unit Unit1;

{ ENHANCEMENT IDEAS:
  1. Print option
  2. Coordinate memo scroll or have a way to sync positions or use a grid
  3. Allow whole word delimiter chars to be modified
  4. backup option for changed files

  ADDED in VERSION 1.2:
  - command line for file association done
  - don't update files that haven't changed
  - return counts
  - restore/save checkbox
  - updated for text DFM files in Delphi 5

  ADDED IN VERSION 2.0:
  - added feature to fix unit and identifier name mismatch compiler warning.
  - added option to add .pas and .dcu unit names in the matching files dialog
  - added Sort button to sort both find and replace lists.
  - fixed problem with quotes not saved in ini file.
  - added validation of number of files to search/replace.
  - add source directory validation to "Show Matching Files" button.
  - changed file extension from ".REP" to ".rep"
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, abcbtn, ExtCtrls, abcedbtn, abcedtdr, Grids, abcfx,
  Menus, abcpanel, abcsplit, abcversn, abclabel, ImgList, abcmemo;

type

  TfrmMain = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    New1: TMenuItem;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    ImageList1: TImageList;
    abcEdgePanel1: TabcEdgePanel;
    pagMain: TPageControl;
    tabStrings: TTabSheet;
    Panel1: TPanel;
    Label6: TLabel;
    abcTriSplit1: TabcTriSplit;
    abcSplitPane1: TabcSplitPane;
    memOldStrings: TabcROMemo;
    abcSplitPane2: TabcSplitPane;
    memNewStrings: TabcROMemo;
    abcSplitPane3: TabcSplitPane;
    memDescription: TMemo;
    tabSource: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Label9: TLabel;
    dirSource: TabcDirectoryEdit;
    cboFiletype: TComboBox;
    rdgFileSelect: TRadioGroup;
    lstSelectFiles: TListBox;
    btnShowFiles: TButton;
    tabDest: TTabSheet;
    Label3: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    lblProgress: TLabel;
    dirDest: TabcDirectoryEdit;
    btnGo: TabcPicBtn;
    prgAllFiles: TProgressBar;
    btnStop: TabcPicBtn;
    btnPause: TabcPicBtn;
    prgCurrentFile: TProgressBar;
    chkCaseInsensitive: TCheckBox;
    chkWholeWordsOnly: TCheckBox;
    chkCopyUnchangedFiles: TCheckBox;
    chkFindOnly: TCheckBox;
    tabResults: TTabSheet;
    memResults: TMemo;
    tabHelp: TTabSheet;
    memHelp: TMemo;
    tabAbout: TTabSheet;
    Bevel2: TBevel;
    abcEffectsImage1: TabcEffectsImage;
    abcVersionLabel1: TabcVersionLabel;
    abcURLLabel1: TabcURLLabel;
    Bevel1: TBevel;
    chkFixUnitIdentifiers: TCheckBox;
    btnSort: TButton;
    Label5: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pagMainChange(Sender: TObject);
    procedure rdgFileSelectClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure memOldStringsChange(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure btnGoClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnShowFilesClick(Sender: TObject);
    procedure dirSourceChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure abcSplitPane2Resize(Sender: TObject);
    procedure abcURLLabel1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chkFixUnitIdentifiersClick(Sender: TObject);
    procedure btnSortClick(Sender: TObject);
  private
    { Private declarations }
    FDataChanged: Boolean;
    FFileName: TFileName;
    FPaused: Boolean;
    FStopped: Boolean;
    FLabelSave: string;
    FNextFileToProcess: Integer;
    FSourceFiles: TStringList;
    FSourceDir, FDestDir: string;
    FCurrentDestSubDir: string;
    FReplacements, FChangedFiles: Integer;
    {UI Methods}
    function PromptToSave: TModalResult;
    function SaveChanges(AlwaysPrompt: Boolean): Boolean;
    procedure DoSaveAs(FileName: TFileName);
    function ValidReplacementCount: Boolean;
    function ValidSourceDir: Boolean;
    function ValidDestDir: Boolean;
    procedure Clear;
    procedure OpenFile(FileName: TFileName);
    procedure FileOpen;
    procedure SetFileName(Value: TFileName);
    procedure ShowMatchingFiles;
    procedure PopulateFileSelectList;
    procedure QuickSortStrings(const lst1, lst2: TStringList; L, R: Integer; SCompare: TStringListSortCompare);
    procedure SortFindReplaceStrings;
    procedure AddUnitNamesFrom(const FileNames: TStrings);
    {String Replace Methods}
    function GetFilesToReplace: Boolean;
    procedure ReplaceAllFiles;
    procedure ReplaceInFile(Source, Dest: TFileName);
    function ReplaceTextStream(Mem: TMemoryStream): Integer;
    function ReplaceAllInString(var S: string): Integer;
    procedure UpdateCurrentFileCount(i: integer);
    procedure UpdateProgressCount(i: integer);
    procedure ResetCurrentFileCount;
    procedure ResetProgressCount;
    procedure InitProgressBars;
    {advertising}
    procedure StartShow;
    procedure StopShow;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

{$R GReplace_img.res}

{$WARN UNIT_PLATFORM OFF}

uses FileCtrl, IniFiles, abcutilf, Unit2;

const
  SEC_GENERAL = 'OST File Replace';

  SEC_DESCRIPTION = 'Description';
  SEC_REPLACE = 'Replace';
  SEC_REPLACE_WITH = 'ReplaceWith';

  ITEM_COUNT = 'ItemCount';
  ITEM_PREFIX = 'Item';
  ITEM_SOURCE = 'SourceDir';
  ITEM_DEST = 'DestDir';
  ITEM_PATTERN = 'FileTypes';
  ITEM_FILE_SELECT = 'FileSelection';
  ITEM_CASE = 'CaseInsensitive';
  ITEM_WHOLE_WORDS = 'WholeWordsOnly';
  ITEM_COPY_UNCHANGED = 'CopyUnchangedFiles';
  ITEM_FIND_ONLY = 'FindOnly';
  ITEM_UNITS = 'FixUnitIdentifiers';
  ITEM_VERSION = 'GReplaceVersion';

  C_CAPTION = 'Global Search & Replace';

{String Replace Methods}

procedure TfrmMain.AddUnitNamesFrom(const FileNames: TStrings);
var
  i, iPos: integer;
  sPasFileName: string;
  strFind, strRepl: TStringList;
begin
  //add all .pas or .dcu file names to replacements
  strFind := TStringList.Create;
  strRepl := TStringList.Create;
  try
    strFind.Assign(memOldStrings.Lines);
    strRepl.Assign(memNewStrings.Lines);
    with FileNames do
      for i := 0 to Count - 1 do
        if (LowerCase(ExtractFileExt(Strings[i])) = '.pas') or
           (LowerCase(ExtractFileExt(Strings[i])) = '.dcu') then
        begin
          sPasFileName := ExtractFileName(Strings[i]);
          iPos := Pos('.', sPasFileName);
          sPasFileName := Copy(sPasFileName, 1, iPos-1);
          if strFind.IndexOf(sPasFileName) = -1 then
          begin
            strFind.Add(sPasFileName);
            strRepl.Add(sPasFileName);
          end;
        end;
    memOldStrings.Lines.Assign(strFind);
    memNewStrings.Lines.Assign(strRepl);
  finally
    strFind.Free;
    strRepl.Free;
  end;
end;

function TfrmMain.GetFilesToReplace: Boolean;
var
  i: integer;
begin
  FSourceDir := dirSource.Text;
  if not abcLastCharInSet(FSourceDir, ['\','/']) then
    FSourceDir := FSourceDir + '\';
  FDestDir := dirDest.Text;
  if not abcLastCharInSet(FDestDir, ['\','/']) then
    FDestDir := FDestDir + '\';
  FSourceFiles.Clear;
  if rdgFileSelect.ItemIndex = 2 then
  begin
    {Selected Files}
    with lstSelectFiles do
      for i := 0 to Items.Count - 1 do
        if Selected[i] then
          FSourceFiles.Add(FSourceDir + Items[i]);
  end
  else
    abcGetFileNames(FSourceDir, cboFileType.Text, True,
      (rdgFileSelect.ItemIndex=0), FSourceFiles);

  if chkFixUnitIdentifiers.Checked then
  begin
    AddUnitNamesFrom(FSourceFiles);
    Result := ValidReplacementCount;
    if not Result then Exit;
  end;
  Result := (FSourceFiles.Count > 0);
  if not Result then
    MessageDlg('Could not find any files to search or replace.',
               mtWarning, [mbOk], 0);
end;

procedure TfrmMain.ReplaceAllFiles;
var
  i: integer;
  TimeStamp: Double;
  TargetFile, TargetDir: string;
begin
  if not FPaused then
  begin
    TimeStamp := GetTickCount;
    if not GetFilesToReplace then Exit;
    FNextFileToProcess := 0;
    FCurrentDestSubDir := '';
    FReplacements := 0;
    FChangedFiles := 0;
    memResults.Lines.Clear;
  end
  else
    TimeStamp := -1;
  if FSourceFiles.Count = 0 then
  begin
    MessageDlg('No files found matching your specification.', mtError, [mbOk], 0);
    pagMain.ActivePage := tabSource;
  end
  else
  begin
    if not FPaused then InitProgressBars;
    FPaused := False;
    FStopped := False;
    {process each file}
    for i := FNextFileToProcess to FSourceFiles.Count - 1 do
    begin
      UpdateProgressCount(i);
      ResetCurrentFileCount;
          FCurrentDestSubDir := '';
      TargetFile := abcReplaceStr(FSourceDir, FDestDir, FSourceFiles[i], True);
      TargetDir := ExtractFilePath(TargetFile);
      if TargetDir <> FCurrentDestSubDir then
      begin
        if not DirectoryExists(TargetDir) and not chkFindOnly.Checked then
          ForceDirectories(TargetDir);
        FCurrentDestSubDir := TargetDir;
      end;
      ReplaceInFile(FSourceFiles[i], TargetFile);
      Application.ProcessMessages;
      if FStopped or FPaused then
        Break
      else
        FNextFileToProcess := i+1;
    end;
  end;
  {total progress}

  if not FPaused then
  begin
    if (TimeStamp <> -1) then
    begin
      TimeStamp := (GetTickCount - TimeStamp) / 1000;
      lblProgress.Caption := 'Processed '
        + InttoStr(FSourceFiles.Count)
        + ' files in '
        + Format('%.3f', [TimeStamp])
        + ' seconds with '
        + InttoStr(FReplacements)
        + ' matches in '
        + InttoStr(FChangedFiles)
        + ' files:';
      memResults.Lines.Insert(0,'');
      memResults.Lines.Insert(0,lblProgress.Caption);
      pagMain.ActivePage := tabResults;
    end;
    ResetProgressCount;
    ResetCurrentFileCount;
  end;
end;

procedure TfrmMain.UpdateCurrentFileCount(i: integer);
begin
  prgCurrentFile.Position := i;
  prgCurrentFile.Repaint;
  Application.ProcessMessages;
end;

procedure TfrmMain.ResetCurrentFileCount;
begin
  prgCurrentFile.Position := 0;
  prgCurrentFile.Repaint;
  Application.ProcessMessages;
end;

procedure TfrmMain.ResetProgressCount;
begin
  prgAllFiles.Position := 0;
  prgAllFiles.Repaint;
  Application.ProcessMessages;
end;

procedure TfrmMain.UpdateProgressCount(i: integer);
begin
  prgAllFiles.Position := i+1;
  prgAllFiles.Repaint;
  lblProgress.Caption := 'File ' + InttoStr(i+1) + FLabelSave;
  Application.ProcessMessages;
end;

procedure TfrmMain.InitProgressBars;
begin
  prgCurrentFile.Max := memOldStrings.Lines.Count;
  prgCurrentFile.Position := 0;
  prgAllFiles.Max := FSourceFiles.Count;
  prgAllFiles.Position := FNextFileToProcess;
  FLabelSave := ' of ' + InttoStr(FSourceFiles.Count);
end;

procedure TfrmMain.ReplaceInFile(Source, Dest: TFileName);
var
  Mem1, Mem2: TMemoryStream;
  reps: integer;
  IsBinaryDFM: Boolean;
begin
  Mem1 := TMemoryStream.Create;
  try
    Mem1.LoadFromFile(Source);
    Mem1.Seek(0,0);
    IsBinaryDFM := (UpperCase(ExtractFileExt(Source)) = '.DFM') and
                   (TestStreamFormat(Mem1) = sofBinary);
    if IsBinaryDFM then
    begin
      Mem2 := TMemoryStream.Create;
      try
        ObjectResourceToText(Mem1, Mem2);
        Mem2.Seek(0,0);
        reps := ReplaceTextStream(Mem2);
        if (reps > 0) and not chkFindOnly.Checked then
        begin
          Mem1.Clear;
          Mem2.Seek(0,0);
          ObjectTextToResource(Mem2, Mem1);
          Mem1.Seek(0,0);
        end;
      finally
        Mem2.Free;
      end;
    end
    else
      reps := ReplaceTextStream(Mem1);

    if reps > 0 then
    begin
      if chkFindOnly.Checked then
        memResults.Lines.Add(Source)
      else
      begin
        Mem1.SaveToFile(Dest);
        memResults.Lines.Add(Dest);
      end;
      FReplacements := FReplacements + reps;
      FChangedFiles := FChangedFiles + 1;
    end
    else
      if chkCopyUnchangedFiles.Checked and (Source <> Dest) and not chkFindOnly.Checked then
        CopyFile(PChar(Source), PChar(Dest), FALSE);

  finally
    Mem1.Free;
  end;
end;

function TfrmMain.ReplaceTextStream(Mem: TMemoryStream): Integer;
var
  sStream: TStringStream;
  sTemp: string;
begin
  sStream := TStringStream.Create('');
  try
    Mem.Seek(0,0);
    sStream.CopyFrom(Mem, Mem.Size);
    sTemp := sStream.DataString;
    Result := ReplaceAllInString(sTemp);
    Mem.Clear;
    if sTemp <> '' then
    begin
      Mem.SetSize(Length(sTemp));
      Mem.Write(sTemp[1], Length(sTemp));
      Mem.Seek(0,0);
    end;
  finally
    sStream.Free;
  end;
end;

function TfrmMain.ReplaceAllInString(var S: string): Integer;
var
  i: integer;
  sTemp: string;
begin
  Result := 0;
  for i := 0 to memOldStrings.Lines.Count - 1 do
  begin
    UpdateCurrentFileCount(i+1);
    Result := Result + abcReplaceAllCount(
      memOldStrings.Lines[i], memNewStrings.Lines[i], S, sTemp,
      not chkCaseInsensitive.Checked, chkWholeWordsOnly.Checked);
    if Result > 0 then
      S := sTemp;
  end;
end;

{UI Methods}

procedure TfrmMain.QuickSortStrings(const lst1, lst2: TStringList; L, R: Integer; SCompare: TStringListSortCompare);
var
  I, J, P: Integer;
begin
  repeat
    I := L;
    J := R;
    P := (L + R) shr 1;
    repeat
      while SCompare(lst1, I, P) < 0 do Inc(I);
      while SCompare(lst1, J, P) > 0 do Dec(J);
      if I <= J then
      begin
        lst1.Exchange(I, J);
        lst2.Exchange(I, J);
        if P = I then
          P := J
        else if P = J then
          P := I;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSortStrings(lst1, lst2, L, J, SCompare);
    L := I;
  until I >= R;
end;

function CompareStrings(List: TStringList; Index1, Index2: Integer): Integer;
begin
  Result := AnsiCompareStr(List[Index1], List[Index2]);
end;

procedure TfrmMain.SortFindReplaceStrings;
//need to sort both lists concurrently based on find strings alpha
var
  lst1, lst2: TStringList;
begin
  if (memOldStrings.Lines.Count > 1) and
     (memNewStrings.Lines.Count > 1) and
     (memOldStrings.Lines.Count = memNewStrings.Lines.Count) then
  begin
    lst1 := TStringList.Create;
    lst2 := TStringList.Create;
    Screen.Cursor := crHourglass;
    try
      lst1.Assign(memOldStrings.Lines);
      lst2.Assign(memNewStrings.Lines);
      QuickSortStrings(lst1, lst2, 0, lst1.Count - 1, CompareStrings);
      memOldStrings.Lines.Assign(lst1);
      memNewStrings.Lines.Assign(lst2);
    finally
      lst1.Free;
      lst2.Free;
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TfrmMain.ShowMatchingFiles;
var
  Dialog: TfrmFileList;
begin
  Dialog := TfrmFileList.Create(Self);
  with Dialog do
  try
    abcGetFileNames(dirSource.Text, cboFileType.Text, True,
      (rdgFileSelect.ItemIndex=0), Memo1.Lines);
    if Memo1.Lines.Count = 0 then
      ShowMessage('No matching files')
    else
    begin
      ShowModal;
      if (ModalResult = mrOk) and chkAddUnits.Checked then
      begin
        Self.Repaint;
        Screen.Cursor := crHourglass;
        AddUnitNamesFrom(Memo1.Lines);
        Screen.Cursor := crDefault;
      end;
    end;
  finally
    Free;
  end;
end;

function TfrmMain.PromptToSave: TModalResult;
const
  DEFAULT_FILE = 'Untitled file';
begin
  if not FDataChanged then
    Result := mrNone
  else
  begin
    if FFileName = '' then
      FFileName := DEFAULT_FILE;
    Result := MessageDlg('Save changes to ' + FFileName + '?', mtConfirmation, [mbYes, mbNo, mbCancel],0);
    if FFileName = DEFAULT_FILE then
      FFileName := '';
  end;
  if Result = mrYes then
    if not SaveChanges(False) then
      Result := mrCancel;
end;

function TfrmMain.SaveChanges(AlwaysPrompt: Boolean): Boolean;
begin
  Result := False;
  if AlwaysPrompt or (FFileName = '') then
  begin
    SaveDialog1.FileName := '';
    if SaveDialog1.Execute then
      DoSaveAs(SaveDialog1.FileName)
    else
      Exit;
  end
  else
    DoSaveAs(FFileName);
  Result := True;
end;

procedure TfrmMain.DoSaveAs(FileName: TFileName);
var
  IniFile: TIniFile;
  i: integer;
begin
  IniFile := TIniFile.Create(FileName);
  with IniFile do
  try
    WriteString(SEC_GENERAL, ITEM_SOURCE, dirSource.Text);
    WriteString(SEC_GENERAL, ITEM_DEST, dirDest.Text);
    WriteString(SEC_GENERAL, ITEM_PATTERN, cboFileType.Text);

    WriteInteger(SEC_GENERAL, ITEM_FILE_SELECT, rdgFileSelect.ItemIndex);

    WriteBool(SEC_GENERAL, ITEM_CASE, chkCaseInsensitive.Checked);

    WriteBool(SEC_GENERAL, ITEM_WHOLE_WORDS, chkWholeWordsOnly.Checked);

    WriteBool(SEC_GENERAL, ITEM_COPY_UNCHANGED, chkCopyUnchangedFiles.Checked);

    WriteBool(SEC_GENERAL, ITEM_FIND_ONLY, chkFindOnly.Checked);

    WriteBool(SEC_GENERAL, ITEM_UNITS, chkFixUnitIdentifiers.Checked);

    WriteString(SEC_GENERAL, ITEM_VERSION, abcVersionLabel1.ValueByKey['FileVersion']);

    EraseSection(SEC_DESCRIPTION);
    WriteInteger(SEC_DESCRIPTION, ITEM_COUNT, memDescription.Lines.Count);
    with memDescription.Lines do
      for i := 0 to Count - 1 do
        WriteString(SEC_DESCRIPTION, ITEM_PREFIX + IntToStr(i), Strings[i]);

    EraseSection(SEC_REPLACE);
    WriteInteger(SEC_REPLACE, ITEM_COUNT, memOldStrings.Lines.Count);
    with memOldStrings.Lines do
      for i := 0 to Count - 1 do
        WriteString(SEC_REPLACE, ITEM_PREFIX + IntToStr(i), '"' + Strings[i] + '"');

    EraseSection(SEC_REPLACE_WITH);
    WriteInteger(SEC_REPLACE_WITH, ITEM_COUNT, memNewStrings.Lines.Count);
    with memNewStrings.Lines do
      for i := 0 to Count - 1 do
        WriteString(SEC_REPLACE_WITH, ITEM_PREFIX + IntToStr(i), '"' + Strings[i] + '"');

  finally
    Free;
  end;
  FDataChanged := False;
  SetFileName(FileName);
end;

procedure TfrmMain.FileOpen;
begin
  if OpenDialog1.Execute then
    OpenFile(OpenDialog1.FileName);
end;

procedure TfrmMain.OpenFile(FileName: TFileName);
var
  IniFile: TIniFile;
  i, iTemp: integer;
  strFind, strRepl, strDesc: TStringList;
begin
  Screen.Cursor := crHourGlass;
  Repaint;
  try
    Clear;
    IniFile := TIniFile.Create(FileName);
    strFind := TStringList.Create;
    strRepl := TStringList.Create;
    strDesc := TStringList.Create;
    with IniFile do
    try
      dirSource.Text := ReadString(SEC_GENERAL, ITEM_SOURCE, '');
      dirDest.Text := ReadString(SEC_GENERAL, ITEM_DEST, '');

      cboFileType.Text := ReadString(SEC_GENERAL, ITEM_PATTERN, '');

      iTemp := ReadInteger(SEC_GENERAL, ITEM_FILE_SELECT, 0);
      if (iTemp > -1) and (iTemp < rdgFileSelect.Items.Count) then
        rdgFileSelect.ItemIndex := iTemp;

      chkCaseInsensitive.Checked := ReadBool(SEC_GENERAL, ITEM_CASE, False);

      chkWholeWordsOnly.Checked := ReadBool(SEC_GENERAL, ITEM_WHOLE_WORDS, False);

      chkCopyUnchangedFiles.Checked := ReadBool(SEC_GENERAL, ITEM_COPY_UNCHANGED, False);

      chkFindOnly.Checked := ReadBool(SEC_GENERAL, ITEM_FIND_ONLY, False);

      chkFixUnitIdentifiers.Checked := ReadBool(SEC_GENERAL, ITEM_UNITS, False);

      iTemp := ReadInteger(SEC_DESCRIPTION, ITEM_COUNT, 0);
      for i := 0 to iTemp - 1 do
        strDesc.Add(ReadString(SEC_DESCRIPTION, ITEM_PREFIX + IntToStr(i), ''));

      iTemp := ReadInteger(SEC_REPLACE, ITEM_COUNT, 0);
      for i := 0 to iTemp - 1 do
        strFind.Add(ReadString(SEC_REPLACE, ITEM_PREFIX + IntToStr(i), ''));

      iTemp := ReadInteger(SEC_REPLACE_WITH, ITEM_COUNT, 0);
      for i := 0 to iTemp - 1 do
        strRepl.Add(ReadString(SEC_REPLACE_WITH, ITEM_PREFIX + IntToStr(i), ''));

      memDescription.Lines.Assign(strDesc);
      memOldStrings.Lines.Assign(strFind);
      memNewStrings.Lines.Assign(strRepl);

    finally
      Free;
      strFind.Free;
      strRepl.Free;
      strDesc.Free;
    end;
    FDataChanged := False;
    SetFileName(FileName);
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmMain.SetFileName(Value: TFileName);
begin
  FFileName := Value;
  if Value = '' then
    Caption := C_CAPTION
  else
    Caption := C_CAPTION + ' - [' + ExtractFileName(Value) + ']';
end;

procedure TfrmMain.Clear;
begin
  memOldStrings.Lines.Clear;
  memNewStrings.Lines.Clear;
  memDescription.Lines.Text := 'DESCRIPTION:';
  dirSource.Text := '';
  cboFileType.ItemIndex := 0;
  rdgFileSelect.ItemIndex := 0;
  lstSelectFiles.Items.Clear;
  dirDest.Text := '';
  chkCaseInsensitive.Checked := False;
  chkFindOnly.Checked := False;
  chkWholeWordsOnly.Checked := False;
  chkCopyUnchangedFiles.Checked := False;
  chkFixUnitIdentifiers.Checked := False;
  SetFileName('');
  FDataChanged := False;
end;

function TfrmMain.ValidReplacementCount: Boolean;
begin
  Result := False;
  if memOldStrings.Lines.Count < memNewStrings.Lines.Count then
  begin
    pagMain.ActivePage := tabStrings;
    memOldStrings.SetFocus;
    MessageDlg('Number of strings to be replaced is less than number of replacement strings.',
               mtWarning, [mbOk], 0);
  end
  else if memNewStrings.Lines.Count < memOldStrings.Lines.Count then
  begin
    pagMain.ActivePage := tabStrings;
    memNewStrings.SetFocus;
    MessageDlg('Number of replacement strings is less than number of strings to be replaced.',
               mtWarning, [mbOk], 0);
  end
  else if memOldStrings.Lines.Count = 0 then
  begin
    pagMain.ActivePage := tabStrings;
    memOldStrings.SetFocus;
    MessageDlg('There is nothing to replace.  Please enter a string.',
               mtWarning, [mbOk], 0);
  end
  else
    Result := True;
end;

function TfrmMain.ValidSourceDir: Boolean;
begin
  Result := DirectoryExists(dirSource.Text);
  if not Result then
  begin
    pagMain.ActivePage := tabSource;
    dirSource.SetFocus;
    MessageDlg('Source Directory does not exist.',
               mtWarning, [mbOk], 0);
  end;
end;

function TfrmMain.ValidDestDir: Boolean;
var
  sDir: string;
begin
  if dirDest.Text = '' then
    dirDest.Text := dirSource.Text;
  if dirDest.Text = dirSource.Text then
  begin
    Result := MessageDlg('Destination same as source. Changed files will be overwritten.',
               mtWarning, [mbOk, mbCancel], 0) = mrOk;
    Exit;
  end
  else if DirectoryExists(dirDest.Text) then
  begin
    Result := MessageDlg('Destination already exists. Files may be overwritten.',
               mtWarning, [mbOk, mbCancel], 0) = mrOk;
    Exit;
  end;
  sDir := dirDest.Text;
  Result := CreateDirectory(PChar(sDir), nil);
  if not Result then
    MessageDlg('Unable to create destination directory ' + sDir,
               mtError, [mbOk], 0);
end;

{Form Events}

procedure TfrmMain.StopShow;
begin
  abcEffectsImage1.Stop;
end;

procedure TfrmMain.StartShow;
begin
  Application.ProcessMessages;
  abcEffectsImage1.Play(True);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  pagMain.ActivePage := tabStrings;
  cboFileType.ItemIndex := 0;
  FSourceFiles := TStringList.Create;
  rdgFileSelectClick(Self);
  FDataChanged := False;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FSourceFiles.Free;
end;

procedure TfrmMain.pagMainChange(Sender: TObject);
begin
  if pagMain.ActivePage = tabAbout then
  begin
    if ImageList1.Count = 0 then
      ImageList1.ResourceLoad(rtBitmap, 'GReplace_ImageList1', clNone);
    StartShow;
  end
  else
    StopShow;
end;

procedure TfrmMain.PopulateFileSelectList;
begin
  Screen.Cursor := crHourglass;
  try
    lstSelectFiles.Clear;
    abcGetFileNames(dirSource.Text, cboFiletype.Text, False, False, lstSelectFiles.Items);
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmMain.rdgFileSelectClick(Sender: TObject);
begin
  case rdgFileSelect.ItemIndex of
    0,1:
      begin
        btnShowFiles.Enabled := True;
        lstSelectFiles.Enabled := False;
        Label9.Enabled := False;
      end;
    2:
      begin
        btnShowFiles.Enabled := False;
        lstSelectFiles.Enabled := True;
        Label9.Enabled := True;
        PopulateFileSelectList;
      end;
  end;
  memOldStringsChange(Self);
end;

procedure TfrmMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.memOldStringsChange(Sender: TObject);
begin
  FDataChanged := True;
end;

procedure TfrmMain.New1Click(Sender: TObject);
begin
  if PromptToSave <> mrCancel then
    Clear;
end;

procedure TfrmMain.Open1Click(Sender: TObject);
begin
  if PromptToSave <> mrCancel then
    FileOpen;
end;

procedure TfrmMain.Save1Click(Sender: TObject);
begin
  SaveChanges(False);
end;

procedure TfrmMain.SaveAs1Click(Sender: TObject);
begin
  SaveChanges(True);
end;

procedure TfrmMain.btnGoClick(Sender: TObject);
begin
  if (chkFixUnitIdentifiers.Checked or ValidReplacementCount) and ValidSourceDir and (chkFindOnly.Checked or ValidDestDir) then
    ReplaceAllFiles;
end;

procedure TfrmMain.btnPauseClick(Sender: TObject);
begin
  FPaused := True;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
  FStopped := True;
end;

procedure TfrmMain.btnShowFilesClick(Sender: TObject);
begin
  if ValidSourceDir then
    ShowMatchingFiles;
end;

procedure TfrmMain.dirSourceChange(Sender: TObject);
begin
  memOldStringsChange(Self);
  if rdgfileSelect.ItemIndex = 2 then
    PopulateFileSelectList;
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := (PromptToSave <> mrCancel);
end;

procedure TfrmMain.abcSplitPane2Resize(Sender: TObject);
begin
  Label6.Left := abcSplitPane2.Left + 10;
end;

procedure TfrmMain.abcURLLabel1Click(Sender: TObject);
begin
  StopShow;
end;

procedure TfrmMain.FormShow(Sender: TObject);
var
  s: string;
  i: integer;
begin
  //command line support
  s := '';
  for i := 1 to ParamCount do
    s := s + ParamStr(i);
  if FileExists(s) then
    OpenFile(s);
end;

procedure TfrmMain.chkFixUnitIdentifiersClick(Sender: TObject);
begin
  if chkFixUnitIdentifiers.Checked then
  begin
    chkCaseInsensitive.Checked := True;
    chkWholeWordsOnly.Checked := True;
  end;
  FDataChanged := True;
end;

procedure TfrmMain.btnSortClick(Sender: TObject);
begin
  if ValidReplacementCount then
    SortFindReplaceStrings;
end;

end.
