unit abcanfr;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls, Forms,
  abcpanel, abcutilg, abcutilf, abcsubfm, abcfx, abcufx;

type

  TabcCustomPageFrame = class;

  TabcPage = class(TCustomControl)
  private
    FFrame: TabcCustomPageFrame;
    FAutoCreate: Boolean;
    FAutoDestroy: Boolean;
    FFormName: string;
    FOnCreateForm: TabcFormNotifyEvent;
    FOnDestroyForm: TabcFormNotifyEvent;
    FOnLoadForm: TabcFormNotifyEvent;
    FFormActions: TabcSubFormActions;
    FChangingTransition: Boolean;
    FParentTransition: Boolean;
    FTransition: TabcTransition;
    FSaveImage: TBitmap;
    FPaintImage: Boolean;
    FOnFormChange: TNotifyEvent;
    FOnFormChanging: TabcFormChangingEvent;
    function GetPageIndex: Integer;
    procedure SetFrame(AFrame: TabcCustomPageFrame);
    procedure SetPageIndex(Value: Integer);
    function GetFormLoaded: Boolean;
    procedure SetAutoCreate(Value: Boolean);
    procedure SetFormLoaded(Value: Boolean);
    procedure DoSetFormLoaded(Value: Boolean);
    procedure SetFormName(Value: string);
    procedure SetAutoDestroy(const Value: Boolean);
    procedure SetFormActions(const Value: TabcSubFormActions);
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
    procedure CMRelease(var Message: TMessage); message CM_RELEASE;
    procedure TransitionChanged(Sender: TObject);
    procedure SetParentTransition(const Value: Boolean);
    procedure SetTransition(const Value: TabcTransition);
    function IsTransitionStored: Boolean;
    procedure CancelTransition(Sender:TObject; var Cancel: Boolean);
    procedure SavePageImage;
    procedure ClearPageImage;
  protected
    FSkipFormPropertyChanges: Boolean;
{$IFDEF DELPHI5}
    FForm: TScrollingWinControl;
{$ELSE}
    FForm: TForm;
{$ENDIF}
    procedure CreateParams(var Params: TCreateParams); override;
    procedure ReadState(Reader: TReader); override;
    procedure CreateHandle; override;
{$IFDEF DELPHI5}
    function CreateForm: TScrollingWinControl; virtual;
    function FindExistingForm: TScrollingWinControl; virtual;
{$ELSE}
    function CreateForm: TForm; virtual;
    function FindExistingForm: TForm; virtual;
{$ENDIF}
    procedure DestroyForm; virtual;
    procedure Paint; override;
{$IFDEF DELPHI5}
    procedure SetForm(Value: TScrollingWinControl); virtual;
    procedure DoSetForm(Value: TScrollingWinControl); virtual;
{$ELSE}
    procedure SetForm(Value: TForm); virtual;
    procedure DoSetForm(Value: TForm); virtual;
{$ENDIF}
    function CanChangeForm(NewFormName: string): Boolean; virtual;
    function CanChangePage: Boolean; virtual;
{$IFDEF DELPHI6}
    procedure SetParent(AParent: TWinControl); override;
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Release;
    procedure FreeForm;
    property Frame: TabcCustomPageFrame read FFrame write SetFrame;
{$IFDEF DELPHI5}
    property Form: TScrollingWinControl read FForm write SetForm;
{$ELSE}
    property Form: TForm read FForm write SetForm;
{$ENDIF}
    property FormLoaded: Boolean read GetFormLoaded write SetFormLoaded;
    property Visible default False;
  published
    property Anchors;
    property AutoCreate: Boolean read FAutoCreate write SetAutoCreate default True;
    property AutoDestroy: Boolean read FAutoDestroy write SetAutoDestroy default True;
    property Caption;
    property Constraints;
    property Color;
    property ParentColor;
    property DragCursor;
    property DragKind;
    property DragMode;
    property FormActions: TabcSubFormActions read FFormActions write SetFormActions
      default [afaCreate, afaActivate, afaDeActivate, afaHide, afaClose, afaDestroy];
    property FormName: string read FFormName write SetFormName;
    property Enabled;
    property Font;
    property Height stored False;
    property Left stored False;
    property PageIndex: Integer read GetPageIndex write SetPageIndex stored False;
    property ParentFont;
    property ParentShowHint;
    property ParentTransition: Boolean read FParentTransition write SetParentTransition default True;
    property PopupMenu;
    property ShowHint;
    property Transition: TabcTransition read FTransition write SetTransition stored IsTransitionStored;
    property Top stored False;
    property Width stored False;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnCreateForm: TabcFormNotifyEvent read FOnCreateForm write FOnCreateForm;
    property OnDestroyForm: TabcFormNotifyEvent read FOnDestroyForm write FOnDestroyForm;
    property OnLoadForm: TabcFormNotifyEvent read FOnLoadForm write FOnLoadForm;
    property OnFormChange: TNotifyEvent read FOnFormChange write FOnFormChange;
    property OnFormChanging: TabcFormChangingEvent read FOnFormChanging write FOnFormChanging;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnStartDock;
    property OnEndDock;
  end;

  { TabcCustomPageFrame }

  TabcPageChangingEvent = procedure(Sender: TObject; Page: TabcPage;
    var AllowChange: Boolean) of object;

  TabcCustomPageFrame = class(TabcCustomEdgePanel)
  private
    FPages: TList;
    FActivePage, FNewActivePage: TabcPage;
    FNewDockPage: TabcPage;
    FUnDockPage: TabcPage;
    FActivatingPage: Boolean;
    FCreatePagesOnLoad: Boolean;
    FOnChange: TNotifyEvent;
    FOnChanging: TabcPageChangingEvent;
    function GetInnerRect: TRect;
    function GetPage(Index: Integer): TabcPage;
    function GetPageCount: Integer;
    procedure InsertPage(Page: TabcPage);
    procedure MovePage(CurIndex, NewIndex: Integer);
    procedure RemovePage(Page: TabcPage);
    procedure CMDockClient(var Message: TCMDockClient); message CM_DOCKCLIENT;
    procedure CMDockNotification(var Message: TCMDockNotification); message CM_DOCKNOTIFICATION;
    procedure CMUnDockClient(var Message: TCMUnDockClient); message CM_UNDOCKCLIENT;
  protected
    procedure DoAddDockClient(Client: TControl; const ARect: TRect); override;
    procedure DockOver(Source: TDragDockObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean); override;
    procedure DoRemoveDockClient(Client: TControl); override;
    function GetPageFromDockClient(Client: TControl): TabcPage;
    procedure GetSiteInfo(Client: TControl; var InfluenceRect: TRect;
      MousePos: TPoint; var CanDock: Boolean); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Loaded; override;
    function CanChange(Page: TabcPage): Boolean; dynamic;
    procedure Change; dynamic;
    procedure ActivatePage(Page: TabcPage); dynamic;
    procedure SetActivePage(Page: TabcPage); virtual;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    procedure ShowControl(AControl: TControl); override;
    {internal}
    property InnerRect: TRect read GetInnerRect;  {client rect minus bevels etc.}
    {for published access in descendent}
    property BevelInner default bvNone;
    property BevelOuter default bvRaised;
    property BorderStyle default bsNone;
    property CreatePagesOnLoad: Boolean read FCreatePagesOnLoad
      write FCreatePagesOnLoad default False;
    property ParentColor default True;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnChanging: TabcPageChangingEvent read FOnChanging write FOnChanging;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function FindNextPage(CurPage: TabcPage;
      GoForward: Boolean): TabcPage;
    procedure SelectNextPage(GoForward: Boolean);
    property PageCount: Integer read GetPageCount;
    property Pages[Index: Integer]: TabcPage read GetPage;
    property TabStop default True;
    {can be published, public here for designer access}
    property ActivePage: TabcPage read FActivePage write SetActivePage;
  end;

  TabcAnimationFrame = class(TabcCustomPageFrame)
  private
    FAnimation: TabcWindowAnimationStyle;
    FAnimationSteps: Byte;
    FAnimationTime: Word;
    FPaintImage: Boolean;
    FSaveImage: TBitmap;
    FTransition: TabcTransition;
    procedure CancelTransition(Sender:TObject; var Cancel: Boolean);
    procedure SavePageImage;
    procedure ClearPageImage;
    procedure SetTransition(Value: TabcTransition);
    procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
  protected
    procedure AnimatePage(Page: TabcPage);
    procedure TransitionPage(Page: TabcPage);
    procedure ActivatePage(Page: TabcPage); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ActivePage;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property DockSite;
    property ParentBiDiMode;
    property Align;
    property Animation: TabcWindowAnimationStyle read FAnimation write FAnimation default awaMoveDown;
    property AnimationSteps: Byte read FAnimationSteps write FAnimationSteps default 5;
    property AnimationTime: Word read FAnimationTime write FAnimationTime default 500;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Color;
    property CreatePagesOnLoad;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Transition: TabcTransition read FTransition write SetTransition;
    property Visible;
    property OnChange;
    property OnChanging;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnStartDrag;
    property OnDockDrop;
    property OnDockOver;
    property OnEndDock;
    property OnGetSiteInfo;
    property OnStartDock;
    property OnUnDock;
  end;

implementation

uses abcconst, Consts, ComStrs, Math;

{$IFDEF DELPHI5}
type
  TScrollingWinControlClass = class of TScrollingWinControl;
{$ENDIF}

{ TabcPage }

constructor TabcPage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alClient;
  Visible := False;
  TabStop := False;
  ControlStyle := ControlStyle + [csAcceptsControls, csNoDesignVisible, csReplicatable];
  ParentColor := True;
  ParentFont := True;
  ParentShowHint := True;
  FForm := nil;
  FFormName := '';
  FAutoCreate := True;
  FAutoDestroy := True;
  FFormActions := [afaCreate, afaActivate, afaDeActivate, afaHide, afaClose, afaDestroy];
  FTransition := TabcTransition.Create;
  FTransition.OnChange := TransitionChanged;
  FTransition.OnEffectStep := CancelTransition;
  FParentTransition := True;
end;

destructor TabcPage.Destroy;
begin
  if FFrame <> nil then FFrame.RemovePage(Self);
  if afaDestroy in FormActions then DestroyForm;
  FTransition.Free;
  ClearPageImage;
  inherited Destroy;
end;

procedure TabcPage.CMRelease(var Message: TMessage);
begin
  Free;
end;

procedure TabcPage.Release;
begin
  PostMessage(Handle, CM_RELEASE,0,0);
end;

function TabcPage.GetPageIndex: Integer;
begin
  if FFrame <> nil then
    Result := FFrame.FPages.IndexOf(Self)
  else
    Result := -1;
end;

procedure TabcPage.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params.WindowClass do
    style := style and not (CS_HREDRAW or CS_VREDRAW);
end;

procedure TabcPage.CMShowingChanged(var Message: TMessage);
var
  FocusCandidate: TWinControl;
begin
  inherited;
  if (csDesigning in ComponentState) then Exit;
  if Showing then
    if (Form <> nil) then
      with Form do
      begin
        if not Showing then Show;
        if afaActivate in FormActions then Perform(CM_ACTIVATE,0,0);
        if CanFocus then SetFocus;
      end
    else
      if afaActivate in FormActions then
        if CanFocus then
        begin
          FocusCandidate := FindNextControl(nil, True, True, True);
          if (FocusCandidate <> nil) and FocusCandidate.CanFocus then
            FocusCandidate.SetFocus;
        end;
end;

procedure TabcPage.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  if Reader.Parent is TabcCustomPageFrame then
    Frame := TabcCustomPageFrame(Reader.Parent);
end;

procedure TabcPage.SetFrame(AFrame: TabcCustomPageFrame);
begin
  if FFrame <> AFrame then
  begin
    if FFrame <> nil then FFrame.RemovePage(Self);
    Parent := AFrame;
    if AFrame <> nil then AFrame.InsertPage(Self);
  end;
end;

procedure TabcPage.SetPageIndex(Value: Integer);
var
  MaxPageIndex: Integer;
begin
  if FFrame <> nil then
  begin
    MaxPageIndex := FFrame.FPages.Count - 1;
    if Value > MaxPageIndex then
      raise EListError.CreateFmt(SPageIndexError, [Value, MaxPageIndex]);
    FFrame.MovePage(PageIndex, Value);
  end;
end;

{$IFDEF DELPHI5}
function TabcPage.CreateForm: TScrollingWinControl;
{$ELSE}
function TabcPage.CreateForm: TForm;
{$ENDIF}
var
  AClass: TPersistentClass;
{$IFDEF DELPHI5}
  AFormClass: TScrollingWinControlClass;
{$ELSE}
  AFormClass: TFormClass;
{$ENDIF}
begin
  if FFormName = '' then
    raise Exception.Create(sabcErrFormNameExpected);
  AClass := FindClass('T' + FFormName);
{$IFDEF DELPHI5}
  AFormClass := TScrollingWinControlClass(AClass);
  Result := AFormClass.Create(Self);
{$ELSE}
  AFormClass := TFormClass(AClass);
  Result := AFormClass.Create(Self);
{$ENDIF}
  if Assigned(FOnCreateForm) then FOnCreateForm(Self, Result);
end;

procedure TabcPage.FreeForm;
begin
  DestroyForm;
end;

procedure TabcPage.DestroyForm;
begin
  if FForm <> nil then
  begin
    if Assigned(FOnDestroyForm) then FOnDestroyForm(Self, FForm);
    if not (csDestroying in FForm.ComponentState) then
{$IFDEF DELPHI5}
//        if FForm is TForm then
//          TForm(FForm).Release
//        else
        FForm.Free;
{$ELSE}
      FForm.Release;
{$ENDIF}
    FForm := nil;
  end;
end;

{$IFDEF DELPHI5}
function TabcPage.FindExistingForm: TScrollingWinControl;
{$ELSE}
function TabcPage.FindExistingForm: TForm;
{$ENDIF}
var
  AClassName: string;
  i: integer;
begin
  Result := nil;
  if FFormName = '' then
    raise Exception.Create(sabcErrFormNameExpected);
  AClassName := 'T' + FFormName;
  with Application do
    for i := 0 to ComponentCount - 1 do
      if Components[i].ClassName = AClassName then
      begin
{$IFDEF DELPHI5}
        Result := TScrollingWinControl(Components[i]);
{$ELSE}
        Result := TForm(Components[i]);
{$ENDIF}
        Break;
      end;
end;

procedure TabcPage.CreateHandle;
begin
  inherited CreateHandle;
  if not (csDesigning in ComponentState) and
     ([afaCreate, afaShowExisting] * FormActions <> []) and
     (FForm = nil) and
     (FFormName <> '') then
    FormLoaded := True;
end;

function TabcPage.GetFormLoaded: Boolean;
begin
  Result := FForm <> nil;
end;

function TabcPage.CanChangePage: Boolean;
begin
  Result := True;
  if (csLoading in ComponentState) or (FForm = nil) then
    Exit;
  if ([afaClose, afaDestroy] * FormActions <> []) then
    if FForm is TForm then
      Result := TForm(FForm).CloseQuery;
end;

function TabcPage.CanChangeForm(NewFormName: string): Boolean;
begin
  Result := True;
  if (csLoading in ComponentState) or (FForm = nil) then
    Exit;
  if (FForm <> nil) and ([afaClose, afaDestroy] * FormActions <> []) then
    if FForm is TForm then
    begin
      Result := TForm(FForm).CloseQuery;
      if not Result then Exit;
    end;
  if Assigned(FOnFormChanging) then FOnFormChanging(Self, NewFormName, Result);
end;

procedure TabcPage.SavePageImage;
begin
  ClearPageImage;
  FSaveImage := abcCopyWindowFromDesktop(Self, True);
end;

procedure TabcPage.ClearPageImage;
begin
  if FSaveImage <> nil then
  begin
    FSaveImage.Free;
    FSaveImage := nil;
  end;
end;

procedure TabcPage.Paint;
begin
  if FPaintImage then
    Canvas.Draw(0,0,FSaveImage)
  else
  begin
    inherited Paint;
    if (FForm <> nil) then FForm.Repaint;
  end;
end;

procedure TabcPage.SetAutoCreate(Value: Boolean);
begin
  if Value <> FAutoCreate then begin
    FAutoCreate := Value;
    if Value then
      FFormActions := FFormActions + [afaCreate]
    else
      FFormActions := FFormActions - [afaCreate];
    if not (csDesigning in ComponentState) then
      if Value then
        SetFormName(FFormName)
      else if FForm <> nil then
        SetForm(nil);
  end;
end;

procedure TabcPage.SetAutoDestroy(const Value: Boolean);
begin
  if Value <> FAutoDestroy then begin
    FAutoDestroy := Value;
    if Value then
      FFormActions := FFormActions + [afaDestroy]
    else
      FFormActions := FFormActions - [afaDestroy];
  end;
end;

procedure TabcPage.SetFormActions(const Value: TabcSubFormActions);
begin
  FFormActions := Value;
  if afaCreate in Value then
    AutoCreate := True
  else
    AutoCreate := False;
  if afaDestroy in Value then
    AutoDestroy := True
  else
    AutoDestroy := False;
end;

{$IFDEF DELPHI5}
procedure TabcPage.SetForm(Value: TScrollingWinControl);
{$ELSE}
procedure TabcPage.SetForm(Value: TForm);
{$ENDIF}
var
  NewFormName: string;
begin
  if Value <> nil then
    NewFormName := Copy(Value.ClassName, 2, 256)
  else
    NewFormName := '';
  if CanChangeForm(NewFormName) then
    DoSetForm(Value);
end;

{$IFDEF DELPHI5}
procedure TabcPage.DoSetForm(Value: TScrollingWinControl);
{$ELSE}
procedure TabcPage.DoSetForm(Value: TForm);
{$ENDIF}
var
  ClientR, OffScreenR, DrawR, RemainingR: TRect;
  NewAlign: TAlign;
  bmp: TBitmap;
  DrawTransition: TabcTransition;
  hasHandle: Boolean;
  isActivePage: Boolean;
begin
  hasHandle := HandleAllocated;
  isActivePage :=
    (FFrame <> nil) and (
      (FFrame.ActivePage = Self) or
      (FFrame.FActivatingPage and (FFrame.FNewActivePage = Self)));
  if (Value <> FForm) then
  begin
    ClientR := ClientRect;
    AdjustClientRect(ClientR);

    if ParentTransition and (Parent is TabcAnimationFrame) then
      DrawTransition := TabcAnimationFrame(Parent).Transition
    else
      DrawTransition := Transition;

    if FForm <> nil then
    begin
      SavePageImage;
      FPaintImage := True;
      with FForm do
      begin
        if afaDeActivate in FormActions then Perform(CM_DEACTIVATE,0,0);
        if afaHide in FormActions then Hide;
{$IFDEF DELPHI5}
        if afaClose in FormActions then
          if FForm is TForm then
            TForm(FForm).Close
          else
            FForm.Visible := False;
{$ELSE}
        if afaClose in FormActions then Close;
{$ENDIF}
        Parent := nil;
      end;
      if afaDestroy in FormActions then
        DestroyForm;
    end;
    FForm := nil;

    if (Value <> nil) then
    begin
      with Value do
      begin
        if not FSkipFormPropertyChanges then
        begin
          {change the form style}
{$IFDEF DELPHI5}
          if Value is TForm then
            with TForm(Value) do
            begin
              BorderStyle := bsNone;
              BorderIcons := [];
              FormStyle := fsNormal;
            end;
{$ELSE}
          BorderStyle := bsNone;
          BorderIcons := [];
          FormStyle := fsNormal;
{$ENDIF}

          if Align = alNone then
            NewAlign := alClient
          else
            NewAlign := Align;

          //size and show new form off screen client aligned
          Align := alNone;
{$IFDEF DELPHI5}
          if Value is TForm then
            TForm(Value).AutoScroll := True;
{$ELSE}
          AutoScroll := True;
{$ENDIF}
        end
        else
        begin
          NewAlign := Align;
          Align := alNone;
        end;

        OffScreenR := ClientR;
        case NewAlign of
          alNone, alClient: ;
          alTop:
            OffScreenR.Bottom := OffScreenR.Top + Height;
          alBottom:
            OffScreenR.Top := OffScreenR.Bottom - Height;
          alLeft:
            OffScreenR.Right := OffScreenR.Left + Width;
          alRight:
            OffScreenR.Left := OffScreenR.Right - Width;
        end;
        DrawR := OffScreenR;
        OffsetRect(OffScreenR, -OffScreenR.Right, -OffScreenR.Bottom);
        with OffScreenR do SetBounds(Left, Top, Right-Left, Bottom-Top);
        Parent := Self;
        if isActivePage then
        begin
          Show;
          if (Self.Parent <> nil) and Self.Parent.Showing then
          begin
            if DrawTransition.Enabled then
              DrawTransition.DrawWindowAt(Value, Self.Canvas, True, DrawR);
            SubtractRect(RemainingR, ClientR, DrawR);
            if DrawTransition.NoRedraw and DrawTransition.Enabled then
              SetWindowPos(Handle, HWND_TOP, DrawR.Left, DrawR.Top, Width, Height, SWP_NOREDRAW)
            else
              SetWindowPos(Handle, HWND_TOP, DrawR.Left, DrawR.Top, Width, Height, SWP_SHOWWINDOW);
          end;
          if afaActivate in FormActions then
          begin
            SetFocus;
            Perform(CM_ACTIVATE,0,0);
          end;
        end;
        Align := NewAlign;
      end;
      FPaintImage := False;
      if not IsRectEmpty(RemainingR) then
        Invalidate;
      FForm := Value;
      if Assigned(FOnLoadForm) then FOnLoadForm(Self, Value);
    end
    else if DrawTransition.Enabled and not (csDestroying in ComponentState) and Parent.Showing and isActivePage then
    begin
      FPaintImage := False;
      bmp := abcGetControlBitmap(Self);
      FPaintImage := True;
      with bmp do
      try
        DrawTransition.DrawAt(Canvas, Self.Canvas, Rect(0,0,Width, Height));
      finally
        Free;
      end;
    end;
    FPaintImage := False;
    ClearPageImage;
    if Assigned(FOnFormChange) then FOnFormChange(Self);
  end;
  if not hasHandle and HandleAllocated then
    DestroyHandle;
end;

procedure TabcPage.SetFormLoaded(Value: Boolean);
var
  NewFormName: string;
begin
  if Value then
    NewFormName := FormName
  else
    NewFormName := '';
  if CanChangeForm(NewFormName) then
    DoSetFormLoaded(Value);
end;

procedure TabcPage.DoSetFormLoaded(Value: Boolean);
var
{$IFDEF DELPHI5}
  AForm: TScrollingWinControl;
{$ELSE}
  AForm: TForm;
{$ENDIF}
begin
  AForm := nil;
  if Value then
  begin
    if (afaShowExisting in FormActions) then
      AForm := FindExistingForm;
    if (AForm = nil) and (afaCreate in FormActions) then
      AForm := CreateForm;
    DoSetForm(AForm);
  end
  else
    DoSetForm(nil);
end;

procedure TabcPage.SetFormName(Value: string);
begin
  if CanChangeForm(Value) then
  begin
    FFormName := Value;
    if ([csLoading, csDesigning] * ComponentState = []) and
       ([afaCreate, afaShowExisting] * FormActions <> []) and
       HandleAllocated then
      DoSetFormLoaded(Value <> '');
  end;
end;

{$IFDEF DELPHI6}
procedure TabcPage.SetParent(AParent: TWinControl);
begin
  if (AParent <> nil) and not (AParent is TabcCustomPageFrame) then
    Abort
  else
    inherited;
end;
{$ENDIF}

procedure TabcPage.SetParentTransition(const Value: Boolean);
begin
  FParentTransition := Value;
end;

procedure TabcPage.SetTransition(const Value: TabcTransition);
begin
  FTransition.Assign(Value);
end;

procedure TabcPage.TransitionChanged(Sender: TObject);
begin
  if not FChangingTransition then
    ParentTransition := False;
end;

function TabcPage.IsTransitionStored: Boolean;
begin
  Result := not ParentTransition;
end;

procedure TabcPage.CancelTransition(Sender:TObject; var Cancel: Boolean);
begin
  Cancel := Application.Terminated or (csDestroying in ComponentState);
end;

{ TabcCustomPageFrame }


constructor TabcCustomPageFrame.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 300;
  Height := 200;
  TabStop := True;
  {tip: Removing csAcceptsControls can cause awful designer painting bugs - fix is in CreateParams}
  ControlStyle := [csCaptureMouse, csClickEvents, csDoubleClicks, csReplicatable];
  FPages := TList.Create;
  BevelInner := bvNone;
  BevelOuter := bvRaised;
  BorderStyle := bsNone;
  ParentColor := True;
  TabStop := False;
end;

destructor TabcCustomPageFrame.Destroy;
var
  i: integer;
begin
  for i := 0 to FPages.Count - 1 do
    TabcPage(FPages[i]).FFrame := nil;
  FPages.Free;
  inherited Destroy;
end;

procedure TabcCustomPageFrame.Loaded;
var
  i: Integer;
begin
  inherited Loaded;
  if CreatePagesOnLoad then
    for I := 0 to FPages.Count - 1 do
      TabcPage(FPages[i]).HandleNeeded;
end;

procedure TabcCustomPageFrame.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or WS_CLIPCHILDREN;
    ExStyle := ExStyle or WS_EX_CONTROLPARENT;
  end;
end;

function TabcCustomPageFrame.CanChange(Page: TabcPage): Boolean;
begin
  Result := True;
  if (csLoading in ComponentState) then Exit;
  if ActivePage <> nil then
  begin
    Result := ActivePage.CanChangePage;
    if not Result then Exit;
  end;
  if Assigned(FOnChanging) then FOnChanging(Self, Page, Result);
end;

procedure TabcCustomPageFrame.Change;
begin
  abcDesignerModified(Self);
  if Assigned(FOnChange) then FOnChange(Self);
end;

function TabcCustomPageFrame.GetInnerRect: TRect;
var
  bevelSize: Integer;
begin
  Result := ClientRect;
  bevelSize := BorderWidth;
  if BevelOuter <> bvNone then
    bevelSize := bevelSize + BevelWidth;
  if BevelInner <> bvNone then
    bevelSize := bevelSize + BevelWidth;
  with Result do
  begin
    if beLeft in BevelEdges then
      Left := Left + bevelSize;
    if beTop in BevelEdges then
      Top := Top + bevelSize;
    if beRight in BevelEdges then
      Right := Right - bevelSize;
    if beBottom in BevelEdges then
      Bottom := Bottom - bevelSize;
  end;
end;

function TabcCustomPageFrame.FindNextPage(CurPage: TabcPage;
  GoForward: Boolean): TabcPage;
var
  I: Integer;
begin
  Result := nil;
  if FPages.Count <> 0 then
  begin
    I := FPages.IndexOf(CurPage);
    if GoForward then
      if I = (FPages.Count-1) then
        I := 0
      else
        Inc(I)
    else
      if (I = 0) or (I = -1) then
        I := FPages.Count - 1
      else
        Dec(I);
    Result := FPages[I];
  end;
end;

procedure TabcCustomPageFrame.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
begin
  for I := 0 to FPages.Count - 1 do Proc(TComponent(FPages[I]));
end;

procedure TabcCustomPageFrame.SetChildOrder(Child: TComponent; Order: Integer);
begin
  TabcPage(Child).PageIndex := Order;
end;

function TabcCustomPageFrame.GetPage(Index: Integer): TabcPage;
begin
  Result := FPages[Index];
end;

function TabcCustomPageFrame.GetPageCount: Integer;
begin
  Result := FPages.Count;
end;

procedure TabcCustomPageFrame.InsertPage(Page: TabcPage);
begin
  FPages.Add(Page);
  Page.FFrame := Self;
end;

procedure TabcCustomPageFrame.MovePage(CurIndex, NewIndex: Integer);
begin
  FPages.Move(CurIndex, NewIndex);
end;

procedure TabcCustomPageFrame.RemovePage(Page: TabcPage);
var
  index: integer;
begin
  index := Page.PageIndex;
  if FActivePage = Page then SetActivePage(nil);
  Page.FFrame := nil;
  FPages.Remove(Page);
  if (index > 0) and (index = PageCount) then
    index := index - 1;
  if index < PageCount then
    SetActivePage(Pages[index]);
end;

procedure TabcCustomPageFrame.SelectNextPage(GoForward: Boolean);
var
  Page: TabcPage;
begin
  Page := FindNextPage(ActivePage, GoForward);
  ActivePage := Page;
end;

procedure TabcCustomPageFrame.ActivatePage(Page: TabcPage);
begin
  Page.BringToFront;
  Page.Visible := True;
end;

procedure TabcCustomPageFrame.SetActivePage(Page: TabcPage);
var
  ParentForm: TCustomForm;
begin
  if FActivatingPage or   {Page activation is not reentrant}
     ((Page <> nil) and (Page.Frame <> Self)) or
     not CanChange(Page) then
    Exit;
  if FActivePage <> Page then
    if (csDesigning in ComponentState) and ([csLoading, csReading] * ComponentState <> []) then
    begin
      FActivePage := Page;
      Page.Visible := True;
    end
    else
    begin
      ParentForm := GetParentForm(Self);
      if (ParentForm <> nil) and (FActivePage <> nil) and
        FActivePage.ContainsControl(ParentForm.ActiveControl) then
      begin
        ParentForm.ActiveControl := FActivePage;
        if ParentForm.ActiveControl <> FActivePage then
        begin
          FActivatingPage := False;
          Exit;
        end;
      end;
      if Page <> nil then
      begin
        FActivatingPage := True;
        FNewActivePage := Page;
        try
          ActivatePage(Page);
        finally
          FActivatingPage := False;
          FNewActivePage := nil;
        end;
        if (ParentForm <> nil) and (FActivePage <> nil) and
          (ParentForm.ActiveControl = FActivePage) then
          if Page.CanFocus then
            ParentForm.ActiveControl := Page else
            ParentForm.ActiveControl := Self;
      end;
      if FActivePage <> nil then FActivePage.Visible := False;
      FActivePage := Page;
      ReAlign;
      if (ParentForm <> nil) and (FActivePage <> nil) and
        (ParentForm.ActiveControl = FActivePage) then
        FActivePage.SelectFirst;
      Change;
    end;
end;

procedure TabcCustomPageFrame.ShowControl(AControl: TControl);
begin
  if (AControl is TabcPage) and (TabcPage(AControl).Frame = Self) then
    SetActivePage(TabcPage(AControl));
end;

// Dock Site Support
procedure TabcCustomPageFrame.DoAddDockClient(Client: TControl; const ARect: TRect);
begin
  if FNewDockPage <> nil then Client.Parent := FNewDockPage;
end;

procedure TabcCustomPageFrame.DockOver(Source: TDragDockObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
var
  R: TRect;
begin
  GetWindowRect(Handle, R);
  Source.DockRect := R;
  DoDockOver(Source, X, Y, State, Accept);
end;

procedure TabcCustomPageFrame.DoRemoveDockClient(Client: TControl);
begin
  if (FUnDockPage <> nil) and not (csDestroying in ComponentState) then
  begin
    FUnDockPage.Free;
    FUnDockPage := nil;
  end;
end;

function TabcCustomPageFrame.GetPageFromDockClient(Client: TControl): TabcPage;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to PageCount - 1 do
  begin
    if (Client.Parent = Pages[I]) and (Client.HostDockSite = Self) then
    begin
      Result := Pages[I];
      Exit;
    end;
  end;
end;

procedure TabcCustomPageFrame.GetSiteInfo(Client: TControl; var InfluenceRect: TRect;
  MousePos: TPoint; var CanDock: Boolean);
begin
  CanDock := GetPageFromDockClient(Client) = nil;
  inherited GetSiteInfo(Client, InfluenceRect, MousePos, CanDock);
end;

procedure TabcCustomPageFrame.CMDockClient(var Message: TCMDockClient);
var
  IsVisible: Boolean;
  DockCtl: TControl;
begin
  Message.Result := 0;
  FNewDockPage := TabcPage.Create(Self);
  try
    try
      DockCtl := Message.DockSource.Control;
      if DockCtl is TCustomForm then
        FNewDockPage.Caption := TCustomForm(DockCtl).Caption;
      FNewDockPage.Frame := Self;
      DockCtl.Dock(Self, Message.DockSource.DockRect);
    except
      FNewDockPage.Free;
      raise;
    end;
    IsVisible := DockCtl.Visible;
    FNewDockPage.Visible := IsVisible;
    if IsVisible then ActivePage := FNewDockPage;
    DockCtl.Align := alClient;
  finally
    FNewDockPage := nil;
  end;
end;

procedure TabcCustomPageFrame.CMDockNotification(var Message: TCMDockNotification);
var
  I: Integer;
  S: string;
  Page: TabcPage;
begin
  Page := GetPageFromDockClient(Message.Client);
  if Page <> nil then
    case Message.NotifyRec.ClientMsg of
      WM_SETTEXT:
        begin
          S := PChar(Message.NotifyRec.MsgLParam);
          { Search for first CR/LF and end string there }
          for I := 1 to Length(S) do
            if S[I] in [#13, #10] then
            begin
              SetLength(S, I - 1);
              Break;
            end;
          Page.Caption := S;
        end;
      CM_VISIBLECHANGED:
        Page.Visible := Boolean(Message.NotifyRec.MsgWParam);
    end;
  inherited;
end;

procedure TabcCustomPageFrame.CMUnDockClient(var Message: TCMUnDockClient);
var
  Page: TabcPage;
begin
  Message.Result := 0;
  Page := GetPageFromDockClient(Message.Client);
  if Page <> nil then
  begin
    FUnDockPage := Page;
    Message.Client.Align := alNone;
  end;
end;

{ TabcAnimationFrame }

constructor TabcAnimationFrame.Create(AOwner: TComponent);
begin
  inherited Create(aOwner);
  FTransition := TabcTransition.Create;
  FTransition.OnEffectStep := CancelTransition;
  FAnimationSteps := 5;
  FAnimationTime := 500;
  FAnimation := awaMoveDown;
end;

destructor TabcAnimationFrame.Destroy;
begin
  FTransition.Free;
  ClearPageImage;
  inherited;
end;

procedure TabcAnimationFrame.AnimatePage(Page: TabcPage);
var
  FromR, ToR: TRect;
begin
  if (csDestroying in ComponentState) then Exit;
  if (ActivePage <> nil) then
  begin
    ShowWindow(ActivePage.Handle, SW_HIDE);
    if (ActivePage.Form <> nil) then
      with ActivePage, Form do
      begin
        if afaDeActivate in FormActions then Perform(CM_DEACTIVATE,0,0);
        if afaHide in FormActions then Hide;
{$IFDEF DELPHI5}
        if afaClose in FormActions then
          if Form is TForm then
            TForm(Form).Close
          else
            Form.Visible := False;
{$ELSE}
        if afaClose in FormActions then Close;
{$ENDIF}
        if afaDestroy in FormActions then
          ActivePage.DestroyForm;
      end;
    ActivePage.Hide;
  end;
  with Page do
  begin
    Align := alNone;
    SetBounds(-Width,-Height, Width, Height);
  end;
  inherited ActivatePage(Page);
  ToR := InnerRect;
  ToR.Right := ToR.Right - ToR.Left;
  ToR.Bottom := ToR.Bottom - ToR.Top;
  FromR := Rect(ToR.Left, ToR.Top, 0, 0);
  if not (csLoading in ComponentState) then
    abcAnimateControl(Page, FromR, ToR, FAnimationSteps, FAnimationTime, FAnimation);
  Page.Align := alClient;
end;

procedure TabcAnimationFrame.ActivatePage(Page: TabcPage);
var
  UseTransition: Boolean;
  oldActivePage: TabcPage;
begin
  if (csDestroying in ComponentState) then Exit;
  if not (csDesigning in ComponentState) then
  begin
    UseTransition :=
      (Transition.Enabled or
      ((Page <> nil) and Showing and not Page.ParentTransition and Page.Transition.Enabled))
      and not (csLoading in ComponentState);
    oldActivePage := ActivePage;
    if UseTransition then
      TransitionPage(Page)
    else
      AnimatePage(Page);
    if oldActivePage <> nil then
      oldActivePage.DestroyHandle;
  end
  else if not (csLoading in ComponentState) then
  begin
    Page.Align := alClient;
    inherited ActivatePage(Page);
  end;
end;

procedure TabcAnimationFrame.TransitionPage(Page: TabcPage);
var
  ClientR, OffScreenR: TRect;
  bmp: TBitmap;
  DrawTransition: TabcTransition;
begin
  ClientR := ClientRect;
  AdjustClientRect(ClientR);
  SavePageImage;
  FPaintImage := True;
  if ActivePage <> nil then
  begin
    ShowWindow(ActivePage.Handle, SW_HIDE);
    if (ActivePage.Form <> nil) then
      with ActivePage, Form do
      begin
        if afaDeActivate in FormActions then Perform(CM_DEACTIVATE,0,0);
        if afaHide in FormActions then Hide;
{$IFDEF DELPHI5}
        if afaClose in FormActions then
          if Form is TForm then
            TForm(Form).Close
          else
            Form.Visible := False;
{$ELSE}
        if afaClose in FormActions then Close;
{$ENDIF}
        if afaDestroy in FormActions then
          ActivePage.DestroyForm;
      end;
    ActivePage.Hide;
  end;
  if Page <> nil then
    with Page do
    begin
      Align := alNone;
      OffScreenR := ClientR;
      OffsetRect(OffScreenR, -OffScreenR.Right, -OffScreenR.Top);
      with OffScreenR do SetBounds(Left, Top, Right-Left, Bottom-Top);
      if (Form <> nil) and not Form.Showing then
        Form.Show;
      BringToFront;
      Show;
      if ParentTransition then
        DrawTransition := Self.Transition
      else
        DrawTransition := Transition;
      if Parent.Showing then
      begin
        DrawTransition.DrawWindow(Page, Self.Canvas, False, ClientR.Left, ClientR.Top);
        if DrawTransition.NoRedraw and DrawTransition.Enabled then
          SetWindowPos(Handle, HWND_TOP, ClientR.Left, ClientR.Top, Width, Height, SWP_NOREDRAW);
      end;
      Align := alClient;
    end
  else
  begin
    FPaintImage := False;
    bmp := abcGetControlBitmap(Self);
    FPaintImage := True;
    with bmp do
    try
      if Showing then
        Transition.DrawAt(Canvas, Self.Canvas, Rect(0,0,Width, Height));
    finally
      Free;
    end;
  end;
  FPaintImage := False;
  ClearPageImage;
end;

procedure TabcAnimationFrame.SetTransition(Value: TabcTransition);
begin
  FTransition.Assign(Value);
end;

procedure TabcAnimationFrame.CancelTransition(Sender:TObject; var Cancel: Boolean);
begin
  Cancel := Application.Terminated or (csDestroying in ComponentState) or not Visible;
end;

procedure TabcAnimationFrame.SavePageImage;
begin
  ClearPageImage;
  FSaveImage := abcCopyWindowFromDesktop(Self, True);
end;

procedure TabcAnimationFrame.ClearPageImage;
begin
  if FSaveImage <> nil then
  begin
    FSaveImage.Free;
    FSaveImage := nil;
  end;
end;

procedure TabcAnimationFrame.Paint;
begin
  if FPaintImage then
    Canvas.Draw(0,0,FSaveImage)
  else
    inherited;
end;

procedure TabcAnimationFrame.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  if not FPaintImage then
    inherited
  else
    Message.Result := 1;
end;

end.
