unit abcapp;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ global singleton Application and Screen Event Lists for ABC }

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, ActnList;

type

  { TabcApplicationEvents }

  { This class allows multiple components to receive notification of
    application events }

  TabcApplicationEvents = class(TObject)
  private
    FSaveOnActionExecute, FSaveOnActionUpdate: TActionEvent;
    FSaveOnShortCut: TShortCutEvent;
    FOnActionExecuteEvents: TList;
    FOnActionUpdateEvents: TList;
    FOnShortCutEvents: TList;
    FSaveOnMinimize, FSaveOnRestore: TNotifyEvent;
    FOnMinimizeEvents: TList;
    FOnRestoreEvents: TList;
    FSaveOnActivate, FSaveOnDeactivate: TNotifyEvent;
    FSaveOnException: TExceptionEvent;
    FSaveOnHelp: THelpEvent;
    FSaveOnHint: TNotifyEvent;
    FSaveOnShowHint: TShowHintEvent;
    FSaveOnIdle: TIdleEvent;
    FSaveOnMessage: TMessageEvent;
    FOnActivateEvents: TList;
    FOnDeactivateEvents: TList;
    FOnExceptionEvents: TList;
    FOnHelpEvents: TList;
    FOnHintEvents: TList;
    FOnShowHintEvents: TList;
    FOnIdleEvents: TList;
    FOnMessageEvents: TList;

    procedure AddTMethod(var List: TList; Method: TMethod);
    procedure RemoveTMethod(var List: TList; Method: TMethod);
    procedure RemoveAllTMethods(var List: TList);
    procedure AppOnActionExecute(Action: TBasicAction; var Handled: Boolean);
    procedure AppOnActionUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure AppOnShortCut(var Msg: TWMKey; var Handled: Boolean);
    procedure AppOnMinimize(Sender: TObject);
    procedure AppOnRestore(Sender: TObject);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
    procedure AppOnException(Sender: TObject; E: Exception);
    function AppOnHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
    procedure AppOnHint(Sender: TObject);
    procedure AppOnShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnMessage(var Msg: TMsg; var Handled: Boolean);

  public
    destructor Destroy; override;
    procedure RegisterOnActionExecute(OnActionExecute: TActionEvent);
    procedure UnRegisterOnActionExecute(OnActionExecute: TActionEvent);
    procedure RegisterOnActionUpdate(OnActionUpdate: TActionEvent);
    procedure UnRegisterOnActionUpdate(OnActionUpdate: TActionEvent);
    procedure RegisterOnShortCut(OnShortCut: TShortCutEvent);
    procedure UnRegisterOnShortCut(OnShortCut: TShortCutEvent);
    procedure RegisterOnMinimize(OnMinimize: TNotifyEvent);
    procedure UnRegisterOnMinimize(OnMinimize: TNotifyEvent);
    procedure RegisterOnRestore(OnRestore: TNotifyEvent);
    procedure UnRegisterOnRestore(OnRestore: TNotifyEvent);
    procedure RegisterOnActivate(OnActivate: TNotifyEvent);
    procedure UnRegisterOnActivate(OnActivate: TNotifyEvent);
    procedure RegisterOnDeactivate(OnDeactivate: TNotifyEvent);
    procedure UnRegisterOnDeactivate(OnDeactivate: TNotifyEvent);
    procedure RegisterOnException(OnException: TExceptionEvent);
    procedure UnRegisterOnException(OnException: TExceptionEvent);
    procedure RegisterOnHelp(OnHelp: THelpEvent);
    procedure UnRegisterOnHelp(OnHelp: THelpEvent);
    procedure RegisterOnHint(OnHint: TNotifyEvent);
    procedure UnRegisterOnHint(OnHint: TNotifyEvent);
    procedure RegisterOnShowHint(OnShowHint: TShowHintEvent);
    procedure UnRegisterOnShowHint(OnShowHint: TShowHintEvent);
    procedure RegisterOnIdle(OnIdle: TIdleEvent);
    procedure UnRegisterOnIdle(OnIdle: TIdleEvent);
    procedure RegisterOnMessage(OnMessage: TMessageEvent);
    procedure UnRegisterOnMessage(OnMessage: TMessageEvent);
  end;

  { TabcScreenEvents }

  { This class allows multiple components to receive notification of
    Screen events }

  TabcScreenEvents = class(TObject)
  private
    FSaveOnActiveControlChange, FSaveOnActiveFormChange: TNotifyEvent;
    FOnActiveControlChangeEvents: TList;
    FOnActiveFormChangeEvents: TList;

    procedure AddTMethod(var List: TList; Method: TMethod);
    procedure RemoveTMethod(var List: TList; Method: TMethod);
    procedure RemoveAllTMethods(var List: TList);

    procedure AppOnActiveControlChange(Sender: TObject);
    procedure AppOnActiveFormChange(Sender: TObject);
  public
    destructor Destroy; override;
    procedure RegisterOnActiveControlChange(OnActiveControlChange: TNotifyEvent);
    procedure UnRegisterOnActiveControlChange(OnActiveControlChange: TNotifyEvent);
    procedure RegisterOnActiveFormChange(OnActiveFormChange: TNotifyEvent);
    procedure UnRegisterOnActiveFormChange(OnActiveFormChange: TNotifyEvent);
  end;


function abcApplicationEvents: TabcApplicationEvents;

function abcScreenEvents: TabcScreenEvents;

var
  abcDesigning: Boolean;

implementation

type
  pMethod = ^TMethod;

var
  GlobalabcApplicationEvents: TabcApplicationEvents;
  GlobalabcScreenEvents: TabcScreenEvents;


{ TabcApplicationEvents }

function abcApplicationEvents: TabcApplicationEvents;
begin
  if GlobalabcApplicationEvents = nil then
    GlobalabcApplicationEvents := TabcApplicationEvents.Create;
  Result := GlobalabcApplicationEvents;
end;

destructor TabcApplicationEvents.Destroy;
begin
  RemoveAllTMethods(FOnActionExecuteEvents);
  RemoveAllTMethods(FOnActionUpdateEvents);
  RemoveAllTMethods(FOnShortCutEvents);
  Application.OnActionExecute := FSaveOnActionExecute;
  Application.OnActionUpdate := FSaveOnActionUpdate;
  Application.OnShortCut := FSaveOnShortCut;
  RemoveAllTMethods(FOnMinimizeEvents);
  RemoveAllTMethods(FOnRestoreEvents);
  Application.OnMinimize := FSaveOnMinimize;
  Application.OnRestore := FSaveOnRestore;
  RemoveAllTMethods(FOnActivateEvents);
  RemoveAllTMethods(FOnDeactivateEvents);
  RemoveAllTMethods(FOnExceptionEvents);
  RemoveAllTMethods(FOnHelpEvents);
  RemoveAllTMethods(FOnHintEvents);
  RemoveAllTMethods(FOnShowHintEvents);
  RemoveAllTMethods(FOnIdleEvents);
  RemoveAllTMethods(FOnMessageEvents);
  Application.OnActivate := FSaveOnActivate;
  Application.OnDeactivate := FSaveOnDeactivate;
  Application.OnException := FSaveOnException;
  Application.OnHelp := FSaveOnHelp;
  Application.OnHint := FSaveOnHint;
  Application.OnShowHint := FSaveOnShowHint;
  Application.OnIdle := FSaveOnIdle;
  Application.OnMessage := FSaveOnMessage;
  inherited Destroy;
  GlobalabcApplicationEvents := nil;
end;

procedure TabcApplicationEvents.AddTMethod(var List: TList; Method: TMethod);
var
  pM: pMethod;
begin
  New(pM);
  pM^ := Method;
  if List = nil then
    List := TList.Create;
  List.Add(pM);
end;

procedure TabcApplicationEvents.RemoveTMethod(var List: TList; Method: TMethod);
var
  pM: pMethod;
  i: integer;
begin
  if abcDesigning then Exit;
  if List <> nil then
    with List do
      for i := Count - 1 downto 0 do
        with pMethod(Items[i])^ do
          if (Code = Method.Code) and (Data = Method.Data) then
          begin
            pM := pMethod(Items[i]);
            Delete(i);
            Dispose(pM);
            Break;
          end;
end;

procedure TabcApplicationEvents.RemoveAllTMethods(var List: TList);
var
  pM: pMethod;
  i: integer;
begin
  if List <> nil then
  begin
    with List do
      for i := Count - 1 downto 0 do
      begin
        pM := pMethod(Items[i]);
        Delete(i);
        Dispose(pM);
      end;
    List.Free;
    List := nil;
  end;
end;

procedure TabcApplicationEvents.AppOnActionExecute(Action: TBasicAction; var Handled: Boolean);
var
  i: integer;
  AnActionEvent: TActionEvent;
begin
  with FOnActionExecuteEvents do
    for i := 0 to Count - 1 do
    begin
      AnActionEvent := TActionEvent(Items[i]^);
      AnActionEvent(Action, Handled);
    end;
end;

procedure TabcApplicationEvents.AppOnActionUpdate(Action: TBasicAction; var Handled: Boolean);
var
  i: integer;
  AnActionEvent: TActionEvent;
begin
  with FOnActionUpdateEvents do
    for i := 0 to Count - 1 do
    begin
      AnActionEvent := TActionEvent(Items[i]^);
      AnActionEvent(Action, Handled);
    end;
end;

procedure TabcApplicationEvents.AppOnShortCut(var Msg: TWMKey; var Handled: Boolean);
var
  i: integer;
  AShortCutEvent: TShortCutEvent;
begin
  with FOnShortCutEvents do
    for i := 0 to Count - 1 do
    begin
      AShortCutEvent := TShortCutEvent(Items[i]^);
      AShortCutEvent(Msg, Handled);
    end;
end;

procedure TabcApplicationEvents.AppOnMinimize(Sender: TObject);
var
  i: integer;
  ANotifyEvent: TNotifyEvent;
begin
  with FOnMinimizeEvents do
    for i := 0 to Count - 1 do
    begin
      ANotifyEvent := TNotifyEvent(Items[i]^);
      ANotifyEvent(Sender);
    end;
end;

procedure TabcApplicationEvents.AppOnRestore(Sender: TObject);
var
  i: integer;
  ANotifyEvent: TNotifyEvent;
begin
  with FOnRestoreEvents do
    for i := 0 to Count - 1 do
    begin
      ANotifyEvent := TNotifyEvent(Items[i]^);
      ANotifyEvent(Sender);
    end;
end;

procedure TabcApplicationEvents.AppOnActivate(Sender: TObject);
var
  i: integer;
  ANotifyEvent: TNotifyEvent;
begin
  with FOnActivateEvents do
    for i := 0 to Count - 1 do
    begin
      ANotifyEvent := TNotifyEvent(Items[i]^);
      ANotifyEvent(Sender);
    end;
end;

procedure TabcApplicationEvents.AppOnDeactivate(Sender: TObject);
var
  i: integer;
  ANotifyEvent: TNotifyEvent;
begin
  with FOnDeactivateEvents do
    for i := 0 to Count - 1 do
    begin
      ANotifyEvent := TNotifyEvent(Items[i]^);
      ANotifyEvent(Sender);
    end;
end;

procedure TabcApplicationEvents.AppOnException(Sender: TObject; E: Exception);
var
  i: integer;
  AnExceptionEvent: TExceptionEvent;
begin
  with FOnExceptionEvents do
    for i := 0 to Count - 1 do
    begin
      AnExceptionEvent := TExceptionEvent(Items[i]^);
      AnExceptionEvent(Sender, E);
    end;
end;

function TabcApplicationEvents.AppOnHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
var
  i: integer;
  AHelpEvent: THelpEvent;
begin
  Result := False;
  with FOnHelpEvents do
    for i := 0 to Count - 1 do
    begin
      AHelpEvent := THelpEvent(Items[i]^);
      Result := Result or AHelpEvent(Command, Data, CallHelp);
    end;
end;

procedure TabcApplicationEvents.AppOnHint(Sender: TObject);
var
  i: integer;
  ANotifyEvent: TNotifyEvent;
begin
  with FOnHintEvents do
    for i := 0 to Count - 1 do
    begin
      ANotifyEvent := TNotifyEvent(Items[i]^);
      ANotifyEvent(Sender);
    end;
end;

procedure TabcApplicationEvents.AppOnShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
var
  i: integer;
  AShowHintEvent: TShowHintEvent;
begin
  with FOnShowHintEvents do
    for i := 0 to Count - 1 do
    begin
      AShowHintEvent := TShowHintEvent(Items[i]^);
      AShowHintEvent(HintStr, CanShow, HintInfo);
    end;
end;

procedure TabcApplicationEvents.AppOnIdle(Sender: TObject; var Done: Boolean);
var
  i: integer;
  AnIdleEvent: TIdleEvent;
begin
  with FOnIdleEvents do
    for i := 0 to Count - 1 do
    begin
      AnIdleEvent := TIdleEvent(Items[i]^);
      AnIdleEvent(Sender, Done);
    end;
end;

procedure TabcApplicationEvents.AppOnMessage(var Msg: TMsg; var Handled: Boolean);
var
  i: integer;
  AMessageEvent: TMessageEvent;
begin
  with FOnMessageEvents do
    for i := 0 to Count - 1 do
    begin
      AMessageEvent := TMessageEvent(Items[i]^);
      AMessageEvent(Msg, Handled);
    end;
end;

procedure TabcApplicationEvents.RegisterOnActionExecute(OnActionExecute: TActionEvent);
begin
  if abcDesigning then Exit;
  if FOnActionExecuteEvents = nil then
  begin
    FSaveOnActionExecute := Application.OnActionExecute;
    if Assigned(FSaveOnActionExecute) then
      AddTMethod(FOnActionExecuteEvents, TMethod(FSaveOnActionExecute));
    Application.OnActionExecute := AppOnActionExecute;
  end;
  AddTMethod(FOnActionExecuteEvents, TMethod(OnActionExecute));
end;

procedure TabcApplicationEvents.UnRegisterOnActionExecute(OnActionExecute: TActionEvent);
begin
  RemoveTMethod(FOnActionExecuteEvents, TMethod(OnActionExecute));
end;

procedure TabcApplicationEvents.RegisterOnActionUpdate(OnActionUpdate: TActionEvent);
begin
  if abcDesigning then Exit;
  if FOnActionUpdateEvents = nil then
  begin
    FSaveOnActionUpdate := Application.OnActionUpdate;
    if Assigned(FSaveOnActionUpdate) then
      AddTMethod(FOnActionUpdateEvents, TMethod(FSaveOnActionUpdate));
    Application.OnActionUpdate := AppOnActionUpdate;
  end;
  AddTMethod(FOnActionUpdateEvents, TMethod(OnActionUpdate));
end;

procedure TabcApplicationEvents.UnRegisterOnActionUpdate(OnActionUpdate: TActionEvent);
begin
  RemoveTMethod(FOnActionUpdateEvents, TMethod(OnActionUpdate));
end;

procedure TabcApplicationEvents.RegisterOnShortCut(OnShortCut: TShortCutEvent);
begin
  if abcDesigning then Exit;
  if FOnShortCutEvents = nil then
  begin
    FSaveOnShortCut := Application.OnShortCut;
    if Assigned(FSaveOnShortCut) then
      AddTMethod(FOnShortCutEvents, TMethod(FSaveOnShortCut));
    Application.OnShortCut := AppOnShortCut;
  end;
  AddTMethod(FOnShortCutEvents, TMethod(OnShortCut));
end;

procedure TabcApplicationEvents.UnRegisterOnShortCut(OnShortCut: TShortCutEvent);
begin
  RemoveTMethod(FOnShortCutEvents, TMethod(OnShortCut));
end;

procedure TabcApplicationEvents.RegisterOnMinimize(OnMinimize: TNotifyEvent);
begin
  if abcDesigning then Exit;
  if FOnMinimizeEvents = nil then
  begin
    FSaveOnMinimize := Application.OnMinimize;
    if Assigned(FSaveOnMinimize) then
      AddTMethod(FOnMinimizeEvents, TMethod(FSaveOnMinimize));
    Application.OnMinimize := AppOnMinimize;
  end;
  AddTMethod(FOnMinimizeEvents, TMethod(OnMinimize));
end;

procedure TabcApplicationEvents.UnRegisterOnMinimize(OnMinimize: TNotifyEvent);
begin
  RemoveTMethod(FOnMinimizeEvents, TMethod(OnMinimize));
end;

procedure TabcApplicationEvents.RegisterOnRestore(OnRestore: TNotifyEvent);
begin
  if abcDesigning then Exit;
  if FOnRestoreEvents = nil then
  begin
    FSaveOnRestore := Application.OnRestore;
    if Assigned(FSaveOnRestore) then
      AddTMethod(FOnRestoreEvents, TMethod(FSaveOnRestore));
    Application.OnRestore := AppOnRestore;
  end;
  AddTMethod(FOnRestoreEvents, TMethod(OnRestore));
end;

procedure TabcApplicationEvents.UnRegisterOnRestore(OnRestore: TNotifyEvent);
begin
  RemoveTMethod(FOnRestoreEvents, TMethod(OnRestore));
end;

procedure TabcApplicationEvents.RegisterOnActivate(OnActivate: TNotifyEvent);
begin
  if abcDesigning then Exit;
  if FOnActivateEvents = nil then
  begin
    FSaveOnActivate := Application.OnActivate;
    if Assigned(FSaveOnActivate) then
      AddTMethod(FOnActivateEvents, TMethod(FSaveOnActivate));
    Application.OnActivate := AppOnActivate;
  end;
  AddTMethod(FOnActivateEvents, TMethod(OnActivate));
end;

procedure TabcApplicationEvents.UnRegisterOnActivate(OnActivate: TNotifyEvent);
begin
  RemoveTMethod(FOnActivateEvents, TMethod(OnActivate));
end;

procedure TabcApplicationEvents.RegisterOnDeactivate(OnDeactivate: TNotifyEvent);
begin
  if abcDesigning then Exit;
  if FOnDeactivateEvents = nil then
  begin
    FSaveOnDeactivate := Application.OnDeactivate;
    if Assigned(FSaveOnDeactivate) then
      AddTMethod(FOnDeactivateEvents, TMethod(FSaveOnDeactivate));
    Application.OnDeactivate := AppOnDeactivate;
  end;
  AddTMethod(FOnDeactivateEvents, TMethod(OnDeactivate));
end;

procedure TabcApplicationEvents.UnRegisterOnDeactivate(OnDeactivate: TNotifyEvent);
begin
  RemoveTMethod(FOnDeactivateEvents, TMethod(OnDeactivate));
end;

procedure TabcApplicationEvents.RegisterOnException(OnException: TExceptionEvent);
begin
  if abcDesigning then Exit;
  if FOnExceptionEvents = nil then
  begin
    FSaveOnException := Application.OnException;
    if Assigned(FSaveOnException) then
      AddTMethod(FOnExceptionEvents, TMethod(FSaveOnException));
    Application.OnException := AppOnException;
  end;
  AddTMethod(FOnExceptionEvents, TMethod(OnException));
end;

procedure TabcApplicationEvents.UnRegisterOnException(OnException: TExceptionEvent);
begin
  RemoveTMethod(FOnExceptionEvents, TMethod(OnException));
end;

procedure TabcApplicationEvents.RegisterOnHelp(OnHelp: THelpEvent);
begin
  if abcDesigning then Exit;
  if FOnHelpEvents = nil then
  begin
    FSaveOnHelp := Application.OnHelp;
    if Assigned(FSaveOnHelp) then
      AddTMethod(FOnHelpEvents, TMethod(FSaveOnHelp));
    Application.OnHelp := AppOnHelp;
  end;
  AddTMethod(FOnHelpEvents, TMethod(OnHelp));
end;

procedure TabcApplicationEvents.UnRegisterOnHelp(OnHelp: THelpEvent);
begin
  RemoveTMethod(FOnHelpEvents, TMethod(OnHelp));
end;

procedure TabcApplicationEvents.RegisterOnHint(OnHint: TNotifyEvent);
begin
  if abcDesigning then Exit;
  if FOnHintEvents = nil then
  begin
    FSaveOnHint := Application.OnHint;
    if Assigned(FSaveOnHint) then
      AddTMethod(FOnHintEvents, TMethod(FSaveOnHint));
    Application.OnHint := AppOnHint;
  end;
  AddTMethod(FOnHintEvents, TMethod(OnHint));
end;

procedure TabcApplicationEvents.UnRegisterOnHint(OnHint: TNotifyEvent);
begin
  RemoveTMethod(FOnHintEvents, TMethod(OnHint));
end;

procedure TabcApplicationEvents.RegisterOnShowHint(OnShowHint: TShowHintEvent);
begin
  if abcDesigning then Exit;
  if FOnShowHintEvents = nil then
  begin
    FSaveOnShowHint := Application.OnShowHint;
    if Assigned(FSaveOnShowHint) then
      AddTMethod(FOnShowHintEvents, TMethod(FSaveOnShowHint));
    Application.OnShowHint := AppOnShowHint;
  end;
  AddTMethod(FOnShowHintEvents, TMethod(OnShowHint));
end;

procedure TabcApplicationEvents.UnRegisterOnShowHint(OnShowHint: TShowHintEvent);
begin
  RemoveTMethod(FOnShowHintEvents, TMethod(OnShowHint));
end;

procedure TabcApplicationEvents.RegisterOnIdle(OnIdle: TIdleEvent);
begin
  if abcDesigning then Exit;
  if FOnIdleEvents = nil then
  begin
    FSaveOnIdle := Application.OnIdle;
    if Assigned(FSaveOnIdle) then
      AddTMethod(FOnIdleEvents, TMethod(FSaveOnIdle));
    Application.OnIdle := AppOnIdle;
  end;
  AddTMethod(FOnIdleEvents, TMethod(OnIdle));
end;

procedure TabcApplicationEvents.UnRegisterOnIdle(OnIdle: TIdleEvent);
begin
  RemoveTMethod(FOnIdleEvents, TMethod(OnIdle));
end;

procedure TabcApplicationEvents.RegisterOnMessage(OnMessage: TMessageEvent);
begin
  if abcDesigning then Exit;
  if FOnMessageEvents = nil then
  begin
    FSaveOnMessage := Application.OnMessage;
    if Assigned(FSaveOnMessage) then
      AddTMethod(FOnMessageEvents, TMethod(FSaveOnMessage));
    Application.OnMessage := AppOnMessage;
  end;
  AddTMethod(FOnMessageEvents, TMethod(OnMessage));
end;

procedure TabcApplicationEvents.UnRegisterOnMessage(OnMessage: TMessageEvent);
begin
  RemoveTMethod(FOnMessageEvents, TMethod(OnMessage));
end;

{ TabcScreenEvents }

function abcScreenEvents: TabcScreenEvents;
begin
  if GlobalabcScreenEvents = nil then
    GlobalabcScreenEvents := TabcScreenEvents.Create;
  Result := GlobalabcScreenEvents;
end;

destructor TabcScreenEvents.Destroy;
begin
  RemoveAllTMethods(FOnActiveControlChangeEvents);
  RemoveAllTMethods(FOnActiveFormChangeEvents);
  Screen.OnActiveControlChange := FSaveOnActiveControlChange;
  Screen.OnActiveFormChange := FSaveOnActiveFormChange;
  inherited Destroy;
  GlobalabcScreenEvents := nil;
end;

procedure TabcScreenEvents.AddTMethod(var List: TList; Method: TMethod);
var
  pM: pMethod;
begin
  New(pM);
  pM^ := Method;
  if List = nil then List := TList.Create;
  List.Add(pM);
end;

procedure TabcScreenEvents.RemoveTMethod(var List: TList; Method: TMethod);
var
  pM: pMethod;
  i: integer;
begin
  if abcDesigning then Exit;
  if List <> nil then
    with List do
      for i := Count - 1 downto 0 do
        with pMethod(Items[i])^ do
          if (Code = Method.Code) and (Data = Method.Data) then
          begin
            pM := pMethod(Items[i]);
            Delete(i);
            Dispose(pM);
            Break;
          end;
end;

procedure TabcScreenEvents.RemoveAllTMethods(var List: TList);
var
  pM: pMethod;
  i: integer;
begin
  if List <> nil then
  begin
    with List do
      for i := Count - 1 downto 0 do
      begin
        pM := pMethod(Items[i]);
        Delete(i);
        Dispose(pM);
      end;
    List.Free;
    List := nil;
  end;
end;

procedure TabcScreenEvents.AppOnActiveControlChange(Sender: TObject);
var
  i: integer;
  ANotifyEvent: TNotifyEvent;
begin
  with FOnActiveControlChangeEvents do
    for i := 0 to Count - 1 do
    begin
      ANotifyEvent := TNotifyEvent(Items[i]^);
      ANotifyEvent(Sender);
    end;
end;

procedure TabcScreenEvents.AppOnActiveFormChange(Sender: TObject);
var
  i: integer;
  ANotifyEvent: TNotifyEvent;
begin
  with FOnActiveFormChangeEvents do
    for i := 0 to Count - 1 do
    begin
      ANotifyEvent := TNotifyEvent(Items[i]^);
      ANotifyEvent(Sender);
    end;
end;

procedure TabcScreenEvents.RegisterOnActiveControlChange(OnActiveControlChange: TNotifyEvent);
begin
  if abcDesigning then Exit;
  if FOnActiveControlChangeEvents = nil then
  begin
    FSaveOnActiveControlChange := Screen.OnActiveControlChange;
    if Assigned(FSaveOnActiveControlChange) then
      AddTMethod(FOnActiveControlChangeEvents, TMethod(FSaveOnActiveControlChange));
    Screen.OnActiveControlChange := AppOnActiveControlChange;
  end;
  AddTMethod(FOnActiveControlChangeEvents, TMethod(OnActiveControlChange));
end;

procedure TabcScreenEvents.UnRegisterOnActiveControlChange(OnActiveControlChange: TNotifyEvent);
begin
  RemoveTMethod(FOnActiveControlChangeEvents, TMethod(OnActiveControlChange));
end;

procedure TabcScreenEvents.RegisterOnActiveFormChange(OnActiveFormChange: TNotifyEvent);
begin
  if abcDesigning then Exit;
  if FOnActiveFormChangeEvents = nil then
  begin
    FSaveOnActiveFormChange := Screen.OnActiveFormChange;
    if Assigned(FSaveOnActiveFormChange) then
      AddTMethod(FOnActiveFormChangeEvents, TMethod(FSaveOnActiveFormChange));
    Screen.OnActiveFormChange := AppOnActiveFormChange;
  end;
  AddTMethod(FOnActiveFormChangeEvents, TMethod(OnActiveFormChange));
end;

procedure TabcScreenEvents.UnRegisterOnActiveFormChange(OnActiveFormChange: TNotifyEvent);
begin
  RemoveTMethod(FOnActiveFormChangeEvents, TMethod(OnActiveFormChange));
end;

initialization

finalization
  GlobalabcApplicationEvents.Free;
  GlobalabcScreenEvents.Free;

end.
