unit abcatlab;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

NOTE:  How to extend this unit to add attached labels to other control classes.

 1. Your component class must inherit from TControl
 2. Add the unit containing your control class to the interface uses clause.
 3. Copy the interface and implementation of TabcLEdit, replacing references to TabcLEdit with your
    new class name.
 4. Modify the implementation of TabcSubLabel.SetAttachedTo to add your new class.

 You may prefer to copy this unit and clone TabcSubLabel into another classname for
 use with your other components.  For instance, DB controls are defined in another unit
 abcdbatl, so that components in this unit can be used without adding the DB unit to the
 project.  A different attached label class was implemented for those controls in abcdbatl.

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Controls, Graphics, Menus, StdCtrls, ExtCtrls,
  Mask, abcspin, Forms, Dialogs, ComCtrls;

type

  { TabcSubLabel }

  TabcSubLabel = class(TCustomLabel)
  private
    FAttachedTo: TControl;
    FEnabledChanging: Boolean;
    FLocked: Boolean;
    FMovingAttached: Boolean;
    FVisibleChanging: Boolean;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure Click; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetName(const NewName: TComponentName); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetAttachedTo(Value: TControl); virtual;
  public
    constructor CreateAttached(AOwner: TComponent; AControl: TControl);
    destructor Destroy; override;
    procedure AttachedMove(dX, dY: Integer);
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property AttachedTo: TControl read FAttachedTo;
  published
    {property Align;   NOT PUBLISHED }
    property Alignment;
    property AutoSize;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    {property Enabled; NOT PUBLISHED, set by attached control}
    {property FocusControl; NOT PUBLISHED, set to attached control if it's a TWinControl}
    property Font;
    property Locked: Boolean read FLocked write FLocked;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    {property Visible; NOT PUBLISHED, set by attached control}
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
{$IFDEF DELPHI6}
    property OnMouseEnter;
    property OnMouseLeave;
{$ENDIF}
  end;

  { TabcLEdit }

  TabcLEdit = class(TEdit)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLMaskEdit }

  TabcLMaskEdit = class(TMaskEdit)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLSpinEdit }

  TabcLSpinEdit = class(TabcSpinEdit)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLMemo }

  TabcLMemo = class(TMemo)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLListBox }

  TabcLListBox = class(TListBox)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLComboBox }

  TabcLComboBox = class(TComboBox)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLImage }

  TabcLImage = class(TImage)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLPanel }

  TabcLPanel = class(TPanel)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLScrollBox }

  TabcLScrollBox = class(TScrollBox)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

  { TabcLTreeView }
  TabcLTreeView = class(TTreeView)
  private
    FSubLabel: TabcSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcSubLabel read FSubLabel;
  end;

implementation

uses abcconst, abcutilf;

const
  CREATELABELX = -100;  {default X offset of label relative to attached control}
  CREATELABELY = 2;     {default Y offset of label relative to attached control}

(* for testing new Delphi Version

procedure DebugComponentState(AComponent: TComponent; ACaption: string);
var
  s: string;
begin
  s := '';
  if csLoading in AComponent.ComponentState then
    s := s + 'csLoading, ';
  if csReading in AComponent.ComponentState then
    s := s + 'csReading, ';
  if csWriting in AComponent.ComponentState then
    s := s + 'csWriting, ';
  if csDestroying in AComponent.ComponentState then
    s := s + 'csDestroying, ';
  if csDesigning in AComponent.ComponentState then
    s := s + 'csDesigning, ';
  if csAncestor in AComponent.ComponentState then
    s := s + 'csAncestor, ';
  if csUpdating in AComponent.ComponentState then
    s := s + 'csUpdating, ';
  if csFixups in AComponent.ComponentState then
    s := s + 'csFixups, ';
  if csFreeNotification in AComponent.ComponentState then
    s := s + 'csFreeNotification, ';
  if csInline in AComponent.ComponentState then
    s := s + 'csInline, ';
  ShowMessage(ACaption + ' ComponentState: ' + #13#10 + s);
end;

*)

{ TabcSubLabel }

constructor TabcSubLabel.CreateAttached(AOwner: TComponent; AControl: TControl);
begin
  inherited Create(AOwner);
  FAttachedTo := AControl;
  if (AControl is TWinControl) then FocusControl := TWinControl(AControl);
  SetBounds(AControl.Left + CREATELABELX,
            AControl.Top + CREATELABELY,
            Width,
            Height);
end;

destructor TabcSubLabel.Destroy;
begin
  if [csDesigning, csAncestor] * ComponentState = [csDesigning] then
    if (FAttachedTo <> nil) and
       ([csDestroying, csLoading] * FAttachedTo.ComponentState = []) then
      if (MessageDlg('This label is attached to ' + FAttachedTo.Name + '.' +
          #13#10 +
          #13#10 +
          'Are you sure you want to permanently delete it?',
          mtWarning,
          [mbYes, mbNo],
          0) = mrNo) then Abort;
  FAttachedTo := nil;
  inherited Destroy;
end;

procedure TabcSubLabel.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = FAttachedTo) and
     not (csDestroying in ComponentState) then
    FAttachedTo := nil;
end;

procedure TabcSubLabel.Loaded;
begin
  inherited Loaded;
  if FAttachedTo <> nil then
  begin
    if Enabled <> FAttachedTo.Enabled then begin
      FEnabledChanging := True;
      Enabled := FAttachedTo.Enabled;
    end;
    if Visible <> FAttachedTo.Visible then begin
      FVisibleChanging := True;
      Visible := FAttachedTo.Visible;
    end;
  end;
end;

procedure TabcSubLabel.SetAttachedTo(Value: TControl);
begin
  if Value <> FAttachedTo then
  begin
    if Value = nil then
      Free
    else
    begin
      if (Value is TabcLEdit) then
        TabcLEdit(Value).SetSubLabel(Self)
      else if (Value is TabcLMaskEdit) then
        TabcLMaskEdit(Value).SetSubLabel(Self)
      else if (Value is TabcLSpinEdit) then
        TabcLSpinEdit(Value).SetSubLabel(Self)
      else if (Value is TabcLMemo) then
        TabcLMemo(Value).SetSubLabel(Self)
      else if (Value is TabcLListBox) then
        TabcLListBox(Value).SetSubLabel(Self)
      else if (Value is TabcLComboBox) then
        TabcLComboBox(Value).SetSubLabel(Self)
      else if (Value is TabcLImage) then
        TabcLImage(Value).SetSubLabel(Self)
      else if (Value is TabcLPanel) then
        TabcLPanel(Value).SetSubLabel(Self)
      else if (Value is TabcLScrollBox) then
        TabcLScrollBox(Value).SetSubLabel(Self)
      else if (Value is TabcLTreeView) then
        TabcLTreeView(Value).SetSubLabel(Self)
      {
      add any new classes here, e.g.
      else if (Value is TNewClass) then
        TNewClass(Value).SetSubLabel(Self)
      }
      else
        Exit;
      FAttachedTo := Value;
      Parent := FAttachedTo.Parent;
      Enabled := FAttachedTo.Enabled;
      Visible := FAttachedTo.Visible;
      if FAttachedTo is TWinControl then
        FocusControl := TWinControl(FAttachedTo);
    end;
  end;
end;

procedure TabcSubLabel.AttachedMove(dX, dY: Integer);
begin
  if (Parent <> nil) and abcDescendsFromClassByName(Parent, 'TabcCustomToolbar') then Exit;
  if [csAncestor, csDesigning, csLoading] * ComponentState = [csAncestor, csDesigning, csLoading] then
    Exit;
  FMovingAttached := True;
  SetBounds(Left + dX, Top + dY, Width, Height);
  FMovingAttached := False;
end;

procedure TabcSubLabel.Click;
begin
  if FAttachedTo is TWinControl then
    TWinControl(FAttachedTo).SetFocus;
  inherited Click;
end;

procedure TabcSubLabel.CMEnabledChanged(var Message: TMessage);
begin
  if FEnabledChanging then begin
    inherited;
    FEnabledChanging := False;
  end
  else begin
    FEnabledChanging := True;
    if FAttachedto <> nil then Enabled := FAttachedTo.Enabled;
  end;
end;

procedure TabcSubLabel.CMVisibleChanged(var Message: TMessage);
begin
  if FVisibleChanging then begin
    inherited;
    FVisibleChanging := False;
  end
  else begin
    FVisibleChanging := True;
    if FAttachedTo <> nil then Visible := FAttachedTo.Visible;
  end;
end;

procedure TabcSubLabel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if FLocked and not FMovingAttached and (FAttachedTo <> nil) then begin
      with FAttachedTo do SetBounds(
        Left + ALeft - Self.Left,
        Top + ATop - Self.Top,
        Width,
        Height);
      inherited SetBounds(Left, Top, AWidth, AHeight);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcSubLabel.SetName(const NewName: TComponentName);
begin
  if [csAncestor, csDesigning] * ComponentState = [csDesigning] then
    if (FAttachedTo <> nil) then
      if (NewName <> FAttachedTo.Name + 'SubLabel') then
        raise Exception.Create('Name cannot be changed');
  inherited SetName(NewName);
end;

procedure TabcSubLabel.SetParent(AParent: TWinControl);
begin
  if (AParent <> nil) and (FAttachedTo <> nil) and
    (AParent <> FAttachedTo.Parent) then
    Abort
  else
    inherited;
end;

{ TabcLEdit }

constructor TabcLEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLEdit.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLEdit.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLEdit.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLEdit.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLEdit.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLEdit.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLEdit.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLEdit.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLEdit.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLEdit.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLEdit.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then
      FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLMaskEdit }

constructor TabcLMaskEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLMaskEdit.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLMaskEdit.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLMaskEdit.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLMaskEdit.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLMaskEdit.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLMaskEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLMaskEdit.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLMaskEdit.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLMaskEdit.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLMaskEdit.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLMaskEdit.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLMaskEdit.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLSpinEdit }

constructor TabcLSpinEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLSpinEdit.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLSpinEdit.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLSpinEdit.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLSpinEdit.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLSpinEdit.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLSpinEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLSpinEdit.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLSpinEdit.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLSpinEdit.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLSpinEdit.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLSpinEdit.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLSpinEdit.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLMemo }

constructor TabcLMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLMemo.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLMemo.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLMemo.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLMemo.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLMemo.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLMemo.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLMemo.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLMemo.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLMemo.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLMemo.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLMemo.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLMemo.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLListBox }

constructor TabcLListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLListBox.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLListBox.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLListBox.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLListBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLListBox.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLListBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLListBox.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLListBox.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLListBox.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLListBox.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLListBox.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLListBox.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLComboBox }

constructor TabcLComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLComboBox.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLComboBox.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLComboBox.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLComboBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLComboBox.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLComboBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLComboBox.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLComboBox.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLComboBox.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLComboBox.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLComboBox.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLComboBox.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLImage }

constructor TabcLImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLImage.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLImage.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLImage.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLImage.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLImage.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLImage.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLImage.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLImage.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLImage.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLImage.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLImage.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLImage.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLPanel }

constructor TabcLPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLPanel.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLPanel.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLPanel.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLPanel.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLPanel.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLPanel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLPanel.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLPanel.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLPanel.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLPanel.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLPanel.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLPanel.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLScrollBox }

constructor TabcLScrollBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLScrollBox.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLScrollBox.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLScrollBox.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLScrollBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLScrollBox.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLScrollBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLScrollBox.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLScrollBox.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLScrollBox.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLScrollBox.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLScrollBox.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLScrollBox.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

{ TabcLTreeView }
constructor TabcLTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLTreeView.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLTreeView.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLTreeView.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLTreeView.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLTreeView.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLTreeView.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLTreeView.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLTreeView.SetSubLabel(Value: TabcSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLTreeView.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLTreeView.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLTreeView.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLTreeView.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcSubLabel) then
    if (FindLabel is TabcSubLabel) then
    begin
      if (TabcSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcSubLabel(FindLabel).FAttachedTo = self) then
        TabcSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

initialization
{$IFDEF DELPHI6}
  ActivateClassGroup(TControl);
{$ENDIF}
  RegisterClass(TabcSubLabel);

finalization
  UnRegisterClass(TabcSubLabel);

end.
