unit abcbar;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls, Forms, Dialogs,
  abcbtn, abcpanel, abcutilg, abcanfr, abcinpla, abcmsg, abcbarlv
  ,ImgList, ActnList;

type

{$IFNDEF DELPHI5}
  TImageIndex = type Integer;
{$ENDIF}

  TabcCustomButtonBar = class;

  { TabcButtonPageActionLink }

  TabcButtonPage = class;

  TabcButtonPageActionLink = class(TControlActionLink)
  protected
    FClient: TabcButtonPage;
    procedure AssignClient (AClient: TObject); override;
    function IsImageIndexLinked: Boolean; override;
    procedure SetImageIndex (Value: Integer); override;
  public
    destructor Destroy; override;
  end;

  TabcButtonPageActionLinkClass = class of TabcButtonPageActionLink;

  TabcControlRenameEvent = procedure(Sender: TObject; var NewCaption: string) of object;

  TabcButtonPage = class(TWinControl)
  private
    FButtonBar: TabcCustomButtonBar;
    FButtonEnabled: Boolean;
    FButtonVisible: Boolean;
    FButtonShowing: Boolean;
    FButtonPicture: TPicture;
    FImageIndex: TImageIndex;
    FOnRename: TabcControlRenameEvent;
    function GetPageIndex: Integer;
    function IsImageIndexStored: Boolean;
    procedure PictureChanged(Sender: TObject);
    procedure SetButtonBar(AButtonBar: TabcCustomButtonBar);
    procedure SetButtonEnabled(const Value: Boolean);
    procedure SetButtonPicture(Value: TPicture);
    procedure SetButtonShowing(Value: Boolean);
    procedure SetButtonVisible(Value: Boolean);
    procedure SetImageIndex (Value: TImageIndex);
    procedure SetPageIndex(Value: Integer);
    procedure UpdateButtonShowing;
    procedure AMSelectedChanged(var Message: TMessage); message AM_SELECTEDCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    procedure ActionChange (Sender: TObject; CheckDefaults: Boolean); override;
    function GetActionLinkClass: TControlActionLinkClass; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure ReadState(Reader: TReader); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure RequestAlign; override;
{$IFDEF DELPHI6}
    procedure SetParent(AParent: TWinControl); override;
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Activate: Boolean;
    function IsActive: Boolean;
    procedure Move(Down: Boolean);
    property ButtonBar: TabcCustomButtonBar read FButtonBar write SetButtonBar;
  published
    property Action;
    property ButtonEnabled: Boolean read FButtonEnabled write SetButtonEnabled default True;
    property ButtonPicture: TPicture read FButtonPicture write SetButtonPicture;
    property ButtonVisible: Boolean read FButtonVisible write SetButtonVisible default True;
    property Caption;
    property Enabled;
    property Font;
    property Height stored False;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex
      stored IsImageIndexStored default -1;
    property Left stored False;
    property PageIndex: Integer read GetPageIndex write SetPageIndex stored False;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Top stored False;
    property Visible stored False;
    property Width stored False;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
    property OnDragDrop;
    property OnDragOver;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnRename: TabcControlRenameEvent read FOnRename write FOnRename;
  end;

  { TabcCustomButtonBar }

  TabcBarButtonAlign = (abbCenter, abbLeft);

  TabcButtonPageChangingEvent = procedure(Sender: TObject; Page: TabcButtonPage;
    var AllowChange: Boolean) of object;

  TabcButtonBarOption =
    (abbAnimate, abbHotTrack, abbDragTrack, abbInPlaceEdit, abbButtonDown, abbShowFocus);
  TabcButtonBarOptions = set of TabcButtonBarOption;

  TabcCustomButtonBar = class(TabcCustomEdgePanel)
  private
    FPages: TList;
    FNewDockPage: TabcButtonPage;
    FUnDockPage: TabcButtonPage;
    FActiveFontColor: TColor;
    FActivePage: TabcButtonPage;
    FActivePageChanging: Boolean;
    FAnimationSteps: Byte;
    FAnimationTime: Word;
    FButtons: TList;
    FButtonAlign: TabcBarButtonAlign;
    FButtonColor: TColor;
    FButtonCursor: TCursor;
    FButtonHeight: Integer;
    FButtonHighlight: TabcGlyphHighlight;
    FButtonStyle: TabcButtonStyle;
    FDragTracking: Boolean;
    FEditing: Boolean;
    FImages: TCustomImageList;
    FOptions: TabcButtonBarOptions;
    FOnChange: TNotifyEvent;
    FOnChanging: TabcButtonPageChangingEvent;
    FSaveAnchorSize: TPoint;
    procedure SetActiveFontColor(Value: TColor);
    procedure AnimatePageChange(FromPage, ToPage: TabcButtonPage);
    procedure KeepButtonDown(Value: Boolean);
    function GetActivePageIndex:Integer;
    procedure SetActivePageIndex(Value: Integer);
    function GetInnerRect: TRect;
    function GetPage(Index: Integer): TabcButtonPage;
    function GetPageCount: Integer;
    procedure InsertPage(Page: TabcButtonPage);
    procedure EnableButton(Page: TabcButtonPage; ButtonEnabled: Boolean);
    procedure ShowButtonFocus(Page: TabcButtonPage; Focused: Boolean);
    procedure ShowButton(Page: TabcButtonPage);
    procedure HideButton(Page: TabcButtonPage);
    procedure MovePage(CurIndex, NewIndex: Integer);
    procedure MoveButton(CurIndex, NewIndex: Integer);
    procedure RemovePage(Page: TabcButtonPage);
    procedure SetActivePage(Page: TabcButtonPage);
    procedure ShowPage(Page: TabcButtonPage);
    procedure SetAnimationSteps(Value: Byte);
    procedure ClearButtonAction(Page: TabcButtonPage);
    procedure UpdateButton(Page: TabcButtonPage);
    procedure ActiveButtonHighlight(Page: TabcButtonPage);
    procedure SetButtonAlign(Value: TabcBarButtonAlign);
    procedure SetButtonColor(Value: TColor);
    procedure SetButtonCursor(Value: TCursor);
    procedure SetButtonHeight(Value: Integer);
    procedure SetButtonHighlight(Value: TabcGlyphHighlight);
    procedure SetButtonStyle(Value: TabcButtonStyle);
    procedure SetImages (Value: TCustomImageList);
    procedure SetOptions(Value: TabcButtonBarOptions);
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMDockClient(var Message: TCMDockClient); message CM_DOCKCLIENT;
    procedure CMDockNotification(var Message: TCMDockNotification); message CM_DOCKNOTIFICATION;
    procedure CMUnDockClient(var Message: TCMUnDockClient); message CM_UNDOCKCLIENT;
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
  protected
    procedure DoAddDockClient(Client: TControl; const ARect: TRect); override;
    procedure DockOver(Source: TDragDockObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean); override;
    procedure DoRemoveDockClient(Client: TControl); override;
    function GetPageFromDockClient(Client: TControl): TabcButtonPage;
    procedure GetSiteInfo(Client: TControl; var InfluenceRect: TRect;
      MousePos: TPoint; var CanDock: Boolean); override;
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoEnter; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function CanChange(Page: TabcButtonPage): Boolean; dynamic;
    procedure Change; dynamic;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    procedure ShowControl(AControl: TControl); override;
    {internal}
    function AllButtonsHidden: Boolean;
    property InnerRect: TRect read GetInnerRect;  {client rect minus bevels etc.}

    {for published access in descendent}
    property ActiveFontColor: TColor read FActiveFontColor write SetActiveFontColor
      default clNone;
    property ActivePage: TabcButtonPage read FActivePage write SetActivePage;
    property AnimationSteps: Byte read FAnimationSteps write SetAnimationSteps default 8;
    property AnimationTime: Word read FAnimationTime write FAnimationTime default 128;
    property BevelInner default bvNone;
    property BevelOuter default bvLowered;
    property BorderStyle default bsSingle;
    property Color default clBtnShadow;
    property Images: TCustomImageList read FImages write SetImages;
    property Options: TabcButtonBarOptions read FOptions write SetOptions
      default [abbAnimate, abbHotTrack, abbDragTrack, abbInPlaceEdit];
    property ParentColor default False;
    property ButtonAlign: TabcBarbuttonAlign read FButtonAlign write SetButtonAlign default abbCenter;
    property ButtonColor: TColor read FButtonColor write SetButtonColor default clBtnFace;
    property ButtonCursor: TCursor read FButtonCursor write SetButtonCursor default crHandPoint;
    property ButtonHeight: Integer read FButtonHeight write SetButtonHeight default 21;
    property ButtonHighlight: TabcGlyphHighlight read FButtonHighlight write SetButtonHighlight default aghNormal;
    property ButtonStyle: TabcButtonStyle read FButtonStyle write SetButtonStyle default absMidHighlight;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnChanging: TabcButtonPageChangingEvent read FOnChanging write FOnChanging;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Edit; dynamic;
    function AddPage(ACaption: string; DoEdit, AppendPage: Boolean): TabcButtonPage;
    function AddPageWithList(ACaption: string; DoEdit, AppendPage: Boolean): TabcButtonList;
    function RemoveActivePage(Confirm: Boolean): Boolean;
    function FindNextPage(CurPage: TabcButtonPage;
      GoForward, CheckButtonVisible: Boolean): TabcButtonPage;
    procedure SelectNextPage(GoForward: Boolean);
    property ActivePageIndex: Integer read GetActivePageIndex write SetActivePageIndex;
    property Editing: Boolean read FEditing;
    property PageCount: Integer read GetPageCount;
    property Pages[Index: Integer]: TabcButtonPage read GetPage;
    property TabStop default True;
  end;

  TabcButtonBar = class(TabcCustomButtonBar)
  public
    procedure SetButtonTransparentColor(Value: TColor);
  published
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property DockSite;
    property ParentBiDiMode;
    property ActiveFontColor;
    property ActivePage;
    property Align;
    property AnimationSteps;
    property AnimationTime;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property ButtonAlign;
    property ButtonColor;
    property ButtonCursor;
    property ButtonHeight;
    property ButtonHighlight;
    property ButtonStyle;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Images;
    property Options;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnChanging;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnStartDrag;
    property OnDockDrop;
    property OnDockOver;
    property OnEndDock;
    property OnGetSiteInfo;
    property OnStartDock;
    property OnUnDock;
  end;

implementation

uses abcutilf, abcconst, Math, Consts, ComStrs;

type
  TControlAccess = class(TControl);


{ TabcButtonPage }

constructor TabcButtonPage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alClient;
  ControlStyle := ControlStyle + [csAcceptsControls, csNoDesignVisible];
  ParentColor := True;
  ParentFont := True;
  ParentShowHint := True;
  FButtonEnabled := True;
  FButtonVisible := True;
  FButtonPicture := TPicture.Create;
  FButtonPicture.OnChange := PictureChanged;
  FImageIndex := -1;
end;

destructor TabcButtonPage.Destroy;
begin
  if FButtonBar <> nil then FButtonBar.RemovePage(Self);
  FButtonPicture.Free;
  inherited Destroy;
end;

procedure TabcButtonPage.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params.WindowClass do
    style := style and not (CS_HREDRAW or CS_VREDRAW);
end;

procedure TabcButtonPage.AMSelectedChanged(var Message: TMessage);
begin
  Broadcast(Message);
end;

function TabcButtonPage.GetPageIndex: Integer;
begin
  if FButtonBar <> nil then
    Result := FButtonBar.FPages.IndexOf(Self)
  else
    Result := -1;
end;

function TabcButtonPage.IsActive: Boolean;
begin
  Result := (ButtonBar <> nil) and (ButtonBar.ActivePage = self);
end;

function TabcButtonPage.Activate: Boolean;
begin
  if (ButtonBar <> nil) then
    ButtonBar.ActivePage := self;
  Result := IsActive;
end;

procedure TabcButtonPage.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  if Reader.Parent is TabcCustomButtonBar then
    ButtonBar := TabcCustomButtonBar(Reader.Parent);
end;

{$IFDEF DELPHI6}
procedure TabcButtonPage.SetParent(AParent: TWinControl);
begin
  if (AParent <> nil) and not (AParent is TabcCustomButtonBar) then
    Abort
  else
    inherited;
end;
{$ENDIF}

procedure TabcButtonPage.Move(Down: Boolean);
var
  newIndex: integer;
begin
  if (ButtonBar <> nil) then
  begin
    newIndex := PageIndex;
    if Down then
    begin
      newIndex := newIndex + 1;
      if newIndex < ButtonBar.PageCount then
        PageIndex := newIndex;
    end
    else
    begin
      newIndex := newIndex - 1;
      if newIndex >= 0 then
        PageIndex := newIndex;
    end;
  end;
end;

procedure TabcButtonPage.RequestAlign;
begin
end;

procedure TabcButtonPage.SetButtonBar(AButtonBar: TabcCustomButtonBar);
begin
  if FButtonBar <> AButtonBar then
  begin
    if FButtonBar <> nil then FButtonBar.RemovePage(Self);
    Parent := AButtonBar;
    if AButtonBar <> nil then AButtonBar.InsertPage(Self);
  end;
end;

procedure TabcButtonPage.SetPageIndex(Value: Integer);
var
  MaxPageIndex: Integer;
begin
  if FButtonBar <> nil then
  begin
    MaxPageIndex := FButtonBar.FPages.Count - 1;
    if (Value > MaxPageIndex) or (Value < 0) then
      raise EListError.CreateFmt(SPageIndexError, [Value, MaxPageIndex]);
    FButtonBar.MovePage(PageIndex, Value);
  end;
end;

procedure TabcButtonPage.SetButtonPicture(Value: TPicture);
begin
  if Value = nil then
    FButtonPicture.Graphic := nil
  else
    FButtonPicture.Graphic := Value.Graphic;
end;

procedure TabcButtonPage.PictureChanged(sender: TObject);
begin
  if FButtonBar <> nil then
    FButtonBar.UpdateButton(Self);
end;

procedure TabcButtonPage.SetButtonShowing(Value: Boolean);
begin
  if PageIndex <> -1 then
    if Value then
    begin
      FButtonShowing := True;
      FButtonBar.ShowButton(Self);
    end else
    begin
      FButtonBar.HideButton(Self);
      FButtonShowing := False;
    end;
end;

procedure TabcButtonPage.DoEnter;
begin
  if PageIndex <> -1 then
    FButtonBar.ShowButtonFocus(Self, True);
  if (ControlCount > 0) and (Controls[0] is TabcCustomButtonList) then
    TabcCustomButtonList(Controls[0]).SetFocus;
  inherited;
end;

procedure TabcButtonPage.DoExit;
begin
  if PageIndex <> -1 then
    FButtonBar.ShowButtonFocus(Self, False);
  inherited;
end;

procedure TabcButtonPage.SetButtonEnabled(const Value: Boolean);
begin
  if FButtonEnabled <> Value then
  begin
    FButtonEnabled := Value;
    if FButtonBar <> nil then
      FButtonBar.EnableButton(Self, Value);
  end;
end;

procedure TabcButtonPage.SetButtonVisible(Value: Boolean);
begin
  if FButtonVisible <> Value then
  begin
    FButtonVisible := Value;
    UpdateButtonShowing;
  end;
end;

procedure TabcButtonPage.SetImageIndex (Value: TImageIndex);
begin
  if Value <> FImageIndex then
  begin
    FImageIndex := Value;
    if not (csLoading in ComponentState) and
       (FButtonBar <> nil) and Assigned(FButtonBar.Images) then
      PictureChanged(nil);
  end;
end;

procedure TabcButtonPage.UpdateButtonShowing;
begin
  SetButtonShowing((FButtonBar <> nil) and FButtonVisible);
end;

procedure TabcButtonPage.CMTextChanged(var Message: TMessage);
begin
  if FButtonBar <> nil then
    FButtonBar.UpdateButton(Self);
end;

function TabcButtonPage.IsImageIndexStored: Boolean;
begin
  Result := (ActionLink = nil) or not TabcButtonPageActionLink(ActionLink).IsImageIndexLinked;
end;

procedure TabcButtonPage.ActionChange (Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
    begin
      if not CheckDefaults or (Self.ImageIndex = -1) then
        Self.ImageIndex := ImageIndex;
    end;
  PictureChanged(nil);
end;

function TabcButtonPage.GetActionLinkClass: TControlActionLinkClass;
begin
  Result := TabcButtonPageActionLink;
end;

{ TabcButtonPageActionLink }

destructor TabcButtonPageActionLink.Destroy;
begin
  if (FClient <> nil) and not (csDestroying in FClient.ComponentState) then
    if FClient.FButtonBar <> nil then
      FClient.FButtonBar.ClearButtonAction(FClient);
  inherited;
end;

procedure TabcButtonPageActionLink.AssignClient (AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TabcButtonPage;
end;

function TabcButtonPageActionLink.IsImageIndexLinked: Boolean;
begin
  Result := inherited IsImageIndexLinked and
    (FClient.ImageIndex = (Action as TCustomAction).ImageIndex);
end;

procedure TabcButtonPageActionLink.SetImageIndex (Value: Integer);
begin
  if IsImageIndexLinked then FClient.ImageIndex := Value;
end;

{ TabcCustomButtonBar }

type
  TabcBarBtn = class(TabcPicSpeedBtn)
  private
    FPage: TabcButtonPage;
    FEditing: Boolean;
    procedure CMDesignHitTest(var Msg: TCMDesignHitTest); message CM_DESIGNHITTEST;
    function GetDockClient: TControl;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
  protected
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure MouseDown(Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer); override;
    procedure MouseEnter; override;
  public
    constructor CreateOnBar(AOwner: TabcCustomButtonBar; APage: TabcButtonPage);
    procedure Click; override;
    procedure Edit;
    procedure SetButtonAlign(Value: TabcBarButtonAlign);
    property Page: TabcButtonPage read FPage;
  end;

constructor TabcBarBtn.CreateOnBar(AOwner: TabcCustomButtonBar; APage: TabcButtonPage);
begin
  Create(AOwner);
  FPage := APage;
  Popup := False;
  Caption := APage.Caption;
  Enabled := APage.ButtonEnabled;
  Hint := APage.Hint;
  Images := AOwner.Images;
  ImageIndex := APage.ImageIndex;
  Action := APage.Action;
  Color := AOwner.ButtonColor;
  Cursor := AOwner.ButtonCursor;
  Height := AOwner.ButtonHeight;
  PictureHighlight := AOwner.ButtonHighlight;
  Style := AOwner.ButtonStyle;
  SetButtonAlign(AOwner.ButtonAlign);
  Parent := AOwner;
end;

procedure TabcBarBtn.CMDesignHitTest(var Msg: TCMDesignHitTest);
begin
  Msg.Result := Longint(BOOL(Page <> Page.ButtonBar.ActivePage));
end;

procedure TabcBarBtn.Click;
begin
  if Page.ButtonBar.Editing then Exit;
  Page.ButtonBar.ActivePage := Page;
  if Down and not (abbButtonDown in Page.ButtonBar.Options) then
    Down := False;
  inherited Click;
end;

procedure TabcBarBtn.MouseEnter;
begin
  if Page.ButtonBar.Editing then Exit;
  inherited MouseEnter;
  if abbHotTrack in Page.ButtonBar.Options then
    Click;
end;

procedure TabcBarBtn.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Page.ButtonBar.Editing then Exit;
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbRight) and (abbInPlaceEdit in Page.ButtonBar.Options) then
    Edit;
end;

procedure TabcBarBtn.DragOver(Source: TObject; X, Y: Integer; State: TDragState;
  var Accept: Boolean);
begin
  inherited DragOver(Source, X, Y, State, Accept);
  if Source is TabcCustomButtonList then
    with Page.ButtonBar do
    try
      FDragTracking := True;
      ActivePage := Page;
    finally
      FDragTracking := False;
    end;
end;

procedure TabcBarBtn.SetButtonAlign(Value: TabcBarButtonAlign);
begin
  case Value of
    abbCenter:
      Margin := -1;
    abbLeft:
      Margin := 5;
  end;
end;

function TabcBarBtn.GetDockClient: TControl;
begin
  Result := nil;
  if Page.ButtonBar.DockSite and (Page.ControlCount > 0) then
  begin
    Result := Page.Controls[0];
    if Result.HostDockSite <> Page.ButtonBar then
      Result := nil;
  end;
end;

procedure TabcBarBtn.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  DockCtl: TControl;
begin
  inherited;
  if (Shift = [ssLeft]) then
  begin
    DockCtl := GetDockClient;
    if DockCtl <> nil then
      DockCtl.BeginDrag(False)
    else if Page.DragMode = dmAutomatic then
      Page.BeginDrag(False);
  end;
end;

procedure TabcBarBtn.WMLButtonDblClk(var Message: TWMLButtonDblClk);
var
  DockCtl: TControl;
begin
  inherited;
  DockCtl := GetDockClient;
  if DockCtl <> nil then
    DockCtl.ManualDock(nil, nil, alNone)
  else if (Page.DragMode = dmAutomatic) and (Page.DragKind = dkDock) then
    Page.ManualDock(nil, nil, alNone);

end;

procedure TabcBarBtn.Edit;
var
  sEdit: string;
  EditFont: TFont;
  EditRect: TRect;
  barWasEditing: Boolean;
begin
  if (not Enabled) or (not Visible) or
     FEditing  or (csDesigning in ComponentState) then Exit;
  barWasEditing := Page.ButtonBar.Editing;
  Page.ButtonBar.FEditing := True;
  FEditing := True;
  try
    sEdit := Caption;
    EditFont := TFont.Create;
    try
      EditFont.Assign(Font);
      EditFont.Color := clBlack;
      EditRect := ClientRect;
      if abcEditInPlace(
        Self,
        EditRect,
        sEdit,
        False,
        bsSingle,
        EditFont,
        clWindow,
        taCenter) then
      begin
        if Assigned(Page.FOnRename) then
          Page.OnRename(Page, sEdit);
        Page.Caption := sEdit;
      end;
    finally
      EditFont.Free;
    end;
  finally
    FEditing := False;
    if not barWasEditing then
      Page.ButtonBar.FEditing := False;
  end;
end;

procedure TabcCustomButtonBar.Edit;
begin
  if ActivePageIndex <> -1 then
  try
    FEditing := True;
    TabcBarBtn(FButtons[ActivePageIndex]).Edit;
  finally
    FEditing := False;
  end;
end;

{ TabcCustomButtonBar }

constructor TabcCustomButtonBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TabStop := True;
  {tip: Removing csAcceptsControls can cause awful designer painting bugs - fix is in CreateParams}
  ControlStyle := [csCaptureMouse, csClickEvents, csOpaque, csDoubleClicks, csReplicatable];
  FButtons := TList.Create;
  FPages := TList.Create;
  BevelOuter := bvLowered;
  BevelInner := bvNone;
  BevelOuter := bvNone;
  BorderStyle := bsSingle;
  FButtonHeight := 21;
  FButtonStyle := absMidHighlight;
  FButtonCursor := crHandPoint;
  FButtonColor := clBtnFace;
  ParentColor := False;
  Color := clBtnShadow;
  FAnimationSteps := 8;
  FAnimationTime := 128;
  FOptions := [abbAnimate, abbHotTrack, abbDragTrack, abbInPlaceEdit];
  FActiveFontColor := clNone;
  SetBounds(Left, Top, 160, 300);
end;

destructor TabcCustomButtonBar.Destroy;
var
  I: Integer;
begin
  for I := 0 to FPages.Count - 1 do
    TabcButtonPage(FPages[I]).FButtonBar := nil;
  FPages.Free;
  FButtons.Free;
  inherited Destroy;
end;

procedure TabcCustomButtonBar.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or WS_CLIPCHILDREN;
    ExStyle := ExStyle or WS_EX_CONTROLPARENT;
  end;
end;

function TabcCustomButtonBar.AddPage(ACaption: string; DoEdit, AppendPage: Boolean): TabcButtonPage;
begin
  Result := TabcButtonPage.Create(Owner);
  with Result do
  begin
    Caption := ACaption;
    ButtonBar := Self;
    if AppendPage then
      PageIndex := PageCount - 1
    else
      PageIndex := ActivePageIndex + 1;
  end;
  if DoEdit then
  begin
    ActivePageIndex := Result.PageIndex;
    Edit;
  end;
end;

function TabcCustomButtonBar.AddPageWithList(ACaption: string; DoEdit, AppendPage: Boolean): TabcButtonList;
var
  NewPage: TabcButtonPage;
begin
  NewPage := AddPage(ACaption, False, AppendPage);
  Result := TabcButtonList.Create(Owner);
  Result.Parent := NewPage;
  if DoEdit then
  begin
    ActivePageIndex := NewPage.PageIndex;
    Edit;
  end;
end;

function TabcCustomButtonBar.RemoveActivePage(Confirm: Boolean): Boolean;
var
  i: integer;
begin
  Result := False;
  if ActivePage <> nil then
    if not Confirm or
       (MessageDlg(
          Format(sabcConfirmRemoveButtonBarGroup, [ActivePage.Caption]),
          mtWarning, [mbYes, mbNo], 0)
          = mrYes) then
    with ActivePage do
    begin
      for i := ControlCount - 1 downto 0 do
        Controls[i].Free;
      Free;
      Result := True;
    end;
end;

procedure TabcCustomButtonBar.DoEnter;
begin
  inherited;
  if (ActivePage <> nil) and ActivePage.CanFocus then
    ActivePage.SetFocus;
end;

procedure TabcCustomButtonBar.Loaded;
var
  i: integer;
begin
  for i := 0 to PageCount - 1 do
    Pages[i].UpdateButtonShowing;
  inherited Loaded;
  if (csDesigning in ComponentState) then
    ReAlign;
end;

procedure TabcCustomButtonBar.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FImages) then
    Images := nil;
end;

function TabcCustomButtonBar.CanChange(Page: TabcButtonPage): Boolean;
begin
  Result := True;
  if Assigned(FOnChanging) and not (csLoading in ComponentState) then
    FOnChanging(Self, Page, Result);
end;

procedure TabcCustomButtonBar.Change;
var
  Form: TCustomForm;
begin
  if csDesigning in ComponentState then
  begin
    Form := GetParentForm(Self);
    if (Form <> nil) and
       (Form.Designer <> nil) and
       not (csLoading in ComponentState)
     then
       Form.Designer.Modified;
  end;
  if Assigned(FOnChange) and not (csLoading in ComponentState) then
    FOnChange(Self);
end;

function TabcCustomButtonBar.GetInnerRect: TRect;
var
  bevelSize: Integer;
begin
  Result := ClientRect;
  bevelSize := BorderWidth;
  if BevelOuter <> bvNone then
    bevelSize := bevelSize + BevelWidth;
  if BevelInner <> bvNone then
    bevelSize := bevelSize + BevelWidth;
  with Result do
  begin
    if beLeft in BevelEdges then
      Left := Left + bevelSize;
    if beTop in BevelEdges then
      Top := Top + bevelSize;
    if beRight in BevelEdges then
      Right := Right - bevelSize;
    if beBottom in BevelEdges then
      Bottom := Bottom - bevelSize;
  end;
end;

procedure TabcCustomButtonBar.AlignControls(AControl: TControl;
  var Rect: TRect);
var
  i, activeIndex: integer;
  alignRect: TRect;
begin
  if (PageCount = 0) or (ActivePage = nil) or (csLoading in ComponentState) or not Showing then Exit;

  alignRect := InnerRect;
  activeIndex := ActivePage.PageIndex;

  for i := 0 to activeIndex do
    with TabcBarBtn(FButtons[i]) do
      if Visible then
        with alignRect do
        begin
          SetBounds(Left, Top, Right - Left, FButtonHeight);
          Top := Top + FButtonHeight;
        end
      else if (csDesigning in Self.ComponentState) then
        with alignRect do SetBounds(Left, - FButtonHeight, Right - Left, FButtonHeight);

  for i := PageCount - 1 downto activeIndex + 1 do
    with TabcBarBtn(FButtons[i]) do
      if Visible then
        with alignRect do
        begin
          SetBounds(Left, Bottom - FButtonHeight, Right - Left, FButtonHeight);
          Bottom := Bottom - FButtonHeight;
        end
      else if (csDesigning in Self.ComponentState) then
        with alignRect do SetBounds(Left, - FButtonHeight, Right - Left, FButtonHeight);

  if not IsRectEmpty(alignRect) then
    with alignRect do
      ActivePage.SetBounds(Left, Top, Right - Left, Bottom - Top);

end;

procedure TabcCustomButtonBar.AnimatePageChange(FromPage, ToPage: TabcButtonPage);
var
  i, j, iAnimateSteps: integer;
  FromIndex, ToIndex: integer;
  alignRect, oldPageRect, newPageRect: TRect;
  pageHeight, stepHeight, remainder: integer;
  NextTime, StepTime: DWORD;
begin
  FromIndex := FPages.IndexOf(FromPage);
  ToIndex := FPages.IndexOf(ToPage);

  if (FromIndex = ToIndex) or (PageCount = 0) then Exit;
  if (FromIndex = -1) or (ToIndex = -1) then
  begin
    ReAlign;
    Exit;
  end;

  alignRect := InnerRect;
  {NB assumes all pages visible}
  pageHeight := (alignRect.Bottom - alignRect.Top) - (PageCount * FButtonHeight);
  if pageHeight <= 0 then Exit;

  if (abbAnimate in Options) and Showing then
  begin
    iAnimateSteps := AnimationSteps;
    StepTime := AnimationTime div iAnimateSteps;
  end
  else
  begin
    iAnimateSteps := 1;
    StepTime := 0;
  end;

  NextTime := GetTickCount;

  stepHeight := pageHeight div iAnimateSteps;
  remainder := pageHeight - stepHeight * iAnimateSteps;

  oldPageRect := FromPage.BoundsRect;
  with oldPageRect do
  begin
    Right := Right - Left;
    Bottom := Bottom - Top;
  end;
  with alignRect do
    if ToIndex < FromIndex then  {move buttons down}
      newPageRect := Classes.Rect(Left, Top + (ToIndex+1)*FButtonHeight, Right - Left, 0)
    else
      newPageRect := Classes.Rect(Left, Bottom - (PageCount-ToIndex-1)*FButtonHeight, Right - Left, 0);

  DisableAlign;
  try
    for i := 1 to iAnimateSteps do
    begin
      NextTime := NextTime + StepTime;
      if i = iAnimateSteps then
        stepHeight := stepHeight + remainder;
      if ToIndex < FromIndex then  {move buttons down}
      begin
        with oldPageRect do
        begin
          Top := Top + StepHeight;
          Bottom := Bottom - StepHeight;
          FromPage.SetBounds(Left, Top, Right, Bottom);
        end;
        for j := FromIndex downto ToIndex + 1 do
          with TabcBarBtn(FButtons[j]) do
            if Visible then
              Top := Top + stepHeight;
        with newPageRect do
        begin
          Bottom := Bottom + stepHeight;
          ToPage.SetBounds(Left, Top, Right, Bottom);
        end;
      end
      else  {move buttons up}
      begin
        with FromPage do Height := Height - StepHeight;
        for j := FromIndex+1 to ToIndex do
          with TabcBarBtn(FButtons[j]) do
            if Visible then
              Top := Top - stepHeight;
        with newPageRect do
        begin
          Top := Top - stepHeight;
          Bottom := Bottom + stepHeight;
          ToPage.SetBounds(Left, Top, Right, Bottom);
        end;
      end;
      if (abbAnimate in Options) and Showing and not FDragTracking then
        repeat
          Application.ProcessMessages;
        until GetTickCount >= NextTime;
    end;
    FromPage.Visible := False;
  finally
    EnableAlign;
  end;
end;

procedure TabcCustomButtonBar.SetButtonHeight(Value: Integer);
var
  i: integer;
begin
  if Value <> FButtonHeight then
  begin
    if Value < 0 then
      raise EInvalidOperation.CreateFmt(SPropertyOutOfRange, [Self.Classname]);
    FButtonHeight := Value;
    DisableAlign;
    try
      with FButtons do
        for i := 0 to Count - 1 do
          TabcBarBtn(Items[i]).Height := Value;
    finally
      EnableAlign;
    end;
  end;
end;

procedure TabcCustomButtonBar.SetButtonAlign(Value: TabcBarButtonAlign);
var
  i: integer;
begin
  if Value <> FButtonAlign then
  begin
    FButtonAlign := Value;
    with FButtons do
        for i := 0 to Count - 1 do
          TabcBarBtn(Items[i]).SetButtonAlign(Value);
  end;
end;

procedure TabcCustomButtonBar.SetButtonCursor(Value: TCursor);
var
  i: integer;
begin
  if Value <> FButtonCursor then
  begin
    FButtonCursor := Value;
    with FButtons do
        for i := 0 to Count - 1 do
          TabcBarBtn(Items[i]).Cursor := Value;
  end;
end;

procedure TabcCustomButtonBar.SetButtonColor(Value: TColor);
var
  i: integer;
begin
  if Value <> FButtonColor then
  begin
    FButtonColor := Value;
    with FButtons do
        for i := 0 to Count - 1 do
          TabcBarBtn(Items[i]).Color := Value;
  end;
end;

procedure TabcCustomButtonBar.SetButtonHighlight(Value: TabcGlyphHighlight);
var
  i: integer;
begin
  if Value <> FButtonHighlight then
  begin
    FButtonHighlight := Value;
    DisableAlign;
    try
      with FButtons do
        for i := 0 to Count - 1 do
          TabcBarBtn(Items[i]).PictureHighlight := Value;
    finally
      EnableAlign;
    end;
  end;
end;

procedure TabcCustomButtonBar.SetButtonStyle(Value: TabcButtonStyle);
var
  i: integer;
begin
  if Value <> FButtonStyle then
  begin
    FButtonStyle := Value;
    DisableAlign;
    try
      with FButtons do
        for i := 0 to Count - 1 do
          TabcBarBtn(Items[i]).Style := Value;
    finally
      EnableAlign;
    end;
  end;
end;

function TabcCustomButtonBar.FindNextPage(CurPage: TabcButtonPage;
  GoForward, CheckButtonVisible: Boolean): TabcButtonPage;
var
  I, StartIndex: Integer;
begin
  Result := nil;
  if FPages.Count <> 0 then
  begin
    StartIndex := FPages.IndexOf(CurPage);
    if StartIndex = -1 then
      if GoForward then StartIndex := FPages.Count - 1 else StartIndex := 0;
    I := StartIndex;
    repeat
      if GoForward then
      begin
        Inc(I);
        if I = FPages.Count then I := 0;
      end else
      begin
        if I = 0 then I := FPages.Count;
        Dec(I);
      end;
      Result := FPages[I];
      if not CheckButtonVisible or Result.ButtonVisible then Exit;
    until I = StartIndex;
  end;
end;

procedure TabcCustomButtonBar.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
begin
  for I := 0 to FPages.Count - 1 do Proc(TComponent(FPages[I]));
end;

procedure TabcCustomButtonBar.SetChildOrder(Child: TComponent; Order: Integer);
begin
  TabcButtonPage(Child).PageIndex := Order;
end;

function TabcCustomButtonBar.GetPage(Index: Integer): TabcButtonPage;
begin
  Result := FPages[Index];
end;

function TabcCustomButtonBar.GetPageCount: Integer;
begin
  Result := FPages.Count;
end;

procedure TabcCustomButtonBar.InsertPage(Page: TabcButtonPage);
var
  NewButton: TabcBarBtn;
begin
  DisableAlign;
  try
    NewButton := TabcBarBtn.CreateOnBar(Self, Page);
    NewButton.Parent := nil;
    NewButton.Parent := Self;
    FPages.Add(Page);
    Page.FButtonBar := Self;
    FButtons.Add(NewButton);
    if FButtons.Count = 1 then
      NewButton.GroupIndex := GetTickCount
    else
      NewButton.GroupIndex := TabcBarBtn(FButtons[0]).GroupIndex;
    NewButton.AllowAllUp := not (abbButtonDown in Options);
  finally
    EnableAlign;
  end;
end;

procedure TabcCustomButtonBar.HideButton(Page: TabcButtonPage);
begin
  if Page.PageIndex <> -1 then
  begin
    TabcBarBtn(FButtons[Page.PageIndex]).Visible := False;
    if (csLoading in ComponentState) then Exit;
    if AllButtonsHidden then
      ActivePage := nil
    else if (Page = ActivePage) then
    begin
      if Page.PageIndex = 0 then
        SelectNextPage(True)
      else
        SelectNextPage(False);
      if Page = ActivePage then
        ActivePage := nil;
    end;
    Realign;
  end;
end;

function TabcCustomButtonBar.AllButtonsHidden: Boolean;
var
  i: integer;
begin
  Result := True;
  for i := 0 to PageCount - 1 do
    if TabcBarBtn(FButtons[i]).Visible then
    begin
      Result := False;
      Break;
    end;
end;

procedure TabcCustomButtonBar.EnableButton(Page: TabcButtonPage; ButtonEnabled: Boolean);
begin
  TabcBarBtn(FButtons[Page.PageIndex]).Enabled := ButtonEnabled;
end;

procedure TabcCustomButtonBar.ShowButton(Page: TabcButtonPage);
begin
  UpdateButton(Page);
  TabcBarBtn(FButtons[Page.PageIndex]).Visible := True;
  if (csLoading in ComponentState) then Exit;
  if ActivePage = nil then
    ActivePage := Page;
end;

procedure TabcCustomButtonBar.ShowButtonFocus(Page: TabcButtonPage; Focused: Boolean);
begin
  if Page <> nil then
    if abbShowFocus in Options then
      TabcBarBtn(FButtons[Page.PageIndex]).ShowFocus := Focused
    else
      TabcBarBtn(FButtons[Page.PageIndex]).ShowFocus := False;
end;

procedure TabcCustomButtonBar.MoveButton(CurIndex, NewIndex: Integer);
begin
  FButtons.Move(CurIndex, NewIndex);
  Realign;
end;

procedure TabcCustomButtonBar.MovePage(CurIndex, NewIndex: Integer);
begin
  FPages.Move(CurIndex, NewIndex);
  MoveButton(CurIndex, NewIndex);
end;

procedure TabcCustomButtonBar.RemovePage(Page: TabcButtonPage);
var
  btn: TabcBarBtn;
  index: integer;
  wasActive: Boolean;
begin
  index := Page.PageIndex;
  wasActive := (FActivePage = Page) and not (csDestroying in ComponentState);
  btn := FButtons[index];
  if wasActive then SetActivePage(nil);
  Page.FButtonBar := nil;
  FPages.Remove(Page);
  FButtons.Remove(btn);
  btn.Free;
  if wasActive then
  begin
    if (index > 0) and (index = PageCount) then
      index := index - 1;
    if (index < PageCount) then
      SetActivePage(Pages[index]);
  end;
end;

procedure TabcCustomButtonBar.SelectNextPage(GoForward: Boolean);
var
  Page: TabcButtonPage;
begin
  Page := FindNextPage(ActivePage, GoForward, not AllButtonsHidden);
  ActivePage := Page;
end;

function TabcCustomButtonBar.GetActivePageIndex:Integer;
begin
  if FActivePage = nil then
    Result := -1
  else
    Result := FPages.IndexOf(FActivePage);
end;

procedure TabcCustomButtonBar.ActiveButtonHighlight(Page: TabcButtonPage);
begin
  if (Page <> nil) and (ActiveFontColor <> clNone) then
    with TabcBarBtn(FButtons[Page.PageIndex]) do
      if (Page = ActivePage) then
        Font.Color := ActiveFontColor
      else
        ParentFont := True;
end;

procedure TabcCustomButtonBar.SetActiveFontColor(Value: TColor);
begin
  if Value <> FActiveFontColor then
  begin
    FActiveFontColor := Value;
    if ActivePage <> nil then
      ActiveButtonHighlight(ActivePage);
  end;
end;

procedure TabcCustomButtonBar.SetActivePageIndex(Value: Integer);
begin
  if (Value >= 0) and (Value < PageCount) then
    ActivePage := Pages[Value]
  else
    if Value = -1 then
      ActivePage := nil;
end;

procedure TabcCustomButtonBar.ShowPage(Page: TabcButtonPage);
var
  ParentForm: TCustomForm;
  OldActivePage: TabcButtonPage;
begin
  ParentForm := GetParentForm(Self);
  if (ParentForm <> nil) and (FActivePage <> nil) and
    FActivePage.ContainsControl(ParentForm.ActiveControl) then
    ParentForm.ActiveControl := FActivePage;
  if Page <> nil then
  begin
    if (FSaveAnchorSize.X = 0) and (FSaveAnchorSize.Y = 0) then
      FSaveAnchorSize := Point(Width, Height);
    Page.SetBounds(0,0,FSaveAnchorSize.X,FSaveAnchorSize.Y);
    Page.BringToFront;
    Page.Visible := True;
    if (ParentForm <> nil) and (FActivePage <> nil) and
      (ParentForm.ActiveControl = FActivePage) then
      if Page.CanFocus then
        ParentForm.ActiveControl := Page
      else
        ParentForm.ActiveControl := Self;
  end;
  DisableAlign;
  try
    AnimatePageChange(FActivePage, Page);
    OldActivePage := FActivePage;
    FActivePage := Page;
    ActiveButtonHighlight(OldActivePage);
    ActiveButtonHighlight(FActivePage);
    KeepButtonDown(abbButtonDown in Options);
    if OldActivePage <> nil then
    begin
      OldActivePage.SendToBack;
      OldActivePage.Visible := False;
    end;
  finally
    EnableAlign;
    ReAlign;
  end;
  if (ParentForm <> nil) and (FActivePage <> nil) and
    (ParentForm.ActiveControl = FActivePage) then
    FActivePage.SelectFirst;
end;

procedure TabcCustomButtonBar.SetActivePage(Page: TabcButtonPage);
begin
  if FActivePageChanging or
    ((Page <> nil) and (Page.ButtonBar <> Self)) then
    Exit;
  FActivePageChanging := True;
  try
    if FActivePage <> Page then
    begin
      if (csLoading in ComponentState) then
      begin
        FActivePage := Page;
        Exit;
      end;
      if not CanChange(Page) then
        Exit;
      ShowPage(Page);
      Change;
    end;
  finally
    FActivePageChanging := False;
  end;
end;

procedure TabcCustomButtonBar.SetAnimationSteps(Value: Byte);
begin
  if Value = 0 then
    FAnimationSteps := 1
  else
    FAnimationSteps := Value;
end;

procedure TabcCustomButtonBar.SetOptions(Value: TabcButtonBarOptions);
begin
  if Value <> FOptions then
  begin
    FOptions := Value;
    if (abbButtonDown in Value) <> (abbButtonDown in FOptions) then
      KeepButtonDown(abbButtonDown in FOptions);
    if (abbShowFocus in Value) <> (abbShowFocus in FOptions) then
      ShowButtonFocus(ActivePage, True);
  end;
end;

procedure TabcCustomButtonBar.KeepButtonDown(Value: Boolean);
var
  i, activePg: integer;
begin
  activePg := ActivePageIndex;
  for i := 0 to FButtons.Count - 1 do
    with TabcBarBtn(FButtons[i]) do
    begin
      AllowAllUp := not Value;
      if i = activePg then
        Down := Value
      else
        Down := False;
    end;
end;

procedure TabcCustomButtonBar.SetImages(Value: TCustomImageList);
var
  i: integer;
begin
  if Value <> FImages then
  begin
    FImages := Value;
    for i := 0 to FButtons.Count - 1 do
      TabcBarBtn(FButtons[i]).Images := Value;
  end;
end;

procedure TabcCustomButtonBar.ShowControl(AControl: TControl);
begin
  if (AControl is TabcButtonPage) and (TabcButtonPage(AControl).ButtonBar = Self) then
    SetActivePage(TabcButtonPage(AControl))
  else if (AControl is TabcBarBtn) then
    inherited ShowControl(AControl);
end;

procedure TabcCustomButtonBar.UpdateButton(Page: TabcButtonPage);
begin
  with TabcBarBtn(FButtons[Page.PageIndex]) do
  begin
    Caption := Page.Caption;
    Picture := Page.ButtonPicture;
    Hint := Page.Hint;
    Images := Page.ButtonBar.Images;
    ImageIndex := Page.ImageIndex;
    Action := Page.Action;
    if ActiveFontColor <> clNone then
      if Page = ActivePage then
        Font.Color := ActiveFontColor
      else
        ParentFont := True;
  end;
end;

procedure TabcCustomButtonBar.ClearButtonAction(Page: TabcButtonPage);
begin
  TabcBarBtn(FButtons[Page.PageIndex]).Action := nil;
end;

procedure TabcCustomButtonBar.CMDialogChar(var Message: TCMDialogChar);
{don't allow accelerator to show page if button is not visible}
var
  i: integer;
begin
  with Message do
    if (ssAlt in KeyDataToShiftState(KeyData)) then
      for i := 0 to PageCount - 1 do
        if IsAccel(CharCode, Pages[i].Caption) and Pages[i].ButtonVisible then
        begin
          ActivePage := Pages[i];
          SelectFirst;
          Result := 1;
          Exit;
        end;
  inherited;
end;

procedure TabcCustomButtonBar.CMDialogKey(var Message: TCMDialogKey);
begin
  if (Focused or Windows.IsChild(Handle, Windows.GetFocus)) and 
    (Message.CharCode = VK_TAB) and (GetKeyState(VK_CONTROL) < 0) then
  begin
    SelectNextPage(GetKeyState(VK_SHIFT) >= 0);
    Message.Result := 1;
  end else
    inherited;
end;

procedure TabcCustomButtonBar.DoAddDockClient(Client: TControl; const ARect: TRect);
begin
  if FNewDockPage <> nil then Client.Parent := FNewDockPage;
end;

procedure TabcCustomButtonBar.DockOver(Source: TDragDockObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
var
  R: TRect;
begin
  if (ActivePage <> nil) and ActivePage.HandleAllocated then
    GetWindowRect(ActivePage.Handle, R)
  else
    GetWindowRect(Handle, R);
  Source.DockRect := R;
  DoDockOver(Source, X, Y, State, Accept);
end;

procedure TabcCustomButtonBar.DoRemoveDockClient(Client: TControl);
begin
  if (FUnDockPage <> nil) and not (csDestroying in ComponentState) then
  begin
    RemovePage(FUndockPage);
    FUnDockPage.Free;
    FUnDockPage := nil;
  end;
end;

function TabcCustomButtonBar.GetPageFromDockClient(Client: TControl): TabcButtonPage;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to PageCount - 1 do
  begin
    if (Client.Parent = Pages[I]) and (Client.HostDockSite = Self) then
    begin
      Result := Pages[I];
      Exit;
    end;
  end;
end;

procedure TabcCustomButtonBar.GetSiteInfo(Client: TControl; var InfluenceRect: TRect;
  MousePos: TPoint; var CanDock: Boolean);
begin
  CanDock := GetPageFromDockClient(Client) = nil;
  inherited GetSiteInfo(Client, InfluenceRect, MousePos, CanDock);
end;

procedure TabcCustomButtonBar.CMDockClient(var Message: TCMDockClient);
var
  IsVisible: Boolean;
  DockCtl: TControl;
begin
  Message.Result := 0;
  FNewDockPage := TabcButtonPage.Create(Self);
  try
    try
      DockCtl := Message.DockSource.Control;
      FNewDockPage.Caption := TControlAccess(DockCtl).Caption;
      if FNewDockPage.Caption = '' then
        FNewDockPage.Caption := DockCtl.Name;
      FNewDockPage.ButtonBar := Self;
      DockCtl.Dock(Self, Message.DockSource.DockRect);
    except
      FNewDockPage.Free;
      raise;
    end;
    IsVisible := DockCtl.Visible;
    FNewDockPage.ButtonVisible := IsVisible;
    if IsVisible then ActivePage := FNewDockPage;
    DockCtl.Align := alClient;
  finally
    FNewDockPage := nil;
  end;
end;

procedure TabcCustomButtonBar.CMDockNotification(var Message: TCMDockNotification);
var
  I: Integer;
  S: string;
  Page: TabcButtonPage;
begin
  Page := GetPageFromDockClient(Message.Client);
  if Page <> nil then
    case Message.NotifyRec.ClientMsg of
      WM_SETTEXT:
        begin
          S := PChar(Message.NotifyRec.MsgLParam);
          { Search for first CR/LF and end string there }
          for I := 1 to Length(S) do
            if S[I] in [#13, #10] then
            begin
              SetLength(S, I - 1);
              Break;
            end;
          Page.Caption := S;
        end;
      CM_VISIBLECHANGED:
        with Page do
        begin
          Visible := Boolean(Message.NotifyRec.MsgWParam);
          ButtonVisible := Boolean(Message.NotifyRec.MsgWParam);;
        end;
    end;
  inherited;
end;

procedure TabcCustomButtonBar.CMUnDockClient(var Message: TCMUnDockClient);
var
  Page: TabcButtonPage;
begin
  Message.Result := 0;
  Page := GetPageFromDockClient(Message.Client);
  if Page <> nil then
  begin
    FUnDockPage := Page;
    Message.Client.Align := alNone;
  end;
end;

procedure TabcCustomButtonBar.CMShowingChanged(var Message: TMessage);
begin
  inherited;
  if Showing then
    ReAlign;
end;

procedure TabcButtonBar.SetButtonTransparentColor(Value: TColor);
var
  i: integer;
begin
  with FButtons do
    for i := 0 to Count - 1 do
      TabcBarBtn(Items[i]).TransparentColor := Value;
end;

end.
