unit abcbarlv;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls,
  ImgList, ActnList,
  abcwav, abctileb, abcinpla, abcmsg;

type
  {$IFNDEF DELPHI5}
    TImageIndex = type Integer;
  {$ENDIF}

  { TabcButtonList }

  TabcCustomButtonList = class;

  { Items and Collection }

  TabcButtonListItems = class;
  TabcButtonListItem = class;

{ TabcButtonListItemActionLink }

  TabcButtonListItemActionLink = class(TActionLink)
  private
    FClient: TabcCustomButtonList;
    FItem: TabcButtonListItem;
    procedure UpdateItem;
  protected
    procedure AssignClient(AClient: TObject); override;
    function IsCaptionLinked: Boolean; override;
    function IsEnabledLinked: Boolean; override;
    function IsImageIndexLinked: Boolean; override;
    procedure SetCaption(const Value: string); override;
    procedure SetEnabled(Value: Boolean); override;
    procedure SetImageIndex (Value: Integer); override;
  end;

  TabcButtonListItem = class(TCollectionItem)
  private
    FImageRect: TRect;
    FTextRect: TRect;
    FCaption: string;
    FData: TObject;
    FEnabled: Boolean;
    FImageIndex: TImageIndex;
    FLink: TComponent;
    FSoundOver: Integer;
    FSoundDown: Integer;
    FTag: Integer;
    FActionLink: TabcButtonListItemActionLink;
    procedure SetCaption(Value: string);
    procedure SetEnabled(Value: Boolean);
    procedure SetImageIndex(Value: TImageIndex);
    procedure ActionChange(Sender: TObject);
    function GetAction: TBasicAction;
    procedure SetAction(Value: TBasicAction);
    function IsCaptionStored: Boolean;
    function IsImageIndexStored: Boolean;
  protected
    function GetDisplayName: string; override;
    procedure SetDisplayName(const Value: string); override;
    function GetCollectionAsList: TabcButtonListItems;
    function GetButtonList: TabcCustomButtonList;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Move(Down: Boolean);
    function MaxY: Integer;
    property CollectionAsList: TabcButtonListItems read GetCollectionAsList;
    property ButtonList: TabcCustomButtonList read GetButtonList;
    property Data: TObject read FData write FData;
    property ImageRect: TRect read FImageRect;
    property TextRect: TRect read FTextRect;
  published
    property Action: TBasicAction read GetAction write SetAction;
    property Caption: string read FCaption write SetCaption
      stored IsCaptionStored;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex
      stored IsImageIndexStored default -1;
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Link: TComponent read FLink write FLink;
    property SoundOver: Integer read FSoundOver write FSoundOver default -1;
    property SoundDown: Integer read FSoundDown write FSoundDown default -1;
    property Tag: Integer read FTag write FTag;
  end;

  TabcButtonListItems = class(TCollection)
  private
    FList: TabcCustomButtonList;
    FOnUpdate: TNotifyEvent;
    function GetItem(Index: Integer): TabcButtonListItem;
    procedure SetItem(Index: Integer; Value: TabcButtonListItem);
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AList: TabcCustomButtonList);
    function Add: TabcButtonListItem;
    property Items[Index: Integer]: TabcButtonListItem read GetItem write SetItem; default;
    property ButtonList: TabcCustomButtonList read FList;
    property OnUpdate: TNotifyEvent read FOnUpdate write FOnUpdate;
  end;

  TabcButtonListViewStyle = (abvIcon, abvSmallIcon);

  TabcButtonListOption = (ablAccelChars, ablAcceptItems, ablAllowDrag,
    ablInplaceEdit, ablKeepSelected, ablWordWrap, ablFlyoutHints,
    ablHideAccelChars, ablAllowDragCopy, ablAllowDelete, ablConfirmDelete,
    ablAcceptDroppedFiles, ablHideCaptions);

  TabcButtonListOptions = set of TabcButtonListOption;

  TabcButtonListItemNotifyEvent = procedure
    (Sender: TObject; Item: TabcButtonListItem) of object;

  TabcButtonListMoveItemEvent = procedure
    (Sender: TObject; Item: TabcButtonListItem;
     Source, Dest: TabcCustomButtonList; SourceIndex, DestIndex: Integer;
     var Accept: Boolean) of object;

  TabcButtonListItemRenameEvent = procedure
    (Sender: TObject; Item: TabcButtonListItem; var NewCaption: string) of object;

  TabcSimpleFontHighlight = class(TPersistent)
  private
    FUnderline: Boolean;
    FFontColor: TColor;
    FOnChange: TNotifyEvent;
    procedure SetFontColor(Value: TColor);
    procedure SetUnderline(Value: Boolean);
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property FontColor: TColor read FFontColor write SetFontColor
      default clNone;
    property Underline: Boolean read FUnderline write SetUnderline default False;
  end;

  TabcScrollButtonStyle = (asbDefault, asbFlat, asbRect);

  TabcCustomButtonList = class(TCustomControl)
  private
    FLargeChangeLink: TChangeLink;
    FSmallChangeLink: TChangeLink;
    FDrawImageList: TImageList;
    FTextFlags: DWORD;

    FUpSBVisible: Boolean;
    FUpSBDown: Boolean;
    FUpSBRect: TRect;
    FDownSBVisible: Boolean;
    FDownSBDown: Boolean;
    FDownSBRect: TRect;
    FScrollTimer: TTimer;

    FTopItemIndex: Integer;
    FActiveIndex: Integer;
    FSelectedIndex: Integer;
    FGroupIndex: Integer;
    FItems: TabcButtonListItems;

    FDragLineColor: TColor;

    FDraggedIndex: Integer;
    FDropTargetIndex: Integer;
    FDragLineY: Integer;
    FEditing: Boolean;
    FFrameBottomColor: TColor;
    FFrameTopColor: TColor;
    FHintWindow: THintWindow;
    FOptions: TabcButtonListOptions;
    FScrollButtonStyle: TabcScrollButtonStyle;
    FViewStyle: TabcButtonListViewStyle;
    FSoundList: TabcCustomWaveList;
    FBackground: TabcTileBack;
    FHighlightActive: TabcSimpleFontHighlight;
    FHighlightSelected: TabcSimpleFontHighlight;
    FSmallImages: TImageList;
    FLargeImages: TImageList;
    FBeginDrag: Boolean;
    FDragCopy: Boolean;
    FMouseDownAt: TPoint;
    FItemSpace: Byte;
    FOnCopyItem: TabcButtonListMoveItemEvent;
    FAfterCopyItem: TabcButtonListItemNotifyEvent;
    FOnDeleteItem: TabcButtonListMoveItemEvent;
    FOnMoveItem: TabcButtonListMoveItemEvent;
    FOnRenameItem: TabcButtonListItemRenameEvent;
    FOnActivateItem: TabcButtonListItemNotifyEvent;
    FOnSelectItem: TabcButtonListItemNotifyEvent;
    FOnUnselectItem: TabcButtonListItemNotifyEvent;
    FOnClickItem: TabcButtonListItemNotifyEvent;
    FOnDblClickItem: TabcButtonListItemNotifyEvent;
    FActiveChanging: Boolean;
    FSelectedChanging: Boolean;
    FTransparent: Boolean;
    procedure SetTransparent(const Value: Boolean);
    procedure BackgroundChanged(Sender: TObject);
    procedure ClearDragEffects;
    function GetActiveItem: TabcButtonListItem;
    function GetDraggedItem: TabcButtonListItem;
    procedure GetDropTargetAtPos(X, Y: Integer);
    function GetSelectedItem: TabcButtonListItem;
    procedure SetBackground(const Value: TabcTileBack);
    procedure SetItems(const Value: TabcButtonListItems);
    procedure SetFrameBottomColor(const Value: TColor);
    procedure SetFrameTopColor(const Value: TColor);
    procedure SetLargeImages(const Value: TImageList);
    procedure SetHighlightActive(Value: TabcSimpleFontHighlight);
    procedure SetHighlightSelected(Value: TabcSimpleFontHighlight);
    procedure SetOptions(const Value: TabcButtonListOptions);
    procedure SetActiveIndex(const Value: Integer);
    procedure SetGroupIndex(const Value: Integer);
    procedure SetSelectedIndex(const Value: Integer);
    procedure SetSelectedItem(const Value: TabcButtonListItem);
    procedure SetSmallImages(const Value: TImageList);
    procedure SetTopItemIndex(Value: Integer);
    procedure SetViewStyle(const Value: TabcButtonListViewStyle);
    procedure SetItemSpace(Value: Byte);
    procedure AMSelectedChanged(var Message: TMessage); message AM_SELECTEDCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
    procedure WMMove(var Message: TWMMove); message WM_MOVE;
{$IFDEF DELPHI5}
    procedure WMContextMenu(var Message: TWMContextMenu); message WM_CONTEXTMENU;
{$ENDIF}
    procedure StartTimedScroll(AnInterval: Integer);
    procedure StopTimedScroll;
    procedure ScrollItems(ScrollUp: Boolean);
    procedure ScrollTimerExpired(Sender: TObject);
    property TopItemIndex: Integer read FTopItemIndex write SetTopItemIndex;
    procedure ImageListChanged(Sender: TObject);
    procedure FontHighlightChanged(Sender: TObject);
    procedure SetScrollButtonStyle(const Value: TabcScrollButtonStyle);
    procedure UpdateExclusive;
  protected
    {override methods}
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
    procedure SetParent(AParent: TWinControl); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure DblClick; override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;

    {internal methods}
    procedure UpdateScrollButtonState;
    function IsItemFullyVisible(Index: Integer): Boolean;
    function IsItemPartlyVisible(Index: Integer): Boolean;
    procedure LayoutChanged;
    procedure DoDrawImage(Index: Integer; Rect: TRect; Active, Selected: Boolean); dynamic;
    procedure DoDrawText(Index: Integer; var Rect: TRect; Flags: DWORD; Active, Selected: Boolean); dynamic;
    procedure DoDrawScrollButton(IsUpSB, IsDown: Boolean); dynamic;
    procedure DoDrawDragLineAt(Pos: Integer; Rect: TRect); dynamic;
    procedure DoMoveItem(Item: TabcButtonListItem; NewIndex: Integer); dynamic;
    procedure DoCopyItem(Item: TabcButtonListItem; NewIndex: Integer); dynamic;
    procedure DoDeleteItem(Item: TabcButtonListItem); dynamic;
    procedure SetDragCopy(Value: Boolean);
    procedure KeySelectNext(GoForward: Boolean);
    {internal properties for descendants if required}
    property TextFlags: DWORD read FTextFlags write FTextFlags;
    property DraggedItem: TabcButtonListItem read GetDraggedItem;
    property DraggedIndex: Integer read FDraggedIndex;
    property DragCopy: Boolean read FDragCopy write SetDragCopy;

    {publishable properties}
    property Background: TabcTileBack read FBackground write SetBackground;
    property DragLineColor: TColor read FDragLineColor write FDragLineColor default clBlack;
    property FrameBottomColor: TColor read FFrameBottomColor write SetFrameBottomColor default clBlack;
    property FrameTopColor: TColor read FFrameTopColor write SetFrameTopColor default clBtnFace;
    property GroupIndex: Integer read FGroupIndex write SetGroupIndex default 0;
    property HighlightActive: TabcSimpleFontHighlight read FHighlightActive write SetHighlightActive;
    property HighlightSelected: TabcSimpleFontHighlight read FHighlightSelected write SetHighlightSelected;
    property ItemSpace: Byte read FItemSpace write SetItemSpace default 5;
    property Items: TabcButtonListItems read FItems write SetItems;
    property LargeImages: TImageList read FLargeImages write SetLargeImages;
    property Options: TabcButtonListOptions read FOptions write SetOptions
      default [ablAcceptItems, ablAllowDrag, ablWordWrap];
    property ScrollButtonStyle: TabcScrollButtonStyle read FScrollButtonStyle write SetScrollButtonStyle
      default asbDefault;
    property SelectedIndex: Integer read FSelectedIndex write SetSelectedIndex default -1;
    property SmallImages: TImageList read FSmallImages write SetSmallImages;
    property SoundList: TabcCustomWaveList read FSoundList write FSoundList;
    property Transparent: Boolean read FTransparent write SetTransparent default False;
    property ViewStyle: TabcButtonListViewStyle read FViewStyle write SetViewStyle default abvIcon;

    {events}
    property OnActivateItem: TabcButtonListItemNotifyEvent read FOnActivateItem write FOnActivateItem;
    property OnClickItem: TabcButtonListItemNotifyEvent read FOnClickItem write FOnClickItem;
    property OnCopyItem: TabcButtonListMoveItemEvent read FOnCopyItem write FOnCopyItem;
    property OnDblClickItem: TabcButtonListItemNotifyEvent read FOnDblClickItem write FOnDblClickItem;
    property AfterCopyItem: TabcButtonListItemNotifyEvent read FAfterCopyItem write FAfterCopyItem;
    property OnDeleteItem: TabcButtonListMoveItemEvent read FOnDeleteItem write FOnDeleteItem;
    property OnMoveItem: TabcButtonListMoveItemEvent read FOnMoveItem write FOnMoveItem;
    property OnRenameItem: TabcButtonListItemRenameEvent read FOnRenameItem write FOnRenameItem;
    property OnSelectItem: TabcButtonListItemNotifyEvent read FOnSelectItem write FOnSelectItem;
    property OnUnselectItem: TabcButtonListItemNotifyEvent read FOnUnselectItem write FOnUnselectItem;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Invalidate; override;
    procedure ClickItem(Item: TabcButtonListItem);
    function AddItem(ACaption: string; AImageIndex: Integer; DoEdit: Boolean): TabcButtonListItem;
    function RemoveActiveItem(Confirm: Boolean): Boolean;
    procedure Edit(const ItemIndex: Integer); dynamic;
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    procedure ScrollItemInView(Index: Integer; Partial: Boolean);
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    function GetIndexAtPos(X, Y: Integer): Integer;
    function GetItemAtPos(X, Y: Integer): TabcButtonListItem;
    property Editing: Boolean read FEditing;
    property ActiveItem: TabcButtonListItem read GetActiveItem;
    property SelectedItem: TabcButtonListItem read GetSelectedItem write SetSelectedItem;
    property ActiveIndex: Integer read FActiveIndex write SetActiveIndex;
  end;

  TabcButtonList = class(TabcCustomButtonList)
  published
    property Action;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property ParentBiDiMode;
    property Align;
    property Background;
    property Color default clBtnShadow;
    property DragLineColor;
    property Enabled;
    property Font;
    property FrameBottomColor;
    property FrameTopColor;
    property GroupIndex;
    property HighlightActive;
    property HighlightSelected;
    property ItemSpace;
    property Items;
    property LargeImages;
    property Options;
    property ParentColor default False;
    property ParentFont default False;
    property ParentShowHint;
    property PopupMenu;
    property ScrollButtonStyle;
    property SelectedIndex;
    property ShowHint;
    property SmallImages;
    property SoundList;
    property TabOrder;
//    property Transparent; not working with doublebuffered window
    property ViewStyle;
    property Visible;
    property OnActivateItem;
    property OnClick;
    property OnClickItem;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
    property OnDblClick;
    property OnDblClickItem;
    property OnEnter;
    property OnExit;
    property OnCopyItem;
    property AfterCopyItem;
    property OnDeleteItem;
    property OnMoveItem;
    property OnRenameItem;
    property OnSelectItem;
    property OnUnselectItem;
  end;

implementation

uses abcbar, abcconst, abcutilf, CommCtrl;

{ TabcCustomButtonList }

const
  C_FRAME_OFFSET = 2;

function Max(i,j: Integer): Integer;
begin
  if i > j then
    Result := i
  else
    Result := j;
end;

{ TabcCustomButtonList }

constructor TabcCustomButtonList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLargeChangeLink := TChangeLink.Create;
  FLargeChangeLink.OnChange := ImageListChanged;
  FSmallChangeLink := TChangeLink.Create;
  FSmallChangeLink.OnChange := ImageListChanged;
  ControlStyle := ControlStyle + [csOpaque, csReplicatable, csClickEvents, csDoubleClicks];

  DoubleBuffered := True;
  
  FBackground := TabcTileBack.Create;
  FBackground.OnChange := BackgroundChanged;
  FItems := TabcButtonListItems.Create(Self);
  FItemSpace := 5;
  FOptions := [ablAcceptItems, ablAllowDrag, ablWordWrap];
  FViewStyle := abvIcon;
  FActiveIndex := -1;
  FSelectedIndex := -1;
  FDraggedIndex := -1;
  FDropTargetIndex := -1;
  FDragLineY := -1;
  FHighlightActive := TabcSimpleFontHighlight.Create;
  FHighlightActive.OnChange := FontHighlightChanged;
  FHighlightSelected := TabcSimpleFontHighlight.Create;
  FHighlightSelected.OnChange := FontHighlightChanged;
  ParentFont := False;
  ParentColor := False;
  Color := clBtnShadow;
  Font.Color := clWindow;
  FFrameTopColor := clBtnFace;
  FFrameBottomColor := clBlack;
  FDragLineColor := clBlack;
  DragKind := dkDrag;
  SetBounds(Left, Top, 150, 300);
end;

procedure TabcCustomButtonList.CreateParams(var Params: TCreateParams);
begin
  inherited;
  with Params do
    if Transparent then
      ExStyle := ExStyle or WS_EX_TRANSPARENT;
end;

destructor TabcCustomButtonList.Destroy;
begin
  FLargeChangeLink.Free;
  FSmallChangeLink.Free;
  FItems.Free;
  FBackground.Free;
  FScrollTimer.Free;
  FHintWindow.Free;
  FHighlightActive.Free;
  FHighlightSelected.Free;
  inherited Destroy;
end;

procedure TabcCustomButtonList.Loaded;
begin
  inherited Loaded;
  FItems.Update(nil);  {calls LayoutChanged}
end;

procedure TabcCustomButtonList.Notification(AComponent: TComponent;
  Operation: TOperation);
var
  i: integer;
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
  begin
    if AComponent = LargeImages then LargeImages := nil;
    if AComponent = SmallImages then SmallImages := nil;
    if AComponent = SoundList then SoundList := nil;
    for i := 0 to Items.Count - 1 do
    begin
      if AComponent = Items[i].Link then
        Items[i].Link := nil;
      if AComponent is TBasicAction then
        if AComponent = Items[i].Action then
          Items[i].Action := nil;
    end;
  end;
end;

procedure TabcCustomButtonList.ImageListChanged(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then
    if Assigned(FDrawImageList) and (Sender = FDrawImageList) then
      LayoutChanged;
end;

function TabcCustomButtonList.AddItem(ACaption: string; AImageIndex: Integer; DoEdit: Boolean): TabcButtonListItem;
begin
  Result := Items.Add;
  with Result do
  begin
    Caption := ACaption;
    ImageIndex := AImageIndex;
  end;
  if DoEdit then
    Edit(Result.Index);
end;

function TabcCustomButtonList.RemoveActiveItem(Confirm: Boolean): Boolean;
begin
  Result := False;
  if ActiveIndex <> -1 then
    if not Confirm or
       (MessageDlg(
          Format(sabcConfirmRemoveButtonListItem, [ActiveItem.Caption]),
          mtWarning, [mbYes, mbNo], 0)
          = mrYes) then
    begin
      ActiveItem.Free;
      Result := True;
    end;
end;

procedure TabcCustomButtonList.Edit(const ItemIndex: Integer);
var
  Item: TabcButtonListItem;
  sEdit: string;
  EditFont: TFont;
  EditRect: TRect;
begin
  if (not Enabled) or (not Visible) or
     (ItemIndex < 0) or (ItemIndex >= Items.Count) or
     not Items[ItemIndex].Enabled or
     FEditing or (csDesigning in ComponentState) then Exit;
  FEditing := True;
  try
    if not IsItemFullyVisible(ItemIndex) then
      ScrollItemInView(ItemIndex, False);
    Item := Items[ItemIndex];
    sEdit := Item.Caption;
    EditFont := TFont.Create;
    try
      EditFont.Assign(Font);
      EditFont.Color := clBlack;
      EditRect := Item.TextRect;
      EditRect.Left := FItemSpace;
      EditRect.Right := Width - FItemSpace;
      if (EditRect.Bottom - EditRect.Top) < (- 2 * Font.Height + 8) then
        EditRect.Bottom := EditRect.Top - 2 * Font.Height + 8
      else
        EditRect.Bottom := EditRect.Bottom + 4;
      if abcEditInPlace(
        Self,
        EditRect,
        sEdit,
        False,
        bsSingle,
        EditFont,
        clWindow,
        taCenter) then
      begin
        if Assigned(FOnRenameItem) then
          FOnRenameItem(Self, Item, sEdit);
        Item.Caption := sEdit;
      end;
    finally
      EditFont.Free;
    end;
  finally
    FEditing := False;
  end;
end;

function TabcCustomButtonList.GetActiveItem: TabcButtonListItem;
begin
  if ActiveIndex = -1 then
    Result := nil
  else
    Result := Items[ActiveIndex];
end;

function TabcCustomButtonList.GetDraggedItem: TabcButtonListItem;
begin
  if DraggedIndex = -1 then
    Result := nil
  else
    Result := Items[DraggedIndex];
end;

function TabcCustomButtonList.GetSelectedItem: TabcButtonListItem;
begin
  if SelectedIndex = -1 then
    Result := nil
  else
    Result := Items[SelectedIndex];
end;

procedure TabcCustomButtonList.SetBackground(const Value: TabcTileBack);
begin
  FBackground.Assign(Value);
end;

procedure TabcCustomButtonList.BackgroundChanged(Sender: TObject);
begin
  Repaint;
end;

procedure TabcCustomButtonList.SetItems(const Value: TabcButtonListItems);
begin
  FItems := Value;
  LayoutChanged;
end;

procedure TabcCustomButtonList.SetFrameBottomColor(const Value: TColor);
begin
  if Value <> FFrameBottomColor then
  begin
    FFrameBottomColor := Value;
    Repaint;
  end;
end;

procedure TabcCustomButtonList.SetFrameTopColor(const Value: TColor);
begin
  if Value <> FFrameTopColor then
  begin
    FFrameTopColor := Value;
    Repaint;
  end;
end;

procedure TabcCustomButtonList.SetLargeImages(const Value: TImageList);
begin
  if LargeImages <> nil then LargeImages.UnRegisterChanges(FLargeChangeLink);
  FLargeImages := Value;
  if LargeImages <> nil then
  begin
    LargeImages.RegisterChanges(FLargeChangeLink);
    LargeImages.FreeNotification(Self);
  end;
  if (ViewStyle = abvIcon) or not Assigned(SmallImages) then
    LayoutChanged;
end;

procedure TabcCustomButtonList.SetItemSpace(Value: Byte);
begin
  if Value <> FItemSpace then
  begin
    FItemSpace := Value;
    LayoutChanged;
  end;
end;

procedure TabcCustomButtonList.SetOptions(const Value: TabcButtonListOptions);
begin
  if [ablAccelChars, ablHideAccelChars] * Value = [ablAccelChars, ablHideAccelChars] then
    if ablAccelChars in FOptions then
      FOptions := Value - [ablAccelChars]
    else
      FOptions := Value - [ablHideAccelChars]
  else
    FOptions := Value;
  if not (ablKeepSelected in Value) then
    FSelectedIndex := -1;
  LayoutChanged;
end;

procedure TabcCustomButtonList.SetActiveIndex(const Value: Integer);
begin
  if (Value <> FActiveIndex) and not FActiveChanging then
  begin
    FActiveChanging := True;
    try
      if (Value < -1) or (Value >= Items.Count) or
         ((Value > -1) and not Items[Value].Enabled) then
        Exit;
      ScrollItemInView(Value, True);
      FActiveIndex := Value;
      Repaint;
      if (Value > -1) and (Value < Items.Count) then
      begin
        if (FSoundList <> nil) then
          FSoundList.PlayItem(Items[Value].SoundOver);
        if Assigned(FOnActivateItem) then
          FOnActivateItem(Self, ActiveItem);
      end;
    finally
      FActiveChanging := False;
    end;
  end;
end;

procedure TabcCustomButtonList.SetSelectedIndex(const Value: Integer);
begin
  if (Value <> FSelectedIndex) and not FSelectedChanging then
  begin
    FSelectedChanging := True;
    try
      if (Value < -1) or (Value >= Items.Count) or
         ((Value > -1) and not Items[Value].Enabled) then
        Exit;
      if (SelectedItem <> nil) and Assigned(FOnUnselectItem) then
        FOnUnselectItem(Self, SelectedItem);
      FSelectedIndex := Value;
      if (csLoading in ComponentState) then Exit;
      if (csDesigning in ComponentState) then
        Options := Options + [ablKeepSelected];
      ScrollItemInView(Value, False);
      if (Value > -1) and (Value < Items.Count) and not (csDesigning in ComponentState) then
      begin
        if (FSoundList <> nil) then
          FSoundList.PlayItem(Items[Value].SoundDown);
        if Assigned(FOnSelectItem) then
          FOnSelectItem(Self, SelectedItem);
      end;
      if (Value <> -1) then
        UpdateExclusive;
    finally
      FSelectedChanging := False;
    end;
  end;
end;

procedure TabcCustomButtonList.UpdateExclusive;
var
  Msg: TMessage;
  SendTo: TWinControl;
begin
  if (FGroupIndex <> 0) and (ablKeepSelected in Options) and (SelectedIndex <> -1) and (Parent <> nil) then
  begin
    Msg.Msg := AM_SELECTEDCHANGED;
    Msg.WParam := FGroupIndex;
    Msg.LParam := Longint(Self);
    Msg.Result := 0;
    SendTo := Parent;
    if (SendTo.Parent <> nil) and (SendTo.Parent is TabcCustomButtonBar) then
      SendTo := SendTo.Parent;
    SendTo.Broadcast(Msg);
  end;
end;

procedure TabcCustomButtonList.SetGroupIndex(const Value: Integer);
begin
  if FGroupIndex <> Value then
  begin
    FGroupIndex := Value;
    UpdateExclusive;
  end;
end;

procedure TabcCustomButtonList.AMSelectedChanged(var Message: TMessage);
begin
  with Message do
    if (WParam = GroupIndex) and (LParam <> Longint(Self)) and (ablKeepSelected in Options) then
      SelectedIndex := -1;
end;

procedure TabcCustomButtonList.SetSelectedItem(const Value: TabcButtonListItem);
begin
  SelectedIndex := Value.Index;
end;

procedure TabcCustomButtonList.SetSmallImages(const Value: TImageList);
begin
  if SmallImages <> nil then SmallImages.UnRegisterChanges(FSmallChangeLink);
  FSmallImages := Value;
  if SmallImages <> nil then
  begin
    SmallImages.RegisterChanges(FSmallChangeLink);
    SmallImages.FreeNotification(Self);
  end;
  if (ViewStyle = abvSmallIcon) then
    LayoutChanged;
end;

procedure TabcCustomButtonList.SetViewStyle(const Value: TabcButtonListViewStyle);
begin
  FViewStyle := Value;
  LayoutChanged;
end;

procedure TabcCustomButtonList.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  Repaint;
end;

procedure TabcCustomButtonList.CMFontChanged(var Message: TMessage);
begin
  inherited;
  LayoutChanged;
end;

procedure TabcCustomButtonList.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  StopTimedScroll;
end;

procedure TabcCustomButtonList.CMShowingChanged(var Message: TMessage);
begin
  inherited;
  if Showing and (SelectedIndex <> -1) then
    ScrollItemInView(SelectedIndex, False);
end;

procedure TabcCustomButtonList.DoDeleteItem(Item: TabcButtonListItem);
var
  Accept: Boolean;
begin
  if Item = nil then Exit;
  Accept := True;
  if not (ablConfirmDelete in Options) or
      (MessageDlg(
        Format(sabcButtonListDelete, [Item.Caption]),
        mtWarning,
        [mbYes, mbNo],
        0) = mrYes) then
  begin
    if Assigned(FOnDeleteItem) then
      FOnDeleteItem(Self, Item, Self, Self, Item.Index, -1, Accept);
    if Accept then
      Item.Free;
  end;
end;

procedure TabcCustomButtonList.DoCopyItem(Item: TabcButtonListItem; NewIndex: Integer);
var
  Accept: Boolean;
  Source: TabcCustomButtonList;
  NewItem: TabcButtonListItem;
begin
  Accept := True;
  Source := Item.ButtonList;

  if Source <> Self then
  begin
    if Assigned(Source.FOnCopyItem) then
      Source.FOnCopyItem(Self, Item, Source, Self, Item.Index, NewIndex, Accept);
    if not Accept then Exit;
  end;

  if Assigned(FOnCopyItem) then
    FOnCopyItem(Self, Item, Source, Self, Item.Index, NewIndex, Accept);
  if not Accept then Exit;

  NewItem := TabcButtonListItem(Items.Insert(FDropTargetIndex));
  NewItem.Assign(Item);
  if Assigned(FAfterCopyItem) then
    FAfterCopyItem(Self, NewItem);
  LayoutChanged;
  ClearDragEffects;
  TabcCustomButtonList(Source).LayoutChanged;
  TabcCustomButtonList(Source).ClearDragEffects;
end;

procedure TabcCustomButtonList.DoMoveItem(Item: TabcButtonListItem; NewIndex: Integer);
var
  Accept: Boolean;
  Source: TabcCustomButtonList;
  NewItem: TabcButtonListItem;
begin
  Accept := True;
  Source := Item.ButtonList;
  if Source = Self then
  begin
    if Assigned(FOnMoveItem) then
      FOnMoveItem(Self, Item, Self, Self, Item.Index, NewIndex, Accept);
    if not Accept then Exit;
    Item.Index := NewIndex;
  end
  else
  begin
    if Assigned(Source.FOnMoveItem) then
      Source.FOnMoveItem(Self, Item, Source, Self, Item.Index, NewIndex, Accept);
    if not Accept then Exit;
    if Assigned(FOnMoveItem) then
      FOnMoveItem(Self, Item, Source, Self, Item.Index, NewIndex, Accept);
    if not Accept then Exit;
    NewItem := TabcButtonListItem(Items.Insert(FDropTargetIndex));
    NewItem.Assign(Item);
    Item.Free;
    LayoutChanged;
    ClearDragEffects;
    TabcCustomButtonList(Source).LayoutChanged;
    TabcCustomButtonList(Source).ClearDragEffects;
  end;
end;

procedure TabcCustomButtonList.DragDrop(Source: TObject; X, Y: Integer);
var
  DroppedItem: TabcButtonListItem;
  DestIndex: Integer;
begin
  inherited;
  if (Source is TabcCustomButtonList) and (ablAcceptItems in Options) and not Editing then
  begin
    DroppedItem := TabcCustomButtonList(Source).DraggedItem;
    if (DroppedItem = nil) or (FDropTargetIndex = -1) then
      Exit;
    if Source = Self then
      if FDropTargetIndex = 0 then
        DestIndex := 0
      else if FDraggedIndex >= FDropTargetIndex then
        DestIndex := FDropTargetIndex
      else
        DestIndex := FDropTargetIndex - 1
    else
      DestIndex := FDropTargetIndex;
    if TabcCustomButtonList(Source).DragCopy then
      DoCopyItem(DroppedItem, DestIndex)
    else
      DoMoveItem(DroppedItem, DestIndex);
  end;
end;

procedure TabcCustomButtonList.DragOver(Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
var
  lastDropTarget, i: Integer;
begin
  inherited;
  if Editing then Exit;
  if (Source is TabcCustomButtonList) then
    if (Source = Self) or (ablAcceptItems in Options) then
    begin
      Accept := False;
      if (Y <= 2 * FItemSpace) and FUpSBVisible then
      begin
        if not (Assigned(FScrollTimer) and FScrollTimer.Enabled) then
        begin
          FDropTargetIndex := -1;
          FDragLineY := -1;
          FUpSBDown := True;
          StartTimedScroll(150);
        end
      end
      else if (Y >= Height - 2 * FItemSpace) and FDownSBVisible then
      begin
        if not (Assigned(FScrollTimer) and FScrollTimer.Enabled) then
        begin
          FDropTargetIndex := -1;
          FDragLineY := -1;
          FDownSBDown := True;
          StartTimedScroll(150);
        end
      end
      else
      begin
        FUpSBDown := False;
        FDownSBDown := False;
        lastDropTarget := FDropTargetIndex;
        GetDropTargetAtPos(X, Y);
        Accept := (FDropTargetIndex <> -1);
        i := GetIndexAtPos(X, Y);
        if i <> ActiveIndex then
          ActiveIndex := i
        else if lastDropTarget <> FDropTargetIndex then
          Repaint;
        if Accept and (ablAllowDragCopy in TabcCustomButtonList(Source).Options) then
          //check Ctrl Key State
          TabcCustomButtonList(Source).DragCopy := GetKeyState(VK_CONTROL) and $80 <> 0;
      end;
    end;
end;

procedure TabcCustomButtonList.ClearDragEffects;
begin
  FDraggedIndex := -1;
  FDropTargetIndex := -1;
  FDragLineY := -1;
  FUpSBDown := False;
  FDownSBDown := False;
  DragCopy := False;
  Repaint;
end;

procedure TabcCustomButtonList.DoEndDrag(Target: TObject; X, Y: Integer);
begin
  inherited;
  ClearDragEffects;
end;

procedure TabcCustomButtonList.SetDragCopy(Value: Boolean);
const
  DRAG_CURS: array[Boolean] of TCursor = (crDrag, crMultiDrag);
begin
  if Value <> FDragCopy then
  begin
    FDragCopy := Value and (ablAllowDragCopy in Options);
    DragCursor := DRAG_CURS[FDragCopy];
  end;
end;

procedure TabcCustomButtonList.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  p: TPoint;
  i: Integer;
begin
  if not Editing then
  begin
    p := Point(X, Y);
    if (Button = mbLeft) then
    begin
      SetFocus;
      if FUpSBVisible and PtInRect(FUpSBRect, p) then
      begin
        FUpSBDown := True;
        DoDrawScrollButton(True, True);
        StartTimedScroll(200);
      end
      else if FDownSBVisible and PtInRect(FDownSBRect, p) then
      begin
        FDownSBDown := True;
        DoDrawScrollButton(False, True);
        StartTimedScroll(200);
      end
      else
      begin
        i := GetIndexAtPos(X, Y);
        if (i > -1) and ((i = ActiveIndex) or (i = SelectedIndex)) then
        begin
          FMouseDownAt := p;
          FDraggedIndex := i;
          if (i = ActiveIndex) then
            SelectedIndex := i;
        end;
      end;
    end
    else
      if (Button = mbRight) and (ablInPlaceEdit in Options) then
        if (ActiveIndex > -1) and PtInRect(ActiveItem.TextRect, p) then
          Edit(ActiveIndex)
        else if (SelectedIndex > -1) and PtInRect(SelectedItem.TextRect, p) then
          Edit(SelectedIndex);
  end;
  inherited;
end;

procedure TabcCustomButtonList.MouseMove(Shift: TShiftState; X,
  Y: Integer);
var
  i: Integer;
begin
  inherited;
  if FUpSBDown or FDownSBDown or Editing then
    Exit;
  i := GetIndexAtPos(X, Y);
  ActiveIndex := i;
  if (cslButtonDown in ControlState) and (ablAllowDrag in Options) and
     (FDraggedIndex > -1) and (abs(FMouseDownAt.X - X + FMouseDownAt.Y - Y) > FItemSpace) and not Dragging then
  begin
    {start dragging}
    FBeginDrag := True;
    BeginDrag(True, -1);
  end;
end;

procedure TabcCustomButtonList.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  wasSelected, mouseItem: TabcButtonListItem;
begin
  inherited;
  if not Editing and (Button = mbLeft) then
  begin
    if FUpSBDown then
      FUpSBDown := False
    else if FDownSBDown then
      FDownSBDown := False
    else if (SelectedIndex > -1) then
    begin
      wasSelected := SelectedItem;
      mouseItem := GetItemAtPos(X, Y);
      if (ablKeepSelected in Options) then
        FActiveIndex := -1
      else
        FSelectedIndex := -1;
      if FBeginDrag then
        FBeginDrag := False
      else
      begin
        FDraggedIndex := -1;
        if not (ablKeepSelected in Options) or (wasSelected = mouseItem) then
        begin
        if (wasSelected <> nil) then
          with wasSelected do
            if (FActionLink <> nil) then
              FActionLink.Execute;
          if Assigned(FOnClickItem) then
          FOnClickItem(Self, WasSelected);
        end;
      end;
    end
    else
      Exit;
    Repaint;
  end;
end;

procedure TabcCustomButtonList.ClickItem(Item: TabcButtonListItem);
var
  x, y: integer;
begin
  if Item = nil then Exit;
  with Item.FImageRect do
  begin
    x := (Left + Right) div 2;
    y := (Top + Bottom) div 2;
  end;
  MouseDown(mbLeft, [], x, y);
  MouseUp(mbLeft, [], x, y);
end;

procedure TabcCustomButtonList.DblClick;
var
  ptMouse: TPoint;
  i: integer;
begin
  inherited;
  if Assigned(OnDblClickItem) and HandleAllocated and (GetCapture = Handle) then
  begin
    GetCursorPos(ptMouse);
    ptMouse := ScreenToClient(ptMouse);
    i := GetIndexAtPos(ptMouse.X, ptMouse.Y);
    if (i <> -1) and Items[i].Enabled then
      FOnDblClickItem(Self, Items[i]);
  end;
end;

procedure TabcCustomButtonList.SetBounds(ALeft, ATop, AWidth,
  AHeight: Integer);
var
  bAdjustSelectedView: Boolean;
begin
  bAdjustSelectedView := Showing and (Height = 0) and (AHeight > 0) and (SelectedIndex > -1);
  inherited;
  LayoutChanged;
  if bAdjustSelectedView then
    ScrollItemInView(SelectedIndex, False);
end;

procedure TabcCustomButtonList.SetParent(AParent: TWinControl);
begin
  inherited;
  if not (csDestroying in ComponentState) then
    if AParent is TabcButtonPage then
      Align := alClient;
end;

procedure TabcCustomButtonList.CMHintShow(var Message: TMessage);
begin
  if (ActiveIndex > -1) and (ablFlyoutHints in Options) then
    with Message do
    begin
      if (ablWordWrap in Options) then
      begin
        {no clip so only show if bottom item partly showing}
        if (ActiveItem.TextRect.Top < Height) and (ActiveItem.TextRect.Bottom > Height) then
        with PHintInfo(LParam)^ do
        begin
          HintPos := ClientToScreen(ActiveItem.TextRect.TopLeft);
          HintMaxWidth := ActiveItem.TextRect.Right - ActiveItem.TextRect.Left;
          HintStr := ActiveItem.Caption;
        end
      end
      else if (Canvas.TextWidth(ActiveItem.Caption) >
        (Width - ActiveItem.TextRect.Left)) then
        {clipped}
        with PHintInfo(LParam)^ do
        begin
          HintPos := ClientToScreen(ActiveItem.TextRect.TopLeft);
          HintStr := ActiveItem.Caption;
        end;
      Result := 0;
    end;
end;

procedure TabcCustomButtonList.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if Shift = [] then
    case Key of
      VK_RIGHT, VK_DOWN:
        KeySelectNext(True);
      VK_LEFT, VK_UP:
        KeySelectNext(False);
      VK_RETURN, VK_SPACE:
        if ActiveItem <> nil then
          ClickItem(ActiveItem);
      VK_ESCAPE:
        SelectedIndex := -1;
      VK_DELETE:
        if (ablAllowDelete in Options) and not Dragging then
          DoDeleteItem(ActiveItem);
    end;
end;

procedure TabcCustomButtonList.KeySelectNext(GoForward: Boolean);
var
  NewIndex, SaveNew: Integer;
begin
  if (ActiveIndex = -1) then
    NewIndex := SelectedIndex
  else
    NewIndex := ActiveIndex;
  SaveNew := NewIndex;
  repeat
    if GoForward then
      NewIndex := NewIndex + 1
    else
      NewIndex := NewIndex - 1;
    if NewIndex >= Items.Count then
      NewIndex := 0
    else if NewIndex < 0 then
      NewIndex := Items.Count - 1;
  until (NewIndex = SaveNew) or Items[NewIndex].Enabled;
  if Items[NewIndex].Enabled then
  begin
    ActiveIndex := NewIndex;
    if ActiveIndex = (Items.Count - 1) then
      ScrollItemInView(ActiveIndex, False);
  end;
end;

procedure TabcCustomButtonList.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
 inherited;
 Message.Result := DLGC_WANTARROWS or DLGC_WANTALLKEYS;
end;

{$IFDEF DELPHI5}
procedure TabcCustomButtonList.WMContextMenu(var Message: TWMContextMenu);
begin
  inherited;
  ActiveIndex := -1;
end;
{$ENDIF}

procedure TabcCustomButtonList.CMDialogChar(var Message: TCMDialogChar);
var
  i: integer;
  pClient, pScreen: TPoint;
  shift: TShiftState;
begin
  if (ablAccelChars in Options) then
    with Message do
      if (ssAlt in KeyDataToShiftState(KeyData)) then
        for i := 0 to Items.Count - 1 do
          if IsAccel(CharCode, Items[i].Caption) then
          begin
            //activate parent button page if not selected
            if Parent is TabcButtonPage then
              if not TabcButtonPage(Parent).Activate then
              begin
                inherited;
                Exit;
              end;
            with Items[i].ImageRect do
            begin
              pClient.X := Left + (Right - Left) div 2;
              pClient.Y:= Top + (Bottom - Top) div 2;
              pScreen := ClientToScreen(pClient);
              SetCursorPos(pScreen.X, pScreen.Y);
              Application.ProcessMessages;
            end;
            shift := KeyDataToShiftState(Message.KeyData);
            MouseDown(mbLeft, shift, pClient.X, pClient.Y);
            MouseUp(mbLeft, shift, pClient.X, pClient.Y);
            Application.ProcessMessages;
            Exit;
          end;
  inherited;
end;

procedure TabcCustomButtonList.GetDropTargetAtPos(X, Y: Integer);
{updates FDropTargetIndex and FDragLineY
 Sets FDropTargetIndex to -1 ... Items.Count
      -1 means no drop
      value in range means insert before that item
      value at end means insert at end
 Sets FDragLineY to -1 if FDropTargetIndex is -1, else to line draw pos.
}
var
  i, bottomEdge: Integer;
begin
  bottomEdge := Height - FItemSpace;
  FDropTargetIndex := -1;
  if Items.Count > 0 then
  begin
    if (Y < FItemSpace) or (Y > bottomEdge) then
      FDropTargetIndex := -1
    else if Y < Items[FTopItemIndex].ImageRect.Top then
      FDropTargetIndex := 0
    else if Y > Items[Items.Count - 1].ImageRect.Bottom then
      FDropTargetIndex := Items.Count
    else
      for i := FTopItemIndex to Items.Count - 2 do
        if (Y >= Items[i].MaxY) and
           (Y <= Items[i+1].ImageRect.Top) then
        begin
          FDropTargetIndex := i+1;
          Break;
        end;
    {update line position}
    if FDropTargetIndex = -1 then
      FDragLineY := -1
    else if FDropTargetIndex = Items.Count then
      FDragLineY := Items[Items.Count - 1].MaxY + FItemSpace
    else
      FDragLineY := Items[FDropTargetIndex].ImageRect.Top - FItemSpace;
  end
  else
  begin
    FDropTargetIndex := 0;
    FDragLineY := FItemSpace;
  end;
end;

function TabcCustomButtonList.GetIndexAtPos(X, Y: Integer): Integer;
var
  i, bottomEdge: Integer;
  p: TPoint;
begin
  if not (csDestroying in ComponentState) then
  begin
    p := Point(X, Y);
    bottomEdge := Height - FItemSpace;
    for i := FTopItemIndex to Items.Count - 1 do
      with Items[i] do
        if PtInRect(ImageRect, p) or PtInRect(TextRect, p) then
        begin
          Result := i;
          Exit;
        end
        else if ImageRect.Top > bottomEdge then
          Break;
  end;
  Result := -1;
end;

function TabcCustomButtonList.GetItemAtPos(X, Y: Integer): TabcButtonListItem;
var
  i: Integer;
begin
  i := GetIndexAtPos(X, Y);
  if i = -1 then
    Result := nil
  else
    Result := Items[i];
end;

procedure TabcCustomButtonList.SetHighlightActive(Value: TabcSimpleFontHighlight);
begin
  FHighlightActive.Assign(Value);
end;

procedure TabcCustomButtonList.SetHighlightSelected(Value: TabcSimpleFontHighlight);
begin
  FHighlightSelected.Assign(Value);
end;

procedure TabcCustomButtonList.FontHighlightChanged(Sender: TObject);
begin
  Repaint;
end;

procedure TabcCustomButtonList.Paint;
var
  PaintRect, TextDrawRect: TRect;
  i: Integer;
begin
  if (csDestroying in ComponentState) then Exit;
  with Canvas do
  begin

    Brush.Color := Color;
    Brush.Style := bsSolid;
    Pen.Style := psSolid;
    Font := Self.Font;
    PaintRect := ClientRect;

    if not Transparent then
    begin
      FillRect(PaintRect);
      Background.Draw(Canvas, PaintRect);
    end;

    Brush.Style := bsClear;

    { Clip remaining drawing to inner rect }

    InflateRect(PaintRect, - FItemSpace, - FItemSpace);

    {Draw Visible Items}

    for i := FTopItemIndex to Items.Count - 1 do
    begin
      if (Items[i].ImageRect.Top > PaintRect.Bottom) then
        Break;
      DoDrawImage(i, Items[i].ImageRect, (i = FActiveIndex), (i = FSelectedIndex));
      if not (ablHideCaptions in Options) then
      begin
        if (Items[i].TextRect.Top > PaintRect.Bottom) then
          Break;
        TextDrawRect := Items[i].TextRect;
        DoDrawText(i, TextDrawRect, Self.TextFlags, (i = FActiveIndex), (i = FSelectedIndex));
      end;
    end;

    { Draw Drag Line }
    if FDragLineY <> -1 then
      DoDrawDragLineAt(FDragLineY, PaintRect);

    {Draw Scroll Buttons}

    if FUpSBVisible then
      DoDrawScrollButton(True, FUpSBDown);

    if FDownSBVisible then
      DoDrawScrollButton(False, FDownSBDown);

    {designtime border}
    if csDesigning in ComponentState then
    begin
      Pen.Style := psDash;
      Pen.Color := clBlack;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  end;
end;

procedure TabcCustomButtonList.DoDrawDragLineAt(Pos: Integer; Rect: TRect);
var
  pts: array[0..2] of TPoint;
begin
  with Canvas do
  begin
    with Pen do
    begin
      Color := FDragLineColor;
      Mode := pmCopy;
      Style := psSolid;
      Width := 1;
    end;
    MoveTo(Rect.Left, Pos);
    LineTo(Rect.Right, Pos);

    {draw line ends}
    with Brush do
    begin
      Color := FDragLineColor;
      Style := bsSolid;
    end;
    pts[0] := Point(Rect.Left+10, Pos);
    if FDropTargetIndex = 0 then
      pts[1] := Point(Rect.Left, Pos)
    else
      pts[1] := Point(Rect.Left, Pos-5);
    if (FDropTargetIndex = Items.Count) and (FDropTargetIndex <> 0) then
      pts[2] := Point(Rect.Left, Pos)
    else
      pts[2] := Point(Rect.Left, Pos+5);
    Polygon(pts);
    pts[0] := Point(Rect.Right-10, Pos);
    if FDropTargetIndex = 0 then
      pts[1] := Point(Rect.Right, Pos)
    else
      pts[1] := Point(Rect.Right, Pos-5);
    if (FDropTargetIndex = Items.Count) and (FDropTargetIndex <> 0) then
      pts[2] := Point(Rect.Right, Pos)
    else
      pts[2] := Point(Rect.Right, Pos+5);
    Polygon(pts);
  end;
end;

procedure TabcCustomButtonList.DoDrawScrollButton(IsUpSB, IsDown: Boolean);
const
  UpDownFlags: array[Boolean] of Integer = (DFCS_SCROLLDOWN, DFCS_SCROLLUP);
  PushedFlags: array[Boolean] of Integer = (0, DFCS_PUSHED);
  EnabledFlags: array[Boolean] of Integer = (DFCS_INACTIVE, 0);
  StyleFlags: array[TabcScrollButtonStyle] of Integer = (0, DFCS_FLAT, DFCS_MONO);
var
  FrameRect: TRect;
  Flags: Integer;
begin
  if IsUpSB then
    FrameRect := FUpSBRect
  else
    FrameRect := FDownSBRect;
  Flags := UpDownFlags[IsUpSB] + PushedFlags[IsDown] + EnabledFlags[Enabled] + StyleFlags[ScrollButtonStyle];
  DrawFrameControl(Canvas.Handle, FrameRect, DFC_SCROLL, Flags);
end;

function GetRGBColor(Value: TColor): DWORD;
begin
  Result := ColorToRGB(Value);
  case Result of
    clNone: Result := CLR_NONE;
    clDefault: Result := CLR_DEFAULT;
  end;
end;

procedure TabcCustomButtonList.DoDrawImage(Index: Integer; Rect: TRect; Active, Selected: Boolean);
const
  //replaced ILD_SELECTED with ILD_BLEND50 for improved icon visibility when disabled
  DrawingStyles: array[Boolean] of Longint = (ILD_BLEND50, ILD_TRANSPARENT);
  BlendColors: array[Boolean] of TColor = (clGrayText, clNone);
var
  DrawR: TRect;
  bItemEnabled: Boolean;
begin
  if not Assigned(FDrawImageList) or (Items[Index].ImageIndex = -1) then
    Exit;
  DrawR := Rect;
  if Active or Selected then
  begin
    if ViewStyle = abvSmallIcon then
      InflateRect(DrawR, -1, -1);
    if Selected then
      Frame3D(Canvas, DrawR, FrameBottomColor, FrameTopColor, 1)
    else if Active then
      Frame3D(Canvas, DrawR, FrameTopColor, FrameBottomColor, 1);
  end;
  DrawR := Rect;
  InflateRect(DrawR, -C_FRAME_OFFSET, -C_FRAME_OFFSET);
  //draw with changed drawing style in image list, without causing recursive imagelist change event
  bItemEnabled := Enabled and Items[Index].Enabled;
  with FDrawImageList do
    if HandleAllocated then
      ImageList_DrawEx(Handle, Items[Index].ImageIndex, Canvas.Handle,
        DrawR.Left, DrawR.Top, 0, 0,
        GetRGBColor(BkColor), GetRGBColor(BlendColors[bItemEnabled]),
        DrawingStyles[bItemEnabled]);
end;

procedure TabcCustomButtonList.DoDrawText(Index: Integer; var Rect: TRect; Flags: DWORD; Active, Selected: Boolean);
var
  Text, SaveText: string;
  bItemEnabled: Boolean;
begin
  Text := Items[Index].Caption;
  if (ablHideAccelChars in Options) then
    Text := abcReplaceAll('&', '', Text, False, False);
  if (Flags and DT_CALCRECT <> 0) and ((Text = '') or (ablAccelChars in Options) and
    (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';
  SaveText := Text;
  Flags := DrawTextBiDiModeFlags(Flags);
  Canvas.Font := Font;
  bItemEnabled := Enabled and Items[Index].Enabled;
  if not bItemEnabled then
    if (Color = clBtnShadow) or (Color = clGrayText) then
      Canvas.Font.Color := clBtnFace
    else
      Canvas.Font.Color := clGrayText
  else if Active then
  begin
    if HighlightActive.FontColor <> clNone then
      Canvas.Font.Color := HighlightActive.FontColor;
    if HighlightActive.Underline then
      Canvas.Font.Style := Canvas.Font.Style + [fsUnderline];
  end
  else if Selected then
  begin
    if HighlightSelected.FontColor <> clNone then
      Canvas.Font.Color := HighlightSelected.FontColor;
    if HighlightSelected.Underline then
      Canvas.Font.Style := Canvas.Font.Style + [fsUnderline];
  end;
  DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
end;

procedure TabcCustomButtonList.ScrollTimerExpired(Sender: TObject);
begin
  if FDownSBDown then
    ScrollItems(False)
  else if FUpSBDown then
    ScrollItems(True)
  else
    FScrollTimer.Enabled := False;
end;

procedure TabcCustomButtonList.StopTimedScroll;
begin
  if Assigned(FScrollTimer) then
    FScrollTimer.Enabled := False;
end;

procedure TabcCustomButtonList.StartTimedScroll(AnInterval: Integer);
begin
  if not Assigned(FScrollTimer) then
  begin
    FScrollTimer := TTimer.Create(Self);
    FScrollTimer.OnTimer := ScrollTimerExpired;
  end;
  ScrollTimerExpired(Self);
  with FScrollTimer do
  begin
    Interval := AnInterval;
    Enabled := True;
  end;
end;

procedure TabcCustomButtonList.SetTopItemIndex(Value: Integer);
var
  i, VOffset: Integer;
begin
  if Value <> FTopItemIndex then
  begin
    VOffset := Items[FTopItemIndex].ImageRect.Top - Items[Value].ImageRect.Top;
    for i := 0 to Items.Count - 1 do
      with Items[i] do
      begin
        OffsetRect(FImageRect, 0, VOffset);
        OffsetRect(FTextRect, 0, VOffset);
      end;
    FTopItemIndex := Value;
  end;
end;

procedure TabcCustomButtonList.ScrollItems(ScrollUp: Boolean);
begin
  if ScrollUp then
    if FUpSBVisible then
      TopItemIndex := TopItemIndex - 1
    else
      Exit
  else
    if FDownSBVisible then
      TopItemIndex := TopItemIndex + 1
    else
      Exit;
  UpdateScrollButtonState;
  Repaint;
end;

procedure TabcCustomButtonList.UpdateScrollButtonState;
begin
  if (csDestroying in ComponentState) then Exit;
  if Items.Count = 0 then
    FDownSBVisible := False
  else
    with Items[Items.Count - 1] do
      FDownSBVisible :=
        MaxY > (Height - FItemSpace);

  if not FDownSBVisible then
    FDownSBDown := False;

  FUpSBVisible := FTopItemIndex > 0;

  if not FUpSBVisible then
    FUpSBDown := False;
end;

procedure TabcCustomButtonList.ScrollItemInView(Index: Integer; Partial: Boolean);
begin
  if (csDestroying in ComponentState) then Exit;
  if (csLoading in ComponentState) or not Showing or (Height = 0) then Exit;
  if (Index > -1) and (Index < Items.Count) then
    if Index < TopItemIndex then
      TopItemIndex := Index
    else if Partial then
      while not IsItemPartlyVisible(Index) do
        TopItemIndex := TopItemIndex + 1
    else
      while not IsItemFullyVisible(Index) do
        TopItemIndex := TopItemIndex + 1;
  UpdateScrollButtonState;
  Repaint;
end;

function TabcCustomButtonList.IsItemFullyVisible(Index: Integer): Boolean;
begin
  Result := (Index > -1) and (Index < Items.Count) and (Index >= TopItemIndex) and
    (Items[Index].MaxY < (Height - FItemSpace));
end;

function TabcCustomButtonList.IsItemPartlyVisible(Index: Integer): Boolean;
begin
  Result := (Index > -1) and (Index < Items.Count) and (Index >= TopItemIndex) and
    (Items[Index].ImageRect.Top < (Height - FItemSpace));
end;

procedure TabcCustomButtonList.LayoutChanged;
const
  DefaultImageSizes: array[TabcButtonListViewStyle] of Integer = (32,16);
  TextOffset = 2;
  Alignments: array[TabcButtonListViewStyle] of DWORD = (DT_CENTER, DT_LEFT);
  WordWraps: array[Boolean] of DWORD =
    (DT_END_ELLIPSIS or DT_SINGLELINE, DT_WORDBREAK);
  AccelChars: array[Boolean] of DWORD = (DT_NOPREFIX, 0);
var
  i, ImageLeft, TextLeft, ImageWidth, ImageHeight, VOffset,
  VScrollW, VScrollH: Integer;
  PaintRect, CalcImageRect, CalcTextRect: TRect;
begin
  if [csReading, csLoading, csDestroying] * ComponentState <> [] then Exit;

  if ActiveIndex >= Items.Count then
    FActiveIndex := -1;

  if SelectedIndex >= Items.Count then
    FSelectedIndex := -1;

  if (ActiveIndex > -1) and not Items[ActiveIndex].Enabled then
    ActiveIndex := -1;

  if (SelectedIndex > -1) and not Items[SelectedIndex].Enabled then
    SelectedIndex := -1;

  {recalculate item rects}

  PaintRect := Rect(0,0,Width,Height);
  InflateRect(PaintRect, -FItemSpace, -FItemSpace);

  VScrollW := GetSystemMetrics(SM_CXVSCROLL);
  VScrollH := GetSystemMetrics(SM_CYVSCROLL);

  FUpSBRect := PaintRect;
  with FUpSBRect do
  begin
    Left := Right - VScrollW;
    Bottom := Top + VScrollH;
  end;

  FDownSBRect := PaintRect;
  with FDownSBRect do
  begin
    Left := Right - VScrollW;
    Top := Bottom - VScrollH;
  end;

  if (ViewStyle = abvSmallIcon) and Assigned(SmallImages) then
    FDrawImageList := SmallImages
  else
    FDrawImageList := LargeImages;

  if Assigned(FDrawImageList) then
  begin
    ImageWidth := FDrawImageList.Width + C_FRAME_OFFSET * 2;
    ImageHeight := FDrawImageList.Height + C_FRAME_OFFSET * 2;
  end
  else
  begin
    ImageWidth := DefaultImageSizes[ViewStyle]+ C_FRAME_OFFSET * 2;
    ImageHeight := ImageWidth;
  end;

  if (ViewStyle = abvIcon) then
  begin
    ImageLeft := (PaintRect.Right - PaintRect.Left - ImageWidth) div 2;
    TextLeft := PaintRect.Left;
  end
  else
  begin
    ImageLeft := FItemSpace + 1;
    TextLeft := ImageLeft + ImageWidth + FItemSpace;
  end;

  FTextFlags := DT_EXPANDTABS or
                Alignments[ViewStyle] or WordWraps[ablWordWrap in Options] or
                AccelChars[ablAccelChars in Options];

  CalcImageRect := Rect(ImageLeft, 0, ImageLeft + ImageWidth, 0);
  CalcTextRect := Rect(TextLeft, 0, PaintRect.Right - PaintRect.Left, 0);

  for i := 0 to Items.Count - 1 do
  begin
    if (ViewStyle = abvIcon) then
    begin
      with CalcImageRect do
      begin
        if not (ablHideCaptions in Options) then
          Top := CalcTextRect.Bottom + 2 * FItemSpace
        else
          Top := Bottom + 2 * FItemSpace;
        Bottom := Top + ImageHeight;
      end;
      with CalcTextRect do
      begin
        Top := CalcImageRect.Bottom;
        Left := TextLeft;
        Right := PaintRect.Right - PaintRect.Left;
        Bottom := Top;
      end;
    end
    else
    begin
      with CalcImageRect do
      begin
        Top := Max(CalcImageRect.Bottom, CalcTextRect.Bottom) + 2 * FItemSpace;
        Bottom := Top + ImageHeight;
      end;

      if not (ablHideCaptions in Options) then
        with CalcTextRect do
        begin
          Top := CalcImageRect.Top;
          Left := TextLeft;
          Right := PaintRect.Right - PaintRect.Left;
          Bottom := Top;
        end;

    end;

    if not (ablHideCaptions in Options) then
    begin
      DoDrawText(i, CalcTextRect, FTextFlags or DT_CALCRECT, False, False);
      {center text}
      if (ViewStyle = abvIcon) then
        with CalcTextRect do
          if Right < PaintRect.Right then
            OffsetRect(CalcTextRect, (PaintRect.Right - PaintRect.Left - Right) div 2, 0);
    end;
    with Items[i] do
    begin
      FImageRect := CalcImageRect;
      FTextRect := CalcTextRect;
    end;
  end;

  { offset image and text rects for scrolled item }

  if FTopItemIndex > 0 then
  begin
    if FTopItemIndex < Items.Count then
    begin
      VOffset := 2 * FItemSpace - Items[FTopItemIndex].ImageRect.Top;
      for i := 0 to Items.Count - 1 do
        with Items[i] do
        begin
          OffsetRect(FImageRect, 0, VOffset);
          OffsetRect(FTextRect, 0, VOffset);
        end;
    end
    else
      FTopItemIndex := 0;
  end;

  UpdateScrollButtonState;

  Repaint;
end;

procedure TabcCustomButtonList.SetScrollButtonStyle(
  const Value: TabcScrollButtonStyle);
begin
  if FScrollButtonStyle <> Value then
  begin
    FScrollButtonStyle := Value;
    Repaint;
  end;
end;

procedure TabcCustomButtonList.AlignControls(AControl: TControl;
  var Rect: TRect);
begin
  inherited;
  if Transparent then Invalidate;
end;

procedure TabcCustomButtonList.Invalidate;
var
 R: TRect;
 i: Integer;
begin
  if Transparent and (Parent <> nil) and Parent.HandleAllocated then
  begin
    R := BoundsRect;
    InvalidateRect(Parent.Handle, @R, True);
    for i := 0 to ControlCount - 1 do
      Controls[i].Invalidate;
  end;
  inherited;
end;

procedure TabcCustomButtonList.SetTransparent(const Value: Boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    if Value then
      ControlStyle := ControlStyle - [csOpaque]
    else
      ControlStyle := ControlStyle + [csOpaque];
    if HandleAllocated then RecreateWnd;
    if not (csDestroying in ComponentState) then Invalidate;
  end;
end;

procedure TabcCustomButtonList.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  if not Transparent then
    inherited
  else
    Message.Result := 1;
end;

procedure TabcCustomButtonList.WMMove(var Message: TWMMove);
begin
  inherited;
  if Transparent then Invalidate;
end;

{ TabcButtonListItem }

constructor TabcButtonListItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FEnabled := True;
  FImageIndex := -1;
  FSoundOver := -1;
  FSoundDown := -1;
end;

destructor TabcButtonListItem.Destroy;
begin
  if FActionLink <> nil then
    FActionLink.Free;
  inherited;
end;

procedure TabcButtonListItem.Assign(Source: TPersistent);
begin
  if Source is TabcButtonListItem then
    with TabcButtonListItem(Source) do
    begin
      Self.Action := Action;
      Self.FCaption := Caption;
      Self.FData := Data;
      Self.FEnabled := Enabled;
      Self.FImageIndex := ImageIndex;
      Self.FLink := Link;
      Self.FSoundOver := SoundOver;
      Self.FSoundDown := SoundDown;
      Self.Tag := Tag;
    end
  else
    inherited Assign(Source);
end;

function TabcButtonListItem.GetButtonList: TabcCustomButtonList;
var
  collList: TabcButtonListItems;
begin
  collList := GetCollectionAsList;
  if collList <> nil then
    Result := collList.ButtonList
  else
    Result := nil;
end;

function TabcButtonListItem.GetCollectionAsList: TabcButtonListItems;
begin
  if Collection is TabcButtonListItems then
    Result := TabcButtonListItems(Collection)
  else
    Result := nil;
end;

function TabcButtonListItem.GetDisplayName: string;
begin
  Result := Caption;
end;

procedure TabcButtonListItem.SetDisplayName(const Value: string);
begin
  Caption := Value;
end;

procedure TabcButtonListItem.SetCaption(Value: string);
begin
  FCaption := Value;
  Changed(True);
end;

procedure TabcButtonListItem.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    Changed(False);
  end;
end;

procedure TabcButtonListItem.SetImageIndex(Value: TImageIndex);
begin
  FImageIndex := Value;
  Changed(True);
end;

function TabcButtonListItem.MaxY: Integer;
begin
  Result := Max(TextRect.Bottom, ImageRect.Bottom);
end;

procedure TabcButtonListItem.Move(Down: Boolean);
var
  newIndex: integer;
begin
  if Collection <> nil then
  begin
    newIndex := Index;
    if Down then
    begin
      newIndex := newIndex + 1;
      if newIndex < Collection.Count then
        Index := newIndex;
    end
    else
    begin
      newIndex := newIndex - 1;
      if newIndex >= 0 then
        Index := newIndex;
    end;
  end;
end;

{ TabcButtonListItems }

constructor TabcButtonListItems.Create(AList: TabcCustomButtonList);
begin
  inherited Create(TabcButtonListItem);
  FList := AList;
end;

function TabcButtonListItems.Add: TabcButtonListItem;
begin
  Result := TabcButtonListItem(inherited Add);
end;

function TabcButtonListItems.GetItem(Index: Integer): TabcButtonListItem;
begin
  Result := TabcButtonListItem(inherited GetItem(Index));
end;

function TabcButtonListItems.GetOwner: TPersistent;
begin
  Result := FList;
end;

procedure TabcButtonListItems.SetItem(Index: Integer;
  Value: TabcButtonListItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TabcButtonListItems.Update(Item: TCollectionItem);
begin
  if FList <> nil then
    if Item = nil then
      FList.LayoutChanged
    else
      FList.Repaint;
end;

{ TabcSimpleFontHighlight }

constructor TabcSimpleFontHighlight.Create;
begin
  FFontColor := clNone;
end;

procedure TabcSimpleFontHighlight.Assign(Source: TPersistent);
begin
  if Source is TabcSimpleFontHighlight then
    with TabcSimpleFontHighlight(Source) do
    begin
      Self.FFontColor := FontColor;
      Self.FUnderline := Underline;
      if Assigned(FOnChange) then
        FOnChange(Self);
    end
  else
    inherited Assign(Source);
end;

procedure TabcSimpleFontHighlight.SetFontColor(Value: TColor);
begin
  if Value <> FFontColor then
  begin
    FFontColor := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TabcSimpleFontHighlight.SetUnderline(Value: Boolean);
begin
  if Value <> FUnderline then
  begin
    FUnderline := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

function TabcButtonListItem.IsCaptionStored: Boolean;
begin
  Result := (FActionLink = nil) or not TabcButtonListItemActionLink(FActionLink).IsCaptionLinked;
end;

function TabcButtonListItem.IsImageIndexStored: Boolean;
begin
  Result := (FActionLink = nil) or not TabcButtonListItemActionLink(FActionLink).IsImageIndexLinked;
end;

procedure TabcButtonListItem.ActionChange(Sender: TObject);
begin
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
    begin
      Self.Caption := Caption;
      Self.ImageIndex := ImageIndex;
    end;
end;

function TabcButtonListItem.GetAction: TBasicAction;
begin
  if FActionLink <> nil then
    Result := FActionLink.Action
  else
    Result := nil;
end;

procedure TabcButtonListItem.SetAction(Value: TBasicAction);
begin
  if Value = nil then
  begin
    FActionLink.Free;
    FActionLink := nil;
  end
  else
  begin
    if FActionLink = nil then
      FActionLink := TabcButtonListItemActionLink.Create(ButtonList);
    FActionLink.Action := Value;
    FActionLink.OnChange := ActionChange;
    ActionChange(Value);
    Value.FreeNotification(ButtonList);
  end;
end;

procedure TabcButtonListItemActionLink.AssignClient(AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TabcCustomButtonList;
  UpdateItem;
end;

procedure TabcButtonListItemActionLink.UpdateItem;
var
  i: Integer;
begin
  if FClient <> nil then
    with FClient.Items do
      for i := 0 to Count - 1 do
        if Items[i].FActionLink = Self then
          FItem := Items[i];
end;

function TabcButtonListItemActionLink.IsCaptionLinked: Boolean;
begin
  if FItem = nil then
    UpdateItem;
  Result := inherited IsCaptionLinked and
    (FItem <> nil) and (FItem.Caption = (Action as TCustomAction).Caption);
end;

function TabcButtonListItemActionLink.IsEnabledLinked: Boolean;
begin
  if FItem = nil then
    UpdateItem;
  Result := inherited IsEnabledLinked and
    (FItem <> nil) and (FItem.Enabled = (Action as TCustomAction).Enabled);
end;

function TabcButtonListItemActionLink.IsImageIndexLinked: Boolean;
begin
  if FItem = nil then
    UpdateItem;
  Result := inherited IsImageIndexLinked and
    (FItem <> nil) and (FItem.ImageIndex = (Action as TCustomAction).ImageIndex);
end;

procedure TabcButtonListItemActionLink.SetCaption(const Value: string);
begin
  if IsCaptionLinked then
    FItem.Caption := Value;
end;

procedure TabcButtonListItemActionLink.SetEnabled(Value: Boolean);
begin
  if IsEnabledLinked then
    FItem.Enabled := Value;
end;

procedure TabcButtonListItemActionLink.SetImageIndex (Value: Integer);
begin
  if IsImageIndexLinked then
    FItem.ImageIndex := Value;
end;

end.
