unit abcbtn;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{$S-,W-,R-}
{$C PRELOAD}

interface

uses Windows, Messages, SysUtils, Classes, Controls, Menus, Graphics,
  StdCtrls, ExtCtrls, Forms, Buttons, CommCtrl, ImgList, ActnList,
  abcexctl, abcutilg, abcwav;

type

  {$IFNDEF DELPHI5}
    TImageIndex = type Integer;
  {$ENDIF}

  TabcButtonStyle = (absAutoDetect, absNew, absWin31, absThin, absThinBlack,
                     absThinGray,
                     absThinHighlight, absMidHighlight, absThickHighlight,
                     absFramed, absFramedBlack, absFramedRaised,
                     absFramedHighlight, absFramedBlackHighlight,
                     absRecessed, absRecessedBlack, absRecessedRaised,
                     absRecessedHighlight, absRecessedBlackHighlight);

  TabcButtonLayout = (ablGlyphLeft, ablGlyphRight, ablGlyphTop, ablGlyphBottom,
                      ablGlyphCenter, ablGlyphTiled);

  TabcGlyphHighlight = (aghNormal, aghGray, aghFade, aghDarken);

  TabcButtonState = (absUp, absDisabled, absDown, absExclusive, absMouseOver);

  TabcNumGlyphs = 1..5;

  { TabcPicBtnActionLink }

  TabcPicBtn = class;

  TabcPicBtnActionLink = class(TControlActionLink)
  protected
    FClient: TabcPicBtn;
    procedure AssignClient (AClient: TObject); override;
    function IsImageIndexLinked: Boolean; override;
    procedure SetImageIndex (Value: Integer); override;
  end;

  TabcPicBtnActionLinkClass = class of TabcPicBtnActionLink;

  { TabcPicBtn }

  TabcPicBtn = class(TButton)
  private
    FCanvas: TCanvas;
    FDisabledShadow: Boolean;
    FGlyph: Pointer;
    FIsFocused: Boolean;
    FLayout: TabcButtonLayout;
    FMargin: Integer;
    FMouseOver: Boolean;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FPictureHighlight: TabcGlyphHighlight;
    FPopup: Boolean;
    FSpacing: Integer;
    FStyle: TabcButtonStyle;
    FTextStyle: TabcTextStyle;
    FSoundList: TabcCustomWaveList;
    FSoundDown: Integer;
    FSoundOver: Integer;
    function HasMouseEffect: Boolean;
    procedure DrawItem(const DrawItemStruct: TDrawItemStruct);
    function GetImageIndex: TImageIndex;
    function GetImages: TCustomImageList;
    function  GetNumGlyphs: TabcNumGlyphs;
    function GetPicture: TPicture;
    function GetTransparentColor: TColor;
    procedure GlyphChanged(Sender: TObject);
    function IsImageIndexStored: Boolean;
    function IsTransColorStored: Boolean;
    function MouseIsOver: Boolean;
    procedure SetDisabledShadow(Value: Boolean);
    procedure SetImageIndex (Value: TImageIndex);
    procedure SetImages (Value: TCustomImageList);
    procedure SetLayout(Value: TabcButtonLayout);
    procedure SetMargin(Value: Integer);
    procedure SetNumGlyphs(Value: TabcNumGlyphs);
    procedure SetPicture(Value: TPicture);
    procedure SetPictureHighlight(Value: TabcGlyphHighlight);
    procedure SetPopup(Value: Boolean);
    procedure SetSpacing(Value: Integer);
    procedure SetStyle(Value: TabcButtonStyle);
    procedure SetTextStyle(Value:TabcTextStyle);
    procedure SetTransparentColor(Value: TColor);
    procedure UpdateMouseOver;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
  protected
    procedure ActionChange (Sender: TObject; CheckDefaults: Boolean); override;
    function GetActionLinkClass: TControlActionLinkClass; override;
    procedure CreateHandle; override;
    procedure CreateParams(var Params: TCreateParams); override;
    function  GetPalette: HPALETTE; override;
    procedure MouseEnter; dynamic;
    procedure MouseLeave; dynamic;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetButtonStyle(ADefault: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property MouseOver: Boolean read FMouseOver;
  published
    property TextStyle : TabcTextStyle read FTextStyle write SetTextStyle default tsNormal;
    property Picture: TPicture read GetPicture write SetPicture;
    property PictureHighlight: TabcGlyphHighlight
      read FPictureHighlight write SetPictureHighlight default aghNormal;
    property Popup: Boolean read FPopup write SetPopup default False;
    property TransparentColor: TColor
      read GetTransparentColor write SetTransparentColor stored IsTransColorStored;
    property Cancel;
    property Caption;
    property Color default clBtnFace;
    property Default;
    property DisabledShadow: Boolean read FDisabledShadow write SetDisabledShadow;
    property Enabled;
    property ImageIndex: TImageIndex read GetImageIndex write SetImageIndex
      stored IsImageIndexStored default -1;
    property Images: TCustomImageList read GetImages write SetImages;
    property Layout: TabcButtonLayout read FLayout write SetLayout default ablGlyphLeft;
    property Margin: Integer read FMargin write SetMargin default -1;
    property ModalResult;
    property NumGlyphs: TabcNumGlyphs read GetNumGlyphs write SetNumGlyphs default 1;
    property ParentColor default False;
    property ParentShowHint;
    property ShowHint;
    property SoundDown: Integer read FSoundDown write FSoundDown default -1;
    property SoundList: TabcCustomWaveList read FSoundList write FSoundList;
    property SoundOver: Integer read FSoundOver write FSoundOver default -1;
    property Style: TabcButtonStyle read FStyle write SetStyle default absAutoDetect;
    property Spacing: Integer read FSpacing write SetSpacing default 4;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnEnter;
    property OnExit;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

  { TabcPicSpeedBtnActionLink }

  TabcCustomPicSpeedBtn = class;

  TabcPicSpeedBtnActionLink = class(TControlActionLink)
  protected
    FClient: TabcCustomPicSpeedBtn;
    procedure AssignClient (AClient: TObject); override;
    function IsCheckedLinked: Boolean; override;
    function IsImageIndexLinked: Boolean; override;
    procedure SetChecked (Value: Boolean); override;
    procedure SetImageIndex (Value: Integer); override;
  end;

  TabcPicSpeedBtnActionLinkClass = class of TabcPicSpeedBtnActionLink;

  { TabcCustomPicSpeedBtn }

  TabcCancelEvent = procedure (Sender:TObject; var Cancel: Boolean) of Object;
  TabcMouseCancelEvent = procedure(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer; var Cancel: Boolean) of object;

  TabcCustomPicSpeedBtn = class(TGraphicControl)
  private
    FAllowAllUp: Boolean;
    FBeforeClick: TabcCancelEvent;
    FBeforeMouseDown: TabcMouseCancelEvent;
    FBrushPattern: Pointer;
    FDisabledShadow: Boolean;
    FDown: Boolean;
    FDragging: Boolean;
    FGlyph: Pointer;
    FGroupIndex: Integer;
    FLayout: TabcButtonLayout;
    FMargin: Integer;
    FMouseCancel: Boolean;
    FMouseOver: Boolean;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FPictureHighlight: TabcGlyphHighlight;
    FPopup: Boolean;
    FSpacing: Integer;
    FStyle: TabcButtonStyle;
    FTextStyle: TabcTextStyle;
    FSoundList: TabcCustomWaveList;
    FSoundDown: Integer;
    FSoundOver: Integer;
    FShowFocus: Boolean;
    function HasMouseEffect: Boolean;
    function CancelBeforeMouseDown(var Message: TWMMouse; Button: TMouseButton;
      Shift: TShiftState): Boolean;
    function GetImageIndex: TImageIndex;
    function GetImages: TCustomImageList;
    function GetNumGlyphs: TabcNumGlyphs;
    function GetPicture: TPicture;
    function GetTransparentColor: TColor;
    procedure GlyphChanged(Sender: TObject);
    function IsCheckedStored: Boolean;
    function IsImageIndexStored: Boolean;
    function IsTransColorStored: Boolean;
    function MouseIsOver: Boolean;
    procedure ReleaseBrushPattern;
    procedure SetAllowAllUp(Value: Boolean);
    procedure SetDisabledShadow(Value: Boolean);
    procedure SetDown(Value: Boolean);
    procedure SetGroupIndex(Value: Integer);
    procedure SetImageIndex (Value: TImageIndex);
    procedure SetImages (Value: TCustomImageList);
    procedure SetLayout(Value: TabcButtonLayout);
    procedure SetMargin(Value: Integer);
    procedure SetNumGlyphs(Value: TabcNumGlyphs);
    procedure SetPicture(Value: TPicture);
    procedure SetPictureHighlight(Value: TabcGlyphHighlight);
    procedure SetPopup(Value: Boolean);
    procedure SetShowFocus(Value: Boolean);
    procedure SetSpacing(Value: Integer);
    procedure SetStyle(Value: TabcButtonStyle);
    procedure SetTextStyle(Value:TabcTextStyle);
    procedure SetTransparentColor(Value: TColor);
    procedure UpdateExclusive;
    procedure UpdateMouseOver;
    procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure WMLButtonDblClk(var Message: TWMLButtonDown); message WM_LBUTTONDBLCLK;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMMButtonDown(var Message: TWMMButtonDown); message WM_MBUTTONDOWN;
    procedure WMRButtonDown(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
    procedure WMNCLButtonDown(var Message: TWMNCLButtonDown); message WM_NCLBUTTONDOWN;
    procedure WMNCRButtonDown(var Message: TWMNCRButtonDown); message WM_NCRBUTTONDOWN;
  protected
    FState: TabcButtonState;
    procedure ActionChange (Sender: TObject; CheckDefaults: Boolean); override;
    function GetActionLinkClass: TControlActionLinkClass; override;
    procedure AssignTo (Dest: TPersistent); override;
    function GetPalette: HPALETTE; override;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseEnter; dynamic;
    procedure MouseLeave; dynamic;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Paint; override;
    procedure ValidClick; dynamic;

    property MouseOver: Boolean read FMouseOver;
    property State: TabcButtonState read FState;

    {for descendant in TabcButtonBar}
    property ShowFocus: Boolean read FShowFocus write SetShowFocus default False;

    {publishable properties}
    property AllowAllUp: Boolean read FAllowAllUp write SetAllowAllUp default False;
    property GroupIndex: Integer read FGroupIndex write SetGroupIndex default 0;
    property Down: Boolean read FDown write SetDown
      stored IsCheckedStored default False;
    property Color default clBtnFace;
    property DisabledShadow: Boolean read FDisabledShadow write SetDisabledShadow default True;
    property ImageIndex: TImageIndex read GetImageIndex write SetImageIndex
      stored IsImageIndexStored default -1;
    property Images: TCustomImageList read GetImages write SetImages;
    property Layout: TabcButtonLayout read FLayout write SetLayout default ablGlyphLeft;
    property Margin: Integer read FMargin write SetMargin default -1;
    property ParentColor default False;
    property Picture: TPicture read GetPicture write SetPicture;
    property PictureHighlight: TabcGlyphHighlight
      read FPictureHighlight write SetPictureHighlight default aghNormal;
    property NumGlyphs: TabcNumGlyphs read GetNumGlyphs write SetNumGlyphs default 1;   {after picture!}
    property Popup: Boolean read FPopup write SetPopup default True;
    property SoundDown: Integer read FSoundDown write FSoundDown default -1;
    property SoundList: TabcCustomWaveList read FSoundList write FSoundList;
    property SoundOver: Integer read FSoundOver write FSoundOver default -1;
    property TransparentColor: TColor
      read GetTransparentColor write SetTransparentColor stored IsTransColorStored;
    property Spacing: Integer read FSpacing write SetSpacing default 4;
    property Style: TabcButtonStyle read FStyle write SetStyle default absThin;
    property TextStyle : TabcTextStyle read FTextStyle write SetTextStyle default tsNormal;
    property BeforeClick: TabcCancelEvent read FBeforeClick write FBeforeClick;
    property BeforeMouseDown: TabcMouseCancelEvent read FBeforeMouseDown write FBeforeMouseDown;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  public
    procedure Click; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TabcPicSpeedBtn }

  TabcPicSpeedBtn = class(TabcCustomPicSpeedBtn)
  public
    property MouseOver;
  published
    property Action;
    property Align;
    property AllowAllUp;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property GroupIndex;
    property Down;
    property Caption;
    property Color;
    property DisabledShadow;
    property Enabled;
    property Font;
    property ImageIndex;
    property Images;
    property Layout;
    property Margin;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property Picture;
    property PictureHighlight;
    property NumGlyphs;
    property Popup;
    property ParentBiDiMode;
    property PopupMenu;
    property ShowHint;
    property SoundDown;
    property SoundList;
    property SoundOver;
    property Spacing;
    property Style;
    property TextStyle;
    property TransparentColor;
    property Visible;
    property BeforeClick;
    property BeforeMouseDown;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TabcRepSpeedBtn }

  TabcRepSpeedBtn = class(TabcPicSpeedBtn)
  private
    FInitRepeatPause: Word;
    FOnTimer: TNotifyEvent;
    FRepeatEnabled: Boolean;
    FRepeatPause: Word;
    FRepeatTimer: TTimer;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y:Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property RepeatEnabled: Boolean read FRepeatEnabled write FRepeatEnabled default True;
    property RepeatInitialpause: Word read FInitRepeatPause write FInitRepeatPause default 400;
    property Repeatpause: Word read FRepeatPause write FRepeatPause default 100;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  end;

  { TabcToggleBtn }

  TabcToggleBtn = class(TabcPicSpeedBtn)
  private
    FHintDown: string;
    FHintUp: string;
    FOnClickDown: TNotifyEvent;
    FOnClickUp: TNotifyEvent;
    function GetHint: string;
    procedure SetHintDown(Value: string);
    procedure SetHintUp(Value: string);
    procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
  published
    property AllowAllUp default True;
    property Hint: string read GetHint; {remove Hint from OI}
    property HintDown: string read FHintDown write SetHintDown;
    property HintUp: string read FHintUp write SetHintUp;
    property Popup default False;
    property OnClickDown: TNotifyEvent read FOnClickDown write FOnClickDown;
    property OnClickUp: TNotifyEvent read FOnClickUp write FOnClickUp;
  end;

  { TabcMouseRegion }

  TabcMouseRegion = class(TGraphicControl)
  private
    FDown: Boolean;
    FDragging: Boolean;
    FMouseOver: Boolean;
    FSoundList: TabcCustomWaveList;
    FSoundDown: Integer;
    FSoundLeave: Integer;
    FSoundOver: Integer;
    FOutline: Boolean;
    FOutlineColor: TColor;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    function MouseIsOver: Boolean;
    procedure SetOutline(Value: Boolean);
    procedure SetOutlineColor(Value: TColor);
    procedure UpdateMouseOver;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMLButtonDblClk(var Message: TWMLButtonDown); message WM_LBUTTONDBLCLK;
  protected
    FState: TButtonState;
    procedure MouseEnter; dynamic;
    procedure MouseLeave; dynamic;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
    property MouseOver: Boolean read FMouseOver;
  published
    property Action;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property Caption;
    property Enabled;
    property Outline: Boolean read FOutline write SetOutline;
    property OutlineColor: TColor read FOutlineColor write SetOutlineColor default clGray;
    property ParentBiDiMode;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property SoundDown: Integer read FSoundDown write FSoundDown default -1;
    property SoundLeave: Integer read FSoundLeave write FSoundLeave default -1;
    property SoundList: TabcCustomWaveList read FSoundList write FSoundList;
    property SoundOver: Integer read FSoundOver write FSoundOver default -1;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

  { TabcMenuButton }

  TabcMenuButton = class(TSpeedButton)
  private
    FMenuItem: TMenuItem;
    function GetHint: string;
    function GetMenuCaption: string;
    function GetVisible: Boolean;
    function IsMenuPropStored: Boolean;
    procedure SetHint(Value: string);
    procedure SetMenuCaption(Value: string);
    procedure SetMenuItem(Value: TMenuItem);
    procedure SetVisible(Value: Boolean);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    function GetEnabled: Boolean; override;
    procedure SetEnabled(Value: Boolean); override;
  public
    procedure Click; override;
  published
    property Enabled stored IsMenuPropStored;
    property Hint: string read GetHint write SetHint stored IsMenuPropStored;
    property MenuCaption: string read GetMenuCaption write SetMenuCaption stored False;
    property MenuItem: TMenuItem read FMenuItem write SetMenuItem;
    property Visible: Boolean read GetVisible write SetVisible stored IsMenuPropStored;
  end;

  { TabcSizeButton }

  TabcSizeButton = class(TButton)
  private
    FMarginH: Integer;
    FMarginV: Integer;
    procedure AdjustBounds;
    procedure DoDrawText(var Rect: TRect; Flags: Word);
    procedure SetMarginH(Value: Integer);
    procedure SetMarginV(Value: Integer);
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  protected
    procedure AdjustSize; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AutoSize default True;
    property MarginH: Integer read FMarginH write SetMarginH default 8;
    property MarginV: Integer read FMarginV write SetMarginV default 8;
    property OnResize;
  end;

  { TabcFormButton }

  TabcFormButton = class(TButton)
  private
    FAfterShow: TNotifyEvent;
    FForm: TForm;
    FFormModalResult: TModalResult;
    FFormName: TComponentName;
    FShowModal: Boolean;
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    destructor Destroy; override;
    procedure Click; override;
    property Form: TForm read FForm;
    property FormModalResult: TModalResult read FFormModalResult;
  published
    property FormName: TComponentName read FFormName write FFormName;
    property ShowModal: Boolean read FShowModal write FShowModal default False;
    property AfterShow: TNotifyEvent read FAfterShow write FAfterShow;
  end;

  { TabcFormPicBtn }

  TabcFormPicBtn = class(TabcPicBtn)
  private
    FAfterShow: TNotifyEvent;
    FForm: TForm;
    FFormModalResult: TModalResult;
    FFormName: TComponentName;
    FShowModal: Boolean;
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    destructor Destroy; override;
    procedure Click; override;
    property Form: TForm read FForm;
    property FormModalResult: TModalResult read FFormModalResult;
  published
    property FormName: TComponentName read FFormName write FFormName;
    property ShowModal: Boolean read FShowModal write FShowModal default False;
    property AfterShow: TNotifyEvent read FAfterShow write FAfterShow;
  end;

  { TabcFormPicSpeedBtn }

  TabcFormPicSpeedBtn = class(TabcPicSpeedBtn)
  private
    FAfterShow: TNotifyEvent;
    FForm: TForm;
    FFormModalResult: TModalResult;
    FFormName: TComponentName;
    FShowModal: Boolean;
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    destructor Destroy; override;
    procedure Click; override;
    property Form: TForm read FForm;
    property FormModalResult: TModalResult read FFormModalResult;
  published
    property FormName: TComponentName read FFormName write FFormName;
    property ShowModal: Boolean read FShowModal write FShowModal default False;
    property AfterShow: TNotifyEvent read FAfterShow write FAfterShow;
  end;

  { TabcWaveBtn }

  TabcWaveBtn = class(TabcPicBtn)
  private
    FPlaySync: Boolean;
    FWave: TabcWave;
    function GetSound: TabcWaveImage;
    procedure SetPlaySync(Value: Boolean);
    procedure SetSound(Value: TabcWaveImage);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    property Wave: TabcWave read FWave;
  published
    property Sound: TabcWaveImage read GetSound write SetSound;
    property PlaySync: Boolean read FPlaySync write SetPlaySync default False;
  end;

  { TabcWaveFileBtn }

  TabcWaveFileBtn = class(TabcPicBtn)
  private
    FPlaySync: Boolean;
    FWaveFile: TabcWaveFile;
    function GetFileName: TFileName;
    procedure SetFileName(Value: TFileName);
    procedure SetPlaySync(Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    property WaveFile: TabcWaveFile read FWaveFile;
  published
    property FileName: TFileName read GetFileName write SetFileName;
    property PlaySync: Boolean read FPlaySync write SetPlaySync default False;
  end;

implementation

uses Consts, abcgreg;

const
  clABCIconTransparent = clFuchsia;

type
  TabcButtonFill = (abfSolid, abfPattern, abfNone);

type
  TabcGlyphList = class(TImageList)
  private
    Used: TBits;
    FCount: Integer;
    function AllocateIndex: Integer;
  public
    constructor CreateGlyphList(AWidth, AHeight: Integer);
    destructor Destroy; override;
    function Add(Image, Mask: TBitmap): Integer;
    function AddMasked(Image: TBitmap; MaskColor: TColor): Integer;
    procedure Delete(Index: Integer);
    property Count: Integer read FCount;
  end;

  TGlyphCache = class
  private
    GlyphLists: TList;
  public
    constructor Create;
    destructor Destroy; override;
    function GetList(AWidth, AHeight: Integer): TabcGlyphList;
    procedure ReturnList(List: TabcGlyphList);
    function Empty: Boolean;
  end;

  TCustomImageListAccess = class(TCustomImageList);

  TSmallPictureStream = class(TMemoryStream)
  protected
    function Realloc(var NewCapacity: Longint): Pointer; override;
  public
    procedure SavePicture(const P: TPicture);
    procedure RestorePicture(const P: TPicture);
  end;

  TabcButtonGlyph = class
  private
    FStream: TSmallPictureStream;
    FOriginal: TPicture;
    FOriginalSize: TPoint;
    FGlyphHighlight: TabcGlyphHighlight;
    FGlyphList: TabcGlyphList;
    FImageIndex: TImageIndex;
    FImageList: TCustomImageList;
    FImageChangeLink: TChangeLink;
    FIndexs: array[TabcButtonState] of Integer;
    FBtnFaceColor: TColor;
    FTransparentColorChanged: Boolean;
    FTransparentColor: TColor;
    FNumGlyphs: TabcNumGlyphs;
    FOnChange: TNotifyEvent;
    procedure PictureChanged(Sender: TObject);
    procedure SetGlyphHighlight(Value: TabcGlyphHighlight);
    function GetPicture: TPicture;
    procedure SetPicture(Value: TPicture);
    procedure SetTransparentColor(Value: TColor);
    procedure SetNumGlyphs(Value: TabcNumGlyphs);
    procedure UpdateNumGlyphs;
    procedure Invalidate;
    function CreateButtonGlyph(State: TabcButtonState): Integer;
    procedure DrawButtonGlyph(Canvas: TCanvas; State: TabcButtonState;
      Layout: TabcButtonLayout; Pos, Size: TPoint; Rect: TRect; IsMouseOver: Boolean);
    procedure DrawButtonGlyphTiled(Canvas: TCanvas; State: TabcButtonState; Rect: TRect; Size: TPoint);
    procedure DrawButtonText(Canvas: TCanvas; const Caption: string;
      TextBounds: TRect; State: TabcButtonState; TextStyle: TabcTextStyle;
      TextShadow: Boolean; BiDiFlags: Longint);
    procedure CalcButtonLayout(Canvas: TCanvas; const Client: TRect;
      const Caption: string; Layout: TabcButtonLayout; Margin, Spacing: Integer;
      var GlyphPos, GlyphSize: TPoint; var TextBounds: TRect; BiDiFlags: Longint);
    procedure RestoreGlyph;
  public
    constructor Create;
    destructor Destroy; override;
    function Draw(Canvas: TCanvas; const Client, TiledClient: TRect;
      const Caption: string; Layout: TabcButtonLayout;
      Margin, Spacing: Integer; State: TabcButtonState;
      TextStyle: TabcTextStyle; TextShadow: Boolean; IsMouseOver: Boolean;
      BiDiFlags: Longint): TRect;  { returns the text rectangle }
    property GlyphHighlight: TabcGlyphHighlight read FGlyphHighlight write SetGlyphHighlight;
    property TransparentColorChanged: Boolean read FTransparentColorChanged;
    property Picture: TPicture read GetPicture write SetPicture;
    property NumGlyphs: TabcNumGlyphs read FNumGlyphs write SetNumGlyphs;
    property TransparentColor: TColor read FTransparentColor write SetTransparentColor;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

{ TabcGlyphList }

constructor TabcGlyphList.CreateGlyphList(AWidth, AHeight: Integer);
begin
  inherited CreateSize(AWidth, AHeight);
  Used := TBits.Create;
end;

destructor TabcGlyphList.Destroy;
begin
  Used.Free;
  inherited Destroy;
end;

function TabcGlyphList.Add(Image, Mask: TBitmap): Integer;
begin
  Result := AllocateIndex;
  Replace(Result, Image, Mask);
  Inc(FCount);
end;

function TabcGlyphList.AddMasked (Image: TBitmap; MaskColor: TColor): Integer;
  procedure BugfreeReplaceMasked (Index: Integer; NewImage: TBitmap; MaskColor: TColor);
    procedure CheckImage (Image: TGraphic);
    begin
      if Image = nil then Exit;
      if (Image.Height < Height) or (Image.Width < Width) then
        raise EInvalidOperation.Create(SInvalidImageSize);
    end;
  var
    TempIndex: Integer;
    Image, Mask: TBitmap;
  begin
    if HandleAllocated then begin
      CheckImage(NewImage);
      TempIndex := inherited AddMasked(NewImage, MaskColor);
      if TempIndex <> -1 then
        try
          Image := TBitmap.Create;
          Mask := TBitmap.Create;
          try
            Image.Height := Height;
            Image.Width := Width;
            Mask.Monochrome := True;
            { ^ Prevents the "invisible glyph" problem when used with certain
                color schemes. (fixed in Delphi 3.01) }
            Mask.Height := Height;
            Mask.Width := Width;
            ImageList_Draw (Handle, TempIndex, Image.Canvas.Handle, 0, 0, ILD_NORMAL);
            ImageList_Draw (Handle, TempIndex, Mask.Canvas.Handle, 0, 0, ILD_MASK);
            if not ImageList_Replace(Handle, Index, Image.Handle, Mask.Handle) then
              raise EInvalidOperation.Create(SReplaceImage);
          finally
            Image.Free;
            Mask.Free;
          end;
        finally
          inherited Delete(TempIndex);
        end
      else
        raise EInvalidOperation.Create(SReplaceImage);
    end;
    Change;
  end;
begin
  Result := AllocateIndex;
  { This works two very serious bugs in the Delphi 2/BCB and Delphi 3
    implementations of the ReplaceMasked method. In the Delphi 2 and BCB
    versions of the ReplaceMasked method, it incorrectly uses ILD_NORMAL as
    the last parameter for the second ImageList_Draw call, in effect causing
    all white colors to be considered transparent also. And in the Delphi 2/3
    and BCB versions it doesn't set Monochrome to True on the Mask bitmap,
    causing the bitmaps to be invisible on certain color schemes. }
  BugfreeReplaceMasked (Result, Image, MaskColor);
  Inc (FCount);
end;

function TabcGlyphList.AllocateIndex: Integer;
begin
  Result := Used.OpenBit;
  if Result >= Used.Size then
  begin
    Result := inherited Add(nil, nil);
    Used.Size := Result + 1;
  end;
  Used[Result] := True;
end;

procedure TabcGlyphList.Delete(Index: Integer);
begin
  if Used[Index] then
  begin
    Dec(FCount);
    Used[Index] := False;
  end;
end;

{ TGlyphCache }

constructor TGlyphCache.Create;
begin
  inherited Create;
  GlyphLists := TList.Create;
end;

destructor TGlyphCache.Destroy;
begin
  GlyphLists.Free;
  inherited Destroy;
end;

function TGlyphCache.GetList(AWidth, AHeight: Integer): TabcGlyphList;
var
  I: Integer;
begin
  for I := GlyphLists.Count - 1 downto 0 do
  begin
    Result := GlyphLists[I];
    with Result do
      if (AWidth = Width) and (AHeight = Height) then Exit;
  end;
  Result := TabcGlyphList.CreateGlyphList(AWidth, AHeight);
  GlyphLists.Add(Result);
end;

function TGlyphCache.Empty: Boolean;
begin
  Result := GlyphLists.Count = 0;
end;

procedure TGlyphCache.ReturnList(List: TabcGlyphList);
begin
  if List = nil then Exit;
  if List.Count = 0 then
  begin
    GlyphLists.Remove(List);
    List.Free;
  end;
end;

var
  GlyphCache: TGlyphCache = nil;

{Shared Brush Patterns}

var
  BrushPatternList: TList;

type
  TabcBrushPattern = class(TObject)
  private
    FBitmap: TBitmap;
    FColor: TColor;
    FUsageCount: Integer;
    procedure CreateBrushPattern;
  public
    constructor Create(AColor: TColor);
    destructor Destroy; override;
    procedure Lock;
    procedure Release;
    property Bitmap: TBitmap read FBitmap;
    property Color: TColor read FColor;
  end;

constructor TabcBrushPattern.Create(AColor: TColor);
begin
  inherited Create;
  FColor := AColor;
  CreateBrushPattern;
end;

procedure TabcBrushPattern.CreateBrushPattern;
var
  X, Y: Integer;
begin
  FBitmap := TBitmap.Create;
  with FBitmap do begin
    Width := 8;
    Height := 8;
    with Canvas do begin
      Brush.Style := bsSolid;
      Brush.Color := FColor;
      FillRect(Rect(0, 0, Width, Height));
      for Y := 0 to 7 do
        for X := 0 to 7 do
          if (Y mod 2) = (X mod 2) then
            Pixels[X, Y] := clBtnHighlight;
    end;
  end;
end;

destructor TabcBrushPattern.Destroy;
begin
  FBitmap.Free;
  inherited Destroy;
end;

procedure TabcBrushPattern.Lock;
begin
  Inc(FUsageCount);
end;

procedure TabcBrushPattern.Release;
begin
  Dec(FUsageCount);
  if FUsageCount < 1 then begin
    BrushPatternList.Remove(Self);
    Free;
  end;
end;

function GetBrushPattern(AColor: TColor): Pointer;
var
  I: integer;
begin
  with BrushPatternList do begin
    for I := 0 to Count - 1 do
      with TabcBrushPattern(Items[I]) do
        if Color = AColor then begin
          Result := TabcBrushPattern(Items[I]);
          Exit;
        end;
    {create a new brush pattern and add to list}
    Result := TabcBrushPattern.Create(AColor);
    Add(Result);
  end;
end;

{ TSmallPictureStream }

function TSmallPictureStream.Realloc(var NewCapacity: Integer): Pointer;
const
  SmallMemoryDelta = $50;
begin
  if NewCapacity > 0 then
    NewCapacity := (NewCapacity + (SmallMemoryDelta - 1)) and not (SmallMemoryDelta - 1);
  Result := Memory;
  if NewCapacity <> Capacity then
    ReallocMem(Result, NewCapacity);
end;

procedure TSmallPictureStream.SavePicture(const P: TPicture);
var
  CName: string[63];
begin
  Position := 0;
  if P.Graphic <> nil then
    CName := P.Graphic.ClassName
  else
    CName := '';
  Write(CName, Length(CName) + 1);
  if P.Graphic <> nil then
    P.Graphic.SaveToStream(Self);
end;

procedure TSmallPictureStream.RestorePicture(const P: TPicture);
var
  CName: string[63];
  NewGraphic: TGraphic;
  GraphicClass: TGraphicClass;
begin
  Position := 0;
  Read(CName[0], 1);
  Read(CName[1], Integer(CName[0]));
  if CName <> '' then
  begin
    GraphicClass := abcFileFormatsList.FindClassName(CName);
    if GraphicClass = nil then
      raise EClassNotFound.Create('Graphic Class "%s" needs to be registered with the abcFileFormatsList in unit abcgreg');
  end
  else
    GraphicClass := nil;
  NewGraphic := nil;
  if GraphicClass <> nil then
  begin
    NewGraphic := GraphicClass.Create;
    try
      NewGraphic.LoadFromStream(Self);
    except
      NewGraphic.Free;
      raise;
    end;
  end;
  P.Graphic := NewGraphic;
  if NewGraphic <> nil then
    NewGraphic.Free;
end;

{ TabcButtonGlyph }
{ NB This is similar to TButtonGlyph in buttons.pas with additional code for
  tiling, raised/lowered text, and button face color, icon and metafile translation to bit cache}

constructor TabcButtonGlyph.Create;
var
  I: TabcButtonState;
begin
  inherited Create;
  FOriginal := TPicture.Create;
  if ABC_GLYPHS_SAVE_GDI then
    FStream := TSmallPictureStream.Create;
  FOriginal.OnChange := PictureChanged;
  FOriginalSize := Point(0, 0);
  FNumGlyphs := 1;
  FImageIndex := -1;
  for I := Low(I) to High(I) do
    FIndexs[I] := -1;
  if GlyphCache = nil then GlyphCache := TGlyphCache.Create;
  FTransparentColor := clABCIconTransparent;
end;

destructor TabcButtonGlyph.Destroy;
begin
  FOriginal.Free;
  FImageChangeLink.Free;
  Invalidate;
  if Assigned(GlyphCache) and GlyphCache.Empty then
  begin
    GlyphCache.Free;
    GlyphCache := nil;
  end;
  if ABC_GLYPHS_SAVE_GDI then
    FStream.Free;
  inherited Destroy;
end;

procedure TabcButtonGlyph.Invalidate;
var
  I: TabcButtonState;
begin
  for I := Low(I) to High(I) do
    if FIndexs[I] <> -1 then
    begin
      FGlyphList.Delete(FIndexs[I]);
      FIndexs[I] := -1;
    end;
  GlyphCache.ReturnList(FGlyphList);
  FGlyphList := nil;
end;

procedure TabcButtonGlyph.PictureChanged(Sender: TObject);
begin
  if Sender = FOriginal then
  begin
    if FOriginal.Graphic is TBitmap then
      FTransparentColor := FOriginal.Bitmap.TransparentColor
    else
      FTransparentColor := clABCIconTransparent;

    FTransparentColorChanged := False;
  end;
  Invalidate;
  if ABC_GLYPHS_SAVE_GDI then
  begin
    FOriginalSize := Point(0, 0);
    FStream.Clear;
  end;
  FOriginalSize := Point(FOriginal.Width, FOriginal.Height);
  if Assigned(FOnChange) then FOnChange (Self);
end;

procedure TabcButtonGlyph.SetNumGlyphs (Value: TabcNumGlyphs);
begin
  Invalidate;
  if ABC_GLYPHS_SAVE_GDI then
  begin
    FOriginalSize := Point(0, 0);
    FStream.Clear;
  end;
  if ((FImageList <> nil) and (FImageIndex >= 0) and (FImageIndex < FImageList.Count)) or
     (Value < Low(TabcNumGlyphs)) or
     (Value > High(TabcNumGlyphs)) then
    FNumGlyphs := 1
  else
    FNumGlyphs := Value;
  PictureChanged (nil);
end;

procedure TabcButtonGlyph.SetPicture(Value: TPicture);
begin
  Invalidate;
  if ABC_GLYPHS_SAVE_GDI then
  begin
    FOriginalSize := Point(0, 0);
    FStream.Clear;
  end;
  FOriginal.Assign(Value);
  FOriginalSize := Point(FOriginal.Width, FOriginal.Height);
  UpdateNumGlyphs;
end;

procedure TabcButtonGlyph.UpdateNumGlyphs;
var
  Glyphs: Integer;
begin
  if (FOriginal.Width <> 0) and (FOriginal.Height <> 0) and
     (FOriginal.Width mod FOriginal.Height = 0) then begin
    Glyphs := FOriginal.Width div FOriginal.Height;
    if Glyphs > High(TabcNumGlyphs) then Glyphs := 1;
  end
  else
    Glyphs := 1;
  SetNumGlyphs (Glyphs);
end;

function TabcButtonGlyph.CreateButtonGlyph(State: TabcButtonState): Integer;
const
  ROP_DSPDxax = $00E20746;
var
  PictureBmp, TmpImage, MonoBmp, DrawBmp: TBitmap;
  IWidth, IHeight, iTemp: Integer;
  IRect, ORect: TRect;
  I: TabcButtonState;
  DestDC: HDC;
begin
  if (State = absDown) and (NumGlyphs < 3) then State := absMouseOver;

  Result := FIndexs[State];
  if Result <> -1 then Exit;    {if glyph is already cached}

  if ((FOriginalSize.X or FOriginalSize.Y) = 0) and
     ((FImageList = nil) or (FImageIndex < 0) or (FImageIndex >= FImageList.Count)) then Exit;
  {create temporary PictureBmp to convert TPicture or copy imagelist image}
  PictureBmp := TBitmap.Create;
  try
    with PictureBmp do
      if (FImageList <> nil) and (FImageIndex >= 0) and (FImageIndex < FImageList.Count) then
      begin
        Width := TCustomImageListAccess(FImageList).Width;
        Height := TCustomImageListAccess(FImageList).Height;
        ImageList_Draw (FImageList.Handle, FImageIndex, Canvas.Handle,
          0, 0, ILD_NORMAL);
        if not FTransparentColorChanged then
          FTransparentColor := TransparentColor;
      end
      else
      begin
        Width := FOriginalSize.X;
        Height := FOriginalSize.Y;
        if ABC_GLYPHS_SAVE_GDI then
          RestoreGlyph;
        with Canvas do begin
          if FOriginal.Graphic is TIcon then begin
            Brush.Color := FTransparentColor;
            FillRect(Rect(0,0,Width,Height));
            Brush.Color := clBlack;
          end;
          CopyMode := cmSrcCopy;
          Draw(0,0,FOriginal.Graphic);
          if not FTransparentColorChanged then
            if (FOriginal.Graphic is TBitmap) or
               (FOriginal.Graphic is TMetaFile) then
              FTransparentColor := TransparentColor;
        end;
      end;
    IWidth := PictureBmp.Width div FNumGlyphs;
    IHeight := PictureBmp.Height;
    if FGlyphList = nil then
    begin
      if GlyphCache = nil then GlyphCache := TGlyphCache.Create;
      FGlyphList := GlyphCache.GetList(IWidth, IHeight);
    end;
    TmpImage := TBitmap.Create;
    try
      TmpImage.Width := IWidth;
      TmpImage.Height := IHeight;
      IRect := Rect(0, 0, IWidth, IHeight);
      TmpImage.Canvas.Brush.Color := FBtnFaceColor;
      I := State;
      if Ord(I) >= NumGlyphs then
        I := absUp;
      ORect := Rect(Ord(I) * IWidth, 0, (Ord(I) + 1) * IWidth, IHeight);
      case State of
        absMouseOver, absDown:
          begin
            TmpImage.Canvas.BrushCopy(IRect, PictureBmp, ORect, FTransparentColor);
            FIndexs[State] := FGlyphList.Add(TmpImage, nil);
          end;
        absExclusive:
          begin
            TmpImage.Canvas.CopyRect(IRect, PictureBmp.Canvas, ORect);
            FIndexs[State] := FGlyphList.AddMasked(TmpImage, FTransparentColor);
          end;
        absDisabled:
          begin
            MonoBmp := TBitmap.Create;
            try
              with MonoBmp do
              begin
                Monochrome := True;
                Width := IWidth;
                Height := IHeight;
              end;
              if NumGlyphs > 1 then
              with TmpImage.Canvas do
              begin    { Change white & gray to clBtnHighlight and clBtnShadow }
                CopyRect(IRect, PictureBmp.Canvas, ORect);
                { Convert white to clBtnHighlight }
                PictureBmp.Canvas.Brush.Color := clWhite;
                MonoBmp.Canvas.CopyRect(IRect, PictureBmp.Canvas, ORect);
                Brush.Color := clBtnHighlight;
                DestDC := Handle;
                SetTextColor(DestDC, clBlack);
                SetBkColor(DestDC, clWhite);
                BitBlt(DestDC, 0, 0, IWidth, IHeight,
                       MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
                { Convert gray to clBtnShadow }
                PictureBmp.Canvas.Brush.Color := clGray;
                MonoBmp.Canvas.CopyRect(IRect, PictureBmp.Canvas, ORect);
                Brush.Color := clBtnShadow;
                DestDC := Handle;
                SetTextColor(DestDC, clBlack);
                SetBkColor(DestDC, clWhite);
                BitBlt(DestDC, 0, 0, IWidth, IHeight,
                       MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
                { Convert transparent color to Button Face Color }
                PictureBmp.Canvas.Brush.Color := ColorToRGB(FTransparentColor);
                MonoBmp.Canvas.CopyRect(IRect, PictureBmp.Canvas, ORect);
                Brush.Color := FBtnFaceColor;
                DestDC := Handle;
                SetTextColor(DestDC, clBlack);
                SetBkColor(DestDC, clWhite);
                BitBlt(DestDC, 0, 0, IWidth, IHeight,
                       MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
              end
              else
              begin
                { Create a disabled version }
                TmpImage.Canvas.CopyRect(IRect, PictureBmp.Canvas, ORect);
                abcGenerateMaskBitmap(MonoBmp, TmpImage, IRect.TopLeft, IRect.BottomRight,
                  [FTransparentColor, clWhite, clSilver]);
                with TmpImage.Canvas do
                begin
                  Brush.Color := FBtnFaceColor;
                  FillRect(IRect);
                  Brush.Color := clBtnHighlight;
                  SetTextColor(Handle, clWhite);
                  SetBkColor(Handle, clBlack);
                  BitBlt(Handle, 1, 1, IWidth, IHeight, MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
                  Brush.Color := clBtnShadow;
                  SetTextColor(Handle, clWhite);
                  SetBkColor(Handle, clBlack);
                  BitBlt(Handle, 0, 0, IWidth, IHeight, MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
                end;
              end;
              FIndexs[State] := FGlyphList.Add(TmpImage, nil);
            finally
              MonoBmp.Free;
            end;
          end;
        absUp:
          begin
            case GlyphHighlight of
              aghNormal:
                TmpImage.Canvas.BrushCopy(IRect, PictureBmp, ORect, FTransparentColor);
              aghGray:
                begin
                  TmpImage.Canvas.BrushCopy(IRect, PictureBmp, ORect, FTransparentColor);
                  abcGrayScaleBitmap(TmpImage);
                end;
              aghFade:
                begin
                  with TmpImage do Canvas.FillRect(Rect(0,0,Width,Height));
                  iTemp := CreateButtonGlyph(absMouseOver);
                  ImageList_DrawEx(FGlyphList.Handle, iTemp, TmpImage.Canvas.Handle, 0, 0, 0, 0,
                    ColorToRGB(FBtnFaceColor), CLR_NONE, ILD_BLEND25);
                end;
              aghDarken:
                begin
                  with TmpImage do Canvas.FillRect(Rect(0,0,Width,Height));
                  iTemp := CreateButtonGlyph(absMouseOver);
                  DrawBmp := TBitmap.Create;
                  try
                    FGlyphList.GetBitmap(iTemp, DrawBmp);
                    BitBlt(TmpImage.Canvas.Handle, 0, 0, DrawBmp.Width, DrawBmp.Height,
                           DrawBmp.Canvas.Handle, 0, 0, SRCAND);
                  finally
                    DrawBmp.Free;
                  end;
                end;
            end;
            FIndexs[State] := FGlyphList.Add(TmpImage, nil);
          end;
      end;
    finally
      TmpImage.Free;
    end;
    Result := FIndexs[State];
    if ABC_GLYPHS_SAVE_GDI then
    begin
      FStream.Clear;
      FStream.SavePicture(FOriginal);
      FOriginal.OnChange := nil;
      FOriginal.Assign(nil);
      FOriginal.OnChange := PictureChanged;
    end;
  finally
    if PictureBmp <> nil then
      PictureBmp.Free;
  end;
end;

procedure TabcButtonGlyph.CalcButtonLayout(Canvas: TCanvas; const Client: TRect;
  const Caption: string; Layout: TabcButtonLayout; Margin, Spacing: Integer;
  var GlyphPos, GlyphSize: TPoint; var TextBounds: TRect; BiDiFlags: Longint);
var
  TextPos: TPoint;
  ClientSize, TextSize: TPoint;
  TotalSize: TPoint;
begin

  if (BiDiFlags and DT_RIGHT) = DT_RIGHT then
    if Layout = ablGlyphLeft then Layout := ablGlyphRight
    else
      if Layout = ablGlyphRight then Layout := ablGlyphLeft;

  { calculate the item sizes }
  ClientSize := Point(Client.Right - Client.Left,
                      Client.Bottom - Client.Top);

  if (FImageList <> nil) and (FImageIndex >= 0) and (FImageIndex < FImageList.Count) then
    GlyphSize := Point(TCustomImageListAccess(FImageList).Width div FNumGlyphs,
      TCustomImageListAccess(FImageList).Height)
  else if FOriginal <> nil then
    GlyphSize := Point(FOriginalSize.X div FNumGlyphs, FOriginalSize.Y)
  else
    GlyphSize := Point(0, 0);

  if Length(Caption) > 0 then
  begin
    TextBounds := Rect(0, 0, Client.Right - Client.Left, 0);
    DrawText(Canvas.Handle, PChar(Caption), Length(Caption), TextBounds,
      DT_CALCRECT or BiDiFlags);
    TextSize := Point(TextBounds.Right - TextBounds.Left, TextBounds.Bottom -
      TextBounds.Top);
  end
  else
  begin
    TextBounds := Rect(0, 0, 0, 0);
    TextSize := Point(0,0);
  end;

  { If the layout has the glyph on the right or the left, then both the
    text and the glyph are centered vertically.  If the glyph is on the top
    or the bottom, then both the text and the glyph are centered horizontally.}

  if Layout in [ablGlyphLeft, ablGlyphRight] then
  begin
    GlyphPos.Y := (ClientSize.Y - GlyphSize.Y + 1) div 2;
    TextPos.Y := (ClientSize.Y - TextSize.Y + 1) div 2;
  end
  else begin
    if Layout in [ablGlyphTop, ablGlyphBottom] then
    begin
      GlyphPos.X := (ClientSize.X - GlyphSize.X + 1) div 2;
      TextPos.X := (ClientSize.X - TextSize.X + 1) div 2;
    end
    else
    begin {ablGlyphCenter or ablGlyphTiled}
      TextPos.X := (ClientSize.X - TextSize.X + 1) div 2;
      TextPos.Y := (ClientSize.Y - TextSize.Y + 1) div 2;
      if Layout = ablGlyphCenter then
      begin
        GlyphPos.X := (ClientSize.X - GlyphSize.X + 1) div 2;
        GlyphPos.Y := (ClientSize.Y - GlyphSize.Y + 1) div 2;
      end
      else
      begin{ablGlyphTiled}
        GlyphPos.X := 0;
        GlyphPos.Y := 0;
      end;
    end;
  end;

  { if there is no text or no bitmap, then Spacing is irrelevant }
  if (TextSize.X = 0) or (GlyphSize.X = 0) then
    Spacing := 0;

  { adjust Margin and Spacing }
  if Margin = -1 then
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(GlyphSize.X + TextSize.X, GlyphSize.Y + TextSize.Y);
      if Layout in [ablGlyphLeft, ablGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X) div 3
      else
        Margin := (ClientSize.Y - TotalSize.Y) div 3;
      Spacing := Margin;
    end
    else
    begin
      TotalSize := Point(GlyphSize.X + Spacing + TextSize.X, GlyphSize.Y +
        Spacing + TextSize.Y);
      if Layout in [ablGlyphLeft, ablGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X + 1) div 2
      else
        Margin := (ClientSize.Y - TotalSize.Y + 1) div 2;
    end;
  end
  else
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(ClientSize.X - (Margin + GlyphSize.X), ClientSize.Y -
        (Margin + GlyphSize.Y));
      if Layout in [ablGlyphLeft, ablGlyphRight] then
        Spacing := (TotalSize.X - TextSize.X) div 2
      else
        Spacing := (TotalSize.Y - TextSize.Y) div 2;
    end;
  end;

  case Layout of
    ablGlyphLeft:
      begin
        GlyphPos.X := Margin;
        TextPos.X := GlyphPos.X + GlyphSize.X + Spacing;
      end;
    ablGlyphRight:
      begin
        GlyphPos.X := ClientSize.X - Margin - GlyphSize.X;
        TextPos.X := GlyphPos.X - Spacing - TextSize.X;
      end;
    ablGlyphTop:
      begin
        GlyphPos.Y := Margin;
        TextPos.Y := GlyphPos.Y + GlyphSize.Y + Spacing;
      end;
    ablGlyphBottom:
      begin
        GlyphPos.Y := ClientSize.Y - Margin - GlyphSize.Y;
        TextPos.Y := GlyphPos.Y - Spacing - TextSize.Y;
      end;
  end;

  { fixup the result variables }
  Inc(GlyphPos.X, Client.Left);
  Inc(GlyphPos.Y, Client.Top);
  OffsetRect(TextBounds, TextPos.X + Client.Left, TextPos.Y + Client.Top);
end;

procedure TabcButtonGlyph.SetGlyphHighlight(Value: TabcGlyphHighlight);
begin
  if Value <> FGlyphHighlight then
  begin
    FGlyphHighlight := Value;
    Invalidate;
  end;
end;

function TabcButtonGlyph.Draw(Canvas: TCanvas; const Client, TiledClient: TRect;
  const Caption: string; Layout: TabcButtonLayout; Margin, Spacing: Integer;
  State: TabcButtonState; TextStyle: TabcTextStyle; TextShadow: Boolean;
  IsMouseOver: Boolean; BiDiFlags: Longint): TRect;
var
  GlyphPos, GlyphSize: TPoint;
begin
  CalcButtonLayout(Canvas, Client, Caption, Layout, Margin, Spacing,
    GlyphPos, GlyphSize, Result, BiDiFlags);
  if Layout = ablGlyphTiled then
    DrawButtonGlyphTiled(Canvas, State, TiledClient, GlyphSize)
  else
    DrawButtonGlyph(Canvas, State, Layout, GlyphPos, GlyphSize, Client, IsMouseOver);
  DrawButtonText(Canvas, Caption, Result, State, TextStyle, TextShadow, BiDiFlags);
end;

procedure TabcButtonGlyph.DrawButtonGlyph(Canvas: TCanvas; State: TabcButtonState;
  Layout: TabcButtonLayout; Pos, Size: TPoint; Rect: TRect; IsMouseOver: Boolean);
var
  Index: Integer;
begin
  if FOriginal = nil then Exit;
  if (Size.X = 0) or (Size.Y = 0) then Exit;
  Index := CreateButtonGlyph(State);
  if State = absExclusive then
    ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, Pos.X, Pos.Y, 0, 0,
      clNone, clNone, ILD_Transparent)
  else
    ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, Pos.X, Pos.Y, 0, 0,
      ColorToRGB(FBtnFaceColor), clNone, ILD_NORMAL);
end;

procedure TabcButtonGlyph.DrawButtonGlyphTiled(Canvas: TCanvas; State: TabcButtonState; Rect: TRect; Size: TPoint);
var
  Index: Integer;
  XPos, YPos: Word;
begin
  if FOriginal = nil then Exit;
  if (Size.X = 0) or (Size.Y = 0) then Exit;
  Index := CreateButtonGlyph(State);
  XPos:= Rect.Left;
  while XPos < Rect.Right do begin
    YPos:= Rect.Top;
    while YPos < Rect.Bottom do begin

      if State = absExclusive then
        ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, XPos, YPos, 0, 0,
          clNone, clNone, ILD_Transparent)
      else
        ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, XPos, YPos, 0, 0,
          ColorToRGB(FBtnFaceColor), clNone, ILD_TRANSPARENT);
      YPos:= YPos + Size.Y;
    end;
    XPos:= XPos + Size.X;
  end;
end;

procedure TabcButtonGlyph.DrawButtonText(Canvas: TCanvas; const Caption: string;
  TextBounds: TRect; State: TabcButtonState; TextStyle: TabcTextStyle;
  TextShadow: Boolean; BiDiFlags: Longint);
var
  SaveColor, TopColor, BottomColor: TColor;
  TmpRect: TRect;
begin
  with Canvas do begin
    Brush.Style := bsClear;
    if State = absDisabled then
      if TextShadow then begin
        OffsetRect(TextBounds, 1, 1);
        Font.Color := clWhite;
        DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
          DT_CENTER or DT_VCENTER or BiDiFlags);
        OffsetRect(TextBounds, -1, -1);
        Font.Color := clDkGray;
        DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
          DT_CENTER or DT_VCENTER or BiDiFlags);
      end
      else begin
        Font.Color := clBtnShadow;
        DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, 0);
      end
    else if TextStyle = tsNormal then
        DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
          DT_CENTER or DT_VCENTER or DT_SINGLELINE)
    else begin
      SaveColor := Canvas.Font.Color;
      TopColor := clBtnHighlight;
      BottomColor := clBtnShadow;
      if TextStyle = tsLowered then begin
        TopColor := clBtnShadow;
        BottomColor := clBtnHighlight;
      end;
      TmpRect := TextBounds;
      OffsetRect(TmpRect, 1, 1);
      Font.Color := BottomColor;
      DrawText(Handle, PChar(Caption), Length(Caption), TmpRect,
        DT_CENTER or DT_VCENTER or DT_SINGLELINE);
      TmpRect := TextBounds;
      OffsetRect(TmpRect, -1, -1);
      Font.Color := TopColor;
      DrawText(Handle, PChar(Caption), Length(Caption), TmpRect,
        DT_CENTER or DT_VCENTER or DT_SINGLELINE);
      Font.Color := SaveColor;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or DT_SINGLELINE);
    end;
  end;
end;

procedure TabcButtonGlyph.SetTransparentColor(Value: TColor);
begin
  if Value <> FTransparentColor then begin
    FTransparentColor := Value;
    FTransparentColorChanged := True;
    Invalidate;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

function TabcButtonGlyph.GetPicture: TPicture;
begin
  if ABC_GLYPHS_SAVE_GDI then
    RestoreGlyph;
  Result := FOriginal;
end;

procedure TabcButtonGlyph.RestoreGlyph;
begin
  if (FOriginal.Graphic = nil) and (FStream.Size > 0) then
    FStream.RestorePicture(FOriginal);
end;

{ abcDrawButtonFace - draws button faces for speedbutton and bitbutton styles

  Face Styles:

  if Popup is true then button draws flat using the Fill option,
  unless one of the mouseover, down, focused flags is set.

  - absThin and absThinBlack are simplest buttons,
    these have a single line border and never draw a focus rect around the outside.
  - absNew and absWin31 are regular style buttons as in Win95 and Win3.1.

  Result is the remaining client rect, adjusted as follows:

  Style+Options           Adjustment to Rect
  -------------           ----L--T--R--B----
  Popup                       No change unless MouseOver or Down or Focused
  absThin+MouseOver          +1 +1 -1 -1
  absThin+Down               +1 +1 -1 -1
  absThin+Focused            +1 +1 -1 -1
  absThinBlack               (same as absThin)
  absNew                     +1 +1 -2 -2
  absNew+Focused             +2 +2 -3 -3
  absNew+Down                +2 +2 -2 -2
  absNew+Down+Focused        +3 +3 -3 -3
  absWin31                   +3 +3 -3 -3
  absWin31+Down              +2 +2 -1 -1
  absWin31+Focus             +4 +4 -4 -4
  absWin31+Down+Focused      +3 +3 -2 -2
  absThinGray                (same as absThin)

  absThinHighlight

  absMidHighlight

  absThickHighlight

  absFramed
  absFramedBlack
  absFramedRaised
  absFramedHighlight
  absFramedBlackHighlight

  absRecessed
  absRecessedBlack
  absRecessedRaised
  absRecessedHighlight
  absRecessedBlackHighlight
}

function abcDrawButtonFace(Canvas: TCanvas; const Client: TRect;
  Style: TabcButtonStyle; Popup, IsMouseOver, IsRounded, IsDown, IsFocused: Boolean;
  Fill: TabcButtonFill; BtnFaceColor: TColor; BrushPattern: TBitmap): TRect;
const
  CL_LIGHTER_GRAY = $DEDEDE;
var
  I: Integer;
begin
  Result := Client;
  with Canvas do begin
    if Fill <> abfNone then begin
      if (Fill = abfPattern) and (BrushPattern <> nil) then
        Canvas.Brush.Bitmap := BrushPattern
      else begin
        Brush.Color := BtnFaceColor;
        Brush.Style := bsSolid;
      end;
      FillRect(Result);
      Canvas.Brush.Bitmap := nil;
    end;

    if Popup and not (IsMouseOver or IsDown or IsFocused) then Exit;  {drawn flat only}

    if (Style = absThin) then {win97 popup style}
      if IsDown then
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST)
      else
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST)
    else if (Style = absThinBlack) then {win97 popup style variant}
      if IsDown then
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST)
      else
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST)
    else if (Style = absNew) or ((Style = absAutoDetect) and NewStyleControls) then begin {win95 style}
      if IsDown then
        if IsFocused then begin
          DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_MONO or BF_ADJUST);
          DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_FLAT or BF_ADJUST);
        end
      else begin
        DrawEdge(Handle, Result, EDGE_SUNKEN, BF_RECT or BF_SOFT or BF_ADJUST)
      end
      else begin
        if IsFocused then
          DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_MONO or BF_ADJUST);
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_BOTTOMRIGHT or BF_SOFT or BF_ADJUST);
      end;
    end
    else if (Style = absThinGray) then
      if IsDown then
        Frame3D(Canvas, Result, clBtnShadow, CL_LIGHTER_GRAY, 1)
      else
        Frame3D(Canvas, Result, CL_LIGHTER_GRAY, clBtnShadow, 1)
    else if (Style = absThinHighlight) then begin
      if IsDown then
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST)
      else if IsMouseOver then
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST)
      else
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST)
    end
    else if (Style = absMidHighlight) then begin
      if IsDown then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        Frame3D(Canvas, Result, clBtnShadow, CL_LIGHTER_GRAY, 1);
      end
      else if IsMouseOver then
      begin
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        Frame3D(Canvas, Result, CL_LIGHTER_GRAY, clBtnShadow, 1);
      end
      else
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST)
    end
    else if (Style = absThickHighlight) then begin
      if IsDown then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        Frame3D(Canvas, Result, clBtnShadow, CL_LIGHTER_GRAY, 1);
      end
      else if IsMouseOver then
      begin
        Pen.Color := clBtnShadow;
        MoveTo(Result.Left-1, Result.Bottom-1);
        LineTo(Result.Right, Result.Bottom-1);
        Result.Bottom := Result.Bottom - 1;
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        Frame3D(Canvas, Result, CL_LIGHTER_GRAY, clBtnShadow, 1);
      end
      else
      begin
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST);
        Pen.Color := clBtnShadow;
        MoveTo(Result.Left-1, Result.Bottom-1);
        LineTo(Result.Right, Result.Bottom-1);
        Result.Bottom := Result.Bottom - 1;
      end
    end
    else if (Style in [absFramed, absRecessed]) then
    begin
      if IsDown then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      end
      else
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
        if (Style = absRecessed) then InflateRect(Result,-1,-1);
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST);
      end
    end
    else if (Style in [absFramedBlack, absRecessedBlack]) then
    begin
      if IsDown then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
      end
      else
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        if (Style = absRecessedBlack) then InflateRect(Result,-1,-1);
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
      end
    end
    else if (Style in [absFramedRaised, absRecessedRaised]) then
    begin
      if IsDown then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      end
      else
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        if (Style = absRecessedRaised) then InflateRect(Result,-1,-1);
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        Frame3D(Canvas, Result, CL_LIGHTER_GRAY, clBtnShadow, 1);
      end
    end
    else if (Style in [absFramedHighlight, absRecessedHighlight]) then
    begin
      if IsDown then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      end
      else if IsMouseOver then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
        if (Style = absRecessedHighlight) then InflateRect(Result,-1,-1);
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST);
        Frame3D(Canvas, Result, CL_LIGHTER_GRAY, clBtnShadow, 1);
      end
      else
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
        if (Style = absRecessedHighlight) then InflateRect(Result,-1,-1);
        DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST);
      end
    end
    else if (Style in [absFramedBlackHighlight, absRecessedBlackHighlight]) then
    begin
      if IsDown then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
      end
      else if IsMouseOver then
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        if (Style = absRecessedBlackHighlight) then InflateRect(Result,-1,-1);
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        Frame3D(Canvas, Result, CL_LIGHTER_GRAY, clBtnShadow, 1);
      end
      else
      begin
        DrawEdge(Handle, Result, BDR_SUNKENOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
        if (Style = absRecessedBlackHighlight) then InflateRect(Result,-1,-1);
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_RECT or BF_SOFT or BF_ADJUST);
      end
    end
    else begin {win31 style}
      DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_FLAT or BF_RECT or BF_MONO or BF_ADJUST);
      if IsRounded then begin
        Pixels[Client.Left, Client.Top] := BtnFaceColor;
        Pixels[Client.Left, Client.Bottom - 1] := BtnFaceColor;
        Pixels[Client.Right - 1, Client.Top] := BtnFaceColor;
        Pixels[Client.Right - 1, Client.Bottom - 1] := BtnFaceColor;
      end;
      if IsFocused then
        DrawEdge(Handle, Result, BDR_RAISEDOUTER, BF_FLAT or BF_RECT or BF_MONO or BF_ADJUST);
      if IsDown then
        DrawEdge(Handle, Result, BDR_SUNKENINNER, BF_TOPLEFT or BF_SOFT or BF_ADJUST)
      else
        for I := 1 to 2 do
          DrawEdge(Handle, Result, BDR_RAISEDINNER, BF_RECT or BF_ADJUST);
    end;
  end;
end;

{ TabcPicBtn }

constructor TabcPicBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FGlyph := TabcButtonGlyph.Create;
  TabcButtonGlyph(FGlyph).FBtnFaceColor := clBtnFace;
  TabcButtonGlyph(FGlyph).OnChange := GlyphChanged;
  FTextStyle := tsNormal;
  FCanvas := TCanvas.Create;
  FLayout := ablGlyphLeft;
  FSpacing := 4;
  FMargin := -1;
  Color := clBtnFace;
  FSoundDown := -1;
  FSoundOver := -1;
end;

destructor TabcPicBtn.Destroy;
begin
  if not (csDestroying in ComponentState) then Destroying;
  TabcButtonGlyph(FGlyph).Free;
  FGlyph := nil;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TabcPicBtn.CMColorChanged(var Message: TMessage);
var
  State: TabcButtonState;
begin
  with TabcButtonGlyph(FGlyph) do begin
    FBtnFaceColor := Color;
    Invalidate;
    if Enabled then
      State := absUp
    else
      State := absDisabled;
    CreateButtonGlyph(State);
  end;
  inherited;
end;

procedure TabcPicBtn.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csDestroying in ComponentState) then Invalidate;
end;

procedure TabcPicBtn.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

function TabcPicBtn.HasMouseEffect: Boolean;
begin
  Result := Popup or
    (PictureHighlight <> aghNormal) or
    (Style in [absThinHighlight, absMidHighlight, absThickHighlight,
               absFramedHighlight, absFramedBlackHighlight,
               absRecessedHighlight, absRecessedBlackHighlight]);
end;

procedure TabcPicBtn.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not ForegroundTask then Exit;
  if not FMouseOver and Enabled and (DragMode <> dmAutomatic)
    and (GetCapture = 0) then
  begin
    FMouseOver := True;
    if HasMouseEffect then
      Repaint;
    if (SoundOver > -1) and (SoundList <> nil) then
      SoundList.PlayItem(SoundOver);
    MouseEnter;
  end;
end;

procedure TabcPicBtn.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseOver := False;
  if Enabled and (DragMode <> dmAutomatic) and (GetCapture = 0) then
  begin
    if HasMouseEffect then
      Repaint;
    MouseLeave;
  end;
end;

procedure TabcPicBtn.CNDrawItem(var Message: TWMDrawItem);
begin
  DrawItem(Message.DrawItemStruct^);
end;

procedure TabcPicBtn.CNMeasureItem(var Message: TWMMeasureItem);
begin
  with Message.MeasureItemStruct^ do
  begin
    itemWidth := Width;
    itemHeight := Height;
  end;
end;

procedure TabcPicBtn.CreateHandle;
var
  State: TabcButtonState;
begin
  if Enabled then
    State := absUp
  else
    State := absDisabled;
  inherited CreateHandle;
  TabcButtonGlyph(FGlyph).CreateButtonGlyph(State);
end;

procedure TabcPicBtn.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do Style := Style or BS_OWNERDRAW;
end;

procedure TabcPicBtn.DrawItem(const DrawItemStruct: TDrawItemStruct);
var
  IsMouseOver, IsDown, IsDefault: Boolean;
  State: TabcButtonState;
  R, TileRect, ClipRect: TRect;
  FillBtnFace: TabcButtonFill;
begin
  if (csDestroying in ComponentState) then Exit;
  FCanvas.Handle := DrawItemStruct.hDC;

  IsMouseOver := Enabled and MouseOver;

  IsDown := DrawItemStruct.itemState and ODS_SELECTED <> 0;
  IsDefault := DrawItemStruct.itemState and ODS_FOCUS <> 0;

  if not Enabled then
    State := absDisabled
  else if IsDown then
    State := absDown
  else if (IsMouseOver or IsDefault or FIsFocused) then
    State := absMouseOver
  else
    State := absUp;

  if (Layout <> ablGlyphTiled) or
     (Color <> TransparentColor) or
     (Picture.Graphic = nil) then
    FillBtnFace := abfSolid
  else
    FillBtnFace := abfNone;

  abcDrawButtonFace(FCanvas, ClientRect, FStyle, Popup, IsMouseOver,
    True, IsDown, (FIsFocused or IsDefault), FillBtnFace, Color, nil);

  R := ClientRect;
  TileRect := R;
  InflateRect(R, -1, -1);
  FCanvas.Font := Self.Font;

  if IsDown then begin
    OffsetRect(R, 1, 1);
    OffsetRect(TileRect, 1, 1);
  end;

  ClipRect := R;
  InflateRect(ClipRect, -1, -1);

  IsMouseOver := IsMouseOver and not (IsDefault or FIsFocused);

  ClipRect :=
    TabcButtonGlyph(FGlyph).Draw(FCanvas, R, TileRect, Caption, FLayout,
      FMargin, FSpacing, State, FTextStyle,  FDisabledShadow, IsMouseOver,
      DrawTextBiDiModeFlags(0));

  abcDrawButtonFace(FCanvas, ClientRect, FStyle, Popup, IsMouseOver,
    True, IsDown, (FIsFocused or IsDefault), abfNone, Color, nil);

  if IsDefault then begin
    if (Style = absWin31) or ((Style = absAutoDetect) and not NewStyleControls) then begin
      InflateRect(ClipRect, 2, 1)
    end
    else begin
      ClipRect := ClientRect;
      InflateRect(ClipRect, -4, -4);
    end;
    FCanvas.Font.Color := clBlack;
    FCanvas.Brush.Color := Color;
    DrawFocusRect(FCanvas.Handle, ClipRect);
  end;
  FCanvas.Handle := 0;
end;

function TabcPicBtn.GetNumGlyphs: TabcNumGlyphs;
begin
  Result := TabcButtonGlyph(FGlyph).NumGlyphs;
end;

function TabcPicBtn.GetPalette: HPALETTE;
begin
  if Picture.Graphic is TBitmap then
    Result := Picture.Bitmap.Palette
  else
    Result := inherited GetPalette;
end;

function TabcPicBtn.GetPicture: TPicture;
begin
  Result := TabcButtonGlyph(FGlyph).Picture;
end;

function TabcPicBtn.GetTransparentColor: TColor;
begin
  Result := TabcButtonGlyph(FGlyph).TransparentColor;
end;

procedure TabcPicBtn.GlyphChanged(Sender: TObject);
begin
  Invalidate;
end;

function TabcPicBtn.IsTransColorStored: Boolean;
begin
  Result := TabcButtonGlyph(FGlyph).TransparentColorChanged;
end;

procedure TabcPicBtn.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

function TabcPicBtn.MouseIsOver: Boolean;
var
  P: TPoint;
begin
  GetCursorPos(P);
  Result := (FindDragTarget(P, True) = Self);
end;

procedure TabcPicBtn.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled and
     (SoundDown > -1) and (SoundList <> nil) then
    SoundList.PlayItem(SoundDown);
end;

procedure TabcPicBtn.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TabcPicBtn.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if Dragging then begin
    UpdateMouseOver;
    Invalidate;
  end;
end;

procedure TabcPicBtn.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
    if (AComponent = Images) then
      SetImages(nil)
    else if (AComponent = FSoundList) then
      FSoundList := nil;
end;

procedure TabcPicBtn.SetButtonStyle(ADefault: Boolean);
begin
  if ADefault <> FIsFocused then
  begin
    FIsFocused := ADefault;
    Refresh;
  end;
end;

procedure TabcPicBtn.SetDisabledShadow(Value: Boolean);
begin
  if Value <> FDisabledShadow then begin
    FDisabledShadow := Value;
    if not Enabled then Invalidate;
  end;
end;

procedure TabcPicBtn.SetPictureHighlight(Value: TabcGlyphHighlight);
begin
  if Value <> FPictureHighlight then begin
    FPictureHighlight := Value;
    TabcButtonGlyph(FGlyph).GlyphHighlight := Value;
    Repaint;
  end;
end;

procedure TabcPicBtn.SetLayout(Value: TabcButtonLayout);
begin
  if FLayout <> Value then
  begin
    FLayout := Value;
    Invalidate;
  end;
end;

procedure TabcPicBtn.SetMargin(Value: Integer);
begin
  if (Value <> FMargin) and (Value >= - 1) then
  begin
    FMargin := Value;
    Invalidate;
  end;
end;

procedure TabcPicBtn.SetNumGlyphs(Value: TabcNumGlyphs);
begin
  if Value < 0 then
    Value := 1
  else if Value > High(TabcNumGlyphs) then
    Value := High(TabcNumGlyphs);
  if Value <> TabcButtonGlyph(FGlyph).NumGlyphs then begin
    TabcButtonGlyph(FGlyph).NumGlyphs := Value;
    Invalidate;
  end;
end;

procedure TabcPicBtn.SetPicture(Value: TPicture);
begin
  TabcButtonGlyph(FGlyph).Picture := Value;
end;

procedure TabcPicBtn.SetPopup(Value: Boolean);
begin
  if Value <> FPopup then begin
    FPopup := Value;
    Invalidate;
  end;
end;

procedure TabcPicBtn.SetSpacing(Value: Integer);
begin
  if FSpacing <> Value then
  begin
    FSpacing := Value;
    Invalidate;
  end;
end;

procedure TabcPicBtn.SetStyle(Value: TabcButtonStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TabcPicBtn.SetTextStyle( Value : TabcTextStyle );
begin
  if Value <> FTextStyle then begin
    FTextStyle := Value;
    Invalidate;
  end;
end;

procedure TabcPicBtn.SetTransparentColor(Value: TColor);
begin
  TabcButtonGlyph(FGlyph).TransparentColor := Value;
end;

procedure TabcPicBtn.UpdateMouseOver;
begin
  if Popup then
    if MouseIsOver then
      Perform(CM_MOUSEENTER, 0, 0)
    else
      Perform(CM_MOUSELEAVE, 0, 0);
end;

procedure TabcPicBtn.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  Perform(WM_LBUTTONDOWN, Message.Keys, Longint(Message.Pos));
end;

function TabcPicBtn.IsImageIndexStored: Boolean;
begin
  Result := (ActionLink = nil) or not TabcPicBtnActionLink(ActionLink).IsImageIndexLinked;
end;

procedure TabcPicBtn.ActionChange (Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
    begin
      if not CheckDefaults or (Self.ImageIndex = -1) then
        Self.ImageIndex := ImageIndex;
    end;
end;

function TabcPicBtn.GetActionLinkClass: TControlActionLinkClass;
begin
  Result := TabcPicBtnActionLink;
end;

function TabcPicBtn.GetImageIndex: TImageIndex;
begin
  Result := TabcButtonGlyph(FGlyph).FImageIndex;
end;

procedure TabcPicBtn.SetImageIndex (Value: TImageIndex);
begin
  with TabcButtonGlyph(FGlyph) do
    if FImageIndex <> Value then
    begin
      FImageIndex := Value;
      if Assigned(FImageList) then
        UpdateNumGlyphs;
    end;
end;

function TabcPicBtn.GetImages: TCustomImageList;
begin
  if FGlyph <> nil then
    Result := TabcButtonGlyph(FGlyph).FImageList
  else
    Result := nil;
end;

procedure TabcPicBtn.SetImages (Value: TCustomImageList);
begin
  if FGlyph <> nil then
    with TabcButtonGlyph(FGlyph) do
      if FImageList <> Value then
      begin
        if FImageList <> nil then
          FImageList.UnRegisterChanges (FImageChangeLink);
        FImageList := Value;
        if FImageList <> nil then
        begin
          if FImageChangeLink = nil then
          begin
            FImageChangeLink := TChangeLink.Create;
            FImageChangeLink.OnChange := PictureChanged;
          end;
          FImageList.RegisterChanges (FImageChangeLink);
          FImageList.FreeNotification (Self);
        end
        else
        begin
          FImageChangeLink.Free;
          FImageChangeLink := nil;
        end;
        if not (csDestroying in ComponentState) then UpdateNumGlyphs;
      end;
end;

{ TabcPicBtnActionLink }

procedure TabcPicBtnActionLink.AssignClient (AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TabcPicBtn;
end;

function TabcPicBtnActionLink.IsImageIndexLinked: Boolean;
begin
  Result := inherited IsImageIndexLinked and
    (FClient.ImageIndex = (Action as TCustomAction).ImageIndex);
end;

procedure TabcPicBtnActionLink.SetImageIndex (Value: Integer);
begin
  if IsImageIndexLinked then FClient.ImageIndex := Value;
end;

{ TabcCustomPicSpeedBtn }

constructor TabcCustomPicSpeedBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetBounds(0, 0, 23, 22);
  ControlStyle := [csCaptureMouse, csOpaque, csDoubleClicks, csReplicatable];
  FPopup := True;
  FStyle := absThin;
  FGlyph := TabcButtonGlyph.Create;
  TabcButtonGlyph(FGlyph).FBtnFaceColor := clBtnFace;
  TabcButtonGlyph(FGlyph).OnChange := GlyphChanged;
  FTextStyle := tsNormal;
  ParentFont := True;
  FSpacing := 4;
  FMargin := -1;
  FLayout := ablGlyphLeft;
  Color := clBtnFace;
  FDisabledShadow := True;
  FSoundDown := -1;
  FSoundOver := -1;
end;

destructor TabcCustomPicSpeedBtn.Destroy;
begin
  if not (csDestroying in ComponentState) then Destroying;
  ReleaseBrushPattern;
  TabcButtonGlyph(FGlyph).Free;
  FGlyph := nil;
  inherited Destroy;
end;

function TabcCustomPicSpeedBtn.CancelBeforeMouseDown(var Message: TWMMouse;
  Button: TMouseButton; Shift: TShiftState): Boolean;
var
  Cancel: Boolean;
  ParentForm: TCustomForm;
begin
  Cancel := False;
  if Assigned(FBeforeMouseDown) then
    with Message do begin
      FBeforeMouseDown(Self, Button,
        KeysToShiftState(Keys) + Shift, XPos, YPos, Cancel);
      if Cancel then begin
        FMouseCancel := True;
        ParentForm := GetParentForm(Self);
        if ParentForm <> nil then ParentForm.SendCancelMode(Self);
      end;
    end;
  Result := Cancel;
end;

procedure TabcCustomPicSpeedBtn.Click;
var
  Cancel: Boolean;
begin
  Cancel := False;
  if Assigned(FBeforeClick) then FBeforeClick(Self, Cancel);
  if not Cancel then begin
    ValidClick;
    inherited Click;
  end;
end;

procedure TabcCustomPicSpeedBtn.CMButtonPressed(var Message: TMessage);
var
  Sender: TabcCustomPicSpeedBtn;
begin
  if Message.WParam = FGroupIndex then
  begin
    Sender := TabcCustomPicSpeedBtn(Message.LParam);
    if Sender <> Self then
    begin
      if Sender.Down and FDown then
      begin
        FDown := False;
        FState := absUp;
        Invalidate;
      end;
      FAllowAllUp := Sender.AllowAllUp;
    end;
  end;
end;

procedure TabcCustomPicSpeedBtn.CMColorChanged(var Message: TMessage);
begin
  TabcButtonGlyph(FGlyph).FBtnFaceColor := Color;
  ReleaseBrushPattern;
  if (FState = absMouseOver) and Popup and not MouseOver then
    FState := absUp;
  with TabcButtonGlyph(FGlyph) do begin
    Invalidate;
    CreateButtonGlyph(FState);
  end;
  inherited;
end;

procedure TabcCustomPicSpeedBtn.CMDialogChar(var Message: TCMDialogChar);
var
  shift: TShiftState;
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled then
    begin
      shift := KeyDataToShiftState(Message.KeyData);
      if ssAlt in shift then
      begin
        MouseDown(mbLeft, shift, 0, 0);
        MouseUp(mbLeft, shift, 0, 0);
        Result := 1;
      end
      else
        inherited;
    end
    else
      inherited;
end;

procedure TabcCustomPicSpeedBtn.CMEnabledChanged(var Message: TMessage);
const
  NewState: array[Boolean] of TabcButtonState = (absDisabled, absUp);
begin
  if not (csDestroying in ComponentState) then
  begin
    TabcButtonGlyph(FGlyph).CreateButtonGlyph(NewState[Enabled]);
    UpdateMouseOver;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
end;

function TabcCustomPicSpeedBtn.HasMouseEffect: Boolean;
begin
  Result := Popup or
    (PictureHighlight <> aghNormal) or
    (Style in [absThinHighlight, absMidHighlight, absThickHighlight,
               absFramedHighlight, absFramedBlackHighlight,
               absRecessedHighlight, absRecessedBlackHighlight]);
end;

procedure TabcCustomPicSpeedBtn.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not ForegroundTask then Exit;
  if not FMouseOver and Enabled and (DragMode <> dmAutomatic)
    and (GetCapture = 0) then
  begin
    FMouseOver := True;
    if not Down then
      FState := absMouseOver;
    if HasMouseEffect then
      Repaint;
    if (SoundOver > -1) and (SoundList <> nil) then
      SoundList.PlayItem(SoundOver);
    MouseEnter;
  end;
end;

procedure TabcCustomPicSpeedBtn.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseOver := False;
  if Enabled and (DragMode <> dmAutomatic) and (GetCapture = 0) then
  begin
    if not Down then
      FState := absUp;
    if HasMouseEffect then
      Repaint;
    MouseLeave;
  end;
end;

procedure TabcCustomPicSpeedBtn.CMSysColorChange(var Message: TMessage);
begin
  ReleaseBrushPattern;
  with TabcButtonGlyph(FGlyph) do begin
    Invalidate;
    CreateButtonGlyph(FState);
  end;
end;

procedure TabcCustomPicSpeedBtn.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

function TabcCustomPicSpeedBtn.GetNumGlyphs: TabcNumGlyphs;
begin
  Result := TabcButtonGlyph(FGlyph).NumGlyphs;
end;

function TabcCustomPicSpeedBtn.GetPalette: HPALETTE;
begin
  if Picture.Graphic is TBitmap then
    Result := Picture.Bitmap.Palette
  else
    Result := inherited GetPalette;
end;

function TabcCustomPicSpeedBtn.GetPicture: TPicture;
begin
  if csDestroying in ComponentState then
    Result := nil
  else
    Result := TabcButtonGlyph(FGlyph).Picture;
end;

function TabcCustomPicSpeedBtn.GetTransparentColor: TColor;
begin
  Result := TabcButtonGlyph(FGlyph).TransparentColor;
end;

procedure TabcCustomPicSpeedBtn.GlyphChanged(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then Invalidate;
end;

function TabcCustomPicSpeedBtn.IsTransColorStored: Boolean;
begin
  Result := TabcButtonGlyph(FGlyph).TransparentColorChanged;
end;

procedure TabcCustomPicSpeedBtn.Loaded;
var
  State: TabcButtonState;
begin
  inherited Loaded;
  if Enabled then
    State := absUp
  else
    State := absDisabled;
  TabcButtonGlyph(FGlyph).CreateButtonGlyph(State);
  Invalidate;
end;

procedure TabcCustomPicSpeedBtn.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then
  begin
    if not FDown then begin
      FState := absDown;
      Repaint;
      if (SoundDown > -1) and (SoundList <> nil) then
        SoundList.PlayItem(SoundDown);
    end;
    FDragging := True;
  end;
end;

procedure TabcCustomPicSpeedBtn.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

function TabcCustomPicSpeedBtn.MouseIsOver: Boolean;
var
  P: TPoint;
begin
  GetCursorPos(P);
  Result := (FindDragTarget(P, True) = Self);
end;

procedure TabcCustomPicSpeedBtn.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TabcCustomPicSpeedBtn.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TabcButtonState;
begin
  inherited MouseMove(Shift, X, Y);
  if FDragging then begin
    if not FDown then
      NewState := absMouseOver
    else
      NewState := absExclusive;
    if (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight) then
      if FDown then
        NewState := absExclusive
      else
        NewState := absDown;
    if NewState <> FState then
    begin
      FState := NewState;
      Repaint;
    end;
  end
  else if not FMouseOver then
    UpdateMouseOver;
end;

procedure TabcCustomPicSpeedBtn.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  if FMouseCancel then begin
    FMouseCancel := False;
    Exit;
  end;
  inherited MouseUp(Button, Shift, X, Y);
  if FDragging then begin
    FDragging := False;
    DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
    if FGroupIndex = 0 then begin
      if DoClick then
        FState := absMouseOver;
      FMouseOver := False;
      if not (FState in [absExclusive, absDown]) then Repaint;
    end
    else begin
      if DoClick then
        SetDown(not FDown)
      else begin
        if FDown then FState := absExclusive;
        Repaint;
      end;
    end;
    if DoClick then Click;
    if MouseIsOver then
      FMouseOver := True
    else
      Perform(CM_MOUSELEAVE,0,0);
    Repaint;
  end;
end;

procedure TabcCustomPicSpeedBtn.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
    if (AComponent = Images) then
      SetImages(nil)
    else if (AComponent = FSoundList) then
      FSoundList := nil;
end;

procedure TabcCustomPicSpeedBtn.Paint;
var
  PaintRect, TileRect, ClipRect: TRect;
  IsMouseOver: Boolean;
  FillBtnFace: TabcButtonFill;
  PatternBMP: TBitmap;
begin
  if (csDestroying in ComponentState) then Exit;
  if not Enabled and not (csDesigning in ComponentState) then begin
    FState := absDisabled;
    FDragging := False;
  end
  else if FState = absDisabled then
    if FDown and (GroupIndex <> 0) then
      FState := absExclusive
    else
      FState := absUp;

  IsMouseOver := Enabled and MouseOver;

  if (FState = absExclusive) then begin
    FillBtnFace := abfPattern;
    if FBrushPattern = nil then begin
      FBrushPattern := GetBrushPattern(Color);
      TabcBrushPattern(FBrushPattern).Lock;
    end;
    PatternBMP := TabcBrushPattern(FBrushPattern).Bitmap;
  end
  else begin
    if (Layout <> ablGlyphTiled) or
            (Color <> TransparentColor) or
            (Picture.Graphic = nil) then
      FillBtnFace := abfSolid
    else
      FillBtnFace := abfNone;
    PatternBMP := nil;
  end;

  Canvas.Font := Self.Font;

  PaintRect := abcDrawButtonFace(Canvas, Rect(0, 0, Width, Height), FStyle,
    Popup, IsMouseOver, False, State in [absDown, absExclusive], False,
    FillBtnFace, Color, PatternBMP);

  ClipRect := PaintRect;
  OffsetRect(ClipRect, Left, Top);

  TileRect := ClientRect;
  if State in [absDown, absExclusive] then begin
    OffsetRect(PaintRect, 1, 1);
    OffsetRect(TileRect, 1, 1);
  end;

  TabcButtonGlyph(FGlyph).Draw(Canvas, PaintRect, TileRect, Caption, FLayout,
    FMargin, FSpacing, FState, FTextStyle, FDisabledShadow, IsMouseOver,
    DrawTextBiDiModeFlags(0));

  abcDrawButtonFace(Canvas, Rect(0, 0, Width, Height), FStyle,
    Popup, IsMouseOver, False, State in [absDown, absExclusive], False,
    abfNone, Color, PatternBMP);

  if (csDesigning in ComponentState) and Popup then
    with Canvas do begin
      Pen.Color := clBtnShadow;
      Pen.Style := psSolid;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
    
  if ShowFocus then
  begin
    ClipRect := ClientRect;
    InflateRect(ClipRect, -2, -2);
    Canvas.Font.Color := clBlack;
    Canvas.Brush.Color := Color;
    DrawFocusRect(Canvas.Handle, ClipRect);
  end;
end;

procedure TabcCustomPicSpeedBtn.ReleaseBrushPattern;
begin
  if FBrushPattern <> nil then begin
    TabcBrushPattern(FBrushPattern).Release;
    FBrushPattern := nil;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetAllowAllUp(Value: Boolean);
begin
  if FAllowAllUp <> Value then
  begin
    FAllowAllUp := Value;
    UpdateExclusive;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetDisabledShadow(Value: Boolean);
begin
  if Value <> FDisabledShadow then begin
    FDisabledShadow := Value;
    if not Enabled then Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetDown(Value: Boolean);
begin
  if FGroupIndex = 0 then Value := False;
  if Value <> FDown then
  begin
    if FDown and (not FAllowAllUp) then Exit;
    FDown := Value;
    if Value then
    begin
      if FState in [absUp, absMouseOver] then Invalidate;
      FState := absExclusive;
    end
    else
    begin
      if MouseOver then
        FState := absMouseOver
      else
        FState := absUp;
      Invalidate;
    end;
    if Value then UpdateExclusive;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetPictureHighlight(Value: TabcGlyphHighlight);
begin
  if Value <> FPictureHighlight then begin
    FPictureHighlight := Value;
    TabcButtonGlyph(FGlyph).GlyphHighlight := Value;
    Repaint;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetGroupIndex(Value: Integer);
begin
  if FGroupIndex <> Value then
  begin
    FGroupIndex := Value;
    UpdateExclusive;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetLayout(Value: TabcButtonLayout);
begin
  if FLayout <> Value then
  begin
    FLayout := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetMargin(Value: Integer);
begin
  if (Value <> FMargin) and (Value >= -1) then
  begin
    FMargin := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetNumGlyphs(Value: TabcNumGlyphs);
begin
  if Value < 0 then
    Value := 1
  else if Value > High(TabcNumGlyphs) then
    Value := High(TabcNumGlyphs);
  if Value <> TabcButtonGlyph(FGlyph).NumGlyphs then begin
    TabcButtonGlyph(FGlyph).NumGlyphs := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetSpacing(Value: Integer);
begin
  if Value <> FSpacing then
  begin
    FSpacing := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetPicture(Value: TPicture);
begin
  if not (csDestroying in ComponentState) then
    TabcButtonGlyph(FGlyph).Picture := Value;
end;

procedure TabcCustomPicSpeedBtn.SetPopup(Value: Boolean);
begin
  if Value <> FPopup then begin
    FPopup := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetShowFocus(Value: Boolean);
begin
  if Value <> FShowFocus then begin
    FShowFocus := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetStyle(Value: TabcButtonStyle);
begin
  if Value <> FStyle then begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetTextStyle( Value : TabcTextStyle );
begin
  if Value <> FTextStyle then begin
    FTextStyle := Value;
    Invalidate;
  end;
end;

procedure TabcCustomPicSpeedBtn.SetTransparentColor(Value: TColor);
begin
  TabcButtonGlyph(FGlyph).TransparentColor := Value;
end;

procedure TabcCustomPicSpeedBtn.UpdateExclusive;
var
  Msg: TMessage;
begin
  if (FGroupIndex <> 0) and (Parent <> nil) then
  begin
    Msg.Msg := CM_BUTTONPRESSED;
    Msg.WParam := FGroupIndex;
    Msg.LParam := Longint(Self);
    Msg.Result := 0;
    Parent.Broadcast(Msg);
  end;
end;

procedure TabcCustomPicSpeedBtn.UpdateMouseOver;
begin
  if Popup then
    if MouseIsOver then
      Perform(CM_MOUSEENTER, 0, 0)
    else
      Perform(CM_MOUSELEAVE, 0, 0);
end;

procedure TabcCustomPicSpeedBtn.ValidClick;
begin
end;

procedure TabcCustomPicSpeedBtn.WMLButtonDown(var Message: TWMLButtonDown);
begin
  if not CancelBeforeMouseDown(TWMMouse(Message), mbLeft, [ssLeft]) then inherited;
end;

procedure TabcCustomPicSpeedBtn.WMLButtonDblClk(var Message: TWMLButtonDown);
begin
  if not CancelBeforeMouseDown(TWMMouse(Message), mbLeft, [ssDouble, SSLeft]) then begin
    inherited;
    if FDown then DblClick;
  end;
end;

procedure TabcCustomPicSpeedBtn.WMMButtonDown(var Message: TWMMButtonDown);
begin
  if not CancelBeforeMouseDown(TWMMouse(Message), mbMiddle, [ssMiddle]) then inherited;
end;

procedure TabcCustomPicSpeedBtn.WMNCLButtonDown(var Message: TWMNCLButtonDown);
begin
  if not CancelBeforeMouseDown(TWMMouse(Message), mbLeft, [ssLeft]) then inherited;
end;

procedure TabcCustomPicSpeedBtn.WMNCRButtonDown(var Message: TWMNCRButtonDown);
begin
  if not CancelBeforeMouseDown(TWMMouse(Message), mbRight, [ssRight]) then inherited;
end;

procedure TabcCustomPicSpeedBtn.WMRButtonDown(var Message: TWMRButtonDown);
begin
  if not CancelBeforeMouseDown(TWMMouse(Message), mbRight, [ssRight]) then inherited;
end;

function TabcCustomPicSpeedBtn.IsCheckedStored: Boolean;
begin
  Result := (ActionLink = nil) or not TabcPicSpeedBtnActionLink(ActionLink).IsCheckedLinked;
end;

function TabcCustomPicSpeedBtn.IsImageIndexStored: Boolean;
begin
  Result := (ActionLink = nil) or not TabcPicSpeedBtnActionLink(ActionLink).IsImageIndexLinked;
end;

procedure TabcCustomPicSpeedBtn.ActionChange (Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
    begin
      if not CheckDefaults or (Self.Down = False) then
        Self.Down := Checked;
      if not CheckDefaults or (Self.ImageIndex = -1) then
        Self.ImageIndex := ImageIndex;
    end;
end;

function TabcCustomPicSpeedBtn.GetActionLinkClass: TControlActionLinkClass;
begin
  Result := TabcPicSpeedBtnActionLink;
end;

procedure TabcCustomPicSpeedBtn.AssignTo (Dest: TPersistent);
begin
  inherited;
  if Dest is TCustomAction then
    TCustomAction(Dest).Checked := Self.Down;
end;

function TabcCustomPicSpeedBtn.GetImageIndex: TImageIndex;
begin
  Result := TabcButtonGlyph(FGlyph).FImageIndex;
end;

procedure TabcCustomPicSpeedBtn.SetImageIndex (Value: TImageIndex);
begin
  with TabcButtonGlyph(FGlyph) do
    if FImageIndex <> Value then
    begin
      FImageIndex := Value;
      if Assigned(FImageList) then
        UpdateNumGlyphs;
    end;
end;

function TabcCustomPicSpeedBtn.GetImages: TCustomImageList;
begin
  if FGlyph <> nil then
    Result := TabcButtonGlyph(FGlyph).FImageList
  else
    Result := nil;
end;

procedure TabcCustomPicSpeedBtn.SetImages (Value: TCustomImageList);
begin
  if FGlyph <> nil then
    with TabcButtonGlyph(FGlyph) do
      if FImageList <> Value then
      begin
        if FImageList <> nil then
          FImageList.UnRegisterChanges (FImageChangeLink);
        FImageList := Value;
        if FImageList <> nil then
        begin
          if FImageChangeLink = nil then
          begin
            FImageChangeLink := TChangeLink.Create;
            FImageChangeLink.OnChange := PictureChanged;
          end;
          FImageList.RegisterChanges (FImageChangeLink);
          FImageList.FreeNotification (Self);
        end
        else
        begin
          FImageChangeLink.Free;
          FImageChangeLink := nil;
        end;
        if not (csDestroying in ComponentState) then UpdateNumGlyphs;
      end;
end;

{ TabcPicSpeedBtnActionLink }

procedure TabcPicSpeedBtnActionLink.AssignClient (AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TabcCustomPicSpeedBtn;
end;

function TabcPicSpeedBtnActionLink.IsCheckedLinked: Boolean;
begin
  Result := inherited IsCheckedLinked and
    (FClient.Down = (Action as TCustomAction).Checked);
end;

function TabcPicSpeedBtnActionLink.IsImageIndexLinked: Boolean;
begin
  Result := inherited IsImageIndexLinked and
    (FClient.ImageIndex = (Action as TCustomAction).ImageIndex);
end;

procedure TabcPicSpeedBtnActionLink.SetChecked (Value: Boolean);
begin
  if IsCheckedLinked then FClient.Down := Value;
end;

procedure TabcPicSpeedBtnActionLink.SetImageIndex (Value: Integer);
begin
  if IsImageIndexLinked then FClient.ImageIndex := Value;
end;

{ TabcRepSpeedBtn }

constructor TabcRepSpeedBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInitRepeatPause := 400;
  FRepeatPause := 100;
  FRepeatEnabled := True;
end;

destructor TabcRepSpeedBtn.Destroy;
begin
  FRepeatTimer.Free;
  inherited Destroy;
end;

procedure TabcRepSpeedBtn.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown (Button, Shift, X, Y);
  if FRepeatEnabled and (Assigned(FOnTimer)) then begin
    if FRepeatTimer = nil then
      FRepeatTimer := TTimer.Create(Self);
    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := FInitRepeatPause;
    FRepeatTimer.Enabled  := True;
  end;
end;

procedure TabcRepSpeedBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil then
    FRepeatTimer.Enabled  := False;
end;

procedure TabcRepSpeedBtn.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Enabled := False;
  if (FState = absDown) and MouseCapture then
    begin
      if Assigned(FOnTimer) then FOnTimer(Self);
      if FRepeatPause <> 0 then
        begin
          FRepeatTimer.Interval := FRepeatPause;
          FRepeatTimer.Enabled := True;
        end;
    end;
end;

{ TabcToggleBtn }

constructor TabcToggleBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  GroupIndex := GetTickCount;
  AllowAllUp := True;
  Popup := False;
end;

procedure TabcToggleBtn.Click;
begin
  inherited Click;
  if Down then begin
    inherited Hint := HintDown;
    if Assigned(FOnClickDown) then FOnClickDown(Self);
  end
  else begin
    inherited Hint := HintUp;
    if Assigned(FOnClickUp) then FOnClickUp(Self);
  end;
end;

procedure TabcToggleBtn.CMButtonPressed(var Message: TMessage);
begin
  inherited;
  if Message.WParam = FGroupIndex then
    if Down then
      inherited Hint := FHintDown
    else
      inherited Hint := FHintUp;
end;

function TabcToggleBtn.GetHint: string;
begin
  Result := inherited Hint;
end;

procedure TabcToggleBtn.SetHintDown(Value: string);
begin
  if Value <> FHintDown then begin
    FHintDown := Value;
    if Down then inherited Hint := Value;
  end;
end;

procedure TabcToggleBtn.SetHintUp(Value: string);
begin
  if Value <> FHintUp then begin
    FHintUp := Value;
    if not Down then inherited Hint := Value;
  end;
end;

{ TabcMouseRegion }

constructor TabcMouseRegion.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOutlineColor := clGray;
  ControlStyle := [csCaptureMouse, csDoubleClicks];
  SetBounds(0, 0, 40, 40);
  FSoundDown := -1;
  FSoundLeave := -1;
  FSoundOver := -1;
end;

procedure TabcMouseRegion.Click;
{make protected method public}
begin
  inherited Click;
end;

procedure TabcMouseRegion.CMDialogChar(var Message: TCMDialogChar);
var
  shift: TShiftState;
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled then
    begin
      shift := KeyDataToShiftState(Message.KeyData);
      if ssAlt in shift then
      begin
        MouseDown(mbLeft, shift, 0, 0);
        MouseUp(mbLeft, shift, 0, 0);
        Result := 1;
      end
      else
        inherited;
    end
    else
      inherited;
end;

procedure TabcMouseRegion.CMEnabledChanged(var Message: TMessage);
begin
  if not (csDestroying in ComponentState) then
    UpdateMouseOver;
end;

procedure TabcMouseRegion.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not ForegroundTask then Exit;
  if not FMouseOver and Enabled and (DragMode <> dmAutomatic)
    and (GetCapture = 0) then
  begin
    FMouseOver := True;
    if (SoundOver > -1) and (SoundList <> nil) then
      SoundList.PlayItem(SoundOver);
    MouseEnter;
  end;
end;

procedure TabcMouseRegion.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseOver := False;
  if Enabled and (DragMode <> dmAutomatic) and (GetCapture = 0) then
  begin
    if (SoundOver > -1) and (SoundList <> nil) then
      SoundList.PlayItem(SoundLeave);
    MouseLeave;
  end;
end;

procedure TabcMouseRegion.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then
  begin
    if not FDown then
    begin
      FState := bsDown;
      if (SoundDown > -1) and (SoundList <> nil) then
        SoundList.PlayItem(SoundDown);
    end;
    FDragging := True;
  end;
end;

procedure TabcMouseRegion.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TButtonState;
begin
  inherited MouseMove(Shift, X, Y);
  if FDragging then
  begin
    if not FDown then NewState := bsUp
    else NewState := bsExclusive;
    if (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight) then
      if FDown then NewState := bsExclusive else NewState := bsDown;
    if NewState <> FState then
      FState := NewState;
  end
  else if not FMouseOver then
    UpdateMouseOver;
end;

procedure TabcMouseRegion.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if FDragging then
  begin
    FDragging := False;
    DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
    FState := bsUp;
    if DoClick then Click;
    if MouseIsOver then
      FMouseOver := True
    else
      Perform(CM_MOUSELEAVE,0,0);
  end;
end;

procedure TabcMouseRegion.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

function TabcMouseRegion.MouseIsOver: Boolean;
var
  P: TPoint;
begin
  GetCursorPos(P);
  Result := (FindDragTarget(P, True) = Self);
end;

procedure TabcMouseRegion.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TabcMouseRegion.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FSoundList) then
    FSoundList := nil;
end;

procedure TabcMouseRegion.Paint;
begin
  if (csDesigning in ComponentState) or FOutline then
   {Draw Border Outline}
    with Canvas do
    begin
      Pen.Style := psSolid;
      Pen.Color := FOutlineColor;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
end;

procedure TabcMouseRegion.SetOutline(Value: Boolean);
begin
  if Value <> FOutline then begin
    FOutline := Value;
    Invalidate;
  end;
end;

procedure TabcMouseRegion.SetOutlineColor(Value: TColor);
begin
  if Value <> FOutlineColor then begin
    FOutlineColor := Value;
    Invalidate;
  end;
end;

procedure TabcMouseRegion.UpdateMouseOver;
begin
  if MouseIsOver then
    Perform(CM_MOUSEENTER, 0, 0)
  else
    Perform(CM_MOUSELEAVE, 0, 0);
end;

procedure TabcMouseRegion.WMLButtonDblClk(var Message: TWMLButtonDown);
begin
  inherited;
  DblClick;
end;

{ TabcMenuButton }

procedure TabcMenuButton.Click;
begin
  if FMenuItem <> nil then
    FMenuItem.Click;
  inherited Click;
end;

function TabcMenuButton.GetEnabled: Boolean;
begin
  if FMenuItem <> nil then
    Result := FMenuItem.Enabled
  else
    Result := inherited GetEnabled;
end;

function TabcMenuButton.GetHint: string;
begin
  if FMenuItem <> nil then
    Result := FMenuItem.Hint
  else
    Result := inherited Hint;
end;

function TabcMenuButton.GetMenuCaption: string;
begin
  if FMenuItem <> nil then
    Result := FMenuItem.Caption
  else
    Result := '';
end;

function TabcMenuButton.GetVisible: Boolean;
begin
  if FMenuItem <> nil then
    Result := FMenuItem.Visible
  else
    Result := inherited Visible;
end;

function TabcMenuButton.IsMenuPropStored: Boolean;
begin
  Result := FMenuItem = nil;
end;

procedure TabcMenuButton.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FMenuItem) then begin
    FMenuItem := nil;
    Enabled := False;
  end;
end;

procedure TabcMenuButton.SetEnabled(Value: Boolean);
begin
  inherited SetEnabled(Value);
  if FMenuItem <> nil then FMenuItem.Enabled := Value;
end;

procedure TabcMenuButton.SetHint(Value: string);
begin
  inherited Hint := Value;
  if FMenuItem <> nil then FMenuItem.Hint := Value;
end;

procedure TabcMenuButton.SetMenuCaption(Value: string);
begin
  if FMenuItem <> nil then FMenuItem.Caption := Value;
end;

procedure TabcMenuButton.SetMenuItem(Value: TMenuItem);
begin
  FMenuItem := Value;
  if Value <> nil then begin
    SetEnabled(FMenuItem.Enabled);
    SetHint(FMenuItem.Hint);
    SetVisible(FMenuItem.Visible);
  end;
end;

procedure TabcMenuButton.SetVisible(Value: Boolean);
begin
  inherited Visible := Value;
  if FMenuItem <> nil then FMenuItem.Visible := Value;
end;

{ TabcSizeButton }

constructor TabcSizeButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AutoSize := True;
  FMarginH := 8;
  FMarginV := 8;
  AdjustBounds;
end;

procedure TabcSizeButton.AdjustBounds;
var
  Rect: TRect;
begin
  if not (csReading in ComponentState) and AutoSize
    and HandleAllocated then
  begin
    Rect := ClientRect;
    DoDrawText(Rect, (DT_EXPANDTABS or DT_CALCRECT));
    SetBounds(Left, Top, Rect.Right + 2 * FMarginH, Rect.Bottom + 2 * FMarginV);
  end;
end;

procedure TabcSizeButton.CMFontChanged(var Message: TMessage);
begin
  inherited;
  AdjustBounds;
end;

procedure TabcSizeButton.CMTextChanged(var Message: TMessage);
begin
  inherited;
  AdjustBounds;
end;

procedure TabcSizeButton.DoDrawText(var Rect: TRect; Flags: Word);
var
  Text: array[0..255] of Char;
  DC: HDC;
  SaveFont: HFont;
begin
  GetTextBuf(Text, SizeOf(Text));
  if (Flags and DT_CALCRECT <> 0) and ((Text[0] = #0) or
    (Text[0] = '&') and (Text[1] = #0)) then StrCopy(Text, ' ');
  DC := GetDC(Handle);
  SaveFont := SelectObject(DC, Font.Handle);
  DrawText(DC, Text, StrLen(Text), Rect, Flags);
  SelectObject(DC, SaveFont);
  ReleaseDC(Handle, DC);
end;

procedure TabcSizeButton.AdjustSize;
begin
  AdjustBounds;
end;

procedure TabcSizeButton.SetMarginH(Value: Integer);
begin
  if Value <> FMarginH then begin
    FMarginH := Value;
    AdjustBounds;
  end;
end;

procedure TabcSizeButton.SetMarginV(Value: Integer);
begin
  if Value <> FMarginV then begin
    FMarginV := Value;
    AdjustBounds;
  end;
end;

{ TabcFormButton }

destructor TabcFormButton.Destroy;
begin
  FForm.Free;
  inherited Destroy;
end;

procedure TabcFormButton.Click;
var
  AClass: TPersistentClass;
  AFormClass: TFormClass;
begin
  inherited Click;
  if FFormName = '' then Exit;
  if FForm = nil then begin
    AClass := FindClass('T' + FFormName);
    AFormClass := TFormClass(AClass);
    FForm := AFormClass.Create(Self);
  end;
  if FShowModal then
    FFormModalResult := FForm.ShowModal
  else
    FForm.Show;
  if Assigned(FAfterShow) then FAfterShow(Self);
end;

procedure TabcFormButton.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = FForm) and (Operation = opRemove) then FForm := nil;
end;

{ TabcFormPicBtn }

destructor TabcFormPicBtn.Destroy;
begin
  FForm.Free;
  inherited Destroy;
end;

procedure TabcFormPicBtn.Click;
var
  AClass: TPersistentClass;
  AFormClass: TFormClass;
begin
  inherited Click;
  if FFormName = '' then Exit;
  if FForm = nil then begin
    AClass := FindClass('T' + FFormName);
    AFormClass := TFormClass(AClass);
    FForm := AFormClass.Create(Self);
  end;
  if FShowModal then
    FFormModalResult := FForm.ShowModal
  else
    FForm.Show;
  if Assigned(FAfterShow) then FAfterShow(Self);
end;

procedure TabcFormPicBtn.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = FForm) and (Operation = opRemove) then FForm := nil;
end;

{ TabcFormPicSpeedBtn }

destructor TabcFormPicSpeedBtn.Destroy;
begin
  FForm.Free;
  inherited Destroy;
end;

procedure TabcFormPicSpeedBtn.Click;
var
  AClass: TPersistentClass;
  AFormClass: TFormClass;
begin
  inherited Click;
  if FFormName = '' then Exit;
  if FForm = nil then begin
    AClass := FindClass('T' + FFormName);
    AFormClass := TFormClass(AClass);
    FForm := AFormClass.Create(Self);
  end;
  if FShowModal then
    FFormModalResult := FForm.ShowModal
  else
    FForm.Show;
  if Assigned(FAfterShow) then FAfterShow(Self);
end;

procedure TabcFormPicSpeedBtn.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = FForm) and (Operation = opRemove) then FForm := nil;
end;

{ TabcWaveBtn }

constructor TabcWaveBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWave := TabcWave.Create(Self);
end;

destructor TabcWaveBtn.Destroy;
begin
  FWave.Free;
  inherited Destroy;
end;

procedure TabcWaveBtn.Click;
begin
  if not FWave.Sound.Empty then FWave.Play;
  inherited Click;
end;

function TabcWaveBtn.GetSound: TabcWaveImage;
begin
  Result := FWave.Sound;
end;

procedure TabcWaveBtn.SetPlaySync(Value: Boolean);
begin
  if Value <> FPlaySync then begin
    if Value then
      FWave.PlayOption := poSync
    else
      FWave.PlayOption := poAsync;
    FPlaySync := Value;
  end;
end;

procedure TabcWaveBtn.SetSound(Value: TabcWaveImage);
begin
  if Value <> FWave.Sound then
    FWave.Sound.Assign(Value);
end;

{ TabcWaveFileBtn }

constructor TabcWaveFileBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWaveFile := TabcWaveFile.Create(Self);
end;

destructor TabcWaveFileBtn.Destroy;
begin
  FWaveFile.Free;
  inherited Destroy;
end;

procedure TabcWaveFileBtn.Click;
begin
  if FileName <> '' then FWaveFile.Play;
  inherited Click;
end;

function TabcWaveFileBtn.GetFileName: TFileName;
begin
  Result := FWaveFile.FileName;
end;

procedure TabcWaveFileBtn.SetFileName(Value: TFileName);
begin
  if Value <> FWaveFile.FileName then
    FWaveFile.FileName := Value;
end;

procedure TabcWaveFileBtn.SetPlaySync(Value: Boolean);
begin
  if Value <> FPlaySync then begin
    if Value then
      FWaveFile.PlayOption := poSync
    else
      FWaveFile.PlayOption := poAsync;
    FPlaySync := Value;
  end;
end;

procedure FreeBrushPatterns;
var
  I: integer;
  ABrushPattern: TabcBrushPattern;
begin
  with BrushPatternList do
    for I := Count - 1 downto 0 do begin
      ABrushPattern := TabcBrushPattern(Items[I]);
      Delete(I);
      ABrushPattern.Free;
    end;
  BrushPatternList.Free;
  BrushPatternList := nil;
end;

initialization
  BrushPatternList := TList.Create;

finalization
  FreeBrushPatterns;

end.
