unit abcbtns;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{$S-,W-,R-}
{$C PRELOAD}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Buttons,
  ExtCtrls, Forms, Dialogs, ImgList, CommCtrl,
  abcexctl, abcbtn, abcwav;

type

  {$IFNDEF DELPHI5}
    TImageIndex = type Integer;
  {$ENDIF}

  TabcCustomShapeButton = class(TGraphicControl)
  private
    { Private declarations }
    FChangeLink: TChangeLink;
    FDragging: Boolean;
    FImageList: TImageList;
    FImageListCount: Integer;
    FIndexDisabled: TImageIndex;
    FIndexDown: TImageIndex;
    FIndexFlat: TImageIndex;
    FIndexMouseMask: TImageIndex;
    FIndexUp: TImageIndex;
    FMouseMask: TBitmap;
    FMouseOver: Boolean;
    FOffsetDownX: Integer;
    FOffsetDownY: Integer;
    FOffsetUpX: Integer;
    FOffsetUpY: Integer;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FPopup: Boolean;
    FScaled: Boolean;
    FSoundList: TabcCustomWaveList;
    FSoundDown: Integer;
    FSoundOver: Integer;
    procedure SetImageList(Value: TImageList);
    procedure SetIndexDisabled(Value: TImageIndex);
    procedure SetIndexDown(Value: TImageIndex);
    procedure SetIndexFlat(Value: TImageIndex);
    procedure SetIndexMouseMask(Value: TImageIndex);
    procedure SetIndexUp(Value: TImageIndex);
    procedure SetPopup(Value: Boolean);
    procedure SetScaled(Value: Boolean);

    function PosInShape(X, Y: Integer): Boolean;
    function MouseIsOver: Boolean;
    procedure UpdateMouseMask;

    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMHitTest(var Message: TCMHitTest); message CM_HITTEST;

  protected
    FState: TButtonState;
    procedure ChangeScale(M, D: Integer); override;
    procedure ImageListChanged(Sender: TObject);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseEnter; dynamic;
    procedure MouseLeave; dynamic;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

    procedure Paint; override;

    property ImageList: TImageList read FImageList write SetImageList;
    property IndexDisabled: TImageIndex read FIndexDisabled write SetIndexDisabled default -1;
    property IndexDown: TImageIndex read FIndexDown write SetIndexDown default -1;
    property IndexFlat: TImageIndex read FIndexFlat write SetIndexFlat default -1;
    property IndexMouseMask: TImageIndex read FIndexMouseMask write SetIndexMouseMask default -1;
    property IndexUp: TImageIndex read FIndexUp write SetIndexUp default -1;
    property MouseOver: Boolean read FMouseOver;
    property OffsetDownX: Integer read FOffsetDownX write FOffsetDownX default 1;
    property OffsetDownY: Integer read FOffsetDownY write FOffsetDownY default 1;
    property OffsetUpX: Integer read FOffsetUpX write FOffsetUpX default -1;
    property OffsetUpY: Integer read FOffsetUpY write FOffsetUpY default -1;
    property Popup: Boolean read FPopup write SetPopup default True;
    property Scaled: Boolean read FScaled write SetScaled default False;
    property SoundDown: Integer read FSoundDown write FSoundDown default -1;
    property SoundList: TabcCustomWaveList read FSoundList write FSoundList;
    property SoundOver: Integer read FSoundOver write FSoundOver default -1;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function IsValidIndex(ImageIndex: TImageIndex): Boolean;
  end;

  TabcShapeButton = class(TabcCustomShapeButton)
  public
    procedure Click; override;
    property MouseOver;
  published
    property Action;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property Caption;
    property Enabled;
    property Font;
    property ImageList;
    property IndexDisabled;
    property IndexDown;
    property IndexFlat;
    property IndexMouseMask;
    property IndexUp;
    property OffsetDownX;
    property OffsetDownY;
    property OffsetUpX;
    property OffsetUpY;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property Popup;
    property PopupMenu;
    property Scaled;
    property ShowHint;
    property SoundDown;
    property SoundList;
    property SoundOver;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

uses abcedge;

constructor TabcCustomShapeButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csDoubleClicks, csFixedWidth, csFixedHeight];
  ParentFont := True;
  FChangeLink := TChangeLink.Create;
  FChangeLink.OnChange := ImageListChanged;
  FMouseMask := TBitmap.Create;
  FIndexDisabled := -1;
  FIndexDown := -1;
  FIndexFlat := -1;
  FIndexMouseMask := -1;
  FIndexUp := -1;
  FPopup := True;
  Width := 75;
  Height := 50;
  FOffsetDownX := 1;
  FOffsetDownY := 1;
  FOffsetUpX := -1;
  FOffsetUpY := -1;
  FSoundDown := -1;
  FSoundOver := -1;
end;

destructor TabcCustomShapeButton.Destroy;
begin
  FMouseMask.Free;
  SetImageList(nil);
  FChangeLink.Free;
  inherited Destroy;
end;

procedure TabcCustomShapeButton.Assign(Source: TPersistent);
begin
  if Source is TabcCustomShapeButton then
    with TabcCustomShapeButton(Source) do
    begin
      {copy all published properties}
      Self.Caption := Caption;
      Self.Cursor := Cursor;
      Self.Enabled := Enabled;
      Self.Font.Assign(Font);
      Self.Hint := Hint;
      Self.IndexDisabled := IndexDisabled;
      Self.IndexDown := IndexDown;
      Self.IndexFlat := IndexFlat;
      Self.IndexMouseMask := IndexMouseMask;
      Self.IndexUp := IndexUp;
      Self.ImageList := ImageList;
      Self.OffsetDownX := OffsetDownX;
      Self.OffsetDownY := OffsetDownY;
      Self.OffsetUpX := OffsetUpX;
      Self.OffsetUpY := OffsetUpY;
      Self.ParentColor := ParentColor;
      Self.ParentFont := ParentFont;
      Self.ParentShowHint := ParentShowHint;
      Self.Popup := Popup;
      Self.ShowHint := ShowHint;
      Self.Tag := Tag;
      Self.Visible := Visible;
      Self.OnClick := OnClick;
      Self.OnDblClick := OnDblClick;
      Self.OnMouseDown := OnMouseDown;
      Self.OnMouseEnter := OnMouseEnter;
      Self.OnMouseLeave := OnMouseLeave;
      Self.OnMouseMove := OnMouseMove;
      Self.OnMouseUp := OnMouseUp;
      Self.SetBounds(Self.Left, Self.Top, Width, Height);
    end
  else
    inherited Assign(Source);
end;

procedure TabcCustomShapeButton.SetScaled(Value: Boolean);
begin
  FScaled := Value;
  if Value then
    ControlStyle := ControlStyle - [csFixedWidth, csFixedHeight]
  else
    ControlStyle := ControlStyle + [csFixedWidth, csFixedHeight];
end;

procedure TabcCustomShapeButton.ChangeScale(M, D: Integer);
var
  saveFontHeight: Integer;
begin
  saveFontHeight := Font.Height;
  inherited ChangeScale(M,D);
  if not FScaled then
    Font.Height := saveFontHeight;   //restore unscaled font size
end;

procedure TabcCustomShapeButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
    if (AComponent = FImageList) then
      SetImageList(nil)
    else if (AComponent = FSoundList) then
      FSoundList := nil;
end;

function TabcCustomShapeButton.PosInShape(X, Y: Integer): Boolean;
begin
  if FMouseMask.Empty then
    Result := False
  else
    Result := (FMouseMask.Canvas.Pixels[X,Y] = clBlack);
end;

procedure TabcCustomShapeButton.SetImageList(Value: TImageList);
begin
  if Value <> FImageList then
  begin
    if FImageList <> nil then FImageList.UnRegisterChanges(FChangeLink);
    FImageList := Value;
    if Value <> nil then
    begin
      Value.RegisterChanges(FChangeLink);
      Value.FreeNotification(Self);
    end;
    if not (csDestroying in ComponentState) then ImageListChanged(Self);
  end;
end;

procedure TabcCustomShapeButton.ImageListChanged(Sender: TObject);
begin
  if (FImageList <> nil) then
  begin
    FImageListCount := FImageList.Count;
    if (ComponentState * [csDesigning, csLoading]) = [csDesigning] then
      SetBounds(Left, Top, FImageList.Width, FImageList.Height);
  end;
  UpdateMouseMask;
  Invalidate;
end;

procedure TabcCustomShapeButton.UpdateMouseMask;
begin
  if (csDesigning in ComponentState) then Exit;
  if (FImageList <> nil) then
    if IsValidIndex(FIndexMouseMask) then
    begin
      FImageList.GetBitmap(FIndexMouseMask, FMouseMask);
      FMouseMask.Monochrome := True; {just in case wrong index was used, otherwise button could be completely disabled}
    end
    else
      {create mask from image}
      with FMouseMask do
        if IsValidIndex(FIndexUp) then
        begin
          FImageList.GetBitmap(FIndexUp, FMouseMask);
          Transparent := True;
          Mask(TransparentColor); {create a mask transparent on bottom left pixel, black in solid parts of image}
        end
        else
        begin
          Width := 0;
          Height := 0;
        end
  else
    with FMouseMask do
    begin
      Width := 0;
      Height := 0;
    end;
end;

function TabcCustomShapeButton.IsValidIndex(ImageIndex: TImageIndex): Boolean;
begin
  Result := (ImageIndex < FImageListCount) and (ImageIndex > -1);
end;

procedure TabcCustomShapeButton.SetIndexDisabled(Value: TImageIndex);
begin
  if (Value <> FIndexDisabled) then
  begin
    FIndexDisabled := Value;
    if not Enabled then Invalidate;
  end;
end;

procedure TabcCustomShapeButton.SetIndexDown(Value: TImageIndex);
begin
  if (Value <> FIndexDown) then
  begin
    FIndexDown := Value;
    if (FState = bsDown) then Invalidate;
  end;
end;

procedure TabcCustomShapeButton.SetIndexFlat(Value: TImageIndex);
begin
  if (Value <> FIndexFlat) then
  begin
    FIndexFlat := Value;
    if (FState = bsUp) then Invalidate;
  end;
end;

procedure TabcCustomShapeButton.SetIndexMouseMask(Value: TImageIndex);
begin
  if (Value <> FIndexMouseMask) then
  begin
    FIndexMouseMask := Value;
    UpdateMouseMask;
  end;
end;

procedure TabcCustomShapeButton.SetIndexUp(Value: TImageIndex);
begin
  if (Value <> FIndexUp) then
  begin
    FIndexUp := Value;
    if (FState = bsUp) then Invalidate;
  end;
end;

procedure TabcCustomShapeButton.Paint;
var
  DrawIndex: Integer;
  Rect: TRect;
  FontHeight: Integer;

  procedure DrawDesignOutline;
  begin
    with Canvas do begin
      Pen.Color := clBlack;
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  end;

begin
  {Calc drawing pos and text bounds}
  Rect := ClientRect;
  if (FState = bsUp) and FPopup and MouseOver then
    OffsetRect(Rect, FOffsetUpX, FOffsetUpY)
  else if FState = bsDown then
    OffsetRect(Rect, FOffsetDownX, FOffsetDownY);

  {Draw Glyph}
  if FImageList <> nil then
  begin
    case FState of
      bsUp:
        if FMouseOver or not FPopup then
          DrawIndex := FIndexUp
        else
          DrawIndex := FIndexFlat;
      bsDisabled:
          if IsValidIndex(FIndexDisabled) then
            DrawIndex := FIndexDisabled
          else
          if IsValidIndex(FIndexFlat) then
            DrawIndex := FIndexFlat
          else
            DrawIndex := FIndexUp;
      bsDown:
        DrawIndex := FIndexDown;
    else
      DrawIndex := -1;
    end;
    if DrawIndex = -1 then
      DrawIndex := 0 {FIndexUp};
    if IsValidIndex(DrawIndex) then
      with FImageList do
        if (FState = bsDisabled) and IsValidIndex(FIndexDisabled) then
          ImageList_DrawEx(Handle, DrawIndex, Canvas.Handle,
            Rect.Left, Rect.Top, 0, 0,
            CLR_NONE, clGray, ILD_FOCUS)
        else
          Draw(Canvas, Rect.Left, Rect.Top, DrawIndex)
    else if (csDesigning in ComponentState) then
      DrawDesignOutline;
  end
  else
    if (csDesigning in ComponentState) then
      DrawDesignOutline;

  {Draw Caption}
  if Caption <> '' then
    with Canvas do
    begin
      Brush.Style := bsClear;
      Font := Self.Font;
      FontHeight := TextHeight('W');
      with Rect do
      begin
        Top := ((Bottom + Top) - FontHeight) div 2;
        Bottom := Top + FontHeight;
      end;
      DrawText(Handle, PChar(Caption), -1, Rect, (DT_EXPANDTABS or
        DT_VCENTER or DT_CENTER));
    end;
end;

procedure TabcCustomShapeButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
begin
  if PosInShape(X,Y) then
  begin
    if (Button = mbLeft) and Enabled then
    begin
      if FState <> bsDown then begin
        FState := bsDown;
        Repaint;
      end;
      FDragging := True;
    end;
    if (SoundDown > -1) and (SoundList <> nil) then
      SoundList.PlayItem(SoundDown);
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TabcCustomShapeButton.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

function TabcCustomShapeButton.MouseIsOver: Boolean;
var
  P: TPoint;
begin
  GetCursorPos(P);
  P := ScreenToClient(P);
  Result := PosInShape(P.X, P.Y);
end;

procedure TabcCustomShapeButton.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TabcCustomShapeButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TButtonState;
begin
  if PosInShape(X,Y) then
  begin
    inherited MouseMove(Shift, X, Y);
    if FDragging then
      NewState := bsDown
    else
      NewState := bsUp;
  end
  else if FDragging then
    NewState := bsUp
  else
    NewState := FState;
  if NewState <> FState then
  begin
    FState := NewState;
    Repaint;
  end;
end;

procedure TabcCustomShapeButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if FDragging then begin
    FDragging := False;
    DoClick := PosInShape(X,Y);
    FState := bsUp;
    FMouseOver := False;
    Repaint;
    if DoClick then Click;
    if MouseIsOver then
      FMouseOver := True
    else
      Perform(CM_MOUSELEAVE,0,0);
    Repaint;
  end;
end;

procedure TabcCustomShapeButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled and
      (ssAlt in KeyDataToShiftState(Message.KeyData)) then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TabcCustomShapeButton.CMHitTest(var Message: TCMHitTest);
begin
  with Message do
    if PosInShape(xPos,yPos) then
      Result := HTCLIENT
    else
      Result := HTNOWHERE;
end;

procedure TabcCustomShapeButton.CMEnabledChanged(var Message: TMessage);
begin
  if Enabled then
    FState := bsUp
  else
    FState := bsDisabled;
  if not (csDestroying in ComponentState) and Popup and MouseIsOver then
    if Enabled then
      Perform(CM_MOUSEENTER, 0, 0)
    else
      Perform(CM_MOUSELEAVE, 0, 0);
  Invalidate;
end;

procedure TabcCustomShapeButton.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TabcCustomShapeButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not ForegroundTask then Exit;
  FMouseOver := True;
  if Popup and Enabled then
    Invalidate;
  if (SoundOver > -1) and (SoundList <> nil) then
    SoundList.PlayItem(SoundOver);
  MouseEnter;
end;

procedure TabcCustomShapeButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseOver := False;
  if Popup and Enabled then
    Invalidate;
  MouseLeave;
end;

procedure TabcCustomShapeButton.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TabcCustomShapeButton.SetPopup(Value: Boolean);
begin
  if Value <> FPopup then begin
    FPopup := Value;
    Invalidate;
  end;
end;

{ TabcShapeButton }

procedure TabcShapeButton.Click;
begin
  inherited;
end;

end.
