unit abcbusy;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls,
  abcmem, abcfbusy, abcwav;

type

  { TabcAVISource }

  TabcAVISource = class(TabcResourceData)
  private
    FCommonAVI: TCommonAVI;
    procedure SetCommonAVI(const Value: TCommonAVI);
  protected
    function GetEmpty: Boolean; override;
  public
    procedure Assign(Source: TPersistent); override;
    procedure Clear; override;
  published
    property CommonAVI: TCommonAVI read FCommonAVI write SetCommonAVI default aviNone;
  end;

  { TabcBusy }

  TabcAVIPosition = (avpTop, avpLeft);
  TabcPopupPosition = (appScreenCenter, appMainFormCenter, appActiveFormCenter, appCustom);

  TabcBusyOption = (
    abdShowAVI, abdShowCaption,
    abdShowMessage1, abdShowMessage2,
    abdShowProgress, abdShowCancel);

  TabcBusyOptions = set of TabcBusyOption;

  TabcBusy = class(TComponent)
  private
    FForm: TabcsfBusy;
    FAutoSize: Boolean;
    FAVIPosition: TabcAVIPosition;
    FAVISource: TabcAVISource;
    FAVITransparent: Boolean;
    FCancelled: Boolean;
    FCaption: string;
    FDlgColor: TColor;
    FDlgHeight: Integer;
    FDlgLeft: Integer;
    FDlgPosition: TabcPopupPosition;
    FDlgTop: Integer;
    FDlgWidth: Integer;
    FMessage1: string;
    FMessage2: string;
    FMessage1Font: TFont;
    FMessage2Font: TFont;
    FMinDuration: Word;
    FOptions: TabcBusyOptions;
    FProgressMax: Integer;
    FProgressStep: Integer;
    FProgressPos: Integer;
    FProgressMin: Integer;
    FShowModal: Boolean;
    FSound: TabcCustomWave;
    FOnCancel: TNotifyEvent;
    FOnFormActivate: TNotifyEvent;
    FOnFormClose: TCloseEvent;
    FOnFormCreate: TNotifyEvent;
    FOnFormShow: TNotifyEvent;
    FOnStartAVI: TNotifyEvent;
    FWordWrap: Boolean;
    procedure AVISourceChanged(Sender: TObject);
    procedure SetAVISource(const Value: TabcAVISource);
    procedure SetCaption(const Value: string);
    procedure SetDlgHeight(const Value: Integer);
    procedure SetDlgLeft(const Value: Integer);
    procedure SetDlgTop(const Value: Integer);
    procedure SetDlgWidth(const Value: Integer);
    procedure SetMessage1(const Value: string);
    procedure SetMessage2(const Value: string);
    procedure SetProgressMax(const Value: Integer);
    procedure SetProgressMin(const Value: Integer);
    procedure SetProgressPos(const Value: Integer);
    procedure SetProgressStep(const Value: Integer);
    procedure SetDlgPosition(const Value: TabcPopupPosition);
    procedure CenterOverForm(AForm: TForm);
    procedure SetAutoSize(const Value: Boolean);
    procedure SetDlgColor(const Value: TColor);
    procedure SetMessage1Font(const Value: TFont);
    procedure SetMessage2Font(const Value: TFont);
    function IsMessage1FontStored: Boolean;
    function IsMessage2FontStored: Boolean;
    procedure SetAVIPosition(const Value: TabcAVIPosition);
    procedure SetOnFormClose(const Value: TCloseEvent);
    procedure SetSound(const Value: TabcCustomWave);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure CancelOnClick(Sender: TObject); virtual;
    procedure CreateForm; virtual;
    procedure FormAfterActivate(Sender: TObject); virtual;
    procedure FormShow(Sender: TObject); virtual;
    function GetFormClosed: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Show; virtual;
    procedure ShowModal; virtual;
    procedure Hide;
    procedure StepBy(Delta: Integer);
    procedure StepIt;
    property Cancelled: Boolean read FCancelled;
    property Form: TabcsfBusy read FForm;
    property FormClosed: Boolean read GetFormClosed;
  published
    property AutoSize: Boolean read FAutoSize write SetAutoSize default True;
    property AVIPosition: TabcAVIPosition read FAVIPosition write SetAVIPosition default avpTop;
    property AVISource: TabcAVISource read FAVISource write SetAVISource;
    property AVITransparent: Boolean read FAVITransparent write FAVITransparent default True;
    property Caption: string read FCaption write SetCaption;
    property DlgColor: TColor read FDlgColor write SetDlgColor default clBtnFace;
    property DlgLeft: Integer read FDlgLeft write SetDlgLeft default 0;
    property DlgTop: Integer read FDlgTop write SetDlgTop default 0;
    property DlgWidth: Integer read FDlgWidth write SetDlgWidth default 0;
    property DlgHeight: Integer read FDlgHeight write SetDlgHeight default 0;
    property DlgPosition: TabcPopupPosition read FDlgPosition write SetDlgPosition default appScreenCenter;
    property Message1: string read FMessage1 write SetMessage1;
    property Message1Font: TFont read FMessage1Font write SetMessage1Font stored IsMessage1FontStored;
    property Message2: string read FMessage2 write SetMessage2;
    property Message2Font: TFont read FMessage2Font write SetMessage2Font stored IsMessage2FontStored;
    property MinDuration: Word read FMinDuration write FMinDuration default 2000;
    property Options: TabcBusyOptions read FOptions write FOptions
      default [abdShowAVI, abdShowCaption];
    property ProgressMin: Integer read FProgressMin write SetProgressMin default 0;
    property ProgressMax: Integer read FProgressMax write SetProgressMax default 100;
    property ProgressPos: Integer read FProgressPos write SetProgressPos default 0;
    property ProgressStep: Integer read FProgressStep write SetProgressStep default 10;
    property Sound: TabcCustomWave read FSound write SetSound;
    property WordWrap: Boolean read FWordWrap write FWordWrap default True;
    property OnCancel: TNotifyEvent read FOnCancel write FOnCancel;
    property OnFormActivate: TNotifyEvent read FOnFormActivate write FOnFormActivate;
    property OnFormCreate: TNotifyEvent read FOnFormCreate write FOnFormCreate;
    property OnFormClose: TCloseEvent read FOnFormClose write SetOnFormClose;
    property OnFormShow: TNotifyEvent read FOnFormShow write FOnFormShow;
    property OnStartAVI: TNotifyEvent read FOnStartAVI write FOnStartAVI;
  end;

implementation

uses extctrls, abcfact, abcutilw;

{ TabcAVISource }

procedure TabcAVISource.Assign(Source: TPersistent);
begin
  if (Source is TabcAVISource) then
    FCommonAVI := TabcAVISource(Source).CommonAVI;
  inherited Assign(Source);
end;

procedure TabcAVISource.Clear;
begin
  inherited Clear;
  FCommonAVI := aviNone;
end;

function TabcAVISource.GetEmpty: Boolean;
begin
  Result := inherited GetEmpty and
    (FCommonAVI = aviNone);
end;

procedure TabcAVISource.SetCommonAVI(const Value: TCommonAVI);
begin
  if Value <> FCommonAVI then
  begin
    Changing;
    Clear;
    FCommonAVI := Value;
    Changed;
  end;
end;

{ TabcBusy }

procedure TabcBusy.AVISourceChanged(Sender: TObject);
begin
end;

procedure TabcBusy.CancelOnClick(Sender: TObject);
begin
  FCancelled := True;
  if Assigned(FOnCancel) then
    FOnCancel(Self);
  Hide;
end;

procedure TabcBusy.CenterOverForm(AForm: TForm);
begin
  with FForm do
  begin
    Position := poDesigned;
    Left := AForm.Left + (AForm.Width - Width) div 2;
    Top := AForm.Top + (AForm.Height - Height) div 2;
  end;
end;

constructor TabcBusy.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoSize := True;
  FAVISource := TabcAVISource.Create;
  FAVISource.OnChange := AVISourceChanged;
  FAVITransparent := True;
  FDlgPosition := appScreenCenter;
  FDlgColor := clBtnFace;
  FMinDuration := 2000;
  FOptions := [abdShowAVI, abdShowCaption];
  FProgressMax := 100;
  FProgressStep := 10;
  FMessage1Font := TFont.Create;
  with FMessage1Font do
  begin
    Name := 'MS Sans Serif';
    Size := 8;
  end;
  FMessage2Font := TFont.Create;
  with FMessage2Font do
  begin
    Name := 'MS Sans Serif';
    Size := 8;
  end;
  FWordWrap := True;
end;

destructor TabcBusy.Destroy;
begin
  if FForm <> nil then Hide;
  FAVISource.Free;
  FMessage1Font.Free;
  FMessage2Font.Free;
  inherited Destroy;
end;

procedure TabcBusy.FormShow(Sender: TObject);
begin
  FForm.CloseAfter := GetTickCount + FMinDuration;
  if Assigned(FOnFormShow) then
    FOnFormShow(Self);
end;

procedure TabcBusy.FormAfterActivate(Sender: TObject);
begin
  Application.ProcessMessages;
  if Assigned(FOnFormActivate) then
    FOnFormActivate(Self);
end;

function TabcBusy.GetFormClosed: Boolean;
begin
  Result := (FForm = nil);
end;

procedure TabcBusy.Hide;
begin
  if FForm <> nil then
  begin
    if FForm.HandleAllocated then
      PostMessage(FForm.Handle, WM_CLOSE, 0, 0)
    else
      FForm.Release;
    FForm := nil;
  end;
end;

procedure TabcBusy.SetAutoSize(const Value: Boolean);
begin
  FAutoSize := Value;
(* removed for WordWrap sizing
  if Value then
  begin
    FDlgWidth := 0;
    FDlgHeight := 0;
  end;
*)
end;

procedure TabcBusy.SetAVISource(const Value: TabcAVISource);
begin
  FAVISource.Assign(Value);
end;

procedure TabcBusy.SetCaption(const Value: string);
begin
  FCaption := Value;
  if FForm <> nil then
    FForm.Caption := Value;
end;

procedure TabcBusy.SetDlgColor(const Value: TColor);
begin
  FDlgColor := Value;
  if FForm <> nil then
    FForm.Color := Value;
end;

procedure TabcBusy.SetDlgHeight(const Value: Integer);
begin
  FDlgHeight := Value;
  AutoSize := False;
end;

procedure TabcBusy.SetDlgLeft(const Value: Integer);
begin
  FDlgLeft := Value;
  DlgPosition := appCustom;
end;

procedure TabcBusy.SetDlgPosition(const Value: TabcPopupPosition);
begin
  FDlgPosition := Value;
  if Value <> appCustom then
  begin
    FDlgLeft := 0;
    FDlgTop := 0;
  end;
end;

procedure TabcBusy.SetDlgTop(const Value: Integer);
begin
  FDlgTop := Value;
  DlgPosition := appCustom;
end;

procedure TabcBusy.SetDlgWidth(const Value: Integer);
begin
  FDlgWidth := Value;
//removed for WordWrap sizing
//  AutoSize := False;
end;

procedure TabcBusy.SetMessage1(const Value: string);
begin
  if Value <> FMessage1 then
  begin
    FMessage1 := Value;
    if FForm <> nil then
      FForm.lblMessage1.Caption := Value;
    if (Value <> '') and not (csLoading in ComponentState) then
      Options := Options + [abdShowMessage1];
  end;
end;

procedure TabcBusy.SetMessage2(const Value: string);
begin
  if Value <> FMessage2 then
  begin
    FMessage2 := Value;
    if FForm <> nil then
      FForm.lblMessage2.Caption := Value;
    if (Value <> '') and not (csLoading in ComponentState) then
      Options := Options + [abdShowMessage2];
  end;
end;

procedure TabcBusy.SetProgressMax(const Value: Integer);
begin
  if Value <> FProgressMax then
  begin
    FProgressMax := Value;
    if FForm <> nil then
      FForm.ProgressBar.Max := Value;
    if (Value <> 0) and not (csLoading in ComponentState) then
      Options := Options + [abdShowProgress];
  end;
end;

procedure TabcBusy.SetProgressMin(const Value: Integer);
begin
  if Value <> FProgressMin then
  begin
    FProgressMin := Value;
    if FForm <> nil then
      FForm.ProgressBar.Min := Value;
    if (Value <> 0) and not (csLoading in ComponentState) then
      Options := Options + [abdShowProgress];
  end;
end;

procedure TabcBusy.SetProgressPos(const Value: Integer);
begin
  if Value <> FProgressPos then
  begin
    FProgressPos := Value;
    if FForm <> nil then
      FForm.ProgressBar.Position := Value;
    if (Value <> 0) and not (csLoading in ComponentState) then
      Options := Options + [abdShowProgress];
  end;
end;

procedure TabcBusy.SetProgressStep(const Value: Integer);
begin
  if Value <> FProgressStep then
  begin
    FProgressStep := Value;
    if FForm <> nil then
      FForm.ProgressBar.Step := Value;
    if (Value <> 0) and not (csLoading in ComponentState) then
      Options := Options + [abdShowProgress];
  end;
end;

procedure TabcBusy.Show;
begin
  CreateForm;
  FCancelled := False;
  FShowModal := False;
  FForm.Show;
  Application.ProcessMessages;
end;

procedure TabcBusy.ShowModal;
begin
  CreateForm;
  FCancelled := False;
  FShowModal := True;
  FForm.ShowModal;
end;

procedure TabcBusy.CreateForm;
begin
  if FForm = nil then
  begin
    FForm := TabcsfBusy(abcFormFactory.CreateFormOfClass(TabcsfBusy, Application));
    with FForm do
    begin
      if (abdShowCaption in Options) then
      begin
        BorderStyle := bsToolWindow;
        bvlBorder.Visible := False;
      end
      else
      begin
        BorderStyle := bsNone;
        bvlBorder.Visible := True;
      end;

      Caption := Self.Caption;
      Color := Self.DlgColor;

      with Animate do
        if (abdShowAVI in Options) then
        begin
          Active := False;
          Visible := True;
          //set avi data source
          if not AVISource.Empty then
          begin
            if AVISource.CommonAVI <> aviNone then
              CommonAVI := AVISource.CommonAVI
            else if AVISource.FileName <> '' then
              FileName := AVISource.FileName
            else if not AVISource.Data.Empty then
              if (AVISource.TempFile = '') or not FileExists(AVISource.TempFile) then
                FileName := AVISource.CopyDataToTempFile
              else
                FileName := AVISource.TempFile
            else //resource
            begin
              if AVISource.ResourceModule <> '' then
              begin
                AVISource.LoadRes;
                ResHandle := AVISource.ModuleHandle{ResourceHandle};
              end;
              if AVISource.ResourceName <> '' then
                ResName := AVISource.ResourceName
              else
                ResID := AVISource.ResourceID;
            end;
          end;
          Transparent := FAVITransparent;
          OnStart := OnStartAVI;
        end;

      with lblMessage1 do
        if (abdShowMessage1 in Options) then
        begin
          Visible := True;
          Caption := Message1;
          Font.Assign(FMessage1Font);
        end;

      with lblMessage2 do
        if (abdShowMessage2 in Options) then
        begin
          Visible := True;
          Caption := Message2;
          Font.Assign(FMessage2Font);
        end;

      with ProgressBar do
        if (abdShowProgress in Options) then
        begin
          Visible := True;
          Min := FProgressMin;
          Max := FProgressMax;
          Position := FProgressPos;
          Step := FProgressStep;
        end;

      with btnCancel do
        if (abdShowCancel in Options) then
        begin
          Visible := True;
          OnClick := CancelOnClick;
        end;

      SizeAndArrangeControls(Self.AutoSize, FWordWrap, DlgWidth, DlgHeight, Ord(AVIPosition));

      case DlgPosition of
        appScreenCenter:
          Position := poScreenCenter;
        appMainFormCenter:
          CenterOverForm(Application.MainForm);
        appActiveFormCenter:
          if Screen.ActiveForm <> nil then
            CenterOverForm(Screen.ActiveForm)
          else
            CenterOverForm(Application.MainForm);
        appCustom:
          begin
            Position := poDesigned;
            Left := DlgLeft;
            Top := DlgTop;
          end;
      end;

      Sound := Self.Sound;

      if Assigned(FOnFormCreate) then
        FOnFormCreate(Self);

      OnShow := FormShow;
      AfterActivate := FormAfterActivate;
      UserFormClose := OnFormClose;
    end;
  end;
end;

procedure TabcBusy.StepBy(Delta: Integer);
begin
  if FForm <> nil then
    FForm.ProgressBar.StepBy(Delta);
end;

procedure TabcBusy.StepIt;
begin
  if FForm <> nil then
    FForm.ProgressBar.StepIt;
end;

procedure TabcBusy.SetMessage1Font(const Value: TFont);
begin
  FMessage1Font.Assign(Value);
    if FForm <> nil then
      FForm.lblMessage1.Font.Assign(Value);
end;

procedure TabcBusy.SetMessage2Font(const Value: TFont);
begin
  FMessage2Font.Assign(Value);
  if FForm <> nil then
      FForm.lblMessage2.Font.Assign(Value);
end;

function TabcBusy.IsMessage1FontStored: Boolean;
begin
  with FMessage1Font do
    Result :=
      (Name <> 'MS Sans Serif') or
      (Style <> []) or
      (Size <> 8) or
      (Color <> clWindowText) or
      (Charset <> DEFAULT_CHARSET) or
      (Pitch <> fpDefault);
end;

function TabcBusy.IsMessage2FontStored: Boolean;
begin
  with FMessage2Font do
    Result :=
      (Name <> 'MS Sans Serif') or
      (Style <> []) or
      (Size <> 8) or
      (Color <> clWindowText) or
      (Charset <> DEFAULT_CHARSET) or
      (Pitch <> fpDefault);
end;

procedure TabcBusy.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = Sound) then
    Sound := nil;
end;

procedure TabcBusy.SetAVIPosition(const Value: TabcAVIPosition);
begin
  FAVIPosition := Value;
end;

procedure TabcBusy.SetOnFormClose(const Value: TCloseEvent);
begin
  FOnFormClose := Value;
  if FForm <> nil then
    FForm.UserFormClose := Value;
end;

procedure TabcBusy.SetSound(const Value: TabcCustomWave);
begin
  FSound := Value;
  if FForm <> nil then
    FForm.Sound := Value;
end;

end.
