unit abccombo;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TabcColorComboBox = class(TCustomComboBox)
  private
    FColorValues: TList;
    FShowText: Boolean;
    procedure DoColorChange(Sender: TObject);
    function GetSelectedColor: TColor;
    procedure SetSelectedColor(Value: TColor);
    procedure GetDefaultColorStr(const S: string);
    procedure SetShowText(Value: Boolean);
  protected
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetDefaultColors; virtual;
    procedure ItemsChanged;
    property SelectedColor: TColor read GetSelectedColor write SetSelectedColor;
  published
    property Anchors;
    property Constraints;
    property DragKind;
    {property Style;}
    property Color default clBtnFace;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property Items;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property ShowText: Boolean read FShowText write SetShowText default False;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    {property OnDrawItem;}
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

  { TabcFontComboBox }

  TabcFontOption = (afsScreen, afsPrinter, afsTrueType, afsDevice, afsRaster, afsVector);
  TabcFontOptions = set of TabcFontOption;

  TabcFontComboBox = class(TCustomComboBox)
  private
    FTrueTypeBmp: TBitmap;
    FPrinterBmp: TBitmap;
    FFontInfo: TStrings;
    FFontOptions: TabcFontOptions;
    function IsTrueTypeFont(Index: Integer): Boolean;
    function IsDeviceFont(Index: Integer): Boolean;
    procedure SetFontOptions(Value: TabcFontOptions);
  protected
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Items;
    procedure LoadFontFamilies;
  published
    property FontOptions: TabcFontOptions read FFontOptions write SetFontOptions
      default [afsScreen, afsPrinter, afsTrueType, afsDevice, afsRaster, afsVector];
    property Anchors;
    property Constraints;
    property DragKind;
    {property Style;}
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    {property OnDrawItem;}
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

implementation

uses Printers, abcutilf;

{$R abcrfn32.res}

{ TabcColorComboBox }

constructor TabcColorComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
  FColorValues := TList.Create;
  Color := clBtnFace;
end;

procedure TabcColorComboBox.SetDefaultColors;
begin
  Items.Clear;
  GetColorValues(GetDefaultColorStr);
end;

procedure TabcColorComboBox.GetDefaultColorStr(const S: string);
begin
  Items.Add(S);
end;

destructor TabcColorComboBox.Destroy;
begin
  FColorValues.Free;
  inherited Destroy;
end;

procedure TabcColorComboBox.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
    DoColorChange(Self);
end;

procedure TabcColorComboBox.DoColorChange(Sender: TObject);
var
  i: integer;
  s: string;
  val: TColor;
  check: Longint;
begin
  {recalculate color values for strings}
  {deletes any strings from items that are not colors}
  FColorValues.Clear;
  with Items do
    for i := Count - 1 downto 0 do
    begin
      s := abcLRTrim(Strings[i]);
      if IdentToColor(S, Longint(val)) then
        FColorValues.Insert(0, Pointer(val))
      else
      begin
        check := StrToIntDef(s, - COLOR_ENDCOLORS);
        if check = - COLOR_ENDCOLORS then
          Delete(i)
        else
          FColorValues.Insert(0, Pointer(TColor(check)));
      end;
    end;
end;

procedure TabcColorComboBox.ItemsChanged;
begin
  DoColorChange(Self);
end;

procedure TabcColorComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  SaveBrushColor, SavePenColor: TColor;
  ColorRect: TRect;
begin
  if (csDesigning in ComponentState) then
    inherited DrawItem(Index, Rect, State)
  else
    with Canvas do
    begin
      ColorRect := Rect;
      SaveBrushColor := Brush.Color;
      SavePenColor := Pen.Color;
      if FShowText then
      begin
        with ColorRect do Right := Left + ItemHeight * 2;
        with Rect do Left := Left + ItemHeight * 2;
      end;
      Brush.Color := Color;
      FillRect(ColorRect);
      InflateRect(ColorRect, -2,-2);
      Frame3D(Canvas, ColorRect, clBtnHighlight, clBtnShadow, 1);
      Brush.Color := TColor(FColorValues[Index]);
      FillRect(ColorRect);
      Brush.Color := Color;
      if FShowText then
      begin
        Brush.Color := SaveBrushColor;
        Pen.Color := SavePenColor;
        inherited DrawItem(Index, Rect, State);
      end;
    end;
end;

function TabcColorComboBox.GetSelectedColor: TColor;
begin
  if ItemIndex = -1 then
    Result := clBlack
  else
    Result := TColor(FColorValues.Items[ItemIndex]);
end;

procedure TabcColorComboBox.SetSelectedColor(Value: TColor);
begin
  ItemIndex := FColorValues.IndexOf(Pointer(Value));
end;

procedure TabcColorComboBox.SetShowText(Value: Boolean);
begin
  if Value <> FShowText then
  begin
    FShowText := Value;
    Repaint;
  end;
end;

{ TabcFontComboBox }

function EnumFontFamilyCallBack(lpelf: PEnumLogFont; lpntm: PNewTextMetric;
  FontType: Integer; Combo: TabcFontComboBox): Integer; stdcall;
var
  fontName: string;
  idx: Integer;
begin
  fontName := StrPas(lpelf^.elfLogFont.lfFaceName);
  with Combo.FFontInfo do
  begin
    idx := IndexOf(fontName);
    if idx = -1 then
      AddObject(fontName, Pointer(FontType))
    else
      Objects[idx] := Pointer(Integer(Objects[idx]) or FontType);
  end;
  Result := 1;
end;

constructor TabcFontComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
  FTrueTypeBmp := TBitmap.Create;
  FTrueTypeBmp.Handle := LoadBitmap(HInstance, 'ABC_FONT_TRUETYPE');
  FTrueTypeBmp.TransparentMode := tmAuto;
  FTrueTypeBmp.Transparent := True;
  FPrinterBmp := TBitmap.Create;
  FPrinterBmp.Handle := LoadBitmap(HInstance, 'ABC_FONT_PRINTER');
  FPrinterBmp.TransparentMode := tmAuto;
  FPrinterBmp.Transparent := True;
  FFontInfo := TStringList.Create;
  FFontOptions := [afsScreen, afsPrinter, afsTrueType, afsDevice, afsRaster, afsVector];
end;

destructor TabcFontComboBox.Destroy;
begin
  FFontInfo.Free;
  FTrueTypeBmp.Free;
  FPrinterBmp.Free;
  inherited Destroy;
end;

procedure TabcFontComboBox.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then LoadFontFamilies;
end;

procedure TabcFontComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
begin
  if (csDesigning in ComponentState) then
    inherited DrawItem(Index, Rect, State)
  else
    with Canvas do
    begin
      FillRect(Rect);
      if DroppedDown then
      begin
        if IsTrueTypeFont(Index) then
          Draw(Rect.Left+2, Rect.Top, FTrueTypeBmp)
        else if IsDeviceFont(Index) then
          Draw(Rect.Left+2, Rect.Top, FPrinterBmp);
        TextOut(Rect.Left + 4 + 13, Rect.Top, Items[Index]);
      end
      else
        TextOut(Rect.Left+2, Rect.Top, Items[Index]);
    end;
end;

function TabcFontComboBox.IsTrueTypeFont(Index: Integer): Boolean;
begin
  Result := Integer(FFontInfo.Objects[Index]) and TRUETYPE_FONTTYPE <> 0;
end;

function TabcFontComboBox.IsDeviceFont(Index: Integer): Boolean;
begin
  Result := Integer(FFontInfo.Objects[Index]) and DEVICE_FONTTYPE <> 0;
end;

procedure TabcFontComboBox.LoadFontFamilies;
var
  i, Flags: integer;
  DC: HDC;
begin
  FFontInfo.Clear;
  if (afsPrinter in FFontOptions) and not abcNoDefaultPrinter then
    EnumFontFamilies(Printer.Handle, nil, @EnumFontFamilyCallBack, Integer(Self));
  if (afsScreen in FFontOptions) then
  begin
    DC := GetDC(0);
    try
      EnumFontFamilies(DC, nil, @EnumFontFamilyCallBack, Integer(Self));
    finally
      ReleaseDC(0, DC);
    end;
  end;
  {remove unselected options -
   note all printer fonts flagged as true-type appear to
   also have the device flag set.
   So this filter cannot retrieve only the printer true-type fonts.}

  with FFontInfo do
    for i := Count - 1 downto 0 do
    begin
      Flags := Integer(Objects[i]);
      if ((Flags and TRUETYPE_FONTTYPE <> 0) and not
          (afsTrueType in FFontOptions)) or
         ((Flags and DEVICE_FONTTYPE <> 0) and not
          (afsDevice in FFontOptions)) or
         ((Flags and RASTER_FONTTYPE <> 0) and not
          (afsRaster in FFontOptions)) or
         ((Flags = 0) and not
          (afsVector in FFontOptions)) then
        Delete(i);
    end;
  TStringList(FFontInfo).Sort;
  Items.Assign(FFontInfo);
end;

procedure TabcFontComboBox.SetFontOptions(Value: TabcFontOptions);
begin
  if [afsScreen, afsPrinter] * Value = [] then
    Value := Value + [afsScreen];
  if [afsTrueType, afsDevice, afsRaster, afsVector] * Value = [] then
    Value := Value + [afsTrueType];
  if Value <> FFontOptions then
  begin
    FFontOptions := Value;
    if not (csDesigning in ComponentState) then LoadFontFamilies;
  end;
end;

end.
