unit abccompf;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  Dialogs, IniFiles, abcutilf;

type

  { TabcFormHook }

  TabcFormHook = class(TComponent)
  private
    FObjectInstance: TFarProc;
    FOwnerForm: TForm;
    FOwnerHandle: HWND;
    FOwnerWindowProc: TFarProc;
    FMDIClientObjectInstance: TFarProc;
    FOwnerMDIClientWindowProc: TFarProc;
    FOwnerClientHandle: HWND;
    FRecreateWnd: Boolean;
    function AppWndProc(var Message: TMessage): Boolean;
    procedure NewWndProc(var Message: TMessage);
    procedure NewClientWndProc(var Message: TMessage);
    procedure SubClassOwnerForm;
    procedure RestoreWndProcs;
  protected
    FSubClassMDIClientWindow: Boolean; //normally false unless needed by descendant
    procedure Loaded; override;
    procedure DoOnSubClass; virtual;
    procedure DoOnRestoreWndProc; virtual;
    function DoProcessAppMessage(var Message: TMessage): Boolean; virtual;
    function DoProcessMessage(var Message: TMessage): Boolean; virtual;
    function DoProcessMDIClientMessage(var Message: TMessage): Boolean; virtual;
    procedure AfterProcessMessage(var Message: TMessage); virtual;
    property OwnerForm: TForm read FOwnerForm;
    property OwnerHandle: HWND read FOwnerHandle;
    property OwnerClientHandle: HWND read FOwnerClientHandle;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TabcFormMessageHook }

  TabcMessageEvent = procedure(Sender: TObject; var Message: TMessage; var MsgHandled: Boolean) of object;
  {set MsgHandled to true to stop further processing, defaults to False}

  TabcFormMessageHook = class(TabcFormHook)
  private
    FOnMessage: TabcMessageEvent;
    FAfterMessage: TabcMessageEvent;
  protected
    function DoProcessMessage(var Message: TMessage): Boolean; override;
    procedure AfterProcessMessage(var Message: TMessage); override;
  published
    property OnMessage: TabcMessageEvent read FOnMessage write FOnMessage;
    property AfterMessage: TabcMessageEvent read FAfterMessage write FAfterMessage;
  end;

  { TabcFormSizer }

  TabcInitialPosSize = (ipsDefault, ipsMin, ipsMax);

  TabcFormSizer = class(TabcFormHook)
  private
    FEnabled: Boolean;
    FHeightMax: Integer;
    FHeightMin: Integer;
    FInitialFormPos: TabcInitialPosSize;
    FInitialFormSize: TabcInitialPosSize;
    FLeftMax: Integer;
    FLeftMin: Integer;
    FTopMax:Integer;
    FTopMin:Integer;
    FWidthMax: Integer;
    FWidthMin: Integer;
    FChangingBoundsLimits: Boolean;
    FDimensionChanged: Boolean;
    procedure ResizeForm;
    procedure SetFormBounds;
    procedure SetHeightMax(Value: Integer);
    procedure SetHeightMin(Value: Integer);
    procedure SetLeftMax(Value: Integer);
    procedure SetLeftMin(Value: Integer);
    procedure SetTopMax(Value: Integer);
    procedure SetTopMin(Value: Integer);
    procedure SetWidthMax(Value: Integer);
    procedure SetWidthMin(Value: Integer);
    procedure SetEnabled(const Value: Boolean);
  protected
    function DoProcessMessage(var Message: TMessage): Boolean; override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetMaxBounds(ALeft, ATop, AWidth, AHeight: Integer); virtual;
    procedure SetMinBounds(ALeft, ATop, AWidth, AHeight: Integer); virtual;
    property OwnerForm;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property HeightMax: Integer read FHeightMax write SetHeightMax default -1;
    property HeightMin: Integer read FHeightMin write SetHeightMin default -1;
    property InitialFormPos: TabcInitialPosSize read FInitialFormPos write FInitialFormPos default ipsDefault;
    property InitialFormSize: TabcInitialPosSize read FInitialFormSize write FInitialFormSize default ipsDefault;
    property LeftMax: Integer read FLeftMax write SetLeftMax default -1;
    property LeftMin: Integer read FLeftMin write SetLeftMin default -1;
    property TopMax: Integer read FTopMax write SetTopMax default -1;
    property TopMin: Integer read FTopMin write SetTopMin default -1;
    property WidthMax: Integer read FWidthMax write SetWidthMax default -1;
    property WidthMin: Integer read FWidthMin write SetWidthMin default -1;
  end;

  { TabcFormSaver }

  TabcFormSaverOption = (fsoCaption, fsoColor, fsoCtl3D, fsoCursor, fsoFont,
    fsoLeft, fsoTop, fsoWidth, fsoHeight, fsoHelpContext, fsoHint, fsoShowHint,
    fsoTag, fsoVisible, fsoWindowState);

  TabcFormSaverOptions = set of TabcFormSaverOption;

  TabcFormSaver = class(TComponent)
  private
    FAutoRestore: Boolean;
    FAutoSave: Boolean;
    FIniFile: TIniFile;
    FIniFileName: TFileName;
    FOnRestore: TNotifyEvent;
    FOnSave: TNotifyEvent;
    FOptions: TabcFormSaverOptions;
    FOwnerForm: TForm;
    function GetIniSection: string;
    procedure InitIniFile;
    procedure SetIniFileName(Value: TFileName);
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Restore; virtual;
    procedure Save; virtual;
    property IniFile: TIniFile read FIniFile;
    property IniSection: string read GetIniSection;
    property OwnerForm: TForm read FOwnerForm;
  published
    property AutoRestore: Boolean read FAutoRestore write FAutoRestore default True;
    property AutoSave: Boolean read FAutoSave write FAutoSave default True;
    property IniFileName: TFileName read FIniFileName write SetIniFileName;
    property Options: TabcFormSaverOptions read FOptions write FOptions
      default [fsoColor, fsoCtl3D, fsoFont, fsoLeft, fsoTop, fsoWidth,
               fsoHeight, fsoShowHint, fsoWindowState];
    property OnRestore: TNotifyEvent read FOnRestore write FOnRestore;
    property OnSave: TNotifyEvent read FOnSave write FOnSave;
  end;

implementation

uses abcutilm, abcmsg, abcconst;

{TabcFormHook}

procedure TabcFormHook.NewWndProc(var Message: TMessage);
begin
  if not DoProcessMessage(Message) then
    {pass message to original WndProc}
    with Message do
      Result := CallWindowProc(FOwnerWindowProc, FOwnerHandle, Msg, WParam, LParam);

  if not (csDestroying in ComponentState) then
  begin
    AfterProcessMessage(Message);

    if (Message.Msg = WM_DESTROY) then
    begin
      FRecreateWnd := True;
      RestoreWndProcs;
      PostMessage(Application.Handle, AM_RECREATEHOOK, 0, Integer(Self));
    end;

  end;
end;

procedure TabcFormHook.NewClientWndProc(var Message: TMessage);
begin
  if not DoProcessMDIClientMessage(Message) then
    with Message do
      Result := CallWindowProc(FOwnerMDIClientWindowProc, FOwnerClientHandle, Msg, WParam, LParam);
end;

function TabcFormHook.AppWndProc(var Message: TMessage): Boolean;
begin
  Result := DoProcessAppMessage(Message);
  if (Message.Msg = AM_RECREATEHOOK) and (Message.LParam = Integer(Self)) then
    SubClassOwnerForm;
end;

constructor TabcFormHook.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TForm) then
    FOwnerForm := TForm(AOwner)
  else if (AOwner.Owner is TForm) then
    FOwnerForm := TForm(AOwner.Owner)
  else
    raise EComponentError.Create(sabcErrorOwnerFormExpected);
end;

destructor TabcFormHook.Destroy;
begin
  Destroying;
  RestoreWndProcs;
  inherited Destroy;
end;

procedure TabcFormHook.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) and not FRecreateWnd and (OwnerHandle = 0) then
    SubClassOwnerForm;
end;

procedure TabcFormHook.SubClassOwnerForm;
begin
  if (csDestroying in ComponentState) or (FObjectInstance <> nil) then
    Exit;
  FOwnerHandle := FOwnerForm.Handle;
  FObjectInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(NewWndProc);
  FOwnerWindowProc := Pointer(GetWindowLong(FOwnerHandle, GWL_WNDPROC));
  SetWindowLong(FOwnerHandle, GWL_WNDPROC, Longint(FObjectInstance));
  if not FRecreateWnd then
    Application.HookMainWindow(AppWndProc);
  FRecreateWnd := False;
  if FSubClassMDIClientWindow and (OwnerForm.FormStyle = fsMDIForm) then
  begin
    FOwnerClientHandle := OwnerForm.ClientHandle;
    FMDIClientObjectInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(NewClientWndProc);
    FOwnerMDIClientWindowProc := Pointer(GetWindowLong(FOwnerClientHandle, GWL_WNDPROC));
    SetWindowLong(FOwnerClientHandle, GWL_WNDPROC, Longint(FMDIClientObjectInstance));
  end;
  DoOnSubClass;
end;

procedure TabcFormHook.RestoreWndProcs;
begin
  DoOnRestoreWndProc;
  if not FRecreateWnd then
    Application.UnhookMainWindow(AppWndProc);
  if not (csDestroying in FOwnerForm.ComponentState) then
  begin
    if (FOwnerForm.Handle = FOwnerHandle) then
      SetWindowLong(FOwnerHandle, GWL_WNDPROC, Longint(FOwnerWindowProc));
    if FMDIClientObjectInstance <> nil then
      SetWindowLong(FOwnerClientHandle, GWL_WNDPROC, Longint(FOwnerMDIClientWindowProc));
  end;
  {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FObjectInstance);
  FObjectInstance := nil;
  if FMDIClientObjectInstance <> nil then
  begin
    {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FMDIClientObjectInstance);
    FMDIClientObjectInstance := nil;
  end;
end;

function TabcFormHook.DoProcessMessage(var Message: TMessage): Boolean;
begin
  Result := False;
end;

procedure TabcFormHook.AfterProcessMessage(var Message: TMessage);
begin
end;

function TabcFormHook.DoProcessMDIClientMessage(var Message: TMessage): Boolean;
begin
  Result := False;
end;

function TabcFormHook.DoProcessAppMessage(var Message: TMessage): Boolean;
begin
  Result := False;
end;

procedure TabcFormHook.DoOnSubClass;
begin
end;

procedure TabcFormHook.DoOnRestoreWndProc;
begin
end;

{ TabcFormMessageHook }

function TabcFormMessageHook.DoProcessMessage(var Message: TMessage): Boolean;
begin
  Result := False;
  if Assigned(FOnMessage) then
    FOnMessage(Self, Message, Result);
end;

procedure TabcFormMessageHook.AfterProcessMessage(var Message: TMessage);
var
  Result: Boolean;
begin
  if Assigned(FAfterMessage) then
    FAfterMessage(Self, Message, Result);
end;

{ TabcFormSizer }

constructor TabcFormSizer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := True;
  FHeightMax := -1;
  FHeightMin := -1;
  FLeftMax := -1;
  FLeftMin := -1;
  FTopMax := -1;
  FTopMin := -1;
  FWidthMax := -1;
  FWidthMin := -1;
end;

procedure TabcFormSizer.Loaded;
var
  NewBounds: TRect;
begin
  inherited Loaded;
  if not FEnabled or
     (csDesigning in ComponentState) or
     ((FInitialFormPos = ipsDefault) and
      (FInitialFormSize = ipsDefault)) then Exit;
  NewBounds := OwnerForm.ClientRect;
  with NewBounds do begin
    case FInitialFormPos of
      ipsDefault: ;
      ipsMin:
        begin
          if FLeftMin <> -1 then Left := FLeftMin;
          if FTopMin <> -1 then Top := FTopMin;
        end;
      ipsMax:
        begin
          if FLeftMax <> -1 then Left := FLeftMax;
          if FTopMax <> -1 then Top := FTopMax;
        end;
    end;
    case FInitialFormSize of
      ipsDefault: ;
      ipsMin:
        begin
          if FWidthMin <> -1 then Right  := FWidthMin;
          if FHeightMin <> -1 then Bottom := FHeightMin;
        end;
      ipsMax:
        begin
          if FWidthMax <> -1 then Right  := FWidthMax;
          if FHeightMax <> -1 then Bottom := FHeightMax;
        end;
    end;
    OwnerForm.SetBounds(Left, Top, Right, Bottom);
  end;
end;

function TabcFormSizer.DoProcessMessage(var Message: TMessage): Boolean;
var
  maxWidth, maxHeight: integer;
begin
  Result := False;
  if (Message.Msg = WM_GETMINMAXINFO) and FEnabled then begin
      with TWMGetMinMaxInfo(Message).MinMaxInfo^ do begin
        if FLeftMax <> -1 then
          ptMaxPosition.x := FLeftMax;
        if FTopMax <> -1 then
          ptMaxPosition.y := FTopMax;
        if FWidthMin <> -1 then
          ptMinTrackSize.x := FWidthMin;
        if FHeightMin <> -1 then
          ptMinTrackSize.y := FHeightMin;
        if (FWidthMax <> -1) then begin
          ptMaxSize.x := FWidthMax;
          ptMaxTrackSize.x := FWidthMax;
        end;
        if (FHeightMax <> -1) then begin
          ptMaxSize.y := FHeightMax;
          ptMaxTrackSize.y := FHeightMax;
        end;
        Result := (OwnerForm.FormStyle <> fsMDIChild);
      end;
  end
  else if (Message.Msg = WM_WINDOWPOSCHANGING) and FEnabled then begin
      with TWMWindowPosChanging(Message).WindowPos^ do begin
        if ((Flags and SWP_NOMOVE) = 0) then begin
          x := abcMinMaxNotDefault(x, FLeftMin, FLeftMax, -1);
          y := abcMinMaxNotDefault(y, FTopMin, FTopMax, -1);
          Result := True;
        end;
        if ((Flags and SWP_NOSIZE) = 0) then begin
          if (OwnerForm.FormStyle = fsMDIChild) then begin
            maxWidth := abcMax(cx, FWidthMax);
            maxHeight := abcMax(cy, FHeightMax);
          end
          else begin
            maxWidth := FWidthMax;
            maxHeight := FHeightMax;
          end;
          cx := abcMinMaxNotDefault(cx, FWidthMin, maxWidth, -1);
          cy := abcMinMaxNotDefault(cy, FHeightMin, maxHeight, -1);
          Result := True;
        end;
      end;
  end;
end;

procedure TabcFormSizer.SetMaxBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  FChangingBoundsLimits := True;
  LeftMax := ALeft;
  TopMax := ATop;
  WidthMax := AWidth;
  HeightMax := AHeight;
  FChangingBoundsLimits := False;
  if FDimensionChanged then SetFormBounds;
end;

procedure TabcFormSizer.ResizeForm;
begin
  FDimensionChanged := True;
  if not FChangingBoundsLimits then SetFormBounds;
end;

procedure TabcFormSizer.SetFormBounds;
begin
  with OwnerForm do
    if not (csDesigning in ComponentState) and HandleAllocated and FEnabled then
      SetWindowPos(Handle, Handle, Left, Top, Width, Height, SWP_NOACTIVATE or SWP_NOZORDER);
  FDimensionChanged := False;
  FChangingBoundsLimits := False;
end;

procedure TabcFormSizer.SetLeftMax(Value: Integer);
begin
  if Value <> FLeftMax then
  begin
    FLeftMax := Value;
    if (Value <> -1) then
    begin
      if FLeftMin > FLeftMax then FLeftMin := FLeftMax;
      if FLeftMax < OwnerForm.Left then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetTopMax(Value: Integer);
begin
  if Value <> FTopMax then
  begin
    FTopMax := Value;
    if (Value <> -1) then
    begin
      if FTopMin > FTopMax then FTopMin := FTopMax;
      if FTopMax < OwnerForm.Top then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetWidthMax(Value: Integer);
begin
  if Value <> FWidthMax then
  begin
    FWidthMax := Value;
    if (Value <> -1) then
    begin
      if FWidthMin > FWidthMax then FWidthMin := FWidthMax;
      if FWidthMax < OwnerForm.Width then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetHeightMax(Value: Integer);
begin
  if Value <> FHeightMax then
  begin
    FHeightMax := Value;
    if (Value <> -1) then
    begin
      if FHeightMin > FHeightMax then FHeightMin := FHeightMax;
      if FHeightMax < OwnerForm.Height then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetMinBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  FChangingBoundsLimits := True;
  LeftMin := ALeft;
  TopMin := ATop;
  WidthMin := AWidth;
  HeightMin := AHeight;
  FChangingBoundsLimits := False;
  if FDimensionChanged then SetFormBounds;
end;

procedure TabcFormSizer.SetLeftMin(Value: Integer);
begin
  if Value <> FLeftMin then
  begin
    FLeftMin := Value;
    if (Value <> -1) then
    begin
      if FLeftMax < FLeftMin then FLeftMax := FLeftMin;
      if FLeftMin > OwnerForm.Left then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetTopMin(Value: Integer);
begin
  if Value <> FTopMin then
  begin
    FTopMin := Value;
    if (Value <> -1) then
    begin
      if FTopMax < FTopMin then FTopMax := FTopMin;
      if FTopMin > OwnerForm.Top then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetWidthMin(Value: Integer);
begin
  if Value <> FWidthMin then
  begin
    FWidthMin := Value;
    if (Value <> -1) then
    begin
      if FWidthMax < FWidthMin then FWidthMax := FWidthMin;
      if FWidthMin > OwnerForm.Width then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetHeightMin(Value: Integer);
begin
  if Value <> FHeightMin then
  begin
    FHeightMin := Value;
    if (Value <> -1) then
    begin
      if FHeightMax < FHeightMin then FHeightMax := FHeightMin;
      if FHeightMin > OwnerForm.Height then ResizeForm;
    end;
  end;
end;

procedure TabcFormSizer.SetEnabled(const Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    if Value then ResizeForm;
  end;
end;

{ TabcFormSaver }

constructor TabcFormSaver.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TForm) then
    FOwnerForm := TForm(AOwner)
  else if (AOwner.Owner is TForm) then
    FOwnerForm := TForm(AOwner.Owner)
  else
    raise EComponentError.Create(sabcErrorOwnerFormExpected);
  FOptions := [fsoColor, fsoCtl3D, fsoFont, fsoLeft, fsoTop, fsoWidth,
               fsoHeight, fsoShowHint, fsoWindowState];
  FAutoRestore := True;
  FAutoSave := True;
end;

destructor TabcFormSaver.Destroy;
begin
  Destroying;
  if not (csDesigning in ComponentState) then begin
    if FAutoSave then Save;
    FIniFile.Free;
  end;
  inherited Destroy;
end;

procedure TabcFormSaver.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then begin
    if (FIniFile = nil) then InitIniFile;
    if FAutoRestore then Restore;
  end;
end;

function TabcFormSaver.GetIniSection: string;
begin
  Result := OwnerForm.ClassName + '.' + Name + ' Form Properties';
end;

procedure TabcFormSaver.InitIniFile;
begin
  if FIniFile <> nil then FIniFile.Free;
  if FIniFileName = '' then
    FIniFile := TIniFile.Create(abcReplaceFileExt(Application.ExeName, '.INI'))
  else
    FIniFile := TIniFile.Create(FIniFileName);
end;

procedure TabcFormSaver.Restore;
var
  RestoredBounds: TRect;
  RestoredCaption: string;
  RestoredWindowState: TWindowState;
  SectionName: string;
begin
  {restore properties for Options}
  SectionName := IniSection;
  try
    with FIniFile, OwnerForm do begin
      if fsoCaption in Options then begin
        RestoredCaption := ReadString(SectionName, 'Caption', '');
        if RestoredCaption <> '' then Caption := RestoredCaption;
      end;
      if fsoColor in Options then
        Color := ReadInteger(SectionName, 'Color', Color);
      if fsoCtl3D in Options then
        Ctl3D := ReadBool(SectionName, 'Ctl3D', Ctl3D);
      if fsoCursor in Options then
        Cursor := ReadInteger(SectionName, 'Cursor', Cursor);
      if fsoFont in Options then begin
        Font.Name := ReadString(SectionName, 'Font.Name', Font.Name);
        Font.Size := ReadInteger(SectionName, 'Font.Size', Font.Size);
        Font.Color := ReadInteger(SectionName, 'Font.Color', Font.Color);
        if ReadBool(SectionName, 'Font.Style.Bold', (fsBold in Font.Style)) then
          Font.Style := Font.Style + [fsBold];
        if ReadBool(SectionName, 'Font.Style.Italic', (fsItalic in Font.Style)) then
          Font.Style := Font.Style + [fsItalic];
        if ReadBool(SectionName, 'Font.Style.Underline', (fsUnderline in Font.Style)) then
          Font.Style := Font.Style + [fsUnderline];
        if ReadBool(SectionName, 'Font.Style.Strikeout', (fsStrikeOut in Font.Style)) then
          Font.Style := Font.Style + [fsStrikeOut];
      end;
      RestoredBounds := ClientRect;
      if fsoLeft in Options then
        RestoredBounds.Left := ReadInteger(SectionName, 'Left', Left);
      if fsoTop in Options then
        RestoredBounds.Top := ReadInteger(SectionName, 'Top', Top);
      if fsoWidth in Options then
        RestoredBounds.Right := ReadInteger(SectionName, 'Width', Width);
      if fsoHeight in Options then
        RestoredBounds.Bottom := ReadInteger(SectionName, 'Height', Height);
      if fsoHelpContext in Options then
        HelpContext := ReadInteger(SectionName, 'HelpContext', HelpContext);
      if fsoHint in Options then
        Hint := ReadString(SectionName, 'Hint', Hint);
      if fsoShowHint in Options then
        ShowHint := ReadBool(SectionName, 'ShowHint', ShowHint);
      if fsoTag in Options then
        Tag := ReadInteger(SectionName, 'Tag', Tag);
      if fsoVisible in Options then
        Visible := ReadBool(SectionName, 'Visible', Visible);
      if fsoWindowState in Options then begin
        RestoredWindowState := TWindowState(ReadInteger(SectionName, 'WindowState', Ord(WindowState)));
        WindowState := RestoredWindowState;
      end
      else
        RestoredWindowState := wsNormal;
      if RestoredWindowState = wsNormal then
        with RestoredBounds do SetBounds(Left, Top, Right, Bottom);
    end;
  except
    on E:Exception do
      ShowMessage('Error restoring form property ' + E.Message);
  end;
  if Assigned(FOnRestore) then FOnRestore(Self);
end;

procedure TabcFormSaver.Save;
var
  SectionName: string;
  SaveOptions: TabcFormSaverOptions;
begin
  {adjust Options for autosave when destroying Owner and component.
   This ensures that form's Visible property is not always saved as False.}

  if (csDestroying in ComponentState) and (csDestroying in FOwnerForm.ComponentState) then begin
    SaveOptions := FOptions;
    FOptions := FOptions - [fsoVisible];
  end;

  {save properties for Options}
  SectionName := IniSection;
  with FIniFile, OwnerForm do begin
    if fsoCaption in Options then
      WriteString(SectionName, 'Caption', Caption);
    if fsoColor in Options then
      WriteInteger(SectionName, 'Color', Color);
    if fsoCtl3D in Options then
      WriteBool(SectionName, 'Ctl3D', Ctl3D);
    if fsoCursor in Options then
      WriteInteger(SectionName, 'Cursor', Cursor);
    if fsoFont in Options then begin
      WriteString(SectionName, 'Font.Name', Font.Name);
      WriteInteger(SectionName, 'Font.Size', Font.Size);
      WriteInteger(SectionName, 'Font.Color', Font.Color);
      WriteBool(SectionName, 'Font.Style.Bold', (fsBold in Font.Style));
      WriteBool(SectionName, 'Font.Style.Italic', (fsItalic in Font.Style));
      WriteBool(SectionName, 'Font.Style.Underline', (fsUnderline in Font.Style));
      WriteBool(SectionName, 'Font.Style.Strikeout', (fsStrikeOut in Font.Style));
    end;
    if fsoLeft in Options then
      WriteInteger(SectionName, 'Left', Left);
    if fsoTop in Options then
      WriteInteger(SectionName, 'Top', Top);
    if fsoWidth in Options then
      WriteInteger(SectionName, 'Width', Width);
    if fsoHeight in Options then
      WriteInteger(SectionName, 'Height', Height);
    if fsoHelpContext in Options then
      WriteInteger(SectionName, 'HelpContext', HelpContext);
    if fsoHint in Options then
      WriteString(SectionName, 'Hint', Hint);
    if fsoShowHint in Options then
      WriteBool(SectionName, 'ShowHint', ShowHint);
    if fsoTag in Options then
      WriteInteger(SectionName, 'Tag', Tag);
    if fsoVisible in Options then
      WriteBool(SectionName, 'Visible', Visible);
    if fsoWindowState in Options then
      WriteInteger(SectionName, 'WindowState', Ord(WindowState));
  end;
  if Assigned(FOnSave) then FOnSave(Self);

  if (csDestroying in ComponentState) and (csDestroying in FOwnerForm.ComponentState) then
    FOptions := SaveOptions;

end;

procedure TabcFormSaver.SetIniFileName(Value: TFileName);
begin
  if Value <> FIniFileName then begin
    FIniFileName := Value;
    if not (csDesigning in ComponentState) then InitIniFile;
  end;
end;

end.
