unit abcctl32;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, abcexctl;

type

  { TabcCheckBox }

  TabcCheckBox = class(TCheckBox)
  private
    FAlignHorz: TAlignment;
    FAlignVert: TabcVAlignment;
    FWordWrap: Boolean;
    procedure SetAlignHorz(const Value: TAlignment);
    procedure SetAlignVert(const Value: TabcVAlignment);
    procedure SetWordWrap(const Value: Boolean);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AlignHorz: TAlignment read FAlignHorz write SetAlignHorz default taLeftJustify;
    property AlignVert: TabcVAlignment read FAlignVert write SetAlignVert default vaTop;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default True;
  end;

  { TabcRadioButton }

  TabcRadioButton = class(TRadioButton)
  private
    FAlignHorz: TAlignment;
    FAlignVert: TabcVAlignment;
    FWordWrap: Boolean;
    procedure SetAlignHorz(const Value: TAlignment);
    procedure SetAlignVert(const Value: TabcVAlignment);
    procedure SetWordWrap(const Value: Boolean);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AlignHorz: TAlignment read FAlignHorz write SetAlignHorz default taLeftJustify;
    property AlignVert: TabcVAlignment read FAlignVert write SetAlignVert default vaTop;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default True;
  end;

  { TabcPushButton }

  TabcPushButton = class(TabcRadioButton)
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AlignHorz default taCenter;
    property AlignVert default vaCenter;
    property Height default 25;
    property Width default 100;
  end;

  { TabcButton }

  TabcButton = class(TButton)
  private
    FAlignHorz: TAlignment;
    FAlignVert: TabcVAlignment;
    FWordWrap: Boolean;
    procedure SetAlignHorz(const Value: TAlignment);
    procedure SetAlignVert(const Value: TabcVAlignment);
    procedure SetWordWrap(const Value: Boolean);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AlignHorz: TAlignment read FAlignHorz write SetAlignHorz default taCenter;
    property AlignVert: TabcVAlignment read FAlignVert write SetAlignVert default vaCenter;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default True;
  end;

implementation

{ TabcCheckBox }

constructor TabcCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWordWrap := True;
end;

procedure TabcCheckBox.CreateParams(var Params: TCreateParams);
const
  HorzAlignFlags: array[TAlignment] of DWORD = (BS_LEFT, BS_RIGHT, BS_CENTER);
  VertAlignFlags: array[TabcVAlignment] of DWORD = (BS_TOP, BS_BOTTOM, BS_VCENTER);
  WordWrapFlags: array[Boolean] of DWORD = (0, BS_MULTILINE);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style and not
                  (BS_LEFT or BS_RIGHT or BS_CENTER or
                   BS_TOP or BS_BOTTOM or BS_VCENTER or
                   BS_MULTILINE);
  Params.Style := Params.Style or
                  HorzAlignFlags[FAlignHorz] or
                  VertAlignFlags[FAlignVert] or
                  WordWrapFlags[FWordWrap];
end;

procedure TabcCheckBox.SetAlignHorz(const Value: TAlignment);
begin
  if Value <> FAlignHorz then
  begin
    FAlignHorz := Value;
    RecreateWnd;
  end;
end;

procedure TabcCheckBox.SetAlignVert(const Value: TabcVAlignment);
begin
  if Value <> FAlignVert then
  begin
    FAlignVert := Value;
    RecreateWnd;
  end;
end;

procedure TabcCheckBox.SetWordWrap(const Value: Boolean);
begin
  if Value <> FWordWrap then
  begin
    FWordWrap := Value;
    RecreateWnd;
  end;
end;

{ TabcRadioButton }

constructor TabcRadioButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWordWrap := True;
end;

procedure TabcRadioButton.CreateParams(var Params: TCreateParams);
const
  HorzAlignFlags: array[TAlignment] of DWORD = (BS_LEFT, BS_RIGHT, BS_CENTER);
  VertAlignFlags: array[TabcVAlignment] of DWORD = (BS_TOP, BS_BOTTOM, BS_VCENTER);
  WordWrapFlags: array[Boolean] of DWORD = (0, BS_MULTILINE);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style and not
                  (BS_LEFT or BS_RIGHT or BS_CENTER or
                   BS_TOP or BS_BOTTOM or BS_VCENTER or
                   BS_MULTILINE);
  Params.Style := Params.Style or
                  HorzAlignFlags[FAlignHorz] or
                  VertAlignFlags[FAlignVert] or
                  WordWrapFlags[FWordWrap];
end;

procedure TabcRadioButton.SetAlignHorz(const Value: TAlignment);
begin
  if Value <> FAlignHorz then
  begin
    FAlignHorz := Value;
    RecreateWnd;
  end;
end;

procedure TabcRadioButton.SetAlignVert(const Value: TabcVAlignment);
begin
  if Value <> FAlignVert then
  begin
    FAlignVert := Value;
    RecreateWnd;
  end;
end;

procedure TabcRadioButton.SetWordWrap(const Value: Boolean);
begin
  if Value <> FWordWrap then
  begin
    FWordWrap := Value;
    RecreateWnd;
  end;
end;

{ TabcPushButton }

constructor TabcPushButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAlignHorz := taCenter;
  FAlignVert := vaCenter;
  SetBounds(Left, Top, 100, 25);
end;

procedure TabcPushButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or BS_PUSHLIKE;
end;

{ TabcButton }

constructor TabcButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAlignHorz := taCenter;
  FAlignVert := vaCenter;
  FWordWrap := True;
end;

procedure TabcButton.CreateParams(var Params: TCreateParams);
const
  HorzAlignFlags: array[TAlignment] of DWORD = (BS_LEFT, BS_RIGHT, BS_CENTER);
  VertAlignFlags: array[TabcVAlignment] of DWORD = (BS_TOP, BS_BOTTOM, BS_VCENTER);
  WordWrapFlags: array[Boolean] of DWORD = (0, BS_MULTILINE);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style and not
                  (BS_LEFT or BS_RIGHT or BS_CENTER or
                   BS_TOP or BS_BOTTOM or BS_VCENTER or
                   BS_MULTILINE);
  Params.Style := Params.Style or
                  HorzAlignFlags[FAlignHorz] or
                  VertAlignFlags[FAlignVert] or
                  WordWrapFlags[FWordWrap];
end;

procedure TabcButton.SetAlignHorz(const Value: TAlignment);
begin
  if Value <> FAlignHorz then
  begin
    FAlignHorz := Value;
    RecreateWnd;
  end;
end;

procedure TabcButton.SetAlignVert(const Value: TabcVAlignment);
begin
  if Value <> FAlignVert then
  begin
    FAlignVert := Value;
    RecreateWnd;
  end;
end;

procedure TabcButton.SetWordWrap(const Value: Boolean);
begin
  if Value <> FWordWrap then
  begin
    FWordWrap := Value;
    RecreateWnd;
  end;
end;

end.
