unit abccurlb;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, Menus;

type

  { TabcCurrencyLabel }

  TabcCurrFractionFormat = (ffHundredths, ffUnitsDigits, ffUnitsWords, ffNoFraction);

  TabcCustomCurrencyLabel = class(TCustomLabel)
  private
    FCurrFractionFormat: TabcCurrFractionFormat;
    FCurrFractionText: string;
    FCurrText: string;
    FFirstUpperCase: Boolean;
    FHandleException: Boolean;
    FValue: Double;
    procedure SetValue(AValue: Double);
    procedure SetCurrFractionFormat(Value: TabcCurrFractionFormat);
    procedure SetCurrFractionText(Value: string);
    procedure SetCurrText(Value: string);
    procedure SetFirstUpperCase(Value: Boolean);
  protected
    procedure Loaded; override;
    procedure UpdateCaption; virtual;
    property CurrFractionFormat: TabcCurrFractionFormat read FCurrFractionFormat
      write SetCurrFractionFormat default ffHundredths;
    property CurrFractionText: string read FCurrFractionText write SetCurrFractionText;
    property CurrText: string read FCurrText write SetCurrText;
    property FirstUpperCase: Boolean read FFirstUpperCase write SetFirstUpperCase default True;
    property HandleException: Boolean read FHandleException write FHandleException default True;
    property Value: Double read FValue write SetValue;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TabcCurrencyLabel = class(TabcCustomCurrencyLabel)
  private
    function GetCaption: string;
  public
    property Caption: string read GetCaption;
  published
    property CurrFractionFormat;
    property CurrFractionText;
    property CurrText;
    property FirstUpperCase;
    property HandleException;
    property Value;
    property Align;
    property Alignment;

    property AutoSize;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

function abcNumToWords(Num: Double; CurrText, CurrFractionText: string;
  FractionFormat: TabcCurrFractionFormat; FirstUpperCase, HandleException: Boolean): string;

implementation

uses abcutilf;

resourcestring
  sabcCurrTextDigits = 'zero;one;two;three;four;five;six;seven;eight;nine';
  sabcCurrTextTeens = 'ten;eleven;twelve;thirteen;fourteen;fifteen;sixteen;seventeen;eighteen;nineteen';
  sabcCurrTextTens = 'ten;twenty;thirty;forty;fifty;sixty;seventy;eighty;ninety';
  sabcCurrTextThousands = 'thousand;million;billion;trillion';
  sabcCurrTextAnd = 'and';
  sabcCurrTextHundred = 'hundred';
  sabcCurrTextError = '#ERROR#';
  sabcCurrTextDollars = 'dollars';
  sabcCurrTextCents = 'cents';
  sabcCurrTextHundredths = '/100';
  sabcCurrTextFormatDecimal = '%.2d';

var
  CurrStrings: TStringList;
  CurrStringsDigitsPos, CurrStringsTeensPos, CurrStringsTensPos, CurrStringsThousandsPos: Integer;

function abcNumToWords(Num: Double; CurrText, CurrFractionText: string;
  FractionFormat: TabcCurrFractionFormat; FirstUpperCase, HandleException: Boolean): string;
{
 NB: Currency fraction is currently in hundredths rounded.
}
var
  nRoundedFraction, nCount, nLength, nPos, nDigit: Integer;
  bAllZeros: Boolean;
  sTemp, sBuff: string;
begin
  try
    if FractionFormat <> ffNoFraction then begin
      {convert fraction}
      {NB must use round rather than trunc due to poor precision of floating point
       e.g. 9.11 is stored as 9.10999999999999999 which would truncate 11 cents
       down to 10 cents.
      }
      nRoundedFraction := Round(Frac(Num) * 100);
      case FractionFormat of
        ffHundredths:
          begin
            Result := sabcCurrTextAnd + #32 +
              Format(sabcCurrTextFormatDecimal, [nRoundedFraction]) + sabcCurrTextHundredths;
            if CurrText <> '' then
              Result := Result + #32 + CurrText;
          end;
        ffUnitsDigits:
          begin
            Result := sabcCurrTextAnd + #32 +
              Format(sabcCurrTextFormatDecimal, [nRoundedFraction]);
            if CurrFractionText <> '' then
              Result := Result + #32 + CurrFractionText;
            if CurrText <> '' then
              Result := CurrText + #32 + Result;
          end;
        ffUnitsWords:
          begin
            Result := sabcCurrTextAnd + #32 + abcNumToWords(nRoundedFraction, '', '', ffNoFraction, False, HandleException);
            if CurrFractionText <> '' then
              Result := Result + #32 + CurrFractionText;
            if CurrText <> '' then
              Result := CurrText + #32 + Result;
          end;
      end;
    end
    else
      Result := CurrText;

    {convert whole number to string}
    sTemp := IntToStr(Trunc(Num));
    nLength := Length(sTemp);
    nCount := nLength;
    while nCount > 0 do begin
      nDigit := StrToInt(sTemp[nCount]);
      nPos := nLength - nCount + 1;
      case (nPos Mod 3) of
        0:
          if nDigit > 0 then
            Result := CurrStrings[CurrStringsDigitsPos + nDigit] + #32 + sabcCurrTextHundred + #32 + Result;
        1:
          begin
            bAllZeros := False;
            if nCount = 1 then
              sBuff := CurrStrings[CurrStringsDigitsPos + nDigit] + #32
            else if (sTemp[nCount - 1] = #49 {'1'}) then begin
              sBuff := CurrStrings[CurrStringsTeensPos + nDigit] + #32;
              Dec(nCount);
            end
            else if nDigit > 0 then
              sBuff := CurrStrings[CurrStringsDigitsPos + nDigit] + #32
            else begin
              sBuff := '';
              bAllZeros := True;
              if (nCount > 1) then
                if (sTemp[nCount - 1] <> #48 {'0'}) then
                  bAllZeros := False;
              if (nCount > 2) then
                if (sTemp[nCount - 2] <> #48 {'0'}) then
                  bAllZeros := False;
            end;
            if not bAllZeros and (nPos > 1) then
              sBuff := sBuff + CurrStrings[CurrStringsThousandsPos + nPos div 3] + #32;
            Result := sBuff + Result;
          end;
        2:
          if nDigit > 0 then
            Result := CurrStrings[CurrStringsTensPos + nDigit] + #32 + Result;
      end;
      Dec(nCount);
    end;
    if Result[Length(Result)] = #32 then
      Result := Copy(Result, 1, (Length(Result) - 1));
    if FirstUpperCase then
      Result := UpperCase(Result[1]) + Copy(Result, 2, Length(Result));
  except
    if HandleException then
      Result := sabcCurrTextError
    else
      raise;
  end;
end;

{ TabcCustomCurrencyLabel }

constructor TabcCustomCurrencyLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFirstUpperCase := True;
  FHandleException := True;
  FCurrText := sabcCurrTextDollars;
  FCurrFractionText := sabcCurrTextCents;
  UpdateCaption;
end;

procedure TabcCustomCurrencyLabel.Loaded;
begin
  inherited Loaded;
  UpdateCaption;
end;

procedure TabcCustomCurrencyLabel.SetCurrFractionFormat(Value: TabcCurrFractionFormat);
begin
  if Value <> FCurrFractionFormat then begin
    FCurrFractionFormat := Value;
    UpdateCaption;
  end;
end;

procedure TabcCustomCurrencyLabel.SetCurrFractionText(Value: string);
begin
  if Value <> FCurrFractionText then begin
    FCurrFractionText := Value;
    UpdateCaption;
  end;
end;

procedure TabcCustomCurrencyLabel.SetCurrText(Value: string);
begin
  if Value <> FCurrText then begin
    FCurrText := Value;
    UpdateCaption;
  end;
end;

procedure TabcCustomCurrencyLabel.SetFirstUpperCase(Value: Boolean);
begin
  if Value <> FFirstUpperCase then begin
    FFirstUpperCase := Value;
    UpdateCaption;
  end;
end;

procedure TabcCustomCurrencyLabel.SetValue(AValue: Double);
begin
  if AValue <> FValue then begin
    FValue := AValue;
    UpdateCaption;
  end;
end;

procedure TabcCustomCurrencyLabel.UpdateCaption;
begin
  inherited Caption := abcNumToWords(FValue, FCurrText, FCurrFractionText,
    FCurrFractionFormat, FFirstUpperCase, FHandleException);
end;

{ TabcCurrencyLabel }

function TabcCurrencyLabel.GetCaption: string;
begin
  Result := inherited Caption;
end;

initialization
  CurrStrings := TStringList.Create;
  CurrStringsDigitsPos := 0;
  abcParseStringToList(sabcCurrTextDigits, ';', CurrStrings, True);
  CurrStringsTeensPos := CurrStrings.Count;
  abcParseStringToList(sabcCurrTextTeens, ';', CurrStrings, True);
  CurrStringsTensPos := CurrStrings.Count - 1;
  abcParseStringToList(sabcCurrTextTens, ';', CurrStrings, True);
  CurrStringsThousandsPos := CurrStrings.Count - 1;
  abcParseStringToList(sabcCurrTextThousands, ';', CurrStrings, True);

finalization
  CurrStrings.Free;
  CurrStrings := nil;

end.
