unit abcdaado;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ Dataset Adapter for Delphi ADO Datasets }

interface

uses Classes, Db, ADOdb, abcda;

type

  { TabcADOAdapterReg }

  TabcADOAdapterReg = class(TabcDatasetAdapterReg);

  { TabcADODatasetAdapter }

  TabcCustomADOAdapter = class(TabcDatasetAdapter)
  private
    FCustomADODataset: TCustomADODataset;
  protected
    { SQL Transactions }
    function IsTransactionEnabled: Boolean; override;
    function IsStartTransactionEnabled: Boolean; override;
    function IsCommitEnabled: Boolean; override;
    function IsRollbackEnabled: Boolean; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    procedure Commit; override;
    procedure Rollback; override;
    procedure StartTransaction; override;
  end;

  TabcADODatasetAdapter = class(TabcCustomADOAdapter)
  private
    FADODataset: TADODataset;
  protected
    function IsIndexEnabled: Boolean; override;
    function GetIndexField(Index: Integer): TField; override;
    procedure SetIndexField(Index: Integer; const Value: TField); override;
    function GetIndexName: string; override;
    procedure SetIndexName(const Value: string); override;
    function GetIndexFieldNames: string; override;
    procedure SetIndexFieldNames(const Value: string); override;
    function GetIndexFieldCount: Integer; override;
    function GetIndexValue: string; override;
    function IsParamsEnabled: Boolean; override;
    function GetParamCount: Integer; override;
    function IsFindKeyEnabled: Boolean; override;
    function IsFindNearestEnabled: Boolean; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    procedure GetIndexNames(const Strings: TStrings); override;
    function FindKey(const KeyValues: Variant): Boolean; override;
    procedure FindNearest(const KeyValues: Variant); override;
  end;

  { TabcADOTableAdapter }

  TabcADOTableAdapter = class(TabcCustomADOAdapter)
  private
    FADOTable: TADOTable;
  protected
    function GetDescription: string; override;
    function IsIndexEnabled: Boolean; override;
    function GetIndexDefs: TIndexDefs; override;
    procedure SetIndexDefs(const Value: TIndexDefs); override;
    function GetIndexField(Index: Integer): TField; override;
    procedure SetIndexField(Index: Integer; const Value: TField); override;
    function GetIndexName: string; override;
    procedure SetIndexName(const Value: string); override;
    function GetIndexFieldNames: string; override;
    procedure SetIndexFieldNames(const Value: string); override;
    function GetIndexFieldCount: Integer; override;
    function GetIndexValue: string; override;
    function IsReadOnlyEnabled: Boolean; override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(Value: Boolean); override;
    function IsFindKeyEnabled: Boolean; override;
    function IsFindNearestEnabled: Boolean; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    procedure GetIndexNames(const Strings: TStrings); override;
    function FindKey(const KeyValues: Variant): Boolean; override;
    procedure FindNearest(const KeyValues: Variant); override;
  end;

  { TabcADOQueryAdapter }

  TabcADOQueryAdapter = class(TabcCustomADOAdapter)
  private
    FADOQuery: TADOQuery;
  protected
    function IsParamsEnabled: Boolean; override;
    function GetParamCount: Integer; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
  end;

  { TabcADOStoredProcAdapter }

  TabcADOStoredProcAdapter = class(TabcCustomADOAdapter)
  private
    FADOStoredProc: TADOStoredProc;
  protected
    function IsParamsEnabled: Boolean; override;
    function GetParamCount: Integer; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
  end;

implementation

uses abcutild, abcnpcon, abcutilm, abcutilf;

{ TabcCustomADOAdapter }

constructor TabcCustomADOAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FCustomADODataset := TCustomADODataset(ADataset);
end;

class function TabcCustomADOAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TCustomADODataset;
end;

function TabcCustomADOAdapter.IsTransactionEnabled: Boolean;
begin
  with FCustomADODataset do
    Result := (Connection <> nil) and Connection.Connected;
end;

function TabcCustomADOAdapter.IsCommitEnabled: Boolean;
begin
  Result := IsTransactionEnabled and FCustomADODataset.Connection.InTransaction;
end;

function TabcCustomADOAdapter.IsRollbackEnabled: Boolean;
begin
  Result := IsCommitEnabled;
end;

function TabcCustomADOAdapter.IsStartTransactionEnabled: Boolean;
begin
  Result := IsTransactionEnabled;
end;

procedure TabcCustomADOAdapter.Commit;
begin
  if IsCommitEnabled then
    FCustomADODataset.Connection.CommitTrans;
end;

procedure TabcCustomADOAdapter.Rollback;
begin
  if IsRollbackEnabled then
    FCustomADODataset.Connection.RollbackTrans;
end;

procedure TabcCustomADOAdapter.StartTransaction;
begin
  if IsStartTransactionEnabled then
    FCustomADODataset.Connection.BeginTrans;
end;

{ TabcADODatasetAdapter }

constructor TabcADODatasetAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FADODataset := TADODataset(ADataset);
end;

class function TabcADODatasetAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TADODataset;
end;

function TabcADODatasetAdapter.GetParamCount: Integer;
begin
  Result := FADODataset.Parameters.Count;
end;

function TabcADODatasetAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcADODatasetAdapter.GetIndexField(Index: Integer): TField;
begin
  Result := FADODataset.IndexFields[Index];
end;

function TabcADODatasetAdapter.GetIndexFieldCount: Integer;
begin
  Result := FADODataset.IndexFieldCount;
end;

function TabcADODatasetAdapter.GetIndexFieldNames: string;
begin
  Result := FADODataset.IndexFieldNames;
end;

function TabcADODatasetAdapter.GetIndexName: string;
begin
  Result := FADODataset.IndexFieldNames;
end;

procedure TabcADODatasetAdapter.GetIndexNames(const Strings: TStrings);
var
  i: integer;
begin
  with FADODataset do
    for i := 0 to FieldCount - 1 do
      if Fields[i].DataType in [ftString..ftDateTime] then
        Strings.Add(Fields[i].Name);
end;

function TabcADODatasetAdapter.GetIndexValue: string;
var
  i: integer;
  FieldStr: string;
begin
  Result := '';
  with FADODataset do
    for I := 0 to IndexFieldCount - 1 do begin
      FieldStr := IndexFields[I].AsString;
      if Length(Result) + Length(FieldStr) > 255 then
        Break
      else
        Result := Result + FieldStr + sabcnpBmkSep;
    end;
  abcShorten(Result, 1);
end;

function TabcADODatasetAdapter.IsIndexEnabled: Boolean;
begin
  Result := False;
end;

procedure TabcADODatasetAdapter.SetIndexField(Index: Integer;
  const Value: TField);
begin
  FADODataset.IndexFields[Index] := Value;
end;

procedure TabcADODatasetAdapter.SetIndexFieldNames(const Value: string);
begin
  FADODataset.IndexFieldNames := Value;
end;

procedure TabcADODatasetAdapter.SetIndexName(const Value: string);
begin
  FADODataset.IndexFieldNames := Value;
end;

function TabcADODatasetAdapter.IsFindKeyEnabled: Boolean;
begin
  Result := True;
end;

function TabcADODatasetAdapter.IsFindNearestEnabled: Boolean;
begin
  Result := True;
end;

function TabcADODatasetAdapter.FindKey(const KeyValues: Variant): Boolean;
begin
  Result := FADODataset.Locate(IndexFieldNames, KeyValues, []);
end;

procedure TabcADODatasetAdapter.FindNearest(const KeyValues: Variant);
begin
  FADODataset.Locate(
    IndexFieldNames, KeyValues,
    [loCaseInsensitive, loPartialKey]);
end;

{ TabcADOTableAdapter }

constructor TabcADOTableAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FADOTable := TADOTable(ADataset);
end;

class function TabcADOTableAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TADOTable;
end;

function TabcADOTableAdapter.GetDescription: string;
begin
  if abc_UseTableComponentNames then
    Result := FADOTable.Name
  else
    Result := FADOTable.TableName;
end;

function TabcADOTableAdapter.GetIndexField(Index: Integer): TField;
begin
  Result := FADOTable.IndexFields[Index];
end;

function TabcADOTableAdapter.GetIndexFieldCount: Integer;
begin
  Result := FADOTable.IndexFieldCount;
end;

function TabcADOTableAdapter.GetIndexFieldNames: string;
begin
  Result := FADOTable.IndexFieldNames;
end;

function TabcADOTableAdapter.GetIndexName: string;
begin
  Result := FADOTable.IndexName;
end;

procedure TabcADOTableAdapter.GetIndexNames(const Strings: TStrings);
begin
  FADOTable.GetIndexNames(Strings);
end;

function TabcADOTableAdapter.GetIndexValue: string;
var
  i: integer;
  FieldStr: string;
begin
  Result := '';
  with FADOTable do
    for I := 0 to IndexFieldCount - 1 do begin
      FieldStr := IndexFields[I].AsString;
      if Length(Result) + Length(FieldStr) > 255 then
        Break
      else
        Result := Result + FieldStr + sabcnpBmkSep;
    end;
  Result := Copy(Result, 0, Length(Result) - 1);
end;

function TabcADOTableAdapter.GetReadOnly: Boolean;
begin
  Result := FADOTable.ReadOnly;
end;

function TabcADOTableAdapter.IsIndexEnabled: Boolean;
begin
  Result := True;
end;

function TabcADOTableAdapter.IsReadOnlyEnabled: Boolean;
begin
  Result := True;
end;

function TabcADOTableAdapter.GetIndexDefs: TIndexDefs;
begin
  Result := FADOTable.IndexDefs;
end;

procedure TabcADOTableAdapter.SetIndexDefs(const Value: TIndexDefs);
begin
  FADOTable.IndexDefs := Value;
end;

procedure TabcADOTableAdapter.SetIndexField(Index: Integer;
  const Value: TField);
begin
  FADOTable.IndexFields[Index] := Value;
end;

procedure TabcADOTableAdapter.SetIndexFieldNames(const Value: string);
begin
  FADOTable.IndexFieldNames := Value;
end;

procedure TabcADOTableAdapter.SetIndexName(const Value: string);
var
  WasActive: Boolean;
begin
  with FADOTable do
  try
    DisableControls;
    WasActive := Active;
    Close;
    IndexName := Value;
    if WasActive then Open;
  finally
    EnableControls;
  end;
end;

procedure TabcADOTableAdapter.SetReadOnly(Value: Boolean);
begin
  FADOTable.ReadOnly := Value;
end;

function TabcADOTableAdapter.IsFindKeyEnabled: Boolean;
begin
  Result := True;
end;

function TabcADOTableAdapter.IsFindNearestEnabled: Boolean;
begin
  Result := True;
end;

function TabcADOTableAdapter.FindKey(const KeyValues: Variant): Boolean;
begin
  Result := FADOTable.Locate(IndexFieldNames, KeyValues, []);
end;

procedure TabcADOTableAdapter.FindNearest(const KeyValues: Variant);
begin
  FADOTable.Locate(
    IndexFieldNames, KeyValues,
    [loCaseInsensitive, loPartialKey]);
end;

{ TabcADOQueryAdapter }

constructor TabcADOQueryAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FADOQuery := TADOQuery(ADataset);
end;

class function TabcADOQueryAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TADOQuery;
end;

function TabcADOQueryAdapter.GetParamCount: Integer;
begin
  Result := FADOQuery.Parameters.Count;
end;

function TabcADOQueryAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

{ TabcADOStoredProcAdapter }

constructor TabcADOStoredProcAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FADOStoredProc := TADOStoredProc(ADataset);
end;

class function TabcADOStoredProcAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TADOStoredProc;
end;

function TabcADOStoredProcAdapter.GetParamCount: Integer;
begin
  Result := FADOStoredProc.Parameters.Count;
end;

function TabcADOStoredProcAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

initialization
  abcDSAdapterFactory.RegisterAdapter(TabcADODatasetAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcADOTableAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcADOQueryAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcADOStoredProcAdapter);

finalization
  abcDSAdapterFactory.UnRegisterAdapter(TabcADODatasetAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcADOTableAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcADOQueryAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcADOStoredProcAdapter);

end.
