unit abcdabde;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ Dataset Adapter classes for BDE }

{$I abcdefs.inc}

interface

uses Windows, Classes, DB, BDE, DBCommon, DBTables, abcda, Forms
     {$IFDEF DELPHI6}, Variants {$ENDIF}
     ;

type

  { TabcBDEAdapterReg }

  TabcBDEAdapterReg = class(TabcDatasetAdapterReg);

  { TabcDBDatasetAdapter for TDBDataset }

  TabcBDEFilterObject = class;

  TabcDBDatasetAdapter = class(TabcDatasetAdapter)
  private
    FDBDataset: TDBDataset;
    FFilter: TabcBDEFilterObject;
    procedure FilterNeeded;
  protected
    { SQL Transactions }
    function IsTransactionEnabled: Boolean; override;
    function IsStartTransactionEnabled: Boolean; override;
    function IsCommitEnabled: Boolean; override;
    function IsRollbackEnabled: Boolean; override;
    { Dataset Filters - implementing multiple BDE filters }
    function IsSingleFilter: Boolean; override;
    function GetFilterText: string; override;
    function GetFiltered: Boolean; override;
    function GetFilterGoToFirst: Boolean; override;
    function GetFilterOptions: TFilterOptions; override;
    function GetFilterPriority: Word; override;
    function GetOnActivateFilter: TNotifyEvent; override;
    function GetOnDeactivateFilter: TNotifyEvent; override;
    function GetOnFilterRecord: TFilterRecordEvent; override;
    function GetFilterFieldDelimiter: TabcFilterFieldDelimiter; override;
    function GetFilterPartialMatch: TabcFilterPartialMatch; override;
    procedure SetFilterText(Value: string); override;
    procedure SetFiltered(Value: Boolean); override;
    procedure SetFilterGoToFirst(Value: Boolean); override;
    procedure SetFilterOptions(const Value: TFilterOptions); override;
    procedure SetFilterPriority(const Value: Word); override;
    procedure SetOnActivateFilter(const Value: TNotifyEvent); override;
    procedure SetOnDeactivateFilter(const Value: TNotifyEvent); override;
    procedure SetOnFilterRecord(const Value: TFilterRecordEvent); override;

    function GetNetUserName: string; override;

  public
    constructor Create(const ADataset: TDataset); override;
    destructor Destroy; override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatabaseInfo(const Strings: TStrings); override;
    procedure GetSessionInfo(const Strings: TStrings); override;
    procedure GetDBEngineInfo(const Strings: TStrings); override;
    { SQL Transactions }
    procedure StartTransaction; override;
    procedure Commit; override;
    procedure Rollback; override;
    { Filter Event Buffer values }
    function FilterParamByName(const FieldName: string): TParam; override;
    { Update Buffers }
    procedure FlushBuffers; override;
  end;

  { TabcTableAdapter for BDE TTable }

  TabcTableAdapter = class(TabcDBDatasetAdapter)
  private
    FTable: TTable;
  protected
    function GetDescription: string; override;
    { Empty Table }
    function IsEmptyTableEnabled: Boolean; override;
    { Indexes }
    function IsIndexEnabled: Boolean; override;
    function GetIndexDefs: TIndexDefs; override;
    procedure SetIndexDefs(const Value: TIndexDefs); override;
    function GetIndexField(Index: Integer): TField; override;
    procedure SetIndexField(Index: Integer; const Value: TField); override;
    function GetIndexName: string; override;
    procedure SetIndexName(const Value: string); override;
    function GetIndexFieldNames: string; override;
    procedure SetIndexFieldNames(const Value: string); override;
    function GetIndexFieldCount: Integer; override;
    function GetIndexValue: string; override;
    { Range Filters }
    function IsRangeEnabled: Boolean; override;
    function GetKeyExclusive: Boolean; override;
    procedure SetKeyExclusive(Value: Boolean); override;
    { Find }
    function IsFindKeyEnabled: Boolean; override;
    function IsFindNearestEnabled: Boolean; override;
    function GetSearchMethod: TabcSearchMethod; override;
    { ReadOnly }
    function IsReadOnlyEnabled: Boolean; override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(Value: Boolean); override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetIndexInfo(const Strings: TStrings); override;
    { Empty Table }
    procedure EmptyTable; override;
    { Indexes }
    procedure GetIndexNames(const Strings: TStrings); override;
    { Range Filters }
    procedure ApplyRange; override;
    procedure CancelRange; override;
    procedure EditRangeStart; override;
    procedure EditRangeEnd; override;
    procedure SetRange(const StartValues, EndValues: Variant); override;
    procedure SetRangeStart; override;
    procedure SetRangeEnd; override;
    { Find Key }
    procedure EditKey; override;
    function FindKey(const KeyValues: Variant): Boolean; override;
    procedure FindNearest(const KeyValues: Variant); override;
    procedure GoToKey; override;
    procedure GoToNearest; override;
    procedure SetKey; override;
  end;

  { TabcQueryAdapter for BDE TQuery }

  TabcQueryAdapter = class(TabcDBDatasetAdapter)
  private
    FQuery: TQuery;
  protected
    { Query Params }
    function IsParamsEnabled: Boolean; override;
    function GetParams: TParams; override;
    procedure SetParams(Value: TParams); override;
    function GetParamCount: Integer; override;
    {Query ReadOnly}
    function IsReadOnlyEnabled: Boolean; override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(Value: Boolean); override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetParamInfo(const Strings: TStrings); override;
  end;

  { TabcStoredProcAdapter for BDE TStoredProc }

  TabcStoredProcAdapter = class(TabcDBDatasetAdapter)
  private
    FStoredProc: TStoredProc;
  protected
    { Params }
    function IsParamsEnabled: Boolean; override;
    function GetParams: TParams; override;
    procedure SetParams(Value: TParams); override;
    function GetParamCount: Integer; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetParamInfo(const Strings: TStrings); override;
  end;

  { TabcBDEFilterObject }

  TabcBDEFilterObject = class
  private
    FActive: Boolean;
    FDataset: TDBDataset;
    FFilter: string;
    FFilterOptions: TFilterOptions;
    FFilterFuncFieldBuf: PChar;
    FFilterFuncRecBuf: Pointer;
    FFilterFuncParam: TParam;
    FGoToFirst: Boolean;
    FHandle, FFuncHandle: hDBIFilter;
    FOnActivate: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;
    FOnFilterRecord: TFilterRecordEvent;
    FPriority: Word;
    procedure DoDbiDropFilter;
    procedure DoDbiAddFilter;
    procedure DoDbiActivateFilter;
    procedure DoDbiDeactivateFilter;
    procedure SetActive(Value: Boolean);
    procedure SetFilter(Value: string);
    procedure SetFilterOptions(Value: TFilterOptions);
    procedure SetOnFilterRecord(Value: TFilterRecordEvent);
    procedure SetPriority(Value: Word);
  protected
    property Handle: hDBIFilter read FHandle;
  public
    constructor Create(ADBDataset: TDBDataset);
    destructor Destroy; override;
    procedure Activate;
    function CanActivate: Boolean;
    procedure Deactivate;
    function ParamByName(const FieldName: string): TParam;
    function HandleAllocated: Boolean;
    property Active: Boolean read FActive write SetActive;
    property Filter: string read FFilter write SetFilter;
    property FilterOptions: TFilterOptions read FFilterOptions write SetFilterOptions default [];
    property GoToFirst: Boolean read FGoToFirst write FGoToFirst default True;
    property Priority: Word read FPriority write SetPriority default 1;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnFilterRecord: TFilterRecordEvent read FOnFilterRecord write SetOnFilterRecord;
  end;

implementation

uses SysUtils, SysConst, abcconst, abcnpcon, abcfiltr, abcutild;

const
  ABC_TRANSISOLATIONS: array[TTransIsolation] of string =
    (sabcnpiDirtyRead, sabcnpiReadCommitted, sabcnpiRepeatableRead);

  ABC_TABLETYPES: array[TTableType] of string =
    (sabcnpittDefault, sabcnpittParadox, sabcnpittDBase, sabcnpittASCII, sabcnpittFoxPro);

  ABC_UPDATEMODES: array[TUpdateMode] of string =
    (sabcnpiupWhereAll, sabcnpiupWhereChanged, sabcnpiupWhereKeyOnly);
  ABC_PARAMBINDMODES: array[TParamBindMode] of string =
    (sabcnpipbByName, sabcnpipbByNumber);
  ABC_PARAMTYPES: array[TParamType] of string =
    (sabcnpiptUnknown, sabcnpiptInput, sabcnpiptOutput, sabcnpiptInputOutput,
     sabcnpiptResult);

{ Filter Callback Function }

function AddFilterCallback(CallBack: Pointer; Filter: TabcBDEFilterObject; var Handle: hDBIFilter): Boolean;
var
  DBResult: DBIResult;
begin
  Result := False;
  if CallBack = nil then Exit;
  DBResult := DbiAddFilter(Filter.FDataset.Handle, Longint(Filter), Filter.FPriority + 1, False,
    nil, pfGENFilter(CallBack), Handle);
  if DBResult <> DBIERR_NONE then Handle := nil;
  Check(DBResult);
end;

function FilterCallback(ulClientData: Longint; pRecBuf: Pointer; iPhyRecNum: Longint): Smallint; stdcall;
var
  Source: TabcBDEFilterObject;
  Accept: Boolean;
begin
  Result := 1;
  Source := TabcBDEFilterObject(ulClientData);
  try
    with Source do
      if Assigned(FOnFilterRecord) then begin
        FFilterFuncRecBuf:= pRecBuf;
        Accept := True;
        OnFilterRecord(FDataset, Accept);
        Result := Ord(Accept);
      end;
  except
    Application.HandleException(Source);
  end;
end;

{ TabcDBDatasetAdapter for TDBDataset }

constructor TabcDBDatasetAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FDBDataset := TDBDataset(ADataset);
end;

destructor TabcDBDatasetAdapter.Destroy;
begin
  FFilter.Free;
  inherited;
end;

class function TabcDBDatasetAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TDBDataset;
end;

function TabcDBDatasetAdapter.IsTransactionEnabled: Boolean;
begin
  Result := False;
  if (FDBDataset.Database <> nil) then
    with FDBDataset.Database do
      Result := IsSQLBased or (TransIsolation = tiDirtyRead);
end;

function TabcDBDatasetAdapter.IsStartTransactionEnabled: Boolean;
begin
  Result := IsTransactionEnabled;
  if Result then
    Result := not FDBDataset.Database.InTransaction;
end;

function TabcDBDatasetAdapter.IsCommitEnabled: Boolean;
begin
  Result := IsTransactionEnabled;
  if Result then
    Result := FDBDataset.Database.InTransaction;
end;

function TabcDBDatasetAdapter.IsRollbackEnabled: Boolean;
begin
  Result := IsCommitEnabled;
end;

procedure TabcDBDatasetAdapter.StartTransaction;
begin
  if CanStartTransaction then
    FDBDataset.Database.StartTransaction;
end;

procedure TabcDBDatasetAdapter.Commit;
begin
  if CanCommit then
    FDBDataset.Database.Commit;
end;

procedure TabcDBDatasetAdapter.Rollback;
begin
  if CanRollback then
    FDBDataset.Database.Rollback;
end;

procedure TabcDBDatasetAdapter.GetDatabaseInfo(const Strings: TStrings);
var
  Indent: Integer;
  TempStrings: TStringList;
  I: Integer;
begin
  Indent := 0;
  AddStrIndent(sabcnpiDatabase,0,Strings);
  Inc(Indent);
  if FDBDataset.Database <> nil then
    with FDBDataset.Database do begin
      AddStrIndent(sabcnpiDBCompName + Name, Indent, Strings);
      AddStrIndent(sabcnpiDBName + DatabaseName, Indent, Strings);
      AddStrIndent(sabcnpiAlias + AliasName, Indent, Strings);
      AddStrIndent(sabcnpiConnected + ABC_BOOLEANS[Connected], Indent, Strings);
      AddStrIndent(sabcnpiDSCount + InttoStr(DatasetCount), Indent, Strings);
      AddStrIndent(sabcnpiDriver + DriverName, Indent, Strings);
      AddStrIndent(sabcnpiSQLBased + ABC_BOOLEANS[IsSQLBased], Indent, Strings);
      AddStrIndent(sabcnpiKeepConn + ABC_BOOLEANS[Connected], Indent, Strings);
      AddStrIndent(sabcnpiLogin + ABC_BOOLEANS[Connected], Indent, Strings);
      AddStrIndent(sabcnpiTemp + ABC_BOOLEANS[Temporary], Indent, Strings);
      AddStrIndent(sabcnpiTransIsol + ABC_TRANSISOLATIONS[TransIsolation], Indent, Strings);
      AddStrIndent('',0,Strings);

      {Params}
      AddStrIndent(sabcnpiParams, Indent, Strings);
      Inc(Indent);
      if Params.Count > 0 then
        for I := 0 to Params.Count - 1 do
          AddStrIndent(Params.Strings[I], Indent, Strings)
      else
        AddStrIndent(sabcnpiParamsEmpty,Indent,Strings);
      Dec(Indent);
      AddStrIndent('',0,Strings);

      {Tables}
      AddStrIndent(sabcnpiTBNames, Indent, Strings);
      Inc(Indent);
      TempStrings := TStringList.Create;
      try
        Session.GetTableNames(DatabaseName, '',
          True, True, TempStrings);
        for I := 0 to TempStrings.Count - 1 do
          AddStrIndent(TempStrings.Strings[I], Indent, Strings);
      finally
        TempStrings.Free;
      end;
      Dec(Indent);
      AddStrIndent('',0,Strings);

      {Stored Procs}
      AddStrIndent(sabcnpiSPNames, Indent, Strings);
      Inc(Indent);
      if IsSQLBased then begin
        try
          TempStrings := TStringList.Create;
          try
            Session.GetStoredProcNames(DatabaseName, TempStrings);
            for I := 0 to TempStrings.Count - 1 do
              AddStrIndent(TempStrings.Strings[I], Indent, Strings);
          finally
            TempStrings.Free;
          end;
        except
          on E: Exception do AddStrIndent(E.Message, Indent, Strings);
        end;
      end
      else
        AddStrIndent(sabcnpiNotSupported,Indent,Strings);
      Dec(Indent);
      AddStrIndent('',0,Strings);

      {Alias Params}
      if AliasName <> '' then begin
        AddStrIndent(sabcnpiAliasParams, Indent, Strings);
        Inc(Indent);
        TempStrings := TStringList.Create;
        try
          try
            Session.GetAliasParams(AliasName, TempStrings);
            for I := 0 to TempStrings.Count - 1 do
              AddStrIndent(TempStrings.Strings[I], Indent, Strings);
          finally
            TempStrings.Free;
          end;
        except
          on E: Exception do AddStrIndent(sabcnpiException + E.Message,Indent, Strings);
        end;
        Dec(Indent);
        AddStrIndent('',0,Strings);
      end;

      {Driver Params}
      if DriverName <> '' then begin
        AddStrIndent(sabcnpiDriverParams, Indent, Strings);
        Inc(Indent);
        TempStrings := TStringList.Create;
        try
          try
            Session.GetDriverParams(DriverName, TempStrings);
            for I := 0 to TempStrings.Count - 1 do
              AddStrIndent(TempStrings.Strings[I], Indent, Strings);
          finally
            TempStrings.Free;
          end;
        except
          on E: Exception do AddStrIndent(sabcnpiException + E.Message,Indent, Strings);
        end;
        AddStrIndent('',0,Strings);
      end;
    end
  else
    AddStrIndent(sabcnpiDBNil, Indent, Strings);
  AddStrIndent('',0,Strings);
end;

procedure TabcDBDatasetAdapter.GetSessionInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;
  TempStrings: TStringList;
begin
  Indent := 0;
  AddStrIndent(sabcnpiSession,0,Strings);
  Inc(Indent);
  TempStrings := TStringList.Create;
  try
    with FDBDataset.DBSession do begin
      AddStrIndent(sabcnpiDatabaseCount + IntToStr(DatabaseCount),Indent,Strings);

      AddStrIndent(sabcnpiDatabaseNames,Indent,Strings);
      TempStrings.Clear;
      GetDatabaseNames(TempStrings);
      Inc(Indent);
      for I := 0 to TempStrings.Count - 1 do
        AddStrIndent(TempStrings.Strings[I], Indent, Strings);
      Dec(Indent);

      AddStrIndent(sabcnpiKeepConnections + ABC_BOOLEANS[KeepConnections],Indent,Strings);
      AddStrIndent(sabcnpiNetFileDir + NetFileDir,Indent,Strings);
      AddStrIndent(sabcnpiPrivateDir + PrivateDir,Indent,Strings);

      AddStrIndent(sabcnpiBDEDriverNames,Indent,Strings);
      TempStrings.Clear;
      GetDriverNames(TempStrings);
      Inc(Indent);
      for I := 0 to TempStrings.Count - 1 do
        AddStrIndent(TempStrings.Strings[I], Indent, Strings);
    end;
  finally
    TempStrings.Free;
  end;
  AddStrIndent('',0,Strings);
end;

procedure TabcDBDatasetAdapter.GetDBEngineInfo(const Strings: TStrings);
var
  Indent: Integer;
  Status: SYSInfo;
  Client: CLIENTInfo;
  Config: SYSConfig;
  Version: SYSVersion;
  DBiFnResult: DBIResult;
begin
  Indent := 0;
  AddStrIndent(sabcnpiBDE,Indent,Strings);
  Inc(Indent);
  AddStrIndent('',0,Strings);

  {load BDE System Info}
  AddStrIndent(sabcnpiSysInfo,Indent,Strings);
  Inc(Indent);
  DBiFnResult := DbiGetSysInfo(Status);
  if DBiFnResult = DBIERR_NONE then begin
    AddStrIndent(sabcnpiBufferSpace + IntToStr(Status.iBufferSpace),Indent,Strings);
    AddStrIndent(sabcnpiHeapSpace + IntToStr(Status.iHeapSpace),Indent,Strings);
    AddStrIndent(sabcnpiDrivers + IntToStr(Status.iDrivers),Indent,Strings);
    AddStrIndent(sabcnpiClients + IntToStr(Status.iClients),Indent,Strings);
    AddStrIndent(sabcnpiSessions + IntToStr(Status.iSessions),Indent,Strings);
    AddStrIndent(sabcnpiDatabases + IntToStr(Status.iDatabases),Indent,Strings);
    AddStrIndent(sabcnpiCursors + IntToStr(Status.iCursors),Indent,Strings);
  end
  else
    AddStrIndent(sabcnpiDBIErrStatus + InttoStr(DBiFnResult),Indent,Strings);
  Dec(Indent);
  AddStrIndent('',0,Strings);

  {load BDE Client Info}
  AddStrIndent(sabcnpiClientInfo,Indent,Strings);
  Inc(Indent);
  DBiFnResult := DbiGetClientInfo(Client);
  if DBiFnResult = DBIERR_NONE then begin
    AddStrIndent(sabcnpiNameEq + StrPas(Client.szName),Indent,Strings);
    AddStrIndent(sabcnpiSessions + IntToStr(Client.iSessions),Indent,Strings);
    AddStrIndent(sabcnpiWorkDir + StrPas(Client.szWorkDir),Indent,Strings);
    AddStrIndent(sabcnpiLang + StrPas(Client.szLang),Indent,Strings);
  end
  else
    AddStrIndent(sabcnpiDBIErrClientStatus + InttoStr(DBiFnResult),Indent,Strings);
  Dec(Indent);
  AddStrIndent('',0,Strings);

  {load BDE Config Info}
  AddStrIndent(sabcnpiConfigInfo,Indent,Strings);
  Inc(Indent);
  DBiFnResult := DbiGetSysConfig(Config);
  if DBiFnResult = DBIERR_NONE then begin
    AddStrIndent(sabcnpiLocalShare + ABC_BOOLEANS[Config.bLocalShare],Indent,Strings);
    AddStrIndent(sabcnpiNetProtocol + IntToStr(Config.iNetProtocol),Indent,Strings);
    AddStrIndent(sabcnpiNetShare + ABC_BOOLEANS[Config.bNetShare],Indent,Strings);
    AddStrIndent(sabcnpiNetType + StrPas(Config.szNetType),Indent,Strings);
    AddStrIndent(sabcnpiUserName + StrPas(Config.szUserName),Indent,Strings);
    AddStrIndent(sabcnpiIniFile + StrPas(Config.szIniFile),Indent,Strings);
    AddStrIndent(sabcnpiLangDriver + StrPas(Config.szLangDriver),Indent,Strings);
  end
  else
    AddStrIndent(sabcnpiDBIErrConfig + InttoStr(DBiFnResult),Indent,Strings);
  Dec(Indent);
  AddStrIndent('',0,Strings);

  {load BDE Version Info}
  AddStrIndent(sabcnpiVersionInfo,Indent,Strings);
  Inc(Indent);
  DBiFnResult := DbiGetSysVersion(Version);
  if DBiFnResult = DBIERR_NONE then begin
    AddStrIndent(sabcnpiEngineversion + IntToStr(Version.iVersion),Indent,Strings);
    AddStrIndent(sabcnpiClientInterfacelevel + IntToStr(Version.iIntfLevel),Indent,Strings);
    AddStrIndent(sabcnpiVersiondate + DateToStr(Version.dateVer),Indent,Strings);
    AddStrIndent(sabcnpiVersiontime + TimeToStr(Version.timeVer),Indent,Strings);
  end
  else
    AddStrIndent(sabcnpiDBIErrSysVersion + InttoStr(DBiFnResult),Indent,Strings);

  AddStrIndent('',0,Strings);
  AddStrIndent('',0,Strings);
end;

procedure TabcDBDatasetAdapter.FilterNeeded;
//defers allocation of filter resources until needed
begin
  if FFilter = nil then
    FFilter := TabcBDEFilterObject.Create(FDBDataset);
end;

function TabcDBDatasetAdapter.GetFilterText: string;
begin
  FilterNeeded;
  Result := FFilter.Filter;
end;

function TabcDBDatasetAdapter.IsSingleFilter: Boolean;
begin
  Result := False;
end;

function TabcDBDatasetAdapter.GetFiltered: Boolean;
begin
  FilterNeeded;
  Result := FFilter.Active;
end;

function TabcDBDatasetAdapter.GetFilterGoToFirst: Boolean;
begin
  FilterNeeded;
  Result := FFilter.GoToFirst;
end;

function TabcDBDatasetAdapter.GetFilterOptions: TFilterOptions;
begin
  FilterNeeded;
  Result := FFilter.FilterOptions;
end;

function TabcDBDatasetAdapter.GetFilterPriority: Word;
begin
  FilterNeeded;
  Result := FFilter.Priority;
end;

function TabcDBDatasetAdapter.GetOnActivateFilter: TNotifyEvent;
begin
  FilterNeeded;
  Result := FFilter.OnActivate;
end;

function TabcDBDatasetAdapter.GetOnDeactivateFilter: TNotifyEvent;
begin
  FilterNeeded;
  Result := FFilter.OnDeactivate;
end;

function TabcDBDatasetAdapter.GetOnFilterRecord: TFilterRecordEvent;
begin
  FilterNeeded;
  Result := FFilter.OnFilterRecord;
end;

function TabcDBDatasetAdapter.GetFilterFieldDelimiter: TabcFilterFieldDelimiter;
begin
  Result := affSquareBracket;
end;

function TabcDBDatasetAdapter.GetFilterPartialMatch: TabcFilterPartialMatch;
begin
  Result := afpAsterisk;
end;

procedure TabcDBDatasetAdapter.SetFilterText(Value: string);
begin
  FilterNeeded;
  FFilter.Filter := Value;
end;

procedure TabcDBDatasetAdapter.SetFiltered(Value: Boolean);
begin
  FilterNeeded;
  FFilter.Active := Value;
end;

procedure TabcDBDatasetAdapter.SetFilterGoToFirst(Value: Boolean);
begin
  FilterNeeded;
  FFilter.GoToFirst := Value;
end;

procedure TabcDBDatasetAdapter.SetFilterOptions(const Value: TFilterOptions);
begin
  FilterNeeded;
  FFilter.FilterOptions := Value;
end;

procedure TabcDBDatasetAdapter.SetFilterPriority(const Value: Word);
begin
  FilterNeeded;
  FFilter.Priority := Value;
end;

procedure TabcDBDatasetAdapter.SetOnActivateFilter(const Value: TNotifyEvent);
begin
  FilterNeeded;
  FFilter.OnActivate := Value;
end;

procedure TabcDBDatasetAdapter.SetOnDeactivateFilter(const Value: TNotifyEvent);
begin
  FilterNeeded;
  FFilter.OnDeactivate := Value;
end;

procedure TabcDBDatasetAdapter.SetOnFilterRecord(const Value: TFilterRecordEvent);
begin
  FilterNeeded;
  FFilter.OnFilterRecord := Value;
end;

function TabcDBDatasetAdapter.FilterParamByName(const FieldName: string): TParam;
begin
  FilterNeeded;
  Result := FFilter.ParamByName(FieldName);
end;

procedure TabcDBDatasetAdapter.FlushBuffers;
begin
  FDBDataset.FlushBuffers;
end;

function TabcDBDatasetAdapter.GetNetUserName: string;
var
  NetUser: array[0..255] of char;
begin
  Result := '';
  if DbiGetNetUserName(NetUser) = 0 then
    Result := StrPas(NetUser);
end;

{ TabcBDEFilterObject }

constructor TabcBDEFilterObject.Create(ADBDataset: TDBDataset);
begin
  FDataset := ADBDataset;
  GetMem(FFilterFuncFieldBuf, 256);
  FFilterFuncParam := TParam.Create(nil);
  FHandle := nil;
  FPriority := 1;
  FGoToFirst := True;
end;

destructor TabcBDEFilterObject.Destroy;
begin
  if HandleAllocated and (FDataSet <> nil) and
  not (csDestroying in FDataSet.ComponentState) and
  FDataSet.Active then DoDbiDropFilter;
  FFilterFuncParam.Free;
  FreeMem(FFilterFuncFieldBuf, 256);
  inherited Destroy;
end;

procedure TabcBDEFilterObject.Activate;
begin
  SetActive(True);
end;

function TabcBDEFilterObject.CanActivate: Boolean;
begin
  Result := (FDataSet <> nil) and (FDataSet.Handle <> nil)
            and FDataSet.Active and ((Filter <> '') or Assigned(FOnFilterRecord));
end;

procedure TabcBDEFilterObject.Deactivate;
begin
  SetActive(False);
end;

procedure TabcBDEFilterObject.DoDbiActivateFilter;
begin
  if HandleAllocated then begin
    if FHandle <> nil then
      DbiActivateFilter(FDataSet.Handle, FHandle);
    if FFuncHandle <> nil then
      DbiActivateFilter(FDataSet.Handle, FFuncHandle);
    FDataSet.Refresh;
    if FGoToFirst then FDataSet.First;
  end
  else
    raise EabcInvalidFilterOperation.Create(sabcErrFilterHandle);
  if Assigned(FOnActivate) then FOnActivate(Self);
end;

procedure TabcBDEFilterObject.DoDbiAddFilter;
var
  Parser: TExprParser;
  DBResult: DBIResult;
begin
  if (FDataSet = nil) then
    raise EabcInvalidFilterOperation.Create(sabcErrDataSetRequired);
  if (FDataSet.Handle = nil) then
    raise EabcInvalidFilterOperation.Create(sabcErrDataSetInactive);
  if (FFilter = '') and not Assigned(FOnFilterRecord) then
    raise EabcInvalidFilterOperation.Create(sabcErrFilterExprEmpty);
  DoDbiDropFilter;
  if FFilter <> '' then begin
{$IFDEF DELPHI5}
    Parser := TExprParser.Create(FDataSet, FFilter, FFilterOptions, [], '', nil, FldTypeMap);
{$ELSE}
    Parser := TExprParser.Create(FDataSet, FFilter, FFilterOptions, [], '', nil);
{$ENDIF}
    try
      DBResult := DbiAddFilter(FDataSet.Handle, 0, FPriority, False,
        PCANExpr(Parser.FilterData), nil, FHandle);
      if DBResult <> DBIERR_NONE then FHandle := nil;
      Check(DBResult);
    finally
      Parser.Free;
    end;
  end;
  if Assigned(FOnFilterRecord) then begin
    AddFilterCallback(@FilterCallback, Self, FFuncHandle);
  end;
end;

procedure TabcBDEFilterObject.DoDbiDeactivateFilter;
begin
  if HandleAllocated then begin
    if FHandle <> nil then DbiDeactivateFilter(FDataSet.Handle, FHandle);
    if FFuncHandle <> nil then DbiDeactivateFilter(FDataSet.Handle, FFuncHandle);
    FDataSet.Refresh;
    if FGoToFirst then FDataSet.First;
    if Assigned(FOnDeactivate) then FOnDeactivate(Self);
  end;
end;

procedure TabcBDEFilterObject.DoDbiDropFilter;
begin
  if HandleAllocated then begin
    if Active then DoDbiDeactivateFilter;
    if FHandle <> nil then begin
      DbiDropFilter(FDataSet.Handle, FHandle);
      FHandle := nil;
    end;
    if FFuncHandle <> nil then begin
      DbiDropFilter(FDataSet.Handle, FFuncHandle);
      FHandle := nil;
    end;
    FActive := False;
  end;
end;

function TabcBDEFilterObject.ParamByName(const FieldName: string): TParam;
var
  Field: TField;
  pbBlank: BOOL;
begin
  if (FFuncHandle = nil) then
    raise EabcInvalidFilterOperation.Create(sabcErrFilterHandle);
  Field := FDataSet.FieldByName(FieldName);
  dbiGetField(FDataSet.Handle, Field.FieldNo, FFilterFuncRecBuf, FFilterFuncFieldBuf, pbBlank);
  with FFilterFuncParam do begin
    DataType:= Field.DataType;
    if DataType = ftString then
      NativeToAnsiBuf(FDataSet.Locale, FFilterFuncFieldBuf, FFilterFuncFieldBuf, 255);
    SetData(FFilterFuncFieldBuf);
    if pbBlank then Clear;
  end;
  Result := FFilterFuncParam;
end;

function TabcBDEFilterObject.HandleAllocated: Boolean;
begin
  Result := (FHandle <> nil) or (FFuncHandle <> nil);
end;

procedure TabcBDEFilterObject.SetActive(Value: Boolean);
begin
  if Value <> FActive then begin
    if Value then begin
      if not HandleAllocated then DoDbiAddFilter;
      DoDbiActivateFilter;
    end
    else begin
      DoDbiDeactivateFilter;
    end;
    FActive := Value;
  end;
end;

procedure TabcBDEFilterObject.SetFilter(Value: string);
begin
  FFilter := Value;
  if (Value = '') and Active then DoDbiDropFilter;
  if CanActivate then begin
    DoDbiAddFilter;
    if FActive then Activate;
  end;
end;

procedure TabcBDEFilterObject.SetFilterOptions(Value: TFilterOptions);
begin
  FFilterOptions := Value;
  if CanActivate then begin
    DoDbiAddFilter;
    if FActive then Activate;
  end;
end;

procedure TabcBDEFilterObject.SetOnFilterRecord(Value: TFilterRecordEvent);
begin
  if Assigned(FOnFilterRecord) and Active then
    Active := False;
  FOnFilterRecord := Value;
end;

procedure TabcBDEFilterObject.SetPriority(Value: Word);
begin
  if Value <> FPriority then begin
    FPriority := Value;
    if Active then begin
      DoDbiDropFilter;
      Activate;
    end
    else
      DoDbiDropFilter;
  end;
end;

{ TabcTableAdapter for BDE TTable }

constructor TabcTableAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FTable := TTable(ADataset);
end;

class function TabcTableAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TTable;
end;

function TabcTableAdapter.GetDescription: string;
begin
  if abc_UseTableComponentNames then
    Result := FTable.Name
  else
    Result := FTable.TableName;
end;

function TabcTableAdapter.IsEmptyTableEnabled: Boolean;
begin
  Result := FTable.Exclusive;
end;

function TabcTableAdapter.IsIndexEnabled: Boolean;
begin
  Result := True;
end;

function TabcTableAdapter.IsRangeEnabled: Boolean;
begin
  Result := True;
end;

function TabcTableAdapter.IsFindKeyEnabled: Boolean;
begin
  Result := True;
end;

function TabcTableAdapter.IsFindNearestEnabled: Boolean;
begin
  Result := True;
end;

function TabcTableAdapter.GetSearchMethod: TabcSearchMethod;
begin
  Result := asmFindKey;
end;

procedure TabcTableAdapter.EmptyTable;
begin
  FTable.EmptyTable;
end;

function TabcTableAdapter.GetIndexDefs: TIndexDefs;
begin
  Result := FTable.IndexDefs;
end;

procedure TabcTableAdapter.SetIndexDefs(const Value: TIndexDefs);
begin
  FTable.IndexDefs := Value;
end;

function TabcTableAdapter.GetIndexField(Index: Integer): TField;
begin
  Result := FTable.IndexFields[Index];
end;

procedure TabcTableAdapter.SetIndexField(Index: Integer; const Value: TField);
begin
  FTable.IndexFields[Index] := Value;
end;

procedure TabcTableAdapter.GetIndexNames(const Strings: TStrings);
begin
  FTable.GetIndexNames(Strings);
end;

function TabcTableAdapter.GetIndexName: string;
begin
  try
    Result := FTable.IndexName;
  except
    Result := '';
  end;
end;

procedure TabcTableAdapter.SetIndexName(const Value: string);
begin
  FTable.IndexName := Value;
end;

function TabcTableAdapter.GetIndexFieldNames: string;
begin
  Result := FTable.IndexFieldNames;
end;

procedure TabcTableAdapter.SetIndexFieldNames(const Value: string);
begin
  FTable.IndexFieldNames := Value;
end;

function TabcTableAdapter.GetIndexFieldCount: Integer;
begin
  Result := FTable.IndexFieldCount;
end;

function TabcTableAdapter.GetIndexValue: string;
var
  i: integer;
  FieldStr: string;
begin
  Result := '';
  with FTable do
    for I := 0 to IndexFieldCount - 1 do begin
      FieldStr := IndexFields[I].AsString;
      if Length(Result) + Length(FieldStr) > 255 then
        Break
      else
        Result := Result + FieldStr + sabcnpBmkSep;
    end;
  Result := Copy(Result, 0, Length(Result) - 1);
end;

procedure TabcTableAdapter.ApplyRange;
begin
  FTable.ApplyRange;
end;

procedure TabcTableAdapter.CancelRange;
begin
  FTable.CancelRange;
end;

procedure TabcTableAdapter.EditRangeStart;
begin
  FTable.EditRangeStart;
end;

procedure TabcTableAdapter.EditRangeEnd;
begin
  FTable.EditRangeEnd;
end;

procedure TabcTableAdapter.SetRange(const StartValues, EndValues: Variant);
var
  i: integer;
begin
  with FTable do
  begin
    SetRangeStart;
    if VarIsArray(StartValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := StartValues[i]
    else
      IndexFields[0].Value := StartValues;
    SetRangeEnd;
    if VarIsArray(EndValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := EndValues[i]
    else
      IndexFields[0].Value := EndValues;
    ApplyRange;
  end;
end;

procedure TabcTableAdapter.SetRangeStart;
begin
  FTable.SetRangeStart;
end;

procedure TabcTableAdapter.SetRangeEnd;
begin
  FTable.SetRangeEnd;
end;

function TabcTableAdapter.GetKeyExclusive: Boolean;
begin
  Result := FTable.KeyExclusive;
end;

procedure TabcTableAdapter.SetKeyExclusive(Value: Boolean);
begin
  FTable.KeyExclusive := Value;
end;

function TabcTableAdapter.FindKey(const KeyValues: Variant): Boolean;
var
  i: integer;
begin
  with FTable do
  begin
    SetKey;
    if VarIsArray(KeyValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := KeyValues[i]
    else
      IndexFields[0].Value := KeyValues;
    Result := GoToKey;
  end;
end;

procedure TabcTableAdapter.FindNearest(const KeyValues: Variant);
var
  i: integer;
begin
  with FTable do
  begin
    SetKey;
    if VarIsArray(KeyValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := KeyValues[i]
    else
      IndexFields[0].Value := KeyValues;
    GoToNearest;
  end;
end;

procedure TabcTableAdapter.EditKey;
begin
  FTable.EditKey;
end;

procedure TabcTableAdapter.GoToKey;
begin
  FTable.GoToKey;
end;

procedure TabcTableAdapter.GoToNearest;
begin
  FTable.GoToNearest;
end;

procedure TabcTableAdapter.SetKey;
begin
  FTable.SetKey;
end;

procedure TabcTableAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent: Integer;
begin
  inherited;
  Indent := 1;
  with FTable do begin
    AddStrIndent(sabcnpiTableProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiTBName + TableName, Indent, Strings);
    AddStrIndent(sabcnpiTBType + ABC_TABLETYPES[TableType], Indent, Strings);
    AddStrIndent(sabcnpiIxFieldCount + IntToStr(IndexFieldCount), Indent, Strings);
    if State in [dsInsert, dsEdit] then begin
      AddStrIndent(sabcnpiKeyExcl + ABC_BOOLEANS[KeyExclusive], Indent, Strings);
      AddStrIndent(sabcnpiKeyFieldCount + IntToStr(KeyFieldCount), Indent, Strings);
    end
    else begin
      AddStrIndent(sabcnpiNoKeyExcl, Indent, Strings);
      AddStrIndent(sabcnpiNoKeyFieldCount, Indent, Strings);
    end;
    AddStrIndent(sabcnpiExcl + ABC_BOOLEANS[Exclusive], Indent, Strings);
    AddStrIndent(sabcnpiIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent(sabcnpiIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiMasterFields + MasterFields, Indent, Strings);
    if MasterSource = nil then
      AddStrIndent(sabcnpiMasterSourceNil, Indent, Strings)
    else
      AddStrIndent(sabcnpiMasterSource + MasterSource.Name, Indent, Strings);
    AddStrIndent(sabcnpiReadOnly + ABC_BOOLEANS[ReadOnly], Indent, Strings);
    AddStrIndent(sabcnpiUpdateMode + ABC_UPDATEMODES[UpdateMode], Indent, Strings);
    AddStrIndent('',0,Strings);
  end;
end;

procedure TabcTableAdapter.GetIndexInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;
  OptionsStr: string;
begin
  Indent := 0;
  AddStrIndent(sabcnpiIndexes,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FTable do begin
    IndexDefs.Update;
    AddStrIndent(sabcnpiCurrentIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiCurrentIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent('',0,Strings);
    AddStrIndent(sabcnpiCurrentIndexDefs, Indent, Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    with IndexDefs do
      if Count > 0 then begin
        for I := 0 to Count - 1 do
          with Items[I] do begin
            AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
            AddStrIndent(sabcnpiExpression + Expression, Indent, Strings);
            AddStrIndent(sabcnpiFieldsEq + Fields, Indent, Strings);
            OptionsStr := '';
           if ixPrimary in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixPrimary;
            if ixUnique in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixUnique;
            if ixDescending in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixDescending;
            if ixCaseInsensitive in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixCaseInsensitive;
            if ixExpression in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixExpression;
            if Length(OptionsStr) > 0 then
              OptionsStr := sabcnpiLeftBr + Copy(OptionsStr, 3, Length(OptionsStr)) + sabcnpiRightBr;
            AddStrIndent(sabcnpiOptions + OptionsStr, Indent, Strings);
            AddStrIndent('',0,Strings);
          end;
      end
      else
        AddStrIndent(sabcnpiNoIndexDefs, Indent, Strings);
  end;
  AddStrIndent('',0,Strings);
end;

{ ReadOnly }

function TabcTableAdapter.IsReadOnlyEnabled: Boolean;
begin
  Result := True;
end;

function TabcTableAdapter.GetReadOnly: Boolean;
begin
  Result := FTable.ReadOnly;
end;

procedure TabcTableAdapter.SetReadOnly(Value: Boolean);
begin
  FTable.ReadOnly := Value;
end;

{ TabcQueryAdapter for BDE TQuery }

constructor TabcQueryAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FQuery := TQuery(ADataset);
end;

class function TabcQueryAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TQuery;
end;

function TabcQueryAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcQueryAdapter.GetParams: TParams;
begin
  Result := FQuery.Params;
end;

procedure TabcQueryAdapter.SetParams(Value: TParams);
begin
  FQuery.Params := Value;
end;

function TabcQueryAdapter.GetParamCount: Integer;
begin
  Result := FQuery.ParamCount;
end;

procedure TabcQueryAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent, I: Integer;
  TempStrings: TStringList;
begin
  inherited;
  Indent := 1;
  with FQuery do begin
    AddStrIndent(sabcnpiQueryProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiSQL, Indent, Strings);
    Inc(Indent);
    for I := 0 to SQL.Count - 1 do
      AddStrIndent(SQL.Strings[I], Indent, Strings);
    Dec(Indent);
    AddStrIndent('',0,Strings);

    AddStrIndent(sabcnpiSQLText, Indent, Strings);
    Inc(Indent);
    if Length(Text) > 0 then begin
      TempStrings := TStringList.Create;
      try
        TempStrings.Text := Text;
        for I := 0 to TempStrings.Count - 1 do
          AddStrIndent(TempStrings.Strings[I], Indent, Strings);
      finally
        TempStrings.Free;
      end;
    end
    else
      AddStrIndent(sabcnpiTextEmpty, Indent, Strings);
    Dec(Indent);
    AddStrIndent('',0,Strings);

    if DataSource = nil then
      AddStrIndent(sabcnpiDSNil, Indent, Strings)
    else
      AddStrIndent(sabcnpiDSEq + DataSource.Name, Indent, Strings);
    AddStrIndent(sabcnpiLocal + ABC_BOOLEANS[Local], Indent, Strings);
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    AddStrIndent(sabcnpiPrepared + ABC_BOOLEANS[Prepared], Indent, Strings);
    AddStrIndent(sabcnpiRequestLive + ABC_BOOLEANS[RequestLive], Indent, Strings);
    AddStrIndent(sabcnpiUniDirectional + ABC_BOOLEANS[UniDirectional], Indent, Strings);
    AddStrIndent(sabcnpiUpdateMode + ABC_UPDATEMODES[UpdateMode], Indent, Strings);
    AddStrIndent('',0,Strings);
  end;
end;

procedure TabcQueryAdapter.GetParamInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;

  procedure AddAParam(AParam: TParam);
  begin
    with AParam do begin
      AddStrIndent('',0,Strings);
      AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
      AddStrIndent(sabcnpiDataType + ABC_FIELDTYPES[DataType], Indent, Strings);
      AddStrIndent(sabcnpiBound + ABC_BOOLEANS[Bound], Indent, Strings);
      AddStrIndent(sabcnpiIsNull + ABC_BOOLEANS[IsNull], Indent, Strings);
      AddStrIndent(sabcnpiParamType + ABC_PARAMTYPES[ParamType], Indent, Strings);
      AddStrIndent(sabcnpiText + Text, Indent, Strings);
    end;
  end;
begin
  Indent := 0;
  AddStrIndent(sabcnpiParams,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FQuery do begin
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    Inc(Indent);
    for I := 0 to ParamCount - 1 do AddAParam(Params[I])
  end;
  AddStrIndent('',0,Strings);
end;

function TabcQueryAdapter.IsReadOnlyEnabled: Boolean;
begin
  Result := True;
end;

function TabcQueryAdapter.GetReadOnly: Boolean;
begin
  Result := not FQuery.CanModify;
end;

procedure TabcQueryAdapter.SetReadOnly(Value: Boolean);
//or could set cachedupdates true...
begin
  FQuery.RequestLive := Value;
end;

{ TabcStoredProcAdapter for BDE TStoredProc }

constructor TabcStoredProcAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FStoredProc := TStoredProc(ADataset);
end;

class function TabcStoredProcAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TStoredProc;
end;

function TabcStoredProcAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcStoredProcAdapter.GetParams: TParams;
begin
  Result := FStoredProc.Params;
end;

procedure TabcStoredProcAdapter.SetParams(Value: TParams);
begin
  FStoredProc.Params := Value;
end;

function TabcStoredProcAdapter.GetParamCount: Integer;
begin
  Result := FStoredProc.ParamCount;
end;

procedure TabcStoredProcAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent: Integer;
begin
  inherited;
  Indent := 1;
  with FStoredProc do begin
    AddStrIndent(sabcnpiSPProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiSPName + StoredProcName, Indent, Strings);
    AddStrIndent(sabcnpiDescAvail + ABC_BOOLEANS[DescriptionsAvailable], Indent, Strings);
    AddStrIndent(sabcnpiOverload + IntToStr(Overload), Indent, Strings);
    AddStrIndent(sabcnpiParamBindMode + ABC_PARAMBINDMODES[ParamBindMode], Indent, Strings);
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    AddStrIndent(sabcnpiPrepared + ABC_BOOLEANS[Prepared], Indent, Strings);
  end;
end;

procedure TabcStoredProcAdapter.GetParamInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;

  procedure AddAParam(AParam: TParam);
  begin
    with AParam do begin
      AddStrIndent('',0,Strings);
      AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
      AddStrIndent(sabcnpiDataType + ABC_FIELDTYPES[DataType], Indent, Strings);
      AddStrIndent(sabcnpiBound + ABC_BOOLEANS[Bound], Indent, Strings);
      AddStrIndent(sabcnpiIsNull + ABC_BOOLEANS[IsNull], Indent, Strings);
      AddStrIndent(sabcnpiParamType + ABC_PARAMTYPES[ParamType], Indent, Strings);
      AddStrIndent(sabcnpiText + Text, Indent, Strings);
    end;
  end;
begin
  Indent := 0;
  AddStrIndent(sabcnpiParams,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FStoredProc do begin
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    AddStrIndent(sabcnpiParamBindMode + ABC_PARAMBINDMODES[ParamBindMode], Indent, Strings);
    Inc(Indent);
    for I := 0 to ParamCount - 1 do AddAParam(Params.Items[I]);
  end;
  AddStrIndent('',0,Strings);
end;

initialization
  abcDSAdapterFactory.RegisterAdapter(TabcDBDatasetAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcStoredProcAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcQueryAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcTableAdapter);

finalization
  abcDSAdapterFactory.UnRegisterAdapter(TabcTableAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcQueryAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcStoredProcAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcDBDatasetAdapter);

end.
