unit abcdacli;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ Dataset Adapter for TClientDataset }

{$I abcdefs.inc}

interface

uses Classes, Db, DbClient, abcda
     {$IFDEF DELPHI7}, Variants {$ENDIF}
     ;

type

  { TabcClientAdapterReg }

  TabcClientAdapterReg = class(TabcDatasetAdapterReg);

  { TabcClientDatasetAdapter for TClientDataset }

  TabcClientDatasetAdapter = class(TabcDatasetAdapter)
  private
    FClientDataset: TClientDataset;
  protected
    { Indexes }
    function IsIndexEnabled: Boolean; override;
    function GetIndexDefs: TIndexDefs; override;
    procedure SetIndexDefs(const Value: TIndexDefs); override;
    function GetIndexField(Index: Integer): TField; override;
    procedure SetIndexField(Index: Integer; const Value: TField); override;
    function GetIndexName: string; override;
    procedure SetIndexName(const Value: string); override;
    function GetIndexFieldNames: string; override;
    procedure SetIndexFieldNames(const Value: string); override;
    function GetIndexFieldCount: Integer; override;
    function GetIndexValue: string; override;

    { Range Filters }
    function IsRangeEnabled: Boolean; override;
    function GetKeyExclusive: Boolean; override;
    procedure SetKeyExclusive(Value: Boolean); override;

    { Find }
    function IsFindKeyEnabled: Boolean; override;
    function IsFindNearestEnabled: Boolean; override;
    function GetSearchMethod: TabcSearchMethod; override;

    { Query Params }
    function IsParamsEnabled: Boolean; override;
    function GetParams: TParams; override;
    procedure SetParams(Value: TParams); override;
    function GetParamCount: Integer; override;

    {Table/Query ReadOnly}
    function IsReadOnlyEnabled: Boolean; override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(Value: Boolean); override;

    function GetFilterPartialMatch: TabcFilterPartialMatch; override;

  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;

    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetIndexInfo(const Strings: TStrings); override;
    procedure GetParamInfo(const Strings: TStrings); override;
    procedure GetDatabaseInfo(const Strings: TStrings); override;

    { Indexes }
    procedure GetIndexNames(const Strings: TStrings); override;

    { Range Filters }
    procedure ApplyRange; override;
    procedure CancelRange; override;
    procedure EditRangeStart; override;
    procedure EditRangeEnd; override;
    procedure SetRange(const StartValues, EndValues: Variant); override;
    procedure SetRangeStart; override;
    procedure SetRangeEnd; override;

    { Find Key }
    procedure EditKey; override;
    function FindKey(const KeyValues: Variant): Boolean; override;
    procedure FindNearest(const KeyValues: Variant); override;
    procedure GoToKey; override;
    procedure GoToNearest; override;
    procedure SetKey; override;
  end;

implementation

uses SysUtils, SysConst, abcconst, abcnpcon, abcfiltr, abcutilm;

resourcestring
  sabcnpiClientDatasetProps = '';

const
  ABC_UPDATEMODES: array[TUpdateMode] of string =
    (sabcnpiupWhereAll, sabcnpiupWhereChanged, sabcnpiupWhereKeyOnly);
  ABC_PARAMTYPES: array[TParamType] of string =
    (sabcnpiptUnknown, sabcnpiptInput, sabcnpiptOutput, sabcnpiptInputOutput,
     sabcnpiptResult);

 { TabcClientDatasetAdapter }

constructor TabcClientDatasetAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FClientDataset := TClientDataset(ADataset);
end;

class function TabcClientDatasetAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TClientDataset;
end;

function TabcClientDatasetAdapter.IsIndexEnabled: Boolean;
begin
  Result := True;
end;

function TabcClientDatasetAdapter.IsRangeEnabled: Boolean;
begin
  Result := True;
end;

function TabcClientDatasetAdapter.IsFindKeyEnabled: Boolean;
begin
  Result := True;
end;

function TabcClientDatasetAdapter.IsFindNearestEnabled: Boolean;
begin
  Result := True;
end;

function TabcClientDatasetAdapter.GetSearchMethod: TabcSearchMethod;
begin
  Result := asmFindKey;
end;

function TabcClientDatasetAdapter.GetIndexDefs: TIndexDefs;
begin
  Result := FClientDataset.IndexDefs;
end;

procedure TabcClientDatasetAdapter.SetIndexDefs(const Value: TIndexDefs);
begin
  FClientDataset.IndexDefs := Value;
end;

function TabcClientDatasetAdapter.GetIndexField(Index: Integer): TField;
begin
  Result := FClientDataset.IndexFields[Index];
end;

procedure TabcClientDatasetAdapter.SetIndexField(Index: Integer; const Value: TField);
begin
  FClientDataset.IndexFields[Index] := Value;
end;

procedure TabcClientDatasetAdapter.GetIndexNames(const Strings: TStrings);
begin
  FClientDataset.GetIndexNames(Strings);
end;

function TabcClientDatasetAdapter.GetIndexName: string;
begin
  Result := FClientDataset.IndexName;
end;

procedure TabcClientDatasetAdapter.SetIndexName(const Value: string);
begin
  FClientDataset.IndexName := Value;
end;

function TabcClientDatasetAdapter.GetIndexFieldNames: string;
begin
  Result := FClientDataset.IndexFieldNames;
end;

procedure TabcClientDatasetAdapter.SetIndexFieldNames(const Value: string);
begin
  FClientDataset.IndexFieldNames := Value;
end;

function TabcClientDatasetAdapter.GetIndexFieldCount: Integer;
begin
  Result := FClientDataset.IndexFieldCount;
end;

function TabcClientDatasetAdapter.GetIndexValue: string;
var
  i: integer;
  FieldStr: string;
begin
  Result := '';
  with FClientDataset do
    for I := 0 to IndexFieldCount - 1 do begin
      FieldStr := IndexFields[I].AsString;
      if Length(Result) + Length(FieldStr) > 255 then
        Break
      else
        Result := Result + FieldStr + sabcnpBmkSep;
    end;
  Result := Copy(Result, 0, Length(Result) - 1);
end;

procedure TabcClientDatasetAdapter.ApplyRange;
begin
  FClientDataset.ApplyRange;
end;

procedure TabcClientDatasetAdapter.CancelRange;
begin
  FClientDataset.CancelRange;
end;

procedure TabcClientDatasetAdapter.EditRangeStart;
begin
  FClientDataset.EditRangeStart;
end;

procedure TabcClientDatasetAdapter.EditRangeEnd;
begin
  FClientDataset.EditRangeEnd;
end;

procedure TabcClientDatasetAdapter.SetRange(const StartValues, EndValues: Variant);
var
  i: integer;
begin
  with FClientDataset do
  begin
    SetRangeStart;
    if VarIsArray(StartValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := StartValues[i]
    else
      IndexFields[0].Value := StartValues;
    SetRangeEnd;
    if VarIsArray(EndValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := EndValues[i]
    else
      IndexFields[0].Value := EndValues;
    ApplyRange;
  end;
end;

procedure TabcClientDatasetAdapter.SetRangeStart;
begin
  FClientDataset.SetRangeStart;
end;

procedure TabcClientDatasetAdapter.SetRangeEnd;
begin
  FClientDataset.SetRangeEnd;
end;

function TabcClientDatasetAdapter.GetKeyExclusive: Boolean;
begin
  Result := FClientDataset.KeyExclusive;
end;

procedure TabcClientDatasetAdapter.SetKeyExclusive(Value: Boolean);
begin
  FClientDataset.KeyExclusive := Value;
end;

function TabcClientDatasetAdapter.FindKey(const KeyValues: Variant): Boolean;
var
  i: integer;
begin
  with FClientDataset do
  begin
    SetKey;
    if VarIsArray(KeyValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := KeyValues[i]
    else
      IndexFields[0].Value := KeyValues;
    Result := GoToKey;
  end;
end;

procedure TabcClientDatasetAdapter.FindNearest(const KeyValues: Variant);
var
  i: integer;
begin
  with FClientDataset do
  begin
    SetKey;
    if VarIsArray(KeyValues) then
      for i := 0 to IndexFieldCount-1 do
        IndexFields[i].Value := KeyValues[i]
    else
      IndexFields[0].Value := KeyValues;
    GoToNearest;
  end;
end;

procedure TabcClientDatasetAdapter.EditKey;
begin
  FClientDataset.EditKey;
end;

procedure TabcClientDatasetAdapter.GoToKey;
begin
  FClientDataset.GoToKey;
end;

procedure TabcClientDatasetAdapter.GoToNearest;
begin
  FClientDataset.GoToNearest;
end;

procedure TabcClientDatasetAdapter.SetKey;
begin
  FClientDataset.SetKey;
end;

function TabcClientDatasetAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcClientDatasetAdapter.GetParams: TParams;
begin
  Result := FClientDataset.Params;
end;

procedure TabcClientDatasetAdapter.SetParams(Value: TParams);
begin
  FClientDataset.Params := Value;
end;

function TabcClientDatasetAdapter.GetParamCount: Integer;
begin
  Result := FClientDataset.Params.Count;
end;

procedure TabcClientDatasetAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent: Integer;
begin
  inherited;
  Indent := 1;
  with FClientDataset do begin
    AddStrIndent(sabcnpiClientDatasetProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiIxFieldCount + IntToStr(IndexFieldCount), Indent, Strings);
    if State in [dsInsert, dsEdit] then begin
      AddStrIndent(sabcnpiKeyExcl + ABC_BOOLEANS[KeyExclusive], Indent, Strings);
      AddStrIndent(sabcnpiKeyFieldCount + IntToStr(KeyFieldCount), Indent, Strings);
    end
    else begin
      AddStrIndent(sabcnpiNoKeyExcl, Indent, Strings);
      AddStrIndent(sabcnpiNoKeyFieldCount, Indent, Strings);
    end;
    AddStrIndent(sabcnpiIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent(sabcnpiIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiMasterFields + MasterFields, Indent, Strings);
    if MasterSource = nil then
      AddStrIndent(sabcnpiMasterSourceNil, Indent, Strings)
    else
      AddStrIndent(sabcnpiMasterSource + MasterSource.Name, Indent, Strings);
    AddStrIndent(sabcnpiReadOnly + ABC_BOOLEANS[ReadOnly], Indent, Strings);
    AddStrIndent('',0,Strings);
  end;
end;

procedure TabcClientDatasetAdapter.GetIndexInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;
  OptionsStr: string;
begin
  Indent := 0;
  AddStrIndent(sabcnpiIndexes,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FClientDataset do begin
    IndexDefs.Update;
    AddStrIndent(sabcnpiCurrentIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiCurrentIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent('',0,Strings);
    AddStrIndent(sabcnpiCurrentIndexDefs, Indent, Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    with IndexDefs do
      if Count > 0 then begin
        for I := 0 to Count - 1 do
          with Items[I] do begin
            AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
            AddStrIndent(sabcnpiExpression + Expression, Indent, Strings);
            AddStrIndent(sabcnpiFieldsEq + Fields, Indent, Strings);
            OptionsStr := '';
           if ixPrimary in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixPrimary;
            if ixUnique in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixUnique;
            if ixDescending in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixDescending;
            if ixCaseInsensitive in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixCaseInsensitive;
            if ixExpression in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixExpression;
            if Length(OptionsStr) > 0 then
              OptionsStr := sabcnpiLeftBr + Copy(OptionsStr, 3, Length(OptionsStr)) + sabcnpiRightBr;
            AddStrIndent(sabcnpiOptions + OptionsStr, Indent, Strings);
            AddStrIndent('',0,Strings);
          end;
      end
      else
        AddStrIndent(sabcnpiNoIndexDefs, Indent, Strings);
  end;
  AddStrIndent('',0,Strings);
end;

procedure TabcClientDatasetAdapter.GetParamInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;

  procedure AddAParam(AParam: TParam);
  begin
    with AParam do begin
      AddStrIndent('',0,Strings);
      AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
      AddStrIndent(sabcnpiDataType + ABC_FIELDTYPES[DataType], Indent, Strings);
      AddStrIndent(sabcnpiBound + ABC_BOOLEANS[Bound], Indent, Strings);
      AddStrIndent(sabcnpiIsNull + ABC_BOOLEANS[IsNull], Indent, Strings);
      AddStrIndent(sabcnpiParamType + ABC_PARAMTYPES[ParamType], Indent, Strings);
      AddStrIndent(sabcnpiText + Text, Indent, Strings);
    end;
  end;
begin
  Indent := 0;
  AddStrIndent(sabcnpiParams,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FClientDataset do begin
    AddStrIndent(sabcnpiParamCount + IntToStr(Params.Count), Indent, Strings);
    Inc(Indent);
    for I := 0 to Params.Count - 1 do AddAParam(Params[I])
  end;
  AddStrIndent('',0,Strings);
end;

procedure TabcClientDatasetAdapter.GetDatabaseInfo(const Strings: TStrings);
begin
end;

function TabcClientDatasetAdapter.IsReadOnlyEnabled: Boolean;
begin
  Result := True;
end;

function TabcClientDatasetAdapter.GetReadOnly: Boolean;
begin
  Result := FClientDataset.ReadOnly;
end;

procedure TabcClientDatasetAdapter.SetReadOnly(Value: Boolean);
begin
  FClientDataset.ReadOnly := Value;
end;

function TabcClientDatasetAdapter.GetFilterPartialMatch: TabcFilterPartialMatch;
begin
  Result := afpAsterisk;
end;

initialization
  abcDSAdapterFactory.RegisterAdapter(TabcClientDatasetAdapter);

finalization
  abcDSAdapterFactory.UnRegisterAdapter(TabcClientDatasetAdapter);

end.
