unit abcdadbx;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ Dataset Adapter classes for DBExpress }

{$I abcdefs.inc}

interface

uses Classes, Db, DBXpress, SqlExpr, abcda;

type

  { TabcSQLAdapterReg }

  TabcSQLAdapterReg = class(TabcDatasetAdapterReg);

  { TabcSQLCustomDatasetAdapter }

  TabcSQLCustomDatasetAdapter = class(TabcDatasetAdapter)
  private
    FCustomSQLDataset: TCustomSQLDataSet;
  protected
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
  end;

  { TabcSQLTableAdapter }

  TabcSQLTableAdapter = class(TabcSQLCustomDatasetAdapter)
  private
    FTable: TSQLTable;
  protected
    function GetDescription: string; override;
    { Indexes }
    function IsIndexEnabled: Boolean; override;
    function GetIndexDefs: TIndexDefs; override;
    procedure SetIndexDefs(const Value: TIndexDefs); override;
    function GetIndexField(Index: Integer): TField; override;
    procedure SetIndexField(Index: Integer; const Value: TField); override;
    function GetIndexName: string; override;
    procedure SetIndexName(const Value: string); override;
    function GetIndexFieldNames: string; override;
    procedure SetIndexFieldNames(const Value: string); override;
    function GetIndexFieldCount: Integer; override;
    function GetIndexValue: string; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetIndexInfo(const Strings: TStrings); override;
    { Indexes }
    procedure GetIndexNames(const Strings: TStrings); override;
  end;

  { TabcSQLQueryAdapter }

  TabcSQLQueryAdapter = class(TabcSQLCustomDatasetAdapter)
  private
    FQuery: TSQLQuery;
  protected
    { Query Params }
    function IsParamsEnabled: Boolean; override;
    function GetParams: TParams; override;
    procedure SetParams(Value: TParams); override;
    function GetParamCount: Integer; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetParamInfo(const Strings: TStrings); override;
  end;

  { TabcSQLStoredProcAdapter }

  TabcSQLStoredProcAdapter = class(TabcSQLCustomDatasetAdapter)
  private
    FStoredProc: TSQLStoredProc;
  protected
    { Params }
    function IsParamsEnabled: Boolean; override;
    function GetParams: TParams; override;
    procedure SetParams(Value: TParams); override;
    function GetParamCount: Integer; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetParamInfo(const Strings: TStrings); override;
  end;

  { TabcSQLDatasetAdapter }

  TabcSQLDatasetAdapter = class(TabcSQLCustomDatasetAdapter)
  private
    FSQLDataset: TSQLDataSet;
  protected
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
  end;

implementation

uses SysUtils, SysConst, abcconst, abcnpcon, abcutild, abcutilf, abcutilm;

const
  ABC_PARAMTYPES: array[TParamType] of string =
    (sabcnpiptUnknown, sabcnpiptInput, sabcnpiptOutput, sabcnpiptInputOutput,
     sabcnpiptResult);

{ TabcSQLCustomDatasetAdapter }

constructor TabcSQLCustomDatasetAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FCustomSQLDataset := TCustomSQLDataSet(ADataset);
end;

class function TabcSQLCustomDatasetAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TCustomSQLDataSet;
end;

{ TabcSQLTableAdapter for TSQLTable }

constructor TabcSQLTableAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FTable := TSQLTable(ADataset);
end;

class function TabcSQLTableAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TSQLTable;
end;

function TabcSQLTableAdapter.GetDescription: string;
begin
  if abc_UseTableComponentNames then
    Result := FTable.Name
  else
    Result := FTable.TableName;
end;

function TabcSQLTableAdapter.IsIndexEnabled: Boolean;
begin
  Result := True;
end;

function TabcSQLTableAdapter.GetIndexDefs: TIndexDefs;
begin
  Result := FTable.IndexDefs;
end;

procedure TabcSQLTableAdapter.SetIndexDefs(const Value: TIndexDefs);
begin
  FTable.IndexDefs := Value;
end;

function TabcSQLTableAdapter.GetIndexField(Index: Integer): TField;
begin
  Result := FTable.IndexFields[Index];
end;

procedure TabcSQLTableAdapter.SetIndexField(Index: Integer; const Value: TField);
begin
  FTable.IndexFields[Index] := Value;
end;

procedure TabcSQLTableAdapter.GetIndexNames(const Strings: TStrings);
begin
  FTable.GetIndexNames(Strings);
end;

function TabcSQLTableAdapter.GetIndexName: string;
begin
  Result := FTable.IndexName;
end;

procedure TabcSQLTableAdapter.SetIndexName(const Value: string);
var
  WasActive: Boolean;
begin
  with FTable do
  try
    DisableControls;
    WasActive := Active;
    Close;
    IndexName := Value;
    if WasActive then Open;
  finally
    EnableControls;
  end;
end;

function TabcSQLTableAdapter.GetIndexFieldNames: string;
begin
  Result := FTable.IndexFieldNames;
end;

procedure TabcSQLTableAdapter.SetIndexFieldNames(const Value: string);
begin
  FTable.IndexFieldNames := Value;
end;

function TabcSQLTableAdapter.GetIndexFieldCount: Integer;
begin
  Result := FTable.IndexFieldCount;
end;

function TabcSQLTableAdapter.GetIndexValue: string;
var
  i: integer;
  FieldStr: string;
begin
  Result := '';
  with FTable do
    for I := 0 to IndexFieldCount - 1 do begin
      FieldStr := IndexFields[I].AsString;
      if Length(Result) + Length(FieldStr) > 255 then
        Break
      else
        Result := Result + FieldStr + sabcnpBmkSep;
    end;
  Result := Copy(Result, 0, Length(Result) - 1);
end;

procedure TabcSQLTableAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent: Integer;
begin
  inherited;
  Indent := 1;
  with FTable do begin
    AddStrIndent(sabcnpiTableProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiTBName + TableName, Indent, Strings);
    AddStrIndent(sabcnpiIxFieldCount + IntToStr(IndexFieldCount), Indent, Strings);
    AddStrIndent(sabcnpiIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent(sabcnpiIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiReadOnly + ABC_BOOLEANS[ReadOnly], Indent, Strings);
    AddStrIndent('',0,Strings);
  end;
end;

procedure TabcSQLTableAdapter.GetIndexInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;
  OptionsStr: string;
begin
  Indent := 0;
  AddStrIndent(sabcnpiIndexes,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FTable do begin
    IndexDefs.Update;
    AddStrIndent(sabcnpiCurrentIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiCurrentIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent('',0,Strings);
    AddStrIndent(sabcnpiCurrentIndexDefs, Indent, Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    with IndexDefs do
      if Count > 0 then begin
        for I := 0 to Count - 1 do
          with Items[I] do begin
            AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
            AddStrIndent(sabcnpiExpression + Expression, Indent, Strings);
            AddStrIndent(sabcnpiFieldsEq + Fields, Indent, Strings);
            OptionsStr := '';
           if ixPrimary in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixPrimary;
            if ixUnique in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixUnique;
            if ixDescending in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixDescending;
            if ixCaseInsensitive in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixCaseInsensitive;
            if ixExpression in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixExpression;
            if Length(OptionsStr) > 0 then
              OptionsStr := sabcnpiLeftBr + Copy(OptionsStr, 3, Length(OptionsStr)) + sabcnpiRightBr;
            AddStrIndent(sabcnpiOptions + OptionsStr, Indent, Strings);
            AddStrIndent('',0,Strings);
          end;
      end
      else
        AddStrIndent(sabcnpiNoIndexDefs, Indent, Strings);
  end;
  AddStrIndent('',0,Strings);
end;

{ TabcSQLQueryAdapter for BDE TQuery }

constructor TabcSQLQueryAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FQuery := TSQLQuery(ADataset);
end;

class function TabcSQLQueryAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TSQLQuery;
end;

function TabcSQLQueryAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcSQLQueryAdapter.GetParams: TParams;
begin
  Result := FQuery.Params;
end;

procedure TabcSQLQueryAdapter.SetParams(Value: TParams);
begin
  FQuery.Params := Value;
end;

function TabcSQLQueryAdapter.GetParamCount: Integer;
begin
  Result := FQuery.Params.Count;
end;

procedure TabcSQLQueryAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent, I: Integer;
  TempStrings: TStringList;
begin
  inherited;
  Indent := 1;
  with FQuery do begin
    AddStrIndent(sabcnpiQueryProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiSQL, Indent, Strings);
    Inc(Indent);
    for I := 0 to SQL.Count - 1 do
      AddStrIndent(SQL.Strings[I], Indent, Strings);
    Dec(Indent);
    AddStrIndent('',0,Strings);

    AddStrIndent(sabcnpiSQLText, Indent, Strings);
    Inc(Indent);
    if Length(Text) > 0 then begin
      TempStrings := TStringList.Create;
      try
        TempStrings.Text := Text;
        for I := 0 to TempStrings.Count - 1 do
          AddStrIndent(TempStrings.Strings[I], Indent, Strings);
      finally
        TempStrings.Free;
      end;
    end
    else
      AddStrIndent(sabcnpiTextEmpty, Indent, Strings);
    Dec(Indent);
    AddStrIndent('',0,Strings);

    if DataSource = nil then
      AddStrIndent(sabcnpiDSNil, Indent, Strings)
    else
      AddStrIndent(sabcnpiDSEq + DataSource.Name, Indent, Strings);
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    AddStrIndent(sabcnpiPrepared + ABC_BOOLEANS[Prepared], Indent, Strings);
    AddStrIndent(sabcnpiUniDirectional + ABC_BOOLEANS[IsUniDirectional], Indent, Strings);
    AddStrIndent('',0,Strings);
  end;
end;

procedure TabcSQLQueryAdapter.GetParamInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;

  procedure AddAParam(AParam: TParam);
  begin
    with AParam do begin
      AddStrIndent('',0,Strings);
      AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
      AddStrIndent(sabcnpiDataType + ABC_FIELDTYPES[DataType], Indent, Strings);
      AddStrIndent(sabcnpiBound + ABC_BOOLEANS[Bound], Indent, Strings);
      AddStrIndent(sabcnpiIsNull + ABC_BOOLEANS[IsNull], Indent, Strings);
      AddStrIndent(sabcnpiParamType + ABC_PARAMTYPES[ParamType], Indent, Strings);
      AddStrIndent(sabcnpiText + Text, Indent, Strings);
    end;
  end;
begin
  Indent := 0;
  AddStrIndent(sabcnpiParams,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FQuery do begin
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    Inc(Indent);
    for I := 0 to ParamCount - 1 do AddAParam(Params[I])
  end;
  AddStrIndent('',0,Strings);
end;

{ TabcSQLStoredProcAdapter for BDE TStoredProc }

constructor TabcSQLStoredProcAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FStoredProc := TSQLStoredProc(ADataset);
end;

class function TabcSQLStoredProcAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TSQLStoredProc;
end;

function TabcSQLStoredProcAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcSQLStoredProcAdapter.GetParams: TParams;
begin
  Result := FStoredProc.Params;
end;

procedure TabcSQLStoredProcAdapter.SetParams(Value: TParams);
begin
  FStoredProc.Params := Value;
end;

function TabcSQLStoredProcAdapter.GetParamCount: Integer;
begin
  Result := FStoredProc.Params.Count;
end;

procedure TabcSQLStoredProcAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent: Integer;
begin
  inherited;
  Indent := 1;
  with FStoredProc do begin
    AddStrIndent(sabcnpiSPProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiSPName + StoredProcName, Indent, Strings);
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    AddStrIndent(sabcnpiPrepared + ABC_BOOLEANS[Prepared], Indent, Strings);
  end;
end;

procedure TabcSQLStoredProcAdapter.GetParamInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;

  procedure AddAParam(AParam: TParam);
  begin
    with AParam do begin
      AddStrIndent('',0,Strings);
      AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
      AddStrIndent(sabcnpiDataType + ABC_FIELDTYPES[DataType], Indent, Strings);
      AddStrIndent(sabcnpiBound + ABC_BOOLEANS[Bound], Indent, Strings);
      AddStrIndent(sabcnpiIsNull + ABC_BOOLEANS[IsNull], Indent, Strings);
      AddStrIndent(sabcnpiParamType + ABC_PARAMTYPES[ParamType], Indent, Strings);
      AddStrIndent(sabcnpiText + Text, Indent, Strings);
    end;
  end;
begin
  Indent := 0;
  AddStrIndent(sabcnpiParams,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FStoredProc do begin
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    Inc(Indent);
    for I := 0 to ParamCount - 1 do AddAParam(Params.Items[I]);
  end;
  AddStrIndent('',0,Strings);
end;

{ TabcSQLDatasetAdapter }

constructor TabcSQLDatasetAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FSQLDataset := TSQLDataSet(ADataset);
end;

class function TabcSQLDatasetAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TSQLDataSet;
end;

initialization
  abcDSAdapterFactory.RegisterAdapter(TabcSQLDatasetAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcSQLStoredProcAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcSQLQueryAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcSQLTableAdapter);

finalization
  abcDSAdapterFactory.UnRegisterAdapter(TabcSQLTableAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcSQLQueryAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcSQLStoredProcAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcSQLDatasetAdapter);

end.
