unit abcdaib;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ Dataset Adapter classes for Interbase }

{$I abcdefs.inc}

interface

uses Classes, Db, IBTable, IBQuery, IBStoredProc, IBCustomDataset, abcda;

type

  { TabcIBAdapterReg }

  TabcIBAdapterReg = class(TabcDatasetAdapterReg);

  { TabcIBCustomDatasetAdapter }

  TabcIBCustomDatasetAdapter = class(TabcDatasetAdapter)
  private
    FIBDataset: TIBCustomDataset;
  protected
    { SQL Transactions }
    function IsTransactionEnabled: Boolean; override;
    function IsStartTransactionEnabled: Boolean; override;
    function IsCommitEnabled: Boolean; override;
    function IsRollbackEnabled: Boolean; override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { SQL Transactions }
    procedure StartTransaction; override;
    procedure Commit; override;
    procedure Rollback; override;
  end;

  { TabcIBTableAdapter }

  TabcIBTableAdapter = class(TabcIBCustomDatasetAdapter)
  private
    FTable: TIBTable;
  protected
    function GetDescription: string; override;
    { Empty Table }
    function IsEmptyTableEnabled: Boolean; override;
    { Indexes }
    function IsIndexEnabled: Boolean; override;
    function GetIndexDefs: TIndexDefs; override;
    procedure SetIndexDefs(const Value: TIndexDefs); override;
    function GetIndexField(Index: Integer): TField; override;
    procedure SetIndexField(Index: Integer; const Value: TField); override;
    function GetIndexName: string; override;
    procedure SetIndexName(const Value: string); override;
    function GetIndexFieldNames: string; override;
    procedure SetIndexFieldNames(const Value: string); override;
    function GetIndexFieldCount: Integer; override;
    function GetIndexValue: string; override;
    { Find }
    function IsFindKeyEnabled: Boolean; override;
    function IsFindNearestEnabled: Boolean; override;
    { ReadOnly }
    function IsReadOnlyEnabled: Boolean; override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(Value: Boolean); override;
    { Filters }
    function IsFilterEnabled: Boolean; override;
    function GetFilterText: string; override;
    function GetFiltered: Boolean; override;
    function GetFilterGoToFirst: Boolean; override;
    function GetFilterOptions: TFilterOptions; override;
    function GetFilterPriority: Word; override;
    function GetOnActivateFilter: TNotifyEvent; override;
    function GetOnDeactivateFilter: TNotifyEvent; override;
    function GetOnFilterRecord: TFilterRecordEvent; override;
    procedure SetFilterText(Value: string); override;
    procedure SetFiltered(Value: Boolean); override;
    procedure SetFilterGoToFirst(Value: Boolean); override;
    procedure SetFilterOptions(const Value: TFilterOptions); override;
    procedure SetFilterPriority(const Value: Word); override;
    procedure SetOnActivateFilter(const Value: TNotifyEvent); override;
    procedure SetOnDeactivateFilter(const Value: TNotifyEvent); override;
    procedure SetOnFilterRecord(const Value: TFilterRecordEvent); override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetIndexInfo(const Strings: TStrings); override;
    { Empty Table }
    procedure EmptyTable; override;
    { Indexes }
    procedure GetIndexNames(const Strings: TStrings); override;
    { Find Key }
    function FindKey(const KeyValues: Variant): Boolean; override;
    procedure FindNearest(const KeyValues: Variant); override;
  end;

  { TabcIBQueryAdapter }

  TabcIBQueryAdapter = class(TabcIBCustomDatasetAdapter)
  private
    FQuery: TIBQuery;
  protected
    { Query Params }
    function IsParamsEnabled: Boolean; override;
    function GetParams: TParams; override;
    procedure SetParams(Value: TParams); override;
    function GetParamCount: Integer; override;
    {Query ReadOnly}
    function IsReadOnlyEnabled: Boolean; override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(Value: Boolean); override;
    { Filters - disabled }
    function IsFilterEnabled: Boolean; override;
    procedure SetFilterText(Value: string); override;
    procedure SetFiltered(Value: Boolean); override;
    procedure SetFilterOptions(const Value: TFilterOptions); override;
    procedure SetOnFilterRecord(const Value: TFilterRecordEvent); override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetParamInfo(const Strings: TStrings); override;
  end;

  { TabcIBStoredProcAdapter }

  TabcIBStoredProcAdapter = class(TabcIBCustomDatasetAdapter)
  private
    FStoredProc: TIBStoredProc;
  protected
    { Params }
    function IsParamsEnabled: Boolean; override;
    function GetParams: TParams; override;
    procedure SetParams(Value: TParams); override;
    function GetParamCount: Integer; override;
    { Filters - disabled }
    function IsFilterEnabled: Boolean; override;
    procedure SetFilterText(Value: string); override;
    procedure SetFiltered(Value: Boolean); override;
    procedure SetFilterOptions(const Value: TFilterOptions); override;
    procedure SetOnFilterRecord(const Value: TFilterRecordEvent); override;
  public
    constructor Create(const ADataset: TDataset); override;
    class function DatasetClass: TabcDatasetClass; override;
    { Debugging Information }
    procedure GetDatasetInfo(const Strings: TStrings); override;
    procedure GetParamInfo(const Strings: TStrings); override;
  end;

  { TabcIBDatasetAdapter }

  TabcIBDatasetAdapter = class(TabcIBCustomDatasetAdapter)
  protected
    { Filters - disabled }
    function IsFilterEnabled: Boolean; override;
    procedure SetFilterText(Value: string); override;
    procedure SetFiltered(Value: Boolean); override;
    procedure SetFilterOptions(const Value: TFilterOptions); override;
    procedure SetOnFilterRecord(const Value: TFilterRecordEvent); override;
  public
    class function DatasetClass: TabcDatasetClass; override;
  end;

implementation

uses SysUtils, SysConst, abcconst, abcnpcon, abcutild, abcutilf, abcutilm;

const
  ABC_PARAMTYPES: array[TParamType] of string =
    (sabcnpiptUnknown, sabcnpiptInput, sabcnpiptOutput, sabcnpiptInputOutput,
     sabcnpiptResult);

{ TabcIBCustomDatasetAdapter }

constructor TabcIBCustomDatasetAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FIBDataset := TIBCustomDataset(ADataset);
end;

class function TabcIBCustomDatasetAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TIBCustomDataset;
end;

function TabcIBCustomDatasetAdapter.IsTransactionEnabled: Boolean;
begin
  Result := (FIBDataset.Transaction <> nil);
end;

function TabcIBCustomDatasetAdapter.IsStartTransactionEnabled: Boolean;
begin
  Result := IsTransactionEnabled;
  if Result then
    Result := not FIBDataset.Transaction.InTransaction;
end;

function TabcIBCustomDatasetAdapter.IsCommitEnabled: Boolean;
begin
  Result := IsTransactionEnabled;
  if Result then
    Result := FIBDataset.Transaction.InTransaction;
end;

function TabcIBCustomDatasetAdapter.IsRollbackEnabled: Boolean;
begin
  Result := IsCommitEnabled;
end;

procedure TabcIBCustomDatasetAdapter.StartTransaction;
begin
  if CanStartTransaction then
    FIBDataset.Transaction.StartTransaction;
end;

procedure TabcIBCustomDatasetAdapter.Commit;
begin
  with FIBDataset do
    if CanCommit then
    begin
      Transaction.Commit;
      StartTransaction;
      Open;
    end;
end;

procedure TabcIBCustomDatasetAdapter.Rollback;
begin
  with FIBDataset do
    if CanRollback then
    begin
      Transaction.Rollback;
      StartTransaction;
      Open;
    end;
end;

{ TabcIBTableAdapter for TIBTable }

constructor TabcIBTableAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FTable := TIBTable(ADataset);
end;

class function TabcIBTableAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TIBTable;
end;

function TabcIBTableAdapter.GetDescription: string;
begin
  if abc_UseTableComponentNames then
    Result := FTable.Name
  else
    Result := FTable.TableName;
end;

function TabcIBTableAdapter.IsEmptyTableEnabled: Boolean;
begin
  Result := not FTable.ReadOnly;
end;

function TabcIBTableAdapter.IsIndexEnabled: Boolean;
begin
  Result := True;
end;

function TabcIBTableAdapter.IsFindKeyEnabled: Boolean;
begin
  Result := True;
end;

function TabcIBTableAdapter.IsFindNearestEnabled: Boolean;
begin
  Result := True;
end;

procedure TabcIBTableAdapter.EmptyTable;
begin
  FTable.EmptyTable;
end;

function TabcIBTableAdapter.GetIndexDefs: TIndexDefs;
begin
  Result := FTable.IndexDefs;
end;

procedure TabcIBTableAdapter.SetIndexDefs(const Value: TIndexDefs);
begin
  FTable.IndexDefs := Value;
end;

function TabcIBTableAdapter.GetIndexField(Index: Integer): TField;
begin
  Result := FTable.IndexFields[Index];
end;

procedure TabcIBTableAdapter.SetIndexField(Index: Integer; const Value: TField);
begin
  FTable.IndexFields[Index] := Value;
end;

procedure TabcIBTableAdapter.GetIndexNames(const Strings: TStrings);
begin
  FTable.GetIndexNames(Strings);
end;

function TabcIBTableAdapter.GetIndexName: string;
begin
  Result := FTable.IndexName;
end;

procedure TabcIBTableAdapter.SetIndexName(const Value: string);
var
  WasActive: Boolean;
begin
  FTable.IndexName := Value;
  Exit;
  with FTable do
  try
    DisableControls;
    WasActive := Active;
    Close;
    IndexName := Value;
    if WasActive then Open;
  finally
    EnableControls;
  end;
end;

function TabcIBTableAdapter.GetIndexFieldNames: string;
begin
  Result := FTable.IndexFieldNames;
end;

procedure TabcIBTableAdapter.SetIndexFieldNames(const Value: string);
begin
  FTable.IndexFieldNames := Value;
end;

function TabcIBTableAdapter.GetIndexFieldCount: Integer;
begin
  Result := FTable.IndexFieldCount;
end;

function TabcIBTableAdapter.GetIndexValue: string;
var
  i: integer;
  FieldStr: string;
begin
  Result := '';
  with FTable do
    for I := 0 to IndexFieldCount - 1 do begin
      FieldStr := IndexFields[I].AsString;
      if Length(Result) + Length(FieldStr) > 255 then
        Break
      else
        Result := Result + FieldStr + sabcnpBmkSep;
    end;
  Result := Copy(Result, 0, Length(Result) - 1);
end;

function TabcIBTableAdapter.FindKey(const KeyValues: Variant): Boolean;
begin
  Result := FTable.Locate(IndexFieldNames, KeyValues, []);
end;

procedure TabcIBTableAdapter.FindNearest(const KeyValues: Variant);
begin
  FTable.Locate(
    IndexFieldNames, KeyValues,
    [loCaseInsensitive, loPartialKey]);
end;

procedure TabcIBTableAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent: Integer;
begin
  inherited;
  Indent := 1;
  with FTable do begin
    AddStrIndent(sabcnpiTableProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiTBName + TableName, Indent, Strings);
    AddStrIndent(sabcnpiIxFieldCount + IntToStr(IndexFieldCount), Indent, Strings);
    AddStrIndent(sabcnpiIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent(sabcnpiIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiReadOnly + ABC_BOOLEANS[ReadOnly], Indent, Strings);
    AddStrIndent('',0,Strings);
  end;
end;

procedure TabcIBTableAdapter.GetIndexInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;
  OptionsStr: string;
begin
  Indent := 0;
  AddStrIndent(sabcnpiIndexes,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FTable do begin
    IndexDefs.Update;
    AddStrIndent(sabcnpiCurrentIndexName + IndexName, Indent, Strings);
    AddStrIndent(sabcnpiCurrentIndexFieldNames + IndexFieldNames, Indent, Strings);
    AddStrIndent('',0,Strings);
    AddStrIndent(sabcnpiCurrentIndexDefs, Indent, Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    with IndexDefs do
      if Count > 0 then begin
        for I := 0 to Count - 1 do
          with Items[I] do begin
            AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
            AddStrIndent(sabcnpiExpression + Expression, Indent, Strings);
            AddStrIndent(sabcnpiFieldsEq + Fields, Indent, Strings);
            OptionsStr := '';
           if ixPrimary in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixPrimary;
            if ixUnique in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixUnique;
            if ixDescending in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixDescending;
            if ixCaseInsensitive in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixCaseInsensitive;
            if ixExpression in Options then
              OptionsStr := OptionsStr + sabcnpiSep + sabcnpiixExpression;
            if Length(OptionsStr) > 0 then
              OptionsStr := sabcnpiLeftBr + Copy(OptionsStr, 3, Length(OptionsStr)) + sabcnpiRightBr;
            AddStrIndent(sabcnpiOptions + OptionsStr, Indent, Strings);
            AddStrIndent('',0,Strings);
          end;
      end
      else
        AddStrIndent(sabcnpiNoIndexDefs, Indent, Strings);
  end;
  AddStrIndent('',0,Strings);
end;

{ ReadOnly }

function TabcIBTableAdapter.IsReadOnlyEnabled: Boolean;
begin
  Result := True;
end;

function TabcIBTableAdapter.GetReadOnly: Boolean;
begin
  Result := FTable.ReadOnly;
end;

procedure TabcIBTableAdapter.SetReadOnly(Value: Boolean);
begin
  FTable.ReadOnly := Value;
end;

{ Filters }

function TabcIBTableAdapter.IsFilterEnabled: Boolean;
begin
  Result := True;
end;

function TabcIBTableAdapter.GetFiltered: Boolean;
begin
  Result := FTable.Filtered;
end;

function TabcIBTableAdapter.GetFilterGoToFirst: Boolean;
begin
  Result := False;
end;

function TabcIBTableAdapter.GetFilterOptions: TFilterOptions;
begin
  Result := [];
end;

function TabcIBTableAdapter.GetFilterPriority: Word;
begin
  Result := 0;
end;

function TabcIBTableAdapter.GetFilterText: string;
begin
  Result := FTable.Filter;
end;

function TabcIBTableAdapter.GetOnActivateFilter: TNotifyEvent;
begin
  Result := nil;
end;

function TabcIBTableAdapter.GetOnDeactivateFilter: TNotifyEvent;
begin
  Result := nil;
end;

function TabcIBTableAdapter.GetOnFilterRecord: TFilterRecordEvent;
begin
  Result := nil;
end;

procedure TabcIBTableAdapter.SetFiltered(Value: Boolean);
begin
  FTable.Filtered := Value;
  
(*
  with FTable do
  try
    DisableControls;
    Filtered := Value;
  finally
    EnableControls;
  end;
*)

end;

procedure TabcIBTableAdapter.SetFilterGoToFirst(Value: Boolean);
begin
end;

procedure TabcIBTableAdapter.SetFilterOptions(const Value: TFilterOptions);
begin
end;

procedure TabcIBTableAdapter.SetFilterPriority(const Value: Word);
begin
end;

procedure TabcIBTableAdapter.SetFilterText(Value: string);
var
  flt: string;
begin
  flt := Value;
  if flt <> '' then
  // Remove [] from field names as used in BDE filters...
  begin
    flt := abcReplaceAll('[', '', flt, True, False);
    flt := abcReplaceAll(']', '', flt, True, False);
  end;
  if flt <> FTable.Filter then
    FTable.Filter := flt;
end;

procedure TabcIBTableAdapter.SetOnActivateFilter(
  const Value: TNotifyEvent);
begin
end;

procedure TabcIBTableAdapter.SetOnDeactivateFilter(
  const Value: TNotifyEvent);
begin
end;

procedure TabcIBTableAdapter.SetOnFilterRecord(
  const Value: TFilterRecordEvent);
begin
end;

{ TabcIBQueryAdapter for BDE TQuery }

constructor TabcIBQueryAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FQuery := TIBQuery(ADataset);
end;

class function TabcIBQueryAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TIBQuery;
end;

function TabcIBQueryAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcIBQueryAdapter.GetParams: TParams;
begin
  Result := FQuery.Params;
end;

procedure TabcIBQueryAdapter.SetParams(Value: TParams);
begin
  FQuery.Params := Value;
end;

function TabcIBQueryAdapter.GetParamCount: Integer;
begin
  Result := FQuery.ParamCount;
end;

procedure TabcIBQueryAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent, I: Integer;
  TempStrings: TStringList;
begin
  inherited;
  Indent := 1;
  with FQuery do begin
    AddStrIndent(sabcnpiQueryProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiSQL, Indent, Strings);
    Inc(Indent);
    for I := 0 to SQL.Count - 1 do
      AddStrIndent(SQL.Strings[I], Indent, Strings);
    Dec(Indent);
    AddStrIndent('',0,Strings);

    AddStrIndent(sabcnpiSQLText, Indent, Strings);
    Inc(Indent);
    if Length(Text) > 0 then begin
      TempStrings := TStringList.Create;
      try
        TempStrings.Text := Text;
        for I := 0 to TempStrings.Count - 1 do
          AddStrIndent(TempStrings.Strings[I], Indent, Strings);
      finally
        TempStrings.Free;
      end;
    end
    else
      AddStrIndent(sabcnpiTextEmpty, Indent, Strings);
    Dec(Indent);
    AddStrIndent('',0,Strings);

    if DataSource = nil then
      AddStrIndent(sabcnpiDSNil, Indent, Strings)
    else
      AddStrIndent(sabcnpiDSEq + DataSource.Name, Indent, Strings);
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    AddStrIndent(sabcnpiPrepared + ABC_BOOLEANS[Prepared], Indent, Strings);
    AddStrIndent(sabcnpiUniDirectional + ABC_BOOLEANS[UniDirectional], Indent, Strings);
    AddStrIndent('',0,Strings);
  end;
end;

procedure TabcIBQueryAdapter.GetParamInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;

  procedure AddAParam(AParam: TParam);
  begin
    with AParam do begin
      AddStrIndent('',0,Strings);
      AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
      AddStrIndent(sabcnpiDataType + ABC_FIELDTYPES[DataType], Indent, Strings);
      AddStrIndent(sabcnpiBound + ABC_BOOLEANS[Bound], Indent, Strings);
      AddStrIndent(sabcnpiIsNull + ABC_BOOLEANS[IsNull], Indent, Strings);
      AddStrIndent(sabcnpiParamType + ABC_PARAMTYPES[ParamType], Indent, Strings);
      AddStrIndent(sabcnpiText + Text, Indent, Strings);
    end;
  end;
begin
  Indent := 0;
  AddStrIndent(sabcnpiParams,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FQuery do begin
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    Inc(Indent);
    for I := 0 to ParamCount - 1 do AddAParam(Params[I])
  end;
  AddStrIndent('',0,Strings);
end;

function TabcIBQueryAdapter.IsReadOnlyEnabled: Boolean;
begin
  Result := True;
end;

function TabcIBQueryAdapter.GetReadOnly: Boolean;
begin
  Result := FQuery.CanModify;
end;

procedure TabcIBQueryAdapter.SetReadOnly(Value: Boolean);
begin
//  FQuery.RequestLive := Value;
end;

function TabcIBQueryAdapter.IsFilterEnabled: Boolean;
begin
  Result := False;
end;

procedure TabcIBQueryAdapter.SetFiltered(Value: Boolean);
begin
end;

procedure TabcIBQueryAdapter.SetFilterOptions(const Value: TFilterOptions);
begin
end;

procedure TabcIBQueryAdapter.SetFilterText(Value: string);
begin
end;

procedure TabcIBQueryAdapter.SetOnFilterRecord(
  const Value: TFilterRecordEvent);
begin
end;

{ TabcIBStoredProcAdapter for BDE TStoredProc }

constructor TabcIBStoredProcAdapter.Create(const ADataset: TDataset);
begin
  inherited;
  FStoredProc := TIBStoredProc(ADataset);
end;

class function TabcIBStoredProcAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TIBStoredProc;
end;

function TabcIBStoredProcAdapter.IsParamsEnabled: Boolean;
begin
  Result := True;
end;

function TabcIBStoredProcAdapter.GetParams: TParams;
begin
  Result := FStoredProc.Params;
end;

procedure TabcIBStoredProcAdapter.SetParams(Value: TParams);
begin
  FStoredProc.Params := Value;
end;

function TabcIBStoredProcAdapter.GetParamCount: Integer;
begin
  Result := FStoredProc.ParamCount;
end;

procedure TabcIBStoredProcAdapter.GetDatasetInfo(const Strings: TStrings);
var
  Indent: Integer;
begin
  inherited;
  Indent := 1;
  with FStoredProc do begin
    AddStrIndent(sabcnpiSPProps,Indent,Strings);
    AddStrIndent('',0,Strings);
    Inc(Indent);
    AddStrIndent(sabcnpiSPName + StoredProcName, Indent, Strings);
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    AddStrIndent(sabcnpiPrepared + ABC_BOOLEANS[Prepared], Indent, Strings);
  end;
end;

procedure TabcIBStoredProcAdapter.GetParamInfo(const Strings: TStrings);
var
  Indent: Integer;
  I: Integer;

  procedure AddAParam(AParam: TParam);
  begin
    with AParam do begin
      AddStrIndent('',0,Strings);
      AddStrIndent(sabcnpiNameEq + Name, Indent, Strings);
      AddStrIndent(sabcnpiDataType + ABC_FIELDTYPES[DataType], Indent, Strings);
      AddStrIndent(sabcnpiBound + ABC_BOOLEANS[Bound], Indent, Strings);
      AddStrIndent(sabcnpiIsNull + ABC_BOOLEANS[IsNull], Indent, Strings);
      AddStrIndent(sabcnpiParamType + ABC_PARAMTYPES[ParamType], Indent, Strings);
      AddStrIndent(sabcnpiText + Text, Indent, Strings);
    end;
  end;
begin
  Indent := 0;
  AddStrIndent(sabcnpiParams,Indent,Strings);
  AddStrIndent('',0,Strings);
  Inc(Indent);
  with FStoredProc do begin
    AddStrIndent(sabcnpiParamCount + IntToStr(ParamCount), Indent, Strings);
    Inc(Indent);
    for I := 0 to ParamCount - 1 do AddAParam(Params.Items[I]);
  end;
  AddStrIndent('',0,Strings);
end;

function TabcIBStoredProcAdapter.IsFilterEnabled: Boolean;
begin
  Result := False;
end;

procedure TabcIBStoredProcAdapter.SetFiltered(Value: Boolean);
begin
end;

procedure TabcIBStoredProcAdapter.SetFilterOptions(const Value: TFilterOptions);
begin
end;

procedure TabcIBStoredProcAdapter.SetFilterText(Value: string);
begin
end;

procedure TabcIBStoredProcAdapter.SetOnFilterRecord(
  const Value: TFilterRecordEvent);
begin
end;

{ TabcIBDatasetAdapter }

class function TabcIBDatasetAdapter.DatasetClass: TabcDatasetClass;
begin
  Result := TIBDataset;
end;

function TabcIBDatasetAdapter.IsFilterEnabled: Boolean;
begin
  Result := False;
end;

procedure TabcIBDatasetAdapter.SetFiltered(Value: Boolean);
begin
end;

procedure TabcIBDatasetAdapter.SetFilterOptions(const Value: TFilterOptions);
begin
end;

procedure TabcIBDatasetAdapter.SetFilterText(Value: string);
begin
end;

procedure TabcIBDatasetAdapter.SetOnFilterRecord(
  const Value: TFilterRecordEvent);
begin
end;

initialization
  abcDSAdapterFactory.RegisterAdapter(TabcIBDatasetAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcIBStoredProcAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcIBQueryAdapter);
  abcDSAdapterFactory.RegisterAdapter(TabcIBTableAdapter);

finalization
  abcDSAdapterFactory.UnRegisterAdapter(TabcIBTableAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcIBQueryAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcIBStoredProcAdapter);
  abcDSAdapterFactory.UnRegisterAdapter(TabcIBDatasetAdapter);

end.
