unit abcdbacc;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)
{$I abcdefs.inc}

interface

{$IFDEF INFOPOWER}
uses
  SysUtils, Classes, Forms, DB, DBTables, BDE, Dialogs, wwTable, wwQuery, abcapp;
{$ELSE}
uses
  SysUtils, Classes, Forms, DB, DBTables, BDE, Dialogs, abcapp;
{$ENDIF}

type

  { TabcLocalTable }

{$IFDEF INFOPOWER}
  TabcLocalTable = class(TwwTable)
{$ELSE}
  TabcLocalTable = class(TTable)
{$ENDIF}
  private
    FLocalPath: Boolean;
    function GetDatabaseName: TFileName;
    function IsDatabaseNameStored: Boolean;
    procedure SetDatabaseName(Value: TFileName);
    procedure SetLocalPath(Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property DatabaseName: TFileName read GetDatabaseName write SetDatabaseName stored IsDatabaseNameStored;
    property LocalPath: Boolean read FLocalPath write SetLocalPath default True;
  end;

  { TabcLocalQuery }

{$IFDEF INFOPOWER}
  TabcLocalQuery = class(TwwQuery)
{$ELSE}
  TabcLocalQuery = class(TQuery)
{$ENDIF}
  private
    FLocalPath: Boolean;
    function GetDatabaseName: TFileName;
    function IsDatabaseNameStored: Boolean;
    procedure SetDatabaseName(Value: TFileName);
    procedure SetLocalPath(Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property DatabaseName: TFileName read GetDatabaseName write SetDatabaseName stored IsDatabaseNameStored;
    property LocalPath: Boolean read FLocalPath write SetLocalPath default True;
  end;

  { TabcLocalDatabase }

  TabcLocalDatabase = class(TDatabase)
  private
    FLocalPath: Boolean;
    procedure SetLocalPath(Value: Boolean);
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property LocalPath: Boolean read FLocalPath write SetLocalPath default True;
  end;

  { TabcDBIdleTime }

  TabcDBIdleTime = class(TComponent)
  private
    FOnIdle: TIdleEvent;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DoIdle(Sender: TObject; var Done: Boolean);
    procedure FlushBDE;
  published
    property OnIdle: TIdleEvent read FOnIdle write FOnIdle;
  end;

  TabcProjectNameFunction = function: string;

var
  abcProjectNameFunc: TabcProjectNameFunction;

implementation

{ TabcLocalTable }

constructor TabcLocalTable.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  LocalPath := True;
end;

function TabcLocalTable.GetDatabaseName: TFileName;
begin
  Result := inherited DatabaseName;
end;

function TabcLocalTable.IsDatabaseNameStored: Boolean;
begin
  Result := not FLocalPath;
end;

procedure TabcLocalTable.SetDatabaseName(Value: TFileName);
begin
  if Value <> DatabaseName then begin
    inherited DatabaseName := Value;
    FLocalPath := False;
  end;
end;

procedure TabcLocalTable.SetLocalPath(Value: Boolean);
var
  LocalDir: string;
begin
  if Value <> FLocalPath then begin
    FLocalPath := Value;
    if Value then begin
      if Assigned(abcProjectNameFunc) then
        LocalDir := ExtractFilePath(abcProjectNameFunc)
      else
        LocalDir := ExtractFilePath(Application.ExeName);
      System.Delete(LocalDir, Length(LocalDir), 1);
      inherited DatabaseName := LocalDir;
    end;
  end;
end;

{ TabcLocalQuery }

constructor TabcLocalQuery.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  LocalPath := True;
end;

function TabcLocalQuery.GetDatabaseName: TFileName;
begin
  Result := inherited DatabaseName;
end;

function TabcLocalQuery.IsDatabaseNameStored: Boolean;
begin
  Result := not FLocalPath;
end;

procedure TabcLocalQuery.SetDatabaseName(Value: TFileName);
begin
  if Value <> DatabaseName then begin
    inherited DatabaseName := Value;
    FLocalPath := False;
  end;
end;

procedure TabcLocalQuery.SetLocalPath(Value: Boolean);
var
  LocalDir: string;
begin
  if Value <> FLocalPath then begin
    FLocalPath := Value;
    if Value then begin
      if Assigned(abcProjectNameFunc) then
        LocalDir := ExtractFilePath(abcProjectNameFunc)
      else
        LocalDir := ExtractFilePath(Application.ExeName);
      System.Delete(LocalDir, Length(LocalDir), 1);
      inherited DatabaseName := LocalDir;
    end;
  end;
end;

{ TabcLocalDatabase }

constructor TabcLocalDatabase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  LocalPath := True;
end;

procedure TabcLocalDatabase.Loaded;
var
  LocalDir: string;
  I: integer;
  Found: Boolean;
begin
  if FLocalPath then begin
    if Assigned(abcProjectNameFunc) then
      LocalDir := ExtractFilePath(abcProjectNameFunc)
    else
      LocalDir := ExtractFilePath(Application.ExeName);
    Delete(LocalDir, Length(LocalDir), 1);
    {locate params entry}
    Found := False;
    with Params do
      for I := 0 to Count - 1 do
        if Copy(Strings[I],1,5) = 'Path=' then begin
          Strings[I] := 'Path=' + LocalDir;
          Found := True;
        end;
    if not Found then Params.Add('Path=' + LocalDir);
  end;
  inherited Loaded;
end;

procedure TabcLocalDatabase.SetLocalPath(Value: Boolean);
var
  LocalDir: string;
begin
  if Value <> FLocalPath then begin
    FLocalPath := Value;
    if Value then begin
      if Assigned(abcProjectNameFunc) then
        LocalDir := ExtractFilePath(abcProjectNameFunc)
      else
        LocalDir := ExtractFilePath(Application.ExeName);
      Delete(LocalDir, Length(LocalDir), 1);
      {Set directory in Params - assumes driver is STANDARD}
      DriverName := 'STANDARD';
      {Standard Driver for dBase or Paradox uses single Path Param}
      Params.Clear;
      Params.Add('Path=' + LocalDir);
    end;
  end;
end;

constructor TabcDBIdleTime.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in AOwner.ComponentState) then
    abcApplicationEvents.RegisterOnIdle(DoIdle);
end;

destructor TabcDBIdleTime.Destroy;
begin
  if not (csDesigning in ComponentState) then
    abcApplicationEvents.UnRegisterOnIdle(DoIdle);
  inherited Destroy;
end;

procedure TabcDBIdleTime.DoIdle(Sender: TObject; var Done: Boolean);
begin
  FlushBDE;
  if Assigned(FOnIdle) then FOnIdle(Sender, Done);
end;

procedure TabcDBIdleTime.FlushBDE;
{
  Modified for Delphi 3 - DbiUseIdleTime no longer supported,
  replaced with DbiSaveChanges on all dbdataset cursors
}
var
  Sess, Db, Ds: Integer;
begin
  for Sess := 0 to Sessions.Count - 1 do
    with Sessions.Sessions[Sess] do
      for Db := 0 to DatabaseCount - 1 do
        with Databases[Db] do
          for Ds := 0 to DataSetCount - 1 do
            with DataSets[Ds] do
              if (Handle <> nil) and CanModify then DbiSaveChanges(Handle);
end;

end.
