unit abcdbatl;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

NOTE:  How to extend this unit to add attached labels to other control classes.

 1. Your component class must inherit from TControl
 2. Add the unit containing your control class to the interface uses clause.
 3. Copy the interface and implementation of TabcLDBText, replacing references to TabcLDBText with your
    new class name.
 4. Modify the implementation of TabcDBSubLabel.SetAttachedTo to add your new class.
 5. Add a register procedure entry for your new class.

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Controls, Graphics, Menus, StdCtrls,
  DB, DBCtrls, DBLookup, abcdbaut, Dialogs;

type

  { TabcDBSubLabel }

  TabcDBSubLabel = class(TCustomLabel)
  private
    FAttachedTo: TControl;
    FCaptionAuto: Boolean;
    FEnabledChanging: Boolean;
    FLocked: Boolean;
    FMovingAttached: Boolean;
    FVisibleChanging: Boolean;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure SetCaptionAuto(Value: Boolean);
  protected
    procedure Click; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetName(const NewName: TComponentName); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetAttachedTo(Value: TControl); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    constructor CreateAttached(AOwner: TComponent; AControl: TControl);
    destructor Destroy; override;
    procedure AttachedMove(dX, dY: Integer);
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure SetCaptionToField;
    property AttachedTo: TControl read FAttachedTo;
  published
    {property Align;   NOT PUBLISHED }
    property Alignment;
    property AutoSize;
    property Caption;
    property CaptionAuto: Boolean read FCaptionAuto write SetCaptionAuto default True;
    property Color;
    property DragCursor;
    property DragMode;
    {property Enabled; NOT PUBLISHED, set by attached control}
    {property FocusControl; NOT PUBLISHED, set to attached control if it's a TWinControl}
    property Font;
    property Locked: Boolean read FLocked write FLocked;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    {property Visible; NOT PUBLISHED, set by attached control}
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { Database Controls }

  { TabcLDBText }

  TabcLDBText = class(TDBText)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBEdit }

  TabcLDBEdit = class(TDBEdit)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBAutoEdit }

  TabcLDBAutoEdit = class(TabcDBAutoEdit)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBMemo }

  TabcLDBMemo = class(TDBMemo)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBImage }

  TabcLDBImage = class(TDBImage)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBListBox }

  TabcLDBListBox = class(TDBListBox)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBComboBox }

  TabcLDBComboBox = class(TDBComboBox)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBLookupList }

  TabcLDBLookupList = class(TDBLookupList)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
    function GetField: TField;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Field: TField read GetField;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBLookupCombo }

  TabcLDBLookupCombo = class(TDBLookupCombo)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
    function GetField: TField;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Field: TField read GetField;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBLookupListBox }

  TabcLDBLookupListBox = class(TDBLookupListBox)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Field: TField read GetField;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBLookupComboBox }

  TabcLDBLookupComboBox = class(TDBLookupComboBox)
  private
    FSubLabel: TabcDBSubLabel;
    FSaveParent: TWinControl;
    FLabelDeleted: Boolean;
    procedure ReadLabelDeleted(Reader: TReader);
    procedure WriteLabelDeleted(Writer: TWriter);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSubLabel(Value: TabcDBSubLabel); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Field: TField read GetField;
    property SubLabel: TabcDBSubLabel read FSubLabel;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { Following two controls set their own caption from the datafield }

  { TabcLDBCheckBox }

  TabcLDBCheckBox = class(TDBCheckBox)
  private
    FCaptionAuto: Boolean;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetCaptionAuto(Value: Boolean);
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetCaptionToField;
  published
    property CaptionAuto: Boolean read FCaptionAuto write SetCaptionAuto default True;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  { TabcLDBRadioGroup }

  TabcLDBRadioGroup = class(TDBRadioGroup)
  private
    FCaptionAuto: Boolean;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetCaptionAuto(Value: Boolean);
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetCaptionToField;
  published
    property CaptionAuto: Boolean read FCaptionAuto write SetCaptionAuto default True;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

implementation

uses abcconst, abcutilf;

const
  CREATELABELX = -100;  {default X offset of label relative to attached control}
  CREATELABELY = 2;     {default Y offset of label relative to attached control}

(* for testing new Delphi Version

procedure DebugComponentState(AComponent: TComponent; ACaption: string);
var
  s: string;
begin
  s := '';
  if csLoading in AComponent.ComponentState then
    s := s + 'csLoading, ';
  if csReading in AComponent.ComponentState then
    s := s + 'csReading, ';
  if csWriting in AComponent.ComponentState then
    s := s + 'csWriting, ';
  if csDestroying in AComponent.ComponentState then
    s := s + 'csDestroying, ';
  if csDesigning in AComponent.ComponentState then
    s := s + 'csDesigning, ';
  if csAncestor in AComponent.ComponentState then
    s := s + 'csAncestor, ';
  if csUpdating in AComponent.ComponentState then
    s := s + 'csUpdating, ';
  if csFixups in AComponent.ComponentState then
    s := s + 'csFixups, ';
  if csFreeNotification in AComponent.ComponentState then
    s := s + 'csFreeNotification, ';
  if csInline in AComponent.ComponentState then
    s := s + 'csInline, ';
  ShowMessage(ACaption + ' ComponentState: ' + #13#10 + s);
end;

*)

{ TabcDBSubLabel }

constructor TabcDBSubLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCaptionAuto := True;
end;

constructor TabcDBSubLabel.CreateAttached(AOwner: TComponent; AControl: TControl);
begin
  Create(AOwner);
  FAttachedTo := AControl;
  if (AControl is TWinControl) then FocusControl := TWinControl(AControl);
  SetBounds(AControl.Left + CREATELABELX,
            AControl.Top + CREATELABELY,
            Width,
            Height);
end;

destructor TabcDBSubLabel.Destroy;
begin
  if [csDesigning, csAncestor] * ComponentState = [csDesigning] then
    if (FAttachedTo <> nil) and
       ([csDestroying, csLoading] * FAttachedTo.ComponentState = []) then
      if (MessageDlg('This label is attached to ' + FAttachedTo.Name + '.' +
          #13#10 +
          #13#10 +
          'Are you sure you want to permanently delete it?',
          mtWarning,
          [mbYes, mbNo],
          0) = mrNo) then Abort;
  FAttachedTo := nil;
  inherited Destroy;
end;

procedure TabcDBSubLabel.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = FAttachedTo) and
     not (csDestroying in ComponentState) then
    FAttachedTo := nil;
end;

procedure TabcDBSubLabel.Loaded;
begin
  inherited Loaded;
  if FAttachedTo <> nil then
  begin
    if Enabled <> FAttachedTo.Enabled then begin
      FEnabledChanging := True;
      Enabled := FAttachedTo.Enabled;
    end;
    if Visible <> FAttachedTo.Visible then begin
      FVisibleChanging := True;
      Visible := FAttachedTo.Visible;
    end;
  end;
end;

procedure TabcDBSubLabel.SetAttachedTo(Value: TControl);
begin
  if Value <> FAttachedTo then
  begin
    if Value = nil then
      Free
    else
    begin
      if (Value is TabcLDBText) then
        TabcLDBText(Value).SetSubLabel(Self)
      else if (Value is TabcLDBEdit) then
        TabcLDBEdit(Value).SetSubLabel(Self)
      else if (Value is TabcLDBAutoEdit) then
        TabcLDBAutoEdit(Value).SetSubLabel(Self)
      else if (Value is TabcLDBMemo) then
        TabcLDBMemo(Value).SetSubLabel(Self)
      else if (Value is TabcLDBImage) then
        TabcLDBImage(Value).SetSubLabel(Self)
      else if (Value is TabcLDBListBox) then
        TabcLDBListBox(Value).SetSubLabel(Self)
      else if (Value is TabcLDBComboBox) then
        TabcLDBComboBox(Value).SetSubLabel(Self)
      else if (Value is TabcLDBLookupList) then
        TabcLDBLookupList(Value).SetSubLabel(Self)
      else if (Value is TabcLDBLookupCombo) then
        TabcLDBLookupCombo(Value).SetSubLabel(Self)
      else if (Value is TabcLDBLookupListBox) then
        TabcLDBLookupListBox(Value).SetSubLabel(Self)
      else if (Value is TabcLDBLookupComboBox) then
        TabcLDBLookupComboBox(Value).SetSubLabel(Self)
      {
      add any new classes here, e.g.
      else if (Value is TNewClass) then
        TNewClass(Value).SetSubLabel(Self)
      }
      else
        Exit;
      FAttachedTo := Value;
      Parent := FAttachedTo.Parent;
      Enabled := FAttachedTo.Enabled;
      Visible := FAttachedTo.Visible;
      if FAttachedTo is TWinControl then
        FocusControl := TWinControl(FAttachedTo);
    end;
  end;
end;

procedure TabcDBSubLabel.AttachedMove(dX, dY: Integer);
begin
  if (Parent <> nil) and abcDescendsFromClassByName(Parent, 'TabcCustomToolbar') then Exit;
  if [csAncestor, csDesigning, csLoading] * ComponentState = [csAncestor, csDesigning, csLoading] then
    Exit;
  FMovingAttached := True;
  SetBounds(Left + dX, Top + dY, Width, Height);
  FMovingAttached := False;
end;

procedure TabcDBSubLabel.Click;
begin
  if FAttachedTo is TWinControl then
    TWinControl(FAttachedTo).SetFocus;
  inherited Click;
end;

procedure TabcDBSubLabel.CMEnabledChanged(var Message: TMessage);
begin
  if FEnabledChanging then begin
    inherited;
    FEnabledChanging := False;
  end
  else begin
    FEnabledChanging := True;
    if FAttachedto <> nil then Enabled := FAttachedTo.Enabled;
  end;
end;

procedure TabcDBSubLabel.CMVisibleChanged(var Message: TMessage);
begin
  if FVisibleChanging then begin
    inherited;
    FVisibleChanging := False;
  end
  else begin
    FVisibleChanging := True;
    if FAttachedTo <> nil then Visible := FAttachedTo.Visible;
  end;
end;

procedure TabcDBSubLabel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if FLocked and not FMovingAttached and (FAttachedTo <> nil) then begin
      with FAttachedTo do SetBounds(
        Left + ALeft - Self.Left,
        Top + ATop - Self.Top,
        Width,
        Height);
      inherited SetBounds(Left, Top, AWidth, AHeight);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcDBSubLabel.SetCaptionAuto(Value: Boolean);
begin
  FCaptionAuto := Value;
  if Value then SetCaptionToField;
end;

procedure TabcDBSubLabel.SetCaptionToField;
var
  Field: TField;
begin
//  Field := nil;
  if FCaptionAuto and (FAttachedTo <> nil) then
  begin
    if (FAttachedTo is TabcLDBText) then
      Field := TabcLDBText(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBEdit) then
      Field := TabcLDBEdit(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBAutoEdit) then
      Field := TabcLDBAutoEdit(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBMemo) then
      Field := TabcLDBMemo(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBImage) then
      Field := TabcLDBImage(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBListBox) then
      Field := TabcLDBListBox(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBComboBox) then
      Field := TabcLDBComboBox(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBLookupList) then
      Field := TabcLDBLookupList(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBLookupCombo) then
      Field := TabcLDBLookupCombo(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBLookupListBox) then
      Field := TabcLDBLookupListBox(FAttachedTo).Field
    else if (FAttachedTo is TabcLDBLookupComboBox) then
      Field := TabcLDBLookupComboBox(FAttachedTo).Field
    {
    add any new classes here, e.g.
    else if (FAttachedTo is TNewClass) then
      Field := TNewClass(FAttachedTo).Field
    }
    else
      Exit;
    if Field <> nil then
      Caption := Field.DisplayName;
  end;
end;

procedure TabcDBSubLabel.SetName(const NewName: TComponentName);
begin
  if [csAncestor, csDesigning] * ComponentState = [csDesigning] then
    if (FAttachedTo <> nil) then
      if (NewName <> FAttachedTo.Name + 'SubLabel') then
        raise Exception.Create('Name cannot be changed');
  inherited SetName(NewName);
end;

procedure TabcDBSubLabel.SetParent(AParent: TWinControl);
begin
  if (AParent <> nil) and (FAttachedTo <> nil) and
    (AParent <> FAttachedTo.Parent) then
    Abort
  else
    inherited;
end;

{ Database Controls }

{ TabcLDBText }

constructor TabcLDBText.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
  begin
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
    FSubLabel.AttachedMove(0,-CREATELABELY);  {standard offset doesn't apply here}
  end;
end;

destructor TabcLDBText.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBText.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBText.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBText.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBText.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBText.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBText.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBText.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBText.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBText.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBText.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBText.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBText.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBText.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBText.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBText.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBEdit }

constructor TabcLDBEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBEdit.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBEdit.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBEdit.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBEdit.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBEdit.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBEdit.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBEdit.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBEdit.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBEdit.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBEdit.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBEdit.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBEdit.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBEdit.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBEdit.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then
    FSubLabel.SetCaptionToField;
end;

procedure TabcLDBEdit.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBAutoEdit }

constructor TabcLDBAutoEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBAutoEdit.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBAutoEdit.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBAutoEdit.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBAutoEdit.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBAutoEdit.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBAutoEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBAutoEdit.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBAutoEdit.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBAutoEdit.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBAutoEdit.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBAutoEdit.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBAutoEdit.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBAutoEdit.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBAutoEdit.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBAutoEdit.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBAutoEdit.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBMemo }

constructor TabcLDBMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBMemo.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBMemo.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBMemo.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBMemo.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBMemo.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBMemo.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBMemo.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBMemo.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBMemo.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBMemo.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBMemo.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBMemo.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBMemo.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBMemo.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBMemo.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBMemo.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBImage }

constructor TabcLDBImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBImage.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBImage.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBImage.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBImage.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBImage.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBImage.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBImage.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBImage.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBImage.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBImage.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBImage.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBImage.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBImage.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBImage.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBImage.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBImage.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBListBox }

constructor TabcLDBListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBListBox.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBListBox.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBListBox.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBListBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBListBox.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBListBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBListBox.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBListBox.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBListBox.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBListBox.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBListBox.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBListBox.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBListBox.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBListBox.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBListBox.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBListBox.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBComboBox }

constructor TabcLDBComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBComboBox.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBComboBox.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBComboBox.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBComboBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBComboBox.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBComboBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBComboBox.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBComboBox.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBComboBox.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBComboBox.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBComboBox.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBComboBox.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBComboBox.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBComboBox.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBComboBox.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBComboBox.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBLookupList }

constructor TabcLDBLookupList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBLookupList.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBLookupList.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBLookupList.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBLookupList.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBLookupList.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBLookupList.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBLookupList.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBLookupList.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBLookupList.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBLookupList.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBLookupList.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBLookupList.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBLookupList.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBLookupList.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

function TabcLDBLookupList.GetField: TField;
var
  I: integer;
begin
  Result := nil;
  if (DataField <> '') and (DataSource <> nil) then
    if DataSource.DataSet <> nil then
      with DataSource.DataSet do
        for I := 0 to FieldCount - 1 do
          if Fields[I].FieldName = DataField then begin
            Result := Fields[I];
            Break;
          end;
end;

procedure TabcLDBLookupList.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBLookupList.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBLookupCombo }

constructor TabcLDBLookupCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBLookupCombo.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBLookupCombo.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBLookupCombo.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBLookupCombo.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBLookupCombo.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBLookupCombo.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBLookupCombo.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBLookupCombo.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBLookupCombo.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBLookupCombo.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBLookupCombo.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBLookupCombo.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBLookupCombo.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBLookupCombo.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

function TabcLDBLookupCombo.GetField: TField;
var
  I: integer;
begin
  Result := nil;
  if (DataField <> '') and (DataSource <> nil) then
    if DataSource.DataSet <> nil then
      with DataSource.DataSet do
        for I := 0 to FieldCount - 1 do
          if Fields[I].FieldName = DataField then begin
            Result := Fields[I];
            Break;
          end;
end;

procedure TabcLDBLookupCombo.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBLookupCombo.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBLookupListBox }

constructor TabcLDBLookupListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBLookupListBox.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBLookupListBox.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBLookupListBox.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBLookupListBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBLookupListBox.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBLookupListBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBLookupListBox.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBLookupListBox.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBLookupListBox.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBLookupListBox.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBLookupListBox.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBLookupListBox.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBLookupListBox.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBLookupListBox.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

function TabcLDBLookupListBox.GetField: TField;
var
  I: integer;
begin
  Result := nil;
  if (DataField <> '') and (DataSource <> nil) then
    if DataSource.DataSet <> nil then
      with DataSource.DataSet do
        for I := 0 to FieldCount - 1 do
          if Fields[I].FieldName = DataField then begin
            Result := Fields[I];
            Break;
          end;
end;

procedure TabcLDBLookupListBox.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBLookupListBox.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBLookupComboBox }

constructor TabcLDBLookupComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if [csReading, csUpdating] * AOwner.ComponentState = [] then  {i.e. neither value in set}
    FSubLabel := TabcDBSubLabel.CreateAttached(AOwner, Self);
end;

destructor TabcLDBLookupComboBox.Destroy;
begin
  if [csAncestor] * ComponentState = [] then
    if (FSaveParent <> nil) and not (csDestroying in FSaveParent.ComponentState) then
      begin
        Destroying;
        if FSubLabel <> nil then
        begin
          FSubLabel.Destroying;
          FSubLabel.Free;
        end;
      end;
  FSubLabel := nil;
  inherited Destroy;
end;

procedure TabcLDBLookupComboBox.SetName(const NewName: TComponentName);
var
  LabelName: string;
  Found: TComponent;
begin
  inherited SetName(NewName);
  LabelName := Name + 'SubLabel';
  Found := Owner.FindComponent(LabelName);
  if (Found is TabcDBSubLabel) then
  begin
    FSubLabel.Free;
    SetSubLabel(TabcDBSubLabel(Found));
  end
  else
    if FSubLabel <> nil then FSubLabel.Name := LabelName;
end;

procedure TabcLDBLookupComboBox.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
     (AComponent = SubLabel) and
     not (csDestroying in ComponentState) then
    FSubLabel := nil;
end;

procedure TabcLDBLookupComboBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_ENABLEDCHANGED, 0, 0);
end;

procedure TabcLDBLookupComboBox.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (FSubLabel <> nil) then FSubLabel.Perform(CM_VISIBLECHANGED, 0, 0);
end;

procedure TabcLDBLookupComboBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldLeft, OldTop: Integer;
begin
  if (FSubLabel <> nil) then
  begin
    OldLeft := Left;
    OldTop := Top;
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    if [csLoading, csUpdating] * Owner.ComponentState = [] then
      FSubLabel.AttachedMove(ALeft - OldLeft, ATop - OldTop);
  end
  else
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcLDBLookupComboBox.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (ComponentState * [csDesigning, csDestroying] = [csDesigning]) and (FSubLabel <> nil) then
    FSubLabel.Parent := AParent;
  FSaveParent := AParent;
end;

procedure TabcLDBLookupComboBox.SetSubLabel(Value: TabcDBSubLabel);
begin
  if Value <> FSubLabel then
  begin
    if FSubLabel <> nil then
    begin
      FSubLabel.Destroying;
      FSubLabel.Free;
    end;
    FSubLabel := Value;
  end;
end;

procedure TabcLDBLookupComboBox.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty(
    'LabelDeleted',
    ReadLabelDeleted,
    WriteLabelDeleted,
    ((Filer.Ancestor=nil) and (FSubLabel = nil))
  );
end;

procedure TabcLDBLookupComboBox.ReadLabelDeleted(Reader: TReader);
begin
  FLabelDeleted := Reader.ReadBoolean;
end;

procedure TabcLDBLookupComboBox.WriteLabelDeleted(Writer: TWriter);
begin
  Writer.WriteBoolean(True);
end;

procedure TabcLDBLookupComboBox.Loaded;
var
  FindLabel: TComponent;
begin
  inherited Loaded;
  if FLabelDeleted then
    FSubLabel.Free
  else
  begin
    {fixup reference to control}
    FindLabel := Owner.FindComponent(Name + 'SubLabel');
    if (FindLabel is TabcDBSubLabel) then
    begin
      if (TabcDBSubLabel(FindLabel).FAttachedTo = nil) or
         (TabcDBSubLabel(FindLabel).FAttachedTo = self) then
        TabcDBSubLabel(FindLabel).SetAttachedTo(self);
    end
    else
      FSubLabel := nil;
    if (FSubLabel = nil) then FSubLabel := TabcDBSubLabel.CreateAttached(Owner, Self);
    FSubLabel.SetAttachedTo(Self);
  end;
end;

function TabcLDBLookupComboBox.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBLookupComboBox.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

function TabcLDBLookupComboBox.GetField: TField;
var
  I: integer;
begin
  Result := nil;
  if (DataField <> '') and (DataSource <> nil) then
    if DataSource.DataSet <> nil then
      with DataSource.DataSet do
        for I := 0 to FieldCount - 1 do
          if Fields[I].FieldName = DataField then begin
            Result := Fields[I];
            Break;
          end;
end;

procedure TabcLDBLookupComboBox.SetDataField(Value: string);
begin
  inherited DataField := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

procedure TabcLDBLookupComboBox.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  if (FSubLabel <> nil) then FSubLabel.SetCaptionToField;
end;

{ TabcLDBCheckBox }

constructor TabcLDBCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCaptionAuto := True;
end;

function TabcLDBCheckBox.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBCheckBox.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBCheckBox.SetCaptionAuto(Value: Boolean);
begin
  FCaptionAuto := Value;
  if Value then SetCaptionToField;
end;

procedure TabcLDBCheckBox.SetCaptionToField;
begin
  if FCaptionAuto then
    if Field <> nil then
      Caption := Field.DisplayName;
end;

procedure TabcLDBCheckBox.SetDataField(Value: string);
begin
  inherited DataField := Value;
  SetCaptionToField;
end;

procedure TabcLDBCheckBox.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  SetCaptionToField;
end;

{ TabcLDBRadioGroup }

constructor TabcLDBRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCaptionAuto := True;
end;

function TabcLDBRadioGroup.GetDataField: string;
begin
  Result := inherited DataField;
end;

function TabcLDBRadioGroup.GetDataSource: TDataSource;
begin
  Result := inherited DataSource;
end;

procedure TabcLDBRadioGroup.SetCaptionAuto(Value: Boolean);
begin
  FCaptionAuto := Value;
  if Value then SetCaptionToField;
end;

procedure TabcLDBRadioGroup.SetCaptionToField;
begin
  if FCaptionAuto then
    if Field <> nil then
      Caption := Field.DisplayName;
end;

procedure TabcLDBRadioGroup.SetDataField(Value: string);
begin
  inherited DataField := Value;
  SetCaptionToField;
end;

procedure TabcLDBRadioGroup.SetDataSource(Value: TDataSource);
begin
  inherited DataSource := Value;
  SetCaptionToField;
end;

initialization
{$IFDEF DELPHI6}
  ActivateClassGroup(TControl);
{$ENDIF}
  RegisterClass(TabcDBSubLabel);

finalization
  UnRegisterClass(TabcDBSubLabel);

end.
