unit abcdbaut;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Classes, Messages, Controls, DB, DBCtrls, Forms, abcforms;

type

  TEditFieldOption = (efMemo, efGraphic, efBlob);
  TabcEditFieldOptions = set of TEditFieldOption;

  TabcEditBlobOption = (ebCustom, ebMemo, ebGraphic{, ebRTF});

  TabcCustomFieldEditEvent = procedure (Sender: TObject; Owner: TComponent;
    Dataset: TDataset; Field: TField; PopupFormStyle: TabcPopupFormStyle;
    ReadOnly: Boolean; HelpContext: THelpContext; var Form: TForm) of object;

  { TabcCustomDBAutoEdit }

  TabcCustomDBAutoEdit = class(TDBEdit)
  private
    FOnCustomBlobEdit: TabcCustomFieldEditEvent;
    FPopupForm: TForm;
    FPopupAutoEdit: TabcEditFieldOptions;
    FPopupEditBlobAs: TabcEditBlobOption;
    FPopupFormStyle: TabcPopupFormStyle;
    function GetPopupForm: TForm;
    procedure CMLostFocus(var Message: TCMLostFocus); message CM_LOSTFOCUS;
  protected
    procedure DblClick; override;
    property PopupForm: TForm read GetPopupForm;
    property PopupAutoEdit: TabcEditFieldOptions read FPopupAutoEdit
      write FPopupAutoEdit default [efMemo, efGraphic];
    property PopupEditBlobAs: TabcEditBlobOption read FPopupEditBlobAs
      write FPopupEditBlobAs default ebCustom;
    property OnCustomBlobEdit: TabcCustomFieldEditEvent read FOnCustomBlobEdit write FOnCustomBlobEdit;
    property PopupFormStyle: TabcPopupFormStyle read FPopupFormStyle write FPopupFormStyle default pfsNormal;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  { TabcDBAutoEdit }

  TabcDBAutoEdit = class(TabcCustomDBAutoEdit)
  published
    property PopupAutoEdit;
    property PopupEditBlobAs;
    property PopupFormStyle;
    property OnCustomBlobEdit;
  end;

implementation

uses abcfdbme, abcfdbim;

{ TabcCustomDBAutoEdit }

constructor TabcCustomDBAutoEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPopupAutoEdit := [efMemo, efGraphic];
  FPopupFormStyle := pfsNormal;
end;

procedure TabcCustomDBAutoEdit.CMLostFocus(var Message: TCMLostFocus);
begin
  inherited;
  Invalidate;
end;

procedure TabcCustomDBAutoEdit.DblClick;
begin
  if PopupForm <> nil then
    FPopupForm.Show
  else if (Field <> nil) then
    case Field.DataType of
      ftUnknown..ftVarBytes:
        Exit;
      ftBlob:
        if efBlob in FPopupAutoEdit then
          case FPopupEditBlobAs of
            ebMemo:
              FPopupForm := ExecabcafMemoFieldEdit(Self, DataSource.DataSet,
                Field, PopupFormStyle, ReadOnly, HelpContext);
            ebGraphic:
              FPopupForm := ExecabcafGraphicFieldEdit(Self, DataSource.DataSet,
                Field, PopupFormStyle, ReadOnly, HelpContext);
            ebCustom:
              if Assigned(FOnCustomBlobEdit) then
                FOnCustomBlobEdit(Self, Self, DataSource.DataSet,
                  Field, PopupFormStyle, ReadOnly, HelpContext, FPopupForm);
//            ebRTF:
              //!;
          end;
      ftMemo:
        if efMemo in FPopupAutoEdit then
          FPopupForm := ExecabcafMemoFieldEdit(Self, DataSource.DataSet,
            Field, PopupFormStyle, ReadOnly, HelpContext);
      ftGraphic:
        if efGraphic in FPopupAutoEdit then
          FPopupForm := ExecabcafGraphicFieldEdit(Self, DataSource.DataSet,
            Field, PopupFormStyle, ReadOnly, HelpContext);
    end;
  inherited DblClick;
end;

function TabcCustomDBAutoEdit.GetPopupForm: TForm;
var
  I: Integer;
begin
  Result := nil;
  if FPopupForm <> nil then begin
    for I := 0 to ComponentCount - 1 do
      if Components[I] = FPopupForm then begin
        Result := FPopupForm;
        Exit;
      end;
    FPopupForm := nil;
  end;
end;

end.
