unit abcdbctl;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, DB, DBCtrls, Mask, Buttons, abcexctl, Menus, abcda;

type

  TabcSuperNavButton = class;
  TabcSuperNavDataLink = class;
  TabcGlyphColor = (gcBlack, gcColored);
  TSuperNavBtn = (snbFirst, snbMoveBack, snbPrior, snbNext, snbMoveFwd, snbLast,
                  snbAppend, snbBookmark, snbInsert, snbDelete, snbEdit, snbPost,
                  snbCancel, snbRefresh);
  TabcSuperNavBtnSet = set of TSuperNavBtn;

  TabcSuperNavClick = procedure (Sender: TObject; Button: TSuperNavBtn) of object;

  TabcNavSpeed = (nsFast, nsMedium, nsSlow);

{ TabcDBSuperNav }

  TabcDBSuperNav = class (TCustomPanel)
  private
    FDataLink: TabcSuperNavDataLink;
    FVisibleButtons: TabcSuperNavBtnSet;
    FHints: TStrings;
    ButtonWidth: Integer;
    MinBtnSize: TPoint;
    FBeforeAction: TabcSuperNavClick;
    FOnNavClick: TabcSuperNavClick;
    FocusedButton: TSuperNavBtn;
    FConfirmDelete: Boolean;
    FNavSpeed: TabcNavSpeed;
    FMoveBy: Word;
    FGlyphColor: TabcGlyphColor;
    FFlat: Boolean;
    procedure SetFlat(Value: Boolean);
    procedure SetNavSpeed(Value: TabcNavSpeed);
    procedure SetGlyphColor(Value: TabcGlyphColor);
    procedure LoadGlyphs;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure InitButtons;
    procedure InitHints;
    procedure ClickButton(Sender: TObject);
    procedure BookmarkClick(Sender: TObject);
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SetVisible(Value: TabcSuperNavBtnSet);
    procedure AdjustNavSize (var W: Integer; var H: Integer);
    procedure SetHints(Value: TStrings);
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    Buttons: array[TSuperNavBtn] of TabcSuperNavButton;
    procedure DataChanged;
    procedure EditingChanged;
    procedure ActiveChanged;
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure BtnClick(Index: TSuperNavBtn); virtual;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property VisibleButtons: TabcSuperNavBtnSet read FVisibleButtons write SetVisible
      default [snbFirst, snbMoveBack, snbPrior, snbNext, snbMoveFwd, snbLast,
               snbAppend, snbBookmark, snbInsert, snbDelete, snbEdit, snbPost,
               snbCancel, snbRefresh];
    property Align;
    property Anchors;
    property Constraints;
    property DragKind;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Ctl3D;
    property Hints: TStrings read FHints write SetHints;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete default True;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    {New Properties}
    property NavSpeed: TabcNavSpeed read FNavSpeed write SetNavSpeed default nsMedium;
    property MoveBy: Word read FMoveBy write FMoveBy default 50;
    property GlyphColor: TabcGlyphColor read FGlyphColor write SetGlyphColor default gcColored;

    property BeforeAction: TabcSuperNavClick read FBeforeAction write FBeforeAction;
    property OnClick: TabcSuperNavClick read FOnNavClick write FOnNavClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnStartDrag;
    property Flat: Boolean read FFlat write SetFlat default False;
    property OnEndDock;
    property OnStartDock;
  end;

{ TabcSuperNavButton }

  TabcSuperNavButton = class(TSpeedButton)
  private
    FIndex: TSuperNavBtn;
    FNavStyle: TNavButtonStyle;
    FRepeatTimer: TTimer;
    {new fields}
    FInitRepeatPause: integer;
    FRepeatPause: integer;
    FBookmark: TBookmark;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    destructor Destroy; override;
    property NavStyle: TNavButtonStyle read FNavStyle write FNavStyle;
    property Index : TSuperNavBtn read FIndex write FIndex;
  end;

{ TabcSuperNavDataLink }

  TabcSuperNavDataLink = class(TDataLink)
  private
    FNavigator: TabcDBSuperNav;
  protected
    procedure EditingChanged; override;
    procedure DataSetChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create(ANav: TabcDBSuperNav);
    destructor Destroy; override;
  end;

  TabcRecCountDataLink = class;

{ TabcDBRecordCount }

  TabcDBRecordCount = class(TCustomLabel)
  private
    FCurrentRecord, FRecordCount: Longint;
    FDataLink: TabcRecCountDataLink;
    FDataSet: TDataSet;
    FDisabledCaption: string;
    FMask: string;
    procedure SetDataset(Value: TDataset);
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetMask(Value: string);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CalcRecNumCount(var ARecNum, ARecCount: Integer);
    function CalcCaptionFor(const ARecNum, ARecCount: Integer): string;
  protected
    procedure ActiveChanged;
    procedure DataChange;
    procedure RefreshRecordCount;
    procedure RefreshCaption;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Loaded; override;
    function GetLabelText: string; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DisabledCaption: string read FDisabledCaption write FDisabledCaption;
    property Mask: string read FMask write SetMask;
    {from CustomLabel}
    property Align;
    property Alignment;
    property Anchors;
    property Constraints;
    property DragKind;
    property BiDiMode;
    property ParentBiDiMode;
    property AutoSize;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Transparent;
    property ShowHint;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

{ TabcRecCountDataLink }

  TabcRecCountDataLink = class(TDataLink)
  private
    FRecordCount: TabcDBRecordCount;
  protected
    procedure EditingChanged; override;
    procedure DataSetChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create(ARecordCount: TabcDBRecordCount);
    destructor Destroy; override;
  end;

  TabcDBStateDataLink = class;

  TabcDBState = class(TCustomLabel)
  private
    FDataLink: TabcDBStateDataLink;
    FDataSet: TDataSet;
    FAdapter: TabcDatasetAdapter;
    FDisabledCaption: string;
    FMask: string;
    FStates: TStrings;
    procedure SetDataset(Value: TDataset);
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetMask(Value: string);
    procedure SetStates(Value: TStrings);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure ActiveChanged;
    procedure DataChange;
    procedure RefreshState;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DisabledCaption: string read FDisabledCaption write FDisabledCaption;
    property Mask: string read FMask write SetMask;
    property States: TStrings read FStates write SetStates;
    {from CustomLabel}
    property Align;
    property Alignment;
    property Anchors;
    property Constraints;
    property DragKind;
    property BiDiMode;
    property ParentBiDiMode;
    property AutoSize;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Transparent;
    property ShowHint;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

{ TabcDBStateDataLink }

  TabcDBStateDataLink = class(TDataLink)
  private
    FState: TabcDBState;
  protected
    procedure EditingChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create(AState: TabcDBState);
    destructor Destroy; override;
  end;

{ TabcDBEditLabel }

  TabcDBEditLabel = class(TWinControl)
  private
    { sub components }
    fLabel: TLabel;
    fDBEdit: TDBEdit;
    { property values }
    FCaptionAuto: Boolean;
    fSaveLabelWidth: integer;
    FLabelOffsetX: Integer;
    FLabelOffsetY: Integer;
    FCalcLayout: Boolean;
    FSizing: Boolean;
    FLabelFixed: Boolean;
    fLayout: TabcEditLabelLayout;
    fLabelWidth: integer;
    fLabelHeight: integer;
    fOnChange: TNotifyEvent;
    { property access methods }
    function GetField: TField;
    function  GetDataField: string;
    procedure SetDataField(Value: string);
    function  GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);

    function GetMasked: Boolean;
    function GetEditText: string;
    procedure SetEditText(Value: string);
    function GetText: string;
    procedure SetText(Value: string);

    function  GetCursor: TCursor;
    procedure SetCursor(Value: TCursor);
    function  GetDragCursor: TCursor;
    procedure SetDragCursor(Value: TCursor);
    function  GetHint: string;
    procedure SetHint(Value: string);
    function  GetParentShowHint: Boolean;
    procedure SetParentShowHint(Value: Boolean);
    function  GetEditPopupMenu: TPopupMenu;
    procedure SetPopupMenu(Value: TPopupMenu);
    function  GetShowHint: Boolean;
    procedure SetShowHint(Value: Boolean);

    procedure SetLabelOffsetX(Value: Integer);
    procedure SetLabelOffsetY(Value: Integer);
    procedure setLabelWidth(setTo: integer);
    procedure setLabelHeight(setTo: integer);
    function  getHelpContext: THelpContext;
    procedure setHelpContext(setTo: THelpContext);
    function  getLabelFont: TFont;
    procedure setLabelFont(setTo: TFont);
    function  getEditFont: TFont;
    procedure setEditFont(setTo: TFont);
    function  getLabelParentFont: boolean;
    procedure setLabelParentFont(setTo: boolean);
    function  getEditParentFont: boolean;
    procedure setEditParentFont(setTo: boolean);
    function  getAlignment: TAlignment;
    procedure setAlignment(setTo: TAlignment);
    function  getCaption: TCaption;
    procedure setCaption(Value: TCaption);
    procedure setCaptionAuto(Value: Boolean);
    function  getShowAccelChar: boolean;
    procedure setShowAccelChar(setTo: boolean);
    function  getTabStop: boolean;
    procedure setTabStop(setTo: boolean);
    function  getAutoSelect: boolean;
    procedure setAutoSelect(setTo: boolean);
    function  getBorderStyle: TBorderStyle;
    procedure setBorderStyle(setTo: TBorderStyle);
    function  getCharCase: TEditCharCase;
    procedure setCharCase(setTo: TEditCharCase);
    function  getEditColor: TColor;
    procedure setEditColor(setTo: TColor);
    function  getEditParentColor: boolean;
    procedure setEditParentColor(setTo: boolean);
    function  getCtl3D: boolean;
    procedure setCtl3D(setTo: boolean);
    function  getParentCtl3D: boolean;
    procedure setParentCtl3D(setTo: boolean);
    function  getMaxLength: integer;
    procedure setMaxLength(setTo: integer);
    function  getPasswordChar: char;
    procedure setPasswordChar(setTo: char);
    function  getReadOnly: boolean;
    procedure setReadOnly(setTo: boolean);
    function  getModified: Boolean;
    procedure setModified(setTo: Boolean);
    function  getSelLength: Integer;
    procedure setSelLength(setTo: Integer);
    function  getSelStart: Integer;
    procedure setSelStart(setTo: Integer);
    function  getSelText: string;
    procedure setSelText(setTo: string);
    { event property access methods }
    function  getOnChange: TNotifyEvent;
    procedure setOnChange(setTo: TNotifyEvent);
    function  getOnClick: TNotifyEvent;
    procedure setOnClick(setTo: TNotifyEvent);
    function  getOnDblClick: TNotifyEvent;
    procedure setOnDblClick(setTo: TNotifyEvent);
    function  getOnEnter: TNotifyEvent;
    procedure setOnEnter(setTo: TNotifyEvent);
    function  getOnExit: TNotifyEvent;
    procedure setOnExit(setTo: TNotifyEvent);
    function  getOnKeyDown: TKeyEvent;
    procedure setOnKeyDown(setTo: TKeyEvent);
    function  getOnKeyPress: TKeyPressEvent;
    procedure setOnKeyPress(setTo: TKeyPressEvent);
    function  getOnKeyUp: TKeyEvent;
    procedure setOnKeyUp(setTo: TKeyEvent);
    function  getOnMouseDown: TMouseEvent;
    procedure setOnMouseDown(setTo: TMouseEvent);
    function  getOnMouseMove: TMouseMoveEvent;
    procedure setOnMouseMove(setTo: TMouseMoveEvent);
    function  getOnMouseUp: TMouseEvent;
    procedure setOnMouseUp(setTo: TMouseEvent);
    function  GetOnDragDrop: TDragDropEvent;
    procedure SetOnDragDrop(Value: TDragDropEvent);
    function  GetOnDragOver: TDragOverEvent;
    procedure SetOnDragOver(Value: TDragOverEvent);
    function  GetOnEndDrag: TEndDragEvent;
    procedure SetOnEndDrag(Value: TEndDragEvent);
    function  GetOnStartDrag: TStartDragEvent;
    procedure SetOnStartDrag(Value: TStartDragEvent);
    { internal support methods }
    procedure initLayout;
    procedure textChanged(Sender: TObject);
    procedure labelClicked(Sender: TObject);
    function  isLabelFontStored: boolean;
    function  isEditFontStored: boolean;
    function  isEditColorStored: boolean;
    function  isCtl3DStored: boolean;
    function  isShowHintStored: boolean;
    { message handling methods }
    procedure WMSize(var Message: TMessage); message WM_Size;
    procedure WMNCHitTest(var Message: TMessage); message WM_NCHitTest;
    procedure CMTextChanged(var Message: TMessage); message CM_TextChanged;
    procedure ActiveChange(Sender: TObject);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure SetDragMode(Value: TDragMode); override;
    function  GetPopupMenu: TPopupMenu; override;
    procedure ChangeScale(M, D: Integer); override;
    procedure paintWindow(DC: HDC); override;
    procedure setLayout(setTo: TabcEditLabelLayout); virtual;
    procedure DoEnter; override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Loaded; override;
    procedure SetFocus; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    function UseRightToLeftAlignment: Boolean; override;
    procedure Clear;
    procedure ClearSelection;
    procedure CopyToClipboard;
    procedure CutToClipboard;
    function Focused: Boolean; override;
    procedure PasteFromClipboard;
    function  GetSelTextBuf(Buffer: PChar; BufSize: Integer): Integer;
    procedure SelectAll;
    procedure SetSelTextBuf(Buffer: PChar);
    property Modified: Boolean read GetModified write SetModified;
    property SelLength: Integer read GetSelLength write SetSelLength;
    property SelStart: Integer read GetSelStart write SetSelStart;
    property SelText: string read GetSelText write SetSelText;
    property Field: TField read GetField;
    procedure ValidateEdit;
    function GetTextLen: Integer;
    property IsMasked: Boolean read GetMasked;
    property EditText: string read GetEditText write SetEditText;
    property Text: string read GetText write SetText;
    procedure SetCaptionToField;
  published
    { label properties }
    property CaptionAuto: Boolean read FCaptionAuto write SetCaptionAuto; {NB Must be loaded before caption}
    property Caption: TCaption read getCaption write setCaption;
    property Alignment: TAlignment read getAlignment write setAlignment default taLeftJustify;
    property ShowAccelChar: boolean read getShowAccelChar write setShowAccelChar default True;
    property LabelFont: TFont read getLabelFont write setLabelFont stored isLabelFontStored;
    property LabelParentFont: boolean read getLabelParentFont write setLabelParentFont default True;
    { edit properties }
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property HelpContext: THelpContext read getHelpContext write setHelpContext default 0;
    property Enabled;
    property TabStop: boolean read getTabStop write setTabStop default True;
    property EditFont: TFont read getEditFont write setEditFont stored isEditFontStored;
    property EditParentFont: boolean read getEditParentFont write setEditParentFont default True;
    property AutoSelect: boolean read getAutoSelect write setAutoSelect default True;
    property BorderStyle: TBorderStyle read getBorderStyle write setBorderStyle default bsSingle;
    property CharCase: TEditCharCase read getCharCase write setCharCase default ecNormal;
    property EditColor: TColor read getEditColor write setEditColor stored isEditColorStored;
    property EditParentColor: boolean read getEditParentColor write setEditParentColor default False;
    property Ctl3D: boolean read getCtl3D write setCtl3D stored isCtl3DStored;
    property ParentCtl3D: boolean read getParentCtl3D write setParentCtl3D default True;
    property MaxLength: integer read getMaxLength write setMaxLength default 0;
    property PasswordChar: char read getPasswordChar write setPasswordChar default #0;
    property ReadOnly: boolean read getReadOnly write setReadOnly default false;
    { common properties }
    property LabelOffsetX: Integer read FLabelOffsetX write SetLabelOffsetX;
    property LabelOffsetY: Integer read FLabelOffsetY write SetLabelOffsetY;
    property LabelWidth: integer read fLabelWidth write setLabelWidth;
    property LabelHeight: integer read fLabelHeight write setLabelHeight;
    property Layout: TabcEditLabelLayout read fLayout write setLayout default elLeft;
    { shared properties }
    property Cursor: TCursor read GetCursor write SetCursor;
    property DragCursor: TCursor read GetDragCursor write SetDragCursor;
    property DragMode;
    property Hint: string read GetHint write SetHint;
    property ParentShowHint: Boolean read GetParentShowHint write SetParentShowHint;
    property PopupMenu: TPopupMenu read GetEditPopupMenu write SetPopupMenu;
    property ShowHint: Boolean read GetShowHint write SetShowHint stored isShowHintStored;

    { event properties }
    property OnChange: TNotifyEvent read getOnChange write setOnChange;
    property OnClick: TNotifyEvent read getOnClick write setOnClick;
    property OnEnter: TNotifyEvent read getOnEnter write setOnEnter;
    property OnExit: TNotifyEvent read getOnExit write setOnExit;
    property OnKeyDown: TKeyEvent read getOnKeyDown write setOnKeyDown;
    property OnKeyPress: TKeyPressEvent read getOnKeyPress write setOnKeyPress;
    property OnKeyUp: TKeyEvent read getOnKeyUp write setOnKeyUp;
    property OnMouseDown: TMouseEvent read getOnMouseDown write setOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read getOnMouseMove write setOnMouseMove;
    property OnMouseUp: TMouseEvent read getOnMouseUp write setOnMouseUp;
    property OnDblClick: TNotifyEvent read getOnDblClick write setOnDblClick;
    property OnDragDrop: TDragDropEvent read getOnDragDrop write setOnDragDrop;
    property OnDragOver: TDragOverEvent read getOnDragOver write setOnDragOver;
    property OnEndDrag: TEndDragEvent read getOnEndDrag write setOnEndDrag;

    { properties redeclared as published }
    property Anchors;
    property Constraints;
    property BiDiMode;
    property ParentBiDiMode;
    {property Text;}
    property Color;
    property ParentColor;
    property TabOrder;
    property Visible;
    property OnStartDrag: TStartDragEvent read GetOnStartDrag write SetOnStartDrag;
  end;

implementation

uses Clipbrd, DBConsts, abcutild
{$IFDEF DELPHI6}
     , VDBConsts
{$ENDIF}
;

{$R navbtn32.res}

{ TabcDBSuperNav }

const
  SMoveBackRecord = 'Move back';
  SMoveFwdRecord =  'Move forward';
  SBookmarkRecord = 'Bookmark';
  SAppendRecord =   'Append record';

  BtnTypeName: array[TSuperNavBtn] of PChar = ('FIRST', 'MOVEBACK', 'PRIOR',
    'NEXT', 'MOVEFWD', 'LAST', 'APPEND', 'BOOKMARK', 'INSERT', 'DELETE', 'EDIT',
    'POST', 'CANCEL', 'REFRESH');

  BtnHintId: array[TSuperNavBtn] of string = (SFirstRecord, SMoveBackRecord,
    SPriorRecord, SNextRecord, SMoveFwdRecord, SLastRecord, SAppendRecord,
    SBookmarkRecord, SInsertRecord, SDeleteRecord, SEditRecord, SPostEdit,
    SCancelEdit, SRefreshRecord);

constructor TabcDBSuperNav.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] + [csOpaque];
  if not NewStyleControls then ControlStyle := ControlStyle + [csFramed];
  FDataLink := TabcSuperNavDataLink.Create(Self);
  FVisibleButtons := [snbFirst, snbMoveBack, snbPrior, snbNext, snbMoveFwd, snbLast,
    snbAppend, snbBookmark, snbInsert, snbDelete, snbEdit, snbPost, snbCancel,
    snbRefresh];
  FHints := TStringList.Create;
  FGlyphColor := gcColored;
  InitButtons;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  Width := 241+28+28+28+28;
  Height := 25;
  ButtonWidth := 0;
  FocusedButton := snbFirst;
  FConfirmDelete := True;
  {New property defaults}
  FNavSpeed := nsMedium;
  FMoveBy := 50;
end;

destructor TabcDBSuperNav.Destroy;
begin
  FHints.Free;
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TabcDBSuperNav.SetNavSpeed(Value: TabcNavSpeed);
var
  I: TSuperNavBtn;
  InitRepeatPause, RepeatPause: integer;
begin
  if Value <> FNavSpeed then begin
    FNavSpeed := Value;
    case Value of
      nsFast: begin
        InitRepeatPause := 200;
        RepeatPause := 10;
      end;
      nsMedium: begin
        InitRepeatPause := 200;
        RepeatPause := 50;
      end;
      nsSlow: begin
        InitRepeatPause := 400;
        RepeatPause := 100;
      end;
    else
      begin
        InitRepeatPause := 400;
        RepeatPause := 100;
      end;
    end;
    for I := Low(Buttons) to High(Buttons) do begin
      Buttons[I].FInitRepeatPause := InitRepeatPause;
      Buttons[I].FRepeatPause := RepeatPause;
    end;
  end;
end;

procedure TabcDBSuperNav.InitButtons;
var
  I: TSuperNavBtn;
  Btn: TabcSuperNavButton;
  X: Integer;
begin
  MinBtnSize := Point(20, 18);
  X := 0;
  for I := Low(Buttons) to High(Buttons) do
  begin
    Btn := TabcSuperNavButton.Create (Self);
    {new fields}
    Btn.FInitRepeatPause := 200;
    Btn.FRepeatPause := 50;
    Btn.Index := I;
    Btn.Visible := I in FVisibleButtons;
    Btn.Enabled := True;
    Btn.SetBounds (X, 0, MinBtnSize.X, MinBtnSize.Y);
    Btn.NumGlyphs := 2;
    Btn.OnClick := ClickButton;
    Btn.OnMouseDown := BtnMouseDown;
    Btn.Parent := Self;
    Btn.Flat := Flat;
    Buttons[I] := Btn;
    X := X + MinBtnSize.X;
  end;
  InitHints;
  with Buttons[snbBookmark] do begin
    {set special properties}
    GroupIndex := 29011; {unique group}
    AllowAllUp := True;
    Down := False;
    NumGlyphs := 4;
    OnClick := BookmarkClick;
  end;
  Buttons[snbPrior].NavStyle := Buttons[snbPrior].NavStyle + [nsAllowTimer];
  Buttons[snbNext].NavStyle  := Buttons[snbNext].NavStyle + [nsAllowTimer];
  Buttons[snbMoveBack].NavStyle  := Buttons[snbMoveBack].NavStyle + [nsAllowTimer];
  Buttons[snbMoveFwd].NavStyle  := Buttons[snbMoveFwd].NavStyle + [nsAllowTimer];
  LoadGlyphs;
end;

procedure TabcDBSuperNav.SetFlat(Value: Boolean);
var
  I: TSuperNavBtn;
begin
  if Value <> FFlat then begin
    FFlat := Value;
    for I := Low(Buttons) to High(Buttons) do
      Buttons[I].Flat := Value;
  end;
end;

procedure TabcDBSuperNav.SetGlyphColor(Value: TabcGlyphColor);
begin
  if Value <> FGlyphColor then begin
    FGlyphColor := Value;
    LoadGlyphs;
  end;
end;

procedure TabcDBSuperNav.LoadGlyphs;
var
  I: TSuperNavBtn;
  ResName: string;
begin
  for I := Low(Buttons) to High(Buttons) do
    with Buttons[I] do begin
      if FGlyphColor = gcColored then begin
        FmtStr(ResName, 'SUNAV_%s', [BtnTypeName[I]]);
        Glyph.Handle := LoadBitmap(HInstance, PChar(ResName));
        Glyph.PixelFormat := pf4Bit;
      end
      else begin
        FmtStr(ResName, 'SUNBW_%s', [BtnTypeName[I]]);
        Glyph.Handle := LoadBitmap(HInstance, PChar(ResName));
        Glyph.PixelFormat := pf1Bit;
      end;
      Enabled := False;
      Enabled := True;
    end;
end;

procedure TabcDBSuperNav.InitHints;
var
  I: Integer;
  J: TSuperNavBtn;
begin
  for J := Low(Buttons) to High(Buttons) do
    Buttons[J].Hint := BtnHintId[J];
  {new buttons - since no string res for them}
  Buttons[snbMoveBack].Hint := 'Move back';
  Buttons[snbMoveFwd].Hint := 'Move forward';
  Buttons[snbBookmark].Hint := 'Bookmark';
  Buttons[snbAppend].Hint := 'Append record';

  J := Low(Buttons);
  for I := 0 to (FHints.Count - 1) do
  begin
    if FHints.Strings[I] <> '' then Buttons[J].Hint := FHints.Strings[I];
    if J = High(Buttons) then Exit;
    Inc(J);
  end;
end;

procedure TabcDBSuperNav.SetHints(Value: TStrings);
begin
  FHints.Assign(Value);
  InitHints;
end;

procedure TabcDBSuperNav.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure TabcDBSuperNav.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcDBSuperNav.SetVisible(Value: TabcSuperNavBtnSet);
var
  I: TSuperNavBtn;
  W, H: Integer;
begin
  W := Width;
  H := Height;
  FVisibleButtons := Value;
  for I := Low(Buttons) to High(Buttons) do
    Buttons[I].Visible := I in FVisibleButtons;
  AdjustNavSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
  Invalidate;
end;

procedure TabcDBSuperNav.AdjustNavSize (var W: Integer; var H: Integer);
var
  Count: Integer;
  MinW: Integer;
  I: TSuperNavBtn;
  Space, Temp, Remain: Integer;
  X: Integer;
begin
  if (csLoading in ComponentState) then Exit;
  if Buttons[snbFirst] = nil then Exit;

  Count := 0;
  for I := Low(Buttons) to High(Buttons) do
    if Buttons[I].Visible then Inc(Count);
  if Count = 0 then Inc(Count);

  MinW := Count * MinBtnSize.X;
  if W < MinW then W := MinW;
  if H < MinBtnSize.Y then H := MinBtnSize.Y;

  ButtonWidth := W div Count;
  Temp := Count * ButtonWidth;
  if Align = alNone then W := Temp;

  X := 0;
  Remain := W - Temp;
  Temp := Count div 2;
  for I := Low(Buttons) to High(Buttons) do
  begin
    if Buttons[I].Visible then
    begin
      Space := 0;
      if Remain <> 0 then
      begin
        Dec (Temp, Remain);
        if Temp < 0 then
        begin
          Inc (Temp, Count);
          Space := 1;
        end;
      end;
      Buttons[I].SetBounds (X, 0, ButtonWidth + Space, Height);
      Inc (X, ButtonWidth + Space);
    end
    else
      Buttons[I].SetBounds (Width + 1, 0, ButtonWidth, Height);
  end;
end;

procedure TabcDBSuperNav.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  AdjustNavSize (W, H);
  inherited SetBounds (ALeft, ATop, W, H);
end;

procedure TabcDBSuperNav.WMSize(var Message: TWMSize);
var
  W, H: Integer;
begin
  inherited;

  { check for minimum size }
  W := Width;
  H := Height;
  AdjustNavSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;

procedure TabcDBSuperNav.ClickButton(Sender: TObject);
begin
  BtnClick(TabcSuperNavButton(Sender).Index);
end;

procedure TabcDBSuperNav.BookmarkClick(Sender: TObject);
begin
  if (DataSource <> nil) and (DataSource.State <> dsInactive) then
    with Sender as TabcSuperNavButton do
      begin
        AllowAllUp := not AllowAllUp;
        Down := not Down;
        if Down then
          FBookmark := DataSource.DataSet.GetBookmark
        else begin
          DataSource.DataSet.GotoBookmark(FBookmark);
          DataSource.DataSet.FreeBookmark(FBookmark);
        end;
      end;
  if not (csDesigning in ComponentState) and Assigned(FOnNavClick) then
    FOnNavClick(Self, snbBookmark);
end;

procedure TabcDBSuperNav.BtnMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  OldFocus: TSuperNavBtn;
begin
  OldFocus := FocusedButton;
  FocusedButton := TabcSuperNavButton(Sender).Index;
  if TabStop and (GetFocus <> Handle) and CanFocus then
  begin
    SetFocus;
    if (GetFocus <> Handle) then
      Exit;
  end
  else if TabStop and (GetFocus = Handle) and (OldFocus <> FocusedButton) then
  begin
    Buttons[OldFocus].Invalidate;
    Buttons[FocusedButton].Invalidate;
  end;
end;

procedure TabcDBSuperNav.BtnClick(Index: TSuperNavBtn);
begin
  if not (csDesigning in ComponentState) and Assigned(FBeforeAction) then
    FBeforeAction(Self, Index);
  if (DataSource <> nil) and (DataSource.State <> dsInactive) then
  begin
    with DataSource.DataSet do
    begin
      case Index of
        snbPrior: Prior;
        snbMoveBack: MoveBy(-FMoveBy);
        snbNext: Next;
        snbMoveFwd: MoveBy(FMoveBy);
        snbFirst: First;
        snbLast: Last;
        snbAppend: Append;
        snbInsert: Insert;
        snbEdit: Edit;
        snbCancel: Cancel;
        snbPost: Post;
        snbRefresh: Refresh;
        snbDelete:
          if not FConfirmDelete or
            (MessageDlg(SDeleteRecordQuestion, mtConfirmation,
            mbOKCancel, 0) <> idCancel) then Delete;
      end;
    end;
  end;
  if not (csDesigning in ComponentState) and Assigned(FOnNavClick) then
    FOnNavClick(Self, Index);
end;

procedure TabcDBSuperNav.WMSetFocus(var Message: TWMSetFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure TabcDBSuperNav.WMKillFocus(var Message: TWMKillFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure TabcDBSuperNav.KeyDown(var Key: Word; Shift: TShiftState);
var
  NewFocus: TSuperNavBtn;
  OldFocus: TSuperNavBtn;
begin
  OldFocus := FocusedButton;
  case Key of
    VK_RIGHT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus < High(Buttons) then
            NewFocus := Succ(NewFocus);
        until (NewFocus = High(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate;
        end;
      end;
    VK_LEFT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus > Low(Buttons) then
            NewFocus := Pred(NewFocus);
        until (NewFocus = Low(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate;
        end;
      end;
    VK_SPACE:
      begin
        if Buttons[FocusedButton].Enabled then
          Buttons[FocusedButton].Click;
      end;
  end;
end;

procedure TabcDBSuperNav.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TabcDBSuperNav.DataChanged;
var
  UpEnable, DnEnable: Boolean;
begin
  UpEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.BOF;
  DnEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.EOF;
  Buttons[snbFirst].Enabled := UpEnable;
  Buttons[snbMoveBack].Enabled := UpEnable;
  Buttons[snbPrior].Enabled := UpEnable;
  Buttons[snbNext].Enabled := DnEnable;
  Buttons[snbMoveFwd].Enabled := DnEnable;
  Buttons[snbLast].Enabled := DnEnable;
  Buttons[snbBookmark].Enabled := Enabled
    and not (FDataLink.DataSet.BOF and FDataLink.DataSet.EOF);
  Buttons[snbDelete].Enabled := Enabled and FDataLink.Active and
    FDataLink.DataSet.CanModify and
    not (FDataLink.DataSet.BOF and FDataLink.DataSet.EOF);
end;

procedure TabcDBSuperNav.EditingChanged;
var
  CanModify: Boolean;
begin
  CanModify := Enabled and FDataLink.Active and FDataLink.DataSet.CanModify;
  Buttons[snbAppend].Enabled := CanModify;
  Buttons[snbInsert].Enabled := CanModify;
  Buttons[snbEdit].Enabled := CanModify and not FDataLink.Editing;
  Buttons[snbPost].Enabled := CanModify and FDataLink.Editing;
  Buttons[snbCancel].Enabled := CanModify and FDataLink.Editing;
  Buttons[snbRefresh].Enabled := CanModify;
end;

procedure TabcDBSuperNav.ActiveChanged;
var
  I: TSuperNavBtn;
begin
  if not (Enabled and FDataLink.Active) then
    for I := Low(Buttons) to High(Buttons) do
      Buttons[I].Enabled := False
  else
  begin
    DataChanged;
    EditingChanged;
  end;
end;

procedure TabcDBSuperNav.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csLoading in ComponentState) then
    ActiveChanged;
end;

procedure TabcDBSuperNav.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if not (csLoading in ComponentState) then
    ActiveChanged;
  if Value <> nil then Value.FreeNotification(self);
end;

function TabcDBSuperNav.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcDBSuperNav.Loaded;
var
  W, H: Integer;
begin
  inherited Loaded;
  W := Width;
  H := Height;
  AdjustNavSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
  InitHints;
  ActiveChanged;
end;

{TabcSuperNavButton}

destructor TabcSuperNavButton.Destroy;
begin
  if FRepeatTimer <> nil then
    FRepeatTimer.Free;
  inherited Destroy;
end;

procedure TabcSuperNavButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown (Button, Shift, X, Y);
  if nsAllowTimer in FNavStyle then
  begin
    if FRepeatTimer = nil then
      FRepeatTimer := TTimer.Create(Self);
    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := FInitRepeatPause;
    FRepeatTimer.Enabled  := True;
  end;
end;

procedure TabcSuperNavButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
                                  X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil then
    FRepeatTimer.Enabled  := False;
end;

procedure TabcSuperNavButton.TimerExpired(Sender: TObject);
begin
  if FRepeatTimer.Interval = 10 then
    {issue an extra click to speed it up
     since timer minimum is really 50ms}
    if (FState = bsDown) and MouseCapture then begin
      try
        Click;
      except
        FRepeatTimer.Enabled := False;
        raise;
      end;
    end;

  FRepeatTimer.Interval := FRepeatPause;
  if (FState = bsDown) and MouseCapture then
  begin
    try
      Click;
    except
      FRepeatTimer.Enabled := False;
      raise;
    end;
  end;
end;

procedure TabcSuperNavButton.Paint;
var
  R: TRect;
begin
  inherited Paint;
  if (GetFocus = Parent.Handle) and
     (FIndex = TabcDBSuperNav (Parent).FocusedButton) then
  begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if FState = bsDown then
      OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
  end;
end;

{ TabcSuperNavDataLink }

constructor TabcSuperNavDataLink.Create(ANav: TabcDBSuperNav);
begin
  inherited Create;
  FNavigator := ANav;
  VisualControl := True;
end;

destructor TabcSuperNavDataLink.Destroy;
begin
  FNavigator := nil;
  inherited Destroy;
end;

procedure TabcSuperNavDataLink.EditingChanged;
begin
  if FNavigator <> nil then FNavigator.EditingChanged;
end;

procedure TabcSuperNavDataLink.DataSetChanged;
begin
  if FNavigator <> nil then FNavigator.DataChanged;
end;

procedure TabcSuperNavDataLink.ActiveChanged;
begin
  if FNavigator <> nil then FNavigator.ActiveChanged;
end;


{ TabcDBRecordCount }

constructor TabcDBRecordCount.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcRecCountDataLink.Create(Self);
  ShowAccelChar := False;
  Width := 95;
  Height := 25;
  FMask := 'Record #R of #N';
  FDisabledCaption := 'Disabled';
  FCurrentRecord := 5;
  FRecordCount := 0;
end;

destructor TabcDBRecordCount.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FDataset := nil;
  inherited Destroy;
end;

procedure TabcDBRecordCount.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FDataset := Value;
  end;
end;

procedure TabcDBRecordCount.ActiveChanged;
begin
  SetDataset(FDataLink.DataSet);
  DataChange;
end;

procedure TabcDBRecordCount.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  RefreshCaption;
end;

procedure TabcDBRecordCount.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcDBRecordCount.Loaded;
begin
  inherited Loaded;
  ActiveChanged;
end;

procedure TabcDBRecordCount.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if not (csLoading in ComponentState) then
    DataChange;
end;

function TabcDBRecordCount.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcDBRecordCount.DataChange;
begin
  if not (csDestroying in ComponentState) then
    if (FDataSet <> nil) then
      RefreshRecordCount
    else
      if csDesigning in ComponentState then
        Caption := FMask
      else begin
        FCurrentRecord := 0;
        FRecordCount := 0;
        RefreshCaption;
      end;
end;

procedure TabcDBRecordCount.CalcRecNumCount(var ARecNum, ARecCount: Integer);
begin
  if (FDataSet <> nil) and FDataSet.Active then
    with FDataSet do begin
      ARecCount := RecordCount;
      UpdateCursorPos;
      if EOF then
        if State = dsInsert then
          ARecNum := ARecCount + 1
        else
          ARecNum := ARecCount
      else
        ARecNum := RecNo;
    end
  else begin
    ARecNum := 0;
    ARecCount := 0;
  end;
end;

function TabcDBRecordCount.CalcCaptionFor(const ARecNum, ARecCount: Integer): string;
const
  RecMarker = '#R';
  CountMarker = '#N';
var
  MarkerPos: integer;
begin
  if not Enabled or (FDataSet = nil) then
    if (csDesigning in ComponentState) then
      Result := Name
    else
      Result := FDisabledCaption
  else
  begin
    Result := FMask;
    MarkerPos := Pos(RecMarker, Result);
    if MarkerPos > 0 then
    begin
      Delete(Result, MarkerPos, Length(RecMarker));
      Insert(InttoStr(ARecNum), Result, MarkerPos);
    end;
    MarkerPos := Pos(CountMarker, Result);
    if MarkerPos > 0 then
    begin
      Delete(Result, MarkerPos, Length(CountMarker));
      Insert(InttoStr(ARecCount), Result, MarkerPos);
    end;
  end;
end;

function TabcDBRecordCount.GetLabelText: string;
var
  TempCurRec, TempRecCount: LongInt;
begin
  if not (csPaintCopy in ControlState) then
    Result := inherited GetLabelText
  else
  begin
    CalcRecNumCount(TempCurRec, TempRecCount);
    Result := CalcCaptionFor(TempCurRec, TempRecCount);
  end;
end;

procedure TabcDBRecordCount.RefreshRecordCount;
var
  TempCurRec, TempRecCount: LongInt;
begin
  CalcRecNumCount(TempCurRec, TempRecCount);
  if (FCurrentRecord <> TempCurRec) or (FRecordCount <> TempRecCount) then
  begin
    FCurrentRecord := TempCurRec;
    FRecordCount := TempRecCount;
    RefreshCaption;
  end;
end;

procedure TabcDBRecordCount.RefreshCaption;
begin
  Caption := CalcCaptionFor(FCurrentRecord, FRecordCount);
end;

procedure TabcDBRecordCount.SetMask(Value: string);
begin
  if Value <> FMask then
  begin
    FMask := Value;
    FCurrentRecord := 5; {to force a recalc of caption}
    FRecordCount := 0;
    DataChange;
  end;
end;

function TabcDBRecordCount.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TabcDBRecordCount.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

{ TabcRecCountDataLink }

constructor TabcRecCountDataLink.Create(ARecordCount: TabcDBRecordCount);
begin
  inherited Create;
  FRecordCount := ARecordCount;
  VisualControl := True;
end;

destructor TabcRecCountDataLink.Destroy;
begin
  FRecordCount := nil;
  inherited Destroy;
end;

procedure TabcRecCountDataLink.EditingChanged;
begin
  if FRecordCount <> nil then FRecordCount.DataChange;
end;

procedure TabcRecCountDataLink.DataSetChanged;
begin
  if FRecordCount <> nil then FRecordCount.DataChange;
end;

procedure TabcRecCountDataLink.ActiveChanged;
begin
  if FRecordCount <> nil then FRecordCount.ActiveChanged;
end;

{ TabcDBState }

constructor TabcDBState.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcDBStateDataLink.Create(Self);
  FStates := TStringList.Create;
  FStates.Add('Inactive');
  FStates.Add('Browse');
  FStates.Add('Edit');
  FStates.Add('Insert');
  FStates.Add('Set Key');
  FStates.Add('Calculating Fields');
  ShowAccelChar := False;
  Width := 95;
  Height := 25;
  FMask := '#D #S';
  FDisabledCaption := 'Disabled';
end;

destructor TabcDBState.Destroy;
begin
  FDataLink.Free;
  FStates.Free;
  inherited Destroy;
end;

procedure TabcDBState.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FDataset := Value;
    FAdapter.Free;
    FAdapter := nil;
    if Value <> nil then
      FAdapter := abcDSAdapterFactory.CreateAdapter(FDataSet);  
  end;
end;

procedure TabcDBState.ActiveChanged;
begin
  SetDataset(FDataLink.DataSet);
  DataChange;
end;

procedure TabcDBState.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  RefreshState;
end;

procedure TabcDBState.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcDBState.Loaded;
begin
  inherited Loaded;
  ActiveChanged;
end;

procedure TabcDBState.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if not (csLoading in ComponentState) then
    DataChange;
end;

function TabcDBState.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcDBState.DataChange;
begin
  if (FDataSet = nil) then
    if (csDesigning in ComponentState) then
      Caption := Name
    else
      Caption := FDisabledCaption
  else
    RefreshState;
end;

procedure TabcDBState.RefreshState;
const
  DataSetMarker = '#D';
  StateMarker = '#S';
  DefaultStateNames: array[0..5] of string = ('Inactive', 'Browse', 'Edit', 'Insert', 'Set Key', 'Calculating Fields');
var
  Index: integer;
  TempCaption: string;
  StateName: string;
  MarkerPos: integer;
begin
  if not Enabled or (FDataSet = nil) then
    if (csDesigning in ComponentState) then
      Caption := Name
    else
      Caption := FDisabledCaption
  else begin
    TempCaption := FMask;
    {dataset tablename or componentname}
    MarkerPos := Pos(DataSetMarker, TempCaption);
    if MarkerPos > 0 then begin
      Delete(TempCaption, MarkerPos, Length(DataSetMarker));
      if FAdapter <> nil then
        Insert(FAdapter.Description, TempCaption, MarkerPos);
    end;
    {dataset state description}
    MarkerPos := Pos(StateMarker, TempCaption);
    if MarkerPos > 0 then begin
      Index := Ord(FDataSet.State);
      if Index > FStates.Count - 1 then
        StateName := DefaultStateNames[Index]
      else
        StateName := FStates.Strings[Index];
      Delete(TempCaption, MarkerPos, Length(StateMarker));
      Insert(StateName, TempCaption, MarkerPos);
    end;
    Caption := TempCaption;
  end;
end;

procedure TabcDBState.SetMask(Value: string);
begin
  if Value <> FMask then begin
    FMask := Value;
    DataChange;
  end;
end;

function TabcDBState.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TabcDBState.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

{ TabcDBStateDataLink }

constructor TabcDBStateDataLink.Create(AState: TabcDBState);
begin
  inherited Create;
  FState := AState;
  VisualControl := True;
end;

destructor TabcDBStateDataLink.Destroy;
begin
  FState := nil;
  inherited Destroy;
end;

procedure TabcDBStateDataLink.EditingChanged;
begin
  if FState <> nil then FState.DataChange;
end;

procedure TabcDBStateDataLink.ActiveChanged;
begin
  if FState <> nil then FState.ActiveChanged;
end;

procedure TabcDBState.SetStates(Value: TStrings);
begin
  FStates.Assign(Value);
  DataChange;
end;

{ TabcDBEditLabel }

{ TAbcElSubDBEdit }

type
  TAbcElSubDBEdit = class(TDBEdit)
  private
    procedure WMSize(var Message: TMessage); message WM_Size;
  protected
    procedure Change; override;
  protected
    procedure Click; override;
    procedure DblClick; override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure DoStartDrag(var DragObject: TDragObject); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    constructor Create(AOwner: TComponent); override;
  end;

constructor TAbcElSubDBEdit.Create(AOwner: TComponent);
var
  Link: integer;
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  Text := '';
  Link := Perform(CM_GETDATALINK, 0, 0);
  if Link <> 0 then TFieldDataLink(Link).OnActiveChange := TabcDBEditLabel(AOwner).ActiveChange;
end;

procedure TAbcElSubDBEdit.Change;
begin
  inherited Change;
  if TabcDBEditLabel(Owner).FCaptionAuto then
    if Field <> nil then
      if TabcDBEditLabel(Owner).fLabel.Caption <> Field.DisplayName then
        TabcDBEditLabel(Owner).fLabel.Caption := Field.DisplayName;
end;

procedure TAbcElSubDBEdit.Click;
begin
  if Assigned(OnClick) then OnClick(Owner);
end;

procedure TAbcElSubDBEdit.DblClick;
begin
  if Assigned(OnDblClick) then OnDblClick(Owner);
end;

procedure TAbcElSubDBEdit.DoEndDrag(Target: TObject; X, Y: Integer);
begin
  if Assigned(OnEndDrag) then OnEndDrag(Owner, Target, X, Y);
end;

procedure TAbcElSubDBEdit.DoEnter;
begin
  if Assigned(OnEnter) then OnEnter(Owner);
end;

procedure TAbcElSubDBEdit.DoExit;
begin
  if Assigned(OnExit) then OnExit(Owner);
end;

procedure TAbcElSubDBEdit.DoStartDrag(var DragObject: TDragObject);
begin
  if Assigned(OnStartDrag) then OnStartDrag(Owner, DragObject);
end;

procedure TAbcElSubDBEdit.DragDrop(Source: TObject; X, Y: Integer);
begin
  if Assigned(OnDragDrop) then OnDragDrop(Owner, Source, X, Y);
end;

procedure TAbcElSubDBEdit.DragOver(Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := True;
  if Assigned(OnDragOver) then
    OnDragOver(Owner, Source, X, Y, State, Accept) else
    Accept := False;
end;

procedure TAbcElSubDBEdit.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if Assigned(OnKeyUp) then OnKeyUp(Owner, Key, Shift);
end;

procedure TAbcElSubDBEdit.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseDown) then OnMouseDown(Owner, Button, Shift, X, Y);
end;

procedure TAbcElSubDBEdit.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseMove) then OnMouseMove(Owner, Shift, X, Y);
end;

procedure TAbcElSubDBEdit.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseUp) then OnMouseUp(Owner, Button, Shift, X, Y);
end;

procedure TAbcElSubDBEdit.WMSize(var Message: TMessage);
begin
  inherited;
  sendMessage(parent.handle, WM_Size, 0, parent.width + (longint(parent.height) shl 16));
end;

{ TabcDBEditLabel }

constructor TabcDBEditLabel.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  fDBEdit := TAbcElSubDBEdit.Create(Self);
  fDBEdit.parent := self;
  fDBEdit.autosize := true;
  fDBEdit.onChange := textChanged;
  fDBEdit.tabStop := true;
  fLabel := TLabel.Create(Self);
  fLabel.parent := self;
  fLabel.autosize := false;
  fLabel.focusControl := fDBEdit;
  fLabel.transparent := true;
  fLabel.wordWrap := true;
  fLabel.onClick := labelClicked;
  fLabel.showhint := false;
  fLabel.Caption := 'Caption';
  fCaptionAuto := true;
  fDBEdit.Height := 25;
  LabelHeight := 28;
  Width := 223;
  fLayout := elLeft;
  fLabelWidth := 100;
  initLayout;
end;

destructor TabcDBEditLabel.Destroy;
begin
  fLabel.Free;
  fDBEdit.Free;
  inherited Destroy;
end;

procedure TabcDBEditLabel.WMSize(var Message: TMessage);
begin
  inherited;
  if not FCalcLayout then begin
    FSizing := True;
    initLayout;
  end;
end;

procedure TabcDBEditLabel.WMNCHitTest(var Message: TMessage);
var
  WindowRect: TRect;
  WinMidX: word;
begin
  if (csDesigning in ComponentState) then begin
    {get window position to find out if mouse is near left or right end}
    GetWindowRect(Handle, WindowRect);
    {compare WindowRect.Left, WindowRect.Right, Message.LParamLO}
    WinMidX := WindowRect.Left + (WindowRect.Right - WindowRect.Left) div 2;
    if Message.LParamLO < WinMidX then
      FLabelFixed := False
    else
      FLabelFixed := True;
  end;
  inherited;
end;

procedure TabcDBEditLabel.paintWindow(DC: HDC);
begin
  inherited paintWindow(DC);
  if csDesigning in ComponentState then
    drawFocusRect(DC, CLientRect);
end;

procedure TabcDBEditLabel.CMTextChanged(var Message: TMessage);
begin
  inherited;
  if fDBEdit.Text <> inherited Text then fDBEdit.Text := inherited Text;
end;

{CMEnter Proc deleted}
procedure TabcDBEditLabel.DoEnter;
begin
  fDBEdit.SetFocus;
end;

procedure TabcDBEditLabel.SetFocus;
begin
  fDBEdit.SetFocus;
end;

function TabcDBEditLabel.Focused;
begin
  Result := fDBEdit.Focused;
end;

function TabcDBEditLabel.isLabelFontStored: boolean;
begin
  result := not LabelParentFont;
end;

function TabcDBEditLabel.isEditFontStored: boolean;
begin
  result := not EditParentFont;
end;

function TabcDBEditLabel.isCtl3DStored: boolean;
begin
  result := not ParentCtl3D;
end;

function TabcDBEditLabel.isEditColorStored: boolean;
begin
  result := not EditParentColor;
end;

function TabcDBEditLabel.isShowHintStored: boolean;
begin
  result := not ParentShowHint;
end;

procedure TabcDBEditLabel.textChanged(Sender: TObject);
begin
  if not(csLoading in ComponentState) then
    if Assigned(fOnChange) then fOnChange(Sender);
end;

procedure TabcDBEditLabel.labelClicked(Sender: TObject);
begin
  fDBEdit.setfocus;
  fDBEdit.selectAll;
end;

procedure TabcDBEditLabel.Loaded;
begin
  inherited Loaded;
  initLayout;
end;

procedure TabcDBEditLabel.initLayout;
var
  AdjustYOffset: Byte;
begin
  if (csLoading in ComponentState) then Exit;
  if FCalcLayout then
    Exit
  else
    FCalcLayout := True;
  if fLayout = elTop then begin
    if FSizing then begin
      LabelHeight := Height - FLabelOffsetY - fDBEdit.Height - 2;
      if LabelOffsetX > 0 then
        LabelWidth := Width - 2 - LabelOffsetX
      else
        LabelWidth := Width - 2;
    end
    else begin
      Height := fLabel.Height + FLabelOffsetY + fDBEdit.Height + 2;
      if FLabelOffsetX > 0 then begin
        if Width <> fLabel.Width + FLabelOffsetX + 2 then Width := fLabel.Width + FLabelOffsetX + 2;
      end
      else
        if Width <> fLabel.Width - FLabelOffsetX + 2 then Width := fLabel.Width - FLabelOffsetX + 2;
    end;

    if fDBEdit.top <> (Height - fDBEdit.height - 1) then fDBEdit.top := Height - fDBEdit.height - 1;

    if FLabelOffsetX > 0 then begin
      if fDBEdit.left <> FLabelOffsetX + 1 then fDBEdit.left := FLabelOffsetX + 1;
      if fLabel.left <> 1 then fLabel.left := 1;
      if fDBEdit.width <> (Width - FLabelOffsetX - 2) then fDBEdit.width := Width - FLabelOffsetX - 2;
    end
    else begin
      if fLabel.left <> 1 - FLabelOffsetX then fLabel.left := 1 - FLabelOffsetX;
      if fDBEdit.left <> 1 then fDBEdit.left := 1;
      if fDBEdit.width <> (Width - 2) then fDBEdit.width := Width - 2;
    end;
  end
  else begin
    if FSizing then
      LabelHeight := Height
    else
      Height := fLabel.Height;
    if FLabelFixed then begin
      if fLabel.width <> fLabelWidth then fLabel.width := fLabelWidth;
      if fDBEdit.width <> (self.width - fLabelWidth - FLabelOffsetX - 2) then
        fDBEdit.width := (self.width - fLabelWidth - FLabelOffsetX - 2);
    end
    else begin
      if fLabel.width <> (self.width - fDBEdit.width - FLabelOffsetX - 2) then
        fLabel.width := (self.width - fDBEdit.width - FLabelOffsetX - 2);
      fLabelWidth := fLabel.width;
    end;

    if fLabel.left <> 1 then fLabel.left := 1;

    if NewStyleControls then
      AdjustYOffset := 3
    else
      AdjustYOffset := 5;

    if BorderStyle = bsNone then
      AdjustYOffset := AdjustYOffset - 2;

    if FLabelOffsetY < 0 then begin
      if fLabel.top <> (AdjustYOffset - FLabelOffsetY) then fLabel.top := AdjustYOffset - FLabelOffsetY;
    end
    else
      if fLabel.top <> AdjustYOffset then fLabel.top := AdjustYOffset;

    if FLabelOffsetY > 0 then begin
      if fDBEdit.top <> (FLabelOffsetY + 1) then fDBEdit.top := FLabelOffsetY + 1;
    end
    else
      if fDBEdit.top <> 1 then fDBEdit.top := 1;

    if fDBEdit.left <> (fLabelWidth + FLabelOffsetX) then fDBEdit.left := (fLabelWidth + FLabelOffsetX);
  end;

  FCalcLayout := False;
  FSizing := False;
end;

procedure TabcDBEditLabel.ChangeScale(M, D: Integer);
begin
  inherited ChangeScale(M,D);
  fDBEdit.Autosize := False;
  fDBEdit.Autosize := True;
  InitLayout;
end;

procedure TabcDBEditLabel.setLayout(setTo: TabcEditLabelLayout);
begin
  if (fLayout = elLeft) and (setTo = elTop)
    and not (csLoading in ComponentState) then
    fSaveLabelWidth := LabelWidth;
  fLayout := setTo;
  if (csLoading in ComponentState) then Exit;
  if fLayout = elTop then begin
    if FLabelOffsetX > 0 then begin
      Width := fDBEdit.Width + FLabelOffsetX + 2;
      fLabel.left := 1;
    end
    else begin
      Width := fDBEdit.Width + 2;
      fLabel.left := 1 - FLabelOffsetX;
    end;
    if not (csLoading in ComponentState) then
      LabelHeight := Height;
  end
  else begin
    if fSaveLabelWidth <> 0 then begin
      fLabelWidth := fSaveLabelWidth;
      fSaveLabelWidth := 0;
    end;
    FLabelFixed := True;
    Width := fLabelWidth + FLabelOffsetX + fDBEdit.Width  + 2;
    Height := Height - fDBEdit.Height - FLabelOffsetY - 2;
  end;
  initLayout;
end;

procedure TabcDBEditLabel.SetLabelOffsetX(Value: Integer);
begin
  if FLabelOffsetX <> Value then
    begin
      FLabelOffsetX := Value;
      initLayout;
    end;
end;

procedure TabcDBEditLabel.SetLabelOffsetY(Value: Integer);
begin
  if FLabelOffsetY <> Value then
    begin
      FLabelOffsetY := Value;
      initLayout;
    end;
end;

procedure TabcDBEditLabel.setLabelWidth(setTo: integer);
begin
  if fLabelWidth <> setTo then
    begin
      fLabelWidth := setTo;
      FLabelFixed := True;
      fLabel.Width := fLabelWidth;
      initLayout;
    end;
end;

procedure TabcDBEditLabel.setLabelHeight(setTo: integer);
begin
  if fLabelHeight <> setTo then
    begin
      fLabelHeight := setTo;
      FLabelFixed := True;
      fLabel.Height := fLabelHeight;
      initLayout;
    end;
end;

procedure TabcDBEditLabel.Clear;
begin
  fDBEdit.Clear;
end;

procedure TabcDBEditLabel.ClearSelection;
begin
  fDBEdit.ClearSelection;
end;

procedure TabcDBEditLabel.CopyToClipboard;
begin
  fDBEdit.CopyToClipboard;
end;

procedure TabcDBEditLabel.CutToClipboard;
begin
  fDBEdit.CutToClipboard;
end;

procedure TabcDBEditLabel.PasteFromClipboard;
begin
  fDBEdit.PasteFromClipboard;
end;

function  TabcDBEditLabel.GetSelTextBuf(Buffer: PChar; BufSize: Integer): Integer;
begin
  result := fDBEdit.GetSelTextBuf(Buffer, BufSize);
end;

procedure TabcDBEditLabel.SelectAll;
begin
  fDBEdit.SelectAll;
end;

procedure TabcDBEditLabel.SetSelTextBuf(Buffer: PChar);
begin
  fDBEdit.SetSelTextBuf(Buffer);
end;

function TabcDBEditLabel.getModified: Boolean;
begin
  result := fDBEdit.Modified;
end;

procedure TabcDBEditLabel.setModified(setTo: Boolean);
begin
  if fDBEdit.Modified <> setTo then fDBEdit.Modified := setTo;
end;

function TabcDBEditLabel.getSelLength: Integer;
begin
  result := fDBEdit.SelLength;
end;

procedure TabcDBEditLabel.setSelLength(setTo: Integer);
begin
  if fDBEdit.SelLength <> setTo then fDBEdit.SelLength := setTo;
end;

function TabcDBEditLabel.getSelStart: Integer;
begin
  result := fDBEdit.SelStart;
end;

procedure TabcDBEditLabel.setSelStart(setTo: Integer);
begin
  if fDBEdit.SelStart <> setTo then fDBEdit.SelStart := setTo;
end;

function TabcDBEditLabel.getSelText: string;
begin
  result := fDBEdit.SelText;
end;

procedure TabcDBEditLabel.setSelText(setTo: string);
begin
  if fDBEdit.SelText <> setTo then fDBEdit.SelText := setTo;
end;

function TabcDBEditLabel.getOnChange: TNotifyEvent;
begin
  result := fOnChange;
end;

procedure TabcDBEditLabel.setOnChange(setTo: TNotifyEvent);
begin
  fOnChange := setTo;
end;

function TabcDBEditLabel.getOnClick: TNotifyEvent;
begin
  result := fDBEdit.OnClick;
end;

procedure TabcDBEditLabel.setOnClick(setTo: TNotifyEvent);
begin
  fDBEdit.OnClick := setTo;
end;

function TabcDBEditLabel.getOnDblClick: TNotifyEvent;
begin
  result := fDBEdit.OnDblClick;
end;

procedure TabcDBEditLabel.setOnDblClick(setTo: TNotifyEvent);
begin
  fDBEdit.OnDblClick := setTo;
end;

function TabcDBEditLabel.getOnEnter: TNotifyEvent;
begin
  result := fDBEdit.OnEnter;
end;

procedure TabcDBEditLabel.setOnEnter(setTo: TNotifyEvent);
begin
  fDBEdit.OnEnter := setTo;
end;

function TabcDBEditLabel.getOnExit: TNotifyEvent;
begin
  result := fDBEdit.OnExit;
end;

procedure TabcDBEditLabel.setOnExit(setTo: TNotifyEvent);
begin
  fDBEdit.OnExit := setTo;
end;

function TabcDBEditLabel.getOnKeyDown: TKeyEvent;
begin
  result := fDBEdit.OnKeyDown;
end;

procedure TabcDBEditLabel.setOnKeyDown(setTo: TKeyEvent);
begin
  fDBEdit.OnKeyDown := setTo;
end;

function TabcDBEditLabel.getOnKeyPress: TKeyPressEvent;
begin
  result := fDBEdit.OnKeyPress;
end;

procedure TabcDBEditLabel.setOnKeyPress(setTo: TKeyPressEvent);
begin
  fDBEdit.OnKeyPress := setTo;
end;

function TabcDBEditLabel.getOnKeyUp: TKeyEvent;
begin
  result := fDBEdit.OnKeyUp;
end;

procedure TabcDBEditLabel.setOnKeyUp(setTo: TKeyEvent);
begin
  fDBEdit.OnKeyUp := setTo;
end;

function TabcDBEditLabel.getOnMouseDown: TMouseEvent;
begin
  result := fDBEdit.OnMouseDown;
end;

procedure TabcDBEditLabel.setOnMouseDown(setTo: TMouseEvent);
begin
  fDBEdit.OnMouseDown := setTo;
end;

function TabcDBEditLabel.getOnMouseMove: TMouseMoveEvent;
begin
  result := fDBEdit.OnMouseMove;
end;

procedure TabcDBEditLabel.setOnMouseMove(setTo: TMouseMoveEvent);
begin
  fDBEdit.OnMouseMove := setTo;
end;

function TabcDBEditLabel.getOnMouseUp: TMouseEvent;
begin
  result := fDBEdit.OnMouseUp;
end;

procedure TabcDBEditLabel.setOnMouseUp(setTo: TMouseEvent);
begin
  fDBEdit.OnMouseUp := setTo;
end;

function TabcDBEditLabel.GetOnDragDrop: TDragDropEvent; begin Result := fDBEdit.OnDragDrop; end;

procedure TabcDBEditLabel.SetOnDragDrop(Value: TDragDropEvent);
begin
  fDBEdit.OnDragDrop := Value;
end;

function TabcDBEditLabel.GetOnDragOver: TDragOverEvent; begin Result := fDBEdit.OnDragOver; end;

procedure TabcDBEditLabel.SetOnDragOver(Value: TDragOverEvent);
begin
  fDBEdit.OnDragOver := Value;
end;

function TabcDBEditLabel.GetOnEndDrag: TEndDragEvent; begin Result := fDBEdit.OnEndDrag; end;

procedure TabcDBEditLabel.SetOnEndDrag(Value: TEndDragEvent);
begin
  fDBEdit.OnEndDrag := Value;
end;

function TabcDBEditLabel.GetOnStartDrag: TStartDragEvent; begin Result := fDBEdit.OnStartDrag; end;

procedure TabcDBEditLabel.SetOnStartDrag(Value: TStartDragEvent);
begin
  fDBEdit.OnStartDrag := Value;
end;

function TabcDBEditLabel.getAutoSelect: boolean;
begin
  result := fDBEdit.AutoSelect;
end;

procedure TabcDBEditLabel.setAutoSelect(setTo: boolean);
begin
  if fDBEdit.AutoSelect <> setTo then fDBEdit.AutoSelect := setTo;
end;

function TabcDBEditLabel.getBorderStyle: TBorderStyle;
begin
  result := fDBEdit.BorderStyle;
end;

procedure TabcDBEditLabel.setBorderStyle(setTo: TBorderStyle);
begin
  if fDBEdit.BorderStyle <> setTo then fDBEdit.BorderStyle := setTo;
end;

function TabcDBEditLabel.getCharCase: TEditCharCase;
begin
  result := fDBEdit.CharCase;
end;

procedure TabcDBEditLabel.setCharCase(setTo: TEditCharCase);
begin
  if fDBEdit.CharCase <> setTo then fDBEdit.CharCase := setTo;
end;

function TabcDBEditLabel.getEditColor: TColor;
begin
  result := fDBEdit.Color;
end;

procedure TabcDBEditLabel.setEditColor(setTo: TColor);
begin
  if fDBEdit.Color <> setTo then fDBEdit.Color := setTo;
end;

function TabcDBEditLabel.getEditParentColor: boolean;
begin
  result := fDBEdit.ParentColor;
end;

procedure TabcDBEditLabel.setEditParentColor(setTo: boolean);
begin
  if fDBEdit.ParentColor <> setTo then fDBEdit.ParentColor := setTo;
end;

function TabcDBEditLabel.getCtl3D: boolean;
begin
  result := fDBEdit.Ctl3D;
end;

procedure TabcDBEditLabel.setCtl3D(setTo: boolean);
begin
  if fDBEdit.Ctl3D <> setTo then fDBEdit.Ctl3D := setTo;
end;

function TabcDBEditLabel.getParentCtl3D: boolean;
begin
  result := fDBEdit.ParentCtl3D;
end;

procedure TabcDBEditLabel.setParentCtl3D(setTo: boolean);
begin
  if fDBEdit.ParentCtl3D <> setTo then fDBEdit.ParentCtl3D := setTo;
end;

function TabcDBEditLabel.getMaxLength: integer;
begin
  result := fDBEdit.MaxLength;
end;

procedure TabcDBEditLabel.setMaxLength(setTo: integer);
begin
  if fDBEdit.MaxLength <> setTo then fDBEdit.MaxLength := setTo;
end;

function TabcDBEditLabel.getPasswordChar: char;
begin
  result := fDBEdit.PasswordChar;
end;

procedure TabcDBEditLabel.setPasswordChar(setTo: char);
begin
  if fDBEdit.PasswordChar <> setTo then fDBEdit.PasswordChar := setTo;
end;

function TabcDBEditLabel.getReadOnly: boolean;
begin
  result := fDBEdit.ReadOnly;
end;

procedure TabcDBEditLabel.setReadOnly(setTo: boolean);
begin
  if fDBEdit.ReadOnly <> setTo then fDBEdit.ReadOnly := setTo;
end;

procedure TabcDBEditLabel.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  fDBEdit.Enabled := Enabled;
  fLabel.Enabled := Enabled;
end;

function TabcDBEditLabel.getTabStop: boolean;
begin
  result := fDBEdit.TabStop;
end;

procedure TabcDBEditLabel.setTabStop(setTo: boolean);
begin
  if TabStop <> setTo then fDBEdit.TabStop := setTo;
end;

function TabcDBEditLabel.getAlignment: TAlignment;
begin
  result := fLabel.Alignment;
end;

procedure TabcDBEditLabel.setAlignment(setTo: TAlignment);
begin
  if fLabel.Alignment <> setTo then fLabel.Alignment := setTo;
end;

function TabcDBEditLabel.getCaption: TCaption;
begin
  result := fLabel.Caption;
end;

procedure TabcDBEditLabel.setCaption(Value: TCaption);
begin
  if not FCaptionAuto then begin
    fLabel.Caption := Value;
  end;
end;

procedure TabcDBEditLabel.setCaptionAuto(Value: Boolean);
begin
  FCaptionAuto := Value;
  if Value then SetCaptionToField;
end;

function TabcDBEditLabel.getShowAccelChar: boolean;
begin
  result := fLabel.ShowAccelChar;
end;

procedure TabcDBEditLabel.setShowAccelChar(setTo: boolean);
begin
  if fLabel.ShowAccelChar <> setTo then fLabel.ShowAccelChar := setTo;
end;

function TabcDBEditLabel.getLabelFont: TFont;
begin
  result := fLabel.Font;
end;

procedure TabcDBEditLabel.setLabelFont(setTo: TFont);
begin
  if fLabel.Font <> setTo then begin
    fLabel.Font := setTo;
    if fLayout = elLeft then begin
      FLabelOffsetY := -(fLabel.Font.Height - fDBEdit.Font.Height);
      initLayout;
    end;
  end;

end;

function TabcDBEditLabel.getEditFont: TFont;
begin
  result := fDBEdit.Font;
end;

procedure TabcDBEditLabel.setEditFont(setTo: TFont);
begin
  if fDBEdit.Font <> setTo then begin
    fDBEdit.Font := setTo;
    if fLayout = elLeft then begin
      FLabelOffsetY := -(fLabel.Font.Height - fDBEdit.Font.Height);
      initLayout;
    end;
  end;
end;

function TabcDBEditLabel.getLabelParentFont: boolean;
begin
  result := fLabel.ParentFont;
end;

procedure TabcDBEditLabel.setLabelParentFont(setTo: boolean);
begin
  if fLabel.ParentFont <> setTo then fLabel.ParentFont := setTo;
end;

function TabcDBEditLabel.getEditParentFont: boolean;
begin
  result := fDBEdit.ParentFont;
end;

procedure TabcDBEditLabel.setEditParentFont(setTo: boolean);
begin
  if fDBEdit.ParentFont <> setTo then fDBEdit.ParentFont := setTo;
end;

function TabcDBEditLabel.getHelpContext: THelpContext;
begin
  result := fDBEdit.HelpContext;
end;

procedure TabcDBEditLabel.setHelpContext(setTo: THelpContext);
begin
  if fDBEdit.HelpContext <> setTo then fDBEdit.HelpContext := setTo;
end;

function TabcDBEditLabel.GetCursor: TCursor;
begin
  result := fDBEdit.Cursor;
end;

procedure TabcDBEditLabel.SetCursor(Value: TCursor);
begin
  if Value <> fDBEdit.Cursor then
    fDBEdit.Cursor := Value;
end;

function TabcDBEditLabel.GetDragCursor: TCursor;
begin
  result := fDBEdit.DragCursor;
end;

procedure TabcDBEditLabel.SetDragCursor(Value: TCursor);
begin
  if Value <> fDBEdit.DragCursor then
    fDBEdit.DragCursor := Value;
end;

procedure TabcDBEditLabel.SetDragMode(Value: TDragMode);
begin
  inherited SetDragMode(Value);
  if Value <> fDBEdit.DragMode then
    fDBEdit.DragMode := Value;
end;

function TabcDBEditLabel.GetHint: string;
begin
  Result := fDBEdit.Hint;
end;

procedure TabcDBEditLabel.SetHint(Value: string);
begin
  if Value <> fDBEdit.Hint then
    fDBEdit.Hint := Value;
end;

function TabcDBEditLabel.GetParentShowHint: Boolean;
begin
  Result := fDBEdit.ParentShowHint;
end;

procedure TabcDBEditLabel.SetParentShowHint(Value: Boolean);
begin
  if Value <> fDBEdit.ParentShowHint then
    fDBEdit.ParentShowHint := Value;
end;

function TabcDBEditLabel.GetShowHint: Boolean;
begin
  Result := fDBEdit.ShowHint;
end;

procedure TabcDBEditLabel.SetShowHint(Value: Boolean);
begin
  if Value <> fDBEdit.ShowHint then begin
    ParentShowHint := False;
    fDBEdit.ShowHint := Value;
  end;
end;

function TabcDBEditLabel.GetPopupMenu: TPopupMenu;
begin
  Result := fDBEdit.PopupMenu;
end;

function TabcDBEditLabel.GetEditPopupMenu: TPopupMenu;
begin
  Result := fDBEdit.PopupMenu;
end;

procedure TabcDBEditLabel.SetPopupMenu(Value: TPopupMenu);
begin
  if Value <> fDBEdit.PopupMenu then
    fDBEdit.PopupMenu := Value;
end;

function TabcDBEditLabel.GetDataField: string;
begin
  Result := fDBEdit.DataField;
end;

procedure TabcDBEditLabel.SetDataField(Value: string);
begin
  if fDBEdit.DataField <> Value then
    fDBEdit.DataField := Value;
  SetCaptionToField;
end;

function TabcDBEditLabel.GetDataSource: TDataSource;
begin
  Result := fDBEdit.DataSource;
end;

procedure TabcDBEditLabel.SetDataSource(Value: TDataSource);
begin
  if fDBEdit.DataSource <> Value then
    fDBEdit.DataSource := Value;
end;

function TabcDBEditLabel.GetField: TField;
begin
  Result := fDBEdit.Field;
end;

procedure TabcDBEditLabel.ValidateEdit;
begin
  fDBEdit.ValidateEdit;
end;

function TabcDBEditLabel.GetTextLen: Integer;
begin
  Result := fDBEdit.GetTextLen;
end;

function TabcDBEditLabel.GetMasked: Boolean;
begin
  Result := fDBEdit.IsMasked;
end;

function TabcDBEditLabel.GetEditText: string;
begin
  Result := fDBEdit.EditText;
end;

procedure TabcDBEditLabel.SetEditText(Value: string);
begin
  fDBEdit.EditText := Value;
end;

function TabcDBEditLabel.GetText: string;
begin
  Result := fDBEdit.Text;
end;

procedure TabcDBEditLabel.SetText(Value: string);
begin
  fDBEdit.Text := Value;
end;

procedure TabcDBEditLabel.SetCaptionToField;
begin
  if FCaptionAuto then
    if Field <> nil then
      fLabel.Caption := Field.DisplayName;
end;

procedure TabcDBEditLabel.ActiveChange(Sender: TObject);
begin
  SetCaptiontoField;
end;

procedure TabcDBEditLabel.CMGetDataLink(var Message: TMessage);
begin
  with Message do
    Result := fDBEdit.Perform(Msg, WParam, LParam);
end;

function TabcDBEditLabel.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := fDBEdit.ExecuteAction(Action);
end;

function TabcDBEditLabel.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := fDBEdit.UpdateAction(Action);
end;

function TabcDBEditLabel.UseRightToLeftAlignment: Boolean;
begin
  Result := DBUseRightToLeftAlignment(Self, Field);
end;

end.
