unit abcdbcur;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  DBCtrls,
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, Menus, abccurlb, DB;
type

  { TabcDBCurrencyLabel }

  TabcDBCurrencyLabel = class(TabcCustomCurrencyLabel)
  private
    FDataLink: TFieldDataLink;
    procedure DataChange(Sender: TObject);
    function GetCaption: string;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetValue: Double;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    function GetLabelText: string; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    property Caption: string read GetCaption;
    property Field: TField read GetField;
    property Value: Double read GetValue;
  published
    property CurrFractionFormat;
    property CurrFractionText;
    property CurrText;
    property FirstUpperCase;
    property HandleException;
    property Align;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property Alignment;
    property AutoSize default False;
    property Color;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Transparent;
    property ShowHint;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnEndDock;
    property OnStartDock;
  end;

implementation

uses DBConsts;

resourcestring
  sabcCurrTextDBNull = '#NULL#';

{ TabcDBCurrencyLabel }

constructor TabcDBCurrencyLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AutoSize := False;
  ShowAccelChar := False;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
end;

destructor TabcDBCurrencyLabel.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TabcDBCurrencyLabel.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

function TabcDBCurrencyLabel.GetLabelText: string;
begin
  if not (csPaintCopy in ControlState) then
    Result := inherited GetLabelText
  else
    if FDataLink.Field <> nil then
      if FDataLink.Field.IsNull then
        Result := sabcCurrTextDBNull
      else
        Result := abcNumToWords(FDataLink.Field.AsFloat, CurrText, CurrFractionText,
          CurrFractionFormat, FirstUpperCase, HandleException)
    else
      Result := abcNumToWords(0, CurrText, CurrFractionText,
        CurrFractionFormat, FirstUpperCase, HandleException);
end;

function TabcDBCurrencyLabel.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcDBCurrencyLabel.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

function TabcDBCurrencyLabel.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TabcDBCurrencyLabel.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TabcDBCurrencyLabel.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TabcDBCurrencyLabel.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
    if FDataLink.Field.IsNull then begin
      inherited Value := 0;
      inherited Caption := sabcCurrTextDBNull;
    end
    else
      inherited Value := FDataLink.Field.AsFloat
  else
    inherited Value := 0;
end;

function TabcDBCurrencyLabel.GetCaption: string;
begin
  Result := inherited Caption;
end;

function TabcDBCurrencyLabel.GetValue: Double;
begin
  Result := inherited Value;
end;

procedure TabcDBCurrencyLabel.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

function TabcDBCurrencyLabel.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TabcDBCurrencyLabel.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

end.
