unit abcdbdtp;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, CommCtrl, DB, DBCtrls;

type

{ following is a copy of Delphi TCommonCalendar, TMonthCalendar and TDateTimePicker
  with some method and property access changes needed:
    - added TabcCommonCalendar.SetSelectedDateRange
    - made published properties protected so selected properties could be left out of final classes
    - fixed calendar autosizing
    - added custom formatting options
}

  TabcCommonCalendar = class;

  TabcMonthCalColors = class(TPersistent)
  private
    Owner: TabcCommonCalendar;
    FBackColor: TColor;
    FTextColor: TColor;
    FTitleBackColor: TColor;
    FTitleTextColor: TColor;
    FMonthBackColor: TColor;
    FTrailingTextColor: TColor;
    procedure SetColor(Index: Integer; Value: TColor);
    procedure SetAllColors;
  public
    constructor Create(AOwner: TabcCommonCalendar);
    procedure Assign(Source: TPersistent); override;
  published
    property BackColor: TColor index 0 read FBackColor write SetColor default clWindow;
    property TextColor: TColor index 1 read FTextColor write SetColor default clWindowText;
    property TitleBackColor: TColor index 2 read FTitleBackColor write SetColor default clActiveCaption;
    property TitleTextColor: TColor index 3 read FTitleTextColor write SetColor default clWhite;
    property MonthBackColor: TColor index 4 read FMonthBackColor write SetColor default clWhite;
    property TrailingTextColor: TColor index 5 read FTrailingTextColor
      write SetColor default clInactiveCaptionText;
  end;

  TabcCommonCalendar = class(TWinControl)
  private
    FCalColors: TabcMonthCalColors;
    FCalExceptionClass: ExceptClass;
    FDateTime: TDateTime;
    FEndDate: TDate;
    FFirstDayOfWeek: TCalDayOfWeek;
    FMaxDate: TDate;
    FMaxSelectRange: Integer;
    FMinDate: TDate;
    FMonthDelta: Integer;
    FMultiSelect: Boolean;
    FShowToday: Boolean;
    FShowTodayCircle: Boolean;
    FWeekNumbers: Boolean;
    FOnGetMonthInfo: TOnGetMonthInfoEvent;
    function DoStoreEndDate: Boolean;
    function DoStoreMaxDate: Boolean;
    function DoStoreMinDate: Boolean;
    function GetDate: TDate;
    procedure SetCalColors(Value: TabcMonthCalColors);
    procedure SetDate(Value: TDate);
    procedure SetDateTime(Value: TDateTime);
    procedure SetEndDate(Value: TDate);
    procedure SetFirstDayOfWeek(Value: TCalDayOfWeek);
    procedure SetMaxDate(Value: TDate);
    procedure SetMaxSelectRange(Value: Integer);
    procedure SetMinDate(Value: TDate);
    procedure SetMonthDelta(Value: Integer);
    procedure SetMultiSelect(Value: Boolean);
    procedure SetRange(MinVal, MaxVal: TDate);
    procedure SetSelectedRange(Date, EndDate: TDate);
    procedure SetShowToday(Value: Boolean);
    procedure SetShowTodayCircle(Value: Boolean);
    procedure SetWeekNumbers(Value: Boolean);
    procedure AutoAdjustSize;
  protected
    procedure CheckEmptyDate; virtual;
    procedure CheckValidDate(Value: TDate); virtual;
    procedure CreateWnd; override;
    function GetCalendarHandle: HWND; virtual; abstract;
    function GetCalStyles: DWORD; virtual;
    function MsgSetCalColors(ColorIndex: Integer; ColorValue: TColor): Boolean; virtual; abstract;
    function MsgSetDateTime(Value: TSystemTime): Boolean; virtual; abstract;
    function MsgSetRange(Flags: Integer; SysTime: PSystemTime): Boolean; virtual; abstract;
    property CalColors: TabcMonthCalColors read FCalColors write SetCalColors;
    property CalendarHandle: HWND read GetCalendarHandle;
    property CalExceptionClass: ExceptClass read FCalExceptionClass write FCalExceptionClass;
    property Date: TDate read GetDate write SetDate;
    property DateTime: TDateTime read FDateTime write SetDateTime;
    property EndDate: TDate read FEndDate write SetEndDate stored DoStoreEndDate;
    property FirstDayOfWeek: TCalDayOfWeek read FFirstDayOfWeek write SetFirstDayOfWeek
      default dowLocaleDefault;
    property MaxDate: TDate read FMaxDate write SetMaxDate stored DoStoreMaxDate;
    property MaxSelectRange: Integer read FMaxSelectRange write SetMaxSelectRange default 31;
    property MinDate: TDate read FMinDate write SetMinDate stored DoStoreMinDate;
    property MonthDelta: Integer read FMonthDelta write SetMonthDelta default 1;
    property MultiSelect: Boolean read FMultiSelect write SetMultiSelect default False;
    property ShowToday: Boolean read FShowToday write SetShowToday default True;
    property ShowTodayCircle: Boolean read FShowTodayCircle write
      SetShowTodayCircle default True;
    property WeekNumbers: Boolean read FWeekNumbers write SetWeekNumbers default False;
    property OnGetMonthInfo: TOnGetMonthInfoEvent read FOnGetMonthInfo write FOnGetMonthInfo;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BoldDays(Days: array of LongWord; var MonthBoldInfo: LongWord);
    procedure SetSelectedDateRange(ADate, AnEndDate: TDate);
  end;

  TabcCustomMonthCalendar = class(TabcCommonCalendar)
  private
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
  protected
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
    procedure ConstrainedResize(var MinWidth, MinHeight, MaxWidth,
      MaxHeight: Integer); override;
    procedure CreateParams(var Params: TCreateParams); override;
    function GetCalendarHandle: HWND; override;
    function MsgSetCalColors(ColorIndex: Integer; ColorValue: TColor): Boolean; override;
    function MsgSetDateTime(Value: TSystemTime): Boolean; override;
    function MsgSetRange(Flags: Integer; SysTime: PSystemTime): Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TabcFormatDateEvent = procedure(Sender: TObject; Value: TDateTime;
    var FormatString: string) of object;

  TabcDateTimeKind = (adkDate, adkTime, adkDateTime);

  TabcCustomDateTimePicker = class(TabcCommonCalendar)
  private
    FCalAlignment: TDTCalAlignment;
    FChanging: Boolean;
    FChecked: Boolean;
    FDateFormat: TDTDateFormat;
    FDateMode: TDTDateMode;
    FDroppedDown: Boolean;
    FDateTimeKind: TabcDateTimeKind;
    FLastChange: TSystemTime;
    FParseInput: Boolean;
    FShowCheckbox: Boolean;
    FOnUserInput: TDTParseInputEvent;
    FOnCloseUp: TNotifyEvent;
    FOnChange: TNotifyEvent;
    FOnDropDown: TNotifyEvent;
    FDateFormatString: string;
    FDateFormatZero: string;
    FShowingZeroDate: Boolean;
    FLastFormatString: string;
    FOnFormatDate: TabcFormatDateEvent;
    FDefaultDate: TDateTime;
    procedure AdjustHeight;
    function GetTime: TTime;
    procedure SetCalAlignment(Value: TDTCalAlignment);
    procedure SetChecked(Value: Boolean);
    procedure SetDateMode(Value: TDTDateMode);
    procedure SetDateFormat(Value: TDTDateFormat);
    procedure SetDateTimeKind(Value: TabcDateTimeKind);
    procedure SetParseInput(Value: Boolean);
    procedure SetShowCheckbox(Value: Boolean);
    procedure SetTime(Value: TTime);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
    procedure SetDateFormatString(const Value: string);
    procedure SetDateFormatZero(const Value: string);
    function GetDateFormatString: string;
    procedure ReadOldKindProperty(Reader: TReader);
  protected
    procedure CheckEmptyDate; override;
    procedure CheckValidDate(Value: TDate); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DefineProperties(Filer: TFiler); override;
    procedure Change; dynamic;
    procedure GetDisplayFormat(Value: TSystemTime); dynamic;
    procedure SetDefaultDropDownDate; dynamic;
    function GetCalendarHandle: HWND; override;
    function MsgSetCalColors(ColorIndex: Integer; ColorValue: TColor): Boolean; override;
    function MsgSetDateTime(Value: TSystemTime): Boolean; override;
    function MsgSetRange(Flags: Integer; SysTime: PSystemTime): Boolean; override;

    property CalAlignment: TDTCalAlignment read FCalAlignment write SetCalAlignment;
    property Time: TTime read GetTime write SetTime;
    property ShowCheckbox: Boolean read FShowCheckbox write SetShowCheckbox default False;
    property Checked: Boolean read FChecked write SetChecked default True;
    property DateFormat: TDTDateFormat read FDateFormat write SetDateFormat;
    property DateFormatString: string read FDateFormatString write SetDateFormatString;
    property DateFormatZero: string read FDateFormatZero write SetDateFormatZero;
    property DateMode: TDTDateMode read FDateMode write SetDateMode;
    property DefaultDate: TDateTime read FDefaultDate write FDefaultDate;
    property DateTimeKind: TabcDateTimeKind read FDateTimeKind write SetDateTimeKind;
    property ParseInput: Boolean read FParseInput write SetParseInput;
    property OnCloseUp: TNotifyEvent read FOnCloseUp write FOnCloseUp;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnDropDown: TNotifyEvent read FOnDropDown write FOnDropDown;
    property OnFormatDate: TabcFormatDateEvent read FOnFormatDate write FOnFormatDate;
    property OnUserInput: TDTParseInputEvent read FOnUserInput write FOnUserInput;

  public
    constructor Create(AOwner: TComponent); override;
    property DateTime;
    property DroppedDown: Boolean read FDroppedDown;
  end;

{ TabcDateTimeDataLink }

  TabcDateTimeDataLink = class(TDataLink)
  private
    FField1, FField2: TField;
    FFieldName1, FFieldName2: string;
    FControl: TComponent;
    FEditing: Boolean;
    FModified: Boolean;
    FOnDataChange: TNotifyEvent;
    FOnEditingChange: TNotifyEvent;
    FOnUpdateData: TNotifyEvent;
    FOnActiveChange: TNotifyEvent;
    function GetCanModify: Boolean;
    procedure SetEditing(Value: Boolean);
    procedure SetField1(Value: TField);
    procedure SetField2(Value: TField);
    procedure SetFieldName1(const Value: string);
    procedure SetFieldName2(const Value: string);
    procedure UpdateField1;
    procedure UpdateField2;
  protected
    procedure ActiveChanged; override;
    procedure EditingChanged; override;
    procedure FocusControl(Field: TFieldRef); override;
    procedure LayoutChanged; override;
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
  public
    constructor Create;
    function Edit: Boolean;
    procedure Modified;
    procedure Reset;
    property CanModify: Boolean read GetCanModify;
    property Control: TComponent read FControl write FControl;
    property Editing: Boolean read FEditing;
    property Field1: TField read FField1;
    property Field2: TField read FField2;
    property FieldName1: string read FFieldName1 write SetFieldName1;
    property FieldName2: string read FFieldName2 write SetFieldName2;
    property OnDataChange: TNotifyEvent read FOnDataChange write FOnDataChange;
    property OnEditingChange: TNotifyEvent read FOnEditingChange write FOnEditingChange;
    property OnUpdateData: TNotifyEvent read FOnUpdateData write FOnUpdateData;
    property OnActiveChange: TNotifyEvent read FOnActiveChange write FOnActiveChange;
  end;

  { TabcDBDateTimePicker }

  TabcDBDateTimePicker = class(TabcCustomDateTimePicker)
  private
    FDataLink: TabcDateTimeDataLink;
    FChanging: Boolean;
    function GetDataField: string;
    function GetDataFieldCheckBox: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetFieldCheckBox: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataFieldCheckBox(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure ActiveChanged(Sender: TObject);
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    procedure Change; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    property Field: TField read GetField;
    property FieldCheckBox: TField read GetFieldCheckBox;
    property Date;
    property Time;
    property Checked;
  published
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property Constraints;
    property DragKind;
    property CalAlignment;
    property CalColors;
    property Color stored True default clWindow;
    property DateFormat;
    property DateMode;
    property DateTimeKind default adkDate;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property MaxDate;
    property MinDate;
    property ParseInput;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnCloseUp;
    property OnChange;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
    property OnDropDown;
    property OnDblClick;
    property OnStartDock;
    property OnEndDock;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDrag;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnUserInput;
    property DataField: string read GetDataField write SetDataField;
    property DataFieldCheckBox: string read GetDataFieldCheckBox write SetDataFieldCheckBox;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DateFormatString;
    property DateFormatZero;
    property DefaultDate;
    property OnFormatDate;
  end;

  { TabcDBMonthCalendar }

  TabcDBMonthCalendar = class(TabcCustomMonthCalendar)
  private
    FDataLink: TabcDateTimeDataLink;
    FOnChange: TNotifyEvent;
    FChanging: Boolean;
    function GetDataField: string;
    function GetDataFieldEndDate: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetFieldEndDate: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataFieldEndDate(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure ActiveChanged(Sender: TObject);
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Change; dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    property Field: TField read GetField;
    property FieldEndDate: TField read GetFieldEndDate;
    property Date;
    property EndDate;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property Constraints;
    property DragKind;
    property AutoSize;
    property BorderWidth;
    property CalColors;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FirstDayOfWeek;
    property Font;
    property ImeMode;
    property ImeName;
    property MaxDate;
    property MaxSelectRange;
    property MinDate;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property ShowToday;
    property ShowTodayCircle;
    property TabOrder;
    property TabStop;
    property Visible;
    property WeekNumbers;
    property OnClick;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDock;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetMonthInfo;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    property DataField: string read GetDataField write SetDataField;
    property DataFieldEndDate: string read GetDataFieldEndDate write SetDataFieldEndDate;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

implementation

uses Consts, {$IFDEF DELPHI6}RTLConsts,{$ENDIF} ComStrs;

{ from SysUtils }
{$IFNDEF DELPHI5}
procedure ReplaceTime(var DateTime: TDateTime; const NewTime: TDateTime);
begin
  DateTime := Trunc(DateTime);
  if DateTime >= 0 then
    DateTime := DateTime + Abs(Frac(NewTime))
  else
    DateTime := DateTime - Abs(Frac(NewTime));
end;

procedure ReplaceDate(var DateTime: TDateTime; const NewDate: TDateTime);
var
  Temp: TDateTime;
begin
  Temp := NewDate;
  ReplaceTime(Temp, DateTime);
  DateTime := Temp;
end;
{$ENDIF}

{ from Delphi ComCtrls }

procedure SetComCtlStyle(Ctl: TWinControl; Value: Integer; UseStyle: Boolean);
var
  Style: Integer;
begin
  if Ctl.HandleAllocated then
  begin
    Style := GetWindowLong(Ctl.Handle, GWL_STYLE);
    if not UseStyle then Style := Style and not Value
    else Style := Style or Value;
    SetWindowLong(Ctl.Handle, GWL_STYLE, Style);
  end;
end;

{ TabcMonthCalColors }

const
  ColorIndex: array[0..5] of Integer = (MCSC_BACKGROUND, MCSC_TEXT,
    MCSC_TITLEBK, MCSC_TITLETEXT, MCSC_MONTHBK, MCSC_TRAILINGTEXT);

constructor TabcMonthCalColors.Create(AOwner: TabcCommonCalendar);
begin
  Owner := AOwner;
  FBackColor := clWindow;
  FTextColor := clWindowText;
  FTitleBackColor := clActiveCaption;
  FTitleTextColor := clWhite;
  FMonthBackColor := clWhite;
  FTrailingTextColor := clInactiveCaptionText;
end;

procedure TabcMonthCalColors.Assign(Source: TPersistent);
var
  SourceName: string;
begin
  if Source = nil then SourceName := 'nil'
  else SourceName := Source.ClassName;
  if (Source = nil) or not (Source is TabcMonthCalColors) then
    raise EConvertError.CreateFmt(SAssignError, [SourceName, ClassName]);
  FBackColor := TabcMonthCalColors(Source).BackColor;
  FTextColor := TabcMonthCalColors(Source).TextColor;
  FTitleBackColor := TabcMonthCalColors(Source).TitleBackColor;
  FTitleTextColor := TabcMonthCalColors(Source).TitleTextColor;
  FMonthBackColor := TabcMonthCalColors(Source).MonthBackColor;
  FTrailingTextColor := TabcMonthCalColors(Source).TrailingTextColor;
end;

procedure TabcMonthCalColors.SetColor(Index: Integer; Value: TColor);
begin
  case Index of
    0: FBackColor := Value;
    1: FTextColor := Value;
    2: FTitleBackColor := Value;
    3: FTitleTextColor := Value;
    4: FMonthBackColor := Value;
    5: FTrailingTextColor := Value;
  end;
  if Owner.HandleAllocated then
    Owner.MsgSetCalColors(ColorIndex[Index], ColorToRGB(Value));
end;

procedure TabcMonthCalColors.SetAllColors;
begin
  SetColor(0, FBackColor);
  SetColor(1, FTextColor);
  SetColor(2, FTitleBackColor);
  SetColor(3, FTitleTextColor);
  SetColor(4, FMonthBackColor);
  SetColor(5, FTrailingTextColor);
end;

{ TCommonCalendar }

constructor TabcCommonCalendar.Create(AOwner: TComponent);
begin
  CheckCommonControl(ICC_DATE_CLASSES);
  inherited Create(AOwner);
  FShowToday := True;
  FShowTodayCircle := True;
  ControlStyle := [csOpaque, csClickEvents, csDoubleClicks, csReflector];
  FCalColors := TabcMonthCalColors.Create(Self);
  FDateTime := Now;
  FFirstDayOfWeek := dowLocaleDefault;
  FMaxSelectRange := 31;
  FMonthDelta := 1;
end;

destructor TabcCommonCalendar.Destroy;
begin
  inherited Destroy;
  FCalColors.Free;
end;

procedure TabcCommonCalendar.BoldDays(Days: array of LongWord; var MonthBoldInfo: LongWord);
var
  I: LongWord;
begin
  MonthBoldInfo := 0;
  for I := Low(Days) to High(Days) do
    if (Days[I] > 0) and (Days[I] < 32) then
      MonthBoldInfo := MonthBoldInfo or ($00000001 shl (Days[I] - 1));
end;

procedure TabcCommonCalendar.CheckEmptyDate;
begin
  // do nothing
end;

procedure TabcCommonCalendar.CheckValidDate(Value: TDate);
begin
  if (FMaxDate <> 0.0) and (Value > FMaxDate) then
    raise CalExceptionClass.CreateFmt(SDateTimeMax, [DateToStr(FMaxDate)]);
  if (FMinDate <> 0.0) and (Value < FMinDate) then
    raise CalExceptionClass.CreateFmt(SDateTimeMin, [DateToStr(FMinDate)]);
end;

procedure TabcCommonCalendar.CreateWnd;
begin
  inherited CreateWnd;
  FCalColors.SetAllColors;
  SetRange(FMinDate, FMaxDate);
  SetMaxSelectRange(FMaxSelectRange);
  SetMonthDelta(FMonthDelta);
  SetFirstDayOfWeek(FFirstDayOfWeek);
  if FMultiSelect then
    SetSelectedRange(FDateTime, FEndDate)
  else
    SetDateTime(FDateTime);
end;

function TabcCommonCalendar.GetCalStyles: DWORD;
const
  ShowTodayFlags: array[Boolean] of DWORD = (MCS_NOTODAY, 0);
  ShowTodayCircleFlags: array[Boolean] of DWORD = (MCS_NOTODAYCIRCLE, 0);
  WeekNumFlags: array[Boolean] of DWORD = (0, MCS_WEEKNUMBERS);
  MultiSelFlags: array[Boolean] of DWORD = (0, MCS_MULTISELECT);
begin
  Result := MCS_DAYSTATE or ShowTodayFlags[FShowToday] or
    ShowTodayCircleFlags[FShowTodayCircle] or WeekNumFlags[FWeekNumbers] or
    MultiSelFlags[FMultiSelect];
end;

function TabcCommonCalendar.DoStoreEndDate: Boolean;
begin
  Result := FMultiSelect;
end;

function TabcCommonCalendar.DoStoreMaxDate: Boolean;
begin
  Result := FMaxDate <> 0.0;
end;

function TabcCommonCalendar.DoStoreMinDate: Boolean;
begin
  Result := FMinDate <> 0.0;
end;

function TabcCommonCalendar.GetDate: TDate;
begin
  Result := TDate(FDateTime);
end;

procedure TabcCommonCalendar.SetCalColors(Value: TabcMonthCalColors);
begin
  if FCalColors <> Value then FCalColors.Assign(Value);
end;

procedure TabcCommonCalendar.SetDate(Value: TDate);
begin
  ReplaceTime(TDateTime(Value), FDateTime);
  if Value = 0.0 then
    CheckEmptyDate;
  try
    CheckValidDate(Trunc(Value));
    SetDateTime(Value);
  except
    SetDateTime(FDateTime);
    raise;
  end;
end;

procedure TabcCommonCalendar.SetDateTime(Value: TDateTime);
var
  ST: TSystemTime;
begin
  DateTimeToSystemTime(Value, ST);
  if FMultiSelect then
    SetSelectedRange(Value, FEndDate)
  else begin
    if HandleAllocated then
      if not MsgSetDateTime(ST) then
        raise ECommonCalendarError.Create(sFailSetCalDateTime);
    FDateTime := Value;
  end;
end;

procedure TabcCommonCalendar.SetEndDate(Value: TDate);
var
  TruncValue: TDate;
begin
  TruncValue := Trunc(Value);
  if Trunc(FEndDate) <> TruncValue then
  begin
    Value := TruncValue + 0.0;
    if Value = 0.0 then CheckEmptyDate;
    SetSelectedRange(Date, TruncValue);
  end;
end;

procedure TabcCommonCalendar.SetSelectedDateRange(ADate, AnEndDate: TDate);
var
  TruncValue: TDate;
begin
  try
    ReplaceTime(TDateTime(ADate), FDateTime);
    if ADate = 0.0 then CheckEmptyDate;
    CheckValidDate(Trunc(ADate));
    TruncValue := Trunc(AnEndDate);
    if Trunc(FEndDate) <> TruncValue then
    begin
      AnEndDate := TruncValue + 0.0;
      if AnEndDate = 0.0 then CheckEmptyDate;
    end;
    CheckValidDate(Trunc(AnEndDate));
    SetSelectedRange(ADate, AnEndDate);
  except
    SetSelectedRange(FDateTime, FEndDate);
    raise;
  end;
end;

procedure TabcCommonCalendar.SetFirstDayOfWeek(Value: TCalDayOfWeek);
var
  DOWFlag: Integer;
  A: array[0..1] of char;
begin
  if HandleAllocated then
  begin
    if Value = dowLocaleDefault then
    begin
      GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IFIRSTDAYOFWEEK, A, SizeOf(A));
      DOWFlag := Ord(A[0]) - Ord('0');
    end
    else
      DOWFlag := Ord(Value);
    if CalendarHandle <> 0 then
      MonthCal_SetFirstDayOfWeek(CalendarHandle, DOWFlag);
  end;
  FFirstDayOfWeek := Value;
end;

procedure TabcCommonCalendar.SetMaxDate(Value: TDate);
begin
  if (FMinDate <> 0.0) and (Value < FMinDate) then
    raise CalExceptionClass.CreateFmt(SDateTimeMin, [DateToStr(FMinDate)]);
  if FMaxDate <> Value then
  begin
    SetRange(FMinDate, Value);
    FMaxDate := Value;
  end;
end;

procedure TabcCommonCalendar.SetMaxSelectRange(Value: Integer);
begin
  if FMultiSelect and HandleAllocated then
    if not MonthCal_SetMaxSelCount(CalendarHandle, Value) then
      raise ECommonCalendarError.Create(sFailSetCalMaxSelRange);
  FMaxSelectRange := Value;
end;

procedure TabcCommonCalendar.SetMinDate(Value: TDate);
begin
  if (FMaxDate <> 0.0) and (Value > FMaxDate) then
    raise CalExceptionClass.CreateFmt(SDateTimeMax, [DateToStr(FMaxDate)]);
  if FMinDate <> Value then
  begin
    SetRange(Value, FMaxDate);
    FMinDate := Value;
  end;
end;

procedure TabcCommonCalendar.SetMonthDelta(Value: Integer);
begin
  if HandleAllocated and (CalendarHandle <> 0) then
    MonthCal_SetMonthDelta(CalendarHandle, Value);
  FMonthDelta := Value;
end;

procedure TabcCommonCalendar.SetMultiSelect(Value: Boolean);
begin
  if FMultiSelect <> Value then
  begin
    FMultiSelect := Value;
    if Value then FEndDate := FDateTime
    else FEndDate := 0.0;
    RecreateWnd;
  end;
end;

procedure TabcCommonCalendar.SetRange(MinVal, MaxVal: TDate);
var
  STA: packed array[1..2] of TSystemTime;
  Flags: DWORD;
  TruncDate, TruncEnd, TruncMin, TruncMax: Int64;
begin
  Flags := 0;
  TruncMin := Trunc(MinVal);
  TruncMax := Trunc(MaxVal);
  TruncDate := Trunc(FDateTime);
  TruncEnd := Trunc(FEndDate);
  if TruncMin <> 0 then
  begin
    if TruncDate < TruncMin then SetDate(MinVal);
    if TruncEnd < TruncMin then SetEndDate(MinVal);
    Flags := Flags or GDTR_MIN;
    DateTimeToSystemTime(TruncMin, STA[1]);
  end;
  if TruncMax <> 0 then
  begin
    if TruncDate > TruncMax then SetDate(MaxVal);
    if TruncEnd > TruncMax then SetEndDate(MaxVal);
    Flags := Flags or GDTR_MAX;
    DateTimeToSystemTime(TruncMax, STA[2]);
  end;
  if HandleAllocated then
    if not MsgSetRange(Flags, @STA[1]) then
      raise ECommonCalendarError.Create(sFailSetCalMinMaxRange);
end;

procedure TabcCommonCalendar.SetSelectedRange(Date, EndDate: TDate);
var
  DateArray: array[1..2] of TSystemTime;
begin
  if not FMultiSelect then
    SetDateTime(Date)
  else begin
    DateTimeToSystemTime(Date, DateArray[1]);
    DateTimeToSystemTime(EndDate, DateArray[2]);
    if HandleAllocated then
      if not MonthCal_SetSelRange(Handle, @DateArray[1]) then
        raise ECommonCalendarError.Create(sFailsetCalSelRange);
    FDateTime := Date;
    FEndDate := EndDate;
  end;
end;

procedure TabcCommonCalendar.SetShowToday(Value: Boolean);
begin
  if FShowToday <> Value then
  begin
    FShowToday := Value;
    SetComCtlStyle(Self, MCS_NOTODAY, not Value);
    AutoAdjustSize;
  end;
end;

procedure TabcCommonCalendar.SetShowTodayCircle(Value: Boolean);
begin
  if FShowTodayCircle <> Value then
  begin
    FShowTodayCircle := Value;
    SetComCtlStyle(Self, MCS_NOTODAYCIRCLE, not Value);
    AutoAdjustSize;
  end;
end;

procedure TabcCommonCalendar.SetWeekNumbers(Value: Boolean);
begin
  if FWeekNumbers <> Value then
  begin
    FWeekNumbers := Value;
    SetComCtlStyle(Self, MCS_WEEKNUMBERS, Value);
    AutoAdjustSize;
  end;
end;

procedure TabcCommonCalendar.AutoAdjustSize;
var
  NewWidth, NewHeight: Integer;
begin
  if AutoSize then
  begin
    NewWidth := 0;
    NewHeight := 0;
    if CanAutoSize(NewWidth, NewHeight) then
      SetBounds(Left, Top, NewWidth, NewHeight);
  end;
end;

function IsBlankSysTime(const ST: TSystemTime): Boolean;
type
  TFast = array [0..3] of DWORD;
begin
  Result := (TFast(ST)[0] or TFast(ST)[1] or TFast(ST)[2] or TFast(ST)[3]) = 0;
end;

{ TabcCustomMonthCalendar }

constructor TabcCustomMonthCalendar.Create(AOwner: TComponent);
begin
  FCalExceptionClass := EMonthCalError;
  inherited Create(AOwner);
  Width := 191;
  Height := 154;
end;

procedure TabcCustomMonthCalendar.CMFontChanged(var Message: TMessage);
begin
  inherited;
  if HandleAllocated then Perform(WM_SIZE, 0, 0);
end;

procedure TabcCustomMonthCalendar.CNNotify(var Message: TWMNotify);
var
  ST: PSystemTime;
  I, MonthNo: Integer;
  CurState: PMonthDayState;
begin
  with Message, NMHdr^ do
  begin
    case code of
      MCN_GETDAYSTATE:
        with PNmDayState(NMHdr)^ do
        begin
          FillChar(prgDayState^, cDayState * SizeOf(TMonthDayState), 0);
          if Assigned(FOnGetMonthInfo) then
          begin
            CurState := prgDayState;
            for I := 0 to cDayState - 1 do
            begin
              MonthNo := stStart.wMonth + I;
              if MonthNo > 12 then MonthNo := MonthNo - 12;
              FOnGetMonthInfo(Self, MonthNo, CurState^);
              Inc(CurState);
            end;
          end;
        end;
      MCN_SELECT, MCN_SELCHANGE:
        begin
          ST := @PNMSelChange(NMHdr).stSelStart;
          if not IsBlankSysTime(ST^) then
            FDateTime := SystemTimeToDateTime(ST^);
          if FMultiSelect then
          begin
            ST := @PNMSelChange(NMHdr).stSelEnd;
            if not IsBlankSysTime(ST^) then
              FEndDate := SystemTimeToDateTime(ST^);
          end;
        end;
    end;
  end;
  inherited;
end;

procedure TabcCustomMonthCalendar.ConstrainedResize(var MinWidth, MinHeight, MaxWidth,
  MaxHeight: Integer);
var
  R: TRect;
  CtlMinWidth, CtlMinHeight: Integer;
begin
  if HandleAllocated then
  begin
    MonthCal_GetMinReqRect(Handle, R);
    with R do
    begin
      CtlMinHeight := Bottom - Top;
      CtlMinWidth := Right - Left;
    end;
    if MinHeight < CtlMinHeight then MinHeight := CtlMinHeight;
    if MinWidth < CtlMinWidth then MinWidth := CtlMinWidth;
  end;
  inherited ConstrainedResize(MinWidth, MinHeight, MaxWidth, MaxHeight);
end;

procedure TabcCustomMonthCalendar.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, MONTHCAL_CLASS);
  with Params do
  begin
    Style := Style or GetCalStyles;
    WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW) or
      CS_DBLCLKS;
  end;
end;

function TabcCustomMonthCalendar.GetCalendarHandle: HWND;
begin
  Result := Handle;
end;

function TabcCustomMonthCalendar.MsgSetCalColors(ColorIndex: Integer; ColorValue: TColor): Boolean;
begin
  Result := True;
  if HandleAllocated then
    Result := MonthCal_SetColor(Handle, ColorIndex, ColorValue) <> DWORD($FFFFFFFF);
end;

function TabcCustomMonthCalendar.MsgSetDateTime(Value: TSystemTime): Boolean;
begin
  Result := True;
  if HandleAllocated then
    Result := MonthCal_SetCurSel(Handle, Value);
end;

function TabcCustomMonthCalendar.MsgSetRange(Flags: Integer; SysTime: PSystemTime): Boolean;
begin
  Result := True;
  if HandleAllocated then
    if Flags <> 0 then Result := MonthCal_SetRange(Handle, Flags, SysTime);
end;

function TabcCustomMonthCalendar.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
var
  R: TRect;
begin
  if HandleAllocated then
  begin
    Result := True;
    Perform(MCM_GETMINREQRECT, 0, Longint(@R));
    with R do
    begin
      NewWidth := Right - Left;
      NewHeight := Bottom - Top;
    end;
  end
  else Result := False;
end;

{ TabcCustomDateTimePicker }

constructor TabcCustomDateTimePicker.Create(AOwner: TComponent);
begin
  FCalExceptionClass := EDateTimeError;
  FChanging := False;
  inherited Create(AOwner);
  DateTimeToSystemTime(FDateTime, FLastChange);
  FShowCheckbox := False;
  FChecked := True;
  ControlStyle := ControlStyle + [csFixedHeight, csReflector];
  Color := clWindow;
  ParentColor := False;
  TabStop := True;
  Width := 186;
  AdjustHeight;
end;

procedure TabcCustomDateTimePicker.AdjustHeight;
var
  DC: HDC;
  SaveFont: HFont;
  SysMetrics, Metrics: TTextMetric;
begin
  DC := GetDC(0);
  try
    GetTextMetrics(DC, SysMetrics);
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
  finally
    ReleaseDC(0, DC);
  end;
  Height := Metrics.tmHeight + (GetSystemMetrics(SM_CYBORDER) * 8);
end;

procedure TabcCustomDateTimePicker.CheckEmptyDate;
begin
  if not FShowCheckbox then
    raise EDateTimeError.Create(SNeedAllowNone);
  FChecked := False;
  Invalidate;
end;

procedure TabcCustomDateTimePicker.CheckValidDate(Value: TDate);
begin
  inherited;
end;

procedure TabcCustomDateTimePicker.CreateParams(var Params: TCreateParams);
const
  Formats: array[TDTDateFormat] of DWORD = (DTS_SHORTDATEFORMAT,
    DTS_LONGDATEFORMAT);
var
  ACalAlignment: TDTCalAlignment;
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, DATETIMEPICK_CLASS);
  with Params do
  begin
    Style := Style or Formats[FDateFormat];
    if FDateMode = dmUpDown then Style := Style or DTS_UPDOWN;
    if FDateTimeKind = adkTime then Style := Style or DTS_TIMEFORMAT;
    ACalAlignment := FCalAlignment;
    if UseRightToLeftAlignment then
      if ACalAlignment = dtaLeft then
        ACalAlignment := dtaRight
      else
        ACalAlignment := dtaLeft;
    if ACalAlignment = dtaRight then Style := Style or DTS_RIGHTALIGN;
    if FParseInput then Style := Style or DTS_APPCANPARSE;
    if FShowCheckbox then Style := Style or DTS_SHOWNONE;
    WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW);
  end;
end;

procedure TabcCustomDateTimePicker.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TabcCustomDateTimePicker.CreateWnd;
begin
  FLastFormatString := GetDateFormatString;
  inherited CreateWnd;
  SetChecked(FChecked);
  DateTime_SetFormat(Handle, PChar(FLastFormatString));
end;

procedure TabcCustomDateTimePicker.CMColorChanged(var Message: TMessage);
begin
  inherited;
  InvalidateRect(Handle, nil, True);
end;

procedure TabcCustomDateTimePicker.CMFontChanged(var Message: TMessage);
begin
  inherited;
  AdjustHeight;
  InvalidateRect(Handle, nil, True);
end;

procedure TabcCustomDateTimePicker.CNNotify(var Message: TWMNotify);
var
  DT: TDateTime;
  AllowChange: Boolean;
begin
  with Message, NMHdr^ do
  begin
    Result := 0;
    case code of
      DTN_CLOSEUP:
        begin
          FDroppedDown := False;
          SetDate(SystemTimeToDateTime(FLastChange));
          if Assigned(FOnCloseUp) then FOnCloseUp(Self);
        end;
      DTN_DATETIMECHANGE:
        begin
          with PNMDateTimeChange(NMHdr)^ do
          begin
            if FDroppedDown and (dwFlags = GDT_VALID) then
            begin
              FLastChange := st;
              FDateTime := SystemTimeToDateTime(FLastChange);
            end
            else begin
              if FShowCheckbox and IsBlankSysTime(st) then
                FChecked := False
              else if dwFlags = GDT_VALID then
              begin
                FLastChange := st;
                DT := SystemTimeToDateTime(st);
                case DateTimeKind of
                  adkDate:
                    SetDate(DT);
                  adkTime:
                    SetTime(DT);
                  adkDateTime:
                    SetDateTime(DT);
                end;
                if FShowCheckbox then FChecked := True;
              end;
            end;
            Change;
          end;
        end;
      DTN_DROPDOWN:
        begin
          DateTimeToSystemTime(Date, FLastChange);
          FDroppedDown := True;
          SetDefaultDropDownDate;
          if Assigned(FOnDropDown) then FOnDropDown(Self);
        end;
      DTN_USERSTRING:
        begin
          AllowChange := Assigned(FOnUserInput);
          with PNMDateTimeString(NMHdr)^ do
          begin
            if AllowChange then
            begin
              DT := 0.0;
              FOnUserInput(Self, pszUserString, DT, AllowChange);
              DateTimeToSystemTime(DT, st);
            end;
            dwFlags := Ord(not AllowChange);
          end;
        end;
    else
      inherited;
    end;
  end;
end;

function TabcCustomDateTimePicker.GetCalendarHandle: HWND;
begin
  Result := DateTime_GetMonthCal(Handle);
end;

function TabcCustomDateTimePicker.GetTime: TTime;
begin
  Result := TTime(FDateTime);
end;

function TabcCustomDateTimePicker.MsgSetCalColors(ColorIndex: Integer; ColorValue: TColor): Boolean;
begin
  Result := True;
  if HandleAllocated then
    Result := DateTime_SetMonthCalColor(Handle, ColorIndex, ColorValue) <> DWORD($FFFFFFFF);
end;

function TabcCustomDateTimePicker.MsgSetDateTime(Value: TSystemTime): Boolean;
begin
  Result := True;
  if HandleAllocated then
    if not FChanging then
    begin
      FChanging := True;
      try
        Result := DateTime_SetSystemTime(Handle, GDT_VALID, Value);
        GetDisplayFormat(Value);
        Invalidate;
        if FShowCheckbox and not (csLoading in ComponentState)then
          FChecked := Result;
      finally
        FChanging := False;
      end;
    end;
end;

function TabcCustomDateTimePicker.MsgSetRange(Flags: Integer; SysTime: PSystemTime): Boolean;
begin
  Result := True;
  if HandleAllocated then
    if Flags <> 0 then Result := DateTime_SetRange(Handle, Flags, SysTime);
end;

procedure TabcCustomDateTimePicker.SetCalAlignment(Value: TDTCalAlignment);
begin
  if FCalAlignment <> Value then
  begin
    FCalAlignment := Value;
    if not (csDesigning in ComponentState) then
      SetComCtlStyle(Self, DTS_RIGHTALIGN, Value = dtaRight);
  end;
end;

procedure TabcCustomDateTimePicker.SetChecked(Value: Boolean);
var
  ST: TSystemTime;
begin
  FChecked := Value;
  if FShowCheckbox then
  begin
    if Value then SetDateTime(FDateTime)
    else DateTime_SetSystemTime(Handle, GDT_NONE, ST);
    Invalidate;
  end;
end;

procedure TabcCustomDateTimePicker.SetDateFormat(Value: TDTDateFormat);
begin
  if FDateFormat <> Value then
  begin
    FDateFormat := Value;
    RecreateWnd;
  end;
end;

procedure TabcCustomDateTimePicker.SetDateMode(Value: TDTDateMode);
begin
  if FDateMode <> Value then
  begin
    FDateMode := Value;
    RecreateWnd;
  end;
end;

procedure TabcCustomDateTimePicker.SetDateTimeKind(Value: TabcDateTimeKind);
begin
  if FDateTimeKind <> Value then
  begin
    FDateTimeKind := Value;
    RecreateWnd;
  end;
end;

procedure TabcCustomDateTimePicker.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  //this property was removed
  Filer.DefineProperty('Kind', ReadOldKindProperty, nil, False);
end;

procedure TabcCustomDateTimePicker.ReadOldKindProperty(Reader: TReader);
begin
  if Reader.ReadIdent = 'dtkDate' then
    DateTimeKind := adkDate
  else
    DateTimeKind := adkTime;
end;


procedure TabcCustomDateTimePicker.SetParseInput(Value: Boolean);
begin
  if FParseInput <> Value then
  begin
    FParseInput := Value;
    if not (csDesigning in ComponentState) then
      SetComCtlStyle(Self, DTS_APPCANPARSE, Value);
  end;
end;

procedure TabcCustomDateTimePicker.SetShowCheckbox(Value: Boolean);
begin
  if FShowCheckbox <> Value then
  begin
    FShowCheckbox := Value;
    RecreateWnd;
  end;
end;

procedure TabcCustomDateTimePicker.SetTime(Value: TTime);
begin
  if Abs(Frac(FDateTime)) <> Abs(Frac(Value)) then
  begin
    ReplaceDate(TDateTime(Value), FDateTime);
    if Value = 0.0 then
    begin
      if not FShowCheckbox then
        raise EDateTimeError.Create(SNeedAllowNone);
      FChecked := False;
      Invalidate;
    end
    else
      SetDateTime(Value);
  end;
end;

procedure TabcCustomDateTimePicker.SetDateFormatString(
  const Value: string);
begin
  if Value <> FDateFormatString then
  begin
    FDateFormatString := Value;
    if HandleAllocated and (Value <> '') then
      DateTime_SetFormat(Handle, PChar(Value));
  end;
end;

procedure TabcCustomDateTimePicker.SetDateFormatZero(const Value: string);
begin
  if Value <> FDateFormatZero then
  begin
    FDateFormatZero := Value;
    if HandleAllocated and FShowingZeroDate then
      DateTime_SetFormat(Handle, PChar(Value));
  end;
end;

function TabcCustomDateTimePicker.GetDateFormatString: string;
var
  Locale: LCID;
  function GetDatePart: string;
  begin
    if DateFormat = dfShort then
      Result := GetLocaleStr(Locale, LOCALE_SSHORTDATE, 'M/dd/yyyy')
    else
      Result := GetLocaleStr(Locale, LOCALE_SLONGDATE, 'M/dd/yyyy');
  end;

  function GetTimePart: string;
  begin
    Result := GetLocaleStr(Locale, LOCALE_STIMEFORMAT, 'h:mm:ss tt');
  end;

begin
  Result := DateFormatString;
  if Result = '' then
  begin
    Locale := GetThreadLocale;
    case DateTimeKind of
      adkDate:
        Result := GetDatePart;
      adkTime:
        Result := GetTimePart;
      adkDateTime:
        Result := GetDatePart + ' ' + GetTimePart;
    end;
  end;
end;

procedure TabcCustomDateTimePicker.GetDisplayFormat(Value: TSystemTime);
var
  sFormatString: string;
begin
  sFormatString := '';
  if Assigned(FOnFormatDate) then
    FOnFormatDate(Self, SystemTimeToDateTime(Value), sFormatString)
  else
    with Value do
      if ((DateTimeKind <> adkTime) and
          (wYear = 1899) and
          (wMonth = 12) and
          (wDay = 30)
         ) then
        sFormatString := DateFormatZero
      else
        sFormatString := GetDateFormatString;

  if sFormatString = '' then
    sFormatString :=  '''''';
  if sFormatString <> FLastFormatString then
  begin
    FLastFormatString := sFormatString;
    DateTime_SetFormat(Handle, PChar(sFormatString));
  end;
end;

procedure TabcCustomDateTimePicker.SetDefaultDropDownDate;
var
  defDate: TDateTime;
begin
  if Trunc(Date) = 0 then
  begin
    defDate := DefaultDate;
    if Trunc(defDate) = 0 then
      defDate := SysUtils.Date;
    DateTimeToSystemTime(defDate, FLastChange);
    DateTime_SetSystemTime(GetCalendarHandle, GDT_VALID, FLastChange);
    Change;
  end;
end;

{ TabcDateTimeDataLink }

constructor TabcDateTimeDataLink.Create;
begin
  inherited Create;
  VisualControl := True;
end;

procedure TabcDateTimeDataLink.SetEditing(Value: Boolean);
begin
  if FEditing <> Value then
  begin
    FEditing := Value;
    FModified := False;
    if Assigned(FOnEditingChange) then FOnEditingChange(Self);
  end;
end;

procedure TabcDateTimeDataLink.SetFieldName1(const Value: string);
begin
  if FFieldName1 <> Value then
  begin
    FFieldName1 := Value;
    UpdateField1;
  end;
end;

procedure TabcDateTimeDataLink.SetFieldName2(const Value: string);
begin
  if FFieldName2 <> Value then
  begin
    FFieldName2 := Value;
    UpdateField2;
  end;
end;

procedure TabcDateTimeDataLink.SetField1(Value: TField);
begin
  if FField1 <> Value then
  begin
    FField1 := Value;
    EditingChanged;
    RecordChanged(nil);
  end;
end;

procedure TabcDateTimeDataLink.SetField2(Value: TField);
begin
  if FField2 <> Value then
  begin
    FField2 := Value;
    EditingChanged;
    RecordChanged(nil);
  end;
end;

procedure TabcDateTimeDataLink.UpdateField1;
begin
  if Active and (FFieldName1 <> '') then
  begin
    FField1 := nil;
    if Assigned(FControl) then
      SetField1(GetFieldProperty(DataSource.DataSet, FControl, FFieldName1))
    else
      SetField1(DataSource.DataSet.FieldByName(FFieldName1));
  end
  else
    SetField1(nil);
end;

procedure TabcDateTimeDataLink.UpdateField2;
begin
  if Active and (FFieldName2 <> '') then
  begin
    FField2 := nil;
    if Assigned(FControl) then
      SetField2(GetFieldProperty(DataSource.DataSet, FControl, FFieldName2))
    else
      SetField2(DataSource.DataSet.FieldByName(FFieldName2));
  end
  else
    SetField2(nil);
end;

function TabcDateTimeDataLink.Edit: Boolean;
begin
  if CanModify then inherited Edit;
  Result := FEditing;
end;

function TabcDateTimeDataLink.GetCanModify: Boolean;
begin
  Result := not ReadOnly and (Field1 <> nil) and Field1.CanModify;
end;

procedure TabcDateTimeDataLink.Modified;
begin
  FModified := True;
end;

procedure TabcDateTimeDataLink.Reset;
begin
  RecordChanged(nil);
end;

procedure TabcDateTimeDataLink.ActiveChanged;
begin
  UpdateField1;
  UpdateField2;
  if Assigned(FOnActiveChange) then FOnActiveChange(Self);
end;

procedure TabcDateTimeDataLink.EditingChanged;
begin
  SetEditing(inherited Editing and CanModify);
end;

procedure TabcDateTimeDataLink.FocusControl(Field: TFieldRef);
begin
  if (Field^ <> nil) and ((Field^ = FField1) or (Field^ = FField2)) and (FControl is TWinControl) then
    if TWinControl(FControl).CanFocus then
    begin
      Field^ := nil;
      TWinControl(FControl).SetFocus;
    end;
end;

procedure TabcDateTimeDataLink.RecordChanged(Field: TField);
begin
  if (Field = nil) or (Field = FField1) or (Field = FField2) then
  begin
    if Assigned(FOnDataChange) then FOnDataChange(Self);
    FModified := False;
  end;
end;

procedure TabcDateTimeDataLink.LayoutChanged;
begin
  UpdateField1;
  UpdateField2;
end;

procedure TabcDateTimeDataLink.UpdateData;
begin
  if FModified then
  begin
    if (Field1 <> nil) and Assigned(FOnUpdateData) then FOnUpdateData(Self);
    FModified := False;
  end;
end;

{ TabcDBDateTimePicker }

constructor TabcDBDateTimePicker.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcDateTimeDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnActiveChange := ActiveChanged;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TabcDBDateTimePicker.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TabcDBDateTimePicker.Loaded;
begin
  inherited Loaded;
  if (csDesigning in ComponentState) then
    DataChange(Self);
end;

procedure TabcDBDateTimePicker.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcDBDateTimePicker.ActiveChanged(Sender: TObject);
begin
  if Enabled then
    Perform(CM_ENABLEDCHANGED,0,0);
end;

procedure TabcDBDateTimePicker.DataChange(Sender: TObject);
begin
  if FChanging then Exit;
  if FDataLink.Field1 <> nil then
    case DateTimeKind of
      adkDate:
        Date := FDataLink.Field1.AsDateTime;
      adkTime:
        Time := FDataLink.Field1.AsDateTime;
      adkDateTime:
        DateTime := FDataLink.Field1.AsDateTime;
    end
  else
    DateTime := 0;
  if FDataLink.Field2 <> nil then
    Checked := FDataLink.Field2.AsBoolean;
end;

procedure TabcDBDateTimePicker.UpdateData(Sender: TObject);
begin
  if FDataLink.Field2 <> nil then
    FDataLink.Field2.AsBoolean := Checked;
  if not ShowCheckBox or Checked then
    case DateTimeKind of
      adkDate:
        FDataLink.Field1.AsDateTime :=
          Trunc(Date) + Frac(FDataLink.Field1.AsDateTime);
      adkTime:
        FDataLink.Field1.AsDateTime :=
          Trunc(FDataLink.Field1.AsDateTime) + Frac(Time);
      adkDateTime:
        FDataLink.Field1.AsDateTime := DateTime;
    end;
end;

procedure TabcDBDateTimePicker.Change;
begin
  if (csDesigning in ComponentState) then Exit;
  FChanging := True;
  try
    FDataLink.Edit;
    inherited Change;
    FDataLink.Modified;
  finally
    FChanging := False;
  end;
end;

function TabcDBDateTimePicker.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcDBDateTimePicker.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TabcDBDateTimePicker.GetDataField: string;
begin
  Result := FDataLink.FieldName1;
end;

procedure TabcDBDateTimePicker.SetDataField(const Value: string);
begin
  FDataLink.FieldName1 := Value;
end;

function TabcDBDateTimePicker.GetDataFieldCheckBox: string;
begin
  Result := FDataLink.FieldName2;
end;

procedure TabcDBDateTimePicker.SetDataFieldCheckBox(const Value: string);
begin
  FDataLink.FieldName2 := Value;
  ShowCheckBox := (Value <> '');
end;

function TabcDBDateTimePicker.GetField: TField;
begin
  Result := FDataLink.Field1;
end;

function TabcDBDateTimePicker.GetFieldCheckBox: TField;
begin
  Result := FDataLink.Field2;
end;

procedure TabcDBDateTimePicker.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if not FDataLink.Edit then
    Key := 0;
end;

procedure TabcDBDateTimePicker.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key in [#32..#255]) and (FDataLink.Field1 <> nil) and
    not FDataLink.Field1.IsValidChar(Key) then
  begin
    MessageBeep(0);
    Key := #0;
  end;
  case Key of
    #32..#255:
      FDataLink.Edit;
    #27:
      FDataLink.Reset;
  end;
end;

procedure TabcDBDateTimePicker.CMEnabledChanged(var Message: TMessage);
var
  CanEnable: Boolean;
  Form: TCustomForm;
begin
  CanEnable := Enabled and FDatalink.CanModify;
  if not CanEnable and (Parent <> nil) then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then Form.DefocusControl(Self, False);
  end;
  if HandleAllocated and not (csDesigning in ComponentState) then
    EnableWindow(Handle, CanEnable);
end;

procedure TabcDBDateTimePicker.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

procedure TabcDBDateTimePicker.CMGetDatalink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

function TabcDBDateTimePicker.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TabcDBDateTimePicker.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

{ TabcDBMonthCalendar }

constructor TabcDBMonthCalendar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcDateTimeDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnActiveChange := ActiveChanged;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TabcDBMonthCalendar.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TabcDBMonthCalendar.Loaded;
begin
  inherited Loaded;
  if (csDesigning in ComponentState) then
    DataChange(Self);
end;

procedure TabcDBMonthCalendar.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcDBMonthCalendar.ActiveChanged(Sender: TObject);
begin
  if Enabled then
    Perform(CM_ENABLEDCHANGED,0,0);
end;

procedure TabcDBMonthCalendar.DataChange(Sender: TObject);
begin
  if FChanging then Exit;
  if FDataLink.Field1 <> nil then
  begin
    if (FDataLink.Field2 <> nil) then
      SetSelectedDateRange(FDataLink.Field1.AsDateTime, FDataLink.Field2.AsDateTime)
    else
      Date := FDataLink.Field1.AsDateTime;
  end;
end;

procedure TabcDBMonthCalendar.UpdateData(Sender: TObject);
begin
  if (FDataLink.Field1 <> nil) then
    FDataLink.Field1.AsDateTime := Date;
  if (FDataLink.Field2 <> nil) then
    FDataLink.Field2.AsDateTime := EndDate;
end;

procedure TabcDBMonthCalendar.Change;
begin
  if (csDesigning in ComponentState) then Exit;
  FChanging := True;
  try
    FDataLink.Edit;
    if Assigned(FOnChange) then FOnChange(Self);
    FDataLink.Modified;
    FDataLink.UpdateRecord;
  finally
    FChanging := False;
  end;
end;

function TabcDBMonthCalendar.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcDBMonthCalendar.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TabcDBMonthCalendar.GetDataField: string;
begin
  Result := FDataLink.FieldName1;
end;

procedure TabcDBMonthCalendar.SetDataField(const Value: string);
begin
  FDataLink.FieldName1 := Value;
end;

function TabcDBMonthCalendar.GetDataFieldEndDate: string;
begin
  Result := FDataLink.FieldName2;
end;

procedure TabcDBMonthCalendar.SetDataFieldEndDate(const Value: string);
begin
  FDataLink.FieldName2 := Value;
  MultiSelect := (Value <> '');
end;

function TabcDBMonthCalendar.GetField: TField;
begin
  Result := FDataLink.Field1;
end;

function TabcDBMonthCalendar.GetFieldEndDate: TField;
begin
  Result := FDataLink.Field2;
end;

procedure TabcDBMonthCalendar.CMGetDatalink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

function TabcDBMonthCalendar.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TabcDBMonthCalendar.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

procedure TabcDBMonthCalendar.CNNotify(var Message: TWMNotify);
begin
  inherited;
  with Message.NMHdr^ do
    if (code = MCN_SELECT) or (code = MCN_SELCHANGE) then
      Change;
end;

procedure TabcDBMonthCalendar.CMEnabledChanged(var Message: TMessage);
var
  CanEnable: Boolean;
  Form: TCustomForm;
begin
  CanEnable := Enabled and FDatalink.CanModify;
  if not CanEnable and (Parent <> nil) then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then Form.DefocusControl(Self, False);
  end;
  if HandleAllocated and not (csDesigning in ComponentState) then
    EnableWindow(Handle, CanEnable);
end;

end.
