unit abcdbfx;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Menus, Forms,
  DB, DBCtrls,
  abcfx, abcufx, abcbtn, abcgreg
  ;

type

  { TabcDBEffectsImageDataLink }

  TabcDBEffectsImageDataLink = class(TDataLink)
  private
    FField1, FField2: TField;
    FFieldName1, FFieldName2: string;
    FEditing: Boolean;
    FModified: Boolean;
    FOnDataChange: TNotifyEvent;
    FOnEditingChange: TNotifyEvent;
    FOnUpdateData: TNotifyEvent;
    FOnActiveChange: TNotifyEvent;
    function GetCanModify: Boolean;
    procedure SetEditing(Value: Boolean);
    procedure SetField1(Value: TField);
    procedure SetField2(Value: TField);
    procedure SetFieldName1(const Value: string);
    procedure SetFieldName2(const Value: string);
    procedure UpdateField1;
    procedure UpdateField2;
  protected
    procedure ActiveChanged; override;
    procedure EditingChanged; override;
    procedure LayoutChanged; override;
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
  public
    constructor Create;
    function Edit: Boolean;
    procedure Modified;
    procedure Reset;
    property CanModify: Boolean read GetCanModify;
    property Editing: Boolean read FEditing;
    property Field1: TField read FField1;
    property Field2: TField read FField2;
    property FieldName1: string read FFieldName1 write SetFieldName1;
    property FieldName2: string read FFieldName2 write SetFieldName2;
    property OnDataChange: TNotifyEvent read FOnDataChange write FOnDataChange;
    property OnEditingChange: TNotifyEvent read FOnEditingChange write FOnEditingChange;
    property OnUpdateData: TNotifyEvent read FOnUpdateData write FOnUpdateData;
    property OnActiveChange: TNotifyEvent read FOnActiveChange write FOnActiveChange;
  end;

  { TabcCustomDBEffectsImage }

  TabcFileNameEvent = procedure (Sender: TObject; var FileName: string) of object;

  TabcCustomDBEffectsImage = class(TabcCustomEffectsImage)
  private
    FDataLink: TabcDBEffectsImageDataLink;
    FAutoDisplay: Boolean;
    FPictureLoaded: Boolean;
    FQuickDraw: Boolean;
    FOnLoadFromFile: TabcFileNameEvent;
    FOnSaveToFile: TabcFileNameEvent;
    FOnSetFileName: TabcFileNameEvent;
    FWriteParadoxBlobHeader: Boolean;
    FFilePath: string;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataFieldCaption: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetFieldCaption: TField;
    procedure SetAutoDisplay(Value: Boolean);
    procedure SetDataField(const Value: string);
    procedure SetDataFieldCaption(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure UpdateData(Sender: TObject);
    procedure WMSize(var Message: TMessage); message WM_SIZE;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(Value: Boolean); override;
    function DoEdit: Boolean; override;
    procedure PictureChanged(Sender: TObject); override;
    procedure LoadPictureFromBlobField; dynamic;
    procedure LoadCaption;
    {can be published}
    property AutoDisplay: Boolean read FAutoDisplay write SetAutoDisplay default True;
    property DataField: string read GetDataField write SetDataField;
    property DataFieldCaption: string read GetDataFieldCaption write SetDataFieldCaption;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property FilePath: string read FFilePath write FFilePath;
    property QuickDraw: Boolean read FQuickDraw write FQuickDraw default True;
    property OnLoadFromFile: TabcFileNameEvent read FOnLoadFromFile write FOnLoadFromFile;
    property OnSaveToFile: TabcFileNameEvent read FOnSaveToFile write FOnSaveToFile;
    property OnSetFileName: TabcFileNameEvent read FOnSetFileName write FOnSetFileName;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    procedure LoadPicture;
    property Field: TField read GetField;
    property FieldCaption: TField read GetFieldCaption;
    property WriteParadoxBlobHeader: Boolean read FWriteParadoxBlobHeader
      write FWriteParadoxBlobHeader;
      { NB this is set to true, override in descendant if you want blobs to store
        native file format only and don't want to use TGraphicField methods to
        access the data.
      }
  published
  end;

  { TabcDBEffectsImage }

  TabcDBEffectsImage = class(TabcCustomDBEffectsImage)
  private
    FAnimationStopped: Boolean;
    FOnAnimateNext: TabcCancelEvent;
    FPlaying, FLoop: Boolean;
    FPlayItemChanging: Boolean;
    FDataset: TDataset;
  protected
    procedure TerminateThread; override;
    procedure DoEffectEnd; override;
    procedure DoStop; override;
    procedure CheckAnimationStop;
  public
    procedure Play(Loop: Boolean);
    property Picture;
  published
    property Align;
    property Alignment;
    property CaptionStyle;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property AutoDisplay;
    property Color;
    property DataField;
    property DataFieldCaption;
    property DataSource;
    property DragCursor;
    property DragMode;
    property Effect;
    property EffectPause;
    property EffectSelection;
    property EffectSelIndex;
    property EffectSize;
    property EffectSpeed;
    property Enabled;
    property FilePath;
    property Font;
    property Layout;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property PictureSize;
    property PopupMenu;
    property QuickDraw;
    property ShowAccelChar;
    property ShowHint;
    property SuspendOnAppInactive;
    property Transparent;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnLoadFromFile;
    property OnSaveToFile;
    property OnSetFileName;
    property OnStartDock;
    property OnEndDock;
    property OnEffectEnd;
    property OnEffectStep;
    property OnAnimateNext: TabcCancelEvent read FOnAnimateNext write FOnAnimateNext;
  end;

var
  abcWriteParadoxBlobHeaders: Boolean = True;

implementation

uses Clipbrd, Consts;

const
  ERR_BITMAP_NEEDED = 'A bitmap image is needed';

{ TabcDBEffectsImageDataLink }

constructor TabcDBEffectsImageDataLink.Create;
begin
  inherited Create;
  VisualControl := True;
end;

procedure TabcDBEffectsImageDataLink.SetEditing(Value: Boolean);
begin
  if FEditing <> Value then
  begin
    FEditing := Value;
    FModified := False;
    if Assigned(FOnEditingChange) then FOnEditingChange(Self);
  end;
end;

procedure TabcDBEffectsImageDataLink.SetFieldName1(const Value: string);
begin
  if FFieldName1 <> Value then
  begin
    FFieldName1 := Value;
    UpdateField1;
  end;
end;

procedure TabcDBEffectsImageDataLink.SetFieldName2(const Value: string);
begin
  if FFieldName2 <> Value then
  begin
    FFieldName2 := Value;
    UpdateField2;
  end;
end;

procedure TabcDBEffectsImageDataLink.SetField1(Value: TField);
begin
  if FField1 <> Value then
  begin
    FField1 := Value;
    EditingChanged;
    RecordChanged(nil);
  end;
end;

procedure TabcDBEffectsImageDataLink.SetField2(Value: TField);
begin
  if FField2 <> Value then
  begin
    FField2 := Value;
    EditingChanged;
    RecordChanged(nil);
  end;
end;

procedure TabcDBEffectsImageDataLink.UpdateField1;
begin
  if Active and (FFieldName1 <> '') then
  begin
    FField1 := nil;
    SetField1(DataSource.DataSet.FieldByName(FFieldName1));
  end
  else
    SetField1(nil);
end;

procedure TabcDBEffectsImageDataLink.UpdateField2;
begin
  if Active and (FFieldName2 <> '') then
  begin
    FField2 := nil;
    SetField2(DataSource.DataSet.FieldByName(FFieldName2));
  end
  else
    SetField2(nil);
end;

function TabcDBEffectsImageDataLink.Edit: Boolean;
begin
  if CanModify then inherited Edit;
  Result := FEditing;
end;

function TabcDBEffectsImageDataLink.GetCanModify: Boolean;
begin
  Result := not ReadOnly and (Field1 <> nil) and Field1.CanModify;
end;

procedure TabcDBEffectsImageDataLink.Modified;
begin
  FModified := True;
end;

procedure TabcDBEffectsImageDataLink.Reset;
begin
  RecordChanged(nil);
end;

procedure TabcDBEffectsImageDataLink.ActiveChanged;
begin
  UpdateField1;
  UpdateField2;
  if Assigned(FOnActiveChange) then FOnActiveChange(Self);
end;

procedure TabcDBEffectsImageDataLink.EditingChanged;
begin
  SetEditing(inherited Editing and CanModify);
end;

procedure TabcDBEffectsImageDataLink.RecordChanged(Field: TField);
begin
  if (Field = nil) or (Field = FField1) or (Field = FField2) then
  begin
    if Assigned(FOnDataChange) then FOnDataChange(Self);
    FModified := False;
  end;
end;

procedure TabcDBEffectsImageDataLink.LayoutChanged;
begin
  UpdateField1;
  UpdateField2;
end;

procedure TabcDBEffectsImageDataLink.UpdateData;
begin
  if FModified then
  begin
    if (Field1 <> nil) and Assigned(FOnUpdateData) then FOnUpdateData(Self);
    FModified := False;
  end;
end;

{ TabcDBEffectsImage }

constructor TabcCustomDBEffectsImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ParentColor := False;
  FAutoDisplay := True;
  FDataLink := TabcDBEffectsImageDataLink.Create;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
  FQuickDraw := True;
  FWriteParadoxBlobHeader := abcWriteParadoxBlobHeaders;
end;

destructor TabcCustomDBEffectsImage.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

function TabcCustomDBEffectsImage.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcCustomDBEffectsImage.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(self);
end;

function TabcCustomDBEffectsImage.GetDataField: string;
begin
  Result := FDataLink.FieldName1;
end;

function TabcCustomDBEffectsImage.GetDataFieldCaption: string;
begin
  Result := FDataLink.FieldName2;
end;

procedure TabcCustomDBEffectsImage.SetDataField(const Value: string);
begin
  FDataLink.FieldName1 := Value;
end;

procedure TabcCustomDBEffectsImage.SetDataFieldCaption(const Value: string);
begin
  FDataLink.FieldName2 := Value;
end;

function TabcCustomDBEffectsImage.GetField: TField;
begin
  Result := FDataLink.Field1;
end;

function TabcCustomDBEffectsImage.GetFieldCaption: TField;
begin
  Result := FDataLink.Field2;
end;

procedure TabcCustomDBEffectsImage.SetAutoDisplay(Value: Boolean);
begin
  if FAutoDisplay <> Value then
  begin
    FAutoDisplay := Value;
    if Value then LoadPicture;
  end;
end;

procedure TabcCustomDBEffectsImage.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
     (AComponent = DataSource) then
    DataSource := nil;
end;

procedure TabcCustomDBEffectsImage.PictureChanged(Sender: TObject);
begin
  if FPictureLoaded then FDataLink.Modified;
  FPictureLoaded := FAutoDisplay;
  inherited PictureChanged(Sender);
end;

procedure TabcCustomDBEffectsImage.LoadCaption;
begin
  if (FieldCaption <> nil) then
    Caption := FieldCaption.AsString;
end;

procedure TabcCustomDBEffectsImage.LoadPicture;
var
  fName: string;
begin
  if not FPictureLoaded then
  begin
    LoadCaption;
    if (Field = nil) then
      Picture.Graphic := nil
    else if Field.IsBlob then
      LoadPictureFromBlobField
    else if (Field.DataType = ftString) then
    begin
      fName := FilePath + Field.AsString;
      if Assigned(FOnLoadFromFile) then
        FOnLoadFromFile(Self, fName);
      if FileExists(fName) then
        Picture.LoadFromFile(fName);
    end;
  end;
end;

type
  TGraphicFixupConstructor = class(TGraphic);
  TGraphicFixupConstructorClass = class of TGraphicFixupConstructor;

procedure TabcCustomDBEffectsImage.LoadPictureFromBlobField;
// load BMP, ICO, WMF, JPG formats using a stream
// detect existence of Paradox Graphic Blob Header
// other graphics types are not supported, but could be in future
// for JPEG support add units JPEG and abcjpeg to project

var
  memStream: TMemoryStream;
  Ext: string;
  NewGraphic: TGraphic;
  GraphicClass: TGraphicClass;
begin
  memStream := TMemoryStream.Create;
  try
    TBlobField(Field).SaveToStream(memStream);
    Ext := abcGetGraphicFileExtFromStream(memStream);
    if Ext = '' then
    begin
      Picture.Graphic := nil;
      Exit;
    end;
    GraphicClass := abcGetGraphicClassFromFileExt(Ext);
    if GraphicClass = nil then
      raise EInvalidGraphic.CreateFmt(SUnknownExtension, [Ext]);

    NewGraphic := TGraphicFixupConstructorClass(GraphicClass).Create;

    try
      NewGraphic.OnProgress := Picture.OnProgress;

      memStream.Seek(0,0);

      abcSkipParadoxBlobHeader(memStream);

      NewGraphic.LoadFromStream(memStream);

      Picture.Graphic := NewGraphic;
    finally
      NewGraphic.Free;
    end;
  finally
    memStream.Free;
  end;
end;

procedure TabcCustomDBEffectsImage.DataChange(Sender: TObject);
begin
  FPictureIsChanging := True;
  try
    Picture.Graphic := nil;
    FPictureLoaded := False;
    if FAutoDisplay then
      LoadPicture
    else
      LoadCaption;
  finally
    FPictureIsChanging := False;
    PictureChanged(nil);
  end;
end;

procedure TabcCustomDBEffectsImage.UpdateData(Sender: TObject);
var
  fName: string;
  memStream: TMemoryStream;
  ParadoxBlobHeader: TabcParadoxGraphicHeader;
  DataSize: Longint;
begin
  if Picture.Graphic.Empty then
    Field.Clear
  else
  begin
    if Field.IsBlob then
    begin
      if Picture.Graphic is TBitmap then
        Field.Assign(Picture.Graphic)
      else
      begin
        {write other graphics formats to a stream}
        memStream := TMemoryStream.Create;
        try
          if FWriteParadoxBlobHeader then
          begin
            with ParadoxBlobHeader do
            begin
              Count := 1;
              HType := $0100;
              Size := 0;
            end;
            memStream.WriteBuffer(ParadoxBlobHeader, SizeOf(TabcParadoxGraphicHeader));
          end;
          Picture.Graphic.SaveToStream(memStream);
          memStream.Seek(0,0);
          if FWriteParadoxBlobHeader then
            {update size in header}
            begin
              DataSize := memStream.Size - SizeOf(TabcParadoxGraphicHeader);
              memStream.Seek(4,0);
              memStream.WriteBuffer(DataSize, SizeOf(Longint));
            end;
          TBlobField(Field).LoadFromStream(memStream);
        finally
          memStream.Free;
        end;
      end
    end
    else if (Field.DataType = ftString) then
    begin
      fName := FilePath + Field.AsString;
      {save file}
      if Assigned(FOnSaveToFile) then
        FOnSaveToFile(Self, fName);
      if fName <> '' then
        Picture.SaveToFile(fName);
      {set filename}
      if Assigned(FOnSetFileName) then
        FOnSetFileName(Self, fName);
      Field.AsString := fName;
    end;
  end;
end;

procedure TabcCustomDBEffectsImage.WMSize(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

function TabcCustomDBEffectsImage.GetReadOnly: Boolean;
begin
  Result := not FDataLink.CanModify;
end;

procedure TabcCustomDBEffectsImage.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

function TabcCustomDBEffectsImage.DoEdit: Boolean;
begin
  Result := FDataLink.CanModify and FDataLink.Edit;
end;

procedure TabcCustomDBEffectsImage.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

procedure TabcCustomDBEffectsImage.CMTextChanged(var Message: TMessage);
begin
end;

function TabcCustomDBEffectsImage.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TabcCustomDBEffectsImage.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

{ TabcDBEffectsImage }

{ Play method to auto scroll through an image dataset }

procedure TabcDBEffectsImage.CheckAnimationStop;
begin
  if Assigned(OnAnimateNext) then
    OnAnimateNext(Self, FAnimationStopped);
  if Application.Terminated or (csDestroying in ComponentState) or not Visible then
    FAnimationStopped := True;
end;

procedure TabcDBEffectsImage.DoEffectEnd;
begin
  inherited;
  if FPlaying then
  begin
    CheckAnimationStop;
    if FAnimationStopped then
    begin
      FPlaying := False;
      Exit;
    end;
    FPlayItemChanging := True;
    with FDataset do
    try
      DisableControls;
      Next;
      if EOF then
        if FLoop then
          First
        else
          FPlaying := False;
    finally
      EnableControls;
      FPlayItemChanging := False;
    end;
  end;
end;

procedure TabcDBEffectsImage.DoStop;
begin
  FPlayItemChanging := False;
  inherited;
end;

procedure TabcDBEffectsImage.Play(Loop: Boolean);
begin
  if (DataSource = nil) then Exit;
  FDataset := DataSource.Dataset;
  if (FDataset = nil) or (FDataset.State <> dsBrowse) or (FDataset.BOF and FDataset.EOF) then
    Exit;
  FPlayItemChanging := True;
  FAnimationStopped := False;
  FPlaying := True;
  FLoop := Loop;
  if NeverPainted then
    Exit;
  TerminateThread;
  with FDataset do
  try
    DisableControls;
    Next;
    if EOF then
      First;
  finally
    EnableControls;
    FPlayItemChanging := False;
  end;
end;

procedure TabcDBEffectsImage.TerminateThread;
begin
  if not FPlayItemChanging then
    FPlaying := False;
  inherited;
end;

end.
