unit abcdbgrd;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, SysUtils, Messages, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, BDE, DB, DBGrids, DBTables, DBCtrls,
  Mask, Buttons, abcexctl, abcdbctl, Grids, Menus, abcdbgrf;

type

  {TabcSubQuery - used to register with no icon and restore the link to the container}

  TabcSubQuery = class(TQuery)
  private
    FParent: TComponent;
    procedure SetParent(AParent: TComponent);
  protected
    procedure ReadState(Reader: TReader); override;
  public
    function HasParent: Boolean; override;
    property Parent: TComponent read FParent write SetParent stored False;
  end;

  {TabcSubTable - used to register with no icon and restore the link to the container}

  TabcSubTable = class(TTable)
  private
    FParent: TComponent;
    procedure SetParent(AParent: TComponent);
  protected
    procedure ReadState(Reader: TReader); override;
  public
    function HasParent: Boolean; override;
    property Parent: TComponent read FParent write SetParent stored False;
  end;

  {TabcSubDataSource - used to register with no icon and restore the link to the container}

  TabcSubDataSource = class(TDataSource)
  private
    FParent: TComponent;
    procedure SetParent(AParent: TComponent);
  protected
    procedure ReadState(Reader: TReader); override;
  public
    function HasParent: Boolean; override;
    property Parent: TComponent read FParent write SetParent stored False;
  end;

  TabcSubNavAlignment = (naTop, naBottom);

  {TabcDBQueryGrid}

  TabcDBQueryGrid = class(TCustomPanel)
  private
    {Sub Components}
    FSubQuery: TabcSubQuery;
    FSubDataSource: TabcSubDataSource;
    FSubNavigator: TabcDBSuperNav;
    FSubGrid: TabcDBFixedGrid;
    FColumn: TColumn; {workaround for bug in Delphi column streaming}
    {Shared Properties}
    FDragCursor: TCursor;
    FDragMode: TDragMode;
    FHint: string;
    FHelpContext: THelpContext;
    FParentColor: Boolean;
    FParentCtl3D: Boolean;
    FParentFont: Boolean;
    FParentShowHint: Boolean;
    FPopupMenu: TPopupMenu;
    FTabOrder: TTabOrder;
    FTabStop: Boolean;
    FVisible: Boolean;
    FReadOnly: Boolean;
    FSubNavAlign: TabcSubNavAlignment;
    FHints: TStrings;
    function GetFixedCols: Integer;
    procedure SetFixedCols(Value: Integer);

    {Shared Property Access Methods - Properties}
    function  GetColor: TColor;
    function  GetCtl3D: Boolean;
    function  GetFont: TFont;
    function  GetShowHint: Boolean;

    function  IsColorStored: Boolean;
    function  IsCtl3DStored: Boolean;
    function  IsFontStored: Boolean;
    function  IsShowHintStored: Boolean;

    procedure SetReadOnly(Value: Boolean);
    procedure SetSubNavAlign (Value: TabcSubNavAlignment);
    procedure SetDragCursor(Value: TCursor);
    procedure SetColor(Value: TColor);
    procedure SetCtl3D(Value: Boolean);
    procedure SetFont(Value: TFont);
    procedure SetHint(Value: string);
    procedure SetHelpContext(Value: THelpContext);
    procedure SetParentColor(Value: Boolean);
    procedure SetParentCtl3D(Value: Boolean);
    procedure SetParentFont(Value: Boolean);
    procedure SetParentShowHint(Value: Boolean);
    procedure SetPopupMenu(Value: TPopupMenu);
    procedure SetShowHint(Value: Boolean);
    procedure SetTabOrder(Value: TTabOrder);
    procedure SetTabStop(Value: Boolean);
    procedure SetVisible(Value: Boolean);

    {SubQuery Property Access Methods - Properties}
    function GetParams: TParams;
    function GetSQL: TStrings;
    function GetUniDirectional: Boolean;
    procedure SetParams(Value: TParams);
    procedure SetSQL(Value: TStrings);
    procedure SetUniDirectional(Value: Boolean);

    function GetActive: Boolean;
    function GetAutoCalcFields: Boolean;
    function GetDatabaseName: TFileName;
    procedure SetActive(Value: Boolean);
    procedure SetAutoCalcFields(Value: Boolean);
    procedure SetDatabaseName(Value: TFileName);
    {SubQuery Property Access Methods - Events}
    function GetBeforeOpen: TDataSetNotifyEvent;
    function GetAfterOpen: TDataSetNotifyEvent;
    function GetBeforeClose: TDataSetNotifyEvent;
    function GetAfterClose: TDataSetNotifyEvent;
    function GetBeforeInsert: TDataSetNotifyEvent;
    function GetAfterInsert: TDataSetNotifyEvent;
    function GetBeforeEdit: TDataSetNotifyEvent;
    function GetAfterEdit: TDataSetNotifyEvent;
    function GetBeforePost: TDataSetNotifyEvent;
    function GetAfterPost: TDataSetNotifyEvent;
    function GetBeforeCancel: TDataSetNotifyEvent;
    function GetAfterCancel: TDataSetNotifyEvent;
    function GetBeforeDelete: TDataSetNotifyEvent;
    function GetAfterDelete: TDataSetNotifyEvent;
    function GetOnNewRecord: TDataSetNotifyEvent;
    function GetOnCalcFields: TDataSetNotifyEvent;
    procedure SetBeforeOpen(Value: TDataSetNotifyEvent);
    procedure SetAfterOpen(Value: TDataSetNotifyEvent);
    procedure SetBeforeClose(Value: TDataSetNotifyEvent);
    procedure SetAfterClose(Value: TDataSetNotifyEvent);
    procedure SetBeforeInsert(Value: TDataSetNotifyEvent);
    procedure SetAfterInsert(Value: TDataSetNotifyEvent);
    procedure SetBeforeEdit(Value: TDataSetNotifyEvent);
    procedure SetAfterEdit(Value: TDataSetNotifyEvent);
    procedure SetBeforePost(Value: TDataSetNotifyEvent);
    procedure SetAfterPost(Value: TDataSetNotifyEvent);
    procedure SetBeforeCancel(Value: TDataSetNotifyEvent);
    procedure SetAfterCancel(Value: TDataSetNotifyEvent);
    procedure SetBeforeDelete(Value: TDataSetNotifyEvent);
    procedure SetAfterDelete(Value: TDataSetNotifyEvent);
    procedure SetOnNewRecord(Value: TDataSetNotifyEvent);
    procedure SetOnCalcFields(Value: TDataSetNotifyEvent);

    {SubDataSource Property Access Methods - Events}
    function GetOnStateChange: TNotifyEvent;
    function GetOnDataChange: TDataChangeEvent;
    function GetOnUpdateData: TNotifyEvent;
    procedure SetOnStateChange(Value: TNotifyEvent);
    procedure SetOnDataChange(Value: TDataChangeEvent);
    procedure SetOnUpdateData(Value: TNotifyEvent);

    {SubNavigator Property Access Methods - Properties}
    {NB: No Event Properties published for the Navigator}
    function GetConfirmDelete: Boolean;                       {protected}
    function GetGlyphColor: TabcGlyphColor;
    function GetHints: TStrings;
    function GetNavSpeed: TabcNavSpeed;
    function GetMoveBy: Word;
    function GetVisibleButtons: TabcSuperNavBtnSet;
    procedure SetConfirmDelete(Value: Boolean);               {protected}
    procedure SetGlyphColor(Value: TabcGlyphColor);
    procedure SetHints(Value: TStrings);
    procedure SetNavSpeed(Value: TabcNavSpeed);
    procedure SetMoveBy(Value: Word);
    procedure SetVisibleButtons(Value: TabcSuperNavBtnSet);
    function GetNavHeight: Integer;
    procedure SetNavHeight(Value: Integer);

    {SubGrid Property Access Methods - Properties}
    function GetFixedColor: TColor;
    function GetOptions: TDBGridOptions;
    function GetTitleFont: TFont;
    procedure SetFixedColor(Value: TColor);
    procedure SetOptions(Value: TDBGridOptions);
    procedure SetTitleFont(Value: TFont);
    function GetDefaultDrawing: Boolean;                      {protected}
    procedure SetDefaultDrawing(Value: Boolean);              {protected}
    {SubGrid Property Access Methods - Events}
    function GetOnColEnter: TNotifyEvent;
    function GetOnColExit: TNotifyEvent;
    function GetOnDrawDataCell: TDrawDataCellEvent;           {protected}
    function GetOnDblClick: TNotifyEvent;
    function GetOnDragDrop: TDragDropEvent;
    function GetOnDragOver: TDragOverEvent;
    function GetOnEndDrag: TEndDragEvent;
    function GetOnEnter: TNotifyEvent;
    function GetOnExit: TNotifyEvent;
    function GetOnKeyDown: TKeyEvent;
    function GetOnKeyPress: TKeyPressEvent;
    function GetOnKeyUp: TKeyEvent;
    procedure SetOnColEnter(Value: TNotifyEvent);
    procedure SetOnColExit(Value: TNotifyEvent);
    procedure SetOnDrawDataCell(Value: TDrawDataCellEvent);   {protected}
    procedure SetOnDblClick(Value: TNotifyEvent);
    procedure SetOnDragDrop(Value: TDragDropEvent);
    procedure SetOnDragOver(Value: TDragOverEvent);
    procedure SetOnEndDrag(Value: TEndDragEvent);
    procedure SetOnEnter(Value: TNotifyEvent);
    procedure SetOnExit(Value: TNotifyEvent);
    procedure SetOnKeyDown(Value: TKeyEvent);
    procedure SetOnKeyPress(Value: TKeyPressEvent);
    procedure SetOnKeyUp(Value: TKeyEvent);
    function GetColumns: TDBGridColumns;
    procedure SetColumns(Value: TDBGridColumns);
    function GetOnColumnMoved: TMovedEvent;                     {protected}
    procedure SetOnColumnMoved(Value: TMovedEvent);             {protected}
    function GetOnDrawColumnCell: TDrawColumnCellEvent;         {protected}
    procedure SetOnDrawColumnCell(Value: TDrawColumnCellEvent); {protected}
    function GetOnEditButtonClick: TNotifyEvent;
    procedure SetOnEditButtonClick(Value: TNotifyEvent);
    function GetOnStartDrag: TStartDragEvent;
    procedure SetOnStartDrag(Value: TStartDragEvent);
    {Initialise Properties}
    procedure InitSubQueryProperties(var AQuery: TabcSubQuery);
    procedure InitSubDataSourceProperties(var ADataSource: TabcSubDataSource);
    procedure InitSubSuperNavProperties(var ADBSuperNav: TabcDBSuperNav);
    procedure InitSubGridProperties(var ADBGrid: TabcDBFixedGrid);
    procedure InitSharedProperties;

    procedure AddSubQuery(ASubQuery: TabcSubQuery);
    procedure AddSubDataSource(ASubDataSource: TabcSubDataSource);
    procedure NameSubQuery(AOwner: TComponent);
    procedure NameSubDataSource(AOwner: TComponent);

  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure Loaded; override;
    procedure SetDragMode(Value: TDragMode); override;
    procedure SetEnabled(Value: Boolean); override;

    {Grid Owner Draw Behaviour}
    property DefaultDrawing: Boolean read GetDefaultDrawing write SetDefaultDrawing default True;
    property OnDrawDataCell: TDrawDataCellEvent read GetOnDrawDataCell write SetOnDrawDataCell;
    property OnColumnMoved: TMovedEvent read GetOnColumnMoved write SetOnColumnMoved;
    property OnDrawColumnCell: TDrawColumnCellEvent read GetOnDrawColumnCell write SetOnDrawColumnCell;

    {Navigator ConfirmDelete}
    property ConfirmDelete: Boolean read GetConfirmDelete write SetConfirmDelete default True;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    {Runtime properties to allow regular names to be used in code}
    {Note that the same published properties are named A1..., A2, A3 and then
     renamed by the property editor to make them sort to the top of the list}
    property DatabaseName: TFileName read GetDatabaseName write SetDatabaseName;
    property SQL: TStrings read GetSQL write SetSQL;
    property Active: Boolean read GetActive write SetActive default False;

    {Runtime subcomponent properties for access to their methods}
    property Query: TabcSubQuery read FSubQuery write FSubQuery;
    property DataSource: TabcSubDataSource read FSubDataSource;
    property Navigator: TabcDBSuperNav read FSubNavigator;
    property Grid: TabcDBFixedGrid read FSubGrid;

  published
    {Shared & Containing Panel Properties}
    property Align;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property Enabled;
    property Color: TColor read GetColor write SetColor stored IsColorStored;
    property Ctl3D: Boolean read GetCtl3D write SetCtl3D stored IsCtl3DStored;
    property DragCursor: TCursor read FDragCursor write SetDragCursor;
    property DragMode: TDragMode read FDragMode write SetDragMode;
    property Font: TFont read GetFont write SetFont stored IsFontStored;
    property Hint: string read FHint write SetHint;
    property HelpContext: THelpContext read FHelpContext write SetHelpContext;
    property NavAlign: TabcSubNavAlignment read FSubNavAlign write SetSubNavAlign default naTop;
    property ParentColor: Boolean read FParentColor write SetParentColor default False;
    property ParentCtl3D: Boolean read FParentCtl3D write SetParentCtl3D default True;
    property ParentFont: Boolean read FParentFont write SetParentFont default True;
    property ParentShowHint: Boolean read FParentShowHint write SetParentShowHint default True;
    property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly default True;
    property ShowHint: Boolean read GetShowHint write SetShowHint stored IsShowHintStored;
    property TabOrder: TTabOrder read FTabOrder write SetTabOrder default 0;
    property TabStop: Boolean read FTabStop write SetTabStop default True;
    property Visible: Boolean read FVisible write SetVisible default True;

    {SubQuery Properties}
    property A1DatabaseName: TFileName read GetDatabaseName write SetDatabaseName stored False;
    property A2SQL: TStrings read GetSQL write SetSQL stored False;
    property A3Active: Boolean read GetActive write SetActive stored False;
    property AutoCalcFields: Boolean read GetAutoCalcFields write SetAutoCalcFields stored False;
    property Params: TParams read GetParams write SetParams stored False;
    property UniDirectional: Boolean read GetUniDirectional write SetUniDirectional stored False;

    {SubNavigator Properties}
    property GlyphColor: TabcGlyphColor read GetGlyphColor write SetGlyphColor default gcColored;
    property Hints: TStrings read GetHints write SetHints;
    property NavHeight: Integer read GetNavHeight write SetNavHeight;
    property NavSpeed: TabcNavSpeed read GetNavSpeed write SetNavSpeed default nsMedium;
    property MoveBy: Word read GetMoveBy write SetMoveBy default 50;
    property VisibleButtons: TabcSuperNavBtnSet read GetVisibleButtons write SetVisibleButtons
      default [snbFirst, snbMoveBack, snbPrior, snbNext, snbMoveFwd, snbLast, snbBookmark, snbRefresh];

    {SubGrid Properties}
    property Columns: TDBGridColumns read GetColumns write SetColumns;
    property FixedCols: Integer read GetFixedCols write SetFixedCols;
    property FixedColor: TColor read GetFixedColor write SetFixedColor default clBtnFace;
    property Options: TDBGridOptions read GetOptions write SetOptions
      default [dgTitles, dgIndicator, dgColumnResize, dgColLines,
      dgRowLines, dgTabs, dgConfirmDelete, dgCancelOnExit];
    property TitleFont: TFont read GetTitleFont write SetTitleFont;

    {SubQuery Events}
    property BeforeOpen: TDataSetNotifyEvent read GetBeforeOpen write SetBeforeOpen stored False;
    property AfterOpen: TDataSetNotifyEvent read GetAfterOpen write SetAfterOpen stored False;
    property BeforeClose: TDataSetNotifyEvent read GetBeforeClose write SetBeforeClose stored False;
    property AfterClose: TDataSetNotifyEvent read GetAfterClose write SetAfterClose stored False;
    property BeforeInsert: TDataSetNotifyEvent read GetBeforeInsert write SetBeforeInsert stored False;
    property AfterInsert: TDataSetNotifyEvent read GetAfterInsert write SetAfterInsert stored False;
    property BeforeEdit: TDataSetNotifyEvent read GetBeforeEdit write SetBeforeEdit stored False;
    property AfterEdit: TDataSetNotifyEvent read GetAfterEdit write SetAfterEdit stored False;
    property BeforePost: TDataSetNotifyEvent read GetBeforePost write SetBeforePost stored False;
    property AfterPost: TDataSetNotifyEvent read GetAfterPost write SetAfterPost stored False;
    property BeforeCancel: TDataSetNotifyEvent read GetBeforeCancel write SetBeforeCancel stored False;
    property AfterCancel: TDataSetNotifyEvent read GetAfterCancel write SetAfterCancel stored False;
    property BeforeDelete: TDataSetNotifyEvent read GetBeforeDelete write SetBeforeDelete stored False;
    property AfterDelete: TDataSetNotifyEvent read GetAfterDelete write SetAfterDelete stored False;
    property OnNewRecord: TDataSetNotifyEvent read GetOnNewRecord write SetOnNewRecord stored False;
    property OnCalcFields: TDataSetNotifyEvent read GetOnCalcFields write SetOnCalcFields stored False;

    {SubDataSource Events}
    property OnStateChange: TNotifyEvent read GetOnStateChange write SetOnStateChange stored False;
    property OnDataChange: TDataChangeEvent read GetOnDataChange write SetOnDataChange stored False;
    property OnUpdateData: TNotifyEvent read GetOnUpdateData write SetOnUpdateData stored False;

    {SubNavigator - No Events Used}

    {SubGrid Events - all published except owner drawing and column move events}
    property OnColEnter: TNotifyEvent read GetOnColEnter write SetOnColEnter;
    property OnColExit: TNotifyEvent read GetOnColExit write SetOnColExit;
    property OnDblClick: TNotifyEvent read GetOnDblClick write SetOnDblClick;
    property OnDragDrop: TDragDropEvent read GetOnDragDrop write SetOnDragDrop;
    property OnDragOver: TDragOverEvent read GetOnDragOver write SetOnDragOver;
    property OnEndDrag: TEndDragEvent read GetOnEndDrag write SetOnEndDrag;
    property OnEnter: TNotifyEvent read GetOnEnter write SetOnEnter;
    property OnExit: TNotifyEvent read GetOnExit write SetOnExit;
    property OnKeyDown: TKeyEvent read GetOnKeyDown write SetOnKeyDown;
    property OnKeyPress: TKeyPressEvent read GetOnKeyPress write SetOnKeyPress;
    property OnKeyUp: TKeyEvent read GetOnKeyUp write SetOnKeyUp;
    property OnEditButtonClick: TNotifyEvent read GetOnEditButtonClick write SetOnEditButtonClick;
    property OnStartDrag: TStartDragEvent read GetOnStartDrag write SetOnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

  {TabcDBTableGrid}

  TabcDBTableGrid = class(TCustomPanel)
  private
    {Sub Components}
    FSubTable: TabcSubTable;
    FSubDataSource: TabcSubDataSource;
    FSubNavigator: TabcDBSuperNav;
    FSubGrid: TabcDBFixedGrid;
    FColumn: TColumn; {workaround for bug in Delphi column streaming}
    {Shared Properties}
    FDragCursor: TCursor;
    FDragMode: TDragMode;
    FHint: string;
    FHelpContext: THelpContext;
    FParentColor: Boolean;
    FParentCtl3D: Boolean;
    FParentFont: Boolean;
    FParentShowHint: Boolean;
    FPopupMenu: TPopupMenu;
    FTabOrder: TTabOrder;
    FTabStop: Boolean;
    FVisible: Boolean;
    FReadOnly: Boolean;
    FSubNavAlign: TabcSubNavAlignment;
    FHints: TStrings;
    function GetFixedCols: Integer;
    procedure SetFixedCols(Value: Integer);

    {Shared Property Access Methods - Properties}
    function  GetColor: TColor;
    function  GetCtl3D: Boolean;
    function  GetFont: TFont;
    function  GetShowHint: Boolean;

    function  IsColorStored: Boolean;
    function  IsCtl3DStored: Boolean;
    function  IsFontStored: Boolean;
    function  IsShowHintStored: Boolean;

    procedure SetReadOnly(Value: Boolean);
    procedure SetSubNavAlign (Value: TabcSubNavAlignment);
    procedure SetDragCursor(Value: TCursor);
    procedure SetColor(Value: TColor);
    procedure SetCtl3D(Value: Boolean);
    procedure SetFont(Value: TFont);
    procedure SetHint(Value: string);
    procedure SetHelpContext(Value: THelpContext);
    procedure SetParentColor(Value: Boolean);
    procedure SetParentCtl3D(Value: Boolean);
    procedure SetParentFont(Value: Boolean);
    procedure SetParentShowHint(Value: Boolean);
    procedure SetPopupMenu(Value: TPopupMenu);
    procedure SetShowHint(Value: Boolean);
    procedure SetTabOrder(Value: TTabOrder);
    procedure SetTabStop(Value: Boolean);
    procedure SetVisible(Value: Boolean);

    {SubTable Property Access Methods - Properties}
    function GetActive: Boolean;
    function GetAutoCalcFields: Boolean;
    function GetDatabaseName: TFileName;
    function GetExclusive: Boolean;
    function GetIndexFieldNames: string;
    function GetIndexName: string;
    function GetMasterFields: string;
    function GetMasterSource: TDataSource;
    function GetTableName: TFileName;
    function GetTableType: TTableType;
    procedure SetActive(Value: Boolean);
    procedure SetAutoCalcFields(Value: Boolean);
    procedure SetDatabaseName(Value: TFileName);
    procedure SetExclusive(Value: Boolean);
    procedure SetIndexFieldNames(Value: string);
    procedure SetIndexName(Value: string);
    procedure SetMasterFields(Value: string);
    procedure SetMasterSource(Value: TDataSource);
    procedure SetTableName(Value: TFileName);
    procedure SetTableType(Value: TTableType);
    {SubTable Property Access Methods - Events}
    function GetBeforeOpen: TDataSetNotifyEvent;
    function GetAfterOpen: TDataSetNotifyEvent;
    function GetBeforeClose: TDataSetNotifyEvent;
    function GetAfterClose: TDataSetNotifyEvent;
    function GetBeforeInsert: TDataSetNotifyEvent;
    function GetAfterInsert: TDataSetNotifyEvent;
    function GetBeforeEdit: TDataSetNotifyEvent;
    function GetAfterEdit: TDataSetNotifyEvent;
    function GetBeforePost: TDataSetNotifyEvent;
    function GetAfterPost: TDataSetNotifyEvent;
    function GetBeforeCancel: TDataSetNotifyEvent;
    function GetAfterCancel: TDataSetNotifyEvent;
    function GetBeforeDelete: TDataSetNotifyEvent;
    function GetAfterDelete: TDataSetNotifyEvent;
    function GetOnNewRecord: TDataSetNotifyEvent;
    function GetOnCalcFields: TDataSetNotifyEvent;
    procedure SetBeforeOpen(Value: TDataSetNotifyEvent);
    procedure SetAfterOpen(Value: TDataSetNotifyEvent);
    procedure SetBeforeClose(Value: TDataSetNotifyEvent);
    procedure SetAfterClose(Value: TDataSetNotifyEvent);
    procedure SetBeforeInsert(Value: TDataSetNotifyEvent);
    procedure SetAfterInsert(Value: TDataSetNotifyEvent);
    procedure SetBeforeEdit(Value: TDataSetNotifyEvent);
    procedure SetAfterEdit(Value: TDataSetNotifyEvent);
    procedure SetBeforePost(Value: TDataSetNotifyEvent);
    procedure SetAfterPost(Value: TDataSetNotifyEvent);
    procedure SetBeforeCancel(Value: TDataSetNotifyEvent);
    procedure SetAfterCancel(Value: TDataSetNotifyEvent);
    procedure SetBeforeDelete(Value: TDataSetNotifyEvent);
    procedure SetAfterDelete(Value: TDataSetNotifyEvent);
    procedure SetOnNewRecord(Value: TDataSetNotifyEvent);
    procedure SetOnCalcFields(Value: TDataSetNotifyEvent);

    {SubDataSource Property Access Methods - Events}
    function GetOnStateChange: TNotifyEvent;
    function GetOnDataChange: TDataChangeEvent;
    function GetOnUpdateData: TNotifyEvent;
    procedure SetOnStateChange(Value: TNotifyEvent);
    procedure SetOnDataChange(Value: TDataChangeEvent);
    procedure SetOnUpdateData(Value: TNotifyEvent);

    {SubNavigator Property Access Methods - Properties}
    {NB: No Event Properties published for the Navigator}
    function GetConfirmDelete: Boolean;
    function GetGlyphColor: TabcGlyphColor;
    function GetHints: TStrings;
    function GetNavSpeed: TabcNavSpeed;
    function GetMoveBy: Word;
    function GetVisibleButtons: TabcSuperNavBtnSet;
    procedure SetConfirmDelete(Value: Boolean);
    procedure SetGlyphColor(Value: TabcGlyphColor);
    procedure SetHints(Value: TStrings);
    procedure SetNavSpeed(Value: TabcNavSpeed);
    procedure SetMoveBy(Value: Word);
    procedure SetVisibleButtons(Value: TabcSuperNavBtnSet);
    function GetNavHeight: Integer;
    procedure SetNavHeight(Value: Integer);

    {SubGrid Property Access Methods - Properties}
    function GetFixedColor: TColor;
    function GetOptions: TDBGridOptions;
    function GetTitleFont: TFont;
    procedure SetFixedColor(Value: TColor);
    procedure SetOptions(Value: TDBGridOptions);
    procedure SetTitleFont(Value: TFont);
    function GetDefaultDrawing: Boolean;                      {protected}
    procedure SetDefaultDrawing(Value: Boolean);              {protected}
    {SubGrid Property Access Methods - Events}
    function GetOnColEnter: TNotifyEvent;
    function GetOnColExit: TNotifyEvent;
    function GetOnDrawDataCell: TDrawDataCellEvent;           {protected}
    function GetOnDblClick: TNotifyEvent;
    function GetOnDragDrop: TDragDropEvent;
    function GetOnDragOver: TDragOverEvent;
    function GetOnEndDrag: TEndDragEvent;
    function GetOnEnter: TNotifyEvent;
    function GetOnExit: TNotifyEvent;
    function GetOnKeyDown: TKeyEvent;
    function GetOnKeyPress: TKeyPressEvent;
    function GetOnKeyUp: TKeyEvent;
    procedure SetOnColEnter(Value: TNotifyEvent);
    procedure SetOnColExit(Value: TNotifyEvent);
    procedure SetOnDrawDataCell(Value: TDrawDataCellEvent);   {protected}
    procedure SetOnDblClick(Value: TNotifyEvent);
    procedure SetOnDragDrop(Value: TDragDropEvent);
    procedure SetOnDragOver(Value: TDragOverEvent);
    procedure SetOnEndDrag(Value: TEndDragEvent);
    procedure SetOnEnter(Value: TNotifyEvent);
    procedure SetOnExit(Value: TNotifyEvent);
    procedure SetOnKeyDown(Value: TKeyEvent);
    procedure SetOnKeyPress(Value: TKeyPressEvent);
    procedure SetOnKeyUp(Value: TKeyEvent);
    function GetColumns: TDBGridColumns;
    procedure SetColumns(Value: TDBGridColumns);
    function GetOnColumnMoved: TMovedEvent;                     {protected}
    procedure SetOnColumnMoved(Value: TMovedEvent);             {protected}
    function GetOnDrawColumnCell: TDrawColumnCellEvent;         {protected}
    procedure SetOnDrawColumnCell(Value: TDrawColumnCellEvent); {protected}
    function GetOnEditButtonClick: TNotifyEvent;
    procedure SetOnEditButtonClick(Value: TNotifyEvent);
    function GetOnStartDrag: TStartDragEvent;
    procedure SetOnStartDrag(Value: TStartDragEvent);

    {Initialise Properties}
    procedure InitSubTableProperties(var ATable: TabcSubTable);
    procedure InitSubDataSourceProperties(var ADataSource: TabcSubDataSource);
    procedure InitSubSuperNavProperties(var ADBSuperNav: TabcDBSuperNav);
    procedure InitSubGridProperties(var ADBGrid: TabcDBFixedGrid);
    procedure InitSharedProperties;

    procedure AddSubTable(ASubTable: TabcSubTable);
    procedure AddSubDataSource(ASubDataSource: TabcSubDataSource);
    procedure NameSubTable(AOwner: TComponent);
    procedure NameSubDataSource(AOwner: TComponent);

  protected

    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure Loaded; override;
    procedure SetDragMode(Value: TDragMode); override;
    procedure SetEnabled(Value: Boolean); override;

    {Grid Owner Draw Behaviour}
    property DefaultDrawing: Boolean read GetDefaultDrawing write SetDefaultDrawing default True;
    property OnDrawDataCell: TDrawDataCellEvent read GetOnDrawDataCell write SetOnDrawDataCell;
    property OnColumnMoved: TMovedEvent read GetOnColumnMoved write SetOnColumnMoved;
    property OnDrawColumnCell: TDrawColumnCellEvent read GetOnDrawColumnCell write SetOnDrawColumnCell;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    {Runtime properties to allow regular names to be used in code}
    {Note that the same published properties are named A1..., A2, A3 and then
     renamed by the property editor to make them sort to the top of the list}
    property DatabaseName: TFileName read GetDatabaseName write SetDatabaseName;
    property TableName: TFileName read GetTableName write SetTableName;
    property Active: Boolean read GetActive write SetActive default False;

    {Runtime subcomponent properties for access to their methods}
    property Table: TabcSubTable read FSubTable write FSubTable;
    property DataSource: TabcSubDataSource read FSubDataSource;
    property Navigator: TabcDBSuperNav read FSubNavigator;
    property Grid: TabcDBFixedGrid read FSubGrid;

  published
    {Shared & Container Control Properties}
    property Align;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property Enabled;
    property Color: TColor read GetColor write SetColor stored IsColorStored;
    property Ctl3D: Boolean read GetCtl3D write SetCtl3D stored IsCtl3DStored;
    property DragCursor: TCursor read FDragCursor write SetDragCursor;
    property DragMode: TDragMode read FDragMode write SetDragMode;
    property Font: TFont read GetFont write SetFont stored IsFontStored;
    property Hint: string read FHint write SetHint;
    property HelpContext: THelpContext read FHelpContext write SetHelpContext;
    property NavAlign: TabcSubNavAlignment read FSubNavAlign write SetSubNavAlign default naTop;
    property ParentColor: Boolean read FParentColor write SetParentColor default False;
    property ParentCtl3D: Boolean read FParentCtl3D write SetParentCtl3D default True;
    property ParentFont: Boolean read FParentFont write SetParentFont default True;
    property ParentShowHint: Boolean read FParentShowHint write SetParentShowHint default True;
    property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly default False;
    property ShowHint: Boolean read GetShowHint write SetShowHint stored IsShowHintStored;
    property TabOrder: TTabOrder read FTabOrder write SetTabOrder default 0;
    property TabStop: Boolean read FTabStop write SetTabStop default True;
    property Visible: Boolean read FVisible write SetVisible default True;

    {SubTable Properties}
    property A1DatabaseName: TFileName read GetDatabaseName write SetDatabaseName stored False;
    property A2TableName: TFileName read GetTableName write SetTableName stored False;
    property A3Active: Boolean read GetActive write SetActive stored False;
    property AutoCalcFields: Boolean read GetAutoCalcFields write SetAutoCalcFields stored False;
    property Exclusive: Boolean read GetExclusive write SetExclusive stored False;
    property IndexFieldNames: string read GetIndexFieldNames write SetIndexFieldNames stored False;
    property IndexName: string read GetIndexName write SetIndexName stored False;
    property MasterFields: string read GetMasterFields write SetMasterFields stored False;
    property MasterSource: TDataSource read GetMasterSource write SetMasterSource stored False;
    property TableType: TTableType read GetTableType write SetTableType stored False;

    {SubNavigator Properties}
    property ConfirmDelete: Boolean read GetConfirmDelete write SetConfirmDelete default True;
    property GlyphColor: TabcGlyphColor read GetGlyphColor write SetGlyphColor default gcColored;
    property Hints: TStrings read GetHints write SetHints;
    property NavHeight: Integer read GetNavHeight write SetNavHeight;
    property NavSpeed: TabcNavSpeed read GetNavSpeed write SetNavSpeed default nsMedium;
    property MoveBy: Word read GetMoveBy write SetMoveBy default 50;
    property VisibleButtons: TabcSuperNavBtnSet read GetVisibleButtons write SetVisibleButtons
      default [snbFirst, snbMoveBack, snbPrior, snbNext, snbMoveFwd, snbLast,
        snbAppend, snbBookmark, snbInsert, snbDelete, snbEdit, snbPost, snbCancel,
        snbRefresh];

    {SubGrid Properties}
    property Columns: TDBGridColumns read GetColumns write SetColumns;
    property FixedCols: Integer read GetFixedCols write SetFixedCols;
    property FixedColor: TColor read GetFixedColor write SetFixedColor default clBtnFace;
    property Options: TDBGridOptions read GetOptions write SetOptions
      default [dgEditing, dgTitles, dgIndicator, dgColumnResize, dgColLines,
      dgRowLines, dgTabs, dgConfirmDelete, dgCancelOnExit];
    property TitleFont: TFont read GetTitleFont write SetTitleFont;

    {SubTable Events}
    property BeforeOpen: TDataSetNotifyEvent read GetBeforeOpen write SetBeforeOpen stored False;
    property AfterOpen: TDataSetNotifyEvent read GetAfterOpen write SetAfterOpen stored False;
    property BeforeClose: TDataSetNotifyEvent read GetBeforeClose write SetBeforeClose stored False;
    property AfterClose: TDataSetNotifyEvent read GetAfterClose write SetAfterClose stored False;
    property BeforeInsert: TDataSetNotifyEvent read GetBeforeInsert write SetBeforeInsert stored False;
    property AfterInsert: TDataSetNotifyEvent read GetAfterInsert write SetAfterInsert stored False;
    property BeforeEdit: TDataSetNotifyEvent read GetBeforeEdit write SetBeforeEdit stored False;
    property AfterEdit: TDataSetNotifyEvent read GetAfterEdit write SetAfterEdit stored False;
    property BeforePost: TDataSetNotifyEvent read GetBeforePost write SetBeforePost stored False;
    property AfterPost: TDataSetNotifyEvent read GetAfterPost write SetAfterPost stored False;
    property BeforeCancel: TDataSetNotifyEvent read GetBeforeCancel write SetBeforeCancel stored False;
    property AfterCancel: TDataSetNotifyEvent read GetAfterCancel write SetAfterCancel stored False;
    property BeforeDelete: TDataSetNotifyEvent read GetBeforeDelete write SetBeforeDelete stored False;
    property AfterDelete: TDataSetNotifyEvent read GetAfterDelete write SetAfterDelete stored False;
    property OnNewRecord: TDataSetNotifyEvent read GetOnNewRecord write SetOnNewRecord stored False;
    property OnCalcFields: TDataSetNotifyEvent read GetOnCalcFields write SetOnCalcFields stored False;

    {SubDataSource Events}
    property OnStateChange: TNotifyEvent read GetOnStateChange write SetOnStateChange stored False;
    property OnDataChange: TDataChangeEvent read GetOnDataChange write SetOnDataChange stored False;
    property OnUpdateData: TNotifyEvent read GetOnUpdateData write SetOnUpdateData stored False;

    {SubNavigator - No Events Used}

    {SubGrid Events - all published except owner drawing and column move events}
    property OnColEnter: TNotifyEvent read GetOnColEnter write SetOnColEnter;
    property OnColExit: TNotifyEvent read GetOnColExit write SetOnColExit;
    property OnDblClick: TNotifyEvent read GetOnDblClick write SetOnDblClick;
    property OnDragDrop: TDragDropEvent read GetOnDragDrop write SetOnDragDrop;
    property OnDragOver: TDragOverEvent read GetOnDragOver write SetOnDragOver;
    property OnEndDrag: TEndDragEvent read GetOnEndDrag write SetOnEndDrag;
    property OnEnter: TNotifyEvent read GetOnEnter write SetOnEnter;
    property OnExit: TNotifyEvent read GetOnExit write SetOnExit;
    property OnKeyDown: TKeyEvent read GetOnKeyDown write SetOnKeyDown;
    property OnKeyPress: TKeyPressEvent read GetOnKeyPress write SetOnKeyPress;
    property OnKeyUp: TKeyEvent read GetOnKeyUp write SetOnKeyUp;
    property OnEditButtonClick: TNotifyEvent read GetOnEditButtonClick write SetOnEditButtonClick;
    property OnStartDrag: TStartDragEvent read GetOnStartDrag write SetOnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

implementation

uses Clipbrd, DBConsts;

{TabcSubQuery}

procedure TabcSubQuery.ReadState(Reader: TReader);
begin
  if Reader.Parent is TabcDBQueryGrid then
    TabcDBQueryGrid(Reader.Parent).FSubQuery.Name := '';
  inherited ReadState(Reader);
  if Reader.Parent is TabcDBQueryGrid then Parent := Reader.Parent;
end;

procedure TabcSubQuery.SetParent(AParent: TComponent);
begin
  if AParent <> nil then TabcDBQueryGrid(AParent).AddSubQuery(self);
end;

function TabcSubQuery.HasParent: Boolean;
begin
  HasParent := True;
end;

{TabcSubTable}

procedure TabcSubTable.ReadState(Reader: TReader);
begin
  if Reader.Parent is TabcDBTableGrid then
    TabcDBTableGrid(Reader.Parent).FSubTable.Name := '';
  inherited ReadState(Reader);
  if Reader.Parent is TabcDBTableGrid then Parent := Reader.Parent;
end;

procedure TabcSubTable.SetParent(AParent: TComponent);
begin
  if AParent <> nil then TabcDBTableGrid(AParent).AddSubTable(self);
end;

function TabcSubTable.HasParent: Boolean;
begin
  HasParent := True;
end;

{TabcSubDataSource}

procedure TabcSubDataSource.ReadState(Reader: TReader);
begin
  if Reader.Parent is TabcDBQueryGrid then
    TabcDBQueryGrid(Reader.Parent).FSubDataSource.Name := ''
  else if Reader.Parent is TabcDBTableGrid then
    TabcDBTableGrid(Reader.Parent).FSubDataSource.Name := '';
  inherited ReadState(Reader);
  if (Reader.Parent is TabcDBQueryGrid) or
     (Reader.Parent is TabcDBTableGrid) then
     Parent := Reader.Parent;
end;

procedure TabcSubDataSource.SetParent(AParent: TComponent);
begin
  if AParent is TabcDBQueryGrid then begin
    if AParent <> nil then TabcDBQueryGrid(AParent).AddSubDataSource(self);
  end
  else
    if AParent is TabcDBTableGrid then begin
      if AParent <> nil then TabcDBTableGrid(AParent).AddSubDataSource(self);
    end;
end;

function TabcSubDataSource.HasParent: Boolean;
begin
  HasParent := True;
end;

{TabcDBQueryGrid}

constructor TabcDBQueryGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csReplicatable];
  Width := 360;
  Height := 240;
  FSubQuery := TabcSubQuery.Create(AOwner);
  FSubDataSource := TabcSubDataSource.Create(AOwner);
  if not (csLoading in AOwner.ComponentState) and
     not (csReading in AOwner.ComponentState) then begin
    NameSubQuery(AOwner);
    NameSubDataSource(AOwner);
  end;
  FSubNavigator := TabcDBSuperNav.Create(Self);
  FSubNavigator.Parent := self;
  FSubGrid := TabcDBFixedGrid.Create(Self);
  FSubGrid.Parent := self;
  {add a dummy column to work around bug loading grid columns}
  if (csLoading in AOwner.ComponentState) or
     (csReading in AOwner.ComponentState) then begin
    FColumn := TColumn.Create(FSubGrid.Columns);
  end;
  InitSubQueryProperties(FSubQuery);
  InitSubDataSourceProperties(FSubDataSource);
  InitSubSuperNavProperties(FSubNavigator);
  InitSubGridProperties(FSubGrid);
  InitSharedProperties;
  FHints := TStringList.Create;
end;

destructor TabcDBQueryGrid.Destroy;
begin
  FHints.Free;
  FSubGrid.Free;
  FSubNavigator.Free;
  FSubDataSource.Free;
  FSubQuery.Free;
  inherited Destroy;
end;

procedure TabcDBQueryGrid.AddSubQuery(ASubQuery: TabcSubQuery);
begin
  FSubQuery.Free;
  FSubQuery := ASubQuery;
  FSubDataSource.DataSet := FSubQuery;
  ASubQuery.FParent := TComponent(Self);
end;

procedure TabcDBQueryGrid.AddSubDataSource(ASubDataSource: TabcSubDataSource);
begin
  FSubDataSource.Free;
  FSubDataSource := ASubDataSource;
  FSubGrid.DataSource := FSubDataSource;
  FSubNavigator.DataSource := FSubDataSource;
  ASubDataSource.FParent := TComponent(Self);
end;

procedure TabcDBQueryGrid.NameSubQuery(AOwner: TComponent);
var
  I,J: integer;
begin
  J := 0;
  with AOwner do
    for I := 0 to ComponentCount - 1 do
      if Components[I] is TabcSubQuery then
        Inc(J);
  FSubQuery.Name := 'SubQuery' + InttoStr(J);
end;

procedure TabcDBQueryGrid.NameSubDataSource(AOwner: TComponent);
var
  I,J: integer;
begin
  J := 0;
  with AOwner do
    for I := 0 to ComponentCount - 1 do
      if Components[I] is TabcSubDataSource then
        Inc(J);
  FSubDataSource.Name := 'SubDataSource' + InttoStr(J);
end;

{Component Filing}

procedure TabcDBQueryGrid.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
  Proc(FSubQuery);
  Proc(FSubDataSource);
end;

procedure TabcDBQueryGrid.InitSubQueryProperties(var AQuery: TabcSubQuery);
begin
  with AQuery do begin
    AutoCalcFields := True;
    UpdateMode := upWhereAll;
  end;
  AQuery.Tag := Tag;
end;

procedure TabcDBQueryGrid.InitSubDataSourceProperties(var ADataSource: TabcSubDataSource);
begin
  ADataSource.AutoEdit := True;
  ADataSource.Dataset := FSubQuery as TDataSet;
  ADataSource.Enabled := Enabled;
  ADataSource.Tag := Tag;
end;

procedure TabcDBQueryGrid.InitSubSuperNavProperties(var ADBSuperNav: TabcDBSuperNav);
begin
  ADBSuperNav.Align := alTop;
  ADBSuperNav.ConfirmDelete := True;
  ADBSuperNav.Cursor := Cursor;
  ADBSuperNav.DataSource := FSubDataSource;
  ADBSuperNav.DragCursor := DragCursor;
  ADBSuperNav.DragMode := DragMode;
  ADBSuperNav.Enabled := Enabled;
  ADBSuperNav.GlyphColor := gcColored;
  ADBSuperNav.Height := 25;
  ADBSuperNav.HelpContext := HelpContext;
  ADBSuperNav.Hint := Hint;
  ADBSuperNav.MoveBy := 50;
  ADBSuperNav.NavSpeed := nsMedium;
  ADBSuperNav.Ctl3D := Ctl3D;
  ADBSuperNav.ParentCtl3D := ParentCtl3D;
  ADBSuperNav.ShowHint := ShowHint;
  ADBSuperNav.ParentShowHint := ParentShowHint;
  ADBSuperNav.PopupMenu := PopupMenu;
  ADBSuperNav.TabOrder := TabOrder;
  ADBSuperNav.TabStop := TabStop;
  ADBSuperNav.Tag := Tag;
  ADBSuperNav.Visible := True;
  ADBSuperNav.VisibleButtons :=  [snbFirst, snbMoveBack, snbPrior, snbNext,
    snbMoveFwd, snbLast, snbBookmark, snbRefresh];
end;

procedure TabcDBQueryGrid.InitSubGridProperties(var ADBGrid: TabcDBFixedGrid);
begin
  ADBGrid.Align := alClient;
  ADBGrid.DataSource := FSubDataSource;
  ADBGrid.Cursor := Cursor;
  ADBGrid.DefaultDrawing := True;
  ADBGrid.DragCursor := DragCursor;
  ADBGrid.DragMode := DragMode;
  ADBGrid.Enabled := Enabled;
  ADBGrid.HelpContext := HelpContext;
  ADBGrid.Hint := Hint;
  ADBGrid.Color := Color;
  ADBGrid.ParentColor := ParentColor;
  ADBGrid.Ctl3D := Ctl3D;
  ADBGrid.ParentCtl3D := ParentCtl3D;
  ADBGrid.Font := Font;
  ADBGrid.ParentFont := ParentFont;
  ADBGrid.ShowHint := ShowHint;
  ADBGrid.ParentShowHint := ParentShowHint;
  ADBGrid.PopupMenu := PopupMenu;
  ADBGrid.ReadOnly := False;
  ADBGrid.TabOrder := TabOrder;
  ADBGrid.TabStop := TabStop;
  ADBGrid.Tag := Tag;
  ADBGrid.Visible := True;
  ADBGrid.Options := [dgTitles, dgIndicator, dgColumnResize, dgColLines,
    dgRowLines, dgTabs, dgConfirmDelete, dgCancelOnExit];
end;

procedure TabcDBQueryGrid.Loaded;
begin
  inherited Loaded;
  {Set common properties shared between the main component and sub components}
  FSubQuery.Tag := Tag;
  FSubDataSource.Enabled := Enabled;
  FSubDataSource.Tag := Tag;
  FSubNavigator.Cursor := Cursor;
  FSubNavigator.DragCursor := DragCursor;
  FSubNavigator.DragMode := DragMode;
  FSubNavigator.Enabled := Enabled;
  FSubNavigator.HelpContext := HelpContext;
  FSubNavigator.Hint := Hint;
  FSubNavigator.Ctl3D := Ctl3D;
  FSubNavigator.ParentCtl3D := ParentCtl3D;
  FSubNavigator.ShowHint := ShowHint;
  FSubNavigator.ParentShowHint := ParentShowHint;
  FSubNavigator.PopupMenu := PopupMenu;
  FSubNavigator.TabOrder := TabOrder;
  FSubNavigator.TabStop := TabStop;
  FSubNavigator.Tag := Tag;
  FSubNavigator.Visible := Visible;
  FSubGrid.Cursor := Cursor;
  FSubGrid.DragCursor := DragCursor;
  FSubGrid.DragMode := DragMode;
  FSubGrid.Enabled := Enabled;
  FSubGrid.HelpContext := HelpContext;
  FSubGrid.Hint := Hint;
  FSubGrid.Color := Color;
  FSubGrid.ParentColor := ParentColor;
  FSubGrid.Ctl3D := Ctl3D;
  FSubGrid.ParentCtl3D := ParentCtl3D;
  FSubGrid.Font := Font;
  FSubGrid.ParentFont := ParentFont;
  FSubGrid.ShowHint := ShowHint;
  FSubGrid.ParentShowHint := ParentShowHint;
  FSubGrid.PopupMenu := PopupMenu;
  FSubGrid.TabOrder := TabOrder;
  FSubGrid.TabStop := TabStop;
  FSubGrid.Tag := Tag;
  FSubGrid.Visible := Visible;
  SetReadOnly(FReadOnly);
  with FSubGrid.Columns do
    if (Count = 1) and (Columns[0].FieldName = 'ABCNullColumn') then
      RebuildColumns;
  if FHints.Count > 0 then begin
    FSubNavigator.Hints := FHints;
    FHints.Clear;
  end;
end;

procedure TabcDBQueryGrid.InitSharedProperties;
begin
  DragCursor := crDrag;
  DragMode :=  dmManual;
  Enabled := True;
  Color := clWindow;
  Ctl3D := True;
  ParentCtl3D := True;
  ParentFont := True;
  ParentShowHint := True;
  ReadOnly := True;
  TabStop := True;
  Visible := True;
  NavAlign := naTop;
end;

{Shared Property Access Methods - Properties}

function  TabcDBQueryGrid.GetColor: TColor; begin Result := inherited Color; end;
function  TabcDBQueryGrid.GetCtl3D: Boolean; begin Result := inherited Ctl3D; end;
function  TabcDBQueryGrid.GetFont: TFont; begin Result := inherited Font; end;
function  TabcDBQueryGrid.GetShowHint: Boolean; begin Result := inherited ShowHint; end;

function TabcDBQueryGrid.IsColorStored: Boolean; begin Result := not ParentColor; end;
function TabcDBQueryGrid.IsCtl3DStored: Boolean; begin Result := not ParentCtl3D; end;
function TabcDBQueryGrid.IsFontStored: Boolean; begin Result := not ParentFont; end;
function TabcDBQueryGrid.IsShowHintStored: Boolean; begin Result := not ParentShowHint; end;

procedure TabcDBQueryGrid.SetDragCursor(Value: TCursor);
begin
  if Value <> FDragCursor then begin
    FSubNavigator.DragCursor := Value;
    FSubGrid.DragCursor := Value;
    FDragCursor := Value;
  end;
end;

procedure TabcDBQueryGrid.SetDragMode(Value: TDragMode);
begin
  if Value <> FDragMode then begin
    FSubNavigator.DragMode := Value;
    FSubGrid.DragMode := Value;
    FDragMode := Value;
  end;
end;

procedure TabcDBQueryGrid.SetEnabled(Value: Boolean);
begin
  if Value <> Enabled then begin
    FSubDataSource.Enabled := Value;
    FSubNavigator.Enabled := Value;
    FSubGrid.Enabled := Value;
    inherited Enabled := Value;
  end;
end;

procedure TabcDBQueryGrid.SetColor(Value: TColor);
begin
  if Value <> inherited Color then begin
    ParentColor := False;
    inherited Color := Value;
    FSubGrid.Color := Value;
  end;
end;

procedure TabcDBQueryGrid.SetCtl3D(Value: Boolean);
begin
  if Value <> inherited Ctl3D then begin
    ParentCtl3D := False;
    FSubNavigator.Ctl3D := Value;
    FSubGrid.Ctl3D := Value;
    inherited Ctl3D := Value;
  end;
end;

procedure TabcDBQueryGrid.SetFont(Value: TFont);
begin
  if Value <> inherited Font then begin
    ParentFont := False;
    FSubGrid.Font := Value;
    inherited Font := Value;
  end;
end;

procedure TabcDBQueryGrid.SetHint(Value: string);
begin
  if Value <> FHint then begin
    FSubNavigator.Hint := Value;
    FSubGrid.Hint := Value;
    FHint := Value;
  end;
end;

procedure TabcDBQueryGrid.SetHelpContext(Value: THelpContext);
begin
  if Value <> FHelpContext then begin
    FSubNavigator.HelpContext := Value;
    FSubGrid.HelpContext := Value;
    FHelpContext := Value;
  end;
end;

procedure TabcDBQueryGrid.SetParentColor(Value: Boolean);
begin
  if Value <> FParentColor then begin
    FParentColor := Value;
    inherited ParentColor := Value;
    FSubGrid.ParentColor := Value;
  end;
end;

procedure TabcDBQueryGrid.SetParentCtl3D(Value: Boolean);
begin
  if Value <> FParentCtl3D then begin
    FParentCtl3D := Value;
    inherited ParentCtl3D := Value;
    FSubNavigator.ParentCtl3D := Value;
    FSubGrid.ParentCtl3D := Value;
  end;
end;

procedure TabcDBQueryGrid.SetParentFont(Value: Boolean);
begin
  if Value <> FParentFont then begin
    FParentFont := Value;
    inherited ParentFont := Value;
    FSubGrid.ParentFont := Value;
  end;
end;

procedure TabcDBQueryGrid.SetParentShowHint(Value: Boolean);
begin
  if Value <> FParentShowHint then begin
    FParentShowHint := Value;
    inherited ParentShowHint := Value;
    FSubNavigator.ParentShowHint := Value;
    FSubGrid.ParentShowHint := Value;
  end;
end;

procedure TabcDBQueryGrid.SetPopupMenu(Value: TPopupMenu);
begin
  if Value <> FPopupMenu then begin
    FSubNavigator.PopupMenu := Value;
    FSubGrid.PopupMenu := Value;
    FPopupMenu := Value;
  end;
end;

procedure TabcDBQueryGrid.SetShowHint(Value: Boolean);
begin
  if Value <> inherited ShowHint then begin
    ParentShowHint := False;
    FSubNavigator.ShowHint := Value;
    FSubGrid.ShowHint := Value;
    inherited ShowHint := Value;
  end;
end;

procedure TabcDBQueryGrid.SetTabOrder(Value: TTabOrder);
begin
  if Value <> FTabOrder then begin
    FSubNavigator.TabOrder := Value;
    FSubGrid.TabOrder := Value;
    FTabOrder := Value;
  end;
end;

procedure TabcDBQueryGrid.SetTabStop(Value: Boolean);
begin
  if Value <> FTabStop then begin
    FSubNavigator.TabStop := Value;
    FSubGrid.TabStop := Value;
    FTabStop := Value;
  end;
end;

procedure TabcDBQueryGrid.SetVisible(Value: Boolean);
begin
  if Value <> FVisible then begin
    FSubNavigator.Visible := Value;
    FSubGrid.Visible := Value;
    FVisible := Value;
    inherited Visible := Value;
  end;
end;

procedure TabcDBQueryGrid.SetReadOnly(Value: Boolean);
{Note - this always updates the properties because it is used in
 SetActive with checking of the Query.CanModify property}
begin
  FSubQuery.RequestLive := not Value;
  if Active and not FSubQuery.CanModify then begin
    Value := True;
    FSubQuery.RequestLive := False;
  end;
  FSubDataSource.AutoEdit := not Value;
  FSubGrid.ReadOnly := Value;
  if Value then begin
    FSubNavigator.VisibleButtons := FSubNavigator.VisibleButtons -
      [snbInsert, snbDelete, snbEdit, snbPost, snbCancel, snbAppend];
    FSubGrid.Options := FSubGrid.Options - [dgEditing];
  end
  else begin
    FSubNavigator.VisibleButtons := FSubNavigator.VisibleButtons +
      [snbAppend, snbInsert, snbDelete, snbEdit, snbPost, snbCancel, snbRefresh];
    FSubGrid.Options := FSubGrid.Options + [dgEditing];
  end;
  FReadOnly := Value;
end;

procedure TabcDBQueryGrid.SetSubNavAlign (Value: TabcSubNavAlignment);
begin
  if Value = naTop then begin
    FSubNavigator.Align := alTop;
    FSubNavAlign := naTop;
  end
  else begin    {value = naBottom}
    FSubNavigator.Align := alBottom;
    FSubNavAlign := naBottom;
  end;
end;

{SubQuery Property Access Methods - Properties}
function TabcDBQueryGrid.GetActive: Boolean;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.Active
  else
    Result := False;
end;

function TabcDBQueryGrid.GetAutoCalcFields: Boolean;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AutoCalcFields
  else
    Result := False;
end;

function TabcDBQueryGrid.GetDatabaseName: TFileName;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.DatabaseName
  else
    Result := '';
end;

function TabcDBQueryGrid.GetParams: TParams;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.Params
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetSQL: TStrings;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.SQL
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetUniDirectional: Boolean;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.UniDirectional
  else
    Result := False;
end;

procedure TabcDBQueryGrid.SetActive(Value: Boolean);
{Note - this also sets the ReadOnly property when
 Activate returns Query.CanModify = false}
begin
  FSubQuery.Active := Value;
  if Value and not FSubQuery.CanModify and not ReadOnly then
    SetReadOnly(True);
end;
procedure TabcDBQueryGrid.SetAutoCalcFields(Value: Boolean);
begin
  if FSubQuery.AutoCalcFields <> Value then
    FSubQuery.AutoCalcFields := Value;
end;
procedure TabcDBQueryGrid.SetDatabaseName(Value: TFileName);
begin
  if FSubQuery.DatabaseName <> Value then
    FSubQuery.DatabaseName := Value;
end;
procedure TabcDBQueryGrid.SetSQL(Value: TStrings);
begin
  if FSubQuery.SQL <> Value then
    FSubQuery.SQL := Value;
end;
procedure TabcDBQueryGrid.SetParams(Value: TParams);
begin
  if FSubQuery.Params <> Value then
    FSubQuery.Params := Value;
end;
procedure TabcDBQueryGrid.SetUniDirectional(Value: Boolean);
begin
  if FSubQuery.UniDirectional <> Value then
    FSubQuery.UniDirectional := Value;
end;

{SubQuery Property Access Methods - Events}
function TabcDBQueryGrid.GetBeforeOpen: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.BeforeOpen
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetAfterOpen: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AfterOpen
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetBeforeClose: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.BeforeClose
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetAfterClose: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AfterClose
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetBeforeInsert: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.BeforeInsert
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetAfterInsert: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AfterInsert
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetBeforeEdit: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.BeforeEdit
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetAfterEdit: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AfterEdit
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetBeforePost: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.BeforePost
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetAfterPost: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AfterPost
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetBeforeCancel: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.BeforeCancel
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetAfterCancel: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AfterCancel
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetBeforeDelete: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.BeforeDelete
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetAfterDelete: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.AfterDelete
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnNewRecord: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.OnNewRecord
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnCalcFields: TDataSetNotifyEvent;
begin
  if FSubQuery <> nil then
    Result := FSubQuery.OnCalcFields
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetBeforeOpen(Value: TDataSetNotifyEvent); begin FSubQuery.BeforeOpen := Value; end;
procedure TabcDBQueryGrid.SetAfterOpen(Value: TDataSetNotifyEvent); begin FSubQuery.AfterOpen := Value; end;
procedure TabcDBQueryGrid.SetBeforeClose(Value: TDataSetNotifyEvent); begin FSubQuery.BeforeClose := Value; end;
procedure TabcDBQueryGrid.SetAfterClose(Value: TDataSetNotifyEvent); begin FSubQuery.AfterClose := Value; end;
procedure TabcDBQueryGrid.SetBeforeInsert(Value: TDataSetNotifyEvent); begin FSubQuery.BeforeInsert := Value; end;
procedure TabcDBQueryGrid.SetAfterInsert(Value: TDataSetNotifyEvent); begin FSubQuery.AfterInsert := Value; end;
procedure TabcDBQueryGrid.SetBeforeEdit(Value: TDataSetNotifyEvent); begin FSubQuery.BeforeEdit := Value; end;
procedure TabcDBQueryGrid.SetAfterEdit(Value: TDataSetNotifyEvent); begin FSubQuery.AfterEdit := Value; end;
procedure TabcDBQueryGrid.SetBeforePost(Value: TDataSetNotifyEvent); begin FSubQuery.BeforePost := Value; end;
procedure TabcDBQueryGrid.SetAfterPost(Value: TDataSetNotifyEvent); begin FSubQuery.AfterPost := Value; end;
procedure TabcDBQueryGrid.SetBeforeCancel(Value: TDataSetNotifyEvent); begin FSubQuery.BeforeCancel := Value; end;
procedure TabcDBQueryGrid.SetAfterCancel(Value: TDataSetNotifyEvent); begin FSubQuery.AfterCancel := Value; end;
procedure TabcDBQueryGrid.SetBeforeDelete(Value: TDataSetNotifyEvent); begin FSubQuery.BeforeDelete := Value; end;
procedure TabcDBQueryGrid.SetAfterDelete(Value: TDataSetNotifyEvent); begin FSubQuery.AfterDelete := Value; end;
procedure TabcDBQueryGrid.SetOnNewRecord(Value: TDataSetNotifyEvent); begin FSubQuery.OnNewRecord := Value; end;
procedure TabcDBQueryGrid.SetOnCalcFields(Value: TDataSetNotifyEvent); begin FSubQuery.OnCalcFields := Value; end;

{SubDataSource Property Access Methods - Events}
function TabcDBQueryGrid.GetOnStateChange: TNotifyEvent;
begin
  if FSubDataSource <> nil then
    Result := FSubDataSource.OnStateChange
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnDataChange: TDataChangeEvent;
begin
  if FSubDataSource <> nil then
    Result := FSubDataSource.OnDataChange
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnUpdateData: TNotifyEvent;
begin
  if FSubDataSource <> nil then
    Result := FSubDataSource.OnUpdateData
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetOnStateChange(Value: TNotifyEvent); begin FSubDataSource.OnStateChange := Value; end;
procedure TabcDBQueryGrid.SetOnDataChange(Value: TDataChangeEvent); begin FSubDataSource.OnDataChange := Value; end;
procedure TabcDBQueryGrid.SetOnUpdateData(Value: TNotifyEvent); begin FSubDataSource.OnUpdateData := Value; end;

{SubNavigator Property Access Methods - Properties}

function TabcDBQueryGrid.GetConfirmDelete: Boolean;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.ConfirmDelete
  else
    Result := False;
end;

function TabcDBQueryGrid.GetGlyphColor: TabcGlyphColor;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.GlyphColor
  else
    Result := gcBlack;
end;


function TabcDBQueryGrid.GetHints: TStrings;
begin
  if (csLoading in ComponentState) or (FSubNavigator = nil) then
    Result := FHints
  else
    Result := FSubNavigator.Hints;
end;

function TabcDBQueryGrid.GetNavHeight: Integer;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.Height
  else
    Result := 0;
end;

procedure TabcDBQueryGrid.SetNavHeight(Value: Integer);
begin
  if Value <> FSubNavigator.Height then
    FSubNavigator.Height := Value;
end;

function TabcDBQueryGrid.GetNavSpeed: TabcNavSpeed;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.NavSpeed
  else
    Result := nsFast;
end;

function TabcDBQueryGrid.GetMoveBy: Word;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.MoveBy
  else
    Result := 0;
end;

function TabcDBQueryGrid.GetVisibleButtons: TabcSuperNavBtnSet;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.VisibleButtons
  else
    Result := [];
end;

procedure TabcDBQueryGrid.SetConfirmDelete(Value: Boolean);
begin
  if FSubNavigator.ConfirmDelete <> Value then
    FSubNavigator.ConfirmDelete := Value;
end;

procedure TabcDBQueryGrid.SetGlyphColor(Value: TabcGlyphColor);
begin
  if FSubNavigator.GlyphColor <> Value then
    FSubNavigator.GlyphColor := Value;
end;
procedure TabcDBQueryGrid.SetHints(Value: TStrings);
begin
  if FSubNavigator.Hints <> Value then
    FSubNavigator.Hints := Value;
end;
procedure TabcDBQueryGrid.SetNavSpeed(Value: TabcNavSpeed);
begin
  if FSubNavigator.NavSpeed <> Value then
    FSubNavigator.NavSpeed := Value;
end;
procedure TabcDBQueryGrid.SetMoveBy(Value: Word);
begin
  if FSubNavigator.MoveBy <> Value then
    FSubNavigator.MoveBy := Value;
end;
procedure TabcDBQueryGrid.SetVisibleButtons(Value: TabcSuperNavBtnSet);
begin
  if FSubNavigator.VisibleButtons <> Value then
    FSubNavigator.VisibleButtons := Value;
end;

{SubGrid Property Access Methods - Properties}

function TabcDBQueryGrid.GetFixedCols: Integer;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.FixedCols
  else
    Result := 0;
end;

procedure TabcDBQueryGrid.SetFixedCols(Value: Integer); begin FSubGrid.FixedCols := Value; end;

function TabcDBQueryGrid.GetDefaultDrawing: Boolean;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.DefaultDrawing
  else
    Result := False;
end;

function TabcDBQueryGrid.GetFixedColor: TColor;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.FixedColor
  else
    Result := clBlack;
end;

function TabcDBQueryGrid.GetOptions: TDBGridOptions;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.Options
  else
    Result := [];
end;

function TabcDBQueryGrid.GetTitleFont: TFont;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.TitleFont
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetDefaultDrawing(Value: Boolean);
begin
  if FSubGrid.DefaultDrawing <> Value then
    FSubGrid.DefaultDrawing := Value;
end;
procedure TabcDBQueryGrid.SetFixedColor(Value: TColor);
begin
  if FSubGrid.FixedColor <> Value then begin
    FSubGrid.FixedColor := Value;
  end;
end;
procedure TabcDBQueryGrid.SetOptions(Value: TDBGridOptions);
begin
  if FSubGrid.Options <> Value then begin
    FSubGrid.Options := Value;
  end;
end;
procedure TabcDBQueryGrid.SetTitleFont(Value: TFont);
begin
  if FSubGrid.TitleFont <> Value then begin
    FSubGrid.TitleFont := Value;
  end;
end;

function TabcDBQueryGrid.GetColumns: TDBGridColumns;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.Columns
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetColumns(Value: TDBGridColumns); begin FSubGrid.Columns := Value; end;

{SubGrid Property Access Methods - Events}
function TabcDBQueryGrid.GetOnColEnter: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnColEnter
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnColExit: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnColExit
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnDrawDataCell: TDrawDataCellEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDrawDataCell
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnDblClick: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDblClick
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnDragDrop: TDragDropEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDragDrop
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnDragOver: TDragOverEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDragOver
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnEndDrag: TEndDragEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnEndDrag
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnEnter: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnEnter
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnExit: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnExit
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnKeyDown: TKeyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnKeyDown
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnKeyPress: TKeyPressEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnKeyPress
  else
    Result := nil;
end;

function TabcDBQueryGrid.GetOnKeyUp: TKeyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnKeyUp
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetOnColEnter(Value: TNotifyEvent); begin FSubGrid.OnColEnter := Value; end;
procedure TabcDBQueryGrid.SetOnColExit(Value: TNotifyEvent); begin FSubGrid.OnColExit := Value; end;
procedure TabcDBQueryGrid.SetOnDrawDataCell(Value: TDrawDataCellEvent); begin FSubGrid.OnDrawDataCell := Value; end;
procedure TabcDBQueryGrid.SetOnDblClick(Value: TNotifyEvent); begin FSubGrid.OnDblClick := Value; end;
procedure TabcDBQueryGrid.SetOnDragDrop(Value: TDragDropEvent); begin FSubGrid.OnDragDrop := Value; end;
procedure TabcDBQueryGrid.SetOnDragOver(Value: TDragOverEvent); begin FSubGrid.OnDragOver := Value; end;
procedure TabcDBQueryGrid.SetOnEndDrag(Value: TEndDragEvent); begin FSubGrid.OnEndDrag := Value; end;
procedure TabcDBQueryGrid.SetOnEnter(Value: TNotifyEvent); begin FSubGrid.OnEnter := Value; end;
procedure TabcDBQueryGrid.SetOnExit(Value: TNotifyEvent); begin FSubGrid.OnExit := Value; end;
procedure TabcDBQueryGrid.SetOnKeyDown(Value: TKeyEvent); begin FSubGrid.OnKeyDown := Value; end;
procedure TabcDBQueryGrid.SetOnKeyPress(Value: TKeyPressEvent); begin FSubGrid.OnKeyPress := Value; end;
procedure TabcDBQueryGrid.SetOnKeyUp(Value: TKeyEvent); begin FSubGrid.OnKeyUp := Value; end;

function TabcDBQueryGrid.GetOnColumnMoved: TMovedEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnColumnMoved
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetOnColumnMoved(Value: TMovedEvent); begin FSubGrid.OnColumnMoved := Value; end;

function TabcDBQueryGrid.GetOnDrawColumnCell: TDrawColumnCellEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDrawColumnCell
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetOnDrawColumnCell(Value: TDrawColumnCellEvent); begin FSubGrid.OnDrawColumnCell := Value; end;

function TabcDBQueryGrid.GetOnEditButtonClick: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnEditButtonClick
  else
    Result := nil;
end;

procedure TabcDBQueryGrid.SetOnEditButtonClick(Value: TNotifyEvent); begin FSubGrid.OnEditButtonClick := Value; end;

function TabcDBQueryGrid.GetOnStartDrag: TStartDragEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnStartDrag
  else
    Result := nil;
end;


procedure TabcDBQueryGrid.SetOnStartDrag(Value: TStartDragEvent); begin FSubGrid.OnStartDrag := Value; end;

{TabcDBTableGrid}

procedure TabcDBTableGrid.AddSubTable(ASubTable: TabcSubTable);
begin
  FSubTable.Free;
  FSubTable := ASubTable;
  FSubDataSource.DataSet := FSubTable;
  ASubTable.FParent := TComponent(Self);
end;

procedure TabcDBTableGrid.AddSubDataSource(ASubDataSource: TabcSubDataSource);
begin
  FSubDataSource.Free;
  FSubDataSource := ASubDataSource;
  FSubGrid.DataSource := FSubDataSource;
  FSubNavigator.DataSource := FSubDataSource;
  ASubDataSource.FParent := TComponent(Self);
end;

procedure TabcDBTableGrid.NameSubTable(AOwner: TComponent);
var
  I,J: integer;
begin
  J := 0;
  with AOwner do
    for I := 0 to ComponentCount - 1 do
      if Components[I] is TabcSubTable then
        Inc(J);
  FSubTable.Name := 'SubTable' + InttoStr(J);
end;

procedure TabcDBTableGrid.NameSubDataSource(AOwner: TComponent);
var
  I,J: integer;
begin
  J := 0;
  with AOwner do
    for I := 0 to ComponentCount - 1 do
      if Components[I] is TabcSubDataSource then
        Inc(J);
  FSubDataSource.Name := 'SubDataSource' + InttoStr(J);
end;

{Component Filing}

procedure TabcDBTableGrid.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
  Proc(FSubTable);
  Proc(FSubDataSource);
end;

constructor TabcDBTableGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csReplicatable];
  Width := 360;
  Height := 240;
  FSubTable := TabcSubTable.Create(AOwner);
  FSubDataSource := TabcSubDataSource.Create(AOwner);

  if not (csLoading in AOwner.ComponentState) and
     not (csReading in AOwner.ComponentState) then begin
    NameSubTable(AOwner);
    NameSubDataSource(AOwner);
  end;

  FSubNavigator := TabcDBSuperNav.Create(self);
  FSubNavigator.Parent := self;
  FSubGrid := TabcDBFixedGrid.Create(self);
  FSubGrid.Parent := self;
  {add a dummy column to work around bug loading grid columns}
  if (csLoading in AOwner.ComponentState) or
     (csReading in AOwner.ComponentState) then begin
    FColumn := TColumn.Create(FSubGrid.Columns);
    FColumn.FieldName := 'ABCNullColumn';
  end;
  InitSubTableProperties(FSubTable);
  InitSubDataSourceProperties(FSubDataSource);
  InitSubSuperNavProperties(FSubNavigator);
  InitSubGridProperties(FSubGrid);
  InitSharedProperties;
  FHints := TStringList.Create;
end;

destructor TabcDBTableGrid.Destroy;
begin
  FHints.Free;
  FSubGrid.Free;
  FSubNavigator.Free;
  FSubDataSource.Free;
  FSubTable.Free;
  inherited Destroy;
end;

procedure TabcDBTableGrid.InitSubTableProperties(var ATable: TabcSubTable);
begin
  with ATable do begin
    Active := False;
    AutoCalcFields := True;
    Exclusive := False;
    ReadOnly := False;
    TableType := ttDefault;
    UpdateMode := upWhereAll;
  end;
  ATable.Tag := Tag;
end;

procedure TabcDBTableGrid.InitSubDataSourceProperties(var ADataSource: TabcSubDataSource);
begin
  ADataSource.AutoEdit := True;
  ADataSource.Dataset := FSubTable as TDataSet;
  ADataSource.Enabled := Enabled;
  ADataSource.Tag := Tag;
end;

procedure TabcDBTableGrid.InitSubSuperNavProperties(var ADBSuperNav: TabcDBSuperNav);
begin
  ADBSuperNav.Align := alTop;
  ADBSuperNav.ConfirmDelete := True;
  ADBSuperNav.Ctl3D := Ctl3D;
  ADBSuperNav.Cursor := Cursor;
  ADBSuperNav.DataSource := FSubDataSource;
  ADBSuperNav.DragCursor := DragCursor;
  ADBSuperNav.DragMode := DragMode;
  ADBSuperNav.Enabled := Enabled;
  ADBSuperNav.GlyphColor := gcColored;
  ADBSuperNav.Height := 25;
  ADBSuperNav.HelpContext := HelpContext;
  ADBSuperNav.Hint := Hint;
  ADBSuperNav.MoveBy := 50;
  ADBSuperNav.NavSpeed := nsMedium;
  ADBSuperNav.ParentCtl3D := ParentCtl3D;
  ADBSuperNav.ParentShowHint := ParentShowHint;
  ADBSuperNav.PopupMenu := PopupMenu;
  ADBSuperNav.ShowHint := ShowHint;
  ADBSuperNav.TabOrder := TabOrder;
  ADBSuperNav.TabStop := TabStop;
  ADBSuperNav.Tag := Tag;
  ADBSuperNav.Visible := True;
  ADBSuperNav.VisibleButtons :=  [snbFirst, snbMoveBack, snbPrior, snbNext,
    snbMoveFwd, snbLast, snbAppend, snbBookmark, snbInsert, snbDelete, snbEdit, snbPost,
    snbCancel, snbRefresh];
end;

procedure TabcDBTableGrid.InitSubGridProperties(var ADBGrid: TabcDBFixedGrid);
begin
  ADBGrid.Align := alClient;
  ADBGrid.DataSource := FSubDataSource;
  ADBGrid.Color := Color;
  ADBGrid.Ctl3D := Ctl3D;
  ADBGrid.Cursor := Cursor;
  ADBGrid.DefaultDrawing := True;
  ADBGrid.DragCursor := DragCursor;
  ADBGrid.DragMode := DragMode;
  ADBGrid.Enabled := Enabled;
  ADBGrid.Font := Font;
  ADBGrid.HelpContext := HelpContext;
  ADBGrid.Hint := Hint;
  ADBGrid.ParentColor := ParentColor;
  ADBGrid.ParentCtl3D := ParentCtl3D;
  ADBGrid.ParentFont := ParentFont;
  ADBGrid.ParentShowHint := ParentShowHint;
  ADBGrid.PopupMenu := PopupMenu;
  ADBGrid.ReadOnly := False;
  ADBGrid.ShowHint := ShowHint;
  ADBGrid.TabOrder := TabOrder;
  ADBGrid.TabStop := TabStop;
  ADBGrid.Tag := Tag;
  ADBGrid.Visible := True;
end;

procedure TabcDBTableGrid.Loaded;
begin
  inherited Loaded;
  {Set common properties shared between the main component and sub components}
  FSubTable.Tag := Tag;
  FSubDataSource.Enabled := Enabled;
  FSubDataSource.Tag := Tag;
  FSubNavigator.Ctl3D := Ctl3D;
  FSubNavigator.Cursor := Cursor;
  FSubNavigator.DragCursor := DragCursor;
  FSubNavigator.DragMode := DragMode;
  FSubNavigator.Enabled := Enabled;
  FSubNavigator.HelpContext := HelpContext;
  FSubNavigator.Hint := Hint;
  FSubNavigator.ParentCtl3D := ParentCtl3D;
  FSubNavigator.ParentShowHint := ParentShowHint;
  FSubNavigator.PopupMenu := PopupMenu;
  FSubNavigator.ShowHint := ShowHint;
  FSubNavigator.TabOrder := TabOrder;
  FSubNavigator.TabStop := TabStop;
  FSubNavigator.Tag := Tag;
  FSubNavigator.Visible := Visible;
  FSubGrid.Color := Color;
  FSubGrid.Ctl3D := Ctl3D;
  FSubGrid.Cursor := Cursor;
  FSubGrid.DragCursor := DragCursor;
  FSubGrid.DragMode := DragMode;
  FSubGrid.Enabled := Enabled;
  FSubGrid.Font := Font;
  FSubGrid.HelpContext := HelpContext;
  FSubGrid.Hint := Hint;
  FSubGrid.ParentColor := ParentColor;
  FSubGrid.ParentCtl3D := ParentCtl3D;
  FSubGrid.ParentFont := ParentFont;
  FSubGrid.ParentShowHint := ParentShowHint;
  FSubGrid.PopupMenu := PopupMenu;
  FSubGrid.ShowHint := ShowHint;
  FSubGrid.TabOrder := TabOrder;
  FSubGrid.TabStop := TabStop;
  FSubGrid.Tag := Tag;
  FSubGrid.Visible := Visible;
  with FSubGrid.Columns do
    if (Count = 1) and (Columns[0].FieldName = 'ABCNullColumn') then
      RebuildColumns;
  if FHints.Count > 0 then begin
    FSubNavigator.Hints := FHints;
    FHints.Clear;
  end;
end;

procedure TabcDBTableGrid.InitSharedProperties;
begin
  DragCursor := crDrag;
  DragMode :=  dmManual;
  Enabled := True;
  Color := clWindow;
  Ctl3D := True;
  ParentCtl3D := True;
  ParentFont := True;
  ParentShowHint := True;
  TabStop := True;
  Visible := True;
  ReadOnly := False;
  NavAlign := naTop;
end;

{Shared Property Access Methods - Properties}

function TabcDBTableGrid.GetColor: TColor; begin Result := inherited Color; end;
function TabcDBTableGrid.GetCtl3D: Boolean; begin Result := inherited Ctl3D; end;
function TabcDBTableGrid.GetFont: TFont; begin Result := inherited Font; end;
function TabcDBTableGrid.GetShowHint: Boolean; begin Result := inherited ShowHint; end;

function TabcDBTableGrid.IsColorStored: Boolean; begin Result := not ParentColor; end;
function TabcDBTableGrid.IsCtl3DStored: Boolean; begin Result := not ParentCtl3D; end;
function TabcDBTableGrid.IsFontStored: Boolean; begin Result := not ParentFont; end;
function TabcDBTableGrid.IsShowHintStored: Boolean; begin Result := not ParentShowHint; end;

procedure TabcDBTableGrid.SetDragCursor(Value: TCursor);
begin
  if Value <> FDragCursor then begin
    FSubNavigator.DragCursor := Value;
    FSubGrid.DragCursor := Value;
    FDragCursor := Value;
  end;
end;

procedure TabcDBTableGrid.SetDragMode(Value: TDragMode);
begin
  if Value <> FDragMode then begin
    FSubNavigator.DragMode := Value;
    FSubGrid.DragMode := Value;
    FDragMode := Value;
  end;
end;

procedure TabcDBTableGrid.SetEnabled(Value: Boolean);
begin
  if Value <> Enabled then begin
    FSubDataSource.Enabled := Value;
    FSubNavigator.Enabled := Value;
    FSubGrid.Enabled := Value;
    inherited Enabled := Value;
  end;
end;

procedure TabcDBTableGrid.SetColor(Value: TColor);
begin
  if Value <> inherited Color then begin
    ParentColor := False;
    inherited Color := Value;
    FSubGrid.Color := Value;
  end;
end;

procedure TabcDBTableGrid.SetCtl3D(Value: Boolean);
begin
  if Value <> inherited Ctl3D then begin
    ParentCtl3D := False;
    inherited Ctl3D := Value;
    FSubNavigator.Ctl3D := Value;
    FSubGrid.Ctl3D := Value;
  end;
end;

procedure TabcDBTableGrid.SetFont(Value: TFont);
begin
  if Value <> inherited Font then begin
    ParentFont := False;
    inherited Font := Value;
    FSubGrid.Font := Value;
  end;
end;

procedure TabcDBTableGrid.SetHint(Value: string);
begin
  if Value <> FHint then begin
    FSubNavigator.Hint := Value;
    FSubGrid.Hint := Value;
    FHint := Value;
  end;
end;

procedure TabcDBTableGrid.SetHelpContext(Value: THelpContext);
begin
  if Value <> FHelpContext then begin
    FSubNavigator.HelpContext := Value;
    FSubGrid.HelpContext := Value;
    FHelpContext := Value;
  end;
end;

procedure TabcDBTableGrid.SetParentColor(Value: Boolean);
begin
  if Value <> FParentColor then begin
    FParentColor := Value;
    inherited ParentColor := Value;
    FSubGrid.ParentColor := Value;
  end;
end;

procedure TabcDBTableGrid.SetParentCtl3D(Value: Boolean);
begin
  if Value <> FParentCtl3D then begin
    FParentCtl3D := Value;
    inherited ParentCtl3D := Value;
    FSubNavigator.ParentCtl3D := Value;
    FSubGrid.ParentCtl3D := Value;
  end;
end;

procedure TabcDBTableGrid.SetParentFont(Value: Boolean);
begin
  if Value <> FParentFont then begin
    FParentFont := Value;
    inherited ParentFont := Value;
    FSubGrid.ParentFont := Value;
  end;
end;

procedure TabcDBTableGrid.SetParentShowHint(Value: Boolean);
begin
  if Value <> FParentShowHint then begin
    FParentShowHint := Value;
    inherited ParentShowHint := Value;
    FSubNavigator.ParentShowHint := Value;
    FSubGrid.ParentShowHint := Value;
  end;
end;

procedure TabcDBTableGrid.SetPopupMenu(Value: TPopupMenu);
begin
  if Value <> FPopupMenu then begin
    FSubNavigator.PopupMenu := Value;
    FSubGrid.PopupMenu := Value;
    FPopupMenu := Value;
  end;
end;

procedure TabcDBTableGrid.SetShowHint(Value: Boolean);
begin
  if Value <> inherited ShowHint then begin
    ParentShowHint := False;
    inherited ShowHint := Value;
    FSubNavigator.ShowHint := Value;
    FSubGrid.ShowHint := Value;
  end;
end;

procedure TabcDBTableGrid.SetTabOrder(Value: TTabOrder);
begin
  if Value <> FTabOrder then begin
    FSubNavigator.TabOrder := Value;
    FSubGrid.TabOrder := Value;
    FTabOrder := Value;
  end;
end;

procedure TabcDBTableGrid.SetTabStop(Value: Boolean);
begin
  if Value <> FTabStop then begin
    FSubNavigator.TabStop := Value;
    FSubGrid.TabStop := Value;
    FTabStop := Value;
  end;
end;

procedure TabcDBTableGrid.SetVisible(Value: Boolean);
begin
  if Value <> FVisible then begin
    FSubNavigator.Visible := Value;
    FSubGrid.Visible := Value;
    FVisible := Value;
    inherited Visible := Value;
  end;
end;

procedure TabcDBTableGrid.SetReadOnly(Value: Boolean);
{Note - this always updates the properties because it is used in
 SetActive with checking of the Table.CanModify property}
begin
  FSubTable.ReadOnly := Value;
  FSubDataSource.AutoEdit := not Value;
  FSubGrid.ReadOnly := Value;
  if Value then begin
    FSubNavigator.VisibleButtons := FSubNavigator.VisibleButtons -
      [snbInsert, snbDelete, snbEdit, snbPost, snbCancel, snbAppend];
    FSubGrid.Options := FSubGrid.Options - [dgEditing];
  end
  else begin
    FSubNavigator.VisibleButtons := FSubNavigator.VisibleButtons +
      [snbAppend, snbInsert, snbDelete, snbEdit, snbPost, snbCancel, snbRefresh];
    FSubGrid.Options := FSubGrid.Options + [dgEditing];
  end;
  FReadOnly := Value;
end;

procedure TabcDBTableGrid.SetSubNavAlign (Value: TabcSubNavAlignment);
begin
  if Value = naTop then begin
    FSubNavigator.Align := alTop;
    FSubNavAlign := naTop;
  end
  else begin    {value = naBottom}
    FSubNavigator.Align := alBottom;
    FSubNavAlign := naBottom;
  end;
end;

{SubTable Property Access Methods - Properties}

function TabcDBTableGrid.GetActive: Boolean;
begin
  if FSubTable <> nil then
    Result := FSubTable.Active
  else
    Result := False;
end;

function TabcDBTableGrid.GetAutoCalcFields: Boolean;
begin
  if FSubTable <> nil then
    Result := FSubTable.AutoCalcFields
  else
    Result := False;
end;

function TabcDBTableGrid.GetDatabaseName: TFileName;
begin
  if FSubTable <> nil then
    Result := FSubTable.DatabaseName
  else
    Result := '';
end;

function TabcDBTableGrid.GetExclusive: Boolean;
begin
  if FSubTable <> nil then
    Result := FSubTable.Exclusive
  else
    Result := False;
end;

function TabcDBTableGrid.GetIndexFieldNames: string;
begin
  if FSubTable <> nil then
    Result := FSubTable.IndexFieldNames
  else
    Result := '';
end;

function TabcDBTableGrid.GetIndexName: string;
begin
  if FSubTable <> nil then
    Result := FSubTable.IndexName
  else
    Result := '';
end;

function TabcDBTableGrid.GetMasterFields: string;
begin
  if FSubTable <> nil then
    Result := FSubTable.MasterFields
  else
    Result := '';
end;

function TabcDBTableGrid.GetMasterSource: TDataSource;
begin
  if FSubTable <> nil then
    Result := FSubTable.MasterSource
  else
    Result := nil;
end;

function TabcDBTableGrid.GetTableName: TFileName;
begin
  if FSubTable <> nil then
    Result := FSubTable.TableName
  else
    Result := '';
end;

function TabcDBTableGrid.GetTableType: TTableType;
begin
  if FSubTable <> nil then
    Result := FSubTable.TableType
  else
    Result := ttDefault;
end;

procedure TabcDBTableGrid.SetActive(Value: Boolean);
{Note - this also sets the ReadOnly property when
 Activate returns Table.CanModify = false}
begin
  if FSubTable.Active <> Value then
    FSubTable.Active := Value;
  if Value and not FSubTable.CanModify and not ReadOnly then begin
    FSubTable.Active := False;
    ReadOnly := True;
    FSubTable.Active := True;
  end;
end;
procedure TabcDBTableGrid.SetAutoCalcFields(Value: Boolean);
begin
  if FSubTable.AutoCalcFields <> Value then begin
    FSubTable.AutoCalcFields := Value;
  end;
end;
procedure TabcDBTableGrid.SetDatabaseName(Value: TFileName);
begin
  if FSubTable.DatabaseName <> Value then begin
    FSubTable.DatabaseName := Value;
  end;
end;
procedure TabcDBTableGrid.SetExclusive(Value: Boolean);
begin
  if FSubTable.Exclusive <> Value then begin
    FSubTable.Exclusive := Value;
  end;
end;
procedure TabcDBTableGrid.SetIndexFieldNames(Value: string);
begin
  if FSubTable.IndexFieldNames <> Value then begin
    FSubTable.IndexFieldNames := Value;
  end;
end;
procedure TabcDBTableGrid.SetIndexName(Value: string);
begin
  if FSubTable.IndexName <> Value then begin
    FSubTable.IndexName := Value;
  end;
end;
procedure TabcDBTableGrid.SetMasterFields(Value: string);
begin
  if FSubTable.MasterFields <> Value then begin
    FSubTable.MasterFields := Value;
  end;
end;
procedure TabcDBTableGrid.SetMasterSource(Value: TDataSource);
begin
  if FSubTable.MasterSource <> Value then begin
    FSubTable.MasterSource := Value;
  end;
end;
procedure TabcDBTableGrid.SetTableName(Value: TFileName);
begin
  if FSubTable.TableName <> Value then begin
    FSubTable.TableName := Value;
  end;
end;
procedure TabcDBTableGrid.SetTableType(Value: TTableType);
begin
  if FSubTable.TableType <> Value then begin
    FSubTable.TableType := Value;
  end;
end;

{SubTable Property Access Methods - Events}
function TabcDBTableGrid.GetBeforeOpen: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.BeforeOpen
  else
    Result := nil;
end;

function TabcDBTableGrid.GetAfterOpen: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.AfterOpen
  else
    Result := nil;
end;

function TabcDBTableGrid.GetBeforeClose: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.BeforeClose
  else
    Result := nil;
end;

function TabcDBTableGrid.GetAfterClose: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.AfterClose
  else
    Result := nil;
end;

function TabcDBTableGrid.GetBeforeInsert: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.BeforeInsert
  else
    Result := nil;
end;

function TabcDBTableGrid.GetAfterInsert: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.AfterInsert
  else
    Result := nil;
end;

function TabcDBTableGrid.GetBeforeEdit: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.BeforeEdit
  else
    Result := nil;
end;

function TabcDBTableGrid.GetAfterEdit: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.AfterEdit
  else
    Result := nil;
end;

function TabcDBTableGrid.GetBeforePost: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.BeforePost
  else
    Result := nil;
end;

function TabcDBTableGrid.GetAfterPost: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.AfterPost
  else
    Result := nil;
end;

function TabcDBTableGrid.GetBeforeCancel: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.BeforeCancel
  else
    Result := nil;
end;

function TabcDBTableGrid.GetAfterCancel: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.AfterCancel
  else
    Result := nil;
end;

function TabcDBTableGrid.GetBeforeDelete: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.BeforeDelete
  else
    Result := nil;
end;

function TabcDBTableGrid.GetAfterDelete: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.AfterDelete
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnNewRecord: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.OnNewRecord
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnCalcFields: TDataSetNotifyEvent;
begin
  if FSubTable <> nil then
    Result := FSubTable.OnCalcFields
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetBeforeOpen(Value: TDataSetNotifyEvent); begin FSubTable.BeforeOpen := Value; end;
procedure TabcDBTableGrid.SetAfterOpen(Value: TDataSetNotifyEvent); begin FSubTable.AfterOpen := Value; end;
procedure TabcDBTableGrid.SetBeforeClose(Value: TDataSetNotifyEvent); begin FSubTable.BeforeClose := Value; end;
procedure TabcDBTableGrid.SetAfterClose(Value: TDataSetNotifyEvent); begin FSubTable.AfterClose := Value; end;
procedure TabcDBTableGrid.SetBeforeInsert(Value: TDataSetNotifyEvent); begin FSubTable.BeforeInsert := Value; end;
procedure TabcDBTableGrid.SetAfterInsert(Value: TDataSetNotifyEvent); begin FSubTable.AfterInsert := Value; end;
procedure TabcDBTableGrid.SetBeforeEdit(Value: TDataSetNotifyEvent); begin FSubTable.BeforeEdit := Value; end;
procedure TabcDBTableGrid.SetAfterEdit(Value: TDataSetNotifyEvent); begin FSubTable.AfterEdit := Value; end;
procedure TabcDBTableGrid.SetBeforePost(Value: TDataSetNotifyEvent); begin FSubTable.BeforePost := Value; end;
procedure TabcDBTableGrid.SetAfterPost(Value: TDataSetNotifyEvent); begin FSubTable.AfterPost := Value; end;
procedure TabcDBTableGrid.SetBeforeCancel(Value: TDataSetNotifyEvent); begin FSubTable.BeforeCancel := Value; end;
procedure TabcDBTableGrid.SetAfterCancel(Value: TDataSetNotifyEvent); begin FSubTable.AfterCancel := Value; end;
procedure TabcDBTableGrid.SetBeforeDelete(Value: TDataSetNotifyEvent); begin FSubTable.BeforeDelete := Value; end;
procedure TabcDBTableGrid.SetAfterDelete(Value: TDataSetNotifyEvent); begin FSubTable.AfterDelete := Value; end;
procedure TabcDBTableGrid.SetOnNewRecord(Value: TDataSetNotifyEvent); begin FSubTable.OnNewRecord := Value; end;
procedure TabcDBTableGrid.SetOnCalcFields(Value: TDataSetNotifyEvent); begin FSubTable.OnCalcFields := Value; end;

{SubDataSource Property Access Methods - Events}
function TabcDBTableGrid.GetOnStateChange: TNotifyEvent;
begin
  if FSubDataSource <> nil then
    Result := FSubDataSource.OnStateChange
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnDataChange: TDataChangeEvent;
begin
  if FSubDataSource <> nil then
    Result := FSubDataSource.OnDataChange
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnUpdateData: TNotifyEvent;
begin
  if FSubDataSource <> nil then
    Result := FSubDataSource.OnUpdateData
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetOnStateChange(Value: TNotifyEvent); begin FSubDataSource.OnStateChange := Value; end;
procedure TabcDBTableGrid.SetOnDataChange(Value: TDataChangeEvent); begin FSubDataSource.OnDataChange := Value; end;
procedure TabcDBTableGrid.SetOnUpdateData(Value: TNotifyEvent); begin FSubDataSource.OnUpdateData := Value; end;

{SubNavigator Property Access Methods - Properties}

function TabcDBTableGrid.GetConfirmDelete: Boolean;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.ConfirmDelete
  else
    Result := False;
end;

function TabcDBTableGrid.GetGlyphColor: TabcGlyphColor;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.GlyphColor
  else
    Result := gcBlack;
end;

function TabcDBTableGrid.GetHints: TStrings;
begin
  if (csLoading in ComponentState) or (FSubNavigator = nil) then
    Result := FHints
  else
    Result := FSubNavigator.Hints;
end;

function TabcDBTableGrid.GetNavHeight: Integer;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.Height
  else
    Result := 0;
end;

procedure TabcDBTableGrid.SetNavHeight(Value: Integer);
begin
  if Value <> FSubNavigator.Height then
    FSubNavigator.Height := Value;
end;

function TabcDBTableGrid.GetNavSpeed: TabcNavSpeed;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.NavSpeed
  else
    Result := nsFast;
end;

function TabcDBTableGrid.GetMoveBy: Word;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.MoveBy
  else
    Result := 0;
end;

function TabcDBTableGrid.GetVisibleButtons: TabcSuperNavBtnSet;
begin
  if FSubNavigator <> nil then
    Result := FSubNavigator.VisibleButtons
  else
    Result := [];
end;

procedure TabcDBTableGrid.SetConfirmDelete(Value: Boolean);
begin
  if FSubNavigator.ConfirmDelete <> Value then begin
    FSubNavigator.ConfirmDelete := Value;
  end;
end;
procedure TabcDBTableGrid.SetGlyphColor(Value: TabcGlyphColor);
begin
  if FSubNavigator.GlyphColor <> Value then begin
    FSubNavigator.GlyphColor := Value;
  end;
end;
procedure TabcDBTableGrid.SetHints(Value: TStrings);
begin
  if FSubNavigator.Hints <> Value then begin
    FSubNavigator.Hints := Value;
  end;
end;
procedure TabcDBTableGrid.SetNavSpeed(Value: TabcNavSpeed);
begin
  if FSubNavigator.NavSpeed <> Value then begin
    FSubNavigator.NavSpeed := Value;
  end;
end;
procedure TabcDBTableGrid.SetMoveBy(Value: Word);
begin
  if FSubNavigator.MoveBy <> Value then begin
    FSubNavigator.MoveBy := Value;
  end;
end;
procedure TabcDBTableGrid.SetVisibleButtons(Value: TabcSuperNavBtnSet);
begin
  if FSubNavigator.VisibleButtons <> Value then begin
    FSubNavigator.VisibleButtons := Value;
  end;
end;

{SubGrid Property Access Methods - Properties}

function TabcDBTableGrid.GetFixedCols: Integer;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.FixedCols
  else
    Result := 0;
end;

procedure TabcDBTableGrid.SetFixedCols(Value: Integer);
begin
  if FSubGrid.FixedCols <> Value then
    FSubGrid.FixedCols := Value;
end;

function TabcDBTableGrid.GetDefaultDrawing: Boolean;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.DefaultDrawing
  else
    Result := False;
end;

function TabcDBTableGrid.GetFixedColor: TColor;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.FixedColor
  else
    Result := clBlack;
end;

function TabcDBTableGrid.GetOptions: TDBGridOptions;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.Options
  else
    Result := [];
end;

function TabcDBTableGrid.GetTitleFont: TFont;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.TitleFont
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetDefaultDrawing(Value: Boolean);
begin
  if FSubGrid.DefaultDrawing <> Value then
    FSubGrid.DefaultDrawing := Value;
end;
procedure TabcDBTableGrid.SetFixedColor(Value: TColor);
begin
  if FSubGrid.FixedColor <> Value then begin
    FSubGrid.FixedColor := Value;
  end;
end;
procedure TabcDBTableGrid.SetOptions(Value: TDBGridOptions);
begin
  if FSubGrid.Options <> Value then begin
    FSubGrid.Options := Value;
  end;
end;
procedure TabcDBTableGrid.SetTitleFont(Value: TFont);
begin
  if FSubGrid.TitleFont <> Value then begin
    FSubGrid.TitleFont := Value;
  end;
end;

function TabcDBTableGrid.GetColumns: TDBGridColumns;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.Columns
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetColumns(Value: TDBGridColumns); begin FSubGrid.Columns := Value; end;

{SubGrid Property Access Methods - Events}
function TabcDBTableGrid.GetOnColEnter: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnColEnter
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnColExit: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnColExit
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnDrawDataCell: TDrawDataCellEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDrawDataCell
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnDblClick: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDblClick
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnDragDrop: TDragDropEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDragDrop
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnDragOver: TDragOverEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDragOver
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnEndDrag: TEndDragEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnEndDrag
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnEnter: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnEnter
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnExit: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnExit
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnKeyDown: TKeyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnKeyDown
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnKeyPress: TKeyPressEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnKeyPress
  else
    Result := nil;
end;

function TabcDBTableGrid.GetOnKeyUp: TKeyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnKeyUp
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetOnColEnter(Value: TNotifyEvent); begin FSubGrid.OnColEnter := Value; end;
procedure TabcDBTableGrid.SetOnColExit(Value: TNotifyEvent); begin FSubGrid.OnColExit := Value; end;
procedure TabcDBTableGrid.SetOnDrawDataCell(Value: TDrawDataCellEvent); begin FSubGrid.OnDrawDataCell := Value; end;
procedure TabcDBTableGrid.SetOnDblClick(Value: TNotifyEvent); begin FSubGrid.OnDblClick := Value; end;
procedure TabcDBTableGrid.SetOnDragDrop(Value: TDragDropEvent); begin FSubGrid.OnDragDrop := Value; end;
procedure TabcDBTableGrid.SetOnDragOver(Value: TDragOverEvent); begin FSubGrid.OnDragOver := Value; end;
procedure TabcDBTableGrid.SetOnEndDrag(Value: TEndDragEvent); begin FSubGrid.OnEndDrag := Value; end;
procedure TabcDBTableGrid.SetOnEnter(Value: TNotifyEvent); begin FSubGrid.OnEnter := Value; end;
procedure TabcDBTableGrid.SetOnExit(Value: TNotifyEvent); begin FSubGrid.OnExit := Value; end;
procedure TabcDBTableGrid.SetOnKeyDown(Value: TKeyEvent); begin FSubGrid.OnKeyDown := Value; end;
procedure TabcDBTableGrid.SetOnKeyPress(Value: TKeyPressEvent); begin FSubGrid.OnKeyPress := Value; end;
procedure TabcDBTableGrid.SetOnKeyUp(Value: TKeyEvent); begin FSubGrid.OnKeyUp := Value; end;
function TabcDBTableGrid.GetOnColumnMoved: TMovedEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnColumnMoved
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetOnColumnMoved(Value: TMovedEvent); begin FSubGrid.OnColumnMoved := Value; end;
function TabcDBTableGrid.GetOnDrawColumnCell: TDrawColumnCellEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnDrawColumnCell
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetOnDrawColumnCell(Value: TDrawColumnCellEvent); begin FSubGrid.OnDrawColumnCell := Value; end;
function TabcDBTableGrid.GetOnEditButtonClick: TNotifyEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnEditButtonClick
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetOnEditButtonClick(Value: TNotifyEvent); begin FSubGrid.OnEditButtonClick := Value; end;
function TabcDBTableGrid.GetOnStartDrag: TStartDragEvent;
begin
  if FSubGrid <> nil then
    Result := FSubGrid.OnStartDrag
  else
    Result := nil;
end;

procedure TabcDBTableGrid.SetOnStartDrag(Value: TStartDragEvent); begin FSubGrid.OnStartDrag := Value; end;

initialization
  RegisterClass(TabcSubQuery);
  RegisterClass(TabcSubTable);
  RegisterClass(TabcSubDataSource);

finalization
  UnRegisterClass(TabcSubQuery);
  UnRegisterClass(TabcSubTable);
  UnRegisterClass(TabcSubDataSource);

end.
