unit abcdbhnc;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  abcfact, abcapp;

type

  { TabcCustomDBHintManager }

  TabcShowHint = (ashNone, ashShort, ashLong);

  TabcHintKeyEvent = procedure (Sender: TObject;
    const Control: TControl; var FormClass, ControlName: string) of object;

  TabcCustomDBHintManager = class(TComponent)
  private
    FEditing: Boolean;
    FEnabled: Boolean;
    FHintEditor: TForm;
    FKeyFirstParentForm: Boolean;
    FOnHint: TNotifyEvent;
    FOnShowHint: TShowHintEvent;
    FReadOnly: Boolean;
    FShowHint: TabcShowHint;
    FOnGetKey: TabcHintKeyEvent;
    function GetHintColor: TColor;
    function GetHintPause: Integer;
    procedure SetHintColor(Value: TColor);
    procedure SetHintPause(Value: Integer);
    function GetHintHidePause: Integer;
    function GetHintShortPause: Integer;
    procedure SetHintHidePause(Value: Integer);
    procedure SetHintShortPause(Value: Integer);
    procedure SetShowHint(Value: TabcShowHint);
    procedure SetEnabled(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
  protected
    procedure Loaded; override;
    procedure DoOnShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
    procedure DoOnHint(Sender: TObject);
    function GetHintInfo(const Control: TControl): string;

    { Virtual functions for data access }
    function DoGetHint(const Control: TControl; const FormClass, ControlName: string;
      var ShortHint, LongHint: string): Boolean; virtual;
    procedure DoStoreHint(const Control: TControl;
      const FormClass, ControlName, ShortHint, LongHint: string); virtual;
    procedure DoDeleteHint(const Control: TControl; const FormClass, ControlName: string); virtual;
    procedure DoSetDatasetState(Active: Boolean; ReadOnly: Boolean); virtual;
    function DoDataSetIsActive: Boolean; virtual;
    procedure DoEdit; virtual;
    procedure DoOnCloseEditor; virtual;  {called by the editor form when it closes}

    { protected exception layer }
    function GetHint(const Control: TControl; const FormClass, ControlName: string;
      var ShortHint, LongHint: string): Boolean;
    procedure StoreHint(const Control: TControl;
      const FormClass, ControlName, ShortHint, LongHint: string);
    procedure DeleteHint(const Control: TControl; const FormClass, ControlName: string);
    procedure SetDatasetState(Active: Boolean; ReadOnly: Boolean);
    function DataSetIsActive: Boolean;

    {can be published in descendant}
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property HintColor: TColor read GetHintColor write SetHintColor default clInfoBk;
    property HintPause: Integer read GetHintPause write SetHintPause default 500;
    property HintShortPause: Integer read GetHintShortPause write SetHintShortPause default 50;
    property HintHidePause: Integer read GetHintHidePause write SetHintHidePause default 3000;
    property KeyFirstParentForm: Boolean read FKeyFirstParentForm write FKeyFirstParentForm default True;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly default True;
    property ShowHint: TabcShowHint read FShowHint write SetShowHint default ashNone;
    property OnGetKey: TabcHintKeyEvent read FOnGetKey write FOnGetKey;
    property OnHint: TNotifyEvent read FOnHint write FOnHint;
    property OnShowHint: TShowHintEvent read FOnShowHint write FOnShowHint;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure OnCloseEditor; {called by the editor form when it closes}
    procedure SetHintText(Control: TControl; Value: string);
    procedure DeleteHintText(Control: TControl);
    procedure Edit;
  end;

  TabcHelpKeyEvent = procedure (Sender: TObject; var HelpKey: string) of object;

  TabcCustomDBHelpManager = class(TComponent)
  private
    FReadOnly: Boolean;
    FEditing: Boolean;
    FEnabled: Boolean;
    FHelpViewer: TForm;
    FKeyFirstParentForm: Boolean;
    FOnGetKey: TabcHelpKeyEvent;
    FOnShowForm: TNotifyEvent;
    FOnCloseForm: TCloseEvent;
    FTemplate: TFileName;
    FDefaultTopic: string;
    procedure SetEnabled(Value: Boolean);
    procedure SetOnShowForm(Value: TNotifyEvent);
    procedure SetOnCloseForm(Value: TCloseEvent);
    procedure SetReadOnly(Value: Boolean);
  protected
    procedure Loaded; override;
    function DoOnHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
    function GetHelpForm: TForm;
    { Virtual functions for data access }
    function DoGetHelp(const FormClass: string; var HelpText: string): Boolean; virtual;
    procedure DoStoreHelp(const FormClass, HelpText: string); virtual;
    procedure DoDeleteHelp(const FormClass: string); virtual;
    procedure DoSetDatasetState(Active: Boolean; ReadOnly: Boolean); virtual;
    function DoDataSetIsActive: Boolean; virtual;

    procedure DoOnCloseEditor; virtual;  {called by the editor form when it closes}
    procedure DoEdit(const Form: TForm; const HelpTopic: string); virtual;

    procedure DisplayHelpViewer(const Form: TForm; const HelpTopic, HelpText: string; IsReadOnly: Boolean); virtual;

    { protected exception layer }
    function GetHelp(const FormClass: string; var HelpText: string): Boolean;
    procedure StoreHelp(const FormClass, HelpText: string);
    procedure DeleteHelp(const FormClass: string);
    procedure SetDatasetState(Active: Boolean; ReadOnly: Boolean);
    function DataSetIsActive: Boolean;

    {can be published in descendant}
    property DefaultTopic: string read FDefaultTopic write FDefaultTopic;
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property KeyFirstParentForm: Boolean read FKeyFirstParentForm write FKeyFirstParentForm default True;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly default True;
    property OnGetKey: TabcHelpKeyEvent read FOnGetKey write FOnGetKey;
    property OnShowForm: TNotifyEvent read FOnShowForm write SetOnShowForm;
    property OnCloseForm: TCloseEvent read FOnCloseForm write SetOnCloseForm;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Edit;
    procedure EditDefaultTopic;
    procedure EditHelpTopic(const HelpTopic: string);
    procedure Show;
    procedure ShowDefaultTopic;
    procedure ShowHelpTopic(const HelpTopic: string);
    procedure OnCloseEditor; {called by the editor form when it closes}
    procedure SetHelpText(const Form: TForm; const HelpTopic, HelpText: string);
    procedure DeleteHelpText(const Form: TForm; const HelpTopic: string);
    property Template: TFileName read FTemplate write FTemplate;
  end;

implementation

uses extctrls, abcfhnt, abcfhlp, abcutilf;

{ TabcCustomDBHintManager }

constructor TabcCustomDBHintManager.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  abcApplicationEvents.RegisterOnHint(DoOnHint);
  abcApplicationEvents.RegisterOnShowHint(DoOnShowHint);

  HintPause := 500;
  HintHidePause := 3000;
  FReadOnly := True;
  FEnabled := True;
  FKeyFirstParentForm := True;
end;

destructor TabcCustomDBHintManager.Destroy;
begin
  abcApplicationEvents.UnRegisterOnHint(DoOnHint);
  abcApplicationEvents.UnRegisterOnShowHint(DoOnShowHint);
  if (FHintEditor <> nil) then
  begin
    if FHintEditor.Visible then
      FHintEditor.Close;
    FHintEditor.Free;
    FHintEditor := nil;
  end;
  inherited Destroy;
end;

procedure TabcCustomDBHintManager.SetEnabled(Value: Boolean);
begin
  FEnabled := Value;
  if not Value and (FHintEditor <> nil) then
  begin
    if FHintEditor.Visible then
      FHintEditor.Close;
    FHintEditor.Free;
    FHintEditor := nil;
  end;
end;

procedure TabcCustomDBHintManager.SetReadOnly(Value: Boolean);
begin
  if Value <> FReadOnly then
  begin
    FReadOnly := Value;
    if not (csDesigning in ComponentState) then
    begin
      if Value and (FHintEditor <> nil) then
      begin
        if FHintEditor.Visible then
          FHintEditor.Close;
        FHintEditor.Free;
        FHintEditor := nil;
      end;
      SetDatasetState((ShowHint <> ashNone), Value);
    end;
  end;
end;

procedure TabcCustomDBHintManager.Edit;
begin
  DoEdit;
end;

procedure TabcCustomDBHintManager.DoEdit;
begin
  ReadOnly := False;
  if ReadOnly then Exit;
  if ShowHint = ashNone then
    ShowHint := ashShort;
  FEditing := True;
  if FHintEditor = nil then
    FHintEditor := abcFormFactory.CreateFormOfClass(TabcsfHintEditor, Self);
  FHintEditor.Show;
end;

procedure TabcCustomDBHintManager.OnCloseEditor;
{called by the editor form when it closes}
begin
  DoOnCloseEditor;
end;

procedure TabcCustomDBHintManager.DoOnCloseEditor;
begin
  FEditing := False;
end;

procedure TabcCustomDBHintManager.DoOnHint(Sender: TObject);
begin
  if Enabled and Assigned(FOnHint) then FOnHint(Sender);
end;

procedure TabcCustomDBHintManager.DoOnShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
var
  newHintStr: string;
begin
  CanShow := Enabled and (FShowHint <> ashNone) and DataSetIsActive;
  if not CanShow then Exit;

  newHintStr := GetHintInfo(HintInfo.HintControl);

  if (FShowHint = ashLong) then
    newHintStr := GetLongHint(HintInfo.HintControl.Hint)
  else
    newHintStr := GetShortHint(HintInfo.HintControl.Hint);

  if newHintStr <> '' then
    HintStr := newHintStr;
  HintInfo.HintStr := HintStr;

  if FEditing then
    CanShow := False;
  if Assigned(FOnShowHint) then FOnShowHint(HintStr, CanShow, HintInfo);
  if FEditing then
    TabcsfHintEditor(FHintEditor).EditControlHint(HintInfo.HintControl, HintInfo.HintPos);
end;

function TabcCustomDBHintManager.GetHintInfo(const Control: TControl): string;
var
  ParentForm: TCustomForm;
  ShortHint, LongHint: string;
  aFormClassName: string;
begin
  Result := '';
  if FKeyFirstParentForm then
    ParentForm := abcFirstFormParentOf(Control)
  else
    ParentForm := GetParentForm(Control);

  if ParentForm = nil then
    aFormClassName := ''
  else
    aFormClassName := ParentForm.ClassName;

  if GetHint(Control, aFormClassName, Control.Name, ShortHint, LongHint) then
  begin
    if (ShortHint = '') then
      Result := LongHint
    else
      if LongHint = '' then
        Result := ShortHint
      else
        Result := ShortHint + '|' + LongHint;
    if Result <> '' then
      Control.Hint := Result;
  end;
end;

procedure TabcCustomDBHintManager.SetHintText(Control: TControl; Value: string);
var
  ParentForm: TCustomForm;
  sFormClass, sControlName, sShort, sLong: string;
begin
  if ReadOnly then Exit;
  Control.Hint := Value;

  if FKeyFirstParentForm then
    ParentForm := abcFirstFormParentOf(Control)
  else
    ParentForm := GetParentForm(Control);

  if ParentForm = nil then
    sFormClass := ''
  else
    sFormClass := ParentForm.ClassName;
  sControlName := Control.Name;
  sShort := GetShortHint(Value);
  sLong := GetLongHint(Value);
  StoreHint(Control, sFormClass, sControlName, sShort, sLong);
end;

procedure TabcCustomDBHintManager.DeleteHintText(Control: TControl);
var
  ParentForm: TCustomForm;
  sFormClass, sControlName: string;
begin
  if ReadOnly then Exit;
  Control.Hint := '';

  if FKeyFirstParentForm then
    ParentForm := abcFirstFormParentOf(Control)
  else
    ParentForm := GetParentForm(Control);

  if ParentForm = nil then
    sFormClass := ''
  else
    sFormClass := ParentForm.ClassName;

  sControlName := Control.Name;
  DeleteHint(Control, sFormClass, sControlName);
end;

function TabcCustomDBHintManager.GetHintColor: TColor;
begin
  Result := Application.HintColor;
end;

function TabcCustomDBHintManager.GetHintPause: Integer;
begin
  Result := Application.HintPause;
end;

function TabcCustomDBHintManager.GetHintHidePause: Integer;
begin
  Result := Application.HintHidePause;
end;

function TabcCustomDBHintManager.GetHintShortPause: Integer;
begin
  Result := Application.HintShortPause;
end;

procedure TabcCustomDBHintManager.SetHintColor(Value: TColor);
begin
  Application.HintColor := Value;
end;

procedure TabcCustomDBHintManager.SetHintPause(Value: Integer);
begin
  Application.HintPause := Value;
end;

procedure TabcCustomDBHintManager.SetHintHidePause(Value: Integer);
begin
  Application.HintHidePause := Value;
end;

procedure TabcCustomDBHintManager.SetHintShortPause(Value: Integer);
begin
  Application.HintShortPause := Value;
end;

procedure TabcCustomDBHintManager.SetShowHint(Value: TabcShowHint);
begin
  FShowHint := Value;
  if ([csDesigning, csLoading] * ComponentState) = [] then
    SetDatasetState((Value <> ashNone), ReadOnly);
end;

procedure TabcCustomDBHintManager.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
    SetDatasetState((ShowHint <> ashNone), ReadOnly);
end;

{ protected exception layer }

function TabcCustomDBHintManager.GetHint(const Control: TControl;
  const FormClass, ControlName: string; var ShortHint, LongHint: string): Boolean;
var
  searchClass, searchControl: string;
begin
  searchClass := FormClass;
  searchControl := ControlName;
  try
    if Assigned(FOnGetKey) then
      FOnGetKey(Self, Control, searchClass, searchControl);
    Result := DoGetHint(Control, searchClass, searchControl, ShortHint, LongHint);
  except
    Result := False;
  end;
end;

procedure TabcCustomDBHintManager.StoreHint(const Control: TControl;
  const FormClass, ControlName, ShortHint, LongHint: string);
var
  saveFormClass, saveControl: string;
begin
  saveFormClass := FormClass;
  saveControl := ControlName;
  try
    if Assigned(FOnGetKey) then
      FOnGetKey(Self, Control, saveFormClass, saveControl);
    DoStoreHint(Control, saveFormClass, saveControl, ShortHint, LongHint);
  except
  end;
end;

procedure TabcCustomDBHintManager.DeleteHint(const Control: TControl; const FormClass, ControlName: string);
var
  searchClass, searchControl: string;
begin
  searchClass := FormClass;
  searchControl := ControlName;
  try
    if Assigned(FOnGetKey) then
      FOnGetKey(Self, Control, searchClass, searchControl);
    DoDeleteHint(Control, searchClass, searchControl);
  except
  end;
end;

procedure TabcCustomDBHintManager.SetDatasetState(Active: Boolean; ReadOnly: Boolean);
begin
  try
    DoSetDatasetState(Active, ReadOnly);
  except
  end;
end;

function TabcCustomDBHintManager.DataSetIsActive: Boolean;
begin
  try
    Result := DoDataSetIsActive;
  except
    Result := False;
  end;
end;

{DB Interface Methods for Hint Text reading and storage}

procedure TabcCustomDBHintManager.DoSetDatasetState(Active: Boolean; ReadOnly: Boolean);
begin
end;

function TabcCustomDBHintManager.DoDataSetIsActive: Boolean;
begin
  Result := False;
end;

function TabcCustomDBHintManager.DoGetHint(const Control: TControl;
  const FormClass, ControlName: string; var ShortHint, LongHint: string): Boolean;
begin
  Result := False;
end;

procedure TabcCustomDBHintManager.DoStoreHint(const Control: TControl;
  const FormClass, ControlName, ShortHint, LongHint: string);
begin
end;

procedure TabcCustomDBHintManager.DoDeleteHint(const Control: TControl; const FormClass, ControlName: string);
begin
end;

{ TabcCustomDBHelpManager }

constructor TabcCustomDBHelpManager.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  abcApplicationEvents.RegisterOnHelp(DoOnHelp);
  FEnabled := True;
  FKeyFirstParentForm := True;
  FReadOnly := True;
end;

destructor TabcCustomDBHelpManager.Destroy;
begin
  abcApplicationEvents.UnRegisterOnHelp(DoOnHelp);
  if (FHelpViewer <> nil) then
  begin
    if FHelpViewer.Visible then
      FHelpViewer.Close;
    FHelpViewer.Free;
    FHelpViewer := nil;
  end;
  inherited Destroy;
end;

procedure TabcCustomDBHelpManager.SetEnabled(Value: Boolean);
begin
  FEnabled := Value;
  if not Value and (FHelpViewer <> nil) then
  begin
    if FHelpViewer.Visible then
      FHelpViewer.Close;
    FHelpViewer.Free;
    FHelpViewer := nil;
  end;
  if (ComponentState * [csDesigning, csLoading]) = [] then
    SetDatasetState(Enabled, ReadOnly);
end;

procedure TabcCustomDBHelpManager.SetReadOnly(Value: Boolean);
begin
  if Value <> FReadOnly then
  begin
    FReadOnly := Value;
    if not (csDesigning in ComponentState) then
    begin
      if (FHelpViewer <> nil) then
      begin
        if FHelpViewer.Visible then
          FHelpViewer.Close;
        FHelpViewer.Free;
        FHelpViewer := nil;
      end;
      SetDatasetState(Enabled, Value);
    end;
  end;
end;

procedure TabcCustomDBHelpManager.SetOnShowForm(Value: TNotifyEvent);
begin
  FOnShowForm := Value;
  if FHelpViewer <> nil then
    TabcsfHelpViewer(FHelpViewer).OnShowExtension := Value;
end;

procedure TabcCustomDBHelpManager.SetOnCloseForm(Value: TCloseEvent);
begin
  FOnCloseForm := Value;
  if FHelpViewer <> nil then
    TabcsfHelpViewer(FHelpViewer).OnCloseExtension := Value;
end;

procedure TabcCustomDBHelpManager.Edit;
var
  HelpTopic: string;
  HelpForm: TForm;
begin
  HelpForm := GetHelpForm;

  if HelpForm <> nil then
    HelpTopic := HelpForm.ClassName
  else
    HelpTopic := DefaultTopic;
  DoEdit(HelpForm, HelpTopic);
end;

procedure TabcCustomDBHelpManager.EditDefaultTopic;
begin
  if (DefaultTopic <> '') then
    DoEdit(nil, DefaultTopic);
end;

procedure TabcCustomDBHelpManager.EditHelpTopic(const HelpTopic: string);
begin
  DoEdit(nil, HelpTopic);
end;

procedure TabcCustomDBHelpManager.DoEdit(const Form: TForm; const HelpTopic: string);
var
  HelpText: string;
begin

  SetDatasetState(True, False);

  FEditing := True;

  GetHelp(HelpTopic, HelpText);

  DisplayHelpViewer(Form, HelpTopic, HelpText, False);

end;

procedure TabcCustomDBHelpManager.Show;
var
  CallHelp: Boolean;
begin
  DoOnHelp(0,0,CallHelp);
end;

procedure TabcCustomDBHelpManager.ShowDefaultTopic;
begin
  ShowHelpTopic(DefaultTopic);
end;

procedure TabcCustomDBHelpManager.ShowHelpTopic(const HelpTopic: string);
var
  HelpText: string;
begin
  if not GetHelp(HelpTopic, HelpText) and ReadOnly then Exit;
  DisplayHelpViewer(nil, HelpTopic, HelpText, ReadOnly);
end;

procedure TabcCustomDBHelpManager.OnCloseEditor;
begin
  DoOnCloseEditor;
end;

procedure TabcCustomDBHelpManager.DoOnCloseEditor;
begin
  FEditing := False;
end;

function TabcCustomDBHelpManager.GetHelpForm: TForm;
begin
  if FKeyFirstParentForm and (Screen.ActiveControl <> nil) then
  begin
    Result := TForm(abcFirstFormParentOf(Screen.ActiveControl));
    if Result = nil then
      Result := Application.MainForm;
    if Result <> nil then
    begin
      if (Result.FormStyle = fsMDIForm) then
        if (Result.ActiveMDIChild <> nil) and (Result.ActiveMDIChild.WindowState <> wsMinimized) then
          Result := Result.ActiveMDIChild;
    end;
  end
  else
  begin
    Result := Screen.ActiveForm;
    if Result = nil then
      Result := Application.MainForm;
  end;
end;

procedure TabcCustomDBHelpManager.DisplayHelpViewer(const Form: TForm; const HelpTopic, HelpText: string; IsReadOnly: Boolean);
begin
  if FHelpViewer = nil then
    FHelpViewer := abcFormFactory.CreateFormOfClass(TabcsfHelpViewer, Self);
  with TabcsfHelpViewer(FHelpViewer) do
  begin
    DisplayOnly := IsReadOnly;
    Template := Self.Template;
    OnShowExtension := FOnShowForm;
    OnCloseExtension := FOnCloseForm;
    ShowHelp(Form, HelpTopic, HelpText);
  end;
end;

function TabcCustomDBHelpManager.DoOnHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
var
  HelpTopic, HelpText: string;
  HelpForm: TForm;
  ShowHelpViewer: Boolean;
begin
  Result := False;

  if not Enabled then Exit;

  HelpText := '';
  HelpTopic := '';
  HelpForm := GetHelpForm;

  if HelpForm <> nil then
    HelpTopic := HelpForm.ClassName;

  ShowHelpViewer := GetHelp(HelpTopic, HelpText) or not ReadOnly;

  if not ShowHelpViewer then
  begin
    HelpForm := nil;
    HelpTopic := DefaultTopic;
    ShowHelpViewer := GetHelp(HelpTopic, HelpText);
  end;

  if ShowHelpViewer then
  begin
    CallHelp := False;
    Result := True;
    DisplayHelpViewer(HelpForm, HelpTopic, HelpText, ReadOnly);
  end
  else
    CallHelp := True;
end;

procedure TabcCustomDBHelpManager.SetHelpText(const Form: TForm; const HelpTopic, HelpText: string);
var
  sTopic: string;
begin
  SetDatasetState(True, False);
  if Form = nil then
    sTopic := HelpTopic
  else
    sTopic := Form.ClassName;
  StoreHelp(sTopic, HelpText);
end;

procedure TabcCustomDBHelpManager.DeleteHelpText(const Form: TForm; const HelpTopic: string);
var
  sTopic: string;
begin
  SetDatasetState(True, False);
  if Form = nil then
    sTopic := HelpTopic
  else
    sTopic := Form.ClassName;
  DeleteHelp(sTopic);
end;

procedure TabcCustomDBHelpManager.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
  begin
    SetEnabled(Enabled);
    {add local path to help file template}
    if (FTemplate <> '') and (ExtractFilePath(FTemplate) = '') then
      FTemplate := ExtractFilePath(Application.ExeName) + FTemplate;
  end;
end;

{ protected exception layer }

function TabcCustomDBHelpManager.GetHelp(const FormClass: string; var HelpText: string): Boolean;
var
  searchClass: string;
begin
  searchClass := FormClass;
  try
    if Assigned(FOnGetKey) then
      FOnGetKey(Self, searchClass);
    Result := DoGetHelp(searchClass, HelpText);
  except
    Result := False;
  end;
end;

procedure TabcCustomDBHelpManager.StoreHelp(const FormClass, HelpText: string);
var
  saveClass: string;
begin
  saveClass := FormClass;
  try
    if Assigned(FOnGetKey) then
      FOnGetKey(Self, saveClass);
    DoStoreHelp(saveClass, HelpText);
  except
  end;
end;

procedure TabcCustomDBHelpManager.DeleteHelp(const FormClass: string);
var
  searchClass: string;
begin
  searchClass := FormClass;
  try
    if Assigned(FOnGetKey) then
      FOnGetKey(Self, searchClass);
    DoDeleteHelp(searchClass);
  except
  end;
end;

procedure TabcCustomDBHelpManager.SetDatasetState(Active: Boolean; ReadOnly: Boolean);
begin
  try
    DoSetDatasetState(Active, ReadOnly);
  except
  end;
end;

function TabcCustomDBHelpManager.DataSetIsActive: Boolean;
begin
  try
    Result := DoDataSetIsActive;
  except
    Result := False;
  end;
end;

{DB Interface Methods for Help Text reading and storage}

procedure TabcCustomDBHelpManager.DoSetDatasetState(Active: Boolean; ReadOnly: Boolean);
begin
end;

function TabcCustomDBHelpManager.DoDataSetIsActive: Boolean;
begin
  Result := False;
end;

function TabcCustomDBHelpManager.DoGetHelp(const FormClass: string; var HelpText: string): Boolean;
begin
  Result := False;
end;

procedure TabcCustomDBHelpManager.DoStoreHelp(const FormClass, HelpText: string);
begin
end;

procedure TabcCustomDBHelpManager.DoDeleteHelp(const FormClass: string);
begin
end;

end.
