unit abcdbhnt;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  DB, abcdbhnc, abcda
{$IFDEF DELPHI6}
  , Variants
{$ENDIF}
  ;
type

  { TabcDBHintManager }

  TabcDBHintManager = class(TabcCustomDBHintManager)
  private
    FHintTable: TDataset;
    FAdapter: TabcDatasetAdapter;
    procedure SetHintTable(Value: TDataset);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoSetDatasetState(Active: Boolean; ReadOnly: Boolean); override;
    function DoGetHint(const Control: TControl;
      const FormClass, ControlName: string; var ShortHint, LongHint: string): Boolean; override;
    procedure DoStoreHint(const Control: TControl;
      const FormClass, ControlName, ShortHint, LongHint: string); override;
    procedure DoDeleteHint(const Control: TControl; const FormClass, ControlName: string); override;
    function DoDataSetIsActive: Boolean; override;
  public
    destructor Destroy; override;
  published
    property HintTable: TDataset read FHintTable write SetHintTable;
    property Enabled;
    property HintColor;
    property HintPause;
    property HintShortPause;
    property HintHidePause;
    property KeyFirstParentForm;
    property ReadOnly;
    property ShowHint;
    property OnGetKey;
    property OnHint;
    property OnShowHint;
  end;

  { TabcDBHelpManager }

  TabcDBHelpManager = class(TabcCustomDBHelpManager)
  private
    FHelpTable: TDataset;
    FAdapter: TabcDatasetAdapter;
    procedure SetHelpTable(Value: TDataset);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoSetDatasetState(Active: Boolean; ReadOnly: Boolean); override;
    function DoGetHelp(const FormClass: string; var HelpText: string): Boolean; override;
    procedure DoStoreHelp(const FormClass, HelpText: string); override;
    procedure DoDeleteHelp(const FormClass: string); override;
    function DoDataSetIsActive: Boolean; override;
  public
    destructor Destroy; override;
  published
    property DefaultTopic;
    property HelpTable: TDataset read FHelpTable write SetHelpTable;
    property Enabled;
    property KeyFirstParentForm;
    property ReadOnly;
    property Template;
    property OnCloseForm;
    property OnGetKey;
    property OnShowForm;
  end;

implementation

{ TabcDBHintManager }

destructor TabcDBHintManager.Destroy;
begin
  FAdapter.Free;
  FAdapter := nil;
  inherited;
end;

procedure TabcDBHintManager.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = HintTable) and (Operation = opRemove) then
    HintTable := nil;
end;

procedure TabcDBHintManager.SetHintTable(Value: TDataset);
begin
  if Value <> FHintTable then
  begin
    FAdapter.Free;
    FAdapter := nil;
    FHintTable := Value;
    if Value = nil then
      Enabled := False
    else
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
    if ([csDesigning, csLoading] * ComponentState) = [] then
      SetDatasetState((ShowHint <> ashNone), ReadOnly);
  end;
end;

procedure TabcDBHintManager.DoSetDatasetState(Active: Boolean; ReadOnly: Boolean);
begin
  if (HintTable = nil) or (csDesigning in ComponentState) then Exit;
  HintTable.Active := False;
  if Active then
  begin
    if FAdapter.CanSetReadOnly then
      FAdapter.ReadOnly := ReadOnly;
    HintTable.Active := Active;
  end;
end;

function TabcDBHintManager.DoDataSetIsActive: Boolean;
begin
  Result := (HintTable <> nil) and HintTable.Active and
    (FAdapter.CanFindKey or FAdapter.CanLocate);
end;

function TabcDBHintManager.DoGetHint(const Control: TControl;
  const FormClass, ControlName: string; var ShortHint, LongHint: string): Boolean;
begin
  if DataSetIsActive then
    with FAdapter, HintTable do
    begin
      if (SearchMethod = asmFindKey) then
        Result := FindKey(VarArrayOf([FormClass, ControlName]))
      else
        Result := FAdapter.Locate('FORMCLASS;CONTROLNAME', VarArrayOf([FormClass, ControlName]), []);
      if Result then
      begin
        ShortHint := FieldByName('SHORTHINT').AsString;
        LongHint := FieldByName('LONGHINT').AsString;
      end
      else
      begin
        ShortHint := '';
        LongHint := '';
      end;
    end
  else
    Result := False;
end;

procedure TabcDBHintManager.DoStoreHint(const Control: TControl;
  const FormClass, ControlName, ShortHint, LongHint: string);
var
  recLocated: Boolean;
begin
  if DataSetIsActive then
    with FAdapter, HintTable do
    begin
      if (SearchMethod = asmFindKey) then
        recLocated := FindKey(VarArrayOf([FormClass, ControlName]))
      else
        recLocated := FAdapter.Locate('FORMCLASS;CONTROLNAME', VarArrayOf([FormClass, ControlName]), []);
      if recLocated then
      begin
        Edit;
        FieldByName('SHORTHINT').AsString := ShortHint;
        FieldByName('LONGHINT').AsString := LongHint;
        Post;
      end
      else
      begin
        Insert;
        FieldByName('FORMCLASS').AsString := FormClass;
        FieldByName('CONTROLNAME').AsString := ControlName;
        FieldByName('SHORTHINT').AsString := ShortHint;
        FieldByName('LONGHINT').AsString := LongHint;
        Post;
      end;
      FlushBuffers;
    end;
end;

procedure TabcDBHintManager.DoDeleteHint(const Control: TControl;
  const FormClass, ControlName: string);
var
  recLocated: Boolean;
begin
  if DataSetIsActive then
    with FAdapter, HintTable do
    begin
      if (SearchMethod = asmFindKey) then
        recLocated := FindKey(VarArrayOf([FormClass, ControlName]))
      else
        recLocated := FAdapter.Locate('FORMCLASS;CONTROLNAME', VarArrayOf([FormClass, ControlName]), []);
      if recLocated then
      begin
        Delete;
        FlushBuffers;
      end;
    end;
end;

{ TabcDBHelpManager }

destructor TabcDBHelpManager.Destroy;
begin
  FAdapter.Free;
  FAdapter := nil;
  inherited;
end;

procedure TabcDBHelpManager.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = HelpTable) and (Operation = opRemove) then
    HelpTable := nil;
end;

procedure TabcDBHelpManager.SetHelpTable(Value: TDataset);
begin
  if Value <> FHelpTable then
  begin
    FAdapter.Free;
    FAdapter := nil;
    FHelpTable := Value;
    if Value = nil then
      Enabled := False
    else
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
    if ([csDesigning, csLoading] * ComponentState) = [] then
      SetDatasetState(Enabled, ReadOnly);
  end;
end;

procedure TabcDBHelpManager.DoSetDatasetState(Active: Boolean; ReadOnly: Boolean);
begin
  if (HelpTable = nil) or (csDesigning in ComponentState) then Exit;
  if (HelpTable.Active = Active) and (FAdapter.ReadOnly = ReadOnly) then Exit;
  HelpTable.Active := False;
  if Active then
  begin
    if FAdapter.CanSetReadOnly then
      FAdapter.ReadOnly := ReadOnly;
    HelpTable.Active := Active;
  end;
end;

function TabcDBHelpManager.DoDataSetIsActive: Boolean;
begin
  Result := (HelpTable <> nil) and HelpTable.Active and
    (FAdapter.CanFindKey or FAdapter.CanLocate);
end;

function TabcDBHelpManager.DoGetHelp(const FormClass: string; var HelpText: string): Boolean;
begin
  if DataSetIsActive then
    with FAdapter, FHelpTable do
    begin
      if (SearchMethod = asmFindKey) then
        Result := FindKey(VarArrayOf([FormClass]))
      else
        Result := FAdapter.Locate('FORMCLASS', VarArrayOf([FormClass]), []);
      if Result then
        HelpText := FieldByName('HELPTEXT').AsString
      else
        HelpText := '';
    end
  else
    Result := False;
end;

procedure TabcDBHelpManager.DoStoreHelp(const FormClass, HelpText: string);
var
  recLocated: Boolean;
begin
  if DataSetIsActive then
    with FAdapter, HelpTable do
    begin
      if (SearchMethod = asmFindKey) then
        recLocated := FindKey(VarArrayOf([FormClass]))
      else
        recLocated := FAdapter.Locate('FORMCLASS', VarArrayOf([FormClass]), []);
      if recLocated then
      begin
        Edit;
        FieldByName('HELPTEXT').AsString := HelpText;
        Post;
      end
      else
      begin
        Insert;
        FieldByName('FORMCLASS').AsString := FormClass;
        FieldByName('HELPTEXT').AsString := HelpText;
        Post;
      end;
      FlushBuffers;
    end;
end;

procedure TabcDBHelpManager.DoDeleteHelp(const FormClass: string);
var
  recLocated: Boolean;
begin
  if DataSetIsActive then
    with FAdapter, HelpTable do
    begin
      if (SearchMethod = asmFindKey) then
        recLocated := FindKey(VarArrayOf([FormClass]))
      else
        recLocated := FAdapter.Locate('FORMCLASS', VarArrayOf([FormClass]), []);
      if recLocated then
      begin
        Delete;
        FlushBuffers;
      end;
    end;
end;

end.
