unit abcdbrng;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Messages, Classes, Graphics, Controls, StdCtrls, Forms, Menus,
  DB, DBCtrls, abcda;

type
  TabcApplyRangeWhen = (arwByFieldType, arwChange, arwEnter);

  TabcDBRangeEditDataLink = class;

  TabcCustomRangeEdit = class(TCustomEdit)
  private
    { Private declarations }
    FApplyRangeWhen: TabcApplyRangeWhen;
    FDataLink: TabcDBRangeEditDataLink;
    FGoToFirst: Boolean;
    FIxName: string;
    FIxFieldNames: string;
    FIxFieldCount: Integer;
    FIxFieldType: TFieldType;
    FIxField: TField;
    FSeparator: string;
    FDataset: TDataset;
    FAdapter: TabcDatasetAdapter;
    procedure SetDataset(Value: TDataset);
    procedure ActiveChanged;
    procedure DataSetChanged;
    procedure IndexChanged;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    { Protected declarations }
    procedure Change; override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ApplyRangeWhen: TabcApplyRangeWhen read FApplyRangeWhen write FApplyRangeWhen default arwByFieldType;
    property GoToFirst: Boolean read FGoToFirst write FGoToFirst default True;
    property Separator: string read FSeparator write FSeparator;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function UseRightToLeftAlignment: Boolean; override;
    procedure ApplyRange;
  end;

  TabcDBRangeEdit = class(TabcCustomRangeEdit)
  published
    property ApplyRangeWhen;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property DataSource;
    property GoToFirst;
    property Separator;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    {property Enabled;}
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

  { TabcDBRangeEditDataLink }

  TabcDBRangeEditDataLink = class(TDataLink)
  private
    FOwner: TabcCustomRangeEdit;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
  public
    constructor Create(AOwner: TabcCustomRangeEdit);
    destructor Destroy; override;
  end;


implementation

uses abcutild, abcutilf;

{ TabcDBRangeEditDataLink }

constructor TabcDBRangeEditDataLink.Create(AOwner: TabcCustomRangeEdit);
begin
  inherited Create;
  FOwner := AOwner;
  VisualControl := True;
end;

destructor TabcDBRangeEditDataLink.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

procedure TabcDBRangeEditDataLink.ActiveChanged;
begin
  if FOwner <> nil then FOwner.ActiveChanged;
end;

procedure TabcDBRangeEditDataLink.DataSetChanged;
begin
  if FOwner <> nil then FOwner.DataSetChanged;
end;

{ TabcCustomRangeEdit }

constructor TabcCustomRangeEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FDataLink := TabcDBRangeEditDataLink.Create(Self);
  FGoToFirst := True;
  FSeparator := '-';
end;

destructor TabcCustomRangeEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FDataset := nil;
  FAdapter.Free;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcCustomRangeEdit.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FAdapter.Free;
    FAdapter := nil;
    FDataset := Value;
    if Value <> nil then
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
  end;
end;

procedure TabcCustomRangeEdit.ActiveChanged;
begin
  SetDataset(FDataLink.Dataset);
  if FDataLink.Active and FAdapter.CanSetRange then begin
    Enabled := True;
    IndexChanged;
  end
  else
    Enabled := False;
end;

procedure TabcCustomRangeEdit.ApplyRange;
var
  SepPos: Integer;
  StartRange, EndRange: string;
begin
  if FDataLink.Active then
    with FAdapter do
      {set dataset range}
      if Text = '' then begin
        CancelRange;
        if FGoToFirst then FDataset.First;
      end
      else begin
        SepPos := Pos(FSeparator, Text);
        StartRange := '';
        EndRange := '';
        if SepPos = 0 then begin
          StartRange := Text;
          if FIxFieldType = ftString then
            EndRange := Text + #255
          else
            EndRange := Text;
        end
        else begin
          StartRange := abcLRTrim(Copy(Text, 1, SepPos - 1));
          EndRange := abcLRTrim(Copy(Text, SepPos + Length(Separator), Length(Text)));
          if EndRange <> '' then begin
            if FIxFieldType = ftString then
               EndRange := EndRange + #255;
          end
          else
            {set max value for field type}
            EndRange := abcMaxFieldType(FIxFieldType);
        end;
        try
          SetRangeStart;
          IndexFields[0].AsString := StartRange;
          SetRangeEnd;
          IndexFields[0].AsString := EndRange;
          ApplyRange;
          if FGoToFirst then FDataset.First;
        except
          on Exception do begin
            CancelRange;
            if FGoToFirst then FDataset.First;
            raise;
          end;
        end;
      end;
end;

procedure TabcCustomRangeEdit.Change;
begin
  inherited Change;
  if (FApplyRangeWhen = arwChange) or
     ((FApplyRangeWhen = arwByFieldType) and (FIxFieldType = ftString)) then
    ApplyRange;
end;

procedure TabcCustomRangeEdit.DataSetChanged;
begin
  if FAdapter <> nil then
    with FAdapter do
      if (IndexName <> FIxName) or (IndexFieldNames <> FIxFieldNames) or
         (IndexFieldCount <> FIxFieldCount) then
        IndexChanged;
end;

function TabcCustomRangeEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcCustomRangeEdit.IndexChanged;
begin
  with FAdapter do
    if CanSetRange and CanIndex and (IndexDefs <> nil) then
    begin
      IndexDefs.Update;
      FIxName := IndexName;
      FIxFieldNames := IndexFieldNames;
      FIxFieldCount := IndexFieldCount;
      if IndexFieldCount > 0 then
      begin
        FIxField := IndexFields[0];
        FIxFieldType := FIxField.DataType;
      end
      else
      begin
        FIxField := nil;
        FIxFieldType := ftUnknown;
      end;
      if FIxFieldType in [ftString..ftDateTime] then
        Enabled := True
      else
        Enabled := False;
    end
    else
    begin
      FIxName := '';
      FIxFieldNames := '';
      FIxFieldCount := 0;
      FIxField := nil;
      FIxFieldType := ftUnknown;
      Enabled := False;
    end;
  Caption := '';
end;

procedure TabcCustomRangeEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key = #13) then
    if (FApplyRangeWhen = arwEnter) or
       ((FApplyRangeWhen = arwByFieldType) and (FIxFieldType <> ftString)) then begin
      ApplyRange;
      Key := #0;
    end;
end;

procedure TabcCustomRangeEdit.Loaded;
begin
  inherited Loaded;
  ActiveChanged;
end;

procedure TabcCustomRangeEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcCustomRangeEdit.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

procedure TabcCustomRangeEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

function TabcCustomRangeEdit.UseRightToLeftAlignment: Boolean;
begin
  if FIxField <> nil then
    Result := DBUseRightToLeftAlignment(Self, FIxField)
  else
    Result := inherited UseRightToLeftAlignment;
end;

end.
