unit abcdbtre;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, DB, ExtCtrls, abcnp;

type

  TabcDBTreeView = class;

{ TabcDBTreeLevel }

  TabcDBTreeLevel = class(TCollectionItem)
  private
    FDataSource: TDataSource;
    FDataField: string;
    FImageField: string;
    FSelImageField: string;
    FStateImageField: string;
    FOverlayImageField: string;
    function GetDataSet: TDataSet;
    function GetField: TField;
    function GetFieldImage: TField;
    function GetFieldSelImage: TField;
    function GetFieldStateImage: TField;
    function GetFieldOverlayImage: TField;
    procedure SetDataSource(Value: TDataSource);
    procedure SetDataField(const Value: string);
    procedure SetImageField(const Value: string);
    procedure SetSelImageField(const Value: string);
    procedure SetStateImageField(const Value: string);
    procedure SetOverlayImageField(const Value: string);
  protected
    procedure SyncDataSource(ABookmark: TBookmarkStr);
  public
    procedure Assign(Source: TPersistent); override;
    function GetDisplayName: string; override;
    property DataSet: TDataSet read GetDataSet;
    property Field: TField read GetField;
    property FieldImage: TField read GetFieldImage;
    property FieldSelImage: TField read GetFieldSelImage;
    property FieldStateImage: TField read GetFieldStateImage;
    property FieldOverlayImage: TField read GetFieldOverlayImage;
  published
    property DataField: string read FDataField write SetDataField;
    property DataSource: TDataSource read FDataSource write SetDataSource;
    property ImageField: string read FImageField write SetImageField;
    property SelImageField: string read FSelImageField write SetSelImageField;
    property StateImageField: string read FStateImageField write SetStateImageField;
    property OverlayImageField: string read FOverlayImageField write SetOverlayImageField;
  end;

{ TabcDBTreeSources }

  TabcDBTreeSources = class(TCollection)
  private
    FTreeView: TabcDBTreeView;
    function GetItem(Index: Integer): TabcDBTreeLevel;
    procedure SetItem(Index: Integer; Value: TabcDBTreeLevel);
  protected
    function GetOwner: TPersistent; override;
  public
    constructor Create(ATreeView: TabcDBTreeView);
    function Add: TabcDBTreeLevel;
    procedure NotifyRemoved(ADataSource: TDataSource);
    property Items[Index: Integer]: TabcDBTreeLevel read GetItem write SetItem; default;
  end;

  TabcDBTreeOption = (adtSyncOnClick, adtExpandOnDblClick, adtClearOnCollapse);

  TabcDBTreeOptions = set of TabcDBTreeOption;

  TabcDBTreeCalcTextEvent =
    procedure(Sender: TObject; Level: Integer; Dataset: TDataset;
              var DisplayText: string) of object;


{  TabcDBTreeView }

  TabcTreeNodeMouseMoveEvent = procedure(Sender: TCustomTreeView; Node: TTreeNode; Shift: TShiftState; X, Y: Integer) of object;

  TabcDBTreeView = class(TTreeView)
  private
    FOptions: TabcDBTreeOptions;
    FDataSources: TabcDBTreeSources;
    FOnDeletion: TTVExpandedEvent;
    FOnGetText: TabcDBTreeCalcTextEvent;
    FSyncABCNavPanel: Boolean;
    FSyncSourceToNodeCount: Integer;
    FSyncToSourceTimer: TTimer;
    FSyncToSourceDataset: TDataset;
    FMouseOverNode: TTreeNode;
    FOnMouseMoveOverNode: TabcTreeNodeMouseMoveEvent;
    FInMouseMove: Boolean;
    procedure OnSyncToSourceTimer(Sender: TObject);
    procedure StopSyncTimer;
    procedure SetDataSources(const Value: TabcDBTreeSources);
    procedure DoOnDeletion(Sender: TObject; Node: TTreeNode);
    function GetSelectedBookmark: TBookmarkStr;
    function GetSelectedDataField: string;
    function GetSelectedDataSource: TDataSource;
    procedure SetDataField(Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Loaded; override;
    function CanChange(Node: TTreeNode): Boolean; override;
    procedure Change(Node: TTreeNode); override;
    function CanExpand(Node: TTreeNode): Boolean; override;
    procedure Expand(Node: TTreeNode); override;
    procedure Collapse(Node: TTreeNode); override;
    procedure Click; override;
    procedure DblClick; override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure ReloadChild(const Node: TTreeNode; DoExpand, Recurse: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadTreeLevel(Index: Integer; ParentNode: TTreeNode); virtual;
    procedure SyncSourceToNode(Node: TTreeNode);
    procedure SyncNodeToSource(Dataset: TDataset);
    procedure ClearCollapsed(Node: TTreeNode);
    procedure ReloadChildren(const Node: TTreeNode; DoExpand, DrillDown: Boolean);
    function SelectNodeContaining(aBookmark: TBookmarkStr; aTreeLevel: integer): Boolean;
    function LocateNodeWithBookmark(aBookmark: TBookmarkStr; aTreeLevel: integer): TTreeNode;
    function AddNodeWithBookmark(aBookmark: TBookmarkStr; aTreeLevel: integer): TTreeNode;
    function UpdateNodeWithBookmark(oldBookmark, newBookmark: TBookmarkStr; aTreeLevel: integer): Boolean;
    function DeleteNodeWithBookmark(aBookmark: TBookmarkStr; aTreeLevel: integer): Boolean;
    property SelectedBookmark: TBookmarkStr read GetSelectedBookmark;
  published
    {for ABC Nav Panel integration}
    property SyncABCNavPanel: Boolean read FSyncABCNavPanel write FSyncABCNavPanel default True;
    property DataField: string read GetSelectedDataField write SetDataField stored False;
    property DataSource: TDataSource read GetSelectedDataSource write SetDataSource stored False;
    property DataSources: TabcDBTreeSources read FDataSources write SetDataSources;
    property Options: TabcDBTreeOptions read FOptions write FOptions default
      [adtSyncOnClick, adtExpandOnDblClick];
    property OnDeletion: TTVExpandedEvent read FOnDeletion write FOnDeletion;
    property OnGetText: TabcDBTreeCalcTextEvent read FOnGetText write FOnGetText;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnMouseMoveOverNode: TabcTreeNodeMouseMoveEvent read FOnMouseMoveOverNode write FOnMouseMoveOverNode;

  end;

implementation

const
  SCROLLING_TIMER_DELAY = 400;

type
  PBookmark = ^TBookmarkStr;

function NewPBookmark(const aBookmark: TBookmarkStr): PBookmark;
begin
  New(Result);
  Result^ := aBookmark;
end;

{ TabcDBTreeView }

constructor TabcDBTreeView.Create(AOwner: TComponent);
begin
  inherited Create(aOwner);
  if csLoading in ComponentState then
    FSyncSourceToNodeCount := 1;
  inherited OnDeletion := DoOnDeletion;
  FDataSources := TabcDBTreeSources.Create(Self);
  FOptions := [adtSyncOnClick, adtExpandOnDblClick];
end;

destructor TabcDBTreeView.Destroy;
begin
  StopSyncTimer;
  FSyncToSourceTimer.Free;
  FDataSources.Free;
  inherited Destroy;
end;

procedure TabcDBTreeView.DoOnDeletion(Sender: TObject; Node: TTreeNode);
begin
  if Assigned(FOnDeletion) then
    FOnDeletion(Sender, Node);
  Dispose(Node.Data);
end;

function TabcDBTreeView.CanExpand(Node: TTreeNode): Boolean;
begin
  Result := inherited CanExpand(Node) or
    (Node.Level < (DataSources.Count - 1));
end;

procedure TabcDBTreeView.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Node: TTreeNode;
begin
  inherited;
  if FInMouseMove then
    Exit;
  FInMouseMove := True;
  try
  if Assigned(OnMouseMoveOverNode) then
  begin
    Node := GetNodeAt(X, Y);
    if Node <> FMouseOverNode then
    begin
      FMouseOverNode := Node;
      OnMouseMoveOverNode(Self, Node, Shift, X, Y);
    end;
  end;
  finally
    FInMouseMove := False;
  end;
end;

procedure TabcDBTreeView.Click;
var
  sel: TTreeNode;
begin
  if (adtSyncOnClick in Options) and (Selected <> nil) then
  begin
    sel := Selected;
    SyncSourceToNode(sel);
    Selected := sel;
  end;
  inherited Click;
end;

procedure TabcDBTreeView.DblClick;
begin
  if (adtExpandOnDblClick in Options) and
    (Selected <> nil) and
    not Selected.Expanded and
    not Selected.HasChildren and
    CanExpand(Selected) then
  begin
    Expand(Selected);
    Selected.Expand(False);
  end;
  inherited DblClick;
end;

procedure TabcDBTreeView.Expand(Node: TTreeNode);
begin
  if not Node.HasChildren then
    if Node.Level < DataSources.Count - 1 then
      LoadTreeLevel(Node.Level + 1, Node);
  inherited Expand(Node);
end;

procedure TabcDBTreeView.SetDataSources(const Value: TabcDBTreeSources);
begin
  FDataSources.Assign(Value);
end;

procedure TabcDBTreeView.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent is TDataSource) and (Operation = opRemove) then
    FDataSources.NotifyRemoved(TDataSource(AComponent));
end;

procedure TabcDBTreeView.Loaded;
begin
  inherited Loaded;
  FSyncSourceToNodeCount := 0;
end;

procedure TabcDBTreeView.ClearCollapsed(Node: TTreeNode);
begin
  if Node <> nil then
    Node.DeleteChildren;
end;

procedure TabcDBTreeView.Collapse(Node: TTreeNode);
begin
  if adtClearOnCollapse in Options then
    ClearCollapsed(Node);
end;

procedure TabcDBTreeView.LoadTreeLevel(Index: Integer; ParentNode: TTreeNode);
var
  NewNode: TTreeNode;
  dst: TDataSet;
  DisplayText: string;
begin
  dst := DataSources.Items[Index].DataSet;
  if (dst = nil) or not (dst.Active) then Exit;
  Items.BeginUpdate;
  try
    with DataSources.Items[Index], dst do
    begin
      SyncSourceToNode(ParentNode);
      dst.DisableControls;
      try
        First;
        while not EOF do
        begin
          if DataField <> '' then
            DisplayText := FieldByName(DataField).AsString
          else
            DisplayText := '';
          if Assigned(FOnGetText) then
            FOnGetText(Self, Index, dst, DisplayText);
          if ParentNode = nil then
            NewNode := Items.AddObject(nil, DisplayText, NewPBookmark(Bookmark))
          else
            NewNode := Items.AddChildObject(ParentNode, DisplayText, NewPBookmark(Bookmark));
          with NewNode do
          begin
            if ImageField <> '' then
              ImageIndex := FieldByName(ImageField).AsInteger;
            if SelImageField <> '' then
              SelectedIndex := FieldByName(SelImageField).AsInteger;
            if StateImageField <> '' then
              StateIndex := FieldByName(StateImageField).AsInteger;
            if OverlayImageField <> '' then
              OverlayIndex := FieldByName(OverlayImageField).AsInteger;
          end;
          Next;
        end;
      finally
        dst.EnableControls;
      end;
    end;
  finally
    Items.EndUpdate;
  end;
end;

function TabcDBTreeView.AddNodeWithBookmark(aBookmark: TBookmarkStr; aTreeLevel: integer): TTreeNode;
//returns new node or nil if there was an error
var
  ParentNode, NextSiblingNode: TTreeNode;
  dst: TDataSet;
  seekBookmark, DisplayText: string;
begin
  Result := nil;
  dst := DataSources.Items[aTreeLevel].DataSet;
  if (dst = nil) or not (dst.Active) then Exit;
  Items.BeginUpdate;
  try
    if aTreeLevel = 0 then
      ParentNode := nil
    else
      ParentNode := LocateNodeWithBookmark(DataSources.Items[aTreeLevel-1].DataSet.Bookmark, aTreeLevel-1);

    with DataSources.Items[aTreeLevel], dst do
    begin
      DisableControls;
      try
        //check for insert or append
        Next;
        if EOF then
          NextSiblingNode := nil
        else
          if ParentNode = nil then
            NextSiblingNode := LocateNodeWithBookmark(dst.Bookmark, aTreeLevel)
          else
          begin
            //faster to search immediately through parent's children
            seekBookmark := dst.Bookmark;
            NextSiblingNode := ParentNode.GetFirstChild;
            while NextSiblingNode <> nil do
              if CompareStr(seekBookmark, TBookmarkStr(NextSiblingNode.Data^)) = 0 then
                Break
              else
                NextSiblingNode := NextSiblingNode.GetNextSibling;
          end;
        dst.Bookmark := aBookmark;
        if DataField <> '' then
          DisplayText := FieldByName(DataField).AsString
        else
          DisplayText := '';
        if Assigned(FOnGetText) then
          FOnGetText(Self, Index, dst, DisplayText);

        if (NextSiblingNode <> nil) then
          Result := Items.InsertObject(NextSiblingNode, DisplayText, NewPBookmark(aBookmark))
        else if (ParentNode <> nil) then
          Result := Items.AddChildObject(ParentNode, DisplayText, NewPBookmark(aBookmark))
        else
          Result := Items.AddObject(nil, DisplayText, NewPBookmark(aBookmark));

        with Result do
        begin
          if ImageField <> '' then
            ImageIndex := FieldByName(ImageField).AsInteger;
          if SelImageField <> '' then
            SelectedIndex := FieldByName(SelImageField).AsInteger;
          if StateImageField <> '' then
            StateIndex := FieldByName(StateImageField).AsInteger;
          if OverlayImageField <> '' then
            OverlayIndex := FieldByName(OverlayImageField).AsInteger;
        end;

      finally
        EnableControls;
      end;
    end;
  finally
    Items.EndUpdate;
  end;
end;

function TabcDBTreeView.UpdateNodeWithBookmark(oldBookmark, newBookmark: TBookmarkStr; aTreeLevel: integer): Boolean;
var
  node: TTreeNode;
  dst: TDataSet;
  DisplayText: string;
begin
  node := LocateNodeWithBookmark(oldBookmark, aTreeLevel);
  Result := node <> nil;
  if Result then
  begin
    //update the node information from the dataset
    dst := DataSources.Items[aTreeLevel].DataSet;
    if (dst = nil) or not (dst.Active) then Exit;
    Items.BeginUpdate;
    dst.DisableControls;
    try
      SyncSourceToNode(node);
      with DataSources.Items[aTreeLevel], dst do
      begin
        if DataField <> '' then
          DisplayText := FieldByName(DataField).AsString
        else
          DisplayText := '';
        if Assigned(FOnGetText) then
          FOnGetText(Self, Index, dst, DisplayText);
        with node do
        begin
          Text := displayText;
          if ImageField <> '' then
            ImageIndex := FieldByName(ImageField).AsInteger;
          if SelImageField <> '' then
            SelectedIndex := FieldByName(SelImageField).AsInteger;
          if StateImageField <> '' then
            StateIndex := FieldByName(StateImageField).AsInteger;
          if OverlayImageField <> '' then
            OverlayIndex := FieldByName(OverlayImageField).AsInteger;
          if CompareStr(newBookmark, oldBookmark) <> 0 then
          begin
            Dispose(node.Data);
            node.Data := NewPBookmark(newBookmark);
          end;
        end;
      end;
    finally
      dst.EnableControls;
      Items.EndUpdate;
    end;
  end;
end;

function TabcDBTreeView.DeleteNodeWithBookmark(aBookmark: TBookmarkStr; aTreeLevel: integer): Boolean;
var
  node: TTreeNode;
begin
  node := LocateNodeWithBookmark(aBookmark, aTreeLevel);
  Result := False;
  if node <> nil then
  begin
    Items.BeginUpdate;
    try
      node.Delete;
      Result := True;
    finally
      Items.EndUpdate;
    end;
  end;
end;

function TabcDBTreeView.SelectNodeContaining(aBookmark: TBookmarkStr; aTreeLevel: integer): Boolean;
var
  node: TTreeNode;
begin
  node := LocateNodeWithBookmark(aBookmark, aTreeLevel);
  Result := node <> nil;
  if Result then
  begin
    node.MakeVisible;
    node.Selected := True;
  end;
end;

function TabcDBTreeView.LocateNodeWithBookmark(aBookmark: TBookmarkStr; aTreeLevel: integer): TTreeNode;
//this is the slow part, since nodes/bookmarks are not indexed in any way
//would be much faster if we could do a search on a sorted list of bookmarks, for example
var
  next: TTreeNode;
begin
  //searches all nodes for matching bookmark
  Result := nil;
  if Items.Count = 0 then Exit;
  Result := Items[0];
  while Result <> nil do
  begin
    if Result.Level > aTreeLevel then Exit;
    if Result.Level < aTreeLevel then
      if Result.HasChildren then
        Result := Result.GetFirstChild
      else
        Result := Result.GetNextSibling
    else {level = aTreeLevel}
    begin
      if CompareStr(aBookmark, TBookmarkStr(Result.Data^)) = 0 then //bookmark found
        Exit
      else
      begin
        next := Result.GetNextSibling;
        while next = nil do
        begin
          Result := Result.parent;
          if Result = nil then
            Exit;
          next := Result.GetNextSibling;
        end;
        Result := next;
      end;
    end;
  end;
end;

procedure TabcDBTreeView.OnSyncToSourceTimer(Sender: TObject);
var
  i, seekLevel: integer;
begin
  StopSyncTimer;
  if FSyncSourceToNodeCount = 0 then
  begin
    seekLevel := -1;
    if (FSyncToSourceDataset <> nil) and FSyncToSourceDataset.Active then
    begin
      //find level to sync
      with DataSources do
        for i := 0 to Count - 1 do
          if Items[i].Dataset = FSyncToSourceDataset then
          begin
            seekLevel := i;
            Break;
          end;
      if seekLevel = -1 then Exit;
      //find node for current bookmark
      SelectNodeContaining(FSyncToSourceDataset.Bookmark, seekLevel);
    end;
  end;
end;

procedure TabcDBTreeView.StopSyncTimer;
begin
  if FSyncToSourceTimer <> nil then
    FSyncToSourceTimer.Enabled := False;
end;

procedure TabcDBTreeView.SyncNodeToSource(Dataset: TDataset);
begin
  if FSyncSourceToNodeCount = 0 then
  begin
    if FSyncToSourceTimer = nil then
    begin
      FSyncToSourceTimer := TTimer.Create(self);
      with FSyncToSourceTimer do
      begin
        Enabled := False;
        Interval := SCROLLING_TIMER_DELAY;
        OnTimer := OnSyncToSourceTimer;
      end;
    end;
    FSyncToSourceDataset := Dataset;
    FSyncToSourceTimer.Enabled := False; //restart timer every time we get a scroll event
    FSyncToSourceTimer.Enabled := True;
  end;
end;

procedure TabcDBTreeView.SyncSourceToNode(Node: TTreeNode);
var
  ds: TDataset;
begin
  if Node <> nil then
  begin
    if Node.Level < DataSources.Count then
    begin
      ds := DataSources.Items[Node.Level].Dataset;
      ds.DisableControls;
    end
    else
      ds := nil;
    try
      FSyncSourceToNodeCount := FSyncSourceToNodeCount + 1;
      SyncSourceToNode(Node.Parent);
      if ds <> nil then
        DataSources.Items[Node.Level].SyncDataSource(TBookmarkStr(Node.Data^));
      FSyncSourceToNodeCount := FSyncSourceToNodeCount - 1;
    finally
      if ds <> nil then
        ds.EnableControls;
    end;
  end;
end;

function TabcDBTreeView.CanChange(Node: TTreeNode): Boolean;
begin
  Result := inherited CanChange(Node);
  if Result then
    FSyncSourceToNodeCount := FSyncSourceToNodeCount + 1;
end;

procedure TabcDBTreeView.Change(Node: TTreeNode);
begin
  StopSyncTimer;
  inherited;
  FSyncSourceToNodeCount := FSyncSourceToNodeCount - 1;
end;

function TabcDBTreeView.GetSelectedBookmark: TBookmarkStr;
begin
  if Selected <> nil then
    Result := TBookmarkStr(Selected.Data^);
end;

function TabcDBTreeView.GetSelectedDataField: string;
begin
  if not (csDesigning in ComponentState) and
     (Selected <> nil) and (DataSources.Count > Selected.Level) then
    Result := DataSources.Items[Selected.Level].DataField
  else
    Result := '';
end;

procedure TabcDBTreeView.SetDataField(Value: string);
begin
end;

procedure TabcDBTreeView.SetDataSource(Value: TDataSource);
begin
end;

function TabcDBTreeView.GetSelectedDataSource: TDataSource;
begin
  if not (csDesigning in ComponentState) and
     (Selected <> nil) and (DataSources.Count > Selected.Level) then
    Result := DataSources.Items[Selected.Level].DataSource
  else
    Result := nil;
end;

procedure TabcDBTreeView.ReloadChildren(const Node: TTreeNode; DoExpand, DrillDown: Boolean);
begin
  Items.BeginUpdate;
  try
    ReloadChild(Node, DoExpand, DrillDown);
  finally
    Items.EndUpdate;
  end;
end;

procedure TabcDBTreeView.ReloadChild(const Node: TTreeNode; DoExpand, Recurse: Boolean);
var
  NextNode: TTreeNode;
begin
  if Node = nil then
  begin
    Items.Clear;
    LoadTreeLevel(0,nil);
    if Recurse and (Items.Count > 0) then
    begin
      NextNode := Items[0];
      repeat
        ReloadChild(NextNode, DoExpand, Recurse);
        NextNode := NextNode.GetNextSibling;
      until (NextNode = nil);
    end;
  end
  else
  begin
    Node.DeleteChildren;
    if DoExpand then
    begin
      Expand(Node);
      Node.Expand(True);
    end
    else if Node.Level < DataSources.Count - 1 then
      LoadTreeLevel(Node.Level + 1, Node)
    else
      Exit;
    if Recurse and Node.HasChildren then
    begin
      NextNode := Node.GetFirstChild;
      repeat
        ReloadChild(NextNode, DoExpand, Recurse);
        NextNode := Node.GetNextChild(NextNode);
      until (NextNode = nil);
    end;
  end;
end;

{ TabcDBTreeLevel }

procedure TabcDBTreeLevel.Assign(Source: TPersistent);
begin
  if Source is TabcDBTreeLevel then
  begin
    DataSource := TabcDBTreeLevel(Source).DataSource;
    DataField := TabcDBTreeLevel(Source).DataField;
    ImageField := TabcDBTreeLevel(Source).ImageField;
    SelImageField := TabcDBTreeLevel(Source).SelImageField;
    StateImageField := TabcDBTreeLevel(Source).StateImageField;
    OverlayImageField := TabcDBTreeLevel(Source).OverlayImageField;
  end
  else
    inherited Assign(Source);
end;

function TabcDBTreeLevel.GetDisplayName: string;
begin
  if DataSource <> nil then
  begin
    Result := DataSource.Name;
    if DataSource.Owner <> TabcDBTreeSources(Collection).FTreeView.Owner then
      Result := DataSource.Owner.Name + '.' + Result;
  end
  else
    Result := '';
  if Result = '' then
    Result := inherited GetDisplayName;
end;

function TabcDBTreeLevel.GetDataSet: TDataSet;
begin
  if DataSource = nil then
    Result := nil
  else
    Result := DataSource.DataSet;
end;

function TabcDBTreeLevel.GetField: TField;
var
  ds: TDataSet;
begin
  ds := DataSet;
  if ds = nil then
    Result := nil
  else
    Result := ds.FindField(DataField);
end;

function TabcDBTreeLevel.GetFieldImage: TField;
var
  ds: TDataSet;
begin
  ds := DataSet;
  if ds = nil then
    Result := nil
  else
    Result := ds.FindField(ImageField);
end;

function TabcDBTreeLevel.GetFieldSelImage: TField;
var
  ds: TDataSet;
begin
  ds := DataSet;
  if ds = nil then
    Result := nil
  else
    Result := ds.FindField(SelImageField);
end;

function TabcDBTreeLevel.GetFieldStateImage: TField;
var
  ds: TDataSet;
begin
  ds := DataSet;
  if ds = nil then
    Result := nil
  else
    Result := ds.FindField(StateImageField);
end;

function TabcDBTreeLevel.GetFieldOverlayImage: TField;
var
  ds: TDataSet;
begin
  ds := DataSet;
  if ds = nil then
    Result := nil
  else
    Result := ds.FindField(OverlayImageField);
end;

procedure TabcDBTreeLevel.SetDataSource(Value: TDataSource);
begin
  if DataSource <> Value then
  begin
    FDataSource := Value;
    Changed(False);
  end;
end;

procedure TabcDBTreeLevel.SetDataField(const Value: string);
begin
  if DataField <> Value then
  begin
    FDataField := Value;
    Changed(False);
  end;
end;

procedure TabcDBTreeLevel.SetImageField(const Value: string);
begin
  if ImageField <> Value then
  begin
    FImageField := Value;
    Changed(False);
  end;
end;

procedure TabcDBTreeLevel.SetSelImageField(const Value: string);
begin
  if SelImageField <> Value then
  begin
    FSelImageField := Value;
    Changed(False);
  end;
end;

procedure TabcDBTreeLevel.SetStateImageField(const Value: string);
begin
  if StateImageField <> Value then
  begin
    FStateImageField := Value;
    Changed(False);
  end;
end;

procedure TabcDBTreeLevel.SetOverlayImageField(const Value: string);
begin
  if OverlayImageField <> Value then
  begin
    FOverlayImageField := Value;
    Changed(False);
  end;
end;

procedure TabcDBTreeLevel.SyncDataSource(ABookmark: TBookmarkStr);
var
  dst: TDataSet;
  tree: TabcDBTreeView;
  parForm: TCustomForm;
begin
  dst := DataSet;
  if (dst <> nil) and dst.Active then
    if dst.Bookmark <> ABookmark then
      dst.Bookmark := ABookmark;
  tree := TabcDBTreeSources(Collection).FTreeView;
  if tree.SyncABCNavPanel then
  begin
    parForm := GetParentForm(tree);
    parForm.ActiveControl := nil;
    parForm.ActiveControl := tree;
  end;
end;

{ TabcDBTreeSources }

constructor TabcDBTreeSources.Create(ATreeView: TabcDBTreeView);
begin
  inherited Create(TabcDBTreeLevel);
  FTreeView := ATreeView;
end;

function TabcDBTreeSources.Add: TabcDBTreeLevel;
begin
  Result := TabcDBTreeLevel(inherited Add);
end;

function TabcDBTreeSources.GetOwner: TPersistent;
begin
  Result := FTreeView;
end;

function TabcDBTreeSources.GetItem(Index: Integer): TabcDBTreeLevel;
begin
  Result := TabcDBTreeLevel(inherited GetItem(Index));
end;

procedure TabcDBTreeSources.SetItem(Index: Integer; Value: TabcDBTreeLevel);
begin
  inherited SetItem(Index, Value);
end;

procedure TabcDBTreeSources.NotifyRemoved(ADataSource: TDataSource);
var
  i: Integer;
begin
  for i := 0 to Count - 1 do
    if Items[i].DataSource = ADataSource then
    begin
      Items[i].DataSource := nil;
      if FTreeView <> nil then
        if (csDesigning in FTreeView.ComponentState) then
          GetParentForm(FTreeView).Designer.Modified;
    end;
end;

end.
