unit abcdfabh;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  abcexctl, StdCtrls, ExtCtrls, abclabel, ShellAPI, abcversn,
  abcdlg;

type
  TabcdfHintAbout = class(TForm)
    imgComponent: TImage;
    btnCancel: TButton;
    btnExplore: TButton;
    Bevel2: TBevel;
    lbmVersion: TabcMultilineLabel;
    abcURLLabel1: TabcURLLabel;
    btnHelp: TButton;
    imgComponent1: TImage;
    imgComponent2: TImage;
    Panel1: TPanel;
    lblCompDesc: TLabel;
    abcMailToLabel1: TabcMailToLabel;
    abcVersionLabel1: TabcVersionLabel;
    lblURLCheckRelease: TabcMouseLabel;
    abcMessageDlg1: TabcMessageDlg;
    Bevel1: TBevel;
    procedure btnHelpClick(Sender: TObject);
    procedure btnExploreClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCancelClick(Sender: TObject);
    procedure lblURLCheckReleaseClick(Sender: TObject);
  private
    FabcClassName: string;
    FSaveAppOnShowHint: TShowHintEvent;
    FExploreApp: string;
    procedure AppShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
    procedure SetabcClassName(const Value: string);
    procedure LoadComponentBitmap;
    procedure LoadDescriptions;
  public
    { Public declarations }
    property abcClassName: string read FabcClassName write SetabcClassName;
  end;

var
  abcdfHintAbout: TabcdfHintAbout;

procedure abcShowIDEHintAbout;

resourcestring
  abcIDEReleaseURL = 'http://www.obsof.com/products/abc/release.txt';

implementation

uses Registry, WinInet, abcdsncn, abcutilf;

{$R *.DFM}

{.$R abcr32a.res}

function abcIDEGetLatestRelease: string;
const
  bufSize = 1024;
var
  hInt, hUrl: HINTERNET;
  buf: PChar;
  bufRead: DWORD;
begin
  Result := '';
  hInt := nil;
  hUrl := nil;
  buf := nil;
  try
    hInt := InternetOpen(
      'ABC for Delphi 6.0',
      INTERNET_OPEN_TYPE_DIRECT,
      nil,
      nil,
      0);
  if hInt <> nil then
  begin
    hUrl := InternetOpenUrl(
    hInt,
    PChar(abcIDEReleaseURL),
    nil,
    0,
    INTERNET_FLAG_NO_CACHE_WRITE or
    INTERNET_FLAG_PRAGMA_NOCACHE or
    INTERNET_FLAG_RELOAD or 
    INTERNET_FLAG_RESYNCHRONIZE,
    0);

    if hUrl <> nil then
    begin
      buf := AllocMem(bufSize);
      if InternetReadFile(hUrl, buf, bufSize - 1, bufRead) and (bufRead > 0) then
        Result := buf;
    end;
  end;

  finally
    if buf <> nil then FreeMem(buf);
    if hUrl <> nil then InternetCloseHandle(hUrl);
    if hInt <> nil then InternetCloseHandle(hInt);
  end;
end;

procedure abcShowIDEHintAbout;
begin
  if abcdfHintAbout = nil then
    abcdfHintAbout := TabcdfHintAbout.Create(Application);
  abcdfHintAbout.Caption := sabcdfHintAboutCaption;
  abcdfHintAbout.lblCompDesc.Caption := sabcdfHintAboutlblCompDescCaption;
  abcdfHintAbout.Show;
end;

{ TabcdfAbout }

procedure TabcdfHintAbout.SetabcClassName(const Value: string);
begin
  FabcClassName := Value;
  LoadComponentBitmap;
  LoadDescriptions;
end;

procedure TabcdfHintAbout.btnHelpClick(Sender: TObject);
begin
  Close;
  if FabcClassName <> '' then
    //load help on component
    Application.HelpCommand(HELP_PARTIALKEY, Integer(PChar(FabcClassName)))
  else
    //load the abc help file
    ShellExecute(
      Application.Handle,
      'open',
      'winhlp32 abc6.hlp',
      '',
      '',
      SW_SHOWNORMAL);
end;

procedure TabcdfHintAbout.btnExploreClick(Sender: TObject);
begin
  if FileExists(FExploreApp) then
    ShellExecute(
      Application.Handle,
      'open',
      PChar(FExploreApp),
      '',
      '',
      SW_SHOWNORMAL);
  Close;
end;

type
  PBmpData = ^TBmpData;
  TBmpData = record
    ResName: string;
    hBmp: HBITMAP;
  end;

function LoadModuleBitmap(HInstance: Integer; Data: Pointer): Boolean;
begin
  with PBmpData(Data)^ do
  begin
    hBmp := LoadBitmap(HInstance, PChar(ResName));
    Result := hBmp = 0;
  end;
end;

procedure TabcdfHintAbout.LoadComponentBitmap;
var
  aClass: TPersistentClass;
  bmpData: TBmpData;
  srcBitmap, maskedBitmap: TBitmap;
  srcR, destR: TRect;
begin
  aClass := GetClass(FabcClassName);
  if aClass = nil then
    aClass := GetClass('TComponent');
  bmpData.hBmp := 0;
  while bmpData.hBmp = 0 do
  begin
    bmpData.ResName := UpperCase(aClass.ClassName);
    EnumResourceModules(LoadModuleBitmap, @bmpData);
    if bmpData.hBmp = 0 then
      aClass := TPersistentClass(aClass.ClassParent);
    if aClass = TPersistent then
      Exit;
  end;
  srcBitmap := TBitmap.Create;
  maskedBitmap := TBitmap.Create;
  with maskedBitmap do
  try
    srcBitmap.Handle := bmpData.hBmp;
    Width := srcBitmap.Width + 2;
    Height := srcBitmap.Height + 2;
    srcR := Rect(0,0,srcBitmap.Width-1, srcBitmap.Height-1);
    destR := Rect(0,0,Width,Height);
    Frame3D(Canvas, destR, clBtnHighlight, clBtnShadow, 1);
    with Canvas do
    begin
      Brush.Color := clBtnFace;
      BrushCopy(destR, srcBitmap, srcR, srcBitmap.TransparentColor);
    end;
    imgComponent.Picture.Graphic := maskedBitmap;
    imgComponent1.Picture.Graphic := maskedBitmap;
    imgComponent2.Picture.Graphic := maskedBitmap;
  finally
    srcBitmap.Free;
    Free;
    DeleteObject(bmpData.hBmp);
  end;
end;

procedure TabcdfHintAbout.LoadDescriptions;
begin
  lblCompDesc.Caption := abcIDEComponentDescription(FabcClassName);
end;

procedure TabcdfHintAbout.FormCreate(Sender: TObject);
begin
  FSaveAppOnShowHint := Application.OnShowHint;
  Application.OnShowHint := AppShowHint;
  with lbmVersion.Lines do
  begin
    Clear;
    Add('Release ' + sabcRelease + ' ' + sabcEdition);
    Add(sabcCopyright);
  end;

  //locate Explore demo app
  with TRegistry.Create do
  try
    RootKey := HKEY_LOCAL_MACHINE;
    if OpenKeyReadOnly(sabcdsgnRegKey) then
    begin
      FExploreApp := ReadString(sabcdsgnRegExplore);
      if not FileExists(FExploreApp) then
        btnExplore.Enabled := False;
    end;
  finally
    Free;
  end;

  //email body for support

  with abcMailToLabel1.Mail.Body do
  begin
    Add('  ABC ' + sabcRelease + ' ' + sabcEdition);
    //Delphi or BCB version
    with abcVersionLabel1 do
      Add('  ' + VersionInfo[0].Value + ' Build ' + VersionInfo[1].Value + ' ' + VersionInfo[2].Value);
    //Windows version
    Add('');
    abcGetWindowsVersionStrings(abcMailToLabel1.Mail.Body);
    Add('');
  end;
end;

procedure TabcdfHintAbout.AppShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
begin
  if Assigned(FSaveAppOnShowHint) then
    FSaveAppOnShowHint(HintStr, CanShow, HintInfo);
  if Copy(HintStr,1,3) = 'abc' then
    abcClassName := 'T' + HintStr
  else
    lblCompDesc.Caption := sabcdfHintAboutlblCompDescCaption;
end;

procedure TabcdfHintAbout.FormDestroy(Sender: TObject);
begin
  Application.OnShowHint := FSaveAppOnShowHint;
  abcdfHintAbout := nil;
end;

procedure TabcdfHintAbout.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TabcdfHintAbout.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TabcdfHintAbout.lblURLCheckReleaseClick(Sender: TObject);
var
  sLatestVersion: string;
begin
  sLatestVersion := '';
  Screen.Cursor := crHourglass;
  try
    sLatestVersion := abcIDEGetLatestRelease;
  finally
    Screen.Cursor := crDefault;
  end;
  with abcMessageDlg1, Message do
  begin
    Clear;
    DlgType := mtInformation;
    if Trim(sLatestVersion) = '' then
    begin
      Add('');
      Add('Could not read the release URL');
      Add('');
      Add(abcIDEReleaseURL);
      DlgType := mtWarning;
    end
    else if AnsiCompareStr(sabcRelease, sLatestVersion) <> 0 then
    begin
      Add('');
      Add('You have release ' + sabcRelease);
      Add('');
      Add('The current release is ' + sLatestVersion);
    end
    else
    begin
      Add('');
      Add('You have the current release ' + sabcRelease);
    end;
    Execute;
  end;
end;

end.
