unit abcdffxp;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons;

type
  TabcdfImageEffectSetSelector = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    SrcList: TListBox;
    DstList: TListBox;
    SrcLabel: TLabel;
    DstLabel: TLabel;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FInitList: TList;
  public
    { Public declarations }
    property InitList: TList read FInitList write FInitList;
  end;

var
  abcdfImageEffectSetSelector: TabcdfImageEffectSetSelector;

function abcDlgEditTransitionEffectList(EffectList: TList): Boolean;

implementation

{$R *.DFM}

function abcDlgEditTransitionEffectList(EffectList: TList): Boolean;
var
  i: integer;
begin
  with TabcdfImageEffectSetSelector.Create(nil) do
  try
    InitList := EffectList;
    Result := (ShowModal = mrOk);
    if Result then
    begin
      {update list items}
      EffectList.Clear;
      with DstList.Items do
        for i := 0 to Count - 1 do
          EffectList.Add(Objects[i]);
    end;
  finally
    Free;
  end;
end;

procedure TabcdfImageEffectSetSelector.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TabcdfImageEffectSetSelector.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
end;

procedure TabcdfImageEffectSetSelector.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
    DstList.Items.AddObject(SrcList.Items[I],
      SrcList.Items.Objects[I]);
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
end;

procedure TabcdfImageEffectSetSelector.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
    if SrcList.Items.IndexOf(DstList.Items[I]) = -1 then
      SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
  DstList.Items.Clear;
  SetItem(DstList, 0);
end;

procedure TabcdfImageEffectSetSelector.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := 0 to List.Items.Count - 1 do
    if List.Selected[I] then
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
      List.Items.Delete(I);
end;

procedure TabcdfImageEffectSetSelector.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;
  IncludeBtn.Enabled := not SrcEmpty;
  IncAllBtn.Enabled := not SrcEmpty;
  ExcludeBtn.Enabled := not DstEmpty;
  ExAllBtn.Enabled := not DstEmpty;
end;

function TabcdfImageEffectSetSelector.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TabcdfImageEffectSetSelector.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

procedure TabcdfImageEffectSetSelector.FormCreate(Sender: TObject);
var
  i: integer;
begin
  with SrcList.Items do
    for i := 0 to Count - 1 do
      Objects[i] := Pointer(i);
end;

procedure TabcdfImageEffectSetSelector.FormShow(Sender: TObject);
var
  i,j: integer;
begin
  if InitList <> nil then
  begin
    for i := 0 to InitList.Count - 1 do
    begin
      j := SrcList.Items.IndexOfObject(InitList[i]);
      if j <> -1 then
      begin
        DstList.Items.AddObject(SrcList.Items[j], SrcList.Items.Objects[j]);
        DstList.Items.Delete(j);
      end;
    end;
  end;
  SrcList.Sorted := True;
  SetButtons;
end;

end.
