unit abcdflin;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses Windows, Classes, Graphics, Forms, Controls, Buttons, StdCtrls, ExtCtrls,
     Dialogs, abcspin, abcshape, TabNotBk, ComCtrls;

type
  TabcdfLineEdit = class(TForm)
    Panel1: TPanel;
    OKBtn: TBitBtn;
    SaveDialog: TSaveDialog;
    CancelBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    ColorDialog1: TColorDialog;
    TabbedNotebook1: TTabbedNotebook;
    GroupBox1: TGroupBox;
    fraAH: TabcFrame;
    lnAHBoth: TabcLine;
    lnAHEnd: TabcLine;
    lnAHStart: TabcLine;
    lnAHNone: TabcLine;
    GroupBox2: TGroupBox;
    fraAS: TabcFrame;
    lnASSolid: TabcLine;
    lnASBox: TabcLine;
    lnASLine: TabcLine;
    GroupBox3: TGroupBox;
    lnArrSize: TabcLine;
    abcSpinEdit1: TabcSpinEdit;
    GroupBox7: TGroupBox;
    fraDI: TabcFrame;
    lnDIHorz: TabcLine;
    lnDIVert: TabcLine;
    lnDITopBtm: TabcLine;
    lnDIBtmTop: TabcLine;
    GroupBox4: TGroupBox;
    lnColor: TabcLine;
    BitBtn1: TBitBtn;
    GroupBox5: TGroupBox;
    fraST: TabcFrame;
    lnSTClear: TabcLine;
    lnSTDash: TabcLine;
    lnSTDashDot: TabcLine;
    lnSTDashDotDot: TabcLine;
    lnSTDot: TabcLine;
    lnSTSolid: TabcLine;
    GroupBox6: TGroupBox;
    lnWidth: TabcLine;
    abcSpinEdit2: TabcSpinEdit;
    procedure Panel1Resize(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure abcSpinEdit2Change(Sender: TObject);
    procedure lnAHBothMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure abcSpinEdit1Change(Sender: TObject);
    procedure lnAHBothClick(Sender: TObject);
    procedure lnASSolidClick(Sender: TObject);
    procedure lnDIHorzClick(Sender: TObject);
    procedure lnSTSolidClick(Sender: TObject);
  private
    { Private declarations }
    FHLine: TabcLine;
    FUpdateLine: TabcLine;
    procedure GetLineProps(ALine: TabcLine);
    procedure SetLineProps(ALine: TabcLine);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;


function ExecabcdfLineEdit(ALine: TabcLine): boolean;

implementation

{$R *.DFM}

function ExecabcdfLineEdit(ALine: TabcLine): boolean;
begin
  Result := False;
  with TabcdfLineEdit.Create(nil) do
  try
    GetLineProps(ALine);
    if ShowModal = mrOK then begin
      SetLineProps(ALine);
      Result := True;
    end;
  finally
    Free;
  end;
end;

{ TabcsfLineEdit }

procedure TabcdfLineEdit.GetLineProps(ALine: TabcLine);
var
  Sender: TabcLine;
begin
  Sender := nil;
  with ALine do begin
    {arrow heads}
    case Arrows of
      arBoth: Sender := lnAHBoth;
      arEnd: Sender := lnAHEnd;
      arStart: Sender := lnAHStart;
      arNone: Sender := lnAHNone;
    end;
    lnAHBothClick(Sender);
    {arrow style}
    case ArrowStyle of
      ahSolid: Sender := lnASSolid;
      ahBox: Sender := lnASBox;
      ahLine: Sender := lnASLine;
    end;
    lnASSolidClick(Sender);
    {arrow size}
    abcSpinEdit1.Value := ArrowSize;
    abcSpinEdit1Change(Self);
    {direction}
    case Direction of
      diHorizontal:  Sender := lnDIHorz;
      diVertical:    Sender := lnDIVert;
      diToptoBottom: Sender := lnDITopBtm;
      diBottomtoTop: Sender := lnDIBtmTop;
    end;
    lnDIHorzClick(Sender);
    {color}
    lnColor.Color := Color;
    FUpdateLine.Color := Color;
    {line style}
    case LineStyle of
      psSolid:       Sender := lnSTSolid;
      psDot:         Sender := lnSTDot;
      psDash:        Sender := lnSTDash;
      psDashDot:     Sender := lnSTDashDot;
      psDashDotDot:  Sender := lnSTDashDotDot;
      psClear:       Sender := lnSTClear;
      psInsideFrame: Sender := lnSTSolid;
    end;
    lnSTSolidClick(Sender);
    {line width}
    abcSpinEdit2.Value := LineWidth;
    abcSpinEdit2Change(Self);
  end;
end;

procedure TabcdfLineEdit.SetLineProps(ALine: TabcLine);
begin
  with ALine do begin
    Arrows := FUpdateLine.Arrows;
    ArrowStyle := FUpdateLine.ArrowStyle;
    ArrowSize := FUpdateLine.ArrowSize;
    Direction := FUpdateLine.Direction;
    Color := FUpdateLine.Color;
    LineStyle := FUpdateLine.LineStyle;
    LineWidth := FUpdateLine.LineWidth;
  end;
end;

constructor TabcdfLineEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FUpdateLine := TabcLine.Create(Self);
end;

destructor TabcdfLineEdit.Destroy;
begin
  FUpdateLine.Free;
  inherited Destroy;
end;

procedure TabcdfLineEdit.Panel1Resize(Sender: TObject);
const
  Offset = 5;
var
  LastPos: Integer;
begin
  LastPos := Panel1.Width - Offset;
  with CancelBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with OkBtn do
    if Visible then
      Left := LastPos - Width;
end;

procedure TabcdfLineEdit.BitBtn1Click(Sender: TObject);
begin
  with ColorDialog1 do begin
    Color := lnColor.Color;
    if Execute then begin
      lnColor.Color := Color;
      FUpdateLine.Color := Color;
    end;
  end;
end;

procedure TabcdfLineEdit.abcSpinEdit2Change(Sender: TObject);
begin
  lnWidth.LineWidth := abcSpinEdit2.Value;
  FUpdateLine.LineWidth := abcSpinEdit2.Value;
end;

procedure TabcdfLineEdit.lnAHBothMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if Sender is TabcLine then begin
    if Sender <> FHLine then begin
      if FHLine <> nil then
        FHLine.Color := clBlue;
      FHLine := TabcLine(Sender);
      FHLine.Color := clRed;
    end;
  end
  else begin
    if FHLine <> nil then begin
      FHLine.Color := clBlue;
      FHLine := nil;
    end;
  end;
end;

procedure TabcdfLineEdit.abcSpinEdit1Change(Sender: TObject);
begin
  lnArrSize.ArrowSize := abcSpinEdit1.Value;
  FUpdateLine.ArrowSize := abcSpinEdit1.Value;
end;

procedure TabcdfLineEdit.lnAHBothClick(Sender: TObject);
begin
  if Sender is TabcLine then
    with TabcLine(Sender) do begin
      FUpdateLine.Arrows := Arrows;
      fraAH.SetBounds(Left-10, Top, Width+20, Height);
    end;
end;

procedure TabcdfLineEdit.lnASSolidClick(Sender: TObject);
begin
  if Sender is TabcLine then
    with TabcLine(Sender) do begin
      FUpdateLine.ArrowStyle := ArrowStyle;
      fraAS.SetBounds(Left-10, Top, Width+20, Height);
    end;
end;

procedure TabcdfLineEdit.lnDIHorzClick(Sender: TObject);
begin
  if Sender is TabcLine then
    with TabcLine(Sender) do begin
      FUpdateLine.Direction := Direction;
      fraDI.SetBounds(Left-10, Top-10, Width+20, Height+20);
    end;
end;

procedure TabcdfLineEdit.lnSTSolidClick(Sender: TObject);
begin
  if Sender is TabcLine then
    with TabcLine(Sender) do begin
      FUpdateLine.LineStyle := LineStyle;
      fraST.SetBounds(Left-10, Top, Width+20, Height);
    end;
end;

end.
