unit abcdfmm;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses SysUtils, Windows, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Dialogs, abcmm, abchandl, abcmem, mplayer;

type
  TabcdfMCIStreamEdit = class(TForm)
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    Bevel1: TBevel;
    btnLoad: TButton;
    btnSave: TButton;
    btnClear: TButton;
    btnPlay: TButton;
    imgSound: TImage;
    lblStatus: TLabel;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    MCIStreamPlayer: TabcMCIStreamPlayer;
    cboDeviceType: TComboBox;
    Label1: TLabel;
    btnStop: TButton;
    procedure btnLoadClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure cboDeviceTypeChange(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
  private
    { Private declarations }
    procedure CheckStatus;
  public
    { Public declarations }
  end;

function ExecabcdfMCIStreamEdit(AStreamPlayer: TabcMCIStreamPlayer): Boolean;

implementation

uses consts;

{$R *.DFM}

function ExecabcdfMCIStreamEdit(AStreamPlayer: TabcMCIStreamPlayer): Boolean;
begin
  Result := False;
  with TabcdfMCIStreamEdit.Create(nil) do
  try
    MCIStreamPlayer.MCIData := AStreamPlayer.MCIData;
    MCIStreamPlayer.DeviceType := AStreamPlayer.DeviceType;
    if not ((MCIStreamPlayer.MCIData.Empty) or
            (MCIStreamPlayer.DeviceType = dtAutoSelect)) then
      MCIStreamPlayer.Open;
    cboDeviceType.ItemIndex := Ord(MCIStreamPlayer.DeviceType) - 1;
    OpenDialog.Filter := SMPOpenFilter;
    SaveDialog.Filter := SMPOpenFilter;
    CheckStatus;
    if (ShowModal = mrOK) then begin
      AStreamPlayer.MCIData.Assign(MCIStreamPlayer.MCIData);
      {set type also}
      AStreamPlayer.DeviceType := MCIStreamPlayer.DeviceType;
      Result := True;
    end;
    MCIStreamPlayer.Close;
  finally
    Free;
  end;
end;

procedure TabcdfMCIStreamEdit.CheckStatus;
begin
  if MCIStreamPlayer.MCIData.Empty then begin
    if Visible then btnOK.SetFocus;
    btnSave.Enabled := False;
    btnClear.Enabled := False;
    btnPlay.Enabled := False;
    btnStop.Enabled := False;
    lblStatus.Visible := False;
    imgSound.Visible := False;
  end
  else begin
    btnSave.Enabled := True;
    btnClear.Enabled := True;
    btnPlay.Enabled := True;
    lblStatus.Caption := InttoStr(MCIStreamPlayer.MCIData.Size) + ' Bytes';
    lblStatus.Visible := True;
    imgSound.Visible := True;
  end;
end;

procedure TabcdfMCIStreamEdit.btnLoadClick(Sender: TObject);
var
  FileExt: string;
begin
  if OpenDialog.Execute then begin
    FileExt := UpperCase(ExtractFileExt(OpenDialog.FileName));
    if FileExt = '.WAV' then
      cboDeviceType.ItemIndex := 11
    else if (FileExt = '.MID') or (FileExt = '.RMI') then
      cboDeviceType.ItemIndex := 8
    else if (FileExt = '.AVI') then
      cboDeviceType.ItemIndex := 0
    else begin
      cboDeviceType.ItemIndex := -1;
      ShowMessage('Please select device type');
    end;
    cboDeviceTypeChange(Self);
    MCIStreamPlayer.Close;
    MCIStreamPlayer.MCIData.LoadFromFile(OpenDialog.FileName);
    if cboDeviceType.ItemIndex <> -1 then
      MCIStreamPlayer.Open;
    CheckStatus;
  end;
end;

procedure TabcdfMCIStreamEdit.btnSaveClick(Sender: TObject);
begin
  if SaveDialog.Execute then begin
    MCIStreamPlayer.MCIData.SaveToFile(SaveDialog.FileName);
  end;
end;

procedure TabcdfMCIStreamEdit.btnClearClick(Sender: TObject);
begin
  MCIStreamPlayer.MCIData.Clear;
  CheckStatus;
end;

procedure TabcdfMCIStreamEdit.btnPlayClick(Sender: TObject);
begin
  if cboDeviceType.ItemIndex = -1 then
    ShowMessage('Please select device type')
  else begin
    MCIStreamPlayer.Close;
    MCIStreamPlayer.Open;
    MCIStreamPlayer.Play;
    btnStop.Enabled := True;
  end;
end;

procedure TabcdfMCIStreamEdit.cboDeviceTypeChange(Sender: TObject);
begin
  with MCIStreamPlayer, cboDeviceType do
  case ItemIndex of
    0:  DeviceType := dtAVIVideo;
    1:  DeviceType := dtCDAudio;
    2:  DeviceType := dtDAT;
    3:  DeviceType := dtDigitalVideo;
    4:  DeviceType := dtMMMovie;
    5:  DeviceType := dtOther;
    6:  DeviceType := dtOverlay;
    7:  DeviceType := dtScanner;
    8:  DeviceType := dtSequencer;
    9:  DeviceType := dtVCR;
    10: DeviceType := dtVideoDisc;
    11: DeviceType := dtWaveAudio;
  end;
  if cboDeviceType.ItemIndex <> -1 then
    MCIStreamPlayer.Open;
  CheckStatus;
end;

procedure TabcdfMCIStreamEdit.btnStopClick(Sender: TObject);
begin
  MCIStreamPlayer.Stop;
  btnPlay.Enabled := True;
  btnStop.Enabled := False;
end;

end.
