unit abcdfnav;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{ Dialog for ABC Navigator Panel component editor }

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, Grids, Outline, abcbtn,
  abcspin, INIFiles, abcnp, abcexctl;

type

  TabcdfNavPanelEdit = class(TForm)
    Notebook: TNotebook;
    btnNext: TBitBtn;
    btnPrev: TBitBtn;
    btnCancel: TBitBtn;
    lblPageHint: TLabel;
    Image1: TImage;
    lstSelected: TListBox;
    outButtons: TOutline;
    Label1: TLabel;
    Label2: TLabel;
    btnAdd: TButton;
    btnAddAll: TButton;
    btnRemove: TButton;
    btnRemoveAll: TButton;
    btnUp: TBitBtn;
    btnDown: TBitBtn;
    rgGlyphStyles: TRadioGroup;
    chkCaption: TCheckBox;
    rgCaptionTextStyle: TRadioGroup;
    rgGlyphPos: TRadioGroup;
    btnButtonFont: TButton;
    FontDialog1: TFontDialog;
    rgBtnSize: TRadioGroup;
    grpCustomSize: TGroupBox;
    lblBtnWidth: TLabel;
    speBtnWidth: TabcSpinEdit;
    lblBtnHeight: TLabel;
    speBtnHeight: TabcSpinEdit;
    rgGlyphOrient: TRadioGroup;
    GroupBox1: TGroupBox;
    btnGlyphSample: TabcPicSpeedBtn;
    chkHints: TCheckBox;
    BtnColor: TBitBtn;
    ColorDialog1: TColorDialog;
    grdHints: TStringGrid;
    grdCaptions: TStringGrid;
    GroupBox2: TGroupBox;
    btnCaptionSample: TabcPicSpeedBtn;
    grpSizePreview: TGroupBox;
    Bevel1: TBevel;
    pnlSample: TPanel;
    btnSizeSample: TabcPicSpeedBtn;
    Label5: TLabel;
    speBtnMargin: TabcSpinEdit;
    Label7: TLabel;
    cboBtnStyle: TComboBox;
    grpPanelMargin: TGroupBox;
    Label4: TLabel;
    spePanelMarginH: TabcSpinEdit;
    Label6: TLabel;
    spePanelMarginV: TabcSpinEdit;
    grpBtnAlignment: TGroupBox;
    Label9: TLabel;
    speBtnOffsetH: TabcSpinEdit;
    Label10: TLabel;
    speBtnOffsetV: TabcSpinEdit;
    chkPopup: TCheckBox;
    Label3: TLabel;
    cboBtnPicHighlight: TComboBox;
    speBtnRows: TabcSpinEdit;
    speBtnCols: TabcSpinEdit;
    lblAlignRows: TLabel;
    lblAlignCols: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure CancelClick(Sender: TObject);
    procedure PrevClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    procedure rgBtnSizeClick(Sender: TObject);
    procedure speBtnWidthChange(Sender: TObject);
    procedure btnRemoveAllClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnAddAllClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure rgCaptionTextStyleClick(Sender: TObject);
    procedure rgGlyphPosClick(Sender: TObject);
    procedure btnButtonFontClick(Sender: TObject);
    procedure GlyphSampleChange(Sender: TObject);
    procedure BtnColorClick(Sender: TObject);
    procedure chkCaptionClick(Sender: TObject);
    procedure spePanelMarginChange(Sender: TObject);
    procedure outButtonsDblClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure speBtnMarginChange(Sender: TObject);
    procedure cboBtnStyleClick(Sender: TObject);
    procedure speBtnRowsChange(Sender: TObject);
    procedure speBtnColsChange(Sender: TObject);
    procedure chkPopupClick(Sender: TObject);
    procedure cboBtnPicHighlightClick(Sender: TObject);
  private
    { Private declarations }
    FNewControlCount: Integer;
    FButtonCount: Integer;
    FCtrlCols: Integer;
    FCtrlRows: Integer;
    FByExactCols: Boolean;
    FINIFile: TINIFile;
    FInitSizing: Boolean;
    FPanel: TabcNavPanel;
    FToolbar: TabcNavToolbar;
    FRowColChanging: Boolean;
    procedure CenterSizeSample(AWidth, AHeight: Integer);
    procedure CreateButtons;
    procedure CreateButton(Index: Integer);
    function GetColCount(var Items, Rows: Integer): Integer;
    function GetRowCount(var Items, Cols: Integer): Integer;
    procedure InitOutline;
    procedure InitCaptionSample;
    procedure InitDefaultCaptions;
    procedure InitDefaultHints;
    procedure InitButtonSize;
    procedure InitNavGlyphSample;
    procedure RefreshButtons;
    procedure AdjustAllButtons;
    function GetINIFileBtnInfo(Ident: string; Vert: Boolean;
      var BtnFileName: TFileName; var BtnNumGlyphs: Integer): Boolean;
    procedure SetButtonGlyph(AControl: TControl);
    procedure LoadBitmapRes(AButton: TabcCustomPicSpeedBtn; AResName: string);
  public
    { Public declarations }
  end;

function ExecabcdfNavPanelEdit(Win: TWinControl): Boolean;

var
  abcdfNavPanelEdit: TabcdfNavPanelEdit;

implementation

{$R *.DFM}

uses abcdsncn, abcutilm, abcnpbtn, abcnpedt, abcnpflt, abcnpinf;

const

  ABCINIFileName = 'abc_4_d.ini';

  { page numbers }
  pgButtons      = 0;
  pgGlyphs       = 1;
  pgCaptionStyle = 2;
  pgButtonSize   = 3;
  pgCaptions     = 4;
  pgHints        = 5;

function ExecabcdfNavPanelEdit(Win: TWinControl): Boolean;

  procedure EnableControls(Win: TWinControl; Value: Boolean);
  var
    i: integer;
  begin
    Win.Enabled := Value;
    with Win do
      for i := 0 to ControlCount - 1 do
        Controls[i].Enabled := Value;
  end;

begin
  Result := False;
  with TabcdfNavPanelEdit.Create(nil) do
  try
    if Win is TabcNavToolbar then
    begin
      FToolbar := TabcNavToolbar(Win);
      EnableControls(grpBtnAlignment, False);
      EnableControls(grpPanelMargin, False);
    end
    else
    begin
      FToolbar := nil;
      EnableControls(grpBtnAlignment, True);
      EnableControls(grpPanelMargin, True);
    end;
    FPanel := TabcNavPanel(Win);
    if ShowModal = mrOK then
    begin
      CreateButtons;
      Result := True;
    end;
  finally
    Free;
  end;
end;

{ TabcsfNavPanelEdit }

type
  TPicSpeedBtnAccess = class(TabcCustomPicSpeedBtn);

procedure TabcdfNavPanelEdit.LoadBitmapRes(AButton: TabcCustomPicSpeedBtn; AResName: string);
begin
  with TPicSpeedBtnAccess(AButton) do
  begin
    Picture.Bitmap.LoadFromResourceName(HInstance, AResName);
    Picture.Bitmap.PixelFormat := pf4Bit;
    NumGlyphs := 2;
  end;
end;

procedure TabcdfNavPanelEdit.CreateButtons;
var
  I: integer;
begin
  if FToolbar <> nil then FToolbar.BeginUpdate;
  try
    for I := 0 to lstSelected.Items.Count - 1 do
      CreateButton(I);
    AdjustAllButtons;
    if chkHints.Checked then FPanel.ShowHint := True;
  finally
    if FToolbar <> nil then FToolbar.EndUpdate;
  end;
  //need to call designer.modified again?
end;

procedure TabcdfNavPanelEdit.AdjustAllButtons;
{
Resizes panel and aligns buttons and other controls in rows/cols.
Order depends on horz or vert layout.
Horz = layout row then col.
Vert = layout col then row.
}

var
  I,
  BtnWidth, BtnHeight, BtnOffsetH, BtnOffsetV,
  CurrCol, CurrRow, CurrLeft, CurrTop,
  CtrlColWidth, CtrlRowHeight,
  PanelMarginH, PanelMarginV,
  PanelWidth, PanelHeight: integer;
  ButtonList: TList;
  ControlList: TList;

  procedure SetButtonProps(Button: TabcPicSpeedBtn);
  begin
    with Button do
    begin
      if rgGlyphStyles.ItemIndex <> 7 then
        SetButtonGlyph(TControl(ButtonList.Items[I]));
      Color := btnGlyphSample.Color;
      Style := btnGlyphSample.Style;
      PictureHighlight := btnGlyphSample.PictureHighlight;
      Popup := btnGlyphSample.Popup;
      if chkCaption.Checked then
      begin
        Font := btnCaptionSample.Font;
        Layout := btnCaptionSample.Layout;
        TextStyle := btnCaptionSample.TextStyle;
        Margin := btnCaptionSample.Margin;
      end
      else
        Margin := -1;
    end;
  end;

begin
  BtnOffsetH := speBtnOffsetH.Value;
  BtnOffsetV := speBtnOffsetV.Value;
  PanelMarginH := spePanelMarginH.Value;
  PanelMarginV := spePanelMarginV.Value;
  CurrCol := 0;
  CurrRow := 0;
  CurrLeft := PanelMarginH;
  CurrTop := PanelMarginV;
  CtrlRowHeight := 0;
  CtrlColWidth := 0;
  BtnWidth := btnSizeSample.Width;
  BtnHeight := btnSizeSample.Height;
  ButtonList := TList.Create;
  try
    ControlList := TList.Create;
    try
      with FPanel do begin
        for I := 0 to ControlCount - 1 do
          if Controls[I] is TabcCustomPicSpeedBtn then
            ButtonList.Add(Controls[I])
          else
            ControlList.Add(Controls[I]);

        if rgGlyphOrient.ItemIndex = 0 then
        begin
          {Lay out buttons in rows}
          PanelWidth := (BtnWidth + BtnOffsetH) * FCtrlCols + 2 * PanelMarginH - BtnOffsetH;
          for I := 0 to ButtonList.Count - 1 do
            with TabcPicSpeedBtn(ButtonList.Items[I]) do
            begin
              SetButtonProps(TabcPicSpeedBtn(ButtonList.Items[I]));
              SetBounds(CurrLeft, CurrTop, BtnWidth, BtnHeight);
              Inc(CurrCol);
              if CurrCol = FCtrlCols then
              begin
                CurrCol := 0;
                CurrTop := CurrTop + BtnHeight + BtnOffsetV;
                CurrLeft := PanelMarginH;
              end
              else
                Inc(CurrLeft, BtnWidth + BtnOffsetH);
            end;

          if ButtonList.Count > 0 then
            if ControlList.Count > 0 then
              CurrTop := CurrTop + 1 - BtnOffsetV
            else
              CurrTop := CurrTop + PanelMarginV;

          if CurrLeft <> PanelMarginH then
            CurrTop := CurrTop + BtnHeight;
          CurrLeft := PanelMarginH + 1;

          for I := 0 to ControlList.Count - 1 do
            with TControl(ControlList.Items[I]) do
            begin
              if CurrLeft <> PanelMarginH + 1 then
              begin
                if PanelWidth < (CurrLeft + PanelMarginH + Width + 1) then
                begin
                  CurrLeft := PanelMarginH + 1;
                  CurrTop := CurrTop + CtrlRowHeight + 1;
                  CtrlRowHeight := 0;
                end
              end
              else if PanelWidth < (Width + 2 * PanelMarginH + 2) then
                PanelWidth := Width + 2 * PanelMarginH + 2;
              SetBounds(CurrLeft, CurrTop, Width, Height);
              CurrLeft := CurrLeft + Width + 2;
              if CtrlRowHeight < Height then CtrlRowHeight := Height;
            end;
          if ControlList.Count = 0 then
            PanelHeight := CurrTop
          else
            PanelHeight := CurrTop + CtrlRowHeight + PanelMarginV + 1;
        end
        else
        begin
          {Lay out buttons in cols}
          PanelHeight := (BtnHeight + BtnOffsetV) * FCtrlRows + 2 * PanelMarginV - BtnOffsetV;
          for I := 0 to ButtonList.Count - 1 do
            with TabcPicSpeedBtn(ButtonList.Items[I]) do
            begin
              SetButtonProps(TabcPicSpeedBtn(ButtonList.Items[I]));
              SetBounds(CurrLeft, CurrTop, BtnWidth, BtnHeight);
              Inc(CurrRow);
              if CurrRow = FCtrlRows then
              begin
                CurrRow := 0;
                CurrLeft := CurrLeft + BtnWidth + BtnOffsetH;
                CurrTop := PanelMarginV;
              end
              else
                Inc(CurrTop, BtnHeight + BtnOffsetV);
            end;
          if ButtonList.Count > 0 then
            if ControlList.Count > 0 then
              CurrLeft := CurrLeft + 1 - BtnOffsetH
            else
              CurrLeft := CurrLeft + PanelMarginH;
          if CurrTop <> PanelMarginV then
            CurrLeft := CurrLeft + BtnWidth;
          CurrTop := PanelMarginV + 1;

          for I := 0 to ControlList.Count - 1 do
            with TControl(ControlList.Items[I]) do
            begin
              if CurrTop <> PanelMarginV + 1 then
              begin
                if PanelHeight < (CurrTop + PanelMarginV + Height + 1) then
                begin
                  CurrTop := PanelMarginV + 1;
                  CurrLeft := CurrLeft + CtrlColWidth + 1;
                  CtrlColWidth := 0;
                end
              end
              else if PanelHeight < (Height + 2 * PanelMarginV + 2) then
                PanelHeight := Height + 2 * PanelMarginV + 2;
              SetBounds(CurrLeft, CurrTop, Width, Height);
              CurrTop := CurrTop + Height + 2;
              if CtrlColWidth < Width then CtrlColWidth := Width;
            end;
          if ControlList.Count = 0 then
            PanelWidth := CurrLeft
          else
            PanelWidth := CurrLeft + CtrlColWidth + PanelMarginH + 1;
        end;
        SetBounds(Left, Top, PanelWidth, PanelHeight);
      end;
    finally
      ControlList.Free;
    end;
  finally
    ButtonList.Free;
  end;
end;

function TabcdfNavPanelEdit.GetINIFileBtnInfo(Ident: string; Vert: Boolean;
  var BtnFileName: TFileName; var BtnNumGlyphs: Integer): Boolean;
const
  Section = 'NavPanel Custom Buttons';
var
  FindIdent: string;
  FoundPath: string;
begin
  Result := False;
  if Vert then
    FindIdent := Ident + 'Vert'
  else
    FindIdent := Ident;
  with FINIFile do
  begin
    FoundPath := ReadString(Section, 'Path', '');
    BtnFileName := ReadString(Section, FindIdent, '');
    if (BtnFileName = '') and Vert then
      BtnFileName := ReadString(Section, Ident, '');
    if BtnFileName = '' then
      Exit
    else
    begin
      BtnFileName := FoundPath + '\' + BtnFileName;
      if FileExists(BtnFileName) then
        Result := True
      else begin
        BtnFileName := '';
        Exit;
      end;
    end;
    BtnNumGlyphs := ReadInteger(Section, Ident + 'NumGlyphs', 2);
  end;
end;

procedure TabcdfNavPanelEdit.SetButtonGlyph(AControl: TControl);
var
  BmpResName: string;
  ArrayIndex: Integer;
  INIIdent: string;
  INIGlyphFileName: TFileName;
  ININumGlyphs: Integer;
begin
  if rgGlyphStyles.ItemIndex = 6 then
  begin
    {Custom Button}
    if AControl is TabcToggleBtn then
      INIIdent := 'Toggle'
    else if AControl is TabcPicSpeedBtn then
      INIIdent := 'Custom'
    else
      INIIdent := Copy(AControl.ClassName, 2, 63);
    if (AControl is TabcCustomPicSpeedBtn) and
        GetINIFileBtnInfo(INIIdent,
                         (rgGlyphOrient.ItemIndex = 1),
                         INIGlyphFileName,
                         ININumGlyphs) then
      with TPicSpeedBtnAccess(AControl) do
      begin
        Picture.LoadFromFile(INIGlyphFileName);
        NumGlyphs := ININumGlyphs;
      end;
  end
  else
  begin
    ArrayIndex := 0;
    while not (AControl is ABCDesignNavComponentClass(ArrayIndex)) do
    begin
      Inc(ArrayIndex);
      if ArrayIndex > ABC_DESIGNER_BTN_COUNT - 1 then Exit;
    end;
    if rgGlyphOrient.ItemIndex = 0 then
      BmpResName := ABCDesignNavBmpResName(ArrayIndex)
    else
      BmpResName := ABCDesignNavVertBmpResName(ArrayIndex);
    if BmpResName = '' then Exit;

    {set glyph}
    BmpResName := 'ABCNP_' + BmpResName + IntToStr(rgGlyphStyles.ItemIndex);
    LoadBitmapRes(TabcCustomPicSpeedBtn(AControl), BmpResName);
  end;
end;

procedure TabcdfNavPanelEdit.CreateButton(Index: Integer);
var
  Btn: TControl;
  BtnClass: TComponentClass;
  NewNamePrefix, NewName: string;
  NewNameCount: Integer;
  ArrayIndex: Integer;
begin
  ArrayIndex := Word(lstSelected.Items.Objects[Index]);
  BtnClass := ABCDesignNavComponentClass(ArrayIndex);
  if BtnClass = nil then Exit;
  {Create and name the control}
  Btn := TControl(BtnClass.Create(FPanel.Owner));
  {get valid name - NB This changes for Delphi 2}
  NewNamePrefix := Copy(Btn.ClassName, 2, 255);
  NewNameCount := 0;
  repeat
    Inc(NewNameCount);
    NewName := NewNamePrefix + IntToStr(NewNameCount);
  until (FPanel.Owner.FindComponent(NewName) = nil);
  Btn.Name := NewName;

  {class specific additional properties}
  if Btn is TabcCustomnpBtn then
    with TabcCustomnpBtn(Btn) do begin
      Parent := FPanel;
      {set Caption}
      if chkCaption.Checked then
        Caption := grdCaptions.Cols[1].Strings[Index + 1];
      if chkHints.Checked then
        {set Hint or HintUp and HintDown}
        if (Btn is TabcnpBookmark) then
          with TabcnpBookmark(Btn) do begin
            HintUp := grdHints.Cols[1].Strings[Index + 1];
            HintDown := grdHints.Cols[2].Strings[Index + 1];
          end
        else if (Btn is TabcnpFilterExpression) then
          with TabcnpFilterExpression(Btn) do begin
            HintUp := grdHints.Cols[1].Strings[Index + 1];
            HintDown := grdHints.Cols[2].Strings[Index + 1];
          end
        else if (Btn is TabcnpFilterActive) then
          with TabcnpFilterActive(Btn) do begin
            HintUp := grdHints.Cols[1].Strings[Index + 1];
            HintDown := grdHints.Cols[2].Strings[Index + 1];
          end
        else
          Hint := grdHints.Cols[1].Strings[Index + 1];
      if (Btn is TabcnpBlobEdit) or
         (Btn is TabcnpFilterActive) or
         (Btn is TabcnpFindReplace) then
        try
          FPanel.AutoField := True;
        except
        end;
    end
  else if Btn is TabcnpIndexCombo then
    with TabcnpIndexCombo(Btn) do begin
      Parent := FPanel;
      {set Hint}
      if chkHints.Checked then
        Hint := grdHints.Cols[1].Strings[Index + 1];
      if FPanel.Datasource <> nil then
        try
          LoadItemsFromDataSet;
        except
        end;
    end
  else if Btn is TabcToggleBtn then
    with TabcToggleBtn(Btn) do begin
      Parent := FPanel;
      {set Caption}
      if chkCaption.Checked then
        Caption := grdCaptions.Cols[1].Strings[Index + 1];
      {set Hint}
      if chkHints.Checked then begin
        HintUp := grdHints.Cols[1].Strings[Index + 1];
        HintDown := grdHints.Cols[2].Strings[Index + 1];
      end;
      {custom button glyph}
      if rgGlyphStyles.ItemIndex = 6 then
        SetButtonGlyph(Self);
    end
  else if Btn is TabcPicSpeedBtn then
    with TabcPicSpeedBtn(Btn) do begin
      Parent := FPanel;
      {set Caption}
      if chkCaption.Checked then
        Caption := grdCaptions.Cols[1].Strings[Index + 1];
      {set Hint}
      if chkHints.Checked then
        Hint := grdHints.Cols[1].Strings[Index + 1];
      {custom button glyph}
      if rgGlyphStyles.ItemIndex = 6 then
        SetButtonGlyph(Self);
    end
  else if (Btn is TabcnpRangeEdit) then
    with TabcnpRangeEdit(Btn) do begin
      Parent := FPanel;
      {set Hint}
      if chkHints.Checked then
        Hint := grdHints.Cols[1].Strings[Index + 1];
    end
  else if (Btn is TabcnpState) then
    TabcnpState(Btn).Parent := FPanel
  else if (Btn is TabcnpRecordCount) then
    TabcnpRecordCount(Btn).Parent := FPanel
  else
    Parent := FPanel;

(*
  if Btn <> nil then
    if (csDesigning in Btn.ComponentState) then
      if (Btn.Owner is TForm) and (TForm(Btn.Owner).Designer <> nil) then
        TForm(Btn.Owner).Designer.Modified;
*)

end;

procedure TabcdfNavPanelEdit.InitOutline;
{
  Load strings in outline page 1
  Also store the item's array index in the outline?
}
var
  I: integer;
  ParentIndex: Longint;
begin
  ParentIndex := 0;
  for I := 0 to ABC_DESIGNER_BTN_COUNT - 1 do
  with OutButtons do begin
    case I of
    0:
      ParentIndex := Add(0, sabcnpdsgnListGroup1);
    6:
      ParentIndex := Add(0, sabcnpdsgnListGroup2);
    18:
      ParentIndex := Add(0, sabcnpdsgnListGroup3);
    21:
      ParentIndex := Add(0, sabcnpdsgnListGroup4);
    26:
      ParentIndex := Add(0, sabcnpdsgnListGroup5);
    29:
      ParentIndex := Add(0, sabcnpdsgnListGroup6);
    end;
    AddChildObject(ParentIndex, ABCDesignNavBtnListName(I), Pointer(I));
  end;
end;

procedure TabcdfNavPanelEdit.InitDefaultHints;
var
  I, CurrRow, ArrayIndex: Integer;
  DefaultHint, DefaultHintDown: String;
begin
  with grdHints do begin
    RowCount := 0; {clear cells}
    RowCount := lstSelected.Items.Count + 1;
    FixedRows := 1;
    Cells[0,0] := 'BUTTON';
    Cells[1,0] := 'HINT or HINT UP';
    Cells[2,0] := 'HINT DOWN';
    CurrRow := 0;
    for I := 0 to lstSelected.Items.Count - 1 do begin
      Inc(CurrRow);
      ArrayIndex := Word(lstSelected.Items.Objects[I]);
      DefaultHint := ABCDesignNavComponentHint(ArrayIndex);
      DefaultHintDown := ABCDesignNavComponentHintDown(ArrayIndex);
      Cells[0,CurrRow] := lstSelected.Items.Strings[I];
      Cells[1,CurrRow] := DefaultHint;
      Cells[2,CurrRow] := DefaultHintDown;
    end;
  end;
end;

procedure TabcdfNavPanelEdit.InitCaptionSample;
begin
  with btnCaptionSample do begin
    Picture := btnGlyphSample.Picture;
    NumGlyphs := 2;
    Color := btnGlyphSample.Color;
    Style := btnGlyphSample.Style;
    PictureHighlight := btnGlyphSample.PictureHighlight;
    Popup := btnGlyphSample.Popup;
  end;
end;

procedure TabcdfNavPanelEdit.InitDefaultCaptions;
var
  I, CurrRow, ArrayIndex: Integer;
  DefaultCaption: String;
begin
  with grdCaptions do begin
    RowCount := 0; {clear cells}
    RowCount := lstSelected.Items.Count + 1;
    FixedRows := 1;
    ColWidths[1] := Width - ColWidths[0];
    Cells[0,0] := 'BUTTON';
    Cells[1,0] := 'CAPTION';
    CurrRow := 0;
    for I := 0 to lstSelected.Items.Count - 1 do begin
      Inc(CurrRow);
      ArrayIndex := Word(lstSelected.Items.Objects[I]);
      DefaultCaption := ABCDesignNavComponentCaption(ArrayIndex);
      Cells[0,CurrRow] := lstSelected.Items.Strings[I];
      Cells[1,CurrRow] := DefaultCaption;
    end;
  end;
end;

procedure TabcdfNavPanelEdit.InitButtonSize;
var
  I: Integer;
  CompClass: TComponentClass;
begin
  FInitSizing := True;

  FButtonCount := 0;
  for I := 0 to FPanel.ControlCount - 1 do
    if FPanel.Controls[I] is TabcCustomPicSpeedBtn then
      Inc(FButtonCount);
  for I := 0 to lstSelected.Items.Count - 1 do
    {find the button classes}
  begin
    CompClass := ABCDesignNavComponentClass(Word(lstSelected.Items.Objects[I]));
    if (CompClass <> nil) and CompClass.InheritsFrom(TabcCustomPicSpeedBtn) then
      Inc(FButtonCount);
  end;
  if FButtonCount < 2 then begin
    speBtnRows.MinValue := FButtonCount;
    speBtnCols.MinValue := FButtonCount;
    speBtnRows.Value := FButtonCount;
    speBtnCols.Value := FButtonCount;
    speBtnRows.Enabled := False;
    speBtnCols.Enabled := False;
    lblAlignRows.Enabled := False;
    lblAlignCols.Enabled := False;
  end
  else begin
    speBtnRows.MinValue := 1;
    speBtnCols.MinValue := 1;
    speBtnRows.MaxValue := FButtonCount;
    speBtnCols.MaxValue := FButtonCount;
    speBtnRows.Enabled := True;
    speBtnCols.Enabled := True;
    lblAlignRows.Enabled := True;
    lblAlignCols.Enabled := True;
    FRowColChanging := True;
    if rgGlyphOrient.ItemIndex = 0 then begin
      {Horz}
      speBtnRows.Value := 1;
      speBtnCols.Value := FButtonCount;
    end
    else begin
      {Vert}
      speBtnRows.Value := FButtonCount;
      speBtnCols.Value := 1;
    end;
    FCtrlRows := speBtnRows.Value;
    FCtrlCols := speBtnCols.Value;
    FRowColChanging := False;
  end;
  if chkPopup.Checked then begin
    speBtnOffsetH.Value := 0;
    speBtnOffsetV.Value := 0;
  end
  else begin
    speBtnOffsetH.Value := -1;
    speBtnOffsetV.Value := -1;
  end;

  spePanelMarginH.Value := 1;
  spePanelMarginV.Value := 1;

  {set up size sample button}
  with btnSizeSample do begin
    Picture := btnGlyphSample.Picture;
    NumGlyphs := 2;
    Color := btnGlyphSample.Color;
    Style := btnGlyphSample.Style;
    PictureHighlight := btnGlyphSample.PictureHighlight;
    Popup := btnGlyphSample.Popup;

    if chkCaption.Checked then begin
      Caption := 'Caption';
      Font := btnCaptionSample.Font;
      Layout := btnCaptionSample.Layout;
      Margin := btnCaptionSample.Margin;
      TextStyle := btnCaptionSample.TextStyle;
      rgBtnSize.ItemIndex := 6;
      CenterSizeSample(50, 50);
    end
    else begin
      Caption := '';
      Layout := btnGlyphSample.Layout;
      Margin := -1;
      TextStyle := btnGlyphSample.TextStyle;
      rgBtnSize.ItemIndex := 2;
      CenterSizeSample(23, 22);
    end;
  end;
  FInitSizing := False;
end;

procedure TabcdfNavPanelEdit.FormCreate(Sender: TObject);
begin
  FINIFile := TINIFile.Create(ABCINIFileName);
  InitOutline;
  NoteBook.PageIndex := pgButtons;
  PrevClick(Self);
  RefreshButtons;
  InitNavGlyphSample;
end;

procedure TabcdfNavPanelEdit.FormDestroy(Sender: TObject);
begin
  FINIFile.Free;
end;

procedure TabcdfNavPanelEdit.InitNavGlyphSample;
//var
//  tempbmp: TBitmap;
begin
  LoadBitmapRes(btnGlyphSample, 'ABCNP_FIRST2');
(*
  tempbmp := TBitmap.Create;
  try
    tempbmp.Handle := LoadBitmap(HInstance, 'ABCNP_FIRST2');
    with btnGlyphSample do begin
      Picture.Graphic := tempbmp;
      NumGlyphs := 2;
    end;
  finally
    tempbmp.free;
  end;
*)

  cboBtnStyle.ItemIndex := 3;
  cboBtnStyleClick(Self);
end;

procedure TabcdfNavPanelEdit.CancelClick(Sender: TObject);
begin
  Close;
end;

procedure TabcdfNavPanelEdit.PrevClick(Sender: TObject);
begin
  case NoteBook.PageIndex of
    pgButtons:
      Exit;
    pgGlyphs:
      NoteBook.PageIndex := pgButtons;
    pgCaptionStyle:
      NoteBook.PageIndex := pgGlyphs;
    pgButtonSize:
      if chkCaption.Checked then
        NoteBook.PageIndex := pgCaptionStyle
      else
        NoteBook.PageIndex := pgGlyphs;
    pgCaptions:
      NoteBook.PageIndex := pgButtonSize;
    pgHints:
      if chkCaption.Checked then
        NoteBook.PageIndex := pgCaptions
      else
        NoteBook.PageIndex := pgButtonSize;
  end;
  RefreshButtons;
end;

procedure TabcdfNavPanelEdit.NextClick(Sender: TObject);
begin
  case NoteBook.PageIndex of
    pgButtons:
      begin
        FNewControlCount := lstSelected.Items.Count;
        NoteBook.PageIndex := pgGlyphs;
      end;
    pgGlyphs:
      if chkCaption.Checked then begin
        InitCaptionSample;
        NoteBook.PageIndex := pgCaptionStyle;
      end
      else begin
        InitButtonSize;
        NoteBook.PageIndex := pgButtonSize;
      end;
    pgCaptionStyle:
      begin
        InitButtonSize;
        NoteBook.PageIndex := pgButtonSize;
      end;
    pgButtonSize:
      if FNewControlCount > 0 then
        if chkCaption.Checked then begin
          InitDefaultCaptions;
          NoteBook.PageIndex := pgCaptions;
        end
        else if chkHints.Checked then begin
          InitDefaultHints;
          NoteBook.PageIndex := pgHints;
        end
        else begin
          ModalResult := mrOK;
          Exit;
        end
      else begin
        ModalResult := mrOK;
        Exit;
      end;
    pgCaptions:
      if chkHints.Checked then begin
        InitDefaultHints;
        NoteBook.PageIndex := pgHints;
      end
      else begin
        ModalResult := mrOK;
        Exit;
      end;
    pgHints:
      begin
        ModalResult := mrOK;
        Exit;
      end;
  end;
  RefreshButtons;
end;

procedure TabcdfNavPanelEdit.RefreshButtons;
begin
  case NoteBook.PageIndex of
    pgButtons:
      lblPageHint.Caption := sabcnpdsgnButtons;
    pgGlyphs:
      lblPageHint.Caption := sabcnpdsgnGlyphs;
    pgHints:
      lblPageHint.Caption := sabcnpdsgnHints;
    pgCaptions:
      lblPageHint.Caption := sabcnpdsgnCaptions;
    pgCaptionStyle:
      lblPageHint.Caption := sabcnpdsgnCaptionStyle;
    pgButtonSize:
      lblPageHint.Caption := sabcnpdsgnButtonSize;
  end;

  btnPrev.Enabled := Notebook.PageIndex > 0;
  if (Notebook.PageIndex = pgHints) or
     ((Notebook.PageIndex = pgCaptions) and not chkHints.Checked) or
     ((Notebook.PageIndex = pgButtonSize) and
      ((FNewControlCount = 0) or not
      (chkCaption.Checked or chkHints.Checked))) then
  begin
    btnNext.Caption := '&Create';
    btnNext.Glyph.Handle := LoadBitmap(HInstance, 'ABCNPDSGN_CREATE');
    btnNext.NumGlyphs := 1;
  end
  else
  begin
    btnNext.Caption := '&Next';
    btnNext.Glyph.Handle := LoadBitmap(HInstance, 'ABCNPDSGN_NEXT');
    btnNext.NumGlyphs := 2;
  end;
end;

procedure TabcdfNavPanelEdit.CenterSizeSample(AWidth, AHeight: Integer);
var
  NewLeft, NewTop, NewWidth, NewHeight: Integer;
begin
  NewWidth := AWidth + 2 * spePanelMarginH.Value;
  NewHeight := AHeight + 2 * spePanelMarginV.Value;
  NewLeft := (grpSizePreview.Width - NewWidth + 1) div 2;
  NewTop := (grpSizePreview.Height - NewHeight + 1) div 2;
  pnlSample.SetBounds(NewLeft, NewTop, NewWidth, NewHeight);
  BtnSizeSample.SetBounds(spePanelMarginH.Value, spePanelMarginV.Value, AWidth, AHeight);
end;

procedure TabcdfNavPanelEdit.rgBtnSizeClick(Sender: TObject);
begin
  with btnSizeSample do
  case rgBtnSize.ItemIndex of
    0:
      CenterSizeSample(15, 15);
    1:
      CenterSizeSample(20, 20);
    2:
      CenterSizeSample(23, 22);
    3:
      CenterSizeSample(25, 25);
    4:
      CenterSizeSample(30, 30);
    5:
      CenterSizeSample(32, 32);
    6:
      CenterSizeSample(40, 40);
    7:
      CenterSizeSample(50, 50);
    8:
      CenterSizeSample(80, 30);
    9:
      CenterSizeSample(120, 60);
    10:
      CenterSizeSample(speBtnWidth.Value, speBtnHeight.Value);
  end;
  grpCustomSize.Visible := (rgBtnSize.ItemIndex = (rgBtnSize.Items.Count - 1));
end;

procedure TabcdfNavPanelEdit.speBtnWidthChange(Sender: TObject);
begin
  CenterSizeSample(speBtnWidth.Value, speBtnHeight.Value);
end;

procedure TabcdfNavPanelEdit.btnRemoveAllClick(Sender: TObject);
begin
  lstSelected.Clear;
  outButtons.SetFocus;
end;

procedure TabcdfNavPanelEdit.btnUpClick(Sender: TObject);
var
  SaveIndex: integer;
begin
  with lstSelected do
    if (Items.Count > 0) and (ItemIndex > 0) then begin
      SaveIndex := ItemIndex - 1;
      Items.Exchange(ItemIndex, ItemIndex - 1);
      ItemIndex := SaveIndex;
      SetFocus;
    end;
end;

procedure TabcdfNavPanelEdit.btnDownClick(Sender: TObject);
var
  SaveIndex: integer;
begin
  with lstSelected do
    if (Items.Count > 0) and (ItemIndex < Items.Count - 1) then begin
      SaveIndex := ItemIndex + 1;
      Items.Exchange(ItemIndex, ItemIndex + 1);
      ItemIndex := SaveIndex;
      SetFocus;
    end;
end;

procedure TabcdfNavPanelEdit.btnAddClick(Sender: TObject);
var
  Child: Longint;
begin
  with outButtons do
    if Items[SelectedItem].HasItems then begin
      Child := Items[SelectedItem].GetFirstChild;
      while Child <> -1 do begin
        lstSelected.Items.AddObject(Items[Child].Text, Items[Child].Data);
        Child := Items[SelectedItem].GetNextChild(Child);
      end;
    end
    else
     lstSelected.Items.AddObject(Items[SelectedItem].Text, Items[SelectedItem].Data);
end;

procedure TabcdfNavPanelEdit.btnAddAllClick(Sender: TObject);
var
  I: Longint;
begin
  with outButtons do
    for I := 1 to ItemCount do
      if not Items[I].HasItems then
        lstSelected.Items.AddObject(Items[I].Text, Items[I].Data);
end;

procedure TabcdfNavPanelEdit.btnRemoveClick(Sender: TObject);
var
  SaveIndex: Integer;
begin
  with lstSelected do begin
    SaveIndex := ItemIndex;
    if ItemIndex > -1 then
      Items.Delete(ItemIndex);
    if Items.Count > 0 then begin
      if SaveIndex < Items.Count then
        ItemIndex := SaveIndex
      else
        ItemIndex := Items.Count - 1;
      SetFocus;
    end;
  end;
end;

procedure TabcdfNavPanelEdit.rgCaptionTextStyleClick(Sender: TObject);
begin
  case rgCaptionTextStyle.ItemIndex of
    0: btnCaptionSample.TextStyle := tsNormal;
    1: btnCaptionSample.TextStyle := tsRaised;
    2: btnCaptionSample.TextStyle := tsLowered;
  end;
end;

procedure TabcdfNavPanelEdit.rgGlyphPosClick(Sender: TObject);
begin
  case rgGlyphPos.ItemIndex of
    0: btnCaptionSample.Layout := ablGlyphLeft;
    1: btnCaptionSample.Layout := ablGlyphRight;
    2: btnCaptionSample.Layout := ablGlyphTop;
    3: btnCaptionSample.Layout := ablGlyphBottom;
    4: btnCaptionSample.Layout := ablGlyphCenter;
    5: btnCaptionSample.Layout := ablGlyphTiled;
  end;
end;

procedure TabcdfNavPanelEdit.btnButtonFontClick(Sender: TObject);
begin
  with FontDialog1 do
    if Execute then
      btnCaptionSample.Font := Font;
end;

procedure TabcdfNavPanelEdit.GlyphSampleChange(Sender: TObject);
var
  bmpname: string;
  INIGlyphFileName: TFileName;
  ININumGlyphs: Integer;
begin
  if rgGlyphStyles.ItemIndex = 7 then
    {None item, clear picture}
    btnGlyphSample.Picture.Graphic := nil
  else if rgGlyphStyles.ItemIndex = 6 then
  begin
    {Custom button from INI File}
    if GetINIFileBtnInfo('npFirst',
                         (rgGlyphOrient.ItemIndex = 1),
                         INIGlyphFileName,
                         ININumGlyphs) then
      with btnGlyphSample do
      begin
        Picture.LoadFromFile(INIGlyphFileName);
        NumGlyphs := ININumGlyphs;
      end
    else
      btnGlyphSample.Picture.Graphic := nil;
  end
  else
  begin
    if rgGlyphOrient.ItemIndex = 0 then
      bmpname := 'ABCNP_FIRST'
    else
      bmpname := 'ABCNP_VFIRST';
    bmpname := bmpname + IntToStr(rgGlyphStyles.ItemIndex);
    LoadBitmapRes(btnGlyphSample, bmpname);
  end;
end;

procedure TabcdfNavPanelEdit.BtnColorClick(Sender: TObject);
begin
  with ColorDialog1 do
    if Execute then
      btnGlyphSample.Color := Color;
end;

procedure TabcdfNavPanelEdit.chkCaptionClick(Sender: TObject);
begin
  with btnGlyphSample do
    if chkCaption.Checked then
    begin
      Caption := 'Caption';
      SetBounds(70,15,75,50);
    end
    else
    begin
      Caption := '';
      SetBounds(95,25,25,25);
    end;
end;

procedure TabcdfNavPanelEdit.spePanelMarginChange(Sender: TObject);
begin
  if not FInitSizing then
    CenterSizeSample(btnSizeSample.Width, btnSizeSample.Height);
end;

procedure TabcdfNavPanelEdit.outButtonsDblClick(Sender: TObject);
begin
  with outButtons do
    if not Items[SelectedItem].HasItems then
     lstSelected.Items.AddObject(Items[SelectedItem].Text, Items[SelectedItem].Data);
end;

procedure TabcdfNavPanelEdit.speBtnMarginChange(Sender: TObject);
begin
  btnCaptionSample.Margin := speBtnMargin.Value;
end;

procedure TabcdfNavPanelEdit.cboBtnStyleClick(Sender: TObject);
begin
  btnGlyphSample.Style := TabcButtonStyle(cboBtnStyle.ItemIndex);
end;

procedure TabcdfNavPanelEdit.cboBtnPicHighlightClick(Sender: TObject);
begin
  btnGlyphSample.PictureHighlight := TabcGlyphHighlight(cboBtnPicHighlight.ItemIndex);
end;

function TabcdfNavPanelEdit.GetColCount(var Items, Rows: Integer): Integer;
begin
  if Items = 0 then
  begin
    Rows := 0;
    Result := 0;
  end
  else if Rows >= Items then
  begin
    Rows := Items;
    Result := 1;
  end
  else
  begin
    if Rows = 0 then Rows := 1;
    Result := abcCeil(Items / Rows);
  end;
end;

function TabcdfNavPanelEdit.GetRowCount(var Items, Cols: Integer): Integer;
begin
  if Items = 0 then
  begin
    Cols := 0;
    Result := 0;
  end
  else if Cols >= Items then
  begin
    Cols := Items;
    Result := 1;
  end
  else
  begin
    if Cols = 0 then Cols := 1;
    Result := abcCeil(Items / Cols);
  end;
end;

procedure TabcdfNavPanelEdit.speBtnRowsChange(Sender: TObject);
var
  bIncrease: Boolean;
begin
  if not FRowColChanging then
  begin
    FRowColChanging := True;
    FByExactCols := False;
    bIncrease := speBtnRows.Value > FCtrlRows;
    FCtrlRows := speBtnRows.Value;
    FCtrlCols := GetColCount(FButtonCount, FCtrlRows);
    if rgGlyphOrient.ItemIndex = 0 then
    begin
      {Horz - want exact rows for num cols}
      while (FCtrlCols = speBtnCols.Value) and (FCtrlCols <> 1) and (FCtrlCols <> FButtonCount) do
      begin
        if bIncrease then
          Inc(FCtrlRows)
        else
          Dec(FCtrlRows);
        FCtrlCols := GetColCount(FButtonCount, FCtrlRows);
      end;
      FCtrlRows := GetRowCount(FButtonCount, FCtrlCols);
    end;
    speBtnRows.Value := FCtrlRows;
    speBtnCols.Value := FCtrlCols;
    FRowColChanging := False;
  end;
end;

procedure TabcdfNavPanelEdit.speBtnColsChange(Sender: TObject);
var
  bIncrease: Boolean;
begin
  if not FRowColChanging then
  begin
    FRowColChanging := True;
    FByExactCols := True;
    bIncrease := speBtnCols.Value > FCtrlCols;
    FCtrlCols := speBtnCols.Value;
    FCtrlRows := GetRowCount(FButtonCount, FCtrlCols);
    if rgGlyphOrient.ItemIndex = 1 then
    begin
      {Vert - want exact cols for num rows}
      while (FCtrlRows = speBtnRows.Value) and (FCtrlRows <> 1) and (FCtrlRows <> FButtonCount) do
      begin
        if bIncrease then
          Inc(FCtrlCols)
        else
          Dec(FCtrlCols);
        FCtrlRows := GetRowCount(FButtonCount, FCtrlCols);
      end;
      FCtrlCols := GetColCount(FButtonCount, FCtrlRows);
    end;
    speBtnRows.Value := FCtrlRows;
    speBtnCols.Value := FCtrlCols;
    FRowColChanging := False;
  end;
end;

procedure TabcdfNavPanelEdit.chkPopupClick(Sender: TObject);
begin
  btnGlyphSample.Popup := chkPopup.Checked;
end;

end.
