unit abcdfnpb;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{ Dialog for ABC Navigator Panel Button component editor }

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, ExtCtrls, StdCtrls, Buttons, abcbtn, abcspin, Dialogs, INIFiles, abcnpbtn,
  abcexctl;

type

  TabcdfNavBtnEdit = class(TForm)
    Notebook: TNotebook;
    btnNext: TBitBtn;
    btnPrev: TBitBtn;
    btnCancel: TBitBtn;
    lblPageHint: TLabel;
    Image1: TImage;
    rgGlyphStyles: TRadioGroup;
    FontDialog1: TFontDialog;
    rgBtnSize: TRadioGroup;
    grpCustomSize: TGroupBox;
    lblBtnWidth: TLabel;
    speBtnWidth: TabcSpinEdit;
    lblBtnHeight: TLabel;
    speBtnHeight: TabcSpinEdit;
    ColorDialog1: TColorDialog;
    grpSizePreview: TGroupBox;
    Bevel1: TBevel;
    btnSizeSample: TabcPicSpeedBtn;
    edHint: TEdit;
    edHintDown: TEdit;
    lblHint: TLabel;
    lblHintDown: TLabel;
    lblCaption: TLabel;
    edCaption: TEdit;
    Label7: TLabel;
    cboBtnStyle: TComboBox;
    rgGlyphOrient: TRadioGroup;
    BtnColor: TBitBtn;
    GroupBox1: TGroupBox;
    btnGlyphSample: TabcPicSpeedBtn;
    chkHints: TCheckBox;
    chkCaption: TCheckBox;
    rgGlyphPos: TRadioGroup;
    rgCaptionTextStyle: TRadioGroup;
    btnButtonFont: TButton;
    GroupBox2: TGroupBox;
    btnCaptionSample: TabcPicSpeedBtn;
    Label5: TLabel;
    speBtnMargin: TabcSpinEdit;
    chkPopup: TCheckBox;
    Label3: TLabel;
    cboBtnPicHighlight: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure CancelClick(Sender: TObject);
    procedure PrevClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    procedure rgBtnSizeClick(Sender: TObject);
    procedure speBtnWidthChange(Sender: TObject);
    procedure rgCaptionTextStyleClick(Sender: TObject);
    procedure rgGlyphPosClick(Sender: TObject);
    procedure btnButtonFontClick(Sender: TObject);
    procedure GlyphSampleChange(Sender: TObject);
    procedure BtnColorClick(Sender: TObject);
    procedure chkCaptionClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cboBtnStyleClick(Sender: TObject);
    procedure speBtnMarginChange(Sender: TObject);
    procedure chkPopupClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure cboBtnPicHighlightClick(Sender: TObject);
  private
    { Private declarations }
    FINIFile: TINIFile;
    FButton: TabcCustomPicSpeedBtn;
    FFirstActivate: Boolean;
    procedure CenterSizeSample(AWidth, AHeight: Integer);
    procedure InitButtonValues;
    procedure InitCaptionSample;
    procedure InitButtonSize;
    procedure InitNavGlyphSample;
    procedure InitCaptionHint;
    procedure RefreshButtons;
    procedure UpdateButton;
    function GetINIFileBtnInfo(Ident: string; Vert: Boolean;
      var BtnFileName: TFileName; var BtnNumGlyphs: Integer): Boolean;
    procedure SetButtonGlyph(AControl: TControl);
    procedure LoadBitmapRes(AButton: TabcCustomPicSpeedBtn; AResName: string);
  public
    { Public declarations }
  end;

function ExecabcdfNavBtnEdit(Button: TabcCustomPicSpeedBtn): Boolean;

implementation

{$R *.DFM}

uses abcdsncn, abcutilf, abcnpedt, abcnpflt, abcnpinf;

const

  ABCINIFileName = 'abc_4_d.ini';

  { page numbers }
  pgGlyphs       = 0;
  pgCaptionStyle = 1;
  pgButtonSize   = 2;
  pgCaptionHint  = 3;

type
  TabcNavBtnEditProps = class(TabcCustomPicSpeedBtn)
  public
    property Caption;
    property Color;
    property Font;
    property Layout;
    property Margin;
    property Picture;
    property NumGlyphs;
    property Popup;
    property Style;
    property TextStyle;
  end;

function ExecabcdfNavBtnEdit(Button: TabcCustomPicSpeedBtn): Boolean;
begin
  Result := False;
  with TabcdfNavBtnEdit.Create(nil) do
  try
    FButton := Button;
    InitButtonValues;
    if ShowModal = mrOK then begin
      UpdateButton;
      Result := True;
    end;
  finally
    Free;
  end;
end;

{ TabcdfNavBtnEdit }

type
  TAccess = class(TabcCustomPicSpeedBtn);

procedure TabcdfNavBtnEdit.LoadBitmapRes(AButton: TabcCustomPicSpeedBtn; AResName: string);
begin
  with TAccess(AButton) do
  begin
    Picture.Bitmap.LoadFromResourceName(HInstance, AResName);
    Picture.Bitmap.PixelFormat := pf4Bit;
    NumGlyphs := 2;
  end;
end;

procedure TabcdfNavBtnEdit.InitButtonValues;
begin
  with TabcNavBtnEditProps(FButton) do begin
    cboBtnStyle.ItemIndex := Ord(Style);
    cboBtnStyleClick(Self);
    cboBtnPicHighlight.ItemIndex := Ord(PictureHighlight);
    cboBtnPicHighlightClick(Self);
    chkPopup.Checked := Popup;
    ColorDialog1.Color := Color;
    btnGlyphSample.Color := Color;
    chkHints.Checked := Hint <> '';
    chkCaption.Checked := Caption <> '';
    rgGlyphPos.ItemIndex := Ord(Layout);
    rgCaptionTextStyle.ItemIndex := Ord(TextStyle);
    FontDialog1.Font.Assign(Font);
    btnCaptionSample.Font.Assign(Font);
    speBtnMargin.Value := Margin;
    if (FButton is TabcnpBookmark) then
      with TabcnpBookmark(FButton) do begin
        edHint.Text := HintUp;
        edHintDown.Text := HintDown;
      end
    else if (FButton is TabcnpFilterExpression) then
      with TabcnpFilterExpression(FButton) do begin
        edHint.Text := HintUp;
        edHintDown.Text := HintDown;
      end
    else if (FButton is TabcnpFilterActive) then
      with TabcnpFilterActive(FButton) do begin
        edHint.Text := HintUp;
        edHintDown.Text := HintDown;
      end
    else if (FButton is TabcToggleBtn) then
      with TabcToggleBtn(FButton) do begin
        edHint.Text := HintUp;
        edHintDown.Text := HintDown;
      end
    else
      edHint.Text := Hint;
    edCaption.Text := Caption;
  end;
end;

procedure TabcdfNavBtnEdit.UpdateButton;
begin
  with TabcNavBtnEditProps(FButton) do begin
    SetButtonGlyph(TControl(FButton));
    Color := btnGlyphSample.Color;
    Style := btnGlyphSample.Style;
    PictureHighlight := btnGlyphSample.PictureHighlight;
    Popup := btnGlyphSample.Popup;
    if chkHints.Checked then
      if (FButton is TabcnpBookmark) then
        with TabcnpBookmark(FButton) do begin
          HintUp := edHint.Text;
          HintDown := edHintDown.Text;
        end
      else if (FButton is TabcnpFilterExpression) then
        with TabcnpFilterExpression(FButton) do begin
          HintUp := edHint.Text;
          HintDown := edHintDown.Text;
        end
      else if (FButton is TabcnpFilterActive) then
        with TabcnpFilterActive(FButton) do begin
          HintUp := edHint.Text;
          HintDown := edHintDown.Text;
        end
      else if (FButton is TabcToggleBtn) then
        with TabcToggleBtn(FButton) do begin
          HintUp := edHint.Text;
          HintDown := edHintDown.Text;
        end
      else
        Hint := edHint.Text;
    if chkCaption.Checked then begin
      Font := btnCaptionSample.Font;
      Layout := btnCaptionSample.Layout;
      TextStyle := btnCaptionSample.TextStyle;
      Caption := edCaption.Text;
      Margin := btnCaptionSample.Margin;
    end
    else
      Margin := -1;

    SetBounds(Left, Top, btnSizeSample.Width, btnSizeSample.Height);
  end;
end;

function TabcdfNavBtnEdit.GetINIFileBtnInfo(Ident: string; Vert: Boolean;
  var BtnFileName: TFileName; var BtnNumGlyphs: Integer): Boolean;
const
  Section = 'NavPanel Custom Buttons';
var
  FindIdent: string;
  FoundPath: string;
begin
  Result := False;
  if Vert then
    FindIdent := Ident + 'Vert'
  else
    FindIdent := Ident;
  with FINIFile do begin
    FoundPath := ReadString(Section, 'Path', '');
    BtnFileName := ReadString(Section, FindIdent, '');
    if (BtnFileName = '') and Vert then
      BtnFileName := ReadString(Section, Ident, '');
    if BtnFileName = '' then
      Exit
    else begin
      BtnFileName := FoundPath + '\' + BtnFileName;
      if FileExists(BtnFileName) then
        Result := True
      else begin
        BtnFileName := '';
        Exit;
      end;
    end;
    BtnNumGlyphs := ReadInteger(Section, Ident + 'NumGlyphs', 2);
  end;
end;

procedure TabcdfNavBtnEdit.SetButtonGlyph(AControl: TControl);
var
  BmpResName: string;
  ArrayIndex: Integer;
  INIIdent: string;
  INIGlyphFileName: TFileName;
  ININumGlyphs: Integer;
begin
  if rgGlyphStyles.ItemIndex = 6 then begin
    {Custom Button}
    if AControl is TabcToggleBtn then
      INIIdent := 'Toggle'
    else if AControl is TabcPicSpeedBtn then
      INIIdent := 'Custom'
    else
      INIIdent := Copy(AControl.ClassName, 2, 63);
    if GetINIFileBtnInfo(INIIdent,
                         (rgGlyphOrient.ItemIndex = 1),
                         INIGlyphFileName,
                         ININumGlyphs) then
      with TabcCustomnpBtn(AControl) do begin
        Picture.LoadFromFile(INIGlyphFileName);
        NumGlyphs := ININumGlyphs;
      end;
  end
  else begin
    ArrayIndex := 0;
    while not (AControl is ABCDesignNavComponentClass(ArrayIndex)) do begin
      Inc(ArrayIndex);
      if ArrayIndex > ABC_DESIGNER_BTN_COUNT-1 then Exit;
    end;
    if rgGlyphOrient.ItemIndex = 0 then
      BmpResName := ABCDesignNavBmpResName(ArrayIndex)
    else
      BmpResName := ABCDesignNavVertBmpResName(ArrayIndex);
    if BmpResName = '' then Exit;

    {set glyph}
    BmpResName := 'ABCNP_' + BmpResName + IntToStr(rgGlyphStyles.ItemIndex);
    LoadBitmapRes(TabcCustomPicSpeedBtn(AControl), BmpResName);
  end;
end;

procedure TabcdfNavBtnEdit.InitCaptionHint;
var
  ArrayIndex: Integer;
begin
  ArrayIndex := 0;
  while not (FButton is ABCDesignNavComponentClass(ArrayIndex)) do
    Inc(ArrayIndex);
  if chkHints.Checked then begin
    edHint.Text := ABCDesignNavComponentHint(ArrayIndex);
    edHint.Visible := True;
    lblHint.Visible := True;
    if ABCDesignNavComponentHintDown(ArrayIndex) <> sabcnpHintDownNone then begin
      edHintDown.Text := ABCDesignNavComponentHintDown(ArrayIndex);
      edHintDown.Visible := True;
      lblHintDown.Visible := True;
      edCaption.Top := 100;
      lblCaption.Top := 103;
    end
    else begin
      edCaption.Top := 60;
      lblCaption.Top := 63;
    end;
  end
  else begin
    edHint.Visible := False;
    lblHint.Visible := False;
    edHintDown.Visible := False;
    lblHintDown.Visible := False;
    edCaption.Top := 20;
    lblCaption.Top := 23;
  end;
  if chkCaption.Checked then begin
    edCaption.Text := ABCDesignNavComponentCaption(ArrayIndex);
    edCaption.Visible := True;
    lblCaption.Visible := True;
  end;
end;

procedure TabcdfNavBtnEdit.InitCaptionSample;
begin
  with btnCaptionSample do begin
    Picture := btnGlyphSample.Picture;
    NumGlyphs := 2;
    Color := btnGlyphSample.Color;
    Style := btnGlyphSample.Style;
    PictureHighlight := btnGlyphSample.PictureHighlight;
  end;
end;

procedure TabcdfNavBtnEdit.InitButtonSize;
begin
  with btnSizeSample do begin
    Picture := btnGlyphSample.Picture;
    NumGlyphs := 2;
    Color := btnGlyphSample.Color;
    Popup := btnGlyphSample.Popup;
    Style := btnGlyphSample.Style;
    PictureHighlight := btnGlyphSample.PictureHighlight;
    if chkCaption.Checked then begin
      Caption := 'Caption';
      Font := btnCaptionSample.Font;
      Layout := btnCaptionSample.Layout;
      Margin := btnCaptionSample.Margin;
      TextStyle := btnCaptionSample.TextStyle;
      rgBtnSize.ItemIndex := 6;
      CenterSizeSample(50, 50);
    end
    else begin
      Caption := '';
      Layout := btnGlyphSample.Layout;
      Margin := -1;
      TextStyle := btnGlyphSample.TextStyle;
      rgBtnSize.ItemIndex := 2;
      CenterSizeSample(23, 22);
    end;
  end;
end;

procedure TabcdfNavBtnEdit.FormCreate(Sender: TObject);
begin
  FINIFile := TINIFile.Create(ABCINIFileName);
  NoteBook.PageIndex := pgGlyphs;
  FFirstActivate := True;
  PrevClick(Self);
  RefreshButtons;
  InitNavGlyphSample;
end;

procedure TabcdfNavBtnEdit.FormDestroy(Sender: TObject);
begin
  FINIFile.Free;
end;

procedure TabcdfNavBtnEdit.InitNavGlyphSample;
begin
  LoadBitmapRes(btnGlyphSample, 'ABCNP_FIRST2');
end;

procedure TabcdfNavBtnEdit.CancelClick(Sender: TObject);
begin
  Close;
end;

procedure TabcdfNavBtnEdit.PrevClick(Sender: TObject);
begin
  case NoteBook.PageIndex of
    pgGlyphs:
      Exit;
    pgCaptionStyle:
      NoteBook.PageIndex := pgGlyphs;
    pgButtonSize:
      if chkCaption.Checked then
        NoteBook.PageIndex := pgCaptionStyle
      else
        NoteBook.PageIndex := pgGlyphs;
    pgCaptionHint:
      NoteBook.PageIndex := pgButtonSize;
  end;
  RefreshButtons;
end;

procedure TabcdfNavBtnEdit.NextClick(Sender: TObject);
begin
  case NoteBook.PageIndex of
    pgGlyphs:
      if chkCaption.Checked then begin
        InitCaptionSample;
        NoteBook.PageIndex := pgCaptionStyle;
      end
      else begin
        InitButtonSize;
        NoteBook.PageIndex := pgButtonSize;
      end;
    pgCaptionStyle:
      begin
        InitButtonSize;
        NoteBook.PageIndex := pgButtonSize;
      end;
    pgButtonSize:
      if chkCaption.Checked or chkHints.Checked then
      begin
        InitCaptionHint;
        NoteBook.PageIndex := pgCaptionHint;
      end
      else begin
        ModalResult := mrOK;
        Exit;
      end;
    pgCaptionHint:
      begin
        ModalResult := mrOK;
        Exit;
      end;
  end;
  RefreshButtons;
end;

procedure TabcdfNavBtnEdit.RefreshButtons;
begin
  case NoteBook.PageIndex of
    pgGlyphs:
      lblPageHint.Caption := sabcnpdsgnBtnGlyphs;
    pgCaptionStyle:
      lblPageHint.Caption := sabcnpdsgnBtnCaptionStyle;
    pgButtonSize:
      lblPageHint.Caption := sabcnpdsgnBtnButtonSize;
    pgCaptionHint:
      lblPageHint.Caption := sabcnpdsgnBtnCaptionHint;
  end;

  btnPrev.Enabled := Notebook.PageIndex > 0;
  if ((Notebook.PageIndex = pgCaptionHint) or
      ((Notebook.PageIndex = pgButtonSize) and not
       (chkCaption.Checked or chkHints.Checked))) then
  begin
    btnNext.Caption := '&Create';
    btnNext.Glyph.Handle := LoadBitmap(HInstance, 'ABCNPDSGN_CREATE');
    btnNext.NumGlyphs := 1;
  end
  else
  begin
    btnNext.Caption := '&Next';
    btnNext.Glyph.Handle := LoadBitmap(HInstance, 'ABCNPDSGN_NEXT');
    btnNext.NumGlyphs := 2;
  end;
end;

procedure TabcdfNavBtnEdit.CenterSizeSample(AWidth, AHeight: Integer);
var
  NewLeft, NewTop: Integer;
begin
  NewLeft := (grpSizePreview.Width - AWidth + 1) div 2;
  NewTop := (grpSizePreview.Height - AHeight + 1) div 2;
  BtnSizeSample.SetBounds(NewLeft, NewTop, AWidth, AHeight);
end;

procedure TabcdfNavBtnEdit.rgBtnSizeClick(Sender: TObject);
begin
  with btnSizeSample do
  case rgBtnSize.ItemIndex of
    0:
      CenterSizeSample(15, 15);
    1:
      CenterSizeSample(20, 20);
    2:
      CenterSizeSample(23, 22);
    3:
      CenterSizeSample(25, 25);
    4:
      CenterSizeSample(30, 30);
    5:
      CenterSizeSample(32, 32);
    6:
      CenterSizeSample(40, 40);
    7:
      CenterSizeSample(50, 50);
    8:
      CenterSizeSample(80, 30);
    9:
      CenterSizeSample(120, 60);
    10:
      CenterSizeSample(speBtnWidth.Value, speBtnHeight.Value);
  end;
  grpCustomSize.Visible := (rgBtnSize.ItemIndex = (rgBtnSize.Items.Count - 1));
end;

procedure TabcdfNavBtnEdit.speBtnWidthChange(Sender: TObject);
begin
  CenterSizeSample(speBtnWidth.Value, speBtnHeight.Value);
end;

procedure TabcdfNavBtnEdit.rgCaptionTextStyleClick(Sender: TObject);
begin
  case rgCaptionTextStyle.ItemIndex of
    0: btnCaptionSample.TextStyle := tsNormal;
    1: btnCaptionSample.TextStyle := tsRaised;
    2: btnCaptionSample.TextStyle := tsLowered;
  end;
end;

procedure TabcdfNavBtnEdit.rgGlyphPosClick(Sender: TObject);
begin
  case rgGlyphPos.ItemIndex of
    0: btnCaptionSample.Layout := ablGlyphLeft;
    1: btnCaptionSample.Layout := ablGlyphRight;
    2: btnCaptionSample.Layout := ablGlyphTop;
    3: btnCaptionSample.Layout := ablGlyphBottom;
    4: btnCaptionSample.Layout := ablGlyphCenter;
    5: btnCaptionSample.Layout := ablGlyphTiled;
  end;
end;

procedure TabcdfNavBtnEdit.btnButtonFontClick(Sender: TObject);
begin
  with FontDialog1 do
    if Execute then
      btnCaptionSample.Font := Font;
end;

procedure TabcdfNavBtnEdit.GlyphSampleChange(Sender: TObject);
var
  bmpname: string;
  INIGlyphFileName: TFileName;
  ININumGlyphs: Integer;
begin
  if rgGlyphStyles.ItemIndex = 7 then
    {None item, clear picture}
    btnGlyphSample.Picture.Graphic := nil
  else if rgGlyphStyles.ItemIndex = 6 then
  begin
    {Custom button from INI File}
    if GetINIFileBtnInfo('npFirst',
                         (rgGlyphOrient.ItemIndex = 1),
                         INIGlyphFileName,
                         ININumGlyphs) then
      with btnGlyphSample do
      begin
        Picture.LoadFromFile(INIGlyphFileName);
        NumGlyphs := ININumGlyphs;
      end
    else
      btnGlyphSample.Picture.Graphic := nil;
  end
  else
  begin
    if rgGlyphOrient.ItemIndex = 0 then
      bmpname := 'ABCNP_FIRST'
    else
      bmpname := 'ABCNP_VFIRST';
    bmpname := bmpname + IntToStr(rgGlyphStyles.ItemIndex);
    LoadBitmapRes(btnGlyphSample, bmpname);
  end;
end;

procedure TabcdfNavBtnEdit.BtnColorClick(Sender: TObject);
begin
  with ColorDialog1 do
    if Execute then
      btnGlyphSample.Color := Color;
end;

procedure TabcdfNavBtnEdit.chkCaptionClick(Sender: TObject);
begin
  with btnGlyphSample do
    if chkCaption.Checked then 
    begin
      Caption := 'Caption';
      SetBounds(70,15,75,50);
    end
    else 
    begin
      Caption := '';
      SetBounds(95,25,25,25);
    end;
end;

procedure TabcdfNavBtnEdit.cboBtnStyleClick(Sender: TObject);
begin
  btnGlyphSample.Style := TabcButtonStyle(cboBtnStyle.ItemIndex);
end;

procedure TabcdfNavBtnEdit.cboBtnPicHighlightClick(Sender: TObject);
begin
  btnGlyphSample.PictureHighlight := TabcGlyphHighlight(cboBtnPicHighlight.ItemIndex);
end;

procedure TabcdfNavBtnEdit.speBtnMarginChange(Sender: TObject);
begin
  btnCaptionSample.Margin := speBtnMargin.Value;
end;

procedure TabcdfNavBtnEdit.chkPopupClick(Sender: TObject);
begin
  btnGlyphSample.Popup := chkPopup.Checked;
end;

procedure TabcdfNavBtnEdit.FormActivate(Sender: TObject);
begin
  if FFirstActivate then 
  begin
    InitButtonValues;
    FFirstActivate := False;
  end;
end;

end.
