unit abcdfsbd;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtDlgs, ExtCtrls, StdCtrls, ComCtrls, ImgList,
  abcbtns,
{$IFDEF DELPHI6}
  DesignIntf, DesignEditors
{$ELSE}
  DsgnIntf
{$ENDIF}
  ;

type
  TabcdfEdgeDesigner = class(TForm)
    PageControl: TPageControl;
    tabSource: TTabSheet;
    tabFlat: TTabSheet;
    tabUp: TTabSheet;
    tabDown: TTabSheet;
    tabTest: TTabSheet;
    tabParams: TTabSheet;
    tabDisabled: TTabSheet;
    tabMask: TTabSheet;
    imgSource: TImage;
    imgFlat: TImage;
    imgUp: TImage;
    imgDown: TImage;
    imgDisabled: TImage;
    imgMask: TImage;

    rdgPixelFormat: TRadioGroup;
    Label1: TLabel;
    shpTrans: TShape;
    ColorDialog: TColorDialog;
    OpenDialog: TOpenPictureDialog;
    Label2: TLabel;
    Label3: TLabel;
    shpHighlight: TShape;
    shpShadow: TShape;
    SaveDialog: TSaveDialog;
    Panel1: TPanel;
    btnOpen: TButton;
    btnSave: TButton;
    btnOK: TButton;
    btnCancel: TButton;
    btnShape: TabcShapeButton;
    imlTest: TImageList;
    chkListSave: TCheckBox;
    chkPopup: TCheckBox;
    chkEnabled: TCheckBox;
    btnClear: TButton;
    procedure FormDestroy(Sender: TObject);
    procedure shpTransMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure rdgPixelFormatClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure PageControlChange(Sender: TObject);
    procedure chkPopupClick(Sender: TObject);
    procedure chkEnabledClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure chkListSaveClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    FbmpSource: TBitmap;
    FbmpFlat: TBitmap;
    FbmpUp: TBitmap;
    FbmpDown: TBitmap;
    FbmpDisabled: TBitmap;
    FbmpMask: TBitmap;
    FChanging: Boolean;
    FSaveFlags: array[0..4] of Boolean;
    FSourceLoaded: Boolean;
    procedure MakeBitmaps;
    procedure EnableButtons;
    function GetActiveImage: TImage;
    procedure LoadActiveImage(FileName: TFileName);
  end;

procedure abcDesignEdgeBitmaps(Button: TabcShapeButton);

implementation

{$R *.DFM}

uses abcedge, abcutilf, abcprop;

const
  C_PAGE_SOURCE = 0;
  C_PAGE_PARAMS = 1;
  C_PAGE_FLAT = 2;
  C_PAGE_UP = 3;
  C_PAGE_DOWN = 4;
  C_PAGE_DISABLED = 5;
  C_PAGE_MASK = 6;
  C_PAGE_TEST = 7;

  C_INDEX_FLAT = 0;
  C_INDEX_UP = 1;
  C_INDEX_DOWN = 2;
  C_INDEX_DISABLED = 3;
  C_INDEX_MASK = 4;

procedure abcDesignEdgeBitmaps(Button: TabcShapeButton);
var
  TargetList: TImageList;
begin
  with TabcdfEdgeDesigner.Create(nil) do
  try
    if ShowModal = mrOk then
    begin
      TargetList := Button.ImageList;
      if TargetList = nil then
      begin
        TargetList := TImageList.Create(Button.Owner);
        if (csDesigning in Button.ComponentState) then
        begin

{$IFDEF DELPHI6}
          TargetList.Name := abcFormDesignerUniqueName(GetParentForm(Button).Designer,'ImageList');
{$ELSE}
          TargetList.Name := abcDesignerUniqueName(IFormDesigner(GetParentForm(Button).Designer),'ImageList');
{$ENDIF}
          Button.ImageList := TargetList;
        end;
      end;
      {save images}
      with TargetList, Button do
      begin
        if TargetList.Width < FbmpSource.Width then
          TargetList.Width := FbmpSource.Width;
        if TargetList.Height < FbmpSource.Height then
          TargetList.Height := FbmpSource.Height;

        {replace or add individual bitmaps}
        if FSaveFlags[C_INDEX_FLAT] = True then
          if (IndexFlat > -1) and (IndexFlat < Count) then
            ReplaceMasked(IndexFlat, FbmpFlat, FbmpFlat.TransparentColor)
          else
            IndexFlat := AddMasked(FbmpFlat, FbmpFlat.TransparentColor)
        else
          IndexFlat := -1;

        if FSaveFlags[C_INDEX_UP] = True then
          if (IndexUp > -1) and (IndexUp < Count) then
            ReplaceMasked(IndexUp, FbmpUp, FbmpUp.TransparentColor)
          else
            IndexUp := AddMasked(FbmpUp, FbmpUp.TransparentColor)
        else
          IndexUp := -1;

        if FSaveFlags[C_INDEX_DOWN] = True then
          if (IndexDown > -1) and (IndexDown < Count) then
            ReplaceMasked(IndexDown, FbmpDown, FbmpDown.TransparentColor)
          else
            IndexDown := AddMasked(FbmpDown, FbmpDown.TransparentColor)
        else
          IndexDown := -1;

        if FSaveFlags[C_INDEX_DISABLED] = True then
          if (IndexDisabled > -1) and (IndexDisabled < Count) then
            ReplaceMasked(IndexDisabled, FbmpDisabled, FbmpDisabled.TransparentColor)
          else
            IndexDisabled := AddMasked(FbmpDisabled, FbmpDisabled.TransparentColor)
        else
          IndexDisabled := -1;

        if FSaveFlags[C_INDEX_MASK] = True then
          if (IndexMouseMask > -1) and (IndexMouseMask < Count) then
            ReplaceMasked(IndexMouseMask, FbmpMask, FbmpMask.TransparentColor)
          else
            IndexMouseMask := AddMasked(FbmpMask, FbmpMask.TransparentColor)
        else
          IndexMouseMask := -1;
      end;
      {other button properties}
      Button.Popup := chkPopup.Checked;
    end;
  finally
    Free;
  end;
end;

procedure TabcdfEdgeDesigner.EnableButtons;
begin
  btnCancel.Enabled := True;
  btnOk.Enabled := FSourceLoaded;
  btnOpen.Enabled := not (PageControl.ActivePage.TabIndex in [C_PAGE_PARAMS, C_PAGE_TEST])
    and (FSourceLoaded or (PageControl.ActivePage.TabIndex = C_PAGE_SOURCE));
  btnSave.Enabled :=
    not (PageControl.ActivePage.TabIndex in [C_PAGE_PARAMS, C_PAGE_TEST])
    and (GetActiveImage.Picture.Graphic <> nil);
  {checkbox and save flags}
  case PageControl.ActivePage.TabIndex of
    C_PAGE_SOURCE, C_PAGE_PARAMS, C_PAGE_TEST:
      chkListSave.Visible := False;
    C_PAGE_FLAT:
    begin
      chkListSave.Visible := True;
      chkListSave.Checked := FSaveFlags[C_INDEX_FLAT];
    end;
    C_PAGE_UP:
    begin
      chkListSave.Visible := True;
      chkListSave.Checked := FSaveFlags[C_INDEX_UP];
    end;
    C_PAGE_DOWN:
    begin
      chkListSave.Visible := True;
      chkListSave.Checked := FSaveFlags[C_INDEX_DOWN];
    end;
    C_PAGE_DISABLED:
    begin
      chkListSave.Visible := True;
      chkListSave.Checked := FSaveFlags[C_INDEX_DISABLED];
    end;
    C_PAGE_MASK:
    begin
      chkListSave.Visible := True;
      chkListSave.Checked := FSaveFlags[C_INDEX_MASK];
    end;
  end;
end;

function TabcdfEdgeDesigner.GetActiveImage: TImage;
begin
  if PageControl.ActivePage = tabSource then
    Result := imgSource
  else if PageControl.ActivePage = tabFlat then
    Result := imgFlat
  else if PageControl.ActivePage = tabUp then
    Result := imgUp
  else if PageControl.ActivePage = tabDown then
    Result := imgDown
  else if PageControl.ActivePage = tabDisabled then
    Result := imgDisabled
  else if PageControl.ActivePage = tabMask then
    Result := imgMask
  else
    Result := nil;
end;

procedure TabcdfEdgeDesigner.MakeBitmaps;
var
  bLeft, bTop, bWidth, bHeight: Integer;
begin
  FbmpFlat.Free;
  FbmpUp.Free;
  FbmpDown.Free;
  FbmpDisabled.Free;
  FbmpMask.Free;

  if rdgPixelFormat.ItemIndex = Ord(pfCustom) then
  begin
    FChanging := True;
    rdgPixelFormat.ItemIndex := Ord(pf8Bit);
    FChanging := False;
  end;

  FbmpFlat := TBitmap.Create;
  with FbmpFlat do
  begin
    Assign(FbmpSource);
    if TransparentColor <> shpTrans.Brush.Color then
      TransparentColor := shpTrans.Brush.Color;
    if PixelFormat <> TPixelFormat(rdgPixelFormat.ItemIndex) then
      PixelFormat := TPixelFormat(rdgPixelFormat.ItemIndex);
    abcMakeEdgeBitmaps(FbmpFlat,
                       shpHighlight.Brush.Color,
                       shpShadow.Brush.Color,
                       FbmpUp,
                       FbmpDown);
    imgFlat.Picture.Graphic := FbmpFlat;
    imgUp.Picture.Graphic := FbmpUp;
    imgDown.Picture.Graphic := FbmpDown;
  end;
  {load imagelist}
  with imlTest do
  begin
    Clear;
    Width := FbmpFlat.Width;
    Height := FbmpFlat.Height;
    AddMasked(FbmpFlat, shpTrans.Brush.Color);
    AddMasked(FbmpUp, shpTrans.Brush.Color);
    AddMasked(FbmpDown, shpTrans.Brush.Color);
    {disabled}
    FbmpDisabled := TBitmap.Create;
    with FbmpDisabled do
    begin
      Assign(FbmpFlat);
      TransparentColor := shpTrans.Brush.Color;
      Transparent := True;
    end;
    DrawingStyle := dsFocus;
    BlendColor := clGray;
    Draw(FbmpDisabled.Canvas, 0, 0, 0);
    DrawingStyle := dsNormal;
    imgDisabled.Picture.Graphic := FbmpDisabled;
    AddMasked(FbmpDisabled, shpTrans.Brush.Color);
    {mask}
    FbmpMask := TBitmap.Create;
    with FbmpMask do
    begin
      Assign(FbmpFlat);
      Mask(shpTrans.Brush.Color);
      Monochrome := True;
    end;
    imgMask.Picture.Graphic := FbmpMask;
    AddMasked(FbmpMask, shpTrans.Brush.Color);
  end;

  bWidth := FbmpSource.Width;
  bHeight := FbmpSource.Height;
  bLeft := (tabSource.Width - bWidth) div 2;
  bTop := (tabSource.Height - bHeight) div 2;
  btnShape.SetBounds(bLeft, bTop, bWidth, bHeight);

  FSaveFlags[C_INDEX_FLAT] := True;
  FSaveFlags[C_INDEX_UP] := True;
  FSaveFlags[C_INDEX_DOWN] := True;
  FSaveFlags[C_INDEX_DISABLED] := False;
  FSaveFlags[C_INDEX_MASK] := False;
  FSourceLoaded := True;
  EnableButtons;
end;

procedure TabcdfEdgeDesigner.FormDestroy(Sender: TObject);
begin
  FbmpSource.Free;
  FbmpFlat.Free;
  FbmpUp.Free;
  FbmpDown.Free;
  FbmpDisabled.Free;
  FbmpMask.Free;
end;

procedure TabcdfEdgeDesigner.shpTransMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Sender is TShape then
  with TShape(Sender).Brush do
  begin
    ColorDialog.Color := Color;
    if ColorDialog.Execute then
    begin
      Color := ColorDialog.Color;
      MakeBitmaps;
    end;
  end;
end;

procedure TabcdfEdgeDesigner.rdgPixelFormatClick(Sender: TObject);
begin
  if not FChanging then MakeBitmaps;
end;

procedure TabcdfEdgeDesigner.FormCreate(Sender: TObject);
begin
  shpHighlight.Brush.color := clBtnHighlight;
  shpShadow.Brush.color := clBtnShadow;
  PageControl.ActivePage := tabSource;
  EnableButtons;
end;

procedure TabcdfEdgeDesigner.btnSaveClick(Sender: TObject);
begin
  with SaveDialog do
    if Execute then
      GetActiveImage.Picture.SaveToFile(FileName);
end;

procedure TabcdfEdgeDesigner.btnOpenClick(Sender: TObject);
begin
  with OpenDialog do
    if Execute then
      LoadActiveImage(FileName);
end;

procedure TabcdfEdgeDesigner.LoadActiveImage(FileName: TFileName);
begin
  if PageControl.ActivePage = tabSource then
  begin
    FChanging := True;
    FbmpSource.Free;
    FbmpSource := TBitmap.Create;
    FbmpSource.LoadFromFile(FileName);
    imgSource.Picture.Graphic := FbmpSource;
    rdgPixelFormat.ItemIndex := Ord(FbmpSource.PixelFormat);
    shpTrans.Brush.Color := FbmpSource.TransparentColor;
    if Height < FbmpSource.Height + 100 then
      Height := FbmpSource.Height + 100;
    FChanging := False;
    MakeBitmaps;
  end
  else if PageControl.ActivePage = tabFlat then
  begin
    FbmpFlat.Free;
    FbmpFlat := TBitmap.Create;
    FbmpFlat.LoadFromFile(FileName);
    imgFlat.Picture.Graphic := FbmpFlat;
    imlTest.ReplaceMasked(C_INDEX_FLAT, FbmpFlat, shpTrans.Brush.Color);
    FSaveFlags[C_INDEX_FLAT] := True;
  end
  else if PageControl.ActivePage = tabUp then
  begin
    FbmpUp.Free;
    FbmpUp := TBitmap.Create;
    FbmpUp.LoadFromFile(FileName);
    imgUp.Picture.Graphic := FbmpUp;
    imlTest.ReplaceMasked(C_INDEX_UP, FbmpUp, shpTrans.Brush.Color);
    FSaveFlags[C_INDEX_UP] := True;
  end
  else if PageControl.ActivePage = tabDown then
  begin
    FbmpDown.Free;
    FbmpDown := TBitmap.Create;
    FbmpDown.LoadFromFile(FileName);
    imgDown.Picture.Graphic := FbmpDown;
    imlTest.ReplaceMasked(C_INDEX_DOWN, FbmpDown, shpTrans.Brush.Color);
    FSaveFlags[C_INDEX_UP] := True;
  end
  else if PageControl.ActivePage = tabDisabled then
  begin
    FbmpDisabled.Free;
    FbmpDisabled := TBitmap.Create;
    FbmpDisabled.LoadFromFile(FileName);
    imgDisabled.Picture.Graphic := FbmpDisabled;
    imlTest.ReplaceMasked(C_INDEX_DISABLED, FbmpDisabled, shpTrans.Brush.Color);
    FSaveFlags[C_INDEX_DISABLED] := True;
  end
  else if PageControl.ActivePage = tabMask then
  begin
    FbmpMask.Free;
    FbmpMask := TBitmap.Create;
    FbmpMask.LoadFromFile(FileName);
    imgMask.Picture.Graphic := FbmpMask;
    imlTest.ReplaceMasked(C_INDEX_MASK, FbmpMask, shpTrans.Brush.Color);
    FSaveFlags[C_INDEX_MASK] := True;
  end;
  EnableButtons;
end;

procedure TabcdfEdgeDesigner.PageControlChange(Sender: TObject);
begin
  EnableButtons;
end;

procedure TabcdfEdgeDesigner.chkPopupClick(Sender: TObject);
begin
  btnShape.Popup := chkPopup.Checked;
end;

procedure TabcdfEdgeDesigner.chkEnabledClick(Sender: TObject);
begin
  btnShape.Enabled := chkEnabled.Checked;
end;

procedure TabcdfEdgeDesigner.btnClearClick(Sender: TObject);
var
  i: integer;
begin
  FChanging := True;
  for i := Low(FSaveFlags) to High(FSaveFlags) do
    FSaveFlags[i] := False;
  FbmpSource.Free;
  FbmpSource := nil;
  FbmpFlat.Free;
  FbmpFlat := nil;
  FbmpUp.Free;
  FbmpUp := nil;
  FbmpDown.Free;
  FbmpDown := nil;
  FbmpDisabled.Free;
  FbmpDisabled := nil;
  FbmpMask.Free;
  FbmpMask := nil;
  imgSource.Picture.Graphic := nil;
  imgFlat.Picture.Graphic := nil;
  imgUp.Picture.Graphic := nil;
  imgDown.Picture.Graphic := nil;
  imgDisabled.Picture.Graphic := nil;
  imgMask.Picture.Graphic := nil;
  imlTest.Clear;
  FChanging := False;
  PageControl.ActivePage := tabSource;
  chkPopup.Checked := true;
  chkEnabled.Checked := True;
  FSourceLoaded := False;
  EnableButtons;
end;

procedure TabcdfEdgeDesigner.chkListSaveClick(Sender: TObject);
var
  flagIndex: Integer;
begin
  flagIndex := PageControl.ActivePage.TabIndex - 2;
  if (flagIndex >= Low(FSaveFlags)) and  (flagIndex <= High(FSaveFlags)) then
    FSaveFlags[flagIndex] := chkListSave.Checked;
end;

procedure TabcdfEdgeDesigner.FormResize(Sender: TObject);
var
  NewLeft, NewTop: integer;
begin
  with btnShape do
  begin
    NewLeft := (tabTest.Width - Width) div 2;
    NewTop := (tabTest.Height - Height) div 2;
    setBounds(NewLeft, NewTop, Width, Height);
  end;
end;

end.
