unit abcdfwvl;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses Windows, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls;

type
  TabcdfWaveListEdit = class(TForm)
    btnClose: TBitBtn;
    Bevel1: TBevel;
    Label1: TLabel;
    lstSounds: TListBox;
    btnAdd: TButton;
    btnEdit: TButton;
    btnDelete: TButton;
    btnUp: TButton;
    btnDown: TButton;
    procedure btnAddClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
  private
    { Private declarations }
    FChanged: Boolean;
    procedure CheckButtons;
  public
    { Public declarations }
  end;

function ExecabcdfWaveListEdit(Sounds: TStrings): Boolean;

implementation

uses abcwav, abcdfwav;

{$R *.DFM}

function ExecabcdfWaveListEdit(Sounds: TStrings): Boolean;
begin
  with TabcdfWaveListEdit.Create(nil) do
  try
    with lstSounds do begin
      Items.Assign(Sounds);
      if Items.Count > 0 then ItemIndex := 0;
    end;
    CheckButtons;
    ShowModal;
    Sounds.Assign(lstSounds.Items);
    Result := FChanged;
  finally
    Free;
  end;
end;

procedure TabcdfWaveListEdit.CheckButtons;
begin
  if lstSounds.Items.Count = 0 then begin
    if Visible then btnClose.SetFocus;
    btnEdit.Enabled := False;
    btnDelete.Enabled := False;
    btnUp.Enabled := False;
    btnDown.Enabled := False;
  end
  else begin
    btnEdit.Enabled := True;
    btnDelete.Enabled := True;
    btnUp.Enabled := True;
    btnDown.Enabled := True;
  end;
end;

procedure TabcdfWaveListEdit.btnAddClick(Sender: TObject);
var
  Sound: TabcWaveImage;
  SName: string;
begin
  Sound := TabcWaveImage.Create;
  SName := '[Empty Wave Image]';
  with lstSounds do begin
    if ExecabcdfWaveEditListItem(SName, Sound) then begin
      ItemIndex := Items.AddObject(SName, Sound);
      FChanged := True;
    end
    else
      Sound.Free;
    CheckButtons;
    SetFocus;
  end;
end;

procedure TabcdfWaveListEdit.btnEditClick(Sender: TObject);
var
  Sound: TabcWaveImage;
  SName: string;
  SaveIndex: Integer;
begin
  with lstSounds do begin
    SaveIndex := ItemIndex;
    Sound := TabcWaveImage(Items.Objects[lstSounds.ItemIndex]);
    SName := Items.Strings[lstSounds.ItemIndex];
    if ExecabcdfWaveEditListItem(SName, Sound) then begin
      FChanged := True;
      Items.Strings[ItemIndex] := SName;
      ItemIndex := SaveIndex;
    end;
    SetFocus;
  end;
end;

procedure TabcdfWaveListEdit.btnDeleteClick(Sender: TObject);
var
  Sound: TabcWaveImage;
  SaveIndex: Integer;
begin
  FChanged := True;
  with lstSounds do begin
    SaveIndex := ItemIndex;
    Sound := TabcWaveImage(Items.Objects[ItemIndex]);
    Items.Delete(ItemIndex);
    Sound.Free;
    CheckButtons;
    if SaveIndex < Items.Count then
      ItemIndex := SaveIndex
    else
      ItemIndex := Items.Count - 1;
    SetFocus;
  end;
end;

procedure TabcdfWaveListEdit.btnUpClick(Sender: TObject);
var
  NewIndex: Integer;
begin
  with lstSounds do begin
    if ItemIndex > 0 then begin
      FChanged := True;
      NewIndex := ItemIndex - 1;
      Items.Exchange(NewIndex, ItemIndex);
      ItemIndex := NewIndex;
    end;
    SetFocus;
  end;
end;

procedure TabcdfWaveListEdit.btnDownClick(Sender: TObject);
var
  NewIndex: Integer;
begin
  with lstSounds do begin
    if ItemIndex < (Items.Count - 1) then begin
      FChanged := True;
      NewIndex := ItemIndex + 1;
      Items.Exchange(ItemIndex, NewIndex);
      ItemIndex := NewIndex;
    end;
    SetFocus;
  end;
end;

end.
