unit abcdfx32;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, abcspin, ComCtrls, Buttons, Menus, ImgList,
  abcfx, abcufx;

type
  TabcdfImageEffectChooser = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    trvEffects: TTreeView;
    ImageList1: TImageList;
    pagPreview: TPageControl;
    tabPreview: TTabSheet;
    tabSpeed: TTabSheet;
    tabSelection: TTabSheet;
    Bevel1: TBevel;
    Image2: TImage;
    Image1: TImage;
    abcEffectsImage1: TabcEffectsImage;
    Label1: TLabel;
    Label2: TLabel;
    speSpeed: TabcSpinEdit;
    speSize: TabcSpinEdit;
    lstSelection: TListBox;
    popTreeSelection: TPopupMenu;
    mnuAddtoSelection1: TMenuItem;
    popListSelection: TPopupMenu;
    mnuListDelete: TMenuItem;
    mnuListDeleteAll: TMenuItem;
    btnAdd: TButton;
    Button1: TButton;
    chkEnabled: TCheckBox;
    chkNoRedraw: TCheckBox;
    Bevel2: TBevel;
    Bevel3: TBevel;
    lblPause: TLabel;
    spePause: TabcSpinEdit;
    procedure abcEffectsImage1Click(Sender: TObject);
    procedure speSizeChange(Sender: TObject);
    procedure speSpeedChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure trvEffectsGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure trvEffectsGetSelectedIndex(Sender: TObject; Node: TTreeNode);
    procedure trvEffectsClick(Sender: TObject);
    procedure mnuAddtoSelection1Click(Sender: TObject);
    procedure lstSelectionKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure popListSelectionPopup(Sender: TObject);
    procedure mnuListDeleteClick(Sender: TObject);
    procedure mnuListDeleteAllClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure trvEffectsStartDrag(Sender: TObject;
      var DragObject: TDragObject);
    procedure lstSelectionDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lstSelectionDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure spePauseChange(Sender: TObject);
  private
    { Private declarations }
    FImage2Loaded: Boolean;
    procedure AddAllChildrenToSelected(Node: TTreeNode);
    procedure AddNodeToSelected(Node: TTreeNode);
    procedure InitListSelections;
    procedure ListDeleteSelected;
  public
    { Public declarations }
    procedure SwapImage;
  end;

var
  abcdfImageEffectChooser: TabcdfImageEffectChooser;

function abcDlgChooseTransition(Transition: TabcTransition): Boolean;

function abcDlgChooseTransitionEffect(Image: TabcCustomEffectsImage): Boolean;

implementation

{$R *.DFM}

function abcDlgChooseTransition(Transition: TabcTransition): Boolean;
var
  i: integer;
begin
  Result := False;
  if Transition = nil then
    Exit;
  with TabcdfImageEffectChooser.Create(nil) do
  try
    speSize.Value := Transition.EffectSize;
    speSpeed.Value := Transition.EffectSpeed;
    spePause.Visible := False;
    lblPause.Visible := False;
    chkEnabled.Checked := Transition.Enabled;
    chkNoRedraw.Visible := True;
    chkNoRedraw.Checked := Transition.NoRedraw;
    abcEffectsImage1.Effect := Transition.Effect;
    with Transition.EffectSelection do
      for i := 0 to Count - 1 do
      begin
        abcEffectsImage1.EffectSelection.Add(Items[i]);
      end;
    InitListSelections;
    speSize.Enabled := abcSizedEffect(Transition.Effect);
    Result := (ShowModal = mrOk);
    if Result then
    begin
      Transition.Effect := abcEffectsImage1.Effect;
      Transition.EffectSpeed := abcEffectsImage1.EffectSpeed;
      Transition.EffectSize := abcEffectsImage1.EffectSize;
      Transition.EffectSelection.Clear;
      with abcEffectsImage1.EffectSelection do
        for i := 0 to Count - 1 do
          Transition.EffectSelection.Add(Items[i]);
      Transition.Enabled := chkEnabled.Checked;
      Transition.NoRedraw := chkNoRedraw.Checked;
    end;
  finally
    Free;
  end;
end;

function abcDlgChooseTransitionEffect(Image: TabcCustomEffectsImage): Boolean;
var
  ImageCast: TabcEffectsImage;
  i: integer;
begin
  Result := False;
  if Image = nil then
    Exit;
  ImageCast := TabcEffectsImage(Image);
  with TabcdfImageEffectChooser.Create(nil) do
  try
    speSize.Value := ImageCast.EffectSize;
    speSpeed.Value := ImageCast.EffectSpeed;
    spePause.Value := ImageCast.EffectPause;
    chkEnabled.Checked := ImageCast.Enabled;
    chkNoRedraw.Visible := False;
    abcEffectsImage1.Effect := ImageCast.Effect;
    with ImageCast.EffectSelection do
      for i := 0 to Count - 1 do
      begin
        abcEffectsImage1.EffectSelection.Add(Items[i]);
      end;
    InitListSelections;
    speSize.Enabled := abcSizedEffect(ImageCast.Effect);
    Result := (ShowModal = mrOk);
    if Result then
    begin
      ImageCast.Effect := abcEffectsImage1.Effect;
      ImageCast.EffectPause := abcEffectsImage1.EffectPause;
      ImageCast.EffectSpeed := abcEffectsImage1.EffectSpeed;
      ImageCast.EffectSize := abcEffectsImage1.EffectSize;
      ImageCast.EffectSelection.Clear;
      with abcEffectsImage1.EffectSelection do
        for i := 0 to Count - 1 do
          ImageCast.EffectSelection.Add(Items[i]);
      ImageCast.Enabled := chkEnabled.Checked;
    end;
  finally
    Free;
  end;
end;

procedure TabcdfImageEffectChooser.SwapImage;
begin
  with abcEffectsImage1 do
    if FImage2Loaded then
      Picture := Image1.Picture
    else
      Picture := Image2.Picture;
  FImage2Loaded := not FImage2Loaded;
end;

procedure TabcdfImageEffectChooser.abcEffectsImage1Click(Sender: TObject);
begin
  SwapImage;
end;

procedure TabcdfImageEffectChooser.speSizeChange(Sender: TObject);
begin
  with abcEffectsImage1 do
    EffectSize := speSize.Value;
end;

procedure TabcdfImageEffectChooser.speSpeedChange(Sender: TObject);
begin
  with abcEffectsImage1 do
    EffectSpeed := speSpeed.Value;
end;

procedure TabcdfImageEffectChooser.InitListSelections;
var
  i, found, numParents: integer;
begin
  {init tree node effect numbers for the lowest level}
  {and synchronise lstSelection items with image list}
  lstSelection.Items.Clear;
  with abcEffectsImage1 do
    for i := 0 to EffectSelection.Count - 1 do
      lstSelection.Items.AddObject(' ', EffectSelection[i]);
  numParents := 0;
  with trvEffects do
    for i := 0 to Items.Count - 1 do
      with Items[i] do
        if HasChildren then
          numParents := numParents + 1
        else
        begin
          Data := Pointer(i - numParents);
          found := abcEffectsImage1.EffectSelection.IndexOf(Data);
          if found <> -1 then
            lstSelection.Items[found] := Items[i].Text;
        end;
end;

procedure TabcdfImageEffectChooser.AddNodeToSelected(Node: TTreeNode);
begin
  lstSelection.Items.AddObject(Node.Text, Node.Data);
  abcEffectsImage1.EffectSelection.Add(Node.Data);
end;

procedure TabcdfImageEffectChooser.FormCreate(Sender: TObject);
begin
  abcEffectsImage1.EffectSpeed := 0;
  pagPreview.ActivePage := tabPreview;
  SwapImage;
  abcEffectsImage1.EffectSpeed := 500;
end;

procedure TabcdfImageEffectChooser.trvEffectsGetImageIndex(Sender: TObject;
  Node: TTreeNode);
begin
  with Node do
    if HasChildren then
      ImageIndex := 0
    else
      ImageIndex := 1;
end;

procedure TabcdfImageEffectChooser.trvEffectsGetSelectedIndex(
  Sender: TObject; Node: TTreeNode);
begin
  with Node do
    if not HasChildren then
      SelectedIndex := 2;
end;

procedure TabcdfImageEffectChooser.trvEffectsClick(Sender: TObject);
var
  i: integer;
begin
  if (trvEffects.Selected <> nil) then
    with trvEffects.Selected do
      if HasChildren then
      begin
        Expanded := not Expanded;
      end
      else
      begin
        i := Integer(Data);
        if (i <> -1) then
        begin
          abcEffectsImage1.Effect := TabcTransitionEffect(i);
          speSize.Enabled := abcSizedEffect(abcEffectsImage1.Effect);
          SwapImage;
        end
      end;
end;

procedure TabcdfImageEffectChooser.AddAllChildrenToSelected(Node: TTreeNode);
var
  Child: TTreeNode;
begin
  if Node.HasChildren then
  begin
    Child := Node.GetFirstChild;
    while Child <> nil do
    begin
      AddAllChildrenToSelected(Child);
      Child := Node.GetNextChild(Child);
    end;
  end
  else
    AddNodeToSelected(Node);
end;

procedure TabcdfImageEffectChooser.mnuAddtoSelection1Click(Sender: TObject);
begin
  with trvEffects do
    if (Selected <> nil) then
      if (Selected.HasChildren) then
        AddAllChildrenToSelected(Selected)
      else
        AddNodeToSelected(Selected);
end;

procedure TabcdfImageEffectChooser.ListDeleteSelected;
var
  saveIndex: integer;
begin
  with lstSelection do
    if ItemIndex > -1 then
    begin
      saveIndex := ItemIndex;
      ItemIndex := saveIndex - 1;
      Items.Delete(saveIndex);
      abcEffectsImage1.EffectSelection.Delete(saveIndex);
    end;
end;

procedure TabcdfImageEffectChooser.lstSelectionKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_DELETE then ListDeleteSelected;
end;

procedure TabcdfImageEffectChooser.popListSelectionPopup(Sender: TObject);
begin
  if lstSelection.Items.Count < 1 then Abort;
  mnuListDelete.Enabled := (lstSelection.ItemIndex >= 0);
end;

procedure TabcdfImageEffectChooser.mnuListDeleteClick(Sender: TObject);
begin
  ListDeleteSelected;
end;

procedure TabcdfImageEffectChooser.mnuListDeleteAllClick(Sender: TObject);
begin
  lstSelection.Clear;
  abcEffectsImage1.EffectSelection.Clear;
end;

procedure TabcdfImageEffectChooser.btnOKClick(Sender: TObject);
begin
  with abcEffectsImage1 do
    if (lstSelection.Items.Count > 0) and not (Effect in [atfSelectionRandom, atfSelectionSequence]) then
      case MessageDlg(
        'There are items in the Selection list.' + #13#10 + #13#10 +
        'Do you want the transition effect to ' + #13#10 +
        'iterate through this Selection sequence?',
        mtConfirmation,
        [mbYes, mbNo, mbCancel],
        0) of
        mrYes:
          begin
            Effect := atfSelectionSequence;
            ModalResult := mrOk;
          end;
        mrNo:
          ModalResult := mrOk;
        mrCancel:
          Exit;
      end;
end;

procedure TabcdfImageEffectChooser.trvEffectsStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  if trvEffects.Selected = nil then
    trvEffects.EndDrag(False)
  else
    pagPreview.ActivePage := tabSelection;
end;

procedure TabcdfImageEffectChooser.lstSelectionDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := (Source = trvEffects);
end;

procedure TabcdfImageEffectChooser.lstSelectionDragDrop(Sender,
  Source: TObject; X, Y: Integer);
begin
  if Source = trvEffects then
    mnuAddtoSelection1Click(Self);
end;

procedure TabcdfImageEffectChooser.spePauseChange(Sender: TObject);
begin
  with abcEffectsImage1 do
    EffectPause := spePause.Value;
end;

end.
