unit abcedbtn;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, ExtCtrls, abcmsg;

type

  TabcAutoClick = (aclEnterKey, aclOnExit, aclOnEnter);

  TabcAutoClicks = set of TabcAutoClick;

  TabcCustomButtonEdit = class(TEdit)
  private
    FCanvas: TCanvas;
{    FGlyph: TBitmap;              }
    FProcessingEnter:Boolean;
    FNoClick:Boolean;
    FButtonWidth: Integer;
    FButtonVisible: Boolean;
    FPressed: Boolean;
    FTracking: Boolean;
    FAutoClick: TabcAutoClicks;
    FButtonDisabled: Boolean;
    FOnButtonClick: TNotifyEvent;
    procedure SetButtonDisabled(Value: Boolean);
    procedure SetButtonVisible(Value: Boolean);
    procedure SetEditRect;
{    procedure SetGlyph(Value: TBitmap);
    procedure GlyphChanged(Sender: TObject);
    function IsGlyphStored: Boolean;              }
    procedure StopTracking;
    procedure TrackButton(X,Y: Integer);
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMPaste(var Message: TWMPaste); message WM_PASTE;
    procedure WMCut(var Message: TWMCut); message WM_CUT;
    procedure WMCancelMode(var Message: TMessage); message WM_CancelMode;
    procedure WMChar(var Message: TWMChar); message WM_CHAR;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message wm_LButtonDblClk;
    procedure WMPaint(var Message: TWMPaint); message WM_Paint;
    procedure WMSetCursor(var Message: TWMSetCursor); message WM_SetCursor;
    procedure AMPROCESSING_ENTER(var Message: TMessage); message AM_PROCESSING_ENTER;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure PaintWindow(DC: HDC); override;
    procedure Paint; virtual;
    procedure CreateWnd; override;
    procedure AdjustEditRect(var R: TRect); dynamic;

    property Canvas: TCanvas read FCanvas;

    {to publish in descendant classes}

    property AutoClick: TabcAutoClicks read FAutoClick write FAutoClick default [];
    property ButtonDisabled: Boolean read FButtonDisabled write SetButtonDisabled default False;
    property ButtonVisible: Boolean read FButtonVisible write SetButtonVisible default True;
{    property Glyph: TBitmap read FGlyph write SetGlyph stored IsGlyphStored;         }
    property OnButtonClick: TNotifyEvent read FOnButtonClick write FOnButtonClick;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ClickButton; virtual;
  end;

  TabcButtonEdit = class(TabcCustomButtonEdit)
  published
    property AutoClick;
    property ButtonDisabled;
    property ButtonVisible;
{    property Glyph;               }
    property OnButtonClick;
  end;

implementation

constructor TabcCustomButtonEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCanvas := TControlCanvas.Create;
  TControlCanvas(FCanvas).Control := Self;
  FButtonVisible := True;
  FButtonWidth := GetSystemMetrics(SM_CXVSCROLL);
{  FGlyph := TBitmap.Create;
  FGlyph.OnChange := GlyphChanged;                 }
end;

destructor TabcCustomButtonEdit.Destroy;
begin
  FCanvas.Free;
{  FGlyph.Free;                                   }
  inherited Destroy;
end;

procedure TabcCustomButtonEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE;
end;

procedure TabcCustomButtonEdit.ClickButton;
begin
  if Assigned(FOnButtonClick) then FOnButtonClick(Self);
end;

procedure TabcCustomButtonEdit.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if (Button = mbLeft) and
    PtInRect(Rect(Width - FButtonWidth, 0, Width, Height), Point(X,Y)) and
    Enabled and not ButtonDisabled and ButtonVisible then
  begin
    MouseCapture := True;
    FTracking := True;
    TrackButton(X, Y);
  end;
  FNoClick:=FProcessingEnter and (aclOnEnter in FAutoClick);
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TabcCustomButtonEdit.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if FTracking then TrackButton(X, Y);
  inherited MouseMove(Shift, X, Y);
end;

procedure TabcCustomButtonEdit.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  WasPressed: Boolean;
begin
  WasPressed := FPressed;
  StopTracking;
  inherited MouseUp(Button, Shift, X, Y);
  if (Button = mbLeft) and WasPressed and not FNoClick then ClickButton;
end;

procedure TabcCustomButtonEdit.PaintWindow(DC: HDC);
var
  R: TRect;
  Flags: Integer;
  BL, BT, W: Integer;
  BR: HBrush;
begin
  FCanvas.Handle := DC;
  if ButtonVisible then
  begin
    BL := ClientWidth - FButtonWidth;
    SetRect(R, BL, 0, ClientWidth, ClientHeight);
    if FPressed then
      Flags := DFCS_BUTTONPUSH or DFCS_FLAT or DFCS_PUSHED
    else
      Flags := DFCS_BUTTONPUSH;
    DrawFrameControl(DC, R, DFC_BUTTON, Flags);
    {draw button glyph or ...}
    {draw ellipsis ...}
    W := FButtonWidth shr 3;
    if W = 0 then W := 1;
    BL := BL + FButtonWidth div 2 - 1;
    BT := ClientHeight - ClientHeight shr 2 - 2;
    if FPressed then
    begin
      Inc(BL);
      Inc(BT);
    end;
    if not Enabled or ButtonDisabled then
      BR := SelectObject(DC, GetStockObject(GRAY_BRUSH))
    else
      BR := SelectObject(DC, GetStockObject(BLACK_BRUSH));
    PatBlt(DC, BL, BT, W, W, PATCOPY);
    PatBlt(DC, BL - (W * 2), BT, W, W, PATCOPY);
    PatBlt(DC, BL + (W * 2), BT, W, W, PATCOPY);
    SelectObject(DC, BR);
    ExcludeClipRect(DC, R.Left, R.Top, R.Right, R.Bottom);
  end;

  try
    Paint;
  finally
    FCanvas.Handle := 0;
  end;

  inherited PaintWindow(DC);

end;

procedure TabcCustomButtonEdit.Paint;
begin
end;

procedure TabcCustomButtonEdit.SetButtonDisabled(Value: Boolean);
begin
  if Value <> FButtonDisabled then
  begin
    FButtonDisabled := Value;
    if Value then StopTracking;
    Invalidate;
  end;
end;

procedure TabcCustomButtonEdit.StopTracking;
begin
  if FTracking then
  begin
    TrackButton(-1, -1);
    FTracking := False;
    MouseCapture := False;
  end;
end;

procedure TabcCustomButtonEdit.WMCancelMode(var Message: TMessage);
begin
  StopTracking;
  inherited;
end;

procedure TabcCustomButtonEdit.WMChar(var Message: TWMChar);
{remove enter key}
begin
  if Message.CharCode = VK_RETURN then
  begin
    if (aclEnterKey in AutoClick) then ClickButton;
  end
  else
    inherited;
end;

procedure TabcCustomButtonEdit.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  with Message do
    if ButtonVisible and
      PtInRect(Rect(Width - FButtonWidth, 0, Width, Height), Point(XPos, YPos))
      then Exit;
  inherited;
end;

procedure TabcCustomButtonEdit.WMPaint(var Message: TWMPaint);
begin
  PaintHandler(Message);
end;

procedure TabcCustomButtonEdit.TrackButton(X,Y: Integer);
var
  NewState: Boolean;
  R: TRect;
begin
  SetRect(R, ClientWidth - FButtonWidth, 0, ClientWidth, ClientHeight);
  NewState := PtInRect(R, Point(X, Y));
  if FPressed <> NewState then
  begin
    FPressed := NewState;
    InvalidateRect(Handle, @R, False);
  end;
end;

procedure TabcCustomButtonEdit.WMSetCursor(var Message: TWMSetCursor);
var
  P: TPoint;
begin
  if ButtonVisible then
  begin
    GetCursorPos(P);
    if PtInRect(Rect(Width - FButtonWidth, 0, Width, Height), ScreenToClient(P)) then
      Windows.SetCursor(LoadCursor(0, idc_Arrow))
    else
      inherited;
  end
  else
    inherited;
end;

procedure TabcCustomButtonEdit.CreateWnd;
begin
  inherited CreateWnd;
  if ButtonVisible then SetEditRect;
end;

procedure TabcCustomButtonEdit.SetEditRect;
var
  R: TRect;
begin
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@R));
  AdjustEditRect(R);
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@R));
end;

procedure TabcCustomButtonEdit.AdjustEditRect(var R: TRect);
{allow descendant classes to adjust the rect if required}
begin
  Dec(R.Right, FButtonWidth);
  if BorderStyle = bsSingle then
    if Ctl3D then
      Dec(R.Left, 1)
    else
      Dec(R.Left, 2);
end;

procedure TabcCustomButtonEdit.WMSize(var Message: TWMSize);
begin
  inherited;
  if ButtonVisible then SetEditRect;
end;

procedure TabcCustomButtonEdit.WMPaste(var Message: TWMPaste);
begin
  if ReadOnly then Exit;
  inherited;
end;

procedure TabcCustomButtonEdit.WMCut(var Message: TWMPaste);
begin
  if ReadOnly then Exit;
  inherited;
end;

procedure TabcCustomButtonEdit.AMPROCESSING_ENTER(var Message: TMessage);
var
  R: TRect;
begin
  FProcessingEnter:=False;
  {workaround for odd windows caret width on multiline edit}
  if Font.Size < 10 then
  begin
    SendMessage(Handle, EM_GETRECT, 0, LongInt(@R));
    DestroyCaret;
    CreateCaret(Handle, 0, 1, R.Bottom - R.Top);
    ShowCaret(Handle);
  end;
  {autoselect}
  if AutoSelect and
    not (csClicked in ControlState) then
  begin
    SelStart := 0;
    SelLength := Length(Text);
  end;
end;

procedure TabcCustomButtonEdit.DoEnter;
begin
  PostMessage(Handle, AM_PROCESSING_ENTER, 0, 0);
  FProcessingEnter:=True;
  inherited DoEnter;
  if (aclOnEnter in AutoClick) then
    ClickButton;
end;

procedure TabcCustomButtonEdit.DoExit;
begin
  DestroyCaret;
  if (aclOnExit in AutoClick) then
   ClickButton;
  inherited DoExit;
end;

procedure TabcCustomButtonEdit.SetButtonVisible(Value: Boolean);
begin
  if Value <> FButtonVisible then
  begin
    FButtonVisible := Value;
    RecreateWnd;
  end;
end;

(*

procedure TabcCustomButtonEdit.GlyphChanged(Sender: TObject);
begin
end;

function TabcCustomButtonEdit.IsGlyphStored: Boolean;
begin
  Result := not FGlyph.Empty; {(GlyphStyle = csCustom);}
end;

procedure TabcCustomButtonEdit.SetGlyph(Value: TBitmap);
begin
  if Value <> FGlyph then
    FGlyph.Assign(Value);
end;

*)

end.
