unit abcedge;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{
Bitmap edge drawing functions
}

interface

uses Windows, Classes, Graphics;

{NB Caller must free Result of all functions to avoid resource leak.}
function abcGetBlackMask(Bitmap: TBitmap): TBitmap;
function abcGetEdgeMask(Bitmap: TBitmap; Highlight, Shadow: TColor): TBitmap;
function abcGetEdgeUpDownBitmap(Bitmap: TBitmap; Highlight, Shadow: TColor): TBitmap;

{NB Caller must free bmpUp and bmpDown of all functions to avoid resource leak.}
procedure abcMakeEdgeBitmaps(Source: TBitmap; Highlight, Shadow: TColor; var bmpUp, bmpDown: TBitmap);
procedure abcMakeOffsetEdgeBitmaps(Source: TBitmap; Highlight, Shadow: TColor; downXOffset, downYOffset: integer; var bmpUp, bmpDown: TBitmap);

{NB Caller must free region}
function abcGetRegionFromBitmap(Source: TBitmap; TransparentTolerance: TColor; OffsetOrigin: TPoint): HRGN;

implementation

function abcGetBlackMask(Bitmap: TBitmap): TBitmap;
{NB Caller must free result.}
{
  Returns a mask with background Bitmap.TransparentColor
  and black fill of the image colors.
}
begin
  Result := TBitmap.Create;
  Result.Assign(Bitmap);
  Result.Mask(Bitmap.TransparentColor);
end;

function abcGetEdgeMask(Bitmap: TBitmap; Highlight, Shadow: TColor): TBitmap;
{NB Caller must free result.}
var
  x, y: integer;
  TransColor: TColor;
  tmp: TBitmap;
begin
  TransColor := Bitmap.TransparentColor;
  Result := TBitmap.Create;
  Result.Assign(Bitmap);
  tmp := TBitmap.Create;
  try
    with tmp do
    begin
      Assign(Bitmap);
      Mask(TransColor);   {white for transcolor, black otherwise.}
      with Canvas do
        for y := 0 to Height-1 do
          for x := 0 to Width-1 do
            if Pixels[x,y] = clWhite then
              if Pixels[x+1,y] = clBlack then   {Left Edge}
                Result.Canvas.Pixels[x,y] := Highlight
              else
                if Pixels[x-1,y] = clBlack then  {Right Edge}
                  Result.Canvas.Pixels[x,y] := Shadow
                else
                  if Pixels[x,y+1] = clBlack then  {Top Edge}
                    Result.Canvas.Pixels[x,y] := Highlight
                  else
                    if Pixels[x,y-1] = clBlack then  {Bottom Edge}
                      Result.Canvas.Pixels[x,y] := Shadow;

    end;
  finally
    tmp.Free;
  end;
end;

function abcGetEdgeUpDownBitmap(Bitmap: TBitmap; Highlight, Shadow: TColor): TBitmap;
{Returns a two part bitmap wih Up/Down images}
{NB Caller must free result.}
var
  x, y: integer;
  TransColor: TColor;
  tmp: TBitmap;
  xOffset: integer;
begin
  TransColor := Bitmap.TransparentColor;
  Result := TBitmap.Create;
  with Result do
  begin
    xOffset := Bitmap.Width;
    Width := xOffset * 2;
    Height := Bitmap.Height;
    Monochrome := Bitmap.Monochrome;
    PixelFormat := Bitmap.PixelFormat;
    HandleType := Bitmap.HandleType;
    TransparentColor := TransColor;
    Palette := Bitmap.Palette;
    Canvas.Brush.Color := TransColor;
    Canvas.FillRect(Rect(0,0,Width,Height));
    Canvas.Draw(0, 0, Bitmap);
    Canvas.Draw(xOffset, 0, Bitmap);
  end;
  tmp := TBitmap.Create;
  try
    with tmp do
    begin
      Assign(Bitmap);
      Mask(TransColor);   {white for transcolor, black otherwise.}
      with Canvas do
        for y := 0 to Height-1 do
          for x := 0 to Width-1 do
            if Pixels[x,y] = clWhite then
              if Pixels[x+1,y] = clBlack then   {Left Edge}
              begin
                Result.Canvas.Pixels[x,y] := Highlight;
                Result.Canvas.Pixels[x+xOffset,y] := Shadow;
              end
              else
                if Pixels[x-1,y] = clBlack then  {Right Edge}
                begin
                  Result.Canvas.Pixels[x,y] := Shadow;
                  Result.Canvas.Pixels[x+xOffset,y] := Highlight;
                end
                else
                  if Pixels[x,y+1] = clBlack then  {Top Edge}
                  begin
                    Result.Canvas.Pixels[x,y] := Highlight;
                    Result.Canvas.Pixels[x+xOffset,y] := Shadow;
                  end
                  else
                    if Pixels[x,y-1] = clBlack then  {Bottom Edge}
                    begin
                      Result.Canvas.Pixels[x,y] := Shadow;
                      Result.Canvas.Pixels[x+xOffset,y] := Highlight;
                    end;
    end;
  finally
    tmp.Free;
  end;
end;

procedure abcMakeEdgeBitmaps(Source: TBitmap; Highlight, Shadow: TColor; var bmpUp, bmpDown: TBitmap);
{NB Caller must free bmpUp and bmpDown to avoid resource leak.}
var
  x, y: integer;
  tmp: TBitmap;
begin
  bmpUp := TBitmap.Create;
  bmpDown := TBitmap.Create;
  bmpUp.Assign(Source);
  bmpDown.Assign(Source);
  tmp := TBitmap.Create;
  try
    with tmp do
    begin
      Assign(Source);
      Mask(TransparentColor);   {white for transcolor, black otherwise.}
      with Canvas do
        for y := 0 to Height-1 do
          for x := 0 to Width-1 do
            if Pixels[x,y] = clWhite then
              if Pixels[x+1,y] = clBlack then   {Left Edge}
              begin
                bmpUp.Canvas.Pixels[x,y] := Highlight;
                bmpDown.Canvas.Pixels[x,y] := Shadow;
              end
              else
                if Pixels[x-1,y] = clBlack then  {Right Edge}
                begin
                  bmpUp.Canvas.Pixels[x,y] := Shadow;
                  bmpDown.Canvas.Pixels[x,y] := Highlight;
                end
                else
                  if Pixels[x,y+1] = clBlack then  {Top Edge}
                  begin
                    bmpUp.Canvas.Pixels[x,y] := Highlight;
                    bmpDown.Canvas.Pixels[x,y] := Shadow;
                  end
                  else
                    if Pixels[x,y-1] = clBlack then  {Bottom Edge}
                    begin
                      bmpUp.Canvas.Pixels[x,y] := Shadow;
                      bmpDown.Canvas.Pixels[x,y] := Highlight;
                    end;
    end;
  finally
    tmp.Free;
  end;
end;

procedure abcMakeOffsetEdgeBitmaps(Source: TBitmap; Highlight, Shadow: TColor;
  downXOffset, downYOffset: integer; var bmpUp, bmpDown: TBitmap);
{NB Caller must free bmpUp and bmpDown to avoid resource leak.}
var
  x, y: integer;
  tmp: TBitmap;
begin
  bmpUp := TBitmap.Create;
  bmpDown := TBitmap.Create;
  bmpUp.Assign(Source);
  with bmpDown do
  begin
    Assign(Source);
    with Canvas do
    begin
      Brush.Style := bsSolid;
      Brush.Color := Source.TransparentColor;
      FillRect(Rect(0,0,Width, Height));
      Draw(downXOffset, downYOffset, Source);
    end;
  end;

  tmp := TBitmap.Create;
  try
    with tmp do
    begin
      Assign(Source);
      Mask(TransparentColor);   {white for transcolor, black otherwise.}
      with Canvas do
        for y := 0 to Height-1 do
          for x := 0 to Width-1 do
            if Pixels[x,y] = clWhite then
              if Pixels[x+1,y] = clBlack then   {Left Edge}
              begin
                bmpUp.Canvas.Pixels[x,y] := Highlight;
                bmpDown.Canvas.Pixels[x+downXOffset,y+downYOffset] := Shadow;
              end
              else
                if Pixels[x-1,y] = clBlack then  {Right Edge}
                begin
                  bmpUp.Canvas.Pixels[x,y] := Shadow;
                  bmpDown.Canvas.Pixels[x+downXOffset,y+downYOffset] := Highlight;
                end
                else
                  if Pixels[x,y+1] = clBlack then  {Top Edge}
                  begin
                    bmpUp.Canvas.Pixels[x,y] := Highlight;
                    bmpDown.Canvas.Pixels[x+downXOffset,y+downYOffset] := Shadow;
                  end
                  else
                    if Pixels[x,y-1] = clBlack then  {Bottom Edge}
                    begin
                      bmpUp.Canvas.Pixels[x,y] := Shadow;
                      bmpDown.Canvas.Pixels[x+downXOffset,y+downYOffset] := Highlight;
                    end;
    end;
  finally
    tmp.Free;
  end;
end;

{ abcGetRegionFromBitmap }
const
  MAXRECT = 2000;

type
  TabcPolyRectRegionData = record
    case Integer of
      0: (Data: TRgnData);
      1: (Header: TRgnDataHeader; RgnRects: array[0..MAXRECT-1] of TRect);
  end;

function abcGetRegionFromBitmap(Source: TBitmap; TransparentTolerance: TColor; OffsetOrigin: TPoint): HRGN;
{NB Caller must free region}
var
  x, x0, y: Integer;
  minR, minG, minB, maxR, maxG, maxB, pixRGB: Byte;
  RgnDataSize: DWORD;
  RgnRectCount: Integer;
  PolyRectRegionData: TabcPolyRectRegionData;
  Rgn, RgnPoints: HRgn;
  hMemDC: HDC;
  bmDIB: BITMAP;
  bmInfo: BITMAPINFO;
  hDIB: HBITMAP;
  pDIBBits: Pointer;
  pPixel: pInteger;
  dibWidthBytes: Integer;
  bUseOffset: Boolean;

  function MinBytes(x, y: Byte): Byte;
  begin
    if x < y then
      Result := x
    else
      Result := y;
  end;

  function IntToByte(i: integer): Byte;
  begin
    if i < 0 then
      Result := 0
    else if i > 255 then
      Result := 255
    else
      Result := i;
  end;

  procedure CombineRgnPoints;
  begin
    CombineRgn(Rgn, Rgn, RgnPoints, RGN_OR);
  	DeleteObject(RgnPoints);
  end;

  procedure MakeRgnPoints;
  begin
    {make region}
    PolyRectRegionData.Header.nCount := RgnRectCount;
    RgnDataSize := SizeOf(TRgnDataHeader) + RgnRectCount * SizeOf(TRect);
    RgnPoints := ExtCreateRegion(nil,RgnDataSize,PolyRectRegionData.Data);
    RgnRectCount := 0;
    if RgnPoints <> 0 then
      if Rgn = 0 then
        Rgn := RgnPoints
      else
        CombineRgnPoints;
    RgnPoints := 0;
  end;

  procedure AddRectToRgn(L,T,R,B: Integer);
  var
    rTemp: TRect;
  begin
    rTemp := Rect(L,T,R,B);
    if bUseOffset then
      OffsetRect(rTemp, OffsetOrigin.X, OffsetOrigin.Y);
    PolyRectRegionData.RgnRects[RgnRectCount] := rTemp;
    RgnRectCount := RgnRectCount + 1;
    if RgnRectCount = MAXRECT then
      MakeRgnPoints;
  end;

begin
  Result := 0;
  RgnRectCount := 0;
  Rgn := 0;
  RgnPoints := 0;
  bUseOffset := (OffsetOrigin.X <> 0) or (OffsetOrigin.Y <> 0);

  if Source = nil then Exit;

  with PolyRectRegionData.Header do
  begin
    dwSize := SizeOf(TRgnDataHeader);
    iType := RDH_RECTANGLES;
    nCount := 0;
    nRgnSize := 0;
    rcBound := Rect(0,0,0,0);
  end;

  with Source do
  begin
    minR := IntToByte(GetRValue(DWORD(TransparentColor)) - GetRValue(DWORD(TransparentTolerance)));
    minG := IntToByte(GetGValue(DWORD(TransparentColor)) - GetGValue(DWORD(TransparentTolerance)));
    minB := IntToByte(GetBValue(DWORD(TransparentColor)) - GetBValue(DWORD(TransparentTolerance)));
    maxR := IntToByte(minR + 2 * GetRValue(DWORD(TransparentTolerance)));
    maxG := IntToByte(minG + 2 * GetGValue(DWORD(TransparentTolerance)));
    maxB := IntToByte(minB + 2 * GetBValue(DWORD(TransparentTolerance)));

    //copy bitmap to 32 bit DIB

    hMemDC := CreateCompatibleDC(0);
    if hMemDC = 0 then Exit;
    hDIB := 0;
    try

      with bmInfo.bmiHeader do
      begin

        biSize := SizeOf(BITMAPINFOHEADER);
        biWidth := Width;
        biHeight := Height;
        biPlanes := 1;
        biBitCount := 32;
        biCompression := BI_RGB;
        biSizeImage := 0;
        biXPelsPerMeter := 0;
        biYPelsPerMeter := 0;
        biClrUsed := 0;
        biClrImportant := 0;
      end;

      hDIB := CreateDIBSection(hMemDC, bmInfo, DIB_RGB_COLORS, pDIBBits, 0, 0);
      if hDIB = 0 then Exit;
      GDIFlush;
      SelectObject(hMemDC, hDIB);
      GetObject(hDIB, SizeOf(BITMAP), @bmDIB);
      while (bmDIB.bmWidthBytes mod 4) > 0 do
        inc(bmDIB.bmWidthBytes);
      BitBlt(hMemDC, 0, 0, Width, Height, Canvas.Handle, 0, 0, SRCCOPY);

      with bmDIB do
      begin
        pDIBBits := bmBits;
        dibWidthBytes := bmWidthBytes;
        inc(PChar(pDIBBits), (bmHeight - 1) * bmWidthBytes);
      end;

      //process DIB pixels
      //bitmap is upside down and color values are reversed

      for y := 0 to Height - 1 do
      begin
        x := -1;
        while x + 1 < Width do
        begin
          x := x + 1;

          x0 := x;
          pPixel := pDIBBits;
          Inc(PChar(pPixel), x * SizeOf(Integer));
          while x < Width do
          begin
            pixRGB := GetBValue(DWORD(pPixel^));
            if (pixRGB >= minR) and (pixRGB <= maxR) then
            begin
              pixRGB := GetGValue(DWORD(pPixel^));
              if (pixRGB >= minG) and (pixRGB <= maxG) then
              begin
                pixRGB := GetRValue(DWORD(pPixel^));
                if (pixRGB >= minB) and (pixRGB <= maxB) then
                  Break;
              end;
            end;
            Inc(PChar(pPixel), SizeOf(LongInt));
            x := x + 1;
          end;
          if x > x0 then
            AddRecttoRgn(x0,y,x,y+1);
        end;
        Dec(PChar(pDIBBits), dibWidthBytes);
      end;
    finally
      DeleteDC(hMemDC);
      if hDIB <> 0 then
        DeleteObject(hDIB);
    end;
  end;

  {add remaining points to Rgn}
  if RgnRectCount > 0 then
    MakeRgnPoints;
  Result := Rgn;
end;

end.
