unit abcedtdr;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, abcedbtn, abcfdir;

type

  TabcDirDialogWrapper = class(TPersistent)
  private
    FDirDialog: TabcDirectoryDialog;
    FExecuteResult: Boolean;
    function GetCtl3D: Boolean;
    procedure SetCtl3D(Value: Boolean);
    function GetDirButtonStyle: TabcDirButtonStyle;
    procedure SetDirButtonStyle(Value: TabcDirButtonStyle);
    function GetDirDialogStyle: TabcDirDialogStyle;
    procedure SetDirDialogStyle(Value: TabcDirDialogStyle);
    function GetDirectory: TFileName;
    procedure SetDirectory(Value: TFileName);
    function GetDirEditStyle: TabcDirEditStyle;
    procedure SetDirEditStyle(Value: TabcDirEditStyle);
    function GetDirLabelStyle: TabcDirLabelStyle;
    procedure SetDirLabelStyle(Value: TabcDirLabelStyle);
    function GetHistoryList: TStrings;
    procedure SetHistoryList(Value: TStrings);
    function GetHelpContext: THelpContext;
    procedure SetHelpContext(Value: THelpContext);
    function GetOptions: TabcDirectoryOptions;
    procedure SetOptions(Value: TabcDirectoryOptions);
    function GetTag: Integer;
    procedure SetTag(Value: Integer);
    function GetTitle: string;
    procedure SetTitle(Value: string);
  public
    constructor Create;
    destructor Destroy; override;
    property Dialog: TabcDirectoryDialog read FDirDialog;
    function Execute: Boolean;
    property ExecuteResult: Boolean read FExecuteResult;
  published
    property Ctl3D: Boolean read GetCtl3D write SetCtl3D default True;
    property DirButtonStyle: TabcDirButtonStyle read GetDirButtonStyle write SetDirButtonStyle
      default dbsButton;
    property DirDialogStyle: TabcDirDialogStyle read GetDirDialogStyle write SetDirDialogStyle
      default ddsBtnBottom;
    property Directory: TFileName read GetDirectory write SetDirectory;
    property DirEditStyle: TabcDirEditStyle read GetDirEditStyle write SetDirEditStyle
      default desLabel;
    property DirLabelStyle: TabcDirLabelStyle read GetDirLabelStyle write SetDirLabelStyle
      default dlsFolder;
    property HelpContext: THelpContext read GetHelpContext write SetHelpContext default 0;
    property HistoryList: TStrings read GetHistoryList write SetHistoryList;
    property Options: TabcDirectoryOptions read GetOptions write SetOptions
      default [adrPathMustExist, adrUpdateHistory];
    property Tag: Integer read GetTag write SetTag default 0;
    property Title: string read GetTitle write SetTitle;
  end;

  TabcDirectoryEdit = class(TabcCustomButtonEdit)
  private
    { Private declarations }
    FDirDialog: TabcDirDialogWrapper;
    FAfterExecute: TNotifyEvent;
    FBeforeExecute: TNotifyEvent;
    function GetClickOnEnterKey: Boolean;
    procedure SetClickOnEnterKey(Value: Boolean);
  public
    procedure ClickButton; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ButtonDisabled;
    property ButtonVisible;
    property ClickOnEnterKey: Boolean read GetClickOnEnterKey write SetClickOnEnterKey default True;
    property DirDialog: TabcDirDialogWrapper read FDirDialog write FDirDialog;
    property AfterExecute: TNotifyEvent read FAfterExecute write FAfterExecute;
    property BeforeExecute: TNotifyEvent read FBeforeExecute write FBeforeExecute;
  end;

implementation

{ TabcDirDialogWrapper }

constructor TabcDirDialogWrapper.Create;
begin
  FDirDialog := TabcDirectoryDialog.Create(nil);
end;

destructor TabcDirDialogWrapper.Destroy;
begin
  FDirDialog.Free;
  inherited Destroy;
end;

function TabcDirDialogWrapper.Execute: Boolean;
begin
  Result := FDirDialog.Execute;
  FExecuteResult := Result;
end;

function TabcDirDialogWrapper.GetCtl3D: Boolean;
begin
  Result := FDirDialog.Ctl3D;
end;

procedure TabcDirDialogWrapper.SetCtl3D(Value: Boolean);
begin
  FDirDialog.Ctl3D := Value;
end;

function TabcDirDialogWrapper.GetDirButtonStyle: TabcDirButtonStyle;
begin
  Result := FDirDialog.DirButtonStyle;
end;

procedure TabcDirDialogWrapper.SetDirButtonStyle(Value: TabcDirButtonStyle);
begin
  FDirDialog.DirButtonStyle := Value;
end;

function TabcDirDialogWrapper.GetDirDialogStyle: TabcDirDialogStyle;
begin
  Result := FDirDialog.DirDialogStyle;
end;

procedure TabcDirDialogWrapper.SetDirDialogStyle(Value: TabcDirDialogStyle);
begin
  FDirDialog.DirDialogStyle := Value;
end;

function TabcDirDialogWrapper.GetDirectory: TFileName;
begin
  Result := FDirDialog.Directory;
end;

procedure TabcDirDialogWrapper.SetDirectory(Value: TFileName);
begin
  FDirDialog.Directory := Value;
end;

function TabcDirDialogWrapper.GetDirEditStyle: TabcDirEditStyle;
begin
  Result := FDirDialog.DirEditStyle;
end;

procedure TabcDirDialogWrapper.SetDirEditStyle(Value: TabcDirEditStyle);
begin
  FDirDialog.DirEditStyle := Value;
end;

function TabcDirDialogWrapper.GetDirLabelStyle: TabcDirLabelStyle;
begin
  Result := FDirDialog.DirLabelStyle;
end;

procedure TabcDirDialogWrapper.SetDirLabelStyle(Value: TabcDirLabelStyle);
begin
  FDirDialog.DirLabelStyle := Value;
end;

function TabcDirDialogWrapper.GetHistoryList: TStrings;
begin
  Result := FDirDialog.HistoryList;
end;

procedure TabcDirDialogWrapper.SetHistoryList(Value: TStrings);
begin
  FDirDialog.HistoryList := Value;
end;

function TabcDirDialogWrapper.GetHelpContext: THelpContext;
begin
  Result := FDirDialog.HelpContext;
end;

procedure TabcDirDialogWrapper.SetHelpContext(Value: THelpContext);
begin
  FDirDialog.HelpContext := Value;
end;

function TabcDirDialogWrapper.GetOptions: TabcDirectoryOptions;
begin
  Result := FDirDialog.Options;
end;

procedure TabcDirDialogWrapper.SetOptions(Value: TabcDirectoryOptions);
begin
  FDirDialog.Options := Value;
end;

function TabcDirDialogWrapper.GetTag: Integer;
begin
  Result := FDirDialog.Tag;
end;

procedure TabcDirDialogWrapper.SetTag(Value: Integer);
begin
  FDirDialog.Tag := Value;
end;

function TabcDirDialogWrapper.GetTitle: string;
begin
  Result := FDirDialog.Title;
end;

procedure TabcDirDialogWrapper.SetTitle(Value: string);
begin
  FDirDialog.Title := Value;
end;

{ TabcDirectoryEdit }

constructor TabcDirectoryEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FDirDialog := TabcDirDialogWrapper.Create;
  ClickOnEnterKey := True;
end;

destructor TabcDirectoryEdit.Destroy;
begin
  FDirDialog.Free;
  inherited Destroy;
end;

procedure TabcDirectoryEdit.ClickButton;
begin
  if Assigned(FBeforeExecute) then FBeforeExecute(Self);
  if DirDialog.Execute then Text := DirDialog.Directory;
  if Assigned(FAfterExecute) then FAfterExecute(Self);
end;

function TabcDirectoryEdit.GetClickOnEnterKey: Boolean;
begin
  Result := aclEnterKey in inherited AutoClick;
end;

procedure TabcDirectoryEdit.SetClickOnEnterKey(Value: Boolean);
begin
  if Value then
    inherited AutoClick := [aclEnterKey]
  else
    inherited AutoClick := [];
end;

end.
