unit abcedtop;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, abcedbtn;

type

  TabcOpenDialogWrapper = class(TPersistent)
  private
    FExecuteResult: Boolean;
    FOpenDialog: TOpenDialog;
    function GetCtl3D: Boolean;
    procedure SetCtl3D(Value: Boolean);
    function GetDefaultExt: string;
    procedure SetDefaultExt(Value: string);
    function GetFileName: TFileName;
    procedure SetFileName(Value: TFileName);
    function GetFilter: string;
    procedure SetFilter(Value: string);
    function GetFilterIndex: Integer;
    procedure SetFilterIndex(Value: Integer);
    function GetHelpContext: THelpContext;
    procedure SetHelpContext(Value: THelpContext);
    function GetInitialDir: string;
    procedure SetInitialDir(Value: string);
    function GetOptions: TOpenOptions;
    procedure SetOptions(Value: TOpenOptions);
    function GetTag: Integer;
    procedure SetTag(Value: Integer);
    function GetTitle: string;
    procedure SetTitle(Value: string);
{$IFDEF DELPHI6}
    function GetOptionsEx: TOpenOptionsEx;
    procedure SetOptionsEx(Value: TOpenOptionsEx);
{$ENDIF}
  public
    constructor Create;
    destructor Destroy; override;
    property Dialog: TOpenDialog read FOpenDialog;
    function Execute: Boolean;
    property ExecuteResult: Boolean read FExecuteResult;
  published
    property Ctl3D: Boolean read GetCtl3D write SetCtl3D default True;
    property DefaultExt: string read GetDefaultExt write SetDefaultExt;
    property FileName: TFileName read GetFileName write SetFileName;
    property Filter: string read GetFilter write SetFilter;
    property FilterIndex: Integer read GetFilterIndex write SetFilterIndex default 1;
    property HelpContext: THelpContext read GetHelpContext write SetHelpContext default 0;
    property InitialDir: string read GetInitialDir write SetInitialDir;
    property Options: TOpenOptions read GetOptions write SetOptions default [ofHideReadOnly];
{$IFDEF DELPHI6}
    property OptionsEx: TOpenOptionsEx read GetOptionsEx write SetOptionsEx default [];
{$ENDIF}
    property Tag: Integer read GetTag write SetTag default 0;
    property Title: string read GetTitle write SetTitle;
  end;

  TabcFileOpenEdit = class(TabcCustomButtonEdit)
  private
    { Private declarations }
    FOpenDialog: TabcOpenDialogWrapper;
    FAfterExecute: TNotifyEvent;
    FBeforeExecute: TNotifyEvent;
    function GetClickOnEnterKey: Boolean;
    procedure SetClickOnEnterKey(Value: Boolean);
    function GetOnDlgClose: TNotifyEvent;
    procedure SetOnDlgClose(Value: TNotifyEvent);
    function GetOnDlgShow: TNotifyEvent;
    procedure SetOnDlgShow(Value: TNotifyEvent);
    function GetOnDlgCanClose: TCloseQueryEvent;
    procedure SetOnDlgCanClose(Value: TCloseQueryEvent);
    function GetOnDlgFolderChange: TNotifyEvent;
    procedure SetOnDlgFolderChange(Value: TNotifyEvent);
    function GetOnDlgSelectionChange: TNotifyEvent;
    procedure SetOnDlgSelectionChange(Value: TNotifyEvent);
    function GetOnDlgTypeChange: TNotifyEvent;
    procedure SetOnDlgTypeChange(Value: TNotifyEvent);
{$IFDEF DELPHI5}
    function GetOnDlgIncludeItem: TIncludeItemEvent;
    procedure SetOnDlgIncludeItem(Value: TIncludeItemEvent);
{$ENDIF}
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure ClickButton; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property ButtonDisabled;
    property ButtonVisible;
    property ClickOnEnterKey: Boolean read GetClickOnEnterKey write SetClickOnEnterKey default True;
    property OpenDialog: TabcOpenDialogWrapper read FOpenDialog write FOpenDialog;
    property AfterExecute: TNotifyEvent read FAfterExecute write FAfterExecute;
    property BeforeExecute: TNotifyEvent read FBeforeExecute write FBeforeExecute;
    {Dialog Events}
    property OnDlgClose: TNotifyEvent read GetOnDlgClose write SetOnDlgClose;
    property OnDlgShow: TNotifyEvent read GetOnDlgShow write SetOnDlgShow;
    property OnDlgFolderChange: TNotifyEvent read GetOnDlgFolderChange write SetOnDlgFolderChange;
    property OnDlgCanClose: TCloseQueryEvent read GetOnDlgCanClose write SetOnDlgCanClose;
{$IFDEF DELPHI5}
    property OnDlgIncludeItem: TIncludeItemEvent read GetOnDlgIncludeItem write SetOnDlgIncludeItem;
{$ENDIF}
    property OnDlgSelectionChange: TNotifyEvent read GetOnDlgSelectionChange write SetOnDlgSelectionChange;
    property OnDlgTypeChange: TNotifyEvent read GetOnDlgTypeChange write SetOnDlgTypeChange;
  end;

implementation

{ TabcOpenDialogWrapper }

constructor TabcOpenDialogWrapper.Create;
begin
  FOpenDialog := TOpenDialog.Create(nil);
  Filter := 'All Files (*.*)|*.*';
end;

destructor TabcOpenDialogWrapper.Destroy;
begin
  FOpenDialog.Free;
  inherited Destroy;
end;

function TabcOpenDialogWrapper.Execute: Boolean;
begin
  Result := FOpenDialog.Execute;
  FExecuteResult := Result;
end;

function TabcOpenDialogWrapper.GetCtl3D: Boolean;
begin
  Result := FOpenDialog.Ctl3D;
end;

procedure TabcOpenDialogWrapper.SetCtl3D(Value: Boolean);
begin
  FOpenDialog.Ctl3D := Value;
end;

function TabcOpenDialogWrapper.GetDefaultExt: string;
begin
  Result := FOpenDialog.DefaultExt;
end;

procedure TabcOpenDialogWrapper.SetDefaultExt(Value: string);
begin
  FOpenDialog.DefaultExt := Value;
end;

function TabcOpenDialogWrapper.GetFileName: TFileName;
begin
  Result := FOpenDialog.FileName;
end;

procedure TabcOpenDialogWrapper.SetFileName(Value: TFileName);
begin
  FOpenDialog.FileName := Value;
end;

function TabcOpenDialogWrapper.GetFilter: string;
begin
  Result := FOpenDialog.Filter;
end;

procedure TabcOpenDialogWrapper.SetFilter(Value: string);
begin
  FOpenDialog.Filter := Value;
end;

function TabcOpenDialogWrapper.GetFilterIndex: Integer;
begin
  Result := FOpenDialog.FilterIndex;
end;

procedure TabcOpenDialogWrapper.SetFilterIndex(Value: Integer);
begin
  FOpenDialog.FilterIndex := Value;
end;

function TabcOpenDialogWrapper.GetHelpContext: THelpContext;
begin
  Result := FOpenDialog.HelpContext;
end;

procedure TabcOpenDialogWrapper.SetHelpContext(Value: THelpContext);
begin
  FOpenDialog.HelpContext := Value;
end;

function TabcOpenDialogWrapper.GetInitialDir: string;
begin
  Result := FOpenDialog.InitialDir;
end;

procedure TabcOpenDialogWrapper.SetInitialDir(Value: string);
begin
  FOpenDialog.InitialDir := Value;
end;

function TabcOpenDialogWrapper.GetOptions: TOpenOptions;
begin
  Result := FOpenDialog.Options;
end;

procedure TabcOpenDialogWrapper.SetOptions(Value: TOpenOptions);
begin
  FOpenDialog.Options := Value;
end;

{$IFDEF DELPHI6}
function TabcOpenDialogWrapper.GetOptionsEx: TOpenOptionsEx;
begin
  Result := FOpenDialog.OptionsEx;
end;

procedure TabcOpenDialogWrapper.SetOptionsEx(Value: TOpenOptionsEx);
begin
  FOpenDialog.OptionsEx := Value;
end;
{$ENDIF}

function TabcOpenDialogWrapper.GetTag: Integer;
begin
  Result := FOpenDialog.Tag;
end;

procedure TabcOpenDialogWrapper.SetTag(Value: Integer);
begin
  FOpenDialog.Tag := Value;
end;

function TabcOpenDialogWrapper.GetTitle: string;
begin
  Result := FOpenDialog.Title;
end;

procedure TabcOpenDialogWrapper.SetTitle(Value: string);
begin
  FOpenDialog.Title := Value;
end;

{ TabcFileOpenEdit }

constructor TabcFileOpenEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FOpenDialog := TabcOpenDialogWrapper.Create;
  ClickOnEnterKey := True;
end;

destructor TabcFileOpenEdit.Destroy;
begin
  FOpenDialog.Free;
  inherited Destroy;
end;

procedure TabcFileOpenEdit.ClickButton;
begin
  if Assigned(FBeforeExecute) then FBeforeExecute(Self);
  if OpenDialog.Execute then Text := OpenDialog.FileName;
  if Assigned(FAfterExecute) then FAfterExecute(Self);
end;

function TabcFileOpenEdit.GetClickOnEnterKey: Boolean;
begin
  Result := aclEnterKey in inherited AutoClick;
end;

procedure TabcFileOpenEdit.SetClickOnEnterKey(Value: Boolean);
begin
  if Value then
    inherited AutoClick := [aclEnterKey]
  else
    inherited AutoClick := [];
end;

function TabcFileOpenEdit.GetOnDlgClose: TNotifyEvent;
begin
  Result := FOpenDialog.Dialog.OnClose;
end;

procedure TabcFileOpenEdit.SetOnDlgClose(Value: TNotifyEvent);
begin
  FOpenDialog.Dialog.OnClose := Value;
end;

function TabcFileOpenEdit.GetOnDlgShow: TNotifyEvent;
begin
  Result := FOpenDialog.Dialog.OnShow;
end;

procedure TabcFileOpenEdit.SetOnDlgShow(Value: TNotifyEvent);
begin
  FOpenDialog.Dialog.OnShow := Value;
end;

function TabcFileOpenEdit.GetOnDlgCanClose: TCloseQueryEvent;
begin
  Result := FOpenDialog.Dialog.OnCanClose;
end;

procedure TabcFileOpenEdit.SetOnDlgCanClose(Value: TCloseQueryEvent);
begin
  FOpenDialog.Dialog.OnCanClose := Value;
end;

function TabcFileOpenEdit.GetOnDlgFolderChange: TNotifyEvent;
begin
  Result := FOpenDialog.Dialog.OnFolderChange;
end;

procedure TabcFileOpenEdit.SetOnDlgFolderChange(Value: TNotifyEvent);
begin
  FOpenDialog.Dialog.OnFolderChange := Value;
end;

function TabcFileOpenEdit.GetOnDlgSelectionChange: TNotifyEvent;
begin
  Result := FOpenDialog.Dialog.OnSelectionChange;
end;

procedure TabcFileOpenEdit.SetOnDlgSelectionChange(Value: TNotifyEvent);
begin
  FOpenDialog.Dialog.OnSelectionChange := Value;
end;

function TabcFileOpenEdit.GetOnDlgTypeChange: TNotifyEvent;
begin
  Result := FOpenDialog.Dialog.OnTypeChange;
end;

procedure TabcFileOpenEdit.SetOnDlgTypeChange(Value: TNotifyEvent);
begin
  FOpenDialog.Dialog.OnTypeChange := Value;
end;

{$IFDEF DELPHI5}
function TabcFileOpenEdit.GetOnDlgIncludeItem: TIncludeItemEvent;
begin
  Result := FOpenDialog.Dialog.OnIncludeItem;
end;

procedure TabcFileOpenEdit.SetOnDlgIncludeItem(Value: TIncludeItemEvent);
begin
  FOpenDialog.Dialog.OnIncludeItem := Value;
end;
{$ENDIF}

end.
