unit abcedtsv;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, abcedbtn;

type

  TabcSaveDialogWrapper = class(TPersistent)
  private
    FExecuteResult: Boolean;
    FSaveDialog: TSaveDialog;
    function GetCtl3D: Boolean;
    procedure SetCtl3D(Value: Boolean);
    function GetDefaultExt: string;
    procedure SetDefaultExt(Value: string);
    function GetFileName: TFileName;
    procedure SetFileName(Value: TFileName);
    function GetFilter: string;
    procedure SetFilter(Value: string);
    function GetFilterIndex: Integer;
    procedure SetFilterIndex(Value: Integer);
    function GetHelpContext: THelpContext;
    procedure SetHelpContext(Value: THelpContext);
    function GetInitialDir: string;
    procedure SetInitialDir(Value: string);
    function GetOptions: TOpenOptions;
    procedure SetOptions(Value: TOpenOptions);
    function GetTag: Integer;
    procedure SetTag(Value: Integer);
    function GetTitle: string;
    procedure SetTitle(Value: string);
{$IFDEF DELPHI6}
    function GetOptionsEx: TOpenOptionsEx;
    procedure SetOptionsEx(Value: TOpenOptionsEx);
{$ENDIF}
  public
    constructor Create;
    destructor Destroy; override;
    property Dialog: TSaveDialog read FSaveDialog;
    function Execute: Boolean;
    property ExecuteResult: Boolean read FExecuteResult;
  published
    property Ctl3D: Boolean read GetCtl3D write SetCtl3D default True;
    property DefaultExt: string read GetDefaultExt write SetDefaultExt;
    property FileName: TFileName read GetFileName write SetFileName;
    property Filter: string read GetFilter write SetFilter;
    property FilterIndex: Integer read GetFilterIndex write SetFilterIndex default 1;
    property HelpContext: THelpContext read GetHelpContext write SetHelpContext default 0;
    property InitialDir: string read GetInitialDir write SetInitialDir;
    property Options: TOpenOptions read GetOptions write SetOptions default [ofHideReadOnly];
{$IFDEF DELPHI6}
    property OptionsEx: TOpenOptionsEx read GetOptionsEx write SetOptionsEx default [];
{$ENDIF}
    property Tag: Integer read GetTag write SetTag default 0;
    property Title: string read GetTitle write SetTitle;
  end;

  TabcFileSaveEdit = class(TabcCustomButtonEdit)
  private
    { Private declarations }
    FSaveDialog: TabcSaveDialogWrapper;
    FAfterExecute: TNotifyEvent;
    FBeforeExecute: TNotifyEvent;
    function GetClickOnEnterKey: Boolean;
    procedure SetClickonEnterKey(Value: Boolean);
    function GetOnDlgClose: TNotifyEvent;
    procedure SetOnDlgClose(Value: TNotifyEvent);
    function GetOnDlgShow: TNotifyEvent;
    procedure SetOnDlgShow(Value: TNotifyEvent);
    function GetOnDlgCanClose: TCloseQueryEvent;
    procedure SetOnDlgCanClose(Value: TCloseQueryEvent);
    function GetOnDlgFolderChange: TNotifyEvent;
    procedure SetOnDlgFolderChange(Value: TNotifyEvent);
    function GetOnDlgSelectionChange: TNotifyEvent;
    procedure SetOnDlgSelectionChange(Value: TNotifyEvent);
    function GetOnDlgTypeChange: TNotifyEvent;
    procedure SetOnDlgTypeChange(Value: TNotifyEvent);
{$IFDEF DELPHI5}
    function GetOnDlgIncludeItem: TIncludeItemEvent;
    procedure SetOnDlgIncludeItem(Value: TIncludeItemEvent);
{$ENDIF}
  public
    { Public declarations }
    procedure ClickButton; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property ButtonDisabled;
    property ButtonVisible;
    property ClickOnEnterKey: Boolean read GetClickOnEnterKey write SetClickOnEnterKey default True;
    property SaveDialog: TabcSaveDialogWrapper read FSaveDialog write FSaveDialog;
    property AfterExecute: TNotifyEvent read FAfterExecute write FAfterExecute;
    property BeforeExecute: TNotifyEvent read FBeforeExecute write FBeforeExecute;
    property OnDlgClose: TNotifyEvent read GetOnDlgClose write SetOnDlgClose;
    property OnDlgShow: TNotifyEvent read GetOnDlgShow write SetOnDlgShow;
    property OnDlgCanClose: TCloseQueryEvent read GetOnDlgCanClose write SetOnDlgCanClose;
{$IFDEF DELPHI5}
    property OnDlgIncludeItem: TIncludeItemEvent read GetOnDlgIncludeItem write SetOnDlgIncludeItem;
{$ENDIF}
    property OnDlgFolderChange: TNotifyEvent read GetOnDlgFolderChange write SetOnDlgFolderChange;
    property OnDlgSelectionChange: TNotifyEvent read GetOnDlgSelectionChange write SetOnDlgSelectionChange;
    property OnDlgTypeChange: TNotifyEvent read GetOnDlgTypeChange write SetOnDlgTypeChange;
  end;

implementation

{ TabcSaveDialogWrapper }

constructor TabcSaveDialogWrapper.Create;
begin
  FSaveDialog := TSaveDialog.Create(nil);
  Filter := 'All Files (*.*)|*.*';
end;

destructor TabcSaveDialogWrapper.Destroy;
begin
  FSaveDialog.Free;
  inherited Destroy;
end;

function TabcSaveDialogWrapper.Execute: Boolean;
begin
  Result := FSaveDialog.Execute;
  FExecuteResult := Result;
end;

function TabcSaveDialogWrapper.GetCtl3D: Boolean;
begin
  Result := FSaveDialog.Ctl3D;
end;

procedure TabcSaveDialogWrapper.SetCtl3D(Value: Boolean);
begin
  FSaveDialog.Ctl3D := Value;
end;

function TabcSaveDialogWrapper.GetDefaultExt: string;
begin
  Result := FSaveDialog.DefaultExt;
end;

procedure TabcSaveDialogWrapper.SetDefaultExt(Value: string);
begin
  FSaveDialog.DefaultExt := Value;
end;

function TabcSaveDialogWrapper.GetFileName: TFileName;
begin
  Result := FSaveDialog.FileName;
end;

procedure TabcSaveDialogWrapper.SetFileName(Value: TFileName);
begin
  FSaveDialog.FileName := Value;
end;

function TabcSaveDialogWrapper.GetFilter: string;
begin
  Result := FSaveDialog.Filter;
end;

procedure TabcSaveDialogWrapper.SetFilter(Value: string);
begin
  FSaveDialog.Filter := Value;
end;

function TabcSaveDialogWrapper.GetFilterIndex: Integer;
begin
  Result := FSaveDialog.FilterIndex;
end;

procedure TabcSaveDialogWrapper.SetFilterIndex(Value: Integer);
begin
  FSaveDialog.FilterIndex := Value;
end;

function TabcSaveDialogWrapper.GetHelpContext: THelpContext;
begin
  Result := FSaveDialog.HelpContext;
end;

procedure TabcSaveDialogWrapper.SetHelpContext(Value: THelpContext);
begin
  FSaveDialog.HelpContext := Value;
end;

function TabcSaveDialogWrapper.GetInitialDir: string;
begin
  Result := FSaveDialog.InitialDir;
end;

procedure TabcSaveDialogWrapper.SetInitialDir(Value: string);
begin
  FSaveDialog.InitialDir := Value;
end;

function TabcSaveDialogWrapper.GetOptions: TOpenOptions;
begin
  Result := FSaveDialog.Options;
end;

procedure TabcSaveDialogWrapper.SetOptions(Value: TOpenOptions);
begin
  FSaveDialog.Options := Value;
end;

{$IFDEF DELPHI6}
function TabcSaveDialogWrapper.GetOptionsEx: TOpenOptionsEx;
begin
  Result := FSaveDialog.OptionsEx;
end;

procedure TabcSaveDialogWrapper.SetOptionsEx(Value: TOpenOptionsEx);
begin
  FSaveDialog.OptionsEx := Value;
end;
{$ENDIF}

function TabcSaveDialogWrapper.GetTag: Integer;
begin
  Result := FSaveDialog.Tag;
end;

procedure TabcSaveDialogWrapper.SetTag(Value: Integer);
begin
  FSaveDialog.Tag := Value;
end;

function TabcSaveDialogWrapper.GetTitle: string;
begin
  Result := FSaveDialog.Title;
end;

procedure TabcSaveDialogWrapper.SetTitle(Value: string);
begin
  FSaveDialog.Title := Value;
end;

{ TabcFileSaveEdit }

constructor TabcFileSaveEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FSaveDialog := TabcSaveDialogWrapper.Create;
  ClickOnEnterKey := True;
end;

destructor TabcFileSaveEdit.Destroy;
begin
  FSaveDialog.Free;
  inherited Destroy;
end;

procedure TabcFileSaveEdit.ClickButton;
begin
  if Assigned(FBeforeExecute) then FBeforeExecute(Self);
  if SaveDialog.Execute then Text := SaveDialog.FileName;
  if Assigned(FAfterExecute) then FAfterExecute(Self);
end;

function TabcFileSaveEdit.GetClickOnEnterKey: Boolean;
begin
  Result := aclEnterKey in inherited AutoClick;
end;

procedure TabcFileSaveEdit.SetClickOnEnterKey(Value: Boolean);
begin
  if Value then
    inherited AutoClick := [aclEnterKey]
  else
    inherited AutoClick := [];
end;

function TabcFileSaveEdit.GetOnDlgClose: TNotifyEvent;
begin
  Result := FSaveDialog.Dialog.OnClose;
end;

procedure TabcFileSaveEdit.SetOnDlgClose(Value: TNotifyEvent);
begin
  FSaveDialog.Dialog.OnClose := Value;
end;

function TabcFileSaveEdit.GetOnDlgShow: TNotifyEvent;
begin
  Result := FSaveDialog.Dialog.OnShow;
end;

procedure TabcFileSaveEdit.SetOnDlgShow(Value: TNotifyEvent);
begin
  FSaveDialog.Dialog.OnShow := Value;
end;

function TabcFileSaveEdit.GetOnDlgCanClose: TCloseQueryEvent;
begin
  Result := FSaveDialog.Dialog.OnCanClose;
end;

procedure TabcFileSaveEdit.SetOnDlgCanClose(Value: TCloseQueryEvent);
begin
  FSaveDialog.Dialog.OnCanClose := Value;
end;

function TabcFileSaveEdit.GetOnDlgFolderChange: TNotifyEvent;
begin
  Result := FSaveDialog.Dialog.OnFolderChange;
end;

procedure TabcFileSaveEdit.SetOnDlgFolderChange(Value: TNotifyEvent);
begin
  FSaveDialog.Dialog.OnFolderChange := Value;
end;

function TabcFileSaveEdit.GetOnDlgSelectionChange: TNotifyEvent;
begin
  Result := FSaveDialog.Dialog.OnSelectionChange;
end;

procedure TabcFileSaveEdit.SetOnDlgSelectionChange(Value: TNotifyEvent);
begin
  FSaveDialog.Dialog.OnSelectionChange := Value;
end;

function TabcFileSaveEdit.GetOnDlgTypeChange: TNotifyEvent;
begin
  Result := FSaveDialog.Dialog.OnTypeChange;
end;

procedure TabcFileSaveEdit.SetOnDlgTypeChange(Value: TNotifyEvent);
begin
  FSaveDialog.Dialog.OnTypeChange := Value;
end;

{$IFDEF DELPHI5}
function TabcFileSaveEdit.GetOnDlgIncludeItem: TIncludeItemEvent;
begin
  Result := FSaveDialog.Dialog.OnIncludeItem;
end;

procedure TabcFileSaveEdit.SetOnDlgIncludeItem(Value: TIncludeItemEvent);
begin
  FSaveDialog.Dialog.OnIncludeItem := Value;
end;
{$ENDIF}

end.
