unit abcedwin;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}


interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  StdCtrls, DB, DBCtrls, mask, menus, abcdbaut, abcforms,
  abcscrol;

type

  { TabcDBEditWinDataLink }

  TabcCustomDBEditWin = class;

  TabcDBEditWinDataLink = class(TDataLink)
  private
    FDBEditWin: TabcCustomDBEditWin;
    FFieldCount: Integer;
    FFieldMapSize: Integer;
    FModified: Boolean;
    FInUpdateData: Boolean;
    FFieldMap: Pointer;
    function GetDefaultFields: Boolean;
    function GetFields(I: Integer): TField;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
    procedure DataSetScrolled(Distance: Integer); override;
    procedure EditingChanged; override;
    procedure FocusControl(Field: TFieldRef); override;
    procedure LayoutChanged; override;
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
  public
    constructor Create(ADBEditWin: TabcCustomDBEditWin);
    destructor Destroy; override;
    function AddMapping(const FieldName: string): Boolean;
    procedure ClearMapping;
    procedure Modified;
    procedure Reset;
    property DefaultFields: Boolean read GetDefaultFields;
    property FieldCount: Integer read FFieldCount;
    property Fields[I: Integer]: TField read GetFields;
  end;

  { TabcDBEditWinEdit }

  TabcDBEditWinEdit = class(TCustomMaskEdit)
  private
    FAlignment: TAlignment;
    FCanvas: TControlCanvas;
    FDBEditWin: TabcCustomDBEditWin;
    FFocused: Boolean;
    FIndex: Integer;
    FModified: Boolean;
    FPopupAutoEdit: TabcEditFieldOptions;
    FPopupEditBlobAs: TabcEditBlobOption;
    FPopupForm: TForm;
    FPopupFormStyle: TabcPopupFormStyle;
    FOnCustomBlobEdit: TabcCustomFieldEditEvent;
    FTextMargin: Integer;
    procedure CalcTextMargin;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetPopupForm: TForm;
    procedure SetFocused(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
  protected
    procedure Change; override;
    procedure DblClick; override;
    function EditCanModify: Boolean; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Reset; override;
    property Modified: Boolean read FModified write FModified;
    property PopupForm: TForm read GetPopupForm;
  public
    constructor CreateInWindow(ADBEditWin: TabcCustomDBEditWin; AIndex: Integer); virtual;
    destructor Destroy; override;
    function UseRightToLeftAlignment: Boolean; override;
    property Field: TField read GetField;
  published
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DataField: string read GetDataField;
    property DataSource: TDataSource read GetDataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupAutoEdit: TabcEditFieldOptions read FPopupAutoEdit
      write FPopupAutoEdit default [efMemo, efGraphic];
    property PopupEditBlobAs: TabcEditBlobOption read FPopupEditBlobAs
      write FPopupEditBlobAs default ebCustom;
    property PopupFormStyle: TabcPopupFormStyle read FPopupFormStyle write FPopupFormStyle default pfsNormal;
    property OnCustomBlobEdit: TabcCustomFieldEditEvent read FOnCustomBlobEdit write FOnCustomBlobEdit;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TabcCustomDBEditWin }

  TabcLayoutEdit = (leVert, leHorz);
  TabcLayoutLabel = (laLeft, laTop);

  TReadOnlyFieldOption = (rofEditReadOnly, rofEditDisabled, rofEditColor, rofLabelDisabled);
  TabcReadOnlyFieldOptions = set of TReadOnlyFieldOption;

  TabcFieldEditNotifyEvent = procedure (FieldIndex: Integer; FieldName: string;
    Sender: TObject) of Object;

  TabcFieldEditKeyPressEvent = procedure (FieldIndex: Integer; FieldName: string;
    Sender: TObject; var Key: Char) of object;

  TabcFieldEditKeyEvent = procedure (FieldIndex: Integer; FieldName: string;
    Sender: TObject; var Key: Word; Shift: TShiftState) of object;

  TabcCustomDBEditWin = class(TabcCustomScrollBox)
  private
    FAutoCaption: Boolean;
    FDataLink: TabcDBEditWinDataLink;
    FEditAutoSize: Boolean;
    FEditCharWidth: Integer;
    FEditColor: TColor;
    FEditFont: TFont;
    FEditHints: TStrings;
    FEditParentColor: Boolean;
    FEditParentFont: Boolean;
    FEditShowHint: Boolean;
    FEditWidth: Integer;
    FFieldEdits: TList;
    FFieldLabels: TList;
    FFieldOffsetH: Integer;
    FFieldOffsetV: Integer;
    FFieldReadOnly: TabcReadOnlyFieldOptions;
    FLabelAlignment: TAlignment;
    FLabelAutoSize: Boolean;
    FLabelCaptions: TStrings;
    FLabelColor: TColor;
    FLabelFont: TFont;
    FLabelOffsetH: Integer;
    FLabelOffsetV: Integer;
    FLabelHeight: Integer;
    FLabelParentColor: Boolean;
    FLabelParentFont: Boolean;
    FLabelWidth: Integer;
    FLabelWordWrap: Boolean;
    FLayoutEdit: TabcLayoutEdit;
    FLayoutLabel: TabcLayoutLabel;
    FMarginLeft: Integer;
    FMarginTop: Integer;
    FMaxWidth: Integer;
    FOnCustomBlobEdit: TabcCustomFieldEditEvent;
    FOnEditChange: TabcFieldEditNotifyEvent;
    FOnEditClick: TabcFieldEditNotifyEvent;
    FOnEditDblClick: TabcFieldEditNotifyEvent;
    FOnEditEnter: TabcFieldEditNotifyEvent;
    FOnEditExit: TabcFieldEditNotifyEvent;
    FOnEditKeyDown: TabcFieldEditKeyEvent;
    FOnEditKeyPress: TabcFieldEditKeyPressEvent;
    FOnEditKeyUp: TabcFieldEditKeyEvent;
    FPopupAutoEdit: TabcEditFieldOptions;
    FPopupEditBlobAs: TabcEditBlobOption;
    FPopupFormStyle: TabcPopupFormStyle;
    FReadOnly: Boolean;
    FUpdatingFields: Boolean;
    procedure AlignFields;
    procedure AssignEditFont(AFont: TFont);
    function CalcEditWidth(AField: TField): Integer;
    function CanModifyField(Index: Integer): Boolean;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMUIActivate(var Message: TMessage); message CM_UIACTIVATE;
    function CreateEdit(Index: Integer; AField: TField): TabcDBEditWinEdit;
    function CreateLabel(Index: Integer; AField: TField): TLabel;
    procedure DataChanged;
    procedure DefineFieldMap;
    procedure EditChange(Sender: TObject);
    procedure EditClick(Sender: TObject);
    procedure EditDblClick(Sender: TObject);
    procedure EditEnter(Sender: TObject);
    procedure EditExit(Sender: TObject);
    function EditField(Index: Integer): Boolean;
    procedure EditFontChanged(Sender: TObject);
    procedure EditingChanged;
    procedure EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure EditKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    function FieldEditing(Index: Integer): Boolean;
    procedure FieldModified(Index: Integer);
    function GetDataSource: TDataSource;
    function GetFieldCount: Integer;
    function GetFieldEdit(Index: integer): TabcDBEditWinEdit;
    function GetFieldLabel(Index: integer): TLabel;
    function GetFields(Index: Integer): TField;
    function GetSelectedField: TField;
    function GetSelectedIndex: Integer;
    function GetSenderEditIndex(Sender: TObject): Integer;
    function IsEditColorStored: Boolean;
    function IsEditFontStored: Boolean;
    function IsLabelColorStored: Boolean;
    function IsLabelFontStored: Boolean;
    procedure LabelClick(Sender: TObject);
    procedure LabelFontChanged(Sender: TObject);
    procedure RecordChanged(Field: TField);
    procedure ResetField(Index: Integer);
    procedure ResetRecord;
    procedure SetAutoCaption(Value: Boolean);
    procedure SetDataSource(Value: TDataSource);
    procedure SetEditAutoSize(Value: Boolean);
    procedure SetEditColor(Value: TColor);
    procedure SetEditFont(Value: TFont);
    procedure SetEditHints(Value: TStrings);
    procedure SetEditParentColor(Value: Boolean);
    procedure SetEditParentFont(Value: Boolean);
    procedure SetEditShowHint(Value: Boolean);
    procedure SetEditWidth(Value: Integer);
    procedure SetFieldOffsetH(Value: Integer);
    procedure SetFieldOffsetV(Value: Integer);
    procedure SetFieldReadOnly(Value: TabcReadOnlyFieldOptions);
    procedure SetLabelAlignment(Value: TAlignment);
    procedure SetLabelAutoSize(Value: Boolean);
    procedure SetLabelCaptions(Value: TStrings);
    procedure SetLabelColor(Value: TColor);
    procedure SetLabelFont(Value: TFont);
    procedure SetLabelHeight(Value: Integer);
    procedure SetLabelOffsetH(Value: Integer);
    procedure SetLabelOffsetV(Value: Integer);
    procedure SetLabelParentColor(Value: Boolean);
    procedure SetLabelParentFont(Value: Boolean);
    procedure SetLabelWidth(Value: Integer);
    procedure SetLabelWordWrap(Value: Boolean);
    procedure SetLayoutEdit(Value: TabcLayoutEdit);
    procedure SetLayoutLabel(Value: TabcLayoutLabel);
    procedure SetMarginLeft(Value: Integer);
    procedure SetMarginTop(Value: Integer);
    procedure SetMaxWidth(Value: Integer);
    procedure SetOnCustomBlobEdit(Value: TabcCustomFieldEditEvent);
    procedure SetPopupAutoEdit(Value: TabcEditFieldOptions);
    procedure SetPopupEditBlobAs(Value: TabcEditBlobOption);
    procedure SetPopupFormStyle(Value: TabcPopupFormStyle);
    procedure SetReadOnly(Value: Boolean);
    procedure SetSelectedField(Value: TField);
    procedure SetSelectedIndex(Value: Integer);
    procedure UpdateCaptions;
    procedure UpdateEditCharWidth;
    procedure UpdateEditSize;
    procedure UpdateReadOnlyFields;
    procedure UpdateData;
    procedure UpdateField(Index: Integer);
  protected
    procedure AddFields; virtual;
    procedure ClearFields; virtual;
    procedure CreateWnd; override;
    procedure Loaded; override;
    procedure LayoutChanged; virtual;
    procedure LinkActive(Value: Boolean); virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Scroll(Distance: Integer); virtual;
    property AutoCaption: Boolean read FAutoCaption write SetAutoCaption default True;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property EditAutoSize: Boolean read FEditAutoSize write SetEditAutoSize default True;
    property EditColor: TColor read FEditColor write SetEditColor stored IsEditColorStored default clWindow;
    property EditFont: TFont read FEditFont write SetEditFont stored IsEditFontStored;
    property EditHints: TStrings read FEditHints write SetEditHints;
    property LayoutEdit: TabcLayoutEdit read FLayoutEdit write SetLayoutEdit default leVert;
    property EditParentFont: Boolean read FEditParentFont write SetEditParentFont default True;
    property EditParentColor: Boolean read FEditParentColor write SetEditParentColor default False;
    property EditShowHint: Boolean read FEditShowHint write SetEditShowHint default False;
    property EditWidth: Integer read FEditWidth write SetEditWidth default 200;
    property FieldEdit[Index: integer]: TabcDBEditWinEdit read GetFieldEdit;
    property FieldLabel[Index: integer]: TLabel read GetFieldLabel;
    property FieldOffsetH: Integer read FFieldOffsetH write SetFieldOffsetH default 10;
    property FieldOffsetV: Integer read FFieldOffsetV write SetFieldOffsetV default 10;
    property LabelAlignment: TAlignment read FLabelAlignment write SetLabelAlignment default taLeftJustify;
    property LabelAutoSize: Boolean read FLabelAutoSize write SetLabelAutoSize default False;
    property LabelCaptions: TStrings read FLabelCaptions write SetLabelCaptions;
    property LabelColor: TColor read FLabelColor write SetLabelColor stored IsLabelColorStored;
    property LabelFont: TFont read FLabelFont write SetLabelFont stored IsLabelFontStored;
    property LayoutLabel: TabcLayoutLabel read FLayoutLabel write SetLayoutLabel default laLeft;
    property LabelHeight: Integer read FLabelHeight write SetLabelHeight default 24;
    property LabelOffsetH: Integer read FLabelOffsetH write SetLabelOffsetH default 5;
    property LabelOffsetV: Integer read FLabelOffsetV write SetLabelOffsetV default 2;
    property LabelParentColor: Boolean read FLabelParentColor write SetLabelParentColor default True;
    property LabelParentFont: Boolean read FLabelParentFont write SetLabelParentFont default True;
    property LabelWidth: Integer read FLabelWidth write SetLabelWidth default 120;
    property LabelWordWrap: Boolean read FLabelWordWrap write SetLabelWordWrap default False;
    property MarginLeft: Integer read FMarginLeft write SetMarginLeft default 20;
    property MarginTop: Integer read FMarginTop write SetMarginTop default 20;
    property MaxWidth: Integer read FMaxWidth write SetMaxWidth default 500;
    property FieldReadOnly: TabcReadOnlyFieldOptions read FFieldReadOnly write SetFieldReadOnly default
      [rofEditReadOnly, rofEditDisabled, rofEditColor];
    property PopupAutoEdit: TabcEditFieldOptions read FPopupAutoEdit write SetPopupAutoEdit default
      [efMemo, efGraphic];
    property PopupEditBlobAs: TabcEditBlobOption read FPopupEditBlobAs write SetPopupEditBlobAs default ebCustom;
    property PopupFormStyle: TabcPopupFormStyle read FPopupFormStyle write SetPopupFormStyle default pfsNormal;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly default False;
    property OnCustomBlobEdit: TabcCustomFieldEditEvent read FOnCustomBlobEdit write SetOnCustomBlobEdit;
    property OnEditChange: TabcFieldEditNotifyEvent read FOnEditChange write FOnEditChange;
    property OnEditClick: TabcFieldEditNotifyEvent read FOnEditClick write FOnEditClick;
    property OnEditDblClick: TabcFieldEditNotifyEvent read FOnEditDblClick write FOnEditDblClick;
    property OnEditEnter: TabcFieldEditNotifyEvent read FOnEditEnter write FOnEditEnter;
    property OnEditExit: TabcFieldEditNotifyEvent read FOnEditExit write  FOnEditExit;
    property OnEditKeyDown: TabcFieldEditKeyEvent read FOnEditKeyDown write FOnEditKeyDown;
    property OnEditKeyPress: TabcFieldEditKeyPressEvent read FOnEditKeyPress write FOnEditKeyPress;
    property OnEditKeyUp: TabcFieldEditKeyEvent read FOnEditKeyUp write FOnEditKeyUp;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
{$IFDEF DELPHI5}
    function CanFocus: Boolean; override;
{$ELSE}
    function CanFocus: Boolean;
{$ENDIF}
    property FieldCount: Integer read GetFieldCount;
    property Fields[Index: Integer]: TField read GetFields;
    property SelectedField: TField read GetSelectedField write SetSelectedField;
    property SelectedIndex: Integer read GetSelectedIndex write SetSelectedIndex;
  end;

  TabcDBEditWin = class(TabcCustomDBEditWin)
  public
    property FieldEdit;
    property FieldLabel;
  published
    property AutoCaption;
    property DataSource;
    property EditAutoSize;
    property EditColor;
    property EditFont;
    property EditHints;
    property EditParentColor;
    property EditParentFont;
    property EditShowHint;
    property EditWidth;
    property FieldOffsetH;
    property FieldOffsetV;
    property FieldReadOnly;
    property LabelAlignment;
    property LabelAutoSize;
    property LabelCaptions;
    property LabelColor;
    property LabelFont;
    property LabelHeight;
    property LabelOffsetH;
    property LabelOffsetV;
    property LabelParentColor;
    property LabelParentFont;
    property LabelWidth;
    property LabelWordWrap;
    property LayoutEdit;
    property LayoutLabel;
    property MarginLeft;
    property MarginTop;
    property MaxWidth;
    property PopupAutoEdit;
    property PopupEditBlobAs;
    property PopupFormStyle;
    property ReadOnly;
    property OnCustomBlobEdit;
    property OnEditChange;
    property OnEditClick;
    property OnEditDblClick;
    property OnEditEnter;
    property OnEditExit;
    property OnEditKeyDown;
    property OnEditKeyPress;
    property OnEditKeyUp;
    { inherited scrollbox properties }
    property Align;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Constraints;
    {property DockSite;}
    property DragKind;
    property ParentBiDiMode;
    property AutoScroll;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle default bsSingle;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Color nodefault;
    property Ctl3D;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnCanResize;
    property OnConstrainedResize;
    {property OnDockDrop;}
    {property OnDockOver;}
    property OnEndDock;
    {property OnGetSiteInfo;}
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnStartDock;
    {property OnUnDock;}
    property OnStartDrag;
  end;

implementation

uses ExtCtrls, abcfdbme, abcfdbim;

const
  MaxMapSize = 65520 div SizeOf(Integer);

type
  TIntArray = array[0..MaxMapSize] of Integer;
  PIntArray = ^TIntArray;

{ TabcDBEditWinDataLink }

constructor TabcDBEditWinDataLink.Create(ADBEditWin: TabcCustomDBEditWin);
begin
  inherited Create;
  FDBEditWin := ADBEditWin;
  VisualControl := True;
end;

destructor TabcDBEditWinDataLink.Destroy;
begin
  ClearMapping;
  inherited Destroy;
end;

procedure TabcDBEditWinDataLink.ActiveChanged;
begin
  FDBEditWin.LinkActive(Active);
end;

function TabcDBEditWinDataLink.AddMapping(const FieldName: string): Boolean;
var
  Field: TField;
  NewSize: Integer;
  NewMap: Pointer;
begin
  if FFieldCount >= MaxMapSize then raise Exception.Create('Too many columns');
  Field := DataSet.FieldByName(FieldName);
  Result := Field <> nil;
  if Result then
  begin
    if FFieldMapSize = 0 then
    begin
      FFieldMapSize := 8;
      GetMem(FFieldMap, FFieldMapSize * SizeOf(Integer));
    end
    else if FFieldCount = FFieldMapSize then
    begin
      NewSize := FFieldMapSize;
      Inc(NewSize, NewSize);
      if (NewSize > MaxMapSize) or (NewSize < FFieldCount) then
        NewSize := MaxMapSize;
      GetMem(NewMap, NewSize * SizeOf(Integer));
      Move(FFieldMap^, NewMap^, SizeOf(Integer) * FFieldCount);
      FreeMem(FFieldMap, SizeOf(Integer) * FFieldCount);
      FFieldMap := nil;
      FFieldMapSize := NewSize;
      FFieldMap := NewMap;
    end;
    PIntArray(FFieldMap)^[FFieldCount] := Field.Index;
    Inc(FFieldCount);
  end;
end;

procedure TabcDBEditWinDataLink.ClearMapping;
begin
  if FFieldMap <> nil then begin
    FreeMem(FFieldMap, FFieldMapSize * SizeOf(Integer));
    FFieldMap := nil;
    FFieldMapSize := 0;
    FFieldCount := 0;
  end;
end;

procedure TabcDBEditWinDataLink.DataSetChanged;
begin
  FDBEditWin.DataChanged;
  FModified := False;
end;

procedure TabcDBEditWinDataLink.DataSetScrolled(Distance: Integer);
begin
  FDBEditWin.Scroll(Distance);
end;

procedure TabcDBEditWinDataLink.EditingChanged;
begin
  FDBEditWin.EditingChanged;
end;

procedure TabcDBEditWinDataLink.FocusControl(Field: TFieldRef);
begin
  FDBEditWin.FieldEdit[Field^.Index].SetFocus;
end;

function TabcDBEditWinDataLink.GetDefaultFields: Boolean;
begin
  Result := True;
  if DataSet <> nil then Result := DataSet.DefaultFields;
end;

function TabcDBEditWinDataLink.GetFields(I: Integer): TField;
begin
  if I < FFieldCount then
    Result := DataSet.Fields[PIntArray(FFieldMap)^[I]]
  else
    Result := nil;
end;

procedure TabcDBEditWinDataLink.LayoutChanged;
begin
  FDBEditWin.LayoutChanged;
end;

procedure TabcDBEditWinDataLink.Modified;
begin
  FModified := True;
end;

procedure TabcDBEditWinDataLink.RecordChanged(Field: TField);
begin
  if (Field = nil) or not FInUpdateData then
  begin
    FDBEditWin.RecordChanged(Field);
    FModified := False;
  end;
end;

procedure TabcDBEditWinDataLink.Reset;
begin
  Dataset.Cancel;
  FModified := False;
end;

procedure TabcDBEditWinDataLink.UpdateData;
begin
  FInUpdateData := True;
  try
    if FModified then FDBEditWin.UpdateData;
    FModified := False;
  finally
    FInUpdateData := False;
  end;
end;

{ TabcDBEditWinEdit }

constructor TabcDBEditWinEdit.CreateInWindow(ADBEditWin: TabcCustomDBEditWin; AIndex: Integer);
begin
  inherited Create(ADBEditWin);
  FDBEditWin := ADBEditWin;
  FIndex := AIndex;
  FPopupAutoEdit := [efMemo, efGraphic];
  FPopupFormStyle := pfsNormal;
  CalcTextMargin;
end;

destructor TabcDBEditWinEdit.Destroy;
begin
  FCanvas.Free;
  inherited Destroy;
end;

procedure TabcDBEditWinEdit.CalcTextMargin;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  DC := GetDC(0);
  GetTextMetrics(DC, SysMetrics);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  I := SysMetrics.tmHeight;
  if I > Metrics.tmHeight then I := Metrics.tmHeight;
  FTextMargin := I div 4;
end;

procedure TabcDBEditWinEdit.Change;
begin
  FDBEditWin.FieldModified(FIndex);
  inherited Change;
end;

procedure TabcDBEditWinEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  inherited;
end;

procedure TabcDBEditWinEdit.CMExit(var Message: TCMExit);
begin
  try
    FDBEditWin.UpdateField(FIndex);
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused(False);
  SetCursor(0);
  DoExit;
end;

procedure TabcDBEditWinEdit.CMFontChanged(var Message: TMessage);
begin
  inherited;
  CalcTextMargin;
end;

procedure TabcDBEditWinEdit.DataChange(Sender: TObject);
var
  Field: TField;
begin
  Field := FDBEditWin.Fields[FIndex];
  if Field <> nil then begin
    if FAlignment <> Field.Alignment then begin
      EditText := '';
      FAlignment := Field.Alignment;
      Invalidate;
    end;
    EditMask := Field.EditMask;
    if Field.DataType = ftString then
      MaxLength := Field.Size else
      MaxLength := 0;
    if FFocused and FDBEditWin.CanModifyField(FIndex) then
      Text := Field.Text
    else
      EditText := Field.DisplayText;
  end
  else begin
    FAlignment := taLeftJustify;
    Invalidate;
    EditMask := '';
    MaxLength := 0;
    if csDesigning in ComponentState then
      EditText := Name else
      EditText := '';
  end;
end;

function TabcDBEditWinEdit.GetPopupForm: TForm;
var
  I: Integer;
begin
  Result := nil;
  if FPopupForm <> nil then begin
    for I := 0 to ComponentCount - 1 do
      if Components[I] = FPopupForm then begin
        Result := FPopupForm;
        Exit;
      end;
    FPopupForm := nil;
  end;
end;

procedure TabcDBEditWinEdit.DblClick;
begin
  if PopupForm <> nil then
    FPopupForm.Show
  else if (Field <> nil) then
    case Field.DataType of
      ftUnknown..ftVarBytes:
        Exit;
      ftBlob:
        if efBlob in FPopupAutoEdit then
          case FPopupEditBlobAs of
            ebMemo:
              FPopupForm := ExecabcafMemoFieldEdit(Self, FDBEditWin.DataSource.Dataset,
                Field, PopupFormStyle, ReadOnly, FDBEditWin.HelpContext);
            ebGraphic:
              FPopupForm := ExecabcafGraphicFieldEdit(Self, FDBEditWin.DataSource.Dataset,
                Field, PopupFormStyle, ReadOnly, FDBEditWin.HelpContext);
            ebCustom:
              if Assigned(FOnCustomBlobEdit) then
                FOnCustomBlobEdit(Self, Self, FDBEditWin.DataSource.Dataset,
                  Field, PopupFormStyle, ReadOnly, FDBEditWin.HelpContext, FPopupForm);
          end;
      ftMemo:
        if efMemo in FPopupAutoEdit then
          FPopupForm := ExecabcafMemoFieldEdit(Self, FDBEditWin.DataSource.Dataset,
            Field, PopupFormStyle, ReadOnly, FDBEditWin.HelpContext);
      ftGraphic:
        if efGraphic in FPopupAutoEdit then
          FPopupForm := ExecabcafGraphicFieldEdit(Self, FDBEditWin.DataSource.Dataset,
            Field, PopupFormStyle, ReadOnly, FDBEditWin.HelpContext);
    end;
  inherited DblClick;
end;

function TabcDBEditWinEdit.EditCanModify: Boolean;
begin
  Result := FDBEditWin.EditField(FIndex);
end;

function TabcDBEditWinEdit.GetDataField: string;
begin
  if Field = nil then
    Result := ''
  else
    Result := Field.FieldName;
end;

function TabcDBEditWinEdit.GetDataSource: TDataSource;
begin
  Result := FDBEditWin.DataSource;
end;

function TabcDBEditWinEdit.GetField: TField;
begin
  Result := FDBEditWin.Fields[FIndex];
end;

procedure TabcDBEditWinEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
    FDBEditWin.EditField(FIndex);
end;

procedure TabcDBEditWinEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key in [#32..#255]) and (FDBEditWin.Fields[FIndex] <> nil) and
    not FDBEditWin.Fields[FIndex].IsValidChar(Key) then
  begin
    MessageBeep(0);
    Key := #0;
  end;
  case Key of
    ^H, ^V, ^X, #32..#255:
      FDBEditWin.EditField(FIndex);
    #27:
      begin
        if FModified then
          FDBEditWin.ResetField(FIndex)
        else
          FDBEditWin.ResetRecord;
        SelectAll;
        Key := #0;
      end;
  end;
end;

procedure TabcDBEditWinEdit.Reset;
begin
  FDBEditWin.ResetField(FIndex);
  SelectAll;
end;

procedure TabcDBEditWinEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then
  begin
    FFocused := Value;
    if (FAlignment <> taLeftJustify) and not IsMasked then Invalidate;
    FDBEditWin.ResetField(FIndex);
  end;
end;

procedure TabcDBEditWinEdit.UpdateData(Sender: TObject);
begin
  ValidateEdit;
  Field.Text := Text;
end;

procedure TabcDBEditWinEdit.WMCut(var Message: TMessage);
begin
  FDBEditWin.EditField(FIndex);
  inherited;
end;

procedure TabcDBEditWinEdit.WMPaint(var Message: TWMPaint);
var
  Width, Indent, Left, I: Integer;
  R: TRect;
  DC: HDC;
  PS: TPaintStruct;
  S: string;
begin
  if (FAlignment = taLeftJustify) or FFocused then
  begin
    inherited;
    Exit;
  end;
  if FCanvas = nil then
  begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then DC := BeginPaint(Handle, PS);
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    with FCanvas do
    begin
      R := ClientRect;
      if (BorderStyle = bsSingle) and not Ctl3D then
      begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
        InflateRect(R, -1, -1);
      end;
      Brush.Color := Color;
      S := EditText;
      if PasswordChar <> #0 then
      begin
        for I := 1 to Length(S) do
          S[I] := PasswordChar;
      end;
      Width := TextWidth(S);
      if BorderStyle = bsNone then Indent := 0 else Indent := FTextMargin;
      if FAlignment = taRightJustify then
        Left := R.Right - Width - Indent else
        Left := (R.Left + R.Right - Width) div 2;
      TextRect(R, Left, Indent, S);
    end;
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0 then EndPaint(Handle, PS);
  end;
end;

procedure TabcDBEditWinEdit.WMPaste(var Message: TMessage);
begin
  FDBEditWin.EditField(FIndex);
  inherited;
end;

function TabcDBEditWinEdit.UseRightToLeftAlignment: Boolean;
begin
  Result := DBUseRightToLeftAlignment(Self, Field);
end;

{ TabcCustomDBEditWin }

constructor TabcCustomDBEditWin.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls];
  TabStop := True;
  FDataLink := TabcDBEditWinDataLink.Create(Self);
  FEditHints := TStringList.Create;
  FFieldEdits:= TList.Create;
  FFieldLabels := TList.Create;
  FLabelCaptions := TStringList.Create;
  FEditFont := TFont.Create;
  AssignEditFont(Self.Font);
  FEditFont.OnChange := EditFontChanged;
  FLabelFont := TFont.Create;
  FLabelFont.Assign(Self.Font);
  FLabelFont.OnChange := LabelFontChanged;
  FAutoCaption := True;
  FEditAutoSize := True;
  FEditColor := clWindow;
  FEditParentColor := False;
  FEditParentFont := True;
  FEditShowHint := False;
  FEditWidth := 200;
  FFieldOffsetH := 10;
  FFieldOffsetV := 10;
  FLabelAlignment := taLeftJustify;
  FLabelAutoSize := False;
  FLabelColor := clBtnFace;
  FLabelHeight := 24;
  FLabelOffsetH := 5;
  FLabelOffsetV := 2;
  FLabelParentColor := True;
  FLabelParentFont := True;
  FLabelWidth := 120;
  FMarginLeft := 20;
  FMarginTop := 20;
  FMaxWidth := 500;
  FPopupAutoEdit := [efMemo, efGraphic];
  FPopupFormStyle := pfsNormal;
  FFieldReadOnly := [rofEditReadOnly, rofEditDisabled, rofEditColor];
end;

destructor TabcCustomDBEditWin.Destroy;
begin
  ClearFields;
  FDataLink.Free;
  FDataLink := nil;
  FEditFont.Free;
  FLabelFont.Free;
  FEditHints.Free;
  FFieldEdits.Free;
  FFieldLabels.Free;
  FLabelCaptions.Free;
  inherited Destroy;
end;

procedure TabcCustomDBEditWin.AddFields;
var
  I: integer;
  NewLabel: TLabel;
begin
  NewLabel := nil;
  FUpdatingFields := True;
  for I := 0 to FieldCount - 1 do begin
    CreateEdit(I, Fields[I]);
    NewLabel := CreateLabel(I, Fields[I]);
  end;
  if (FieldCount > 0) and FLabelAutoSize then
    FLabelHeight := NewLabel.Height;
  UpdateCaptions;
  UpdateEditSize;
  AlignFields;
  FUpdatingFields := False;
end;

procedure TabcCustomDBEditWin.AlignFields;
var
  I, VMax: integer;
  EditPos, LabelPos: TPoint;
  CtrlOffset, CurrRight, NextWidth: Integer;
  CurrEdit: TabcDBEditWinEdit;
  CurrLabel: TLabel;

  function MaxInt(X,Y: Integer): Integer;
  begin
    if X > Y then
      Result := X
    else
      Result := Y;
  end;

begin
  if FieldCount = 0 then Exit;
  if (csLoading in ComponentState) then Exit;
  HorzScrollBar.Position := 0;
  VertScrollBar.Position := 0;
  {initial offsets}
  if (FLayoutLabel = laTop) then begin
    LabelPos := Point(FMarginLeft, FMarginTop);
    EditPos := Point(FMarginLeft, FMarginTop + FLabelHeight + FLabelOffsetV);
  end
  else if FLayoutEdit = leVert then begin   {laLeft + leVert}
    LabelPos := Point(FMarginLeft, FMarginTop + FLabelOffsetV);
    EditPos := Point(FMarginLeft + FLabelWidth + FLabelOffsetH, FMarginTop);
  end
  else begin     {laLeft + leHorz}
    LabelPos := Point(FMarginLeft, FMarginTop + FLabelOffsetV);
    EditPos := Point(FMarginLeft + TLabel(FFieldLabels[0]).Width + FLabelOffsetH, FMarginTop);
  end;

  if (FLayoutLabel = laLeft) then
    CtrlOffset := MaxInt(
          TabcDBEditWinEdit(FFieldEdits[0]).Height,
          FLabelHeight + FLabelOffsetV)
          + FFieldOffsetV
  else
    CtrlOffset := FLabelHeight + FLabelOffsetV
          + TabcDBEditWinEdit(FFieldEdits[0]).Height + FFieldOffsetV;

  for I := 0 to FieldCount - 1 do begin
    CurrEdit := TabcDBEditWinEdit(FFieldEdits[I]);
    CurrLabel := TLabel(FFieldLabels[I]);
    with CurrEdit do SetBounds(EditPos.X, EditPos.Y, Width, Height);
    with CurrLabel do SetBounds(LabelPos.X, LabelPos.Y, Width, Height);
    if I < (FieldCount - 1) then begin
      {calc next pos}
      if (FLayoutEdit = leVert) then begin
        LabelPos.Y := LabelPos.Y + CtrlOffset;
        EditPos.Y := EditPos.Y + CtrlOffset;
      end
      else begin
        CurrRight := MaxInt(CurrLabel.Left + CurrLabel.Width, CurrEdit.Left + CurrEdit.Width);
        if (FLayoutLabel = laLeft) then
          NextWidth := FFieldOffsetH + TLabel(FFieldLabels[I+1]).Width +
            FLabelOffsetH + TabcDBEditWinEdit(FFieldEdits[I+1]).Width
        else
          NextWidth := FFieldOffsetH +
            MaxInt(TLabel(FFieldLabels[I+1]).Width, TabcDBEditWinEdit(FFieldEdits[I+1]).Width);

        if (CurrRight + NextWidth) > FMaxWidth then begin
          LabelPos.X := FMarginLeft;
          LabelPos.Y := LabelPos.Y + CtrlOffset;
          if (FLayoutLabel = laLeft) then
            EditPos.X := FMarginLeft + TLabel(FFieldLabels[I+1]).Width + FLabelOffsetH
          else
            EditPos.X := FMarginLeft;
          EditPos.Y := EditPos.Y + CtrlOffset;
        end
        else begin
          if (FLayoutLabel = laLeft) then begin
            LabelPos.X := EditPos.X + CurrEdit.Width + FFieldOffsetH;
            EditPos.X := LabelPos.X + TLabel(FFieldLabels[I+1]).Width + FLabelOffsetH;
          end
          else begin
            LabelPos.X := LabelPos.X + FFieldOffsetH + MaxInt(CurrLabel.Width, CurrEdit.Width);
            EditPos.X := LabelPos.X;
          end
        end;
      end
    end
    else begin
      {at end, set VertScroll for bottom margin}
      VMax := EditPos.Y + CurrEdit.Height;
      if VMax < LabelPos.Y + CurrLabel.Height then
        VMax := LabelPos.Y + CurrLabel.Height;
      VMax := VMax + FMarginTop;
      VertScrollBar.Range := VMax;
    end;
  end;
end;

procedure TabcCustomDBEditWin.AssignEditFont(AFont: TFont);
begin
  if (AFont <> FEditFont) and (AFont <> nil) then begin
    FEditFont.Assign(AFont);
    UpdateEditCharWidth;
  end;
end;

function TabcCustomDBEditWin.CalcEditWidth(AField: TField): Integer;
begin
  if AField.DisplayWidth = 0 then
    Result := FEditWidth
  else
    Result := AField.DisplayWidth * FEditCharWidth + Abs(FEditFont.Height) div 4 + 6;
end;

function TabcCustomDBEditWin.CanModifyField(Index: Integer): Boolean;
begin
  Result := not ReadOnly and FDataLink.Editing and Fields[Index].CanModify;
end;

procedure TabcCustomDBEditWin.ClearFields;
var
  I: integer;
  DeletedControl: TControl;
begin
  for I := FieldCount - 1 downto 0 do begin
    DeletedControl := FFieldEdits[I];
    FFieldEdits.Remove(DeletedControl);
    TabcDBEditWinEdit(DeletedControl).Free;
    DeletedControl := FFieldLabels[I];
    FFieldLabels.Remove(DeletedControl);
    TLabel(DeletedControl).Free;
  end;
end;

procedure TabcCustomDBEditWin.CMColorChanged(var Message: TMessage);
begin
  inherited;
  if FEditParentColor then FEditColor := Color;
end;

function TabcCustomDBEditWin.CanFocus: Boolean;
var
  ParentForm: TCustomForm;
begin
  Result := inherited CanFocus;
  if Result then
  begin
    ParentForm := GetParentForm(Self);
    if (ParentForm <> nil) and
       (ParentForm.ActiveControl <> nil) and
       (ParentForm.ActiveControl.Parent = self) then
      Result := False;
  end;
end;

procedure TabcCustomDBEditWin.CMUIActivate(var Message: TMessage);
var
  I: Integer;
  ParentForm: TCustomForm;
begin
  inherited;
  ParentForm := GetParentForm(Self);
  if (ParentForm <> nil) and (ParentForm.ActiveControl = self) then
  for I := 0 to FieldCount - 1 do
    with TabcDBEditWinEdit(FFieldEdits[I]) do
      if Visible and Enabled then
      begin
        SetFocus;
        Exit;
      end;
end;

procedure TabcCustomDBEditWin.CMFontChanged(var Message: TMessage);
begin
  inherited;
  if FEditParentFont then begin
    AssignEditFont(Font);
    FEditParentFont := True;
  end;
  if FLabelParentFont then begin
    FLabelFont.Assign(Font);
    FLabelParentFont := True;
  end;
end;

function TabcCustomDBEditWin.CreateEdit(Index: Integer; AField: TField): TabcDBEditWinEdit;
var
  NewEdit: TabcDBEditWinEdit;
begin
  NewEdit := TabcDBEditWinEdit.CreateInWindow(Self, Index);
  with NewEdit do begin
    Parent := Self;
    if FEditParentColor then
      ParentColor := True
    else
      Color := FEditColor;
    if not FEditParentFont then Font.Assign(FEditFont);
    if FEditHints.Count > Index then Hint := FEditHints.Strings[Index];
    if not FieldEditing(Index) then begin
      if rofEditReadOnly in FFieldReadOnly then ReadOnly := True;
      if rofEditDisabled in FFieldReadOnly then Enabled := False;
      if rofEditColor in FFieldReadOnly then ParentColor := True;
    end;
    if FEditAutoSize then
      SetBounds(Left, Top, CalcEditWidth(AField), Height)
    else
      SetBounds(Left, Top, FEditWidth, Height);
    PopupAutoEdit := Self.PopupAutoEdit;
    PopupEditBlobAs := Self.PopupEditBlobAs;
    PopupFormStyle := Self.PopupFormStyle;
    OnCustomBlobEdit := Self.OnCustomBlobEdit;
    Text := AField.Text;
    OnChange := EditChange;
    OnClick := EditClick;
    OnDblClick := EditDblClick;
    OnEnter := EditEnter;
    OnExit := EditExit;
    OnKeyDown := EditKeyDown;
    OnKeyPress := EditKeyPress;
    OnKeyUp := EditKeyUp;
    Visible := True;
  end;
  FFieldEdits.Add(NewEdit);
  Result := NewEdit;
end;

function TabcCustomDBEditWin.CreateLabel(Index: Integer; AField: TField): TLabel;
var
  NewLabel: TLabel;
begin
  NewLabel := TLabel.Create(Self);
  with NewLabel do begin
    Parent := Self;
    Alignment := FLabelAlignment;
    WordWrap := FLabelWordWrap;
    SetBounds(Left, Top, FLabelWidth, FLabelHeight);
    if FAutoCaption then
      Caption := AField.DisplayName
    else
      if FLabelCaptions.Count > Index then
        Caption := FLabelCaptions.Strings[Index]
      else
        Caption := '';
    AutoSize := FLabelAutoSize;
    if not FLabelParentColor then Color := FLabelColor;
    FocusControl := TabcDBEditWinEdit(FFieldEdits[Index]);
    if not FLabelParentFont then Font.Assign(FLabelFont);
    if FReadOnly or AField.ReadOnly then
      if rofLabelDisabled in FFieldReadOnly then Enabled := False;
    OnClick := LabelClick;
    Visible := True;
  end;
  FFieldLabels.Add(NewLabel);
  Result := NewLabel;
end;

procedure TabcCustomDBEditWin.CreateWnd;
begin
  inherited CreateWnd;
  LayoutChanged;
end;

procedure TabcCustomDBEditWin.Loaded;
begin
  inherited;
  LayoutChanged;
end;

procedure TabcCustomDBEditWin.DataChanged;
var
  I: Integer;
begin
  FUpdatingFields := True;
  if not HandleAllocated then Exit;
  for I := 0 to FieldCount - 1 do
    TabcDBEditWinEdit(FFieldEdits[I]).DataChange(Self);
  FUpdatingFields := False;
end;

procedure TabcCustomDBEditWin.DefineFieldMap;
var
  I: Integer;
begin
  with FDatalink.Dataset do
    for I := 0 to FieldCount - 1 do
      with Fields[I] do if Visible then FDatalink.AddMapping(Fieldname);
  AddFields;
end;

procedure TabcCustomDBEditWin.EditChange(Sender: TObject);
var
  I: integer;
begin
  if Assigned(FOnEditChange) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditChange(I, Fields[I].FieldName, Self);
  end;
end;

procedure TabcCustomDBEditWin.EditClick(Sender: TObject);
var
  I: integer;
begin
  if Assigned(FOnEditClick) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditClick(I, Fields[I].FieldName, Self);
  end;
end;

procedure TabcCustomDBEditWin.EditDblClick(Sender: TObject);
var
  I: integer;
begin
  if Assigned(FOnEditDblClick) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditDblClick(I, Fields[I].FieldName, Self);
  end;
end;

procedure TabcCustomDBEditWin.EditEnter(Sender: TObject);
var
  I: integer;
begin
  if Assigned(FOnEditEnter) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditEnter(I, Fields[I].FieldName, Self);
  end;
end;

procedure TabcCustomDBEditWin.EditExit(Sender: TObject);
var
  I: integer;
begin
  if Assigned(FOnEditExit) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditExit(I, Fields[I].FieldName, Self);
  end;
end;

function TabcCustomDBEditWin.EditField(Index: Integer): Boolean;
begin
  if not FReadOnly and Fields[Index].CanModify then begin
    FDataLink.Edit;
    FDataLink.Modified;
    TabcDBEditWinEdit(FFieldEdits[Index]).Modified := True;
    Result := True;
  end
  else
    Result := False;
end;

procedure TabcCustomDBEditWin.EditFontChanged(Sender: TObject);
var
  I: Integer;
begin
  FEditParentFont := False;
  if not (csLoading in ComponentState) then begin
    for I := 0 to FieldCount - 1 do
      TabcDBEditWinEdit(FFieldEdits[I]).Font.Assign(FEditFont);
    UpdateEditCharWidth;
    UpdateEditSize;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.EditingChanged;
begin
  if not HandleAllocated then Exit;
  UpdateReadOnlyFields;
end;

procedure TabcCustomDBEditWin.EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  I: integer;
begin
  if Assigned(FOnEditKeyDown) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditKeyDown(I, Fields[I].FieldName, Self, Key, Shift);
  end;
end;

procedure TabcCustomDBEditWin.EditKeyPress(Sender: TObject; var Key: Char);
var
  I: integer;
begin
  if Assigned(FOnEditKeyPress) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditKeyPress(I, Fields[I].FieldName, Self, Key);
  end;
end;

procedure TabcCustomDBEditWin.EditKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  I: integer;
begin
  if Assigned(FOnEditKeyUp) then begin
    I := GetSenderEditIndex(Sender);
    if I > -1 then
      FOnEditKeyUp(I, Fields[I].FieldName, Self, Key, Shift);
  end;
end;

procedure TabcCustomDBEditWin.FieldModified(Index: Integer);
begin
  if not FUpdatingFields then begin
    FDataLink.Modified;
    TabcDBEditWinEdit(FFieldEdits[Index]).Modified := True;
 end;
end;

function TabcCustomDBEditWin.FieldEditing(Index: Integer): Boolean;
begin
  Result := not FReadOnly and
            (FDataLink.Editing or FDataLink.DataSource.AutoEdit) and
            Fields[Index].CanModify;
end;

function TabcCustomDBEditWin.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TabcCustomDBEditWin.GetFieldCount: Integer;
begin
  Result := FDatalink.FieldCount;
end;

function TabcCustomDBEditWin.GetFieldEdit(Index: integer): TabcDBEditWinEdit;
begin
  Result := TabcDBEditWinEdit(FFieldEdits[Index]);
end;

function TabcCustomDBEditWin.GetFieldLabel(Index: integer): TLabel;
begin
  Result := TLabel(FFieldLabels[Index]);
end;

function TabcCustomDBEditWin.GetFields(Index: Integer): TField;
begin
  Result := FDatalink.Fields[Index];
end;

function TabcCustomDBEditWin.GetSelectedField: TField;
var
  I: Integer;
begin
  Result := nil;
  I := GetSelectedIndex;
  if I <> -1 then Result := Fields[I];
end;

function TabcCustomDBEditWin.GetSelectedIndex: Integer;
var
  I: Integer;
begin
  Result := -1;
  for I := 0 to FieldCount - 1 do
    if ValidParentForm(Self).ActiveControl = FFieldEdits[I] then begin
      Result := I;
      Break;
    end;
end;

function TabcCustomDBEditWin.GetSenderEditIndex(Sender: TObject): Integer;
var
  I: integer;
begin
  Result := -1;
  for I := 0 to FieldCount - 1 do
    if FFieldEdits[I] = Sender then begin
      Result := I;
      Exit;
    end;
end;

function TabcCustomDBEditWin.IsEditColorStored: Boolean;
begin
  Result := not EditParentColor;
end;

function TabcCustomDBEditWin.IsEditFontStored: Boolean;
begin
  Result := not EditParentFont;
end;

function TabcCustomDBEditWin.IsLabelColorStored: Boolean;
begin
  Result := not LabelParentColor;
end;

function TabcCustomDBEditWin.IsLabelFontStored: Boolean;
begin
  Result := not LabelParentFont;
end;

procedure TabcCustomDBEditWin.LabelClick(Sender: TObject);
begin
  if Sender is TLabel then
    if TLabel(Sender).FocusControl is TabcDBEditWinEdit then
      with TabcDBEditWinEdit(TLabel(Sender).FocusControl) do
        if Visible and Enabled then SetFocus;
end;

procedure TabcCustomDBEditWin.LabelFontChanged(Sender: TObject);
var
  I: Integer;
begin
  FLabelParentFont := False;
  if not (csLoading in ComponentState) then
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).Font.Assign(FLabelFont);
end;

procedure TabcCustomDBEditWin.LayoutChanged;
var
  I: Integer;
begin
  if csLoading in ComponentState then Exit;
  if not HandleAllocated then Exit;
  ClearFields;
  FDatalink.ClearMapping;
  if FDatalink.Active then DefineFieldMap;
  if Focused then
    for I := 0 to FieldCount - 1 do
      with TabcDBEditWinEdit(FFieldEdits[I]) do
        if Visible and Enabled then begin
          SetFocus;
          Exit;
        end;
end;

procedure TabcCustomDBEditWin.LinkActive(Value: Boolean);
begin
  LayoutChanged;
end;

procedure TabcCustomDBEditWin.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcCustomDBEditWin.RecordChanged(Field: TField);
var
  I: Integer;
begin
  if not HandleAllocated then Exit;
  FUpdatingFields := True;
  for I := 0 to FieldCount - 1 do
    if (Field = nil) or (Fields[I] = Field) then begin
      TabcDBEditWinEdit(FFieldEdits[I]).DataChange(Self);
      Break;
    end;
  FUpdatingFields := False;
end;

procedure TabcCustomDBEditWin.ResetField(Index: Integer);
begin
  with TabcDBEditWinEdit(FFieldEdits[Index]) do begin
    FUpdatingFields := True;
    Text := Fields[Index].Text;
    Modified := False;
    FUpdatingFields := False;
  end;
end;

procedure TabcCustomDBEditWin.ResetRecord;
begin
  FDataLink.Reset;
end;

procedure TabcCustomDBEditWin.Scroll(Distance: Integer);
var
  I: Integer;
begin
  if not HandleAllocated then Exit;
  for I := 0 to FieldCount - 1 do
    TabcDBEditWinEdit(FFieldEdits[I]).Text := Fields[I].Text;
end;

procedure TabcCustomDBEditWin.SetAutoCaption(Value: Boolean);
begin
  if Value <> FAutoCaption then begin
    FAutoCaption := Value;
    UpdateCaptions;
  end;
end;

procedure TabcCustomDBEditWin.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

procedure TabcCustomDBEditWin.SetEditAutoSize(Value: Boolean);
begin
  if Value <> FEditAutoSize then begin
    FEditAutoSize := Value;
    UpdateEditSize;
  end;
end;

procedure TabcCustomDBEditWin.SetEditColor(Value: TColor);
var
  I: integer;
begin
  if Value <> FEditColor then begin
    FEditColor := Value;
    for I := 0 to FieldCount - 1 do
      with TabcDBEditWinEdit(FFieldEdits[I]) do
        if (rofEditColor in FFieldReadOnly) then begin
          if not (Self.FReadOnly or Fields[I].ReadOnly) then
             Color := Value;
        end
        else
          Color := Value;
    FEditParentColor := False;
  end;
end;

procedure TabcCustomDBEditWin.SetEditFont(Value: TFont);
begin
  if Value <> FEditFont then begin
    AssignEditFont(Value);
    FEditParentFont := False;
  end;
end;

procedure TabcCustomDBEditWin.SetEditHints(Value: TStrings);
var
  I: integer;
begin
  FEditHints.Assign(Value);
  for I := 0 to FieldCount - 1 do
    with TabcDBEditWinEdit(FFieldEdits[I]) do
      if I < FEditHints.Count then
        Hint := FEditHints.Strings[I]
      else
        Hint := '';
end;

procedure TabcCustomDBEditWin.SetEditParentColor(Value: Boolean);
var
  I: integer;
begin
  if Value <> FEditParentColor then begin
    FEditParentColor := Value;
    for I := 0 to FieldCount - 1 do
      with TabcDBEditWinEdit(FFieldEdits[I]) do
        if (rofEditColor in FFieldReadOnly) then
          if Value or Self.FReadOnly or Fields[I].ReadOnly then
            ParentColor := True
          else
            ParentColor := False
        else
          ParentColor := Value;
    if Value then FEditColor := Color;
  end;
end;

procedure TabcCustomDBEditWin.SetEditParentFont(Value: Boolean);
var
  I: integer;
begin
  if Value <> FEditParentFont then begin
    if Value then AssignEditFont(Font);
    FEditParentFont := Value;
    for I := 0 to FieldCount - 1 do
      TabcDBEditWinEdit(FFieldEdits[I]).ParentFont := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetEditShowHint(Value: Boolean);
var
  I: integer;
begin
  if Value <> FEditShowHint then begin
    FEditShowHint := Value;
    for I := 0 to FieldCount - 1 do
      TabcDBEditWinEdit(FFieldEdits[I]).ShowHint := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetEditWidth(Value: Integer);
var
  I: integer;
begin
  if Value <> FEditWidth then begin
    FEditWidth := Value;
    FEditAutoSize := False;
    for I := 0 to FieldCount - 1 do
      TabcDBEditWinEdit(FFieldEdits[I]).Width := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetFieldOffsetH(Value: Integer);
begin
  if Value <> FFieldOffsetH then begin
    FFieldOffsetH := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetFieldOffsetV(Value: Integer);
begin
  if Value <> FFieldOffsetV then begin
    FFieldOffsetV := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetFieldReadOnly(Value: TabcReadOnlyFieldOptions);
begin
  if Value <> FFieldReadOnly then begin
    FFieldReadOnly := Value;
    UpdateReadOnlyFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelAlignment(Value: TAlignment);
var
  I: integer;
begin
  if Value <> FLabelAlignment then begin
    FLabelAlignment := Value;
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).Alignment := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelAutoSize(Value: Boolean);
var
  I: integer;
begin
  if Value <> FLabelAutoSize then begin
    FLabelAutoSize := Value;
    for I := 0 to FieldCount - 1 do
      with TLabel(FFieldLabels[I]) do begin
        AutoSize := Value;
        if not Value then begin
          Width := FLabelWidth;
          Height := FLabelHeight;
        end;
      end;
    if Value and (FieldCount > 0) then
      FLabelHeight := TLabel(FFieldLabels[0]).Height;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelCaptions(Value: TStrings);
begin
  FLabelCaptions.Assign(Value);
  if not FAutoCaption then begin
    UpdateCaptions;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelColor(Value: TColor);
var
  I: integer;
begin
  if Value <> FLabelColor then begin
    FLabelColor := Value;
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).Color := Value;
    FLabelParentColor := False;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelFont(Value: TFont);
begin
  if Value <> FLabelFont then begin
    FLabelFont.Assign(Value);
    LabelFontChanged(Self);
  end;
end;

procedure TabcCustomDBEditWin.SetLabelHeight(Value: Integer);
var
  I: integer;
begin
  if Value <> FLabelHeight then begin
    FLabelHeight := Value;
    FLabelAutoSize := False;
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).Height := FLabelHeight;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelOffsetH(Value: Integer);
begin
  if Value <> FLabelOffsetH then begin
    FLabelOffsetH := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelOffsetV(Value: Integer);
begin
  if Value <> FLabelOffsetV then begin
    FLabelOffsetV := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelParentColor(Value: Boolean);
var
  I: integer;
begin
  if Value <> FLabelParentColor then begin
    FLabelParentColor := Value;
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).ParentColor := Value;
    if Value then FLabelColor := Color;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelParentFont(Value: Boolean);
var
  I: integer;
begin
  if Value <> FLabelParentFont then begin
    if Value then FLabelFont.Assign(Font);
    FLabelParentFont := Value;
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).ParentFont := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelWidth(Value: Integer);
var
  I: integer;
begin
  if Value <> FLabelWidth then begin
    FLabelWidth := Value;
    FLabelAutoSize := False;
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).Width := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLabelWordWrap(Value: Boolean);
var
  I: integer;
begin
  if Value <> FLabelWordWrap then begin
    FLabelWordWrap := Value;
    for I := 0 to FieldCount - 1 do
      TLabel(FFieldLabels[I]).WordWrap := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetLayoutEdit(Value: TabcLayoutEdit);
begin
  if Value <> FLayoutEdit then begin
    FLayoutEdit := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetLayoutLabel(Value: TabcLayoutLabel);
begin
  if Value <> FLayoutLabel then begin
    FLayoutLabel := Value;
    if Value = laTop then
      LabelAlignment := taLeftJustify;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetMarginLeft(Value: Integer);
var
  I, dX: integer;
begin
  if Value <> FMarginLeft then begin
    dX := Value - FMarginLeft;
    FMarginLeft := Value;
    if (FLayoutEdit = leVert) then begin
      HorzScrollBar.Position := 0;
      VertScrollBar.Position := 0;
      for I := 0 to FieldCount - 1 do begin
        with TLabel(FFieldLabels[I]) do Left := Left + dX;
        with TabcDBEditWinEdit(FFieldEdits[I]) do Left := Left + dX;
      end;
    end
    else
      AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetMarginTop(Value: Integer);
var
  I, dY: integer;
begin
  if Value <> FMarginTop then begin
    dY := Value - FMarginTop;
    FMarginTop := Value;
    HorzScrollBar.Position := 0;
    VertScrollBar.Position := 0;
    for I := 0 to FieldCount - 1 do begin
      with TLabel(FFieldLabels[I]) do Top := Top + dY;
      with TabcDBEditWinEdit(FFieldEdits[I]) do Top := Top + dY;
    end;
  end;
end;

procedure TabcCustomDBEditWin.SetMaxWidth(Value: Integer);
begin
  if Value <> FMaxWidth then begin
    FMaxWidth := Value;
    AlignFields;
  end;
end;

procedure TabcCustomDBEditWin.SetOnCustomBlobEdit(Value: TabcCustomFieldEditEvent);
var
  I: integer;
begin
  FOnCustomBlobEdit := Value;
  for I := 0 to FieldCount - 1 do
    TabcDBEditWinEdit(FFieldEdits[I]).OnCustomBlobEdit := Value;
end;

procedure TabcCustomDBEditWin.SetPopupAutoEdit(Value: TabcEditFieldOptions);
var
  I: integer;
begin
  if Value <> FPopupAutoEdit then begin
    FPopupAutoEdit := Value;
    for I := 0 to FieldCount - 1 do
      TabcDBEditWinEdit(FFieldEdits[I]).PopupAutoEdit := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetPopupEditBlobAs(Value: TabcEditBlobOption);
var
  I: integer;
begin
  if Value <> FPopupEditBlobAs then begin
    FPopupEditBlobAs := Value;
    for I := 0 to FieldCount - 1 do
      TabcDBEditWinEdit(FFieldEdits[I]).PopupEditBlobAs := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetPopupFormStyle(Value: TabcPopupFormStyle);
var
  I: integer;
begin
  if Value <> FPopupFormStyle then begin
    FPopupFormStyle := Value;
    for I := 0 to FieldCount - 1 do
      TabcDBEditWinEdit(FFieldEdits[I]).PopupFormStyle := Value;
  end;
end;

procedure TabcCustomDBEditWin.SetReadOnly(Value: Boolean);
begin
  if Value <> FReadOnly then begin
    FReadOnly := Value;
    UpdateReadOnlyFields;
  end;
end;

procedure TabcCustomDBEditWin.SetSelectedField(Value: TField);
var
  I: Integer;
begin
  for I := 0 to FieldCount - 1 do
    if Fields[I] = Value then begin
      FieldEdit[I].SetFocus;
      Break;
    end;
end;

procedure TabcCustomDBEditWin.SetSelectedIndex(Value: Integer);
begin
  FieldEdit[Value].SetFocus;
end;

procedure TabcCustomDBEditWin.UpdateCaptions;
var
  I: integer;
begin
  for I := 0 to FieldCount - 1 do
    with TLabel(FFieldlabels[I]) do
      if FAutoCaption then
        Caption := Fields[I].DisplayName
      else
        if I < FLabelCaptions.Count then
          Caption := FLabelCaptions.Strings[I]
        else
          Caption := '';
end;

procedure TabcCustomDBEditWin.UpdateData;
var
  I: Integer;
begin
 for I := 0 to FieldCount - 1 do
  with TabcDBEditWinEdit(FFieldEdits[I]) do
    if Modified then UpdateData(Self);
end;

procedure TabcCustomDBEditWin.UpdateEditCharWidth;
const
  CharO = 79;
var
  DC: HDC;
  OldFont: HFont;
begin
  DC := GetDC(0);
  try
    OldFont := SelectObject(DC, FEditFont.Handle);
    GetCharWidth(DC, CharO, CharO, FEditCharWidth);
    SelectObject(DC, OldFont);
  finally
    ReleaseDC(0, DC);
  end;
end;

procedure TabcCustomDBEditWin.UpdateEditSize;
var
  I: integer;
begin
  for I := 0 to FieldCount - 1 do
    with TabcDBEditWinEdit(FFieldEdits[I]) do
      if FEditAutoSize then
        Width := CalcEditWidth(Fields[I])
      else
        Width := FEditWidth;
end;

procedure TabcCustomDBEditWin.UpdateField(Index: Integer);
begin
  if FDataLink.Editing and CanModifyField(Index) then
    if not (Fields[Index] is TBlobField) then
      Fields[Index].Text := TabcDBEditWinEdit(FFieldEdits[Index]).Text;
end;

procedure TabcCustomDBEditWin.UpdateReadOnlyFields;
var
  I: integer;
begin
  for I := 0 to FieldCount - 1 do
    with TabcDBEditWinEdit(FFieldEdits[I]) do
      if not FieldEditing(I) then begin
        if rofEditReadOnly in FFieldReadOnly then
          ReadOnly := True
        else
          ReadOnly := False;
        if rofEditDisabled in FFieldReadOnly then
          Enabled := False
        else
          Enabled := True;
        if (rofEditColor in FFieldReadOnly) or FEditParentColor then
          ParentColor := True
        else
          Color := FEditColor;
        if rofLabelDisabled in FFieldReadOnly then
          TLabel(FFieldLabels[I]).Enabled := False
        else
          TLabel(FFieldLabels[I]).Enabled := True;
      end
      else begin
        ReadOnly := False;
        Enabled := True;
        if FEditParentColor then
          ParentColor := True
        else
          Color := FEditColor;
        TLabel(FFieldLabels[I]).Enabled := True;
      end;
end;

{ Defer action processing to datalink }

function TabcCustomDBEditWin.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := (FDataLink <> nil) and FDataLink.ExecuteAction(Action);
end;

function TabcCustomDBEditWin.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := (FDataLink <> nil) and FDataLink.UpdateAction(Action);
end;

end.
