unit abcexctl;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)
{$S-,W-,R-}
{$C PRELOAD}

{$I abcdefs.inc}

interface

uses Windows, Messages, SysUtils, Classes, Controls, Menus, Graphics,
  StdCtrls, ExtCtrls, Forms, Buttons, CommCtrl, abctileb;

type

  TabcTextStyle = (tsNormal, tsRaised, tsLowered);

  TabcCustomBackground = class(TGraphicControl)
  private
    FActive: Boolean;
    FCreating: Boolean;
    FGradient: TabcGradient;
    function GetColorFrom: TColor;
    function GetColorTo: TColor;
    function GetDrawDirection: TabcDrawDirection;
    function GetDrawSteps: Word;
    function GetDrawMethod: TabcGradientDrawMethod;
    function GetDrawBuffered: Boolean;
    procedure SetActive(Value: Boolean);
    procedure SetColorFrom(Value: TColor);
    procedure SetColorTo(Value: TColor);
    procedure SetDrawDirection(Value: TabcDrawDirection);
    procedure SetDrawSteps(Value: Word);
    procedure SetDrawMethod(Value: TabcGradientDrawMethod);
    procedure SetDrawBuffered(Value: Boolean);
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure GradientChanged(Sender: TObject);
  protected
    procedure ImageChanged; virtual;
    procedure Loaded; override;
    procedure Paint; override;
    property Active: Boolean read FActive write SetActive;
    property Align;
    property ColorFrom: TColor read GetColorFrom write SetColorFrom default clBlue;
    property ColorTo: TColor read GetColorTo write SetColorTo default clBlack;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property DrawDirection: TabcDrawDirection read GetDrawDirection write SetDrawDirection default ddVertical;
    property DrawSteps: Word read GetDrawSteps write SetDrawSteps default 236;
    property DrawMethod: TabcGradientDrawMethod read GetDrawMethod write SetDrawMethod default agdDraw;
    property DrawBuffered: Boolean read GetDrawBuffered write SetDrawBuffered default False;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
  end;

  TabcBackground = class(TabcCustomBackground)
  published
    property Active;
    property Align;
    property Anchors;
    property Constraints;
    property DragKind;
    property ColorFrom;
    property ColorTo;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property DrawDirection;
    property DrawSteps;
    property DrawMethod;
    property DrawBuffered;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

  TabcMDIBackground = class(TabcCustomBackground)
  private
    FClientInstance: TFarProc;
    FClientHandle: HWND;
    FDefClientProc: TFarProc;
    FShowScrollBars: Boolean;
    procedure EraseMDIClientBkgrnd(var Message: TMessage);
    function GetAlign: TAlign;
    function GetCursor: TCursor;
    function GetHint: string;
    function GetLeft: integer;
    function GetTop: integer;
    function GetWidth: integer;
    function GetHeight: integer;
    procedure SubClassMDIClient(AOwner: TForm);
    procedure SetShowScrollBars(Value: Boolean);
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
  protected
    procedure ImageChanged; override;
    procedure Loaded; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Invalidate; override;
  published
    property Active;
    property ColorFrom;
    property ColorTo;
    property DrawDirection;
    property DrawSteps;
    property DrawMethod;
    property DrawBuffered;
    property Align: TAlign read GetAlign;
    property Cursor: TCursor read GetCursor;
    property Hint: string read GetHint;
    property Left: integer read GetLeft;
    property Top: integer read GetTop;
    property Width: integer read GetWidth;
    property Height: integer read GetHeight;
    property Visible;
    property ShowScrollBars: Boolean read FShowScrollBars write SetShowScrollBars default False;
  end;

  TabcTiledImageStyle = (sCenter, sClip, sFit, sHeight, sStretch, sTile, sWidth);

  TabcCustomTiledImage = class(TGraphicControl)
  private
    FPicture: TPicture;
    FTiledImageStyle: TabcTiledImageStyle;
    function GetCanvas: TCanvas;
    procedure PictureChanged(Sender: TObject);
    procedure SetImageStyle(Value: TabcTiledImageStyle);
    procedure SetPicture(Value: TPicture);
  protected
    procedure AdjustSize; override;
    function GetPalette: HPALETTE; override;
    procedure ImageChanged; virtual;
    procedure Paint; override;
    property Align;
    property AutoSize;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ImageStyle: TabcTiledImageStyle read FTiledImageStyle write SetImageStyle default sTile;
    property ParentShowHint;
    property Picture: TPicture read FPicture write SetPicture;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas: TCanvas read GetCanvas;
  published
  end;

  TabcTiledImage = class(TabcCustomTiledImage)
  published
    property Align;
    property Anchors;
    property Constraints;
    property DragKind;
    property AutoSize;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ImageStyle;
    property ParentShowHint;
    property Picture;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

  TabcMDITiledImage = class(TabcCustomTiledImage)
  private
    FClientHandle: HWND;
    FClientInstance: TFarProc;
    FDefClientProc: TFarProc;
    FShowScrollBars: Boolean;
    procedure EraseMDIClientBkgrnd(var Message: TMessage);
    procedure SubClassMDIClient(AOwner: TForm);
    function GetAlign: TAlign;
    function GetCursor: TCursor;
    function GetHint: string;
    function GetLeft: integer;
    function GetTop: integer;
    function GetWidth: integer;
    function GetHeight: integer;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure SetShowScrollBars(Value: Boolean);
  protected
    procedure ImageChanged; override;
    procedure Loaded; override;
    procedure Paint; override;
    procedure PaintBitmap(var Bitmap: TBitmap);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Invalidate; override;
  published
    property ImageStyle;
    property Picture;
    property Visible;
    property Align: TAlign read GetAlign;
    property Cursor: TCursor read GetCursor;
    property Hint: string read GetHint;
    property Left: integer read GetLeft;
    property Top: integer read GetTop;
    property Width: integer read GetWidth;
    property Height: integer read GetHeight;
    property ShowScrollBars: Boolean read FShowScrollBars write SetShowScrollBars default False;
  end;

  { TClock Label }

  TabcClockLabel = class(TCustomLabel)
  private
    FEnabled: Boolean;
    FFormat: string;
    FOnTimer: TNotifyEvent;
    FTimer: TTimer;
    function GetCaption: string;
    function GetInterval: Word;
    procedure SetInterval(Value: Word);
    procedure Timer(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetEnabled(Value: Boolean); override;
  published
    property Enabled;
    property Format: string read FFormat write FFormat;
    property Interval: Word read GetInterval write SetInterval;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
    property Align;
    property Alignment;
    property AutoSize;
    property Anchors;
    property Constraints;
    property DragKind;
    property BiDiMode;
    property ParentBiDiMode;
    property Caption: string read GetCaption;
    property Color;
    property DragCursor;
    property DragMode;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Transparent;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

{ TabcPopupSelect }

  TabcNoSelectionEvent = procedure(Sender: TObject) of object;
  TabcPopupEvent = procedure(Sender: TObject; Items: TStrings; var MatchTo: string) of object;
  TabcSelectionEvent = procedure(Sender: TObject; var Selection: string) of object;

  TabcColItems = 1..50;

  TabcPopupSelect = class(TEdit)
  private
    fBarBreak: Boolean;
    fItemIndex: integer;
    fItems: TStringList;
    fItemsPerCol: TabcColItems;
    fMenu: TPopupMenu;
    fMenuItems: TStringList;  {temporary items on popup}
    fOnNoSelection: TabcNoSelectionEvent;
    fOnPopup: TabcPopupEvent;
    fOnSelection: TabcSelectionEvent;
    fPopupOffsetX: integer;
    fPopupOffsetY: integer;
    fPopupYAdjust: Boolean;
    fSorted: Boolean;
    procedure menuClick(Sender: TObject);
    procedure menuPopup;
    procedure SetItemIndex(Value: integer);
    procedure SetItems(Value: TStringList);
    procedure SetSorted(Value: Boolean);
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LButtonDown;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BarBreak: Boolean read fBarBreak write fBarBreak;
    property Items: TStringList read fItems write SetItems;
    property ItemsPerCol: TabcColItems read fItemsPerCol write fItemsPerCol default 10;
    property ItemIndex: integer read fItemIndex write SetItemIndex default -1;
    property Sorted: Boolean read fSorted write SetSorted;
    property PopupOffsetX: integer read fPopupOffsetX write fPopupOffsetX default -1;
    property PopupOffsetY: integer read fPopupOffsetY write fPopupOffsetY default -1;
    property PopupYAdjust: Boolean read fPopupYAdjust write fPopupYAdjust default True;
    property OnNoSelection: TabcNoSelectionEvent read fOnNoSelection write fOnNoSelection;
    property OnPopup: TabcPopupEvent read fOnPopup write fOnPopup;
    property OnSelection: TabcSelectionEvent read fOnSelection write fOnSelection;
  end;

{ TabcSuperLabel }

  TabcVAlignment = (vaTop, vaBottom, vaCenter);

  TabcSuperLabel = class(TCustomLabel)
  private
    FBColor: TColor;
    FBMargin: Byte;
    FBStyle: TBorderStyle;
    FCtl3D: Boolean;
    FLMargin: Byte;
    FRMargin: Byte;
    FTMargin: Byte;
    FVAlignment: TabcVAlignment;
    function getInhAutoSize: boolean;
  protected
    procedure Paint; override;
    function  DoDrawTheText(Rect: TRect; Flags: Word; Text: pChar): integer;
    procedure setVAlignment(setTo: TabcVAlignment);
    procedure setTMargin(setTo: Byte);
    procedure setLMargin(setTo: Byte);
    procedure setRMargin(setTo: Byte);
    procedure setBMargin(setTo: Byte);
    procedure setBStyle(setTo: TBorderStyle);
    procedure setCtl3D(setTo: Boolean);
    procedure setBColor(setTo: TColor);
    function  getWordWrap: Boolean;
    procedure setWordWrap(setTo: Boolean);
    property  AutoSize: Boolean read getInhAutoSize;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property Alignment;
    property Anchors;
    property Constraints;
    property DragKind;
    property BiDiMode;
    property ParentBiDiMode;
    property BorderColor: TColor read FBColor write setBColor default clWindowFrame;
    property BorderStyle: TBorderStyle read FBStyle write setBStyle default bsNone;
    property BottomMargin: Byte read FBMargin write setBMargin default 0;
    property Caption;
    property Color;
    property Ctl3D: Boolean read FCtl3D write setCtl3D default true;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property LeftMargin: Byte read FLMargin write setLMargin default 0;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property RightMargin: Byte read FRMargin write setRMargin default 0;
    property TopMargin: Byte read FTMargin write setTMargin default 0;
    property Transparent;
    property VerticalAlignment: TabcVAlignment read FVAlignment write setVAlignment default vaTop;
    property Visible;
    property WordWrap read getWordWrap write setWordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

{ TabcAutoLabel }

  TabcAutoLabel = class(TLabel)
  private
    FBorderColor: TColor;
    FBorderStyle: TBorderStyle;
    FCtl3D: Boolean;
    FSeparator: Char;
    FTextStyle: TabcTextStyle;
    FWordWrap: Boolean;
    FYOffset: integer;
    procedure AdjustHeight;
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetCtl3D(Value: boolean);
    procedure SetBorderColor(Value: TColor);
    procedure SetSeparator(Value: Char);
    procedure SetTextStyle(Value:TabcTextStyle);
    procedure SetWordWrap(Value: Boolean);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    procedure Paint; override;
    procedure SetAutoSize(Value: Boolean); override;
    procedure Loaded; override;
    procedure AdjustBounds; override;
    procedure DoDrawText(var Rect: TRect; Flags: Longint); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Alignment default taRightJustify;
    property Align;
    property Anchors;
    property Constraints;
    property DragKind;
    property BiDiMode;
    property ParentBiDiMode;
    property BorderColor: TColor read FBorderColor write SetBorderColor default clWindowFrame;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsNone;
    property Caption;
    property Color;
    property Ctl3D: Boolean read FCtl3D write SetCtl3D default true;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Separator: Char read FSeparator write SetSeparator default ':';
    property ShowAccelChar;
    property ShowHint;
    property TextStyle : TabcTextStyle read FTextStyle write SetTextStyle default tsNormal;
    property Transparent;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default False;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

{ TabcEditLabel }

  TabcEditLabelLayout = (elLeft, elTop);

  TabcEditLabel = class(TWinControl)
  private
    { sub components }
    fLabel: TLabel;
    fEdit: TEdit;
    { property values }
    FSaveLabelWidth: Integer;
    FLabelOffsetX: Integer;
    FLabelOffsetY: Integer;
    FCalcLayout: Boolean;
    FSizing: Boolean;
    FLabelFixed: Boolean;
    fLayout: TabcEditLabelLayout;
    fLabelWidth: integer;
    fLabelHeight: integer;
    fOnChange: TNotifyEvent;
    { property access methods }
    function  GetCursor: TCursor;
    procedure SetCursor(Value: TCursor);
    function  GetDragCursor: TCursor;
    procedure SetDragCursor(Value: TCursor);
    function  GetHint: string;
    procedure SetHint(Value: string);
    function  GetParentShowHint: Boolean;
    procedure SetParentShowHint(Value: Boolean);
    function  GetEditPopupMenu: TPopupMenu;
    procedure SetPopupMenu(Value: TPopupMenu);
    function  GetShowHint: Boolean;
    procedure SetShowHint(Value: Boolean);
    procedure SetLabelOffsetX(Value: Integer);
    procedure SetLabelOffsetY(Value: Integer);
    procedure setLabelWidth(setTo: integer);
    procedure setLabelHeight(setTo: integer);
    function  getHelpContext: THelpContext;
    procedure setHelpContext(setTo: THelpContext);
    function  getLabelFont: TFont;
    procedure setLabelFont(setTo: TFont);
    function  getEditFont: TFont;
    procedure setEditFont(setTo: TFont);
    function  getLabelParentFont: boolean;
    procedure setLabelParentFont(setTo: boolean);
    function  getEditParentFont: boolean;
    procedure setEditParentFont(setTo: boolean);
    function  getAlignment: TAlignment;
    procedure setAlignment(setTo: TAlignment);
    function  getCaption: TCaption;
    procedure setCaption(setTo: TCaption);
    function  getShowAccelChar: boolean;
    procedure setShowAccelChar(setTo: boolean);
    function  getTabStop: boolean;
    procedure setTabStop(setTo: boolean);
    function  getAutoSelect: boolean;
    procedure setAutoSelect(setTo: boolean);
    function  getBorderStyle: TBorderStyle;
    procedure setBorderStyle(setTo: TBorderStyle);
    function  getCharCase: TEditCharCase;
    procedure setCharCase(setTo: TEditCharCase);
    function  getEditColor: TColor;
    procedure setEditColor(setTo: TColor);
    function  getEditParentColor: boolean;
    procedure setEditParentColor(setTo: boolean);
    function  getCtl3D: boolean;
    procedure setCtl3D(setTo: boolean);
    function  getParentCtl3D: boolean;
    procedure setParentCtl3D(setTo: boolean);
    function  getHideSelection: boolean;
    procedure setHideSelection(setTo: boolean);
    function  getMaxLength: integer;
    procedure setMaxLength(setTo: integer);
    function  getOEMConvert: boolean;
    procedure setOEMConvert(setTo: boolean);
    function  getPasswordChar: char;
    procedure setPasswordChar(setTo: char);
    function  getReadOnly: boolean;
    procedure setReadOnly(setTo: boolean);
    function  getModified: Boolean;
    procedure setModified(setTo: Boolean);
    function  getSelLength: Integer;
    procedure setSelLength(setTo: Integer);
    function  getSelStart: Integer;
    procedure setSelStart(setTo: Integer);
    function  getSelText: string;
    procedure setSelText(setTo: string);
    { event property access methods }
    function  getOnChange: TNotifyEvent;
    procedure setOnChange(setTo: TNotifyEvent);
    function  getOnClick: TNotifyEvent;
    procedure setOnClick(setTo: TNotifyEvent);
    function  getOnDblClick: TNotifyEvent;
    procedure setOnDblClick(setTo: TNotifyEvent);
    function  getOnEnter: TNotifyEvent;
    procedure setOnEnter(setTo: TNotifyEvent);
    function  getOnExit: TNotifyEvent;
    procedure setOnExit(setTo: TNotifyEvent);
    function  getOnKeyDown: TKeyEvent;
    procedure setOnKeyDown(setTo: TKeyEvent);
    function  getOnKeyPress: TKeyPressEvent;
    procedure setOnKeyPress(setTo: TKeyPressEvent);
    function  getOnKeyUp: TKeyEvent;
    procedure setOnKeyUp(setTo: TKeyEvent);
    function  getOnMouseDown: TMouseEvent;
    procedure setOnMouseDown(setTo: TMouseEvent);
    function  getOnMouseMove: TMouseMoveEvent;
    procedure setOnMouseMove(setTo: TMouseMoveEvent);
    function  getOnMouseUp: TMouseEvent;
    procedure setOnMouseUp(setTo: TMouseEvent);
    function  GetOnDragDrop: TDragDropEvent;
    procedure SetOnDragDrop(Value: TDragDropEvent);
    function  GetOnDragOver: TDragOverEvent;
    procedure SetOnDragOver(Value: TDragOverEvent);
    function  GetOnEndDrag: TEndDragEvent;
    procedure SetOnEndDrag(Value: TEndDragEvent);
    function  GetOnStartDrag: TStartDragEvent;
    procedure SetOnStartDrag(Value: TStartDragEvent);
    { internal support methods }
    procedure initLayout;
    procedure textChanged(Sender: TObject);
    procedure labelClicked(Sender: TObject);
    function  isLabelFontStored: boolean;
    function  isEditFontStored: boolean;
    function  isEditColorStored: boolean;
    function  isCtl3DStored: boolean;
    function  isShowHintStored: boolean;
    { message handling methods }
    procedure WMSize(var Message: TMessage); message WM_Size;
    procedure WMNCHitTest(var Message: TMessage); message WM_NCHitTest;
    procedure CMTextChanged(var Message: TMessage); message CM_TextChanged;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure SetDragMode(Value: TDragMode); override;
    function  GetPopupMenu: TPopupMenu; override;
    procedure ChangeScale(M, D: Integer); override;
    procedure paintWindow(DC: HDC); override;
    procedure setLayout(setTo: TabcEditLabelLayout); virtual;
    procedure DoEnter; override; {replaces CMEnter proc}
    procedure Loaded; override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor  Destroy; override;
    procedure SetFocus; override;
    procedure Clear;
    procedure ClearSelection;
    procedure CopyToClipboard;
    procedure CutToClipboard;
    function Focused: Boolean; override;
    procedure PasteFromClipboard;
    function  GetSelTextBuf(Buffer: PChar; BufSize: Integer): Integer;
    procedure SelectAll;
    procedure SetSelTextBuf(Buffer: PChar);
    property Modified: Boolean read GetModified write SetModified;
    property SelLength: Integer read GetSelLength write SetSelLength;
    property SelStart: Integer read GetSelStart write SetSelStart;
    property SelText: string read GetSelText write SetSelText;
  published
    { label properties }
    property Alignment: TAlignment read getAlignment write setAlignment default taLeftJustify;
    property Caption: TCaption read getCaption write setCaption;
    property ShowAccelChar: boolean read getShowAccelChar write setShowAccelChar default True;
    property LabelFont: TFont read getLabelFont write setLabelFont stored isLabelFontStored;
    property LabelParentFont: boolean read getLabelParentFont write setLabelParentFont default True;
    { edit properties }
    property HelpContext: THelpContext read getHelpContext write setHelpContext default 0;
    property Enabled;
    property TabStop: boolean read getTabStop write setTabStop default True;
    property EditFont: TFont read getEditFont write setEditFont stored isEditFontStored;
    property EditParentFont: boolean read getEditParentFont write setEditParentFont default True;
    property AutoSelect: boolean read getAutoSelect write setAutoSelect default True;
    property BorderStyle: TBorderStyle read getBorderStyle write setBorderStyle default bsSingle;
    property CharCase: TEditCharCase read getCharCase write setCharCase default ecNormal;
    property EditColor: TColor read getEditColor write setEditColor stored isEditColorStored;
    property EditParentColor: boolean read getEditParentColor write setEditParentColor default False;
    property Ctl3D: boolean read getCtl3D write setCtl3D stored isCtl3DStored;
    property ParentCtl3D: boolean read getParentCtl3D write setParentCtl3D default True;
    property HideSelection: boolean read getHideSelection write setHideSelection default True;
    property MaxLength: integer read getMaxLength write setMaxLength default 0;
    property OEMConvert: boolean read getOEMConvert write setOEMConvert default False;
    property PasswordChar: char read getPasswordChar write setPasswordChar default #0;
    property ReadOnly: boolean read getReadOnly write setReadOnly default false;
    { common properties }
    property LabelOffsetX: Integer read FLabelOffsetX write SetLabelOffsetX;
    property LabelOffsetY: Integer read FLabelOffsetY write SetLabelOffsetY;
    property LabelWidth: integer read fLabelWidth write setLabelWidth;
    property LabelHeight: integer read fLabelHeight write setLabelHeight;
    property Layout: TabcEditLabelLayout read fLayout write setLayout default elLeft;
    { shared properties }
    property Anchors;
    property Constraints;
    property DragKind;
    property BiDiMode;
    property ParentBiDiMode;
    property Cursor: TCursor read GetCursor write SetCursor;
    property DragCursor: TCursor read GetDragCursor write SetDragCursor;
    property DragMode;
    property Hint: string read GetHint write SetHint;
    property ParentShowHint: Boolean read GetParentShowHint write SetParentShowHint;
    property PopupMenu: TPopupMenu read GetEditPopupMenu write SetPopupMenu;
    property ShowHint: Boolean read GetShowHint write SetShowHint stored isShowHintStored;
    { event properties }
    property OnChange: TNotifyEvent read getOnChange write setOnChange;
    property OnClick: TNotifyEvent read getOnClick write setOnClick;
    property OnEnter: TNotifyEvent read getOnEnter write setOnEnter;
    property OnExit: TNotifyEvent read getOnExit write setOnExit;
    property OnKeyDown: TKeyEvent read getOnKeyDown write setOnKeyDown;
    property OnKeyPress: TKeyPressEvent read getOnKeyPress write setOnKeyPress;
    property OnKeyUp: TKeyEvent read getOnKeyUp write setOnKeyUp;
    property OnMouseDown: TMouseEvent read getOnMouseDown write setOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read getOnMouseMove write setOnMouseMove;
    property OnMouseUp: TMouseEvent read getOnMouseUp write setOnMouseUp;
    property OnDblClick: TNotifyEvent read getOnDblClick write setOnDblClick;
    property OnDragDrop: TDragDropEvent read getOnDragDrop write setOnDragDrop;
    property OnDragOver: TDragOverEvent read getOnDragOver write setOnDragOver;
    property OnEndDrag: TEndDragEvent read getOnEndDrag write setOnEndDrag;
    { properties redeclared as published }
    property Text;
    property Color;
    property ParentColor;
    property TabOrder;
    property Visible;
    property OnStartDrag: TStartDragEvent read GetOnStartDrag write SetOnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

  TabcMDIScroll = class(TComponent)
  private
    FClientInstance: TFarProc;
    FClientHandle: HWND;
    FDefClientProc: TFarProc;
    FEnabled: Boolean;
    procedure SetEnabled(Value: Boolean);
  protected
    procedure SubClassMDIClient(AOwner: TForm);
    procedure HideMDIScrollBars(var Message: TMessage);
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled;
  end;

implementation

uses Consts, abcconst;

{ TabcCustomBackground }

constructor TabcCustomBackground.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCreating := True;
  FGradient := TabcGradient.Create;
  FGradient.OnChange := GradientChanged;
  Height := 105;
  Width := 105;
  ControlStyle := ControlStyle + [csOpaque, csReplicatable];
  FActive := True;
  FCreating := False;
end;

destructor TabcCustomBackground.Destroy;
begin
  Destroying;
  FGradient.Free;
  inherited Destroy;
end;

procedure TabcCustomBackground.Loaded;
begin
  inherited Loaded;
//  FGradient.RefreshBuffer;
  ImageChanged;
end;

procedure TabcCustomBackground.GradientChanged(Sender: TObject);
begin
  if not (csLoading in ComponentState) then
    ImageChanged;
end;

procedure TabcCustomBackground.SetActive(Value: Boolean);
begin
  if Value <> FActive then begin
    FActive:= Value;
    ImageChanged;
  end;
end;

function TabcCustomBackground.GetColorFrom: TColor;
begin
  Result := FGradient.ColorFrom;
end;

function TabcCustomBackground.GetColorTo: TColor;
begin
  Result := FGradient.ColorTo;
end;

function TabcCustomBackground.GetDrawDirection: TabcDrawDirection;
begin
  Result := FGradient.Direction;
end;

function TabcCustomBackground.GetDrawSteps: Word;
begin
  Result := FGradient.ColorCount;
end;

function TabcCustomBackground.GetDrawMethod: TabcGradientDrawMethod;
begin
  Result := FGradient.DrawMethod;
end;

function TabcCustomBackground.GetDrawBuffered: Boolean;
begin
  Result := FGradient.DrawBuffered;
end;

procedure TabcCustomBackground.SetColorFrom(Value:TColor);
begin
  FGradient.ColorFrom := Value;
end;

procedure TabcCustomBackground.SetColorTo(Value:TColor);
begin
  FGradient.ColorTo := Value;
end;

procedure TabcCustomBackground.SetDrawDirection(Value: TabcDrawDirection);
begin
  FGradient.Direction := Value;
  if (Value in [ddRLDiagonal,ddCenter]) and ([csLoading, csDesigning] * ComponentState = [csDesigning]) then
    DrawBuffered := True;
end;

procedure TabcCustomBackground.SetDrawSteps(Value: Word);
begin
  if Value > High(TabcColorCount) then
    Value := High(TabcColorCount)
  else if Value < Low(TabcColorCount) then
    Value := Low(TabcColorCount);
  FGradient.ColorCount := Value;
end;

procedure TabcCustomBackground.SetDrawMethod(Value: TabcGradientDrawMethod);
begin
  FGradient.DrawMethod := Value;
end;

procedure TabcCustomBackground.SetDrawBuffered(Value: Boolean);
begin
  FGradient.DrawBuffered := Value;
end;

procedure TabcCustomBackground.ImageChanged;
begin
  if not (csDestroying in ComponentState) then
    Invalidate;
end;

procedure TabcCustomBackground.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  if not (FCreating or (csLoading in ComponentState)) then ImageChanged;
end;

procedure TabcCustomBackground.CMVisibleChanged(var Message: TMessage);
begin
  if Visible then
    ControlStyle := ControlStyle + [csOpaque]
  else
    ControlStyle := ControlStyle - [csOpaque];
  inherited;
end;

procedure TabcCustomBackground.Paint;
begin
  if not (csDestroying in ComponentState) then
    with inherited Canvas do begin
      if csDesigning in ComponentState then begin
        Pen.Style := psDash;
        Brush.Style := bsClear;
        Rectangle(0, 0, Width, Height);
      end;
      if Active then
        FGradient.Draw(inherited Canvas, Rect(0, 0, Width, Height));
    end;
end;

constructor TabcMDIBackground.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TForm) then
    if (TForm(AOwner).FormStyle = fsMDIForm) then
      inherited Align := alClient
    else
      raise EComponentError.Create(sabcErrorOwnerMDIFormExpected)
  else
    raise EComponentError.Create(sabcErrorOwnerMDIFormExpected);
end;

destructor TabcMDIBackground.Destroy;
begin
  Destroying;
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FDefClientProc));
  {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FClientInstance);
  inherited Destroy;
end;

procedure TabcMDIBackground.Loaded;
begin
  inherited Loaded;
  if FClientHandle = 0 then
    if not(csDesigning in ComponentState) then
      SubClassMDIClient(Owner as TForm);
end;

procedure TabcMDIBackground.SubClassMDIClient(AOwner: TForm);
begin
  AOwner.HandleNeeded;
  FClientHandle := AOwner.ClientHandle;
  FClientInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(EraseMDIClientBkgrnd);
  FDefClientProc := Pointer(GetWindowLong(FClientHandle, GWL_WNDPROC));
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FClientInstance));
end;

procedure TabcMDIBackground.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TabcMDIBackground.Invalidate;
begin
  if FClientHandle <> 0 then
    RedrawWindow(FClientHandle, nil, 0, RDW_ERASE + RDW_INVALIDATE)
  else
    inherited;
end;

procedure TabcMDIBackground.EraseMDIClientBkgrnd(var Message: TMessage);
begin
  if Visible then
    case Message.Msg of
      WM_ERASEBKGND:
        begin
          FGradient.DrawToDC(Message.wParam, ClientRect);
          Message.Result := 1;
          Exit;
        end;
      WM_WINDOWPOSCHANGING:
        if not FShowScrollBars then
          ShowScrollBar(FClientHandle, SB_BOTH, False)
        else
          Invalidate;
      WM_HSCROLL, WM_VSCROLL:
        if FShowScrollBars then Invalidate;
    end;
  Message.Result := CallWindowProc(FDefClientProc, FClientHandle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure TabcMDIBackground.ImageChanged;
{erase whole client area, not just current clip region}
begin
  if (csDesigning in ComponentState) then
    inherited ImageChanged
  else
    if not (csLoading in ComponentState) then
      Invalidate;
end;

procedure TabcMDIBackground.Paint;
begin
  if csDesigning in ComponentState then
    inherited Paint;
end;

function TabcMDIBackground.GetAlign: TAlign; begin Result := inherited Align; end;
function TabcMDIBackground.GetCursor: TCursor; begin Result := inherited Cursor; end;
function TabcMDIBackground.GetHint: string; begin Result := inherited Hint; end;
function TabcMDIBackground.GetLeft: integer; begin Result := inherited Left; end;
function TabcMDIBackground.GetTop: integer; begin Result := inherited Top; end;
function TabcMDIBackground.GetWidth: integer; begin Result := inherited Width; end;
function TabcMDIBackground.GetHeight: integer; begin Result := inherited Height; end;

procedure TabcMDIBackground.SetShowScrollBars(Value: Boolean);
var
  I: integer;
  twp: TWindowPos;
  pwp: ^TWindowPos;
  ChildPos: TPoint;
  PChildPos: ^TPoint;
  AMessage: TMessage;
begin
  if Value <> FShowScrollBars then
  begin
    FShowScrollBars := Value;
    if (csDesigning in ComponentState) then Exit;
    if (FClientHandle <> 0) then
      if Value then
      begin
        with TForm(Owner) do
          for I := 0 to MDIChildCount - 1 do
            with MDIChildren[I] do begin
              ChildPos := Point(Left, Top);
              PChildPos := @ChildPos;
              DefMDIChildProc(Handle, WM_MOVE, 0, Longint(PChildPos));
            end;
      end
      else
      begin
        twp.hWnd := TForm(Owner).Handle;
        twp.flags := SWP_DRAWFRAME or SWP_NOACTIVATE or SWP_NOMOVE or
          SWP_NOOWNERZORDER or SWP_NOSIZE or SWP_NOZORDER;
        pwp := @twp;
        AMessage.Msg := WM_WINDOWPOSCHANGING;
        AMessage.wParam := 0;
        AMessage.LParam := Longint(pwp);
        EraseMDIClientBkgrnd(AMessage);
      end;
  end;
end;

{ TabcCustomTiledImage }

constructor TabcCustomTiledImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FTiledImageStyle := sTile;
  Height := 105;
  Width := 105;
end;

destructor TabcCustomTiledImage.Destroy;
begin
  Destroying;
  FPicture.Free;
  inherited Destroy;
end;

function TabcCustomTiledImage.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture.Graphic is TBitmap then
    Result := TBitmap(FPicture.Graphic).Palette;
end;

procedure TabcCustomTiledImage.ImageChanged;
begin
  if not (csDestroying in ComponentState) then Invalidate;
end;

procedure TabcCustomTiledImage.Paint;
var
  Dest: TRect;
  XPos, YPos: Integer;
  PicRatio, ImageRatio: Double;
begin
  if csDesigning in ComponentState then
    with inherited Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  if not ((Picture = nil) or (Picture.Graphic = nil) or (Picture.Graphic.Empty)) then
  case ImageStyle of
    sClip:
      with inherited canvas do
        Draw(0,0,Picture.Graphic);
    sCenter:
    begin
      XPos:= (Width - Picture.Width) div 2;
      YPos:= (Height - Picture.Height) div 2;
      with inherited Canvas do
        Draw(Xpos,YPos, Picture.Graphic);
    end;
    sFit:
    If (Picture.Width > 0) and (Picture.Height > 0) then
    begin
      PicRatio:= Picture.Height / Picture.Width;
      ImageRatio:= Height/Width;
      If PicRatio > ImageRatio then
        begin
          XPos:= Trunc(Height / PicRatio);
          YPos:= Height;
        end
      else
        begin
          XPos:= Width;
          YPos:= Trunc(Width * PicRatio);
        end;
      Dest := Rect(0,0,XPos,YPos);
      with inherited Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
    sHeight:
    begin
      XPos:= Trunc(Picture.Width * (Height / Picture.Height));
      YPos:= Height;
      Dest := Rect(0,0,XPos,YPos);
      with inherited Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
    sStretch:
    begin
      Dest := ClientRect;
      with inherited Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
    sTile:
    begin
      XPos:= 0;
      While XPos < Width do
      begin
        YPos:= 0;
        While YPos < Height do
        begin
          with inherited Canvas do
            Draw(XPos, YPos, Picture.Graphic);
          YPos:= YPos + Picture.Height;
        end;
        XPos:= XPos + Picture.Width;
      end;
    end;
    sWidth:
    begin
      XPos:= Width;
      YPos:= Trunc(Width * (Picture.Height / Picture.Width));
      Dest := Rect(0,0,XPos,YPos);
      with inherited Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
  end;
end;

function TabcCustomTiledImage.GetCanvas: TCanvas;
var
  Bitmap: TBitmap;
begin
  if Picture.Graphic = nil then
  begin
    Bitmap := TBitmap.Create;
    try
      Bitmap.Width := Width;
      Bitmap.Height := Height;
      Picture.Graphic := Bitmap;
    finally
      Bitmap.Free;
    end;
  end;
  if Picture.Graphic is TBitmap then
    Result := TBitmap(Picture.Graphic).Canvas
  else
    raise EInvalidOperation.Create(SImageCanvasNeedsBitmap);
end;

procedure TabcCustomTiledImage.AdjustSize;
begin
  if AutoSize then
    PictureChanged(Self);
  inherited;
end;

procedure TabcCustomTiledImage.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TabcCustomTiledImage.SetImageStyle(Value: TabcTiledImageStyle);
begin
  FTiledImageStyle := Value;
  ImageChanged;
end;

procedure TabcCustomTiledImage.PictureChanged(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then
  begin
    if AutoSize and (Picture.Width > 0) and (Picture.Height > 0) then
      SetBounds(Left, Top, Picture.Width, Picture.Height);
    if (Picture.Graphic is TBitmap) and (Picture.Width = Width) and
      (Picture.Height = Height) then
      ControlStyle := ControlStyle + [csOpaque]
    else
      ControlStyle := ControlStyle - [csOpaque];
    ImageChanged;
  end;
end;

constructor TabcMDITiledImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TForm) then
    if (TForm(AOwner).FormStyle = fsMDIForm) then
      inherited Align := alClient
    else
      raise EComponentError.Create(sabcErrorOwnerMDIFormExpected)
  else
    raise EComponentError.Create(sabcErrorOwnerMDIFormExpected);
end;

destructor TabcMDITiledImage.Destroy;
begin
  Destroying;
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FDefClientProc));
  {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FClientInstance);
  inherited Destroy;
end;

procedure TabcMDITiledImage.Loaded;
begin
  inherited Loaded;
  if FClientHandle = 0 then
    if not(csDesigning in ComponentState) then
      SubClassMDIClient(Owner as TForm);
end;

procedure TabcMDITiledImage.SubClassMDIClient(AOwner: TForm);
begin
  AOwner.HandleNeeded;
  FClientHandle := AOwner.ClientHandle;
  FClientInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(EraseMDIClientBkgrnd);
  FDefClientProc := Pointer(GetWindowLong(FClientHandle, GWL_WNDPROC));
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FClientInstance));
end;

procedure TabcMDITiledImage.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TabcMDITiledImage.Invalidate;
begin
  inherited Invalidate;
  if FClientHandle <> 0 then
    RedrawWindow(FClientHandle, nil, 0, RDW_ERASE + RDW_INVALIDATE);
end;

procedure TabcMDITiledImage.EraseMDIClientBkgrnd(var Message: TMessage);
var
  SrcRect, DestRect: TRect;
  SrcDC, DestDC: THandle;
  TempBitmap: TBitmap;
begin
  if (Message.Msg = WM_ERASEBKGND) and Visible then begin
    DestDC := Message.wParam;
    GetClipBox(DestDC, DestRect);
    TempBitmap := TBitmap.Create;
    try
      TempBitmap.Width := Width;
      TempBitmap.Height := Height;
      if not(ImageStyle in [sTile, sStretch]) or (Picture = nil) or
          (Picture.Graphic = nil) or (Picture.Graphic.Empty) then
        with TempBitmap do begin
          Canvas.Brush.Style := bsSolid;
          Canvas.Brush.Color := ValidParentForm(Self).Color;
          Canvas.FillRect(Rect(0,0,Width, Height));
        end;
      PaintBitmap(TempBitmap);
      SrcDC := TempBitmap.Canvas.Handle;
      GetClipBox(SrcDC, SrcRect);
      BitBlt(DestDC, DestRect.left, DestRect.top,
        (DestRect.Right - DestRect.Left), (DestRect.Bottom - DestRect.Top),
        SrcDC, DestRect.Left, DestRect.Top, SRCCOPY);
    finally
      TempBitmap.Free;
    end;
    Message.Result := 1;
  end
  else
  if (Message.Msg = WM_WINDOWPOSCHANGING) and Visible then begin
    if not FShowScrollBars then
      ShowScrollBar(FClientHandle, SB_BOTH, False);
    Message.Result := CallWindowProc(FDefClientProc, FClientHandle, Message.Msg, Message.WParam, Message.LParam);
    if FShowScrollBars then
      Invalidate;
  end
  else if ((Message.Msg = WM_HSCROLL) or (Message.Msg = WM_VSCROLL)) and Visible and FShowScrollBars then begin
    Message.Result := CallWindowProc(FDefClientProc, FClientHandle, Message.Msg, Message.WParam, Message.LParam);
    Invalidate;
  end
  else
    Message.Result := CallWindowProc(FDefClientProc, FClientHandle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure TabcMDITiledImage.ImageChanged;
{erase whole client area, not just current clip region}
begin
  if not (csLoading in ComponentState) then
    InvalidateRect(FClientHandle, nil, True);
end;

procedure TabcMDITiledImage.Paint;
begin
  if csDesigning in ComponentState then
    inherited Paint;
end;

procedure TabcMDITiledImage.PaintBitmap(var Bitmap: TBitmap);
var
  Dest: TRect;
  XPos, YPos: Integer;
  PicRatio, ImageRatio: Double;
begin
  if Picture.Graphic <> nil then
  case ImageStyle of
    sClip:
      with Bitmap.Canvas do
        Draw(0,0,Picture.Graphic);
    sCenter:
    begin
      XPos:= (Width - Picture.Width) div 2;
      YPos:= (Height - Picture.Height) div 2;
      with Bitmap.Canvas do
        Draw(Xpos,YPos, Picture.Graphic);
    end;
    sFit:
    if (Picture.Width > 0) and (Picture.Height > 0) then
    begin
      PicRatio:= Picture.Height / Picture.Width;
      ImageRatio:= Height/Width;
      if PicRatio > ImageRatio then
        begin
          XPos:= Trunc(Height / PicRatio);
          YPos:= Height;
        end
      else
        begin
          XPos:= Width;
          YPos:= Trunc(Width * PicRatio);
        end;
      Dest := Rect(0,0,XPos,YPos);
      with Bitmap.Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
    sHeight:
    begin
      XPos:= Trunc(Picture.Width * (Height / Picture.Height));
      YPos:= Height;
      Dest := Rect(0,0,XPos,YPos);
      with Bitmap.Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
    sStretch:
    begin
      Dest := ClientRect;
      with Bitmap.Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
    sTile:
    begin
      XPos:= 0;
      while XPos < Width do
      begin
        YPos:= 0;
        while YPos < Height do
        begin
          with Bitmap.Canvas do
            Draw(XPos, YPos, Picture.Graphic);
          YPos:= YPos + Picture.Height;
        end;
        XPos:= XPos + Picture.Width;
      end;
    end;
    sWidth:
    begin
      XPos:= Width;
      YPos:= Trunc(Width * (Picture.Height / Picture.Width));
      Dest := Rect(0,0,XPos,YPos);
      with Bitmap.Canvas do
        StretchDraw(Dest, Picture.Graphic);
    end;
  end;
end;

function TabcMDITiledImage.GetAlign: TAlign; begin Result := inherited Align; end;
function TabcMDITiledImage.GetCursor: TCursor; begin Result := inherited Cursor; end;
function TabcMDITiledImage.GetHint: string; begin Result := inherited Hint; end;
function TabcMDITiledImage.GetLeft: integer; begin Result := inherited Left; end;
function TabcMDITiledImage.GetTop: integer; begin Result := inherited Top; end;
function TabcMDITiledImage.GetWidth: integer; begin Result := inherited Width; end;
function TabcMDITiledImage.GetHeight: integer; begin Result := inherited Height; end;

procedure TabcMDITiledImage.SetShowScrollBars(Value: Boolean);
var
  I: integer;
  twp: TWindowPos;
  pwp: ^TWindowPos;
  ChildPos: TPoint;
  PChildPos: ^TPoint;
  AMessage: TMessage;
begin
  if Value <> FShowScrollBars then
  begin
    FShowScrollBars := Value;
    if (csDesigning in ComponentState) then Exit;
    if (FClientHandle <> 0) then
      if Value then
      begin
        with TForm(Owner) do
          for I := 0 to MDIChildCount - 1 do
            with MDIChildren[I] do begin
              ChildPos := Point(Left, Top);
              PChildPos := @ChildPos;
              DefMDIChildProc(Handle, WM_MOVE, 0, Longint(PChildPos));
            end;
      end
      else
      begin
        twp.hWnd := TForm(Owner).Handle;
        twp.flags := SWP_DRAWFRAME or SWP_NOACTIVATE or SWP_NOMOVE or
          SWP_NOOWNERZORDER or SWP_NOSIZE or SWP_NOZORDER;
        pwp := @twp;
        AMessage.Msg := WM_WINDOWPOSCHANGING;
        AMessage.wParam := 0;
        AMessage.LParam := Longint(pwp);
        EraseMDIClientBkgrnd(AMessage);
      end;
  end;
end;

{ TClock Label }

constructor TabcClockLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTimer := TTimer.Create(Self);
  FTimer.OnTimer := self.Timer;
  FFormat := 'c';
  inherited Caption := FormatDateTime(FFormat, Now);
  FEnabled := True;
end;

destructor TabcClockLabel.Destroy;
begin
  FTimer.Free;
  inherited destroy;
end;

procedure TabcClockLabel.Timer(Sender: TObject);
begin
  inherited Caption := FormatDateTime(FFormat, Now);
  if Assigned(FOnTimer) then FOnTimer(Self);
end;

function TabcClockLabel.GetInterval: Word; begin Result := FTimer.Interval; end;

procedure TabcClockLabel.SetInterval(Value: Word);
begin
  if Value <> FTimer.Interval then
    FTimer.Interval := Value;
end;

procedure TabcClockLabel.SetEnabled(Value: Boolean);
begin
  FTimer.Enabled := Value;
  FEnabled := Value;
  inherited;
end;

function TabcClockLabel.GetCaption: string; begin Result := inherited Caption; end;

{ TabcPopupSelect}

type
  TabcPopupSelectMenu = class(TPopupMenu)
  private
    fWindow: HWND;
    procedure WndProc(var Message: TMessage);
  public
    fSelected: boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Popup(X, Y: Integer); override;
  end;

procedure TabcPopupSelectMenu.WndProc(var Message: TMessage);
begin
  try
    case Message.Msg of
      WM_COMMAND:
        if DispatchCommand(Message.wParam) then Exit;
    end;
    with Message do Result := DefWindowProc(fWindow, Msg, wParam, lParam);
  except
    Application.HandleException(Self);
  end;
end;

constructor TabcPopupSelectMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fWindow := {$IFDEF DELPHI6}Classes.{$ENDIF}AllocateHWnd(WndProc);
end;

destructor TabcPopupSelectMenu.Destroy;
begin
  {$IFDEF DELPHI6}Classes.{$ENDIF}DeallocateHWnd(fWindow);
  inherited Destroy;
end;

procedure TabcPopupSelectMenu.Popup(X, Y: Integer);
begin
  fSelected := false;
  TrackPopupMenu(Items.Handle, TPM_LEFTALIGN or TPM_LEFTBUTTON, X, Y, 0 { reserved}, fWindow, nil);
end;

{ TabcPopupSelect }

constructor TabcPopupSelect.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fItems := TStringList.Create;
  fMenuItems := TStringList.Create;
  fItemIndex := -1;
  fItemsPerCol := 10;
  fPopupOffsetX := -1;
  fPopupOffsetY := -1;
  fPopupYAdjust := True;
end;

destructor TabcPopupSelect.Destroy;
begin
  fMenu.Free;
  fItems.Free;
  fMenuItems.Free;
  inherited Destroy;
end;

procedure TabcPopupSelect.SetItems(Value: TStringList);
begin
  fItems.Assign(Value);
end;

procedure TabcPopupSelect.SetSorted(Value: Boolean);
begin
  if Value <> fSorted then begin
    Items.Sorted := Value;
    fSorted := Value;
  end;
end;

procedure TabcPopupSelect.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key in [vk_up, vk_down] then
    begin
      menuPopup;
      Key := 0;
    end
  else
    inherited KeyDown(Key, Shift);
end;

procedure TabcPopupSelect.WMLButtonDown(var Message: TWMLButtonDown);
begin
  inherited;
  Perform(WM_LBUTTONUP, 0, 0);
  menuPopup;
end;

{replaces previous version of procedure}
procedure TabcPopupSelect.menuPopup;
var
  NewMenuItem: TMenuItem;
  pt: TPoint;
  match: boolean;
  offset, cnt: integer;
  matchTo: string;
  ColItem: integer;
begin
  fMenu.Free;

  pt.X := PopupOffsetX;
  pt.Y := PopupOffsetY;

  pt := ClientToScreen(pt);
  match := false;
  offset := 0;

  fMenu := TabcPopupSelectMenu.Create(Self);

  matchTo := Text;

  fMenuItems.Clear;
  fMenuItems.AddStrings(fItems);

  if Assigned(fOnPopup) then fOnPopup(self, fMenuItems, matchTo);

  ColItem := 0;
  for cnt := 0 to fMenuItems.Count -1 do begin
    ColItem := ColItem + 1;
    NewMenuItem := TMenuItem.Create(fMenu);
    NewMenuItem.caption := fMenuItems.Strings[cnt];
    NewMenuItem.onclick := menuClick;
    if ColItem > fItemsPerCol then begin
      ColItem := 1;
      if fBarBreak then
        NewMenuItem.Break := mbBarBreak
      else
        NewMenuItem.Break := mbBreak;
      if not match then
        offset := 0;
    end;
    fMenu.Items.Add(NewMenuItem);
    if not match then
      if matchTo = fMenuItems.Strings[cnt] then match := true
      else if fMenuItems.Strings[cnt] = '-' then offset := offset + 8
      else offset := offset + 18;
  end;
  if match and PopupYAdjust then pt.Y := pt.Y - offset;
  if pt.Y < 0 then
    pt.Y := 0;

  (fMenu as TabcPopupSelectMenu).Popup(pt.X, pt.Y);

  application.processMessages;
  if not (fMenu as TabcPopupSelectMenu).fSelected then
    if Assigned(fOnNoSelection) then fOnNoSelection(self);
end;

procedure TabcPopupSelect.menuClick(Sender: TObject);
var
  txt: string;
begin
  (fMenu as TabcPopupSelectMenu).fSelected := True;
  txt := (Sender as TMenuitem).Caption;
  fItemIndex := (Sender as TMenuitem).ComponentIndex - 1;
  if Assigned(fOnSelection) then fOnSelection(self, txt);
  text := txt;
end;

procedure TabcPopupSelect.SetItemIndex(Value: Integer);
begin
  if Value <> fItemIndex then begin
    fItemIndex := Value;
    if Value > -1 then
      Text := fItems.Strings[Value]
    else
      Text := '';
  end;
end;

{ TabcSuperLabel }

constructor TabcSuperLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited AutoSize := false;
  FVAlignment := vaTop;
  if NewStyleControls then
    FTMargin := 2
  else
    FTMargin := 4;
  FLMargin := 0;
  FRMargin := 0;
  FBMargin := 0;
  FBStyle := bsNone;
  FCtl3D := True;
  FBColor := clWindowFrame;
  Width := 100;
  Height := 25;
end;

function TabcSuperLabel.DoDrawTheText(Rect: TRect; Flags: Word; Text: pChar): integer;
begin
  if (Flags and DT_CALCRECT <> 0) and ((Text[0] = #0) or ShowAccelChar and
    (Text[0] = '&') and (Text[1] = #0)) then StrCopy(Text, ' ');
  if not ShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Canvas.Font := Font;
  if not Enabled then Canvas.Font.Color := clGrayText;
  DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
  result := Rect.Bottom - Rect.Top + 1;
end;

procedure TabcSuperLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordBreaks: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  R: TRect;
  R2: TRect;
  hRg: Hrgn;
  tlOffset, brOffset: byte;
  pxtr, pxbl: TColor;
  y2, hgt: integer;
  Text: array[0..255] of Char;
begin
  GetTextBuf(Text, SizeOf(Text));
  with Canvas do
  begin
    R := ClientRect;
    pxtr := pixels[R.right, 0];
    pxbl := pixels[0, R.bottom];
    tlOffset := 0;
    brOffset := 0;

    Brush.Style := bsSolid;

    if not Transparent then
      begin
        Brush.Color := Self.Color;
        FillRect(R);
      end;

    if FBStyle = bsSingle then
      if FCtl3D then
        begin
          Inc(R.Top);
          Inc(R.Left);
          Brush.Color := clBtnHighlight;
          FrameRect(R);
          OffsetRect(R, -1, -1);
          Brush.Color := clBtnShadow;
          FrameRect(R);
          pixels[R.right, 0] := pxtr;
          pixels[0, R.bottom] := pxbl;
          tlOffset := 2;
          brOffset := 1;
        end
      else
        begin
          Brush.Color := FBColor;
          FrameRect(R);
          tlOffset := 1;
          brOffset := 1;
        end
    else if (csDesigning in ComponentState) then
      begin
        Brush.Color := clGray;
        FrameRect(R);
        tlOffset := 0;
        brOffset := 0;
      end;

    Brush.Style := bsClear;

    Inc(R.top, FTMargin + tlOffset);
    Inc(R.left, FLMargin + tlOffset);
    Dec(R.right, FRMargin + brOffset);
    Dec(R.bottom, FBMargin + brOffset);
    if R.right <= R.left then R.right := R.left;
    if R.bottom <= R.top then R.bottom := R.top;

    R2 := R;
    OffsetRect(R2, self.left, self.top);
    hRg := CreateRectRgnIndirect(R2);
    SelectClipRgn(Canvas.Handle, hRg);
    DeleteObject(hRg);

    if FVAlignment <> vaTop then
      begin
        y2 := R.bottom;
        hgt := DoDrawTheText(R, DT_EXPANDTABS or WordBreaks[WordWrap] or DT_CALCRECT, Text);
        R.bottom := R.top + hgt - 1;
        if FVAlignment = vaCenter then
            OffsetRect(R, 0, (y2 - R.bottom) div 2)
        else
            OffsetRect(R, 0, y2 - R.bottom);
      end;
    DoDrawTheText(R, DT_EXPANDTABS or WordBreaks[WordWrap] or Alignments[Alignment], Text);
  end;
end;

function TabcSuperLabel.getInhAutoSize: boolean;
begin
  result := inherited AutoSize;
end;

procedure TabcSuperLabel.setVAlignment(setTo: TabcVAlignment);
begin
  if FVAlignment <> setTo then
  begin
    FVAlignment := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setTMargin(setTo: byte);
begin
  if FTMargin <> setTo then
  begin
    FTMargin := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setLMargin(setTo: byte);
begin
  if FLMargin <> setTo then
  begin
    FLMargin := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setRMargin(setTo: byte);
begin
  if FRMargin <> setTo then
  begin
    FRMargin := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setBMargin(setTo: byte);
begin
  if FBMargin <> setTo then
  begin
    FBMargin := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setCtl3D(setTo: boolean);
begin
  if FCtl3D <> setTo then
  begin
    FCtl3D := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setBStyle(setTo: TBorderStyle);
begin
  if FBStyle <> setTo then
  begin
    FBStyle := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setBColor(setTo: TColor);
begin
  if FBColor <> setTo then
  begin
    FBColor := setTo;
    Invalidate;
  end;
end;

procedure TabcSuperLabel.setWordWrap(setTo: boolean);
begin
  if inherited WordWrap <> setTo then
  begin
    inherited WordWrap := setTo;
    Invalidate;
  end;
end;

function TabcSuperLabel.getWordWrap: boolean;
begin
  result := inherited WordWrap;
end;

{ TabcAutoLabel }

constructor TabcAutoLabel.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FTextStyle := tsNormal;
  FBorderStyle := bsNone;
  FBorderColor := clWindowFrame;
  FCtl3D := True;
  FSeparator := ':';
  Alignment := taRightJustify;
  Width := 100;
  AutoSize := True;
end;

procedure TabcAutoLabel.Loaded;
var
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
begin
  inherited Loaded;
  if AutoSize then
    AdjustBounds
  else
    if NewStyleControls then
      {Windows 95 and later}
      FYOffset := 2
    else
    begin
      {Win 3.1 - this works on point sizes 8 and above}
      DC := GetDC(0);
      SaveFont := SelectObject(DC, Font.Handle);
      GetTextMetrics(DC, Metrics);
      SelectObject(DC, SaveFont);
      ReleaseDC(0, DC);
      FYOffset := (Height - Metrics.tmHeight) div 2;
    end;
end;

procedure TabcAutoLabel.SetAutoSize(Value: Boolean);
begin
  if Value <> AutoSize then begin
    inherited SetAutoSize(Value);
    AdjustBounds;
  end;
end;

procedure TabcAutoLabel.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
  AdjustBounds;
end;

procedure TabcAutoLabel.CMFontChanged(var Message: TMessage);
begin
  inherited;
  AdjustBounds;
end;

procedure TabcAutoLabel.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then
  begin
    FWordWrap := Value;
    AdjustBounds;
  end;
end;

procedure TabcAutoLabel.SetTextStyle( Value : TabcTextStyle );
begin
  if Value <> FTextStyle then begin
    FTextStyle := Value;
    Invalidate;
  end;
end;

procedure TabcAutoLabel.SetSeparator(Value: Char);
begin
  if Value <> FSeparator then begin
    FSeparator := Value;
    Invalidate;
  end;
end;

procedure TabcAutoLabel.SetBorderStyle( Value : TBorderStyle );
begin
  if Value <> FBorderStyle then begin
    FBorderStyle := Value;
    Invalidate;
  end;
end;

procedure TabcAutoLabel.SetCtl3D(Value: boolean);
begin
  if FCtl3D <> Value then
  begin
    FCtl3D := Value;
    Invalidate;
  end;
end;

procedure TabcAutoLabel.SetBorderColor(Value: TColor);
begin
  if FBorderColor <> Value then
  begin
    FBorderColor := Value;
    Invalidate;
  end;
end;

procedure TabcAutoLabel.AdjustHeight;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  DC := GetDC(0);
  GetTextMetrics(DC, SysMetrics);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  I := SysMetrics.tmHeight;
  if I > Metrics.tmHeight then I := Metrics.tmHeight;
  Height := Metrics.tmHeight + I div 4 + GetSystemMetrics(SM_CYBORDER) * 4;
  if NewStyleControls then
    {Windows 95 and later}
    FYOffset := 2
  else
    {Win 3.1 - this works on point sizes 8 and above}
    FYOffset := (Height - Metrics.tmHeight) div 2;
end;

procedure TabcAutoLabel.AdjustBounds;
const
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  DC: HDC;
  NewWidth: Integer;
  Rect: TRect;
begin
  if not (csReading in ComponentState) and AutoSize then
  begin
    AdjustHeight;
    Rect := ClientRect;
    DC := GetDC(0);
    Canvas.Handle := DC;
    DoDrawText(Rect, (DT_EXPANDTABS or DT_CALCRECT) or WordWraps[FWordWrap]);
    Canvas.Handle := 0;
    ReleaseDC(0, DC);
    InflateRect(Rect, 5, 5);
    case Alignment of
      taLeftJustify:
        SetBounds(Left, Top, Rect.Right, Rect.Bottom);
      taCenter:
        begin
          NewWidth := (Rect.Right + 4);
          SetBounds(Left - (NewWidth - Width) div 2, Top, NewWidth, Rect.Bottom);
        end;
      taRightJustify:
        begin
          NewWidth := (Rect.Right + 8);
          SetBounds(Left - (NewWidth - Width), Top, NewWidth, Rect.Bottom);
        end;
    end;
  end;
end;

procedure TabcAutoLabel.DoDrawText( var Rect : TRect; Flags : Longint );
var
  Text: array[0..256] of Char;
  SepChar: array[0..1] of char;
  TmpRect: TRect;
  TopColor, BottomColor: TColor;
begin
  GetTextBuf(Text, SizeOf(Text));
  if (Flags and DT_CALCRECT <> 0) and ((Text[0] = #0) or ShowAccelChar and
    (Text[0] = '&') and (Text[1] = #0)) then StrCopy(Text, ' ');
  if not ShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Canvas.Font := Font;
  TopColor := clBtnHighlight;
  BottomColor := clBtnShadow;
  if FTextStyle = tsLowered then begin
    TopColor := clBtnShadow;
    BottomColor := clBtnHighlight;
  end;

  SepChar[0] := FSeparator;
  SepChar[1] := #0;
  StrCat(Text, SepChar);

  OffsetRect(Rect, 0, FYOffset);
  if Alignment = taLeftJustify then
    OffsetRect(Rect, 4, 0)
  else if Alignment = taRightJustify then
    OffsetRect(Rect, -4, 0);

  if FTextStyle in [ tsLowered, tsRaised ] then begin
    TmpRect := Rect;
    OffsetRect( TmpRect, 1, 1 );
    Canvas.Font.Color := BottomColor;
    DrawText(Canvas.Handle, Text, StrLen(Text), TmpRect, Flags);
    TmpRect := Rect;
    OffsetRect( TmpRect, -1, -1 );
    Canvas.Font.Color := TopColor;
    DrawText(Canvas.Handle, Text, StrLen(Text), TmpRect, Flags);
  end;

  Canvas.Font.Color := Font.Color;
  if not Enabled then Canvas.Font.Color := clGrayText;
  DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
end;

procedure TabcAutoLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  Rect: TRect;
  pxtr, pxbl: TColor;
  tlOffset, brOffset: Byte;
begin
  Rect := ClientRect;
  tlOffset := 0;
  brOffset := 0;
  with Canvas do begin
    pxtr := pixels[Rect.right, 0];
    pxbl := pixels[0, Rect.bottom];
    if not Transparent then begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(Rect);
    end;

    Brush.Style := bsClear;

    if FBorderStyle = bsSingle then
      if FCtl3D then
        begin
          Inc(Rect.Top);
          Inc(Rect.Left);
          Brush.Color := clBtnHighlight;
          FrameRect(Rect);
          OffsetRect(Rect, -1, -1);
          Brush.Color := clBtnShadow;
          FrameRect(Rect);
          pixels[Rect.right, 0] := pxtr;
          pixels[0, Rect.bottom] := pxbl;
          tlOffset := 2;
          brOffset := 1;
        end
      else
        begin
          Brush.Color := FBorderColor;
          FrameRect(Rect);
          tlOffset := 1;
          brOffset := 1;
        end
    else if (csDesigning in ComponentState) then
      begin
        Brush.Color := clGray;
        FrameRect(Rect);
        tlOffset := 0;
        brOffset := 0;
      end;

    Brush.Style := bsClear;
    tlOffset := tlOffset + 1;
    InflateRect(Rect, -tlOffset, -tlOffset);
    if Rect.right <= Rect.left then Rect.right := Rect.left;
    if Rect.bottom <= Rect.top then Rect.bottom := Rect.top;
    case Alignment of
      taLeftJustify:
        OffsetRect(Rect, -tlOffset, -tlOffset);
      taCenter:
        OffsetRect(Rect, (tlOffset-brOffset), -tlOffset);
      taRightJustify:
        OffsetRect(Rect, ( 2 * tlOffset - brOffset), -tlOffset);
    end;
    DoDrawText(Rect, DT_EXPANDTABS or WordWraps[WordWrap] or
      Alignments[Alignment]);
  end;
end;

{ TabcElSubEdit for TabcEditLabel }

type
  TabcElSubEdit = class(TEdit)
  private
    procedure WMSize(var Message: TMessage); message WM_Size;
  protected
    procedure Change; override;
    procedure Click; override;
    procedure DblClick; override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure DoStartDrag(var DragObject: TDragObject); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    constructor Create(AOwner: TComponent); override;
  end;

constructor TabcElSubEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  Text := '';
end;

procedure TabcElSubEdit.WMSize(var Message: TMessage);
begin
  inherited;
  sendMessage(parent.handle, WM_Size, 0, parent.width + (longint(parent.height) shl 16));
end;

procedure TabcElSubEdit.Change;
begin
  if Assigned(OnChange) then OnChange(Owner);
end;

procedure TabcElSubEdit.Click;
begin
  if Assigned(OnClick) then OnClick(Owner);
end;

procedure TabcElSubEdit.DblClick;
begin
  if Assigned(OnDblClick) then OnDblClick(Owner);
end;

procedure TabcElSubEdit.DoEndDrag(Target: TObject; X, Y: Integer);
begin
  if Assigned(OnEndDrag) then OnEndDrag(Owner, Target, X, Y);
end;

procedure TabcElSubEdit.DoEnter;
begin
  if Assigned(OnEnter) then OnEnter(Owner);
end;

procedure TabcElSubEdit.DoExit;
begin
  if Assigned(OnExit) then OnExit(Owner);
end;

procedure TabcElSubEdit.DoStartDrag(var DragObject: TDragObject);
begin
  if Assigned(OnStartDrag) then OnStartDrag(Owner, DragObject);
end;

procedure TabcElSubEdit.DragDrop(Source: TObject; X, Y: Integer);
begin
  if Assigned(OnDragDrop) then OnDragDrop(Owner, Source, X, Y);
end;

procedure TabcElSubEdit.DragOver(Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := True;
  if Assigned(OnDragOver) then
    OnDragOver(Owner, Source, X, Y, State, Accept) else
    Accept := False;
end;

procedure TabcElSubEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Assigned(OnKeyDown) then OnKeyDown(Owner, Key, Shift);
end;

procedure TabcElSubEdit.KeyPress(var Key: Char);
begin
  if Assigned(OnKeyPress) then OnKeyPress(Owner, Key);
end;

procedure TabcElSubEdit.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if Assigned(OnKeyUp) then OnKeyUp(Owner, Key, Shift);
end;

procedure TabcElSubEdit.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseDown) then OnMouseDown(Owner, Button, Shift, X, Y);
end;

procedure TabcElSubEdit.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseMove) then OnMouseMove(Owner, Shift, X, Y);
end;

procedure TabcElSubEdit.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseUp) then OnMouseUp(Owner, Button, Shift, X, Y);
end;

{ TabcEditLabel }

constructor TabcEditLabel.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  fEdit := TabcElSubEdit.Create(Self);
  fEdit.parent := self;
  fEdit.autosize := true;
  fEdit.onChange := textChanged;
  fEdit.tabStop := true;
  fLabel := TLabel.Create(Self);
  fLabel.parent := self;
  fLabel.autosize := false;
  fLabel.focusControl := fEdit;
  fLabel.transparent := true;
  fLabel.wordWrap := true;
  fLabel.onClick := labelClicked;
  fLabel.showhint := false;
  FEdit.Height := 25;
  Caption := 'Caption';
  LabelHeight := 28;
  Width := 223;  {sizes edit box part to default TEdit width}
  fLayout := elLeft;
  fLabelWidth := 100;
  initLayout;
end;

destructor TabcEditLabel.Destroy;
begin
  fLabel.Free;
  fEdit.Free;
  inherited Destroy;
end;

procedure TabcEditLabel.WMSize(var Message: TMessage);
begin
  inherited;
  if not FCalcLayout then begin
    FSizing := True;
    initLayout;
  end;
end;

procedure TabcEditLabel.WMNCHitTest(var Message: TMessage);
var
  WindowRect: TRect;
  WinMidX: word;
begin
  if (csDesigning in ComponentState) then begin
    {get window position to find out if mouse is near left or right end}
    GetWindowRect(Handle, WindowRect);
    {compare WindowRect.Left, WindowRect.Right, Message.LParamLO}
    WinMidX := WindowRect.Left + (WindowRect.Right - WindowRect.Left) div 2;
    if Message.LParamLO < WinMidX then
      FLabelFixed := False
    else
      FLabelFixed := True;
  end;
  inherited;
end;

procedure TabcEditLabel.paintWindow(DC: HDC);
begin
  inherited paintWindow(DC);
  if csDesigning in ComponentState then
    drawFocusRect(DC, CLientRect);
end;

procedure TabcEditLabel.CMTextChanged(var Message: TMessage);
begin
  inherited;
  if fEdit.Text <> inherited Text then fEdit.Text := inherited Text;
end;

procedure TabcEditLabel.DoEnter;
begin
  fEdit.SetFocus;
end;

procedure TabcEditLabel.SetFocus;
begin
  fEdit.SetFocus;
end;

function TabcEditLabel.Focused;
begin
  Result := fEdit.Focused;
end;

function TabcEditLabel.isLabelFontStored: boolean;
begin
  result := not LabelParentFont;
end;

function TabcEditLabel.isEditFontStored: boolean;
begin
  result := not EditParentFont;
end;

function TabcEditLabel.isCtl3DStored: boolean;
begin
  result := not ParentCtl3D;
end;

function TabcEditLabel.isEditColorStored: boolean;
begin
  result := not EditParentColor;
end;

function TabcEditLabel.isShowHintStored: boolean;
begin
  result := not ParentShowHint;
end;

procedure TabcEditLabel.textChanged(Sender: TObject);
begin
  text := fEdit.text;
  if not(csLoading in ComponentState) then
    if Assigned(fOnChange) then fOnChange(Sender);
end;

procedure TabcEditLabel.labelClicked(Sender: TObject);
begin
  fEdit.setfocus;
  fEdit.selectAll;
end;

procedure TabcEditLabel.Loaded;
begin
  inherited Loaded;
  initLayout;
end;

procedure TabcEditLabel.initLayout;
var
  AdjustYOffset: Byte;
begin
  if (csLoading in ComponentState) then Exit;
  if FCalcLayout then
    Exit
  else
    FCalcLayout := True;
  if fLayout = elTop then begin
    if FSizing then begin
      LabelHeight := Height - FLabelOffsetY - fEdit.Height - 2;
      if LabelOffsetX > 0 then
        LabelWidth := Width - 2 - LabelOffsetX
      else
        LabelWidth := Width - 2;
    end
    else begin
      Height := fLabel.Height + FLabelOffsetY + fEdit.Height + 2;
      if FLabelOffsetX > 0 then begin
        if Width <> fLabel.Width + FLabelOffsetX + 2 then Width := fLabel.Width + FLabelOffsetX + 2;
      end
      else
        if Width <> fLabel.Width - FLabelOffsetX + 2 then Width := fLabel.Width - FLabelOffsetX + 2;
    end;

    if fEdit.top <> (Height - fEdit.height - 1) then fEdit.top := Height - fEdit.height - 1;

    if FLabelOffsetX > 0 then begin
      if fEdit.left <> FLabelOffsetX + 1 then fEdit.left := FLabelOffsetX + 1;
      if fLabel.left <> 1 then fLabel.left := 1;
      if fEdit.width <> (Width - FLabelOffsetX - 2) then fEdit.width := Width - FLabelOffsetX - 2;
    end
    else begin
      if fLabel.left <> 1 - FLabelOffsetX then fLabel.left := 1 - FLabelOffsetX;
      if fEdit.left <> 1 then fEdit.left := 1;
      if fEdit.width <> (Width - 2) then fEdit.width := Width - 2;
    end;
  end
  else begin
    if FSizing then
      LabelHeight := Height
    else
      Height := fLabel.Height;
    if FLabelFixed then begin
      if fLabel.width <> fLabelWidth then fLabel.width := fLabelWidth;
      if fEdit.width <> (self.width - fLabelWidth - FLabelOffsetX - 2) then
        fEdit.width := (self.width - fLabelWidth - FLabelOffsetX - 2);
    end
    else begin
      if fLabel.width <> (self.width - fEdit.width - FLabelOffsetX - 2) then
        fLabel.width := (self.width - fEdit.width - FLabelOffsetX - 2);
      fLabelWidth := fLabel.width;
    end;

    if fLabel.left <> 1 then fLabel.left := 1;

    if NewStyleControls then
      AdjustYOffset := 3
    else
      AdjustYOffset := 5;

    if BorderStyle = bsNone then
      AdjustYOffset := AdjustYOffset - 2;

    if FLabelOffsetY < 0 then begin
      if fLabel.top <> (AdjustYOffset - FLabelOffsetY) then fLabel.top := AdjustYOffset - FLabelOffsetY;
    end
    else
      if fLabel.top <> AdjustYOffset then fLabel.top := AdjustYOffset;

    if FLabelOffsetY > 0 then begin
      if fEdit.top <> (FLabelOffsetY + 1) then fEdit.top := FLabelOffsetY + 1;
    end
    else
      if fEdit.top <> 1 then fEdit.top := 1;

    if fEdit.left <> (fLabelWidth + FLabelOffsetX) then fEdit.left := (fLabelWidth + FLabelOffsetX);
  end;

  FCalcLayout := False;
  FSizing := False;
end;

procedure TabcEditLabel.ChangeScale(M, D: Integer);
begin
  inherited ChangeScale(M,D);
  fEdit.Autosize := False;
  fEdit.Autosize := True;
  InitLayout;
end;

procedure TabcEditLabel.setLayout(setTo: TabcEditLabelLayout);
begin
  if (fLayout = elLeft) and (setTo = elTop)
    and not (csLoading in ComponentState) then
    fSaveLabelWidth := LabelWidth;
  fLayout := setTo;
  if (csLoading in ComponentState) then Exit;
  if fLayout = elTop then begin
    if FLabelOffsetX > 0 then begin
      Width := fEdit.Width + FLabelOffsetX + 2;
      fLabel.left := 1;
    end
    else begin
      Width := fEdit.Width + 2;
      fLabel.left := 1 - FLabelOffsetX;
    end;
    if not (csLoading in ComponentState) then
      LabelHeight := Height;
  end
  else begin
    if fSaveLabelWidth <> 0 then begin
      fLabelWidth := fSaveLabelWidth;
      fSaveLabelWidth := 0;
    end;
    FLabelFixed := True;
    Width := fLabelWidth + FLabelOffsetX + fEdit.Width  + 2;
    Height := Height - FEdit.Height - FLabelOffsetY - 2;
  end;
  initLayout;
end;

procedure TabcEditLabel.SetLabelOffsetX(Value: Integer);
begin
  if FLabelOffsetX <> Value then
    begin
      FLabelOffsetX := Value;
      initLayout;
    end;
end;
procedure TabcEditLabel.SetLabelOffsetY(Value: Integer);
begin
  if FLabelOffsetY <> Value then
    begin
      FLabelOffsetY := Value;
      initLayout;
    end;
end;

procedure TabcEditLabel.setLabelWidth(setTo: integer);
begin
  if fLabelWidth <> setTo then
    begin
      fLabelWidth := setTo;
      FLabelFixed := True;
      fLabel.Width := fLabelWidth;
      initLayout;
    end;
end;

procedure TabcEditLabel.setLabelHeight(setTo: integer);
begin
  if fLabelHeight <> setTo then
    begin
      fLabelHeight := setTo;
      FLabelFixed := True;
      fLabel.Height := fLabelHeight;
      initLayout;
    end;
end;

procedure TabcEditLabel.Clear;
begin
  fEdit.Clear;
end;

procedure TabcEditLabel.ClearSelection;
begin
  fEdit.ClearSelection;
end;

procedure TabcEditLabel.CopyToClipboard;
begin
  fEdit.CopyToClipboard;
end;

procedure TabcEditLabel.CutToClipboard;
begin
  fEdit.CutToClipboard;
end;

procedure TabcEditLabel.PasteFromClipboard;
begin
  fEdit.PasteFromClipboard;
end;

function  TabcEditLabel.GetSelTextBuf(Buffer: PChar; BufSize: Integer): Integer;
begin
  result := fEdit.GetSelTextBuf(Buffer, BufSize);
end;

procedure TabcEditLabel.SelectAll;
begin
  fEdit.SelectAll;
end;

procedure TabcEditLabel.SetSelTextBuf(Buffer: PChar);
begin
  fEdit.SetSelTextBuf(Buffer);
end;

function TabcEditLabel.getModified: Boolean;
begin
  result := fEdit.Modified;
end;

procedure TabcEditLabel.setModified(setTo: Boolean);
begin
  if fEdit.Modified <> setTo then fEdit.Modified := setTo;
end;

function TabcEditLabel.getSelLength: Integer;
begin
  result := fEdit.SelLength;
end;

procedure TabcEditLabel.setSelLength(setTo: Integer);
begin
  if fEdit.SelLength <> setTo then fEdit.SelLength := setTo;
end;

function TabcEditLabel.getSelStart: Integer;
begin
  result := fEdit.SelStart;
end;

procedure TabcEditLabel.setSelStart(setTo: Integer);
begin
  if fEdit.SelStart <> setTo then fEdit.SelStart := setTo;
end;

function TabcEditLabel.getSelText: string;
begin
  result := fEdit.SelText;
end;

procedure TabcEditLabel.setSelText(setTo: string);
begin
  if fEdit.SelText <> setTo then fEdit.SelText := setTo;
end;

function TabcEditLabel.getOnChange: TNotifyEvent;
begin
  result := fOnChange;
end;

procedure TabcEditLabel.setOnChange(setTo: TNotifyEvent);
begin
  fOnChange := setTo;
end;

function TabcEditLabel.getOnClick: TNotifyEvent;
begin
  result := fEdit.OnClick;
end;

procedure TabcEditLabel.setOnClick(setTo: TNotifyEvent);
begin
  fEdit.OnClick := setTo;
end;

function TabcEditLabel.getOnDblClick: TNotifyEvent;
begin
  result := fEdit.OnDblClick;
end;

procedure TabcEditLabel.setOnDblClick(setTo: TNotifyEvent);
begin
  fEdit.OnDblClick := setTo;
end;

function TabcEditLabel.getOnEnter: TNotifyEvent;
begin
  result := fEdit.OnEnter;
end;

procedure TabcEditLabel.setOnEnter(setTo: TNotifyEvent);
begin
  fEdit.OnEnter := setTo;
end;

function TabcEditLabel.getOnExit: TNotifyEvent;
begin
  result := fEdit.OnExit;
end;

procedure TabcEditLabel.setOnExit(setTo: TNotifyEvent);
begin
  fEdit.OnExit := setTo;
end;

function TabcEditLabel.getOnKeyDown: TKeyEvent;
begin
  result := fEdit.OnKeyDown;
end;

procedure TabcEditLabel.setOnKeyDown(setTo: TKeyEvent);
begin
  fEdit.OnKeyDown := setTo;
end;

function TabcEditLabel.getOnKeyPress: TKeyPressEvent;
begin
  result := fEdit.OnKeyPress;
end;

procedure TabcEditLabel.setOnKeyPress(setTo: TKeyPressEvent);
begin
  fEdit.OnKeyPress := setTo;
end;

function TabcEditLabel.getOnKeyUp: TKeyEvent;
begin
  result := fEdit.OnKeyUp;
end;

procedure TabcEditLabel.setOnKeyUp(setTo: TKeyEvent);
begin
  fEdit.OnKeyUp := setTo;
end;

function TabcEditLabel.getOnMouseDown: TMouseEvent;
begin
  result := fEdit.OnMouseDown;
end;

procedure TabcEditLabel.setOnMouseDown(setTo: TMouseEvent);
begin
  fEdit.OnMouseDown := setTo;
end;

function TabcEditLabel.getOnMouseMove: TMouseMoveEvent;
begin
  result := fEdit.OnMouseMove;
end;

procedure TabcEditLabel.setOnMouseMove(setTo: TMouseMoveEvent);
begin
  fEdit.OnMouseMove := setTo;
end;

function TabcEditLabel.getOnMouseUp: TMouseEvent;
begin
  result := fEdit.OnMouseUp;
end;

procedure TabcEditLabel.setOnMouseUp(setTo: TMouseEvent);
begin
  fEdit.OnMouseUp := setTo;
end;

function TabcEditLabel.GetOnDragDrop: TDragDropEvent; begin Result := fEdit.OnDragDrop; end;

procedure TabcEditLabel.SetOnDragDrop(Value: TDragDropEvent);
begin
  fEdit.OnDragDrop := Value;
end;

function TabcEditLabel.GetOnDragOver: TDragOverEvent; begin Result := fEdit.OnDragOver; end;

procedure TabcEditLabel.SetOnDragOver(Value: TDragOverEvent);
begin
  fEdit.OnDragOver := Value;
end;

function TabcEditLabel.GetOnEndDrag: TEndDragEvent; begin Result := fEdit.OnEndDrag; end;

procedure TabcEditLabel.SetOnEndDrag(Value: TEndDragEvent);
begin
  fEdit.OnEndDrag := Value;
end;

function TabcEditLabel.GetOnStartDrag: TStartDragEvent; begin Result := fEdit.OnStartDrag; end;
procedure TabcEditLabel.SetOnStartDrag(Value: TStartDragEvent);
begin
  fEdit.OnStartDrag := Value;
end;

function TabcEditLabel.getAutoSelect: boolean;
begin
  result := fEdit.AutoSelect;
end;

procedure TabcEditLabel.setAutoSelect(setTo: boolean);
begin
  if fEdit.AutoSelect <> setTo then fEdit.AutoSelect := setTo;
end;

function TabcEditLabel.getBorderStyle: TBorderStyle;
begin
  result := fEdit.BorderStyle;
end;

procedure TabcEditLabel.setBorderStyle(setTo: TBorderStyle);
begin
  if fEdit.BorderStyle <> setTo then fEdit.BorderStyle := setTo;
end;

function TabcEditLabel.getCharCase: TEditCharCase;
begin
  result := fEdit.CharCase;
end;

procedure TabcEditLabel.setCharCase(setTo: TEditCharCase);
begin
  if fEdit.CharCase <> setTo then fEdit.CharCase := setTo;
end;

function TabcEditLabel.getEditColor: TColor;
begin
  result := fEdit.Color;
end;

procedure TabcEditLabel.setEditColor(setTo: TColor);
begin
  if fEdit.Color <> setTo then fEdit.Color := setTo;
end;

function TabcEditLabel.getEditParentColor: boolean;
begin
  result := fEdit.ParentColor;
end;

procedure TabcEditLabel.setEditParentColor(setTo: boolean);
begin
  if fEdit.ParentColor <> setTo then fEdit.ParentColor := setTo;
end;

function TabcEditLabel.getCtl3D: boolean;
begin
  result := fEdit.Ctl3D;
end;

procedure TabcEditLabel.setCtl3D(setTo: boolean);
begin
  if fEdit.Ctl3D <> setTo then fEdit.Ctl3D := setTo;
end;

function TabcEditLabel.getParentCtl3D: boolean;
begin
  result := fEdit.ParentCtl3D;
end;

procedure TabcEditLabel.setParentCtl3D(setTo: boolean);
begin
  if fEdit.ParentCtl3D <> setTo then fEdit.ParentCtl3D := setTo;
end;

function TabcEditLabel.getHideSelection: boolean;
begin
  result := fEdit.HideSelection;
end;

procedure TabcEditLabel.setHideSelection(setTo: boolean);
begin
  if fEdit.HideSelection <> setTo then fEdit.HideSelection := setTo;
end;

function TabcEditLabel.getMaxLength: integer;
begin
  result := fEdit.MaxLength;
end;

procedure TabcEditLabel.setMaxLength(setTo: integer);
begin
  if fEdit.MaxLength <> setTo then fEdit.MaxLength := setTo;
end;

function TabcEditLabel.getOEMConvert: boolean;
begin
  result := fEdit.OEMConvert;
end;

procedure TabcEditLabel.setOEMConvert(setTo: boolean);
begin
  if fEdit.OEMConvert <> setTo then fEdit.OEMConvert := setTo;
end;

function TabcEditLabel.getPasswordChar: char;
begin
  result := fEdit.PasswordChar;
end;

procedure TabcEditLabel.setPasswordChar(setTo: char);
begin
  if fEdit.PasswordChar <> setTo then fEdit.PasswordChar := setTo;
end;

function TabcEditLabel.getReadOnly: boolean;
begin
  result := fEdit.ReadOnly;
end;

procedure TabcEditLabel.setReadOnly(setTo: boolean);
begin
  if fEdit.ReadOnly <> setTo then fEdit.ReadOnly := setTo;
end;

procedure TabcEditLabel.CMEnabledChanged(var Message: TMessage);
begin
  fEdit.Enabled := Enabled;
  fLabel.Enabled := Enabled;
  inherited;
end;

function TabcEditLabel.getTabStop: boolean;
begin
  result := fEdit.TabStop;
end;

procedure TabcEditLabel.setTabStop(setTo: boolean);
begin
  if TabStop <> setTo then fEdit.TabStop := setTo;
end;

function TabcEditLabel.getAlignment: TAlignment;
begin
  result := fLabel.Alignment;
end;

procedure TabcEditLabel.setAlignment(setTo: TAlignment);
begin
  if fLabel.Alignment <> setTo then fLabel.Alignment := setTo;
end;

function TabcEditLabel.getCaption: TCaption;
begin
  result := fLabel.Caption;
end;

procedure TabcEditLabel.setCaption(setTo: TCaption);
begin
  if fLabel.Caption <> setTo then
    fLabel.Caption := setTo;
end;

function TabcEditLabel.getShowAccelChar: boolean;
begin
  result := fLabel.ShowAccelChar;
end;

procedure TabcEditLabel.setShowAccelChar(setTo: boolean);
begin
  if fLabel.ShowAccelChar <> setTo then fLabel.ShowAccelChar := setTo;
end;

function TabcEditLabel.getLabelFont: TFont;
begin
  result := fLabel.Font;
end;

procedure TabcEditLabel.setLabelFont(setTo: TFont);
begin
  if fLabel.Font <> setTo then begin
    fLabel.Font := setTo;
    if fLayout = elLeft then begin
      FLabelOffsetY := -(fLabel.Font.Height - fEdit.Font.Height);
      initLayout;
    end;
  end;

end;

function TabcEditLabel.getEditFont: TFont;
begin
  result := fEdit.Font;
end;

procedure TabcEditLabel.setEditFont(setTo: TFont);
begin
  if fEdit.Font <> setTo then begin
    fEdit.Font := setTo;
    if fLayout = elLeft then begin
      FLabelOffsetY := -(fLabel.Font.Height - fEdit.Font.Height);
      initLayout;
    end;
  end;
end;

function TabcEditLabel.getLabelParentFont: boolean;
begin
  result := fLabel.ParentFont;
end;

procedure TabcEditLabel.setLabelParentFont(setTo: boolean);
begin
  if fLabel.ParentFont <> setTo then fLabel.ParentFont := setTo;
end;

function TabcEditLabel.getEditParentFont: boolean;
begin
  result := fEdit.ParentFont;
end;

procedure TabcEditLabel.setEditParentFont(setTo: boolean);
begin
  if fEdit.ParentFont <> setTo then fEdit.ParentFont := setTo;
end;

function TabcEditLabel.getHelpContext: THelpContext;
begin
  result := fEdit.HelpContext;
end;

procedure TabcEditLabel.setHelpContext(setTo: THelpContext);
begin
  if fEdit.HelpContext <> setTo then fEdit.HelpContext := setTo;
end;

function TabcEditLabel.GetCursor: TCursor;
begin
  result := fEdit.Cursor;
end;

procedure TabcEditLabel.SetCursor(Value: TCursor);
begin
  if Value <> fEdit.Cursor then
    fEdit.Cursor := Value;
end;

function TabcEditLabel.GetDragCursor: TCursor;
begin
  result := fEdit.DragCursor;
end;

procedure TabcEditLabel.SetDragCursor(Value: TCursor);
begin
  if Value <> fEdit.DragCursor then
    fEdit.DragCursor := Value;
end;

procedure TabcEditLabel.SetDragMode(Value: TDragMode);
begin
  inherited SetDragMode(Value);
  if Value <> fEdit.DragMode then fEdit.DragMode := Value;
end;

function TabcEditLabel.GetHint: string;
begin
  Result := fEdit.Hint;
end;

procedure TabcEditLabel.SetHint(Value: string);
begin
  if Value <> fEdit.Hint then
    fEdit.Hint := Value;
end;

function TabcEditLabel.GetParentShowHint: Boolean;
begin
  Result := fEdit.ParentShowHint;
end;

procedure TabcEditLabel.SetParentShowHint(Value: Boolean);
begin
  if Value <> fEdit.ParentShowHint then
    fEdit.ParentShowHint := Value;
end;

function TabcEditLabel.GetShowHint: Boolean;
begin
  Result := fEdit.ShowHint;
end;

procedure TabcEditLabel.SetShowHint(Value: Boolean);
begin
  if Value <> fEdit.ShowHint then begin
    ParentShowHint := False;
    fEdit.ShowHint := Value;
  end;
end;

function TabcEditLabel.GetPopupMenu: TPopupMenu;
begin
  Result := fEdit.PopupMenu;
end;

function TabcEditLabel.GetEditPopupMenu: TPopupMenu;
begin
  Result := fEdit.PopupMenu;
end;

procedure TabcEditLabel.SetPopupMenu(Value: TPopupMenu);
begin
  inherited PopupMenu := Value;
  fEdit.PopupMenu := Value;
end;

constructor TabcMDIScroll.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (AOwner is TForm) or not (TForm(AOwner).FormStyle = fsMDIForm) then
    raise EComponentError.Create(sabcErrorOwnerMDIFormExpected);
end;

destructor TabcMDIScroll.Destroy;
begin
  Destroying;
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FDefClientProc));
  {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FClientInstance);
  inherited Destroy;
end;

procedure TabcMDIScroll.Loaded;
begin
  inherited Loaded;
  if FClientHandle = 0 then
    if not(csDesigning in ComponentState) then
      SubClassMDIClient(Owner as TForm);
end;

procedure TabcMDIScroll.SubClassMDIClient(AOwner: TForm);
begin
  AOwner.HandleNeeded;
  FClientHandle := AOwner.ClientHandle;
  FClientInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(HideMDIScrollBars);
  FDefClientProc := Pointer(GetWindowLong(FClientHandle, GWL_WNDPROC));
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FClientInstance));
end;

procedure TabcMDIScroll.HideMDIScrollBars(var Message: TMessage);
begin
  if (Message.Msg = WM_WINDOWPOSCHANGING) and not FEnabled then
  begin
    {turn off scroll bars}
    ShowScrollBar(FClientHandle, SB_BOTH, False);
    Message.Result := CallWindowProc(FDefClientProc, FClientHandle,
      Message.Msg, Message.WParam, Message.LParam);
  end
  else
    {let original wndproc handle the message}
    Message.Result := CallWindowProc(FDefClientProc, FClientHandle,
      Message.Msg, Message.WParam, Message.LParam);
end;

procedure TabcMDIScroll.SetEnabled(Value: Boolean);
var
  I: integer;
  twp: TWindowPos;
  pwp: ^TWindowPos;
  ChildPos: TPoint;
  PChildPos: ^TPoint;
  AMessage: TMessage;
begin
  if Value <> FEnabled then begin
    FEnabled := Value;
    if (csDesigning in ComponentState) then exit;
    if Value then begin
      with TForm(Owner) do
        for I := 0 to MDIChildCount - 1 do
          with MDIChildren[I] do begin
            ChildPos := Point(Left, Top);
            PChildPos := @ChildPos;
            DefMDIChildProc(Handle, WM_MOVE, 0, Longint(PChildPos));
          end;
    end
    else begin
      twp.hWnd := TForm(Owner).Handle;
      twp.flags := SWP_DRAWFRAME or SWP_NOACTIVATE or SWP_NOMOVE or
        SWP_NOOWNERZORDER or SWP_NOSIZE or SWP_NOZORDER;
      pwp := @twp;
      AMessage.Msg := WM_WINDOWPOSCHANGING;
      AMessage.wParam := 0;
      AMessage.LParam := Longint(pwp);
      HideMDIScrollBars(AMessage);
    end;
  end;
end;

end.
