unit abcfact;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ abstract factory singletons for ABC }

{$I abcdefs.inc}

interface

uses SysUtils, Classes, Forms;

type

  { TabcFormFactory }

  { This class allows you to replace abc forms
    with your own custom descendants using its RegisterFormClass method }

  TabcFormFactory = class(TObject)
  private
    FFormClasses: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    function GetFormClass(Ancestor: TFormClass): TFormClass;
    function CreateFormOfClass(Ancestor: TFormClass; AOwner: TComponent): TForm;
    procedure RegisterBaseFormClass(Ancestor: TFormClass);
    procedure RegisterFormClass(Ancestor, Descendant: TFormClass);
    procedure UnRegisterFormClass(Ancestor: TFormClass);
  end;

function abcFormFactory: TabcFormFactory;

implementation

var
  GlobalabcFormFactory: TabcFormFactory;

function abcFormFactory: TabcFormFactory;
begin
  if GlobalabcFormFactory = nil then
    GlobalabcFormFactory := TabcFormFactory.Create;
  Result := GlobalabcFormFactory;
end;

constructor TabcFormFactory.Create;
begin
  FFormClasses := TStringList.Create;
  FFormClasses.Sorted := True;
end;

destructor TabcFormFactory.Destroy;
begin
  FFormClasses.Free;
  inherited Destroy;
  GlobalabcFormFactory := nil;
end;

function TabcFormFactory.CreateFormOfClass(Ancestor: TFormClass; AOwner: TComponent): TForm;
var
  FoundClass: TFormClass;
begin
  FoundClass := GetFormClass(Ancestor);
  if FoundClass = nil then
    Result := nil
  else
    Result :=  FoundClass.Create(AOwner);
end;

function TabcFormFactory.GetFormClass(Ancestor: TFormClass): TFormClass;
var
  i: integer;
begin
  i := FFormClasses.IndexOf(Ancestor.ClassName);
  if i = -1 then
    Result := nil
  else
    Result := TFormClass(FFormClasses.Objects[i]);
end;

procedure TabcFormFactory.RegisterBaseFormClass(Ancestor: TFormClass);
begin
  RegisterFormClass(Ancestor, Ancestor);
end;

procedure TabcFormFactory.RegisterFormClass(Ancestor, Descendant: TFormClass);
begin
  if not Descendant.InheritsFrom(Ancestor) then
    raise Exception.Create(
      'Form Factory error: ' + Descendant.ClassName +
      ' does not inherit from ' + Ancestor.ClassName);
  UnRegisterFormClass(Ancestor);
{$IFDEF DELPHI6}
  ActivateClassGroup(TForm);
{$ENDIF}
  RegisterClass(Descendant);
  FFormClasses.AddObject(Ancestor.ClassName, TObject(Descendant));
end;

procedure TabcFormFactory.UnRegisterFormClass(Ancestor: TFormClass);
var
  i: integer;
  Removed: TFormClass;
begin
  i := FFormClasses.IndexOf(Ancestor.ClassName);
  if i >= 0 then
  begin
    Removed := TFormClass(FFormClasses.Objects[i]);
    FFormClasses.Delete(i);
    UnregisterClass(Removed);
  end;
end;

initialization

finalization
  GlobalabcFormFactory.Free;

end.
