unit abcfbusy;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ComCtrls,
  abcfact, abcforms, ExtCtrls, abcmsg, abcwav;

type
  TabcsfBusy = class(TabcForm)
    Animate: TAnimate;
    lblMessage1: TLabel;
    lblMessage2: TLabel;
    ProgressBar: TProgressBar;
    btnCancel: TButton;
    lblCheckCaptionSize: TLabel;
    bvlBorder: TBevel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FAfterActivate: TNotifyEvent;
    FCloseAfter: DWORD;
    FUserFormClose: TCloseEvent;
    FSound: TabcCustomWave;
    function CanPlay: Boolean;
    procedure CMActivate(var Message: TCMActivate); message CM_ACTIVATE;
    procedure AMAfterActivate(var Message: TWMNoParams); message AM_AFTERACTIVATE;
    procedure SizeToFitTop(AWordWrap: Boolean; AWidth: Integer);
    procedure SizeToFitLeft(AWordWrap: Boolean; AWidth: Integer);
    procedure MoveControlsToSizeTop(AWordWrap: Boolean);
    procedure MoveControlsToSizeLeft(AWordWrap: Boolean);
    function GetCaptionSize: Integer;
  public
    procedure SizeAndArrangeControls(AAutoSize, AWordWrap: Boolean; AWidth, AHeight, AOrdAVIPosition: Integer);
    property CloseAfter: DWORD read FCloseAfter write FCloseAfter;
    property Sound: TabcCustomWave read FSound write FSound;
    property AfterActivate: TNotifyEvent read FAfterActivate write FAfterActivate;
    property UserFormClose: TCloseEvent read FUserFormClose write FUserFormClose;
  end;

var
  abcsfBusy: TabcsfBusy;

implementation

{$R *.DFM}

uses abcbusy, abcutilm;

const
  HOFFSET = 6;
  VOFFSET = 6;
  MIN_PROGRESSWIDTH = 200;

function TabcsfBusy.CanPlay: Boolean;
begin
  with Animate do
    Result := ((FileName <> '') and FileExists(FileName)) or
              (CommonAVI <> aviNone) or
              (ResName <> '') or
              (ResId <> 0);
end;

procedure TabcsfBusy.CMActivate(var Message: TCMActivate);
begin
  inherited;
  with Animate do
    if Visible and CanPlay then
      Active := True;
  if Sound <> nil then Sound.Play;
  PostMessage(Handle, AM_AFTERACTIVATE, 0, 0);
end;

procedure TabcsfBusy.AMAfterActivate(var Message: TWMNoParams);
begin
  if Assigned(FAfterActivate) then
    FAfterActivate(Self);
end;

procedure TabcsfBusy.SizeAndArrangeControls(AAutoSize, AWordWrap: Boolean; AWidth, AHeight, AOrdAVIPosition: Integer);
begin
  if AAutoSize then
    if AOrdAVIPosition = Ord(avpTop) then
      SizeToFitTop(AWordWrap, AWidth)
    else
      SizeToFitLeft(AWordWrap, Screen.DesktopWidth)
  else
  begin
    SetBounds(Left, Top, AWidth, AHeight);
    if AOrdAVIPosition = Ord(avpTop) then
      MoveControlsToSizeTop(AWordWrap)
    else
      MoveControlsToSizeLeft(AWordWrap);
  end;
end;

function TabcsfBusy.GetCaptionSize: Integer;
var
  WindowDims: TNonClientMetrics;
begin
  //calculate width of caption text using window caption font
  WindowDims.cbSize := SizeOf(TNonClientMetrics);
  Win32Check(SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(TNonClientMetrics), @WindowDims, 0));
  with lblCheckCaptionSize.Font, WindowDims.lfCaptionFont do
  begin
    Charset := TFontCharset(lfCharSet);
    Name := lfFaceName;
    case lfPitchAndFamily and $F of
      VARIABLE_PITCH: Pitch := fpVariable;
      FIXED_PITCH: Pitch := fpFixed;
    else
      Pitch := fpDefault;
    end;
    Height := lfHeight;
    if lfWeight >= FW_BOLD then
      Style := Style + [fsBold];
    if lfItalic = 1 then
      Style := Style + [fsItalic];
    if lfUnderline = 1 then
      Style := Style + [fsUnderLine];
    if lfStrikeOut = 1 then
      Style := Style + [fsStrikeOut];
    lblCheckCaptionSize.Caption := Self.Caption;
    Result := lblCheckCaptionSize.Width;
  end;
end;

procedure TabcsfBusy.SizeToFitTop(AWordWrap: Boolean; AWidth: Integer);
var
  calcWidth, calcHeight, labelWidth: integer;
begin
  calcHeight := VOFFSET;

  if (Caption <> '') and (BorderStyle <> bsNone) then
    calcWidth := GetCaptionSize
  else
    calcWidth := 0;

  with Animate do
    if Visible then
    begin
      Left := HOFFSET;
      Top := calcHeight;
      if calcWidth < Width then
        calcWidth := Width;
      calcHeight := calcHeight + Height + VOFFSET;
    end;

  if calcWidth > (AWidth - 2*HOFFSET) then
    labelWidth := calcWidth
  else
    labelWidth := AWidth - 2*HOFFSET;

  with lblMessage1 do
    if Visible then
    begin
      Left := HOFFSET;
      Top := calcHeight;
      WordWrap := AWordWrap;
      if WordWrap and (Width > labelWidth) then
      begin
        Width := labelWidth;
        AutoSize := False;
        AutoSize := True;
      end;
      calcHeight := calcHeight + Height + VOFFSET;
      if (calcWidth < Width) then
        calcWidth := Width;
    end;

  with lblMessage2 do
    if Visible then
    begin
      Left := HOFFSET;
      Top := calcHeight;
      WordWrap := AWordWrap;
      if WordWrap and (Width > labelWidth) then
      begin
        Width := labelWidth;
        AutoSize := False;
        AutoSize := True;
      end;
      calcHeight := calcHeight + Height + VOFFSET;
      if (calcWidth < Width) then
        calcWidth := Width;
    end;

  with ProgressBar do
    if Visible then
    begin
      Left := HOFFSET;
      if (calcWidth < MIN_PROGRESSWIDTH + HOFFSET) then
        calcWidth := MIN_PROGRESSWIDTH + HOFFSET;
      if btnCancel.Visible then
      begin
        if (calcWidth < MIN_PROGRESSWIDTH + 3 * HOFFSET + btnCancel.Width) then
        begin
          calcWidth := MIN_PROGRESSWIDTH + 3 * HOFFSET + btnCancel.Width;
          Width := MIN_PROGRESSWIDTH;
        end
        else
          Width := calcWidth - HOFFSET - btnCancel.Width;
        btnCancel.Top := calcHeight;
        btnCancel.Left := Left + Width + HOFFSET;
        calcHeight := calcHeight + btnCancel.Height + VOFFSET;
        Top := btnCancel.Top + (btnCancel.Height - Height) div 2;
      end
      else
      begin
        Top := calcHeight;
        calcHeight := calcHeight + Height + VOFFSET;
        Width := calcWidth;
      end;
    end
    else
      with btnCancel do
        if Visible then
        begin
          Top := calcHeight;
          calcHeight := calcHeight + Height + VOFFSET;
          if (calcWidth < Width) then
            calcWidth := Width;
          Left := HOFFSET + (calcWidth - Width) div 2;
        end;

  if BorderStyle <> bsNone then
    calcWidth := calcWidth + 2 * GetSystemMetrics(SM_CXDLGFRAME);


  with Animate do if Visible then
    Left := (calcWidth - Width) div 2;

  if BorderStyle <> bsNone then
    calcHeight := calcHeight + GetSystemMetrics(SM_CYCAPTION) + 2 * GetSystemMetrics(SM_CYDLGFRAME);

  if calcWidth > Screen.Width then
    calcWidth := Screen.Width;

  SetBounds(Left, Top, calcWidth, calcHeight);

end;

procedure TabcsfBusy.SizeToFitLeft(AWordWrap: Boolean; AWidth: Integer);
var
  calcLeft, calcWidth, calcHeight, calcHeightLabels, labelWidth: integer;
begin

  calcHeight := VOFFSET;

  if (Caption <> '') and (BorderStyle <> bsNone) then
    calcWidth := GetCaptionSize
  else
    calcWidth := 0;

  calcLeft := HOFFSET;

  with Animate do
    if Visible then
    begin
      Left := HOFFSET;
      Top := VOFFSET;
      calcLeft := calcLeft + Width + HOFFSET;
      if calcWidth < Width then
        calcWidth := Width;
      calcHeight := calcHeight + Height + VOFFSET;
    end;

  if (calcWidth - calcLeft) > (AWidth - 2*HOFFSET) then
    labelWidth := calcWidth - calcLeft
  else
    labelWidth := AWidth - 2*HOFFSET - calcLeft;

  calcHeightLabels := VOFFSET;

  with lblMessage1 do
    if Visible then
    begin
      Left := calcLeft;
      Top := calcHeightLabels;
      WordWrap := AWordWrap;
      if WordWrap and (Width > labelWidth) then
      begin
        Width := labelWidth;
        AutoSize := False;
        AutoSize := True;
        if Width > labelWidth then
          Width := labelWidth;
      end;
      calcHeightLabels := calcHeightLabels + Height + VOFFSET;
      if (calcWidth < calcLeft + Width) then
        calcWidth := calcLeft + Width;
    end;

  with lblMessage2 do
    if Visible then
    begin
      Left := calcLeft;
      Top := calcHeightLabels;
      WordWrap := AWordWrap;
      if WordWrap and (Width > labelWidth) then
      begin
        Width := labelWidth;
        AutoSize := False;
        AutoSize := True;
        if Width > labelWidth then
          Width := labelWidth;
      end;
      calcHeightLabels := calcHeightLabels + Height + VOFFSET;
      if (calcWidth < calcLeft + Width) then
        calcWidth := calcLeft + Width;
    end;

  with ProgressBar do
    if Visible then
    begin
      Left := calcLeft;
      if (calcWidth < calcLeft + MIN_PROGRESSWIDTH) then
        calcWidth := calcLeft + MIN_PROGRESSWIDTH;
      if btnCancel.Visible then
      begin
        if (calcWidth < calcLeft + MIN_PROGRESSWIDTH + HOFFSET + btnCancel.Width) then
        begin
          calcWidth := calcLeft + MIN_PROGRESSWIDTH + HOFFSET + btnCancel.Width;
          Width := MIN_PROGRESSWIDTH;
        end
        else
          Width := calcWidth - HOFFSET - btnCancel.Width - calcLeft;
        btnCancel.Top := calcHeightLabels;
        btnCancel.Left := Left + Width + HOFFSET;
        calcHeightLabels := calcHeightLabels + btnCancel.Height + VOFFSET;
        Top := btnCancel.Top + (btnCancel.Height - Height) div 2;
      end
      else
      begin
        Top := calcHeightLabels;
        calcHeightLabels := calcHeightLabels + Height + VOFFSET;
        Width := calcWidth - calcLeft;
      end;
    end
    else
      with btnCancel do
        if Visible then
        begin
          Top := calcHeightLabels;
          calcHeightLabels := calcHeightLabels + Height + VOFFSET;
          if (calcWidth < calcLeft + Width) then
            calcWidth := calcLeft + Width;
          Left := calcLeft + (calcWidth - Width - calcLeft) div 2;
        end;

  if calcHeight < calcHeightLabels then
  begin
    if Animate.Visible then
      Animate.Top := VOFFSET + (calcHeightLabels - calcHeight) div 2;
    calcHeight := calcHeightLabels;
  end;

  calcWidth := calcWidth + HOFFSET;

  if BorderStyle <> bsNone then
  begin
    calcWidth := calcWidth + 2 * GetSystemMetrics(SM_CXDLGFRAME);
    calcHeight := calcHeight + GetSystemMetrics(SM_CYCAPTION) + 2 * GetSystemMetrics(SM_CYDLGFRAME);
  end;

  SetBounds(Left, Top, calcWidth, calcHeight);

end;

procedure TabcsfBusy.MoveControlsToSizeTop(AWordWrap: Boolean);
var
  NextTop: integer;
begin
  NextTop := VOFFSET;

  with Animate do
    if Visible then
    begin
      Left := (Self.Width - Width) div 2;
      Top := NextTop;
      NextTop := Height + VOFFSET;
    end;

  with lblMessage1 do
    if Visible then
    begin
      Left := HOFFSET;
      Top := NextTop;
      WordWrap := AWordWrap;
      Width := Self.ClientWidth - 2 * HOFFSET;
      AutoSize := False;
      AutoSize := True;
      NextTop := Height + VOFFSET;
    end;

  with lblMessage2 do
    if Visible then
    begin
      Left := HOFFSET;
      Top := NextTop;
      WordWrap := AWordWrap;
      Width := Self.ClientWidth - 2 * HOFFSET;
      AutoSize := False;
      AutoSize := True;
      NextTop := Height + VOFFSET;
    end;

  with ProgressBar do
    if Visible then
    begin
      Left := HOFFSET;
      if btnCancel.Visible then
      begin
        btnCancel.Top := NextTop;
        btnCancel.Left := Self.Width - HOFFSET;
        Width := btnCancel.Left - 2 * HOFFSET;
        Top := btnCancel.Top + (btnCancel.Height - Height) div 2;
      end
      else
      begin
        Top := NextTop;
        Width := Self.Width - 2 * HOFFSET;
      end;
    end
    else
      with btnCancel do
        if Visible then
        begin
          Left := (Self.Width - Width) div 2;
          Top := NextTop;
        end;
end;

procedure TabcsfBusy.MoveControlsToSizeLeft(AWordWrap: Boolean);
var
  NextTop, LabelsLeft: integer;
begin
  NextTop := VOFFSET;
  LabelsLeft := HOFFSET;

  with Animate do
    if Visible then
    begin
      Left := HOFFSET;
      LabelsLeft := LabelsLeft + Width + HOFFSET;
      Top := NextTop;
    end;

  with lblMessage1 do
    if Visible then
    begin
      Left := LabelsLeft;
      Top := NextTop;
      WordWrap := AWordWrap;
      Width := Self.ClientWidth - LabelsLeft - HOFFSET;
      AutoSize := False;
      AutoSize := True;
      NextTop := Height + VOFFSET;
    end;

  with lblMessage2 do
    if Visible then
    begin
      Left := LabelsLeft;
      Top := NextTop;
      WordWrap := AWordWrap;
      Width := Self.ClientWidth - LabelsLeft - HOFFSET;
      AutoSize := False;
      AutoSize := True;
      NextTop := Height + VOFFSET;
    end;

  with ProgressBar do
    if Visible then
    begin
      Left := LabelsLeft;
      if btnCancel.Visible then
      begin
        btnCancel.Top := NextTop;
        btnCancel.Left := Self.Width - HOFFSET;
        Width := btnCancel.Left - HOFFSET - LabelsLeft;
        Top := btnCancel.Top + (btnCancel.Height - Height) div 2;
      end
      else
      begin
        Top := NextTop;
        Width := Self.Width - 2 * HOFFSET;
      end;
    end
    else
      with btnCancel do
        if Visible then
        begin
          Left := (Self.Width - Width) div 2;
          Top := NextTop;
        end;
end;

procedure TabcsfBusy.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  while GetTickCount < FCloseAfter do
    Application.ProcessMessages;
  if Sound <> nil then Sound.Stop;
  Action := caFree;
  if Assigned(FUserFormClose) then
    FUserFormClose(Self, Action);
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcsfBusy);

end.
