unit abcfdbim;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, StdCtrls, ExtCtrls, Buttons, DB, DBCtrls,
  abcforms, abcbtn, abcpanel, abcfact;

type

  TPicEditorMenuItem = (pmFileNew, pmFileOpen, pmFileSave, pmFileSaveAs, pmFilePrint,
    pmEditCut, pmEditCopy, pmEditPaste, pmLarger, pmSmaller, pmScaled, pmSizeToFit);

  TPicPopupClickEvent = procedure(Sender: TObject; MenuItem: TPicEditorMenuItem) of object;

  TabcafGraphicFieldEdit = class(TabcApplicationForm)
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    PrintDialog1: TPrintDialog;
    PopupMenu1: TPopupMenu;
    FileOpen: TMenuItem;
    FileSaveAs: TMenuItem;
    FilePrint: TMenuItem;
    N3: TMenuItem;
    EditCut: TMenuItem;
    EditCopy: TMenuItem;
    EditPaste: TMenuItem;
    N4: TMenuItem;
    N5: TMenuItem;
    Panel2: TPanel;
    DBImage1: TDBImage;
    ViewLarger: TMenuItem;
    ViewSmaller: TMenuItem;
    ViewActual: TMenuItem;
    ViewStretch: TMenuItem;
    View100: TMenuItem;
    View25: TMenuItem;
    View50: TMenuItem;
    View75: TMenuItem;
    N9: TMenuItem;
    View150: TMenuItem;
    View200: TMenuItem;
    View300: TMenuItem;
    View400: TMenuItem;
    N8: TMenuItem;
    DataSource1: TDataSource;
    pnlToolBar: TabcEdgePanel;
    scrToolbar: TScrollBox;
    psbFileOpen: TabcPicSpeedBtn;
    psbFileSaveAs: TabcPicSpeedBtn;
    psbFilePrint: TabcPicSpeedBtn;
    psbViewLarger: TabcPicSpeedBtn;
    psbViewSmaller: TabcPicSpeedBtn;
    psbViewStretch: TabcPicSpeedBtn;
    psbEditCut: TabcPicSpeedBtn;
    psbEditCopy: TabcPicSpeedBtn;
    psbEditPaste: TabcPicSpeedBtn;
    cboSize: TComboBox;
    EdgePanel1: TabcEdgePanel;
    OKBtn: TBitBtn;
    HelpBtn: TBitBtn;
    psbScrollLeft: TabcPicSpeedBtn;
    psbScrollRight: TabcPicSpeedBtn;
    procedure FileOpenClick(Sender: TObject);
    procedure FileSaveAsClick(Sender: TObject);
    procedure FilePrintClick(Sender: TObject);
    procedure EditCopyClick(Sender: TObject);
    procedure EditCutClick(Sender: TObject);
    procedure EditPasteClick(Sender: TObject);
    procedure pnlToolbarResize(Sender: TObject);
    procedure psbScrollLeftClick(Sender: TObject);
    procedure psbScrollRightClick(Sender: TObject);
    procedure scrToolbarResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HelpBtnClick(Sender: TObject);
    procedure ViewStretchClick(Sender: TObject);
    procedure ViewLargerClick(Sender: TObject);
    procedure ViewSmallerClick(Sender: TObject);
    procedure Panel2Resize(Sender: TObject);
    procedure View100Click(Sender: TObject);
    procedure View25Click(Sender: TObject);
    procedure View50Click(Sender: TObject);
    procedure View75Click(Sender: TObject);
    procedure View150Click(Sender: TObject);
    procedure View200Click(Sender: TObject);
    procedure View300Click(Sender: TObject);
    procedure View400Click(Sender: TObject);
    procedure cboSizeChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EdgePanel1Resize(Sender: TObject);
  private
    FEdited: Boolean;
    FFileName: TFileName;
    FOnChange: TNotifyEvent;
    FOnPopupClick: TPicPopupClickEvent;
    FScrollLeftBtn: byte;
    FScaleSize: Integer;
    function  GetPopupEnabled: Boolean;
    procedure SetPopupEnabled(Value: Boolean);
    function  GetOnPopup: TNotifyEvent;
    procedure SetOnPopup(Value: TNotifyEvent);
    function GetToolbarVisible: Boolean;
    procedure SetToolbarVisible(Value: Boolean);
    procedure CheckScrollButtons;
    procedure SetReadOnly(AReadOnly: Boolean);
  public
    destructor Destroy; override;
    procedure ViewScaled(Percent: Integer);
    property Edited: Boolean read FEdited;
    property FileName: TFileName read FFileName;
    property PopupEnabled: Boolean read GetPopupEnabled write SetPopupEnabled default True;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnPopup: TNotifyEvent read GetOnPopup write SetOnPopup;
    property OnPopupClick: TPicPopupClickEvent read FOnPopupClick write FOnPopupClick;
    property ScaleSize: Integer read FScaleSize;
    property ToolbarVisible: Boolean read GetToolbarVisible write SetToolbarVisible default True;
  end;

function ExecabcafGraphicFieldEdit(
  AOwner: TComponent;
  ADataset: TDataset;
  AField: TField;
  APopupFormStyle: TabcPopupFormStyle;
  AReadOnly: Boolean;
  AHelpContext: THelpContext
  ): TForm;

implementation

{$R *.DFM}

uses Printers, abcconst, abcutilf;

const
  ToolBarHeight = 29;
  ScrollBtnWidth = 13;
  ToolbarBtnWidth = 25;
  ToolbarBtnCount = 12;
  ToolbarMax = ToolbarBtnCount * ToolbarBtnWidth;
  FullWidth = ToolbarMax + 2 * ScrollBtnWidth;

function ExecabcafGraphicFieldEdit(
  AOwner: TComponent;
  ADataset: TDataset;
  AField: TField;
  APopupFormStyle: TabcPopupFormStyle;
  AReadOnly: Boolean;
  AHelpContext: THelpContext
  ): TForm;
var
  Form: TabcafGraphicFieldEdit;
begin
  Form := TabcafGraphicFieldEdit(
    abcFormFactory.CreateFormOfClass(TabcafGraphicFieldEdit, AOwner));
  Result := Form;
  with Form do begin
    if AHelpContext <> 0 then begin
      HelpContext := AHelpContext;
      HelpBtn.Visible := True;
    end;
    DataSource1.Dataset := ADataSet;
    with DBImage1 do begin
      DataField := AField.FieldName;
      ReadOnly := AReadOnly or not AField.CanModify;
      if ReadOnly then
        Caption := 'View ' + AField.DisplayLabel
      else
        Caption := 'Edit ' + AField.DisplayLabel;
      SetReadOnly(ReadOnly);
    end;
    case APopupFormStyle of
      pfsNormal, pfsModal:
        FormStyle := fsNormal;
      pfsMDIChild:
        {make sure MDI form exists}
        if (Application.MainForm = nil) or
          (Application.MainForm.ClientHandle = 0) then
          FormStyle := fsNormal
        else
          FormStyle := fsMDIChild;
      pfsStayOnTop:
        FormStyle := fsStayOnTop;
    end;

    if APopupFormStyle = pfsModal then begin
      OKBtn.Kind := bkOK;
      ShowModal;
    end
    else begin
      OKBtn.Kind := bkClose;
      Show;
    end;
  end;
end;

destructor TabcafGraphicFieldEdit.Destroy;
begin
  inherited Destroy;
end;

procedure TabcafGraphicFieldEdit.SetReadOnly(AReadOnly: Boolean);
begin
  if AReadOnly then begin
    DBImage1.ReadOnly := True;
    FileOpen.Enabled := False;
    EditCut.Enabled := False;
    EditPaste.Enabled := False;
    psbFileOpen.Enabled := False;
    psbEditCut.Enabled := False;
    psbEditPaste.Enabled := False;
  end
  else begin
    DBImage1.ReadOnly := False;
    FileOpen.Enabled := True;
    EditCut.Enabled := True;
    EditPaste.Enabled := True;
    psbFileOpen.Enabled := True;
    psbEditCut.Enabled := True;
    psbEditPaste.Enabled := True;
  end;
end;

function  TabcafGraphicFieldEdit.GetPopupEnabled: Boolean;
begin
  Result := PopupMenu1.AutoPopup;
end;

procedure TabcafGraphicFieldEdit.SetPopupEnabled(Value: Boolean);
begin
  if Value <> PopupEnabled then
    PopupMenu1.AutoPopup := Value;
end;

function  TabcafGraphicFieldEdit.GetOnPopup: TNotifyEvent;
begin
  Result := PopupMenu1.OnPopup;
end;

procedure TabcafGraphicFieldEdit.SetOnPopup(Value: TNotifyEvent);
begin
  PopupMenu1.OnPopup := Value;
end;

function TabcafGraphicFieldEdit.GetToolbarVisible: Boolean;
begin
  Result := pnlToolbar.Visible;
end;

procedure TabcafGraphicFieldEdit.SetToolbarVisible(Value: Boolean);
begin
  pnlToolbar.Visible := Value;
  if (csDesigning in ComponentState) then
    if Value then
      pnlToolbar.Height := ToolBarHeight
    else
      pnlToolbar.Height := 0;
end;

procedure TabcafGraphicFieldEdit.FileOpenClick(Sender: TObject);
begin
  with OpenDialog1 do
    if Execute then begin
      DBImage1.DataSource.DataSet.Edit;
      TGraphicField(DBImage1.Field).LoadFromFile(FileName);
      FFileName := FileName;
    end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmFileOpen);
end;

procedure TabcafGraphicFieldEdit.FileSaveAsClick(Sender: TObject);
begin
  with SaveDialog1 do begin
    if DBImage1.Picture.Graphic is TBitmap then
      Filter := sabcFilterBMPAll
    else if DBImage1.Picture.Graphic is TIcon then
      Filter := sabcFilterICOAll
    else if DBImage1.Picture.Graphic is TMetaFile then
      Filter := sabcFilterWMFAll
    else
      Filter := sabcFilterAll;
    if Execute then begin
      DBImage1.Picture.Graphic.SaveToFile(FileName);
      FFileName := FileName;
    end;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmFileSaveAs);
end;

procedure TabcafGraphicFieldEdit.FilePrintClick(Sender: TObject);
begin
  if PrintDialog1.Execute then abcPrintControlImage(DBImage1);
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmFilePrint);
end;

procedure TabcafGraphicFieldEdit.EditCutClick(Sender: TObject);
begin
  with DBImage1 do begin
    CopyToClipBoard;
    DataSource.DataSet.Edit;
    Picture.Bitmap.Width := 0;
    Picture.Bitmap.Height := 0;
    Invalidate;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmEditCut);
end;

procedure TabcafGraphicFieldEdit.EditCopyClick(Sender: TObject);
begin
  DBImage1.CopyToClipboard;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmEditCopy);
end;

procedure TabcafGraphicFieldEdit.EditPasteClick(Sender: TObject);
begin
  DBImage1.PasteFromClipboard;
  ViewScaled(FScaleSize);
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmEditPaste);
end;

procedure TabcafGraphicFieldEdit.pnlToolbarResize(Sender: TObject);
begin
  if pnlToolbar.Width > FullWidth then begin
    psbScrollRight.Left := FullWidth - ScrollBtnWidth - 1;
    scrToolbar.Width := ToolbarMax;
    psbScrollLeft.Enabled := False;
    psbScrollRight.Enabled := False;
  end
  else begin
    psbScrollRight.Left := pnlToolbar.Width - ScrollBtnWidth;
    scrToolbar.Width := pnlToolbar.Width - 2 * ScrollBtnWidth + 1;
    CheckScrollButtons;
  end;
end;

procedure TabcafGraphicFieldEdit.psbScrollLeftClick(Sender: TObject);
begin
  scrToolbar.ScrollBy(ToolbarBtnWidth,0);
  Dec(FScrollLeftBtn);
  CheckScrollButtons;
end;

procedure TabcafGraphicFieldEdit.psbScrollRightClick(Sender: TObject);
begin
  scrToolbar.ScrollBy(-ToolbarBtnWidth,0);
  Inc(FScrollLeftBtn);
  CheckScrollButtons;
end;

procedure TabcafGraphicFieldEdit.scrToolbarResize(Sender: TObject);
begin
  if FScrollLeftBtn <> 0 then begin
    scrToolbar.ScrollBy(FScrollLeftBtn * ToolbarBtnWidth, 0);
    FScrollLeftBtn := 0;
  end;
  CheckScrollButtons;
end;

procedure TabcafGraphicFieldEdit.CheckScrollButtons;
begin
  if FScrollLeftBtn > 0 then
    psbScrollLeft.Enabled := True
  else
    psbScrollLeft.Enabled := False;
  if scrToolbar.Width < (ToolbarBtnCount - FScrollLeftBtn) * ToolbarBtnWidth then
    psbScrollRight.Enabled := True
  else
    psbScrollRight.Enabled := False;
end;

procedure TabcafGraphicFieldEdit.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TabcafGraphicFieldEdit.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

procedure TabcafGraphicFieldEdit.ViewStretchClick(Sender: TObject);
begin
  DBImage1.Align := alClient;
  cboSize.ItemIndex := -1;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmSizeToFit);
end;

procedure TabcafGraphicFieldEdit.ViewLargerClick(Sender: TObject);
var
  ALeft, ATop, AWidth, AHeight: Integer;
begin
  with DBImage1 do begin
    AWidth := MulDiv(Width, 120, 100);
    AHeight := MulDiv(Height, 120, 100);
    ALeft := Left + (Width - AWidth + 1) div 2;
    ATop := Top + (Height - AHeight + 1) div 2;
    Align := alNone;
    SetBounds(ALeft, ATop, AWidth, AHeight);
    cboSize.ItemIndex := -1;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmLarger);
end;

procedure TabcafGraphicFieldEdit.ViewSmallerClick(Sender: TObject);
var
  ALeft, ATop, AWidth, AHeight: Integer;
begin
  with DBImage1 do begin
    AWidth := MulDiv(Width, 100, 120);
    AHeight := MulDiv(Height, 100, 120);
    ALeft := Left + (Width - AWidth + 1) div 2;
    ATop := Top + (Height - AHeight + 1) div 2;
    Align := alNone;
    SetBounds(ALeft, ATop, AWidth, AHeight);
    cboSize.ItemIndex := -1;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, pmSmaller);
end;

procedure TabcafGraphicFieldEdit.Panel2Resize(Sender: TObject);
var
  ALeft, ATop: Integer;
begin
  with DBImage1 do
    if Align <> alClient then begin
      ALeft := (Panel2.Width - Width + 1) div 2;
      ATop := (Panel2.Height - Height + 1) div 2;
      SetBounds(ALeft, ATop, Width, Height);
    end;
end;

procedure TabcafGraphicFieldEdit.ViewScaled(Percent: Integer);
var
  ALeft, ATop, AWidth, AHeight: Integer;
begin
  with DBImage1 do begin
    AWidth := MulDiv(Picture.Width, Percent, 100);
    AHeight := MulDiv(Picture.Height, Percent, 100);
    ALeft := (Panel2.Width - AWidth + 1) div 2;
    ATop := (Panel2.Height - AHeight + 1) div 2;
    Align := alNone;
    SetBounds(ALeft, ATop, AWidth, AHeight);
    FScaleSize := Percent;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Self, pmScaled);
end;

procedure TabcafGraphicFieldEdit.View100Click(Sender: TObject);
begin
  ViewScaled(100);
end;

procedure TabcafGraphicFieldEdit.View25Click(Sender: TObject);
begin
  ViewScaled(25);
end;

procedure TabcafGraphicFieldEdit.View50Click(Sender: TObject);
begin
  ViewScaled(50);
end;

procedure TabcafGraphicFieldEdit.View75Click(Sender: TObject);
begin
  ViewScaled(75);
end;

procedure TabcafGraphicFieldEdit.View150Click(Sender: TObject);
begin
  ViewScaled(150);
end;

procedure TabcafGraphicFieldEdit.View200Click(Sender: TObject);
begin
  ViewScaled(200);
end;

procedure TabcafGraphicFieldEdit.View300Click(Sender: TObject);
begin
  ViewScaled(300);
end;

procedure TabcafGraphicFieldEdit.View400Click(Sender: TObject);
begin
  ViewScaled(400);
end;

procedure TabcafGraphicFieldEdit.cboSizeChange(Sender: TObject);
begin
  case cboSize.ItemIndex of
    0: View25Click(Self);
    1: View50Click(Self);
    2: View75Click(Self);
    3: View100Click(Self);
    4: View150Click(Self);
    5: View200Click(Self);
    6: View300Click(Self);
    7: View400Click(Self);
  end;
  OKBtn.SetFocus;
end;

procedure TabcafGraphicFieldEdit.FormShow(Sender: TObject);
begin
  cboSize.ItemIndex := 3;
  View100Click(Self);
  pnlToolBar.Width := 0; {force resize for buttons}
end;

procedure TabcafGraphicFieldEdit.EdgePanel1Resize(Sender: TObject);
const
  BtnSpace = 5;
var
  LastBtnLeft: Integer;
begin
  LastBtnLeft := EdgePanel1.Width;
  with HelpBtn do
    if Visible then begin
      Left := LastBtnLeft - Width - BtnSpace;
      LastBtnLeft := Left;
    end;
  with OKBtn do Left := LastBtnLeft - Width - BtnSpace;
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcafGraphicFieldEdit);

end.

