unit abcfdbme;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, StdCtrls, TabNotBk, ExtCtrls, Buttons, DB, DBCtrls,
  abcforms, abcbtn, abcpanel, abcfact;

type

  TEditorMenuItem = (emFileNew, emFileOpen, emFileSave, emFileSaveAs, emFilePrint,
    emEditUndo, emEditCut, emEditCopy, emEditPaste, emEditDelete, emEditSelectAll,
    emEditTimeDate, emEditWordWrap, emEditTabs, emSearchFind, emSearchFindNext,
    emSearchReplace, emOptionsColor, emOptionsFont);

  TPopupClickEvent = procedure(Sender: TObject; MenuItem: TEditorMenuItem) of object;

  TabcafMemoFieldEdit = class(TabcApplicationForm)
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    FontDialog1: TFontDialog;
    ColorDialog1: TColorDialog;
    PrintDialog1: TPrintDialog;
    FindDialog1: TFindDialog;
    ReplaceDialog1: TReplaceDialog;
    PopupMenu1: TPopupMenu;
    FileOpen: TMenuItem;
    FileSaveAs: TMenuItem;
    N1: TMenuItem;
    FilePrint: TMenuItem;
    N2: TMenuItem;
    EditUndo: TMenuItem;
    N3: TMenuItem;
    EditCut: TMenuItem;
    EditCopy: TMenuItem;
    EditPaste: TMenuItem;
    EditDelete: TMenuItem;
    N4: TMenuItem;
    EditSelectAll: TMenuItem;
    EditTimeDate: TMenuItem;
    EditWordWrap: TMenuItem;
    N5: TMenuItem;
    SearchFind: TMenuItem;
    SearchFindNext: TMenuItem;
    SearchReplace: TMenuItem;
    N6: TMenuItem;
    N7: TMenuItem;
    OptionsFont: TMenuItem;
    OptionsColor: TMenuItem;
    Panel2: TPanel;
    DBMemo1: TDBMemo;
    EditTabs: TMenuItem;
    DataSource1: TDataSource;
    EdgePanel1: TabcEdgePanel;
    OKBtn: TBitBtn;
    HelpBtn: TBitBtn;
    pnlToolbar: TabcEdgePanel;
    scrToolbar: TScrollBox;
    psbScrollLeft: TabcPicSpeedBtn;
    psbScrollRight: TabcPicSpeedBtn;
    psbFileOpen: TabcPicSpeedBtn;
    psbFileSaveAs: TabcPicSpeedBtn;
    psbFilePrint: TabcPicSpeedBtn;
    psbEditUndo: TabcPicSpeedBtn;
    psbEditCut: TabcPicSpeedBtn;
    psbEditCopy: TabcPicSpeedBtn;
    psbEditPaste: TabcPicSpeedBtn;
    psbEditDelete: TabcPicSpeedBtn;
    psbEditSelectAll: TabcPicSpeedBtn;
    psbEditTimeDate: TabcPicSpeedBtn;
    tbEditWordWrap: TabcToggleBtn;
    tbEditTabs: TabcToggleBtn;
    psbSearchFind: TabcPicSpeedBtn;
    psbSearchFindNext: TabcPicSpeedBtn;
    psbSearchReplace: TabcPicSpeedBtn;
    psbOptionsColor: TabcPicSpeedBtn;
    psbOptionsFont: TabcPicSpeedBtn;
    procedure FileOpenClick(Sender: TObject);
    procedure DBMemo1Change(Sender: TObject);
    procedure FileSaveAsClick(Sender: TObject);
    procedure OptionsFontClick(Sender: TObject);
    procedure OptionsColorClick(Sender: TObject);
    procedure FilePrintClick(Sender: TObject);
    procedure EditCopyClick(Sender: TObject);
    procedure EditCutClick(Sender: TObject);
    procedure EditDeleteClick(Sender: TObject);
    procedure EditPasteClick(Sender: TObject);
    procedure EditSelectAllClick(Sender: TObject);
    procedure EditTimeDateClick(Sender: TObject);
    procedure EditWordWrapClick(Sender: TObject);
    procedure SearchFindClick(Sender: TObject);
    procedure SearchReplaceClick(Sender: TObject);
    procedure FindDialog1Find(Sender: TObject);
    procedure SearchFindNextClick(Sender: TObject);
    procedure ReplaceDialog1Replace(Sender: TObject);
    procedure EditUndoClick(Sender: TObject);
    procedure pnlToolbarResize(Sender: TObject);
    procedure sbScrollLeftClick(Sender: TObject);
    procedure sbScrollRightClick(Sender: TObject);
    procedure scrToolbarResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure EdgePanel1Resize(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure EditTabsClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FEdited: Boolean;
    FFileName: TFileName;
    FOnChange: TNotifyEvent;
    FOnPopupClick: TPopupClickEvent;
    FFindText: string;
    FFound: Boolean;
    FMatchCase: Boolean;
    FStartPos: integer;
    FReplaceText: string;
    FScrollLeftBtn: byte;
    function GetFilter: string;
    procedure SetFilter(Value: string);
    function  GetPopupEnabled: Boolean;
    procedure SetPopupEnabled(Value: Boolean);
    function  GetOnPopup: TNotifyEvent;
    procedure SetOnPopup(Value: TNotifyEvent);
    function GetToolbarVisible: Boolean;
    procedure SetToolbarVisible(Value: Boolean);
    procedure ActivateFormMemo(Sender: TObject);
    procedure MemoFind;
    procedure MemoFindFromPos;
    procedure MemoFindNext;
    procedure MemoReplaceAll;
    procedure MemoReplaceSelection;
    procedure DisableUndo;
    procedure ResetEdit;
    procedure CheckScrollButtons;
    procedure SetReadOnly(AReadOnly: Boolean);
  public
    property Edited: Boolean read FEdited;
    property FileName: TFileName read FFileName;
    property Filter: string read GetFilter write SetFilter;
    property PopupEnabled: Boolean read GetPopupEnabled write SetPopupEnabled default True;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnPopup: TNotifyEvent read GetOnPopup write SetOnPopup;
    property OnPopupClick: TPopupClickEvent read FOnPopupClick write FOnPopupClick;
    property ToolbarVisible: Boolean read GetToolbarVisible write SetToolbarVisible default True;
  end;

function ExecabcafMemoFieldEdit(
  AOwner: TComponent;
  ADataset: TDataset;
  AField: TField;
  APopupFormStyle: TabcPopupFormStyle;
  AReadOnly: Boolean;
  AHelpContext: THelpContext
  ): TForm;

implementation

{$R *.DFM}

uses abcutilf;

const
  ToolBarHeight = 29;
  ScrollBtnWidth = 13;
  ToolbarBtnWidth = 25;
  ToolbarBtnCount = 17;
  ToolbarMax = ToolbarBtnCount * ToolbarBtnWidth;
  FullWidth = ToolbarMax + 2 * ScrollBtnWidth;

function ExecabcafMemoFieldEdit(
  AOwner: TComponent;
  ADataset: TDataset;
  AField: TField;
  APopupFormStyle: TabcPopupFormStyle;
  AReadOnly: Boolean;
  AHelpContext: THelpContext
  ): TForm;
var
  Form: TabcafMemoFieldEdit;
begin
  Form := TabcafMemoFieldEdit(
    abcFormFactory.CreateFormOfClass(TabcafMemoFieldEdit, AOwner));
  Result := Form;
  with Form do begin
    if AHelpContext <> 0 then begin
      HelpContext := AHelpContext;
      HelpBtn.Visible := True;
    end;
    DataSource1.Dataset := ADataSet;
    with DBMemo1 do begin
      DataField := AField.FieldName;
      ReadOnly := AReadOnly or not AField.CanModify;
      if ReadOnly then
        Caption := 'View ' + AField.DisplayLabel
      else
        Caption := 'Edit ' + AField.DisplayLabel;
      SetReadOnly(ReadOnly);
    end;
    ResetEdit;
    case APopupFormStyle of
      pfsNormal, pfsModal:
        FormStyle := fsNormal;
      pfsMDIChild:
        {make sure MDI form exists}
        if (Application.MainForm = nil) or
          (Application.MainForm.ClientHandle = 0) then
          FormStyle := fsNormal
        else
          FormStyle := fsMDIChild;
      pfsStayOnTop:
        FormStyle := fsStayOnTop;
    end;

    if APopupFormStyle = pfsModal then begin
      OKBtn.Kind := bkOK;
      ShowModal;
    end
    else begin
      OKBtn.Kind := bkClose;
      Show;
    end;
  end;
end;

procedure TabcafMemoFieldEdit.SetReadOnly(AReadOnly: Boolean);
begin
  if AReadOnly then begin
    DBMemo1.ReadOnly := True;
    FileOpen.Enabled := False;
    EditUndo.Enabled := False;
    EditCut.Enabled := False;
    EditPaste.Enabled := False;
    EditDelete.Enabled := False;
    EditTimeDate.Enabled := False;
    EditTabs.Enabled := False;
    SearchReplace.Enabled := False;
    psbFileOpen.Enabled := False;
    psbEditUndo.Enabled := False;
    psbEditCut.Enabled := False;
    psbEditPaste.Enabled := False;
    psbEditDelete.Enabled := False;
    psbEditTimeDate.Enabled := False;
    tbEditTabs.Enabled := False;
    psbSearchReplace.Enabled := False;
  end
  else begin
    DBMemo1.ReadOnly := False;
    FileOpen.Enabled := True;
    EditUndo.Enabled := True;
    EditCut.Enabled := True;
    EditPaste.Enabled := True;
    EditDelete.Enabled := True;
    EditTimeDate.Enabled := True;
    EditTabs.Enabled := True;
    SearchReplace.Enabled := True;
    psbFileOpen.Enabled := True;
    psbEditUndo.Enabled := True;
    psbEditCut.Enabled := True;
    psbEditPaste.Enabled := True;
    psbEditDelete.Enabled := True;
    psbEditTimeDate.Enabled := True;
    tbEditTabs.Enabled := True;
    psbSearchReplace.Enabled := True;
  end;
end;

function  TabcafMemoFieldEdit.GetFilter: string;
begin
  Result := OpenDialog1.Filter;
end;

procedure TabcafMemoFieldEdit.SetFilter(Value: string);
begin
  if Value <> Filter then begin
    OpenDialog1.Filter := Value;
    SaveDialog1.Filter := Value;
  end;
end;

function  TabcafMemoFieldEdit.GetPopupEnabled: Boolean;
begin
  Result := PopupMenu1.AutoPopup;
end;

procedure TabcafMemoFieldEdit.SetPopupEnabled(Value: Boolean);
begin
  if Value <> PopupEnabled then
    PopupMenu1.AutoPopup := Value;
end;

function  TabcafMemoFieldEdit.GetOnPopup: TNotifyEvent;
begin
  Result := PopupMenu1.OnPopup;
end;

procedure TabcafMemoFieldEdit.SetOnPopup(Value: TNotifyEvent);
begin
  PopupMenu1.OnPopup := Value;
end;

function TabcafMemoFieldEdit.GetToolbarVisible: Boolean;
begin
  Result := pnlToolbar.Visible;
end;

procedure TabcafMemoFieldEdit.SetToolbarVisible(Value: Boolean);
begin
  pnlToolbar.Visible := Value;
  if (csDesigning in ComponentState) then
    if Value then
      pnlToolbar.Height := ToolBarHeight
    else
      pnlToolbar.Height := 0;
end;

procedure TabcafMemoFieldEdit.ActivateFormMemo(Sender: TObject);
{
 This procedure activates the form and the memo while the
 Find or Replace dialog box is open.
 Assumes the Sender is either a TFindDialog or a TReplaceDialog.
}
var
  TargetLine, TopLine, LinesHalfHeight: Integer;
begin
  if FormStyle = fsMDIChild then
    BringWindowToTop(Application.MainForm.Handle);
  BringWindowToTop(Handle);
  {Scroll memo selection into center of view}
  with DBMemo1 do begin
    if SelStart <> 0 then begin
      LinesHalfHeight := ((Height div (- Font.Height)) + 1) div 2;
      TargetLine := Perform(EM_LINEFROMCHAR, SelStart, 0);
      TopLine := Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
      Perform(EM_LINESCROLL,0,TargetLine - TopLine - LinesHalfHeight);
    end;
    SetFocus;
  end;
  if Sender is TFindDialog then
    SetWindowPos(
      TFindDialog(Sender).Handle,
      HWND_TOP,
      0,0,0,0,
      SWP_NOACTIVATE + SWP_NOMOVE + SWP_NOSIZE)
  else if Sender is TReplaceDialog then
    SetWindowPos(
      TReplaceDialog(Sender).Handle,
      HWND_TOP,
      0,0,0,0,
      SWP_NOACTIVATE + SWP_NOMOVE + SWP_NOSIZE);
end;

procedure TabcafMemoFieldEdit.DisableUndo;
begin
  DBMemo1.Perform(EM_EMPTYUNDOBUFFER, 0, 0);
  EditUndo.Enabled := False;
  psbEditUndo.Enabled := False;
end;

procedure TabcafMemoFieldEdit.ResetEdit;
begin
  DisableUndo;
  FEdited := False;
end;

procedure TabcafMemoFieldEdit.DBMemo1Change(Sender: TObject);
begin
  if not FEdited then FEdited := True;
  EditUndo.Enabled := DBMemo1.Perform(EM_CANUNDO, 0, 0) <> 0;
  psbEditUndo.Enabled := EditUndo.Enabled;
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TabcafMemoFieldEdit.FileOpenClick(Sender: TObject);
begin
  with OpenDialog1 do
    if Execute then begin
      DBMemo1.DataSource.DataSet.Edit;
      TMemoField(DBMemo1.Field).LoadFromFile(FileName);
      FFileName := FileName;
      ResetEdit;
    end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emFileOpen);
end;

procedure TabcafMemoFieldEdit.FileSaveAsClick(Sender: TObject);
begin
  with SaveDialog1 do
    if Execute then begin
      DBMemo1.Lines.SaveToFile(FileName);
      FFileName := FileName;
      ResetEdit;
    end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emFileSaveAs);
end;

procedure TabcafMemoFieldEdit.FilePrintClick(Sender: TObject);
begin
  if PrintDialog1.Execute then
    if DBMemo1.Lines.Count > 0 then
      abcPrintStringsFormat(DBMemo1.Lines, 20, 60);
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emFilePrint);
end;

procedure TabcafMemoFieldEdit.EditUndoClick(Sender: TObject);
begin
  DBMemo1.Perform(EM_UNDO, 0, 0);
  DisableUndo;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditUndo);
end;

procedure TabcafMemoFieldEdit.EditCutClick(Sender: TObject);
begin
  DBMemo1.CutToClipBoard;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditCut);
end;

procedure TabcafMemoFieldEdit.EditCopyClick(Sender: TObject);
begin
  DBMemo1.CopyToClipboard;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditCopy);
end;

procedure TabcafMemoFieldEdit.EditPasteClick(Sender: TObject);
begin
  DBMemo1.PasteFromClipboard;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditPaste);
end;

procedure TabcafMemoFieldEdit.EditDeleteClick(Sender: TObject);
begin
  DBMemo1.ClearSelection;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditDelete);
end;

procedure TabcafMemoFieldEdit.EditSelectAllClick(Sender: TObject);
begin
  DBMemo1.SelectAll;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditSelectAll);
end;

procedure TabcafMemoFieldEdit.EditTimeDateClick(Sender: TObject);
begin
  abcStrToClip(FormatDateTime(ShortTimeFormat	, Now) + ' ' + FormatDateTime(ShortDateFormat, Now));
  DBMemo1.PasteFromClipboard;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditTimeDate);
end;

procedure TabcafMemoFieldEdit.EditWordWrapClick(Sender: TObject);
begin
  with DBMemo1 do begin
    WordWrap := not WordWrap;
    EditWordWrap.Checked := WordWrap;
    tbEditWordWrap.Down := WordWrap;
    if WordWrap then
      ScrollBars := ssVertical
    else
      ScrollBars := ssBoth;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditWordWrap);
end;

procedure TabcafMemoFieldEdit.EditTabsClick(Sender: TObject);
begin
  with DBMemo1 do begin
    WantTabs := not WantTabs;
    EditTabs.Checked := WantTabs;
    tbEditTabs.Down := WantTabs;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emEditTabs);
end;

procedure TabcafMemoFieldEdit.SearchFindClick(Sender: TObject);
begin
  with FindDialog1 do begin
    FindText := DBMemo1.SelText;
    FFindText := FindText;
    FFound := False;
    Execute;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emSearchFind);
end;

procedure TabcafMemoFieldEdit.SearchFindNextClick(Sender: TObject);
begin
  MemoFindNext;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emSearchFindNext);
end;

procedure TabcafMemoFieldEdit.SearchReplaceClick(Sender: TObject);
begin
  with ReplaceDialog1 do begin
    FindText := DBMemo1.SelText;
    FFound := False;
    Execute;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emSearchReplace);
end;

procedure TabcafMemoFieldEdit.OptionsColorClick(Sender: TObject);
begin
  with ColorDialog1 do begin
    Color := DBMemo1.Color;
    if Execute then
      DBMemo1.Color := Color;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emOptionsColor);
end;

procedure TabcafMemoFieldEdit.OptionsFontClick(Sender: TObject);
begin
  with FontDialog1 do begin
    Font := DBMemo1.Font;
    if Execute then
      DBMemo1.Font := Font;
  end;
  if Assigned(FOnPopupClick) then FOnPopupClick(Sender, emOptionsFont);
end;

procedure TabcafMemoFieldEdit.FindDialog1Find(Sender: TObject);
begin
  FMatchCase := (frMatchCase) in TFindDialog(Sender).Options;
  if FFindText <> TFindDialog(Sender).FindText then FFound := False;
  if FFound then begin
    MemoFindNext;
    if not FFound then
      MessageDlg('Cannot find "' + FFindText + '"', mtInformation, [mbOK], 0);
  end
  else begin
    FFindText := TFindDialog(Sender).FindText;
    MemoFind;
    if not FFound then
      MessageDlg('Cannot find "' + FFindText + '"', mtInformation, [mbOK], 0);
  end;
  ActivateFormMemo(Sender);
end;

procedure TabcafMemoFieldEdit.ReplaceDialog1Replace(Sender: TObject);
begin
  if FFindText <> ReplaceDialog1.FindText then FFound := False;
  FReplaceText := ReplaceDialog1.ReplaceText;
  if (frReplaceAll) in ReplaceDialog1.Options then begin
    FFindText := ReplaceDialog1.FindText;
    FReplaceText := ReplaceDialog1.ReplaceText;
    FMatchCase := (frMatchCase) in ReplaceDialog1.Options;
    MemoReplaceAll;
    DisableUndo;
  end
  else if FFound then begin
    MemoReplaceSelection;
    MemoFindNext;
  end;
  ActivateFormMemo(Sender);
end;

procedure TabcafMemoFieldEdit.MemoFind;
begin
  FStartPos := 0;
  MemoFindFromPos;
end;

procedure TabcafMemoFieldEdit.MemoFindFromPos;
var
  MemoBuffer, FindPtr, FindPos: PChar;
  MemoSize: Integer;
  SearchText: array[0..255] of char;
begin
  FFound := False;
  if FFindText = '' then
    Exit
  else
    StrPCopy(SearchText, FFindText);
  MemoSize := DBMemo1.GetTextLen;
  if FStartPos >= MemoSize then Exit;
  Inc(MemoSize);
  GetMem(MemoBuffer, MemoSize);
  try
    DBMemo1.GetTextBuf(MemoBuffer,MemoSize);
    if not FMatchCase then begin
      StrUpper(SearchText);
      StrUpper(MemoBuffer);
    end;
    FindPtr := MemoBuffer + FStartPos;
    FindPos := StrPos(FindPtr, SearchText);
    if FindPos <> nil then begin
      FFound := True;
      {highlight found text}
      with DBMemo1 do begin
        SelStart := FindPos - MemoBuffer;
        SelLength := Length(FFindText);
      end;
    end;
  finally
    FreeMem(MemoBuffer, MemoSize);
  end;
end;

procedure TabcafMemoFieldEdit.MemoFindNext;
begin
  FStartPos := DBMemo1.SelStart + DBMemo1.SelLength;
  MemoFindFromPos;
end;

procedure TabcafMemoFieldEdit.MemoReplaceSelection;
begin
  abcStrToClip(FReplaceText);
  DBMemo1.PasteFromClipboard;
end;

procedure TabcafMemoFieldEdit.MemoReplaceAll;
begin
  FStartPos := 0;
  MemoFindFromPos;
  while FFound do begin
    MemoReplaceSelection;
    MemoFindNext;
  end;
end;

procedure TabcafMemoFieldEdit.pnlToolbarResize(Sender: TObject);
begin
  if pnlToolbar.Width > FullWidth then begin
    psbScrollRight.Left := FullWidth - ScrollBtnWidth - 1;
    scrToolbar.Width := ToolbarMax;
    psbScrollLeft.Enabled := False;
    psbScrollRight.Enabled := False;
  end
  else begin
    psbScrollRight.Left := pnlToolbar.Width - ScrollBtnWidth;
    scrToolbar.Width := pnlToolbar.Width - 2 * ScrollBtnWidth + 1;
    CheckScrollButtons;
  end;
end;

procedure TabcafMemoFieldEdit.sbScrollLeftClick(Sender: TObject);
begin
  scrToolbar.ScrollBy(ToolbarBtnWidth,0);
  Dec(FScrollLeftBtn);
  CheckScrollButtons;
end;

procedure TabcafMemoFieldEdit.sbScrollRightClick(Sender: TObject);
begin
  scrToolbar.ScrollBy(-ToolbarBtnWidth,0);
  Inc(FScrollLeftBtn);
  CheckScrollButtons;
end;

procedure TabcafMemoFieldEdit.scrToolbarResize(Sender: TObject);
begin
  if FScrollLeftBtn <> 0 then begin
    scrToolbar.ScrollBy(FScrollLeftBtn * ToolbarBtnWidth, 0);
    FScrollLeftBtn := 0;
  end;
  CheckScrollButtons;
end;

procedure TabcafMemoFieldEdit.CheckScrollButtons;
begin
  if FScrollLeftBtn > 0 then
    psbScrollLeft.Enabled := True
  else
    psbScrollLeft.Enabled := False;

  if scrToolbar.Width < (ToolbarBtnCount - FScrollLeftBtn) * ToolbarBtnWidth then
    psbScrollRight.Enabled := True
  else
    psbScrollRight.Enabled := False;
end;

procedure TabcafMemoFieldEdit.FormCreate(Sender: TObject);
begin
  ResetEdit;
end;

procedure TabcafMemoFieldEdit.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TabcafMemoFieldEdit.EdgePanel1Resize(Sender: TObject);
const
  BtnSpace = 5;
var
  LastBtnLeft: Integer;
begin
  {align visible buttons}
  LastBtnLeft := EdgePanel1.Width;
  with HelpBtn do
    if Visible then begin
      Left := LastBtnLeft - Width - BtnSpace;
      LastBtnLeft := Left;
    end;
  with OKBtn do Left := LastBtnLeft - Width - BtnSpace;
end;

procedure TabcafMemoFieldEdit.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

procedure TabcafMemoFieldEdit.FormShow(Sender: TObject);
begin
  pnlToolBar.Width := 0; {force resize for buttons}
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcafMemoFieldEdit);

end.

