unit abcfdir;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

(*

Change the Caption of the following controls for localization,
using a descendant form class.

  lblDirCaptionFolders
  lblFormCaptionFolders
  lblDirCaptionDirectories
  lblFormCaptionDirectories
  lblDrives
  bbtnOK
  bbtnCancel
  bbtnHelp
  btnOK
  btnCancel
  btnHelp
  btnNetwork
  bbtnNetwork

*)

interface

uses Windows, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, SysUtils, FileCtrl, Dialogs, abcutilf, abcforms, abcfact;

type

  { TabcDirectoryDialog }

  TabcDirectoryOption = (adrCreate, adrCreatePrompt, adrExistPrompt, adrNoChangeDrive,
    adrHideDrive, adrHideNetConnect, adrNoValidate, adrPathMustExist, adrPathNotExist,
    adrShowHelp, adrUpdateHistory);

  TabcDirectoryOptions = set of TabcDirectoryOption;

  TabcDirEditStyle = (desLabel, desEdit, desCombo);

  TabcDirButtonStyle = (dbsButton, dbsBitBtn);

  TabcDirDialogStyle = (ddsBtnBottom, ddsBtnRight);

  TabcDirLabelStyle = (dlsFolder, dlsDirectory);

  TabcsfDirectoryDialog = class;

  TabcDirectoryDialog = class(TComponent)
  private
    FCtl3D: Boolean;
    FDirButtonStyle: TabcDirButtonStyle;
    FDirectory: TFileName;
    FDirDialogStyle: TabcDirDialogStyle;
    FDirEditStyle: TabcDirEditStyle;
    FDirLabelStyle: TabcDirLabelStyle;
    FHelpContext: THelpContext;
    FHistoryList: TStrings;
    FOptions: TabcDirectoryOptions;
    FTitle: string;
    function GetDrive: Char;
    procedure InitFormStyle(Form: TabcsfDirectoryDialog);
    procedure SetDirEditStyle(Value: TabcDirEditStyle);
    procedure SetHistoryList(Value: TStrings);
    procedure SetOptions(Value: TabcDirectoryOptions);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean;
    property Drive: Char read GetDrive;
  published
    property Ctl3D: Boolean read FCtl3D write FCtl3D default True;
    property DirButtonStyle: TabcDirButtonStyle read FDirButtonStyle write FDirButtonStyle
      default dbsButton;
    property DirDialogStyle: TabcDirDialogStyle read FDirDialogStyle write FDirDialogStyle
      default ddsBtnBottom;
    property Directory: TFileName read FDirectory write FDirectory;
    property DirEditStyle: TabcDirEditStyle read FDirEditStyle write SetDirEditStyle
      default desLabel;
    property DirLabelStyle: TabcDirLabelStyle read FDirLabelStyle write FDirLabelStyle
      default dlsFolder;
    property HelpContext: THelpContext read FHelpContext write FHelpContext default 0;
    property HistoryList: TStrings read FHistoryList write SetHistoryList;
    property Options: TabcDirectoryOptions read FOptions write SetOptions
      default [adrPathMustExist, adrUpdateHistory];
    property Title: string read FTitle write FTitle;
  end;

  { TabcsfDirectoryDialog }

  TabcsfDirectoryDialog = class(TabcForm)
    bbtnOK: TBitBtn;
    bbtnCancel: TBitBtn;
    bbtnHelp: TBitBtn;
    btnOK: TButton;
    btnCancel: TButton;
    btnHelp: TButton;
    btnNetwork: TButton;
    bbtnNetwork: TBitBtn;
    edDir: TEdit;
    cboDir: TComboBox;
    lblDir: TLabel;
    lblDrives: TLabel;
    DriveComboBox1: TDriveComboBox;
    DirectoryListBox1: TDirectoryListBox;
    lblDirCaptionFolders: TLabel;
    lblDirCaptionDirectories: TLabel;
    lblFormCaptionFolders: TLabel;
    lblformCaptionDirectories: TLabel;
    procedure DirectoryListBox1Change(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnNetworkClick(Sender: TObject);
  private
    { Private declarations }
    FOptions: TabcDirectoryOptions;
    FSelectedDirectory: TFileName;
    function ShowDirMessage(MsgConst: string; Directory: TFileName;
      Warning, Error: Boolean): Word;
    function ValidateDirectory(var ADir: TFileName): Boolean;
  public
    { Public declarations }
  end;

  function SelectFolder(
    var ADirectory: TFileName;
    const ADirectoryOptions: TabcDirectoryOptions;
    const AHelpCtx: LongInt): Boolean;

implementation

{$R *.DFM}

uses abcconst, abcutilw;

var
  NetworkIsConnected: Boolean;

{ SelectFolder }
{ This is a quick wrapper function around the directory dialog
  when you don't want to create your own component }

function SelectFolder(
    var ADirectory: TFileName;
    const ADirectoryOptions: TabcDirectoryOptions;
    const AHelpCtx: LongInt): Boolean;
begin
  with TabcDirectoryDialog.Create(nil) do
    try
      Directory := ADirectory;
      Options := ADirectoryOptions;
      if adrPathMustExist in ADirectoryOptions then
        DirEditStyle := desLabel
      else
        DirEditStyle := desEdit;
      HelpContext := AHelpCtx;
      Result := Execute;
      if Result then ADirectory := Directory;
    finally
      Free;
    end;
end;

{ TabcDirectoryDialog }

constructor TabcDirectoryDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCtl3D := True;
  FHistoryList := TStringList.Create;
  FOptions := [adrPathMustExist, adrUpdateHistory];
end;

destructor TabcDirectoryDialog.Destroy;
begin
  FHistoryList.Free;
  inherited Destroy;
end;

function TabcDirectoryDialog.Execute: Boolean;
var
  Form: TabcsfDirectoryDialog;
  CurIndex: Integer;
begin
  Result := False;
  Form := TabcsfDirectoryDialog(
    abcFormFactory.CreateFormOfClass(TabcsfDirectoryDialog, Self));
  with Form do
    try
      InitFormStyle(Form);
      if ShowModal = mrOK then begin
        Result := True;
        FDirectory := FSelectedDirectory;
        if (adrUpdateHistory in Options) then begin
          CurIndex := FHistoryList.IndexOf(FDirectory);
          case CurIndex of
            -1:
              FHistoryList.Insert(0, FDirectory);
            0:
              {do nothing};
          else
            FHistoryList.Move(CurIndex, 0);
          end;
        end;
      end;
    finally
      Form.Free;
    end;
end;

procedure TabcDirectoryDialog.InitFormStyle(Form: TabcsfDirectoryDialog);
const
  CBtnHSpace = 7;
  CBtnVSpace = 4;
var
  BottomBtnTop, RightBtnLeft, BtnWidth, BtnHeight, NextButtonPos: Integer;
begin
  with Form do begin
    FOptions := Self.FOptions;
    if FDirDialogStyle = ddsBtnRight then begin
      RightBtnLeft := cboDir.Width + 12;
      Width := Width + 4 + btnOk.Width;
      NextButtonPos := lblDirCaptionFolders.Top;
      BtnHeight := btnOk.Height;
      with btnOk do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
      with bbtnOk do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
      NextButtonPos := NextButtonPos + BtnHeight + CBtnVSpace;
      with btnCancel do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
      with bbtnCancel do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
      NextButtonPos := NextButtonPos + BtnHeight + 2 * CBtnVSpace;
      if (adrShowHelp in FOptions) then
      begin
        with btnHelp do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
        with bbtnHelp do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
        NextButtonPos := NextButtonPos + BtnHeight + 2 * CBtnVSpace;
        if FDirButtonStyle = dbsBitBtn then
          bbtnHelp.Visible := True
        else
          btnHelp.Visible := True;
      end;
      if not (adrHideNetConnect in FOptions) then begin
        with btnNetwork do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
        with bbtnNetwork do SetBounds(RightBtnLeft,NextButtonPos,Width,Height);
        if FDirButtonStyle = dbsBitBtn then
        begin
          bbtnNetwork.Visible := True;
          bbtnNetwork.Enabled := NetworkIsConnected;
        end
        else
        begin
          btnNetwork.Visible := True;
          btnNetwork.Enabled := NetworkIsConnected;
        end;
      end;
      ClientWidth := btnOk.Left + btnOk.Width + 4;
      ClientHeight :=  DriveComboBox1.Top + DriveComboBox1.Height + 4;
    end
    else begin
      BtnWidth := btnOk.Width;
      NextButtonPos := DriveComboBox1.Left + DriveComboBox1.Width - BtnWidth;
      BottomBtnTop := btnOk.Top;
      if not (adrHideNetConnect in FOptions) then begin
        with btnNetwork do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
        with bbtnNetwork do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
        NextButtonPos := NextButtonPos - BtnWidth - CBtnHSpace;
        if FDirButtonStyle = dbsBitBtn then
        begin
          bbtnNetwork.Visible := True;
          bbtnNetwork.Enabled := NetworkIsConnected;
        end
        else
        begin
          btnNetwork.Visible := True;
          btnNetwork.Enabled := NetworkIsConnected;
        end;
      end;
      if (adrShowHelp in FOptions) then begin
        with btnHelp do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
        with bbtnHelp do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
        NextButtonPos := NextButtonPos - BtnWidth - CBtnHSpace;
        if FDirButtonStyle = dbsBitBtn then
          bbtnHelp.Visible := True
        else
          btnHelp.Visible := True;
      end;
      with btnCancel do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
      with bbtnCancel do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
      NextButtonPos := NextButtonPos - BtnWidth - CBtnHSpace;
      with btnOk do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
      with bbtnOk do SetBounds(NextButtonPos,BottomBtnTop,Width,Height);
    end;

    if FDirButtonStyle = dbsBitBtn then begin
      bbtnOK.Visible := True;
      bbtnCancel.Visible := True;
    end
    else begin
      btnOK.Visible := True;
      btnCancel.Visible := True;
    end;

    if FDirectory <> '' then
      try
        DirectoryListBox1.Directory := FDirectory;
      except
        on E: Exception do MessageDlg(E.Message,mtError,[mbOK],0);
      end;

    case FDirEditStyle of
      desLabel: ;
      desEdit:
        begin
          lblDir.Visible := False;
          edDir.Visible := True;
          edDir.Text := DirectoryListBox1.Directory;
        end;
      desCombo:
        begin
          lblDir.Visible := False;
          cboDir.Visible := True;
          cboDir.Items.Assign(FHistoryList);
          cboDir.Text := DirectoryListBox1.Directory;
        end;
    end;

    if FDirLabelStyle = dlsFolder then
    begin
      lblDirCaptionFolders.Visible := True;
      lblDirCaptionDirectories.Visible := False;
      if FTitle = '' then
        Caption := lblFormCaptionFolders.Caption
      else
        Caption := FTitle;
    end
    else
    begin
      lblDirCaptionDirectories.Visible := True;
      lblDirCaptionFolders.Visible := False;
      if FTitle = '' then
        Caption := lblFormCaptionDirectories.Caption
      else
        Caption := FTitle;
    end;

    if (adrNoChangeDrive in FOptions) then begin
      lblDrives.Enabled := False;
      DriveComboBox1.Enabled := False;
      if btnNetwork.Visible then
        btnNetwork.Enabled := False
      else if bbtnNetwork.Visible then
        bbtnNetwork.Enabled := False;
    end;

    if (adrHideDrive in FOptions) then begin
      lblDrives.Visible := False;
      DriveComboBox1.Visible := False;
      ClientHeight := lblDrives.Top;
    end;

    Ctl3D := Self.FCtl3D;
    HelpContext := Self.FHelpContext;
  end;
end;

function TabcDirectoryDialog.GetDrive: Char;
begin
  if FDirectory <> '' then
    Result := FDirectory[1]
  else
    Result := #0;
end;

procedure TabcDirectoryDialog.SetDirEditStyle(Value: TabcDirEditStyle);
{
if the edit style is a label, then the Options must include doPathMustExist.
}
begin
  if Value <> FDirEditStyle then begin
    FDirEditStyle := Value;
    if (Value = desLabel) then Options := Options + [adrPathMustExist];
  end;
end;

procedure TabcDirectoryDialog.SetHistoryList(Value: TStrings);
begin
  if Value <> FHistoryList then FHistoryList.Assign(Value);
end;

procedure TabcDirectoryDialog.SetOptions(Value: TabcDirectoryOptions);
{
Check/change option vals.
adrPathMustExist and (adrCreate or adrCreatePrompt or adrPathNotExist) are mutually exclusive
adrPathNotExist & adrExistPrompt are mutually exclusive
adrPathNotExist needs an edit style to enter the value (edit or combo)
}
begin
  if Value <> FOptions then begin
    if (adrExistPrompt in Value) and not (adrExistPrompt in FOptions) then
      if (adrPathNotExist in Value) then Value := Value - [adrPathNotExist];
    if (adrPathMustExist in Value) and not (adrPathMustExist in FOptions) then
      Value := Value - [adrCreate, adrCreatePrompt, adrPathNotExist]
    else if (adrCreate in Value) or
            (adrCreatePrompt in Value) or
            (adrPathNotExist in Value) then begin
      Value := Value - [adrPathMustExist];
      if (adrPathNotExist in Value) then begin
        Value := Value - [adrExistPrompt];
        if (FDirEditStyle = desLabel) then begin
          FDirEditStyle := desEdit;
        end;
      end;
    end;
    if (adrHideDrive in Value) then Value := Value + [adrHideNetConnect];
    FOptions := Value;
  end;
end;

{ TabcsfDirectoryDialog }

function TabcsfDirectoryDialog.ValidateDirectory(var ADir: TFileName): Boolean;
{This function validates directory names for invalid chars
 and expands any .\ or ..\ in path into a fully qualified name.
 It does not test for directory existence, but does test for drive existence.
}
var
  TempPath: TFileName;
  CurDir, sTemp, sFile: string;
  bExpand, bDriveExists: Boolean;
  I, nPos: integer;
  chDrive: Char;

  function BacktoParent: Boolean;
  {steps CurDir up to parent directory}
  var
    Ch: Char;
  begin
    Result := False;
    if Length(CurDir) > 3 then begin
      repeat
        CurDir := Copy(CurDir, 1, Length(CurDir) - 1);
        Ch := CurDir[Length(CurDir)];
      until Ch = '\';
      if Length(CurDir) > 3 then begin
        CurDir := Copy(CurDir, 1, Length(CurDir) - 1);
        Result := True;
      end;
    end;
  end;

  function ValidFilename(sFile: string): Boolean;
  {uses Win95 definition of valid filename}
  const
    InvalidFileCharsWin32: set of char =
      ['\','/',':','*','?','"','<','>','|'];
  var
    J: integer;
  begin
    Result := False;
    {check length ok}
    if (sFile = '') or (Length(sFile) > 255) then Exit;
    for J := 1 to Length(sFile) do
      if sFile[J] in InvalidFileCharsWin32 then Exit;
    Result := True;
  end;

begin
  Result := False;
  TempPath := ADir;

  TempPath := abcReplaceAll('/', '\', TempPath, True, False);

  {expand any path containing .\ or ..\}
  GetDir(0, CurDir);
  bExpand := False;
  if Pos('.\', TempPath) = 1 then begin
    bExpand := True;
    TempPath := Copy(TempPath, 3, Length(TempPath) - 2);
  end;

  while Pos('..\', TempPath) = 1 do begin
    bExpand := True;
    if BacktoParent then
      TempPath := Copy(TempPath, 4, Length(TempPath) - 3)
    else begin
      ShowDirMessage(sabcErrInvalidFileName, ADir, False, True);
      Exit;
    end;
  end;
  if bExpand then
    if TempPath = '' then
      TempPath := CurDir
    else
      if Length(CurDir) > 3 then
        TempPath := CurDir + '\' + TempPath
      else
        TempPath := CurDir + TempPath;

  if TempPath[1] <> CurDir[1] then begin
    {check drive}
    bDriveExists := False;
    chDrive := TempPath[1];
    with DriveComboBox1 do
      for I := 0 to Items.Count-1 do
        if Items.Strings[I][1] = chDrive then begin
          bDriveExists := True;
          Break;
        end;
    if not bDriveExists then begin
      ShowDirMessage(sabcErrInvalidDrive, ADir, False, True);
      Exit;
    end;
  end;

  if copy(TempPath,2,2) <> ':\' then begin
    ShowDirMessage(sabcErrInvalidDrive, ADir, False, True);
    Exit;
  end;

  {parse directories}
  sTemp := Copy(TempPath,4,Length(TempPath)-3);

  while sTemp <> '' do begin
    nPos := Pos('\', sTemp);
    if nPos <> 0 then begin
      sFile := Copy(sTemp,1,nPos - 1);
      sTemp := Copy(sTemp,nPos+1,Length(sTemp));
    end
    else begin
      sFile := sTemp;
      sTemp := '';
    end;
    if not ValidFileName(sFile) then begin
      ShowDirMessage(sabcErrInvalidFileName, ADir, False, True);
      Exit;
    end;
  end;

  if bExpand then ADir := TempPath;

  Result := True;
end;

function TabcsfDirectoryDialog.ShowDirMessage(MsgConst: string; Directory: TFileName;
  Warning, Error: Boolean): Word;
var
  sMsg: string;
  Flags: Word;
begin
  sMsg := Format(MsgConst, [Directory, #13#10, #13#10]);
  if bbtnOk.Visible then
    {show Delphi style message}
    if Error then
      Result := MessageDlg(sMsg, mtError, [mbOK], 0)
    else if Warning then
      Result := MessageDlg(sMsg, mtWarning, [mbOK], 0)
    else
      Result := MessageDlg(sMsg, mtConfirmation, [mbYes, mbNo], 0)
  else begin
    {show Win API style message}
    if Error then
      Flags := MB_ICONSTOP + MB_OK
    else if Warning then
      Flags := MB_ICONEXCLAMATION + MB_OK
    else
      Flags := MB_ICONQUESTION + MB_YESNO;
    Result := Application.MessageBox(pChar(sMsg), pChar(Caption), Flags);
  end;
end;

procedure TabcsfDirectoryDialog.DirectoryListBox1Change(Sender: TObject);
begin
  if edDir.Visible then
    edDir.Text := DirectoryListBox1.Directory
  else if cboDir.Visible then
    cboDir.Text := DirectoryListBox1.Directory;
end;

procedure TabcsfDirectoryDialog.btnHelpClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

procedure TabcsfDirectoryDialog.btnOKClick(Sender: TObject);
var
  bDirExists: Boolean;
begin
  if lblDir.Visible then
    FSelectedDirectory := DirectoryListBox1.Directory
  else if edDir.Visible then
    FSelectedDirectory := edDir.Text
  else
    FSelectedDirectory := cboDir.Text;

  if FSelectedDirectory = '' then Exit;

  {validate}

  if not (adrNoValidate in FOptions) then
    {check for valid chars and expand .\ or ..\ to full path}
    if not ValidateDirectory(FSelectedDirectory) then Exit;

  bDirExists := DirectoryExists(FSelectedDirectory);

  if bDirExists then begin
    if (adrPathNotExist in FOptions) then begin
      ShowDirMessage(sabcErrPathNotExist, FSelectedDirectory, True, False);
      Exit;
    end;
    if (adrExistPrompt in FOptions) then
      if ShowDirMessage(sabcConfirmPathExists, FSelectedDirectory, False, False) = mrNo then
        Exit;
  end
  else
  begin
    if (adrPathMustExist in FOptions) then
    begin
      ShowDirMessage(sabcErrPathMustExist, FSelectedDirectory, True, False);
      Exit;
    end;
    if (adrCreatePrompt in FOptions) then
      if ShowDirMessage(sabcConfirmPathCreate, FSelectedDirectory, False, False) = mrNo then
        Exit;
    if (adrCreate in FOptions) then
      ForceDirectories(FSelectedDirectory);
  end;
  ModalResult := mrOK;
end;

procedure TabcsfDirectoryDialog.btnNetworkClick(Sender: TObject);
begin
  WNetConnectionDialog(Handle, RESOURCETYPE_DISK);
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcsfDirectoryDialog);
  NetworkIsConnected := abcNetworkIsConnected;

end.
