unit abcffind;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, ExtCtrls, abcnpedt, abcforms, abcfact;

type
  TabcsfDBFindReplace = class(TabcSystemForm)
    lblFind: TLabel;
    lblReplace: TLabel;
    edtFind: TEdit;
    edtReplace: TEdit;
    chkMatchCase: TCheckBox;
    chkMatchWholeField: TCheckBox;
    chkSearchAllFields: TCheckBox;
    rgDirection: TRadioGroup;
    btnFindNext: TButton;
    btnClose: TButton;
    btnReplace: TButton;
    btnReplaceAll: TButton;
    btnHelp: TButton;
    lblSearching: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCloseClick(Sender: TObject);
    procedure btnReplaceClick(Sender: TObject);
    procedure btnFindNextClick(Sender: TObject);
    procedure btnReplaceAllClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure edtFindChange(Sender: TObject);
    procedure edtReplaceChange(Sender: TObject);
    procedure chkMatchCaseClick(Sender: TObject);
    procedure chkMatchWholeFieldClick(Sender: TObject);
    procedure chkSearchAllFieldsClick(Sender: TObject);
    procedure rgDirectionClick(Sender: TObject);
  private
    { Private declarations }
    FButton: TabcnpFindReplace;
  public
    { Public declarations }
    constructor CreateFindForm(AOwner: TabcnpFindReplace); virtual;
  end;

  TabcsfDBFindReplaceClass = class of TabcsfDBFindReplace;

implementation

uses abcutild;

{$R *.DFM}

constructor TabcsfDBFindReplace.CreateFindForm(AOwner: TabcnpFindReplace);
var
  Pos: TPoint;
begin
  inherited Create(AOwner);
  FButton := AOwner;
  with FButton do begin
    edtFind.Text := FindText;
    edtReplace.Text := ReplaceText;
    btnFindNext.Enabled := edtFind.Text <> '';
    chkMatchCase.Checked := nfrMatchCase in OptionsFind;
    chkMatchCase.Enabled := not (nfrDisableMatchCase in OptionsFind);
    chkMatchCase.Visible := not (nfrHideMatchCase in OptionsFind);
    chkMatchWholeField.Checked := nfrWholeField in OptionsFind;
    chkMatchWholeField.Enabled := not (nfrDisableWholeField in OptionsFind);
    chkMatchWholeField.Visible := not (nfrHideWholeField in OptionsFind);
    chkSearchAllFields.Checked := nfrAllFields in OptionsFind;
    chkSearchAllFields.Enabled := not (nfrDisableAllFields in OptionsFind);
    chkSearchAllFields.Visible := not (nfrHideAllFields in OptionsFind);
    rgDirection.ItemIndex := Ord(nfrDown in OptionsFind);
    rgDirection.Enabled := not (nfrDisableUpDown in OptionsFind);
    rgDirection.Visible := not (nfrHideUpDown in OptionsFind);
    Self.HelpContext := HelpContext;
    if nfrShowHelp in OptionsFind then begin
      btnHelp.Visible := True;
    end;
    if Action = nfaFind then begin
      btnReplace.Visible := (nfrShowReplace in OptionsReplace) and
        not abcDataSetReadOnly(DataSet) and
        ((nfrAllFields in OptionsFind) or Field.CanModify);
    end
    else begin
      Caption := 'Replace';
      lblReplace.Visible := True;
      edtReplace.Visible := True;
      btnReplace.Caption := '&Replace';
      btnReplaceAll.Visible := nfrShowReplaceAll in OptionsReplace;
    end;
    lblSearching.Color := clActiveCaption;
    lblSearching.Font.Color := clCaptionText;
    Pos := ClientToScreen(Point(0, Height));
    if (Pos.Y + Self.Height) > Screen.Height then
      Pos.Y := Pos.Y - Height - Self.Height;
    if (Pos.X + Self.Width) > Screen.Width then
      Pos.X := Pos.X - Self.Width;
    Self.SetBounds(Pos.X, Pos.Y, Self.Width, Self.Height);
  end;
end;

procedure TabcsfDBFindReplace.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TabcsfDBFindReplace.btnCloseClick(Sender: TObject);
begin
  FButton.Close;
end;

procedure TabcsfDBFindReplace.btnReplaceClick(Sender: TObject);
begin
  if btnReplace.Caption = '&Replace' then
    FButton.Replace
  else begin
    Caption := 'Replace';
    lblReplace.Visible := True;
    edtReplace.Visible := True;
    btnReplaceAll.Visible := nfrShowReplaceAll in FButton.OptionsReplace;
    btnReplace.Caption := '&Replace';
    if edtFind.Text = '' then
      edtFind.SetFocus
    else
      edtReplace.SetFocus;
  end;
end;

procedure TabcsfDBFindReplace.btnFindNextClick(Sender: TObject);
begin
  FButton.Find;
end;

procedure TabcsfDBFindReplace.btnReplaceAllClick(Sender: TObject);
begin
  FButton.ReplaceAll;
end;

procedure TabcsfDBFindReplace.btnHelpClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

procedure TabcsfDBFindReplace.edtFindChange(Sender: TObject);
begin
  FButton.FindText := edtFind.Text;
  btnFindNext.Enabled := edtFind.Text <> '';
end;

procedure TabcsfDBFindReplace.edtReplaceChange(Sender: TObject);
begin
  FButton.ReplaceText := edtReplace.Text;
end;

procedure TabcsfDBFindReplace.chkMatchCaseClick(Sender: TObject);
begin
  with FButton do
    if chkMatchCase.Checked then
      OptionsFind := OptionsFind + [nfrMatchCase]
    else
      OptionsFind := OptionsFind - [nfrMatchCase];
end;

procedure TabcsfDBFindReplace.chkMatchWholeFieldClick(Sender: TObject);
begin
  with FButton do
    if chkMatchWholeField.Checked then
      OptionsFind := OptionsFind + [nfrWholeField]
    else
      OptionsFind := OptionsFind - [nfrWholeField];
end;

procedure TabcsfDBFindReplace.chkSearchAllFieldsClick(Sender: TObject);
begin
  with FButton do
    if chkSearchAllFields.Checked then
      OptionsFind := OptionsFind + [nfrAllFields]
    else
      OptionsFind := OptionsFind - [nfrAllFields];
end;

procedure TabcsfDBFindReplace.rgDirectionClick(Sender: TObject);
begin
  with FButton do
    if rgDirection.ItemIndex = 1 then
      OptionsFind := OptionsFind + [nfrDown]
    else
      OptionsFind := OptionsFind - [nfrDown];
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcsfDBFindReplace);
  
end.
