unit abcffltr;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses SysUtils, Windows, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Dialogs, DB, abcfiltr, abcforms, abcfact, abcda;

type
  TabcafFilter = class(TabcApplicationForm)
    Panel1: TPanel;
    SaveDialog: TSaveDialog;
    CloseBtn: TBitBtn;
    HelpBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    Panel3: TPanel;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    cboField: TComboBox;
    Label2: TLabel;
    cboOperator: TComboBox;
    lblValue: TLabel;
    edValue: TEdit;
    GroupBox2: TGroupBox;
    memExpr: TMemo;
    Label3: TLabel;
    chkCaseInsens: TCheckBox;
    chkPartial: TCheckBox;
    btnAddExpr: TButton;
    btnAndCond: TButton;
    btnOrCond: TButton;
    btnLoad: TBitBtn;
    btnSave: TBitBtn;
    btnPrint: TBitBtn;
    btnClearExpr: TBitBtn;
    btnActivate: TBitBtn;
    btnDeActivate: TBitBtn;
    btnSetFilter: TBitBtn;
    procedure Panel1Resize(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure memExprChange(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure cboOperatorChange(Sender: TObject);
    procedure btnAddExprClick(Sender: TObject);
    procedure btnAndCondClick(Sender: TObject);
    procedure btnOrCondClick(Sender: TObject);
    procedure btnClearExprClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure btnActivateClick(Sender: TObject);
    procedure btnDeActivateClick(Sender: TObject);
    procedure btnSetFilterClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure chkCaseInsensClick(Sender: TObject);
    procedure chkPartialClick(Sender: TObject);
  private
    { Private declarations }
    FFilter: TabcDBFilter;
    FFilterChanged: Boolean;
    procedure EnableFilterButtons;
    function ValidCondition(var Condition: string): Boolean;
    function FieldFromString(const S: string): TField;
  public
    { Public declarations }
  end;

procedure ExecabcafFilter(AOwner: TComponent; AFilter: TabcDBFilter;
  Modal: Boolean; ShowHelpBtn: Boolean; AHelpContext: Word);

implementation

{$R *.DFM}

uses Printers;

const
  SNoEdit = 'Cannot edit filter.';
  SNoFilter = '  Filter is nil';
  SNoDataSource = ' DataSource unavailable';
  SNoDataSet = ' DataSet unavailable';
  SDataSetInactive = ' DataSet inactive';
  SNoFields = ' DataSet has no fields';
  SNoFieldsType = ' DataSet has no fields that can be filtered';

  SFieldRequired = 'Field required for Condition.';
  SOperatorRequired = 'Operator required for Condition.';
  SValueRequired = 'Value required for Condition.';
  SBooleanFieldReqd = 'Operator cannot be used on this field.';
  SExprTooLarge = 'Unable to add Condition.  Expression cannot exceed 255 characters.';

procedure ExecabcafFilter(AOwner: TComponent; AFilter: TabcDBFilter;
  Modal: Boolean; ShowHelpBtn: Boolean; AHelpContext: Word);
var
  Form: TabcafFilter;
  I: Integer;
begin
  {validate Filter is editable}
  if (AFilter = nil) then
    raise EabcInvalidFilterOperation.Create(SNoEdit + SNoFilter);
  if (AFilter.DataSource = nil) then
    raise EabcInvalidFilterOperation.Create(SNoEdit + SNoDataSource);
  if (AFilter.DataSource.DataSet = nil) or (AFilter.Adapter = nil) then
    raise EabcInvalidFilterOperation.Create(SNoEdit + SNoDataSet);
  if (not AFilter.DataSource.DataSet.Active) then
    raise EabcInvalidFilterOperation.Create(SNoEdit + SDataSetInactive);
  if (AFilter.DataSource.DataSet.FieldCount = 0) then
    raise EabcInvalidFilterOperation.Create(SNoEdit + SNoFields);

  Form := TabcafFilter(
    abcFormFactory.CreateFormOfClass(TabcafFilter, AOwner));

  with Form, AFilter do begin
    Caption := 'Edit Filter on ' + Adapter.Description;
    with DataSource.DataSet do
        for I := 0 to FieldCount - 1 do
          if (not Fields[I].Calculated) and (Fields[I].DataType in [ftString..ftDateTime]) then
              cboField.Items.Add(Fields[I].DisplayLabel);
    if (cboField.Items.Count = 0) then
      raise EabcInvalidFilterOperation.Create(SNoEdit + SNoFieldsType);
    HelpBtn.Visible := ShowHelpBtn;
    HelpContext := AHelpContext;
    chkCaseInsens.Checked := (foCaseInsensitive in FilterOptions);
    chkPartial.Checked := not (foNoPartialCompare in FilterOptions);
    memExpr.Lines.Clear;
    if Filter <> '' then memExpr.Lines.Add(Filter);
    FFilter := AFilter;
    FFilterChanged := False;
    EnableFilterButtons;
    memExpr.MaxLength := 0;
    if Modal then
      ShowModal
    else
      Show;
  end;
end;

function TabcafFilter.ValidCondition(var Condition: string): Boolean;
var
  Field: TField;
  StrField, StrOp, StrVal: string;

  function LTrim(S: string): string;
  begin
    if (S[1] = ' ') then
      Result := LTrim(Copy(S, 2, Length(S)))
    else
      Result := S;
  end;

  function RTrim(S: string): string;
  begin
    if (S[Length(S)] = ' ') then
      Result := RTrim(Copy(S, 1, Length(S) - 1))
    else
      Result := S;
  end;

  function LRTrim(S: string): string;
  begin
    Result := LTrim(RTrim(S));
  end;

begin
  {validate user input}
  Result := False;
  Condition := '';
  if (cboField.ItemIndex = -1) then begin
    MessageDlg(SFieldRequired, mtWarning, [mbOK], 0);
    cboField.SetFocus;
    Exit;
  end;
  if (cboOperator.ItemIndex = -1) then begin
    MessageDlg(SOperatorRequired, mtWarning, [mbOK], 0);
    cboOperator.SetFocus;
    Exit;
  end;
  Field := FieldFromString(cboField.Items.Strings[cboField.ItemIndex]);
  if (cboOperator.ItemIndex > 3) then begin
    if (edValue.Text = '') then begin
      MessageDlg(SValueRequired, mtWarning, [mbOK], 0);
      edValue.SetFocus;
      Exit;
    end;
    {validate data type of value to match field type}
    StrVal := LRTrim(edValue.Text);
    try
      case Field.DataType of
        ftString:
          StrVal := '''' + StrVal + '''';
        ftSmallInt..ftWord:
          StrToInt(StrVal);
        ftFloat..ftBCD:
          StrToFloat(StrVal);
        ftDate:
          begin
            StrToDate(StrVal);
            StrVal := '''' + StrVal + '''';
          end;
        ftTime:
          begin
            StrToTime(StrVal);
            StrVal := '''' + StrVal + '''';
          end;
        ftDateTime:
          begin
            StrToDateTime(StrVal);
            StrVal := '''' + StrVal + '''';
          end;
      end;
    except
      on Exception do begin
        edValue.SetFocus;
        raise;
      end;
    end;
  end
  else if (cboOperator.ItemIndex > 1) then
    {Boolean field required}
    if (Field.DataType <> ftBoolean) then begin
      MessageDlg(SBooleanFieldReqd, mtWarning, [mbOK], 0);
      cboField.SetFocus;
      Exit;
    end;
  {Build condition}
  StrField := '[' + Field.FieldName + ']';
  case cboOperator.ItemIndex of
    0: {Field Is Blank}
      begin
        StrOp := ' = ';
        StrVal := 'NULL';
      end;
    1: {Field Is Not Blank}
      begin
        StrOp := ' <> ';
        StrVal := 'NULL';
      end;
    2: {Field Is True}
      begin
        StrOp := '';
        StrVal := '';
      end;
    3: {Field Is Not True}
      begin
        StrField := 'NOT ' + StrField;
        StrOp := '';
        StrVal := '';
      end;
    4: {Equal To}
      begin
        StrOp := ' = ';
        if chkPartial.Checked and
          (Pos('*', StrVal) = 0) then
          StrVal := Copy(StrVal, 1, Length(StrVal)-1) + '*''';
      end;
    5: {Not Equal To}
      begin
        StrOp := ' <> ';
      end;
    6: {Less Than}
      begin
        StrOp := ' < ';
      end;
    7: {Less Than or Equal To}
      begin
        StrOp := ' <= ';
      end;
    8: {Greater Than}
      begin
        StrOp := ' > ';
      end;
    9: {Greater Than or Equal To}
      begin
        StrOp := ' >= ';
      end;
  end;
  Condition := '( ' + StrField + StrOp + StrVal + ' ) ';
  Result := True;
end;

function TabcafFilter.FieldFromString(const S: string): TField;
var
  I: Integer;
begin
  Result := nil;
  with FFilter.DataSource.DataSet do
    for I := 0 to FieldCount - 1 do
      if Fields[I].DisplayLabel = S then begin
        Result := Fields[I];
        Break;
      end;
end;

procedure PrintStrings(Strings: TStrings);
{
Prints a TStrings object to the default printer
}
var
  Py: integer;
  PLines: integer;
  I: integer;

const
  LinesPerPage = 60;
  procedure CheckPageEnd;
  begin
    Inc(PLines);
    Inc(Py,100);
    if PLines > LinesPerPage then begin
      Printer.NewPage;
      Py := 100;
      PLines := 1;
    end;
  end;
begin
  Screen.Cursor := crHourGlass;
  with Printer do
    try
      BeginDoc;
      PLines := 0;
      Py := 0;
      for I := 0 to Strings.Count - 1 do begin
        CheckPageEnd;
        Canvas.TextOut(20,Py,Strings[I]);
      end;
      EndDoc;
    finally
      Screen.Cursor := crDefault;
    end;
end;

procedure TabcafFilter.Panel1Resize(Sender: TObject);
const
  Offset = 8;
var
  LastPos: Integer;
begin
  LastPos := Panel1.Width - Offset;
  with HelpBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with CloseBtn do
    if Visible then
      Left := LastPos - Width;
end;

procedure TabcafFilter.btnSaveClick(Sender: TObject);
begin
  with SaveDialog do
    if Execute then begin
      memExpr.Lines.SaveToFile(FileName);
      OpenDialog.FileName := FileName;
    end;
end;

procedure TabcafFilter.btnPrintClick(Sender: TObject);
var
  PrintList: TStringList;
  StrOptions: string;
begin
  PrintList := TStringList.Create;
  try
    with PrintList do begin
      Add(UpperCase(Caption));
      Add('');
      Add('EXPRESSION:');
      Add('');
      AddStrings(memExpr.Lines);
      Add('');
      Add('OPTIONS:');
      Add('');
      if chkCaseInsens.Checked then
        StrOptions := 'Case Insensitive; '
      else
        StrOptions := 'Case Sensitive; ';
      if chkPartial.Checked then
        StrOptions := StrOptions + 'Partial Compare.'
      else
        StrOptions := StrOptions + 'No Partial Compare.';
      Add(StrOptions);
    end;
    PrintStrings(PrintList);
  finally
    PrintList.Free;
  end;
end;

procedure TabcafFilter.memExprChange(Sender: TObject);
begin
  FFilterChanged := True;
end;

procedure TabcafFilter.btnLoadClick(Sender: TObject);
begin
  with OpenDialog do
    if Execute then begin
      memExpr.Lines.LoadFromFile(FileName);
      SaveDialog.FileName := FileName;
    end;
end;

procedure TabcafFilter.cboOperatorChange(Sender: TObject);
begin
  if (cboOperator.ItemIndex > -1) and
     (cboOperator.ItemIndex < 4) then begin
    lblValue.Enabled := False;
    edValue.Enabled := False;
  end
  else begin
    lblValue.Enabled := True;
    edValue.Enabled := True;
  end;
end;

procedure TabcafFilter.btnAddExprClick(Sender: TObject);
var
  Condition: string;
begin
  if ValidCondition(Condition) then begin
    memExpr.Text := Condition;
  end;
end;

procedure TabcafFilter.btnAndCondClick(Sender: TObject);
var
  Condition: string;
begin
  if ValidCondition(Condition) then begin
    if memExpr.Text = '' then
      memExpr.Text := Condition
    else
      with memExpr.Lines do begin
        Strings[Count - 1] := Strings[Count - 1] + 'AND ';
        memExpr.Lines.Add(Condition);
      end;
  end;
end;

procedure TabcafFilter.btnOrCondClick(Sender: TObject);
var
  Condition: string;
begin
  if ValidCondition(Condition) then begin
    if memExpr.Text = '' then
      memExpr.Text := Condition
    else
      with memExpr.Lines do begin
        Strings[Count - 1] := Strings[Count - 1] + 'OR ';
        memExpr.Lines.Add(Condition);
      end;
  end;
end;

procedure TabcafFilter.btnClearExprClick(Sender: TObject);
begin
  memExpr.Lines.Clear;
end;

procedure TabcafFilter.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

procedure TabcafFilter.btnActivateClick(Sender: TObject);
begin
  if (FFilter.Filter = '') or FFilterChanged then
    btnSetFilterClick(Self);
  if FFilter.Filter <> '' then begin
    FFilter.Active := True;
    EnableFilterButtons;
  end;
end;

procedure TabcafFilter.EnableFilterButtons;
begin
  btnActivate.Enabled := (FFilter.Active = False);
  btnDeActivate.Enabled := (FFilter.Active = True);
end;

procedure TabcafFilter.btnDeActivateClick(Sender: TObject);
begin
  FFilter.Active := False;
  EnableFilterButtons;
end;

procedure TabcafFilter.btnSetFilterClick(Sender: TObject);
begin
  if Length(memExpr.Text) = 0 then
    btnAddExprClick(Self);
  if Length(memExpr.Text) <> 0 then
    with FFilter do begin
      if chkCaseInsens.Checked then
        FilterOptions := FilterOptions + [foCaseInsensitive]
      else
        FilterOptions := FilterOptions - [foCaseInsensitive];
      if chkPartial.Checked then
        FilterOptions := FilterOptions - [foNoPartialCompare]
      else
        FilterOptions := FilterOptions + [foNoPartialCompare];
      Filter := memExpr.Text;
      FFilterChanged := False;
    end;
  EnableFilterButtons;
end;

procedure TabcafFilter.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TabcafFilter.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TabcafFilter.chkCaseInsensClick(Sender: TObject);
begin
  FFilterChanged := True;
end;

procedure TabcafFilter.chkPartialClick(Sender: TObject);
begin
  FFilterChanged := True;
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcafFilter);

end.
