unit abcffndk;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses SysUtils, Windows, Classes, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Grids, DB, abcforms, abcfact, abcda
{$IFDEF DELPHI6}
  , Variants
{$ENDIF}    
  ;

type

  EabcFindKeyDlgException = class(Exception);

  TabcafFindKey = class(TabcApplicationForm)
    Panel1: TPanel;
    OKBtn: TBitBtn;
    Panel2: TPanel;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    KeyGrid: TStringGrid;
    chkNearest: TCheckBox;
    procedure DoFindKey;
    procedure Panel1Resize(Sender: TObject);
    procedure Panel2Resize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure SetDataset(Value: TDataset);
    procedure SetAdapter(Value: TabcDatasetAdapter);
    procedure KeyGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure HelpBtnClick(Sender: TObject);
  private
    FDataset: TDataset;
    FAdapter: TabcDatasetAdapter;
  public
    destructor Destroy; override;
  end;

var
  abcafFindKey: TabcafFindKey;

procedure ExecabcafFindKey(AOwner: TComponent; ADataset: TDataset;
  AnAdapter: TabcDatasetAdapter; ShowHelpBtn: Boolean; Context: THelpContext);

implementation

uses abcconst, abcutilf;

{$R *.DFM}

const
  SIndexField = 'Index Field';
  SValHeading = 'Key Value';
  SFormCaption = 'Find Key in ';

procedure ExecabcafFindKey(AOwner: TComponent; ADataset: TDataset;
  AnAdapter: TabcDatasetAdapter; ShowHelpBtn: Boolean; Context: THelpContext);
var
  Dialog: TabcafFindKey;
  DlgResult: TModalResult;
begin
  if (ADataset = nil) or (AnAdapter = nil) or not AnAdapter.CanFindKey then
    raise EabcFindKeyDlgException.Create(sabcErrTableRequired)
  else if not ADataset.Active then
    raise EabcFindKeyDlgException.Create(sabcErrTableInactive)
  else if (AnAdapter.IndexFieldCount < 1) then
    raise EabcFindKeyDlgException.Create(sabcErrNoIndexFields)
  else
  begin
    Dialog := TabcafFindKey(
      abcFormFactory.CreateFormOfClass(TabcafFindKey, AOwner));
    try
      with Dialog do begin
        SetDataset(ADataset);
        SetAdapter(AnAdapter);
        HelpBtn.Visible := ShowHelpBtn;
        HelpContext := Context;
        DlgResult := ShowModal;
        if (DlgResult = mrOk) then
          DoFindKey;
      end;
    finally
      Dialog.Free;
    end;
  end;
end;

destructor TabcafFindKey.Destroy;
begin
  FDataset := nil;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcafFindKey.DoFindKey;
var
  i, len: integer;
  Args: variant;
begin
  with FAdapter do begin
    len := IndexFieldCount;
    Args := VarArrayCreate([0, len-1], varOleStr);
    for i := 0 to len - 1 do
      Args[i] := KeyGrid.Cells[1,I+1];
    if chkNearest.Checked and CanFindNearest then
      FindNearest(Args)
    else
      FindKey(Args);
  end;
end;

procedure TabcafFindKey.SetDataset(Value: TDataset);
begin
  FDataset := Value;
end;

procedure TabcafFindKey.SetAdapter(Value: TabcDatasetAdapter);
var
  I: integer;
begin
  {Setup field names and search values}
  FAdapter := Value;
  with KeyGrid, Value do begin
    Caption := SFormCaption + Description;
    RowCount := 1;
    RowCount := IndexFieldCount + 1;
    FixedRows := 1;
    if RowCount > 2 then begin
      Cells[0,0] := Cells[0,0] + 's';
      Cells[1,0] := Cells[1,0] + 's';
    end;
    for I := 0 to IndexFieldCount - 1 do begin
      Cells[0, I+1] := IndexFields[I].DisplayName;
      Cells[1, I+1] := IndexFields[I].AsString;
    end;
  end;
end;

procedure TabcafFindKey.Panel1Resize(Sender: TObject);
const
  Offset = 5;
var
  LastPos: Integer;
begin
  LastPos := Panel1.Width - Offset;
  with HelpBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with CancelBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with OkBtn do
    if Visible then 
      Left := LastPos - Width;
end;

procedure TabcafFindKey.Panel2Resize(Sender: TObject);

  function GetVScrollVisibleWidth: Integer;
  var
    Min, Max: Integer;
  begin
    Result := 0;
    with KeyGrid do
      if ((ScrollBars = ssBoth) or (ScrollBars = ssVertical)) and HandleAllocated then begin
        GetScrollRange(Handle, SB_VERT, Min, Max);
        if Min <> Max then Result := GetSystemMetrics(SM_CXVSCROLL);
      end;
  end;

begin
  KeyGrid.DefaultColWidth :=
    (Panel2.Width - 9 - GetVScrollVisibleWidth) div 2;
end;

procedure TabcafFindKey.FormCreate(Sender: TObject);
begin
  with KeyGrid do begin
    {Fixed headings}
    Cells[0,0] := SIndexField;
    Cells[1,0] := SValHeading;
  end;

end;

procedure TabcafFindKey.FormActivate(Sender: TObject);
begin
  {force resize of grid}
  Panel2.Height := Panel2.Height + 1;
end;

procedure TabcafFindKey.KeyGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_TAB) then with KeyGrid do
    if (Shift = []) and (Col = 2) and (Row = RowCount - 1) then
      {tab to next control}
      ActiveControl := chkNearest
    else if (Shift = [ssShift]) and (Col = 1) and (Row = 1) then
      {tab to previous control}
      if HelpBtn.Visible then
        ActiveControl := HelpBtn
      else
        ActiveControl := CancelBtn;
end;

procedure TabcafFindKey.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcafFindKey);

end.
