unit abcfhlp;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, abcdbhnc, ComCtrls, ExtCtrls, Menus, ClipBrd, abcrched,
  abcfact, abcforms;

type

  { TabcsfHelpViewer }

  TabcsfHelpViewer = class(TabcForm)
    Panel1: TPanel;
    btnSave: TButton;
    btnDelete: TButton;
    btnClose: TButton;
    MainMenu1: TMainMenu;
    mnuOptions: TMenuItem;
    mnuOptionsKeeponTop: TMenuItem;
    rchHelp: TabcRichEdit;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnSaveClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure mnuOptionsKeeponTopClick(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
  private
    FHelpManager: TabcCustomDBHelpManager;
    FDisplayOnly: Boolean;
    FHelpForm: TForm;
    FHelpTopic: string;
    FOnShowExtension: TNotifyEvent;
    FOnCloseExtension: TCloseEvent;
    function GetTemplate: TFileName;
    procedure SetTemplate(Value: TFileName);
  protected
    procedure SetDisplayOnly(Value: Boolean); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    procedure ShowHelp(const Form: TForm; const HelpTopic, HelpText: string);
    property DisplayOnly: Boolean read FDisplayOnly write SetDisplayOnly;
    property Template: TFileName read GetTemplate write SetTemplate;
    property OnShowExtension: TNotifyEvent read FOnShowExtension write FOnShowExtension;
    property OnCloseExtension: TCloseEvent read FOnCloseExtension write FOnCloseExtension;
  end;

implementation

uses abcconst;

{$R *.DFM}

{ TabcsfHelpViewer }

constructor TabcsfHelpViewer.Create(AOwner: TComponent);
begin
  if not (AOwner is TabcCustomDBHelpManager) then
    raise Exception.Create(sabcErrorHelpManagerOwner);
  inherited Create(AOwner);
  FHelpManager := TabcCustomDBHelpManager(AOwner);
end;

procedure TabcsfHelpViewer.SetDisplayOnly(Value: Boolean);
begin
  FDisplayOnly := Value;
  if Value then
  begin
    btnSave.Visible := False;
    btnDelete.Visible := False;
    with rchHelp do
    begin
      ReadOnly := True;
      EditorOptions := [];
      PopupOptions := [arpPrint, arpPageSetup, arpCopy, arpSelectAll, arpFind];
    end;
  end
  else
  begin
    btnSave.Visible := True;
    btnDelete.Visible := True;
    with rchHelp do
    begin
      ReadOnly := False;
      EditorOptions := [{areFileNew,}
                        areFileOpen,
                        areFileLoadTemplate,
                        areFileSave,
                        areFilePrint,
                        areFilePageSetup,
                        areEditMenu,
                        areSearchMenu,
                        areFormatMenu,
                        areFileToolbar,
                        areEditToolbar,
                        areFindToolbar,
                        areFontToolbar,
                        areFormatToolbar,
                        areStatusBar,
                        {areConfirmSaveOnClose,}
                        areCancelOnEsc];
      PopupOptions := [arpEdit, {arpCancel,}
                       arpClear, arpLoadTemplate,
                       arpFileOpen, arpFileSaveAs,
                       arpPrint, arpPageSetup,
                       arpUndo, arpCut, arpCopy, arpPaste, arpDelete, arpSelectAll,
                       arpBold, arpItalic, arpUnderline,
                       arpFont, arpBullets,
                       arpFind, arpReplace];
    end;
  end;
end;

procedure TabcsfHelpViewer.ShowHelp(const Form: TForm; const HelpTopic, HelpText: string);
var
  strStream: TStringStream;
  sTopicDisplay: string;
begin
  FHelpForm := Form;
  FHelpTopic := HelpTopic;
  if FHelpForm <> nil then
    sTopicDisplay := Form.Caption
  else
    sTopicDisplay := HelpTopic;
  if DisplayOnly then
    Caption := sabcHelpFormCaption + sTopicDisplay
  else
    Caption := sabcHelpFormEditCaption + sTopicDisplay;

  rchHelp.PlainText := False;
  if (HelpText = '') then
  begin
    rchHelp.Lines.Clear;
    if not DisplayOnly and
       (FHelpManager.Template <> '') and
       FileExists(FHelpManager.Template) then
        rchHelp.Lines.LoadFromFile(FHelpManager.Template);
  end
  else
  begin
    strStream := TStringStream.Create(HelpText);
    try
      rchHelp.Lines.LoadFromStream(strStream);
    finally
      strStream.free;
    end;
  end;
  if not Visible then Show;
  BringToFront;
end;

procedure TabcsfHelpViewer.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caHide;
  if Assigned(FOnCloseExtension) then
    FOnCloseExtension(Self, Action);
  FHelpManager.OnCloseEditor;
end;

function TabcsfHelpViewer.GetTemplate: TFileName;
begin
  Result := rchHelp.FileTemplate;
end;

procedure TabcsfHelpViewer.SetTemplate(Value: TFileName);
begin
  rchHelp.FileTemplate := Value;
end;

procedure TabcsfHelpViewer.btnSaveClick(Sender: TObject);
var
  strStream: TStringStream;
begin
  Screen.Cursor := crHourglass;
  try
    strStream := TStringStream.Create('');
    try
      rchHelp.Lines.SaveToStream(strStream);
      strStream.Seek(0, soFromBeginning);
      FHelpManager.SetHelpText(FHelpForm, FHelpTopic, strStream.DataString);
    finally
      strStream.Free;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TabcsfHelpViewer.FormShow(Sender: TObject);
begin
  SetBounds(Screen.Width-Width, 0, Width, Height);
  if Assigned(FOnShowExtension) then
    FOnShowExtension(Self);
end;

procedure TabcsfHelpViewer.FormResize(Sender: TObject);
begin
  btnClose.Left := Panel1.Width - 4 - btnClose.Width;
  btnDelete.Left := btnClose.Left - 4 - btnDelete.Width;
  btnSave.Left := btnDelete.Left - 4 - btnSave.Width;
end;

procedure TabcsfHelpViewer.btnDeleteClick(Sender: TObject);
begin
  Screen.Cursor := crHourglass;
  try
    FHelpManager.DeleteHelpText(FHelpForm, FHelpTopic);
    rchHelp.Lines.Clear;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TabcsfHelpViewer.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TabcsfHelpViewer.mnuOptionsKeeponTopClick(Sender: TObject);
begin
  with mnuOptionsKeeponTop do
  begin
    Checked := not Checked;
    if Checked then
      FormStyle := fsStayOnTop
    else
      FormStyle := fsNormal;
  end;
end;

procedure TabcsfHelpViewer.Panel1Resize(Sender: TObject);
const
  BTN_SPACE = 8;
begin
  btnClose.Left := Panel1.ClientWidth - BTN_SPACE - btnClose.Width;
  btnDelete.Left := btnClose.Left - BTN_SPACE - btnDelete.Width;
  btnSave.Left := btnDelete.Left - BTN_SPACE - btnSave.Width;
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcsfHelpViewer);

end.
