unit abcfhnt;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, abcdbhnc,
  abcfact, abcforms;

type

  { TabcsfHintEditor }

  TabcsfHintEditor = class(TabcForm)
    Label1: TLabel;
    Label2: TLabel;
    edtShortHint: TEdit;
    btnSave: TButton;
    memLongHint: TMemo;
    btnDelete: TButton;
    chkTrackHint: TCheckBox;
    lblEditHint: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnSaveClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
  private
    FHintControl: TControl;
    FHintManager: TabcCustomDBHintManager;
    function GetHintControlStr: string;
    procedure SetHintControlStr(Value: string);
    procedure SetHintControl(Value: TControl);
  public
    constructor Create(AOwner: TComponent); override;
    procedure EditControlHint(AControl: TControl; Pos: TPoint); virtual;
    property HintControl: TControl read FHintControl write SetHintControl;
    property HintControlStr: string read GetHintControlStr write SetHintControlStr;
  end;

implementation

uses abcutilf, abcconst;

{$R *.DFM}

{ TabcsfHintEditor }

constructor TabcsfHintEditor.Create(AOwner: TComponent);
begin
  if not (AOwner is TabcCustomDBHintManager) then
    raise Exception.Create(sabcErrorHintManagerOwner);
  inherited Create(AOwner);
  FHintManager := TabcCustomDBHintManager(AOwner);
end;

procedure TabcsfHintEditor.EditControlHint(AControl: TControl; Pos: TPoint);
begin
  HintControl := AControl;
  if chkTrackHint.Checked then SetBounds(Pos.X, Pos.Y, Width, Height);
end;

function TabcsfHintEditor.GetHintControlStr: string;
begin
  Result := edtShortHint.Text + '|' + memLongHint.Text;
  if abcLRTrim(Result) = '|' then Result := '';
end;

procedure TabcsfHintEditor.SetHintControlStr(Value: string);
begin
  edtShortHint.Text := GetShortHint(Value);
  memLongHint.Text := GetLongHint(Value);
end;

procedure TabcsfHintEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caHide;
  FHintManager.OnCloseEditor;
end;

procedure TabcsfHintEditor.btnSaveClick(Sender: TObject);
begin
  Screen.Cursor := crHourglass;
  try
    FHintManager.SetHintText(HintControl, HintControlStr);
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TabcsfHintEditor.FormShow(Sender: TObject);
begin
  SetBounds(Screen.Width - Width, 0, Width, Height);
  ShowHint := False;
  HintControl := nil;
end;

procedure TabcsfHintEditor.FormResize(Sender: TObject);
var
  i: integer;
begin
  i := ClientWidth - 4 - btnSave.Width;
  btnSave.Left := i;
  btnDelete.Left := i;
  i := i - 4 - edtShortHint.Left;
  edtShortHint.Width := i;
  chkTrackHint.Top := ClientHeight - chkTrackHint.Height - 4;
  lblEditHint.Top := chkTrackHint.Top - lblEditHint.Height - 4;
  with memLongHint do SetBounds(Left, Top, i, lblEditHint.Top - Top - 6);
end;

procedure TabcsfHintEditor.btnDeleteClick(Sender: TObject);
begin
  Screen.Cursor := crHourglass;
  try
    edtShortHint.Text := '';
    memLongHint.Text := '';
    FHintManager.DeleteHintText(HintControl);
  finally
    Screen.Cursor := crHourglass;
  end;
end;

procedure TabcsfHintEditor.SetHintControl(Value: TControl);
begin
  if Value <> FHintControl then
  begin
    FHintControl := Value;
    if (Value <> nil) then
    begin
      btnSave.Enabled := True;
      btnDelete.Enabled := True;
      Caption := sabcHintFormCaption + ' - ' + HintControl.Name;
      HintControlStr := Value.Hint;
    end
    else
    begin
      btnSave.Enabled := False;
      btnDelete.Enabled := False;
      Caption := sabcHintFormCaption;
      HintControlStr := '';
    end;
  end;
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcsfHintEditor);

end.
