unit abcfiltr;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  DB, abcda;

type

  EabcInvalidFilterOperation = class(Exception);

//  TabcFilterRecordEvent = procedure(Sender: TObject; var Accept: Boolean) of object;  {Accept defaults to True}
  TabcFilterRecordEvent = TFilterRecordEvent;

  {TabcCustomDBFilter -> TabcCustomDBDataLinkFilter -> TabcDBFilter}
  {TabcCustomDBFilter -> TabcDBDataSetFilter, used in Nav Panel filters}

  TabcCustomDBFilter = class(TComponent)
  private
    FActive: Boolean;
    FAdapter: TabcDatasetAdapter;
    FAutoActivate: Boolean;
    FDataSet: TDataset;
    FFilter: string;
    FGoToFirst: Boolean;
    FFilterOptions: TFilterOptions;
    FHelpContext: THelpContext;
    FOnActivate: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;
    FOnFilterRecord: TabcFilterRecordEvent;
    FPriority: Word;
    procedure SetActive(Value: Boolean);
    procedure SetAutoActivate(Value: Boolean);
    procedure SetDataSet(Value: TDataSet);
    procedure SetFilter(Value: string);
    procedure SetFilterOptions(Value: TFilterOptions);
    procedure SetOnFilterRecord(Value: TabcFilterRecordEvent);
    procedure SetPriority(Value: Word);
  protected
    procedure Loaded; override;
    procedure LoadFromStream(Stream: TStream); virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SaveToStream(Stream: TStream); virtual;
    property Active: Boolean read FActive write SetActive;
    property AutoActivate: Boolean read FAutoActivate write SetAutoActivate default True;
    property DataSet: TDataSet read FDataSet write SetDataSet;
    property Filter: string read FFilter write SetFilter;
    property FilterOptions: TFilterOptions read FFilterOptions write SetFilterOptions default [];
    property GoToFirst: Boolean read FGoToFirst write FGoToFirst default True;
    property HelpContext: THelpContext read FHelpContext write FHelpContext;
    property Priority: Word read FPriority write SetPriority default 1;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnFilterRecord: TabcFilterRecordEvent read FOnFilterRecord write SetOnFilterRecord;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Activate;
    property Adapter: TabcDatasetAdapter read FAdapter;
    function CanActivate: Boolean;
    procedure Deactivate;
    function ParamByName(const FieldName: string): TParam;
    procedure LoadFromFile(const FileName: TFileName);
    procedure SaveToFile(const FileName: TFileName);
  end;

  { TabcDBDataSetFilter }

  TabcDBDataSetFilter = class(TabcCustomDBFilter)
  public
    function Edit: Boolean;
    property Active;
    property AutoActivate;
    property DataSet;
    property Filter;
    property FilterOptions;
    property GoToFirst;
    property HelpContext;
    {property Priority;}
    property OnActivate;
    property OnDeactivate;
    property OnFilterRecord;
  end;

  { TabcCustomDBDataLinkFilter }

  TabcFilterDataLink = class;

  TabcCustomDBDataLinkFilter = class(TabcCustomDBFilter)
  private
    FDataLink: TabcFilterDataLink;
    procedure ActiveChanged;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TabcDBFilter }

  TabcDBFilter = class(TabcCustomDBDataLinkFilter)
  public
    procedure Edit(Modal: Boolean; ShowHelpBtn: Boolean);
    property Active;
    property Adapter;
  published
    property AutoActivate;
    property DataSource;
    property Filter;
    property FilterOptions;
    property GoToFirst;
    property HelpContext;
    {property Priority;}
    property OnActivate;
    property OnDeactivate;
    property OnFilterRecord;
  end;

  { TabcFilterDataLink }

  TabcFilterDataLink = class(TDataLink)
  private
    FOwner: TabcCustomDBDataLinkFilter;
  protected
    procedure ActiveChanged; override;
  public
    constructor Create(AOwner: TabcCustomDBDataLinkFilter);
    destructor Destroy; override;
  end;

implementation

uses abcconst, abcffltr, abcfflds;

{ TabcCustomDBFilter }

constructor TabcCustomDBFilter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoActivate := True;
  FGoToFirst := True;
  FPriority := 1;
end;

destructor TabcCustomDBFilter.Destroy;
begin
  FAdapter.Free;
  inherited Destroy;
end;

procedure TabcCustomDBFilter.Activate;
begin
  SetActive(True);
end;

function TabcCustomDBFilter.CanActivate: Boolean;
begin
  Result := (FDataSet <> nil) and
            FDataSet.Active and
            (FAdapter <> nil) and
            FAdapter.CanFilter and
            ((Filter <> '') or Assigned(FOnFilterRecord));
end;

procedure TabcCustomDBFilter.Deactivate;
begin
  SetActive(False);
end;

function TabcCustomDBFilter.ParamByName(const FieldName: string): TParam;
begin
  Result := nil;
  if FAdapter <> nil then
    Result := FAdapter.FilterParamByName(FieldName);
end;

procedure TabcCustomDBFilter.Loaded;
begin
  inherited Loaded;
  SetDataSet(FDataSet);
end;

procedure TabcCustomDBFilter.LoadFromFile(const FileName: TFileName);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TabcCustomDBFilter.LoadFromStream(Stream: TStream);
var
  Reader: TReader;
begin
  Reader := TReader.Create(Stream, 1024);
  try
    Filter := Reader.ReadString;
  finally
    Reader.Free;
  end;
end;

procedure TabcCustomDBFilter.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataSet <> nil) and
    (AComponent = FDataSet) then DataSet := nil;
end;

procedure TabcCustomDBFilter.SaveToFile(const FileName: TFileName);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TabcCustomDBFilter.SaveToStream(Stream: TStream);
var
  Writer: TWriter;
begin
  Writer := TWriter.Create(Stream, 1024);
  try
    Writer.WriteString(FFilter);
  finally
    Writer.Free;
  end;
end;

procedure TabcCustomDBFilter.SetActive(Value: Boolean);
begin
  if Value <> FActive then
  begin
    if FAdapter <> nil then
    begin
      if FAdapter.SingleFilter and (FAdapter.Filter <> FFilter) then
        if Value then
        begin
          FAdapter.Filter := FFilter;
          FAdapter.FilterOptions := FFilterOptions;
          FAdapter.Filtered := True;
        end
        else
          FActive := False
      else
        FAdapter.Filtered := Value;
      if FAdapter.SingleFilter then
        if FAdapter.Filtered then
          if Assigned(FOnActivate) then
            FOnActivate(Self)
          else
        else
          if Assigned(FOnDeactivate) then
            FOnDeactivate(Self);
    end;
    FActive := Value;
  end;
end;

procedure TabcCustomDBFilter.SetAutoActivate(Value: Boolean);
begin
  FAutoActivate := Value;
  if Value and not Active and CanActivate and not (csLoading in ComponentState) then
    SetActive(True);
end;

procedure TabcCustomDBFilter.SetDataSet(Value: TDataSet);
begin
  if not (csLoading in ComponentState) then begin
    FAdapter.Free;
    FAdapter := nil;
    FDataset := Value;
    if Value <> nil then
    begin
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
      with FAdapter do
        if CanFilter then
        begin
          Filter := Self.Filter;
          FilterGoToFirst := Self.GoToFirst;
          FilterOptions := Self.FilterOptions;
          FilterPriority := Self.Priority;
          if not SingleFilter then
          begin
            OnActivateFilter := Self.OnActivate;
            OnDeactivateFilter := Self.OnDeactivate;
          end;
          OnFilterRecord := Self.OnFilterRecord;
          if FAutoActivate and CanActivate then Activate;
        end;
    end;
  end
  else
    FDataSet := Value;
end;

procedure TabcCustomDBFilter.SetFilter(Value: string);
begin
  FFilter := Value;
  if FAdapter <> nil then
  begin
    FAdapter.Filter := Value;
    if FAutoActivate and CanActivate then Activate;
  end;
end;

procedure TabcCustomDBFilter.SetFilterOptions(Value: TFilterOptions);
begin
  FFilterOptions := Value;
  if FAdapter <> nil then
  begin
    FAdapter.FilterOptions := Value;
    if FAutoActivate and CanActivate then Activate;
  end;
end;

procedure TabcCustomDBFilter.SetOnFilterRecord(Value: TabcFilterRecordEvent);
begin
  if Assigned(FOnFilterRecord) and Active then
    Active := False;
  FOnFilterRecord := Value;
  if FAdapter <> nil then
  begin
    FAdapter.OnFilterRecord := Value;
    if FAutoActivate and CanActivate then Activate;
  end;
end;

procedure TabcCustomDBFilter.SetPriority(Value: Word);
begin
  if Value <> FPriority then begin
    FPriority := Value;
    if FAdapter <> nil then
      FAdapter.FilterPriority := Value;
  end;
end;

{ TabcDBDataSetFilter }

function TabcDBDataSetFilter.Edit: Boolean;
{
Returns true if the filter was changed.
}
var
  Expression: string;
  Edited, CaseInsens, Partial: Boolean;
begin
  Result := False;
  Expression := Filter;
  CaseInsens := foCaseInsensitive in FilterOptions;
  Partial := not (foNoPartialCompare in FilterOptions);
  if HelpContext = 0 then
    Edited := ExecabcafDatasetFilter(DataSet, Adapter, Expression, CaseInsens, Partial, False, 0)
  else
    Edited := ExecabcafDatasetFilter(DataSet, Adapter, Expression, CaseInsens, Partial, True, HelpContext);
  if Edited then begin
    Filter := Expression;
    if CaseInsens then
      FilterOptions := FilterOptions + [foCaseInsensitive]
    else
      FilterOptions := FilterOptions - [foCaseInsensitive];
    if Partial then
      FilterOptions := FilterOptions - [foNoPartialCompare]
    else
      FilterOptions := FilterOptions + [foNoPartialCompare];
    Result := True;
  end;
end;

{ TabcCustomDBDataLinkFilter }

constructor TabcCustomDBDataLinkFilter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcFilterDataLink.Create(Self);
end;

destructor TabcCustomDBDataLinkFilter.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TabcCustomDBDataLinkFilter.ActiveChanged;
begin
  DataSet := FDataLink.DataSet;
end;

function TabcCustomDBDataLinkFilter.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcCustomDBDataLinkFilter.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TabcCustomDBDataLinkFilter.SetDataSource(Value: TDataSource);
begin
  Deactivate;
  FDataLink.DataSource := Value;
end;

{ TabcDBFilter }

procedure TabcDBFilter.Edit(Modal: Boolean; ShowHelpBtn: Boolean);
begin
  if (csDesigning in ComponentState) then
    ExecabcafFilter(Application, Self, Modal, ShowHelpBtn, 0)
  else
    ExecabcafFilter(Self, Self, Modal, ShowHelpBtn, FHelpContext);
end;

{ TabcFilterDataLink }

constructor TabcFilterDataLink.Create(AOwner: TabcCustomDBDataLinkFilter);
begin
  inherited Create;
  FOwner := AOwner;
end;

destructor TabcFilterDataLink.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

procedure TabcFilterDataLink.ActiveChanged;
begin
  if FOwner <> nil then
    FOwner.ActiveChanged;
end;

end.
