unit abcfqryp;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses SysUtils, Windows, Classes, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Grids, DB, abcforms, abcfact, abcda;

type

  EabcQueryParamDlgException = class(Exception);

  TabcafQueryParams = class(TabcApplicationForm)
    Panel1: TPanel;
    OKBtn: TBitBtn;
    Panel2: TPanel;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    KeyGrid: TStringGrid;
    chkOpen: TCheckBox;
    procedure SetParams;
    procedure Panel1Resize(Sender: TObject);
    procedure Panel2Resize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure SetDataset(Value: TDataset);
    procedure SetAdapter(Value: TabcDatasetAdapter);
    procedure KeyGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure HelpBtnClick(Sender: TObject);
  private
    FDataset: TDataset;
    FAdapter: TabcDatasetAdapter;
  public
    destructor Destroy; override;
  end;

var
  abcafQueryParams: TabcafQueryParams;

procedure ExecabcafQueryParams(AOwner: TComponent; ADataset: TDataset;
  AnAdapter: TabcDatasetAdapter; ShowHelpBtn: Boolean; Context: THelpContext);

implementation

uses abcconst, abcutilf;

{$R *.DFM}

const
  SParamFieldHeading = 'Parameter';
  SValHeading = 'Value';
  SFormCaption = 'Set Query Parameters ';

  FieldTypes: array[0..16] of string =
    ('Unknown','String','Smallint','Integer','Word','Boolean',
     'Float','Currency','BCD','Date','Time','DateTime','Bytes',
     'VarBytes','Blob','Memo','Graphic');

procedure ExecabcafQueryParams(AOwner: TComponent; ADataset: TDataset;
  AnAdapter: TabcDatasetAdapter; ShowHelpBtn: Boolean; Context: THelpContext);
var
  Dialog: TabcafQueryParams;
  DlgResult: TModalResult;
begin
  if (ADataset = nil) or (AnAdapter = nil) or not AnAdapter.CanSetParams then
    raise EabcQueryParamDlgException.Create(sabcErrQueryRequired)
  else if (AnAdapter.ParamCount < 1) then
    raise EabcQueryParamDlgException.Create(sabcErrQueryParams)
  else begin
    Dialog := TabcafQueryParams(
      abcFormFactory.CreateFormOfClass(TabcafQueryParams, AOwner));
    try
      with Dialog do begin
        SetDataset(ADataset);
        SetAdapter(AnAdapter);
        HelpBtn.Visible := ShowHelpBtn;
        HelpContext := Context;
        DlgResult := ShowModal;
        if (DlgResult = mrOk) then
          SetParams;
      end;
    finally
      Dialog.Free;
    end;
  end;
end;

destructor TabcafQueryParams.Destroy;
begin
  FDataset := nil;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcafQueryParams.SetParams;
var
  I: integer;
  ReadParams: TParams;
begin
  with FAdapter, FDataset do begin
    if Active then Close;
    ReadParams := Params;
    for I := 0 to ParamCount - 1 do
      ReadParams[I].Text := KeyGrid.Cells[1,I+1];
    Params := ReadParams;
    if chkOpen.Checked then
      with FDataset do
      begin
        Open;
        CursorPosChanged;
      end;
  end;
end;

procedure TabcafQueryParams.SetDataset(Value: TDataset);
begin
  FDataset := Value;
end;

procedure TabcafQueryParams.SetAdapter(Value: TabcDatasetAdapter);
{Setup field names and search values}
var
  I: integer;
begin
  FAdapter := Value;
  with KeyGrid, Value do begin
    Caption := SFormCaption + Description;
    RowCount := 1;
    RowCount := ParamCount + 1;
    FixedRows := 1;
    if RowCount > 2 then begin
      Cells[0,0] := Cells[0,0] + 's';
      Cells[1,0] := Cells[1,0] + 's';
    end;
    for I := 0 to ParamCount - 1 do begin
      Cells[0, I+1] := Params[I].Name + ' (' + FieldTypes[Ord(Params[I].DataType)] + ')';
      Cells[1, I+1] := Params[I].AsString;
    end;
  end;
end;

procedure TabcafQueryParams.Panel1Resize(Sender: TObject);
const
  Offset = 5;
var
  LastPos: Integer;
begin
  LastPos := Panel1.Width - Offset;
  with HelpBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with CancelBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with OkBtn do
    if Visible then
      Left := LastPos - Width;
end;

procedure TabcafQueryParams.Panel2Resize(Sender: TObject);

  function GetVScrollVisibleWidth: Integer;
  var
    Min, Max: Integer;
  begin
    Result := 0;
    with KeyGrid do
      if ((ScrollBars = ssBoth) or (ScrollBars = ssVertical)) and HandleAllocated then begin
        GetScrollRange(Handle, SB_VERT, Min, Max);
        if Min <> Max then Result := GetSystemMetrics(SM_CXVSCROLL);
      end;
  end;

begin
  KeyGrid.DefaultColWidth :=
    (Panel2.Width - 9 - GetVScrollVisibleWidth) div 2;
end;

procedure TabcafQueryParams.FormCreate(Sender: TObject);
begin
  with KeyGrid do begin
    {Fixed headings}
    Cells[0,0] := SParamFieldHeading;
    Cells[1,0] := SValHeading;
  end;

end;

procedure TabcafQueryParams.FormActivate(Sender: TObject);
begin
  {force resize of grid}
  Panel2.Height := Panel2.Height + 1;
end;

procedure TabcafQueryParams.KeyGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_TAB) then with KeyGrid do
    if (Shift = []) and (Col = 2) and (Row = RowCount - 1) then
      {tab to next control}
      ActiveControl := OkBtn
    else if (Shift = [ssShift]) and (Col = 1) and (Row = 1) then
      {tab to previous control}
      if HelpBtn.Visible then
        ActiveControl := HelpBtn
      else
        ActiveControl := CancelBtn;
end;

procedure TabcafQueryParams.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcafQueryParams);
  
end.
