unit abcfrnge;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses SysUtils, Windows, Classes, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Grids, DB, abcforms, abcfact, abcda;

type

  EabcRangeDlgException = class(Exception);

  TabcafKeyRange = class(TabcApplicationForm)
    Panel1: TPanel;
    OKBtn: TBitBtn;
    Panel2: TPanel;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    ClearBtn: TBitBtn;
    RangeGrid: TStringGrid;
    procedure ApplyRange;
    procedure RemoveRange;
    procedure Panel1Resize(Sender: TObject);
    procedure Panel2Resize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure SetDataset(Value: TDataset);
    procedure SetAdapter(Value: TabcDatasetAdapter);
    procedure RangeGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure HelpBtnClick(Sender: TObject);
  private
    FDataset: TDataset;
    FAdapter: TabcDatasetAdapter;
  public
    destructor Destroy; override;
  end;

var
  abcafKeyRange: TabcafKeyRange;

procedure ExecabcafKeyRange(AOwner: TComponent; ADataset: TDataset;
  AnAdapter: TabcDatasetAdapter; ShowHelpBtn: Boolean; Context: THelpContext);

implementation

uses abcconst, abcutilf;

{$R *.DFM}

const
  SIndexField = 'Index Field';
  SStartVal = 'Start Value';
  SEndVal = 'End Value';
  SFormCaption = 'Set Range on ';

procedure ExecabcafKeyRange(AOwner: TComponent; ADataset: TDataset;
  AnAdapter: TabcDatasetAdapter; ShowHelpBtn: Boolean; Context: THelpContext);
var
  Dialog: TabcafKeyRange;
  DlgResult: TModalResult;
begin
  if (ADataset = nil) or (AnAdapter = nil) or not AnAdapter.CanSetRange then
    raise EabcRangeDlgException.Create(sabcErrTableRequired)
  else if not ADataset.Active then
    raise EabcRangeDlgException.Create(sabcErrTableInactive)
  else if (AnAdapter.IndexFieldCount < 1) then
    raise EabcRangeDlgException.Create(sabcErrNoIndexFields)
  else begin
    Dialog := TabcafKeyRange(
      abcFormFactory.CreateFormOfClass(TabcafKeyRange, AOwner));
    try
      with Dialog do begin
        SetDataset(ADataset);
        SetAdapter(AnAdapter);
        HelpBtn.Visible := ShowHelpBtn;
        HelpContext := Context;
        DlgResult := ShowModal;
        if (DlgResult = mrOk) then
          ApplyRange
        else if (DlgResult = mrNo) then
          RemoveRange;
      end;
    finally
      Dialog.Free;
    end;
  end;
end;

destructor TabcafKeyRange.Destroy;
begin
  FDataset := nil;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcafKeyRange.ApplyRange;
var
  I: integer;
begin
  with FAdapter do begin
    SetRangeStart;
    for I := 0 to IndexFieldCount - 1 do
      IndexFields[I].AsString := RangeGrid.Cells[1,I+1];
    SetRangeEnd;
    for I := 0 to IndexFieldCount - 1 do
      IndexFields[I].AsString := RangeGrid.Cells[2,I+1];
    ApplyRange;
  end;
end;

procedure TabcafKeyRange.RemoveRange;
begin
  with FDataset, FAdapter do begin
    CancelRange;
    First;
  end;
end;

procedure TabcafKeyRange.SetDataset(Value: TDataset);
begin
  FDataset := Value;
end;

procedure TabcafKeyRange.SetAdapter(Value: TabcDatasetAdapter);
var
  I: integer;
  CurPos: TBookmark;
begin
  FAdapter := Value;
  {Setup Index field names}
  with RangeGrid, FDataset , Value do begin
    Caption := SFormCaption + Description;
    RowCount := 1;
    RowCount := IndexFieldCount + 1;
    FixedRows := 1;
    if RowCount > 2 then begin
      Cells[0,0] := Cells[0,0] + 's';
      Cells[1,0] := Cells[1,0] + 's';
      Cells[2,0] := Cells[2,0] + 's';
    end;
    for I := 0 to IndexFieldCount - 1 do begin
      Cells[0, I+1] := IndexFields[I].DisplayName;
      Cells[1, I+1] := IndexFields[I].AsString;
    end;
    DisableControls;
    CurPos := GetBookmark;
    Last;
    for I := 0 to IndexFieldCount - 1 do begin
      Cells[2, I+1] := IndexFields[I].AsString;
    end;
    GoToBookmark(CurPos);
    FreeBookmark(CurPos);
    EnableControls;
  end;
end;

procedure TabcafKeyRange.Panel1Resize(Sender: TObject);
const
  Offset = 5;
var
  LastPos: Integer;
begin
  LastPos := Panel1.Width - Offset;
  with HelpBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with CancelBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with OkBtn do
    if Visible then begin
      Left := LastPos - Width;
      LastPos := LastPos - Width - Offset;
    end;
  with ClearBtn do
    if Visible then 
      Left := LastPos - Width;
end;

procedure TabcafKeyRange.Panel2Resize(Sender: TObject);

  function GetVScrollVisibleWidth: Integer;
  var
    Min, Max: Integer;
  begin
    Result := 0;
    with RangeGrid do
      if ((ScrollBars = ssBoth) or (ScrollBars = ssVertical)) and HandleAllocated then begin
        GetScrollRange(Handle, SB_VERT, Min, Max);
        if Min <> Max then Result := GetSystemMetrics(SM_CXVSCROLL);
      end;
  end;

begin
  RangeGrid.DefaultColWidth :=
    (Panel2.Width - 9 - GetVScrollVisibleWidth) div 3;
end;

procedure TabcafKeyRange.FormCreate(Sender: TObject);
begin
  with RangeGrid do begin
    {Fixed headings}
    Cells[0,0] := SIndexField;
    Cells[1,0] := SStartVal;
    Cells[2,0] := SEndVal;
  end;

end;

procedure TabcafKeyRange.FormActivate(Sender: TObject);
begin
  Panel2.Height := Panel2.Height + 1;
end;

procedure TabcafKeyRange.RangeGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_TAB) then with RangeGrid do
    if (Shift = []) and (Col = 2) and (Row = RowCount - 1) then
      {tab to next control}
      ActiveControl := ClearBtn
    else if (Shift = [ssShift]) and (Col = 1) and (Row = 1) then
      {tab to previous control}
      if HelpBtn.Visible then
        ActiveControl := HelpBtn
      else
        ActiveControl := CancelBtn;
end;

procedure TabcafKeyRange.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcafKeyRange);

end.
