unit abcfti32;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, abctips, abclabel, abcforms, abcfact, abcctl32,
  ComCtrls;

type
  TabcsfTip = class(TabcForm)
    pnlCenter: TPanel;
    pnlBottom: TPanel;
    btnClose: TButton;
    Shape1: TShape;
    Bevel2: TBevel;
    Bevel1: TBevel;
    Image1: TImage;
    lblDidYouKnow: TLabel;
    lblTip: TLabel;
    btnNext: TButton;
    btnHelp: TButton;
    btnWhatsNew: TButton;
    pnlTop: TPanel;
    lblGreeting: TabcMultilineLabel;
    chkShowNext: TabcCheckBox;
    pnlProgress: TPanel;
    ProgressBar: TProgressBar;
    lblProgress: TLabel;
    procedure btnNextClick(Sender: TObject);
    procedure chkShowNextClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure btnWhatsNewClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FTipComponent: TabcCustomWelcomeTips;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure AdjustFormSize; virtual;
  end;

var
  frmABCTip: TabcsfTip;

implementation

uses abcconst;

{$R *.DFM}

constructor TabcsfTip.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TabcCustomWelcomeTips) then
    FTipComponent := TabcCustomWelcomeTips(AOwner)
  else
    raise Exception.Create(sabcErrorOwnerWelcomeTipsExpected);
  chkShowNext.Checked := FTipComponent.ShowOnStartup;
  btnNextClick(Self);
end;

procedure TabcsfTip.btnNextClick(Sender: TObject);
begin
  lblTip.Caption := FTipComponent.GetNextTip;
end;

procedure TabcsfTip.chkShowNextClick(Sender: TObject);
begin
  FTipComponent.ShowOnStartup := chkShowNext.Checked;
end;

procedure TabcsfTip.btnHelpClick(Sender: TObject);
begin
  Application.HelpContext(btnHelp.HelpContext);
end;

procedure TabcsfTip.btnWhatsNewClick(Sender: TObject);
begin
  Application.HelpContext(btnWhatsNew.HelpContext);
end;

procedure TabcsfTip.AdjustFormSize;
const
  C_SPACE = 2;
  MIN_TOP_HEIGHT = 41;
  MIN_BOTTOM_HEIGHT = 40;
var
  topHeight, bottomHeight, clHeight: integer;
begin
  topHeight := lblGreeting.Top + lblGreeting.Height + C_SPACE;
  if topHeight < MIN_TOP_HEIGHT then
    topHeight := MIN_TOP_HEIGHT;
  bottomHeight := chkShowNext.Top + chkShowNext.Height + C_SPACE;
  if bottomHeight < MIN_BOTTOM_HEIGHT then
    bottomHeight := MIN_BOTTOM_HEIGHT;
  clHeight := topHeight + pnlCenter.Height + bottomHeight;
  if pnlProgress.Visible then
    clHeight := clHeight + pnlProgress.Height;
  ClientHeight := clHeight;
  pnlTop.Height := topHeight;
  pnlBottom.Height := bottomHeight;
end;

procedure TabcsfTip.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TabcsfTip.FormDestroy(Sender: TObject);
begin
  if FTipComponent <> nil then
    FTipComponent.AfterExecute;
end;

procedure TabcsfTip.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TabcsfTip.FormShow(Sender: TObject);
begin
  AdjustFormSize;
end;

initialization
  abcFormFactory.RegisterBaseFormClass(TabcsfTip);

end.
