unit abcfx;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Menus, StdCtrls, imglist,
  abcufx, abcbtn, abclabel, abcapp;

type

  {$IFNDEF DELPHI5}
    TImageIndex = type Integer;
  {$ENDIF}

  TabcPictureSize = (apsNone, apsAutoSize, apsCenter, apsCenterBestFit, apsBestFit, apsStretch);

  TabcEffectSize = 1..Maxint;

  { TabcTransition }

  TabcTransition = class(TPersistent)
  private
    FEffect: TabcTransitionEffect;
    FEffectPause: Word;
    FEffectSelection: TList;
    FEffectSelIndex: Integer;
    FEffectSize: TabcEffectSize;
    FEffectSpeed: Word;
    FEnabled: Boolean;
    FThread: TabcTransitionThread;
    FNoRedraw: Boolean;
    FOnEffectStep: TabcCancelEvent;
    FOnEffectEnd: TNotifyEvent;
    FOnChange: TNotifyEvent;
    FBitmap: TBitmap;
    FWaitFor: Boolean;
    procedure CancelEffectStep(Sender: TObject; var Cancel: Boolean);
    procedure SetEffectSelection(Value: TList);
    procedure SetEffectSelIndex(Value: Integer);
    procedure ReadProcessMessages(Reader: TReader);
    procedure ReadEffectSelections(Reader: TReader);
    procedure WriteEffectSelections(Writer: TWriter);
    procedure SetEffect(const Value: TabcTransitionEffect);
    procedure SetEffectSize(const Value: TabcEffectSize);
    procedure SetEffectSpeed(const Value: Word);
    procedure SetEnabled(const Value: Boolean);
    procedure SetNoRedraw(const Value: Boolean);
    procedure ThreadTerminated(Sender: TObject);
    procedure TerminateThread;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure DataChanged;
    property WaitFor: Boolean read FWaitFor write FWaitFor default True;
    property EffectPause: Word read FEffectPause write FEffectPause default 0;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create;
    destructor Destroy; override;
    procedure DrawAt(const Source, Target: TCanvas; const Dest: TRect);
    procedure DrawWindow(const AWinControl: TWinControl; const Target: TCanvas; ClientOnly: Boolean; X, Y: Integer);
    procedure DrawWindowAt(const AWinControl: TWinControl; const Target: TCanvas; ClientOnly: Boolean; const Dest: TRect);
    procedure DrawForm(const AForm: TCustomForm; const Target: TCanvas);
    procedure DrawFormAt(const AForm: TCustomForm; const Target: TCanvas; const Dest: TRect);
    procedure DrawFormToScreen(const AForm: TCustomForm; const Dest: TRect);
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnEffectStep: TabcCancelEvent read FOnEffectStep write FOnEffectStep;
    property OnEffectEnd: TNotifyEvent read FOnEffectEnd write FOnEffectEnd;
  published
    property Effect: TabcTransitionEffect read FEffect write SetEffect default atfNone;
    property EffectSelection: TList read FEffectSelection write SetEffectSelection
             stored False; {values stored by DefineProperties}
    property EffectSelIndex: Integer read FEffectSelIndex write SetEffectSelIndex default -1;
    property EffectSize: TabcEffectSize read FEffectSize write SetEffectSize default 8;
    property EffectSpeed: Word read FEffectSpeed write SetEffectSpeed default 500;
    property Enabled: Boolean read FEnabled write SetEnabled default False;
    property NoRedraw: Boolean read FNoRedraw write SetNoRedraw default False;
  end;

  { TabcCustomEffectsImage }

  TabcCustomEffectsImage = class(TGraphicControl)
  private
    { Private declarations }
    FAlignment: TAlignment;
    FCaptionStyle: TabcCaptionStyle;
    FDLLList: TList;
    FEffect: TabcTransitionEffect;
    FEffectSelection: TList;
    FEffectSelIndex: Integer;
    FEffectSize: TabcEffectSize;
    FEffectSpeed: Word;
    FEffectPause: Word;
    FImageVisible: Boolean;
    FLayout: TTextLayout;
    FPicture: TPicture;
    FReadOnly: Boolean;
    FOnEffectStep: TabcCancelEvent;
    FOnEffectEnd: TNotifyEvent;
    FShowAccelChar: Boolean;
    FTransparent: Boolean;
    FWordWrap: Boolean;
    FCopyingBackground: Boolean;
    FThread: TabcTransitionThread;
    FSuspendOnAppInactive: Boolean;
    FCanvas: TCanvas;
    FLoadingPicture: Boolean;
    FParentWindowHandle: HWnd;
    FNeverPainted: Boolean;
    procedure GetCanvasHandle;
    procedure ReleaseCanvasHandle;
    procedure ApplicationActivated(Sender: TObject);
    procedure ApplicationDeactivated(Sender: TObject);
    procedure CancelEffectStep(Sender: TObject; var Cancel: Boolean);
    procedure FreeDlls;
    function GetCanvas: TCanvas;
    function GetEffectSelCount: Integer;
    procedure SetAlignment(Value: TAlignment);
    procedure SetCaptionStyle(Value: TabcCaptionStyle);
    procedure SetEffectSelection(Value: TList);
    procedure SetEffectSelIndex(Value: Integer);
    procedure SetImageVisible(Value: Boolean);
    procedure SetLayout(Value: TTextLayout);
    procedure SetPicture(Value: TPicture);
    procedure SetPictureSize(Value: TabcPictureSize);
    procedure SetShowAccelChar(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetWordWrap(Value: Boolean);
    procedure ReadEffectSelections(Reader: TReader);
    procedure WriteEffectSelections(Writer: TWriter);
    procedure ReadProcessMessages(Reader: TReader);
    function GetEffectSel(Index: Integer): TabcTransitionEffect;
    procedure SetEffectSel(Index: Integer; const Value: TabcTransitionEffect);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMBiDiModeChanged(var Message: TMessage); message CM_BIDIMODECHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMCopy(var Message: TMessage); message WM_COPY;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
  protected
    FPictureHasChanged: Boolean;
    FPictureIsChanging: Boolean;
    FPictureSize: TabcPictureSize;
    FPaintPictureSize: TabcPictureSize;
    FBackgroundBitmap: TBitmap;
    FPictureBitmap: TBitmap;
    FScreenBuffer: TBitmap;
    function GetPalette: HPALETTE; override;
    procedure Paint; override;
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    procedure SetParent(AParent: TWinControl); override;
    {for descendant classes}
    procedure PictureChanged(Sender: TObject); virtual;
    procedure AdjustAutoSize;
    function CalcPictureRect: TRect;
    procedure DrawBackground; virtual;
    procedure DrawPicture(R: TRect); virtual;
    procedure DrawImageText(R: TRect); virtual;
    procedure DoDrawText(Canvas: TCanvas; var Rect: TRect; Flags: Longint); virtual;
    function GetReadOnly: Boolean; virtual;
    procedure SetReadOnly(Value: Boolean); virtual;
    function DoEdit: Boolean; virtual;
    procedure DoClear; virtual;
    procedure DoEffectEnd; virtual;
    procedure DoStop; virtual;
    function HasText: Boolean; virtual;
    function IsEmpty: Boolean; virtual;
    procedure TerminateThread; virtual;
    procedure ThreadTerminated(Sender: TObject); virtual;
    procedure CanCreateThread;
    property NeverPainted: Boolean read FNeverPainted;
    {following properties can be published}
    property Alignment: TAlignment read FAlignment write SetAlignment
      default taCenter;
    property CaptionStyle: TabcCaptionStyle read FCaptionStyle write SetCaptionStyle;
    property Effect: TabcTransitionEffect read FEffect write FEffect default atfNone;
    property EffectPause: Word read FEffectPause write FEffectPause default 0;
    property EffectSelection: TList read FEffectSelection write SetEffectSelection
             stored False; {values stored by DefineProperties}
    property EffectSelIndex: Integer read FEffectSelIndex write SetEffectSelIndex default -1;
    property EffectSize: TabcEffectSize read FEffectSize write FEffectSize default 8;
    property EffectSpeed: Word read FEffectSpeed write FEffectSpeed default 500;
    property Layout: TTextLayout read FLayout write SetLayout default tlCenter;
    property Picture: TPicture read FPicture write SetPicture;
    property PictureSize: TabcPictureSize read FPictureSize write SetPictureSize default apsCenterBestFit;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly;
    property ShowAccelChar: Boolean read FShowAccelChar write SetShowAccelChar default True;
    property SuspendOnAppInactive: Boolean read FSuspendOnAppInactive write FSuspendOnAppInactive default True;
    property Transparent: Boolean read FTransparent write SetTransparent default False;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default True;
    property OnEffectStep: TabcCancelEvent read FOnEffectStep write FOnEffectStep;
    property OnEffectEnd: TNotifyEvent read FOnEffectEnd write FOnEffectEnd;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Canvas: TCanvas read GetCanvas;
    procedure Clear;
    procedure HideImage;
    procedure ShowImage;
    procedure CopyToClipboard;
    procedure CutToClipboard;
    procedure AddSelectedEffect(Effect: TabcTransitionEffect);
    procedure RemoveSelectedEffect(Effect: TabcTransitionEffect);
    function IsEffectSelected(Effect: TabcTransitionEffect): Boolean;
    property EffectSelCount: Integer read GetEffectSelCount;
    property EffectSel[Index: Integer]: TabcTransitionEffect read GetEffectSel write SetEffectSel;
    property ImageVisible: Boolean read FImageVisible write SetImageVisible default True;
    procedure PasteFromClipboard;
    procedure FreeDll(Instance: THandle);
    function LoadFromDLL(DLLFileName: string; ResourceID: Integer; const ResourceName: string): THandle;
    procedure LoadFromResourceID(Instance: THandle; const ResourceID: Integer);
    procedure LoadFromResourceName(Instance: THandle; const ResourceName: string);
    procedure LoadFromFile(FileName: TFileName);
    procedure RefreshBackground;
    procedure Stop;
    procedure Suspend;
    procedure Resume;
  published
  end;

  TabcEffectsImage = class(TabcCustomEffectsImage)
  private
    FAnimationStopped: Boolean;
    FImageIndex: TImageIndex;
    FImageList: TImageList;
    FOnAnimateNext: TabcCancelEvent;
    FPlaying, FLoop: Boolean;
    FPlayIndexes: TList;
    FPlayItem: integer;
    FPlayItemChanging: Boolean;
    procedure SetImageIndex(const Value: TImageIndex);
    procedure SetImageList(const Value: TImageList);
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure TerminateThread; override;
    procedure DoEffectEnd; override;
    procedure DoStop; override;
    procedure CheckAnimationStop;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Play(Loop: Boolean);
    procedure PlayRange(FromIndex, ToIndex: TImageIndex; Loop: Boolean);
    procedure PlaySelection(const Selection: array of TImageIndex; Loop: Boolean);
  published
    property Align;
    property Alignment;
    property Caption;
    property CaptionStyle;
    property Color;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property DragCursor;
    property DragMode;
    property Effect;
    property EffectPause;
    property EffectSelection;
    property EffectSelIndex;
    property EffectSize;
    property EffectSpeed;
    property Enabled;
    property Font;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
    property ImageList: TImageList read FImageList write SetImageList;
    property Layout;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property Picture;
    property PictureSize;
    property PopupMenu;
    property ReadOnly;
    property ShowAccelChar;
    property ShowHint;
    property SuspendOnAppInactive;
    property Transparent;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnStartDock;
    property OnEndDock;
    property OnEffectEnd;
    property OnEffectStep;
    property OnAnimateNext: TabcCancelEvent read FOnAnimateNext write FOnAnimateNext;
  end;

implementation

uses Clipbrd, abcconst, abcutilf, abcutilg;

constructor TabcCustomEffectsImage.Create(AOwner: TComponent);
var
  screenpf: TPixelFormat;
begin
  FNeverPainted := True;
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque] - [csSetCaption];
  FCanvas := TCanvas.Create;
  FPicture := TPicture.Create;
  FPictureBitmap := TBitmap.Create;
  screenpf := abcScreenPixelFormat;
  FPictureBitmap.PixelFormat := screenpf;
  FPicture.OnChange := PictureChanged;
  FBackgroundBitmap := TBitmap.Create;
  FBackgroundBitmap.PixelFormat := screenpf;
  FScreenBuffer := TBitmap.Create;
  FScreenBuffer.PixelFormat := screenpf;
  FCaptionStyle := TabcCaptionStyle.Create;
  FCaptionStyle.OnChange := PictureChanged;
  FAlignment := taCenter;
  FLayout := tlCenter;
  FShowAccelChar := True;
  FWordWrap := True;
  SetBounds(Left, Top, 100, 100);
  FEffect := atfNone;
  FEffectSelection := TList.Create;
  FEffectSize := 8;
  FEffectSpeed := 500;
  FPictureSize := apsCenterBestFit;
  FPaintPictureSize := apsCenter;
  FDLLList := TList.Create;
  FImageVisible := True;
  abcApplicationEvents.RegisterOnActivate(ApplicationActivated);
  abcApplicationEvents.RegisterOnDeactivate(ApplicationDeactivated);
  FSuspendOnAppInactive := True;
end;

destructor TabcCustomEffectsImage.Destroy;
begin
  Destroying;
  TerminateThread;
  FPicture.Free;
  FPictureBitmap.Free;
  FBackgroundBitmap.Free;
  FScreenBuffer.Free;
  FEffectSelection.Free;
  FCaptionStyle.Free;
  FreeDLLs;
  FDLLList.Free;
  abcApplicationEvents.UnRegisterOnActivate(ApplicationActivated);
  abcApplicationEvents.UnRegisterOnDeactivate(ApplicationDeactivated);
  FCanvas.Free;
  inherited Destroy;
end;

procedure TabcCustomEffectsImage.Loaded;
begin
  inherited;
  PictureChanged(nil);
end;

procedure TabcCustomEffectsImage.ApplicationActivated(Sender: TObject);
begin
  if SuspendOnAppInactive then
    Resume;
end;

procedure TabcCustomEffectsImage.ApplicationDeactivated(Sender: TObject);
begin
  if SuspendOnAppInactive then
    Suspend;
end;

procedure TabcCustomEffectsImage.SetParent(AParent: TWinControl);
begin
  TerminateThread;
  inherited;
end;

function TabcCustomEffectsImage.DoEdit: Boolean;
begin
  Result := not ReadOnly;
end;

procedure TabcCustomEffectsImage.Clear;
begin
  if IsEmpty then Exit;
  FPictureIsChanging := True;
  DoClear;
  FPictureIsChanging := False;
  PictureChanged(nil);
end;

procedure TabcCustomEffectsImage.DoClear;
begin
  Picture.Graphic := nil;
  Caption := '';
end;

procedure TabcCustomEffectsImage.FreeDlls;
var
  i: integer;
begin
  with FDLLList do
    for i := Count - 1 downto 0 do
    begin
      FreeLibrary(THandle(Items[i]));
      Delete(i);
    end;
end;

procedure TabcCustomEffectsImage.FreeDll(Instance: THandle);
{free all references to Instance}
var
  i: integer;
begin
  with FDLLList do
    for i := Count - 1 downto 0 do
      if Items[i] = Pointer(Instance) then
      begin
        FreeLibrary(THandle(Items[i]));
        Delete(i);
      end;
end;

function TabcCustomEffectsImage.LoadFromDLL(DLLFileName: string; ResourceID: Integer; const ResourceName: string): THandle;
begin
  Result := 0;
  if not DoEdit then Exit;
  Result := LoadLibrary(PChar(DLLFileName));
  if Result <> 0 then
  begin
    FDLLList.Add(Pointer(Result));
    if ResourceName <> '' then
      Picture.Bitmap.LoadFromResourceName(Result, ResourceName)
    else
      Picture.Bitmap.LoadFromResourceID(Result, ResourceID);
  end;
end;

procedure TabcCustomEffectsImage.LoadFromResourceID(Instance: THandle; const ResourceID: Integer);
begin
  if DoEdit then
  begin
    FLoadingPicture := True;
    Picture.Graphic := nil;
    Picture.Bitmap.LoadFromResourceID(Instance, ResourceID);
    FLoadingPicture := False;
    PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.LoadFromResourceName(Instance: THandle; const ResourceName: string);
begin
  if DoEdit then
  begin
    FLoadingPicture := True;
    Picture.Graphic := nil;
    Picture.Bitmap.LoadFromResourceName(Instance, ResourceName);
    FLoadingPicture := False;
    PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.LoadFromFile(FileName: TFileName);
begin
  if DoEdit then
  begin
    FLoadingPicture := True;
    Picture.Graphic := nil;
    Picture.LoadFromFile(FileName);
    FLoadingPicture := False;
    PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('EffectSelections', ReadEffectSelections, WriteEffectSelections, (FEffectSelection.Count > 0));
  //this property was removed
  Filer.DefineProperty('ProcessMessages', ReadProcessMessages, nil, False);
end;

procedure TabcCustomEffectsImage.ReadEffectSelections(Reader: TReader);
begin
  with Reader do
  begin
    ReadListBegin;
    FEffectSelection.Clear;
    while not EndOfList do FEffectSelection.Add(Pointer(Reader.ReadInteger));
    ReadListEnd;
  end;
end;

procedure TabcCustomEffectsImage.WriteEffectSelections(Writer: TWriter);
var
  i: integer;
begin
  with Writer do
  begin
    WriteListBegin;
    for i := 0 to FEffectSelection.Count - 1 do
      WriteInteger(Integer(FEffectSelection[i]));
    WriteListEnd;
  end;
end;

procedure TabcCustomEffectsImage.ReadProcessMessages(Reader: TReader);
begin
  Reader.ReadBoolean;
end;

procedure TabcCustomEffectsImage.CopyToClipboard;
begin
  if Picture.Graphic <> nil then Clipboard.Assign(Picture);
end;

procedure TabcCustomEffectsImage.CutToClipboard;
begin
  if Picture.Graphic <> nil then
    if DoEdit then
    begin
      CopyToClipboard;
      Picture.Graphic := nil;
    end;
end;

procedure TabcCustomEffectsImage.PasteFromClipboard;
begin
  if DoEdit then
    if Clipboard.HasFormat(CF_PICTURE) then
      Picture.Assign(Clipboard)
    else if Clipboard.HasFormat(CF_BITMAP) then
      Picture.Bitmap.Assign(Clipboard)
    else if Clipboard.HasFormat(CF_METAFILEPICT) then
      Picture.Metafile.Assign(Clipboard);
end;

function TabcCustomEffectsImage.GetCanvas: TCanvas;
var
  Bitmap: TBitmap;
begin
  if Picture.Graphic is TBitmap then
    Result := TBitmap(Picture.Graphic).Canvas
  else
    if Picture.Graphic = nil then
    begin
      Bitmap := TBitmap.Create;
      try
        Bitmap.Width := Width;
        Bitmap.Height := Height;
        Picture.Graphic := Bitmap;
      finally
        Bitmap.Free;
      end;
      Result := TBitmap(Picture.Graphic).Canvas;
    end
    else
      raise EInvalidOperation.Create(sabcErrorBitmapNeeded);
end;

function TabcCustomEffectsImage.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture.Graphic <> nil then
    Result := FPicture.Graphic.Palette;
end;

procedure TabcCustomEffectsImage.SetPictureSize(Value: TabcPictureSize);
begin
  FPictureSize := Value;
  PictureChanged(nil);
end;

function TabcCustomEffectsImage.CalcPictureRect: TRect;
var
  PicRatio, ImageRatio: Double;
begin
  Result := ClientRect;
  if (Picture.Width > 0) and
     (Picture.Height > 0) and
     (Width > 0) then
    with Result do
      case FPaintPictureSize of
        apsNone, apsAutoSize, apsCenterBestFit, apsStretch:
          {no adjustment};
        apsCenter:
        begin
          Left := (Width - Picture.Width) div 2;
          Right := Left + Picture.Width;
          Top := (Height - Picture.Height) div 2;
          Bottom := Top + Picture.Height;
        end;
        apsBestFit:
        begin
          PicRatio := Picture.Height / Picture.Width;
          ImageRatio := Height / Width;
          if PicRatio > ImageRatio then
          begin
            Right := Trunc(Bottom / PicRatio);
            Left := (Width - Right) div 2;
            Right := Right + Left;
          end
          else
          begin
            Bottom := Trunc(Right * PicRatio);
            Top := (Height - Bottom) div 2;
            Bottom := Top + Bottom;
          end;
        end;
      end;
end;

procedure TabcCustomEffectsImage.AdjustAutoSize;
begin
  if (PictureSize = apsCenterBestFit) then
    if (Picture.Width > Width) or (Picture.Height > Height) then
      FPaintPictureSize := apsBestFit
    else
      FPaintPictureSize := apsCenter
  else
    FPaintPictureSize := PictureSize;
  if (PictureSize = apsAutoSize) and (Picture.Width > 0) and (Picture.Height > 0) and
    ((Picture.Width <> Width) or (Picture.Height > Height)) then
    SetBounds(Left, Top, Picture.Width, Picture.Height);
end;

procedure TabcCustomEffectsImage.DrawBackground;
begin
  with FScreenBuffer.Canvas do
    if Transparent then
      Draw(0,0,FBackgroundBitmap)
    else
    begin
      Brush.Style := bsSolid;
      Brush.Color := Color;
      FillRect(ClientRect);
    end;
end;

procedure TabcCustomEffectsImage.DrawPicture(R: TRect);
var
  G: TGraphic;
begin
  {copy picture graphic to internal bitmap - for jpeg, wmf, ico conversion into a bitmap}
  with FPictureBitmap do
  begin
    {erase}
    Height := 0;
    Width := 0;
    {draw}
    Height := Picture.Height;
    Width := Picture.Width;

    if Self.Transparent then
      Canvas.Brush.Color := ABC_TRANS_COLOR
    else
      Canvas.Brush.Color := Self.Color;
    Canvas.FillRect(Rect(0,0,Width,Height));

    G := FPicture.Graphic;
    if (G <> nil) and not ((G is TMetaFile) or (G is TIcon)) then
      G.Transparent := Self.Transparent;

    Canvas.Draw(0,0,FPicture.Graphic);
    Transparent := Self.Transparent;
  end;

  {draw picture to buffer}
  with FScreenBuffer.Canvas do
    if FPaintPictureSize in [apsNone, apsAutoSize, apsCenter] then
      Draw(R.Left, R.Top, FPictureBitmap)
    else
      StretchDraw(R, FPictureBitmap);

end;

procedure TabcCustomEffectsImage.PictureChanged(Sender: TObject);
var
  PicR: TRect;
begin
  if FNeverPainted or FPictureIsChanging or FLoadingPicture or ([csLoading, csDestroying] * ComponentState <> []) or Application.Terminated then
    Exit;
  FPictureIsChanging := True;
  try
    TerminateThread;
    AdjustAutoSize;
    FPictureHasChanged := True;
    DrawBackground;
    PicR := CalcPictureRect;

    if ImageVisible then
      DrawPicture(PicR);
    if HasText then
      DrawImageText(PicR);

    if csDesigning in ComponentState then
      with FScreenBuffer.Canvas do
      begin
        Pen.Style := psDash;
        Brush.Style := bsClear;
        Rectangle(0, 0, Width, Height);
      end;

    CanCreateThread;
  finally
    FPictureIsChanging := False;
  end;
  if csDesigning in ComponentState then
    Invalidate;
end;

procedure TabcCustomEffectsImage.DrawImageText(R: TRect);
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  DrawRect, CalcRect: TRect;
  DrawFlags: Longint;

  procedure DrawOffsetText(Offset: Integer; Emboss, Shadow: Boolean);
  var
    bmpSrc, bmpMask: TBitmap;
  begin
    with FScreenBuffer do
    begin
      if Emboss then
      begin
        bmpSrc := TBitmap.Create;
        bmpMask := TBitmap.Create;
        bmpSrc.Assign(FScreenBuffer);
        bmpMask.Width := Width;
        bmpMask.Height := Height;
      end
      else begin
        bmpSrc := nil;
        bmpMask := nil;
      end;

      if not Shadow then begin
        OffsetRect(DrawRect, -Offset, -Offset);
        Canvas.Font.Color := FCaptionStyle.ColorHighlight;
        DoDrawText(Canvas, DrawRect, DrawFlags);
        OffsetRect(DrawRect, 2*Offset, 2*Offset);
      end
      else
        OffsetRect(DrawRect, Offset, Offset);

      Canvas.Font.Color := FCaptionStyle.ColorShadow;
      DoDrawText(Canvas, DrawRect, DrawFlags);
      OffsetRect(DrawRect, -Offset, -Offset);
      if Emboss then
      begin
        with bmpMask.Canvas do
        begin
          Brush.Style := bsClear;
          Font := Self.Font;
          Font.Color := clBlack;
        end;
        DoDrawText(bmpMask.Canvas, DrawRect, DrawFlags);
        SetBkColor(Canvas.Handle, clWhite);
        SetTextColor(Canvas.Handle, clBlack);
        BitBlt(Canvas.Handle, 0, 0, Width, Height, bmpSrc.Canvas.Handle, 0, 0, SRCINVERT);
        BitBlt(Canvas.Handle, 0, 0, Width, Height, bmpMask.Canvas.Handle, 0, 0, SRCAND);
        BitBlt(Canvas.Handle, 0, 0, Width, Height, bmpSrc.Canvas.Handle, 0, 0, SRCINVERT);
        bmpSrc.Free;
        bmpMask.Free;
      end
      else
      begin
        Canvas.Font.Color := Self.Font.Color;
        DoDrawText(Canvas, DrawRect, DrawFlags);
      end;
    end;
  end;

begin
  {draw caption to buffer}
  with FScreenBuffer.Canvas, CaptionStyle do
  begin
    Font := Self.Font;
    Brush.Style := bsClear;
    DrawRect := R;
    DrawFlags := DT_EXPANDTABS or WordWraps[FWordWrap] or Alignments[FAlignment];

    case Effect of
      ceNormal: {do nothing};
      ceRaised..ceEmbossLowered:
        InflateRect(DrawRect, -Offset*2, -Offset*2);
      ceShadowed:
        InflateRect(DrawRect, -Offset, -Offset);
    end;

    if FLayout <> tlTop then
    begin
      CalcRect := DrawRect;
      DoDrawText(FScreenBuffer.Canvas, CalcRect, DrawFlags or DT_CALCRECT);
      if FLayout = tlBottom then
        OffsetRect(DrawRect, 0, R.Bottom - CalcRect.Bottom)
      else
        OffsetRect(DrawRect, 0, (R.Bottom - CalcRect.Bottom) div 2);
    end;

    if not Enabled or (Effect = ceNormal) then
      DoDrawText(FScreenBuffer.Canvas, DrawRect, DrawFlags)
    else
      case FCaptionStyle.Effect of
        ceRaised:         DrawOffsetText(Offset, False, False);
        ceLowered:        DrawOffsetText(-Offset, False, False);
        ceEmbossRaised:   DrawOffsetText(Offset, True, False);
        ceEmbossLowered:  DrawOffsetText(-Offset, True, False);
        ceShadowed:       DrawOffsetText(Offset, False, True);
      end;
  end;
end;

procedure TabcCustomEffectsImage.DoDrawText(Canvas: TCanvas; var Rect: TRect; Flags: Longint);
var
  Text: string;
begin
  Text := Caption;
  if (Flags and DT_CALCRECT <> 0) and ((Text = '') or FShowAccelChar and
    (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';
  if not FShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Flags := DrawTextBiDiModeFlags(Flags);
  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
  end
  else
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
end;

procedure TabcCustomEffectsImage.CancelEffectStep(Sender: TObject; var Cancel: Boolean);
begin
  Cancel := Application.Terminated or (csDestroying in ComponentState) or not Enabled or not Visible;
  if Assigned(FOnEffectStep) then
    FOnEffectStep(Self, Cancel);
end;

procedure TabcCustomEffectsImage.WMPaint(var Message: TWMPaint);
begin
  if FCopyingBackground then Exit;
  if FNeverPainted then
  begin
    FNeverPainted := False;
    RefreshBackground;
    if (csDesigning in ComponentState) or IsEmpty then
    begin
      FPictureHasChanged := False;
      inherited;
    end
    else if FPictureHasChanged then
      CanCreateThread;
  end
  else
    inherited;
end;

procedure TabcCustomEffectsImage.CMColorChanged(var Message: TMessage);
begin
  inherited;
  if not (Transparent or (csLoading in ComponentState)) then
    PictureChanged(nil);
end;

procedure TabcCustomEffectsImage.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Transparent and not (FNeverPainted or (csLoading in ComponentState)) then
    RefreshBackground;
end;

procedure TabcCustomEffectsImage.CMTextChanged(var Message: TMessage);
begin
  PictureChanged(nil);
end;

procedure TabcCustomEffectsImage.CMEnabledChanged(var Message: TMessage);
begin
  PictureChanged(nil);
  if not Enabled then
    Repaint;
end;

procedure TabcCustomEffectsImage.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if Visible then
    CanCreateThread
  else
    TerminateThread;
end;

procedure TabcCustomEffectsImage.CMFontChanged(var Message: TMessage);
begin
  if Caption <> '' then
    PictureChanged(nil);
end;

procedure TabcCustomEffectsImage.CMBiDiModeChanged(var Message: TMessage);
begin
  if Caption <> '' then
    PictureChanged(nil);
end;

procedure TabcCustomEffectsImage.CMDialogChar(var Message: TCMDialogChar);
begin
  if Enabled and ShowAccelChar and
    IsAccel(Message.CharCode, Caption) and
    (ssAlt in KeyDataToShiftState(Message.KeyData)) then
  begin
    Click;
    Message.Result := 1;
  end;
end;

procedure TabcCustomEffectsImage.Paint;
begin
  if FNeverPainted or FCopyingBackground or (FThread <> nil) then
    Exit;
  inherited Canvas.Draw(0,0, FScreenBuffer);
end;

procedure TabcCustomEffectsImage.SetAlignment(Value: TAlignment);
begin
  if Value <> FAlignment then
  begin
    FAlignment := Value;
    if Caption <> '' then
      PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.SetCaptionStyle(Value: TabcCaptionStyle);
begin
  if Value <> FCaptionStyle then begin
    FCaptionStyle.Assign(Value);
    FCaptionStyle.OnChange := PictureChanged;
    PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.SetImageVisible(Value: Boolean);
begin
  if Value <> FImageVisible then
  begin
    FImageVisible := Value;
    PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.HideImage;
begin
  ImageVisible := False;
end;

procedure TabcCustomEffectsImage.ShowImage;
begin
  ImageVisible := True;
end;

function TabcCustomEffectsImage.HasText: Boolean;
begin
  Result := Caption <> '';
end;

function TabcCustomEffectsImage.IsEmpty: Boolean;
begin
  Result := (not HasText) and (Picture.Graphic = nil);
end;

procedure TabcCustomEffectsImage.SetLayout(Value: TTextLayout);
begin
  if Value <> FLayout then
  begin
    FLayout := Value;
    if HasText then
      PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

function TabcCustomEffectsImage.GetReadOnly: Boolean;
begin
  Result := FReadOnly;
end;

procedure TabcCustomEffectsImage.SetReadOnly(Value: Boolean);
begin
  FReadOnly := Value;
end;

procedure TabcCustomEffectsImage.SetShowAccelChar(Value: Boolean);
begin
  if Value <> FShowAccelChar then
  begin
    FShowAccelChar := Value;
    if Caption <> '' then
      PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.RefreshBackground;
begin
  SetBounds(Left, Top, Width, Height);
end;

procedure TabcCustomEffectsImage.SetTransparent(Value: Boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    if FPicture.Graphic <> nil then
      FPicture.Graphic.Transparent := Transparent;
    if not (csLoading in ComponentState) then
      if Value then
        SetBounds(Left, Top, Width, Height)
      else
        PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.SetWordWrap(Value: Boolean);
begin
  if Value <> FWordWrap then
  begin
    FWordWrap := Value;
    if Caption <> '' then
      PictureChanged(nil);
  end;
end;

procedure TabcCustomEffectsImage.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  wasResized: Boolean;
begin
  inherited;
  if FNeverPainted then Exit;
  wasResized := (FScreenBuffer.Width <> Width) or (FScreenBuffer.Height <> Height);
  if wasResized then
  begin
    FScreenBuffer.Width := Width;
    FScreenBuffer.Height := Height;
  end;
  if Transparent then
    {get background bitmap}
    with FBackgroundBitmap do
    try
      Width := Self.Width;
      Height := Self.Height;
      FCopyingBackground := True;
      Canvas.Lock;
      Parent.PaintTo(Canvas.Handle, -Left-1, -Top-1);
    finally
      Canvas.Unlock;
      FCopyingBackground := False;
    end;
  if not FPictureIsChanging then
    if Transparent or wasResized then
      PictureChanged(nil);
end;

procedure TabcCustomEffectsImage.SetEffectSelIndex(Value: Integer);
begin
  if (Value < -1) or (Value >= FEffectSelection.Count) then
    FEffectSelIndex := -1
  else
    FEffectSelIndex := Value;
end;

procedure TabcCustomEffectsImage.SetEffectSelection(Value: TList);
var
  i: integer;
begin
 with FEffectSelection do
  begin
    Clear;
    if Value <> nil then
      for i := 0 to Value.Count - 1 do
        Add(Value[i]);
  end;
end;

procedure TabcCustomEffectsImage.WMCut(var Message: TMessage);
begin
  CutToClipboard;
end;

procedure TabcCustomEffectsImage.WMCopy(var Message: TMessage);
begin
  CopyToClipboard;
end;

procedure TabcCustomEffectsImage.WMPaste(var Message: TMessage);
begin
  PasteFromClipboard;
end;

procedure TabcCustomEffectsImage.AddSelectedEffect(
  Effect: TabcTransitionEffect);
begin
  if not IsEffectSelected(Effect) then
    FEffectSelection.Add(Pointer(Effect));
end;

function TabcCustomEffectsImage.GetEffectSelCount: Integer;
begin
  Result := FEffectSelection.Count;
end;

function TabcCustomEffectsImage.IsEffectSelected(
  Effect: TabcTransitionEffect): Boolean;
begin
  Result := FEffectSelection.IndexOf(Pointer(Effect)) <> -1;
end;

procedure TabcCustomEffectsImage.RemoveSelectedEffect(
  Effect: TabcTransitionEffect);
begin
  FEffectSelection.Remove(Pointer(Effect));
end;

function TabcCustomEffectsImage.GetEffectSel(
  Index: Integer): TabcTransitionEffect;
begin
  if (Index > -1) and (Index < FEffectSelection.Count) then
    Result := TabcTransitionEffect(FEffectSelection.Items[Index])
  else
    Result := atfNone;
end;

procedure TabcCustomEffectsImage.SetEffectSel(Index: Integer;
  const Value: TabcTransitionEffect);
begin
  if (Index > -1) and (Index < FEffectSelection.Count) then
    FEffectSelection.Items[Index] := Pointer(Value)
  else if (Index >= FEffectSelection.Count) then
    AddSelectedEffect(Value);
end;

procedure TabcCustomEffectsImage.GetCanvasHandle;
begin
  FCanvas.Handle := GetDeviceContext(FParentWindowHandle);
end;

procedure TabcCustomEffectsImage.ReleaseCanvasHandle;
var
  OldDC: HDC;
begin
  OldDC := FCanvas.Handle;
  FCanvas.Handle := 0;
  if IsWindow(FParentWindowHandle) then
    ReleaseDC(FParentWindowHandle, OldDC);
end;

procedure TabcCustomEffectsImage.CanCreateThread;
begin
  if FPictureHasChanged and (FThread = nil) and not IsEmpty and
     not FNeverPainted and Visible and
     (Parent <> nil) and Parent.HandleAllocated and
     not (csDesigning in ComponentState) then
  begin
    FPictureHasChanged := False;
    GetCanvasHandle;
    FThread :=
      abcThreadTransition (
        Effect, EffectSelection, FEffectSelIndex,
        FScreenBuffer.Canvas, FCanvas,
        0, 0, Width, Height,
        0, 0,
        EffectSize, EffectSpeed, EffectPause,
        CancelEffectStep,
        ThreadTerminated,
        False,
        False
        );
  end;
end;

procedure TabcCustomEffectsImage.TerminateThread;
begin
  if FThread <> nil then
    with FThread do
    begin
      Suspend;
      OnTerminate := nil;
      FreeOnTerminate := False;
      Resume;
      Terminate;
      WaitFor;
      Free;
      ThreadTerminated(Self);
    end;
end;

procedure TabcCustomEffectsImage.ThreadTerminated(Sender: TObject);
begin
  FThread := nil;
  DoEffectEnd;
end;

procedure TabcCustomEffectsImage.DoEffectEnd;
begin
  ReleaseCanvasHandle;
  if not ((csDestroying in ComponentState) or Application.Terminated) and Assigned(FOnEffectEnd) then
    FOnEffectEnd(Self);
end;

procedure TabcCustomEffectsImage.DoStop;
begin
  TerminateThread;
  Invalidate;
end;

procedure TabcCustomEffectsImage.Stop;
begin
  DoStop;
end;

procedure TabcCustomEffectsImage.Resume;
begin
  if FThread <> nil then
    FThread.Resume;
end;

procedure TabcCustomEffectsImage.Suspend;
begin
  if FThread <> nil then
    FThread.Suspend;
end;

{ TabcEffectsImage }

{ Image List Support with Animations }

constructor TabcEffectsImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FImageIndex := -1;
end;

destructor TabcEffectsImage.Destroy;
begin
  TerminateThread;
  FPlayIndexes.Free;
  inherited;
end;

procedure TabcEffectsImage.Loaded;
begin
  inherited Loaded;
  if ImageList <> nil then
    SetImageIndex(ImageIndex);
end;

procedure TabcEffectsImage.SetImageIndex(const Value: TImageIndex);
var
  WasTransparent: Boolean;
begin
  FImageIndex := Value;
  if not (csLoading in ComponentState) and
     Assigned(ImageList) and
     (Value > -1) and
     (Value < ImageList.Count) then
  begin
    //load imagelist image
    FPictureIsChanging := True;
    WasTransparent := Picture.Bitmap.Transparent;
    Picture.Bitmap.Transparent := False;
    ImageList.GetBitmap(Value, Picture.Bitmap);
    Picture.Bitmap.Transparent := WasTransparent;
    FPictureIsChanging := False;
    PictureChanged(nil);
  end
  else
    SetPicture(nil);
end;

procedure TabcEffectsImage.SetImageList(const Value: TImageList);
begin
  if Value <> FImageList then
  begin
    FImageList := Value;
    if not (csLoading in ComponentState) then
      SetImageIndex(ImageIndex);
  end;
end;

procedure TabcEffectsImage.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (AComponent = FImageList) and (Operation = opRemove) then
    FImageList := nil;
end;

procedure TabcEffectsImage.Play(Loop: Boolean);
begin
  if Assigned(ImageList) and (ImageList.Count > 0) then
    PlayRange(0, ImageList.Count - 1, Loop);
end;

procedure TabcEffectsImage.PlayRange(FromIndex, ToIndex: TImageIndex; Loop: Boolean);
var
  i: integer;
begin
  FAnimationStopped := False;
  TerminateThread;
  if Assigned(ImageList) and (ImageList.Count > 0) then
  begin
    if FPlayIndexes = nil then
      FPlayIndexes := TList.Create
    else
      FPlayIndexes.Clear;
    if FromIndex < ToIndex then
      for i := FromIndex to ToIndex do
        FPlayIndexes.Add(Pointer(i))
    else
        for i := FromIndex downto ToIndex do
          FPlayIndexes.Add(Pointer(i));
    FPlaying := True;
    FLoop := Loop;
    FPlayItem := 0;
    FPlayItemChanging := True;
    ImageIndex := FromIndex;
    FPlayItemChanging := False;
  end;
end;

procedure TabcEffectsImage.PlaySelection(const Selection: array of TImageIndex; Loop: Boolean);
var
  i: integer;
begin
  FAnimationStopped := False;
  TerminateThread;
  if Assigned(ImageList) and (ImageList.Count > 0) then
  begin
    if FPlayIndexes = nil then
      FPlayIndexes := TList.Create
    else
      FPlayIndexes.Clear;
    for i := Low(Selection) to High(Selection) do
      FPlayIndexes.Add(Pointer(Selection[i]));
    FPlaying := True;
    FLoop := Loop;
    FPlayItem := 0;
    FPlayItemChanging := True;
    ImageIndex := TImageIndex(FPlayIndexes[0]);
    FPlayItemChanging := False;
  end;
end;

procedure TabcEffectsImage.DoEffectEnd;
begin
  inherited;
  if FPlaying then
  begin
    CheckAnimationStop;
    if FAnimationStopped then
    begin
      FPlaying := False;
      FPlayIndexes.Clear;
      Exit;
    end;
    FPlayItem := FPlayItem + 1;
    if FPlayItem >= FPlayIndexes.Count then
      if FLoop then
        FPlayItem := 0
      else
      begin
        FPlaying := False;
        FPlayIndexes.Clear;
        Exit;
      end;
    FPlayItemChanging := True;
    ImageIndex :=  TImageIndex(FPlayIndexes[FPlayItem]);
    FPlayItemChanging := False;
  end;
end;

procedure TabcEffectsImage.DoStop;
begin
  FPlayItemChanging := False;
  inherited;
end;

procedure TabcEffectsImage.CheckAnimationStop;
begin
  if Assigned(OnAnimateNext) then
    OnAnimateNext(Self, FAnimationStopped);
  if Application.Terminated or (csDestroying in ComponentState) or not Visible then
    FAnimationStopped := True;
end;

procedure TabcEffectsImage.TerminateThread;
begin
  if not FPlayItemChanging then
    FPlaying := False;
  inherited;
end;

{ TabcTransition }

constructor TabcTransition.Create;
begin
  FEffect := atfNone;
  FEffectSelection := TList.Create;
  FEffectSize := 8;
  FEffectSpeed := 500;
  FEffectSelIndex := -1;
  FWaitFor := True;
end;

destructor TabcTransition.Destroy;
begin
  TerminateThread;
  if FBitmap <> nil then
    FBitmap.Free;
  FEffectSelection.Free;
  inherited;
end;

procedure TabcTransition.Assign(Source: TPersistent);
begin
  if Source is TabcTransition then
    with Source do
    begin
      Self.Effect := Effect;
      Self.EffectSelection := EffectSelection;
      Self.EffectSelIndex := EffectSelIndex;
      Self.EffectSize := EffectSize;
      Self.EffectSpeed := EffectSpeed;
      Self.Enabled := Enabled;
      Self.NoRedraw := NoRedraw;
      Self.OnEffectStep := OnEffectStep;
      Self.OnEffectEnd := OnEffectEnd;
      Self.DataChanged;
    end
  else
    inherited;
end;

procedure TabcTransition.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('EffectSelections', ReadEffectSelections, WriteEffectSelections, (FEffectSelection.Count > 0));
  //this property was removed
  Filer.DefineProperty('ProcessMessages', ReadProcessMessages, nil, False);
end;

procedure TabcTransition.ReadProcessMessages(Reader: TReader);
begin
  Reader.ReadBoolean;
end;

procedure TabcTransition.CancelEffectStep(Sender: TObject; var Cancel: Boolean);
begin
  if Assigned(FOnEffectStep) then
    FOnEffectStep(Self, Cancel);
end;

procedure TabcTransition.DrawAt(const Source, Target: TCanvas; const Dest: TRect);
begin
  if FThread <> nil then Exit;
  with Dest do
    FThread :=
      abcThreadTransition (
        Effect, EffectSelection, FEffectSelIndex,
        Source, Target,
        Left, Top, Right - Left, Bottom - Top,
        0, 0,
        EffectSize, EffectSpeed, EffectPause,
        CancelEffectStep,
        ThreadTerminated,
        True,
        False
        );
  if WaitFor then
    with FThread do
    begin
      FreeOnTerminate := False;
      Resume;
      WaitFor;
      Free;
      ThreadTerminated(Self);
    end
  else
    FThread.Resume;
end;

procedure TabcTransition.ThreadTerminated(Sender: TObject);
begin
  FThread := nil;
  if FBitmap <> nil then
  begin
    FBitmap.Free;
    FBitmap := nil;
  end;
  if Assigned(FOnEffectEnd) then
    FOnEffectEnd(Self);
end;

procedure TabcTransition.TerminateThread;
begin
  if FThread <> nil then
    with FThread do
    begin
      Suspend;
      FreeOnTerminate := False;
      OnTerminate := nil;
      Terminate;
      Resume;
      WaitFor;
      Free;
      ThreadTerminated(Self);
    end;
end;

procedure TabcTransition.DrawWindowAt(const AWinControl: TWinControl; const Target: TCanvas; ClientOnly: Boolean; const Dest: TRect);
//draw window at dest topleft and clip to dest rect
var
  SaveIndex: Integer;
begin
  with Target do
  begin
    SaveIndex := SaveDC(Handle);
    try
      with Dest do
      begin
        DrawWindow(AWinControl, Target, ClientOnly, Left, Top);
      end;
    finally
      RestoreDC(Handle, SaveIndex);
    end;
  end;
end;

procedure TabcTransition.DrawWindow(const AWinControl: TWinControl; const Target: TCanvas; ClientOnly: Boolean; X, Y: Integer);
begin
  if AWinControl = nil then Exit;
  if FBitmap <> nil then
  begin
    FBitmap.Free;
    FBitmap := nil;
  end;
  FBitmap := abcGetControlBitmap(AWinControl);
  DrawAt(FBitmap.Canvas, Target, Rect(X, Y, X+FBitmap.Width, Y+FBitmap.Height));
end;

procedure TabcTransition.DrawFormAt(const AForm: TCustomForm; const Target: TCanvas; const Dest: TRect);
//draw form client area at dest topleft and clip to dest rect
var
  SaveIndex: Integer;
begin
  with Target do
  begin
    Lock;
    SaveIndex := SaveDC(Handle);
    try
      with Dest do
      begin
        DrawWindow(AForm, Target, True, Left, Top);
      end;
    finally
      RestoreDC(Handle, SaveIndex);
      Unlock;
    end;
  end;
end;

procedure TabcTransition.DrawForm(const AForm: TCustomForm; const Target: TCanvas);
var
  R: TRect;
begin
  R := AForm.ClientRect;
  OffsetRect(R, -R.Left, -R.Top);
  DrawWindowAt(AForm, Target, True, R);
end;

procedure TabcTransition.DrawFormToScreen(const AForm: TCustomForm; const Dest: TRect);
var
  Canvas: TCanvas;
  DC: HDC;
  SaveIndex, Width, Height: Integer;
begin
  Canvas := TCanvas.Create;
  try
    Width := Dest.Right - Dest.Left;
    Height := Dest.Bottom - Dest.Top;
    DC := GetDC(0);
    SaveIndex := SaveDC(DC);
    try
      MoveWindowOrg(DC, Dest.Left, Dest.Top);
      IntersectClipRect(DC, 0, 0, Width, Height);
      Canvas.Handle := DC;
      DrawForm(AForm, Canvas);
      Canvas.Handle := 0;
    finally
      RestoreDC(DC, SaveIndex);
      ReleaseDC(0, DC);
    end;
  finally
    Canvas.Free;
  end;
end;

procedure TabcTransition.ReadEffectSelections(Reader: TReader);
begin
  with Reader do
  begin
    ReadListBegin;
    FEffectSelection.Clear;
    while not EndOfList do FEffectSelection.Add(Pointer(Reader.ReadInteger));
    ReadListEnd;
  end;
end;

procedure TabcTransition.WriteEffectSelections(Writer: TWriter);
var
  i: integer;
begin
  with Writer do
  begin
    WriteListBegin;
    for i := 0 to FEffectSelection.Count - 1 do
      WriteInteger(Integer(FEffectSelection[i]));
    WriteListEnd;
  end;
end;

procedure TabcTransition.SetEffectSelection(Value: TList);
var
  i: integer;
begin
  with FEffectSelection do
  begin
    Clear;
    if Value <> nil then
      for i := 0 to Value.Count - 1 do
        Add(Value[i]);
  end;
  DataChanged;
end;

procedure TabcTransition.SetEffectSelIndex(Value: Integer);
begin
  if (Value < -1) or (Value >= FEffectSelection.Count) then
    FEffectSelIndex := -1
  else if Value <> FEffectSelIndex then
  begin
    FEffectSelIndex := Value;
    DataChanged;
  end;
end;

procedure TabcTransition.DataChanged;
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

procedure TabcTransition.SetEffect(const Value: TabcTransitionEffect);
begin
  if Value <> FEffect then
  begin
    FEffect := Value;
    DataChanged;
  end;
end;

procedure TabcTransition.SetEffectSize(const Value: TabcEffectSize);
begin
  if Value <> FEffectSize then
  begin
    FEffectSize := Value;
    DataChanged;
  end;
end;

procedure TabcTransition.SetEffectSpeed(const Value: Word);
begin
  if Value <> FEffectSpeed then
  begin
    FEffectSpeed := Value;
    DataChanged;
  end;
end;

procedure TabcTransition.SetEnabled(const Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    DataChanged;
  end;
end;

procedure TabcTransition.SetNoRedraw(const Value: Boolean);
begin
  if Value <> FNoRedraw then
  begin
    FNoRedraw := Value;
    DataChanged;
  end;
end;

end.
