unit abcfxrch;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses Classes, Windows, Graphics, Controls, StdCtrls, Forms, abcmem, abcrched, abcfx;

type

  { TabcRichText }

  TabcRichText = class(TabcPersistentMemory)
  private
    FRichEdit: TabcRichEdit;
    FEditing: Boolean;
  protected
    procedure Changed; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    function Edit: Boolean;
    procedure LoadFromRichEdit(ARichEdit: TabcRichEdit);
    procedure SaveToRichEdit(ARichEdit: TabcRichEdit);
    procedure PaintToRect(Dest: TCanvas; DestR: TRect; ALayout: TTextLayout);
  end;

  { TabcRichEffectsImage }

  TabcRichEffectsImage = class(TabcEffectsImage)
  private
    FRichText: TabcRichText;
    procedure SetRichText(Value: TabcRichText);
  protected
    procedure DoClear; override;
    procedure DrawImageText(R: TRect); override;
    function HasText: Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property RichText: TabcRichText read FRichText write SetRichText;
  end;

implementation

uses abcuspr, abcutilg;

{ TabcRichText }

constructor TabcRichText.Create;
begin
  inherited Create;
  FRichEdit := TabcRichEdit(TabcRichEdit.CreateParentedControl(Application.Handle));
  FRichEdit.Visible := False;
  FRichEdit.Color := ABC_TRANS_COLOR;
end;

destructor TabcRichText.Destroy;
begin
  FRichEdit.Free;
  FRichEdit := nil;
  inherited;
end;

procedure TabcRichText.Changed;
begin
  if not FEditing then
    SaveToRichEdit(FRichEdit);
  inherited;
end;

function TabcRichText.Edit: Boolean;
begin
  FEditing := True;
  try
    FRichEdit.Color := clWindow;
    Result := FRichEdit.Edit;
    FRichEdit.Color := ABC_TRANS_COLOR;
    if Result then
      LoadFromRichEdit(FRichEdit);
  finally
    FEditing := False;
  end;
end;

procedure TabcRichText.SaveToRichEdit(ARichEdit: TabcRichEdit);
var
  tmpStream: TMemoryStream;
begin
  if ARichEdit = nil then Exit;
  tmpStream := TMemoryStream.Create;
  try
    SaveToStream(tmpStream);
    tmpStream.Seek(0,0);
    ARichEdit.Lines.LoadFromStream(tmpStream);
  finally
    tmpStream.Free;
  end;
end;

procedure TabcRichText.LoadFromRichEdit(ARichEdit: TabcRichEdit);
var
  tmpStream: TMemoryStream;
begin
  if ARichEdit = nil then Exit;
  tmpStream := TMemoryStream.Create;
  try
    ARichEdit.Lines.SaveToStream(tmpStream);
    tmpStream.Seek(0,0);
    LoadFromStream(tmpStream);
  finally
    tmpStream.Free;
  end;
end;

procedure TabcRichText.PaintToRect(Dest: TCanvas; DestR: TRect; ALayout: TTextLayout);
var
  FormatDC: HDC;
  bmpR: TRect;
  bmpTemp: TBitmap;
begin
  bmpTemp := TBitmap.Create;
  with bmpTemp do
  try
    Width := 0;
    Height := 0;
    with DestR do
    begin
      Width := Right - Left;
      Height := Bottom - Top;
    end;
    bmpR := Rect(0,0,Width,Height);
    with Canvas do
    begin
      Brush.Color := ABC_TRANS_COLOR;
      Brush.Style := bsSolid;
      FillRect(bmpR);
    end;
    FormatDC := GetDC(0);
    try
      FRichEdit.PaintToRect(Canvas.Handle, FormatDC, bmpR, ALayout);
    finally
      ReleaseDC(0, FormatDC);
    end;
    abcTransparentBltMasked(
      Dest, DestR.Left, DestR.Top, bmpR.Right, bmpR.Bottom,
      bmpTemp, 0, 0, True);
  finally
    Free;
  end;
end;

{ TabcRichEffectsImage }

constructor TabcRichEffectsImage.Create(AOwner: TComponent);
begin
  inherited;
  FRichText := TabcRichText.Create;
  FRichText.OnChange := PictureChanged;
end;

destructor TabcRichEffectsImage.Destroy;
begin
  Destroying;
  FRichText.Free;
  inherited;
end;

procedure TabcRichEffectsImage.DoClear;
begin
  inherited;
  FRichText.Clear;
end;

procedure TabcRichEffectsImage.DrawImageText(R: TRect);
begin
  inherited;
  with FRichText do
    if not Empty then
      PaintToRect(FScreenBuffer.Canvas, R, Layout);
end;

function TabcRichEffectsImage.HasText: Boolean;
begin
  Result := not FRichText.Empty or (Caption <> '');
end;

procedure TabcRichEffectsImage.SetRichText(Value: TabcRichText);
begin
  if Value = nil then
    FRichText.Clear
  else if Value <> FRichText then
    FRichText.Assign(Value);
end;

end.
