unit abchandl;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

TabcWindowHandle is a non-visual descendant of TComponent that provides an
invisible Window Handle.

It can be used to implement descendant classes that respond to specific messages,
for example notifications that are normally sent to visible controls.

Adding message handlers in descendant classes is done in the same way as you would
do with any TControl descendant.  The WndProc in this class calls Dispatch to
locate a message handler in your class.

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, Classes, Forms;

type
  TabcWindowHandle = class(TComponent)
  private
    FWindowHandle: HWND;
  protected
    procedure WndProc(var Message: TMessage); virtual;
    property Handle: HWND read FWindowHandle;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DefaultHandler(var Message); override;
  published
  end;

implementation

{ TabcWindowHandle }

constructor TabcWindowHandle.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWindowHandle := {$IFDEF DELPHI6}Classes.{$ENDIF}AllocateHWnd(WndProc);
end;

destructor TabcWindowHandle.Destroy;
begin
  {$IFDEF DELPHI6}Classes.{$ENDIF}DeallocateHWnd(FWindowHandle);
  inherited Destroy;
end;

procedure TabcWindowHandle.WndProc(var Message: TMessage);
begin
  Dispatch(Message);
end;

procedure TabcWindowHandle.DefaultHandler(var Message);
begin
  with TMessage(Message) do
    Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
end;

end.
