unit abchgrid;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}


interface

uses SysUtils, Windows, Messages, Classes, Controls, Forms,
  Graphics, Menus, StdCtrls, DB, Grids, DBCtrls;

type
  TabcCustomHDBGrid = class;

  TabcHGridDataLink = class(TDataLink)
  private
    FGrid: TabcCustomHDBGrid;
    FFieldCount: Integer;
    FFieldMapSize: Integer;
    FModified: Boolean;
    FInUpdateData: Boolean;
    FFieldMap: Pointer;
    function GetDefaultFields: Boolean;
    function GetFields(I: Integer): TField;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
    procedure DataSetScrolled(Distance: Integer); override;
    procedure EditingChanged; override;
    procedure LayoutChanged; override;
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
  public
    constructor Create(AGrid: TabcCustomHDBGrid);
    destructor Destroy; override;
    function AddMapping(const FieldName: string): Boolean;
    procedure ClearMapping;
    procedure Modified;
    procedure Reset;
    property DefaultFields: Boolean read GetDefaultFields;
    property FieldCount: Integer read FFieldCount;
    property Fields[I: Integer]: TField read GetFields;
  end;

  TabcHDBGridOption = (hdgEditing, hdgAlwaysShowEditor, hdgTitles, hdgIndicator,
    hdgColumnResize, hdgRowResize, hdgColLines, hdgRowLines, hdgTabs,
    hdgAlwaysShowSelection, hdgConfirmDelete, hdgCancelOnExit, hdgWordWrap);
  TabcHDBGridOptions = set of TabcHDBGridOption;
  TDrawDataCellEvent = procedure (Sender: TObject; const Rect: TRect; Field: TField;
    State: TGridDrawState) of object;

  TabcHDBGridColSizing = (csNumCols, csBestFit, csExact, csField);

  TabcCustomHDBGrid = class(TCustomGrid)
  private
    FIndicators: TImageList;
    FTitleFont: TFont;
    FAlignTitles: TAlignment;
    FReadOnly: Boolean;
    FUpdatingColWidths: Boolean;
    FOptions: TabcHDBGridOptions;
    FScrollBars: TScrollStyle;
    FTitleOffset, FIndicatorOffset: Byte;
    FUpdateLock: Byte;
    FInRowExit: Boolean;
    FDefaultDrawing: Boolean;
    FSelfChangingTitleFont: Boolean;
    FSelCol: Integer;
    FDataLink: TabcHGridDataLink;
    FOnRowEnter: TNotifyEvent;
    FOnRowExit: TNotifyEvent;
    FOnDrawDataCell: TDrawDataCellEvent;
    FEditText: string;
    FColWidth: Integer;
    FTitleWidth: Integer;
    FColSizing: TabcHDBGridColSizing;
    FNumCols: Integer;
    function AcquireFocus: Boolean;
    procedure DataChanged;
    procedure EditingChanged;
    function GetDataSource: TDataSource;
    function GetFieldCount: Integer;
    function GetFields(Index: Integer): TField;
    function GetSelectedField: TField;
    function GetSelectedIndex: Integer;
    procedure MoveRow(ARow: Integer);
    procedure RecordChanged(Field: TField);
    procedure SetAlignTitles(Value: TAlignment);
    procedure SetDataSource(Value: TDataSource);
    procedure SetOptions(Value: TabcHDBGridOptions);
    procedure SetSelectedField(Value: TField);
    procedure SetSelectedIndex(Value: Integer);
    procedure SetTitleFont(Value: TFont);
    procedure TitleFontChanged(Sender: TObject);
    procedure UpdateData;
    procedure UpdateActive;
    procedure SetScrollBars(Value: TScrollStyle);
    procedure UpdateScrollBar;
    procedure UpdateColCount;
    procedure CMExit(var Message: TMessage); message CM_EXIT;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMParentFontChanged(var Message: TMessage); message CM_PARENTFONTCHANGED;
    procedure CMDesignHitTest(var Msg: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure WMSetCursor(var Msg: TWMSetCursor); message WM_SETCURSOR;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure SetColWidth(Value: Integer);
    procedure SetTitleWidth(Value: Integer);
    procedure SetColSizing(Value: TabcHDBGridColSizing);
    procedure SetNumCols(Value: Integer);
    procedure UpdateColumnSizing;
  protected
    FUpdateFields: Boolean;
    FAcquireFocus: Boolean;
    procedure LayoutChanged; virtual;
    function CanEditAcceptKey(Key: Char): Boolean; override;
    function CanEditModify: Boolean; override;
    function GetEditLimit: Integer; override;
    procedure RowMoved(FromIndex, ToIndex: Longint); override;
    procedure RowEnter; dynamic;
    procedure RowExit; dynamic;
    procedure Scroll(Distance: Integer); virtual;
    procedure ColWidthsChanged; override;
    function HighlightCell(DataCol, DataRow: Integer; const Value: string;
      AState: TGridDrawState): Boolean; virtual;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
    function GetEditMask(ACol, ARow: Longint): string; override;
    function GetEditText(ACol, ARow: Longint): string; override;
    procedure SetEditText(ACol, ARow: Longint; const Value: string); override;
    function GetRowField(ARow: Integer): TField;
    function GetFieldValue(ARow: Integer): string;
    procedure DefineFieldMap; virtual;
    procedure DrawDataCell(const Rect: TRect; Field: TField;
      State: TGridDrawState); dynamic;
    procedure SetColumnAttributes; virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure LinkActive(Value: Boolean); virtual;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure TimedScroll(Direction: TGridScrollDirection); override;
    procedure CreateWnd; override;
    property AlignTitles: TAlignment read FAlignTitles write SetAlignTitles default taLeftJustify;
    property DefaultDrawing: Boolean read FDefaultDrawing write FDefaultDrawing default True;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DataLink: TabcHGridDataLink read FDataLink;
    property Options: TabcHDBGridOptions read FOptions write SetOptions
      default [hdgEditing, hdgTitles, hdgIndicator, hdgColumnResize, hdgRowResize, hdgColLines,
      hdgRowLines, hdgTabs, hdgConfirmDelete, hdgCancelOnExit];
    property ParentColor default False;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    property ScrollBars: TScrollStyle read FScrollBars write SetScrollBars default ssBoth;
    property TitleFont: TFont read FTitleFont write SetTitleFont;
    property OnRowEnter: TNotifyEvent read FOnRowEnter write FOnRowEnter;
    property OnRowExit: TNotifyEvent read FOnRowExit write FOnRowExit;
    property OnDrawDataCell: TDrawDataCellEvent read FOnDrawDataCell write FOnDrawDataCell;
    {new properties}
    property ColWidth: Integer read FColWidth write SetColWidth default 100;
    property TitleWidth: Integer read FTitleWidth write SetTitleWidth default 100;
    property ColSizing: TabcHDBGridColSizing read FColSizing write SetColSizing default csNumCols;
    property NumCols: Integer read FNumCols write SetNumCols default 1;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DefaultDrawDataCell(const Rect: TRect; Field: TField; State: TGridDrawState);
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    property EditorMode;
    property FieldCount: Integer read GetFieldCount;
    property Fields[Index: Integer]: TField read GetFields;
    property SelectedField: TField read GetSelectedField write SetSelectedField;
    property SelectedIndex: Integer read GetSelectedIndex write SetSelectedIndex;
  end;

  TabcHDBGrid = class(TabcCustomHDBGrid)
  public
    property Canvas;
  published
    property Align;
    property AlignTitles;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
    property BorderStyle;
    property Color;
    property ColSizing;
    property ColWidth;
    property Ctl3D;
    property DataSource;
    property DefaultDrawing;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FixedColor;
    property Font;
    property NumCols;
    property Options;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ScrollBars;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property TitleFont;
    property TitleWidth;
    property Visible;
    property OnRowEnter;
    property OnRowExit;
    property OnDrawDataCell;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

implementation

uses ExtCtrls, DBConsts, Consts, Dialogs
{$IFDEF DELPHI6}
     , VDBConsts
{$ENDIF}
;

{$R abchgr32.res}

const
  bmArrow = 'HDBGARROW';
  bmEdit = 'HDBEDIT';
  bmInsert = 'HDBINSERT';

  MaxMapSize = 65520 div SizeOf(Integer);

{ Error reporting }

procedure RaiseGridError(const S: string);
begin
  raise EInvalidGridOperation.Create(S);
end;

{ TabcHGridDataLink }

type
  TIntArray = array[0..MaxMapSize] of Integer;
  PIntArray = ^TIntArray;

constructor TabcHGridDataLink.Create(AGrid: TabcCustomHDBGrid);
begin
  inherited Create;
  FGrid := AGrid;
  VisualControl := True;
end;

destructor TabcHGridDataLink.Destroy;
begin
  ClearMapping;
  inherited Destroy;
end;

function TabcHGridDataLink.GetDefaultFields: Boolean;
begin
  Result := True;
  if DataSet <> nil then Result := DataSet.DefaultFields;
end;

function TabcHGridDataLink.GetFields(I: Integer): TField;
begin
  if I < FFieldCount then
    Result := DataSet.Fields[PIntArray(FFieldMap)^[I]]
  else
    Result := nil;
end;

function TabcHGridDataLink.AddMapping(const FieldName: string): Boolean;
var
  Field: TField;
  NewSize: Integer;
  NewMap: Pointer;
begin
  if FFieldCount >= MaxMapSize then RaiseGridError(STooManyColumns);
  Field := DataSet.FieldByName(FieldName);
  Result := Field <> nil;
  if Result then
  begin
    if FFieldMapSize = 0 then
    begin
      FFieldMapSize := 8;
      GetMem(FFieldMap, FFieldMapSize * SizeOf(Integer));
    end
    else if FFieldCount = FFieldMapSize then
    begin
      NewSize := FFieldMapSize;
      Inc(NewSize, NewSize);
      if (NewSize > MaxMapSize) or (NewSize < FFieldCount) then
        NewSize := MaxMapSize;
      GetMem(NewMap, NewSize * SizeOf(Integer));
      Move(FFieldMap^, NewMap^, SizeOf(Integer) * FFieldCount);
      FreeMem(FFieldMap, SizeOf(Integer) * FFieldCount);
      FFieldMap := nil;
      FFieldMapSize := NewSize;
      FFieldMap := NewMap;
    end;
    PIntArray(FFieldMap)^[FFieldCount] := Field.Index;
    Inc(FFieldCount);
  end;
end;

procedure TabcHGridDataLink.ActiveChanged;
begin
  FGrid.LinkActive(Active);
  FModified := False;
end;

procedure TabcHGridDataLink.ClearMapping;
begin
  if FFieldMap <> nil then begin
    FreeMem(FFieldMap, FFieldMapSize * SizeOf(Integer));
    FFieldMap := nil;
    FFieldMapSize := 0;
    FFieldCount := 0;
  end;
end;

procedure TabcHGridDataLink.Modified;
begin
  FModified := True;
end;

procedure TabcHGridDataLink.DataSetChanged;
begin
  FGrid.DataChanged;
  FModified := False;
end;

procedure TabcHGridDataLink.DataSetScrolled(Distance: Integer);
begin
  FGrid.Scroll(Distance);
end;

procedure TabcHGridDataLink.LayoutChanged;
begin
  FGrid.LayoutChanged;
end;

procedure TabcHGridDataLink.EditingChanged;
begin
  FGrid.EditingChanged;
end;

procedure TabcHGridDataLink.RecordChanged(Field: TField);
begin
  if (Field = nil) or not FInUpdateData then
  begin
    FGrid.RecordChanged(Field);
    FModified := False;
  end;
end;

procedure TabcHGridDataLink.UpdateData;
begin
  FInUpdateData := True;
  try
    if FModified then FGrid.UpdateData;
    FModified := False;
  finally
    FInUpdateData := False;
  end;
end;

procedure TabcHGridDataLink.Reset;
begin
  if FModified then RecordChanged(nil) else Dataset.Cancel;
end;

{ TabcCustomHDBGrid }

var
  DrawBitmap: TBitmap;
  UserCount: Integer;

procedure UsesBitmap;
begin
  if UserCount = 0 then
  begin
    DrawBitmap := TBitmap.Create;
    DrawBitmap.Monochrome := True;
  end;
  Inc(UserCount);
end;

procedure ReleaseBitmap;
begin
  Dec(UserCount);
  if UserCount = 0 then DrawBitmap.Free;
end;

function Max(X, Y: Integer): Integer;
begin
  Result := Y;
  if X > Y then Result := X;
end;

procedure WriteText(ACanvas: TCanvas; const ARect: TRect; DX, DY: Integer;
  const Text: string; Format: Word);
var
  S: array[0..255] of Char;
  DrawRect, B, R: TRect;
begin
  DrawRect := ARect;
  if (Format and DT_LEFT) = 1 then begin
    with DrawRect do begin
      Inc(Left, DX);
      Inc(Right, DX);
      Inc(Top, DY);
      Inc(Bottom, DY);
    end;
    DrawText(ACanvas.Handle, StrPCopy(S, Text), Length(Text), DrawRect, Format);
  end
  else if (Format and DT_RIGHT) = 1 then begin
    with DrawRect do begin
      Inc(Left, DX);
      Inc(Right, DX);
      Inc(Top, DY);
      Inc(Bottom, DY);
    end;
    DrawText(ACanvas.Handle, StrPCopy(S, Text), Length(Text), DrawRect, Format);
  end
  else
  begin
    with DrawBitmap, ARect do
    begin
      Width := Max(Width, Right - Left);
      Height := Max(Height, Bottom - Top);
      R := Rect(DX, DY, Right - Left - 1, Bottom - Top - 1);
      B := Rect(0, 0, Right - Left, Bottom - Top);
    end;
    with DrawBitmap.Canvas do
    begin
      Font := ACanvas.Font;
      Font.Color := clBlack;
      FillRect(B);
      DrawText(Handle, StrPCopy(S, Text), Length(Text), R, Format);
    end;
    with DrawBitmap, ARect do ACanvas.CopyRect(ARect, Canvas, B);
  end;
end;

procedure TabcCustomHDBGrid.SetNumCols(Value: Integer);
begin
  if Value < 1 then
    FNumCols := 1
  else
    FNumCols := Value;
  UpdateColumnSizing;
end;

procedure TabcCustomHDBGrid.SetColSizing(Value: TabcHDBGridColSizing);
begin
  FColSizing := Value;
  if Value = csNumCols then Options := Options - [hdgColumnResize];
  UpdateColumnSizing;
  if (Value = csBestFit) then UpdateColumnSizing;
end;

procedure TabcCustomHDBGrid.SetColWidth(Value: Integer);
begin
  FColWidth := Value;
  UpdateColumnSizing;
  if (FColSizing = csBestFit) then UpdateColumnSizing;
end;

procedure TabcCustomHDBGrid.SetTitleWidth(Value: Integer);
begin
  FTitleWidth := Value;
  UpdateColumnSizing;
end;

constructor TabcCustomHDBGrid.Create(AOwner: TComponent);
var
  Bmp: TBitmap;
begin
  inherited Create(AOwner);
  inherited DefaultDrawing := False;
  FAcquireFocus := True;
  Bmp := TBitmap.Create;
  try
    Bmp.Handle := LoadBitmap(HInstance, bmArrow);
    FIndicators := TImageList.CreateSize(Bmp.Width, Bmp.Height);
    FIndicators.AddMasked(Bmp, clWhite);
    Bmp.Handle := LoadBitmap(HInstance, bmEdit);
    FIndicators.AddMasked(Bmp, clWhite);
    Bmp.Handle := LoadBitmap(HInstance, bmInsert);
    FIndicators.AddMasked(Bmp, clWhite);
  finally
    Bmp.Free;
  end;
  FTitleOffset := 1;
  FIndicatorOffset := 1;
  FUpdateFields := True;
  FOptions := [hdgEditing, hdgTitles, hdgIndicator, hdgColumnResize,
    hdgRowResize, hdgColLines, hdgRowLines, hdgTabs, hdgConfirmDelete,
    hdgCancelOnExit, hdgWordWrap];
  UsesBitmap;
  inherited ScrollBars := ssVertical;
  FScrollBars := ssBoth;
  inherited Options := [goFixedHorzLine, goFixedVertLine, goHorzLine,
    goVertLine, goColSizing, goRowSizing, goRowMoving, goTabs, goEditing];
  inherited RowCount := 2;
  inherited ColCount := 2;
  FDataLink := TabcHGridDataLink.Create(Self);
  Color := clWindow;
  ParentColor := False;
  FTitleFont := TFont.Create;
  FTitleFont.OnChange := TitleFontChanged;
  FSaveCellExtents := False;
  FDefaultDrawing := True;
  FAlignTitles := taLeftJustify;
  HideEditor;
  FColWidth := 100;
  DefaultColWidth := 100;
  FTitleWidth := 100;
  FNumCols := 1;
  FColSizing := csNumCols;
end;

destructor TabcCustomHDBGrid.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FIndicators.Free;
  FTitleFont.Free;
  inherited Destroy;
  ReleaseBitmap;
end;

procedure TabcCustomHDBGrid.DefineFieldMap;
var
  I: Integer;
begin
  with FDatalink.Dataset do
    for I := 0 to FieldCount - 1 do
      with Fields[I] do if Visible then FDatalink.AddMapping(Fieldname);
end;

procedure TabcCustomHDBGrid.DrawDataCell(const Rect: TRect; Field: TField;
  State: TGridDrawState);
begin
  if Assigned(FOnDrawDataCell) then FOnDrawDataCell(Self, Rect, Field, State);
end;

procedure TabcCustomHDBGrid.UpdateColumnSizing;
begin
  FUpdatingColWidths := True;
  SetColumnAttributes;
  UpdateColCount;
  FUpdatingColWidths := False;
end;

procedure TabcCustomHDBGrid.SetColumnAttributes;
var
  I: Integer;
  MaxCWidth, CWidth: Integer;
  MaxLblWidth, LblWidth: Integer;
  MaxVisibleWidth: Integer;
  CalcColWidth, CalcCols, Remainder: Integer;

  function GetVScrollVisibleWidth: Integer;
  var
    Min, Max: Integer;
  begin
    Result := 0;
    if ((ScrollBars = ssBoth) or (ScrollBars = ssVertical)) and
    HandleAllocated then begin
      GetScrollRange(Handle, SB_VERT, Min, Max);
      if Min <> Max then Result := GetSystemMetrics(SM_CXVSCROLL);
    end;
  end;

begin
  MaxVisibleWidth := Width - 4 - GetVScrollVisibleWidth;
  if (hdgTitles in Options) then
    MaxVisibleWidth := MaxVisibleWidth - FTitleWidth;
  if (FColWidth > MaxVisibleWidth) and not (FColSizing in [csBestFit, csExact]) then
    FColWidth := MaxVisibleWidth;
  if MaxVisibleWidth < 1 then
    Exit;

  case FColSizing of
    csNumCols:
      begin
        if FNumCols = 1 then begin
          DefaultColWidth := MaxVisibleWidth;
          if (hdgTitles in Options) then
            ColWidths[0] := FTitleWidth;
        end
        else begin
          CalcColWidth := MaxVisibleWidth div FNumCols;
          if CalcColWidth < 1 then Exit;
          DefaultColWidth := CalcColWidth - Ord(hdgColLines in Options);
          if (hdgTitles in Options) then
            ColWidths[0] := FTitleWidth;
          Remainder := MaxVisibleWidth - CalcColWidth * NumCols - Ord(hdgColLines in Options);
          if (CalcColWidth + Remainder) > 0 then
            ColWidths[FTitleOffset] := CalcColWidth + Remainder;
        end;
      end;
    csBestFit:
      begin
        CalcColWidth := FColWidth + 1;
        CalcCols := MaxVisibleWidth div CalcColWidth;
        if CalcCols < 2 then begin
          DefaultColWidth := MaxVisibleWidth;
          if (hdgTitles in Options) then
            ColWidths[0] := FTitleWidth;
        end
        else begin
          DefaultColWidth := CalcColWidth - 1;
          if (hdgTitles in Options) then
            ColWidths[0] := FTitleWidth;
          Remainder := MaxVisibleWidth - (CalcColWidth+1) * CalcCols +1;
          if (CalcColWidth + Remainder) > 0 then
            ColWidths[FTitleOffset] := CalcColWidth + Remainder
          else
            ColWidths[FTitleOffset] := CalcColWidth;
        end;
      end;
(*
      begin
        CalcColWidth := FColWidth + 1;
        CalcCols := MaxVisibleWidth div CalcColWidth;
        if CalcCols < 2 then begin
          DefaultColWidth := MaxVisibleWidth;
          if (hdgTitles in Options) then
            ColWidths[0] := FTitleWidth;
        end
        else begin
          DefaultColWidth := CalcColWidth - 1;
          if (hdgTitles in Options) then
            ColWidths[0] := FTitleWidth;
          Remainder := MaxVisibleWidth - CalcColWidth * CalcCols + 1;
          if Remainder > 0 then
            ColWidths[FTitleOffset] := CalcColWidth + Remainder
          else
            ColWidths[FTitleOffset] := CalcColWidth;
        end;
      end;
*)
    csExact:
      begin
        DefaultColWidth := FColWidth;
        if (hdgTitles in Options) then
          ColWidths[0] := FTitleWidth;
      end;
    csField:
      begin
        MaxCWidth := 0;
        MaxLblWidth := 0;
        for I := 0 to FieldCount - 1 do
          with Fields[I] do
          begin
            Canvas.Font := Font;
            CWidth := DisplayWidth * Canvas.TextWidth('0') + 4;
            if CWidth > MaxCWidth then MaxCWidth := CWidth;
            if hdgTitles in Options then begin
              Canvas.Font := TitleFont;
              LblWidth := Canvas.TextWidth(DisplayLabel) + 4;
              if LblWidth > MaxLblWidth then MaxLblWidth := LblWidth;
            end;
          end;
        if MaxCWidth > MaxVisibleWidth then
          MaxCWidth := MaxVisibleWidth;
        if MaxCWidth <> 0 then DefaultColWidth := MaxCWidth;
        if (hdgTitles in Options) and (MaxLblWidth <> 0) then
          ColWidths[0] := MaxLblWidth;
      end;
  end;
end;

procedure TabcCustomHDBGrid.LayoutChanged;
var
  J: Integer;
begin
  if csLoading in ComponentState then Exit;
  if not HandleAllocated then Exit;
  if FUpdateLock <> 0 then Exit;
  Inc(FUpdateLock);
  try
    FUpdatingColWidths := True;
    try
      FTitleOffset := 0;
      if hdgTitles in Options then FTitleOffset := 1;
      Canvas.Font := Font;
      DefaultRowHeight := Canvas.TextHeight('W') + 2;
      if hdgRowLines in Options then
        DefaultRowHeight := DefaultRowHeight + 1;
      if hdgIndicator in Options then
      begin
        Canvas.Font := FTitleFont;
        RowHeights[0] := Canvas.TextHeight('W') + 4;
      end;
      FIndicatorOffset := 0;
      if hdgIndicator in Options then FIndicatorOffset := 1;
      FDatalink.ClearMapping;
      if FDatalink.Active then DefineFieldMap;
      J := FieldCount;
      if J = 0 then J := 1;
      inherited RowCount := J + FIndicatorOffset;
      inherited FixedRows := FIndicatorOffset;
      SetColumnAttributes;
      UpdateColCount;
    finally
      FUpdatingColWidths := False;
    end;
    UpdateActive;
    Invalidate;
  finally
    Dec(FUpdateLock);
  end;
end;

procedure TabcCustomHDBGrid.LinkActive(Value: Boolean);
begin
  if not Value then HideEditor;
  LayoutChanged;
  UpdateScrollBar;
  if Value and (hdgAlwaysShowEditor in Options) then ShowEditor;
end;

procedure TabcCustomHDBGrid.CreateWnd;
begin
  inherited CreateWnd;
  LayoutChanged;
  UpdateScrollBar;
end;

function TabcCustomHDBGrid.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TabcCustomHDBGrid.CanEditAcceptKey(Key: Char): Boolean;
begin
  Result := FDatalink.Active and (FieldCount > 0) and
    Fields[SelectedIndex].IsValidChar(Key);
end;

function TabcCustomHDBGrid.GetEditLimit: Integer;
begin
  Result := 0;
  if (FieldCount > 0) and (SelectedField is TStringField) then
    Result := TStringField(SelectedField).Size;
end;

function TabcCustomHDBGrid.CanEditModify: Boolean;
begin
  Result := False;
  if not ReadOnly and FDatalink.Active and not FDatalink.Readonly and
    (FieldCount > 0) and Fields[SelectedIndex].CanModify then
  begin
    FDatalink.Edit;
    Result := FDatalink.Editing;
    if Result then FDatalink.Modified;
  end;
end;

function TabcCustomHDBGrid.GetEditMask(ACol, ARow: Longint): string;
begin
  Result := '';
  if FDatalink.Active and (ARow - FIndicatorOffset < FieldCount) then
    Result := Fields[ARow - FIndicatorOffset].EditMask;
end;

function TabcCustomHDBGrid.GetEditText(ACol, ARow: Longint): string;
begin
  Result := '';
  if FDatalink.Active and (ARow - FIndicatorOffset < FieldCount) then
    Result := Fields[ARow - FIndicatorOffset].Text;
  FEditText := Result;
end;

procedure TabcCustomHDBGrid.SetEditText(ACol, ARow: Longint; const Value: string);
begin
  FEditText := Value;
end;

function TabcCustomHDBGrid.GetFieldCount: Integer;
begin
  Result := FDatalink.FieldCount;
end;

function TabcCustomHDBGrid.GetFields(Index: Integer): TField;
begin
  Result := FDatalink.Fields[Index];
end;

procedure TabcCustomHDBGrid.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  LinkActive(FDataLink.Active);
end;

function TabcCustomHDBGrid.GetSelectedField: TField;
begin
  Result := nil;
  if SelectedIndex < FieldCount then Result := Fields[SelectedIndex];
end;

function TabcCustomHDBGrid.GetSelectedIndex: Integer;
begin
  Result := Row - FIndicatorOffset;
end;

procedure TabcCustomHDBGrid.SetSelectedField(Value: TField);
var
  I: Integer;
begin
  for I := 0 to FieldCount - 1 do
    if Fields[I] = Value then SelectedIndex := I;
end;

procedure TabcCustomHDBGrid.SetSelectedIndex(Value: Integer);
begin
  MoveRow(Value);
end;

procedure TabcCustomHDBGrid.DataChanged;
begin
  if not HandleAllocated then Exit;
  UpdateColCount;
  UpdateScrollBar;
  UpdateActive;
  LayoutChanged; {debug}
  InvalidateEditor;
  ValidateRect(Handle, nil);
  Invalidate;
end;

procedure TabcCustomHDBGrid.EditingChanged;
begin
  if hdgIndicator in Options then InvalidateCell(FSelCol, 0);
end;

procedure TabcCustomHDBGrid.RecordChanged(Field: TField);
var
  R: TRect;
  InvBegin, InvEnd: Integer;
begin
  if not HandleAllocated then Exit;
  InvBegin := 0;
  if Field = nil then InvEnd := RowCount - 1 else
  begin
    for InvBegin := 0 to FieldCount - 1 do
      if Fields[InvBegin] = Field then Break;
    InvEnd := InvBegin;
  end;
  R := BoxRect(Col, InvBegin + FIndicatorOffset, Col, InvEnd + FIndicatorOffset);
  InvalidateRect(Handle, @R, False);
  if ((Field = nil) or (SelectedField = Field)) and
    (Fields[SelectedIndex].Text <> FEditText) then
  begin
    InvalidateEditor;
    if InplaceEditor <> nil then InplaceEditor.Deselect;
  end;
end;

procedure TabcCustomHDBGrid.Scroll(Distance: Integer);
var
  OldRect, NewRect: TRect;
begin
  OldRect := BoxRect(Col, 0, Col, RowCount - 1);
  UpdateScrollBar;
  UpdateActive;
  NewRect := BoxRect(Col, 0, Col, RowCount - 1);
  if not (FColSizing in [csBestFit, csNumCols]) then
  begin
    ValidateRect(Handle, @OldRect);
    InvalidateRect(Handle, @OldRect, False);
    InvalidateRect(Handle, @NewRect, False);
  end;
  if Distance <> 0 then
  begin
    HideEditor;
    try
      if (Abs(Distance) > VisibleColCount) or (FColSizing in [csBestFit, csNumCols]) then
      begin
        Invalidate;
        Exit;
      end
      else
      begin
        NewRect := BoxRect(FTitleOffset, FIndicatorOffset, 1000, RowCount - 1);
        ScrollWindow(Handle, -ColWidth * Distance, 0, @NewRect, @NewRect);
        if hdgIndicator in Options then
        begin
          OldRect := BoxRect(FSelCol, 0, FSelCol, RowCount - 1);
          InvalidateRect(Handle, @OldRect, False);
          NewRect := BoxRect(Col, 0, Col, RowCount - 1);
          InvalidateRect(Handle, @NewRect, False);
        end;
      end;
    finally
      if hdgAlwaysShowEditor in Options then ShowEditor;
    end;
  end;
  Update;
end;

procedure TabcCustomHDBGrid.TitleFontChanged(Sender: TObject);
begin
  if (not FSelfChangingTitleFont) and not (csLoading in ComponentState) then
    ParentFont := False;
  if hdgTitles in Options then LayoutChanged;
end;

procedure TabcCustomHDBGrid.UpdateData;
begin
  if FieldCount > 0 then with Fields[SelectedIndex] do Text := FEditText;
end;

procedure TabcCustomHDBGrid.UpdateActive;
var
  NewCol: Integer;
begin
  if FDatalink.Active then
  begin
    NewCol := FDatalink.ActiveRecord + FTitleOffset;
    if Col <> NewCol then
    begin
      if not (hdgAlwaysShowEditor in Options) then HideEditor;
      Col := NewCol;
    end;
    if (FieldCount > 0) and (Fields[SelectedIndex].Text <> FEditText) then
      InvalidateEditor;
  end;
end;

function TabcCustomHDBGrid.GetRowField(ARow: Integer): TField;
begin
  Result := nil;
  if (ARow >= 0) and FDatalink.Active and (ARow < FDataLink.FieldCount) then
    Result := FDatalink.Fields[ARow];
end;

function TabcCustomHDBGrid.GetFieldValue(ARow: Integer): string;
var
  Field: TField;
begin
  Result := '';
  Field := GetRowField(ARow);
  if Field <> nil then Result := Field.DisplayText;
end;

procedure TabcCustomHDBGrid.UpdateScrollBar;
var
  Pos: Integer;
begin
  if FDatalink.Active and HandleAllocated then
    if (FScrollBars in [ssBoth, ssHorizontal]) then
      with FDatalink.DataSet do begin
        SetScrollRange(Self.Handle, SB_HORZ, 0, 4, False);
        if BOF then Pos := 0
        else if EOF then Pos := 4
        else Pos := 2;
        if GetScrollPos(Self.Handle, SB_HORZ) <> Pos then
          SetScrollPos(Self.Handle, SB_HORZ, Pos, True);
      end
    else
      {hide the scroll bar}
      SetScrollRange(Self.Handle, SB_HORZ, 0, 0, True);
end;

procedure TabcCustomHDBGrid.UpdateColCount;
begin
  if ColCount <= FTitleOffset then ColCount := FTitleOffset + 1;
  FixedCols := FTitleOffset;
  with FDataLink do
    if not Active or (RecordCount = 0) then
      ColCount := 1 + FTitleOffset
    else
    begin
      ColCount := 1000;
      FDataLink.BufferCount := VisibleColCount;
      ColCount := RecordCount + FTitleOffset;
      UpdateActive;
    end;
end;

function TabcCustomHDBGrid.AcquireFocus: Boolean;
begin
  Result := True;
  if FAcquireFocus and CanFocus and not (csDesigning in ComponentState) then
  begin
    SetFocus;
    Result := Focused or (InplaceEditor <> nil) and InplaceEditor.Focused;
  end;
end;

procedure TabcCustomHDBGrid.CMParentFontChanged(var Message: TMessage);
begin
  inherited;
  if ParentFont then
  begin
    FSelfChangingTitleFont := True;
    try
      TitleFont := Font;
    finally
      FSelfChangingTitleFont := False;
    end;
    LayoutChanged;
  end;
end;

procedure TabcCustomHDBGrid.CMExit(var Message: TMessage);
begin
  try
    if FDatalink.Active then
      with FDatalink.Dataset do
        if (hdgCancelOnExit in Options) and (State = dsInsert) and
          not Modified and not FDatalink.FModified then
          Cancel else
          FDataLink.UpdateData;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

procedure TabcCustomHDBGrid.CMFontChanged(var Message: TMessage);
begin
  inherited;
  LayoutChanged;
end;

procedure TabcCustomHDBGrid.CMDesignHitTest(var Msg: TCMDesignHitTest);
begin
  inherited;
  if Msg.Result = 0 then
    with MouseCoord(Msg.Pos.X, Msg.Pos.Y) do
      if (X >= FTitleOffset) and (Y < FIndicatorOffset) then Msg.Result := 1;
  if (Msg.Result = 1) and ((FDataLink = nil) or FDataLink.DefaultFields or
    not FDataLink.Active) then
    Msg.Result := 0;
end;

procedure TabcCustomHDBGrid.WMSetCursor(var Msg: TWMSetCursor);
begin
  if (csDesigning in ComponentState) and ((FDataLink = nil) or
    FDataLink.DefaultFields or not FDataLink.Active) then
    Windows.SetCursor(LoadCursor(0, IDC_ARROW))
  else inherited;
end;

procedure TabcCustomHDBGrid.WMSize(var Message: TWMSize);
begin
  inherited;
  if FUpdateLock = 0 then LayoutChanged;
end;

procedure TabcCustomHDBGrid.WMHScroll(var Message: TWMHScroll);
begin
  if not AcquireFocus then Exit;
  if FDatalink.Active then
    with Message, FDataLink.DataSet, FDatalink do
      case ScrollCode of
        SB_LINEUP: MoveBy(-ActiveRecord - 1);
        SB_LINEDOWN: MoveBy(RecordCount - ActiveRecord);
        SB_PAGEUP: MoveBy(-VisibleColCount);
        SB_PAGEDOWN: MoveBy(VisibleColCount);
        SB_THUMBPOSITION:
          begin
            case Pos of
              0: First;
              1: MoveBy(-VisibleColCount);
              2: Exit;
              3: MoveBy(VisibleColCount);
              4: Last;
            end;
          end;
        SB_BOTTOM: Last;
        SB_TOP: First;
      end;
end;

function TabcCustomHDBGrid.HighlightCell(DataCol, DataRow: Integer;
  const Value: string; AState: TGridDrawState): Boolean;
begin
  Result := (gdSelected in AState) and ((hdgAlwaysShowSelection in Options) or
    Focused);
end;

procedure TabcCustomHDBGrid.DefaultDrawDataCell(const Rect: TRect; Field: TField;
  State: TGridDrawState);
const
  Formats: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT,
    DT_CENTER or DT_WORDBREAK or DT_EXPANDTABS or DT_NOPREFIX);
var
  Alignment: TAlignment;
  Value: string;
  DFormat: Word;
begin
  Alignment := taLeftJustify;
  Value := '';
  if Field <> nil then
  begin
    Alignment := Field.Alignment;
    Value := Field.DisplayText;
  end;
  if (hdgWordWrap in Options) then
    DFormat := Formats[Alignment] or DT_WORDBREAK
  else
    DFormat := Formats[Alignment];

  WriteText(Canvas, Rect, 2, 2, Value, DFormat);
end;

procedure TabcCustomHDBGrid.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
var
  OldActive: Integer;
  Indicator: Integer;
  Highlight: Boolean;
  Value: string;

  procedure Display(const S: string; Alignment: TAlignment);
  const
    Formats: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT,
      DT_CENTER or DT_WORDBREAK or DT_EXPANDTABS or DT_NOPREFIX);
  begin
    WriteText(Canvas, ARect, 2, 2, S, Formats[Alignment]);
  end;

begin
  if gdFixed in AState then
    Canvas.Font := TitleFont else
    Canvas.Font := Font;
  with Canvas do
  begin
    if gdFixed in AState then
      Brush.Color := FixedColor
    else
      Brush.Color := Color;
    Dec(ARow, FIndicatorOffset);
    Dec(ACol, FTitleOffset);
    if ACol < 0 then begin
      if (ARow >= 0) and (ARow < FieldCount) then
        Display(Fields[ARow].DisplayLabel, FAlignTitles)
      else
        Display('', taLeftJustify);
    end
    else if (FDataLink = nil) or not FDataLink.Active then
      FillRect(ARect)
    else if ARow < 0 then
    begin
      FillRect(ARect);
      if ACol = FDataLink.ActiveRecord then
      begin
        Indicator := 0;
        if FDataLink.DataSet <> nil then
          case FDataLink.DataSet.State of
            dsEdit: Indicator := 1;
            dsInsert: Indicator := 2;
          end;
        FIndicators.Draw(Canvas, (ARect.Left + ARect.Right - FIndicators.Width) shr 1,
          (ARect.Top + ARect.Bottom - FIndicators.Height) shr 1, Indicator);
        FSelCol := ACol + FTitleOffset;
      end;
    end
    else
    begin
      Value := '';
      OldActive := FDataLink.ActiveRecord;
      try
        FDataLink.ActiveRecord := ACol;
        Value := GetFieldValue(ARow);
        Highlight := HighlightCell(ACol, ARow, Value, AState);
        if Highlight then
        begin
          Brush.Color := clHighlight;
          Font.Color := clHighlightText;
        end;
        if FDefaultDrawing then
          DefaultDrawDataCell(ARect, GetRowField(ARow), AState);
        DrawDataCell(ARect, GetRowField(ARow), AState);
      finally
        FDataLink.ActiveRecord := OldActive;
      end;
      if FDefaultDrawing and Highlight and not (csDesigning in ComponentState) and
        (ValidParentForm(Self).ActiveControl = Self) then
        Windows.DrawFocusRect(Handle, ARect);
    end;
    if (gdFixed in AState) and ([hdgRowLines, hdgColLines] * Options =
      [hdgRowLines, hdgColLines]) then
      with ARect do
      begin
        Pen.Color := clBtnHighlight;
        PolyLine([Point(Left, Bottom - 1), Point(Left, Top), Point(Right, Top)]);
      end;
  end;
end;

procedure TabcCustomHDBGrid.MoveRow(ARow: Integer);
var
  OldRow: Integer;
begin
  FDatalink.UpdateData;
  if ARow >= FieldCount then ARow := FieldCount - 1;
  if ARow < 0 then ARow := 0;
  OldRow := Row - FIndicatorOffset;
  if ARow <> OldRow then
  begin
    if not FInRowExit then
    begin
      FInRowExit := True;
      try
        RowExit;
      finally
        FInRowExit := False;
      end;
      if Row - FIndicatorOffset <> OldRow then Exit;
    end;
    if not (hdgAlwaysShowEditor in Options) then HideEditor;
    Row := ARow + FIndicatorOffset;
    RowEnter;
  end;
end;

procedure TabcCustomHDBGrid.KeyDown(var Key: Word; Shift: TShiftState);
var
  KeyDownEvent: TKeyEvent;

  procedure NextCol;
  begin
    with FDatalink.Dataset do
    begin
      if (State = dsInsert) and not Modified and not FDatalink.FModified then
        if EOF then Exit else Cancel
      else Next;
      if EOF and CanModify and (hdgEditing in Options) then
        Append;
    end;
  end;

  procedure PriorCol;
  begin
    with FDatalink.Dataset do
      if (State = dsInsert) and not Modified and EOF and
        not FDatalink.FModified then
        Cancel else
        Prior;
  end;

  procedure Tab(GoForward: Boolean);
  var
    ARow, Original: Integer;
  begin
    ARow := SelectedIndex;
    Original := ARow;
    while True do
    begin
      if GoForward then
        Inc(ARow)
      else
        Dec(ARow);
      if ARow >= FieldCount then
      begin
        NextCol;
        ARow := 0;
      end
      else
        if ARow < 0 then begin
          PriorCol;
          ARow := FieldCount - 1;
        end;
      if ARow = Original then Exit;
      if TabStops[ARow + FIndicatorOffset] then
      begin
        MoveRow(ARow);
        Exit;
      end;
    end;
  end;

  function DeletePrompt: Boolean;
  begin
    Result := not (hdgConfirmDelete in Options) or
      (MessageDlg(SDeleteRecordQuestion, mtConfirmation, mbOKCancel,
        0) <> idCancel);
  end;

begin
  KeyDownEvent := OnKeyDown;
  if Assigned(KeyDownEvent) then KeyDownEvent(Self, Key, Shift);
  if not FDatalink.Active or not CanGridAcceptKey(Key, Shift) then Exit;
  with FDatalink.DataSet do
    if ssCtrl in Shift then
      case Key of
        VK_UP: MoveRow(0);
        VK_DOWN: MoveRow(FieldCount - 1);
        VK_LEFT, VK_PRIOR:  MoveBy(-FDatalink.ActiveRecord);
        VK_RIGHT, VK_NEXT: MoveBy(FDatalink.BufferCount - FDatalink.ActiveRecord - 1);
        VK_HOME: begin
          First;
          MoveRow(0);
        end;
        VK_END: begin
          Last;
          MoveRow(FieldCount - 1);
        end;
        VK_DELETE: if not ReadOnly and CanModify and DeletePrompt then Delete;
      end
    else
      case Key of
        VK_LEFT: PriorCol;
        VK_RIGHT: NextCol;
        VK_UP:
            MoveRow(SelectedIndex - 1);
        VK_DOWN:
            MoveRow(SelectedIndex + 1);
        VK_HOME:
          if SelectedIndex > 0 then
            MoveRow(0)
          else
            First;
        VK_END:
          if SelectedIndex < (FieldCount - 1) then
            MoveRow(FieldCount - 1)
          else
            Last;
        VK_NEXT:
          if SelectedIndex < (FieldCount - 1) then
            MoveRow(SelectedIndex + VisibleRowCount)
          else
            NextCol;
        VK_PRIOR:
          if SelectedIndex > 0 then
            MoveRow(SelectedIndex - VisibleRowCount)
          else
            PriorCol;
        VK_INSERT: if not ReadOnly then Insert;
        VK_TAB: if not (ssAlt in Shift) then Tab(not (ssShift in Shift));
        VK_ESCAPE:
          begin
            FDatalink.Reset;
            if not (hdgAlwaysShowEditor in Options) then HideEditor;
          end;
        VK_F2: EditorMode := True;
      end;
end;

procedure TabcCustomHDBGrid.KeyPress(var Key: Char);
begin
  if not (hdgAlwaysShowEditor in Options) and (Key = #13) then
    FDatalink.UpdateData;
  inherited KeyPress(Key);
end;

procedure TabcCustomHDBGrid.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  Cell: TGridCoord;
begin
  if not AcquireFocus then Exit;
  if (ssDouble in Shift) and (Button = mbLeft) then
  begin
    DblClick;
    Exit;
  end;
  if Sizing(X, Y) then
    inherited MouseDown(Button, Shift, X, Y)
  else
  begin
    Cell := MouseCoord(X, Y);
    if (((csDesigning in ComponentState) or (hdgColumnResize in Options)) and
      (Cell.Y < FIndicatorOffset))
      or ((hdgRowResize in Options) and (Cell.X < FTitleOffset))
      then inherited MouseDown(Button, Shift, X, Y)
    else
      if FDatalink.Active then
        with Cell do
        begin
          if (X = Col) and (Y = Row) then ShowEditor;
          if Y >= FIndicatorOffset then MoveRow(Y - FIndicatorOffset);
          if X >= FTitleOffset then
            if X - Col <> 0 then FDatalink.Dataset.MoveBy(X - Col);
        end;
  end;
end;

procedure TabcCustomHDBGrid.RowEnter;
begin
  if Assigned(FOnRowEnter) then FOnRowEnter(Self);
end;

procedure TabcCustomHDBGrid.RowExit;
begin
  if Assigned(FOnRowExit) then FOnRowExit(Self);
end;

procedure TabcCustomHDBGrid.RowMoved(FromIndex, ToIndex: Longint);
begin
  if FDatalink.Active and (FieldCount > 0) then
    Fields[FromIndex - FIndicatorOffset].Index := Fields[ToIndex - FIndicatorOffset].Index;
end;

procedure TabcCustomHDBGrid.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
      (AComponent = DataSource)  then
    DataSource := nil;
end;

procedure TabcCustomHDBGrid.TimedScroll(Direction: TGridScrollDirection);
begin
  if FDatalink.Active then
  begin
    with FDatalink do
    begin
      if sdLeft in Direction then
      begin
        DataSet.MoveBy(-ActiveRecord - 1);
        Exclude(Direction, sdLeft);
      end;
      if sdRight in Direction then
      begin
        DataSet.MoveBy(RecordCount - ActiveRecord);
        Exclude(Direction, sdRight);
      end;
    end;
    if Direction <> [] then inherited TimedScroll(Direction);
  end;
end;

function TabcCustomHDBGrid.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := (DataLink <> nil) and DataLink.ExecuteAction(Action);
end;

function TabcCustomHDBGrid.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := (DataLink <> nil) and DataLink.UpdateAction(Action);
end;

procedure TabcCustomHDBGrid.ColWidthsChanged;
var
  I: Integer;
  CWidth: Integer;
  ParentForm: TCustomForm;
begin
  inherited ColWidthsChanged;
  if not FUpdatingColWidths and HandleAllocated then
    case FColSizing of
      csNumCols:
        {do nothing};
      csBestFit, csExact:
        begin
          SetColWidth(ColWidths[FTitleOffset]);
          LayoutChanged;
        end;
      csField:
        if FUpdateFields and FDatalink.Active then begin
          Inc(FUpdateLock);
          try
            Canvas.Font := Font;
            CWidth := Canvas.TextWidth('0');
            for I := 0 to FieldCount - 1 do
              Fields[I].DisplayWidth := (ColWidths[FTitleOffset] +
                CWidth div 2 - 3) div CWidth;
            ParentForm := GetParentForm(Self);
            if (ParentForm <> nil) and (ParentForm.Designer <> nil) then
              ParentForm.Designer.Modified;
          finally
            Dec(FUpdateLock);
          end;
          LayoutChanged;
        end;
    end;
end;

procedure TabcCustomHDBGrid.Loaded;
begin
  inherited Loaded;
  {LayoutChanged;}
  SetColWidth(FColWidth);
end;

procedure TabcCustomHDBGrid.SetOptions(Value: TabcHDBGridOptions);
const
  LayoutOptions = [hdgEditing, hdgAlwaysShowEditor, hdgTitles, hdgIndicator,
    hdgColumnResize, hdgColLines, hdgRowLines, hdgAlwaysShowSelection, hdgWordWrap];
var
  NewGridOptions: TGridOptions;
  ChangedOptions: TabcHDBGridOptions;
begin
  if FOptions <> Value then
  begin
    NewGridOptions := [];
    if hdgColumnResize in Value then begin
      Include(NewGridOptions, goColSizing);
      if FColSizing = csNumCols then
        FColSizing := csExact;
    end;
    if hdgRowResize in Value then
      NewGridOptions := NewGridOptions + [goRowSizing, goRowMoving];
    if hdgColLines in Value then
      NewGridOptions := NewGridOptions + [goFixedVertLine, goVertLine];
    if hdgRowLines in Value then
      NewGridOptions := NewGridOptions + [goFixedHorzLine, goHorzLine];
    if hdgTabs in Value then Include(NewGridOptions, goTabs);
    if hdgEditing in Value then Include(NewGridOptions, goEditing);
    if hdgAlwaysShowEditor in Value then Include(NewGridOptions, goAlwaysShowEditor);
    inherited Options := NewGridOptions;
    ChangedOptions := (FOptions + Value) - (FOptions * Value);
    FOptions := Value;
    if ChangedOptions * LayoutOptions <> [] then LayoutChanged;
    if ChangedOptions * [hdgTitles]  <> [] then UpdateColumnSizing;
  end;
end;

procedure TabcCustomHDBGrid.SetTitleFont(Value: TFont);
begin
  FTitleFont.Assign(Value);
  LayoutChanged;
end;

procedure TabcCustomHDBGrid.SetScrollBars(Value: TScrollStyle);
begin
  if Value <> FScrollBars then begin
    FScrollBars := Value;
    if Value in [ssBoth, ssVertical] then
      inherited ScrollBars := ssVertical
    else
      inherited ScrollBars := ssNone;
    UpdateScrollBar;
  end;
end;

procedure TabcCustomHDBGrid.SetAlignTitles(Value: TAlignment);
begin
  if Value <> FAlignTitles then begin
    FAlignTitles := Value;
    Invalidate;
  end;
end;

end.
