unit abchighl;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, Classes, Controls, Graphics, StdCtrls, ExtCtrls, TypInfo;

type

  TabcFocusChangedEvent = procedure (Sender: TObject; FocusControl: TWinControl) of object;

  TabcActiveHighlight = class(TGraphicControl)
  private
    FActiveControl: TWinControl;
    FColorChange: Boolean;
    FHighlightColor: TColor;
    FMoveCursor: Boolean;
    FOrigColor: TColor;
    FOrigParentColor: Boolean;
    FBitmap: TBitmap;
    FOnFocusChanged: TabcFocusChangedEvent;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
    function GetActiveColorByName: TColor;
    function GetActiveParentColorByName: Boolean;
    function GetCursor: TCursor;
    function GetHeight: integer;
    function GetHint: string;
    function GetWidth: integer;
    procedure SetActiveColorByName(Value: TColor);
    procedure SetActiveParentColorByName(Value: Boolean);
    procedure SetColorChange(Value: Boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetMoveCursor(Value: Boolean);
  protected
    procedure DoHighlightControl(AWinControl: TWinControl); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
    procedure UndoHighlightControl; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ActiveControl: TWinControl read FActiveControl;
  published
    property ColorChange: Boolean read FColorChange write SetColorChange default True;
    property Enabled;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor default clAqua;
    property MoveCursor: Boolean read FMoveCursor write SetMoveCursor default True;
    property OnFocusChanged: TabcFocusChangedEvent read FOnFocusChanged write FOnFocusChanged;
    {hide inherited properties from object inspector}
    property Cursor: TCursor read GetCursor;
    property Hint: string read GetHint;
    property Height: integer read GetHeight;
    property Width: integer read GetWidth;
  end;

implementation

{$R snap32.res}

{TabcActiveHighlight}

constructor TabcActiveHighlight.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Visible := False;
  inherited Width := 28;
  inherited Height := 28;
  FColorChange := True;
  FMoveCursor := True;
  FHighlightColor := clAqua;
end;

destructor TabcActiveHighlight.Destroy;
begin
  FBitmap.Free;
  inherited destroy;
end;

procedure TabcActiveHighlight.DoHighlightControl(AWinControl: TWinControl);
var
  Pos: TPoint;
begin
  if FActiveControl <> nil then UndoHighlightControl;
  FActiveControl := AWinControl;
  if Enabled and (AWinControl <> nil) then
    with AWinControl do begin
      if FColorChange then begin
        FOrigColor := GetActiveColorByName;
        FOrigParentColor := GetActiveParentColorByName;
        SetActiveColorByName(FHighlightColor);
      end;
      if FMoveCursor then begin
        if (AWinControl is TRadioButton) or (AWinControl is TCheckBox) then
          Pos := Point(10, Height div 2)
        else if (AWinControl is TComboBox) or (AWinControl is TListBox) then
          Pos := Point(Width - 10, Height div 2)
        else
          Pos := Point(Width div 2, Height div 2);
        Pos := AWinControl.ClientToScreen(Pos);

        SetCursorPos(Pos.X, Pos.Y);
      end;
    end;
end;

function TabcActiveHighlight.GetActiveColorByName: TColor;
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(FActiveControl.ClassInfo, 'Color');
  if PropInfo <> nil then
    Result := TColor(GetOrdProp(FActiveControl, PropInfo))
  else
    Result := clWhite;
end;

function TabcActiveHighlight.GetActiveParentColorByName: Boolean;
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(FActiveControl.ClassInfo, 'ParentColor');
  if PropInfo <> nil then
    Result := Boolean(GetOrdProp(FActiveControl, PropInfo))
  else
    Result := False;
end;

procedure TabcActiveHighlight.SetActiveColorByName(Value: TColor);
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(FActiveControl.ClassInfo, 'Color');
  if PropInfo <> nil then SetOrdProp(FActiveControl, PropInfo, Value);
end;

procedure TabcActiveHighlight.SetActiveParentColorByName(Value: Boolean);
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(FActiveControl.ClassInfo, 'ParentColor');
  if PropInfo <> nil then SetOrdProp(FActiveControl, PropInfo, Longint(Value));
end;

procedure TabcActiveHighlight.CMFocusChanged(var Message: TCMFocusChanged);
begin
  inherited;
  with Message do begin
    DoHighlightControl(Sender);
    if Assigned(FOnFocusChanged) then FOnFocusChanged(Self, Sender);
  end;
end;

function TabcActiveHighlight.GetCursor: TCursor; begin Result := inherited Cursor; end;
function TabcActiveHighlight.GetHint: string; begin Result := inherited Hint; end;
function TabcActiveHighlight.GetHeight: integer; begin Result := inherited Height; end;
function TabcActiveHighlight.GetWidth: integer; begin Result := inherited Width; end;

procedure TabcActiveHighlight.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (AComponent is TWinControl) and (Operation = opRemove) then
    if TWinControl(AComponent) = FActiveControl then
      FActiveControl := nil;
end;

procedure TabcActiveHighlight.Paint;
{for design time only}
begin
  if csDesigning in ComponentState then
  begin
    if (FBitmap = nil) then
    begin
      FBitmap := TBitmap.Create;
      FBitmap.Handle := LoadBitmap(HInstance, 'SNAPBMP');
    end;
    SetBounds(Left, Top, 28, 28);
    Canvas.Draw(0, 0, FBitmap);
  end;
end;

procedure TabcActiveHighlight.SetColorChange(Value: Boolean);
begin
  if Value <> FColorChange then
  begin
    if (csDesigning in ComponentState) then
      FColorChange := Value
    else
    begin
      if Enabled and (FActiveControl <> nil) then
        if Value then
        begin
          FOrigColor := GetActiveColorByName;
          FOrigParentColor := GetActiveParentColorByName;
          SetActiveColorByName(FHighlightColor);
        end
        else
        begin
          SetActiveColorByName(FOrigColor);
          SetActiveParentColorByName(FOrigParentColor);
        end;
      FColorChange := Value;
    end;
  end;
end;

procedure TabcActiveHighlight.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csDesigning in ComponentState) and (FActiveControl <> nil) then
    if Enabled then
      DoHighlightControl(FActiveControl)
    else if FColorChange then
    begin
      SetActiveColorByName(FOrigColor);
      SetActiveParentColorByName(FOrigParentColor);
    end;
end;

procedure TabcActiveHighlight.SetHighlightColor(Value: TColor);
begin
  if Value <> FHighlightColor then begin
    FHighlightColor := Value;
    if not (csDesigning in ComponentState) and (FActiveControl <> nil) then
      if Enabled and FColorChange then SetActiveColorByName(Value);
  end;
end;

procedure TabcActiveHighlight.SetMoveCursor(Value: Boolean);
var
  Pos: TPoint;
begin
  if Value <> FMoveCursor then begin
    FMoveCursor := Value;
    if not (csDesigning in ComponentState) and
       (FActiveControl <> nil) and
       Value then begin
      if (FActiveControl is TRadioButton) or (FActiveControl is TCheckBox) then
        Pos := Point(10, Height div 2)
      else if (FActiveControl is TComboBox) or (FActiveControl is TListBox) then
        Pos := Point(Width - 10, Height div 2)
      else
        Pos := Point(Width div 2, Height div 2);
      Pos := FActiveControl.ClientToScreen(Pos);
      SetCursorPos(Pos.X, Pos.Y);
    end;
  end;
end;

procedure TabcActiveHighlight.UndoHighlightControl;
begin
  if FActiveControl = nil then Exit;
  if Enabled and FColorChange then
  begin
    SetActiveColorByName(FOrigColor);
    SetActiveParentColorByName(FOrigParentColor);
  end;
end;

end.
