unit abcidcbo;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  StdCtrls, DB, DbCtrls, Menus,
  abcda;

type

  { TabcCustomDBIndexCombo }

  EabcInvalidIndexComboItem = class(Exception);
  EabcInvalidIndexComboOperation = class(Exception);

  TabcDBIndexComboDataLink = class;

  TabcCustomDBIndexCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FGoToFirst: Boolean;
    FDataLink: TabcDBIndexComboDataLink;
    FDataset: TDataset;
    FAdapter: TabcDatasetAdapter;
    FItemIndex: Integer;
    FIxName: string;
    FIxFieldNames: string;
    FIxFieldCount: Integer;
    FIxField: TField;
    FOnIndexChange: TNotifyEvent;
    FTableIndexItems: TStrings;
    FComboChanging: Boolean;
    procedure ActiveChanged;
    procedure AdapterChange;
    function CreateInheritedItems(Value: TStrings; var InheritedStrings: TStrings): Boolean;
    procedure DataSetChanged;
    function GetDataSource: TDataSource;
    function GetSorted: Boolean;
    procedure IndexChanged;
    function ParseItem(Item: string; var strPart: string; var Index: integer): Boolean;
    procedure SetDataSource(Value: TDataSource);
{$IFNDEF DELPHI6}
    procedure SetItemIndex(const Value: Integer);
{$ENDIF}
    procedure SetInheritedItems;
    procedure SetSorted(Value: Boolean);
    procedure SetTableIndexItems(Value: TStrings);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    { Protected declarations }
    procedure Change; override;
    procedure Loaded; override;
    function LoadItemsFromDataSet: Integer;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
{$IFDEF DELPHI6}
    function GetItemIndex: Integer; override;
    procedure SetItemIndex(const Value: Integer); override;
{$ENDIF}
    property GoToFirst: Boolean read FGoToFirst write FGoToFirst default True;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
{$IFDEF DELPHI6}
    property ItemIndex default -1;
{$ELSE}
    property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
{$ENDIF}
    property Sorted: Boolean read GetSorted write SetSorted default False;
    property TableIndexItems: TStrings read FTableIndexItems write SetTableIndexItems;
    property OnIndexChange: TNotifyEvent read FOnIndexChange write FOnIndexChange;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function UseRightToLeftAlignment: Boolean; override;
  published
    { Published declarations }
  end;

  { TabcDBIndexCombo }

  TabcDBIndexCombo = class(TabcCustomDBIndexCombo)
  private
    function GetText: string;
  public
    function LoadItemsFromDataSet: Integer;
    property Text: string read GetText;
  published
    property Anchors;
{$IFDEF DELPHI6}
    property AutoComplete default True;
    property AutoDropDown default False;
{$ENDIF}
    property BiDiMode;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
    {property Style; NOT published}
    property GoToFirst;
{$IFDEF DELPHI6}
    property CharCase;
{$ENDIF}
    property Color;
    property Ctl3D;
    property DataSource;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    {property Enabled;}
    property Font;
    property ItemHeight;
    property ItemIndex;
    {property MaxLength;}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TableIndexItems;
    property TabOrder;
    property TabStop;
    { property Text;   NOT published, made public, read-only}
    property Visible;
    property OnChange;
    property OnClick;
{$IFDEF DELPHI6}
    property OnCloseUp;
{$ENDIF}
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
{$IFDEF DELPHI6}
    property OnSelect;
{$ENDIF}
  end;

  { TabcDBIndexComboDataLink }

  TabcDBIndexComboDataLink = class(TDataLink)
  private
    FOwner: TabcCustomDBIndexCombo;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
  public
    constructor Create(AOwner: TabcCustomDBIndexCombo);
    destructor Destroy; override;
  end;

implementation

uses abcconst;

{ TabcDBIndexComboDataLink }

constructor TabcDBIndexComboDataLink.Create(AOwner: TabcCustomDBIndexCombo);
begin
  inherited Create;
  FOwner := AOwner;
  VisualControl := True;
end;

destructor TabcDBIndexComboDataLink.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

procedure TabcDBIndexComboDataLink.ActiveChanged;
begin
  if FOwner <> nil then FOwner.ActiveChanged;
end;

procedure TabcDBIndexComboDataLink.DataSetChanged;
begin
  if FOwner <> nil then FOwner.DataSetChanged;
end;

{ TabcCustomDBIndexCombo }

constructor TabcCustomDBIndexCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcDBIndexComboDataLink.Create(Self);
  Style := csDropDownList;
  FGoToFirst := True;
  FItemIndex := -1;
  FTableIndexItems := TStringList.Create;
end;

destructor TabcCustomDBIndexCombo.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FAdapter.Free;
  FAdapter := nil;
  FTableIndexItems.Free;
  inherited Destroy;
end;

procedure TabcCustomDBIndexCombo.AdapterChange;
begin
  if FDataLink.DataSet <> FDataset then
  begin
    FDataset := FDataLink.DataSet;
    FAdapter.Free;
    FAdapter := nil;
    if FDataset <> nil then
      FAdapter := abcDSAdapterFactory.CreateAdapter(FDataset);
  end;
end;

procedure TabcCustomDBIndexCombo.ActiveChanged;
begin
  AdapterChange;
  Enabled := FDataLink.Active and FAdapter.CanIndex;
  if Enabled then
    if (FItemIndex = -1) then
      IndexChanged
    else
      SetItemIndex(FItemIndex)
  else
    SetItemIndex(-1);
end;

procedure TabcCustomDBIndexCombo.Change;
begin
  if not FComboChanging then
  try
    FComboChanging := True;
{$IFDEF DELPHI6}
    SetItemIndex(inherited GetItemIndex);
{$ELSE}
    SetItemIndex(inherited ItemIndex);
{$ENDIF}
    if Assigned(OnChange) then OnChange(Self);
  finally
    FComboChanging := False;
  end;
end;

function TabcCustomDBIndexCombo.CreateInheritedItems(Value: TStrings; var InheritedStrings: TStrings): Boolean;
var
  I, intTemp: integer;
  strTemp: string;
begin
  InheritedStrings.Clear;
  for I := 0 to Value.Count - 1 do
    if ParseItem(Value.Strings[I], strTemp, intTemp) then
      InheritedStrings.Add(strTemp);
  Result := True;
end;

procedure TabcCustomDBIndexCombo.DataSetChanged;
begin
  if FDataLink.Active and FAdapter.CanIndex then
    with FAdapter do
      if (IndexName <> FIxName) or (IndexFieldNames <> FIxFieldNames) or
         (IndexFieldCount <> FIxFieldCount) then
        IndexChanged;
end;

function TabcCustomDBIndexCombo.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TabcCustomDBIndexCombo.GetSorted: Boolean;
begin
  Result := inherited Sorted;
end;

procedure TabcCustomDBIndexCombo.IndexChanged;
{
Synchronise ItemIndex of ComboBox with Adapter IndexName
}
var
  I, NewItemIndex, intTemp: integer;
  strTemp: string;

  procedure SetInheritedItemIndex(Value: Integer);
  begin
    if FItemIndex <> Value then
    begin
{$IFDEF DELPHI6}
      inherited SetItemIndex(Value);
{$ELSE}
      inherited ItemIndex := Value;
{$ENDIF}
      FItemIndex := Value;
    end;
  end;

begin
  if not Enabled then Exit;
  if FDataLink.Active and FAdapter.CanIndex then
    with FAdapter do
    begin
      IndexDefs.Update;
      FIxName := IndexName;
      FIxFieldNames := IndexFieldNames;
      FIxFieldCount := IndexFieldCount;
      if FIxFieldCount > 0 then
        FIxField := IndexFields[0]
      else
        FIxField := nil;
      I := 0;
      while I < IndexDefs.Count do
      begin
        if IndexDefs.Items[I].Name = IndexName then Break;
        Inc(I);
      end;
      if I = IndexDefs.Count then
        SetInheritedItemIndex(-1)
      else
      begin
        NewItemIndex := 0;
        while NewItemIndex < FTableIndexItems.Count do
        begin
          if ParseItem(FTableIndexItems.Strings[NewItemIndex], strTemp, intTemp) then
            if intTemp = I then Break;
          Inc(NewItemIndex);
        end;
        if NewItemIndex = FTableIndexItems.Count then
          SetInheritedItemIndex(-1)
        else
          SetInheritedItemIndex(NewItemIndex);
      end;
    end
  else
    SetInheritedItemIndex(-1);
end;

procedure TabcCustomDBIndexCombo.Loaded;
begin
  inherited Loaded;
  SetInheritedItems;
  if Sorted then
    SetSorted(True);
  if FDataLink.Active then
    SetItemIndex(FItemIndex);
end;

function TabcCustomDBIndexCombo.LoadItemsFromDataSet: Integer;
var
  I: Integer;
  TempStrings: TStringList;
begin
  if (DataSource = nil) then
    raise EabcInvalidIndexComboOperation.Create(sabcErrDataSourceRequired);
  if (FDataLink.DataSet = nil) then
    raise EabcInvalidIndexComboOperation.Create(sabcErrDataSetRequired);
  if not FAdapter.CanIndex then
    raise EabcInvalidIndexComboOperation.Create(sabcnpErrTableRequired);
  TempStrings := TStringList.Create;
  try
    with FAdapter do begin
      IndexDefs.Update;
      if IndexDefs.Count > 0 then begin
        for I := 0 to IndexDefs.Count - 1 do
          TempStrings.Add(IndexDefs.Items[I].Name + ';' + IntToStr(I));
        if TempStrings.Strings[0] = ';0' then
          TempStrings.Strings[0] := sabcnpPrimaryKeyText + ';0';
      end;
      TableIndexItems := TempStrings;
      if Sorted then SetSorted(True);
      SetItemIndex(FItemIndex);
      Result := IndexDefs.Count;
    end;
  finally
    TempStrings.Free;
  end;
end;

procedure TabcCustomDBIndexCombo.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

function TabcCustomDBIndexCombo.ParseItem(Item: string; var strPart: string; var Index: integer): Boolean;
{
separates strings of form "MyIndexName;1"
assumes ; is always the separator
}
var
  Separator: integer;
  strTemp: string;
begin
  strPart := '';
  Index := 0;
  Separator := Pos(';', Item);
  if Separator = 0 then
    raise EabcInvalidIndexComboItem.Create(Format(sabcnpErrItemFormat1, [Item]));
  strPart := Copy(Item, 1, Separator - 1);
  if strPart = '' then
    raise EabcInvalidIndexComboItem.Create(Format(sabcnpErrItemFormat2, [Item]));
  strTemp := Copy(Item, Separator + 1, 256);
  try
    Index := StrToInt(strTemp);
  except
    raise EabcInvalidIndexComboItem.Create(Format(sabcnpErrItemFormat3, [Item]));
  end;
  Result := True;
end;

procedure TabcCustomDBIndexCombo.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{$IFDEF DELPHI6}
function TabcCustomDBIndexCombo.GetItemIndex: Integer;
begin
  Result := FItemIndex;
end;

procedure TabcCustomDBIndexCombo.SetItemIndex(const Value: Integer);
var
  strTemp: string;
  intTemp, newValue: integer;
begin
  if (csLoading in ComponentState) then
    FItemIndex := Value
  else
  begin
    if Value > FTableIndexItems.Count - 1 then
      newValue := FTableIndexItems.Count - 1
    else if Value < -1 then
      newValue := -1
    else
      newValue := Value;
    if newValue = -1 then
    begin
      {clears the combo without changing the table index}
      inherited SetItemIndex(newValue);
      FItemIndex := newValue;
      if Assigned(FOnIndexChange) then FOnIndexChange(Self);
      Exit;
    end;
    if (DataSource <> nil) and
       (FDataLink.DataSet <> nil) and
       FAdapter.CanIndex then
      with FAdapter do
        if ParseItem(TableIndexItems.Strings[newValue], strTemp, intTemp) then
        begin
          IndexDefs.Update;
          if (intTemp > (IndexDefs.Count - 1)) or (intTemp < 0) then
            raise EabcInvalidIndexComboOperation.Create
              (Format(sabcnpErrIndexRange, [IntTemp]))
          else
          begin
            IndexName := IndexDefs.Items[intTemp].Name;
            with FDataLink.DataSet do
              if Active and FGoToFirst then First;
          end;
          FIxName := IndexName;
          FIxFieldNames := IndexFieldNames;
          FIxFieldCount := IndexFieldCount;
        end;
    FItemIndex := newValue;
    SendMessage(Handle, CB_SETCURSEL, newValue, 0);
    if Assigned(FOnIndexChange) then FOnIndexChange(Self);
  end;
end;
{$ELSE}
procedure TabcCustomDBIndexCombo.SetItemIndex(const Value: Integer);
var
  strTemp: string;
  intTemp, newValue: integer;
begin
  if not HandleAllocated then
    FItemIndex := Value
  else
  begin
    if Value > FTableIndexItems.Count - 1 then
      newValue := FTableIndexItems.Count - 1
    else if Value < -1 then
      newValue := -1
    else
      newValue := Value;
    if newValue = -1 then
    begin
      {clears the combo without changing the table index}
      FItemIndex := newValue;
      inherited ItemIndex := newValue;
      if Assigned(FOnIndexChange) then FOnIndexChange(Self);
      Exit;
    end;
    if (DataSource <> nil) and
       (FDataLink.DataSet <> nil) and
       FAdapter.CanIndex then
      with FAdapter do
        if ParseItem(TableIndexItems.Strings[newValue], strTemp, intTemp) then
        begin
          IndexDefs.Update;
          if (intTemp > (IndexDefs.Count - 1)) or (intTemp < 0) then
            raise EabcInvalidIndexComboOperation.Create
              (Format(sabcnpErrIndexRange, [IntTemp]))
          else
          begin
            IndexName := IndexDefs.Items[intTemp].Name;
            with FDataLink.DataSet do
              if Active and FGoToFirst then First;
          end;
          FIxName := IndexName;
          FIxFieldNames := IndexFieldNames;
          FIxFieldCount := IndexFieldCount;
        end;
    FItemIndex := newValue;
    inherited ItemIndex := newValue;
    if Assigned(FOnIndexChange) then FOnIndexChange(Self);
  end;
end;
{$ENDIF}

procedure TabcCustomDBIndexCombo.SetTableIndexItems(Value: TStrings);
begin
  FTableIndexItems.Assign(Value);
  SetInheritedItems;
  if Sorted then SetSorted(True);
  if FDataLink.Active then
    SetItemIndex(FItemIndex);
end;

procedure TabcCustomDBIndexCombo.SetInheritedItems;
var
  TempStrings: TStringList;
begin
  if not (csLoading in ComponentState) then begin
    TempStrings := TStringList.Create;
    try
      if CreateInheritedItems(FTableIndexItems, TStrings(TempStrings)) then
        inherited Items := TempStrings;
    finally
      TempStrings.Free;
    end;
  end;
end;

procedure TabcCustomDBIndexCombo.SetSorted(Value: Boolean);
begin
  inherited Sorted := Value;
  if Value then
    TStringList(FTableIndexItems).Sort;
  SetItemIndex(FItemIndex);
end;

procedure TabcCustomDBIndexCombo.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

function TabcCustomDBIndexCombo.UseRightToLeftAlignment: Boolean;
begin
  if FIxField <> nil then
    Result := DBUseRightToLeftAlignment(Self, FIxField)
  else
    Result := inherited UseRightToLeftAlignment;
end;

{ TabcDBIndexCombo }

function TabcDBIndexCombo.GetText: string;
begin
  Result := inherited Text;
end;

function TabcDBIndexCombo.LoadItemsFromDataSet: Integer;
begin
  Result := inherited LoadItemsFromDataSet;
end;

end.
