unit abcinpla;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, Forms;

type

  { TabcInPlaceEdit }

  TabcInPlaceEdit = class(TMemo)
  private
    FAccept: Boolean;
    FEditing: Boolean;
    FControl: TControl;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CNKeyDown(var Message: TWMKeyDown); message CN_KEYDOWN;
  public
    constructor Create(AOwner: TComponent); override;
    function Edit(
      const AControl: TControl;
      const ARect: TRect;
      var EditText: string;
      const ACtl3D: Boolean;
      const ABorderStyle: TBorderStyle;
      const AFont: TFont;
      const AColor: TColor;
      const AAlignment: TAlignment
      ): Boolean;
    procedure Cancel;
    property Editing: Boolean read FEditing;
  end;

{
  abcEditInPlace:
    Pops up a multiline wrapped edit control at ARect.
    ARect must be in client coordinates of AControl.
    User presses ESC or TAB to cancel and result is then FALSE
    or ENTER to accept changes and result is then TRUE.
}

function abcEditInPlace(
  const AControl: TControl;
  const ARect: TRect;
  var EditText: string;
  const ACtl3D: Boolean;
  const ABorderStyle: TBorderStyle;
  const AFont: TFont;
  const AColor: TColor;
  const AAlignment: TAlignment
  ): Boolean;

procedure abcCancelInPlaceEdit;

implementation

var
  InPlaceEdit: TabcInPlaceEdit;

function abcEditInPlace(
  const AControl: TControl;
  const ARect: TRect;
  var EditText: string;
  const ACtl3D: Boolean;
  const ABorderStyle: TBorderStyle;
  const AFont: TFont;
  const AColor: TColor;
  const AAlignment: TAlignment
  ): Boolean;
begin
  if InPlaceEdit = nil then
    InPlaceEdit := TabcInPlaceEdit.Create(nil);
  InPlaceEdit.Cancel;
  Result := InPlaceEdit.Edit(
    AControl, ARect, EditText, ACtl3D, ABorderStyle, AFont, AColor, AAlignment);
end;

procedure abcCancelInPlaceEdit;
begin
  if InPlaceEdit <> nil then
    InPlaceEdit.Cancel;
end;

{ TabcInPlaceEdit }

constructor TabcInPlaceEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  WantReturns := False;
end;

procedure TabcInPlaceEdit.Cancel;
begin
  Hide;
  Parent := nil;
end;

procedure TabcInPlaceEdit.CMExit(var Message: TCMExit);
begin
  Cancel;
end;

procedure TabcInPlaceEdit.CNKeyDown(var Message: TWMKeyDown);
begin
  {TAB or ESC cancels, ENTER accepts}
  with Message do
    if CharCode in [VK_TAB, VK_ESCAPE] then
      Hide
    else if CharCode = VK_RETURN then
    begin
      FAccept := True;
      Hide;
    end
    else
      inherited;
end;

function TabcInPlaceEdit.Edit(const AControl: TControl; const ARect: TRect;
  var EditText: string; const ACtl3D: Boolean;
  const ABorderStyle: TBorderStyle; const AFont: TFont;
  const AColor: TColor; const AAlignment: TAlignment): Boolean;
var
  ParentForm: TCustomForm;
  BRect: TRect;
begin
  Result := False;
  if FEditing or (AControl = nil) then Exit;
  FControl := AControl;
  ParentForm := GetParentForm(AControl);
  if ParentForm = nil then Exit;
  FEditing := True;
  FAccept := False;
  {set style & pos}
  BorderStyle := ABorderStyle;
  Font := AFont;
  Color := AColor;
  Alignment := AAlignment;
  Parent := ParentForm;
  Lines.Text := EditText;
  BRect := ARect;
  BRect.TopLeft := ParentForm.ScreenToClient(
    AControl.ClientToScreen(ARect.TopLeft));
  BRect.BottomRight := ParentForm.ScreenToClient(
    AControl.ClientToScreen(ARect.BottomRight));
  BoundsRect := BRect;
  BringToFront;
  Show;
  Ctl3D := ACtl3D;
  SetFocus;
  SelectAll;
  while Visible do
    if Application.Terminated then
      Cancel
    else
      Application.ProcessMessages;
  Result := FAccept;
  if Result then
    EditText := Lines.Text;
  FEditing := False;
end;


initialization

finalization
  InPlaceEdit.Free;
  InPlaceEdit := nil;
  
end.
