unit abckeys;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  ExtCtrls, Menus;

type

  TabcKeyStateKeys = (ksInsert, ksNumLock, ksCapsLock, ksScrollLock);
  TabcKeyStateKeySet = set of TabcKeyStateKeys;

  { TabcCustomKeyState }

  TabcCustomKeyState = class(TCustomControl)
  private
    FBevel: TPanelBevel;
    FKeyCapsLock: Boolean;
    FKeyInsert: Boolean;
    FKeyNumLock: Boolean;
    FKeyScrollLock: Boolean;
    FKeyStrings: TStrings;
    FPosCapsLock: Integer;
    FPosInsert: Integer;
    FPosNumLock: Integer;
    FPosScrollLock: Integer;
    FShowINSAsOVR: Boolean;
    FShowKeyStates: TabcKeyStateKeySet;
    FToggleOnClick: Boolean;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    function GetClientSize: TRect;
    function GetKeyCapsLock: Boolean;
    function GetKeyInsert: Boolean;
    function GetKeyNumLock: Boolean;
    function GetKeyScrollLock: Boolean;
    function GetKeyStateBoolean(Key: Word): Boolean;
    procedure SetBevel(Value: TPanelBevel);
    procedure SetKeyCapsLock(Value: Boolean);
    procedure SetKeyInsert(Value: Boolean);
    procedure SetKeyNumLock(Value: Boolean);
    procedure SetKeyScrollLock(Value: Boolean);
    procedure SetKeyStateBoolean(Key: Word; Toggled: Boolean);
    procedure SetShowKeyStates(Value: TabcKeyStateKeySet);
    procedure SetShowINSAsOVR(Value: Boolean);
    procedure SetKeyStrings(Value: TStrings);
    procedure SetToggleOnClick(Value: Boolean);
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
  protected
    procedure AdjustSize; override;
    procedure Paint; override;
    procedure Resize; override;
    property AutoSize default True;
    property Bevel: TPanelBevel read FBevel write SetBevel default bvLowered;
    property KeyCapsLock: Boolean read GetKeyCapsLock write SetKeyCapsLock default False;
    property KeyInsert: Boolean read GetKeyInsert write SetKeyInsert default True;
    property KeyNumLock: Boolean read GetKeyNumLock write SetKeyNumLock default False;
    property KeyScrollLock: Boolean read GetKeyScrollLock write SetKeyScrollLock default False;
    property KeyStrings: TStrings read FKeyStrings write SetKeyStrings;
    property ShowINSAsOVR: Boolean read FShowINSAsOVR write SetShowINSAsOVR default True;
    property ShowKeyStates: TabcKeyStateKeySet read FShowKeyStates write SetShowKeyStates default
      [ksInsert, ksNumLock, ksCapsLock, ksScrollLock];
    property ToggleOnClick: Boolean read FToggleOnClick write SetToggleOnClick default True;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure InvalidateKeyState(Key: Word);
  published
  end;

  { TabcKeyState }

  TabcKeyState = class(TabcCustomKeyState)
  published
    property Anchors;
    property Constraints;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Color;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property AutoSize;
    property Bevel;
    property ShowKeyStates;
    property KeyCapsLock;
    property KeyInsert;
    property KeyNumLock;
    property KeyScrollLock;
    property KeyStrings;
    property ShowINSAsOVR;
    property ToggleOnClick;
    property OnResize;
  end;

implementation

uses abcconst;

var
  IsWinNT: Boolean;

constructor TabcCustomKeyState.Create (AOwner: TComponent);
begin
    inherited Create(AOwner);
    ControlStyle := [csCaptureMouse, csClickEvents, csOpaque, csDoubleClicks];
    AutoSize := True;
    FBevel := bvLowered;
    FKeyStrings := TStringList.Create;
    FKeyStrings.Add('OVR');
    FKeyStrings.Add('NUM');
    FKeyStrings.Add('CAPS');
    FKeyStrings.Add('SCR');
    FShowINSAsOVR := True;
    FShowKeyStates := [ksInsert, ksNumLock, ksCapsLock, ksScrollLock];
    FToggleOnClick := True;
    SetKeyInsert(True);
end;

destructor TabcCustomKeyState.Destroy;
begin
  FKeyStrings.Free;
  inherited Destroy;
end;

procedure TabcCustomKeyState.CMFontChanged(var Message: TMessage);
begin
  inherited;
  if AutoSize then
    Resize;
end;

function TabcCustomKeyState.GetClientSize: TRect;
var
  VisibleText: string;
  VisibleItems: Integer;
  szVisibleText: array [0..255] of char;
begin
  SetRectEmpty(Result);
  VisibleText := '';
  VisibleItems := 0;

  if ksInsert in FShowKeyStates then begin
    VisibleText := VisibleText + KeyStrings[0];
    Inc(VisibleItems);
  end;
  if ksNumLock in FShowKeyStates then begin
    VisibleText := VisibleText + KeyStrings[1];
    Inc(VisibleItems);
  end;
  if ksCapsLock in FShowKeyStates then begin
    VisibleText := VisibleText + KeyStrings[2];
    Inc(VisibleItems);
  end;
  if ksScrollLock in FShowKeyStates then begin
    VisibleText := VisibleText + KeyStrings[3];
    Inc(VisibleItems);
  end;

  StrPCopy(szVisibleText, VisibleText);
  with Canvas do begin
    Font := Self.Font;
    DrawText(Handle, szVisibleText, StrLen(szVisibleText), Result,
      (DT_CALCRECT or DT_LEFT));
  end;
  with Result do begin
    {adjust for borders}
    Right := Right + 10 + 9 * (VisibleItems - 1);
    Bottom := Bottom + 10;
  end;
end;

function TabcCustomKeyState.GetKeyCapsLock: Boolean;
begin
  if not (csDesigning in ComponentState) then
    Result := GetKeyStateBoolean(VK_CAPITAL)
  else
    Result := FKeyCapsLock;
end;

function TabcCustomKeyState.GetKeyInsert: Boolean;
begin
  if not (csDesigning in ComponentState) then
    Result := GetKeyStateBoolean(VK_INSERT)
  else
    Result := FKeyInsert;
end;

function TabcCustomKeyState.GetKeyNumLock: Boolean;
begin
  if not (csDesigning in ComponentState) then
    Result := GetKeyStateBoolean(VK_NUMLOCK)
  else
    Result := FKeyNumLock;
end;

function TabcCustomKeyState.GetKeyScrollLock: Boolean;
begin
  if not (csDesigning in ComponentState) then
    Result := GetKeyStateBoolean(VK_SCROLL)
  else
    Result := FKeyScrollLock;
end;

function TabcCustomKeyState.GetKeyStateBoolean(Key: Word): Boolean;
begin
  Result := Boolean(GetKeyState(Key) and $1);
end;

procedure TabcCustomKeyState.InvalidateKeyState(Key: Word);
begin
  if Key in [VK_CAPITAL, VK_INSERT, VK_NUMLOCK, VK_SCROLL] then Invalidate;
end;

procedure TabcCustomKeyState.Paint;
var
  Rect: TRect;
  BorderRect, TxtRect: TRect;
  TopColor, BottomColor: TColor;
  Text: array[0..255] of Char;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    if Bevel = bvLowered then begin
      TopColor := clBtnShadow;
      BottomColor := clBtnHighlight;
    end
    else begin
      TopColor := clBtnHighlight;
      BottomColor := clBtnShadow;
    end;
  end;

begin
  Rect := GetClientRect;

  with Canvas do
  begin
    Brush.Color := Color;
    FillRect(Rect);
    AdjustColors(FBevel);

    Brush.Style := bsClear;
    Font := Self.Font;

    with BorderRect do begin
      Left := Rect.Top + 4;
      Top := Rect.Top + 4;
    end;

    if ksInsert in FShowKeyStates then
      with BorderRect do begin
        FPosInsert := Left - 2;
        StrPCopy(Text, FKeyStrings.Strings[0]);
        DrawText(Handle, Text, StrLen(Text), BorderRect, (DT_CALCRECT or DT_LEFT));
        if FBevel <> bvNone then begin
          InflateRect(BorderRect, 2, 2);
          MoveTo(Left, Bottom);
          Pen.Color := TopColor;
          LineTo(Left, Top);
          LineTo(Right, Top);
          Pen.Color := BottomColor;
          LineTo(Right, Bottom);
          LineTo(Left, Bottom);
          InflateRect(BorderRect, -2, -2);
        end;
        TxtRect := BorderRect;
        if (KeyInsert = FShowINSAsOVR) then begin
          Font.Color := clBtnHighlight;
          OffsetRect(TxtRect, 1, 1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
          Font.Color := clBtnShadow;
          OffsetRect(TxtRect, -1, -1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end
        else begin
          Font.Color := Self.Font.Color;
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end;
        Left := TxtRect.Right + 9;
      end;

    if ksNumLock in FShowKeyStates then
      with BorderRect do begin
        FPosNumLock := Left - 2;
        StrPCopy(Text, FKeyStrings.Strings[1]);
        DrawText(Handle, Text, StrLen(Text), BorderRect, (DT_CALCRECT or DT_LEFT));
        if FBevel <> bvNone then begin
          InflateRect(BorderRect, 2, 2);
          MoveTo(Left, Bottom);
          Pen.Color := TopColor;
          LineTo(Left, Top);
          LineTo(Right, Top);
          Pen.Color := BottomColor;
          LineTo(Right, Bottom);
          LineTo(Left, Bottom);
          InflateRect(BorderRect, -2, -2);
        end;
        TxtRect := BorderRect;
        if KeyNumLock then begin
          Font.Color := Self.Font.Color;
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end
        else begin
          Font.Color := clBtnHighlight;
          OffsetRect(TxtRect, 1, 1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
          Font.Color := clBtnShadow;
          OffsetRect(TxtRect, -1, -1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end;
        Left := TxtRect.Right + 9;
      end;

    if ksCapsLock in FShowKeyStates then
      with BorderRect do begin
        FPosCapsLock := Left - 2;
        StrPCopy(Text, FKeyStrings.Strings[2]);
        DrawText(Handle, Text, StrLen(Text), BorderRect, (DT_CALCRECT or DT_LEFT));
        if FBevel <> bvNone then begin
          InflateRect(BorderRect, 2, 2);
          MoveTo(Left, Bottom);
          Pen.Color := TopColor;
          LineTo(Left, Top);
          LineTo(Right, Top);
          Pen.Color := BottomColor;
          LineTo(Right, Bottom);
          LineTo(Left, Bottom);
          InflateRect(BorderRect, -2, -2);
        end;
        TxtRect := BorderRect;
        if KeyCapsLock then begin
          Font.Color := Self.Font.Color;
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end
        else begin
          Font.Color := clBtnHighlight;
          OffsetRect(TxtRect, 1, 1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
          Font.Color := clBtnShadow;
          OffsetRect(TxtRect, -1, -1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end;
        Left := TxtRect.Right + 9;
      end;

    if ksScrollLock in FShowKeyStates then
      with BorderRect do begin
        FPosScrollLock := Left - 2;
        StrPCopy(Text, FKeyStrings.Strings[3]);
        DrawText(Handle, Text, StrLen(Text), BorderRect, (DT_CALCRECT or DT_LEFT));
        if FBevel <> bvNone then begin
          InflateRect(BorderRect, 2, 2);
          MoveTo(Left, Bottom);
          Pen.Color := TopColor;
          LineTo(Left, Top);
          LineTo(Right, Top);
          Pen.Color := BottomColor;
          LineTo(Right, Bottom);
          LineTo(Left, Bottom);
          InflateRect(BorderRect, -2, -2);
        end;
        TxtRect := BorderRect;
        if KeyScrollLock then begin
          Font.Color := Self.Font.Color;
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end
        else begin
          Font.Color := clBtnHighlight;
          OffsetRect(TxtRect, 1, 1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
          Font.Color := clBtnShadow;
          OffsetRect(TxtRect, -1, -1);
          DrawText(Handle, Text, StrLen(Text), TxtRect, (DT_LEFT));
        end;
        Left := TxtRect.Right + 9;
      end;
  end;
end;

procedure TabcCustomKeyState.AdjustSize;
begin
  Resize;
end;

procedure TabcCustomKeyState.Resize;
var
  CalcSize: TRect;
begin
  if AutoSize then begin
    CalcSize := GetClientSize;
    SetBounds(Left, Top, CalcSize.Right, CalcSize.Bottom);
  end;
  inherited;
end;

procedure TabcCustomKeyState.SetBevel(Value: TPanelBevel);
begin
  if Value <> FBevel then begin
    FBevel := Value;
    Invalidate;
  end;
end;

procedure TabcCustomKeyState.SetKeyCapsLock(Value: Boolean);
begin
  if not (csDesigning in ComponentState) then
    SetKeyStateBoolean(VK_CAPITAL, Value)
  else
    FKeyCapsLock := Value;
  Invalidate;
end;

procedure TabcCustomKeyState.SetKeyInsert(Value: Boolean);
begin
  if not (csDesigning in ComponentState) then
    SetKeyStateBoolean(VK_INSERT, Value)
  else
    FKeyInsert := Value;
  Invalidate;
end;

procedure TabcCustomKeyState.SetKeyNumLock(Value: Boolean);
begin
  if not (csDesigning in ComponentState) then
    SetKeyStateBoolean(VK_NUMLOCK, Value)
  else
    FKeyNumLock := Value;
  Invalidate;
end;

procedure TabcCustomKeyState.SetKeyScrollLock(Value: Boolean);
begin
  if not (csDesigning in ComponentState) then
    SetKeyStateBoolean(VK_SCROLL, Value)
  else
    FKeyScrollLock := Value;
  Invalidate;
end;

procedure TabcCustomKeyState.SetKeyStateBoolean(Key: Word; Toggled: Boolean);
var
  KeyBuffer: TKeyboardState;
begin
  if Key in [VK_CAPITAL, VK_INSERT, VK_NUMLOCK, VK_SCROLL] then begin
    GetKeyboardState(KeyBuffer);
    KeyBuffer[Key] := Ord(Toggled);
    if IsWinNT and (Key in [VK_CAPITAL, VK_NUMLOCK, VK_SCROLL]) then
    begin
      keybd_event(Key, $45, KEYEVENTF_EXTENDEDKEY or 0, 0);
      keybd_event(Key, $45, KEYEVENTF_EXTENDEDKEY or KEYEVENTF_KEYUP, 0);
    end
    else
      SetKeyBoardState(KeyBuffer);
  end;
end;

procedure TabcCustomKeyState.SetKeyStrings(Value: TStrings);
var
  I, SizeOfStrings: Integer;
begin
  if Value.Count <> 4 then
    raise Exception.Create(sabcKeysErrFourExpected)
  else begin
    {check max length of strings}
    SizeOfStrings := 0;
    for I := 0 to Value.Count - 1 do
      Inc(SizeOfStrings, Length(Value.Strings[I]));
    if SizeOfStrings > 255 then
      raise Exception.Create(sabcKeysErrLength)
    else begin
      FKeyStrings.Assign(Value);
      Resize;
    end;
  end;
end;

procedure TabcCustomKeyState.SetShowINSAsOVR(Value: Boolean);
begin
  if Value <> FShowINSAsOVR then begin
    FShowINSAsOVR := Value;
    Invalidate;
  end;
end;

procedure TabcCustomKeyState.SetShowKeyStates(Value: TabcKeyStateKeySet);
begin
  if Value <> FShowKeyStates then begin
    FShowKeyStates := Value;
    Resize;
  end;
end;

procedure TabcCustomKeyState.SetToggleOnClick(Value: Boolean);
begin
  if Value <> FToggleOnClick then FToggleOnClick := Value;
end;

procedure TabcCustomKeyState.WMLButtonUp(var Message: TWMLButtonUp);
var
  ClickPt: TPoint;
begin
  inherited;
  if FToggleOnClick then
    ClickPt := SmallPointToPoint(Message.Pos);
    if PtInRect(ClientRect, ClickPt) then begin
      {locate which part was clicked and toggle state}
      if (ksScrollLock in FShowKeyStates)
          and (ClickPt.X >= FPosScrollLock) then
        KeyScrollLock := not KeyScrollLock
      else
      if (ksCapsLock in FShowKeyStates)
          and (ClickPt.X >= FPosCapsLock) then
        KeyCapsLock := not KeyCapsLock
      else
      if (ksNumLock in FShowKeyStates)
          and (ClickPt.X >= FPosNumLock) then
        KeyNumLock := not KeyNumLock
      else
      if (ksInsert in FShowKeyStates)
          and (ClickPt.X >= FPosInsert) then
        KeyInsert := not KeyInsert;
    end;
end;

initialization
  IsWinNT := GetVersion < $80000000;

end.
