unit abclabel;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)
{$I abcdefs.inc}

interface

uses
  Windows, SysUtils, Messages, Classes, Graphics, Controls,
  StdCtrls, Menus, abcutilg, abcutil, abcutilw, abcwav;

type

  TabcDegrees = 0..359;

  TabcCaptionEffect = (ceNormal, ceRaised, ceLowered, ceEmbossRaised, ceEmbossLowered, ceShadowed);

  TabcBorderEffect = (beNone, beLine, beRaised, beLowered, beFrameRaised, beFrameLowered);

  TabcBorderWidth = 1..MaxInt;

  TabcBorderStyle = class(TPersistent)
  private
    FBoundsAdjustment: Integer;
    FEffect: TabcBorderEffect;
    FColorHighlight: TColor;
    FColorShadow: TColor;
    FWidth: TabcBorderWidth;
    FOnChange: TNotifyEvent;
    procedure SetEffect(Value: TabcBorderEffect);
    procedure SetColorHighlight(Value: TColor);
    procedure SetColorShadow(Value: TColor);
    procedure SetWidth(Value: TabcBorderWidth);
    procedure UpdateBoundsAdjustment;
  protected
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    property BoundsAdjustment: integer read FBoundsAdjustment;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Effect: TabcBorderEffect read FEffect write SetEffect default beNone;
    property ColorHighlight: TColor read FColorHighlight write SetColorHighlight default clBtnHighlight;
    property ColorShadow: TColor read FColorShadow write SetColorShadow default clBtnShadow;
    property Width: TabcBorderWidth read FWidth write SetWidth default 1;
  end;

  TabcCaptionStyle = class(TPersistent)
  private
    FBoundsAdjustment: Integer;
    FEffect: TabcCaptionEffect;
    FColorHighlight: TColor;
    FColorShadow: TColor;
    FOffset: Integer;
    FOnChange: TNotifyEvent;
    procedure SetEffect(Value: TabcCaptionEffect);
    procedure SetColorHighlight(Value: TColor);
    procedure SetColorShadow(Value: TColor);
    procedure SetOffset(Value: Integer);
    procedure UpdateBoundsAdjustment;
  protected
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    property BoundsAdjustment: integer read FBoundsAdjustment;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Effect: TabcCaptionEffect read FEffect write SetEffect default ceNormal;
    property ColorHighlight: TColor read FColorHighlight write SetColorHighlight default clBtnHighlight;
    property Offset: Integer read FOffset write SetOffset default 1;
    property ColorShadow: TColor read FColorShadow write SetColorShadow default clBtnShadow;
  end;

  TabcRotLabel = class(TCustomLabel)
  private
    { Private declarations }
    FAngle: TabcDegrees;
    FAngleSin, FAngleCos: Double;
    FDrawTextPt: TPoint;
    FBorderPoints: array[0..4] of TPoint;
    FBorderStyle: TabcBorderStyle;
    FCaptionStyle: TabcCaptionStyle;
    FRotateCenter: Boolean;
    function  GetRotatedFont: HFont;
    procedure GetTextBounds(var R: TRect);
    procedure DrawBorder;
    procedure DrawNormalText;
    procedure DrawOffsetText(Offset: Integer; Emboss, Shadow: Boolean);
    procedure DrawCaptionAt(AColor: TColor; APoint: TPoint);
    procedure StyleChanged(Sender: TObject);
    function  GetFont: TFont;
    procedure SetAngle(Value: TabcDegrees);
    procedure SetBorderStyle(Value: TabcBorderStyle);
    procedure SetCaptionStyle(Value: TabcCaptionStyle);
    procedure SetFont(Value: TFont);
  protected
    procedure AdjustBounds; override;
    procedure Paint; override;
    procedure Loaded; override;
    procedure SetAutoSize(Value: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    {property Align; NOT published}
    {property Alignment; NOT published}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property Angle: TabcDegrees read FAngle write SetAngle;
    property AutoSize;
    property BorderStyle: TabcBorderStyle read FBorderStyle write SetBorderStyle;
    property Caption;
    property CaptionStyle: TabcCaptionStyle read FCaptionStyle write SetCaptionStyle;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font: TFont read GetFont write SetFont;
    property ParentColor;
    property ParentFont default False;
    property ParentShowHint;
    property PopupMenu;
    property RotateCenter: Boolean read FRotateCenter write FRotateCenter default True;
    {property ShowAccelChar; NOT published}
    property ShowHint;
    property Transparent;
    property Visible;
    {property WordWrap; NOT published}
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnEndDock;
    property OnStartDock;
{$IFDEF DELPHI6}
    property OnMouseEnter;
    property OnMouseLeave;
{$ENDIF}
  end;

  TabcMultilineLabel = class(TCustomLabel)
  private
    { Private declarations }
    FLines: TStrings;
    FOnResize: TNotifyEvent;
    FWantTrailingReturns: Boolean;
    function GetCaption: string;
    procedure SetLines(Value: TStrings);
  protected
    procedure LinesOnChangeHandler(Sender: TObject); dynamic;
    property WantTrailingReturns: Boolean read FWantTrailingReturns write FWantTrailingReturns;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Caption: string read GetCaption;
  published
    property Align;
    property Alignment;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property AutoSize;
    {property Caption;  read only}
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property Lines: TStrings read FLines write SetLines;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize: TNotifyEvent read FOnResize write FOnResize;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
{$IFDEF DELPHI6}
    property OnMouseEnter;
    property OnMouseLeave;
{$ENDIF}
  end;


  { TabcMouseLabel }

  TabcLabelMouseEffect = (almUnderline, almColor, almColorShadow, almSound);
  TabcLabelMouseEffects = set of TabcLabelMouseEffect;

  TabcCustomMouseLabel = class(TCustomLabel)
  private
    FMouseIsDown: Boolean;
    FMouseIsOver: Boolean;
    FMouseDownColor: TColor;
    FMouseDownEffects: TabcLabelMouseEffects;
    FMouseOverColor: TColor;
    FMouseOverEffects: TabcLabelMouseEffects;
{$IFNDEF DELPHI6}
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
{$ENDIF}
    FSoundList: TabcCustomWaveList;
    FSoundDown: Word;
    FSoundOver: Word;
    procedure SetMouseDownColor(Value: TColor);
    procedure SetMouseDownEffects(Value: TabcLabelMouseEffects);
    procedure SetMouseOverColor(Value: TColor);
    procedure SetMouseOverEffects(Value: TabcLabelMouseEffects);
    procedure UpdateTracking;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure DoDrawText(var Rect: TRect; Flags: Longint); override;
{$IFNDEF DELPHI6}
    procedure DoOnMouseEnter; dynamic;
    procedure DoOnMouseLeave; dynamic;
{$ENDIF}    
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Paint; override;
    {public, read only}
    property MouseIsDown: Boolean read FMouseIsDown;
    property MouseIsOver: Boolean read FMouseIsOver;
    property MouseDownEffects: TabcLabelMouseEffects
      read FMouseDownEffects write SetMouseDownEffects default [almUnderline, almSound];
    property MouseDownColor: TColor read FMouseDownColor write SetMouseDownColor default clAqua;
    property MouseOverEffects: TabcLabelMouseEffects
      read FMouseOverEffects write SetMouseOverEffects default [almUnderline, almSound];
    property MouseOverColor: TColor read FMouseOverColor write SetMouseOverColor default clYellow;
    property SoundDown: Word read FSoundDown write FSoundDown default 0;
    property SoundList: TabcCustomWaveList read FSoundList write FSoundList;
    property SoundOver: Word read FSoundOver write FSoundOver default 0;
{$IFNDEF DELPHI6}
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
{$ENDIF}    
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

  TabcMouseLabel = class(TabcCustomMouseLabel)
  public
    property MouseIsDown;
    property MouseIsOver;
  published
    property Action;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property Align;
    property Alignment;
    property AutoSize;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property MouseDownColor;
    property MouseDownEffects;
    property MouseOverColor;
    property MouseOverEffects;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property SoundDown;
    property SoundList;
    property SoundOver;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

  { TabcURLLabel }

  TabcURLLabel = class(TabcMouseLabel)
  private
    FURL: string;
    FShowBrowser: TabcShowWindowStyle;
    procedure SetURL(Value: string);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
  published
    property MouseDownColor default clPurple;
    property MouseDownEffects default [almColor, almSound];
    property MouseOverEffects default [almUnderLine, almSound];
    property ShowBrowser: TabcShowWindowStyle read FShowBrowser write FShowBrowser default aswShowMaximized;
    property URL: string read FURL write SetURL;
    property Cursor default crHandPoint;
  end;

  { TabcMail }

  TabcMail = class(TPersistent)
  private
    FSubject: string;
    FCC: string;
    FBCC: string;
    FAddress: string;
    FOnChange: TNotifyEvent;
    FBody: TStrings;
    FOwner: TComponent;
    procedure SetAddress(const Value: string);
    procedure SetBCC(const Value: string);
    procedure SetBody(const Value: TStrings);
    procedure SetCC(const Value: string);
    procedure SetSubject(const Value: string);
    procedure BodyChanged(Sender: TObject);
  protected
    procedure Change; dynamic;
  public
    constructor Create(AOwner: TComponent); virtual;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Clear;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Address: string read FAddress write SetAddress;
    property CC: string read FCC write SetCC;
    property BCC: string read FBCC write SetBCC;
    property Subject: string read FSubject write SetSubject;
    property Body: TStrings read FBody write SetBody;
  end;

  { TabcMailToLabel }

  TabcMailToLabel = class(TabcMouseLabel)
  private
    FMail: TabcMail;
    FShowWindow: TabcShowWindowStyle;
    procedure SetMail(Value: TabcMail);
    procedure OnMailChange(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    property MouseDownColor default clPurple;
    property MouseDownEffects default [almColor, almSound];
    property MouseOverEffects default [almUnderLine, almSound];
    property Mail: TabcMail read FMail write SetMail;
    property ShowWindow: TabcShowWindowStyle read FShowWindow write FShowWindow default aswShowMaximized;
    property Cursor default crHandPoint;
  end;

implementation

uses abcutilf, Math;

{ TabcBorderStyle }

constructor TabcBorderStyle.Create;
begin
  FColorHighlight := clBtnHighlight;
  FColorShadow := clBtnShadow;
  FWidth := 1;
end;

procedure TabcBorderStyle.Assign(Source: TPersistent);
begin
  if Source is TabcBorderStyle then begin
    FBoundsAdjustment := TabcBorderStyle(Source).BoundsAdjustment;
    FEffect := TabcBorderStyle(Source).Effect;
    FColorHighlight := TabcBorderStyle(Source).ColorHighlight;
    FColorShadow := TabcBorderStyle(Source).ColorShadow;
    FWidth := TabcBorderStyle(Source).Width;
  end
  else
    inherited Assign(Source);
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TabcBorderStyle.SetEffect(Value: TabcBorderEffect);
begin
  if Value <> FEffect then begin
    FEffect := Value;
    UpdateBoundsAdjustment;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcBorderStyle.SetColorHighlight(Value: TColor);
begin
  if Value <> FColorHighlight then begin
    FColorHighlight := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcBorderStyle.SetColorShadow(Value: TColor);
begin
  if Value <> FColorShadow then begin
    FColorShadow := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcBorderStyle.SetWidth(Value: TabcBorderWidth);
begin
  if Value <> FWidth then begin
    FWidth := Value;
    UpdateBoundsAdjustment;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcBorderStyle.UpdateBoundsAdjustment;
begin
  case FEffect of
    beNone:
      FBoundsAdjustment := 0;
    beLine..beLowered:
      FBoundsAdjustment := FWidth div 2 + 1;
    beFrameRaised,beFrameLowered:
      FBoundsAdjustment := 2 * FWidth + 1;
  end;
end;

{ TabcCaptionStyle }

constructor TabcCaptionStyle.Create;
begin
  FColorHighlight := clBtnHighlight;
  FColorShadow := clBtnShadow;
  FOffset := 1;
end;

procedure TabcCaptionStyle.Assign(Source: TPersistent);
begin
  if Source is TabcCaptionStyle then begin
    FBoundsAdjustment := TabcCaptionStyle(Source).BoundsAdjustment;
    FEffect := TabcCaptionStyle(Source).Effect;
    FColorHighlight := TabcCaptionStyle(Source).ColorHighlight;
    FColorShadow := TabcCaptionStyle(Source).ColorShadow;
    FOffset := TabcCaptionStyle(Source).Offset;
  end
  else
    inherited Assign(Source);
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TabcCaptionStyle.SetEffect(Value: TabcCaptionEffect);
begin
  if Value <> FEffect then begin
    FEffect := Value;
    UpdateBoundsAdjustment;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcCaptionStyle.SetColorHighlight(Value: TColor);
begin
  if Value <> FColorHighlight then begin
    FColorHighlight := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcCaptionStyle.SetOffset(Value: Integer);
begin
  if Value <> FOffset then begin
    FOffset := Value;
    UpdateBoundsAdjustment;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcCaptionStyle.SetColorShadow(Value: TColor);
begin
  if Value <> FColorShadow then begin
    FColorShadow := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcCaptionStyle.UpdateBoundsAdjustment;
begin
  case FEffect of
    ceNormal:
      FBoundsAdjustment := 0;
    ceRaised..ceEmbossLowered:
      FBoundsAdjustment := 2 * FOffset;
    ceShadowed:
      FBoundsAdjustment := FOffset;
  end;
end;

{ TabcRotLabel }

constructor TabcRotLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCaptionStyle := TabcCaptionStyle.Create;
  FCaptionStyle.OnChange := StyleChanged;
  FBorderStyle := TabcBorderStyle.Create;
  FBorderStyle.OnChange := StyleChanged;
  ShowAccelChar := False;
  ParentFont := False;
  Font.Name := 'Arial';
  Font.Size := 10;
  FAngleCos := 1; {for initial angle 0 degrees, sin is 0}
  FRotateCenter := True;
end;

destructor TabcRotLabel.Destroy;
begin
  FCaptionStyle.Free;
  FBorderStyle.Free;
  inherited Destroy;
end;

procedure TabcRotLabel.Loaded;
var
  SaveAutoSize, SaveRotate: Boolean;
begin
  inherited Loaded;
  SaveAutoSize := AutoSize;
  SaveRotate := RotateCenter;
  RotateCenter := False;
  AutoSize := True;
  AdjustBounds;
  RotateCenter := SaveRotate;
  AutoSize := SaveAutoSize;
end;

function TabcRotLabel.GetRotatedFont: HFont;
var
  LogFontRec: TLogFont;
begin
  GetObject(Canvas.Font.Handle, SizeOf(TLogFont), @LogFontRec);
  LogFontRec.lfEscapement := FAngle * 10;
  Result := CreateFontIndirect(LogFontRec);
end;

procedure TabcRotLabel.AdjustBounds;
var
  TextRect: TRect;
  NewLeft, NewTop: Integer;
begin
  NewLeft := Left + Width div 2;
  NewTop := Top + Height div 2;
  inherited;
  if [csLoading, csReading] * ComponentState = [] then begin
    TextRect := ClientRect;
    GetTextBounds(TextRect);
    if FCaptionStyle.Effect <> ceNormal then
      InflateRect(TextRect, FCaptionStyle.BoundsAdjustment, FCaptionStyle.BoundsAdjustment);
    if FBorderStyle.Effect <> beNone then begin
      InflateRect(TextRect, FBorderStyle.BoundsAdjustment, FBorderStyle.BoundsAdjustment);
      Dec(TextRect.Left, 2);
    end;
    FDrawTextPt := Point(0, 0);
    abcSetRectPoints(TextRect, FBorderPoints);
    abcRotateWinPolygon(FAngleSin, FAngleCos, FDrawTextPt, FBorderPoints);
    FBorderPoints[4] := FBorderPoints[0];
    TextRect := abcGetPolygonBounds(FBorderPoints);

    {inflate bounds to ensure we don't clip on rotation}
    if (FBorderStyle.Effect <> beNone) then
    begin
      Inc(TextRect.Right, FBorderStyle.BoundsAdjustment);
      Inc(TextRect.Bottom, FBorderStyle.BoundsAdjustment);
    end;

    {offset all points to 0,0 origin so we can autosize relative to (Left, Top)}
    abcOffsetPoint(FDrawTextPt, - TextRect.Left, - TextRect.Top);
    abcOffsetPoints(FBorderPoints, - TextRect.Left, - TextRect.Top);
    OffsetRect(TextRect, - TextRect.Left, - TextRect.Top);
    if AutoSize then
      if FRotateCenter then begin
        NewLeft := NewLeft - TextRect.Right div 2;
        NewTop := NewTop - TextRect.Bottom div 2;
        SetBounds(NewLeft, NewTop, TextRect.Right, TextRect.Bottom);
      end
      else
        SetBounds(Left, Top, TextRect.Right, TextRect.Bottom);
  end;
  Invalidate;
end;

procedure TabcRotLabel.GetTextBounds(var R: TRect);
var
  DC: HDC;
  SaveFont: HFont;
  Text: array[0..255] of Char;
begin
  GetTextBuf(Text, SizeOf(Text));
  if (Text[0] = #0) then StrCopy(Text, ' ');
  DC := GetDC(0);
  with Canvas do begin
    Handle := DC;
    SaveFont := SelectObject(Handle, GetRotatedFont);
    DrawText(Handle, Text, StrLen(Text), R, DT_CALCRECT);
    SaveFont := SelectObject(Handle, SaveFont);
    DeleteObject(SaveFont);
    Handle := 0;
  end;
  ReleaseDC(0, DC);
end;

procedure TabcRotLabel.Paint;
begin
  with Canvas do begin
    if not Transparent then begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;

    if FBorderStyle.Effect <> beNone then DrawBorder;

    Brush.Style := bsClear;
    Font := Self.Font;

    case FCaptionStyle.Effect of
      ceNormal:         DrawNormalText;
      ceRaised:         DrawOffsetText(FCaptionStyle.Offset, False, False);
      ceLowered:        DrawOffsetText(-FCaptionStyle.Offset, False, False);
      ceEmbossRaised:   DrawOffsetText(FCaptionStyle.Offset, True, False);
      ceEmbossLowered:  DrawOffsetText(-FCaptionStyle.Offset, True, False);
      ceShadowed:       DrawOffsetText(FCaptionStyle.Offset, False, True);
    end;

  end;
end;

procedure TabcRotLabel.DrawOffsetText(Offset: Integer; Emboss, Shadow: Boolean);
var
  DrawPt: TPoint;
  DrawColor: TColor;
  AdjustOffsetX, AdjustOffsetY: Integer;
begin
  case Angle of
    0..45, 316..359:
      begin
        AdjustOffsetX := Offset;
        AdjustOffsetY := Offset;
      end;
    46..135:
      begin
        AdjustOffsetX := Offset;
        AdjustOffsetY := -Offset;
      end;
    136..225:
      begin
        AdjustOffsetX := -Offset;
        AdjustOffsetY := -Offset;
      end;
    else {226..315:}
      begin
        AdjustOffsetX := -Offset;
        AdjustOffsetY := Offset;
      end;
  end;
  with Canvas do begin
    DrawPt := FDrawTextPt;
    if not Shadow then begin
      abcOffsetPoint(DrawPt, -AdjustOffsetX, -AdjustOffsetY);
      DrawColor := FCaptionStyle.ColorHighlight;
      DrawCaptionAt(DrawColor, DrawPt);
      abcOffsetPoint(DrawPt, 2*AdjustOffsetX, 2*AdjustOffsetY);
    end
    else
      abcOffsetPoint(DrawPt, AdjustOffsetX, AdjustOffsetY);

    DrawColor := FCaptionStyle.ColorShadow;
    DrawCaptionAt(DrawColor, DrawPt);
    abcOffsetPoint(DrawPt, -AdjustOffsetX, -AdjustOffsetY);

    if not Enabled then
      DrawColor := clGrayText
    else if Emboss then
      DrawColor := Self.Color
    else
      DrawColor := Self.Font.Color;
    DrawCaptionAt(DrawColor, DrawPt);

  end;
end;

procedure TabcRotLabel.DrawNormalText;
var
  DrawColor: TColor;
begin
  if not Enabled then
    DrawColor := clGrayText
  else
    DrawColor := Self.Font.Color;
  DrawCaptionAt(DrawColor, FDrawTextPt);
end;

procedure TabcRotLabel.DrawCaptionAt(AColor: TColor; APoint: TPoint);
var
  SaveFont: HFont;
begin
  with Canvas do begin
    Font.Color := AColor;
    SaveFont := SelectObject(Handle, GetRotatedFont);
    TextOut(APoint.X, APoint.Y, Caption);
    SaveFont := SelectObject(Handle, SaveFont);
    DeleteObject(SaveFont);
  end;
end;

procedure TabcRotLabel.DrawBorder;
var
  DP: array[0..4] of TPoint;
  I: integer;

  procedure DrawRaisedLoweredLines(TopColor, BottomColor: TColor);
  {draw lines using points depending on Angle}
  begin
    with Canvas do
      case FAngle of
        0..44, 315..359:
          begin
            Pen.Color := TopColor;
            PolyLine([DP[3], DP[0], DP[1]]);
            Pen.Color := BottomColor;
            PolyLine([DP[1], DP[2], DP[3]]);
          end;
        45..134:
          begin
            Pen.Color := TopColor;
            PolyLine([DP[0], DP[1], DP[2]]);
            Pen.Color := BottomColor;
            PolyLine([DP[2], DP[3], DP[0]]);
          end;
        135..224:
          begin
            Pen.Color := TopColor;
            PolyLine([DP[1], DP[2], DP[3]]);
            Pen.Color := BottomColor;
            PolyLine([DP[3], DP[0], DP[1]]);
          end;
        225..314:
          begin
            Pen.Color := TopColor;
            PolyLine([DP[2], DP[3], DP[0]]);
            Pen.Color := BottomColor;
            PolyLine([DP[0], DP[1], DP[2]]);
          end;
      end;
  end;

begin
  for I := 0 to 4 do DP[I] := FBorderPoints[I];
  with Canvas do begin
    Pen.Mode := pmCopy;
    Pen.Style := psSolid;
    Pen.Width := FBorderStyle.Width;
    if Transparent then
      Brush.Style := bsClear
    else
      Brush.Style := bsSolid;

    case FBorderStyle.Effect of
      beLine:
        begin
          Pen.Color := FBorderStyle.ColorShadow;
          Polygon(DP);
        end;
      beRaised:
        begin
          DrawRaisedLoweredLines(FBorderStyle.ColorHighlight, FBorderStyle.ColorShadow);
        end;
      beLowered:
        begin
          DrawRaisedLoweredLines(FBorderStyle.ColorShadow, FBorderStyle.ColorHighlight);
        end;
      beFrameRaised:
        begin
          Brush.Style := bsClear;
          Pen.Color := FBorderStyle.ColorHighlight;
          Polygon(DP);
          Pen.Color := FBorderStyle.ColorShadow;
          abcOffsetPoints(DP, FBorderStyle.Width, FBorderStyle.Width);
          Polygon(DP);
        end;
      beFrameLowered:
        begin
          Brush.Style := bsClear;
          Pen.Color := FBorderStyle.ColorShadow;
          Polygon(DP);
          Pen.Color := FBorderStyle.ColorHighlight;
          abcOffsetPoints(DP, FBorderStyle.Width, FBorderStyle.Width);
          Polygon(DP);
        end;
    end;
  end;
end;

function TabcRotLabel.GetFont: TFont; begin Result := inherited Font; end;

procedure TabcRotLabel.SetAngle(Value: TabcDegrees);
begin
  if Value <> FAngle then begin
    FAngle := Value;
    FAngleSin := Sin(DegToRad(FAngle));
    FAngleCos := Cos(DegToRad(FAngle));
    AdjustBounds;
  end;
end;

procedure TabcRotLabel.SetAutoSize(Value: Boolean);
begin
  inherited SetAutoSize(Value);
  if AutoSize then AdjustBounds;
end;

procedure TabcRotLabel.SetBorderStyle(Value: TabcBorderStyle);
begin
  if Value <> FBorderStyle then begin
    FBorderStyle.Assign(Value);
    FBorderStyle.OnChange := StyleChanged;
    AdjustBounds;
  end;
end;

procedure TabcRotLabel.SetCaptionStyle(Value: TabcCaptionStyle);
begin
  if Value <> FCaptionStyle then begin
    FCaptionStyle.Assign(Value);
    FCaptionStyle.OnChange := StyleChanged;
    AdjustBounds;
  end;
end;

procedure TabcRotLabel.SetFont(Value: TFont);
begin
  if inherited Font <> Value then begin
    inherited Font := Value;
    AdjustBounds;
  end;
end;

procedure TabcRotLabel.StyleChanged(Sender: TObject);
begin
  AdjustBounds;
end;

{ TabcMultilineLabel }

constructor TabcMultilineLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLines := TStringList.Create;
  TStringList(FLines).OnChange := LinesOnChangeHandler;
end;

destructor TabcMultilineLabel.Destroy;
begin
  FLines.Free;
  inherited Destroy;
end;

function TabcMultilineLabel.getCaption;
begin
  Result := inherited Caption;
end;

procedure TabcMultilineLabel.LinesOnChangeHandler(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then
    if not FWantTrailingReturns then
      inherited Caption := abcStripTrailingReturns(FLines.Text)
    else
      inherited Caption := FLines.Text;
end;

procedure TabcMultilineLabel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  if Assigned(FOnResize) then
    FOnResize(Self);
end;

procedure TabcMultilineLabel.SetLines(Value: TStrings);
begin
  if Value <> FLines then
    FLines.Assign(Value);
end;

{ TabcCustomMouseLabel }

constructor TabcCustomMouseLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMouseDownEffects := [almUnderline, almSound];
  FMouseDownColor := clAqua;
  FMouseOverEffects := [almUnderline, almSound];
  FMouseOverColor := clYellow;
end;

procedure TabcCustomMouseLabel.DoDrawText(var Rect: TRect; Flags: Longint);
var
  Text: string;
  rOffset: TRect;

  procedure ApplyHighlight(Effects: TabcLabelMouseEffects; EffectColor: TColor);
  begin
    if (almUnderline in Effects) then
      Canvas.Font.Style := Canvas.Font.Style + [fsUnderline];
    if (almColor in Effects) then
      Canvas.Font.Color := EffectColor
    else if (almColorShadow in Effects) then
    begin
      Canvas.Font.Color := EffectColor;
      Windows.CopyRect(rOffset, Rect);
      OffsetRect(rOffset, -1, -1);
      DrawText(Canvas.Handle, PChar(Text), Length(Text), rOffset, Flags);
      OffsetRect(rOffset, 2, 2);
      DrawText(Canvas.Handle, PChar(Text), Length(Text), rOffset, Flags);
      Canvas.Font.Color := Font.Color;
    end;
  end;

begin
  Text := GetLabelText;
  if (Flags and DT_CALCRECT <> 0) and ((Text = '') or ShowAccelChar and
    (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';
  if not ShowAccelChar then Flags := Flags or DT_NOPREFIX;

  Canvas.Font := Font;

  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
  end
  else begin
    if MouseIsDown then
      ApplyHighlight(MouseDownEffects, MouseDownColor)
    else if MouseIsOver then
      ApplyHighlight(MouseOverEffects, MouseOverColor);
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
  end;
end;

procedure TabcCustomMouseLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  Rect, CalcRect: TRect;
  DrawStyle: Integer;
begin
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    DrawStyle := DT_EXPANDTABS or WordWraps[WordWrap] or Alignments[Alignment];
    { Calculate vertical layout }
    if Layout <> tlTop then
    begin
      CalcRect := Rect;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      if Layout = tlBottom then OffsetRect(Rect, 0, Height - CalcRect.Bottom)
      else OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
    end;
    DoDrawText(Rect, DrawStyle);
  end;
end;

procedure TabcCustomMouseLabel.SetMouseDownEffects(Value: TabcLabelMouseEffects);
begin
  if Value <> FMouseDownEffects then
  begin
    if (almColorShadow in Value) and not (almColorShadow in FMouseDownEffects) then
      FMouseDownEffects := Value - [almColor]
    else if (almColor in Value) and not (almColor in FMouseDownEffects) then
      FMouseDownEffects := Value - [almColorShadow]
    else
      FMouseDownEffects := Value;
    if MouseIsDown then
      Invalidate;
  end;
end;

procedure TabcCustomMouseLabel.SetMouseDownColor(Value: TColor);
begin
  if Value <> FMouseDownColor then
  begin
    FMouseDownColor := Value;
    if MouseIsDown then
      Invalidate;
  end;
end;

procedure TabcCustomMouseLabel.SetMouseOverEffects(Value: TabcLabelMouseEffects);
begin
  if Value <> FMouseOverEffects then
  begin
    if (almColorShadow in Value) and not (almColorShadow in FMouseOverEffects) then
      FMouseOverEffects := Value - [almColor]
    else if (almColor in Value) and not (almColor in FMouseOverEffects) then
      FMouseOverEffects := Value - [almColorShadow]
    else
      FMouseOverEffects := Value;
    if MouseIsOver and not MouseIsDown then
      Invalidate;
  end;
end;

procedure TabcCustomMouseLabel.SetMouseOverColor(Value: TColor);
begin
  if Value <> FMouseOverColor then
  begin
    FMouseOverColor := Value;
    if MouseIsOver and not MouseIsDown then
      Invalidate;
  end;
end;

{$IFNDEF DELPHI6}
procedure TabcCustomMouseLabel.DoOnMouseEnter;
begin
  if Assigned(OnMouseEnter) then
    OnMouseEnter(Self);
end;

procedure TabcCustomMouseLabel.DoOnMouseLeave;
begin
  if Assigned(OnMouseLeave) then
    OnMouseLeave(Self);
end;
{$ENDIF}

procedure TabcCustomMouseLabel.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if not MouseIsDown and Enabled then
  begin
    FMouseIsDown := True;
    Repaint;
    if (almSound in MouseDownEffects) and (SoundList <> nil) then
      SoundList.PlayItem(SoundDown);
  end;
end;

procedure TabcCustomMouseLabel.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if MouseIsDown and Enabled then
  begin
    FMouseIsDown := False;
    Repaint;
  end;
  if Dragging then UpdateTracking;
end;

procedure TabcCustomMouseLabel.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = FSoundList) and (Operation = opRemove) then
    FSoundList := nil;
end;

procedure TabcCustomMouseLabel.UpdateTracking;
var
  P: TPoint;
begin
  FMouseIsDown := False;
  if Enabled then
  begin
    GetCursorPos(P);
    FMouseIsOver := not (FindDragTarget(P, True) = Self);
    if FMouseIsOver then
      Perform(CM_MOUSELEAVE, 0, 0)
    else
      Perform(CM_MOUSEENTER, 0, 0);
  end;
end;

procedure TabcCustomMouseLabel.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  UpdateTracking;
end;

procedure TabcCustomMouseLabel.CMMouseEnter(var Message: TMessage);
begin
  if not MouseIsOver and Enabled then
  begin
    FMouseIsOver := True;
    Repaint;
    if (almSound in MouseOverEffects) and (SoundList <> nil) then
      SoundList.PlayItem(SoundOver);
  end;
{$IFNDEF DELPHI6}
  DoOnMouseEnter;
{$ENDIF}
  inherited;
end;

procedure TabcCustomMouseLabel.CMMouseLeave(var Message: TMessage);
begin
  if Enabled and not Dragging then
  begin
    FMouseIsOver := False;
    Repaint;
  end;
{$IFNDEF DELPHI6}
  DoOnMouseLeave;
{$ENDIF}
  inherited;
end;

{ TabcURLLabel }

constructor TabcURLLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Font.Color := clBlue;
  MouseDownColor := clPurple;
  MouseDownEffects := [almColor, almSound];
  MouseOverEffects := [almUnderLine, almSound];
  FShowBrowser := aswShowMaximized;
  Cursor := crHandPoint;
end;

procedure TabcURLLabel.Click;
begin
  inherited Click;
  abcLaunchBrowserAtURL(URL, ShowBrowser);
end;

procedure TabcURLLabel.SetURL(Value: string);
begin
  if Value <> FURL then
  begin
    FURL := Value;
    if (csDesigning in ComponentState) and (Caption = Name) then
      Caption := Value;
  end;
end;

{ TabcMail }

constructor TabcMail.Create(AOwner: TComponent);
begin
  inherited Create;
  FOwner := AOwner;
  FBody := TStringList.Create;
  TStringList(FBody).OnChange := BodyChanged;
end;

destructor TabcMail.Destroy;
begin
  FBody.Free;
  inherited;
end;

procedure TabcMail.Assign(Source: TPersistent);
begin
  if Source = nil then
    Clear
  else if Source is TabcMail then
    with TabcMail(Source) do
    begin
      Self.FAddress := Address;
      Self.FCC := CC;
      Self.FBCC := BCC;
      Self.FSubject := Subject;
      Self.FBody.Assign(Body);
      Change;
    end
  else
    inherited;
end;

procedure TabcMail.BodyChanged(Sender: TObject);
begin
  Change;
end;

procedure TabcMail.Change;
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

procedure TabcMail.Clear;
begin
  FAddress := '';
  FCC := '';
  FBCC := '';
  FSubject := '';
  FBody.Clear;
  Change;
end;

procedure TabcMail.SetAddress(const Value: string);
begin
  if Value <> FAddress then
  begin
    FAddress := Value;
    Change;
  end;
end;

procedure TabcMail.SetBCC(const Value: string);
begin
  if Value <> FBCC then
  begin
    FBCC := Value;
    Change;
  end;
end;

procedure TabcMail.SetBody(const Value: TStrings);
begin
  if Value <> FBody then
  begin
    FBody.Assign(Value);
    Change;
  end;
end;

procedure TabcMail.SetCC(const Value: string);
begin
  if Value <> FCC then
  begin
    FCC := Value;
    Change;
  end;
end;

procedure TabcMail.SetSubject(const Value: string);
begin
  if Value <> FSubject then
  begin
    FSubject := Value;
    Change;
  end;
end;

{ TabcMailToLabel }

constructor TabcMailToLabel.Create(AOwner: TComponent);
begin
  inherited;
  Font.Color := clBlue;
  MouseDownColor := clPurple;
  MouseDownEffects := [almColor, almSound];
  MouseOverEffects := [almUnderLine, almSound];
  FShowWindow := aswShowMaximized;
  Cursor := crHandPoint;
  FMail := TabcMail.Create(Self);
  if (csDesigning in ComponentState) then
    FMail.OnChange := OnMailChange;
end;

destructor TabcMailToLabel.Destroy;
begin
  FMail.Free;
  inherited;
end;

procedure TabcMailToLabel.OnMailChange(Sender: TObject);
begin
  if (Caption = Name) and (Mail.Address <> '') then
    Caption := 'mailto:' + Mail.Address;
end;

procedure TabcMailToLabel.Click;
begin
  inherited;
  with Mail do
    abcSendEmail(Address, CC, BCC, Subject, Body, FShowWindow);
end;

procedure TabcMailToLabel.SetMail(Value: TabcMail);
begin
  if Value <> FMail then
    FMail.Assign(Value);
end;

end.

