unit abclist;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}


interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, StdCtrls,
  Forms, Menus;

type

  { TabcSyncListBox }

  TListSyncOption = (lsHScroll, lsVScroll, lsClick, lsSelItem, lsTopIndex);
  TabcListSyncOptions = set of TListSyncOption;

  TabcCustomSyncListBox = class(TCustomListBox)
  private
    FScrollWidth: Word;
    FPrintIndent: Byte;
    FPrintLines: Byte;
    FPrinting: Boolean;
    FSyncControl: TabcCustomSyncListBox;
    FSyncOptions: TabcListSyncOptions;
    FScrollBySyncCtrl: Boolean;
    procedure CNKeyUp(var Message: TWMKeyUp); message CN_KEYUP;
    procedure LBSetCaretIndex(var Message: TMessage); message LB_SETCARETINDEX;
    procedure LBSetCurSel(var Message: TMessage); message LB_SETCURSEL;
    procedure LBSetSel(var Message: TMessage); message LB_SETSEL;
    procedure LBSetTopIndex(var Message: TMessage); message LB_SETTOPINDEX;
    procedure SendSyncMessage(Message: TMessage);
    procedure SetScrollWidth(Value: Word);
    procedure SetSyncControl(Value: TabcCustomSyncListBox);
    procedure WMHScroll(var Message: TMessage); message WM_HSCROLL;
    procedure WMVScroll(var Message: TMessage); message WM_VSCROLL;
  protected
    procedure Click; override;
    procedure CreateWnd; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    property ScrollWidth: Word read FScrollWidth write SetScrollWidth;
    property PrintIndent: Byte read FPrintIndent write FPrintIndent default 5;
    property PrintLines: Byte read FPrintLines write FPrintLines default 60;
    property Printing: Boolean read FPrinting write FPrinting default True;
    property SyncOptions: TabcListSyncOptions read FSyncOptions write FSyncOptions
      default [lsHScroll, lsVScroll, lsClick, lsSelItem, lsTopIndex];
    property SyncControl: TabcCustomSyncListBox read FSyncControl write SetSyncControl;
    property ScrollBySyncCtrl: Boolean read FScrollBySyncCtrl write FScrollBySyncCtrl; {flag to avoid circular updates}
  public
    constructor Create(AOwner: TComponent); override;
    procedure Print;
  end;

  TabcSyncListBox = class(TabcCustomSyncListBox)
  public
    property ScrollBySyncCtrl;
  published
    property PrintIndent;
    property Printing;
    property PrintLines;
    property ScrollWidth;
    property SyncControl;
    property SyncOptions;
    {inherited properties}
    property Anchors;
{$IFDEF DELPHI6}
    property AutoComplete;
{$ENDIF}
    property BiDiMode;
    property Constraints;
    property ImeMode;
    property ImeName;
    property ParentBiDiMode;
    property Align;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property ItemHeight;
    property Items;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF DELPHI6}
    property OnData;
    property OnDataFind;
    property OnDataObject;
{$ENDIF}
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  { TabcCheckListBox }

  TabcGlyphAlignH = (gahLeft, gahRight);
  TabcGlyphAlignV = (gavBottom, gavCenter, gavTop);

  TabcGlyphSpace = 1..10;

  TCheckOption = (chClick, chClickGlyph, chDblClick, chSpaceKey, chShiftRange,
    chExtendedSelect, chMultiSelect);
  TabcCheckOptions = set of TCheckOption;

  TabcCheckStyle = (csTickSunken, csTickFlat, csTickRaised, csTickTransparent,
    csTick3D, csCrossSunken, csCrossFlat, csCrossRaised, csCrossTransparent,
    csCross3D, csDiamondSunken, csDiamondFlat, csDiamondRaised, csDiamond3D,
    csCustom, csTickFlatWindows);

  TabcCheckNotifyEvent = procedure(Sender: TObject; Index: Integer; var Value: Boolean) of object;

  TabcCustomCheckListBox = class(TabcCustomSyncListBox)
  private
    FAlignment: TAlignment;
    FCheckedColor: TColor;
    FCheckedColorChange: Boolean;
    FCheckedItems: TStrings;
    FDisabledItems: TStrings;
    FDisabledItemsGrayed: Boolean;
    FDestroyingWnd: Boolean;
    FDrawBuffer: TBitmap;
    FExpandTabs: Boolean;
    FGlyph: TBitmap;
    FGlyphAlignH: TabcGlyphAlignH;
    FGlyphAlignV: TabcGlyphAlignV;
    FGlyphForeColor: TColor;
    FGlyphInternalChange: Boolean;
    FGlyphSpace: TabcGlyphSpace;
    FGlyphStyle: TabcCheckStyle;
    FHalfGlyphWidth: Integer;
    FHalfGlyphHeight: Integer;
    FMaskedGlyph: TBitmap;
    FOnCheck: TabcCheckNotifyEvent;
    FOptions: TabcCheckOptions;
    FSaveItemIndex: Integer;
    FWordWrap: Boolean;
    FShowSelectedItem: Boolean;
    FShowFocusRect: Boolean;
    function AdjustFocusRect(R: TRect): TRect;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure DoDrawText(ACanvas: TCanvas; Index: Integer; var Rect: TRect; CalcRect: Boolean);
    procedure DoMouseRangeSelect(Pos: TPoint; Shift: Boolean);
    function GetChecked(Index: Integer): Boolean;
    function GetDisabled(Index: Integer): Boolean;
    function GetColumns: Integer;
    function GetGlyphRect(const rcItem: TRect): TRect;
    procedure GlyphChanged(Sender: TObject);
    function IsGlyphStored: Boolean;
    procedure LBDeleteString(var Message: TMessage); message LB_DELETESTRING;
    procedure LBResetContent(var Message: TMessage); message LB_RESETCONTENT;
    procedure SetAlignment(Value: TAlignment);
    procedure SetChecked(Index: Integer; Value: Boolean);
    procedure SetDisabled(Index: Integer; Value: Boolean);
    procedure SetCheckedColor(Value: TColor);
    procedure SetCheckedColorChange(Value: Boolean);
    procedure SetCheckedItems(Value: TStrings);
    procedure SetDisabledItems(Value: TStrings);
    procedure SetDisabledItemsGrayed(Value: Boolean);
    procedure SetColumns(Value: Integer);
    procedure SetExpandTabs(Value: Boolean);
    procedure SetGlyph(Value: TBitmap);
    procedure SetGlyphAlignH(Value: TabcGlyphAlignH);
    procedure SetGlyphAlignV(Value: TabcGlyphAlignV);
    procedure SetGlyphForeColor(Value: TColor);
    procedure SetGlyphSpace(Value: TabcGlyphSpace);
    procedure SetGlyphStyle(Value: TabcCheckStyle);
    procedure SetOptions(Value: TabcCheckOptions);
    procedure SetWordWrap(Value: Boolean);
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    function GetAsInteger: Integer;
    procedure SetAsInteger(Value: Integer);
    procedure SetShowSelectedItem(Value: Boolean);
    procedure SetShowFocusRect(Value: Boolean);
  protected
    procedure CheckAll;
    procedure CheckAllExceptDisabled;
    procedure DisableAll;
    procedure ClearAll;
    procedure CreateHandle; override;
    procedure DestroyWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect;
      State: TOwnerDrawState); override;
    function IsItemChecked(Item: string): Boolean;
    function IsObjectChecked(AnObject: TObject): Boolean;
    function IsItemDisabled(Item: string): Boolean;
    function IsObjectDisabled(AnObject: TObject): Boolean;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Loaded; override;
    procedure MeasureItem(Index: Integer; var Height: Integer); override;
    procedure RedrawCheckedItems; virtual;
    procedure RedrawItem(Index: Integer); virtual;
    function SetItemState(Item: string; Value: Boolean): Boolean;
    function SetObjectState(AnObject: TObject; Value: Boolean): Boolean;
    function SetItemDisabledState(Item: string; Value: Boolean): Boolean;
    function SetObjectDisabledState(AnObject: TObject; Value: Boolean): Boolean;
    property AsInteger: Integer read GetAsInteger write SetAsInteger;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property Checked[Index: Integer]: Boolean read GetChecked write SetChecked;
    property Disabled[Index: Integer]: Boolean read GetDisabled write SetDisabled;
    property CheckedColor: TColor read FCheckedColor write SetCheckedColor default clWindowText;
    property CheckedColorChange: Boolean read FCheckedColorChange write SetCheckedColorChange default True;
    property CheckedItems: TStrings read FCheckedItems write SetCheckedItems;
    property DisabledItems: TStrings read FDisabledItems write SetDisabledItems;
    property DisabledItemsGrayed: Boolean read FDisabledItemsGrayed write SetDisabledItemsGrayed default True;
    property Columns: Integer read GetColumns write SetColumns default 0;
    property ExpandTabs: Boolean read FExpandTabs write SetExpandTabs default False;
    property Glyph: TBitmap read FGlyph write SetGlyph stored IsGlyphStored;
    property GlyphAlignH: TabcGlyphAlignH read FGlyphAlignH write SetGlyphAlignH default gahLeft;
    property GlyphAlignV: TabcGlyphAlignV read FGlyphAlignV write SetGlyphAlignV default gavCenter;
    property GlyphForeColor: TColor read FGlyphForeColor write SetGlyphForeColor default clBlack;
    property GlyphSpace: TabcGlyphSpace read FGlyphSpace write SetGlyphSpace default 2;
    property GlyphStyle: TabcCheckStyle read FGlyphStyle write SetGlyphStyle default csTickSunken;
    property OnCheck: TabcCheckNotifyEvent read FOnCheck write FOnCheck;
    property Options: TabcCheckOptions read FOptions write SetOptions
      default [chClick, chSpaceKey, chMultiSelect];
    property WordWrap: Boolean read FWordWrap write SetWordWrap default False;
    property ShowSelectedItem: Boolean read FShowSelectedItem write SetShowSelectedItem default True;
    property ShowFocusRect: Boolean read FShowFocusRect write SetShowFocusRect default True;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TabcCheckListBox = class(TabcCustomCheckListBox)
  public
    procedure CheckAll;
    procedure CheckAllExceptDisabled;
    procedure ClearAll;
    procedure DisableAll;
    function IsItemChecked(Item: string): Boolean;
    function IsObjectChecked(AnObject: TObject): Boolean;
    function IsItemDisabled(Item: string): Boolean;
    function IsObjectDisabled(AnObject: TObject): Boolean;
    function SetItemState(Item: string; Value: Boolean): Boolean;
    function SetObjectState(AnObject: TObject; Value: Boolean): Boolean;
    function SetItemDisabledState(Item: string; Value: Boolean): Boolean;
    function SetObjectDisabledState(AnObject: TObject; Value: Boolean): Boolean;
    property AsInteger;
    property Checked;
    property Disabled;
    property ScrollBySyncCtrl;
  published
    property Alignment;
    property CheckedColor;
    property CheckedColorChange;
    property CheckedItems;
    property DisabledItems;
    property DisabledItemsGrayed;
    property Columns;   {replaces inherited Columns from TListBox}
    property ExpandTabs;
    property Glyph;
    property GlyphAlignH;
    property GlyphAlignV;
    property GlyphForeColor;
    property GlyphSpace;
    property GlyphStyle;
    property OnCheck;
    property Options;
    property WordWrap;
    property ShowSelectedItem;
    property ShowFocusRect;
    {synclistbox properties}
    property PrintIndent;
    property Printing;
    property PrintLines;
    property ScrollWidth;
    property SyncControl;
    property SyncOptions;
    {inherited properties}
    property Anchors;
{$IFDEF DELPHI6}
    property AutoComplete;
{$ENDIF}
    property BiDiMode;
    property Constraints;
    property ImeMode;
    property ImeName;
    property ParentBiDiMode;
    property Align;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    {property ExtendedSelect; replaced by Options value}
    property Font;
    property IntegralHeight;
    property ItemHeight;
    property Items;
    {property MultiSelect; replaced by Options value}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    {property Style; set internally}
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF DELPHI6}
    property OnData;
    property OnDataFind;
    property OnDataObject;
{$ENDIF}
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  { TabcTabListBox }

  TabcCustomTabListBox = class(TabcCustomSyncListBox)
  private
    FNumTabs: Word;
    FScrollWidth: Word;
    FTabPositions: string;
    FTabs: array[0..31] of Integer;
    procedure SetScrollWidth(Value: Word);
    procedure SetTabPositions(Value: string);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  protected
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect;
      State: TOwnerDrawState); override;
    property ScrollWidth: Word read FScrollWidth write SetScrollWidth default 0;
    property TabPositions: string read FTabPositions write SetTabPositions;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TabcTabListBox = class(TabcCustomTabListBox)
  public
    property ScrollBySyncCtrl;
  published
    property ScrollWidth;
    property TabPositions;
    {synclistbox properties}
    property PrintIndent;
    property Printing;
    property PrintLines;
    property SyncControl;
    property SyncOptions;
    {inherited properties}
    property Anchors;
{$IFDEF DELPHI6}
    property AutoComplete;
{$ENDIF}
    property BiDiMode;
    property Constraints;
    property ImeMode;
    property ImeName;
    property ParentBiDiMode;
    property Align;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property Items;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF DELPHI6}
    property OnData;
    property OnDataFind;
    property OnDataObject;
{$ENDIF}
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

implementation

{$R abcchk32.res}

uses abcutilf;

{ TabcCustomCheckListBox }

constructor TabcCustomCheckListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Options := [chClick, chSpaceKey, chMultiSelect];
  inherited Style := lbOwnerDrawFixed;
  FCheckedColor := clWindowText;
  FCheckedColorChange := True;
  FCheckedItems := TStringList.Create;
  FDisabledItems := TStringList.Create;
  FGlyphForeColor := clBlack;
  FGlyph := TBitmap.Create;
  FMaskedGlyph := TBitmap.Create;
  FGlyph.OnChange := GlyphChanged;
  GlyphStyle := csTickSunken;
  FGlyphAlignV := gavCenter;
  FGlyphSpace := 2;
  FDrawBuffer := TBitmap.Create;
  FSaveItemIndex := -1;
  FDisabledItemsGrayed := True;
  FShowSelectedItem := True;
  FShowFocusRect := True;
end;

destructor TabcCustomCheckListBox.Destroy;
begin
  FCheckedItems.Free;
  FDisabledItems.Free;
  FGlyph.Free;
  FMaskedGlyph.Free;
  FDrawBuffer.Free;
  inherited Destroy;
end;

function TabcCustomCheckListBox.AdjustFocusRect(R: TRect): TRect;
begin
  if GlyphAlignH = gahLeft then
    Result := Rect(R.Left + FHalfGlyphWidth + 3 * FGlyphSpace,
                   R.Top,
                   R.Right,
                   R.Bottom)
  else
    Result := Rect(R.Left,
                   R.Top,
                   R.Right - FHalfGlyphWidth - 3 * FGlyphSpace,
                   R.Bottom);
end;

procedure TabcCustomCheckListBox.CheckAll;
begin
  if (chMultiSelect in Options) then begin
    CheckedItems.Assign(Items);
    if HandleAllocated then Invalidate;
  end
  else
    if ItemIndex > -1 then
      Checked[ItemIndex] := True;
end;

procedure TabcCustomCheckListBox.CheckAllExceptDisabled;
var
  i: integer;
begin
  if (chMultiSelect in Options) then
  begin
    CheckedItems.Clear;
    with Items do
      for i := 0 to Count - 1 do
        if not Disabled[i] then
          CheckedItems.AddObject(Strings[i], Objects[i]);
    if HandleAllocated then Invalidate;
  end
end;

procedure TabcCustomCheckListBox.ClearAll;
begin
  CheckedItems.Clear;
  DisabledItems.Clear;
  if not (chMultiSelect in Options) then
    ItemIndex := -1;
  if HandleAllocated then Invalidate;
end;

procedure TabcCustomCheckListBox.DisableAll;
begin
  DisabledItems.Assign(Items);
  if HandleAllocated then Invalidate;
end;

procedure TabcCustomCheckListBox.CMColorChanged(var Message: TMessage);
begin
  inherited;
  FGlyphInternalChange := True;
  GlyphChanged(Self);
end;

procedure TabcCustomCheckListBox.CMFontChanged(var Message: TMessage);
begin
  inherited;
  if not (csLoading in ComponentState) and FCheckedColorChange then
    CheckedColor := Font.Color;
end;

procedure TabcCustomCheckListBox.CMShowingChanged(var Message: TMessage);
begin
  inherited;
  if (FSaveItemIndex <> -1) and Visible then begin
    if chMultiSelect in Options then begin
      RedrawCheckedItems;
      ItemIndex := FSaveItemIndex;
    end
    else
      ItemIndex := FSaveItemIndex;
    FSaveItemIndex := -1;
  end;
end;

procedure TabcCustomCheckListBox.CNDrawItem(var Message: TWMDrawItem);
var
  State: TOwnerDrawState;
  rcFocus: TRect;
begin
  with Message.DrawItemStruct^ do begin
{$IFDEF DELPHI5}
    State := TOwnerDrawState(LongRec(itemState).Lo);
{$ELSE}
    State := TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);
{$ENDIF}
    Canvas.Handle := hDC;
    Canvas.Font := Font;
    Canvas.Brush := Brush;
    if ShowSelectedItem and (Integer(itemID) >= 0) and (odSelected in State) then
    begin
      Canvas.Brush.Color := clHighlight;
      Canvas.Font.Color := clHighlightText
    end;
    if Integer(itemID) >= 0 then
      DrawItem(itemID, rcItem, State)
    else
      Canvas.FillRect(rcItem);
    if ShowFocusRect and (odFocused in State) then begin
      rcFocus := AdjustFocusRect(rcItem);
      DrawFocusRect(hDC, rcFocus);
    end;
    Canvas.Handle := 0;
  end;
end;

procedure TabcCustomCheckListBox.CreateHandle;
begin
  inherited CreateHandle;
  FDestroyingWnd := False;
end;

procedure TabcCustomCheckListBox.DestroyWnd;
begin
  FSaveItemIndex := ItemIndex;
  FDestroyingWnd := True;
  inherited DestroyWnd;
end;

procedure TabcCustomCheckListBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  rcOffset, rcColor, rcBMPDest, rcBMPSource: TRect;
  DrawProc: TDrawItemEvent;
begin
  DrawProc := OnDrawItem;
  if (Index > -1) and (DisabledItems.IndexOf(Items[Index]) > -1) then
    State := State + [odDisabled];
  if Assigned(DrawProc) then
    OnDrawItem(Self, Index, Rect, State)
  else begin
    rcOffset := Rect;
    OffsetRect(rcOffset, -Rect.Left, -Rect.Top);
    FDrawBuffer.Width := rcOffset.Right;
    FDrawBuffer.Height := rcOffset.Bottom;
    with FDrawBuffer.Canvas do begin
      Brush.Color := Color;
      FillRect(rcOffset);
      Font := Self.Canvas.Font;
      Brush := Self.Canvas.Brush;
      rcColor := AdjustFocusRect(rcOffset);
      FillRect(rcColor);
      if Index < Items.Count then begin
        if Checked[Index] then begin
          rcBMPSource := Bounds(0, 0, FHalfGlyphWidth, FHalfGlyphHeight);
          if not (odSelected in State) then
            if (odDisabled in State) and DisabledItemsGrayed then
              FDrawBuffer.Canvas.Font.Color := clGrayText
            else if (CheckedColor <> Self.Font.Color) then
              FDrawBuffer.Canvas.Font.Color := CheckedColor;
        end
        else
        begin
          rcBMPSource := Bounds(FHalfGlyphWidth, 0, FHalfGlyphWidth, FHalfGlyphHeight);
          if ([odSelected, odDisabled] * State = [odDisabled]) and DisabledItemsGrayed then
            FDrawBuffer.Canvas.Font.Color := clGrayText;
        end;
        if Disabled[Index] and DisabledItemsGrayed then
          OffsetRect(rcBMPSource, 0, FHalfGlyphHeight);
        rcBMPDest := GetGlyphRect(rcOffset);
        CopyRect(rcBMPDest, FMaskedGlyph.Canvas, rcBMPSource);
        DoDrawText(FDrawBuffer.Canvas, Index, rcColor, False);
      end;
    end;
    Canvas.CopyRect(Rect, FDrawBuffer.Canvas, rcOffset);
  end;
end;

procedure TabcCustomCheckListBox.DoDrawText(ACanvas: TCanvas; Index: Integer; var Rect: TRect; CalcRect: Boolean);
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Flags: Word;
begin
  Flags := DT_NOPREFIX or Alignments[Alignment];
  if CalcRect then Flags := Flags or DT_CALCRECT;
  if ExpandTabs then Flags := Flags or DT_EXPANDTABS;
  if WordWrap then Flags := Flags or DT_WORDBREAK;
  Inc(Rect.Left, FGlyphSpace);
  Dec(Rect.Right, FGlyphSpace);
  DrawText(ACanvas.Handle, PChar(Items[Index]), Length(Items[Index]), Rect, Flags);
end;

procedure TabcCustomCheckListBox.DoMouseRangeSelect(Pos: TPoint; Shift: Boolean);
var
  I, ClickIndex: Integer;
  ClickIndexRect, GlyphRect: TRect;
  wasChecked: Boolean;
begin
  ClickIndex := ItemAtPos(Pos, True);
  if ClickIndex > -1 then begin
    if DisabledItems.IndexOf(Items[ClickIndex]) > -1 then
      Exit;
    if (chClickGlyph in Options) then begin
      {was click in the glyph part?  If not, then exit.}
      ClickIndexRect := ItemRect(ClickIndex);
      GlyphRect := GetGlyphRect(ClickIndexRect);
      if not PtInRect(GlyphRect, Pos) then Exit;
    end;
    if (chMultiSelect in Options) and
       (chExtendedSelect in Options) and
       ( not (chShiftRange in Options) or Shift) then begin
      for I := 0 to Items.Count - 1 do
        if Selected[I] then begin
          Checked[I] := not Checked[I];
          if I <> ClickIndex then Selected[I] := False;
        end;
    end
    else
      if (chMultiSelect in Options) then
      begin
        wasChecked := Checked[ClickIndex];
        Checked[ClickIndex] := not wasChecked;
      end
      else
      begin
        CheckedItems.Clear;
        Checked[ClickIndex] := True;
        if HandleAllocated then Invalidate;
      end
  end;
end;

function TabcCustomCheckListBox.GetChecked(Index: Integer): Boolean;
begin
  Result := CheckedItems.IndexOf(Items.Strings[Index]) <> -1;
end;

function TabcCustomCheckListBox.GetDisabled(Index: Integer): Boolean;
begin
  Result := DisabledItems.IndexOf(Items.Strings[Index]) <> -1;
end;

function TabcCustomCheckListBox.GetColumns: Integer;
begin
  Result := inherited Columns;
end;

function TabcCustomCheckListBox.GetGlyphRect(const rcItem: TRect): TRect;
{returns the glyph rect relative to the item rect}
begin
  case GlyphAlignV of
    gavBottom:
      Result.Top := rcItem.Bottom - FHalfGlyphHeight;
    gavCenter:
      Result.Top := (rcItem.Top + rcItem.Bottom - FHalfGlyphHeight) div 2;
    gavTop:
      Result.Top := rcItem.Top;
  end;
  if GlyphAlignH = gahLeft then
    Result.Left := rcItem.Left + 2 * FGlyphSpace
  else
    Result.Left := rcItem.Right - FHalfGlyphWidth - 2 * FGlyphSpace;
  Result.Right := Result.Left + FHalfGlyphWidth;
  Result.Bottom := Result.Top + FHalfGlyphHeight;
end;

procedure TabcCustomCheckListBox.GlyphChanged(Sender: TObject);
var
  bmpTemp: TBitmap;
  rcCopy: TRect;
begin
{
 rebuilds FMaskedGlyph with color substitution:
   clOlive replaced with control Color
   clFuchsia replaced with GlyphForeColor

 and another copy with clFuchsia replaced with clGrayText for disbaled items
}
  if not FGlyphInternalChange then
    FGlyphStyle := csCustom
  else
    FGlyphInternalChange := False;
  if FGlyph.Empty then begin
    FMaskedGlyph.Width := 0;
    FMaskedGlyph.Height := 0;
    FHalfGlyphWidth := 0;
    FHalfGlyphHeight := 0;
  end
  else begin
    FMaskedGlyph.Width := FGlyph.Width;
    FMaskedGlyph.Height := FGlyph.Height * 2;
    rcCopy := Rect(0,0,FGlyph.Width,FGlyph.Height);
    FHalfGlyphWidth := FGlyph.Width div 2;
    FHalfGlyphHeight := FGlyph.Height;
    bmpTemp := TBitmap.Create;
    try
      bmpTemp.Width := FGlyph.Width;
      bmpTemp.Height := FGlyph.Height;
      {backcolor}
      with bmpTemp.Canvas do begin
        Brush.Color := Color;
        BrushCopy(rcCopy, FGlyph, rcCopy, clOlive);
      end;
      {forecolor}
      with FMaskedGlyph.Canvas do begin
        Brush.Color := FGlyphForeColor;
        BrushCopy(rcCopy, bmpTemp, rcCopy, clFuchsia);
      end;
      {disabled grayed}
      with FMaskedGlyph.Canvas do begin
        Brush.Color := clGrayText;
        BrushCopy(Rect(0,FGlyph.Height,FGlyph.Width,FMaskedGlyph.Height), bmpTemp, rcCopy, clFuchsia);
      end;
      if HandleAllocated then Invalidate;
    finally
      bmpTemp.Free;
    end;
  end;
end;

function TabcCustomCheckListBox.IsGlyphStored: Boolean;
begin
  Result := GlyphStyle = csCustom;
end;

function TabcCustomCheckListBox.IsItemChecked(Item: String): Boolean;
begin
  Result := CheckedItems.IndexOf(Item) <> -1;
end;

function TabcCustomCheckListBox.IsObjectChecked(AnObject: TObject): Boolean;
begin
  Result := CheckedItems.IndexOfObject(AnObject) <> -1;
end;

function TabcCustomCheckListBox.IsItemDisabled(Item: String): Boolean;
begin
  Result := DisabledItems.IndexOf(Item) <> -1;
end;

function TabcCustomCheckListBox.IsObjectDisabled(AnObject: TObject): Boolean;
begin
  Result := DisabledItems.IndexOfObject(AnObject) <> -1;
end;

procedure TabcCustomCheckListBox.KeyDown(var Key: Word; Shift: TShiftState);
var
  I: Integer;
begin
  inherited KeyDown(Key, Shift);
  if (chSpaceKey in Options) and (Key = Ord(' ')) then
    if (chMultiSelect in Options) and
       (chExtendedSelect in Options) and
       ( not (chShiftRange in Options) or (ssShift in Shift)) then
    begin
      for I := 0 to Items.Count - 1 do
        if Selected[I] then begin
          if not Disabled[I] then
            Checked[I] := not Checked[I];
          if I <> ItemIndex then Selected[I] := False;
        end;
    end
    else
    begin
      I := ItemIndex;
      if (I = -1) then
        if (Items.Count > 0) then
          I := 0
        else
          Exit;
      if not Disabled[I] then
        if (chMultiSelect in Options) then
          Checked[I] := not Checked[I]
        else
        begin
          CheckedItems.Clear;
          Checked[I] := True;
          if HandleAllocated then Invalidate;
        end;
    end;
end;

procedure TabcCustomCheckListBox.LBDeleteString(var Message: TMessage);
begin
  Checked[Message.wParam] := False;
  Disabled[Message.wParam] := False;
  inherited;
end;

procedure TabcCustomCheckListBox.LBResetContent(var Message: TMessage);
begin
  inherited;
  if not (FDestroyingWnd or (csLoading in ComponentState)) then ClearAll;
end;

procedure TabcCustomCheckListBox.Loaded;
begin
  inherited Loaded;
  if CheckedItems.Count > 0 then
    ItemIndex := Items.IndexOf(CheckedItems.Strings[0]);
end;

procedure TabcCustomCheckListBox.MeasureItem(Index: Integer; var Height: Integer);
var
  rcDraw: TRect;
begin
  SetRectEmpty(rcDraw);
  if Columns = 0 then
    rcDraw.Right := Width
  else
    rcDraw.Right := (Width + Columns - 3) div Columns;
  rcDraw.Right := rcDraw.Right - FHalfGlyphWidth - 4 * FGlyphSpace;
  DoDrawText(Canvas, Index, rcDraw, True);
  Height := rcDraw.Bottom;
  inherited MeasureItem(Index, Height);
end;

procedure TabcCustomCheckListBox.RedrawCheckedItems;
var
  I, FoundIndex, TopIdx: Integer;
begin
  TopIdx := TopIndex;
  for I := 0 to CheckedItems.Count - 1 do begin
    FoundIndex := Items.IndexOf(CheckedItems.Strings[I]);
    if (FoundIndex > TopIdx) then
      RedrawItem(FoundIndex);
  end;
end;

procedure TabcCustomCheckListBox.RedrawItem(Index: Integer);
var
  rcDraw: tRect;
begin
  if HandleAllocated and
     (Index >= TopIndex) then begin
    rcDraw := ItemRect(Index);
    if rcDraw.Top <= Height then begin
      InvalidateRect(Handle, @rcDraw, False);
      UpdateWindow(Handle);
    end;
  end;
end;

procedure TabcCustomCheckListBox.SetAlignment(Value: TAlignment);
begin
  if Value <> FAlignment then begin
    FAlignment := Value;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetChecked(Index: Integer; Value: Boolean);
var
  strIndex: string;
  FoundIndex: Integer;
  EventValue: Boolean;
begin
  strIndex := Items.Strings[Index];
  FoundIndex := CheckedItems.IndexOf(strIndex);
  if FoundIndex = -1 then begin
    if Value then begin
      if Assigned(FOnCheck) then begin
        EventValue := True;
        FOnCheck(Self, Index, EventValue);
        if EventValue <> Value then Exit;
      end;
      CheckedItems.AddObject(strIndex, Items.Objects[Index]);
      RedrawItem(Index);
    end
  end
  else begin
    if not Value then begin
      if Assigned(FOnCheck) then begin
        EventValue := False;
        FOnCheck(Self, Index, EventValue);
        if EventValue <> Value then Exit;
      end;
      CheckedItems.Delete(FoundIndex);
      RedrawItem(Index);
    end;
  end;
end;

procedure TabcCustomCheckListBox.SetDisabled(Index: Integer; Value: Boolean);
var
  strIndex: string;
  FoundIndex: Integer;
begin
  strIndex := Items.Strings[Index];
  FoundIndex := DisabledItems.IndexOf(strIndex);
  if Value and (FoundIndex = -1) then begin
    DisabledItems.AddObject(strIndex, Items.Objects[Index]);
    RedrawItem(Index);
  end
  else if not Value and (FoundIndex > -1) then
  begin
    DisabledItems.Delete(FoundIndex);
    RedrawItem(Index);
  end;
end;

procedure TabcCustomCheckListBox.SetCheckedColor(Value: TColor);
begin
  if Value <> FCheckedColor then begin
    FCheckedColor := Value;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetCheckedColorChange(Value: Boolean);
begin
  if Value <> FCheckedColorChange then begin
    FCheckedColorChange := Value;
    if Value then CheckedColor := Font.Color;
  end;
end;

procedure TabcCustomCheckListBox.SetCheckedItems(Value: TStrings);
var
  I: Integer;
begin
  if Value <> FCheckedItems then begin
    for I := 0 to Value.Count - 1 do begin
      if (Items.IndexOf(Value.Strings[I]) = -1) then
        raise EListError.Create('Value does not exist in Items');
    end;
    if (chMultiSelect in Options) then
      FCheckedItems.Assign(Value)
    else begin
      FCheckedItems.Clear;
      if Value.Count > 0 then
        FCheckedItems.AddObject(Value.Strings[0], Value.Objects[0]);
    end;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetDisabledItems(Value: TStrings);
var
  I: Integer;
begin
  if Value <> FDisabledItems then begin
    for I := 0 to Value.Count - 1 do begin
      if (Items.IndexOf(Value.Strings[I]) = -1) then
        raise EListError.Create('Value does not exist in Items');
    end;
    FDisabledItems.Assign(Value);
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetDisabledItemsGrayed(Value: Boolean);
begin
  if Value <> FDisabledItemsGrayed then begin
    FDisabledItemsGrayed := Value;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetColumns(Value: Integer);
begin
  if Value <> Columns then begin
    inherited Columns := Value;
    if WordWrap and (Value = 0) then
      Style := lbOwnerDrawVariable
    else
      Style := lbOwnerDrawFixed;
  end;
end;

procedure TabcCustomCheckListBox.SetExpandTabs(Value: Boolean);
begin
  if Value <> FExpandTabs then begin
    FExpandTabs := Value;
    if HandleAllocated then
      if WordWrap then
        RecreateWnd
      else
        Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetGlyph(Value: TBitmap);
begin
  if Value <> FGlyph then
    FGlyph.Assign(Value);
end;

procedure TabcCustomCheckListBox.SetGlyphAlignH(Value: TabcGlyphAlignH);
begin
  if Value <> FGlyphAlignH then begin
    FGlyphAlignH := Value;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetGlyphAlignV(Value: TabcGlyphAlignV);
begin
  if Value <> FGlyphAlignV then begin
    FGlyphAlignV := Value;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetGlyphForeColor(Value: TColor);
begin
  if Value <> FGlyphForeColor then begin
    FGlyphForeColor := Value;
    FGlyphInternalChange := True;
    GlyphChanged(Self);
  end;
end;

procedure TabcCustomCheckListBox.SetGlyphSpace(Value: TabcGlyphSpace);
begin
  if Value <> FGlyphSpace then begin
    FGlyphSpace := Value;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetGlyphStyle(Value: TabcCheckStyle);
begin
  FGlyphStyle := Value;
  FGlyphInternalChange := (Value <> csCustom);
  case Value of
    csTickSunken:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMSCHECK');
    csTickFlat:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMFCHECK');
    csTickRaised:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMRCHECK');
    csTickTransparent:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMTCHECK');
    csTick3D:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBM3DCHECK');
    csCrossSunken:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMSCROSS');
    csCrossFlat:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMFCROSS');
    csCrossRaised:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMRCROSS');
    csCrossTransparent:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMTCROSS');
    csCross3D:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBM3DCROSS');
    csDiamondSunken:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMSDIAM');
    csDiamondFlat:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMFDIAM');
    csDiamondRaised:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMRDIAM');
    csDiamond3D:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBM3DDIAM');
    csCustom: { do nothing };
    csTickFlatWindows:
      FGlyph.Handle := LoadBitmap(HInstance, 'ABCBMFCHECKWIN');
  end;
end;

function TabcCustomCheckListBox.SetItemState(Item: string; Value: Boolean): Boolean;
var
  FoundIndex: Integer;
begin
  FoundIndex := Items.IndexOf(Item);
  Result := (FoundIndex <> -1);
  if Result then Checked[FoundIndex] := Value;
end;

function TabcCustomCheckListBox.SetObjectState(AnObject: TObject; Value: Boolean): Boolean;
var
  FoundIndex: Integer;
begin
  FoundIndex := Items.IndexOfObject(AnObject);
  Result := (FoundIndex <> -1);
  if Result then Checked[FoundIndex] := Value;
end;

function TabcCustomCheckListBox.SetItemDisabledState(Item: string; Value: Boolean): Boolean;
var
  FoundIndex: Integer;
begin
  FoundIndex := Items.IndexOf(Item);
  Result := (FoundIndex <> -1);
  if Result then Disabled[FoundIndex] := Value;
end;

function TabcCustomCheckListBox.SetObjectDisabledState(AnObject: TObject; Value: Boolean): Boolean;
var
  FoundIndex: Integer;
begin
  FoundIndex := Items.IndexOfObject(AnObject);
  Result := (FoundIndex <> -1);
  if Result then Disabled[FoundIndex] := Value;
end;

procedure TabcCustomCheckListBox.SetOptions(Value: TabcCheckOptions);
var
  SaveIndex: Integer;
  RestoreSelected, ChangeExtended: Boolean;
begin
  if Value <> FOptions then begin
    ChangeExtended := (chExtendedSelect in Value) <> (chExtendedSelect in FOptions);
    if ChangeExtended and (chExtendedSelect in Value) then
      Value := Value + [chMultiSelect];
    if (chClick in Value) and not (chClick in FOptions) then
      FOptions := Value - [chClickGlyph, chDblClick]
    else if (chClickGlyph in Value) and not (chClickGlyph in FOptions) then
      FOptions := Value - [chClick, chDblClick]
    else if (chDblClick in Value) and not (chDblClick in FOptions) then
      FOptions := Value - [chClick, chClickGlyph]
    else
      FOptions := Value;
    if not (csDesigning in ComponentState) then
      if ChangeExtended then
      begin
        inherited ExtendedSelect := (chExtendedSelect in Value);
        inherited MultiSelect := (chExtendedSelect in Value);
      end;
    if not (chMultiSelect in FOptions) and (CheckedItems.Count > 1) then
    begin
      SaveIndex := ItemIndex;
      if (ItemIndex > -1) and Checked[ItemIndex] then
        RestoreSelected := True
      else
        RestoreSelected := False;
      ClearAll;
      ItemIndex := SaveIndex;
      if RestoreSelected then
        Checked[SaveIndex] := True;
    end;
  end;
end;

procedure TabcCustomCheckListBox.SetWordWrap(Value: Boolean);
begin
  if Value <> FWordWrap then begin
    FWordWrap := Value;
    if Value and (Columns = 0) then
      Style := lbOwnerDrawVariable
    else
      Style := lbOwnerDrawFixed;
    if HandleAllocated then Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetShowSelectedItem(Value: Boolean);
begin
  if Value <> FShowSelectedItem then
  begin
    FShowSelectedItem := Value;
    Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.SetShowFocusRect(Value: Boolean);
begin
  if Value <> FShowFocusRect then
  begin
    FShowFocusRect := Value;
    Invalidate;
  end;
end;

procedure TabcCustomCheckListBox.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  inherited;
  if chDblClick in Options then
    DoMouseRangeSelect(SmallPointToPoint(Message.Pos), (Message.Keys and MK_SHIFT <> 0));
end;

procedure TabcCustomCheckListBox.WMLButtonUp(var Message: TWMLButtonUp);
begin
  inherited;
  if (chClick in Options) or (chClickGlyph in Options) then
    DoMouseRangeSelect(SmallPointToPoint(Message.Pos), (Message.Keys and MK_SHIFT <> 0));
end;

function TabcCustomCheckListBox.GetAsInteger: Integer;
var
  i, iPowerMax: integer;
begin
  Result := 0;
  iPowerMax := Items.Count - 1;
  for i := 0 to CheckedItems.Count - 1 do
    Result := Result + 1 shl (iPowerMax - Items.IndexOf(CheckedItems[i]));
end;

procedure TabcCustomCheckListBox.SetAsInteger(Value: Integer);
var
  iTemp, i: integer;
begin
  iTemp := Value;
  ClearAll;
  for i := Items.Count - 1 downto 0 do
  begin
    Checked[i] := ((iTemp and $01) <> 0);
    iTemp := iTemp shr 1;
    if iTemp = 0 then Exit;
  end;
end;

{ TabcCheckListBox }

procedure TabcCheckListBox.CheckAll;
begin
  inherited CheckAll;
end;

procedure TabcCheckListBox.CheckAllExceptDisabled;
begin
  inherited CheckAllExceptDisabled;
end;

procedure TabcCheckListBox.ClearAll;
begin
  inherited ClearAll;
end;

procedure TabcCheckListBox.DisableAll;
begin
  inherited DisableAll;
end;

function TabcCheckListBox.IsItemChecked(Item: string): Boolean;
begin
  Result := inherited IsItemChecked(Item);
end;

function TabcCheckListBox.IsObjectChecked(AnObject: TObject): Boolean;
begin
  Result := inherited IsObjectChecked(AnObject);
end;

function TabcCheckListBox.IsItemDisabled(Item: string): Boolean;
begin
  Result := inherited IsItemDisabled(Item);
end;

function TabcCheckListBox.IsObjectDisabled(AnObject: TObject): Boolean;
begin
  Result := inherited IsObjectDisabled(AnObject);
end;

function TabcCheckListBox.SetItemState(Item: string; Value: Boolean): Boolean;
begin
  Result := inherited SetItemState(Item, Value);
end;

function TabcCheckListBox.SetObjectState(AnObject: TObject; Value: Boolean): Boolean;
begin
  Result := inherited SetObjectState(AnObject, Value);
end;

function TabcCheckListBox.SetItemDisabledState(Item: string; Value: Boolean): Boolean;
begin
  Result := inherited SetItemDisabledState(Item, Value);
end;

function TabcCheckListBox.SetObjectDisabledState(AnObject: TObject; Value: Boolean): Boolean;
begin
  Result := inherited SetObjectDisabledState(AnObject, Value);
end;

{ TabcCustomTabListBox }

constructor TabcCustomTabListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := lbOwnerDrawFixed;
end;

procedure TabcCustomTabListBox.CreateWnd;
begin
  inherited CreateWnd;
  SetTabPositions(TabPositions);
  if (ScrollWidth > 0) then
    Perform(LB_SETHORIZONTALEXTENT, ScrollWidth, 0);
end;

procedure TabcCustomTabListBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  DrawProc: TDrawItemEvent;
begin
  DrawProc := OnDrawItem;
  if Assigned(DrawProc) then
    OnDrawItem(Self, Index, Rect, State)
  else begin
    Canvas.FillRect(Rect);
    if Index < Items.Count then
      if (FNumTabs > 0) then
        TabbedTextOut(Canvas.Handle, Rect.Left + 2, Rect.Top,
          pChar(Items.Strings[Index]), Length(Items.Strings[Index]),
          FNumTabs, FTabs, 0)
      else
        Canvas.TextOut(Rect.Left + 2, Rect.Top, Items.Strings[Index]);
  end;
end;

procedure TabcCustomTabListBox.SetScrollWidth(Value: Word);
begin
  if Value <> FScrollWidth then begin
    FScrollWidth := Value;
    if Value <> 0 then Columns := 0;
    if HandleAllocated then Perform(LB_SETHORIZONTALEXTENT, Value, 0);
  end;
end;

procedure TabcCustomTabListBox.CMFontChanged(var Message: TMessage);
begin
  inherited;
  SelectObject(Canvas.Handle, Font.Handle);
  ItemHeight := Canvas.TextHeight('W');
end;

procedure TabcCustomTabListBox.SetTabPositions(Value: string);
{expecting a string of , separated integers in increasing order}
var
  I, ntemp, sepPos: Integer;
  strTemp: string;
begin
  if not HandleAllocated then
    FTabPositions := Value
  else begin
    if Value = '' then
      I := 0
    else begin
      {parse string of integers into tab array}
      strTemp := Value;
      sepPos := Pos(',', strTemp);
      I := 0;
      while (sepPos > 0) and (I < 32) do begin
        FTabs[I] := StrToInt(Copy(strTemp, 1, sepPos - 1));
        strTemp := Copy(strTemp, sepPos + 1, Length(strTemp));
        sepPos := Pos(',', strTemp);
        Inc(I);
      end;
      {remainder of string}
      if (I < 32) and (Length(StrTemp) > 0) then begin
        ntemp := StrToIntDef(strTemp, 0);
        if ntemp <> 0 then begin
          FTabs[I] := ntemp;
          Inc(I);
        end;
      end;
    end;
    FTabPositions := Value;
    FNumTabs := I;
    Invalidate;
  end;
end;

{ TabcCustomSyncListBox }

constructor TabcCustomSyncListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPrintIndent := 5;
  FPrintLines := 60;
  FPrinting := True;
  FSyncOptions := [lsHScroll, lsVScroll, lsClick, lsSelItem, lsTopIndex];
end;

procedure TabcCustomSyncListBox.CreateWnd;
begin
  inherited CreateWnd;
  if (Columns = 0) and (ScrollWidth > 0) then
    Perform(LB_SETHORIZONTALEXTENT, ScrollWidth, 0);
end;

procedure TabcCustomSyncListBox.Print;
begin
  abcPrintStringsFormat(Items, FPrintIndent, FPrintLines);
end;

procedure TabcCustomSyncListBox.LBSetCaretIndex(var Message: TMessage);
begin
  inherited;
  if (FSyncControl <> nil) and
     (lsSelItem in FSyncOptions) and
     not FScrollBySyncCtrl then
    SendSyncMessage(Message);
end;

procedure TabcCustomSyncListBox.LBSetCurSel(var Message: TMessage);
begin
  inherited;
  if (FSyncControl <> nil) and
     (lsSelItem in FSyncOptions) and
     not FScrollBySyncCtrl then
    SendSyncMessage(Message);
end;

procedure TabcCustomSyncListBox.LBSetSel(var Message: TMessage);
begin
  inherited;
  if (FSyncControl <> nil) and
     (lsSelItem in FSyncOptions) and
     not FScrollBySyncCtrl then
    SendSyncMessage(Message);
end;

procedure TabcCustomSyncListBox.LBSetTopIndex(var Message: TMessage);
begin
  inherited;
  if (FSyncControl <> nil) and
     (lsTopIndex in FSyncOptions) and
     not FScrollBySyncCtrl then
    SendSyncMessage(Message);
end;

procedure TabcCustomSyncListBox.SendSyncMessage(Message: TMessage);
begin
  if (FSyncControl <> nil) then begin
    FSyncControl.ScrollBySyncCtrl := True;
    with Message do SendMessage(FSyncControl.Handle, Msg, wParam, lParam);
    FSyncControl.ScrollBySyncCtrl := False;
  end;
end;

procedure TabcCustomSyncListBox.SetScrollWidth(Value: Word);
begin
  if Value <> FScrollWidth then begin
    FScrollWidth := Value;
    if Value <> 0 then Columns := 0;
    if HandleAllocated then Perform(LB_SETHORIZONTALEXTENT, Value, 0);
  end;
end;

procedure TabcCustomSyncListBox.SetSyncControl(Value: TabcCustomSyncListBox);
begin
  if (Value <> FSyncControl) and (Value <> Self) then
    FSyncControl := Value;
end;

procedure TabcCustomSyncListBox.WMHScroll(var Message: TMessage);
begin
  inherited;
  if (FSyncControl <> nil) and
     (lsHScroll in FSyncOptions) and
     not FScrollBySyncCtrl then
    SendSyncMessage(Message);
end;

procedure TabcCustomSyncListBox.Click;
var
  I: Integer;
begin
  inherited Click;
  if (FSyncControl <> nil) and
     (lsClick in FSyncOptions) and
     not FScrollBySyncCtrl then
    begin
      FSyncControl.ScrollBySyncCtrl := True;
      if MultiSelect then begin
        if FSyncControl.MultiSelect then
          for I := 0 to Items.Count - 1 do
            if I < FSyncControl.Items.Count then
              FSyncControl.Selected[I] := Selected[I]
            else
              Break;
      end
      else
        FSyncControl.ItemIndex := ItemIndex;
      FSyncControl.ScrollBySyncCtrl := False;
    end;
end;

procedure TabcCustomSyncListBox.WMVScroll(var Message: TMessage);
begin
  inherited;
  if (FSyncControl <> nil) and
     (lsVScroll in FSyncOptions) and
     not FScrollBySyncCtrl then
    SendSyncMessage(Message);
end;

procedure TabcCustomSyncListBox.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
    if (AComponent is TabcCustomSyncListBox) then
      if TabcCustomSyncListBox(AComponent) = FSyncControl then
        FSyncControl := nil;
end;

procedure TabcCustomSyncListBox.CNKeyUp(var Message: TWMKeyUp);
begin
  if (Message.CharCode = 44) and FPrinting then
    Print
  else
    inherited;
end;

end.
