unit abcmdtab;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, Classes, Controls, Graphics, Forms,
  CommCtrl, ComCtrls, ImgList;

type

  { TabcMDITabs }

  TabcMDITabOption = (amtCaption, amtIcon);
  TabcMDITabOptions = set of TabcMDITabOption;

  TabcMDITabs = class(TCustomTabControl)
  private
    FFormIconImages: TImageList;
    FClientInstance: TFarProc;
    FClientHandle: HWND;
    FDefClientProc: TFarProc;
    FOwnerForm: TForm;
    FSyncTabs: Boolean;
    FDestroyingChild: TForm;
    FOptions: TabcMDITabOptions;
    procedure AddFormIcon(AForm: TForm);
    procedure MDIChildCreated;
    procedure MDIChildDestroyed;
    procedure MDIChildActivated;
    procedure MDIClientProc(var Message: TMessage);
    procedure SubClassMDIClient;
    procedure SetOptions(const Value: TabcMDITabOptions);
  protected
    function CanChange: Boolean; override;
    procedure Change; override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property DisplayRect;
    procedure RefreshTabs;
  published
    property Align default alTop;
    property Anchors;
    property BevelEdges;
    property BevelKind;
    property BevelInner;
    property BevelOuter;
    property BiDiMode;
    property Constraints;
    property Enabled;
    property Font;
    property HotTrack;
    property MultiLine;
    property MultiSelect;
    property OwnerDraw;
    property ParentBiDiMode;
    property Options: TabcMDITabOptions read FOptions write SetOptions
      default [amtCaption, amtIcon];
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property RaggedRight;
    property ScrollOpposite;
    property ShowHint;
    property Style default tsButtons;
    property TabHeight;
    property TabOrder;
    property TabPosition;
    property TabStop;
    property TabWidth;
    property Visible;
    property OnChange;
    property OnChanging;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}    
    property OnDrawTab;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
  end;

implementation

uses abcconst;

constructor TabcMDITabs.Create(AOwner: TComponent);
begin
  inherited;
  if (AOwner is TForm) then
    if (TForm(AOwner).FormStyle = fsMDIForm) then
      inherited Align := alClient
    else
      raise EComponentError.Create(sabcErrorOwnerMDIFormExpected)
  else
    raise EComponentError.Create(sabcErrorOwnerMDIFormExpected);
  FOwnerForm := TForm(AOwner);
  Align := alTop;
  FFormIconImages := TImageList.Create(Self);
  Images := FFormIconImages;
  Style := tsButtons;
  Height := 23;
  FOptions := [amtCaption, amtIcon];
end;

destructor TabcMDITabs.Destroy;
begin
  Destroying;
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FDefClientProc));
  {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FClientInstance);
  Images := nil;
  FFormIconImages.Free;
  inherited;
end;

procedure TabcMDITabs.Loaded;
begin
  inherited Loaded;
  if FClientHandle = 0 then
    if not(csDesigning in ComponentState) then
      SubClassMDIClient;
end;

procedure TabcMDITabs.SubClassMDIClient;
begin
  FClientHandle := FOwnerForm.ClientHandle;
  FClientInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(MDIClientProc);
  FDefClientProc := Pointer(GetWindowLong(FClientHandle, GWL_WNDPROC));
  SetWindowLong(FClientHandle, GWL_WNDPROC, Longint(FClientInstance));
end;

procedure TabcMDITabs.MDIClientProc(var Message: TMessage);
var
  i: integer;
begin
  if Message.Msg = WM_MDIDESTROY then
    for i := 0 to Tabs.Count - 1 do
      if TForm(Tabs.Objects[i]).Handle = THandle(Message.wParam) then
      begin
        FDestroyingChild := TForm(Tabs.Objects[i]);
        Break;
      end;
  Message.Result := CallWindowProc(FDefClientProc, FClientHandle, Message.Msg, Message.WParam, Message.LParam);
  case Message.Msg of
    WM_MDICREATE:
      MDIChildCreated;
    WM_MDIDESTROY:
      MDIChildDestroyed;
    WM_MDIACTIVATE, WM_SETFOCUS:
      MDIChildActivated;
  end;
end;

procedure TabcMDITabs.AddFormIcon(AForm: TForm);
var
  tempIcon: TIcon;
begin
  if (AForm.Icon <> nil) and not AForm.Icon.Empty then
    FFormIconImages.AddIcon(AForm.Icon)
  else if (Application.Icon <> nil) and not Application.Icon.Empty then
    FFormIconImages.AddIcon(Application.Icon)
  else
  begin
    tempIcon := TIcon.Create;
    try
      tempIcon.Handle := LoadIcon(MainInstance, 'MAINICON');
      if tempIcon.Handle = 0 then
        tempIcon.Handle := LoadIcon(0, IDI_APPLICATION);
      FFormIconImages.AddIcon(tempIcon);
    finally
      tempIcon.Free;
    end;
  end;
end;

procedure TabcMDITabs.MDIChildCreated;
var
  ChildForm: TForm;
begin
  FSyncTabs := True;
  with FOwnerForm do
    if MDIChildCount > Tabs.Count then
    begin
      ChildForm := MDIChildren[MDIChildCount-1];
      if amtCaption in Options then
        Tabs.AddObject(ChildForm.Caption, ChildForm)
      else
        Tabs.AddObject('', ChildForm);
      if amtIcon in Options then
        AddFormIcon(ChildForm);
      TabIndex := Tabs.Count - 1;
    end;
  FSyncTabs := False;
end;

procedure TabcMDITabs.MDIChildDestroyed;
var
  i: integer;
begin
  FSyncTabs := True;
  for i := 0 to Tabs.Count do
    if TForm(Tabs.Objects[i]) = FDestroyingChild then
    begin
      Tabs.Delete(i);
      Perform(TCM_REMOVEIMAGE, i, 0);
      Break;
    end;
  FSyncTabs := False;
end;

procedure TabcMDITabs.MDIChildActivated;
var
  i: integer;
begin
  FSyncTabs := True;
  for i := 0 to Tabs.Count - 1 do
    if TForm(Tabs.Objects[i]) = FOwnerForm.ActiveMDIChild then
    begin
      TabIndex := i;
      Break;
    end;
  FSyncTabs := False;
end;

procedure TabcMDITabs.Change;
begin
  if not FSyncTabs then
  begin
    if (TabIndex > -1) and (TabIndex < FOwnerForm.MDIChildCount) then
      with TForm(Tabs.Objects[TabIndex]) do
      begin
        if WindowState = wsMinimized then
          WindowState := wsNormal;
        BringToFront;
        SetFocus;
      end;
    inherited;
  end;
end;

function TabcMDITabs.CanChange: Boolean;
begin
  if FSyncTabs then
    Result := True
  else
    Result := inherited CanChange;
end;

procedure TabcMDITabs.RefreshTabs;
var
  i: integer;
begin
  FFormIconImages.Clear;
  for i := 0 to Tabs.Count - 1 do
  begin
    if amtCaption in Options then
      Tabs[i] := TForm(Tabs.Objects[i]).Caption
    else
      Tabs[i] := '';
    if amtIcon in Options then
      AddFormIcon(TForm(Tabs.Objects[i]));
  end;
end;

procedure TabcMDITabs.SetOptions(const Value: TabcMDITabOptions);
begin
  if Value <> FOptions then
  begin
    FOptions := Value;
    if [csDesigning, csLoading] * ComponentState = [] then
      RefreshTabs;
  end;
end;

end.
