unit abcmemo;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, Classes, Controls, StdCtrls;

type

  TabcMemoSyncOption = (amsCaret, amsHScroll, amsVScroll, amsSetRect, amsMargins);
  TabcMemoSyncOptions = set of TabcMemoSyncOption;


  TabcROMemo = class(TMemo)
  private
    FPrintIndent: Byte;
    FPrintLines: Byte;
    FPrinting: Boolean;
    FMaxEditLines: Word;

    FSyncMemo: TabcROMemo;
    FSyncOptions: TabcMemoSyncOptions;
    FScrollBySyncMemo: Boolean;

    procedure CNKeyUp(var Message: TWMKeyUp); message CN_KEYUP;
    function GetReadOnly: Boolean;
    procedure SetReadOnly(Value: Boolean);
    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMRButtonDown(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;

    procedure SendSyncMessage(Message: TMessage);
    procedure SyncCaretPos;
    procedure SetSyncMemo(Value: TabcROMemo);

    procedure EMLineScroll(var Message: TMessage); message EM_LINESCROLL;
    procedure EMSetMargins(var Message: TMessage); message EM_SETMARGINS;
    procedure EMSetRect(var Message: TMessage); message EM_SETRECT;
    procedure EMSetRectNP(var Message: TMessage); message EM_SETRECTNP;
    procedure EMSetSel(var Message: TMessage); message EM_SETSEL;
    procedure EMScroll(var Message: TMessage); message EM_SCROLL;
    procedure EMScrollCaret(var Message: TMessage); message EM_SCROLLCARET;

    procedure WMHSCroll(var Message: TMessage); message WM_HSCROLL;
    procedure WMVSCroll(var Message: TMessage); message WM_VSCROLL;
    
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
    procedure Print;
    property ScrollBySyncMemo: Boolean read FScrollBySyncMemo write FScrollBySyncMemo; {flag to avoid circular updates}
  published
    property MaxEditLines: Word read FMaxEditLines write FMaxEditLines;
    property PrintIndent: Byte read FPrintIndent write FPrintIndent default 5;
    property PrintLines: Byte read FPrintLines write FPrintLines default 60;
    property Printing: Boolean read FPrinting write FPrinting default True;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly;
    property SyncOptions: TabcMemoSyncOptions read FSyncOptions write FSyncOptions
      default [amsCaret, amsHScroll, amsVScroll, amsSetRect, amsMargins];
    property SyncMemo: TabcROMemo read FSyncMemo write SetSyncMemo;
  end;

implementation

uses abcutilf;

constructor TabcROMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPrintIndent := 5;
  FPrintLines := 60;
  FPrinting := True;
  FSyncOptions := [amsCaret, amsHScroll, amsVScroll, amsSetRect, amsMargins];
end;

procedure TabcROMemo.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
    if AComponent = FSyncMemo then
      FSyncMemo := nil;
end;

procedure TabcROMemo.Click;
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsCaret in FSyncOptions) and
     not FScrollBySyncMemo then
    SyncCaretPos;
end;

procedure TabcROMemo.SyncCaretPos;
var
  firstvisibleLine, syncFirstVisibleLine: Integer;
begin
  firstvisibleLine := SendMessage(Handle, EM_GETFIRSTVISIBLELINE, 0, 0);
  with FSyncMemo do
  begin
    ScrollBySyncMemo := True;
{$IFDEF DELPHI6}
    CaretPos := Self.CaretPos;
{$ENDIF}
    SendMessage(Handle, EM_SCROLLCARET, 0, 0);
    syncFirstVisibleLine := SendMessage(Handle, EM_GETFIRSTVISIBLELINE, 0, 0);
    if firstvisibleLine <> syncFirstVisibleLine then
      SendMessage(Handle, EM_LINESCROLL, 0, firstvisibleLine - syncFirstVisibleLine);
    ScrollBySyncMemo := False;
  end;
end;

procedure TabcROMemo.CNKeyUp(var Message: TWMKeyUp);
begin
  if (Message.CharCode = 44) and FPrinting then
    Print
  else
  begin
    inherited;
  end;
end;

procedure TabcROMemo.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if ReadOnly then
    if (Shift = []) then
      if (Key in [VK_PRIOR..VK_DOWN]) then
        inherited KeyDown(Key, Shift)
      else
        Key := 0
    else if (Shift = [ssCtrl]) then
      if (Key in [VK_END, VK_HOME]) then
        inherited KeyDown(Key, Shift)
      else
        Key := 0
    else
      Key := 0
  else
    inherited KeyDown(Key, Shift);
end;

procedure TabcROMemo.KeyPress(var Key: Char);
begin
  if (Key in [#10,#13]) and
     (FMaxEditLines > 0) and
     (Lines.Count >= FMaxEditLines) then
    Key := #0;
  inherited KeyPress(Key);
end;

procedure TabcROMemo.KeyUp(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key in [VK_END..VK_DOWN] then
    SyncCaretPos;
end;

procedure TabcROMemo.Print;
begin
  abcPrintStringsFormat(Lines, FPrintIndent, FPrintLines);
end;

procedure TabcROMemo.WMLButtonDown(var Message: TWMLButtonDown);
begin
  if ReadOnly then
    SetFocus
  else
    inherited;
end;

procedure TabcROMemo.WMRButtonDown(var Message: TWMRButtonDown);
begin
  if ReadOnly then
    SetFocus
  else
    inherited;
end;

procedure TabcROMemo.WMNCHitTest(var Message: TWMNCHitTest);
begin
  inherited;
  if ReadOnly and not (csDesigning in ComponentState) then
    if (Message.Result = HTCLIENT) and Focused then
      Message.Result := HTNOWHERE;
end;

function TabcROMemo.GetReadOnly: Boolean; begin Result := Inherited ReadOnly; end;

procedure TabcROMemo.SetReadOnly(Value: Boolean);
begin
  inherited ReadOnly := Value;
  if ReadOnly then SelLength := 0;
end;

procedure TabcROMemo.EMLineScroll(var Message: TMessage);
var
  NewMessage: TMessage;
begin
  inherited;
  if (FSyncMemo <> nil) and
     ((amsHScroll in FSyncOptions) or (amsVScroll in FSyncOptions)) and
     not FScrollBySyncMemo then
  begin
    NewMessage := Message;
    if not (amsHScroll in FSyncOptions) then
      NewMessage.WParam := 0;
    if not (amsVScroll in FSyncOptions) then
      NewMessage.LParam := 0;
    SendSyncMessage(NewMessage);
  end;
end;

procedure TabcROMemo.EMScroll(var Message: TMessage);
//same as WM_VSCROLL
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsVScroll in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

procedure TabcROMemo.EMScrollCaret(var Message: TMessage);
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsCaret in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

procedure TabcROMemo.EMSetMargins(var Message: TMessage);
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsMargins in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

procedure TabcROMemo.EMSetRect(var Message: TMessage);
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsSetRect in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

procedure TabcROMemo.EMSetRectNP(var Message: TMessage);
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsSetRect in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

procedure TabcROMemo.EMSetSel(var Message: TMessage);
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsCaret in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

procedure TabcROMemo.SendSyncMessage(Message: TMessage);
begin
  if (FSyncMemo <> nil) then
  begin
    FSyncMemo.ScrollBySyncMemo := True;
    with Message do
      SendMessage(FSyncMemo.Handle, Msg, wParam, lParam);
    FSyncMemo.ScrollBySyncMemo := False;
  end;
end;

procedure TabcROMemo.SetSyncMemo(Value: TabcROMemo);
begin
  if (Value <> FSyncMemo) and (Value <> Self) then
    FSyncMemo := Value;
end;

procedure TabcROMemo.WMHSCroll(var Message: TMessage);
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsHScroll in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

procedure TabcROMemo.WMVSCroll(var Message: TMessage);
begin
  inherited;
  if (FSyncMemo <> nil) and
     (amsVScroll in FSyncOptions) and
     not FScrollBySyncMemo then
    SendSyncMessage(Message);
end;

end.
