unit abcmenu;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
This unit based on code licensed from Andrew Cher http://members.xoom.com/abcpp/
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms, Menus, Buttons, ComCtrls, imglist,
  abcmsg, abctoolb, abctlbr, abctlwn, abctctls, abcmopts, abcwav;

type

  {$IFNDEF DELPHI5}
    TImageIndex = type Integer;
  {$ENDIF}

  { TabcMenuToolbar }

  TabcMenuButtonOrientation = (aorAutomatic, aorHorizontal, aorVertical);

  TabcMenuToolbarSize = (amsOffice, amsDelphi);

  TabcMenuToolbar = class(TabcToolbar)
  private
    SpaceButton: TSpeedButton;
    FMainMenu: TMainMenu;
    KeepSelected: Boolean;
    FUpdating: Boolean;
    FSBPanelNo: Integer;
    FStatusBar: TStatusBar;
    FOnPopup: TNotifyEvent;
    FOnMenuCommand: TNotifyEvent;
    FOnMenuClose: TNotifyEvent;
    FOptions: TabcMenuOptions;
    FMergedMenusList: TList;
    FGrayInactive: Boolean;
    FGrayActivated: Boolean;
    FMMFont: TFont;
    FMergedMenus: Boolean;
    FSystemFont: Boolean;
    FOpaqueButtons: Boolean;
    FButtonOrientation: TabcMenuButtonOrientation;
    FButtonAutoSize : Boolean;
    FSaveMenuItemCount: Integer;
    FSaveMenuVisibleItems: Integer;
    FMergingMenus: Boolean;
    FOldDisabledStyle: Boolean;
    FOwnerForm: TForm;
    FBarSize: TabcMenuToolbarSize;
    FMDIChildMenu: TMainMenu;
    procedure SetMainMenu(Value: TMainMenu);
    procedure SetSystemFont(Value: Boolean);
    procedure SetOptions(Value: TabcMenuOptions);
    function IsFontStored: Boolean;
    procedure SetOpaqueButtons(Value: Boolean);
    procedure SetOldDisabledStyle(Value: Boolean);
    procedure CheckForMDIChildMenu(Sender: TObject);
    procedure SetGrayActivated(Value: Boolean);
    procedure MenuSystemClick(Sender: TObject);
    procedure SetMenuProperties(Menu: TPopupMenu);
    procedure SetButtonOrientation(Value: TabcMenuButtonOrientation);
    procedure SetButtonAutoSize(Value: Boolean);
    procedure SetBarSize(const Value: TabcMenuToolbarSize);
    procedure MergeMenus;
    procedure EndUpdate1;
    procedure HideOwnerFormMenu;
    function CreateButtonForMenu(AMenuItem: TMenuItem; AOwner: TComponent;
      IsSystemButton: Boolean): TabcToolbarButton;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
    procedure CMSysFontChanged(var Message: TMessage); message CM_SYSFONTCHANGED;
    procedure AMCheckForMDIChildMenu(var Message: TMessage); message AM_CHECKFORMDICHILDMENU;
    procedure AMRebuildMenuToolbar(var Message: TMessage); message AM_REBUILDMENUTOOLBAR;
  protected
    procedure DestroyWindowHandle; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
    procedure GetBarSize (var ASize: Integer; const DockType: TabcDockType); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure InitiateAction; override;
    procedure Rebuild; virtual;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    procedure UpdateMenuItem(AMenuItem: TMenuItem); virtual;
    procedure UpdateTopLevelMenuItems;
    property GrayActivated : Boolean read FGrayActivated write SetGrayActivated;
    property ButtonOrientation: TabcMenuButtonOrientation read FButtonOrientation write SetButtonOrientation default aorAutomatic;
    property ButtonAutoSize : Boolean read FButtonAutoSize write SetButtonAutoSize default True;
    property OwnerForm: TForm read FOwnerForm;
  published
    property BarSize: TabcMenuToolbarSize read FBarSize write SetBarSize default amsOffice;
    property FullSize default True;
    property MainMenu: TMainMenu read FMainMenu write SetMainMenu;
    property StatusBar: TStatusBar read FStatusBar write FStatusBar;
    property StatusBarIndex: Integer read FSBPanelNo write FSBPanelNo;
    property Options: TabcMenuOptions read FOptions write SetOptions;
    property GrayWhenInactive: Boolean read FGrayInactive write FGrayInactive default True;
    property SystemFont: Boolean read FSystemFont write SetSystemFont default False;
    property Font stored IsFontStored;
    property OldDisabledStyle: Boolean read FOldDisabledStyle write SetOldDisabledStyle default False;
    property OpaqueButtons: Boolean read FOpaqueButtons write SetOpaqueButtons default True;
    property ParentFont;
    property ParentShowHint;
    property CloseButton default False;
    property OnPopup: TNotifyEvent read FOnPopup write FOnPopup;
    property OnMenuCommand: TNotifyEvent read FOnMenuCommand write FOnMenuCommand;
    property OnMenuClose: TNotifyEvent read FOnMenuClose write FOnMenuClose;
  end;

  { TabcPopupMenu }

  TabcSysMenuItem = (amiRestore, amiMove, amiSize, amiMinimize, amiMaximize, amiClose, amiMDINext, amiSeparator);
  TabcSysMenuItems = set of TabcSysMenuItem;

  TabcPopupMenu = class(TPopupMenu)
  private
    Form: TCustomForm;
    MenuHandle: THandle;
    FSBPanelNo: Integer;
    FStatusBar: TStatusBar;
    FOptions: TabcMenuOptions;
    FFont: TFont;
    FParentFont: Boolean;
    FParentShowHint: Boolean;
    FShowHint: Boolean;
    FSystemFont: Boolean;
    FSystemMenuItems: TList;
    FAppendSystemItems: Boolean;
    FVisibleItems: TabcSysMenuItems;
    FRuntimeCreated: Boolean;
    FInternalFontChange: Boolean;
    FOnMenuCommand: TNotifyEvent;
    FOnMenuClose: TNotifyEvent;
    FOldDisabledStyle: Boolean;
    FTrayIconMenu: Boolean;
    function Button: TabcToolbarButton;
    function IsFontStored: Boolean;
    function IsShowHintStored: Boolean;
    procedure SetFont(Value: TFont);
    procedure SetParentFont(Value: Boolean);
    procedure SetShowHint(Value: Boolean);
    procedure SetParentShowHint(Value: Boolean);
    procedure SetSystemFont(Value: Boolean);
    procedure SetOptions(Value: TabcMenuOptions);
    procedure MDISystemClick(Sender: TObject);
    procedure FontChanged(Sender: TObject);
{$IFDEF DELPHI5}
    procedure RebuildHandle;
{$ENDIF}    
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure WriteState(Writer: TWriter); override;
    procedure MergeSystemMenu(hSysMenu: HMENU; WindowState: TWindowState;
      OnClick: TNotifyEvent); virtual;
    function CalcOpenAnimation: TabcMenuAnimation; virtual;
    function CalcCloseAnimation: TabcMenuAnimation; virtual;
    property AppendSystemItems: Boolean read FAppendSystemItems write FAppendSystemItems default False;
    property VisibleItems: TabcSysMenuItems read FVisibleItems write FVisibleItems
      default [amiRestore, amiMove, amiSize, amiMinimize, amiMaximize, amiClose,
               amiMDINext, amiSeparator];
  public
    MenuItems: TMenuItem;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Popup(X, Y: Integer); override;
    function FormOnScreen: Boolean;
  published
    property StatusBar: TStatusBar read FStatusBar write FStatusBar;
    property StatusBarIndex: Integer read FSBPanelNo write FSBPanelNo;
    property OldDisabledStyle: Boolean read FOldDisabledStyle write FOldDisabledStyle default False;
    property Options: TabcMenuOptions read FOptions write SetOptions stored True;
    property ShowHint: Boolean read FShowHint write SetShowHint stored IsShowHintStored;
    property ParentShowHint: Boolean read FParentShowHint write SetParentShowHint default True;
    property Font: TFont read FFont write SetFont stored IsFontStored;
    property ParentFont: Boolean read FParentFont write SetParentFont default True;
    property SystemFont: Boolean read FSystemFont write SetSystemFont default False;
    property TrayIconMenu: Boolean read FTrayIconMenu write FTrayIconMenu default False;
    property OnMenuCommand: TNotifyEvent read FOnMenuCommand write FOnMenuCommand;
    property OnMenuClose: TNotifyEvent read FOnMenuClose write FOnMenuClose;
  end;

  { TabcSystemMenu }

  TabcSystemMenu = class(TabcPopupMenu)
  private
    FEnabled: Boolean;
    FObjectInstance: TFarProc;
    FOwnerForm: TForm;
    FOwnerHandle: HWND;
    FOwnerWindowProc: TFarProc;
    FPopupMenu: TabcPopupMenu;
    FPopupAt: TPoint;
    FOldMenu: TMainMenu;
    FDoAdjust: Boolean;
    FOnShow: TNotifyEvent;
    FIconicTimer: TTimer;
    FSaveIconicPos: TSmallPoint;
    FSaveIconicAdjust: Boolean;
    procedure MessageProc(var Message: TMessage);
    procedure SubClassOwnerForm;
    procedure SystemClick(Sender: TObject);
    procedure OnIconicTimer(Sender: TObject);
  protected
    procedure Loaded; override;
    function CalcOpenAnimation: TabcMenuAnimation; override;
    function DoProcessMessage(var Message: TMessage): Boolean; virtual;
    procedure DelayPopupSysMenu(Sender: TObject); virtual;
    procedure PopupIconicSysMenu(X, Y: Integer; DoAdjust: Boolean); virtual;
    property OwnerForm: TForm read FOwnerForm;
    property OwnerHandle: HWND read FOwnerHandle;
    property FormMenu: TMainMenu read FOldMenu;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure PopupSysMenu(X, Y: Integer; DoAdjust: Boolean); virtual;
  published
    property Enabled: Boolean read FEnabled write FEnabled default True;
    property VisibleItems;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

  { TabcMenuImages }

  TabcMenuImages = class;
  TabcMenuImageItems = class;

  TabcMenuImageItem = class(TCollectionItem)
  private
    FBitmap, FBitmapChecked: TBitmap;
    FMenuItem: TMenuItem;
    FMenuImages: TabcMenuImages;
    FMenuImageItems: TabcMenuImageItems;
    FImageIndex: TImageIndex;
    FImageIndexChecked: TImageIndex;
    procedure SetMenuItem(Value: TMenuItem);
    procedure SetImageIndex(Value: TImageIndex);
    procedure SetImageIndexChecked(Value: TImageIndex);
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property MenuImages: TabcMenuImages read FMenuImages;
    property MenuImageItems: TabcMenuImageItems read FMenuImageItems;
    procedure Update;
  published
    property MenuItem: TMenuItem read FMenuItem write SetMenuItem;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default - 1;
    property ImageIndexChecked: TImageIndex read FImageIndexChecked write SetImageIndexChecked default - 1;
  end;

  TabcMenuImageItems = class(TCollection)
  private
    FMenuImages: TabcMenuImages;
    function GetItem(Index: Integer): TabcMenuImageItem;
    procedure SetItem(Index: Integer; Value: TabcMenuImageItem);
  protected
    function GetOwner: TPersistent; override;
    procedure MenuItemRemoved(Value: TMenuItem);
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TabcMenuImages);
    function Add: TabcMenuImageItem;
    function IndexOfMenuItem(AMenuItem: TMenuItem): Integer;
    property Items[Index: Integer]: TabcMenuImageItem read GetItem write SetItem; default;
    property MenuImages: TabcMenuImages read FMenuImages;
  end;

  TabcMenuImages = class(TComponent)
  private
    FEnabled: Boolean;
    FImageList: TImageList;
    FItems: TabcMenuImageItems;
    procedure SetEnabled(Value: Boolean);
    procedure SetImageList(Value: TImageList);
    procedure SetItems(Value: TabcMenuImageItems);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure ItemsChanged(Item: TabcMenuImageItem);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Update;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property ImageList: TImageList read FImageList write SetImageList;
    property Items: TabcMenuImageItems read FItems write SetItems;
  end;


  { TabcMainMenu }

  TabcMainMenu = class(TMainMenu)
     FToolbar: TabcMenuToolbar;
  protected
    procedure MenuChanged(Sender: TObject; Source: TMenuItem; Rebuild: Boolean); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    procedure Merge(Menu: TMainMenu);
    procedure Unmerge(Menu: TMainMenu);
  end;

var
  // Keyboard shortcuts
  abcKeyboardShortCuts: TStringList = nil;

  { The delay time to popup a submenu }
  abcSubMenuPopupDelay: Cardinal = 250;

procedure abcKillActivePopupMenu(MultimediaAction: Boolean);

implementation

uses
  abcconst, abcutilg;

{$R abcmnu32.res}

type
  TMenuItemAccess = class(TMenuItem);  //access protected methods of TMenuItem

  TabcMenuHintWindow = class(THintWindow)
  private
    FTimer: TTimer;
    FHints: TStringList;
    procedure OnTimer(Sender: TObject);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ActivateHint(Rect: TRect; const AHint: string); override;
    procedure ShowHint(const AHint: string);
  end;

  TabcPopupMenuForm = class(TCustomForm)
  private
    Buffer, TileBackBuffer: TBitmap;
    FSelectedIndex, LastSelectedIndex, MenuItemCount, VisibleMenuItemCount, MainWidth, DX, DY: Integer;
    OpenAnimation, CloseAnimation: TabcMenuAnimation;
    SelectedChanged, PaintMenu, FromBottomToTop, FromRightToLeft, DrawDisabled,
    IgnoreFirstMouseMove: Boolean;
    PopupMenu: TabcPopupMenu;
    CurSubMenu, ParentPopupMenuForm: TabcPopupMenuForm;
    Timer: TTimer;
    MenuHandle: THandle;
    Options: TabcMenuOptions;
    procedure CheckShowHint(ShowFloatingHint: Boolean);
    procedure SetSelectedIndex(Value: Integer);
    procedure PopupMenuFormTimer(Sender: TObject);
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    procedure WMSetFocus(var Msg: TWMSetFocus); message WM_SETFOCUS;
    procedure WMActivate(var Msg: TWMActivate); message WM_ACTIVATE;
    procedure WMMouseActivate(var Msg: TWMMouseActivate); message WM_MOUSEACTIVATE;
    procedure CMShowingChanged(var Msg: TMessage); message CM_SHOWINGCHANGED;
  protected
    procedure Paint; override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure SetZOrder(TopMost: Boolean); override;
    procedure WriteState(Writer: TWriter); override;
    function GetRealHeight: Integer; virtual;
    function GetRealWidth: Integer; virtual;
    function CurItemTop: Integer; virtual;
  public
    CurMenuItem, MenuItems: TMenuItem;
    Animated: Boolean;
    KillAnimation: Boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetIndexAt(X, Y: Integer): Integer;
    function GetNearestHelpContext: Integer;
    function Animate: Boolean; virtual;
    function AnimatedHide: Boolean; virtual;
    procedure SilentShow;
    procedure SilentHide;
    procedure PopupCurSubMenu;
    procedure BringMenuToFront;
    function GetCurMenuItem: TMenuItem;
    function GetMenuItemIndex(Index: Integer): TMenuItem;
    property SelectedIndex: Integer read FSelectedIndex write SetSelectedIndex;
  end;

  { TabcMenuToolbarButton }

  TabcMenuToolbarButton = class(TabcToolbarButton)
  private
    FPP: TPoint;
    FOrientation: TabcMenuButtonOrientation;
    FHBuffer, FVBuffer: TBitmap;
    FHBState, FVBState: Boolean;
    FSystemButton: Boolean;
    FInClick: Boolean;
    FFreeYourself: Boolean;
    FMenuItem: TMenuItem;
    FReleaseGlyphHandle: Boolean;
    procedure SetOrientation(Value: TabcMenuButtonOrientation);
    procedure DelaySystemMenu(Sender: TObject);
    procedure WMLButtonDblClk(var Msg: TWMMouse); message WM_LBUTTONDBLCLK;
    procedure CMDialogChar(var Msg: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged (var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMSysColorChange(var Msg: TMessage); message CM_SYSCOLORCHANGE;
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
    property MenuItem: TMenuItem read FMenuItem write FMenuItem;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure Click; override;
    function IsVertical: Boolean;
    property Orientation: TabcMenuButtonOrientation read FOrientation write SetOrientation default aorAutomatic;
    property AutoSize default True;
  end;

const
  // directions for GetNextMenuToolbarButton
  DRLEFT  = - 1;
  DRRIGHT = 1;

  VK_0 = Byte('0');
  VK_Z = Byte('Z');

  FORMFLAGS = SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE;

  RANDOM_ANIMATIONS: array[0..2] of TabcMenuAnimation =
    (amnVSlide, amnHSlide, amnPopup);

  ALT_MASK = $20000000;

var
  ActivePopupMenu: TabcPopupMenu = nil;
  ActiveMenuList: TList = nil;
  SystemMenuList: TList = nil;
  ActiveHintWindow: TabcMenuHintWindow = nil;
  Pattern: TBitmap = nil;
  IgnoreNextMenuUp: Boolean = False; // ignore alt key up after alt key down
  CheckMarkBitmap: HBitmap = 0;
  RadioItemBitmap: HBitmap = 0;
  HGetMessageHook: HHook = 0;
  HGetCBTHook: HHook = 0;
  HGetCallWndProcHook: HHook = 0;
  LastabcButtonMouseInControl: TabcToolbarButton = nil;
  CurCaretIndex: Integer = 0;

  pCharBuf: array [0..256] of Char;
  mii: TMenuItemInfo;

  {for Windows NT 3.51}
  MenuItemText: array[0..255] of Char;
  WinVer4OrHigher: Boolean;

  RebuildMenuItem: TMenuItem;

{ Routines }

{for Windows NT 3.51 - emulates GetMenuItemInfo with calls to older menu API functions}

function abcGetMenuItemInfo(p1: HMENU; p2: UINT; p3: BOOL; var p4: TMenuItemInfo): BOOL;
const
  CMD_POS: array[Boolean] of Integer = (MF_BYCOMMAND, MF_BYPOSITION);
var
  nPos, maxLen: Integer;
  menuState: UINT;
  gotMenuString: Boolean;

  function GetMenuItemPosFromID(p1: HMENU; p3: BOOL): Integer;
  var
    i: integer;
  begin
    for i := 0 to GetMenuItemCount(p1) - 1 do
      if Bool(GetMenuItemID(p1, i)) = p3 then
      begin
        Result := i;
        Exit;
      end;
    Result := 0;
  end;

  procedure DoGetMenuString;
  begin
    maxLen := p4.cch;
    p4.cch := GetMenuString(p1,p2,MenuItemText,255,CMD_POS[p3]);
    if p4.cch = 0 then
      p4.dwTypeData[0] := #0
    else
      StrLCopy(p4.dwTypeData, MenuItemText, maxLen);
    gotMenuString := True;
  end;

begin
  if WinVer4OrHigher then
  begin
    Result := GetMenuItemInfo(p1, p2, p3, p4);
    Exit;
  end;
  {Win3 mapping to older api functions}

  menuState := GetMenuState(p1, p2, CMD_POS[p3]);
  if menuState = $FFFFFFFF then
    Result := False
  else
    Result := True;

  gotMenuString := False;

  if (p4.fMask and MIIM_STATE) <> 0 then
  begin
    p4.fState := 0;
    if (menuState and MF_HILITE) <> 0 then
      p4.fState := p4.fState or MFS_HILITE
    else
      p4.fState := p4.fState or MFS_UNHILITE;
    if (menuState and MF_GRAYED) <> 0 then
      p4.fState := p4.fState or MFS_GRAYED;
    if (menuState and MF_DISABLED) <> 0 then
      p4.fState := p4.fState or MFS_DISABLED
    else if (menuState and MF_GRAYED) = 0 then
      p4.fState := p4.fState or MFS_ENABLED;
    if (menuState and MF_CHECKED) <> 0 then
      p4.fState := p4.fState or MFS_CHECKED
    else
      p4.fState := p4.fState or MFS_UNCHECKED;
  end;

  if (p4.fMask and MIIM_ID) <> 0 then
  {application-defined wID value not supported}
  begin
    p4.wID := 0;
  end;

  if (p4.fMask and MIIM_SUBMENU) <> 0 then
  begin
    if p3 then
      nPos := Integer(p3)
    else
      nPos := GetMenuItemPosFromID(p1, p3);
    p4.hSubMenu := GetSubMenu(p1, nPos);
  end;

  if (p4.fMask and MIIM_CHECKMARKS) <> 0 then
  {not supported}
  begin
    p4.hbmpChecked := 0;
    p4.hbmpUnchecked := 0;
  end;

  if (p4.fMask and MIIM_TYPE) <> 0 then
  {MFT_BITMAP, MFT_OWNERDRAW, MFT_RADIOCHECK, MFT_RIGHTJUSTIFY never returned}
  begin
    p4.fType := 0;
    {returns item string data only}
    if not gotMenuString then
      DoGetMenuString;
    if p4.dwTypeData <> nil then
      p4.fType := p4.fType or MFT_STRING;
    if (menuState and MF_SEPARATOR) <> 0 then
      p4.fType := p4.fType or MFT_SEPARATOR;
    if (menuState and MF_MENUBREAK) <> 0 then
      p4.fType := p4.fType or MFT_MENUBREAK;
    if (menuState and MF_MENUBARBREAK) <> 0 then
      p4.fType := p4.fType or MFT_MENUBARBREAK;
  end;

  if (p4.fMask and MIIM_DATA) <> 0 then
  {returns item string data only}
  begin
    if not gotMenuString then DoGetMenuString;
  end;

  if (p4.fMask and MIIM_STRING) <> 0 then
  begin
    if not gotMenuString then DoGetMenuString;
  end;

  if (p4.fMask and MIIM_BITMAP) <> 0 then
  {no bitmaps returned}
  begin
    p4.hbmpChecked := 0;
    p4.hbmpUnchecked := 0;
  end;

  if (p4.fMask and MIIM_FTYPE) <> 0 then
  {MFT_BITMAP, MFT_OWNERDRAW, MFT_RADIOCHECK, MFT_RIGHTJUSTIFY never returned}
  begin
    p4.fType := 0;
    if GetMenuString(p1,p2,MenuItemText,0,CMD_POS[p3]) <> 0 then
      p4.fType := p4.fType or MFT_STRING;
    if (menuState and MF_SEPARATOR) <> 0 then
      p4.fType := p4.fType or MFT_SEPARATOR;
    if (menuState and MF_MENUBREAK) <> 0 then
      p4.fType := p4.fType or MFT_MENUBREAK;
    if (menuState and MF_MENUBARBREAK) <> 0 then
      p4.fType := p4.fType or MFT_MENUBARBREAK;
  end;
end;

function abcCalcMenuItemHeightOf(const AFont: TFont): Integer;
var
  DC: HDC;
  Old: HGDIOBJ;
  Size: TSize;
begin
  Size.cX := 0;
  Size.cY := 0;
  Old := 0;
  DC := CreateCompatibleDC(0);
  try
    Old := SelectObject(DC, AFont.Handle);
    Windows.GetTextExtentPoint(DC, 'W', 1, Size);
    Result := Size.cy + 6;
  finally
    SelectObject(DC, Old);
    DeleteDC(DC);
  end;
end;

function abcSystemMenuOf(AForm: TForm): TabcSystemMenu;
var
  i: integer;
begin
  with SystemMenuList do
    for i := 0 to Count - 1 do
    begin
      Result := TabcSystemMenu(Items[i]);
      if Result.Owner = AForm then
        Exit
{$IFDEF DELPHI5}
      else
        if (Result.Owner is TFrame) then
          if Result.Owner.Owner = AForm then
            Exit
{$ENDIF}
        ;
    end;
  Result := nil;
end;

function AssignedActivePopupMenuForm: Boolean;
begin
  Result := Assigned(ActivePopupMenu) and ActivePopupMenu.FormOnScreen;
end;

function abcToolbarofHWnd(const Wnd: HWnd): TabcMenuToolbar;
var
  I: Integer;
  ParentForm: TCustomForm;
begin
  Result := nil;
  if Application.Terminated then Exit;

  for I := 0 to ActiveMenuList.Count - 1 do
    with TabcMenuToolbar(ActiveMenuList[I]).OwnerForm do
      if (Handle = Wnd) then
      begin
        Result := TabcMenuToolbar(ActiveMenuList[I]);
        Break;
      end
      else
        if Parent <> nil then
        begin
          //check for docked or subforms
          ParentForm := GetParentForm(TabcMenuToolbar(ActiveMenuList[I]).OwnerForm);
          if (ParentForm <> nil) and (ParentForm.Handle = Wnd) then
          begin
            Result := TabcMenuToolbar(ActiveMenuList[I]);
            Break;
          end;
        end;
end;

function abcActiveMenu: TabcMenuToolbar;
// check for current active abcMainMenu
begin
  Result := nil;
  if Application.Terminated then Exit;
  Result := abcToolbarofHWnd(GetActiveWindow);
end;

function abcAssignedActiveMenu: Boolean;
begin
  Result := abcActiveMenu <> nil;
end;

procedure SetStatusBarText(const HintText: String);
var
  I: Integer;
  S: String;
begin
  S := Trim(GetLongHint(HintText));
  // remove '&#13;' symbols from status bar text
  repeat
    I := Pos('&#', S);
    if I = 0 then
      System.Break;
    System.Delete(S, I, 5);
    System.Insert(' ', S, I);
  until False;

  if Assigned(ActivePopupMenu) and
    Assigned(ActivePopupMenu.StatusBar) then
    with ActivePopupMenu, StatusBar do
    begin
      if SimplePanel then
        SimpleText := S
      else if StatusBarIndex < Panels.Count then
        Panels[StatusBarIndex].Text := S;
      Exit;
    end;
  if abcAssignedActiveMenu and Assigned(abcActiveMenu.StatusBar) then
    with abcActiveMenu, StatusBar do
      if SimplePanel then
        SimpleText := S
      else if StatusBarIndex < Panels.Count then
        Panels[StatusBarIndex].Text := S;
end;

procedure FullHideCaret;
begin
  HideCaret(0);
  Inc(CurCaretIndex);
end;

procedure FullShowCaret;
var
  I: Integer;
begin
  for I := CurCaretIndex downto 1 do
    ShowCaret(0);
  CurCaretIndex := 0;
end;

procedure abcProcessMessages(message: UINT);
var
  Msg: TMsg;
begin
  while PeekMessage(Msg, 0, message, message, pm_NoRemove) do
  begin
    case Integer(GetMessage(Msg, 0, message, message)) of
      -1: Exit;
      0:
      begin
        PostQuitMessage(Msg.WParam);
        Exit;
      end;
    else
    end;
    DispatchMessage(Msg);
  end;
end;

procedure SetActivePopupMenuUp(MultimediaAction: Boolean);
var
  F, F1: TabcPopupMenuForm;
begin
  if Assigned(ActivePopupMenu) and
    Assigned(ActivePopupMenu.PopupComponent) and
    (ActivePopupMenu.PopupComponent is TabcMenuToolbarButton) then
  begin
    // down to the last submenu
    F := ActivePopupMenu.Form as TabcPopupMenuForm;
    while Assigned(F) and Assigned(F.CurSubMenu) do
      F := F.CurSubMenu;
    // up to the first level menu
    while Assigned(F) do
    begin
      if MultimediaAction then
        F.AnimatedHide
      else
        F.Height := 0;
      F1 := F;
      F := F.ParentPopupMenuForm;
      if Assigned(F) then
        F1.Release;
    end;

    if Assigned(ActivePopupMenu.OnMenuClose) then
      ActivePopupMenu.OnMenuClose(nil);

    if ActivePopupMenu.Button <> nil then
      with ActivePopupMenu.Button do
      begin
        MouseLeft;
        Down := False;
        Update;
      end;
  end;
end;

procedure abcKillActivePopupMenu(MultimediaAction: Boolean);
begin
  try
    try
      if AssignedActivePopupMenuForm then
        with ActivePopupMenu, Form as TabcPopupMenuForm do
        begin
          if (PopupMenu is TabcSystemMenu) and
            (TabcSystemMenu(PopupMenu).FIconicTimer <> nil) then
            TabcSystemMenu(PopupMenu).FIconicTimer.Enabled := False;
          SetStatusBarText('');
          Timer.Enabled := False;
          SetActivePopupMenuUp(MultimediaAction);
          if not (Assigned(ActivePopupMenu)) then
            Exit;
          KillAnimation := True;
          SilentHide;
          LastabcButtonMouseInControl := Button;
        end;
    except
    end;
  finally
    ActivePopupMenu := nil;
  end;
end;

procedure SetFirstIndexSelected(Button: TabcToolbarButton);
begin
  if Assigned(Button.DropdownMenu) and
    (Button.DropdownMenu is TabcPopupMenu) then
    with TabcPopupMenuForm(TabcPopupMenu(Button.DropdownMenu).Form) do
    begin
      SelectedIndex := 0;
      IgnoreFirstMouseMove := True;
      DrawDisabled := True;
    end;
end;

function GetNextMenuToolbarButton(Control: TControl; Direction: Integer;
  DoClick: Boolean): TabcMenuToolbarButton;
var
  I, J: Integer;
  MTBar: TabcMenuToolbar;
begin
  Result := nil;
  if not Assigned(Control) or not abcAssignedActiveMenu then
    Exit;
  I := 0;
  MTBar := abcActiveMenu;
  for J := 0 to ActiveMenuList.Count - 1 do
    if Control.Parent = ActiveMenuList[J] then
    begin
      MTBar := TabcMenuToolbar(ActiveMenuList[J]);
      I := 0;
      while (I < MTBar.ControlCount) and
        (MTBar.Controls[I] <> Control)
      do
        Inc(I);
      Break;
    end;

  repeat
    Inc(I, Direction);
    if I <= - 1 then I := MTBar.ControlCount - 1;
    if I >= MTBar.ControlCount then I := 0;
  until (MTBar.Controls[I] is TabcMenuToolbarButton) and
    (Assigned(TabcMenuToolbarButton(MTBar.Controls[I]).DropdownMenu) or (not DoClick)) and
    MTBar.Controls[I].Enabled;

  Result := TabcMenuToolbarButton(MTBar.Controls[I]);

  if DoClick and (Result <> Control) and Assigned(Result) and
    Assigned(Result.DropdownMenu) and (Result.DropdownMenu is TabcPopupMenu) then
  begin
    SetFirstIndexSelected(Result);
    Result.Click;
  end;
end;

function SearchForActiveMenuKey(Key: Integer; SetSelected: Boolean): TabcMenuToolbarButton;
var
  I: Integer;
begin
  Result := nil;
  if not abcAssignedActiveMenu then Exit;
  if not abcActiveMenu.Visible then
    abcActiveMenu.Visible := True;
  with abcActiveMenu do
    for I := 0 to ControlCount - 1 do
      if (Controls[I] is TabcMenuToolbarButton) and
        TabcMenuToolbarButton(Controls[I]).Enabled and
        (TabcMenuToolbarButton(Controls[I]).Caption <> '-') and
        IsAccel(Key, TabcMenuToolbarButton(Controls[I]).Caption) then
      begin
        Result := Controls[I] as TabcMenuToolbarButton;
        SetFirstIndexSelected(Result);
        Exit;
      end;
end;

function GetMessageHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
type
  TClickResult = (crDisabled, crClicked, crShortCutMoved);
var
  AMFAH: Boolean;  // menu form is enabled
  AMFCWA: Boolean; // active menu's form's ActiveMdiChild is assigned
  MTBar: TabcMenuToolbar;
  ActiveWindow, ActiveMdi: HWnd;
  P: TPoint;
  I, Msg, Key, HelpCmd: Integer;
  C: TControl;
  F: TabcPopupMenuForm;
  B: TabcToolbarButton;
  ShortCutItem: TMenuItem;
  ShortCut: TShortCut;
  M: TMsg;

  procedure ClearMessage;
  begin
    ZeroMemory(PMsg(lParam), SizeOf(TMsg));
  end;

  function DoClick(Item: TMenuItem): TClickResult;
  begin
    Result := crClicked;
    if Item.Parent <> nil then Result := DoClick(Item.Parent);
    if Result = crClicked then
      if Item.Enabled and Item.Visible then
        try
          Item.Click;
          if ShortCutItem.ShortCut <> ShortCut then
            Result := crShortCutMoved;
        except
        end
      else
        Result := crDisabled;
  end;

  function IsShortCut(MainMenu: TMainMenu): Boolean;
  var
    ClickResult: TClickResult;
  begin
    Result := False;
    if not Assigned(MainMenu) then Exit;

    repeat
      ClickResult := crDisabled;
      ShortCutItem := MainMenu.FindItem(ShortCut, fkShortCut);

      if ShortCutItem <> nil then begin
        abcKillActivePopupMenu(True);
        ClickResult := DoClick(ShortCutItem);
      end;

    until ClickResult <> crShortCutMoved;
    Result := ShortCutItem <> nil;
  end;

  function IsKeyboardShortcut: Boolean;
  var
    I: Integer;
    S: String;
    SC: TShortCut;
  begin
    SC := ShortCut;
    Result := False;
    if not Assigned(abcKeyboardShortCuts) then Exit;

    try
      ShortCutItem := nil;
      S := ShortCutToText(ShortCut);

      for I := 0 to abcKeyboardShortCuts.Count - 1 do
        if S = abcKeyboardShortCuts[I]
      then ShortCutItem := TMenuItem(abcKeyboardShortCuts.Objects[I]);

      if ShortCutItem <> nil then begin
        abcKillActivePopupMenu(True);
        ShortCut := ShortCutItem.ShortCut;
        DoClick(ShortCutItem);
      end;

    except
      ShortCutItem := nil;
    end;

    Result := ShortCutItem <> nil;
    ShortCut := SC;
  end;

  function IsMenuShortCut: Boolean;
  var
    I: Integer;
  begin
    ShortCut := Key;
    if GetKeyState(VK_SHIFT) < 0 then Inc(ShortCut, scShift);
    if GetKeyState(VK_CONTROL) < 0 then Inc(ShortCut, scCtrl);
    if PMsg(lParam)^.lParam and ALT_MASK <> 0 then Inc(ShortCut, scAlt);

    Result := Assigned(MTBar) and (IsShortCut(MTBar.MainMenu) or IsKeyboardShortcut);

    if (not Result) and Assigned(MTBar) and Assigned(MTBar.FMergedMenusList) then
      with MTBar.FMergedMenusList do
        for i := 0 To Count - 1 do
          if IsShortCut(TMainMenu(Items[i])) then
          begin
            Result := True;
            Exit;
          end;
  end;

  procedure SearchForOpenedMenuShortcut;
  var
    I: Integer;
    arrayCaption : array[0..255] of Char;
    miiInfo      : TMenuItemInfo;
    boolFound    : Boolean;
  begin
    // just a symbol key - seeking for menu item
    for I := 0 to F.MenuItems.Count - 1 do
      if F.MenuItems[I].Visible and IsAccel(Key, F.MenuItems[I].Caption) then
        with F.MenuItems[I] do
        begin
          if (not Enabled) then System.Break;
          if Count = 0 then
            try // ordinary item
              abcKillActivePopupMenu(True);
              //sound
              with F.Options do
                if (SoundList <> nil) and (SoundCommand <> -1) then
                  SoundList.PlayItem(SoundCommand);
              F.MenuItems[I].Click;
              Exit;
            except
            end
          else
          begin // SubMenu
            F.SelectedIndex := I;
            F.SelectedChanged := False;
            F.PopupCurSubMenu;
            F.CurSubMenu.SelectedIndex := 0;
          end;
          System.Break;
        end;
    if I >= F.MenuItems.Count then
    begin
      boolFound:=False;
      miiInfo.cbSize:=44;
      if (Application.MainForm.WindowMenu <> nil) then
        for i:= F.MenuItems.Count to GetMenuItemCount(HMENU(Application.MainForm.WindowMenu.Handle)) do
        begin
          miiInfo.fMask :=MIIM_STATE + MIIM_ID;
          if GetMenuItemInfo(HMenu(Application.MainForm.WindowMenu.Handle),i,True,miiInfo) then
          begin
            if (miiInfo.fState And mfs_Disabled = 0) Then
            begin
              GetMenuString(HMenu(Application.MainForm.WindowMenu.Handle), i, arrayCaption, SizeOf(arrayCaption) -1,MF_BYPOSITION);
              boolFound:=IsAccel(Key, arrayCaption);
              if boolFound then
              begin
                abcKillActivePopupMenu(True);
                SendMessage(TForm(Application.MainForm).Handle, wm_Command , miiInfo.wId, 0);
                Exit;
              end;
            end;
          end;
        end;
      if not boolFound then
         MessageBeep(0);
    end;
  end;

  function IsOkControl(C: TControl): Boolean;
  begin
    Result := False;
    if Assigned(C) then
      Result := IsOkControl(C.Parent);
    Result := Result or (C is TabcPopupMenuForm) or
      ((C = ActivePopupMenu.PopupComponent) and (C is TabcToolbarButton));
  end;


begin
  if Code < 0 then
    Result := CallNextHookEx(HGetMessageHook, Code, wParam, lParam)
  else
  begin
    Result := 0;
    ActiveWindow := GetActiveWindow;
    if (Application <> nil) and
       not Application.Terminated and
       Application.Active and
       (ActiveWindow <> 0) and
       not IsIconic(ActiveWindow) then
    begin
      MTBar := abcActiveMenu;
      { If the Owner is not enabled for mouse and keyboard input... exit }
      if Assigned(MTBar) and (not IsWindowEnabled(MTBar.OwnerForm.Handle)) then
        Exit;
      Msg := PMsg(lParam)^.Message;
      Key := PMsg(lParam)^.wParam;

      // check for mouse messages
      if ((Msg >= WM_LBUTTONDBLCLK) and (Msg <= WM_MBUTTONDBLCLK)) or
        ((Msg >= WM_NCRBUTTONDBLCLK) and (Msg <= WM_NCMBUTTONDBLCLK)) or
        (Msg = WM_LBUTTONDOWN) or (Msg = WM_NCLBUTTONDOWN) or
        (Msg = WM_NCRBUTTONDOWN) then
        if AssignedActivePopupMenuForm then
        begin
          GetCursorPos(P);
          C := FindDragTarget(P, True);
          if IsOkControl(C) then Exit;

          abcKillActivePopupMenu(True);
          IgnoreNextMenuUp := False;
          FullShowCaret;
        end
        else if Assigned(MTBar) and Assigned(abcButtonMouseInControl) then
        begin
          abcButtonMouseInControl.MouseLeft;
          MTBar.KeepSelected := False;
          FullShowCaret;
        end;

      // check for system menu calls
      if (Msg = WM_SYSKEYDOWN) and (Key = VK_SPACE) then
      begin
        PostMessage(ActiveWindow, WM_NCLBUTTONDOWN, htSysMenu, 0);
        ClearMessage;
        Exit;
      end;

      // another key?
      if ((Msg = WM_KEYDOWN) or (Msg = WM_KEYUP) or (Msg = WM_SYSKEYDOWN) or
        (Msg = WM_SYSKEYUP)) then
      begin
        // initialization
        AMFAH := False;
        AMFCWA := False;
        ActiveMdi := 0;
        if Assigned(MTBar) then
          with MTBar.Owner as TForm do
          begin
            AMFAH := IsWindowEnabled(Handle) and Enabled;
            if Assigned(ActiveMdiChild) then ActiveMdi := ActiveMdiChild.Handle;
            AMFCWA := (ActiveMdi <> 0) and (Handle = ActiveWindow);
          end;
        // system menu for child window
        if (Msg = WM_SYSKEYDOWN) and ((Key = $BD) or (Key = $6D)) and
          AMFCWA then
        begin
          PostMessage(ActiveMdi, WM_NCLBUTTONDOWN, htSysMenu, 0);
          ClearMessage;
          Exit;
        end;
        // shortcuts
        if ((Msg = WM_KEYDOWN) or (Msg = WM_SYSKEYDOWN)) and Assigned(MTBar) and
          AMFAH and (not AssignedActivePopupMenuForm) and IsMenuShortCut then
        begin
          ClearMessage;
          Exit;
        end;
        // getting the topmost MenuForm
        F := nil;
        if Assigned(ActivePopupMenu) then
        begin
          F := ActivePopupMenu.Form as TabcPopupMenuForm;
          while F.CurSubMenu <> nil do F := F.CurSubMenu;
        end;
        // check for vk_Menu and vk_F10 - key down
        if (((Msg = WM_SYSKEYDOWN) and ((Key = VK_MENU) or (Key = VK_F10))) or
          ((Msg = WM_KEYDOWN) and (Key = VK_F10))) and AMFAH and
          (Assigned(abcButtonMouseInControl) or AssignedActivePopupMenuForm) then
        begin
          abcKillActivePopupMenu(True);
          if Assigned(LastabcButtonMouseInControl) then
          begin
            LastabcButtonMouseInControl.MouseLeft;
            LastabcButtonMouseInControl.Down := False;
          end;
          if Assigned(MTBar) then
            TForm(MTBar.Owner).SetFocus;
          FullShowCaret;
          if Assigned(MTBar) then
            MTBar.KeepSelected := False;
          IgnoreNextMenuUp := True;
          ClearMessage;
          Exit;
        end { WM_SYSKEYDOWN };
        // check for vk_Menu and vk_F10 - key up
        if(((Msg = WM_SYSKEYUP) and ((Key = VK_MENU) or (Key = VK_F10))) or
          ((Msg = WM_KEYUP) and (Key = VK_F10))) and AMFAH and Assigned(MTBar) then
        begin
          if not Assigned(LastabcButtonMouseInControl) then
          begin
            I := 0;
            while (I < MTBar.ControlCount) and
              ((not (MTBar.Controls[I] is TabcMenuToolbarButton)) or
              TabcMenuToolbarButton(MTBar.Controls[I]).FSystemButton)
            do
              Inc(I);
            if (I < 0) or (I >= MTBar.ControlCount) then Exit;
            LastabcButtonMouseInControl := TabcMenuToolbarButton(MTBar.Controls[I])
          end;

          if (TabcMenuToolbarButton(LastabcButtonMouseInControl).FState <> atsMouseIn) and
            (not IgnoreNextMenuUp) then
          begin
            abcButtonMouseInControl := LastabcButtonMouseInControl;
            abcButtonMouseInControl.MouseEntered;
            MTBar.KeepSelected := True;
            FullHideCaret;
          end;

          IgnoreNextMenuUp := False;
          ClearMessage;
          Exit;
        end { WM_SYSKEYUP };
        // check for shortcuts on main menu
        if  (Msg = WM_SYSKEYDOWN) and (Key >= VK_0) and (Key <= VK_Z) and AMFAH then
          if AssignedActivePopupMenuForm
          then
            SearchForOpenedMenuShortcut
          else
          begin
            LastabcButtonMouseInControl := SearchForActiveMenuKey(Key, True);

            if Assigned(LastabcButtonMouseInControl) then
            begin
              if Assigned(abcButtonMouseInControl) then
                abcButtonMouseInControl.MouseLeft;
              abcButtonMouseInControl := nil;
              LastabcButtonMouseInControl.Click;
              LastabcButtonMouseInControl.MouseLeft;
              FullShowCaret;
              MTBar.KeepSelected := False;
              IgnoreNextMenuUp := True;
              ClearMessage;
              Exit;
            end;
          end;
        // check for other keys
        if (Msg = WM_KEYDOWN) or (Msg = WM_SYSKEYDOWN) then
        begin
          IgnoreNextMenuUp := False;

          // -------------------
          // main menu is active
          // abcActiveMenu.KeepSelected (i.e. 'selected by kbd')
          if Assigned(MTBar) and AMFAH and MTBar.KeepSelected and
            Assigned(abcButtonMouseInControl) then
            case Key of
              VK_ESCAPE:
              begin
                FullShowCaret;
                MTBar.KeepSelected := False;
                LastabcButtonMouseInControl := abcButtonMouseInControl;
                abcButtonMouseInControl.MouseLeft;
              end { VK_ESCAPE } ;

              VK_LEFT:
              begin
                LastabcButtonMouseInControl := abcButtonMouseInControl;
                abcButtonMouseInControl.MouseLeft;
                abcButtonMouseInControl := GetNextMenuToolbarButton(LastabcButtonMouseInControl, drLeft, False);
                abcButtonMouseInControl.MouseEntered;
                ClearMessage;
              end { VK_LEFT } ;

              VK_RIGHT:
              begin
                LastabcButtonMouseInControl := abcButtonMouseInControl;
                abcButtonMouseInControl.MouseLeft;
                abcButtonMouseInControl := GetNextMenuToolbarButton(LastabcButtonMouseInControl, drRight, False);
                abcButtonMouseInControl.MouseEntered;
                ClearMessage;
              end { VK_RIGHT } ;

              VK_RETURN, VK_DOWN, VK_UP, VK_SPACE:
              begin
                MTBar.KeepSelected := False;
                SetFirstIndexSelected(abcButtonMouseInControl);
                abcButtonMouseInControl.Click;
                ClearMessage;
              end { VK_RETURN } ;

            else
            begin
              // alphabetical keys - search for key and beep if not found
              B := SearchForActiveMenuKey(Key, True);

              if Assigned(B) then
              begin
                abcButtonMouseInControl.MouseLeft;
                B.Click;
              end
              else
                { Just beep if the key is not "Shift" or "Control"...  }
                { Or else, since a key is pressed it is always beeping }
                if (Key <> vk_Shift) and (Key <> vk_Control) then
                  MessageBeep(0);
            end;
          ClearMessage;
          Exit;
        end;

        // -------------------
        // menu form is opened
        if Assigned(F) then
        begin
          case Key of

            VK_RETURN:
              F.MouseUp(mbLeft, [], 1, 1);

            VK_ESCAPE:
              if not Assigned(F.ParentPopupMenuForm) then
              begin
                abcKillActivePopupMenu(True);
                abcButtonMouseInControl := LastabcButtonMouseInControl;
                if Assigned(LastabcButtonMouseInControl) then
                  LastabcButtonMouseInControl.MouseEntered;

                if abcAssignedActiveMenu then
                  abcActiveMenu.KeepSelected := True;
                FullHideCaret;
              end
              else
              begin
                F.ParentPopupMenuForm.CurSubMenu := nil;
                F.AnimatedHide;
              end { VK_ESCAPE } ;

            VK_LEFT:
              if not Assigned(F.ParentPopupMenuForm) then
                if ActivePopupMenu.Button <> nil then
                  GetNextMenuToolbarButton(ActivePopupMenu.Button, drLeft, True)
                else
                  F.SelectedIndex := 0
              else
              begin
                F.ParentPopupMenuForm.CurSubMenu := nil;
                F.AnimatedHide;
              end { VK_LEFT } ;

            VK_UP:
            begin
              F.SelectedIndex := F.SelectedIndex -1;
              F.Timer.Enabled := False;
            end { VK_UP } ;

            VK_RIGHT:
              if (F.GetCurMenuItem <> nil) and (F.CurMenuItem.Count > 0) then
              begin
                F.PopupCurSubMenu;
                F.CurSubMenu.SelectedIndex := 0;
              end
              else if ActivePopupMenu.Button <> nil then
                GetNextMenuToolbarButton(ActivePopupMenu.Button, drRight, True)
              else
                F.SelectedIndex := -1;

            VK_DOWN:
            begin
              F.SelectedIndex := F.SelectedIndex +1;
              F.Timer.Enabled := False;
            end { VK_DOWN } ;

            VK_F1:
            begin
              I := F.GetNearestHelpContext;
              abcKillActivePopupMenu(True);
              if Assigned(MTBar) and
                 ((biHelp in TForm(MTBar.Owner).BorderIcons) or
                 (GetKeyState(VK_SHIFT)<0) ) then
                HelpCmd := HELP_CONTEXTPOPUP
              else
                HelpCmd := HELP_CONTEXT;
              Application.HelpCommand(HelpCmd, I);
              PeekMessage(M, 0, 0, 0, pm_Remove);
            end;
          else // alphabetical keys - search for shortcut
            if (Key <> VK_SHIFT) and (Key <> VK_CONTROL) then
              SearchForOpenedMenuShortcut;
          end { case } ;

          ClearMessage;
        end { Assigned(F) } ;
      end { another key } ;
      end { keyboard message }
    end { main form is active } ;
    Result := CallNextHookEx(HGetMessageHook, Code, wParam, lParam);
  end;
end;

function GetCBTHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  MTBar: TabcMenuToolbar;
begin
  if (Code < 0) then
    Result := CallNextHookEx(HGetCBTHook, Code, wParam, lParam)
  else
  begin
    if not Application.Terminated then
      if (Code in [HCBT_MINMAX, HCBT_SETFOCUS, HCBT_DESTROYWND]) then
      begin
        MTBar := abcActiveMenu;
        if Assigned(MTBar) and MTBar.HandleAllocated then
          PostMessage(MTBar.Handle, AM_CHECKFORMDICHILDMENU, 0, 0);
      end
      else if (Code = HCBT_MOVESIZE) then
        abcKillActivePopupMenu(False);
    Result := CallNextHookEx(HGetCBTHook, Code, wParam, lParam);
  end;
end;

function GetCallWndProcHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  MTBar: TabcMenuToolbar;
  IsActive: Boolean;
begin
  if (Code < 0) then
    Result := CallNextHookEx(HGetCallWndProcHook, Code, wParam, lParam)
  else
  begin
    if (Code = HC_ACTION) then
      with PCWPStruct(lParam)^ do
        if (Message = WM_ENABLE) or (Message = WM_ACTIVATE) or (Message = WM_ACTIVATEAPP) then
        begin
          if (Message = WM_ENABLE) then
            MTBar := abcToolbarofHWnd(hwnd)
          else
            MTBar := abcActiveMenu;
          if Assigned(MTBar) and MTBar.HandleAllocated then
          begin
            if (Message = WM_ACTIVATE) then
              IsActive := IsWindowEnabled(TForm(MTBar.Owner).Handle)
            else
              IsActive := BOOL(WParam);
            MTBar.GrayActivated := not IsActive;
          end;
        end;

    Result := CallNextHookEx(HGetCallWndProcHook, Code, wParam, lParam);
  end;
end;

// warning! pixel [0, Height-1] is always transparent
procedure TransBlt(Canvas: TCanvas; X, Y, Width, Height: Integer; hBmp: HBitmap);
var
  BitmapDC: HDC;
  X1, Y1, Cur, Trans: Integer;
begin
  BitmapDC := CreateCompatibleDC(Canvas.Handle);
  SelectObject(BitmapDC, hBmp);
  Trans := GetPixel(BitmapDC, 0, Height-1);

  for X1 := 0 to Width-1 do
    for Y1 := 0 to Height-1 do
    begin
      Cur := GetPixel(BitmapDC, X1, Y1);
      if (Cur <> Trans) then
        SetPixel(Canvas.Handle, X + X1, Y + Y1, Cur);
    end;
  DeleteDC(BitmapDC);
end;

procedure DisabledBlt(Canvas: TCanvas; X, Y, Width, Height: Integer; hBmp: HBitmap; OldStyle: Boolean);
const
  DISABLED_BLT_STYLES: array[Boolean] of Byte = (132, 128);
var
  XOffset: Integer;
  BitmapDC: HDC;
  Trans: TColor;

  procedure PaintColor(Color: TColor; Offset: Integer);
  var
    X1, Y1, Cur: Integer;
  begin
    for X1 := XOffset to XOffset + Width-1 do
      for Y1 := 0 to Height-1 do
      begin
        Cur := GetPixel(BitmapDC, X1, Y1);
        if (Cur <> Trans) and (Cur and $000000FF <= 132) and
          (Cur and $0000FF00 shr 08 <= DISABLED_BLT_STYLES[OldStyle]) and
          (Cur and $00FF0000 shr 16 <= 132) then
          SetPixel(Canvas.Handle, X + X1 + Offset, Y + Y1 + Offset, Color);
      end;
  end;

begin
  XOffset := 0;
  BitmapDC := CreateCompatibleDC(Canvas.Handle);
  SelectObject(BitmapDC, hBmp);
  Trans := GetPixel(BitmapDC, 0, Height-1);
  PaintColor(ColorToRgb(clBtnHighlight), 1);
  PaintColor(ColorToRgb(clBtnShadow), 0);
  DeleteDC(BitmapDC);
end;

function abcCreateSystemMenu(
  AForm, AMC: TComponent;
  ExistingMenu: TabcPopupMenu
  ): TabcPopupMenu;

var
  abcSystemMenu: TabcSystemMenu;
  SystemMenuClick: TNotifyEvent;
  hSysMenu: THandle;

begin
  Result := ExistingMenu;
  if Result = nil then
  begin
    Result := TabcPopupMenu.Create(AForm);
    Result.FRuntimeCreated := True;
  end;
  if (AForm = nil) or (AMC = nil) then
    Exit;
  hSysMenu := GetSystemMenu(TWinControl(AMC).Handle, False);
  if (hSysMenu <> 0) then
    try
      if AMC is TForm then
        abcSystemMenu := abcSystemMenuOf(TForm(AMC))
      else
        abcSystemMenu := nil;

      if abcSystemMenu <> nil then
        SystemMenuClick := abcSystemMenu.SystemClick
      else
        SystemMenuClick := Result.MDISystemClick;

      Result.MergeSystemMenu(hSysMenu, TForm(AMC).WindowState, SystemMenuClick);
    finally
      DeleteObject(hSysMenu);
    end;
end;

procedure UpdateMenuItemBitmaps(Form: TForm);
var
  i: Integer;
begin
  if (Form <> nil) then
    with Form do
      for i := 0 to ComponentCount - 1 do
        if Components[i] is TabcMenuImages then
          TabcMenuImages(Components[I]).Update;
end;

procedure CreatePattern;
var
  X, Y: Integer;
begin
  Pattern := TBitmap.Create;
  Pattern.Width := 8;
  Pattern.Height := 8;
  for Y := 0 to 7 do
    for X := 0 to 7 do
      if (Y mod 2) = (X mod 2) then
        Pattern.Canvas.Pixels[X, Y] := clBtnHighlight
      else
        Pattern.Canvas.Pixels[X, Y] := clBtnFace;
end;

procedure PaintMenuIcon(Owner, AMC: TForm; DC: HDC);
var
  tmpIcon: TIcon;
  SizeX, SizeY: Integer;
begin
  SizeX := GetSystemMetrics(SM_CXSMICON);
  SizeY := GetSystemMetrics(SM_CYSMICON);

  tmpIcon := TIcon.Create;
  try
    if Assigned(AMC) and (AMC.Icon <> nil) and not (AMC.Icon.Empty) then
      tmpIcon.Assign(AMC.Icon)
    else if Assigned(Owner) and (Owner.Icon <> nil) and not (Owner.Icon.Empty) then
      tmpIcon.Assign(Owner.Icon)
    else if (Application.Icon <> nil) and not Application.Icon.Empty then
      tmpIcon.Assign(Application.Icon)
    else
      tmpIcon.Handle := LoadIcon(0, IDI_APPLICATION);
    DrawIconEx(DC, 0, 0, tmpIcon.Handle, SizeX, SizeY, 0, 0, DI_NORMAL);
  finally
    tmpIcon.Free;
  end;
end;

procedure abcSetMenuFont(AFont: TFont);
var
  LogFont: TLogFont;
  NewStyle: TFontStyles;
  SysMenuFontHandle: HFONT;
  NonClientMetrics: TNonClientMetrics;
begin
  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    SysMenuFontHandle := CreateFontIndirect(NonClientMetrics.lfMenuFont)
  else
    SysMenuFontHandle := GetStockObject(SYSTEM_FONT);

  if GetObject(SysMenuFontHandle, SizeOf(LogFont), @LogFont) <> 0 then
  with AFont, LogFont do
  begin
    Height := lfHeight;
    NewStyle := [];
    if lfWeight >= FW_BOLD then
      NewStyle := NewStyle + [fsBold];
    if lfItalic = 1 then
      NewStyle := NewStyle + [fsItalic];
    if lfUnderline = 1 then
      NewStyle := NewStyle + [fsUnderline];
    if lfStrikeOut = 1 then
      NewStyle := NewStyle + [fsStrikeOut];
    Style := NewStyle;
    Charset := TFontCharset(lfCharSet);
    Name := lfFaceName;
    case lfPitchAndFamily and $F of
      VARIABLE_PITCH: Pitch := fpVariable;
      FIXED_PITCH: Pitch := fpFixed;
    else
      Pitch := fpDefault;
    end;
    Handle := SysMenuFontHandle;
  end;
end;

{ TabcMainMenu }

procedure TabcMainMenu.MenuChanged(Sender: TObject; Source: TMenuItem; Rebuild: Boolean);
begin
  if Application.Terminated then
  begin
    WindowHandle := 0;
    Exit;
  end;
  inherited;
  if (Sender = nil) and (Source = nil) and (FToolbar <> nil) and ([csDesigning, csLoading, csDestroying] * ComponentState = []) and not Application.Terminated then
    FToolbar.UpdateTopLevelMenuItems;
end;

procedure TabcMainMenu.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FToolbar) then
    FToolbar := nil;
end;

procedure TabcMainMenu.Merge(Menu: TMainMenu);
begin
  if FToolbar <> nil then
  with FToolbar do
  begin
    if Menu <> nil then
      FMergedMenusList.Add(Menu)
    else
      FMergedMenusList.Clear;
    Rebuild;
  end;
end;

procedure TabcMainMenu.Unmerge(Menu: TMainMenu);
begin
  if (Menu <> nil) and (FToolbar <> nil) then
    with FToolbar do
    begin
      FMergedMenusList.Remove(Menu);
      Rebuild;
    end;
end;

{ TabcMenuToolbar }

constructor TabcMenuToolbar.Create(AOwner: TComponent);
var
  I: Integer;
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csActionClient];
  if (AOwner is TForm) then
    FOwnerForm := TForm(AOwner)
  else if (AOwner.Owner is TForm) then
    FOwnerForm := TForm(AOwner.Owner)
  else
    raise EComponentError.Create(sabcErrorOwnerFormExpected);
  Caption := sabcMenuToolbarCaption;
  CloseButton := False;
  FullSize := True;
  FOptions := TabcMenuOptions.Create;
  FGrayInactive := True;
  FOptions.Timeout := 10;
  FMergedMenusList := TList.Create;
  FGrayActivated := False;
  FMMFont := TFont.Create;
  FMergedMenus := False;
  FOpaqueButtons := True;
  FButtonOrientation := aorAutomatic;
  FButtonAutoSize := True;

  if (csDesigning in Owner.ComponentState) then
    for I := 0 to Owner.ComponentCount - 1 do
    begin
      if Owner.Components[I] is TMainMenu then
      begin
        MainMenu := Owner.Components[I] as TMainMenu;
        Break;
      end
    end
  else
  begin
    if (FOwnerForm.FormStyle = fsMdiForm) then
    begin
      if HGetCBTHook = 0 then
        HGetCBTHook := SetWindowsHookEx(WH_CBT, @GetCBTHook, 0, GetCurrentThreadID);
    end;

    if HGetCallWndProcHook = 0 then
      HGetCallWndProcHook := SetWindowsHookEx(WH_CallWndProc, @GetCallWndProcHook,
      0, GetCurrentThreadID);
  end;
  ActiveMenuList.Add(Self);
end;

destructor TabcMenuToolbar.Destroy;
begin
  if (MainMenu <> nil) then
    MainMenu.WindowHandle := 0;
  FOptions.Free;
  ActiveMenuList.Remove(Self);
  FMergedMenusList.Free;
  FMMFont.Free;
  inherited;
end;

procedure TabcMenuToolbar.Loaded;
begin
  BeginUpdate;
  inherited;
  if (csDesigning in ComponentState) then Exit;
  while ControlCount > 0 do
    Controls[ControlCount - 1].Free;
  FMMFont.Assign(Font);
  HideOwnerFormMenu;
  EndUpdate;
  UpdateTopLevelMenuItems;
end;

procedure TabcMenuToolbar.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
    if AComponent = MainMenu then
      MainMenu := nil
    else if AComponent = StatusBar then
      StatusBar := nil
    else if AComponent = Options.SoundList then
      Options.SoundList := nil;
end;

procedure TabcMenuToolbar.GetBarSize (var ASize: Integer; const DockType: TabcDockType);
begin
  inherited;
  if (BarSize = amsDelphi) then
    ASize := ASize + 1;
end;

procedure TabcMenuToolbar.CMFontChanged(var Message: TMessage);
begin
  inherited;
  if ([csDesigning, csLoading, csReading] * ComponentState = [csDesigning]) then
    Options.ItemHeight := abcCalcMenuItemHeightOf(Font);
end;

procedure TabcMenuToolbar.CMShowingChanged(var Message: TMessage);
begin
  inherited;
  if (MainMenu <> nil) then
    if Visible and Showing then
      MainMenu.WindowHandle := Handle
    else
      MainMenu.WindowHandle := 0;
end;

procedure TabcMenuToolbar.DestroyWindowHandle;
begin
  if (MainMenu <> nil) then
    MainMenu.WindowHandle := 0;
  inherited;
end;

procedure TabcMenuToolbar.MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  abcKillActivePopupMenu(False);
  inherited;
end;

procedure TabcMenuToolbar.SetButtonOrientation(Value: TabcMenuButtonOrientation);
var
  i: integer;
begin
  FButtonOrientation := Value;
  for i := 0 to ControlCount - 1 do
    if Controls[i] is TabcMenuToolbarButton then
      TabcMenuToolbarButton(Controls[i]).Orientation := Value;
end;

procedure TabcMenuToolbar.SetButtonAutoSize(Value: Boolean);
var
  i: integer;
begin
  FButtonAutoSize := Value;
  for i := 0 to ControlCount - 1 do
    if Controls[i] is TabcMenuToolbarButton then
      TabcMenuToolbarButton(Controls[i]).AutoSize := Value;
end;

procedure TabcMenuToolbar.SetMenuProperties(Menu: TPopupMenu);
begin
  if not (Menu is TabcPopupMenu) then Exit;
  with TabcPopupMenu(Menu) do
  begin
    Options.Assign(Self.Options);
    Font.Assign(Self.Font);
    StatusBar := Self.StatusBar;
    StatusBarIndex := Self.FSBPanelNo;
    ParentFont := Self.ParentFont;
    SystemFont := Self.SystemFont;
    ShowHint := Self.ShowHint;
    ParentShowHint := Self.ParentShowHint;
    OnPopup := Self.OnPopup;
    OnMenuCommand := Self.OnMenuCommand;
    OnMenuClose := Self.OnMenuClose;
  end;
end;

function TabcMenuToolbar.CreateButtonForMenu(
  AMenuItem: TMenuItem; AOwner: TComponent; IsSystemButton: Boolean
  ): TabcToolbarButton;
var
  B: TabcMenuToolbarButton;
begin
  B := TabcMenuToolbarButton.Create(Self);
  with B do
  begin
    Font := Self.Font;
    ParentFont := False;
    DropdownArrow := False;
    AllowAllUp := True;
    ShowHint := Self.ShowHint;
    ParentShowHint := False;
    Cursor := Self.Cursor;
    FSystemButton := IsSystemButton;
    AutoSize := FButtonAutoSize;
    Orientation := FButtonOrientation;
    Opaque := FOpaqueButtons;

    if AMenuItem <> nil then
    begin
      MenuItem := AMenuItem;
      if MenuItem.Action <> nil then
        MenuItem.Action.Update;
      Caption := AMenuItem.Caption;
      Enabled := AMenuItem.Enabled;
      HelpContext := AMenuItem.HelpContext;
      Hint := AMenuItem.Hint;
      OnClick := AMenuItem.OnClick;
      
      with MenuItem do
        { Verifying the existence of a ImageList in Delphi4 Menu }
        if (ImageIndex <> - 1) and Assigned(GetParentMenu.Images) then
        begin
          if GetParentMenu.Images.BkColor = clNone then
            Glyph.Canvas.Brush.Color := Color;
          GetParentMenu.Images.GetBitmap(ImageIndex, Glyph);
        end
        else
          { Verifying the existence of a Bitmap in Delphi4 MenuItem }
          if Assigned(Bitmap) and (Bitmap.Handle <> 0) then
          begin
            Glyph.Handle := Bitmap.Handle;
            FReleaseGlyphHandle := True;
          end;

      if AMenuItem.Default then
        B.Font.Style := B.Font.Style + [fsBold];
    end;

    if Glyph.Handle = 0 then
    begin
      mii.fMask := miim_CheckMarks;
      if Assigned(MenuItem) and Assigned(MenuItem.Parent) and
        abcGetMenuItemInfo(MenuItem.Parent.Handle, MenuItem.Command, False, mii) and
        (mii.hbmpUnchecked <> 0) then
      begin
        Glyph.Handle := mii.hbmpUnchecked;
        FReleaseGlyphHandle := True;
      end;
    end;

    if Assigned(MenuItem) and (MenuItem.Count > 0) then
    begin
      GroupIndex := 1;
      DropdownMenu := TabcPopupMenu.Create(AOwner);
      TabcPopupMenu(DropDownMenu).FRuntimeCreated := True;
      SetMenuProperties(DropdownMenu);
      TabcPopupMenu(DropdownMenu).MenuItems := MenuItem;
    end;
  end;
  Result := B;
end;

procedure TabcMenuToolbar.Paint;
begin
  if FMergingMenus or FUpdating then
    Exit;
  inherited;
end;

procedure TabcMenuToolbar.MergeMenus;

  { Find the largest MenuItems GroupIndex }
  function MaxGroupIndex(MenuItems: TMenuItem): Byte;
  var
    i: Integer;
  begin
    Result := 0;
    for i := 0 to MenuItems.Count - 1 do
      if MenuItems[i].GroupIndex > Result then
        Result := MenuItems[i].GroupIndex;
  end;

  function CheckForGroupIndex(Index: Integer; MenuItems: TMenuItem): Boolean;
  var
    I: Integer;
    r: TRect;
  begin
    Result := False;
    for I := 0 to MenuItems.Count - 1 do
    begin
      MenuItems[I].InitiateAction;
      if MenuItems[I].Visible and (MenuItems[I].GroupIndex = Index) then
      begin
        InsertControl(CreateButtonForMenu(MenuItems[I], Owner, False));
        { To reduce the flicker to the minimum... }
        if HandleAllocated then
        begin
          r := Controls[ControlCount - 1].BoundsRect;
          ValidateRect(Handle, @r);
        end;

        Result := True;
      end;
    end;
  end;

var
  AMC: TForm;
  G: Boolean;
  i, j: Integer;
  GrpI, GI2: Byte;
  r: TRect;
begin
  FMergingMenus := True;
  try
    G := False;

    if FMDIChildMenu <> nil then
      FMergedMenusList.Remove(FMDIChildMenu);

    AMC := FOwnerForm.ActiveMdiChild;

    if Assigned(AMC) and Assigned(AMC.Menu)
    then
    begin
      FMDIChildMenu := AMC.Menu;
      FMergedMenusList.Add(FMDIChildMenu);
    end;

    UpdateMenuItemBitmaps(FOwnerForm);
    UpdateMenuItemBitmaps(AMC);

    for i := ControlCount - 1 downto 0 do
      Controls[i].Free;

    { To reduce the flicker to the minimum... }
    if HandleAllocated then
    begin
      r := BoundsRect;
      OffsetRect(r, - r.Left, - r.Top);
      ValidateRect(Handle, @r);
    end;

    if MainMenu <> nil then
    begin
      { Find the largest MenuItems GroupIndex }
      GrpI := MaxGroupIndex(MainMenu.Items);
      with FMergedMenusList do
        for i := 0 to Count - 1 do
        with TMainMenu(Items[i]) do
        begin
          GI2 := MaxGroupIndex(Items);
          if GI2 > GrpI then
            GrpI := GI2;
        end;

      // reinsert menu buttons
      for i := 0 to GrpI do
      begin
        with FMergedMenusList do
          for j := Count - 1 downto 0 do
          begin
            G := CheckForGroupIndex(i, TMainMenu(Items[j]).Items);
            if G then
              Break;
          end;
        if not G then
          CheckForGroupIndex(I, MainMenu.Items);
      end;
    end;

    OrderControls(True, abcGetDockTypeOf(DockedTo), nil);

    FMergedMenus := True;

  finally
    FMergingMenus := False;
    Repaint;
  end;
end;

procedure TabcMenuToolbar.Rebuild;
begin
  if FUpdating then Exit;
  BeginUpdate;
  EndUpdate;
end;

procedure TabcMenuToolbar.BeginUpdate;
begin
  if not FUpdating then
  begin
    FUpdating := True;
    inherited BeginUpdate;
  end;
end;

procedure TabcMenuToolbar.EndUpdate;
begin
  if FUpdating then
  begin
    abcKillActivePopupMenu(False);
    MergeMenus;
    EndUpdate1;
  end;
end;

procedure TabcMenuToolbar.EndUpdate1;
var
  I: Integer;
  AMC: TForm;
  B: TabcMenuToolbarButton;
  L: TList;
  ExistingMenu: TabcPopupMenu;
  bmpTemp: TBitmap;

begin
  if not FUpdating then Exit;
  if Application.Terminated then Exit;
  FSaveMenuVisibleItems := 0;
  if MainMenu <> nil then
    with MainMenu do
    begin
      FSaveMenuItemCount := Items.Count;
      for i := 0 to Items.Count - 1 do
        if Items[i].Visible then
          FSaveMenuVisibleItems := FSaveMenuVisibleItems + 1;
    end
  else
    FSaveMenuItemCount := 0;
  AMC := FOwnerForm.ActiveMdiChild;

  // remove all
  I := 0;
  SpaceButton := nil;
  while I < ControlCount do
  begin
    if (Controls[I] is TabcMenuToolbarButton) and
      TabcMenuToolbarButton(Controls[I]).FSystemButton then
    begin
      Controls[I].Free;
      Continue;
    end;

    if Controls[I] is TSpeedButton then
    begin
      Controls[I].Free;
      Continue;
    end;

    Inc(I);
  end;

  if Assigned(AMC) and (AMC.WindowState = wsMaximized) then
  begin
    // create system menu button
    if (AMC is TForm) then
    begin
      if (biSystemMenu) in AMC.BorderIcons then
      begin
        B := TabcMenuToolbarButton(CreateButtonForMenu(nil, Owner, True));

        if B.DropDownMenu is TabcPopupMenu then
        begin
          TabcPopupMenu(B.DropDownMenu).Form.Free;
          TabcPopupMenu(B.DropDownMenu).Form := nil;
        end;
        B.DropdownMenu.Free;
        B.DropdownMenu := nil;

        ExistingMenu := abcSystemMenuOf(AMC);
        if (ExistingMenu = nil) then
        begin
          ExistingMenu := TabcSystemMenu.Create(AMC);
          ExistingMenu.FRuntimeCreated := True;
        end;
        B.DropdownMenu := abcCreateSystemMenu(Owner, AMC, ExistingMenu);
        SetMenuProperties(B.DropdownMenu);
        B.DisplayMode := admGlyphOnly;
        B.Width := 20;
        B.Tag := sc_Close;
        B.OnDblClick := MenuSystemClick;

        bmpTemp := TBitmap.Create;
        with bmpTemp do
        try
          Width := GetSystemMetrics(SM_CXSMICON) + 1;
          Height := GetSystemMetrics(SM_CYSMICON) + 1;
          with Canvas do
          begin
            Brush.Color := Self.Color;
            FillRect(Rect(0,0,Width, Height));
            PaintMenuIcon(FOwnerForm, AMC, Handle);
          end;
          B.Glyph := bmpTemp;
        finally
          Free;
        end;

        L := TList.Create;
        try
          while ControlCount > 0 do
          begin
            L.Add(Controls[0]);
            RemoveControl(Controls[0]);
          end;

          B.Parent := Self;
          while L.Count > 0 do
          begin
            InsertControl(TControl(L[0]));
            L.Delete(0);
          end;
        finally
          L.Free;
        end;
      end;
    end;

    // space between buttons
    SpaceButton := TSpeedButton.Create(Self);
    SpaceButton.Enabled := False;
    SpaceButton.Parent := Self;

    if ([biMaximize, biMinimize] * AMC.BorderIcons <> []) then
    begin
      // minimize button
      with TSpeedButton.Create(Self) do
      begin
        Width := 16;
        Height := 14;
        Glyph.Handle := LoadBitmap(HInstance, 'ABCM_MINIMIZE_SMALL');
        Enabled := biMinimize in AMC.BorderIcons;
        Tag := sc_Minimize;
        OnClick := MenuSystemClick;
        Hint := sabcMenuMDIMinBtnHint;
        ShowHint := Self.ShowHint;
        Parent := Self;
      end;

      // restore button
      with TSpeedButton.Create(Self) do
      begin
        Width := 16;
        Height := 14;
        Glyph.Handle := LoadBitmap(HInstance, 'ABCM_RESTORE_SMALL');
        Enabled := biMaximize in AMC.BorderIcons;
        Tag := sc_Restore;
        OnClick := MenuSystemClick;
        Hint := sabcMenuMDIRestoreBtnHint;
        ShowHint := Self.ShowHint;
        Parent := Self;
      end;

      // space between buttons
      with TSpeedButton.Create(Self) do
      begin
        Height := 0;
        Width := 2;
        Enabled := False;
        Parent := Self;
      end;
    end;

    // close button
    with TSpeedButton.Create(Self) do
    begin
      Width := 16;
      Height := 14;
      Glyph.Handle := LoadBitmap(HInstance, 'ABCM_CLOSE_SMALL');
      Tag := sc_Close;
      OnClick := MenuSystemClick;
      Hint := sabcMenuMDICloseBtnHint;
      ShowHint := Self.ShowHint;
      Parent := Self;
    end;
    OrderControls(True, abcGetDockTypeOf(DockedTo), nil);
    SetBounds(Left, Top, 0, 0);
  end;

  LastabcButtonMouseInControl := nil;

  inherited EndUpdate;

  { To reduce the flicker to the minimum... }
  if FMergedMenus then
  begin
    if Docked then
      SetBounds(Left, Top, 0, 0);
    FMergedMenus := False;
  end;

  if Application.Active and FOwnerForm.Visible then
    FOwnerForm.SetFocus;

  FUpdating := False;
  HideOwnerFormMenu;
  abcProcessMessages(WM_PAINT);
end;

procedure TabcMenuToolbar.UpdateTopLevelMenuItems;
var
  i, j, visibleItemsCount: Integer;
begin
  if (MainMenu = nil) or ([csDesigning, csLoading, csDestroying] * ComponentState <> []) then Exit;
  with MainMenu do
  begin
{$IFDEF DELPHI5}
      if (AutoHotkeys = maAutomatic) then
        Items.RethinkHotKeys;
{$ENDIF}
    if Items.Count <> FSaveMenuItemCount then
      Rebuild
    else
    begin
      visibleItemsCount := 0;
      for i := 0 to Items.Count - 1 do
      begin
        Items[i].InitiateAction;
        if Items[i].Visible then
          visibleItemsCount := visibleItemsCount + 1;
      end;
      if visibleItemsCount <> FSaveMenuVisibleItems then
      begin
        FSaveMenuVisibleItems := visibleItemsCount;
        Rebuild;
      end
      else
      begin
        BeginUpdate;
        {update buttons and popup menus}
        for j := 0 to ControlCount - 1 do
          if Controls[j] is TabcMenuToolbarButton then
            with TabcMenuToolbarButton(Controls[j]) do
            begin
              Font := Self.Font;
              Font.Style := Font.Style - [fsBold];
              if FGrayActivated then
                Font.Color := clBtnShadow;
              if MenuItem <> nil then
              begin
                Caption := MenuItem.Caption;
                Enabled := MenuItem.Enabled;
                Hint := MenuItem.Hint;
                OnClick := MenuItem.OnClick;
                if MenuItem.Default then
                  Font.Style := Font.Style + [fsBold];
              end;
              if DropdownMenu <> nil then
                with TabcPopupMenu(DropdownMenu) do
                begin
                  Font := Self.Font;
                  Font.Style := Font.Style - [fsBold];
                  if (MenuItem <> nil) and MenuItem.Default then
                    Font.Style := Font.Style + [fsBold];
                end;
              FHBState := False;
              FVBState := False;
              SetBounds(Left, Top, 0, 0);
              Paint;
            end;
        inherited EndUpdate;
        FUpdating := False;
        HideOwnerFormMenu;
      end;
    end;
  end;
end;

procedure TabcMenuToolbar.HideOwnerFormMenu;
begin
  if (MainMenu <> nil) then
    // hides only MDIForm's menu
    if (FOwnerForm is TForm) and
      (not (FOwnerForm.Owner is TForm)) and
      (FOwnerForm.Menu <> nil) then
      FOwnerForm.Menu := nil;
end;

procedure TabcMenuToolbar.UpdateMenuItem(AMenuItem: TMenuItem);
var
  i: Integer;
begin
  if (csDesigning in ComponentState) then Exit;
  BeginUpdate;
  for i := 0 to ControlCount - 1 do
    if Controls[i] is TabcMenuToolbarButton then
      with TabcMenuToolbarButton(Controls[i]) do
          if MenuItem = AMenuItem then
          begin
            Caption := AMenuItem.Caption;
            Enabled := AMenuItem.Enabled;
            Hint :=    AMenuItem.Hint;
            OnClick := AMenuItem.OnClick;
            Font := Self.Font;
            if FGrayActivated then
              Font.Color := clBtnShadow;
            if AMenuItem.Default then
              Font.Style := Font.Style + [fsBold]
            else
              Font.Style := Font.Style - [fsBold];
            FHBState := False;
            FVBState := False;
            SetBounds(Left, Top, 0, 0);
            Paint;
            Break;
          end;

  inherited EndUpdate;
  FUpdating := False;
  HideOwnerFormMenu;
end;

procedure TabcMenuToolbar.InitiateAction;
var
  i: integer;
begin
  inherited;
  { Update main menu's top-most items }
  if MainMenu <> nil then
    for i := 0 to MainMenu.Items.Count - 1 do
      with MainMenu.Items[I] do
        if Visible then InitiateAction;
end;

procedure TabcMenuToolbar.MenuSystemClick(Sender: TObject);
var
  AMC: TForm;
begin
  AMC := FOwnerForm.ActiveMdiChild;
  SendMessage(AMC.Handle, WM_SYSCOMMAND, TComponent(Sender).Tag, 0);
end;

procedure TabcMenuToolbar.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  Order: Boolean;
  I, Sum: Integer;
begin
  if (AWidth <> 0) and (AHeight <> 0) and (not Assigned(DockedTo)) and
    (ControlCount > 0) then
    OrderControls(True, abcGetDockTypeOf(DockedTo), nil);

  // arrange panel
  if Assigned(SpaceButton) then
    if Assigned(DockedTo) and (DockedTo.Position in [adpTop, adpBottom]) then
    begin
      Sum := 0;
      for I := 0 to ControlCount - 1 do
        if Controls[I] <> SpaceButton then
          Inc(Sum, Controls[I].Width);

      Order := (SpaceButton.Height = 1) and (SpaceButton.Width = 1);

      SpaceButton.Height := 0;
      SpaceButton.Width := DockedTo.Width - Sum - 13;

      if Order then
        OrderControls(True, abcGetDockTypeOf(DockedTo), DockedTo);
    end
    else
    begin
      SpaceButton.Height := 1;
      SpaceButton.Width := 1;
    end;
  inherited;
end;

procedure TabcMenuToolbar.SetMainMenu(Value: TMainMenu);
var
  i: integer;
begin
  if Value <> FMainMenu then
  begin
    if (FMainMenu <> nil) and (FMainMenu is TabcMainMenu) then
      TabcMainMenu(FMainMenu).FToolbar := nil;

    FMainMenu := Value;

    if (Value <> nil) and (Value is TabcMainMenu) then
      TabcMainMenu(Value).FToolbar := Self;

    if [csLoading, csDesigning, csDestroying] * ComponentState <> [] then
      Exit;

    Rebuild;
    if (Value <> nil) then
    begin
      if HandleAllocated then
        Value.WindowHandle := Handle;
      FSaveMenuItemCount := Value.Items.Count;
      FSaveMenuVisibleItems := 0;
      for i := 0 to FSaveMenuItemCount - 1 do
        if Value.Items[i].Visible then
          FSaveMenuVisibleItems := FSaveMenuVisibleItems + 1;
    end
    else
    begin
      FSaveMenuItemCount := 0;
      FSaveMenuVisibleItems := 0;
    end;
  end;
end;

var
  LastAMC: TForm = nil;
  LastAMCWS: TWindowState = wsNormal;
  LastAMCMnu: TMainMenu = nil;
  LastAMCClassName: string = 'nil';

procedure TabcMenuToolbar.CheckForMDIChildMenu(Sender: TObject);
var
  AMC: TForm;
  WindowStateChanged, MenuChanged, MaximizedChildClosed: Boolean;
  NewAMCClassName: string;

begin
  if (FOwnerForm.FormStyle <> fsMDIForm) then
    Exit;

  AMC := FOwnerForm.ActiveMdiChild;

  WindowStateChanged := Assigned(AMC) and (LastAMCWS <> AMC.WindowState) and
    ((AMC.WindowState = wsMaximized) or (LastAMCWS = wsMaximized));

  if Assigned(AMC) then
    try
      NewAMCClassName := AMC.ClassName;
    except
      NewAMCClassName := 'nil';
    end
  else
    NewAMCClassName := 'nil';

  if (CompareStr(NewAMCClassName, LastAMCClassName) = 0) and
    not WindowStateChanged then
    Exit;

  MenuChanged := (Assigned(AMC) and (LastAMCMnu <> AMC.Menu)) or
    ((not Assigned(AMC)) and Assigned(LastAMCMnu));

  MaximizedChildClosed := (not Assigned(AMC)) and (LastAMCWS = wsMaximized);

  if WindowStateChanged or MenuChanged or MaximizedChildClosed or (LastAMC <> AMC) then
  begin
    BeginUpdate;
    if MenuChanged then MergeMenus;
    EndUpdate1;
  end;

  LastAMCClassName := NewAMCClassName;

  LastAMC := AMC;
  if Assigned(AMC) then
  begin
    LastAMCWS := AMC.WindowState;
    LastAMCMnu := AMC.Menu;
  end
  else
  begin
    LastAMCWS := wsNormal;
    LastAMCMnu := nil;
  end;
end;

procedure TabcMenuToolbar.SetGrayActivated(Value: Boolean);
var
  i: Integer;
begin
  if FGrayInactive and (Value <> FGrayActivated) then
  begin
    for i := 0 to ControlCount - 1 do
      if (Controls[i] is TabcMenuToolbarButton) and
        not TabcMenuToolbarButton(Controls[i]).FSystemButton then
        with TabcMenuToolbarButton(Controls[i]) do
        begin
          FHBState := False;
          FVBState := False;

          if Value then
            Font.Color := clBtnShadow
          else
            Font.Color := Self.Font.Color;
        end;
    abcProcessMessages(WM_PAINT);
    FGrayActivated := Value;
  end;
end;

procedure TabcMenuToolbar.AMCheckForMDIChildMenu(var Message: TMessage);
begin
  CheckForMDiChildMenu(nil);
end;

procedure TabcMenuToolbar.AMRebuildMenuToolbar(var Message: TMessage);
begin
  Rebuild;
  ArrangeControls;
end;

procedure TabcMenuToolbar.CMSysFontChanged(var Message: TMessage);
begin
  inherited;
  SetSystemFont(FSystemFont);
end;

procedure TabcMenuToolbar.SetSystemFont(Value: Boolean);
begin
  FSystemFont := Value;
  if Value then
  begin
    ParentFont := False;
    abcSetMenuFont(Font);
    UpdateTopLevelMenuItems;
  end;
end;

procedure TabcMenuToolbar.SetOptions(Value: TabcMenuOptions);
begin
  FOptions.Assign(Value);
end;

function TabcMenuToolbar.IsFontStored: Boolean;
begin
  Result := not (ParentFont or SystemFont);
end;

procedure TabcMenuToolbar.SetOpaqueButtons(Value: Boolean);
var i: Integer;
begin
  if FOpaqueButtons <> Value then
  begin
    for i := 0 to ControlCount - 1 do
      if (Controls[i] is TabcMenuToolbarButton) and
        not TabcMenuToolbarButton(Controls[i]).FSystemButton then
        TabcMenuToolbarButton(Controls[i]).Opaque := Value;
    FOpaqueButtons := Value;
  end;
end;

procedure TabcMenuToolbar.SetOldDisabledStyle(Value: Boolean);
var i: Integer;
begin
  if FOldDisabledStyle <> Value then
  begin
    for i := 0 to ControlCount - 1 do
      if (Controls[i] is TabcMenuToolbarButton) then
        with TabcMenuToolbarButton(Controls[i]) do
          if not FSystemButton then
          begin
            OldDisabledStyle := Value;
            if (DropdownMenu is TabcPopupMenu) then
              TabcPopupMenu(DropDownMenu).OldDisabledStyle := Value;
          end;
    FOldDisabledStyle := Value;
  end;
end;

procedure TabcMenuToolbar.SetBarSize(const Value: TabcMenuToolbarSize);
begin
  if Value <> FBarSize then
  begin
    FBarSize := Value;
    if (ComponentState * [csLoading, csDestroying, csDesigning] = []) and HandleAllocated then
      PostMessage(Handle, AM_REBUILDMENUTOOLBAR, 0, 0);
  end;
end;

{ TabcMenuToolbarButton }

constructor TabcMenuToolbarButton.Create(AOwner: TComponent);
begin
  inherited;
  FHBuffer := TBitmap.Create;
  FVBuffer := TBitmap.Create;
  AutoSize := True;
end;

destructor TabcMenuToolbarButton.Destroy;
begin
  Destroying;
  if FInClick then
  begin
    FFreeYourself := True;
    Visible := False;
  end
  else
  begin
    Action := nil;
    if FReleaseGlyphHandle then
      Glyph.ReleaseHandle;
    if Assigned(DropdownMenu) then
      if (DropDownMenu.Owner = Owner) or ((DropDownMenu.Owner = Owner.Owner)) then
      begin
        TabcPopupMenu(DropdownMenu).Form.Release;
        TabcPopupMenu(DropdownMenu).Form := nil;
        TabcPopupMenu(DropdownMenu).Free;
      end
      else
        DropDownMenu := nil;
    FHBuffer.Free;
    FVBuffer.Free;
    inherited;
  end;
end;

procedure TabcMenuToolbarButton.CMDialogChar(var Msg: TCMDialogChar);
begin
  if ((Msg.KeyData and ALT_MASK) = 0) then // ie if ssAlt in ShiftState
    if not Assigned(abcButtonMouseInControl) then
      Exit
    else
      if IsAccel(Msg.CharCode, Caption) and Enabled and Visible then
      begin
        abcButtonMouseInControl.MouseLeft;
        inherited;
      end;
end;

var
  LastX: Integer = 0;
  LastY: Integer = 0;
  ClickNextButtonMove: Boolean = False;
  ButtonMoveLastMenu: TabcMenuToolbar;

procedure TabcMenuToolbarButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  AM: TabcMenuToolbar;
begin
  if ClickNextButtonMove then
  begin
    if ButtonMoveLastMenu <> abcActiveMenu then
    begin
      ButtonMoveLastMenu := abcActiveMenu;
      ClickNextButtonMove := False;
      Exit;
    end;
    // remove SelectedIndex
    if Assigned(DropdownMenu) and (DropdownMenu is TabcPopupMenu) then
      with TabcPopupMenuForm(TabcPopupMenu(DropdownMenu).Form) do
        if SelectedIndex <> - 1 then
          FSelectedIndex := - 1;

    if Enabled then
    begin
      ClickNextButtonMove := False;
      Click;
    end
    else
    begin
      AM := TabcMenuToolbar(Owner);
      if (AM <> nil) and (AM.StatusBar <> nil) then
        SetStatusBarText(Hint);
    end;

    Exit;
  end;

  // ignore some little movements

  if (Abs(X - LastX) <= 2) and (Abs(Y - LastY) <= 2) then Exit;

  LastX := X;
  LastY := Y;

  if (not Down) and Assigned(DropdownMenu) and AssignedActivePopupMenuForm and
    (ActivePopupMenu.Button <> nil) and (ActivePopupMenu.Button <> Self) then
  begin
    abcKillActivePopupMenu(True);
    ClickNextButtonMove := True;
    ButtonMoveLastMenu := TabcMenuToolbar(Owner);
  end;

  inherited MouseMove(Shift, X, Y);

  AM := TabcMenuToolbar(Owner);
  if (AM <> nil) and (AM.StatusBar <> nil) then
    SetStatusBarText(Hint);

  LastabcButtonMouseInControl := Self;
end;

procedure TabcMenuToolbarButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  // on mouse click remove SelectedIndex
  if Assigned(DropdownMenu) and (DropdownMenu is TabcPopupMenu) then
    with TabcPopupMenuForm(TabcPopupMenu(DropdownMenu).Form) do
      if SelectedIndex <> - 1 then
        FSelectedIndex := - 1;
    inherited;
end;

procedure TabcMenuToolbarButton.Click;
begin
  if abcAssignedActiveMenu then
    abcActiveMenu.KeepSelected := False;

  if (Down or FSystemButton) and AssignedActivePopupMenuForm and
    (ActivePopupMenu = DropdownMenu) then
  begin
    abcKillActivePopupMenu(True);
    Down := False;
    MouseEntered;
    FullShowCaret;
  end
  else
  begin
    abcKillActivePopupMenu(True);
    abcButtonMouseInControl := nil;
    LastabcButtonMouseInControl := Self;

    // event
    FInClick := True;
    if MenuItem <> nil then
      MenuItem.Click
    else if Assigned(OnClick) then
      OnClick(Self);
    FInClick := False;
    if FFreeYourself then
    begin
      Free;
      Exit;
    end;
    if (Assigned(DropdownMenu) and (DropdownMenu is TPopupMenu)) then
    begin
      Down := True;
      Paint;

      if (DropdownMenu is TabcPopupMenu) then
      begin
        MouseLeft;
        MouseCapture := False;
        if Assigned(DropdownMenu) then
          with TabcPopupMenu(DropdownMenu) do
          begin
            PopupComponent := Self;
            TabcMenuToolbar(Parent).SetMenuProperties(DropdownMenu);
          end;

        // alignment
        FPP := Point(0, Height);
        if (Parent is TabcCustomToolWindow) and abcAssignedActiveMenu then
          case abcGetDockTypeOf(TabcMenuToolbar(Parent).DockedTo) of
            adtLeftRight:
              FPP := Point(Width, 0);
            adtNotDocked:
              if (Parent.ClientHeight div Height >= 2) and       //only when more than one row
                 (Top + Height + 25 < abcActiveMenu.Height) and
                 ((Left + Width * 2) > abcActiveMenu.Width) then
                FPP := Point(Width, 0);
          else
          end;

        FPP := ClientToScreen(FPP);
        if FSystemButton then
        begin
          abcProcessMessages(WM_PAINT);
          DelaySystemMenu(nil);
        end
        else
        begin
          DropdownMenu.Popup(FPP.X, FPP.Y);
          Down := TabcPopupMenu(DropdownMenu).FormOnScreen;
        end;
      end
      else
        FullShowCaret;

    end
    else
    begin
      FullShowCaret;
      MouseLeft;
      abcButtonMouseInControl := nil;

      //sound
      with TabcMenuToolbar(Parent).Options do
        if (SoundList <> nil) and (SoundCommand <> -1) then
          SoundList.PlayItem(SoundCommand);
      MouseEntered;
    end;
  end;
end;

procedure TabcMenuToolbarButton.DelaySystemMenu(Sender: TObject);
var
  F: TComponent;
  AMC: TForm;
  SysMenu: TabcSystemMenu;
  ExistingMenu: TabcPopupMenu;
begin
  // create menu
  F := Owner;
  while not (F is TForm) do
    F := F.Owner;

  AMC := TForm(F).ActiveMdiChild;
  ExistingMenu := abcSystemMenuOf(AMC);

  if (ExistingMenu = nil) and (DropdownMenu is TabcPopupMenu) then
    ExistingMenu := TabcPopupMenu(DropdownMenu);
  DropdownMenu := abcCreateSystemMenu(F, AMC, ExistingMenu);

  if Owner is TabcMenuToolbar then
    TabcMenuToolbar(Owner).SetMenuProperties(DropdownMenu);

  if Assigned(DropdownMenu) and (Owner is TabcMenuToolbar) then
    with DropdownMenu as TabcPopupMenu do
    begin
      if (AMC is TForm) then
        SysMenu := abcSystemMenuOf(TForm(AMC))
      else
        SysMenu := nil;
      if (SysMenu <> nil) and SysMenu.Enabled then
        Options.Assign(SysMenu.Options)
      else
        Options.Assign(TabcMenuToolbar(Self.Owner).Options);
    end;
  if Assigned(DropdownMenu) then
    DropdownMenu.Popup(FPP.X, FPP.Y);
end;

procedure TabcMenuToolbarButton.WMLButtonDblClk(var Msg: TWMMouse);
begin
  if FSystemButton then
  begin
    abcKillActivePopupMenu(False);
    abcActiveMenu.OwnerForm.ActiveMdiChild.Close;
  end
  else
    inherited;
end;

procedure TabcMenuToolbarButton.CMMouseLeave(var Msg: TMessage);
begin
  if not (abcAssignedActiveMenu and abcActiveMenu.KeepSelected) then
    inherited;
end;

procedure TabcMenuToolbarButton.CMMouseEnter(var Msg: TMessage);
begin
  inherited;
  if not Enabled then
     MouseMove([], - 1, - 1);
end;


procedure TabcMenuToolbarButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  I, H, W: Integer;
  S: String;
begin
  if AutoSize then
  try
    if (Parent is TabcToolbar) and Assigned(Canvas) and Assigned(Glyph) then
    begin
      I := 1;
      S := Caption;
      while I <= Length(S) do
      begin
        if S[I] = '&' then
        begin
          Delete(S, I, 1);
          if (I <= Length(S)) and (S[I] = '&') then Inc(I); // if '&&' found...
        end;
        Inc(I);
      end;

      Canvas.Font := Font;

      if FSystemButton then
      begin
        H := 19;
        W := 20;
      end
      else
      begin
        H := Canvas.TextHeight(S) + 6;
        if (Parent is TabcMenuToolbar) and (TabcMenuToolbar(Parent).BarSize = amsDelphi) then
          H := H + 2;
        W := Canvas.TextWidth(S) + Glyph.Height + 10;
      end;

      if Assigned(TabcToolbar(Parent).DockedTo) and
        (TabcToolbar(Parent).DockedTo.Position in [adpLeft, adpRight]) then
      begin
        AWidth := H;
        AHeight := W;
      end
      else
      begin
        AWidth := W;
        AHeight := H;
        if Assigned(TabcToolbar(Parent).DockedTo) then ATop := 0;
      end;
    end;
  except
    Exit;
  end;
  inherited;
end;

procedure TabcMenuToolbarButton.Paint;
const
  dt_Flags = dt_Center + dt_VCenter + dt_SingleLine + dt_NoClip;
var
  R: TRect;
  GlyphPos: TPoint;
  X, Y: Integer;

  procedure CreateRotatedBitmap;
  var
    W, C, P, X, Y: Integer;
  begin
    FVBuffer.Width :=  FHBuffer.Height;
    FVBuffer.Height := FHBuffer.Width;
    C := ColorToRgb(Color);
    W := FHBuffer.Height;

    with FVBuffer.Canvas do
    begin
      Brush.Color := Color;
      Brush.Style := bsSolid;
      FillRect(ClipRect);
    end;

    for X := 0 to FHBuffer.Width - 1 do
      for Y := 0 to FHBuffer.Height - 1 do
      begin
        P := FHBuffer.Canvas.Pixels[X, Y];
        if P <> C then // not a background color
          FVBuffer.Canvas.Pixels[W - Y - 1, X] := P;
      end;
  end;

begin
  with TabcMenuToolbar(Owner) do
    if FMergingMenus or FUpdating then
      Exit;
  if (FState <> atsUp) and FSystemButton then
    FState := atsUp;

  if (FState = atsUp) and abcAssignedActiveMenu and
    TabcMenuToolbar(Owner).KeepSelected and (Application.Active) and
    (LastabcButtonMouseInControl = Self) and (abcButtonMouseInControl = nil) then
  begin
    abcButtonMouseInControl := Self;
    MouseEntered;
  end;

  if (csDesigning in ComponentState) then
  begin
    FHBState := False;
    FVBState := False;
  end;

  if not FHBState then
  begin
    // bounds
    if not IsVertical then
    begin
      FHBuffer.Width := Width;
      FHBuffer.Height := Height;
    end
    else
    begin
      FHBuffer.Width := Height;
      FHBuffer.Height := Width;
    end;

    // clearing the client area
    R := FHBuffer.Canvas.ClipRect;
    FHBuffer.Canvas.Brush.Color := Color;
    FHBuffer.Canvas.Brush.Style := bsSolid;
    FHBuffer.Canvas.FillRect(R);

    if FSystemButton then
      GlyphPos := Point(2, 0)
    else
      GlyphPos := Point(6, 0);

    if IsVertical then
      GlyphPos.Y := Width  div 2 - 7
    else
      GlyphPos.Y := Height div 2 - 7;

    // draw the button
    Inc(R.Left, Glyph.Width);
    FHBuffer.Canvas.Brush.Style := bsClear;
    FHBuffer.Canvas.Font := Font;

    if Enabled then
    begin
      with Glyph do
        if (Width <> 0) and (Height <> 0) then
          TransBlt(FHBuffer.Canvas, GlyphPos.X, GlyphPos.Y, Width, Height, Handle);

      { Implemented the font color... }
      SetTextColor(FHBuffer.Canvas.Handle, ColorToRgb(Font.Color));
      DrawText(FHBuffer.Canvas.Handle, PChar(Caption), Length(Caption), R, dt_Flags);
    end
    else
    begin
      with Glyph do
        if (Width <> 0) and (Height <> 0) then
          DisabledBlt(FHBuffer.Canvas, GlyphPos.X, GlyphPos.Y, Width, Height, Glyph.Handle, OldDisabledStyle);

      OffsetRect(R, 1, 1);
      SetTextColor(FHBuffer.Canvas.Handle, ColorToRgb(clBtnHighlight));
      DrawText(FHBuffer.Canvas.Handle, PChar(Caption), Length(Caption), R, dt_Flags);

      OffsetRect(R, - 1, - 1);
      SetTextColor(FHBuffer.Canvas.Handle, ColorToRgb(clBtnShadow));
      DrawText(FHBuffer.Canvas.Handle, PChar(Caption), Length(Caption), R, dt_Flags);
    end;
    FHBState := True;
  end;

  // generate vertical bitmap
  if IsVertical and not FVBState then
  begin
    CreateRotatedBitmap;
    FVBState := True;
  end;

  // drawing
  X := 0;  Y := 0;
  if (FState = atsDown) or (FState = atsExclusive) then
  begin
    Inc(X);
    Inc(Y);
  end;

  if IsVertical then
    FVBuffer.Transparent := not Opaque
  else
    fHBuffer.Transparent := not Opaque;

  if IsVertical then
    Canvas.Draw(X, Y, FVBuffer)
  else
    Canvas.Draw(X, Y, FHBuffer);

  // border
  R := Canvas.ClipRect;

  if (FState = atsMouseIn) or (csDesigning in ComponentState) then
    DrawEdge(Canvas.Handle, R, bdr_RaisedInner, bf_Rect);

  if (FState = atsDown) or (FState = atsExclusive) then
    DrawEdge(Canvas.Handle, R, bdr_SunkenOuter, bf_Rect);
end;

function TabcMenuToolbarButton.IsVertical: Boolean;
begin
  Result := (FOrientation = aorVertical) or
    ((FOrientation = aorAutomatic) and (Height > Width));
end;

procedure TabcMenuToolbarButton.SetOrientation(Value: TabcMenuButtonOrientation);
begin
  FOrientation := Value;
  Refresh;
end;

procedure TabcMenuToolbarButton.CMSysColorChange(var Msg: TMessage);
begin
  // buffers' state
  FHBState := False;
  FVBState := False;
  // pattern for checkmarks
  Pattern.Free;
  Pattern := nil;
  inherited;
end;

procedure TabcMenuToolbarButton.CMEnabledChanged (var Message: TMessage);
begin
  FHBState := False;
  FVBState := False;
  inherited;
end;

procedure TabcMenuToolbarButton.CMFontChanged (var Message: TMessage);
begin
  FHBState := False;
  FVBState := False;
  inherited;
  if AutoSize then SetBounds(Left, Top, Width, Height);
end;

procedure TabcMenuToolbarButton.CMTextChanged (var Message: TMessage);
begin
  FHBState := False;
  FVBState := False;
  inherited;
  if AutoSize then SetBounds(Left, Top, Width, Height);
end;

{ TabcPopupMenu }

procedure TabcPopupMenu.MDISystemClick(Sender: TObject);
var
  F: TComponent;
  AMC: TForm;
begin
  F := Owner;
  while not (F is TForm) do
    F := F.Owner;
  AMC := TForm(F).ActiveMdiChild;
  if AMC <> nil then
    PostMessage(AMC.Handle, WM_SYSCOMMAND, TComponent(Sender).Tag, 0);
end;

constructor TabcPopupMenu.Create(AOwner: TComponent);
begin
  inherited;
  FOptions := TabcMenuOptions.Create;
  FFont := TFont.Create;
  FFont.OnChange := FontChanged;
  FAppendSystemItems := False;
  FSystemMenuItems := TList.Create;
  FVisibleItems :=
    [amiRestore, amiMove, amiSize, amiMinimize, amiMaximize,
     amiClose, amiMDINext, amiSeparator];

  // this form will be automatically destroyed by the Owner
  if not (csDesigning in ComponentState) then
    Form := TabcPopupMenuForm.Create(Owner);
  ParentFont := True;
  ParentShowHint := True;
end;

destructor TabcPopupMenu.Destroy;
begin
  FSystemMenuItems.Free;
  FOptions.Free;
  FFont.Free;
  inherited;
end;

procedure TabcPopupMenu.WriteState(Writer: TWriter);
begin
  if not FRuntimeCreated then
    inherited;
end;

procedure TabcPopupMenu.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
    if AComponent = StatusBar then
      StatusBar := nil
    else if AComponent = Options.SoundList then
      Options.SoundList := nil;
end;

procedure TabcPopupMenu.MergeSystemMenu(hSysMenu: HMENU; WindowState: TWindowState; OnClick: TNotifyEvent);
const
  MenuCmds: array [0..5] of UINT = (
  SC_RESTORE, SC_MOVE, SC_SIZE, SC_MINIMIZE,
  SC_MAXIMIZE, SC_CLOSE);

  MenuNames: array [0..5] of String = (
  'SYSTEMRESTORE', 'SYSTEMMOVE', 'SYSTEMSIZE', 'SYSTEMMINIMIZE',
  'SYSTEMMAXIMIZE', 'SYSTEMCLOSE');

var
  i, sysIndex: Integer;

  function wIDIsVisible(fType, wID: UINT): Boolean;
  begin
    if fType and mft_Separator <> 0 then
      Result := (amiSeparator in VisibleItems)
    else
      case wID of
        SC_RESTORE:
          Result := (amiRestore in VisibleItems);
        SC_MOVE:
          Result := (amiMove in VisibleItems);
        SC_SIZE:
          Result := (amiSize in VisibleItems);
        SC_MINIMIZE:
          Result := (amiMinimize in VisibleItems);
        SC_MAXIMIZE:
          Result := (amiMaximize in VisibleItems);
        SC_CLOSE:
          Result := (amiClose in VisibleItems);
        SC_NEXTWINDOW:
          Result := (amiMDINext in VisibleItems);
      else
        Result := True;
      end;
  end;

  procedure AppendMenuItems(Items: TMenuItem; InsertAt: Integer; hMenu: THandle; TopLevel: Boolean);
  var
    i, p, NextPos: Integer;
    M: TMenuItem;
    DefPresent: Boolean;
  begin
    DefPresent := False;
    NextPos := InsertAt;
    for I := 0 to GetMenuItemCount(hMenu) - 1 do
    begin
      mii.dwTypeData := pCharBuf;
      mii.cch := SizeOf(pCharBuf) - 1;
      if not abcGetMenuItemInfo(hMenu, I, True, mii) then
        Continue;

      if TopLevel and not wIDIsVisible(mii.fType, mii.wID) then
        Continue;

      // create menu item for Delphi
      M := TMenuItem.Create(Owner);

      // defining the current state
      if ((WindowState = wsNormal) and (mii.wID = SC_RESTORE)) or
        ((WindowState = wsMaximized) and ((mii.wID = SC_MAXIMIZE) or
         (mii.wID = SC_MOVE) or (mii.wID = SC_SIZE))) or
        ((WindowState = wsMinimized) and ((mii.wID = SC_MINIMIZE) or
         (mii.wID = SC_SIZE))) then
        M.Enabled := False;

      if ((mii.wID = SC_CLOSE) and (not DefPresent)) or (mii.fState and mfs_Default <> 0) then
        M.Default := True;

      M.Tag := mii.wID;
      M.OnClick := OnClick;
      DefPresent := DefPresent or (mii.fState and mfs_Default <> 0);

      if mii.fType and mft_Separator <> 0 then
        M.Caption := '-'
      else
      begin
        P := Pos(#9, pCharBuf);
        if P <> 0 then
        begin
          M.Caption := Copy(pCharBuf, 1, P - 1);
          M.ShortCut := TextToShortCut(Copy(pCharBuf, P + 1, MaxInt));
        end
        else
          M.Caption := pCharBuf;

        for P := 0 to 5 do
          if mii.wID = MenuCmds[P] then
          begin
            Owner.FindComponent(MenuNames[P]).Free;
            M.Name := MenuNames[P];
            Break;
          end;
        end;
        if mii.hSubMenu <> 0 then
          AppendMenuItems(M, 0, mii.hSubMenu, False);
        Items.Insert(NextPos, M);
        NextPos := NextPos + 1;
        if TopLevel then
          FSystemMenuItems.Add(M);
    end;
  end;

begin
  //delete previous system items
  with FSystemMenuItems do
    for i := Count - 1 downto 0 do
    begin
      TMenuItem(Items[i]).Free;
      Delete(i);
    end;
  if FAppendSystemItems then
    sysIndex := 0
  else
    sysIndex := Items.Count;

  mii.fMask := miim_Type + miim_State + miim_ID + miim_SubMenu + miim_CheckMarks;

  AppendMenuItems(Items, sysIndex, hSysMenu, True);

end;

function TabcPopupMenu.CalcOpenAnimation: TabcMenuAnimation;
begin
  Result := Options.OpenAnimation;
  if Result = amnSysMenu then
    Result := amnSmartt
  else if Result = amnToolbar then
    if (PopupComponent is TabcToolbarButton) then
      with TabcMenuToolbar(PopupComponent.Owner) do
        if (DockedTo <> nil) and (DockedTo.Position in [adpLeft, adpRight]) then
          Result := amnHSlide
        else
          Result := amnSmartt
    else
      Result := amnSmartt;
end;

function TabcPopupMenu.CalcCloseAnimation: TabcMenuAnimation;
begin
  Result := Options.CloseAnimation;
  if Result in [amnSysMenu, amnToolbar] then
    Result := amnNone;
end;

{$IFDEF DELPHI5}
procedure TabcPopupMenu.RebuildHandle;
begin
  if MenuItems <> nil then
    with MenuItems do
    begin
      Insert(0, RebuildMenuItem);
      Delete(0);
    end;
end;
{$ENDIF}

procedure TabcPopupMenu.Popup(X, Y: Integer);
var
  i: Integer;
begin
  if (PopupComponent is TabcToolbarButton) and
    (not (PopupComponent is TabcMenuToolbarButton)) and FormOnScreen then
  begin
    TabcPopupMenuForm(Form).AnimatedHide;
    abcKillActivePopupMenu(False);
    Exit;
  end;

  abcKillActivePopupMenu(False);
  ActivePopupMenu := Self;

  SetParentFont(ParentFont);
  SetSystemFont(SystemFont);
  SetParentShowHint(ParentShowHint);

  if Form <> nil then
    Form.BiDiMode := BiDiMode;

  // event
  if Assigned(OnPopup) then OnPopup(Self);

  if Owner is TForm then
    TForm(Owner).BringToFront;

  if Form is TabcPopupMenuForm then
    with Form as TabcPopupMenuForm do
    begin
      ParentPopupMenuForm := nil;

      // initialization
      if Assigned(Self.MenuItems) then
        MenuItems := Self.MenuItems
      else
        MenuItems := Self.Items;

      with MenuItems do
        for i := 0 to Count - 1 do
          Items[i].InitiateAction;

{$IFDEF DELPHI5}
      with TMenuItemAccess(MenuItems) do
        if (GetAutoLineReduction and RethinkLines) or
           (GetAutoHotkeys and RethinkHotkeys) then
          RebuildHandle;
{$ENDIF}

      if Self.MenuHandle <> 0 then
        MenuHandle := Self.MenuHandle
      else
        MenuHandle := MenuItems.Handle;

      if Button <> nil then
        Cursor := Button.Cursor;

      OpenAnimation :=  CalcOpenAnimation;
      CloseAnimation := CalcCloseAnimation;
      PopupMenu := Self;
      SelectedChanged := True;

      { Verifying and correcting if the time goes very small... }
      if abcSubMenuPopupDelay < 50 then
        abcSubMenuPopupDelay := 50;
      Timer.Interval := abcSubMenuPopupDelay;

      UpdateMenuItemBitmaps(TForm(Owner));

      // show menu
      if (not IsWindowEnabled(Handle)) then
        EnableWindow(Handle, True);

      SetBounds(X, Y, 0, 0);
      BringMenuToFront;

      // animation
      try
        if Animate then
        begin
          if (PopupComponent is TabcToolbarButton) and
            not (PopupComponent is TabcMenuToolbarButton) then
            while FormOnScreen do
              Application.ProcessMessages;
          Exit;
        end;
      except
      end;

      SilentHide;
      if ActivePopupMenu = Self then
        ActivePopupMenu := nil;
    end;
end;

function TabcPopupMenu.Button: TabcToolbarButton;
begin
  if PopupComponent is TabcMenuToolbarButton then
    Result := TabcMenuToolbarButton(PopupComponent)
  else
    Result := nil;
end;

function TabcPopupMenu.FormOnScreen: Boolean;
begin
  Result := Assigned(Form) and (Form.Width <> 0) and (Form.Height <> 0) and not (csDestroying in Form.ComponentState);
end;

function TabcPopupMenu.IsFontStored: Boolean;
begin
  Result := not (FParentFont or FSystemFont);
end;

function TabcPopupMenu.IsShowHintStored: Boolean;
begin
  Result := not FParentShowHint;
end;

procedure TabcPopupMenu.FontChanged(Sender: TObject);
var
  MinItemHeight: Integer;
begin
  if not FInternalFontChange then
  begin
    FParentFont := False;
    FSystemFont := False;
  end;
  MinItemHeight := abcCalcMenuItemHeightOf(Font);
  if (Options.ItemHeight < MinItemHeight) or SystemFont then
    Options.ItemHeight := MinItemHeight;
  if Form <> nil then
    Form.Font := Font;
end;

procedure TabcPopupMenu.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TabcPopupMenu.SetParentFont(Value: Boolean);
begin
  FParentFont := Value;
  if Value then
  begin
    FSystemFont := False;
    FInternalFontChange := True;
    if Owner is TForm then
      Font := TForm(Owner).Font;
    FInternalFontChange := False;
  end;
end;

procedure TabcPopupMenu.SetSystemFont(Value: Boolean);
begin
  FSystemFont := Value;
  if Value then
  begin
    FParentFont := False;
    FInternalFontChange := True;
    abcSetMenuFont(Font);
    FInternalFontChange := False;
  end;
end;

procedure TabcPopupMenu.SetShowHint(Value: Boolean);
begin
  if FShowHint <> Value then
  begin
    FShowHint := Value;
    FParentShowHint := False;
  end;
end;

procedure TabcPopupMenu.SetParentShowHint(Value: Boolean);
begin
  FParentShowHint := Value;
  if Value then
  begin
    if Owner is TForm then FShowHint := TForm(Owner).ShowHint;
    if Form <> nil then Form.ShowHint := ShowHint;
  end;
end;

procedure TabcPopupMenu.SetOptions(Value: TabcMenuOptions);
begin
  FOptions.Assign(Value);
end;

{ TabcSystemMenu }

constructor TabcSystemMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TForm) then
    FOwnerForm := TForm(AOwner)
  else if (AOwner.Owner is TForm) then
      FOwnerForm := TForm(AOwner.Owner)
  else
    raise EComponentError.Create(sabcErrorOwnerFormExpected);
  Options.OpenAnimation := amnSysMenu;
  FEnabled := True;
  SystemMenuList.Add(Self);
end;

destructor TabcSystemMenu.Destroy;
begin
  SystemMenuList.Remove(Self);
  Destroying;
  FIconicTimer.Free;
  if not (csDestroying in FOwnerForm.ComponentState) and (FOwnerForm.Handle = FOwnerHandle) then
    {restore form's original WndProc}
    SetWindowLong(FOwnerHandle, GWL_WNDPROC, Longint(FOwnerWindowProc));
  {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FObjectInstance);
  inherited Destroy;
end;

procedure TabcSystemMenu.Loaded;
begin
  inherited Loaded;
  if (OwnerHandle = 0) and not (csDesigning in ComponentState) then
    SubClassOwnerForm;
end;

var
  Recall: Boolean = False;

function TabcSystemMenu.DoProcessMessage(var Message: TMessage): Boolean;
var
  fnResult: Boolean;
begin
  Result := False;
  if not Enabled then Exit;
  fnResult := False;  //so as not to confuse with Message.Result
  case Message.Msg of
    WM_NCLBUTTONDOWN:
      if Recall then
      begin
        Recall:= False;
        fnResult := True;
      end
      else
      begin
        if FormOnScreen then
          abcKillActivePopupMenu(False);
        with TWMNCLButtonDown(Message) do
          if HitTest = htSysMenu then
          begin
            if IsIconic(OwnerHandle) then
              PopupIconicSysMenu(XCursor, YCursor, True)
            else
              PopupSysMenu(XCursor, YCursor, True);
            fnResult := True;
          end
          else if HitTest = htCaption then
          begin
            Recall:= True;
            PostMessage(OwnerHandle, WM_NCLBUTTONDOWN, htSysMenu, 0);
            PostMessage(OwnerHandle, WM_NCLBUTTONUP, windows.htNowhere, Message.lParam);
          end;
      end;
    WM_NCLBUTTONDBLCLK:
    begin
      abcKillActivePopupMenu(False);
      if IsIconic(OwnerHandle) then
      begin
        OwnerForm.Show;
        TWMNCHitMessage(Message).HitTest := windows.htCaption;
      end;
    end;
    WM_NCLBUTTONUP:
      with TWMNCLButtonUp(Message) do
      begin
        if (HitTest = htCaption) and IsIconic(OwnerHandle) then
          PopupSysMenu(XCursor, YCursor, True);
        fnResult := (HitTest in [htSysMenu, htCaption]);
      end;
    WM_NCRBUTTONDOWN:
      fnResult := (TWMNCRButtonDown(Message).HitTest in [htSysMenu, htCaption]);
    WM_NCRBUTTONUP:
      with TWMNCRButtonUp(Message) do
        if (HitTest in [htSysMenu, htCaption]) and
          ((OwnerForm.WindowState <> wsMinimized) or (OwnerForm.FormStyle <> fsMDIChild)) then
        begin
          PopupSysMenu(XCursor, YCursor, False);
          fnResult := True;
        end;
  end;
  Result := fnResult;
end;

procedure TabcSystemMenu.SystemClick(Sender: TObject);
begin
  PostMessage(OwnerForm.Handle, WM_SYSCOMMAND, TComponent(Sender).Tag, 0);
end;

procedure TabcSystemMenu.OnIconicTimer(Sender: TObject);
begin
  FIconicTimer.Enabled := False;
  if IsIconic(OwnerHandle) then
    PopupSysMenu(FSaveIconicPos.X, FSaveIconicPos.Y, FSaveIconicAdjust);
end;

procedure TabcSystemMenu.PopupIconicSysMenu(X, Y: Integer; DoAdjust: Boolean);
begin
  FSaveIconicPos.X := X;
  FSaveIconicPos.Y := Y;
  FSaveIconicAdjust := DoAdjust;
  if FIconicTimer = nil then
  begin
    FIconicTimer := TTimer.Create(Self);
    FIconicTimer.OnTimer := OnIconicTimer;
    FIconicTimer.Interval := abcSubMenuPopupDelay;
  end;
  FIconicTimer.Enabled := True;
end;

procedure TabcSystemMenu.PopupSysMenu(X, Y: Integer; DoAdjust: Boolean);
var
  Placement: TWindowPlacement;
begin
  if AssignedActivePopupMenuForm then Exit;
  FDoAdjust := DoAdjust;
  if DoAdjust then
  begin
    FPopupAt := OwnerForm.ClientOrigin;
    Dec(FPopupAt.Y);
    with OwnerForm do
      if WindowState = wsMinimized then  //get the real minimized position
      begin
        Placement.Length := SizeOf(TWindowPlacement);
        GetWindowPlacement(Handle, @Placement);
        FPopupAt := Placement.ptMinPosition;
        if FormStyle = fsMDIChild then
          Windows.ClientToScreen(Application.MainForm.ClientHandle, FPopupAt);
      end;
  end
  else
    FPopupAt := Point(X, Y);
  DelayPopupSysMenu(nil);
end;

function TabcSystemMenu.CalcOpenAnimation: TabcMenuAnimation;
begin
  Result := Options.OpenAnimation;
  if Result = amnSysMenu then
    if FDoAdjust then
      Result := amnSmartt
    else
      Result := amnPopup
  else if Result = amnToolbar then
    if (PopupComponent is TabcToolbarButton) then
      with TabcMenuToolbar(PopupComponent.Owner) do
        if (DockedTo <> nil) and (DockedTo.Position in [adpLeft, adpRight]) then
          Result := amnHSlide
        else
          Result := amnSmartt
    else
      Result := amnSmartt;
end;

procedure TabcSystemMenu.DelayPopupSysMenu(Sender: TObject);
var
  AOwner: TComponent;
begin
  // determining type of the system menu
  if OwnerForm.FormStyle = fsMDIChild then
    AOwner := Application.MainForm
  else
    AOwner := OwnerForm;

  FPopupMenu := TabcSystemMenu(abcCreateSystemMenu(AOwner, OwnerForm, Self));

  if FDoAdjust and (OwnerForm.WindowState <> wsMinimized) then
    PopupComponent := OwnerForm
  else
    PopupComponent := nil;

  if Assigned(OnShow) then
    OnShow(Self);

  Popup(FPopupAt.X, FPopupAt.Y);
end;

procedure TabcSystemMenu.MessageProc(var Message: TMessage);
begin
  if not DoProcessMessage(Message) then
    {pass message to original WndProc}
    with Message do
      Result := CallWindowProc(FOwnerWindowProc, FOwnerHandle, Msg, WParam, LParam);
end;

procedure TabcSystemMenu.SubClassOwnerForm;
begin
  FOwnerHandle := FOwnerForm.Handle;
  FObjectInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(MessageProc);
  FOwnerWindowProc := Pointer(GetWindowLong(FOwnerHandle, GWL_WNDPROC));
  SetWindowLong(FOwnerHandle, GWL_WNDPROC, Longint(FObjectInstance));
end;

{ TabcPopupMenuForm }

{ To use Bitmaps and the ImageList of Delphi4 Menus }
var
  miBitmap: TBitmap;
  UserCount: Integer;

  { Create a Bitmap for MenuItem }
procedure UsesMenuItemBitmap;
begin
  if UserCount = 0 then miBitmap := TBitmap.Create;
  Inc(UserCount);
end;

{ Destroy the Bitmap for MenuItem }
procedure ReleaseMenuItemBitmap;
begin
  Dec(UserCount);
  if UserCount = 0 then miBitmap.Free;
end;

{ TabcPopupMenuForm }

constructor TabcPopupMenuForm.Create(AOwner: TComponent);
begin
  inherited CreateNew(AOwner, 0);
  BorderStyle := bsNone;
  FormStyle := fsStayOnTop;
  FSelectedIndex := - 1;
  LastSelectedIndex := - 1;
  OpenAnimation :=  amnVSlide;
  CloseAnimation := amnVSlide;
  Timer := TTimer.Create(Self);
  Timer.Enabled := False;
  if abcSubMenuPopupDelay < 50 then
    abcSubMenuPopupDelay := 50;
  Timer.Interval := abcSubMenuPopupDelay;
  Timer.OnTimer := PopupMenuFormTimer;
  Buffer := TBitmap.Create;
  Buffer.Canvas.Font.Assign(Font);
  TileBackBuffer := TBitmap.Create;
  UsesMenuItemBitmap;

  SetBounds(0, 0, 0, 0);
  ShowWindow(Handle, SW_SHOWNA);

  if HGetMessageHook = 0 then
    HGetMessageHook := SetWindowsHookEx(wh_GetMessage, @GetMessageHook, 0, GetCurrentThreadID);
end;

destructor TabcPopupMenuForm.Destroy;
begin
  MenuItems := nil;
  ReleaseMenuItemBitmap;
  Timer.Free;
  Buffer.Free;
  TileBackBuffer.Free;

  if Assigned(ActiveHintWindow) then
    ActiveHintWindow.ReleaseHandle;

  if Assigned(ParentPopupMenuForm) and (ParentPopupMenuForm.CurSubMenu = Self) then
    ParentPopupMenuForm.CurSubMenu := nil;

  if Assigned(CurSubMenu) then
  begin
    CurSubMenu.Release;
    CurSubMenu := nil;
  end;

  inherited Destroy;
end;

procedure TabcPopupMenuForm.WriteState(Writer: TWriter);
begin
  // never saved - runtime component only
end;

procedure TabcPopupMenuForm.CheckShowHint(ShowFloatingHint: Boolean);
var
  s: string;
begin
  GetNearestHelpContext;
  
  if GetCurMenuItem = nil then Exit;

  if Assigned(ActivePopupMenu) and not Assigned(ActivePopupMenu.StatusBar) then
    Application.Hint := CurMenuItem.Hint;

  if Assigned(ActivePopupMenu) then
  begin
    if Assigned(ActivePopupMenu.StatusBar) then
      SetStatusBarText(CurMenuItem.Hint);
    if ShowFloatingHint and ActivePopupMenu.ShowHint then
    begin
      s := Trim(GetShortHint(CurMenuItem.Hint));
      Application.CancelHint;
      ActiveHintWindow.ReleaseHandle;
      ActiveHintWindow.ShowHint(S);
    end;
  end;
end;

function TabcPopupMenuForm.GetRealHeight: Integer;
var
  i, iVisible: Integer;

  procedure CalcItemHeight;
  begin
    mii.dwTypeData := pCharBuf;
    mii.cch := SizeOf(pCharBuf) - 1;
    if not abcGetMenuItemInfo(MenuHandle, iVisible, True, mii) then Exit;

    { When it is attributed a Bitmap or a ImageList to an item }
    { of Delphi4 Menus, its type becomes OwnerDraw!!!          }
    { For obtain the text of the item we have to do:           }
    if (mii.fType and mft_OwnerDraw) = mft_OwnerDraw then
      with MenuItems[i] do
        StrCopy(pCharBuf, PChar(Caption + #9 + ShortCutToText(ShortCut)));

    if mii.fType and mft_Separator <> 0 then
      Inc(Result, Options.SeparatorHeight)
    else
      Inc(Result, Options.ItemHeight);

    Inc(Result, Options.Margins.ItemSpace);
    VisibleMenuItemCount := VisibleMenuItemCount + 1;
  end;

begin
  Result := 0;
  VisibleMenuItemCount := 0;
  if (MenuHandle = 0) or not Assigned(Options) then
    Exit;
  iVisible := -1;

  with Options do
  begin
    Result := Margins.Top;
    mii.fMask := miim_Type;
    for i := 0 to MenuItems.Count - 1 do
    begin
      if MenuItems[i].Visible then
        iVisible := iVisible + 1
      else
        Continue;
      CalcItemHeight;
    end;
    if i < MenuItemCount then
      for iVisible := i to MenuItemCount - 1 do
        CalcItemHeight;

    Inc(Result, Margins.Bottom - Margins.ItemSpace);

    if FitToBackground then
      if Result < Background.Height then
        Result := Background.Height;

    if FitToTitle and (not Assigned(ParentPopupMenuForm)) then
    begin
      i := Title.CalcHeight;
      if Result < i then
        Result := i;
    end;
  end;
end;

function TabcPopupMenuForm.GetRealWidth: Integer;
var
  S: String;
  I, iVisible, P, W1, W2, R1, R2: Integer;

  procedure CalcItemWidth;
  begin
    mii.dwTypeData := pCharBuf;
    mii.cch := SizeOf(pCharBuf) - 1;
    if (not abcGetMenuItemInfo(MenuHandle, iVisible, True, mii)) or
      (mii.fType and mft_Separator <> 0) then
      Exit;

    { When it is attributed a Bitmap or a ImageList to an item }
    { of Delphi4 Menus, its type becomes OwnerDraw             }
    { For obtain the text of the item we have to do this       }
    if (mii.fType and mft_OwnerDraw) = mft_OwnerDraw then
      with MenuItems[i] do
        StrCopy(pCharBuf, PChar(Caption + #9 + ShortCutToText(ShortCut)));

    if (Options.Style in
          [ahsRectangle, ahsBevelLowered, ahsBevelRaised, ahsUnderline]) or
       (mii.fState and mfs_Default <> 0) then
      Canvas.Font.Style := [fsBold]
    else
      Canvas.Font.Style := [];

    S := StrPas(pCharBuf);
    P := Pos(#9, S);
    if P <> 0 then S := Copy(S, 1, P - 1);

    W1 := Canvas.TextWidth(S);

    if R1 < W1 then R1 := W1;

    if P <> 0 then
    begin
      W2 := Canvas.TextWidth(Copy(pCharBuf, P + 1, MaxInt));
      if R2 < W2 then R2 := W2;
    end;
  end;

begin
  Result := 0;
  if MenuHandle = 0 then Exit;
  R1 := 0;
  R2 := 0;
  iVisible := -1;

  mii.fMask := miim_Type + miim_State;
  for I := 0 to MenuItems.Count - 1 do
  begin
    if MenuItems[I].Visible then
      iVisible := iVisible + 1
    else
      Continue;
    CalcItemWidth;
  end;
  if i < MenuItemCount then
    for iVisible := i to MenuItemCount - 1 do
      CalcItemWidth;

  MainWidth := R1;
  Result := R1 + R2;

  with Options do
  begin
    with Title do
      if (not Assigned(ParentPopupMenuForm)) and Visible and
        ((Align = altLeft) or (Align = altRight)) then
        Inc(Result, Width);
    if FitToBackground then
    begin
      W1 := Background.Width - Margins.Left - Margins.Right - 4;
      if (Result < W1) then
        Result := W1;
    end;
    if (MinWidth > 0) then
    begin
      W1 := MinWidth - Margins.Left - Margins.Right - 10;
      if (Result < W1) then
        Result := W1;
    end;
  end;

end;

function TabcPopupMenuForm.GetIndexAt(X, Y: Integer): Integer;
var
  I, H: Integer;
begin
  H := Options.Margins.Top;
  Result := - 1;
  if (MenuHandle = 0) or (Y <= H) then Exit;

  mii.fMask := miim_Type;
  for I := 0 to MenuItemCount - 1 do
  begin
    mii.dwTypeData := pCharBuf;
    mii.cch := SizeOf(pCharBuf) - 1;
    if not abcGetMenuItemInfo(MenuHandle, I, True, mii) then Continue;

    if mii.fType and mft_Separator <> 0 then
      Inc(H, Options.SeparatorHeight)
    else
      Inc(H, Options.ItemHeight);

    if Y <= H then
    begin
      Result := I;
      Exit;
    end;

    Inc(H, Options.Margins.ItemSpace);

    if Y <= H then
    begin
      Result := - 1;
      Exit;
    end;
  end;
end;

function TabcPopupMenuForm.CurItemTop: Integer;
var
  I, J: Integer;
begin
  J := 0;
  Result := 0;
  if (not Assigned(MenuItems)) or (SelectedIndex = - 1) or
    (SelectedIndex >= MenuItems.Count) then
    Exit;

  for I := 0 to MenuItems.Count - 1 do
    if MenuItems[I].Visible then
      if J = SelectedIndex then
        Exit
      else
      begin
        Inc(J);
        if MenuItems[I].Caption = '-' then
          Inc(Result, Options.SeparatorHeight)
        else
          Inc(Result, Options.ItemHeight);
      end;
end;

procedure TabcPopupMenuForm.SetSelectedIndex(Value: Integer);
var
  Direction: Integer;
begin
  ActiveHintWindow.ReleaseHandle;

  if (Value <> SelectedIndex) then
  begin
    if Value > FSelectedIndex then
      Direction := 1
    else
      Direction := - 1;

    LastSelectedIndex := FSelectedIndex;
    FSelectedIndex := Value;
    SelectedChanged := True;

    if MenuHandle = 0 then Exit;

    mii.fMask := miim_Type + miim_State;
    repeat
      // in case if LastSelectedItem = -1
      if (FSelectedIndex = LastSelectedIndex) then Break;

      if FSelectedIndex < 0 then FSelectedIndex := MenuItemCount - 1;
      if FSelectedIndex >= MenuItemCount then FSelectedIndex := - 1;

      // in other cases
      mii.dwTypeData := pCharBuf;
      mii.cch := SizeOf(pCharBuf) - 1;
      if (FSelectedIndex > -1) and abcGetMenuItemInfo(MenuHandle, UINT(FSelectedIndex), True, mii) and
        ((mii.fType and mft_Separator = 0) or
         (FSelectedIndex = LastSelectedIndex)) then
        Break;

      Inc(FSelectedIndex, Direction);

    until
      False;

    DrawDisabled := True;
    Paint;

    CheckShowHint(False);
  end;
end;


{ Events }

procedure TabcPopupMenuForm.WMActivate(var Msg: TWMActivate);
var
  F: TForm;
begin
  if (Msg.Active = WA_ACTIVE) and (Owner is TForm) then
  begin
    F := TForm(Owner);
    while (F.FormStyle = fsMdiChild) and
      (F.Owner is TForm)
    do
      F := TForm(F.Owner);
    if (F is TForm) and (F.Visible) and
      not (csDestroying in F.ComponentState) then
      SendMessage(F.Handle, wm_NCActivate, 1, 0);
  end
  else if (Msg.Active = WA_INACTIVE) and Assigned(PopupMenu) and
    PopupMenu.TrayIconMenu and not Assigned(CurSubMenu) then
    SilentHide
  else
    inherited;
end;

procedure TabcPopupMenuForm.WMMouseActivate(var Msg: TWMMouseActivate);
begin
  Msg.Result := ma_NoActivate;
  if (Owner is TForm) then
    SetActiveWindow(TForm(Owner).Handle);
end;

procedure TabcPopupMenuForm.CMMouseLeave(var Msg: TMessage);
begin
  inherited;
  if Animated then Exit;
  mii.fMask := miim_SubMenu;
  if (FSelectedIndex <> -1) and not (abcGetMenuItemInfo(MenuHandle, UINT(FSelectedIndex), True, mii) and
    (mii.hSubMenu <> 0)) then
  begin
    LastSelectedIndex := FSelectedIndex;
    FSelectedIndex := - 1;
    SelectedChanged := True;
    Paint;
  end;

  SetStatusBarText('');
  ActiveHintWindow.ReleaseHandle;
end;

procedure TabcPopupMenuForm.WMSetFocus(var Msg: TWMSetFocus);
begin
  if Animated and (Owner is TForm) and (TForm(Owner).Visible) then
    TForm(Owner).SetFocus
  else
    inherited;
end;

procedure TabcPopupMenuForm.CMShowingChanged(var Msg: TMessage);
begin
  if Showing then
  begin
    ShowWindow(Handle, sw_ShowNormal or sw_ShowNA);
    SetWindowPos(Handle, hwnd_Top, Left, Top, Width, Height, FormFlags);
  end
  else
    ShowWindow(Handle, sw_Hide);
end;

procedure TabcPopupMenuForm.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then
  begin
    inherited;
    Exit;
  end;
  ActiveHintWindow.ReleaseHandle;
  BringMenuToFront;
end;

procedure TabcPopupMenuForm.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  wID: Integer;
  MI: TMenuItem;
begin
  if Button <> mbLeft then
  begin
    inherited;
    Exit;
  end;
  if Animated or KillAnimation then
    Exit;
  SetStatusBarText('');
  ActiveHintWindow.ReleaseHandle;
  if Assigned(CurSubMenu) then
  begin
    if TForm(Owner).Visible then
      TForm(Owner).SetFocus;
    Exit;
  end;
  MI := GetCurMenuItem;
  if Assigned(MI) then
  begin
    if MI.Count > 0 then
    begin
      PopupCurSubMenu;
      Exit;
    end;
    if (not MI.Enabled) or (MI.Caption = '-') then
      Exit;
    Timer.Enabled := False;
    abcKillActivePopupMenu(True);
    KillAnimation := True;
    SendMessage(TForm(Owner).Handle, wm_NCActivate, 1, 0);
    FullShowCaret;

    //sound
    with Options do
      if (SoundList <> nil) and (SoundCommand <> -1) then
        SoundList.PlayItem(SoundCommand);
    if Assigned(PopupMenu.OnMenuCommand) then
      PopupMenu.OnMenuCommand(MI);
    MI.Click;
  end
  else
  begin
    mii.fMask := miim_Type + miim_ID + miim_Submenu;
    mii.dwTypeData := pCharBuf;
    mii.cch := SizeOf(pCharBuf) - 1;
    if (not abcGetMenuItemInfo(MenuHandle, UINT(SelectedIndex), True, mii)) or
      (mii.fType and mft_Separator <> 0) or
      (mii.fState and mfs_Disabled <> 0) then
      Exit;
    wID := mii.wID;
    abcKillActivePopupMenu(True);
    KillAnimation := True;
    SendMessage(TForm(Owner).Handle, wm_NCActivate, 1, 0);
    FullShowCaret;
    //sound
    with Options do
      if (SoundList <> nil) and (SoundCommand <> -1) then
        SoundList.PlayItem(SoundCommand);
    if Assigned(PopupMenu.OnMenuCommand) then
      PopupMenu.OnMenuCommand(nil);
    PostMessage(TForm(Owner).Handle, wm_Command, wID, 0);
    if abcAssignedActiveMenu then
      abcActiveMenu.CheckForMdiChildMenu(nil);
  end;
end;

procedure TabcPopupMenuForm.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  I: Integer;
begin
  if Animated or KillAnimation or
    ((Abs(X - LastX) <= 2) and (Abs(Y - LastY) <= 2)) then
    Exit;

  LastX := X;
  LastY := Y;

  if IgnoreFirstMouseMove then
  begin
    IgnoreFirstMouseMove := False;
    Exit;
  end;

  I := GetIndexAt(X, Y);
  if I <> SelectedIndex then
  begin
    Timer.Enabled := False;
    SetStatusBarText('');
    ActiveHintWindow.ReleaseHandle;
    LastSelectedIndex := FSelectedIndex;
    FSelectedIndex := I;
    SelectedChanged := True;
    DrawDisabled := False;
    Paint;
    CheckShowHint(False);
    Timer.Enabled := True;
  end;
end;

procedure TabcPopupMenuForm.SetZOrder(TopMost: Boolean);
const
  NormalWindowPos: array[Boolean] of HWND = (HWND_BOTTOM, HWND_TOP);
  TopWindowPos: array[Boolean] of HWND = (HWND_NOTOPMOST, HWND_TOPMOST);
begin
  inherited;
  exit;
  
  if Parent <> nil then
    inherited
  else
    if Handle <> 0 then
      if FormStyle = fsStayOnTop then
        SetWindowPos(Handle, TopWindowPos[TopMost], 0, 0, 0, 0, FormFlags)
      else
        SetWindowPos(Handle, NormalWindowPos[TopMost], 0, 0, 0, 0, FormFlags);
end;

procedure TabcPopupMenuForm.PopupMenuFormTimer(Sender: TObject);
var
  fw: HWND;
  C: TComponent;
begin
  Timer.Enabled := False;

  if SelectedChanged then
  begin
    SelectedChanged := False;
    GetCurMenuItem;
    if not DrawDisabled then
      CheckShowHint(True);
    if (not Assigned(CurSubMenu)) or (CurMenuItem <> CurSubMenu.MenuItems) then
      PopupCurSubMenu;
  end;

  fw := GetForegroundWindow;
  C := FindControl(fw);

  if not ((fw = 0) or (C is TCustomForm) or (fw = Application.Handle) or
    (Assigned(ActiveHintWindow) and (fw = ActiveHintWindow.Handle))) then
  begin
    FullShowCaret;
    abcKillActivePopupMenu(False);
    if abcAssignedActiveMenu then
      abcActiveMenu.KeepSelected := False;
  end;

  if not KillAnimation then
    Timer.Enabled := True;
end;

procedure TabcPopupMenuForm.PopupCurSubMenu;
var
  i: integer;
begin
  if Assigned(CurSubMenu) then
  begin
    CurSubMenu.KillAnimation := True;
    if not CurSubMenu.Animated then
    begin
      CurSubMenu.AnimatedHide;
      CurSubMenu.Release;
    end;
    CurSubMenu := nil;
  end;

  abcProcessMessages(WM_PAINT);

  mii.fMask := miim_State + miim_SubMenu;

  if (FSelectedIndex > -1) and (abcGetMenuItemInfo(MenuHandle, UINT(FSelectedIndex), True, mii)) and
    (mii.hSubMenu <> 0) and (mii.fState and mfs_Disabled = 0) then
    try
      CurSubMenu := TabcPopupMenuForm.Create(Owner);
      CurSubMenu.Font.Assign(Font);
      CurSubMenu.ParentPopupMenuForm := Self;
      CurSubMenu.OpenAnimation := OpenAnimation;
      if CurSubMenu.OpenAnimation = amnVSlideUp then
        //only on top level popup
        CurSubMenu.OpenAnimation := amnVSlide;
      CurSubMenu.CloseAnimation := CloseAnimation;
      CurSubMenu.PopupMenu := PopupMenu;
      CurSubMenu.MenuHandle := mii.hSubMenu;
      CurSubMenu.MenuItems := GetCurMenuItem;
      if CurSubMenu.MenuItems <> nil then
        with CurSubMenu.MenuItems do
          for i := 0 to Count - 1 do
            Items[i].InitiateAction;
      if (GetCurMenuItem <> nil) then
        CurMenuItem.Click;
      with ClientToScreen(Point(Width - 7, CurItemTop)) do
        CurSubMenu.SetBounds(X, Y, 0, 0);
      CurSubMenu.Animate;
    except
      CurSubMenu := nil;
    end
  else
    CurSubMenu := nil;

end;

procedure TabcPopupMenuForm.BringMenuToFront;
var
  F: TabcPopupMenuForm;
begin
  if KillAnimation then
    Exit;
  F := Self;
  while Assigned(F) do
  begin
    SetWindowPos(F.Handle, hwnd_TopMost, F.Left, F.Top, F.Width, F.Height, FormFlags);
    F := F.CurSubMenu;
  end;

  if ActiveHintWindow.Showing then
    with ActiveHintWindow do
      SetWindowPos(Handle, hwnd_TopMost, Left, Top, Width, Height, FormFlags);
end;

function TabcPopupMenuForm.GetNearestHelpContext: Integer;
begin
  Result := 0;
  if (GetCurMenuItem <> nil) then
    Result := CurMenuItem.HelpContext;
  if (Result = 0) and (MenuItems <> nil) then
    Result := MenuItems.HelpContext;
  if (Result = 0) and (ParentPopupMenuForm <> nil) then
    Result := ParentPopupMenuForm.GetNearestHelpContext;
  if (Result = 0) and (PopupMenu <> nil) then
  begin
    Result := PopupMenu.HelpContext;
    if (Result = 0) and (PopupMenu.Button <> nil) then
      Result := PopupMenu.Button.HelpContext;
  end;
  HelpContext := Result;
end;

function TabcPopupMenuForm.GetCurMenuItem: TMenuItem;
begin
  Result := GetMenuItemIndex(SelectedIndex);
end;

function TabcPopupMenuForm.GetMenuItemIndex(Index: Integer): TMenuItem;
var
  I, C: Integer;
begin
  Result := nil;
  CurMenuItem := nil;
  if not Assigned(MenuItems) then
    Exit;
  C := - 1;
  for I := 0 to MenuItems.Count - 1 do
    if MenuItems[I].Visible then
    begin
      Inc(C);
      if C = Index then
      begin
        CurMenuItem := MenuItems[I];
        Result := CurMenuItem;
        Exit;
      end;
    end;
end;

{ Main Routines }

function TabcPopupMenuForm.Animate: Boolean;
var
  PlayCount, NewLeft, NewTop, NewWidth, NewHeight, MX, MY, BL, Time, TimeMinus: Integer;
  TimeStart: DWORD;
  CurAnim: TabcMenuAnimation;
  SR: TRect; // original screen rect

begin
  DX := 1;
  DY := 1;
  PlayCount := 3;
  Result := True;
  Animated := True;
  KillAnimation := False;
  Time := PopupMenu.Options.Timeout;
  TimeMinus := 0;

  // assign options property
  Options := PopupMenu.Options;

  SystemParametersInfo(SPI_GETWORKAREA, 0, @SR, 0);

  // calculate normal height and width.
  // mx and my -- width and height of the full opened window
  MenuItemCount := GetMenuItemCount(MenuHandle);

  MX := GetRealWidth + Options.Margins.Left + Options.Margins.Right + 10;
  if MX < 12 then
    MX := 12;
  MY := GetRealHeight + 4;
  if MY < 12 then
    MY := 12;
  Buffer.Width := MX;
  Buffer.Height := MY;

  // code to arrange menu form
  BL := 0;
  if abcAssignedActiveMenu and (ParentPopupMenuForm = nil) and
    (PopupMenu.Button <> nil) then
    BL := abcActiveMenu.ClientToScreen(Point(PopupMenu.Button.Left, 0)).X;

  FromRightToLeft := False;

  if (Left + MX) > SR.Right then
    if (BL <> 0) and (TabcMenuToolbarButton(PopupMenu.Button).IsVertical) then
      Left := BL - MX
    else if Assigned(ParentPopupMenuForm) then
    begin
      if OpenAnimation = amnVSlide then
        Left := ParentPopupMenuForm.Left - MX + 6
      else
        Left := ParentPopupMenuForm.Left + 6;
      FromRightToLeft := True;
    end
    else
      Left := SR.Right - MX;

  if Left <= 0 then Left := 1;

  FromBottomToTop :=
    ((PopupMenu is TabcSystemMenu) and
    (TabcSystemMenu(PopupMenu).OwnerForm.WindowState = wsMinimized)) or
    not (((Top + MY) < SR.Bottom) or (Top < MY) or (OpenAnimation = amnHSlide));

  if (not FromBottomToTop) and (OpenAnimation = amnVSlideUp) and (Top > MY) then
    FromBottomToTop := True;

  if FromBottomToTop and (PopupMenu.PopupComponent is TControl) and
    (Top < TControl(PopupMenu.PopupComponent).Height) then
    FromBottomToTop := False;

  if FromBottomToTop then
  begin
    if ((PopupMenu.PopupComponent is TabcToolbarButton) or
      (PopupMenu.PopupComponent is TButton) or
      (PopupMenu.PopupComponent is TBitBtn)) and
      ((PopupMenu.Button = nil) or (not TabcMenuToolbarButton(PopupMenu.Button).IsVertical)) then
      Top := Top - TControl(PopupMenu.PopupComponent).Height - 2;

    if ((PopupMenu is TabcSystemMenu) and
      (TabcSystemMenu(PopupMenu).OwnerForm.FormStyle = fsMDIChild) and
      TabcSystemMenu(PopupMenu).FDoAdjust and
      (TabcSystemMenu(PopupMenu).OwnerForm.WindowState <> wsMinimized)) then
      Top := Top - GetSystemMetrics(SM_CYCAPTION)- GetSystemMetrics(SM_CYSIZEFRAME)
    else
      if (BL = 0) and (PopupMenu is TabcSystemMenu) and
        (PopupMenu.PopupComponent is TForm) then
        Top := TForm(PopupMenu.PopupComponent).Top + 3;

  end
  else if Top + MY > SR.Bottom then
  begin
    Top := SR.Bottom - MY;

    if BL <> 0 then
      with TabcMenuToolbarButton(PopupMenu.Button) do
        if (BL <> 0) and (not IsVertical) then
          if (Self.Left + MX + Width > SR.Right) then
            Self.Left := BL - MX
          else
            Self.Left := Self.Left + Width;
  end;

  if Top <= 0 then
    Top := 1;

  NewLeft := Left;
  NewTop := Top;
  NewWidth := MX;
  NewHeight := MY;

  // code for calculation animation steps
  CurAnim := OpenAnimation;

  if CurAnim in [amnSysMenu, amnToolbar] then
    CurAnim := amnSmartt
  else if CurAnim = amnRandom then
    CurAnim := RANDOM_ANIMATIONS[Random(3)];

  if CurAnim = amnSmartt then
  begin
    OpenAnimation := amnHSlide;
    CurAnim := amnVSlide;
  end;

  if CurAnim = amnPopup then
  begin
    OpenAnimation := amnHSlide;
    CurAnim := amnUnfold;
  end;

  if CurAnim in [amnVSlide, amnUnfold, amnVSlideUp] then
  begin
    if PopupMenu.Options.DoubleSpeed then
      DY := MY div 6
    else
      DY := MY div 12;
    if MY > 180 then
      Dec(DY, DY div 4);
  end;

  if CurAnim in [amnHSlide, amnUnfold] then
  begin
    if PopupMenu.Options.DoubleSpeed then
      DX := MX div 6
    else
      DX := MX div 12;
  end;

  if DY = 0 then DY := 1;
  if DX < DY then DX := DY;

  if CurAnim in [amnVSlide, amnUnfold, amnVSlideUp] then
    NewHeight := 0;
  if CurAnim in [amnHSlide, amnUnfold] then
    NewWidth :=  0;

  // check if mouse already moved
  abcProcessMessages(WM_MOUSEMOVE);
  if KillAnimation or not Assigned(PopupMenu) then
  begin
    Animated := False;
    Result := False;
    Exit;
  end;

  // start
  PaintMenu := True;
  Paint;
  SilentShow;

  // Animation!
  while (NewWidth < MX) or (NewHeight < MY) do
  begin
    // calculate the time wasted on drawing
    TimeStart := GetTickCount;

    if NewWidth < MX then
      if (NewWidth + DX) >= MX then
        Break
      else
      begin
        NewWidth := NewWidth + DX;
        if FromRightToLeft then NewLeft := Left - DX;
      end;

    if NewHeight < MY then
      if (NewHeight + DY) >= MY then
        Break
      else
      begin
        NewHeight := NewHeight + DY;
        if FromBottomToTop then NewTop := Top - DY;
      end;

    SetBounds(NewLeft, NewTop, NewWidth, NewHeight);
    Paint;

    abcProcessMessages(WM_MOUSEMOVE);

    if KillAnimation or not Assigned(PopupMenu) then
    begin
      Animated := False;
      Result := False;
      Exit;
    end;

    abcProcessMessages(WM_PAINT);

    // it's necessary in case of menu already opened by another thread
    NewWidth := Width;
    NewHeight := Height;

    if PlayCount >= 0 then
      Dec(PlayCount);

    if PlayCount = 0 then
      //sound
      with Options do
        if (SoundList <> nil) and (SoundPopup <> -1) then
          SoundList.PlayItem(SoundPopup);

    // adjust delay
    BL := Time - Integer((GetTickCount - TimeStart));
    if BL < 0 then
    begin
      TimeMinus := TimeMinus + BL;
      BL := 0;
    end;

    if TimeMinus = 0
    then
      Sleep(BL)
    else
    begin
      Inc(TimeMinus, BL);
      if TimeMinus > 0 then
        TimeMinus := 0;
    end;

  end;

  // final setting bounds
  if FromBottomToTop then
    NewTop := Top - (MY - Height) + 1;
  if FromRightToLeft then
    NewLeft := Left - (MX - Width);

  SetBounds(NewLeft, NewTop, MX, MY);
  Paint;

  if PlayCount > 0 then
    //sound
    with Options do
      if (SoundList <> nil) and (SoundPopup <> -1) then
        SoundList.PlayItem(SoundPopup);

  // finalization
  Animated := False;
  KillAnimation := False;
  Timer.Enabled := Assigned(PopupMenu);

end;

function TabcPopupMenuForm.AnimatedHide: Boolean;
var
  NewLeft, NewTop, NewWidth, NewHeight, MX, MY: Integer;
  CurAnim: TabcMenuAnimation;
begin
  Timer.Enabled := False;
  MX := 0;
  MY := 0;
  Result := True;
  Animated := True;
  KillAnimation := False;
  NewLeft := Left;
  NewTop := Top;
  NewWidth := Width;
  NewHeight := Height;

  // event
  if Assigned(PopupMenu) and Assigned(PopupMenu.OnMenuClose) then
    PopupMenu.OnMenuClose(GetCurMenuItem);

  CurAnim := CloseAnimation;          // current effect
  if CurAnim in [amnSysMenu, amnToolbar] then
    CurAnim := amnNone
  else if CurAnim = amnRandom then
    CurAnim := RANDOM_ANIMATIONS[Random(3)];

  if CurAnim = amnSmartt then
    if ParentPopupMenuForm = nil then
      CurAnim := amnVSlide
    else
      CurAnim := amnHSlide;

  if CurAnim = amnPopup then
    if ParentPopupMenuForm = nil then
      CurAnim := amnUnfold
    else
      CurAnim := amnHSlide;

  if not (CurAnim in [amnVSlide, amnUnfold]) then
    MY := Height;
  if not (CurAnim in [amnHSlide, amnUnfold]) then
    MX := Width;

  //sound
  with Options do
    if (SoundList <> nil) and (SoundClose <> -1) then
      SoundList.PlayItem(SoundClose);

  // Animation!
  while Assigned(PopupMenu) and ((Width > MX) or (Height > MY)) do
  begin
    if Width > MX then
      if (Width - DX) > 0 then
      begin
        NewWidth := Width - DX;
        if FromRightToLeft then
          NewLeft := Left + DX;
      end
      else
      begin
        if FromRightToLeft then
          NewLeft := Left + (MX - Width);
        NewWidth := 0;
      end;

    if Height > MY then
      if (Height - DY) > 0 then
      begin
        NewHeight := Height - DY;
        if FromBottomToTop then NewTop := Top + DY;
      end
      else
      begin
        if FromBottomToTop then
          NewTop := Top + (MY - Height);
        NewHeight := 0;
      end;

    if KillAnimation then
    begin
      Animated := False;
      SilentHide;
      Exit;
    end;

    SetBounds(NewLeft, NewTop, NewWidth, NewHeight);
    Paint;
    abcProcessMessages(WM_PAINT);
  end;
  Animated := False;
  KillAnimation := False;
  SilentHide;
end;

procedure TabcPopupMenuForm.SilentShow;
begin
  // special check for tray-icon menu
  if Assigned(PopupMenu) and PopupMenu.TrayIconMenu then
    SetForegroundWindow(Handle);

  SetWindowPos(Handle, hwnd_TopMost, Left, Top, Width, Height, FormFlags);
  FullHideCaret;
  CheckShowHint(False);
end;

procedure TabcPopupMenuForm.SilentHide;
begin
  Timer.Enabled := False;
  if Assigned(CurSubMenu) then
  begin
    CurSubMenu.KillAnimation := True;
    CurSubMenu.SilentHide;
    CurSubMenu.Release;
    CurSubMenu := nil;
  end;

  SetBounds(Left, Top, 0, 0);

  if Assigned(PopupMenu) and (PopupMenu.PopupComponent is TabcToolbarButton) and
    not (PopupMenu.PopupComponent is TabcMenuToolbarButton) then
    TabcToolbarButton(PopupMenu.PopupComponent).CancelMenuClick;

  FSelectedIndex := - 1;
  Sleep(10);
  abcProcessMessages(WM_PAINT);
end;

procedure TabcPopupMenuForm.Paint;
var
  ItemColors: array [Boolean] of TColor;
  H, bmpH, xDest, yDest: Integer;
  R: TRect;
  ItemType: (itString, itSep);
  I, P, X, Y, menu_ItLeft, menu_ScLeft, menu_Right: Integer;
  FreeM, UseMenuItemBitmaps, DestroyBitmap, HighItem: Boolean;
  M: TMenuItem;
  hBmp: HBitmap;
  S, S1: String;
  OldPalette: HPalette;
  RestorePalette: Boolean;
  textLeft, checkbmpLeft, checkbmpTop: integer;
  miBitmapWidth, miBitmapHeight: integer;
  FbmpSized: Boolean;

  procedure CalcHBitmapSize(const hBmp: HBITMAP; var cWidth, cHeight: integer);
  begin
    with TBitmap.Create do
    try
      Handle := hBmp;
      cWidth := Width;
      cHeight := Height;
      ReleaseHandle;
    finally
      Free;
    end;
  end;

  procedure DrawTextAt(X, Y: Integer; const Text: String;
    Default, Enabled, Selected: Boolean; Alignment: TabcMenuAlignment);
  var
    Flags: Integer;
    IncludedBold, IncludedUnd: Boolean;
  begin
    Flags := dt_NoClip + dt_VCenter;
    with Buffer.Canvas do
    begin
      IncludedBold := Default and not (fsBold in Font.Style);
      IncludedUnd := Selected and Enabled and (Options.Style = ahsUnderline) and
        not (fsUnderline in Font.Style);

      if IncludedBold then
        Font.Style := Font.Style + [fsBold];
      if IncludedUnd then
        Font.Style := Font.Style + [fsUnderline];

      R := Rect(X, Y, X + MainWidth, Y + H);
      Brush.Style := bsClear;

      case Alignment of
        ataRightJustify:
          Flags := Flags or dt_Right;
        ataCenter:
          Flags := Flags or dt_Center;
      end;

      if Enabled then
        if Selected then
          if Options.Style = ahsDefault then
            Font.Color := Options.Colors.HighlightText
          else
            Font.Color := Options.Colors.Highlight
        else
          Font.Color := Options.Colors.MenuText
      else
        Font.Color := Options.Colors.DisabledText;

      if not (Enabled or Selected) or not (Enabled or DrawDisabled) then
      begin
        OffsetRect(R, 1, 1);
        Font.Color := Options.Colors.DisabledShadow;
        DrawText(Handle, PChar(Text), - 1, R, Flags);
        OffsetRect(R, - 1, - 1);
        Font.Color := Options.Colors.DisabledText;
      end
      else
        case Options.Style of
          ahsRectangle, ahsUnderline:
            if Selected then
            begin
              if not IncludedBold then
              begin
                Font.Style := Font.Style + [fsBold];
                IncludedBold := True;
              end;
            end;

          ahsBevelLowered:
            if Selected then
            begin
              if not IncludedBold then
              begin
                Font.Style := Font.Style + [fsBold];
                IncludedBold := True;
              end;
              OffsetRect(R, 1, 1);
            end;

          ahsBevelRaised:
            if Selected then
              begin
                if not IncludedBold then
                begin
                  Font.Style := Font.Style + [fsBold];
                  IncludedBold := True;
                end;
                OffsetRect(R, -1, -1);
              end;

          ahsShadow:
            if Selected then
            begin
              Font.Color := Options.Colors.MenuShadow;
              DrawText(Handle, PChar(Text), - 1, R, Flags);
              OffsetRect(R, - 2, - 2);
              Font.Color := Options.Colors.Highlight;
            end;
        end;

      DrawText(Handle, PChar(Text), - 1, R, Flags);

      if IncludedBold then
        Font.Style := Font.Style - [fsBold];
      if IncludedUnd  then
        Font.Style := Font.Style - [fsUnderline];
    end;
  end;

  function GetItemHeight(const M: TMenuItem): Integer;
  begin
    if M.Caption = '-' then
    begin
      ItemType := itSep;
      Result := Options.SeparatorHeight;
    end
    else
    begin
      ItemType := itString;
      Result := Options.ItemHeight;
    end;
  end;

begin
  if (not Assigned(MenuItems)) and (MenuHandle = 0) then
    Exit;

  // initialization
  FreeM := True;
  M := nil;
  Y := Options.Margins.Top + 2;
  OldPalette := 0;
  RestorePalette := False;
  ItemColors[False] := Options.Colors.Menu;
  ItemColors[True] :=  Options.Colors.Highlight;

  // bounds
  menu_ItLeft := 0;
  menu_Right := Buffer.Width;

  if (not Assigned(ParentPopupMenuForm)) and Options.Title.Visible then
    case Options.Title.Align of
      altLeft:
        Inc(menu_ItLeft, Options.Title.Width);
      altRight:
        Dec(menu_Right,  Options.Title.Width);
    end;

  menu_ScLeft := menu_ItLeft + MainWidth + Options.Margins.Left + 10;

  // Palette
  if (not Options.Background.Empty) and (Options.Background.Palette <> 0) then
  begin
    OldPalette := 0;
    RestorePalette := False;

    if Options.Background.Palette <> 0 then
    begin
      OldPalette := SelectPalette(Buffer.Canvas.Handle, Options.Background.Palette, True);
      RealizePalette(Buffer.Canvas.Handle);
      RestorePalette := True;
    end;
  end;

  // paint edge and background
  if PaintMenu then
    with Buffer, Canvas do

    begin
      Font := Self.Font;
      R := ClipRect;
      {background color}
      Brush.Color := ItemColors[False];
      Brush.Style := bsSolid;
      Windows.FillRect(Handle, R, Brush.Handle);
      {title}
      if (not Assigned(ParentPopupMenuForm)) and Options.Title.Visible then
        Options.Title.Paint(Buffer.Canvas);

      // background image
      if not (Options.Background.Empty) then
        if Options.BackTiled then
        begin
          {paint tiles to TileBackBuffer, then to buffer}
          TileBackBuffer.Assign(Options.Background);
          TileBackBuffer.Width := R.Right - R.Left - menu_ItLeft - 3;
          TileBackBuffer.Height := Buffer.Height;
          yDest := 0;
          while yDest < TileBackBuffer.Height do
          begin
            xDest := 0;
            while xDest < TileBackBuffer.Width do
            begin
              BitBlt(TileBackBuffer.Canvas.Handle, xDest, yDest,
                Options.Background.Width, Options.Background.Height,
                Options.Background.Canvas.Handle, 0, 0, SRCCOPY);
              xDest := xDest + Options.Background.Width;
            end;
            yDest := yDest + Options.Background.Height;
          end;
          BitBlt(Handle, menu_ItLeft + 3, 3,
            TileBackBuffer.Width, TileBackBuffer.Height,
            TileBackBuffer.Canvas.Handle, 0, 0, CopyMode);
        end
        else
          BitBlt(Handle, menu_ItLeft + 3, 3,
            menu_Right - menu_ItLeft - 6, Buffer.Height,
            Options.Background.Canvas.Handle, 0, 0, CopyMode);

      {edges}
      DrawEdge(Handle, R, EDGE_RAISED, bf_Rect);

    end;


  // main drawing
  for I := 0 to VisibleMenuItemCount - 1 do
  begin
    // free old M
    if FreeM then
      M.Free;

    M := GetMenuItemIndex(I);
    if Assigned(M) then
    begin // standard menu item found - it's ok
      FreeM := False;

      // it's still is necessary to check for menu item bitmaps
      mii.fMask := miim_Submenu + miim_CheckMarks;
      abcGetMenuItemInfo(MenuHandle, I, True, mii);
    end
    else
    begin
      // menu item like Window Menu's item created by Windows
      // these items doesn't have corresponding TMenuItem object
      mii.dwTypeData := pCharBuf;
      mii.cch := SizeOf(pCharBuf) - 1;
      mii.fMask := miim_Data + miim_ID + miim_State + miim_Submenu + miim_Type + miim_CheckMarks;
      if not abcGetMenuItemInfo(MenuHandle, I, True, mii) then
        Continue;

      if mii.fType and mft_Separator <> 0 then
        pCharBuf := '-';

      M := NewItem(
        StrPas(pCharBuf),
        0,
        mii.fState and mfs_Checked <> 0,
        mii.fState and mfs_Disabled = 0,
        nil,
        0,
        '');

      M.Default := mii.fState and mfs_Default <> 0;
      M.RadioItem := mii.fType and mft_RadioCheck <> 0;

      P := Pos(#9, M.Caption);
      if P <> 0 then
      begin
        M.ShortCut := TextToShortCut(Copy(M.Caption, P + 1, MaxInt));
        M.Caption := Copy(M.Caption, 1, P - 1);
      end;

      FreeM := True;
    end;

    // main drawing
    with M, Buffer, Buffer.Canvas do
      if Visible then
      begin
        H := GetItemHeight(M);

        if PaintMenu or (((I = SelectedIndex) or (I = LastSelectedIndex)) and
          (SelectedIndex <> LastSelectedIndex)) then
        begin
          HighItem := (I = SelectedIndex) and (ItemType = itString) and (Enabled or DrawDisabled);
          hBmp := 0;
          DestroyBitmap := False;
          UseMenuItemBitmaps := False;

          miBitmapWidth := 16;
          miBitmapHeight := 16;
          FbmpSized := False;

          // get menu item's bitmaps
          if ItemType = itString then
          begin
            { Verifying the existence of a ImageList in Delphi4 Menu }
            if (ImageIndex <> - 1) and Assigned(GetParentMenu.Images) then
            begin
              if GetParentMenu.Images.BkColor = clNone then
                miBitmap.Canvas.Brush.Color := Color;
              miBitmap.Width := 0;
              miBitmap.Height := 0;
              GetParentMenu.Images.GetBitmap(ImageIndex, miBitmap);
              miBitmapWidth := GetParentMenu.Images.Width;
              miBitmapHeight := GetParentMenu.Images.Height;
              FbmpSized := True;
              mii.hbmpChecked := miBitmap.Handle;
              mii.hbmpUnChecked := miBitmap.Handle;
            end
            else
              { Verifying the existence of a Bitmap in Delphi4 MenuItem }
              if Assigned(Bitmap) and (Bitmap.Handle <> 0) then
              begin
                mii.hbmpChecked := Bitmap.Handle;
                mii.hbmpUnChecked := Bitmap.Handle;
                miBitmapWidth := Bitmap.Width;
                miBitmapHeight := Bitmap.Height;
                FbmpSized := True;
              end;

            if (mii.hbmpChecked = 0) and (mii.hbmpUnChecked = 0) and
              (Copy(M.Name, 1, 6) = 'SYSTEM') then
            begin
              hBmp := abcLoadBitmapByName('ABCM_' + M.Name);
              if hBmp = 0 then
                hBmp := abcLoadBitmapByName('ABCM_' + M.Caption);
            end;

            if (hBmp <> 0) then
            begin
              if (mii.hbmpChecked = 0) then
                mii.hbmpChecked := hBmp;
              if (mii.hbmpUnChecked = 0) then
                mii.hbmpUnChecked := hBmp;
              DestroyBitmap := True;
              miBitmapWidth := 16;
              miBitmapHeight := 16;
              FbmpSized := True;
            end;

            UseMenuItemBitmaps := (Caption <> '-') and
              ((mii.hbmpChecked <> 0) or (mii.hbmpUnChecked <> 0));
          end;
          { fill the background with a menu color }
          R := Rect(menu_ItLeft + 3, Y, menu_Right - 3, Y + H);
          Brush.Color := ItemColors[False];
          Brush.Style := bsSolid;
          Windows.FillRect(Handle, R, Brush.Handle);

          if not Options.Background.Empty then
          begin
            { fill background with part of bitmap }
            if I = (MenuItemCount - 1) then
              bmpH := H
            else
              bmpH := H + Options.Margins.ItemSpace;

          if Options.BackTiled then
            BitBlt(Canvas.Handle, menu_ItLeft + 3, Y,
              menu_Right - menu_ItLeft - 6, bmpH,
              TileBackBuffer.Canvas.Handle, 0, Y, Canvas.CopyMode)
          else
            BitBlt(Canvas.Handle, menu_ItLeft + 3, Y,
              menu_Right - menu_ItLeft - 6, bmpH,
              Options.Background.Canvas.Handle, 0, Y, Canvas.CopyMode);
          end;

          if HighItem then
          begin
            { draw the highlite style }
            if UseMenuItemBitmaps or Checked then
              X := H + 4
            else
              X := 3;

            R := Rect(menu_ItLeft + X, Y, menu_Right - 3, Y + H);

            if Options.Style = ahsDefault then
            begin
              Brush.Color := ItemColors[HighItem];
              Brush.Style := bsSolid;
              Windows.FillRect(Handle, R, Brush.Handle);
            end
            else
              case Options.Style of
                ahsBevelLowered:
                  DrawEdge(Handle, R, bdr_SunkenOuter, bf_Rect);
                ahsBevelRaised:
                  DrawEdge(Handle, R, bdr_RaisedInner, bf_Rect);
                ahsRectangle :
                  DrawEdge(Handle, R, Edge_Etched, bf_Rect);
              end;
          end;

          if ItemType = itSep then
          begin
            Pen.Color := Options.Colors.Line;
            PolyLine([Point(menu_ItLeft + 4, Y + 3), Point(menu_Right - 4, Y + 3)]);
            Pen.Color := Options.Colors.LineShadow;
            PolyLine([Point(menu_ItLeft + 4, Y + 4), Point(menu_Right - 4, Y + 4)]);
          end;

          if ItemType = itString then
          begin
            S := Caption;
            P := Pos(#9, M.Caption);
            if P <> 0 then
              S := Copy(S, 1, P - 1);

            if Options.Margins.Left > (H+5) then
              textLeft := menu_ItLeft + Options.Margins.Left
            else
              textLeft := menu_ItLeft + H + 5;

            DrawTextAt(textLeft, Y + 2, S, Default,
              Enabled, SelectedIndex = I, Options.Alignment);

            S1 := ShortCutToText(ShortCut);
            if (S1 = '') and (P <> 0) then
              S1 := Copy(M.Caption, P + 1, MaxInt);
            DrawTextAt(menu_ScLeft, Y + 2, S1, Default,
              Enabled, SelectedIndex = I, ataLeftJustify);

            // submenu triangle
            if mii.hSubMenu <> 0 then
            begin
              if not Enabled then
              begin
                if SelectedIndex <> I then
                 begin
                  Brush.Color := Options.Colors.DisabledShadow;
                  Pen.Color := Options.Colors.DisabledShadow;
                  PolyGon([Point(menu_Right - 13, Y + 7),
                  Point(menu_Right - 13, Y + 13),
                  Point(menu_Right - 10, Y + 10)]);
                end;
                Brush.Color := Options.Colors.DisabledText;
                Pen.Color := Options.Colors.DisabledText;
              end
              else if SelectedIndex <> I then
              begin
                Brush.Color := Options.Colors.MenuText;
                Pen.Color := Options.Colors.MenuText;
              end
              else
              begin
                if Options.Style = ahsDefault then
                begin
                  Brush.Color := Options.Colors.HighlightText;
                  Pen.Color := Options.Colors.HighlightText;
                end
                else
                begin
                  Brush.Color := Options.Colors.Highlight;
                  Pen.Color := Options.Colors.Highlight;
                end;
              end;
              Polygon([Point(menu_Right - 14, Y + 6),
                Point(menu_Right - 14, Y + 12),
                Point(menu_Right - 11, Y + 9)]);
            end;

            if Checked then
            begin
              // inner limits
              if (SelectedIndex = I) and (Enabled or DrawDisabled) then
                Pen.Color := Options.Colors.Highlight
              else
                Pen.Color := Options.Colors.Menu;

              R := Rect(menu_ItLeft + 3, Y, menu_ItLeft + H + 3, Y + H);

              DrawEdge(Handle, R, bdr_SunkenOuter, bf_Rect);

              if Pattern = nil then
                CreatePattern;

              if (SelectedIndex = I) and (Enabled or DrawDisabled) then
                Brush.Color := Options.Colors.Menu
              else
                Brush.Bitmap := Pattern;

              FillRect(Rect(menu_ItLeft + 4, Y + 1, menu_ItLeft + H + 2, Y + H - 1));

              // draw mark
              if hBmp = 0 then
                if UseMenuItemBitmaps and (mii.hbmpChecked <> 0) then
                begin
                  hBmp := mii.hbmpChecked;
                  if not FbmpSized then
                  begin
                    CalcHBitmapSize(hBmp, miBitmapWidth, miBitmapHeight);
                    FbmpSized := True;
                  end;
                end
                else
                begin
                  if RadioItem then
                    hBmp := RadioItemBitmap
                  else
                    hBmp := CheckMarkBitmap;
                  miBitmapWidth := 16;
                  miBitmapHeight := 16;
                  FbmpSized := True;
                end;

              if not FbmpSized then
                CalcHBitmapSize(hBmp, miBitmapWidth, miBitmapHeight);

              checkbmpLeft := menu_ItLeft + 3 + (H-miBitmapWidth) div 2;
              checkbmpTop :=  Y + (H-miBitmapHeight) div 2;

              if Enabled then
                TransBlt(Canvas, checkbmpLeft, checkbmpTop, miBitmapWidth, miBitmapHeight, hBmp)
              else
                DisabledBlt(Canvas, checkbmpLeft, checkbmpTop,  miBitmapWidth, miBitmapHeight, hBmp, PopupMenu.OldDisabledStyle);
            end

            else if UseMenuItemBitmaps and (mii.hbmpUnChecked <> 0) then
            begin
              if (SelectedIndex = I) and (Enabled or DrawDisabled) then
              begin
                R := Rect(menu_ItLeft + 3, Y, menu_ItLeft + H + 3, Y + H);
                DrawEdge(Handle, R, bdr_RaisedInner, bf_Rect);
              end;

              if not FbmpSized then
                CalcHBitmapSize(mii.hbmpUnChecked, miBitmapWidth, miBitmapHeight);

              checkbmpLeft := menu_ItLeft + 3 + (H-miBitmapWidth) div 2;
              checkbmpTop :=  Y + (H-miBitmapHeight) div 2;

              if Enabled then
                TransBlt(Canvas, checkbmpLeft, checkbmpTop, miBitmapWidth, miBitmapHeight, mii.hbmpUnChecked)
              else
                DisabledBlt(Canvas, checkbmpLeft, checkbmpTop, miBitmapWidth, miBitmapHeight, mii.hbmpUnChecked, PopupMenu.OldDisabledStyle);
            end { draw bitmaps };

            if DestroyBitmap then
              DeleteObject(hBmp);

          end { ItemType = itString };
        end;

        Inc(Y, H + Options.Margins.ItemSpace);
      end { draw next row };
  end;

  // background image
  if RestorePalette then
    SelectPalette(Buffer.Canvas.Handle, OldPalette, True);

  // adjust buffer & draw it
  if FromBottomToTop then
    Y := 0
  else
    Y := Buffer.Height - Height;
  if FromRightToLeft then
    X := 0
  else
    X := Buffer.Width - Width;

  BitBlt(Canvas.Handle, 0, 0, Buffer.Width, Buffer.Height, Buffer.Canvas.Handle, X, Y, Canvas.CopyMode);

  if FreeM then
    M.Free;
  PaintMenu := False;
  LastSelectedIndex := SelectedIndex;
end;

{ TabcMenuHintWindow }

constructor TabcMenuHintWindow.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not Assigned(ActiveHintWindow) then
    ActiveHintWindow := Self;
  FHints := TStringList.Create;
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.OnTimer := OnTimer;
  Canvas.Font.Color := clInfoText;
  Canvas.Brush.Color := clInfoBk;
end;

destructor TabcMenuHintWindow.Destroy;
begin
  if ActiveHintWindow = Self then
    ActiveHintWindow := nil;
  FTimer.Free;
  FHints.Free;
  inherited;
end;

procedure TabcMenuHintWindow.OnTimer(Sender: TObject);
begin
  FTimer.Enabled := False;
  ReleaseHandle;
end;

procedure TabcMenuHintWindow.Paint;
var
  R: TRect;
  I, H: Integer;
begin
  H := Canvas.TextHeight(FHints[0]);
  R := ClientRect;
  Canvas.FillRect(R);
  Inc(R.Left, 2);
  Inc(R.Top, 2);
  R.Bottom := H + 6;
  for I := 0 to FHints.Count - 1 do
  begin
    DrawText(Canvas.Handle, PChar(FHints[I]), - 1, R, dt_Left or dt_NoPrefix);
    OffsetRect(R, 0, H + 1);
  end;
end;

procedure TabcMenuHintWindow.ActivateHint(Rect: TRect; const AHint: string);
begin
  ShowHint(AHint);
end;

procedure TabcMenuHintWindow.ShowHint(const AHint: string);
var
  P: TPoint;
  S, S1: String;
  MX, MY, DX, I, L: Integer;
  SR: TRect; // original screen rect
begin
  if AHint = '' then Exit;

  Color := clInfoBk;
  FHints.Clear;

  // special symbol processing
  MX := 0;
  MY := 5;
  S := Trim(GetShortHint(AHint));
  repeat
    I := Pos('&#13;', S);
    L := 4;
    if I = 0 then
    begin
      I := Pos(#13, S);
      L := 0;
    end;
    if I = 0 then
      I := MaxInt;

    S1 := Copy(S, 1, I - 1);
    Inc(MY, Canvas.TextHeight(S1) + 1);
    DX := Canvas.TextWidth(S1) + 6;
    if DX > MX then
      MX := DX;

    FHints.Add(S1);
    if I > Length(S) then
      Break;

    Delete(S, 1, I + L);

  until False;

  SystemParametersInfo(SPI_GETWORKAREA, 0, @SR, 0);

  // coordinates
  GetCursorPos(P);  Inc(P.Y, 25);
  if P.X + MX > SR.Right  then
    P.X := P.X - MX - 10;
  if P.Y + MY > SR.Bottom then
    P.Y := P.Y - MY - 35;
  if P.X <= SR.Left then
    P.X := SR.Left + 1;
  if P.Y <= SR.Top  then
    P.Y := SR.Top  + 1;

  // show
  Caption := AHint;
  BoundsRect := Rect(0, 0, MX, MY);
  SetWindowPos(Handle, hwnd_TopMost, P.X, P.Y, 0, 0, swp_ShowWindow or swp_NoSize or swp_NoActivate);

  FTimer.Interval := Application.HintHidePause;
  FTimer.Enabled := False;
  FTimer.Enabled := True;
end;

{ TabcMenuImageItem }

constructor TabcMenuImageItem.Create(Collection: TCollection);
begin
  FMenuImageItems := TabcMenuImageItems(Collection);
  FMenuImages := FMenuImageItems.MenuImages;
  inherited Create(Collection);
  FImageIndex := - 1;
  FImageIndexChecked := - 1;
end;

destructor TabcMenuImageItem.Destroy;
begin
  FBitmap.Free;
  FBitmapChecked.Free;
  inherited Destroy;
end;

procedure TabcMenuImageItem.Assign(Source: TPersistent);
begin
  if Source is TabcMenuImageItem then
    with TabcMenuImageItem(Source) do
    begin
      Self.FMenuItem := MenuItem;
      Self.FImageIndex := ImageIndex;
      Self.FImageIndexChecked := ImageIndexChecked;
      Changed(False);
    end
    else
      inherited Assign(Source);
end;

function TabcMenuImageItem.GetDisplayName: string;
begin
  if (MenuItem <> nil) and (MenuItem.Name <> '') then
    Result := MenuItem.Name
  else
    Result := inherited GetDisplayName;
end;

procedure TabcMenuImageItem.SetMenuItem(Value: TMenuItem);
begin
  FMenuItem := Value;
  Changed(False);
end;

procedure TabcMenuImageItem.SetImageIndex(Value: TImageIndex);
begin
  FImageIndex := Value;
  Changed(False);
end;

procedure TabcMenuImageItem.SetImageIndexChecked(Value: TImageIndex);
begin
  FImageIndexChecked := Value;
  Changed(False);
end;

procedure TabcMenuImageItem.Update;
var
  imgList: TImageList;
  hbmUnchecked, hbmChecked: HBITMAP;

  procedure ClearCheckedBmp;
  begin
    FBitmapChecked.Free;
    FBitmapChecked := nil;
  end;

  procedure ClearUnCheckedBmp;
  begin
    FBitmap.Free;
    FBitmap := nil;
  end;

begin
  if (MenuImages = nil) or ([csLoading, csDesigning] * MenuImages.ComponentState <> []) or
    (MenuItem = nil) or (MenuItem.Parent = nil) then
    Exit;
  imgList := MenuImages.ImageList;
  if (imgList = nil) or not MenuImages.Enabled then
  begin
    ClearUnCheckedBmp;
    ClearCheckedBmp;
  end
  else
  begin
    {unchecked}
    if (ImageIndex > - 1) and (ImageIndex < imgList.Count) then
    begin
      if FBitmap = nil then
        FBitmap := TBitmap.Create;
      imgList.GetBitmap(ImageIndex, FBitmap);
    end
    else
      ClearUnCheckedBmp;
      {checked}
    if (ImageIndexChecked > - 1) and (ImageIndexChecked < imgList.Count) then
    begin
      if FBitmapChecked = nil then
        FBitmapChecked := TBitmap.Create;
      imgList.GetBitmap(ImageIndexChecked, FBitmapChecked);
    end
    else
      ClearCheckedBmp;
  end;
  {pass to menu item}
  if FBitmap = nil then
    hbmUnchecked := 0
  else
    hbmUnchecked := FBitmap.Handle;
  if FBitmapChecked = nil then
    hbmChecked := hbmUnchecked
  else
    hbmChecked := FBitmapChecked.Handle;
  SetMenuItemBitmaps(MenuItem.Parent.Handle, MenuItem.Command, mf_ByCommand,
  hbmUnchecked, hbmChecked);
end;

{ TabcMenuImageItems }

constructor TabcMenuImageItems.Create(AOwner: TabcMenuImages);
begin
  inherited Create(TabcMenuImageItem);
  FMenuImages := AOwner;
end;

procedure TabcMenuImageItems.Update(Item: TCollectionItem);
begin
  FMenuImages.ItemsChanged(TabcMenuImageItem(Item));
end;

function TabcMenuImageItems.Add: TabcMenuImageItem;
begin
  Result := TabcMenuImageItem(inherited Add);
end;

function TabcMenuImageItems.GetItem(Index: Integer): TabcMenuImageItem;
begin
  Result := TabcMenuImageItem(inherited GetItem(Index));
end;

function TabcMenuImageItems.IndexOfMenuItem(AMenuItem: TMenuItem): Integer;
var
  i: integer;
begin
  if AMenuItem <> nil then
    for i := 0 to Count - 1 do
      if Items[i].MenuItem = AMenuItem then
      begin
        Result := i;
        Exit;
      end;
  Result := - 1;
end;

function TabcMenuImageItems.GetOwner: TPersistent;
begin
  Result := FMenuImages;
end;

procedure TabcMenuImageItems.SetItem(Index: Integer; Value: TabcMenuImageItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TabcMenuImageItems.MenuItemRemoved(Value: TMenuItem);
var
  i: integer;
begin
  for i := 0 to Count - 1 do
    with Items[i] do
      if MenuItem = Value then
        MenuItem := nil;
end;

{ TabcMenuImages }

constructor TabcMenuImages.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TabcMenuImageItems.Create(Self);
  FEnabled := True;
end;

destructor TabcMenuImages.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;

procedure TabcMenuImages.ItemsChanged(Item: TabcMenuImageItem);
var
  i: integer;
begin
  if Item = nil then
    with FItems do
      for i := 0 to Count - 1 do
        Items[i].Update
  else
    Item.Update;
end;

procedure TabcMenuImages.Update;
begin
  ItemsChanged(nil);
end;

procedure TabcMenuImages.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
    if AComponent = FImageList then
      ImageList := nil
    else if AComponent is TMenuItem then
      FItems.MenuItemRemoved(TMenuItem(AComponent));
end;

procedure TabcMenuImages.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
  end;
end;

procedure TabcMenuImages.SetImageList(Value: TImageList);
begin
  if Value <> FImageList then
  begin
    FImageList := Value;
  end;
end;

procedure TabcMenuImages.SetItems(Value: TabcMenuImageItems);
begin
  if Value <> FItems then
  begin
    FItems.Assign(Value);
  end;
end;

initialization
  Randomize;
  ActiveHintWindow := TabcMenuHintWindow.Create(nil);
  ActiveMenuList := TList.Create;
  SystemMenuList := TList.Create;

  // structure for quering menus
  mii.cbSize := 44;

  // check marks
  CheckMarkBitmap := LoadBitmap(HInstance, 'ABCM_CHECKMARK');
  RadioItemBitmap := LoadBitmap(HInstance, 'ABCM_RADIOITEM');

  //for NT3.51 support
  WinVer4OrHigher := LoByte(LoWord(GetVersion)) >= 4;

  RebuildMenuItem := TMenuItem.Create(nil);

finalization
  if HGetMessageHook <> 0 then
  begin
    UnhookWindowsHookEx(HGetMessageHook);
    HGetMessageHook := 0;
  end;

  if HGetCBTHook <> 0 then
  begin
    UnhookWindowsHookEx(HGetCBTHook);
    HGetCBTHook := 0;
  end;

  if HGetCallWndProcHook <> 0 then
  begin
    UnhookWindowsHookEx(HGetCallWndProcHook);
    HGetCallWndProcHook := 0;
  end;

  Pattern.Free;

  ActiveHintWindow.Free;

  ActiveMenuList.Free;

  SystemMenuList.Free;

  DeleteObject(CheckMarkBitmap);
  DeleteObject(RadioItemBitmap);

  RebuildMenuItem.Free;

end.

