unit abcmm;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses Windows, Classes, Controls, Messages, MMSystem, SysUtils, MPlayer, abchandl, abcmem;

type

  TabcCustomMCI = class(TabcWindowHandle)
  private
    FAutoRepeat: Boolean;
    FRepeatPlay: Boolean;
    FOnNotify: TNotifyEvent;
    MCIOpened: Boolean;
    FCapabilities: TMPDevCapsSet;
    FCanPlay: Boolean;
    FCanStep: Boolean;
    FCanEject: Boolean;
    FCanRecord: Boolean;
    FHasVideo: Boolean;
    FFlags: Longint;
    FWait: Boolean;
    FNotify: Boolean;
    FUseWait: Boolean;
    FUseNotify: Boolean;
    FUseFrom: Boolean;
    FUseTo: Boolean;
    FDeviceID: Word;
    FDeviceType: TMPDeviceTypes;
    FTo: Longint;
    FFrom: Longint;
    FFrames: Longint;
    FError: Longint;
    FNotifyValue: TMPNotifyValues;
    FDisplay: TWinControl;
    FDWidth: Integer;
    FDHeight: Integer;
    FElementName: string;
    FAutoOpen: Boolean;
    FAutoRewind: Boolean;
    FAutoClose: Boolean;
    FRaiseExceptions: Boolean;
    FReturnString: array[0..255] of char;
    FShareable: Boolean;
    function CheckIfOpen: Boolean;
    procedure SetPosition(Value: Longint);
    procedure SetDeviceType( Value: TMPDeviceTypes );
    procedure SetWait( Flag: Boolean );
    procedure SetNotify( Flag: Boolean );
    procedure SetFrom( Value: Longint );
    procedure SetTo( Value: Longint );
    procedure SetTimeFormat( Value: TMPTimeFormats );
    procedure SetDisplay( Value: TWinControl );
    procedure SetOrigDisplay;
    procedure SetDisplayRect( Value: TRect );
    function GetDisplayRect: TRect;
    procedure GetDeviceCaps;
    function GetStart: Longint;
    function GetLength: Longint;
    function GetMode: TMPModes;
    function GetTracks: Longint;
    function GetPosition: Longint;
    function GetErrorMessage: string;
    function GetReturnString: string;
    function GetTimeFormat: TMPTimeFormats;
    function GetTrackLength(TrackNum: Integer): Longint;
    function GetTrackPosition(TrackNum: Integer): Longint;
    procedure MMNotify(var Message: TMessage); message MM_MCINOTIFY;
  protected
    function New: Boolean; dynamic;
    function OpenFile(AFileName: TFileName; ADeviceType: TMPDeviceTypes): Boolean; dynamic;
    function SaveAs(AFileName: TFileName): Boolean; dynamic;
    function SendString(Command: string): Boolean; dynamic;
    property ReturnString: string read GetReturnString;

    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoNotify; dynamic;
    function Open: Boolean; virtual;
    function Close: Boolean; dynamic;
    function Play: Boolean; dynamic;
    function Stop: Boolean; dynamic;
    function PauseResume: Boolean; dynamic;
    function Step: Boolean; dynamic;
    function Back: Boolean; dynamic;
    function Previous: Boolean; dynamic;
    function Next: Boolean; dynamic;
    function StartRecording: Boolean; dynamic;
    function Eject: Boolean; dynamic;
    function Save: Boolean; dynamic;
    function Pause: Boolean; dynamic;
    function Resume: Boolean; dynamic;
    function Rewind: Boolean; dynamic;
    {runtime properties}
    property TrackLength[TrackNum: Integer]: Longint read GetTrackLength;
    property TrackPosition[TrackNum: Integer]: Longint read GetTrackPosition;
    property Capabilities: TMPDevCapsSet read FCapabilities;
    property Error: Longint read FError;
    property ErrorMessage: string read GetErrorMessage;
    property Start: Longint read GetStart;
    property Length: Longint read GetLength;
    property Tracks: Longint read GetTracks;
    property Frames: Longint read FFrames write FFrames;
    property Mode: TMPModes read GetMode;
    property Position: Longint read GetPosition write SetPosition;
    property Wait: Boolean read FWait write SetWait;
{$IFDEF DELPHI6}
    property MCINotify: Boolean read FNotify write SetNotify;
{$ELSE}
    property Notify: Boolean read FNotify write SetNotify;
{$ENDIF}
    property NotifyValue: TMPNotifyValues read FNotifyValue;
    property StartPos: Longint read FFrom write SetFrom;
    property EndPos: Longint read FTo write SetTo;
    property DeviceID: Word read FDeviceID;
    property TimeFormat: TMPTimeFormats read GetTimeFormat write SetTimeFormat;
    property DisplayRect: TRect read GetDisplayRect write SetDisplayRect;
    {design/runtime properties}
    property AutoRepeat: Boolean read FAutoRepeat write FAutoRepeat default False;
    property AutoClose: Boolean read FAutoClose write FAutoClose default False;
    property AutoOpen: Boolean read FAutoOpen write FAutoOpen default False;
    property AutoRewind: Boolean read FAutoRewind write FAutoRewind default True;
    property DeviceType: TMPDeviceTypes read FDeviceType write SetDeviceType default dtAutoSelect;
    property Display: TWinControl read FDisplay write SetDisplay;
    property FileName: string read FElementName write FElementName;
    property RaiseExceptions: Boolean read FRaiseExceptions write FRaiseExceptions default False;
    property Shareable: Boolean read FShareable write FShareable default False;
    property OnNotify: TNotifyEvent read FOnNotify write FOnNotify;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function IsDeviceSupported(DeviceType: TMPDeviceTypes): Boolean;
    function IsFileExtSupported(FileExt: string): Boolean;
  end;

  TabcMCIFilePlayer = class(TabcCustomMCI)
  public
    function New: Boolean; override;
    function Open: Boolean; override;
    function Close: Boolean; override;
    function Play: Boolean; override;
    function Stop: Boolean; override;
    function Step: Boolean; override;
    function Back: Boolean; override;
    function Previous: Boolean; override;
    function Next: Boolean; override;
    function Rewind: Boolean; override;
    function StartRecording: Boolean; override;
    function Eject: Boolean; override;
    function Save: Boolean; override;
    function SaveAs(AFileName: TFileName): Boolean; override;
    function Pause: Boolean; override;
    function Resume: Boolean; override;
    function PauseResume: Boolean; override;
    property Handle;
    property TrackLength;
    property TrackPosition;
    property Capabilities;
    property Error;
    property ErrorMessage;
    property Start;
    property Length;
    property Tracks;
    property Frames;
    property Mode;
    property Position;
    property Wait;
{$IFDEF DELPHI6}
    property MCINotify;
{$ELSE}
    property Notify;
{$ENDIF}
    property NotifyValue;
    property StartPos;
    property EndPos;
    property DeviceID;
    property TimeFormat;
    property DisplayRect;
  published
    property AutoClose;
    property AutoOpen;
    property AutoRepeat;
    property AutoRewind;
    property DeviceType;
    property Display;
    property FileName;
    property RaiseExceptions;
    property Shareable;
    property OnNotify;
  end;

  TabcMCIStreamPlayer = class(TabcCustomMCI)
  private
    FMCIData: TabcPersistentMemory;
    FDataChanged: Boolean;
    FSaving: Boolean;
    function GetDeviceType: TMPDeviceTypes;
    function GetFileName: TFileName;
    procedure MCIDataChanged(Sender: TObject);
    procedure MCIDataChanging(Sender: TObject);
    procedure SetDeviceType(Value: TMPDeviceTypes);
    procedure SetMCIData(Value: TabcPersistentMemory);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function New: Boolean; override;
    function Open: Boolean; override;
    function Close: Boolean; override;
    function Play: Boolean; override;
    function Stop: Boolean; override;
    function Step: Boolean; override;
    function Back: Boolean; override;
    function Rewind: Boolean; override;
    function Previous: Boolean; override;
    function Next: Boolean; override;
    function StartRecording: Boolean; override;
    function Eject: Boolean; override;
    function Save: Boolean; override;
    function Pause: Boolean; override;
    function Resume: Boolean; override;
    function PauseResume: Boolean; override;
    property Handle;
    property TrackLength;
    property TrackPosition;
    property Capabilities;
    property DataChanged: Boolean read FDataChanged;
    property Error;
    property ErrorMessage;
    property Start;
    property Length;
    property Tracks;
    property Frames;
    property Mode;
    property Position;
    property Wait;
{$IFDEF DELPHI6}
    property MCINotify;
{$ELSE}
    property Notify;
{$ENDIF}
    property NotifyValue;
    property StartPos;
    property EndPos;
    property DeviceID;
    property TimeFormat;
    property DisplayRect;
    property FileName: TFileName read GetFileName;
  published
    property AutoClose;
    property AutoOpen;
    property AutoRepeat;
    property AutoRewind;
    property DeviceType read GetDeviceType write SetDeviceType default dtWaveAudio;
    property Display;
    property MCIData: TabcPersistentMemory read FMCIData write SetMCIData;
    property RaiseExceptions;
    property Shareable;
    property OnNotify;
  end;

implementation

uses Consts, {$IFDEF DELPHI6}RTLConsts,{$ENDIF} abcconst;

const
  mci_Back     = $0899;  { mci_Step reverse }
  DeviceName: array[TMPDeviceTypes] of PChar = ('', 'AVIVideo', 'CDAudio', 'DAT',
    'DigitalVideo', 'MMMovie', 'Other', 'Overlay', 'Scanner', 'Sequencer',
    'VCR', 'Videodisc', 'WaveAudio');

function TabcCustomMCI.IsDeviceSupported(DeviceType: TMPDeviceTypes): Boolean;
var
  Command: string;
begin
  Result := False;
  Command := 'sysinfo ' + StrPas(DeviceName[DeviceType]) + ' quantity';
  if SendString(Command) then
    Result := ReturnString <> '0';
end;

function TabcCustomMCI.IsFileExtSupported(FileExt: string): Boolean;
var
  PFileExt: array[0..3] of char;
  PStr: array[0..255] of char;
  Command: string;
begin
  Result := False;
  {get device name from WIN.INI}
  StrPCopy(PFileExt, FileExt);
  if GetProfileString('mci extensions', PFileExt, '', PStr, SizeOf(PStr)) <> 0 then begin
    Command := 'sysinfo ' + StrPas(PStr) + ' quantity';
    if SendString(Command) then
      Result := ReturnString <> '0';
  end;
end;

function TabcCustomMCI.GetReturnString: string;
begin
  Result := StrPas(FReturnString);
end;

function TabcCustomMCI.New: Boolean;
var
  OpenParm: TMCI_Open_Parms;
  DisplayR: TRect;
begin
  Result := False;
  if MCIOpened then
    if not Close then Exit;
  if FRepeatPlay then FRepeatPlay := False;

  if FDeviceType <> dtAutoSelect then
    OpenParm.lpstrDeviceType := DeviceName[FDeviceType];

  {blank file name}
  FileName := '';
  OpenParm.lpstrElementName := '';

  OpenParm.dwCallback := 0;

  FFlags := 0;
  if FUseWait then
  begin
    if FWait then FFlags := mci_Wait;
    FUseWait := False;
  end
  else FFlags := mci_Wait;
  if FUseNotify then
  begin
    if FNotify then FFlags := FFlags or mci_Notify;
    FUseNotify := False;
  end;

  FFlags := FFlags or mci_Open_Element;
  if FDeviceType <> dtAutoSelect then FFlags := FFlags or mci_Open_Type;
  if FShareable then FFlags := FFlags or mci_Open_Shareable;
  OpenParm.dwCallback := Handle;
  FError := mciSendCommand(0, mci_Open, FFlags, Longint(@OpenParm));

  if FError <> 0 then {problem opening device}
    if RaiseExceptions then
      raise EMCIDeviceError.Create(ErrorMessage)
    else
      Exit
  else begin
    MCIOpened := True;
    FDeviceID := OpenParm.wDeviceID;
    FFrames := Length div 10;
    GetDeviceCaps;
    if FHasVideo then begin
      Display := FDisplay;
      DisplayR := GetDisplayRect;
      FDWidth := DisplayR.Right-DisplayR.Left;
      FDHeight := DisplayR.Bottom-DisplayR.Top;
    end;
    if (FDeviceType = dtCDAudio) or (FDeviceType = dtVideodisc) then
      TimeFormat := tfTMSF;
    Result := True;
  end;
end;

function TabcCustomMCI.OpenFile(AFileName: TFileName; ADeviceType: TMPDeviceTypes): Boolean;
begin
  Result := False;
  if MCIOpened then
    if not Close then Exit;
  FileName := AFileName;
  DeviceType := ADeviceType;
  Result := Open;
end;

function TabcCustomMCI.SaveAs(AFileName: TFileName): Boolean;
begin
  FileName := AFileName;
  Result := Save;
end;

function TabcCustomMCI.SendString(Command: string): Boolean;
var
  pCommand: array [0..255] of char;
begin
  StrPLCopy(pCommand, Command, 255);
  FError := mciSendString(pCommand, FReturnString, 255, Handle);
  Result := (FError = 0);
end;

constructor TabcCustomMCI.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoOpen := False;
  FAutoRewind := True;
  FDeviceType := dtAutoSelect;
end;

destructor TabcCustomMCI.Destroy;
var
  GenParm: TMCI_Generic_Parms;
begin
  if FDeviceID <> 0 then
    mciSendCommand( FDeviceID, mci_Close, mci_Wait, Longint(@GenParm));
  inherited Destroy;
end;

procedure TabcCustomMCI.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) and FAutoOpen then
    Open;
end;

{MCI message generated when Notify=True, and MCI command completes}
procedure TabcCustomMCI.MMNotify(var Message: TMessage);
begin
  case Message.WParam of
    mci_Notify_Successful: FNotifyValue := nvSuccessful;
    mci_Notify_Superseded: FNotifyValue := nvSuperseded;
    mci_Notify_Aborted: FNotifyValue := nvAborted;
    mci_Notify_Failure: FNotifyValue := nvFailure;
  end;
  DoNotify;
end;

{for MCI Commands to make sure device is open, else raise exception}
function TabcCustomMCI.CheckIfOpen: Boolean;
begin
  Result := MCIOpened;
  if not Result and RaiseExceptions then
      raise EMCIDeviceError.Create(sNotOpenErr);
end;

procedure TabcCustomMCI.DoNotify;
begin
  if Assigned(FOnNotify) then FOnNotify(Self);
  if FAutoClose then Close;
  if (Mode = mpStopped) and FAutoRepeat and FRepeatPlay then begin
    Rewind;
    Play;
  end;
end;

{ MCI Commands }

function TabcCustomMCI.Open: Boolean;
const
  DeviceName: array[TMPDeviceTypes] of PChar = ('', 'AVIVideo', 'CDAudio', 'DAT',
    'DigitalVideo', 'MMMovie', 'Other', 'Overlay', 'Scanner', 'Sequencer',
    'VCR', 'Videodisc', 'WaveAudio');
var
  ElementName: array [0..150] of Char;
  OpenParm: TMCI_Open_Parms;
  DisplayR: TRect;
begin
  Result := False;
  if FElementName = '' then
    if RaiseExceptions then
      raise EMCIDeviceError.Create(Format(SFOpenError, ['']))
    else
      Exit;
  if MCIOpened then
    if not Close then Exit;
  if FRepeatPlay then FRepeatPlay := False;
  OpenParm.dwCallback := 0;
  if FDeviceType <> dtAutoSelect then
   OpenParm.lpstrDeviceType := DeviceName[FDeviceType];
  if FElementName <> '' then begin
    StrPLCopy( ElementName, FElementName, 150);
    OpenParm.lpstrElementName := ElementName;
  end;

  FFlags := 0;
  if FUseWait then begin
    if FWait then FFlags := mci_Wait;
    FUseWait := False;
  end
  else
    FFlags := mci_Wait;
  if FUseNotify then begin
    if FNotify then FFlags := FFlags or mci_Notify;
    FUseNotify := False;
  end;

  if FElementName <> '' then FFlags := FFlags or mci_Open_Element;
  if FDeviceType <> dtAutoSelect then FFlags := FFlags or mci_Open_Type;
  if FShareable then FFlags := FFlags or mci_Open_Shareable;
  OpenParm.dwCallback := Handle;
  FError := mciSendCommand(0, mci_Open, FFlags, Longint(@OpenParm));

  if FError <> 0 then
    if RaiseExceptions then
      raise EMCIDeviceError.Create(ErrorMessage)
    else
      Exit
  else begin
    MCIOpened := True;
    FDeviceID := OpenParm.wDeviceID;
    FFrames := Length div 10;
    GetDeviceCaps;
    if FHasVideo then begin
      Display := FDisplay;
      DisplayR := GetDisplayRect;
      FDWidth := DisplayR.Right-DisplayR.Left;
      FDHeight := DisplayR.Bottom-DisplayR.Top;
    end;
    if (FDeviceType = dtCDAudio) or (FDeviceType = dtVideodisc) then
      TimeFormat := tfTMSF;
    Result := True;
  end;
end;

function TabcCustomMCI.Close: Boolean;
var
  GenParm: TMCI_Generic_Parms;
begin
  Result := False;
  if FDeviceID <> 0 then begin
    if FRepeatPlay then FRepeatPlay := False;
    FFlags := 0;
    if FUseWait then begin
      if FWait then FFlags := mci_Wait;
      FUseWait := False;
    end
    else
      FFlags := mci_Wait;
    if FUseNotify then begin
      if FNotify then FFlags := FFlags or mci_Notify;
      FUseNotify := False;
    end;
    GenParm.dwCallback := Handle;
    FError := mciSendCommand( FDeviceID, mci_Close, FFlags, Longint(@GenParm));
    if FError = 0 then begin
      MCIOpened := False;
      FDeviceID := 0;
      Result := True;
    end;
  end
  else
    Result := False;
end;

function TabcCustomMCI.Play: Boolean;
var
  PlayParm: TMCI_Play_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  {if at the end of media, and not using StartPos or EndPos - go to start}
  if FAutoRewind and (Position = Length) then
    if not FUseFrom and not FUseTo then
      if not Rewind then Exit;
  FFlags := 0;
  if FUseNotify then begin
    if FNotify then FFlags := mci_Notify;
    FUseNotify := False;
  end
  else
    FFlags := mci_Notify;
  if FUseWait then begin
    if FWait then FFlags := FFlags or mci_Wait;
    FUseWait := False;
  end;
  if FUseFrom then begin
    FFlags := FFlags or mci_From;
    PlayParm.dwFrom := FFrom;
    FUseFrom := False; {only applies to this mciSendCommand}
  end;
  if FUseTo then begin
    FFlags := FFlags or mci_To;
    PlayParm.dwTo := FTo;
    FUseTo := False; {only applies to this mciSendCommand}
  end;
  PlayParm.dwCallback := Handle;
  if FAutoRepeat then FRepeatPlay := True;
  FError := mciSendCommand(FDeviceID, mci_Play, FFlags, Longint(@PlayParm));
  Result := (FError = 0);
end;

function TabcCustomMCI.StartRecording: Boolean;
var
  RecordParm: TMCI_Record_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;

  FFlags := 0;
  if FUseNotify then begin
    if FNotify then FFlags := mci_Notify;
    FUseNotify := False;
  end
  else
    FFlags := mci_Notify;
  if FUseWait then begin
    if FWait then FFlags := FFlags or mci_Wait;
    FUseWait := False;
  end;

  if FUseFrom then begin
    FFlags := FFlags or mci_From;
    RecordParm.dwFrom := FFrom;
    FUseFrom := False;
  end;
  if FUseTo then begin
    FFlags := FFlags or mci_To;
    RecordParm.dwTo := FTo;
    FUseTo := False;
  end;
  RecordParm.dwCallback := Handle;
  FError := mciSendCommand( FDeviceID, mci_Record, FFlags, Longint(@RecordParm));
  Result := (FError = 0);
end;

function TabcCustomMCI.Stop: Boolean;
var
  GenParm: TMCI_Generic_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;
  FFlags := 0;
  if FUseWait then begin
    if FWait then FFlags := mci_Wait;
    FUseWait := False;
  end
  else
    FFlags := mci_Wait;
  if FUseNotify then begin
    if FNotify then FFlags := FFlags or mci_Notify;
    FUseNotify := False;
  end;
  GenParm.dwCallback := Handle;
  FError := mciSendCommand( FDeviceID, mci_Stop, FFlags, Longint(@GenParm));
  Result := (FError = 0);
end;

function TabcCustomMCI.PauseResume: Boolean;
begin
  Result := False;
  if not MCIOpened then
    if RaiseExceptions then
      raise EMCIDeviceError.Create(sNotOpenErr)
    else
      Exit;
  if Mode = mpPlaying then
    Result := Pause
  else if Mode = mpPaused then
    Result := Resume;
end;

function TabcCustomMCI.Pause: Boolean;
var
  GenParm: TMCI_Generic_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  FFlags := 0;
  if FUseWait then begin
    if FWait then FFlags := mci_Wait;
    FUseWait := False;
  end
  else
    FFlags := mci_Wait;
  if FUseNotify then begin
    if FNotify then FFlags := FFlags or mci_Notify;
    FUseNotify := False;
  end;
  GenParm.dwCallback := Handle;
  FError := mciSendCommand( FDeviceID, mci_Pause, FFlags, Longint(@GenParm));
  Result := (FError = 0);
end;

function TabcCustomMCI.Resume: Boolean;
var
  GenParm: TMCI_Generic_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;

  FFlags := 0;
  if FUseNotify then begin
    if FNotify then FFlags := mci_Notify;
  end
  else
    FFlags := mci_Notify;
  if FUseWait then begin
    if FWait then FFlags := FFlags or mci_Wait;
  end;
  GenParm.dwCallback := Handle;
  FError := mciSendCommand( FDeviceID, mci_Resume, FFlags, Longint(@GenParm));

  {if error calling resume (resume not supported),  call Play}
  if FError <> 0 then
    Result := Play
  else begin
    if FUseNotify then FUseNotify := False;
    if FUseWait then FUseWait := False;
    Result := True;
  end;
end;

function TabcCustomMCI.Next: Boolean;
var
  SeekParm: TMCI_Seek_Parms;
  TempFlags: Longint;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;

  FFlags := 0;
  if FUseWait then begin
    if FWait then FFlags := mci_Wait;
    FUseWait := False;
  end
  else
    FFlags := mci_Wait;
  if FUseNotify then begin
    if FNotify then FFlags := FFlags or mci_Notify;
    FUseNotify := False;
  end;

  TempFlags := FFlags;
  if TimeFormat = tfTMSF then begin
    if Mode = mpPlaying then begin
      if mci_TMSF_Track(Position) = Tracks then
         StartPos := GetTrackPosition(Tracks)
      else
         StartPos := GetTrackPosition((mci_TMSF_Track(Position))+1);
      Result := Play;
      Exit;
    end
    else begin
      if mci_TMSF_Track(Position) = Tracks then
         SeekParm.dwTo := GetTrackPosition(Tracks)
      else
         SeekParm.dwTo := GetTrackPosition((mci_TMSF_Track(Position))+1);
      FFlags := TempFlags or mci_To;
    end;
  end
  else
    FFlags := TempFlags or mci_Seek_To_End;

  SeekParm.dwCallback := Handle;
  FError := mciSendCommand( FDeviceID, mci_Seek, FFlags, Longint(@SeekParm));
  Result := (FError = 0);
end;

function TabcCustomMCI.Previous: Boolean;
var
  SeekParm: TMCI_Seek_Parms;
  tpos,cpos,TempFlags: Longint;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;

  FFlags := 0;
  if FUseWait then begin
    if FWait then FFlags := mci_Wait;
    FUseWait := False;
  end
  else
    FFlags := mci_Wait;
  if FUseNotify then begin
    if FNotify then FFlags := FFlags or mci_Notify;
    FUseNotify := False;
  end;

  TempFlags := FFlags;
  if TimeFormat = tfTMSF then begin
    cpos := Position;
    tpos := GetTrackPosition(mci_TMSF_Track(Position));
    if Mode = mpPlaying then begin
    	if (mci_TMSF_Track(cpos) <> 1) and
      	(mci_TMSF_Minute(cpos) = mci_TMSF_Minute(tpos)) and
      	(mci_TMSF_Second(cpos) = mci_TMSF_Second(tpos)) then
      	StartPos := GetTrackPosition(mci_TMSF_Track(Position)-1)
    	else
      	StartPos := tpos;
      Result := Play;
      Exit;
	 end
	 else begin
    	if (mci_TMSF_Track(cpos) <> 1) and
      	(mci_TMSF_Minute(cpos) = mci_TMSF_Minute(tpos)) and
      	(mci_TMSF_Second(cpos) = mci_TMSF_Second(tpos)) then
      	SeekParm.dwTo := GetTrackPosition(mci_TMSF_Track(Position)-1)
    	else
      	SeekParm.dwTo := tpos;
    	FFlags := TempFlags or mci_To;
	 end;
  end
  else
    FFlags := TempFlags or mci_Seek_To_Start;

  SeekParm.dwCallback := Handle;
  FError := mciSendCommand( FDeviceID, mci_Seek, FFlags, Longint(@SeekParm));
  Result := (FError = 0);
end;

function TabcCustomMCI.Step: Boolean;
var
  AStepParm: TMCI_Anim_Step_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;

  if FHasVideo then begin
    if FAutoRewind and (Position = Length) then
      if not Rewind then Exit;
    FFlags := 0;
    if FUseWait then begin
      if FWait then FFlags := mci_Wait;
      FUseWait := False;
    end
    else
      FFlags := mci_Wait;
    if FUseNotify then begin
      if FNotify then FFlags := FFlags or mci_Notify;
      FUseNotify := False;
    end;
    FFlags := FFlags or mci_Anim_Step_Frames;
    AStepParm.dwFrames := FFrames;
    AStepParm.dwCallback := Handle;
    FError := mciSendCommand(FDeviceID, mci_Step, FFlags, Longint(@AStepParm));
    Result := (FError <> 0);
  end;
end;

function TabcCustomMCI.Back: Boolean;
var
  AStepParm: TMCI_Anim_Step_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;

  if FHasVideo then begin
    FFlags := 0;
    if FUseWait then begin
      if FWait then FFlags := mci_Wait;
      FUseWait := False;
    end
    else FFlags := mci_Wait;
    if FUseNotify then begin
      if FNotify then FFlags := FFlags or mci_Notify;
      FUseNotify := False;
    end;
    FFlags := FFlags or mci_Anim_Step_Frames or mci_Anim_Step_Reverse;
    AStepParm.dwFrames := FFrames;
    AStepParm.dwCallback := Handle;
    FError := mciSendCommand(FDeviceID, mci_Step, FFlags, Longint(@AStepParm));
    Result := (FError = 0);
  end;
end;

function TabcCustomMCI.Eject: Boolean;
var
  SetParm: TMCI_Set_Parms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;

  if FCanEject then begin
    FFlags := 0;
    if FUseWait then begin
      if FWait then FFlags := mci_Wait;
      FUseWait := False;
    end
    else
      FFlags := mci_Wait;
    if FUseNotify then begin
      if FNotify then FFlags := FFlags or mci_Notify;
      FUseNotify := False;
    end;
    FFlags := FFlags or mci_Set_Door_Open;
    SetParm.dwCallback := Handle;
    FError := mciSendCommand(FDeviceID, mci_Set, FFlags, Longint(@SetParm));
    Result := (FError = 0);
  end;
end;

procedure TabcCustomMCI.SetPosition(Value: Longint);
var
  SeekParm: TMCI_Seek_Parms;
begin
  if not CheckIfOpen then Exit;
  if FRepeatPlay then FRepeatPlay := False;
  FFlags := 0;
  if FUseWait then begin
    if FWait then FFlags := mci_Wait;
    FUseWait := False;
  end
  else
    FFlags := mci_Wait;
  if FUseNotify then begin
    if FNotify then FFlags := FFlags or mci_Notify;
    FUseNotify := False;
  end;
  FFlags := FFlags or mci_To;
  SeekParm.dwCallback := Handle;
  SeekParm.dwTo := Value;
  FError := mciSendCommand( FDeviceID, mci_Seek, FFlags, Longint(@SeekParm));
end;

function TabcCustomMCI.Rewind: Boolean;
var
  SeekParm: TMCI_Seek_Parms;
  RFlags: Longint;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  RFlags := mci_Wait or mci_Seek_To_Start;
  FError := mciSendCommand(FDeviceID, mci_Seek, RFlags, Longint(@SeekParm));
  Result := (FError = 0);
end;

function TabcCustomMCI.GetTrackLength(TrackNum: Integer): Longint;
var
  StatusParm: TMCI_Status_Parms;
begin
  Result := 0;
  if not CheckIfOpen then Exit;
  FFlags := mci_Wait or mci_Status_Item or mci_Track;
  StatusParm.dwItem := mci_Status_Length;
  StatusParm.dwTrack := Longint(TrackNum);
  FError := mciSendCommand(FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := StatusParm.dwReturn;
end;

function TabcCustomMCI.GetTrackPosition(TrackNum: Integer): Longint;
var
  StatusParm: TMCI_Status_Parms;
begin
  FFlags := mci_Wait or mci_Status_Item or mci_Track;
  StatusParm.dwItem := mci_Status_Position;
  StatusParm.dwTrack := Longint(TrackNum);
  FError := mciSendCommand( FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := StatusParm.dwReturn;
end;

function TabcCustomMCI.Save: Boolean;
var
  SaveName: array [0..150] of Char;
  SaveParm: TMCI_SaveParms;
begin
  Result := False;
  if not CheckIfOpen then Exit;
  if FElementName <> '' then begin
    StrPLCopy( SaveName, FElementName, 150);
    SaveParm.lpfilename := SaveName;
    FFlags := 0;
    if FUseWait then begin
      if FWait then FFlags := mci_Wait;
      FUseWait := False;
    end
    else
      FFlags := mci_Wait;
    if FUseNotify then begin
      if FNotify then FFlags := FFlags or mci_Notify;
      FUseNotify := False;
    end;
    SaveParm.dwCallback := Handle;
    FFlags := FFlags or mci_Save_File;
    FError := mciSendCommand(FDeviceID, mci_Save, FFlags, Longint(@SaveParm));
    if FError = 0 then
      Result := True
    else if RaiseExceptions then
      raise EMCIDeviceError.Create(ErrorMessage);
  end
  else if RaiseExceptions then
    raise EMCIDeviceError.Create(Format(SFOpenError, ['']));
end;


{ procedures that set control flags for MCI Commands }

procedure TabcCustomMCI.SetWait( Flag: Boolean );
begin
  if Flag <> FWait then FWait := Flag;
  FUseWait := True;
end;

procedure TabcCustomMCI.SetNotify( Flag: Boolean );
begin
  if Flag <> FNotify then FNotify := Flag;
  FUseNotify := True;
end;

procedure TabcCustomMCI.SetFrom( Value: Longint );
begin
  if Value <> FFrom then FFrom := Value;
  FUseFrom := True;
end;

procedure TabcCustomMCI.SetTo( Value: Longint );
begin
  if Value <> FTo then FTo := Value;
  FUseTo := True;
end;


procedure TabcCustomMCI.SetDeviceType( Value: TMPDeviceTypes );
begin
  if Value <> FDeviceType then FDeviceType := Value;
end;

procedure TabcCustomMCI.SetTimeFormat( Value: TMPTimeFormats );
var
  SetParm: TMCI_Set_Parms;
begin
  begin
    FFlags := mci_Notify or mci_Set_Time_Format;
    SetParm.dwTimeFormat := Longint(Value);
    FError := mciSendCommand( FDeviceID, mci_Set, FFlags, Longint(@SetParm) );
  end;
end;

procedure TabcCustomMCI.SetDisplay(Value: TWinControl);
var
  AWindowParm: TMCI_Anim_Window_Parms;
begin
  if (Value <> nil) and MCIOpened and FHasVideo then begin
    FFlags := mci_Wait or mci_Anim_Window_hWnd;
    AWindowParm.Wnd := Longint(Value.Handle);
    FError := mciSendCommand( FDeviceID, mci_Window, FFlags, Longint(@AWindowParm) );
    if FError <> 0 then
      FDisplay := nil
    else
      FDisplay := Value;
  end
  else
    FDisplay := Value;
end;

procedure TabcCustomMCI.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDisplay) then begin
    if MCIOpened then SetOrigDisplay;
    FDisplay := nil;
  end;
end;

procedure TabcCustomMCI.SetOrigDisplay;
var
  AWindowParm: TMCI_Anim_Window_Parms;
begin
  if MCIOpened and FHasVideo then begin
    FFlags := mci_Wait or mci_Anim_Window_hWnd;
    AWindowParm.Wnd := mci_Anim_Window_Default;
    FError := mciSendCommand( FDeviceID, mci_Window, FFlags, Longint(@AWindowParm) );
  end;
end;

procedure TabcCustomMCI.SetDisplayRect(Value: TRect);
var
  RectParms: TMCI_Anim_Rect_Parms;
  WorkR: TRect;
begin
  if MCIOpened and FHasVideo then begin
    {special case, use default width and height}
    if (Value.Bottom = 0) and (Value.Right = 0) then
      with Value do WorkR := Rect(Left, Top, FDWidth, FDHeight)
    else
      WorkR := Value;
    FFlags := mci_Anim_RECT or mci_Anim_Put_Destination;
    RectParms.rc := WorkR;
    FError := mciSendCommand( FDeviceID, mci_Put, FFlags, Longint(@RectParms));
  end;
end;

{ functions to get device capabilities and status }

function TabcCustomMCI.GetDisplayRect: TRect;
var
  RectParms: TMCI_Anim_Rect_Parms;
begin
  if MCIOpened and FHasVideo then
  begin
    FFlags := mci_Anim_Where_Destination;
    FError := mciSendCommand( FDeviceID, mci_Where, FFlags, Longint(@RectParms) );
    Result := RectParms.rc;
  end;
end;

{ fills in static properties upon opening MCI Device }
procedure TabcCustomMCI.GetDeviceCaps;
var
  DevCapParm: TMCI_GetDevCaps_Parms;
  devType: Longint;
  RectParms: TMCI_Anim_Rect_Parms;
  WorkR: TRect;
begin
  FFlags := mci_Wait or mci_GetDevCaps_Item;

  DevCapParm.dwItem := mci_GetDevCaps_Can_Play;
  mciSendCommand(FDeviceID, mci_GetDevCaps, FFlags,  Longint(@DevCapParm) );
  FCanPlay := Boolean(DevCapParm.dwReturn);
  if FCanPlay then Include(FCapabilities, mpCanPlay);

  DevCapParm.dwItem := mci_GetDevCaps_Can_Record;
  mciSendCommand(FDeviceID, mci_GetDevCaps, FFlags,  Longint(@DevCapParm) );
  FCanRecord := Boolean(DevCapParm.dwReturn);
  if FCanRecord then Include(FCapabilities, mpCanRecord);

  DevCapParm.dwItem := mci_GetDevCaps_Can_Eject;
  mciSendCommand(FDeviceID, mci_GetDevCaps, FFlags,  Longint(@DevCapParm) );
  FCanEject := Boolean(DevCapParm.dwReturn);
  if FCanEject then Include(FCapabilities, mpCanEject);

  DevCapParm.dwItem := mci_GetDevCaps_Has_Video;
  mciSendCommand(FDeviceID, mci_GetDevCaps, FFlags,  Longint(@DevCapParm) );
  FHasVideo := Boolean(DevCapParm.dwReturn);
  if FHasVideo then Include(FCapabilities, mpUsesWindow);

  DevCapParm.dwItem := mci_GetDevCaps_Device_Type;
  mciSendCommand(FDeviceID, mci_GetDevCaps, FFlags,  Longint(@DevCapParm) );
  devType := DevCapParm.dwReturn;
  if (devType = mci_DevType_Animation) or
     (devType = mci_DevType_Digital_Video) or
     (devType = mci_DevType_Overlay) or
     (devType = mci_DevType_VCR) then FCanStep := True;
  if FCanStep then Include(FCapabilities, mpCanStep);

  FFlags := mci_Anim_Where_Source;
  FError := mciSendCommand( FDeviceID, mci_Where, FFlags, Longint(@RectParms) );
  if FError <> 0 then begin
    WorkR := RectParms.rc;
    FDWidth := WorkR.Right - WorkR.Left;
    FDHeight := WorkR.Bottom - WorkR.Top;
  end
  else begin
    FError := 0;
    FDWidth := 0;
    FDHeight := 0;
  end;
end;

function TabcCustomMCI.GetStart: Longint;
var
  StatusParm: TMCI_Status_Parms;
begin
  Result := 0;
  if not CheckIfOpen then Exit;
  FFlags := mci_Wait or mci_Status_Item or mci_Status_Start;
  StatusParm.dwItem := mci_Status_Position;
  FError := mciSendCommand( FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := StatusParm.dwReturn;
end;

function TabcCustomMCI.GetLength: Longint;
var
  StatusParm: TMCI_Status_Parms;
begin
  Result := 0;
  if not CheckIfOpen then Exit;
  FFlags := mci_Wait or mci_Status_Item;
  StatusParm.dwItem := mci_Status_Length;
  FError := mciSendCommand( FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := StatusParm.dwReturn;
end;

function TabcCustomMCI.GetTracks: Longint;
var
  StatusParm: TMCI_Status_Parms;
begin
  Result := 0;
  if not CheckIfOpen then Exit;
  FFlags := mci_Wait or mci_Status_Item;
  StatusParm.dwItem := mci_Status_Number_Of_Tracks;
  FError := mciSendCommand( FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := StatusParm.dwReturn;
end;

function TabcCustomMCI.GetMode: TMPModes;
var
  StatusParm: TMCI_Status_Parms;
begin
  FFlags := mci_Wait or mci_Status_Item;
  StatusParm.dwItem := mci_Status_Mode;
  FError := mciSendCommand( FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := TMPModes(StatusParm.dwReturn - 524); {MCI Mode #s are 524+enum}
end;

function TabcCustomMCI.GetPosition: Longint;
var
  StatusParm: TMCI_Status_Parms;
begin
  Result := 0;
  if not CheckIfOpen then Exit;
  FFlags := mci_Wait or mci_Status_Item;
  StatusParm.dwItem := mci_Status_Position;
  FError := mciSendCommand(FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := StatusParm.dwReturn;
end;

function TabcCustomMCI.GetTimeFormat: TMPTimeFormats;
var
  StatusParm: TMCI_Status_Parms;
begin
  Result := TMPTimeFormats(0);
  if not CheckIfOpen then Exit;
  FFlags := mci_Wait or mci_Status_Item;
  StatusParm.dwItem := mci_Status_Time_Format;
  FError := mciSendCommand( FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  Result := TMPTimeFormats(StatusParm.dwReturn);
end;

function TabcCustomMCI.GetErrorMessage: string;
var
  ErrMsg: array[0..255] of Char;
begin
  if not mciGetErrorString(FError, ErrMsg, 255) then
    Result := SMCIUnknownError
  else
    Result := StrPas(ErrMsg);
end;

{ TabcMCIFilePlayer }

function TabcMCIFilePlayer.New: Boolean; begin Result := inherited New; end;
function TabcMCIFilePlayer.Open: Boolean; begin Result := inherited Open; end;
function TabcMCIFilePlayer.Close: Boolean; begin Result := inherited Close; end;
function TabcMCIFilePlayer.Play: Boolean; begin Result := inherited Play; end;
function TabcMCIFilePlayer.Stop: Boolean; begin Result := inherited Stop; end;
function TabcMCIFilePlayer.PauseResume: Boolean; begin Result := inherited PauseResume; end;
function TabcMCIFilePlayer.Step: Boolean; begin Result := inherited Step; end;
function TabcMCIFilePlayer.Back: Boolean; begin Result := inherited Back; end;
function TabcMCIFilePlayer.Previous: Boolean; begin Result := inherited Previous; end;
function TabcMCIFilePlayer.Next: Boolean; begin Result := inherited Next; end;
function TabcMCIFilePlayer.StartRecording: Boolean; begin Result := inherited StartRecording; end;
function TabcMCIFilePlayer.Eject: Boolean; begin Result := inherited Eject; end;
function TabcMCIFilePlayer.Save: Boolean; begin Result := inherited Save; end;
function TabcMCIFilePlayer.SaveAs(AFileName: TFileName): Boolean; begin Result := inherited SaveAs(AFileName); end;
function TabcMCIFilePlayer.Pause: Boolean; begin Result := inherited Pause; end;
function TabcMCIFilePlayer.Resume: Boolean; begin Result := inherited Resume; end;
function TabcMCIFilePlayer.Rewind: Boolean; begin Result := inherited Rewind; end;

{ TabcMCIStreamPlayer }

function TabcMCIStreamPlayer.New: Boolean; begin Result := inherited New; end;

constructor TabcMCIStreamPlayer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited DeviceType := dtWaveAudio;
  FMCIData := TabcPersistentMemory.Create;
  FMCIData.OnChange := MCIDataChanged;
  FMCIData.OnChanging := MCIDataChanging;
end;

destructor TabcMCIStreamPlayer.Destroy;
begin
  Close;
  MCIData.Free;
  inherited Destroy;
end;

function TabcMCIStreamPlayer.GetDeviceType: TMPDeviceTypes;
begin
  Result := inherited DeviceType;
end;

function TabcMCIStreamPlayer.GetFileName: TFileName;
begin
  Result := inherited FileName;
end;

procedure TabcMCIStreamPlayer.MCIDataChanged(Sender: TObject);
begin
  if not ((csLoading in ComponentState) or
         (csDesigning in ComponentState))
     and AutoOpen then Open;
end;

procedure TabcMCIStreamPlayer.MCIDataChanging(Sender: TObject);
begin
  if not FSaving and (FileName <> '') then Close;
end;

procedure TabcMCIStreamPlayer.SetMCIData(Value: TabcPersistentMemory);
begin
  if Value <> FMCIData then
    FMCIData.Assign(Value);
end;

function TabcMCIStreamPlayer.Open: Boolean;
var
  pTempPath: array[0..255] of char;
  pTempFile: array[0..255] of char;
  TempFile: TFileName;
begin
  Result := False;
  if MCIData.Size = 0 then
    if RaiseExceptions then
      raise EMCIDeviceError.Create(sabcErrNoMCIData)
    else
      Exit
  else begin
    {copy memory stream into temp file and then open file}
    GetTempPath(255, pTempPath);
    GetTempFileName(pTempPath, '', 0, pTempFile);
    TempFile := StrPas(pTempFile);
    inherited FileName := TempFile;
    MCIData.SaveToFile(TempFile);
    Result := inherited Open;
  end;
end;

function TabcMCIStreamPlayer.Close;
begin
  if FDataChanged then Save;
  Result := inherited Close;
  if FileName <> '' then begin
    DeleteFile(FileName);
    inherited FileName := '';
  end;
end;

function TabcMCIStreamPlayer.Play: Boolean; begin Result := inherited Play; end;
function TabcMCIStreamPlayer.Stop: Boolean; begin Result := inherited Stop; end;
function TabcMCIStreamPlayer.PauseResume: Boolean; begin Result := inherited PauseResume; end;
function TabcMCIStreamPlayer.Step: Boolean; begin Result := inherited Step; end;
function TabcMCIStreamPlayer.Back: Boolean; begin Result := inherited Back; end;
function TabcMCIStreamPlayer.Previous: Boolean; begin Result := inherited Previous; end;
function TabcMCIStreamPlayer.Next: Boolean; begin Result := inherited Next; end;

function TabcMCIStreamPlayer.StartRecording: Boolean;
begin
  Result := inherited StartRecording;
  if Result then FDataChanged := True;
end;

function TabcMCIStreamPlayer.Eject: Boolean; begin Result := inherited Eject; end;

function TabcMCIStreamPlayer.Save: Boolean;
var
  pTempPath: array[0..255] of char;
  pTempFile: array[0..255] of char;
  TempFile: TFileName;
begin
  Result := False;
  if FileName = '' then begin
    GetTempPath(255, pTempPath);
    GetTempFileName(pTempPath, '', 0, pTempFile);
    TempFile := StrPas(pTempFile);
    inherited FileName := TempFile;
  end;
  if inherited Save then begin
    {now update stream as well}
    FSaving := True;
    MCIData.LoadFromFile(FileName);
    FDataChanged := False;
    FSaving := False;
    Result := True;
  end;
end;

function TabcMCIStreamPlayer.Pause: Boolean; begin Result := inherited Pause; end;
function TabcMCIStreamPlayer.Resume: Boolean; begin Result := inherited Resume; end;
function TabcMCIStreamPlayer.Rewind: Boolean; begin Result := inherited Rewind; end;

procedure TabcMCIStreamPlayer.SetDeviceType(Value: TMPDeviceTypes);
{ stop user from setting dtAutoSelect since we're using .TMP filename }
begin
  if Value <> dtAutoSelect then
    inherited DeviceType := Value;
end;

end.
