unit abcmopts;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
This unit based on code licensed from Andrew Cher http://members.xoom.com/abcpp/
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Classes, Graphics, Menus, Controls, abcwav;

type

  { Types for TabcMenuOptions }

  TabcMenuAlignment = (ataLeftJustify, ataRightJustify, ataCenter{, ataHebrew});

  TabcMenuAnimation = (amnNone, amnVSlide, amnHSlide, amnSmartt, amnUnfold,
    amnPopup, amnRandom, amnSysMenu, amnToolbar, amnVSlideUp);

  TabcMenuHighlightingStyle = (
    ahsDefault, ahsRectangle, ahsBevelLowered, ahsBevelRaised, ahsShadow,
    ahsUnderline);

  { TabcMenuMargins }

  TabcMenuMargins = class(TPersistent)
  private
    FLeft: Integer;
    FTop: Integer;
    FRight: Integer;
    FBottom: Integer;
    FItemSpace: Integer;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
  published
    property Left: Integer read FLeft write FLeft default 25;
    property Top: Integer read FTop write FTop default 1;
    property Right: Integer read FRight write FRight default 25;
    property Bottom: Integer read FBottom write FBottom default 1;
    property ItemSpace: Integer read FItemSpace write FItemSpace default 0;
  end;

  { TabcMenuColors }

  TabcMenuColors = class(TPersistent)
  private
    FBorder: TColor;
    FMenu: TColor;
    FMenuText: TColor;
    FMenuShadow: TColor;
    FHighlight: TColor;
    FHighlightText: TColor;
    FDisabledText: TColor;
    FDisabledShadow: TColor;
    FLine: TColor;
    FLineShadow: TColor;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    function IsDefaultColors: Boolean;
  published
    property Border: TColor read FBorder write FBorder default clNone;
    property Menu: TColor read FMenu write FMenu default clMenu;
    property MenuText: TColor read FMenuText write FMenuText default clMenuText;
    property MenuShadow: TColor read FMenuShadow write FMenuShadow default clBtnShadow;
    property Highlight: TColor read FHighlight write FHighlight default clHighlight;
    property HighlightText: TColor read FHighlightText write FHighlightText default clHighlightText;
    property DisabledText: TColor read FDisabledText write FDisabledText default clBtnShadow;
    property DisabledShadow: TColor read FDisabledShadow write FDisabledShadow default clBtnHighlight;
    property Line: TColor read FLine write FLine default clBtnShadow;
    property LineShadow: TColor read FLineShadow write FLineShadow default clBtnHighlight;
  end;

  { TabcMenuTitle }

  TabcMenuTitleAlign = (altLeft, altRight);

  TabcMenuTitle = class(TPersistent)
  private
    FAlign: TabcMenuTitleAlign;
    FAlignment: TAlignment;
    FText: String;
    FFont: TFont;
    FColorBegin: TColor;
    FColorEnd: TColor;
    FWidth: Integer;
    FDirection: Boolean;
    FVisible: Boolean;
    FBitmap, FBuffer: TBitmap;
    procedure SetFont(Value: TFont);
    procedure SetBitmap(Value: TBitmap);
    function IsBitmapStored: Boolean;
    function IsFontStored: Boolean;

  public
    constructor Create;
    destructor Destroy; override;
    procedure Paint(Canvas: TCanvas);
    function TextAlign(Width, TextWidth: Integer): Integer;
    function CalcHeight: Integer;
    procedure Assign(Source: TPersistent); override;

  published
    property Align: TabcMenuTitleAlign read FAlign write FAlign default altLeft;
    property Alignment: TAlignment read FAlignment write FAlignment default taLeftJustify;
    property Text: String read FText write FText;
    property Font: TFont read FFont write SetFont stored IsFontStored;
    property ColorBegin: TColor read FColorBegin write FColorBegin default clBlack;
    property ColorEnd: TColor read FColorEnd write FColorEnd default clNavy;
    property Width: Integer read FWidth write FWidth default 50;
    property Direction: Boolean read FDirection write FDirection default True;
    property Visible: Boolean read FVisible write FVisible default False;
    property Bitmap: TBitmap read FBitmap write SetBitmap stored IsBitmapStored;
  end;

  { TabcMenuOptions }

  TabcMenuOptions = class(TPersistent)
  private
    FAlignment: TabcMenuAlignment;
    FBackground: TBitmap;
    FBackTiled: Boolean;
//    FCaption: string;
//    FDraggable: Boolean;
    FColors: TabcMenuColors;
    FCtl3D: Boolean;
    FFitToBackground: Boolean;
    FFitToTitle: Boolean;
    FMinWidth: Integer;
    FItemHeight: Integer;
    FMargins: TabcMenuMargins;
    FSeparatorHeight: Integer;
    FStyle: TabcMenuHighlightingStyle;
    FTitle: TabcMenuTitle;
    FOpenAnimation: TabcMenuAnimation;
    FCloseAnimation: TabcMenuAnimation;
    FTimeout: Integer;
    FDoubleSpeed: Boolean;
    FSoundList: TabcCustomWaveList;
    FSoundPopup: Integer;
    FSoundClose: Integer;
    FSoundCommand: Integer;
    procedure SetBackground(Value: TBitmap);
    procedure SetBackTiled(Value: Boolean);
    procedure SetFitToBackground(Value: Boolean);
    procedure SetColors(Value: TabcMenuColors);
    procedure SetMargins(Value: TabcMenuMargins);
    procedure SetTitle(Value: TabcMenuTitle);
    function IsBackgroundStored: Boolean;
    function IsColorsStored: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Alignment: TabcMenuAlignment read FAlignment write FAlignment default ataLeftJustify;
    property Background: TBitmap read FBackground write SetBackground stored IsBackgroundStored;
    property BackTiled: Boolean read FBackTiled write SetBackTiled default False;
    property Colors: TabcMenuColors read FColors write SetColors stored IsColorsStored;
    property Ctl3D: Boolean read FCtl3D write FCtl3D default True;
//    property Caption: string read FCaption write FCaption;
//    property Draggable: Boolean read FDraggable write FDraggable default False;
    property FitToBackground: Boolean read FFitToBackground write SetFitToBackground default False;
    property FitToTitle: Boolean read FFitToTitle write FFitToTitle default False;
    property MinWidth: Integer read FMinWidth write FMinWidth default 0;
    property ItemHeight: Integer read FItemHeight write FItemHeight default 19;
    property Margins: TabcMenuMargins read FMargins write SetMargins;
    property SeparatorHeight: Integer read FSeparatorHeight write FSeparatorHeight default 11;
    property Style: TabcMenuHighlightingStyle read FStyle write FStyle default ahsDefault;
    property Title: TabcMenuTitle read FTitle write SetTitle;
    property OpenAnimation: TabcMenuAnimation read FOpenAnimation write FOpenAnimation default amnSmartt;
    property CloseAnimation: TabcMenuAnimation read FCloseAnimation write FCloseAnimation default amnNone;
    property Timeout: Integer read FTimeout write FTimeout default 20;
    property DoubleSpeed: Boolean read FDoubleSpeed write FDoubleSpeed default False;
    property SoundList: TabcCustomWaveList read FSoundList write FSoundList;  //NB Owner is responsible for notification checking
    property SoundPopup: Integer read FSoundPopup write FSoundPopup default -1;
    property SoundClose: Integer read FSoundClose write FSoundClose default -1;
    property SoundCommand: Integer read FSoundCommand write FSoundCommand default -1;
  end;

implementation

uses
  SysUtils;

{ TabcMenuMargins }

constructor TabcMenuMargins.Create;
begin
  FLeft := 25;
  FTop := 1;
  FRight := 25;
  FBottom := 1;
end;

procedure TabcMenuMargins.Assign(Source: TPersistent);
begin
  if Source is TabcMenuMargins then
    with TabcMenuMargins(Source) do
    begin
      Self.Left := Left;
      Self.Top := Top;
      Self.Right := Right;
      Self.Bottom := Bottom;
      Self.ItemSpace := ItemSpace;
    end
  else
    inherited;
end;

{ TabcMenuColors }

constructor TabcMenuColors.Create;
begin
  FBorder := clBtnShadow;
  FMenu := clMenu;
  FMenuText := clMenuText;
  FMenuShadow := clBtnShadow;
  FHighlight := clHighlight;
  FHighlightText := clHighlightText;
  FDisabledText := clBtnShadow;
  FDisabledShadow := clBtnHighlight;
  FLine := clBtnShadow;
  FLineShadow := clBtnHighlight;
end;

function TabcMenuColors.IsDefaultColors: Boolean;
begin
  Result :=
    (Border = clBtnShadow) and
    (Menu = clMenu) and
    (MenuText = clMenuText) and
    (MenuShadow = clBtnShadow) and
    (Highlight = clHighlight) and
    (HighlightText = clHighlightText) and
    (DisabledText = clBtnShadow) and
    (DisabledShadow = clBtnHighlight) and
    (Line = clBtnShadow) and
    (LineShadow = clBtnHighlight);
end;

procedure TabcMenuColors.Assign(Source: TPersistent);
begin
  if Source is TabcMenuColors then
    with TabcMenuColors(Source) do
    begin
      Self.Border := Border;
      Self.Menu := Menu;
      Self.MenuText := MenuText;
      Self.Highlight := Highlight;
      Self.HighlightText := HighlightText;
      Self.DisabledText := DisabledText;
      Self.DisabledShadow := DisabledShadow;
      Self.Line := Line;
      Self.LineShadow := LineShadow;
    end
  else
    inherited;
end;

{ TabcMenuTitle }

constructor TabcMenuTitle.Create;
begin
  FAlign := altLeft;
  FColorBegin := clBlack;
  FColorEnd := clNavy;
  FDirection := True;
  FWidth := 50;
  FFont := TFont.Create;
  FFont.Name := 'Arial';
  FFont.Size :=  24;
  FFont.Style := [fsBold];
  FFont.Color := clWhite;
  FBitmap := TBitmap.Create;
end;

destructor TabcMenuTitle.Destroy;
begin
  FFont.Free;
  FBitmap.Free;
  FBuffer.Free;
  inherited;
end;

function TabcMenuTitle.IsBitmapStored: Boolean;
begin
  Result := not FBitmap.Empty;
end;

function TabcMenuTitle.IsFontStored: Boolean;
begin
  Result := (Font.Name <> 'Arial')
    or (Font.Size <> 24)
    or (Font.Style <> [fsBold])
    or (Font.Color <> clWhite)
    or (Font.CharSet <> Default_Charset);
end;

procedure TabcMenuTitle.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TabcMenuTitle.SetBitmap(Value: TBitmap);
begin
  FBitmap.Assign(Value);
end;

function TabcMenuTitle.TextAlign(Width, TextWidth: Integer): Integer;
begin
  Result := 0;
  case FAlignment of
    taLeftJustify:
      Result := Width - 10;
    taCenter:
      Result := (Width + TextWidth ) div 2 + 10;
    taRightJustify:
      Result := TextWidth + 30;
  end;
end;

function TabcMenuTitle.CalcHeight: Integer;
begin
  if FText = '' then
    Result := 0
  else
  begin
    if FBuffer = nil then
      FBuffer := TBitmap.Create;
    with FBuffer.Canvas do
    begin
      Font := Self.Font;
      Result := TextWidth(FText) * 11 div 10;
    end;
  end;
  if Result < FBitmap.Height then
    Result := FBitmap.Height;
end;

procedure TabcMenuTitle.Paint(Canvas: TCanvas);
var
  R: TRect;
  lf: TLogFont;
  hfnt, holdfnt: HFont;
  X, Y, W, H, C1, C2, R1, G1, B1: Integer;
  DR, DG, DB, DH: Real;

begin
  if not FVisible then Exit;

  with Canvas.ClipRect do
    case FAlign of
      altLeft:
        R := Rect(2, 2, Width + 2, Bottom - 2);
      altRight:
        R := Rect(Right - Width - 2, 2, Right - 2, Bottom - 2);
    end;

  if FBuffer = nil then
    FBuffer := TBitmap.Create;
  FBuffer.Width := R.Right - R.Left;
  FBuffer.Height := R.Bottom - R.Top;

  with FBuffer.Canvas do
  begin
    // fill bitmap with FColorBegin and FColorEnd
    Brush.Style := bsSolid;

    if FColorBegin <> FColorEnd then
    begin
      // gradient!
      DH := FBuffer.Height/236;
      C1 := ColorToRgb(FColorEnd);
      C2 := ColorToRgb(FColorBegin);
      R1 := GetRValue(DWORD(C1));
      G1 := GetGValue(DWORD(C1));
      B1 := GetBValue(DWORD(C1));
      DR := (GetRValue(DWORD(C2)) - R1)/236;
      DG := (GetGValue(DWORD(C2)) - G1)/236;
      DB := (GetBValue(DWORD(C2)) - B1)/236;

      for X := 0 to 235 do
      begin
        Brush.Color := Rgb(R1 + Round(DR*X), G1 + Round(DG*X), B1 + Round(DB*X));
        FillRect(Rect(0, Round(X*DH), FBuffer.Width, Round((X+1)*DH)));
      end;
    end
    else begin
      // solid color
      Brush.Color := FColorBegin;
      FillRect(ClipRect);
    end;

    if not FBitmap.Empty then begin
      // draw bitmap
      X := (FBuffer.Width - FBitmap.Width) div 2;
      Y := 0;

      case FAlignment of
        taLeftJustify:
          Y := FBuffer.Height - FBitmap.Height;
        taRightJustify:
          Y := 0;
        taCenter:
          Y := (FBuffer.Height - FBitmap.Height) div 2;
      end;

      BitBlt(FBuffer.Canvas.Handle, X, Y, FBitmap.Width, FBitmap.Height,
        FBitmap.Canvas.Handle, 0, 0, SrcCopy);
    end;

    if FText <> '' then
    begin
      // calculate text bounds
      Font := Self.Font;
      W := TextWidth(FText);
      H := TextHeight(FText);

      // initialize TLogFont structure
      ZeroMemory(@lf, SizeOf(lf));
      StrPCopy(lf.lfFaceName, Font.Name);
      lf.lfCharSet := Font.Charset;
      lf.lfHeight := Font.Height;

      if fsBold in Font.Style then
        lf.lfWeight := fw_Bold
      else
        lf.lfWeight := fw_Normal;

      lf.lfItalic := Integer(fsItalic in Font.Style);
      lf.lfUnderline := Integer(fsUnderline in Font.Style);
      lf.lfStrikeOut := Integer(fsStrikeout in Font.Style);

      if FDirection then
      begin
        lf.lfEscapement := 900;
        X := (FBuffer.Width - H) div 2 - 2;
        Y := TextAlign(FBuffer.Height, W);
      end
      else
      begin
        lf.lfEscapement := 2700;
        X := (FBuffer.Width + H) div 2 + 2;
        Y := FBuffer.Height - TextAlign(FBuffer.Height, W);
      end;

      hfnt := CreateFontIndirect(lf);
      holdfnt := SelectObject(Handle, hfnt);

      SetTextColor(Handle, ColorToRgb(Self.Font.Color));
      SetBkMode(Handle, Transparent);
      TextOut(X, Y, FText);

      SelectObject(Handle, holdfnt);
      DeleteObject(hfnt);
    end;
  end;

  BitBlt(Canvas.Handle, R.Left, R.Top, FBuffer.Width, FBuffer.Height,
    FBuffer.Canvas.Handle, 0, 0, SrcCopy);
end;

procedure TabcMenuTitle.Assign(Source: TPersistent);
begin
  if Source is TabcMenuTitle then
    with TabcMenuTitle(Source) do
    begin
      Self.Align := Align;
      Self.Alignment := Alignment;
      Self.Text := Text;
      Self.ColorBegin := ColorBegin;
      Self.ColorEnd := ColorEnd;
      Self.Width := Width;
      Self.Direction := Direction;
      Self.Visible := Visible;
      Self.Font.Assign(Font);
      Self.Bitmap.Assign(Bitmap);
    end
  else
    inherited;
end;


{ TabcMenuOptions }

constructor TabcMenuOptions.Create;
begin
  FItemHeight := 19;
  FSeparatorHeight := 11;
  FMargins := TabcMenuMargins.Create;
  FCtl3D := True;
  FTitle := TabcMenuTitle.Create;
  FColors := TabcMenuColors.Create;
  FBackground := TBitmap.Create;
  FTimeout := 20;
  FDoubleSpeed := False;
  FOpenAnimation := amnSmartt;
  FCloseAnimation := amnNone;
  FSoundPopup := -1;
  FSoundClose := -1;
  FSoundCommand := -1;
end;

destructor TabcMenuOptions.Destroy;
begin
  FColors.Free;
  FTitle.Free;
  FBackground.Free;
  FMargins.Free;
  inherited;
end;

procedure TabcMenuOptions.SetBackground(Value: TBitmap);
begin
  FBackground.Assign(Value);
end;

procedure TabcMenuOptions.SetBackTiled(Value: Boolean);
begin
  if Value <> FBackTiled then
  begin
    FBackTiled := Value;
    if Value then FFitToBackground := False;
  end;
end;

procedure TabcMenuOptions.SetFitToBackground(Value: Boolean);
begin
  if Value <> FFitToBackground then
  begin
    FFitToBackground := Value;
    if Value then FBackTiled := False;
  end;
end;

procedure TabcMenuOptions.SetTitle(Value: TabcMenuTitle);
begin
  FTitle.Assign(Value);
end;

procedure TabcMenuOptions.SetColors(Value: TabcMenuColors);
begin
  FColors.Assign(Value);
end;

procedure TabcMenuOptions.SetMargins(Value: TabcMenuMargins);
begin
  FMargins.Assign(Value);
end;

function TabcMenuOptions.IsBackgroundStored: Boolean;
begin
  Result := not FBackground.Empty;
end;

function TabcMenuOptions.IsColorsStored: Boolean;
begin
  Result := not Colors.IsDefaultColors;
end;

procedure TabcMenuOptions.Assign(Source: TPersistent);
begin
  if Source is TabcMenuOptions then
    with TabcMenuOptions(Source) do
    begin
      Self.ItemHeight := ItemHeight;
      Self.SeparatorHeight := SeparatorHeight;
      Self.Alignment := Alignment;
      Self.Style := Style;
      Self.Ctl3D := Ctl3D;
//      Self.Draggable := Draggable;
//      Self.Caption := Caption;
      Self.FitToBackground := FitToBackground;
      Self.FitToTitle := FitToTitle;
      Self.MinWidth := MinWidth;
      Self.Margins.Assign(Margins);
      Self.Title.Assign(Title);
      Self.Background.Assign(Background);
      Self.BackTiled := BackTiled;
      Self.Colors.Assign(Colors);
      Self.Timeout := Timeout;
      Self.DoubleSpeed := DoubleSpeed;
      Self.OpenAnimation := OpenAnimation;
      Self.CloseAnimation := CloseAnimation;
      Self.SoundList := SoundList;
      Self.SoundPopup := SoundPopup;
      Self.SoundClose := SoundClose;
      Self.SoundCommand := SoundCommand;
    end
  else
    inherited;
end;

end.
