unit abcnp;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)
{$I abcdefs.inc}

{$C PRELOAD}

interface

uses
  Windows, Classes, SysUtils, TypInfo, Graphics, ExtCtrls, DB,
  DBGrids, Forms, Messages, Controls, Menus,
  abchgrid, abcforms, abcutilg, abcpanel, abctlbr, abcapp
{$IFDEF INFOPOWER}
  , wwdbigrd, wwdbgrid
{$ENDIF}
  ;

{
definitions for message procedures:

    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDCheckBrowseMode(var Message: TabcCDCheckBrowseMode); message ABC_CD_CHECKBROWSEMODE;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
    procedure abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled); message ABC_CD_DATASETSCROLLED;
    procedure abcCDFocusControl(var Message: TabcCDFocusControl); message ABC_CD_FOCUSCONTROL;
    procedure abcCDEditingChanged(var Message: TabcCDEditingChanged); message ABC_CD_EDITINGCHANGED;
    procedure abcCDLayoutChanged(var Message: TabcCDLayoutChanged); message ABC_CD_LAYOUTCHANGED;
    procedure abcCDRecordChanged(var Message: TabcCDRecordChanged); message ABC_CD_RECORDCHANGED;
    procedure abcCDUpdateData(var Message: TabcCDUpdateData); message ABC_CD_UPDATEDATA;

    procedure abcCDActiveFieldChanged(var Message: TabcCDActiveFieldChanged); message ABC_CD_ACTIVEFIELDCHANGED;

}

{ Control DataLink Events }

const
  ABC_CD_BASE                     = $0FF0;
  ABC_CD_ACTIVECHANGED            = ABC_CD_BASE + 0;
  ABC_CD_CHECKBROWSEMODE          = ABC_CD_BASE + 1;
  ABC_CD_DATASETCHANGED           = ABC_CD_BASE + 2;
  ABC_CD_DATASETSCROLLED          = ABC_CD_BASE + 3;
  ABC_CD_FOCUSCONTROL             = ABC_CD_BASE + 4;
  ABC_CD_EDITINGCHANGED           = ABC_CD_BASE + 5;
  ABC_CD_LAYOUTCHANGED            = ABC_CD_BASE + 6;
  ABC_CD_RECORDCHANGED            = ABC_CD_BASE + 7;
  ABC_CD_UPDATEDATA               = ABC_CD_BASE + 8;
  ABC_CD_ACTIVEFIELDCHANGED       = ABC_CD_BASE + 9;

type

  { Control DataLink Message Records }

  TabcCDBaseParams = record
    Msg: Cardinal;
    Unused: Integer;
    DataSet: TDataSet;
    Result: Longint;
  end;

  TabcCDActiveChanged = record
    Msg: Cardinal;
    LinkActive: BOOL;
    DataSet: TDataSet;
    Result: Longint;
  end;

  TabcCDCheckBrowseMode = TabcCDBaseParams;

  TabcCDDataSetChanged = TabcCDBaseParams;

  TabcCDDataSetScrolled = record
    Msg: Cardinal;
    Distance: Integer;
    DataSet: TDataSet;
    Result: Longint;
  end;

  TabcCDFocusControl = record
    Msg: Cardinal;
    Unused: Integer;
    FieldRef: TFieldRef;
    Result: Longint;
  end;

  TabcCDEditingChanged = TabcCDBaseParams;

  TabcCDLayoutChanged = TabcCDBaseParams;

  TabcCDRecordChanged = record
    Msg: Cardinal;
    Unused: Integer;
    Field: TField;
    Result: Longint;
  end;

  TabcCDUpdateData = TabcCDBaseParams;

  TabcCDActiveFieldChanged = TabcCDRecordChanged;

  TabcnpFieldEvent = procedure (Sender: TObject; Field: TField) of object;


  { Exception Class used in this module }

  EabcNavPanelException = class(Exception);

  { TabcCustomNavPanel }

  TabcNavPanelDataLink = class;

  TabcCustomNavPanel = class(TabcCustomEdgePanel)
  private
    FAutoDataSource: Boolean;
    FAutoField: Boolean;
    FDataLink: TabcNavPanelDataLink;
    FField: TField;
    FRegisterForm: Boolean;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure SetAutoDataSource(Value: Boolean);
    procedure SetAutoField(Value: Boolean);
    procedure SetRegisterForm(Value: Boolean);
    procedure RegisterOwnerForm(AddForm: Boolean);
    function IsDatasourceStored: Boolean;
  protected
    procedure ActiveChanged; virtual;
    procedure CheckBrowseMode; virtual;
    procedure DataSetChanged; virtual;
    procedure DataSetScrolled(ADistance: Integer); virtual;
    procedure FocusControl(AField: TFieldRef); virtual;
    procedure EditingChanged; virtual;
    procedure FieldChanged; virtual;
    function GetDataSource: TDataSource; virtual;
    procedure LayoutChanged; virtual;
    procedure Loaded; override;
    procedure RecordChanged(AField: TField); virtual;
    procedure SetDataSource(Value: TDataSource); virtual;
    procedure SetField(Value: TField); virtual;
    procedure UpdateData; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    property AutoField: Boolean read FAutoField write SetAutoField default False;
    property AutoDataSource: Boolean read FAutoDataSource write SetAutoDataSource default False;
    property RegisterForm: Boolean read FRegisterForm write SetRegisterForm default True;
    property Align default alTop;
    property BevelEdges default [beBottom];
    property BevelInner default bvRaised;
    property BevelOuter default bvLowered;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property DataSource: TDataSource read GetDataSource write SetDataSource stored IsDatasourceStored;
    property Field: TField read FField write SetField;
  end;

  { TabcCustomNavToolbar }

  TabcNavToolbarDataLink = class;

  TabcCustomNavToolbar = class(TabcToolbar)
  private
    FAutoDataSource: Boolean;
    FAutoField: Boolean;
    FDataLink: TabcNavToolbarDataLink;
    FField: TField;
    FRegisterForm: Boolean;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure SetAutoDataSource(Value: Boolean);
    procedure SetAutoField(Value: Boolean);
    procedure SetRegisterForm(Value: Boolean);
    procedure RegisterOwnerForm(AddForm: Boolean);
    function IsDatasourceStored: Boolean;
  protected
    procedure ActiveChanged; virtual;
    procedure CheckBrowseMode; virtual;
    procedure DataSetChanged; virtual;
    procedure DataSetScrolled(ADistance: Integer); virtual;
    procedure FocusControl(AField: TFieldRef); virtual;
    procedure EditingChanged; virtual;
    procedure FieldChanged; virtual;
    function GetDataSource: TDataSource; virtual;
    procedure LayoutChanged; virtual;
    procedure Loaded; override;
    procedure RecordChanged(AField: TField); virtual;
    procedure SetDataSource(Value: TDataSource); virtual;
    procedure SetField(Value: TField); virtual;
    procedure UpdateData; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    property AutoField: Boolean read FAutoField write SetAutoField default False;
    property AutoDataSource: Boolean read FAutoDataSource write SetAutoDataSource default False;
    property RegisterForm: Boolean read FRegisterForm write SetRegisterForm default True;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property DataSource: TDataSource read GetDataSource write SetDataSource stored IsDatasourceStored;
    property Field: TField read FField write SetField;
  end;

  { Popup Form Property Types }

  TabcnpAutoForm = class;

  TabcnpAfterFormCreateEvent = procedure (Sender: TabcnpAutoForm) of object;
  TabcnpAfterShowModalEvent = procedure (Sender: TForm; ModalResult: Integer) of object;
  TabcnpModalFormAction = (mfaNone, mfaEdit, mfaAppend, mfaInsert);

  TabcnpDataView = (npvGrid, npvHGrid, npvRecord, npvSplit);

  { TabcNavPanel }

  TabcNavPanel = class(TabcCustomNavPanel)
  public
    function ShowDataSource(AOwner: TComponent; ADataSource: TDataSource;
      DataView: TabcnpDataView; APopupFormStyle: TabcPopupFormStyle;
      AReadOnly: Boolean; AHelpContext: THelpContext;
      AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm; virtual;
    function ShowModalDataSource(AOwner: TComponent; ADataSource: TDataSource;
      AReadOnly: Boolean; AHelpContext: THelpContext;
      AModalAction: TabcnpModalFormAction; AModalAutoPost: Boolean;
      AfterFormCreate: TabcnpAfterFormCreateEvent; AfterShowModal: TabcnpAfterShowModalEvent): Integer; virtual;
  published
    property Align;
    {property Alignment; NOT published}
    property AutoDataSource;
    property AutoField;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property DataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    {property Caption; NOT published}
    property Color;
    property Ctl3D;
    property Font;
    {property Locked; NOT published}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property RegisterForm;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
  end;

  { TabcNavToolbar }

  TabcNavToolbar = class(TabcCustomNavToolbar)
  public
    function ShowDataSource(AOwner: TComponent; ADataSource: TDataSource;
      DataView: TabcnpDataView; APopupFormStyle: TabcPopupFormStyle;
      AReadOnly: Boolean; AHelpContext: THelpContext;
      AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm; virtual;
    function ShowModalDataSource(AOwner: TComponent; ADataSource: TDataSource;
      AReadOnly: Boolean; AHelpContext: THelpContext;
      AModalAction: TabcnpModalFormAction; AModalAutoPost: Boolean;
      AfterFormCreate: TabcnpAfterFormCreateEvent; AfterShowModal: TabcnpAfterShowModalEvent): Integer; virtual;
  published
    property AutoDataSource;
    property AutoField;
    property DataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Caption;
    property Color;
    property Ctl3D;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property RegisterForm;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TabcNavPanelNotifier - global instance }

  TabcNavPanelNotifier = class(TComponent)
    procedure GridHookOnColEnter(Sender: TObject);
  private
    FActiveControl: TWinControl;
    FActiveDataSource: TDataSource;
    FActiveField: TField;
    FActiveForm: TForm;
    FDetectFormList: TList;
    FIgnoreNextFocus: Boolean;
    FNotifyDataSourceList: TList;
    FNotifyFieldList: TList;
    FOnActiveControlChange: TNotifyEvent;
    FOnActiveFormChange: TNotifyEvent;
    FSaveGrid: TWinControl;
    FSaveGridOCEMethod: TMethod;
    procedure AddGridHook;
    function GetSaveGridSelectedField: TField;
    procedure NotifyDataSourceChange;
    procedure NotifyFieldChange;
    procedure RemoveGridHook;
    procedure ScreenActiveControlChange(Sender: TObject);
    procedure ScreenActiveFormChange(Sender: TObject);
    procedure UpdateActiveControl;
    procedure AddDataSourceNotification(NavPanel: TabcCustomNavPanel);
    procedure AddFieldNotification(NavPanel: TabcCustomNavPanel);
    procedure RemoveDataSourceNotification(NavPanel: TabcCustomNavPanel);
    procedure RemoveFieldNotification(NavPanel: TabcCustomNavPanel);
    procedure AddDataSourceNotificationTB(NavToolbar: TabcCustomNavToolbar);
    procedure AddFieldNotificationTB(NavToolbar: TabcCustomNavToolbar);
    procedure RemoveDataSourceNotificationTB(NavToolbar: TabcCustomNavToolbar);
    procedure RemoveFieldNotificationTB(NavToolbar: TabcCustomNavToolbar);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure AddForm(AForm: TForm);
    function FocusActiveControl: Boolean;
    procedure IgnoreNextFocus;
    procedure RemoveForm(AForm: TForm);

    procedure UpdateActiveDataSource;

    property ActiveControl: TWinControl read FActiveControl;
    property ActiveDataSource: TDataSource read FActiveDataSource;
    property ActiveField: TField read FActiveField;
    property ActiveForm: TForm read FActiveForm;
    property OnActiveControlChange: TNotifyEvent read FOnActiveControlChange write FOnActiveControlChange;
    property OnActiveFormChange: TNotifyEvent read FOnActiveFormChange write FOnActiveFormChange;
  end;

  { TabcNavFormLink }

  TabcNavFormLink = class(TComponent)
  private
    procedure RegisterOwnerForm(AddForm: Boolean);
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TabcnpAutoForm }

  TabcnpAutoFormCaptionStyle = (afcDataSetState, afcDataSet, afcNoChange);

  TabcnpAutoFormClass = class of TabcnpAutoForm;

  TabcnpAutoForm = class(TabcnpApplicationForm)
  private
    FDataSetCaption: string;
    FDataSource: TDataSource;
    FNavPanel: TabcNavPanel;
    FNavToolbar: TabcNavToolbar;
    FModalAutoPost: Boolean;
    FPopupStyle: TabcPopupFormStyle;
    FReadOnly: Boolean;
    FSaveDSStateChange: TNotifyEvent;
    FCaptionStyle: TabcnpAutoFormCaptionStyle;
    procedure DoDSStateChange(Sender: TObject);
    procedure SetCaptionToDSState;
  protected
    procedure OnDataSourceStateChange(Sender: TObject); virtual;
    procedure ValidateDataSource; virtual;
    property CaptionStyle: TabcnpAutoFormCaptionStyle read FCaptionStyle write FCaptionStyle;
    property DataSetCaption: string read FDataSetCaption write FDataSetCaption;
    property DataSource: TDataSource read FDataSource;
    property NavPanel: TabcNavPanel read FNavPanel;
    property NavToolbar: TabcNavToolbar read FNavToolbar;
    property ModalAutoPost: Boolean read FModalAutoPost;
    property PopupStyle: TabcPopupFormStyle read FPopupStyle;
    property ReadOnly: Boolean read FReadOnly write FReadOnly;
  public
    constructor CreateAutoForm(AOwner: TComponent; ANavPanel: TabcNavPanel;
      ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
      AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext;
      AfterFormCreate: TabcnpAfterFormCreateEvent); virtual;
    constructor CreateAutoFormTB(AOwner: TComponent; ANavToolbar: TabcNavToolbar;
      ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
      AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext;
      AfterFormCreate: TabcnpAfterFormCreateEvent); virtual;
    function ExecModal(AfterShowModal: TabcnpAfterShowModalEvent): Integer; virtual;
    destructor Destroy; override;
  end;

  { TabcNavPanelDataLink }

  TabcNavPanelDataLink = class(TDataLink)
  private
    FOwner: TabcCustomNavPanel;
  protected
    procedure ActiveChanged; override;
    procedure CheckBrowseMode; override;
    procedure DataSetChanged; override;
    procedure DataSetScrolled(Distance: Integer); override;
    procedure FocusControl(Field: TFieldRef); override;
    procedure EditingChanged; override;
    procedure LayoutChanged; override;
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
  public
    constructor Create(AOwner: TabcCustomNavPanel);
    destructor Destroy; override;
  end;

  { TabcNavToolbarDataLink }

  TabcNavToolbarDataLink = class(TDataLink)
  private
    FOwner: TabcCustomNavToolbar;
  protected
    procedure ActiveChanged; override;
    procedure CheckBrowseMode; override;
    procedure DataSetChanged; override;
    procedure DataSetScrolled(Distance: Integer); override;
    procedure FocusControl(Field: TFieldRef); override;
    procedure EditingChanged; override;
    procedure LayoutChanged; override;
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
  public
    constructor Create(AOwner: TabcCustomNavToolbar);
    destructor Destroy; override;
  end;

function NavPanelNotifier: TabcNavPanelNotifier;

implementation

uses abcconst, abcutilf, abcutild,
     abcfnpdg, abcfnpds, abcfnphg, abcfnpsg, abcnpfac, abcnpcon;

var
  GNavPanelNotifier: TabcNavPanelNotifier;

function NavPanelNotifier: TabcNavPanelNotifier;
begin
  if GNavPanelNotifier = nil then
    GNavPanelNotifier := TabcNavPanelNotifier.Create(nil);
  Result := GNavPanelNotifier;
end;

function GetAutoFormClassOfView(DataView: TabcnpDataView): TabcnpAutoFormClass;
begin
  case DataView of
    npvGrid:
      Result := TabcnfDataSourceEditGrid;
    npvHGrid:
      Result := TabcnfDataSourceEditHGrid;
    npvRecord:
      Result := TabcnfDataSourceEditRec;
    npvSplit:
      Result := TabcnfDataSourceEditSplit;
  else
    Result := TabcnfDataSourceEditGrid;
  end;
end;

{ TabcCustomNavPanel }

constructor TabcCustomNavPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcNavPanelDataLink.Create(Self);
  Align := alTop;
  BevelEdges := [beBottom];
  BevelInner := bvRaised;
  BevelOuter := bvLowered;
  FRegisterForm := True;
end;

destructor TabcCustomNavPanel.Destroy;
begin
  if RegisterForm then
    RegisterOwnerForm(False);
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TabcCustomNavPanel.ActiveChanged;
var
  Message: TabcCDActiveChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do begin
    Msg := ABC_CD_ACTIVECHANGED;
    LinkActive := FDataLink.Active;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.FieldChanged;
var
  FieldMsg: TabcCDActiveFieldChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with FieldMsg do
  begin
    Msg := ABC_CD_ACTIVEFIELDCHANGED;
    Unused := 0;
    Field := FField;
    Result := 0;
  end;
  Broadcast(FieldMsg);
end;

procedure TabcCustomNavPanel.CheckBrowseMode;
var
  Message: TabcCDCheckBrowseMode;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_CHECKBROWSEMODE;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csLoading in ComponentState) then
    ActiveChanged;
end;

procedure TabcCustomNavPanel.DataSetChanged;
var
  Message: TabcCDDataSetChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_DATASETCHANGED;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.DataSetScrolled(ADistance: Integer);
var
  Message: TabcCDDataSetScrolled;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_DATASETSCROLLED;
    Distance := ADistance;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.EditingChanged;
var
  Message: TabcCDEditingChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_EDITINGCHANGED;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.FocusControl(AField: TFieldRef);
var
  Message: TabcCDFocusControl;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_FOCUSCONTROL;
    Unused := 0;
    FieldRef := AField;
    Result := 0;
  end;
  Broadcast(Message);
end;

function TabcCustomNavPanel.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcCustomNavPanel.LayoutChanged;
var
  Message: TabcCDLayoutChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_LAYOUTCHANGED;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.Loaded;
begin
  inherited Loaded;
  if RegisterForm then
    RegisterOwnerForm(True);
  ActiveChanged;
end;

procedure TabcCustomNavPanel.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then
    DataSource := nil;
end;

procedure TabcCustomNavPanel.RecordChanged(AField: TField);
var
  Message: TabcCDRecordChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_RECORDCHANGED;
    Unused := 0;
    Field := AField;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.SetAutoDataSource(Value: Boolean);
begin
  if Value <> FAutoDataSource then
  begin
    FAutoDataSource := Value;
    if not (csDesigning in ComponentState) then
      if Value then
        NavPanelNotifier.AddDataSourceNotification(Self)
      else
        NavPanelNotifier.RemoveDataSourceNotification(Self)
    else if Value then
      Datasource := nil;
  end;
end;

procedure TabcCustomNavPanel.SetAutoField(Value: Boolean);
begin
  if Value <> FAutoField then
  begin
    FAutoField := Value;
    if not (csDesigning in ComponentState) then
      if Value then
        NavPanelNotifier.AddFieldNotification(Self)
      else
        NavPanelNotifier.RemoveFieldNotification(Self);
  end;
end;

procedure TabcCustomNavPanel.SetDataSource(Value: TDataSource);
begin
  if not (csDestroying in ComponentState) and (Value <> DataSource) then
  begin
    FDataLink.DataSource := Value;
    if (csDesigning in ComponentState) and (Value <> nil) then
      AutoDataSource := False;
  end;
end;

function TabcCustomNavPanel.IsDatasourceStored: Boolean;
begin
  Result := not AutoDataSource;
end;

procedure TabcCustomNavPanel.SetField(Value: TField);
begin
  if Value <> FField then
  begin
    FField := Value;
    FieldChanged;
  end;
end;

procedure TabcCustomNavPanel.UpdateData;
var
  Message: TabcCDUpdateData;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_UPDATEDATA;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavPanel.SetRegisterForm(Value: Boolean);
begin
  if Value <> FRegisterForm then
  begin
    FRegisterForm := Value;
    if [csLoading, csDesigning] * ComponentState = [] then
      RegisterOwnerForm(Value);
  end;
end;

procedure TabcCustomNavPanel.RegisterOwnerForm(AddForm: Boolean);
var
  RegForm: TForm;
begin
  if (not (csDesigning in ComponentState)) then
  begin
    if (Owner is TForm) then
      RegForm := TForm(Owner)
{$IFDEF DELPHI5}
    else if (Owner <> nil) and (Owner.Owner is TForm) then  //for TFrame
      RegForm := TForm(Owner.Owner)
{$ENDIF}
    else
      RegForm := nil;
    if RegForm <> nil then
      if AddForm then
        NavPanelNotifier.AddForm(RegForm)
      else
        NavPanelNotifier.RemoveForm(RegForm);
  end;
end;

{ TabcCustomNavToolbar }

constructor TabcCustomNavToolbar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TabcNavToolbarDataLink.Create(Self);
  FRegisterForm := True;
end;

destructor TabcCustomNavToolbar.Destroy;
begin
  if RegisterForm then
    RegisterOwnerForm(False);
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TabcCustomNavToolbar.ActiveChanged;
var
  Message: TabcCDActiveChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_ACTIVECHANGED;
    LinkActive := FDataLink.Active;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.FieldChanged;
var
  FieldMsg: TabcCDActiveFieldChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with FieldMsg do
  begin
    Msg := ABC_CD_ACTIVEFIELDCHANGED;
    Unused := 0;
    Field := FField;
    Result := 0;
  end;
  Broadcast(FieldMsg);
end;

procedure TabcCustomNavToolbar.CheckBrowseMode;
var
  Message: TabcCDCheckBrowseMode;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_CHECKBROWSEMODE;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csLoading in ComponentState) then
    ActiveChanged;
end;

procedure TabcCustomNavToolbar.DataSetChanged;
var
  Message: TabcCDDataSetChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_DATASETCHANGED;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.DataSetScrolled(ADistance: Integer);
var
  Message: TabcCDDataSetScrolled;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_DATASETSCROLLED;
    Distance := ADistance;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.EditingChanged;
var
  Message: TabcCDEditingChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_EDITINGCHANGED;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.FocusControl(AField: TFieldRef);
var
  Message: TabcCDFocusControl;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_FOCUSCONTROL;
    Unused := 0;
    FieldRef := AField;
    Result := 0;
  end;
  Broadcast(Message);
end;

function TabcCustomNavToolbar.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TabcCustomNavToolbar.LayoutChanged;
var
  Message: TabcCDLayoutChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_LAYOUTCHANGED;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.Loaded;
begin
  inherited Loaded;
  if RegisterForm then
    RegisterOwnerForm(True);
  ActiveChanged;
end;

procedure TabcCustomNavToolbar.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then
    DataSource := nil;
end;

procedure TabcCustomNavToolbar.RecordChanged(AField: TField);
var
  Message: TabcCDRecordChanged;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_RECORDCHANGED;
    Unused := 0;
    Field := AField;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.SetAutoDataSource(Value: Boolean);
begin
  if Value <> FAutoDataSource then
  begin
    FAutoDataSource := Value;
    if not (csDesigning in ComponentState) then
      if Value then
        NavPanelNotifier.AddDataSourceNotificationTB(Self)
      else
        NavPanelNotifier.RemoveDataSourceNotificationTB(Self)
    else if Value then
      Datasource := nil;
  end;
end;

procedure TabcCustomNavToolbar.SetAutoField(Value: Boolean);
begin
  if Value <> FAutoField then
  begin
    FAutoField := Value;
    if not (csDesigning in ComponentState) then
      if Value then
        NavPanelNotifier.AddFieldNotificationTB(Self)
      else
        NavPanelNotifier.RemoveFieldNotificationTB(Self);
  end;
end;

procedure TabcCustomNavToolbar.SetDataSource(Value: TDataSource);
begin
  if not (csDestroying in ComponentState) and (Value <> DataSource) then
  begin
    FDataLink.DataSource := Value;
    if (csDesigning in ComponentState) and (Value <> nil) then
      AutoDataSource := False;
  end;
end;

function TabcCustomNavToolbar.IsDatasourceStored: Boolean;
begin
  Result := not AutoDataSource;
end;

procedure TabcCustomNavToolbar.SetField(Value: TField);
begin
  if Value <> FField then
  begin
    FField := Value;
    FieldChanged;
  end;
end;

procedure TabcCustomNavToolbar.UpdateData;
var
  Message: TabcCDUpdateData;
begin
  if [csDestroying, csDesigning] * ComponentState <> [] then Exit;
  with Message do
  begin
    Msg := ABC_CD_UPDATEDATA;
    Unused := 0;
    DataSet := FDataLink.DataSet;
    Result := 0;
  end;
  Broadcast(Message);
end;

procedure TabcCustomNavToolbar.SetRegisterForm(Value: Boolean);
begin
  if Value <> FRegisterForm then
  begin
    FRegisterForm := Value;
    if [csLoading, csDesigning] * ComponentState = [] then
      RegisterOwnerForm(Value);
  end;
end;

procedure TabcCustomNavToolbar.RegisterOwnerForm(AddForm: Boolean);
var
  RegForm: TForm;
begin
  if (not (csDesigning in ComponentState)) then
  begin
    if (Owner is TForm) then
      RegForm := TForm(Owner)
{$IFDEF DELPHI5}
    else if (Owner <> nil) and (Owner.Owner is TForm) then  //for TFrame
      RegForm := TForm(Owner.Owner)
{$ENDIF}      
    else
      RegForm := nil;
    if RegForm <> nil then
      if AddForm then
        NavPanelNotifier.AddForm(RegForm)
      else
        NavPanelNotifier.RemoveForm(RegForm);
  end;
end;

{ TabcNavPanelDataLink }

constructor TabcNavPanelDataLink.Create(AOwner: TabcCustomNavPanel);
begin
  inherited Create;
  FOwner := AOwner;
  VisualControl := True;
end;

destructor TabcNavPanelDataLink.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

procedure TabcNavPanelDataLink.ActiveChanged;
begin
  if FOwner <> nil then
    FOwner.ActiveChanged;
end;

procedure TabcNavPanelDataLink.CheckBrowseMode;
begin
  if FOwner <> nil then
    FOwner.CheckBrowseMode;
end;

procedure TabcNavPanelDataLink.DataSetChanged;
begin
  if FOwner <> nil then
    FOwner.DataSetChanged;
end;

procedure TabcNavPanelDataLink.DataSetScrolled(Distance: Integer);
begin
  if FOwner <> nil then
    FOwner.DataSetScrolled(Distance);
end;

procedure TabcNavPanelDataLink.FocusControl(Field: TFieldRef);
begin
  if FOwner <> nil then
    FOwner.FocusControl(Field);
end;

procedure TabcNavPanelDataLink.EditingChanged;
begin
  if FOwner <> nil then
    FOwner.EditingChanged;
end;

procedure TabcNavPanelDataLink.LayoutChanged;
begin
  if FOwner <> nil then
    FOwner.LayoutChanged;
end;

procedure TabcNavPanelDataLink.RecordChanged(Field: TField);
begin
  if FOwner <> nil then
    FOwner.RecordChanged(Field);
end;

procedure TabcNavPanelDataLink.UpdateData;
begin
  if FOwner <> nil then
    FOwner.UpdateData;
end;

{ TabcNavToolbarDatalink }

constructor TabcNavToolbarDatalink.Create(AOwner: TabcCustomNavToolbar);
begin
  inherited Create;
  FOwner := AOwner;
  VisualControl := True;
end;

destructor TabcNavToolbarDatalink.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

procedure TabcNavToolbarDatalink.ActiveChanged;
begin
  if FOwner <> nil then
    FOwner.ActiveChanged;
end;

procedure TabcNavToolbarDatalink.CheckBrowseMode;
begin
  if FOwner <> nil then
    FOwner.CheckBrowseMode;
end;

procedure TabcNavToolbarDatalink.DataSetChanged;
begin
  if FOwner <> nil then
    FOwner.DataSetChanged;
end;

procedure TabcNavToolbarDatalink.DataSetScrolled(Distance: Integer);
begin
  if FOwner <> nil then
    FOwner.DataSetScrolled(Distance);
end;

procedure TabcNavToolbarDatalink.FocusControl(Field: TFieldRef);
begin
  if FOwner <> nil then
    FOwner.FocusControl(Field);
end;

procedure TabcNavToolbarDatalink.EditingChanged;
begin
  if FOwner <> nil then
    FOwner.EditingChanged;
end;

procedure TabcNavToolbarDatalink.LayoutChanged;
begin
  if FOwner <> nil then
    FOwner.LayoutChanged;
end;

procedure TabcNavToolbarDatalink.RecordChanged(Field: TField);
begin
  if FOwner <> nil then
    FOwner.RecordChanged(Field);
end;

procedure TabcNavToolbarDatalink.UpdateData;
begin
  if FOwner <> nil then
    FOwner.UpdateData;
end;

{ TabcNavPanel }

function TabcNavPanel.ShowDataSource(AOwner: TComponent; ADataSource: TDataSource;
  DataView: TabcnpDataView; APopupFormStyle: TabcPopupFormStyle;
  AReadOnly: Boolean; AHelpContext: THelpContext;
  AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm;   {replaces TForm}
begin
  Result := nil;
  {Validate datasource, readonly and modal action}
  if ADataSource = nil then raise EabcNavPanelException.Create(sabcnpDataSourceNil);
  if ADataSource.DataSet = nil then raise EabcNavPanelException.Create(sabcnpDataSetNil);
  if not ADataSource.DataSet.Active then raise EabcNavPanelException.Create(sabcnpDataSetInactive);
  AutoDataSource := True;
  try
    Result := abcnpFormFactory.CreateAutoFormOfClass(
      GetAutoFormClassOfView(DataView),
      AOwner, Self, ADataSource, APopupFormStyle, AReadOnly, False,
      AHelpContext, AfterFormCreate);
    Result.Show;
    DataSource := ADataSource;
  except
    if Result <> nil then Result.Free;
    raise;
  end;
end;

function TabcNavPanel.ShowModalDataSource(AOwner: TComponent; ADataSource: TDataSource;
  AReadOnly: Boolean; AHelpContext: THelpContext;
  AModalAction: TabcnpModalFormAction; AModalAutoPost: Boolean;
  AfterFormCreate: TabcnpAfterFormCreateEvent; AfterShowModal: TabcnpAfterShowModalEvent): Integer;
var
  AForm: TabcnpAutoForm;
begin
  {Validate datasource, readonly and modal action}
  if ADataSource = nil then raise EabcNavPanelException.Create(sabcnpDataSourceNil);
  if ADataSource.DataSet = nil then raise EabcNavPanelException.Create(sabcnpDataSetNil);
  if not ADataSource.DataSet.Active then raise EabcNavPanelException.Create(sabcnpDataSetInactive);
  if (AReadOnly or abcDataSetReadOnly(ADataSource.DataSet)) and (AModalAction <> mfaNone) then
    raise EabcNavPanelException.Create(sabcnpInvalidReadOnly);
  AForm := abcnpFormFactory.CreateAutoFormOfClass(
    TabcnfDataSourceEditRec,
    AOwner, Self, ADataSource, pfsModal, AReadOnly, AModalAutoPost,
    AHelpContext, AfterFormCreate);
  try
    case AModalAction of
      mfaEdit:
        ADataSource.DataSet.Edit;
      mfaAppend:
        ADataSource.DataSet.Append;
      mfaInsert:
        ADataSource.DataSet.Insert;
    end;
    Result := AForm.ExecModal(AfterShowModal);
  except
    ADataSource.DataSet.Cancel;
    raise;
  end;
end;

{ TabcNavToolbar }

function TabcNavToolbar.ShowDataSource(AOwner: TComponent; ADataSource: TDataSource;
  DataView: TabcnpDataView; APopupFormStyle: TabcPopupFormStyle;
  AReadOnly: Boolean; AHelpContext: THelpContext;
  AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm;
begin
  Result := nil;
  {Validate datasource, readonly and modal action}
  if ADataSource = nil then raise EabcNavPanelException.Create(sabcnpDataSourceNil);
  if ADataSource.DataSet = nil then raise EabcNavPanelException.Create(sabcnpDataSetNil);
  if not ADataSource.DataSet.Active then raise EabcNavPanelException.Create(sabcnpDataSetInactive);
  AutoDataSource := True;
  try
    Result := abcnpFormFactory.CreateAutoFormTBOfClass(
      GetAutoFormClassOfView(DataView),
      AOwner, Self, ADataSource, APopupFormStyle, AReadOnly, False,
      AHelpContext, AfterFormCreate);
    Result.Show;
    DataSource := ADataSource;
  except
    if Result <> nil then Result.Free;
    raise;
  end;
end;

function TabcNavToolbar.ShowModalDataSource(AOwner: TComponent; ADataSource: TDataSource;
  AReadOnly: Boolean; AHelpContext: THelpContext;
  AModalAction: TabcnpModalFormAction; AModalAutoPost: Boolean;
  AfterFormCreate: TabcnpAfterFormCreateEvent; AfterShowModal: TabcnpAfterShowModalEvent): Integer;
var
  AForm: TabcnpAutoForm;
begin
  {Validate datasource, readonly and modal action}
  if ADataSource = nil then raise EabcNavPanelException.Create(sabcnpDataSourceNil);
  if ADataSource.DataSet = nil then raise EabcNavPanelException.Create(sabcnpDataSetNil);
  if not ADataSource.DataSet.Active then raise EabcNavPanelException.Create(sabcnpDataSetInactive);
  if (AReadOnly or abcDataSetReadOnly(ADataSource.DataSet)) and (AModalAction <> mfaNone) then
    raise EabcNavPanelException.Create(sabcnpInvalidReadOnly);

  AForm := abcnpFormFactory.CreateAutoFormTBOfClass(
    TabcnfDataSourceEditRec,
    AOwner, Self, ADataSource, pfsModal, AReadOnly, AModalAutoPost,
    AHelpContext, AfterFormCreate);

  try
    case AModalAction of
      mfaEdit:
        ADataSource.DataSet.Edit;
      mfaAppend:
        ADataSource.DataSet.Append;
      mfaInsert:
        ADataSource.DataSet.Insert;
    end;
    Result := AForm.ExecModal(AfterShowModal);
  except
    ADataSource.DataSet.Cancel;
    raise;
  end;
end;

{ TabcNavPanelNotifier - global instance }

constructor TabcNavPanelNotifier.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDetectFormList := TList.Create;
  FNotifyDataSourceList := TList.Create;
  FNotifyFieldList := TList.Create;
  abcScreenEvents.RegisterOnActiveControlChange(ScreenActiveControlChange);
  abcScreenEvents.RegisterOnActiveFormChange(ScreenActiveFormChange);
end;

destructor TabcNavPanelNotifier.Destroy;
begin
  abcScreenEvents.UnRegisterOnActiveControlChange(ScreenActiveControlChange);
  abcScreenEvents.UnRegisterOnActiveFormChange(ScreenActiveFormChange);
  FDetectFormList.Free;
  FNotifyDataSourceList.Free;
  FNotifyFieldList.Free;
  inherited Destroy;
  GNavPanelNotifier := nil;
end;

procedure TabcNavPanelNotifier.GridHookOnColEnter(Sender: TObject);
var
  ANotifyProc: TNotifyEvent;
  NewField: TField;
begin
  ANotifyProc := nil;
  NewField := GetSaveGridSelectedField;
  if NewField <> FActiveField then
  begin
    FActiveField := NewField;
    NotifyFieldChange;
  end;
  @ANotifyProc := FSaveGridOCEMethod.Code;
  if Assigned(ANotifyProc) then ANotifyProc(FSaveGrid);
end;

procedure TabcNavPanelNotifier.AddDataSourceNotification(NavPanel: TabcCustomNavPanel);
begin
  with FNotifyDataSourceList do
    if IndexOf(NavPanel) = -1 then
      Add(NavPanel);
end;

procedure TabcNavPanelNotifier.AddDataSourceNotificationTB(NavToolbar: TabcCustomNavToolbar);
begin
  with FNotifyDataSourceList do
    if IndexOf(NavToolbar) = -1 then
      Add(NavToolbar);
end;

procedure TabcNavPanelNotifier.AddFieldNotification(NavPanel: TabcCustomNavPanel);
begin
  with FNotifyFieldList do
    if IndexOf(NavPanel) = -1 then
      Add(NavPanel);
end;

procedure TabcNavPanelNotifier.AddFieldNotificationTB(NavToolbar: TabcCustomNavToolbar);
begin
  with FNotifyFieldList do
    if IndexOf(NavToolbar) = -1 then
      Add(NavToolbar);
end;

procedure TabcNavPanelNotifier.AddForm(AForm: TForm);
begin
  with FDetectFormList do
    if IndexOf(AForm) = -1 then
      Add(AForm);
end;

procedure TabcNavPanelNotifier.AddGridHook;
var
  GridInfo: PPropInfo;
  GridMethod: TMethod;
begin
  if FSaveGrid is TCustomDBGrid then
  begin
    GridInfo := GetPropInfo(TDBGrid(FSaveGrid).ClassInfo, sabcnpOnColEnter);
    if GridInfo <> nil then
    begin
      GridMethod := GetMethodProp(TDBGrid(FSaveGrid), GridInfo);
      FSaveGridOCEMethod := GridMethod;
      GridMethod.Code := Self.MethodAddress(sabcnpGridHookOnColEnter);
      GridMethod.Data := Self;
      SetMethodProp(TDBGrid(FSaveGrid), GridInfo, GridMethod);
    end;
  end
  else if FSaveGrid is TabcCustomHDBGrid then
  begin
    GridInfo := GetPropInfo(TabcHDBGrid(FSaveGrid).ClassInfo, sabcnpOnRowEnter);
    if GridInfo <> nil then
    begin
      GridMethod := GetMethodProp(TabcHDBGrid(FSaveGrid), GridInfo);
      FSaveGridOCEMethod := GridMethod;
      GridMethod.Code := Self.MethodAddress(sabcnpGridHookOnColEnter);
      GridMethod.Data := Self;
      SetMethodProp(TabcHDBGrid(FSaveGrid), GridInfo, GridMethod);
    end;
  {$IFDEF INFOPOWER}
  end
  else if FSaveGrid is TwwCustomDBGrid then
  begin
    GridInfo := GetPropInfo(TwwDBGrid(FSaveGrid).ClassInfo, sabcnpOnColEnter);
    if GridInfo <> nil then
    begin
      GridMethod := GetMethodProp(TwwDBGrid(FSaveGrid), GridInfo);
      FSaveGridOCEMethod := GridMethod;
      GridMethod.Code := Self.MethodAddress(sabcnpGridHookOnColEnter);
      GridMethod.Data := Self;
      SetMethodProp(TwwDBGrid(FSaveGrid), GridInfo, GridMethod);
    end;
  {$ENDIF}
  end;
end;

function TabcNavPanelNotifier.FocusActiveControl: Boolean;
begin
  Result := False;
  if FActiveControl <> nil then
    try
      FActiveControl.SetFocus;
      Result := FActiveControl.Focused;
    except
    end;
end;

function TabcNavPanelNotifier.GetSaveGridSelectedField: TField;
begin
  if FSaveGrid is TCustomDBGrid then
  begin
    Result := TDBGrid(FSaveGrid).SelectedField
  end
  else if FSaveGrid is TabcCustomHDBGrid then
    Result := TabcHDBGrid(FSaveGrid).SelectedField
{$IFDEF INFOPOWER}
  else if FSaveGrid is TwwCustomDBGrid then
    Result := TwwDBGrid(FSaveGrid).SelectedField
{$ENDIF}
  else
    Result := nil;
end;

procedure TabcNavPanelNotifier.IgnoreNextFocus;
begin
  FIgnoreNextFocus := True;
end;

procedure TabcNavPanelNotifier.NotifyDataSourceChange;
var
  I: Integer;
begin
  if (Application.MainForm = nil) or
     (Application.Terminated) or
     (csDestroying in Application.MainForm.ComponentState) or
     (csDestroying in ComponentState) then
    Exit;
  with FNotifyDataSourceList do
    for I := 0 to Count - 1 do
      if (TObject(Items[I]) is TabcCustomNavToolbar) then
      with TabcCustomNavToolbar(Items[I]) do
      begin
        if AutoDataSource then
          DataSource := FActiveDataSource;
      end
      else if (TObject(Items[I]) is TabcCustomNavPanel) then
      with TabcCustomNavPanel(Items[I]) do
      begin
        if AutoDataSource then
          DataSource := FActiveDataSource;
      end;
end;

procedure TabcNavPanelNotifier.NotifyFieldChange;
var
  I: Integer;
begin
  with FNotifyFieldList do
    for I := 0 to Count - 1 do
      if (TObject(Items[I]) is TabcCustomNavToolbar) then
        with TabcCustomNavToolbar(Items[I]) do
        begin
          if AutoField and
            (AutoDataSource or
            ((FActiveField <> nil) and (FActiveField.Dataset <> nil) and
            (FActiveField.Dataset = Datasource.Dataset))) then
            Field := FActiveField;
        end
      else if (TObject(Items[I]) is TabcCustomNavPanel) then
        with TabcCustomNavPanel(Items[I]) do
        begin
          if AutoField and
            (AutoDataSource or
            ((FActiveField <> nil) and (FActiveField.Dataset <> nil) and
            (FActiveField.Dataset = Datasource.Dataset))) then
            Field := FActiveField;
        end;
end;

procedure TabcNavPanelNotifier.RemoveDataSourceNotification(NavPanel: TabcCustomNavPanel);
begin
  FNotifyDataSourceList.Remove(NavPanel);
end;

procedure TabcNavPanelNotifier.RemoveDataSourceNotificationTB(NavToolbar: TabcCustomNavToolbar);
begin
  FNotifyDataSourceList.Remove(NavToolbar);
end;

procedure TabcNavPanelNotifier.RemoveFieldNotification(NavPanel: TabcCustomNavPanel);
begin
  FNotifyFieldList.Remove(NavPanel);
end;

procedure TabcNavPanelNotifier.RemoveFieldNotificationTB(NavToolbar: TabcCustomNavToolbar);
begin
  FNotifyFieldList.Remove(NavToolbar);
end;

procedure TabcNavPanelNotifier.RemoveForm(AForm: TForm);

  procedure RemoveOwnedReferences(AForm: TForm);
  var
    i: integer;
  begin
    with FNotifyDataSourceList do
      for I := Count - 1 downto 0 do
        if TComponent(Items[i]).Owner = AForm then
          Delete(i);
    with FNotifyFieldList do
      for I := Count - 1 downto 0 do
        if TComponent(Items[i]).Owner = AForm then
          Delete(i);
  end;

begin
  FDetectFormList.Remove(AForm);
  {remove any navpanel or toolbar owned by AForm from notification lists}
  RemoveOwnedReferences(AForm);

  FActiveDataSource := nil;
  if not Application.Terminated then
  begin
    if (FSaveGrid <> nil) and (FSaveGrid.Owner = AForm) then
      RemoveGridHook;
    if (FActiveControl <> nil) then
      if (FActiveControl = AForm) or (FActiveControl.Owner = AForm) then
      begin
        FActiveControl := nil;
        FActiveDataSource := nil;
      end;
    NotifyDataSourceChange;
  end;
end;

procedure TabcNavPanelNotifier.RemoveGridHook;
var
  GridInfo: PPropInfo;
begin
  try
    if FSaveGrid is TCustomDBGrid then
    begin
      GridInfo := GetPropInfo(TDBGrid(FSaveGrid).ClassInfo, sabcnpOnColEnter);
      if GridInfo <> nil then
        SetMethodProp(TDBGrid(FSaveGrid), GridInfo, FSaveGridOCEMethod);
    end
    else if FSaveGrid is TabcCustomHDBGrid then
    begin
      GridInfo := GetPropInfo(TabcHDBGrid(FSaveGrid).ClassInfo, sabcnpOnRowEnter);
      if GridInfo <> nil then
        SetMethodProp(TabcHDBGrid(FSaveGrid), GridInfo, FSaveGridOCEMethod);
{$IFDEF INFOPOWER}
    end
    else if FSaveGrid is TwwCustomDBGrid then
    begin
      GridInfo := GetPropInfo(TwwCustomDBGrid(FSaveGrid).ClassInfo, sabcnpOnColEnter);
      if GridInfo <> nil then
        SetMethodProp(TwwCustomDBGrid(FSaveGrid), GridInfo, FSaveGridOCEMethod);
{$ENDIF}
    end;
  except
    {ignore errors in case grid has been destroyed}
    FSaveGrid := nil;
  end;
  FSaveGridOCEMethod.Code := nil;
  FSaveGridOCEMethod.Data := nil;
  FSaveGrid := nil;
end;

procedure TabcNavPanelNotifier.ScreenActiveControlChange(Sender: TObject);
begin
  if (Application.MainForm = nil) or
     (Application.Terminated) or
     (csDestroying in Application.MainForm.ComponentState) or
     (csDestroying in ComponentState) then
    Exit;
  if (FActiveForm is TabcnpForm) or (FDetectFormList.IndexOf(FActiveForm) > -1) then
    UpdateActiveControl;
  if Assigned(FOnActiveControlChange) then
    FOnActiveControlChange(Sender);
end;

procedure TabcNavPanelNotifier.ScreenActiveFormChange(Sender: TObject);
begin
  if (Application.MainForm = nil) or
     (Application.Terminated) or
     (csDestroying in Application.MainForm.ComponentState) or
     (csDestroying in ComponentState) then
    Exit;
  if not FIgnoreNextFocus then
    FActiveForm := Screen.ActiveForm
  else
    FIgnoreNextFocus := False;
  if Assigned(FOnActiveFormChange) then
    FOnActiveFormChange(Sender);
end;

procedure TabcNavPanelNotifier.UpdateActiveControl;
begin
  if FIgnoreNextFocus then
    FIgnoreNextFocus := False
  else
    if Screen.ActiveControl <> FActiveControl then
    begin
      FActiveControl := Screen.ActiveControl;
      if (FSaveGrid <> nil) and not (csDestroying in FSaveGrid.ComponentState) then
        RemoveGridHook;
      if FActiveControl = nil then
      begin
        FActiveDataSource := nil;
        NotifyDataSourceChange;
      end
      else
      begin
        if FActiveControl.ClassName = 'TGroupButton' then
          FActiveControl := FActiveControl.Parent;
        UpdateActiveDataSource;
      end;
    end;
end;

procedure TabcNavPanelNotifier.UpdateActiveDataSource;
var
  NewDataSource: TDataSource;
  NewField: TField;
  PropInfo: PPropInfo;
  SFieldName: string;
  I: Integer;

  function ParentIsNavPanel(AWinControl: TWinControl): Boolean;
  var
    ThisParent: TWinControl;
  begin
      ThisParent := AWinControl;
      while (ThisParent <> nil) and
            not (ThisParent is TabcCustomNavPanel)
            and not (ThisParent is TabcCustomNavToolbar) do
        ThisParent := ThisParent.Parent;
      Result := (ThisParent <> nil);
  end;

begin
  if FActiveControl <> nil then
    {ignore focus change to windows on NavPanels and NavToolbars}
    if ParentIsNavPanel(FActiveControl) then Exit;

  { detect and notify datasource change }
  if FActiveControl = nil then
    PropInfo := nil
  else
    PropInfo := GetPropInfo(FActiveControl.ClassInfo, sabcnpDataSource);
  if PropInfo <> nil then
    NewDataSource := TDataSource(GetOrdProp(FActiveControl, PropInfo))
  else
    NewDataSource := nil;
  if NewDataSource <> FActiveDataSource then begin
    FActiveDataSource := NewDataSource;
    NotifyDataSourceChange;
  end;

  { detect and notify field change }
  if (FActiveControl is TCustomDBGrid) or
     (FActiveControl is TabcCustomHDBGrid)
{$IFDEF INFOPOWER}
      or (FActiveControl is TwwCustomDBGrid)
{$ENDIF}
  then begin
    if FSaveGrid <> FActiveControl then
    begin
      FSaveGrid := FActiveControl;
      AddGridHook;
    end;
    NewField := GetSaveGridSelectedField;
  end
  else
  begin
    NewField := nil;
    if FActiveControl = nil then
      PropInfo := nil
    else
      PropInfo := GetPropInfo(FActiveControl.ClassInfo, sabcnpDataField);
    if PropInfo <> nil then
    begin
      SFieldName := GetStrProp(FActiveControl, PropInfo);
      if (FActiveDataSource <> nil) and (FActiveDataSource.DataSet <> nil) then
        with FActiveDataSource.DataSet do
          for I := 0 to FieldCount - 1 do
            if Fields[I].FieldName = SFieldName then
            begin
              NewField := Fields[I];
              Break;
            end;
    end;
  end;
  if NewField <> FActiveField then
  begin
    FActiveField := NewField;
    NotifyFieldChange;
  end;
end;

{ TabcNavFormLink }

constructor TabcNavFormLink.Create(AOwner: TComponent);
begin
  inherited;
  if not (AOwner is TForm)
{$IFDEF DELPHI5}
  or (AOwner is TFrame)
{$ENDIF}  
  then
    raise EComponentError.Create(sabcErrorOwnerFormExpected);
end;

destructor TabcNavFormLink.Destroy;
begin
  RegisterOwnerForm(False);
  inherited;
end;

procedure TabcNavFormLink.Loaded;
begin
  inherited;
  RegisterOwnerForm(True);
end;

procedure TabcNavFormLink.RegisterOwnerForm(AddForm: Boolean);
var
  RegForm: TForm;
begin
  if not (csDesigning in ComponentState) then
  begin
    if (Owner is TForm) then
      RegForm := TForm(Owner)
{$IFDEF DELPHI5}
    else if (Owner <> nil) and (Owner.Owner is TForm) then  //for TFrame
      RegForm := TForm(Owner.Owner)
{$ENDIF}
    else
      RegForm := nil;
    if RegForm <> nil then
      if AddForm then
        NavPanelNotifier.AddForm(RegForm)
      else
        NavPanelNotifier.RemoveForm(RegForm);
  end;
end;

{ TabcnpAutoForm }

constructor TabcnpAutoForm.CreateAutoForm(AOwner: TComponent; ANavPanel: TabcNavPanel;
  ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
  AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext; AfterFormCreate: TabcnpAfterFormCreateEvent);
begin
  FPopupStyle := APopupStyle;
  FDataSource := ADataSource;
  ValidateDataSource;
  FDataSetCaption := abcGetDataSetDesc(FDataSource.DataSet);
  FSaveDSStateChange := ADataSource.OnStateChange;
  ADataSource.OnStateChange := DoDSStateChange;
  FNavPanel := ANavPanel;
  FReadOnly := AReadOnly or abcDataSetReadOnly(FDataSource.DataSet);
  FModalAutoPost := AModalAutoPost;
  HelpContext := AHelpContext;
  inherited Create(AOwner);
  NavPanelNotifier.AddForm(Self);
  if CaptionStyle = afcDataSetState then
    SetCaptionToDSState
  else if CaptionStyle = afcDataSet then
    Caption := DataSetCaption;
  case APopupStyle of
    pfsNormal, pfsModal:
      FormStyle := fsNormal;
    pfsMDIChild:
      begin
        FormStyle := fsMDIChild;
        Position := poDefault;
      end;
    pfsStayOnTop:
      FormStyle := fsStayOnTop;
  end;
  {
    NB Descendant classes are responsible for calling AfterFormCreate
    when their constructor finishes, e.g.
    if Assigned(AfterFormCreate) then AfterFormCreate(Self);
  }
end;

constructor TabcnpAutoForm.CreateAutoFormTB(AOwner: TComponent; ANavToolbar: TabcNavToolbar;
  ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
  AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext; AfterFormCreate: TabcnpAfterFormCreateEvent);
begin
  FPopupStyle := APopupStyle;
  FDataSource := ADataSource;
  ValidateDataSource;
  FDataSetCaption := abcGetDataSetDesc(FDataSource.DataSet);
  FSaveDSStateChange := ADataSource.OnStateChange;
  ADataSource.OnStateChange := DoDSStateChange;
  FNavToolbar := ANavToolbar;
  FReadOnly := AReadOnly or abcDataSetReadOnly(FDataSource.DataSet);
  FModalAutoPost := AModalAutoPost;
  HelpContext := AHelpContext;
  inherited Create(AOwner);
  NavPanelNotifier.AddForm(Self);
  if CaptionStyle = afcDataSetState then
    SetCaptionToDSState
  else if CaptionStyle = afcDataSet then
    Caption := DataSetCaption;
  case APopupStyle of
    pfsNormal, pfsModal:
      FormStyle := fsNormal;
    pfsMDIChild:
      begin
        FormStyle := fsMDIChild;
        Position := poDefault;
      end;
    pfsStayOnTop:
      FormStyle := fsStayOnTop;
  end;
  {
    NB Descendant classes are responsible for calling AfterFormCreate
    when their constructor finishes, e.g.
    if Assigned(AfterFormCreate) then AfterFormCreate(Self);
  }
end;

destructor TabcnpAutoForm.Destroy;
begin
  NavPanelNotifier.RemoveForm(Self);
  FDataSource.OnStateChange := FSaveDSStateChange;
  inherited Destroy;
end;

procedure TabcnpAutoForm.DoDSStateChange(Sender: TObject);
begin
  OnDataSourceStateChange(Sender);
  if Assigned(FSaveDSStateChange) then FSaveDSStateChange(Sender);
end;

function TabcnpAutoForm.ExecModal(AfterShowModal: TabcnpAfterShowModalEvent): Integer;
begin
  try
    Result := ShowModal;
    if Assigned(AfterShowModal) then
      AfterShowModal(Self, Result);
    if ModalAutoPost and not ReadOnly then
      if Result = mrOK then
      begin
        if (DataSource.DataSet.State in [dsEdit, dsInsert]) then
          DataSource.DataSet.Post;
      end
      else if Result = mrCancel then
        DataSource.DataSet.Cancel;
  finally
    Release;
  end;
end;

procedure TabcnpAutoForm.OnDataSourceStateChange(Sender: TObject);
begin
  if (CaptionStyle = afcDataSetState) then
    SetCaptionToDSState;
end;

procedure TabcnpAutoForm.SetCaptionToDSState;
begin
  if DataSource <> nil then
    case DataSource.State of
      dsInactive:
        Caption := DataSetCaption + sabcnpCapDSSep + sabcnpiInactive;
      dsBrowse:
        Caption := DataSetCaption + sabcnpCapDSSep + sabcnpiBrowse;
      dsEdit:
        Caption := DataSetCaption + sabcnpCapDSSep + sabcnpiEdit;
      dsInsert:
        Caption := DataSetCaption + sabcnpCapDSSep + sabcnpiInsert;
      dsSetKey:
        Caption := DataSetCaption + sabcnpCapDSSep + sabcnpiSetKey;
      dsCalcFields:
        Caption := DataSetCaption + sabcnpCapDSSep + sabcnpiCalcFields;
    else
        Caption := DataSetCaption;
    end
  else
    Caption := sabcnpCapDSNil;
end;

procedure TabcnpAutoForm.ValidateDataSource;
begin
  if FDataSource = nil then
    raise EabcNavPanelException.Create(sabcnpDataSourceNil)
  else if FDataSource.DataSet = nil then
    raise EabcNavPanelException.Create(sabcnpDataSetNil)
  else if not FDataSource.DataSet.Active then
    raise EabcNavPanelException.Create(sabcnpDataSetInactive);
end;

initialization

finalization
  GNavPanelNotifier.Free;

end.
