unit abcnpbtn;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Menus, Forms, Dialogs, abcnp, DB,
  DBCtrls, DBGrids, Buttons, abcexctl, abcbtn, abcda;

type

  TabcDataSetCheckEnabledEvent = procedure (Sender: TObject; DataSet: TDataSet;
    var Enable: Boolean) of object;

  TabcFieldCheckEnabledEvent = procedure (Sender: TObject; Field: TField;
    var Enable: Boolean) of object;

  { TabcCustomnpBtn }

  TabcCustomnpBtn = class(TabcCustomPicSpeedBtn)
  private
    FAdapter: TabcDatasetAdapter;
    FDataSet: TDataSet;
    FNavPanel: TabcCustomNavPanel;
    FNavToolbar: TabcCustomNavToolbar;
    procedure SetDataset(Value: TDataset);
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
  protected
    procedure CheckEnabled; virtual;
    procedure SetParent(AParent: TWinControl); override;
    property DataSet: TDataSet read FDataSet;
    property Adapter: TabcDatasetAdapter read FAdapter write FAdapter;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property MouseOver;
    property NavPanel: TabcCustomNavPanel read FNavPanel;
    property NavToolbar: TabcCustomNavToolbar read FNavToolbar;
  published
    property Action;
    property Anchors;
    property BiDiMode;
    property Constraints;
    {property AllowAllUp;}
    property GroupIndex;
    property Caption;
    property Color;
    property DisabledShadow;
    {property Down;}
    {property Enabled;}
    property Font;
    property ImageIndex;
    property Images;
    property Layout;
    property Margin;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property ParentBiDiMode;
    property Picture;
    property PictureHighlight;
    property NumGlyphs;
    property Popup;
    property PopupMenu;
    property ShowHint;
    property Spacing;
    property Style;
    property TextStyle;
    property TransparentColor;
    property Visible;
    property BeforeClick;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TabcnpActiveFieldBtn }

  TabcnpActiveFieldBtn = class(TabcCustomnpBtn)
  private
    FField: TField;
    procedure abcCDActiveFieldChanged(var Message: TabcCDActiveFieldChanged); message ABC_CD_ACTIVEFIELDCHANGED;
  protected
    procedure SetParent(AParent: TWinControl); override;
    property Field: TField read FField;
  end;

{-------------------------- RECORD NAVIGATION BUTTONS -------------------------}

  { TabcCustomnpNavBtn }

  TabcCustomnpNavBtn = class(TabcCustomnpBtn)
  private
    procedure abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled); message ABC_CD_DATASETSCROLLED;
  protected
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:Integer); override;
  end;

  { TabcCustomnpRepNavBtn }

  TRepBtnState = set of (rbFocusRect, rbAllowTimer);

  TabcCustomnpRepNavBtn = class(TabcCustomnpNavBtn)
  private
    FAccelerate: Boolean;
    FAccelerateRepeat: Integer;
    FAccelerateTicks: Integer;
    FInitRepeatPause: Word;
    FOnTimer: TNotifyEvent;
    FRepeatPause: Word;
    FRepeatTimer: TTimer;
    FTimeBtnState: TRepBtnState;
    function GetRepeatEnabled: Boolean;
    procedure RepeatClick(Sender: TObject);
    procedure SetRepeatEnabled(Value: Boolean);
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y:Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:Integer); override;
    property TimeBtnState: TRepBtnState read FTimeBtnState write FTimeBtnState;
    {can be published}
    property Accelerate: Boolean read FAccelerate write FAccelerate default True;
    property RepeatEnabled: Boolean read GetRepeatEnabled write SetRepeatEnabled default True;
    property RepeatInitialpause: Word read FInitRepeatPause write FInitRepeatPause default 100;
    property Repeatpause: Word read FRepeatPause write FRepeatPause default 50;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TabcnpFirst }

  TabcnpFirst = class(TabcCustomnpNavBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcnpNext }

  TabcnpNext = class(TabcCustomnpRepNavBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property Accelerate;
    property RepeatEnabled;
    property RepeatInitialpause;
    property Repeatpause;
  end;

  { TabcnpPrev }

  TabcnpPrev = class(TabcCustomnpRepNavBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property Accelerate;
    property RepeatEnabled;
    property RepeatInitialpause;
    property Repeatpause;
  end;

  { TabcnpLast }

  TabcnpLast = class(TabcCustomnpNavBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcCustomnpJumpBtn }

  TabcCustomnpJumpBtn = class(TabcCustomnpRepNavBtn)
  private
    FMoveBy: Integer;
  protected
    property MoveBy: Integer read FMoveBy write FMoveBy default 10;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Accelerate;
    property RepeatEnabled;
    property RepeatInitialpause;
    property Repeatpause;
  end;

  { TabcnpJumpFwd }

  TabcnpJumpFwd = class(TabcCustomnpJumpBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property MoveBy;
  end;

  { TabcnpJumpBack }

  TabcnpJumpBack = class(TabcCustomnpJumpBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property MoveBy;
  end;

{-------------------------- RECORD EDIT & DELETE BUTTONS ----------------------}

  { TabcnpDelete }

  TabcnpDelete = class(TabcCustomnpBtn)
  private
    FConfirm: Boolean;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Confirm: Boolean read FConfirm write FConfirm
      default True;
  end;

  { TabcCustomnpRecEditBtn }

  TabcCustomnpRecEditBtn = class(TabcCustomnpBtn)
  private
    procedure abcCDEditingChanged(var Message: TabcCDEditingChanged); message ABC_CD_EDITINGCHANGED;
  end;

  { TabcnpInsert }

  TabcnpInsert = class(TabcCustomnpRecEditBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcnpAppend }

  TabcnpAppend = class(TabcCustomnpRecEditBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcnpDuplicateRec }

  TabcnpDuplicateRec = class(TabcCustomnpRecEditBtn)
  private
    FInsert: Boolean;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property Insert: Boolean read FInsert write FInsert default False;
  end;

  { TabcnpEdit }

  TabcnpEdit = class(TabcCustomnpRecEditBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcnpCancel }

  TabcnpCancel = class(TabcCustomnpRecEditBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcnpPost }

  TabcnpPost = class(TabcCustomnpRecEditBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcnpRefresh }

  TabcnpRefresh = class(TabcCustomnpRecEditBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;


  { TabcnpEmptyTable }

  TabcnpEmptyTable = class(TabcCustomnpRecEditBtn)
  private
    FConfirm: Boolean;
    FConfirmMsg: string;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Confirm: Boolean read FConfirm write FConfirm default True;
    property ConfirmMsg: string read FConfirmMsg write FConfirmMsg;
  end;

{-------------------------- SQL DATABASE TRANSACTION BUTTONS ------------------}

  { TabcCustomnpSQLDBBtn }

  TabcCustomnpSQLDBBtn = class(TabcCustomnpBtn)
  private
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
  protected
    procedure CheckEnabled; override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:Integer); override;
  public
    destructor Destroy; override;
  end;

  { TabcnpSQLStartTrans }

  TabcnpSQLStartTrans = class(TabcCustomnpSQLDBBtn)
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  end;

  { TabcnpSQLCommit }

  TabcnpSQLCommit = class(TabcCustomnpSQLDBBtn)
  private
    FConfirm: Boolean;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property Confirm: Boolean read FConfirm write FConfirm
      default False;
  end;

  { TabcnpSQLRollback }

  TabcnpSQLRollback = class(TabcCustomnpSQLDBBtn)
  private
    FConfirm: Boolean;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Confirm: Boolean read FConfirm write FConfirm
      default True;
  end;


implementation

uses TypInfo, DBConsts, abcconst, abcutilf, abcnpcon
{$IFDEF DELPHI6}
     , VDBConsts
{$ENDIF}
;

{ TabcCustomnpBtn }

constructor TabcCustomnpBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csDoubleClicks];
  SetBounds(Left, Top, 25, 25);
end;

destructor TabcCustomnpBtn.Destroy;
begin
  FDataSet := nil;
  FAdapter.Free;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcCustomnpBtn.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  with Message do begin
    SetDataset(Dataset);
    if LinkActive then
      CheckEnabled
    else
      Enabled := False;
  end;
end;

procedure TabcCustomnpBtn.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FAdapter.Free;
    FAdapter := nil;
    FDataset := Value;
    if Value <> nil then
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
  end;
end;

procedure TabcCustomnpBtn.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  CheckEnabled;
end;

procedure TabcCustomnpBtn.CheckEnabled;
begin
  {do nothing}
end;

procedure TabcCustomnpBtn.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (csDestroying in ComponentState) then Exit;
  if (AParent <> nil) then
    if (AParent is TabcCustomNavToolbar) then begin
      FNavToolbar := TabcCustomNavToolbar(AParent);
      FNavPanel := nil;
      with TabcNavToolbar(AParent) do begin
        if (DataSource <> nil) then
          FDataSet := DataSource.DataSet;
        CheckEnabled;
      end;
    end
    else
    if (AParent is TabcCustomNavPanel) then begin
      FNavPanel := TabcCustomNavPanel(AParent);
      FNavToolbar := nil;
      with TabcNavPanel(AParent) do begin
        if (DataSource <> nil) then
          FDataSet := DataSource.DataSet;
        CheckEnabled;
      end;
    end
    else begin
      Enabled := False;
      FNavPanel := nil;
      FNavToolbar := nil;
    end
  else begin
    Enabled := False;
    FNavPanel := nil;
    FNavToolbar := nil;
  end;
end;

{ TabcnpActiveFieldBtn }

procedure TabcnpActiveFieldBtn.abcCDActiveFieldChanged(var Message: TabcCDActiveFieldChanged);
begin
  FField := Message.Field;
  CheckEnabled;
end;

procedure TabcnpActiveFieldBtn.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (csDestroying in ComponentState) then Exit;
  if NavToolbar <> nil then
    TabcNavToolbar(NavToolbar).AutoField := True
  else if NavPanel <> nil then
    TabcNavPanel(NavPanel).AutoField := True;
end;

{-------------------------- RECORD NAVIGATION BUTTONS -------------------------}

{ TabcCustomnpNavBtn }

procedure TabcCustomnpNavBtn.abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled);
begin
  CheckEnabled;
end;

procedure TabcCustomnpNavBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcCustomnpRepNavBtn }

constructor TabcCustomnpRepNavBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAccelerate := True;
  FInitRepeatPause := 200;
  FRepeatPause := 50;
  FTimeBtnState := [rbAllowTimer];
  OnTimer := RepeatClick;
end;

destructor TabcCustomnpRepNavBtn.Destroy;
begin
  FRepeatTimer.Free;
  inherited Destroy;
end;

procedure TabcCustomnpRepNavBtn.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown (Button, Shift, X, Y);
  if (rbAllowTimer in FTimeBtnState) and (Assigned(FOnTimer)) then
  begin
    if FRepeatTimer = nil then
      FRepeatTimer := TTimer.Create(Self);
    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := FInitRepeatPause;
    FAccelerateRepeat := 0;
    FAccelerateTicks := 0;
    FRepeatTimer.Enabled  := True;
  end;
end;

procedure TabcCustomnpRepNavBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil then begin
    FRepeatTimer.Enabled  := False;
  end;
end;

procedure TabcCustomnpRepNavBtn.TimerExpired(Sender: TObject);
const
  TicksPerAccel = 4;
var
  I: Integer;
begin
  Inc(FAccelerateTicks);
  FRepeatTimer.Enabled := False;
  if (State = absDown) and MouseCapture then begin
    for I := 0 to FAccelerateRepeat do
      if Assigned(FOnTimer) then FOnTimer(Self);
    if FAccelerateTicks = 1 then begin
      {first tick}
      if FAccelerate and (FRepeatPause < 50) then
        FRepeatTimer.Interval := 50
      else
        FRepeatTimer.Interval := FRepeatPause;
    end
    else if FAccelerate then begin
      if FAccelerateTicks mod TicksPerAccel = 0 then
        {speed up timer events}
        if FRepeatTimer.Interval <= 50 then
          Inc(FAccelerateRepeat)
        else begin
          FRepeatTimer.Interval := Trunc(FRepeatTimer.Interval * 3 div 4);
          if FRepeatTimer.Interval < 50 then
            FRepeatTimer.Interval := 50;
        end;
    end;
    FRepeatTimer.Enabled := True;
  end;
end;

procedure TabcCustomnpRepNavBtn.Paint;
var
  R: TRect;
begin
  inherited Paint;
  if rbFocusRect in FTimeBtnState then
  begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if State = absDown then
      OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
  end;
end;

function TabcCustomnpRepNavBtn.GetRepeatEnabled: Boolean;
begin
  Result := rbAllowTimer in FTimeBtnState;
end;

procedure TabcCustomnpRepNavBtn.RepeatClick(Sender: TObject);
begin
  Click;
end;

procedure TabcCustomnpRepNavBtn.SetRepeatEnabled(Value: Boolean);
begin
  if Value then
    FTimeBtnState := FTimeBtnState + [rbAllowTimer]
  else
    FTimeBtnState := FTimeBtnState - [rbAllowTimer];
end;

{ TabcnpFirst }

procedure TabcnpFirst.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and not DataSet.BOF;
end;

procedure TabcnpFirst.ValidClick;
begin
  DataSet.First;
end;

{ TabcnpPrev }

procedure TabcnpPrev.CheckEnabled;
begin
  with Dataset do
    Enabled := (DataSet <> nil) and Active and not BOF {$IFDEF DELPHI6} and not IsUniDirectional {$ENDIF};
end;

procedure TabcnpPrev.ValidClick;
begin
  DataSet.Prior;
end;

{ TabcnpNext }

procedure TabcnpNext.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and not DataSet.EOF;
end;

procedure TabcnpNext.ValidClick;
begin
  DataSet.Next;
end;

{ TabcnpLast }

procedure TabcnpLast.CheckEnabled;
begin
  with Dataset do
    Enabled := (DataSet <> nil) and Active and not EOF {$IFDEF DELPHI6} and not IsUniDirectional {$ENDIF};
end;

procedure TabcnpLast.ValidClick;
begin
  DataSet.Last;
end;


{ TabcCustomnpJumpBtn }

constructor TabcCustomnpJumpBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMoveBy := 10;
end;

{ TabcnpJumpFwd }

procedure TabcnpJumpFwd.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and not DataSet.EOF;
end;

procedure TabcnpJumpFwd.ValidClick;
begin
  DataSet.MoveBy(FMoveBy);
end;

{ TabcnpJumpBack }

procedure TabcnpJumpBack.CheckEnabled;
begin
  with Dataset do
    Enabled := (DataSet <> nil) and Active and not BOF {$IFDEF DELPHI6} and not IsUniDirectional {$ENDIF};
end;

procedure TabcnpJumpBack.ValidClick;
begin
  DataSet.MoveBy( - FMoveBy );
end;

{-------------------------- RECORD EDIT & DELETE BUTTONS ----------------------}

{ TabcnpDelete }

constructor TabcnpDelete.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FConfirm := True;
end;

procedure TabcnpDelete.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and
    Dataset.CanModify and not (DataSet.BOF and DataSet.EOF);
end;

procedure TabcnpDelete.ValidClick;
begin
  if not FConfirm or
    (MessageDlg(SDeleteRecordQuestion,
    mtConfirmation, mbOKCancel, 0) <> idCancel) then DataSet.Delete;
    NavPanelNotifier.FocusActiveControl;
end;

{ TabcCustomnpRecEditBtn }

procedure TabcCustomnpRecEditBtn.abcCDEditingChanged(var Message: TabcCDEditingChanged);
begin
  CheckEnabled;
end;

{ TabcnpInsert }

procedure TabcnpInsert.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and DataSet.CanModify and (Dataset.State <> dsInsert);
end;

procedure TabcnpInsert.ValidClick;
begin
  DataSet.Insert;
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpAppend }

procedure TabcnpAppend.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and DataSet.CanModify and (Dataset.State <> dsInsert);
end;

procedure TabcnpAppend.ValidClick;
begin
  DataSet.Append;
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpDuplicateRec }

procedure TabcnpDuplicateRec.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and DataSet.CanModify
     and not (DataSet.BOF and DataSet.EOF);
end;

procedure TabcnpDuplicateRec.ValidClick;
var
  StrBuffer: TStringList;
  FieldStream: TMemoryStream;
  I: integer;
begin
  StrBuffer := TStringList.Create;
  try
    with DataSet do begin
      for I := 0 to FieldCount - 1 do
        with Fields[I] do
          if DataType in [ftString..ftDateTime] then
            StrBuffer.Add(AsString)
          else if DataType in [ftBytes..ftGraphic] then begin
            FieldStream := TMemoryStream.Create;
            {fill the stream, depending on field class}
            if (Fields[I] is TMemoField) then
              TMemoField(Fields[I]).SaveToStream(FieldStream)
            else if (Fields[I] is TGraphicField) then
              TGraphicField(Fields[I]).SaveToStream(FieldStream)
            else if (Fields[I] is TBlobField) then
              TBlobField(Fields[I]).SaveToStream(FieldStream)
            else if (Fields[I] is TBytesField) then begin
              FieldStream.SetSize(Fields[I].DataSize);
              TBytesField(Fields[I]).GetData(FieldStream.Memory);
            end
            else if (Fields[I] is TVarBytesField) then begin
              FieldStream.SetSize(Fields[I].DataSize);
              TVarBytesField(Fields[I]).GetData(FieldStream.Memory);
            end;
            StrBuffer.AddObject(IntToStr(FieldStream.Size), FieldStream);
          end;
      Append;
      for I := 0 to FieldCount - 1 do
        if Fields[I].DataType in [ftString..ftDateTime] then
          Fields[I].AsString := StrBuffer.Strings[I]
        else if Fields[I].DataType in [ftBytes..ftGraphic] then begin
            FieldStream := TMemoryStream(StrBuffer.Objects[I]);
            FieldStream.Seek(0,0);
            if (Fields[I] is TMemoField) then
              TMemoField(Fields[I]).LoadFromStream(FieldStream)
            else if (Fields[I] is TGraphicField) then
              TGraphicField(Fields[I]).LoadFromStream(FieldStream)
            else if (Fields[I] is TBlobField) then
              TBlobField(Fields[I]).LoadFromStream(FieldStream)
            else if (Fields[I] is TBytesField) then begin
              TBytesField(Fields[I]).SetData(FieldStream.Memory);
            end
            else if (Fields[I] is TVarBytesField) then
              TVarBytesField(Fields[I]).SetData(FieldStream.Memory);
            StrBuffer.Objects[I] := nil;
            FieldStream.Free;
          end;
    end;
  finally
    if StrBuffer <> nil then
      for I := 0 to StrBuffer.Count - 1 do
        if StrBuffer.Objects[I] <> nil then begin
          FieldStream := TMemoryStream(StrBuffer.Objects[I]);
          FieldStream.Free;
          StrBuffer.Objects[I] := nil;
        end;
    StrBuffer.Free;
  end;
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpEdit }

procedure TabcnpEdit.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and DataSet.CanModify
    and not (DataSet.State in dsEditModes);
end;

procedure TabcnpEdit.ValidClick;
begin
  DataSet.Edit;
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpCancel }

procedure TabcnpCancel.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and DataSet.CanModify
    and (DataSet.State in dsEditModes);
end;

procedure TabcnpCancel.ValidClick;
begin
  DataSet.Cancel;
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpPost }

procedure TabcnpPost.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and DataSet.CanModify
    and (DataSet.State in dsEditModes);
end;

procedure TabcnpPost.ValidClick;
begin
  DataSet.Post;
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpRefresh }

procedure TabcnpRefresh.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.CanModify;
end;

procedure TabcnpRefresh.ValidClick;
begin
  DataSet.Refresh;
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpEmptyTable }

constructor TabcnpEmptyTable.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FConfirm := True;
  FConfirmMsg := sabcnpWarnEmptyTable;
end;

procedure TabcnpEmptyTable.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active
    and not (DataSet.State in dsEditModes)
    and (FAdapter <> nil) and FAdapter.CanEmptyTable;
end;

procedure TabcnpEmptyTable.ValidClick;
var
  Msg: string;
begin
  if FConfirm then begin
    Msg := abcReplaceStr(sabcnpTableMask, FAdapter.Description, FConfirmMsg, False);
    if MessageDlg(Msg, mtWarning, [mbNo, mbYes], 0) = mrYes then
      FAdapter.EmptyTable;
  end
  else
    FAdapter.EmptyTable;
  NavPanelNotifier.FocusActiveControl;
end;

{-------------------------- SQL DATABASE TRANSACTION BUTTONS ------------------}

{ TabcCustomnpSQLDBBtn }

destructor TabcCustomnpSQLDBBtn.Destroy;
begin
  inherited Destroy;
end;

procedure TabcCustomnpSQLDBBtn.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and (FAdapter <> nil) and FAdapter.CanUseTransactions;
end;

procedure TabcCustomnpSQLDBBtn.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  if Message.LinkActive then CheckEnabled;
end;

procedure TabcCustomnpSQLDBBtn.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  inherited;
  CheckEnabled;
end;

procedure TabcCustomnpSQLDBBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpSQLStartTrans }

procedure TabcnpSQLStartTrans.ValidClick;
begin
  FAdapter.StartTransaction;
  DataSet.Refresh;  {needed to force btns to update}
end;

procedure TabcnpSQLStartTrans.CheckEnabled;
begin
  inherited CheckEnabled;
  if Enabled then
    Enabled := (FAdapter <> nil) and FAdapter.CanStartTransaction;
end;

{ TabcnpSQLCommit }

procedure TabcnpSQLCommit.CheckEnabled;
begin
  inherited CheckEnabled;
  if Enabled then
    Enabled := (FAdapter <> nil) and FAdapter.CanCommit;
end;

procedure TabcnpSQLCommit.ValidClick;
begin
  if not FConfirm or (MessageDlg(sabcnpMsgCommit, mtConfirmation, mbOKCancel, 0)
      <> idCancel) then begin
    FAdapter.Commit;
    DataSet.Refresh; {needed to force btns to update}
  end;
end;

{ TabcnpSQLRollback }

constructor TabcnpSQLRollback.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FConfirm := True;
end;

procedure TabcnpSQLRollback.CheckEnabled;
begin
  inherited CheckEnabled;
  if Enabled then
    Enabled := (FAdapter <> nil) and FAdapter.CanRollback;
end;

procedure TabcnpSQLRollback.ValidClick;
begin
  if not FConfirm or (MessageDlg(sabcnpMsgRollback, mtConfirmation, mbOKCancel, 0)
      <> idCancel) then begin
    FAdapter.Rollback;
    DataSet.Refresh;
  end;
end;


end.
