unit abcnpedt;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Messages, Classes, Controls, StdCtrls, Forms, Dialogs, DB,
  DBGrids, abchgrid, abcedwin,
  abcnp, abcnpbtn, abcbtn, abcdbaut, abcforms, abcnpfac, abcfact
{$IFDEF INFOPOWER}
  , wwdbigrd
{$ENDIF}
  ;

type

  EabcNavPanelEditException = class(EabcNavPanelException);

{-------------------------- DATASET FIELD VIEW/EDIT BUTTONS -------------------}

  { TabcnpBlobEdit }

  TabcnpBlobEdit = class(TabcnpActiveFieldBtn)
  private
    FAutoEdit: TabcEditFieldOptions;
    FEditBlobAs: TabcEditBlobOption;
    FPopupFormStyle: TabcPopupFormStyle;
    FHelpContext: THelpContext;
    FReadOnly: Boolean;
    FOnCheckEnabled: TabcFieldCheckEnabledEvent;
    FOnCustomBlobEdit: TabcCustomFieldEditEvent;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    function GetDataSource: TDataSource;
  protected
    procedure EditBlob; dynamic;
    procedure EditCustomBlob; dynamic;
    procedure EditGraphic; dynamic;
    procedure EditMemo; dynamic;
    procedure CheckEnabled; override;
    procedure ValidClick; override;
    property DataSource: TDataSource read GetDataSource;
  published
    property AutoEdit: TabcEditFieldOptions read FAutoEdit write FAutoEdit default
      [efMemo, efGraphic];
    property EditBlobAs: TabcEditBlobOption read FEditBlobAs write FEditBlobAs default ebCustom;
    property HelpContext: THelpContext read FHelpContext write FHelpContext default 0;
    property PopupFormStyle: TabcPopupFormStyle read FPopupFormStyle write FPopupFormStyle default pfsNormal;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    property OnCheckEnabled: TabcFieldCheckEnabledEvent read FOnCheckEnabled write FOnCheckEnabled;
    property OnCustomBlobEdit: TabcCustomFieldEditEvent read FOnCustomBlobEdit write FOnCustomBlobEdit;
  public
    constructor Create(AOwner: TComponent); override;
  end;

{-------------------------- DATASOURCE VIEW/EDIT BUTTONS -------------------}

  { TabcnpDataSourceEdit }

  TabcnpCustomDataSourceEdit = class(TabcPicSpeedBtn)
  private
    FAfterFormCreate: TabcnpAfterFormCreateEvent;
    FAfterShowModal: TabcnpAfterShowModalEvent;
    FBeforeShowForm: TabcCancelEvent;
    FDataSource: TDataSource;
    FForm: TForm;
    FFormName: TComponentName;
    FDataView: TabcnpDataView;
    FHelpContext: THelpContext;
    FModalAction: TabcnpModalFormAction;
    FModalAutoPost: Boolean;
    FModalResult: Integer;
    FPopupFormStyle: TabcPopupFormStyle;
    FReadOnly: Boolean;
    function GetForm: TForm;
    procedure SetDataView(Value: TabcnpDataView);
    procedure SetModalAction(Value: TabcnpModalFormAction);
    procedure SetReadOnly(Value: Boolean);
  protected
    procedure ValidClick; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetParent(AParent: TWinControl); override;
    function FormExists: Boolean;
    procedure ShowForm; virtual;
    procedure ShowCustomForm; virtual;
    procedure ShowDefaultForm; virtual;
    property DataSource: TDataSource read FDataSource write FDataSource;
    property Form: TForm read GetForm;
    property FormName: TComponentName read FFormName write FFormName;
    property DataView: TabcnpDataView read FDataView write SetDataView default npvGrid;
    property HelpContext: THelpContext read FHelpContext write FHelpContext;
    property ModalAction: TabcnpModalFormAction read FModalAction write SetModalAction default mfaNone;
    property ModalAutoPost: Boolean read FModalAutoPost write FModalAutoPost default True;
    property ModalResult: Integer read FModalResult;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly default False;
    property PopupFormStyle: TabcPopupFormStyle read FPopupFormStyle write FPopupFormStyle default pfsNormal;
    property AfterFormCreate: TabcnpAfterFormCreateEvent read FAfterFormCreate write FAfterFormCreate;
    property AfterShowModal: TabcnpAfterShowModalEvent read FAfterShowModal write FAfterShowModal;
    property BeforeShowForm: TabcCancelEvent read FBeforeShowForm write FBeforeShowForm;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TabcnpDataSourceEdit = class(TabcnpCustomDataSourceEdit)
  public
    procedure ShowForm; override;
    property Form;
    property ModalResult;
  published
    property DataSource;
    property DataView;
    property FormName;
    property HelpContext;
    property ModalAction;
    property ModalAutoPost;
    property PopupFormStyle;
    property ReadOnly;
    property AfterFormCreate;
    property AfterShowModal;
    property BeforeShowForm;
  end;

{-------------------------- FIELD/RECORD SEARCH/REPLACE -----------------------}

  TabcnpFindOption = (nfrMatchCase, nfrWholeField, nfrAllFields, nfrDown,
  nfrHideMatchCase, nfrHideWholeField, nfrHideAllFields, nfrHideUpDown,
  nfrDisableMatchCase, nfrDisableWholeField, nfrDisableAllFields, nfrDisableUpDown,
  nfrShowHelp, nfrShowProgress, nfrFocusField);

  TabcnpFindOptions = set of TabcnpFindOption;

  TabcnpReplaceOption = (nfrShowReplace, nfrShowReplaceAll, nfrReplacePost,
    nfrReplaceFindNext);

  TabcnpReplaceOptions = set of TabcnpReplaceOption;

  TabcnpFindAction = (nfaFind, nfaReplace);

  TabcnpFindReplace = class(TabcnpActiveFieldBtn)
  private
    { Private declarations }
    FAction: TabcnpFindAction;
    FFindText: string;
    FFirstFind: Boolean;
    FForm: TForm;
    FHelpContext: THelpContext;
    FOnFind: TabcnpFieldEvent;
    FOnReplace: TabcnpFieldEvent;
    FOptionsFind: TabcnpFindOptions;
    FOptionsReplace: TabcnpReplaceOptions;
    FReplaceText: string;
    FSaveDataSet: TDataSet;
    FSearchCount: Longint;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled); message ABC_CD_DATASETSCROLLED;
    procedure SetFindText(Value: string);
    procedure SetOptionsReplace(Value: TabcnpReplaceOptions);
  protected
    { Protected declarations }
    procedure CheckEnabled; override;
    function DoFindNext(var ABookmark: TBookmark; var AField: TField;
      var ACancelled: Boolean; ACanModify: Boolean; IncludeCurrent: Boolean): Boolean; virtual;
    procedure FocusField(AField: TField); virtual;
    function MatchField(CheckField: TField; CheckCanModify: Boolean): Boolean; virtual;
    property SearchCount: Longint read FSearchCount;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure ValidClick; override;
    procedure Close; virtual;
    function Find: Boolean; virtual;
    function Replace: Boolean; virtual;
    function ReplaceAll: Integer; virtual;
    property DataSet;
    property Field;
    property Form: TForm read FForm;
  published
    { Published declarations }
    property Action: TabcnpFindAction read FAction write FAction default nfaFind;
    property FindText: string read FFindText write SetFindText;
    property HelpContext: THelpContext read FHelpContext write FHelpContext default 0;
    property ReplaceText: string read FReplaceText write FReplaceText;
    property OptionsFind: TabcnpFindOptions read FOptionsFind write FOptionsFind
      default [nfrDown, nfrShowProgress, nfrFocusField];
    property OptionsReplace: TabcnpReplaceOptions read FOptionsReplace write SetOptionsReplace
      default [nfrShowReplace, nfrShowReplaceAll, nfrReplacePost, nfrReplaceFindNext];
    property OnFind: TabcnpFieldEvent read FOnFind write FOnFind;
    property OnReplace: TabcnpFieldEvent read FOnReplace write FOnReplace;
  end;


implementation

uses SysUtils, abcfdbme, abcfdbim, abcffind,
     abcconst, abcnpcon, abcutild, abcutilf;

{-------------------------- DATASET FIELD VIEW/EDIT BUTTONS -------------------}

{ TabcnpBlobEdit }

constructor TabcnpBlobEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoEdit := [efMemo, efGraphic];
end;

procedure TabcnpBlobEdit.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  if not Message.LinkActive then CheckEnabled;
end;

procedure TabcnpBlobEdit.CheckEnabled;
var
  bEnabled: Boolean;
begin
  bEnabled := (DataSet <> nil) and (DataSet.Active) and (Field <> nil)
    and (Field.DataSet = DataSet) and (Field.DataType in [ftBlob..ftTypedBinary]);
  if bEnabled then
    case Field.DataType of
      ftMemo:
        bEnabled := efMemo in FAutoEdit;
      ftGraphic:
        bEnabled := efGraphic in FAutoEdit;
      ftBlob:
        bEnabled := efBlob in FAutoEdit;
      ftTypedBinary:
        bEnabled := efBlob in FAutoEdit;
    else
      bEnabled := False;
    end;
  if Assigned(FOnCheckEnabled) then FOnCheckEnabled(Self, Field, bEnabled);
  Enabled := bEnabled;
end;

procedure TabcnpBlobEdit.ValidClick;
begin
  if (Field is TMemoField) and (efMemo in AutoEdit) then
    EditMemo
  else if (Field is TGraphicField) and (efGraphic in AutoEdit) then
    EditGraphic
  else
    EditBlob;
end;

procedure TabcnpBlobEdit.EditBlob;
begin
  case FEditBlobAs of
    ebCustom:
      EditCustomBlob;
    ebMemo:
      EditMemo;
    ebGraphic:
      EditGraphic;
  end;
end;

procedure TabcnpBlobEdit.EditCustomBlob;
var
  Form: TForm;
begin
  if Assigned(FOnCustomBlobEdit) then
    FOnCustomBlobEdit(Self, NavPanelNotifier.ActiveControl, Dataset, Field, PopupFormStyle, ReadOnly, HelpContext, Form);
end;

procedure TabcnpBlobEdit.EditGraphic;
var
  I: Integer;
begin
  {check if form already exists}
  with NavPanelNotifier.ActiveControl do
    for I := 0 to ComponentCount - 1 do
      if Components[I] is TabcafGraphicFieldEdit then begin
        TabcafGraphicFieldEdit(Components[I]).Show;
        Exit;
      end;
  ExecabcafGraphicFieldEdit(NavPanelNotifier.ActiveControl, Dataset, Field, PopupFormStyle, ReadOnly, HelpContext);
end;

procedure TabcnpBlobEdit.EditMemo;
var
  I: Integer;
begin
  {check if form already exists}
  with NavPanelNotifier.ActiveControl do
    for I := 0 to ComponentCount - 1 do
      if Components[I] is TabcafMemoFieldEdit then begin
        TabcafMemoFieldEdit(Components[I]).Show;
        Exit;
      end;
  ExecabcafMemoFieldEdit(NavPanelNotifier.ActiveControl, Dataset, Field, PopupFormStyle, ReadOnly, HelpContext);
end;

function TabcnpBlobEdit.GetDataSource: TDataSource;
begin
  Result := nil;
  if Parent is TabcCustomNavToolbar then
    Result := TabcNavToolbar(Parent).DataSource
  else if Parent is TabcCustomNavPanel then
    Result := TabcNavPanel(Parent).DataSource;
end;

{-------------------------- DATASOURCE VIEW/EDIT BUTTONS -------------------}

{ TabcnpCustomDataSourceEdit }

constructor TabcnpCustomDataSourceEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FModalAction := mfaNone;
  FModalAutoPost := True;
end;

function TabcnpCustomDataSourceEdit.GetForm: TForm;
begin
  if not FormExists then FForm := nil;
  Result := FForm;
end;

procedure TabcnpCustomDataSourceEdit.ValidClick;
begin
  ShowForm;
end;

procedure TabcnpCustomDataSourceEdit.ShowForm;
var
  Cancel: Boolean;
begin
  if not ((Parent is TabcNavToolbar) or (Parent is TabcNavPanel)) then Exit;
  Cancel := False;
  if Assigned(FBeforeShowForm) then FBeforeShowForm(Self, Cancel);
  if Cancel then Exit;
  if FormExists then begin
    FForm.Show;
    if FForm.WindowState = wsMinimized then
      FForm.WindowState := wsNormal;
  end
  else if FFormName = '' then
    ShowDefaultForm
  else
    ShowCustomForm;
end;

procedure TabcnpCustomDataSourceEdit.ShowDefaultForm;
begin
  if PopupFormstyle = pfsModal then begin
    FModalResult := mrNone;
    if Parent is TabcNavToolbar then
      FModalResult := TabcNavToolbar(Parent).
        ShowModalDataSource(Self, DataSource, ReadOnly, HelpContext,
          ModalAction, ModalAutoPost, AfterFormCreate, AfterShowModal)
    else if Parent is TabcNavPanel then
      FModalResult := TabcNavPanel(Parent).
        ShowModalDataSource(Self, DataSource, ReadOnly, HelpContext,
          ModalAction, ModalAutoPost, AfterFormCreate, AfterShowModal);
  end
  else
    if Parent is TabcNavToolbar then
    FForm := TabcNavToolbar(Parent).
      ShowDataSource(Self, DataSource, DataView, PopupFormStyle, ReadOnly,
        HelpContext, AfterFormCreate)
    else if Parent is TabcNavPanel then
      FForm := TabcNavPanel(Parent).
        ShowDataSource(Self, DataSource, DataView, PopupFormStyle, ReadOnly,
          HelpContext, AfterFormCreate);
end;

procedure TabcnpCustomDataSourceEdit.ShowCustomForm;
var
  AClass: TPersistentClass;
begin
  {try to create a custom form class}
  AClass := FindClass(sabcnpclassPrefix + FFormName);
  if not AClass.InheritsFrom(TabcnpAutoForm) then
    raise EabcNavPanelEditException.Create(sabcnpNotAutoForm);
  if Parent is TabcNavToolbar then
    FForm := TabcnpAutoFormClass(AClass).CreateAutoFormTB(Self, TabcNavToolbar(Parent),
      DataSource, PopupFormStyle, ReadOnly, ModalAutoPost, HelpContext, AfterFormCreate)
  else if Parent is TabcNavPanel then
    FForm := TabcnpAutoFormClass(AClass).CreateAutoForm(Self, TabcNavPanel(Parent),
      DataSource, PopupFormStyle, ReadOnly, ModalAutoPost, HelpContext, AfterFormCreate);
  if (PopupFormStyle = pfsModal) then begin
    {modal dataset state}
    FModalResult := mrNone;
    FModalResult := TabcnpAutoForm(FForm).ExecModal(AfterShowModal);
    FForm := nil;
  end
  else begin
    FForm.Show;
    if Parent is TabcNavToolbar then
    begin
      with TabcNavToolbar(Parent) do if AutoDataSource then DataSource := DataSource;
    end
    else if Parent is TabcNavPanel then
    begin
      with TabcNavPanel(Parent) do if AutoDataSource then DataSource := DataSource;
    end;
  end;
end;

procedure TabcnpCustomDataSourceEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
    if (AComponent = FDataSource) then
      FDataSource := nil
    else if (AComponent = FForm) then
      FForm := nil;
end;

function TabcnpCustomDataSourceEdit.FormExists: Boolean;
var
  I: integer;
begin
  Result := False;
  if (FForm = nil) then Exit;
  for I := 0 to ComponentCount - 1 do
    if Components[I] = FForm then begin
      Result := True;
      Break;
    end;
end;

procedure TabcnpCustomDataSourceEdit.SetDataView(Value: TabcnpDataView);
begin
  if Value <> FDataView then begin
    FDataView := Value;
    if Value <> npvRecord then
      if PopupFormStyle = pfsModal then
        PopupFormStyle := pfsNormal;
  end;
end;

procedure TabcnpCustomDataSourceEdit.SetModalAction(Value: TabcnpModalFormAction);
begin
  if Value <> FModalAction then begin
    FModalAction := Value;
    if Value = mfaNone then
      FReadOnly := True
    else
      FReadOnly := False;
  end;
end;

procedure TabcnpCustomDataSourceEdit.SetParent(AParent: TWinControl);
var
  parentChanging, showWarning: boolean;
begin
  parentChanging := (AParent <> Parent);
  inherited SetParent(AParent);
  if not (csDestroying in ComponentState) then
    if (csDesigning in ComponentState) and parentChanging then
    begin
      if (Parent is TabcCustomNavToolbar) then
        showWarning := not TabcNavToolbar(Parent).AutoDataSource
      else if (AParent is TabcCustomNavPanel) then
        showWarning := not TabcNavPanel(Parent).AutoDataSource
      else
        showWarning := False;
      if showWarning then
        MessageDlg(sabcWarningnpCustomDataSourceEditParent,
          mtWarning, [mbOk], 0);
    end;
end;

procedure TabcnpCustomDataSourceEdit.SetReadOnly(Value: Boolean);
begin
  if Value <> FReadOnly then begin
    FReadOnly := Value;
    if Value then FModalAction := mfaNone;
  end;
end;

{ TabcnpDataSourceEdit }

procedure TabcnpDataSourceEdit.ShowForm;
begin
  inherited ShowForm;
end;

{-------------------------- FIELD/RECORD SEARCH/REPLACE -----------------------}

constructor TabcnpFindReplace.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOptionsFind := [nfrDown, nfrShowProgress, nfrFocusField];
  FOptionsReplace := [nfrShowReplace, nfrShowReplaceAll, nfrReplacePost, nfrReplaceFindNext];
end;

procedure TabcnpFindReplace.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  if not Message.LinkActive then CheckEnabled;
end;

procedure TabcnpFindReplace.abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled);
begin
  CheckEnabled;
end;

procedure TabcnpFindReplace.CheckEnabled;
begin
  if DataSet <> FSaveDataSet then begin
    if FForm <> nil then Close;
    FSaveDataSet := DataSet;
  end;
  Enabled := (DataSet <> nil) and (DataSet.Active) and
             not (DataSet.BOF and DataSet.EOF) and
             (Field <> nil) and (Field.DataType in [ftString..ftBytes]);
end;

procedure TabcnpFindReplace.ValidClick;
var
  FoundClass: TabcsfDBFindReplaceClass;
begin
  if (Action = nfaReplace) then
    if abcDataSetReadOnly(DataSet) then
      raise EabcNavPanelEditException.Create(sabcnpErrReplaceReadOnly)
    else if not Field.CanModify and not (nfrAllFields in FOptionsFind) then
      raise EabcNavPanelEditException.Create(sabcnpErrReplaceFieldNoModify);
  if FForm <> nil then
    FForm.BringToFront
  else begin
    FoundClass := TabcsfDBFindReplaceClass(abcFormFactory.GetFormClass(TabcsfDBFindReplace));
    FForm := FoundClass.CreateFindForm(Self);
    FForm.Show;
  end;
end;

procedure TabcnpFindReplace.Close;
begin
  if FForm <> nil then begin
    FForm.Close;
    FForm := nil;
  end;
end;

function TabcnpFindReplace.MatchField(CheckField: TField; CheckCanModify: Boolean): Boolean;
begin
  Result := False;
  with CheckField do begin
    if not Visible then Exit;
    if not (DataType in [ftString..ftBytes]) then Exit;
    if CheckCanModify and not CanModify then Exit;
    if nfrWholeField in FOptionsFind then
      if nfrMatchCase in FOptionsFind then
        Result := (AsString = FFindText)
      else
        Result := (UpperCase(AsString) = UpperCase(FFindText))
    else
      if nfrMatchCase in FOptionsFind then
        Result := (Pos(FFindText, AsString) > 0)
      else
        Result := (Pos(UpperCase(FFindText), UpperCase(AsString)) > 0);
  end;
end;

function TabcnpFindReplace.DoFindNext(var ABookmark: TBookmark; var AField: TField;
  var ACancelled: Boolean; ACanModify: Boolean; IncludeCurrent: Boolean): Boolean;
var
  I, LastIndex: Integer;
  CurrRec: TBookmark;

  procedure IncSearchCount;
  begin
    Inc(FSearchCount);
    if (nfrShowProgress in FOptionsFind) then
      if FForm <> nil then
        with TabcsfDBFindReplace(FForm).lblSearching do begin
          Caption := Format(sabcnpSearch, [FSearchCount]);
          Refresh;
        end;
  end;
begin
  Result := False;
  ACancelled := False;
  if FFindText = '' then Exit;
  with DataSet do
    while not Result do begin
      if (nfrAllFields in FOptionsFind) then begin
        {search current record past current field}
        IncSearchCount;
        if nfrDown in FOptionsFind then
          for I := AField.Index + 1 to FieldCount - 1 do begin
            if MatchField(Fields[I], ACanModify) then begin
              AField := Fields[I];
              Result := True;
              Break;
            end;
          end
        else
          for I := AField.Index - 1 downto 0 do
            if MatchField(Fields[I], ACanModify) then begin
              AField := Fields[I];
              Result := True;
              Break;
            end;
      end;
      if (nfrDown in FOptionsFind) then begin
        Next;
        if EOF then
          if MessageDlg(sabcnpFindEOF,
            mtConfirmation, [mbYes, mbNo], 0) = mrYes then
            First
          else begin
            ACancelled := True;
            Break;
          end;
      end
      else begin
        Prior;
        if BOF then
          if MessageDlg(sabcnpFindBOF,
            mtConfirmation, [mbYes, mbNo], 0) = mrYes then
            Last
          else begin
            ACancelled := True;
            Break;
          end;
      end;
      {check if we've searched all records}
      CurrRec := GetBookmark;
      IncSearchCount;
      try
        if CompareBookmarks(ABookmark, CurrRec) = 0 then begin
          {return true if found in current rec}
          if nfrAllFields in FOptionsFind then
            if nfrDown in FOptionsFind then begin
              LastIndex := AField.Index - 1 + Ord(IncludeCurrent);
              for I := 0 to LastIndex do begin
                if MatchField(Fields[I], ACanModify) then begin
                  AField := Fields[I];
                  Result := True;
                  Break;
                end;
              end
            end
            else begin
              LastIndex := AField.Index + 1 - Ord(IncludeCurrent);
              for I := FieldCount - 1 downto LastIndex do begin
                if MatchField(Fields[I], ACanModify) then begin
                  AField := Fields[I];
                  Result := True;
                  Break;
                end;
              end
            end
          else
            if MatchField(AField, ACanModify) then Result := True;
          Break;
        end;
      finally
        FreeBookmark(CurrRec);
      end;
      {search current record}
      if (nfrAllFields in FOptionsFind) then begin
        if nfrDown in FOptionsFind then
          for I := 0 to FieldCount - 1 do begin
            if MatchField(Fields[I], ACanModify) then begin
              AField := Fields[I];
              FreeBookmark(ABookmark);
              ABookmark := GetBookmark;
              Result := True;
              Break;
            end;
        end
        else
          for I := FieldCount - 1 downto 0 do
            if MatchField(Fields[I], ACanModify) then begin
              AField := Fields[I];
              FreeBookmark(ABookmark);
              ABookmark := GetBookmark;
              Result := True;
              Break;
            end;
      end
      else
        if MatchField(AField, ACanModify) then begin
          FreeBookmark(ABookmark);
          ABookmark := GetBookmark;
          Result := True;
        end;
    end;
  if Result then
    if Assigned(FOnFind) then FOnFind(Self, AField);
end;

procedure TabcnpFindReplace.FocusField(AField: TField);
var
  ParentForm: TCustomForm;
  SelPos: Integer;
begin
  DataSet.EnableControls;
  ParentForm := GetParentForm(Self);
  if ParentForm <> nil then
    if (ParentForm is TForm) and (TForm(ParentForm).FormStyle = fsMDIForm) then
      TForm(ParentForm).BringToFront
    else
      if NavPanelNotifier.ActiveForm <> nil then
        NavPanelNotifier.ActiveForm.BringToFront;
  with NavPanelNotifier do begin
    if ActiveControl is TCustomDBGrid then
      with TCustomDBGrid(ActiveControl) do begin
        SelectedField := AField;
        SetFocus;
      end
    else if ActiveControl is TabcCustomHDBGrid then
      with TabcCustomHDBGrid(ActiveControl) do begin
        SelectedField := AField;
        SetFocus;
      end
{$IFDEF INFOPOWER}
    else if ActiveControl is TwwCustomDBGrid then
      with TwwCustomDBGrid(ActiveControl) do begin
        SelectedField := AField;
        SetFocus;
      end
{$ENDIF}
    else begin
      AField.FocusControl;
      if Screen.ActiveControl is TCustomEdit then
        with TCustomEdit(Screen.ActiveControl) do begin
          SelPos := Pos(FindText, Text);
          if SelPos > 0 then begin
            SelStart := SelPos - 1;
            SelLength := Length(FindText);
          end;
        end;
    end;
  end;
end;

function TabcnpFindReplace.Find: Boolean;
var
  CurrBookmark: TBookmark;
  CurrField: TField;
  SearchCancelled: Boolean;
begin
  Result := False;
  FSearchCount := 0;
  if (nfrShowProgress in FOptionsFind) and (FForm <> nil) then
    with TabcsfDBFindReplace(FForm).lblSearching do begin
      Caption := sabcnpSearching;
      Visible := True;
    end;
  if FFindText = '' then Exit;
  with DataSet do begin
    CurrBookmark := GetBookmark;
    DisableControls;
    CurrField := Field;
    try
      if FFirstFind then begin
        FFirstFind := False;
        Result := MatchField(CurrField, True);
      end
      else
        Result := False;
      if not Result then begin
        Result := DoFindNext(CurrBookmark, CurrField, SearchCancelled, False, True);
        GoToBookmark(CurrBookmark);
      end;
      if Result then begin
        if (nfrFocusField in FOptionsFind) then FocusField(CurrField);
      end
      else
        if not SearchCancelled then
          if (nfrAllFields in FOptionsFind) then
            MessageDlg(Format(sabcnpFindNotFound, [FFindText]),
              mtWarning, [mbOK], 0)
          else
            MessageDlg(Format(sabcnpFindNotFoundField, [FFindText]),
              mtWarning, [mbOK], 0);
    finally
      FreeBookmark(CurrBookmark);
      EnableControls;
      if (FForm <> nil) then
        TabcsfDBFindReplace(FForm).lblSearching.Visible := False;
    end;
  end;
end;

function TabcnpFindReplace.Replace: Boolean;
var
  CurrBookmark: TBookmark;
  CurrField: TField;
  SearchCancelled: Boolean;
  IsFound: Boolean;
begin
  Result := False;
  FSearchCount := 0;
  if (nfrShowProgress in FOptionsFind) and (FForm <> nil) then
    with TabcsfDBFindReplace(FForm).lblSearching do begin
      Caption := sabcnpSearching;
      Visible := True;
    end;
  if FFindText = '' then Exit;
  with DataSet do begin
    CurrBookmark := GetBookmark;
    DisableControls;
    CurrField := Field;
    try
      IsFound := MatchField(CurrField, True);
      if not IsFound then
        IsFound := DoFindNext(CurrBookmark, CurrField, SearchCancelled, True, True);
      if not IsFound then begin
        if not SearchCancelled then
          if (nfrAllFields in FOptionsFind) then
            MessageDlg(Format(sabcnpFindNotFound, [FFindText]),
              mtWarning, [mbOK], 0)
          else
            MessageDlg(Format(sabcnpFindNotFoundField, [FFindText]),
              mtWarning, [mbOK], 0);
      end
      else begin
        GoToBookmark(CurrBookmark);
        Edit;
        {whole field or partial replace}
        with CurrField do
          if nfrWholeField in FOptionsFind then
            AsString := FReplaceText
          else
            AsString := abcReplaceStr(FFindText, FReplaceText, AsString,
                  (nfrMatchCase in FOptionsFind));
        Result := True;
        if Assigned(FOnReplace) then FOnReplace(Self, CurrField);
        if (nfrReplacePost in FOptionsReplace) and (State = dsEdit) then Post;
        if (nfrReplaceFindNext in FOptionsReplace) then begin
          IsFound := DoFindNext(CurrBookmark, CurrField, SearchCancelled, True, True);
          if IsFound then begin
            GoToBookmark(CurrBookmark);
            FocusField(CurrField);
          end
          else
          if not SearchCancelled then
            if (nfrAllFields in FOptionsFind) then
              MessageDlg(Format(sabcnpFindNotFound, [FFindText]),
                mtWarning, [mbOK], 0)
            else
              MessageDlg(Format(sabcnpFindNotFoundField, [FFindText]),
                mtWarning, [mbOK], 0);
            end;
      end;
      if (nfrFocusField in FOptionsFind) then FocusField(CurrField);
    finally
      FreeBookmark(CurrBookmark);
      EnableControls;
      if (FForm <> nil) then
        TabcsfDBFindReplace(FForm).lblSearching.Visible := False;
    end;
  end;
end;

function TabcnpFindReplace.ReplaceAll: Integer;
var
  CurrBookmark: TBookmark;
  CurrField: TField;
  SearchCancelled: Boolean;
  IsFound: Boolean;
begin
  Result := 0;
  FSearchCount := 0;
  if (nfrShowProgress in FOptionsFind) and (FForm <> nil) then
    with TabcsfDBFindReplace(FForm).lblSearching do begin
      Caption := sabcnpSearching;
      Visible := True;
    end;
  if FFindText = '' then Exit;
  with DataSet do begin
    CurrBookmark := GetBookmark;
    DisableControls;
    CurrField := Field;
    try
      IsFound := MatchField(CurrField, True);
      if not IsFound then
        IsFound := DoFindNext(CurrBookmark, CurrField, SearchCancelled, True, True);
      if not IsFound then
        if not SearchCancelled then
          if (nfrAllFields in FOptionsFind) then
            MessageDlg(Format(sabcnpFindNotFound, [FFindText]),
              mtWarning, [mbOK], 0)
          else
            MessageDlg(Format(sabcnpFindNotFoundField, [FFindText]),
              mtWarning, [mbOK], 0);
      while IsFound do begin
        GoToBookmark(CurrBookmark);
        Edit;
        {whole field or partial replace}
        with CurrField do
          if nfrWholeField in FOptionsFind then
            AsString := FReplaceText
          else
            AsString := abcReplaceStr(FFindText, FReplaceText, AsString,
                  (nfrMatchCase in FOptionsFind));
        Inc(Result);
        if Assigned(FOnReplace) then FOnReplace(Self, CurrField);
        if (nfrReplacePost in FOptionsReplace) and (State = dsEdit) then Post;
        IsFound := DoFindNext(CurrBookmark, CurrField, SearchCancelled, True, False);
      end;
      if (nfrFocusField in FOptionsFind) then FocusField(CurrField);
    finally
      FreeBookmark(CurrBookmark);
      EnableControls;
      if (FForm <> nil) then
        TabcsfDBFindReplace(FForm).lblSearching.Visible := False;
    end;
  end;
end;

procedure TabcnpFindReplace.SetFindText(Value: string);
begin
  if Value <> FFindText then begin
    FFindText := Value;
    FFirstFind := True;
  end;
end;

procedure TabcnpFindReplace.SetOptionsReplace(Value: TabcnpReplaceOptions);
begin
  if Value <> FOptionsReplace then begin
    FOptionsReplace := Value;
    if nfrReplaceFindNext in FOptionsReplace then
      FOptionsReplace := FOptionsReplace + [nfrReplacePost];
  end;
end;

end.
