unit abcnpfac;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{ abstract factory singletons for ABC }

{$I abcdefs.inc}

interface

uses SysUtils, Classes, Forms, DB,
     abcnp, abcforms;

type

  { TabcnpFormFactory }

  { This class allows you to replace abc forms
    with your own custom descendants using its RegisterFormClass method }

  TabcnpFormFactory = class(TObject)
  private
    FFormClasses: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    function GetAutoFormClass(Ancestor: TabcnpAutoFormClass): TabcnpAutoFormClass;
    function GetFormClass(Ancestor: TFormClass): TFormClass;
    function CreateFormOfClass(Ancestor: TFormClass; AOwner: TComponent): TForm;

    function CreateAutoFormOfClass(
      Ancestor: TabcnpAutoFormClass;
      AOwner: TComponent; ANavPanel: TabcNavPanel;
      ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
      AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext;
      AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm;
    function CreateAutoFormTBOfClass(
      Ancestor: TabcnpAutoFormClass;
      AOwner: TComponent; ANavToolbar: TabcNavToolbar;
      ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
      AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext;
      AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm;
    procedure RegisterBaseFormClass(Ancestor: TFormClass);
    procedure RegisterFormClass(Ancestor, Descendant: TFormClass);
    procedure UnRegisterFormClass(Ancestor: TFormClass);
  end;

function abcnpFormFactory: TabcnpFormFactory;

implementation

var
  GlobalabcnpFormFactory: TabcnpFormFactory;

function abcnpFormFactory: TabcnpFormFactory;
begin
  if GlobalabcnpFormFactory = nil then
    GlobalabcnpFormFactory := TabcnpFormFactory.Create;
  Result := GlobalabcnpFormFactory;
end;

constructor TabcnpFormFactory.Create;
begin
  FFormClasses := TStringList.Create;
  FFormClasses.Sorted := True;
end;

destructor TabcnpFormFactory.Destroy;
begin
  FFormClasses.Free;
  inherited Destroy;
  GlobalabcnpFormFactory := nil;
end;

function TabcnpFormFactory.CreateFormOfClass(Ancestor: TFormClass; AOwner: TComponent): TForm;
var
  FoundClass: TFormClass;
begin
  FoundClass := GetFormClass(Ancestor);
  if FoundClass = nil then
    Result := nil
  else
    Result :=  FoundClass.Create(AOwner);
end;


function TabcnpFormFactory.CreateAutoFormOfClass(
  Ancestor: TabcnpAutoFormClass;
  AOwner: TComponent; ANavPanel: TabcNavPanel;
  ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
  AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext;
  AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm;
var
  FoundClass: TabcnpAutoFormClass;
begin
  FoundClass := GetAutoFormClass(Ancestor);
  if FoundClass = nil then
    Result := nil
  else
    Result := FoundClass.CreateAutoForm(
      AOwner, ANavPanel, ADataSource, APopupStyle, AReadOnly,
      AModalAutoPost, AHelpContext, AfterFormCreate);
end;

function TabcnpFormFactory.CreateAutoFormTBOfClass(
  Ancestor: TabcnpAutoFormClass;
  AOwner: TComponent; ANavToolbar: TabcNavToolbar;
  ADataSource: TDataSource; APopupStyle: TabcPopupFormStyle;
  AReadOnly: Boolean; AModalAutoPost: Boolean; AHelpContext: THelpContext;
  AfterFormCreate: TabcnpAfterFormCreateEvent): TabcnpAutoForm;
var
  FoundClass: TabcnpAutoFormClass;
begin
  FoundClass := GetAutoFormClass(Ancestor);
  if FoundClass = nil then
    Result := nil
  else
    Result := FoundClass.CreateAutoFormTB(
      AOwner, ANavToolbar, ADataSource, APopupStyle, AReadOnly,
      AModalAutoPost, AHelpContext, AfterFormCreate);
end;

function TabcnpFormFactory.GetAutoFormClass(Ancestor: TabcnpAutoFormClass): TabcnpAutoFormClass;
var
  i: integer;
begin
  i := FFormClasses.IndexOf(Ancestor.ClassName);
  if i = -1 then
    Result := nil
  else
    Result := TabcnpAutoFormClass(FFormClasses.Objects[i]);
end;

function TabcnpFormFactory.GetFormClass(Ancestor: TFormClass): TFormClass;
var
  i: integer;
begin
  i := FFormClasses.IndexOf(Ancestor.ClassName);
  if i = -1 then
    Result := nil
  else
    Result := TFormClass(FFormClasses.Objects[i]);
end;

procedure TabcnpFormFactory.RegisterBaseFormClass(Ancestor: TFormClass);
begin
  RegisterFormClass(Ancestor, Ancestor);
end;

procedure TabcnpFormFactory.RegisterFormClass(Ancestor, Descendant: TFormClass);
begin
  if not Descendant.InheritsFrom(Ancestor) then
    raise Exception.Create(
      'Form Factory error: ' + Descendant.ClassName +
      ' does not inherit from ' + Ancestor.ClassName);
  UnRegisterFormClass(Ancestor);
{$IFDEF DELPHI6}
  ActivateClassGroup(TForm);
{$ENDIF}
  RegisterClass(Descendant);
  FFormClasses.AddObject(Ancestor.ClassName, TObject(Descendant));
end;

procedure TabcnpFormFactory.UnRegisterFormClass(Ancestor: TFormClass);
var
  i: integer;
  Removed: TFormClass;
begin
  i := FFormClasses.IndexOf(Ancestor.ClassName);
  if i >= 0 then
  begin
    Removed := TFormClass(FFormClasses.Objects[i]);
    FFormClasses.Delete(i);
    UnregisterClass(Removed);
  end;
end;

initialization

finalization
  GlobalabcnpFormFactory.Free;

end.
