unit abcnpflt;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}


interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Menus, Forms, dialogs, 
  StdCtrls, ExtCtrls, DB, Buttons, abcexctl, abcnp, abcnpbtn,
  abcfiltr, abcdbrng, abcda;

type

  { TabcnpBookmark }

  TabcnpBookmark = class(TabcCustomnpBtn)
  private
    FBookmark: TBookmark;
    FHintDown: string;
    FHintUp: string;
    function GetHint: string;
    procedure SetHintDown(Value: string);
    procedure SetHintUp(Value: string);
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
  published
    property Hint: string read GetHint; {remove Hint from OI}
    property HintDown: string read FHintDown write SetHintDown;
    property HintUp: string read FHintUp write SetHintUp;
  end;

  { TabcCustomnpBookmarkMenu }

  TabcMenuItemBreakRange = 0..50;

  TabcCustomnpBookmarkMenu = class(TabcCustomnpRecEditBtn)
  private
    FBookmarks: TList;
    FBreakAtItem: TabcMenuItemBreakRange;
    FCurrentRecItem: Integer;
    FLabelByIndexVals: Boolean;
    FMenu: TPopupMenu;
    FMenuStrings: TStrings;
    FmiGotoBmkSubMenu, FmiAddBmk, FmiRemoveBmkSubMenu, FmiRemoveAll: TMenuItem;
    function GetBookmark(Index: Integer): TBookmark;
    function GetCount: Integer;
    function GetMenuAlign: TPopupAlignment;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
    procedure abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled); message ABC_CD_DATASETSCROLLED;
    procedure CheckMenuItems;
    procedure GotoBookmark(Index: Integer);
    procedure InitMenuStrings;
    procedure miAddBmkClick(Sender: TObject);
    procedure miGoToSubMenuItemClick(Sender: TObject);
    procedure miRemoveAllClick(Sender: TObject);
    procedure miRemoveBmkSubMenuClick(Sender: TObject);
    procedure PopupAt(APoint: TPoint);
    procedure ResetBarBreaks;
    procedure SetBreakAtItem(Value: TabcMenuItemBreakRange);
    procedure SetMenuAlign(Value: TPopupAlignment);
    procedure SetMenuStrings(Value: TStrings);
  protected
    procedure CheckEnabled; override;
    procedure CreatePopupMenu; dynamic;
    procedure Loaded; override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    property Count: Integer read GetCount;
    property Bookmark[Index: Integer]: TBookmark read GetBookmark;
    property BreakAtItem: TabcMenuItemBreakRange read FBreakAtItem
      write SetBreakAtItem default 10;
    property LabelByIndexVals: Boolean read FLabelByIndexVals
      write FLabelByIndexVals default True;
    property MenuAlign: TPopupAlignment read GetMenuAlign write SetMenuAlign
      default paLeft;
    property MenuStrings: TStrings read FMenuStrings write SetMenuStrings;
  public
    procedure AddBookmark;
    procedure RemoveBookmark(Index: Integer);
    procedure CheckAllBookmarks;
    procedure FreeAllBookmarks;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
  end;

  { TabcnpBookmarkMenu }

  TabcnpBookmarkMenu = class(TabcCustomnpBookmarkMenu)
  public
    property Count;
    property Bookmark;
  published
    property BreakAtItem;
    property LabelByIndexVals;
    property MenuAlign;
    property MenuStrings;
  end;

  { TabcCustomnpIndexCombo }

  EabcInvalidIndexComboItem = class(Exception);
  EabcInvalidIndexComboOperation = class(Exception);

  TabcCustomnpIndexCombo = class(TCustomComboBox)
  private
    FAutoLoadItems: Boolean;
    FGoToFirst: Boolean;
    FItemIndex: Integer;
    FIxName: string;
    FIxFieldNames: string;
    FIxFieldCount: Integer;
    FOnIndexChange: TNotifyEvent;
    FDataset: TDataset;
    FLastIndexedDataset: TDataset;
    FAdapter: TabcDatasetAdapter;
    FTableIndexItems: TStrings;
    FIndexChanging: Boolean;
    FComboChanging: Boolean;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
    procedure SetDataset(Value: TDataset);
    procedure CheckEnabled;
    function CreateInheritedItems(Value: TStrings; var InheritedStrings: TStrings): Boolean;
    function GetSorted: Boolean;
    procedure IndexChanged;
    function ParseItem(Item: string; var strPart: string; var Index: integer): Boolean;
{$IFNDEF DELPHI6}
    procedure SetItemIndex(const Value: Integer);
{$ENDIF}
    procedure SetInheritedItems;
    procedure SetSorted(Value: Boolean);
    procedure SetTableIndexItems(Value: TStrings);
    procedure UpdateFocus;
  protected
    procedure Change; override;
    procedure Loaded; override;
    function LoadItemsFromDataSet: Integer;
{$IFDEF DELPHI6}
    function GetItemIndex: Integer; override;
    procedure SetItemIndex(const Value: Integer); override;
{$ENDIF}
    procedure SetParent(AParent: TWinControl); override;
    property AutoLoadItems: Boolean read FAutoLoadItems write FAutoLoadItems default True;
    property GoToFirst: Boolean read FGoToFirst write FGoToFirst default True;
{$IFDEF DELPHI6}
    property ItemIndex default -1;
{$ELSE}
    property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
{$ENDIF}
    property Sorted: Boolean read GetSorted write SetSorted default False;
    property TableIndexItems: TStrings read FTableIndexItems write SetTableIndexItems;
    property OnIndexChange: TNotifyEvent read FOnIndexChange write FOnIndexChange;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TabcnpIndexCombo }

  TabcnpIndexCombo = class(TabcCustomnpIndexCombo)
  private
    function GetText: string;
  public
    function LoadItemsFromDataSet: Integer;
    property Text: string read GetText;
  published
    {property Style; NOT published}
{$IFDEF DELPHI6}
    property AutoComplete default True;
    property AutoDropDown default False;
{$ENDIF}
    property AutoLoadItems;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property GoToFirst;
    property Anchors;
    property BiDiMode;
{$IFDEF DELPHI6}
    property CharCase;
{$ENDIF}
    property Color;
    property Constraints;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    {property Enabled;}
    property Font;
    property ItemHeight;
    property ItemIndex;
    {property MaxLength;}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TableIndexItems;
    property TabOrder;
    property TabStop;
    { property Text;   NOT published, made public, read-only}
    property Visible;
    property OnChange;
    property OnClick;
{$IFDEF DELPHI6}
    property OnCloseUp;
{$ENDIF}
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
{$IFDEF DELPHI6}
    property OnSelect;
{$ENDIF}
  end;

  { TabcCustomnpRangeEdit }

  TabcCustomnpRangeEdit = class(TCustomEdit)
  private
    FGoToFirst: Boolean;
    FIxName: string;
    FIxFieldNames: string;
    FIxFieldCount: Integer;
    FIxFieldType: TFieldType;
    FSeparator: string;
    FDataset: TDataset;
    FAdapter: TabcDatasetAdapter;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
    procedure SetDataset(Value: TDataset);
    procedure CheckEnabled;
    procedure IndexChanged;
    procedure UpdateFocus;
  protected
    procedure KeyPress(var Key: Char); override;
    procedure SetParent(AParent: TWinControl); override;
    property GoToFirst: Boolean read FGoToFirst write FGoToFirst default True;
    property Separator: string read FSeparator write FSeparator;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ApplyRange;
  end;

  TabcnpRangeEdit = class(TabcCustomnpRangeEdit)
  published
    property GoToFirst;
    property Separator;
    property AutoSelect;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    {property Enabled;}
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TabcnpFilterExpression }

  TabcnpFilterExpression = class(TabcCustomnpBtn)
  private
    FAllowEdit: Boolean;
    FFilter: TabcDBDataSetFilter;
    FHintDown: string;
    FHintUp: string;
    FLoadedFilter: string;
    FLoadedActive: Boolean;
    FOnCheckEnabled: TabcDataSetCheckEnabledEvent;
    FOnEdited: TNotifyEvent;
    FOnActivate: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
    procedure CheckButtonState(Sender: TObject);
    function GetActive: Boolean;
    procedure SetActive(Value: Boolean);
    function GetDown: Boolean;
    function GetFilter: string;
    function GetFilterOptions: TFilterOptions;
    function GetGoToFirst: Boolean;
    function GetHelpContext: THelpContext;
    function GetHint: string;
    function GetOnFilterRecord: TabcFilterRecordEvent;
    procedure SetDown(Value: Boolean);
    procedure SetFilter(Value: string);
    procedure SetFilterOptions(Value: TFilterOptions);
    procedure SetGoToFirst(Value: Boolean);
    procedure SetHelpContext(Value: THelpContext);
    procedure SetHintDown(Value: string);
    procedure SetHintUp(Value: string);
    procedure SetOnFilterRecord(Value: TabcFilterRecordEvent);
    procedure DoFilterActiveChangeEvents;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    property Down: Boolean read GetDown write SetDown;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    function Edit: Boolean;
    function ParamByName(const FieldName: string): TParam;
    property DatasetFilter: TabcDBDataSetFilter read FFilter;
  published
    property Active: Boolean read GetActive write SetActive default False;
    property AllowEdit: Boolean read FAllowEdit write FAllowEdit default True;
    property Filter: string read GetFilter write SetFilter;
    property FilterOptions: TFilterOptions read GetFilterOptions write SetFilterOptions default [];
    property GoToFirst: Boolean read GetGoToFirst write SetGoToFirst default True;
    property HelpContext: THelpContext read GetHelpContext write SetHelpContext;
    property HintDown: string read FHintDown write SetHintDown;
    property HintUp: string read FHintUp write SetHintUp;
    property Hint: string read GetHint; {remove Hint from OI}
    property OnCheckEnabled: TabcDataSetCheckEnabledEvent read FOnCheckEnabled write FOnCheckEnabled;
    property OnFilterRecord: TabcFilterRecordEvent read GetOnFilterRecord write SetOnFilterRecord;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnEdited: TNotifyEvent read FOnEdited write FOnEdited;
  end;

  { TabcnpFilterActive }

  TabcnpFilterActive = class(TabcnpActiveFieldBtn)
  private
    FFilter: TabcDBDataSetFilter;
    FHintDown: string;
    FHintUp: string;
    FOnNoFilter: TNotifyEvent;
    FFilterChanging: Boolean;
    function ApplyFilter: Boolean;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
    function GetDown: Boolean;
    function GetFilterOptions: TFilterOptions;
    function GetGoToFirst: Boolean;
    function GetHint: string;
    procedure SetDown(Value: Boolean);
    procedure SetFilterOptions(Value: TFilterOptions);
    procedure SetGoToFirst(Value: Boolean);
    procedure SetHintDown(Value: string);
    procedure SetHintUp(Value: string);
    procedure CheckButtonState(Sender: TObject);
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
    property Down: Boolean read GetDown write SetDown;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    property FilterOptions: TFilterOptions read GetFilterOptions write SetFilterOptions default [];
    property GoToFirst: Boolean read GetGoToFirst write SetGoToFirst default True;
    property HintDown: string read FHintDown write SetHintDown;
    property HintUp: string read FHintUp write SetHintUp;
    property Hint: string read GetHint; {remove Hint from OI}
    property OnNoFilter: TNotifyEvent read FOnNoFilter write FOnNoFilter;
  end;

{-------------------------- FILTER, SEARCH ETC BUTTONS ------------------------}

  { TabcnpRange }

  TabcnpRange = class(TabcCustomnpRecEditBtn)
  private
    FDlgHelpButton: Boolean;
    FDlgHelpContext: THelpContext;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property DlgHelpButton: Boolean read FDlgHelpButton
      write FDlgHelpButton default False;
    property DlgHelpContext: THelpContext read FDlgHelpContext
      write FDlgHelpContext default 0;
  end;

  { TabcnpFindKey }

  TabcnpFindKey = class(TabcCustomnpRecEditBtn)
  private
    FDlgHelpButton: Boolean;
    FDlgHelpContext: THelpContext;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property DlgHelpButton: Boolean read FDlgHelpButton
      write FDlgHelpButton default False;
    property DlgHelpContext: THelpContext read FDlgHelpContext
      write FDlgHelpContext default 0;
  end;

  { TabcnpQueryParams }

  TabcnpQueryParams = class(TabcCustomnpBtn)
  private
    FDlgHelpButton: Boolean;
    FDlgHelpContext: THelpContext;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  published
    property DlgHelpButton: Boolean read FDlgHelpButton
      write FDlgHelpButton default False;
    property DlgHelpContext: THelpContext read FDlgHelpContext
      write FDlgHelpContext default 0;
  end;


implementation

uses abcconst, abcnpcon, abcutild, abcutilf, abcfrnge, abcffndk, abcfqryp;

{ TabcnpBookmark }

constructor TabcnpBookmark.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  GroupIndex := GetTickCount; {try to get a unique group}
  AllowAllUp := True;
  Down := False;
end;

procedure TabcnpBookmark.CheckEnabled;
begin
  with Dataset do
    Enabled := (DataSet <> nil) and Active and not (BOF and EOF) {$IFDEF DELPHI6} and not IsUniDirectional {$ENDIF};
end;

procedure TabcnpBookmark.Click;
begin
  inherited Down := not inherited Down;
  inherited Click;
end;

procedure TabcnpBookmark.ValidClick;
begin
  inherited Down := not inherited Down;
  if Down then begin
    FBookmark := DataSet.GetBookmark;
    inherited Hint := HintDown;
  end
  else begin
    DataSet.GotoBookmark(FBookmark);
    DataSet.FreeBookmark(FBookmark);
    inherited Hint := HintUp;
  end;
  NavPanelNotifier.FocusActiveControl;
end;

function TabcnpBookmark.GetHint: string;
begin
  Result := inherited Hint;
end;

procedure TabcnpBookmark.SetHintDown(Value: string);
begin
  if Value <> FHintDown then begin
    FHintDown := Value;
    if Down then inherited Hint := Value;
  end;
end;

procedure TabcnpBookmark.SetHintUp(Value: string);
begin
  if Value <> FHintUp then begin
    FHintUp := Value;
    if not Down then inherited Hint := Value;
  end;
end;

{ TabcCustomnpBookmarkMenu }

constructor TabcCustomnpBookmarkMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBreakAtItem := 10;
  FCurrentRecItem := -1;
  FLabelByIndexVals := True;
  FBookmarks := TList.Create;
  FMenuStrings := TStringList.Create;
  with FMenuStrings do begin
    Add(sabcnpBmkGoTo);
    Add(sabcnpBmkAdd);
    Add(sabcnpBmkRemove);
    Add(sabcnpBmkRemoveAll);
  end;
  CreatePopupMenu;
end;

destructor TabcCustomnpBookmarkMenu.Destroy;
begin
  FreeAllBookmarks;
  FmiRemoveAll.Free;
  FmiRemoveAll := nil;
  FmiRemoveBmkSubMenu.Free;
  FmiRemoveBmkSubMenu := nil;
  FmiAddBmk.Free;
  FmiAddBmk := nil;
  FmiGotoBmkSubMenu.Free;
  FmiGotoBmkSubMenu := nil;
  FMenu.Free;
  FMenu := nil;
  FBookmarks.Free;
  FBookmarks := nil;
  FMenuStrings.Free;
  FMenuStrings := nil;
  inherited Destroy;
end;

procedure TabcCustomnpBookmarkMenu.AddBookmark;
var
  Current: TBookmark;
  NewMenuItem: TMenuItem;
  NewItemCaption: string;
  I: integer;
begin
  {get bookmark and add to list}
  Current := DataSet.GetBookmark;
  FBookmarks.Add(Current);
  {create a new menu item for the bookmark}
  if FLabelByIndexVals then
    NewItemCaption := Adapter.IndexValue
  else
    NewItemCaption := sabcnpBmk + IntToStr(FBookmarks.Count);
  NewMenuItem := NewItem(NewItemCaption, 0, False, True, miGoToSubMenuItemClick, 0, '');
  FmiGotoBmkSubMenu.Add(NewMenuItem);
  NewMenuItem := NewItem(NewItemCaption, 0, False, True, miRemoveBmkSubMenuClick, 0, '');
  FmiRemoveBmkSubMenu.Add(NewMenuItem);
  I := FmiGotoBmkSubMenu.Count - 1;
  if (I > 0) and (I mod FBreakAtItem = 0) then begin
    FmiGotoBmkSubMenu.Items[I].Break := mbBarBreak;
    FmiRemoveBmkSubMenu.Items[I].Break := mbBarBreak;
  end;
end;

procedure TabcCustomnpBookmarkMenu.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  FreeAllBookmarks;
end;

procedure TabcCustomnpBookmarkMenu.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  inherited;
  CheckAllBookmarks;
end;

procedure TabcCustomnpBookmarkMenu.abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled);
begin
  inherited;
  if FCurrentRecItem <> -1 then begin
    FmiGotoBmkSubMenu.Items[FCurrentRecItem].Checked := False;
    FmiRemoveBmkSubMenu.Items[FCurrentRecItem].Checked := False;
    FCurrentRecItem := -1;
  end;
end;

procedure TabcCustomnpBookmarkMenu.CheckEnabled;
begin
  with Dataset do
    Enabled := (DataSet <> nil) and Active and (State = dsBrowse) {$IFDEF DELPHI6} and not IsUniDirectional {$ENDIF};
end;

procedure TabcCustomnpBookmarkMenu.CheckMenuItems;
var
  I: Integer;
  CheckBookmark: TBookmark;
begin
  if FBookmarks.Count = 0 then begin
    FmiGotoBmkSubMenu.Enabled := False;
    FmiAddBmk.Enabled := True;
    FmiRemoveBmkSubMenu.Enabled := False;
    FmiRemoveAll.Enabled := False;
  end
  else begin
    FmiAddBmk.Enabled := True;
    FmiRemoveBmkSubMenu.Enabled := True;
    FmiRemoveAll.Enabled := True;
    FmiGotoBmkSubMenu.Enabled := True;
    {Is current record an existing bookmark?}
    if FCurrentRecItem <> -1 then begin
      FmiAddBmk.Enabled := False;
      FmiGotoBmkSubMenu.Items[FCurrentRecItem].Checked := True;
      FmiRemoveBmkSubMenu.Items[FCurrentRecItem].Checked := True;
    end
    else begin
      {search for item}
      CheckBookmark := DataSet.GetBookmark;
      try
        for I := 0 to FBookmarks.Count - 1 do begin
          if DataSet.CompareBookmarks(CheckBookmark, FBookmarks.Items[I]) = 0 then begin
            FCurrentRecItem := I;
            FmiAddBmk.Enabled := False;
            FmiGotoBmkSubMenu.Items[I].Checked := True;
            FmiRemoveBmkSubMenu.Items[I].Checked := True;
            Break;
          end;
        end;
      finally
        DataSet.FreeBookmark(CheckBookmark);
      end;
    end;
  end;
end;

procedure TabcCustomnpBookmarkMenu.CreatePopupMenu;
var
  DummyItem1, DummyItem2: TMenuItem;
begin
  DummyItem1 := NewItem('', 0, False, False, nil, 0, sabcnpBmkDummy1);
  DummyItem2 := NewItem('', 0, False, False, nil, 0, sabcnpBmkDummy2);
  FmiGotoBmkSubMenu := NewSubMenu(FMenuStrings.Strings[0], 0, sabcnpBmkSub, [DummyItem1]);
  FmiAddBmk := NewItem(FMenuStrings.Strings[1], 0, False, True, miAddBmkClick, 0, sabcnpBmkSubAdd);
  FmiRemoveBmkSubMenu := NewSubMenu(FMenuStrings.Strings[2], 0, sabcnpBmkSubRemove, [DummyItem2]);
  FmiRemoveAll := NewItem(FMenuStrings.Strings[3], 0, False, False, miRemoveAllClick, 0, sabcnpBmkSubRemoveAll);
  FMenu := NewPopupMenu(Self, Name + sabcnpBmkPopup, paLeft, False,
      [FmiGotoBmkSubMenu, FmiAddBmk, FmiRemoveBmkSubMenu, FmiRemoveAll]);
  FmiGotoBmkSubMenu.Remove(DummyItem1);
  FmiRemoveBmkSubMenu.Remove(DummyItem2);
  DummyItem1.Free;
  DummyItem2.Free;
end;


procedure TabcCustomnpBookmarkMenu.CheckAllBookmarks;
var
  I: integer;
begin
  if (DataSet <> nil) and DataSet.Active and (FmiGotoBmkSubMenu <> nil) then
  begin
    for I := FmiGotoBmkSubMenu.Count - 1 downto 0 do
      if not DataSet.BookmarkValid(GetBookmark(I)) then
        RemoveBookmark(I);
  end
  else
    FreeAllBookmarks;
  FCurrentRecItem := -1;
end;

procedure TabcCustomnpBookmarkMenu.FreeAllBookmarks;
var
  I: integer;
  MenuItem: TMenuItem;
begin
  if (FmiGotoBmkSubMenu <> nil) then
    for I := FmiGotoBmkSubMenu.Count - 1 downto 0 do
      if (DataSet <> nil) and DataSet.Active then
        RemoveBookmark(I)
      else begin
        MenuItem := FmiGotoBmkSubMenu.Items[I];
        FmiGotoBmkSubMenu.Remove(MenuItem);
        MenuItem.Free;
        MenuItem := FmiRemoveBmkSubMenu.Items[I];
        FmiRemoveBmkSubMenu.Remove(MenuItem);
        MenuItem.Free;
      end;
  FBookmarks.Clear;
  FCurrentRecItem := -1;
end;

function TabcCustomnpBookmarkMenu.GetBookmark(Index: Integer): TBookmark;
begin
  if (Index < 0) or (Index > FBookmarks.Count - 1) then
    Result := nil
  else
    Result := FBookmarks.Items[Index];
end;

function TabcCustomnpBookmarkMenu.GetCount: Integer;
begin
  Result := FBookmarks.Count;
end;

function TabcCustomnpBookmarkMenu.GetMenuAlign: TPopupAlignment;
begin
  Result := FMenu.Alignment;
end;

procedure TabcCustomnpBookmarkMenu.GotoBookmark(Index: Integer);
begin
  DataSet.GoToBookmark(FBookmarks.Items[Index]);
end;

procedure TabcCustomnpBookmarkMenu.InitMenuStrings;
begin
  FmiGotoBmkSubMenu.Caption := FMenuStrings.Strings[0];
  FmiAddBmk.Caption := FMenuStrings.Strings[1];
  FmiRemoveBmkSubMenu.Caption := FMenuStrings.Strings[2];
  FmiRemoveAll.Caption := FMenuStrings.Strings[3];
end;

procedure TabcCustomnpBookmarkMenu.Loaded;
begin
  inherited Loaded;
  InitMenuStrings;
end;

procedure TabcCustomnpBookmarkMenu.miAddBmkClick(Sender: TObject);
begin
  AddBookmark;
  NavPanelNotifier.FocusActiveControl;
end;

procedure TabcCustomnpBookmarkMenu.miGoToSubMenuItemClick(Sender: TObject);
begin
  if FCurrentRecItem <> -1 then begin
    FmiGotoBmkSubMenu.Items[FCurrentRecItem].Checked := False;
    FmiRemoveBmkSubMenu.Items[FCurrentRecItem].Checked := False;
    FCurrentRecItem := -1;
  end;
  {find the menu item that was clicked}
  if (Sender is TMenuItem) then
    FCurrentRecItem := FmiGotoBmkSubMenu.IndexOf(TMenuItem(Sender));
  if FCurrentRecItem <> -1 then GotoBookmark(FCurrentRecItem);
  NavPanelNotifier.FocusActiveControl;
end;

procedure TabcCustomnpBookmarkMenu.miRemoveAllClick(Sender: TObject);
begin
  FreeAllBookmarks;
  NavPanelNotifier.FocusActiveControl;
end;

procedure TabcCustomnpBookmarkMenu.miRemoveBmkSubMenuClick(Sender: TObject);
begin
  if FCurrentRecItem <> -1 then begin
    FmiGotoBmkSubMenu.Items[FCurrentRecItem].Checked := False;
    FmiRemoveBmkSubMenu.Items[FCurrentRecItem].Checked := False;
    FCurrentRecItem := -1;
  end;
  {find the menu item that was clicked}
  if (Sender is TMenuItem) then
    FCurrentRecItem := FmiRemoveBmkSubMenu.IndexOf(TMenuItem(Sender));
  if FCurrentRecItem <> -1 then begin
    RemoveBookmark(FCurrentRecItem);
    ResetBarBreaks;
    FCurrentRecItem := -1;
  end;
  NavPanelNotifier.FocusActiveControl;
end;

procedure TabcCustomnpBookmarkMenu.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  PtPopup: TPoint;
  ClRect: TRect;
begin
  inherited MouseUp(Button, Shift, X, Y);
  ClRect := ClientRect;
  PtPopup := Point(X,Y);
  if PtInRect(ClRect, PtPopup) then begin
    PtPopup := ClientToScreen(Point(0,Height));
    PopupAt(PtPopup);
    Perform(CM_MOUSELEAVE, 0, 0);
  end;
end;

procedure TabcCustomnpBookmarkMenu.PopupAt(APoint: TPoint);
begin
  CheckMenuItems;
  FMenu.Popup(APoint.X, APoint.Y);
end;

procedure TabcCustomnpBookmarkMenu.RemoveBookmark(Index: Integer);
var
  BookMark: TBookmark;
  MenuItem: TMenuItem;
begin
  BookMark := FBookmarks.Items[Index];
  DataSet.FreeBookmark(BookMark);
  {remove item from list and menus}
  FBookmarks.Remove(BookMark);
  MenuItem := FmiGotoBmkSubMenu.Items[Index];
  FmiGotoBmkSubMenu.Remove(MenuItem);
  MenuItem.Free;
  MenuItem := FmiRemoveBmkSubMenu.Items[Index];
  FmiRemoveBmkSubMenu.Remove(MenuItem);
  MenuItem.Free;
end;

procedure TabcCustomnpBookmarkMenu.ResetBarBreaks;
var
  I: integer;
begin
  for I := 0 to FmiGotoBmkSubMenu.Count - 1 do begin
    with FmiGotoBmkSubMenu.Items[I] do
      if (Break <> mbNone) then Break := mbNone;
    with FmiRemoveBmkSubMenu.Items[I] do
      if (Break <> mbNone) then Break := mbNone;
    if (FBreakAtItem > 0) and (I > 0) and (I mod FBreakAtItem = 0) then begin
      FmiGotoBmkSubMenu.Items[I].Break := mbBarBreak;
      FmiRemoveBmkSubMenu.Items[I].Break := mbBarBreak;
    end;
  end;
end;

procedure TabcCustomnpBookmarkMenu.SetBreakAtItem(Value: TabcMenuItemBreakRange);
begin
  if Value <> FBreakAtItem then begin
    FBreakAtItem := Value;
    if not (csDesigning in ComponentState) then ResetBarBreaks;
  end;
end;

procedure TabcCustomnpBookmarkMenu.SetMenuAlign(Value: TPopupAlignment);
begin
  FMenu.Alignment := Value;
end;

procedure TabcCustomnpBookmarkMenu.SetMenuStrings(Value: TStrings);
begin
  if Value.Count <> 4 then
    raise Exception.Create(sabcnpBmksErrMenuStrings)
  else begin
    FMenuStrings.Assign(Value);
    InitMenuStrings;
  end;
end;

{ TabcCustomnpIndexCombo }

constructor TabcCustomnpIndexCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csDropDownList;
  FAutoLoadItems := True;
  FGoToFirst := True;
{$IFNDEF DELPHI6}
  FItemIndex := -1;
{$ENDIF}
  FTableIndexItems := TStringList.Create;
  Width := 125;
  Height := 25;
end;

destructor TabcCustomnpIndexCombo.Destroy;
begin
  FTableIndexItems.Free;
  FAdapter.Free;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcCustomnpIndexCombo.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FAdapter.Free;
    FAdapter := nil;
    FDataset := Value;
    if Value <> nil then
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
  end;
end;

procedure TabcCustomnpIndexCombo.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  SetDataset(Message.Dataset);
  if FAutoLoadItems and Message.LinkActive then
    if (FDataset <> nil) and (FDataset <> FLastIndexedDataset) and FAdapter.CanIndex then begin
      LoadItemsFromDataSet;
      FLastIndexedDataset := FDataset;
    end;
  if Message.LinkActive then
    CheckEnabled
  else
    Enabled := False;
  if Enabled then
    if (ItemIndex = -1) then
      IndexChanged
    else
      SetItemIndex(ItemIndex)
  else
    SetItemIndex(-1);
end;

procedure TabcCustomnpIndexCombo.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  if FAdapter <> nil then
    with FAdapter do
      if (IndexName <> FIxName) or (IndexFieldNames <> FIxFieldNames) or
         (IndexFieldCount <> FIxFieldCount) then
        IndexChanged;
end;

procedure TabcCustomnpIndexCombo.CheckEnabled;
begin
  Enabled := (FDataset <> nil) and (FDataset.Active) and (FAdapter <> nil) and FAdapter.CanIndex;
end;

procedure TabcCustomnpIndexCombo.Change;
begin
  if not FComboChanging then
  try
    FComboChanging := True;
{$IFDEF DELPHI6}
    SetItemIndex(inherited GetItemIndex);
{$ELSE}
    SetItemIndex(inherited ItemIndex);
{$ENDIF}
    if Assigned(OnChange) then OnChange(Self);
  finally
    FComboChanging := False;
  end;
end;

function TabcCustomnpIndexCombo.CreateInheritedItems(Value: TStrings; var InheritedStrings: TStrings): Boolean;
var
  I, intTemp: integer;
  strTemp: string;
begin
  InheritedStrings.Clear;
  for I := 0 to Value.Count - 1 do
    if ParseItem(Value.Strings[I], strTemp, intTemp) then
      InheritedStrings.Add(strTemp);
  Result := True;
end;

function TabcCustomnpIndexCombo.GetSorted: Boolean;
begin
  Result := inherited Sorted;
end;

procedure TabcCustomnpIndexCombo.IndexChanged;
{
Synchronise ItemIndex of ComboBox with Table IndexName
}
var
  I, NewItemIndex, intTemp: integer;
  strTemp: string;

  procedure SetInheritedItemIndex(Value: Integer);
  begin
    if FItemIndex <> Value then
    begin
{$IFDEF DELPHI6}
      inherited SetItemIndex(Value);
{$ELSE}
      inherited ItemIndex := Value;
{$ENDIF}
      FItemIndex := Value;
    end;
  end;

begin
  if FIndexChanging then Exit;
  FIndexChanging := True;
  try
    CheckEnabled;
    if Enabled then
      with FAdapter do
      begin
        if IndexDefs <> nil then
          IndexDefs.Update;
        FIxName := IndexName;
        FIxFieldNames := IndexFieldNames;
        FIxFieldCount := IndexFieldCount;
        I := 0;
        while I < IndexDefs.Count do
        begin
          if IndexDefs.Items[I].Name = IndexName then Break;
          Inc(I);
        end;
        if I = IndexDefs.Count then
          SetInheritedItemIndex(-1)
        else
        begin
          NewItemIndex := 0;
          while NewItemIndex < FTableIndexItems.Count do
          begin
            if ParseItem(FTableIndexItems.Strings[NewItemIndex], strTemp, intTemp) then
              if intTemp = I then Break;
            Inc(NewItemIndex);
          end;
          if NewItemIndex = FTableIndexItems.Count then
            SetInheritedItemIndex(-1)
          else
            SetInheritedItemIndex(NewItemIndex);
        end;
      end
    else
      SetInheritedItemIndex(-1);
  finally
    FIndexChanging := False;
  end;
end;

procedure TabcCustomnpIndexCombo.Loaded;
begin
  inherited Loaded;
  SetInheritedItems;
  if Sorted then SetSorted(True);
  if (FDataset <> nil) and FDataset.Active then SetItemIndex(FItemIndex);
end;

function TabcCustomnpIndexCombo.LoadItemsFromDataSet: Integer;
var
  I: Integer;
  TempStrings: TStringList;
  Defs: TIndexDefs;
begin
  Result := 0;
  if (FDataset = nil) then
    raise EabcInvalidIndexComboOperation.Create(sabcnperrTableRequired);
  if FAdapter = nil then Exit;
  Defs := FAdapter.IndexDefs;
  if Defs <> nil then
  begin
    TempStrings := TStringList.Create;
    try
      Defs.Update;
      if Defs.Count > 0 then begin
        for I := 0 to Defs.Count - 1 do
          TempStrings.Add(Defs.Items[I].Name + sabcnpIdxSep1 + IntToStr(I));
        if TempStrings.Strings[0] = sabcnpIdxSep2 then
          TempStrings.Strings[0] := sabcnpPrimaryKeyText + sabcnpIdxSep2;
      end;
      TableIndexItems := TempStrings;
      if Sorted then SetSorted(True);
      SetItemIndex(ItemIndex);
      Result := Defs.Count;
    finally
      TempStrings.Free;
    end;
  end;
end;

function TabcCustomnpIndexCombo.ParseItem(Item: string; var strPart: string; var Index: integer): Boolean;
{
separates strings of form "MyIndexName;1"
assumes ; is always the separator
}
var
  Separator: integer;
  strTemp: string;
begin
  strPart := '';
  Index := 0;
  Separator := Pos(sabcnpIdxSep1, Item);
  if Separator = 0 then
    raise EabcInvalidIndexComboItem.Create(Format(sabcnpErrItemFormat1, [Item]));
  strPart := Copy(Item, 1, Separator - 1);
  if strPart = '' then
    raise EabcInvalidIndexComboItem.Create(Format(sabcnpErrItemFormat2, [Item]));
  strTemp := Copy(Item, Separator + 1, 256);
  try
    Index := StrToInt(strTemp);
  except
    raise EabcInvalidIndexComboItem.Create(Format(sabcnpErrItemFormat3, [Item]));
  end;
  Result := True;
end;

procedure TabcCustomnpIndexCombo.SetInheritedItems;
var
  TempStrings: TStringList;
begin
  if not (csLoading in ComponentState) then begin
    TempStrings := TStringList.Create;
    try
      if CreateInheritedItems(FTableIndexItems, TStrings(TempStrings)) then
        inherited Items := TempStrings;
    finally
      TempStrings.Free;
    end;
  end;
end;

{$IFDEF DELPHI6}
function TabcCustomnpIndexCombo.GetItemIndex: Integer;
begin
  Result := FItemIndex;
end;

procedure TabcCustomnpIndexCombo.SetItemIndex(const Value: Integer);
var
  strTemp: string;
  intTemp, newValue: integer;
begin
  if (csLoading in ComponentState) then
    FItemIndex := Value
  else
  begin
    if Value > FTableIndexItems.Count - 1 then
      newValue := FTableIndexItems.Count - 1
    else if Value < -1 then
      newValue := -1
    else
      newValue := Value;
    if newValue = -1 then
    begin
      {clears the combo without changing the table index}
      inherited SetItemIndex(newValue);
      FItemIndex := newValue;
      if Assigned(FOnIndexChange) then FOnIndexChange(Self);
      Exit;
    end;
    if (FAdapter <> nil) then
      with FAdapter do
        if ParseItem(TableIndexItems.Strings[newValue], strTemp, intTemp) then
        begin
          IndexDefs.Update;
          if (intTemp > (IndexDefs.Count - 1)) or (intTemp < 0) then
            raise EabcInvalidIndexComboOperation.Create
              (Format(sabcnpErrIndexRange, [IntTemp]))
          else
          begin
            FIxName := IndexDefs.Items[intTemp].Name;
            IndexName := FIxName;
            if FDataset.Active and FGoToFirst then FDataset.First;
            UpdateFocus;
          end;
          FIxName := IndexName;
          FIxFieldNames := IndexFieldNames;
          FIxFieldCount := IndexFieldCount;
        end;
    FItemIndex := newValue;
    SendMessage(Handle, CB_SETCURSEL, newValue, 0);
    if Assigned(FOnIndexChange) then FOnIndexChange(Self);
  end;
end;
{$ELSE}
procedure TabcCustomnpIndexCombo.SetItemIndex(const Value: Integer);
var
  strTemp: string;
  intTemp, newValue: integer;
begin
  if (csLoading in ComponentState) then
    FItemIndex := Value
  else
  begin
    if Value > FTableIndexItems.Count - 1 then
      newValue := FTableIndexItems.Count - 1
    else if Value < -1 then
      newValue := -1
    else
      newValue := Value;
    if newValue = -1 then
    begin
      {clears the combo without changing the table index}
      FItemIndex := newValue;
      inherited ItemIndex := newValue;
      if Assigned(FOnIndexChange) then FOnIndexChange(Self);
      Exit;
    end;
    if (FAdapter <> nil) then
      with FAdapter do
        if ParseItem(TableIndexItems.Strings[newValue], strTemp, intTemp) then
        begin
          IndexDefs.Update;
          if (intTemp > (IndexDefs.Count - 1)) or (intTemp < 0) then
            raise EabcInvalidIndexComboOperation.Create
              (Format(sabcnpErrIndexRange, [IntTemp]))
          else
          begin
            IndexName := IndexDefs.Items[intTemp].Name;
            if FDataset.Active and FGoToFirst then FDataset.First;
            UpdateFocus;
          end;
          FIxName := IndexName;
          FIxFieldNames := IndexFieldNames;
          FIxFieldCount := IndexFieldCount;
        end;
    FItemIndex := newValue;
    inherited ItemIndex := newValue;
    if Assigned(FOnIndexChange) then FOnIndexChange(Self);
  end;
end;
{$ENDIF}

procedure TabcCustomnpIndexCombo.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (csDestroying in ComponentState) then Exit;
  if (AParent <> nil) then
    if (Parent is TabcCustomNavToolbar) then
      with TabcNavToolbar(Parent) do
        if (DataSource <> nil) then
          SetDataset(DataSource.DataSet)
        else
          SetDataset(nil)
    else if (Parent is TabcCustomNavPanel) then
      with TabcNavPanel(Parent) do
        if (DataSource <> nil) then
          SetDataset(DataSource.DataSet)
        else
          SetDataset(nil)
    else
      SetDataset(nil)
  else
    SetDataset(nil);
  CheckEnabled;
end;

procedure TabcCustomnpIndexCombo.SetSorted(Value: Boolean);
begin
  inherited Sorted := Value;
  if Value then
    TStringList(FTableIndexItems).Sort;
  SetItemIndex(ItemIndex);
end;

procedure TabcCustomnpIndexCombo.SetTableIndexItems(Value: TStrings);
begin
  FTableIndexItems.Assign(Value);
  SetInheritedItems;
  if Sorted then SetSorted(True);
  if (FDataset <> nil) and FDataset.Active then SetItemIndex(ItemIndex);
end;

procedure TabcCustomnpIndexCombo.UpdateFocus;
{ focus reset and fix for active control lockup with MDI child DB grids }
var
  ChildForm: TForm;
  ParentForm: TCustomForm;
begin
  ParentForm := GetParentForm(Self);
  if (ParentForm is TForm) and (TForm(ParentForm).FormStyle = fsMDIForm) then begin
    ChildForm := TForm(ParentForm).ActiveMDIChild;
    if ChildForm <> nil then
      with ChildForm do begin
        if (ActiveControl <> nil) and ActiveControl.HandleAllocated then
          Windows.SetFocus(ActiveControl.Handle);
      end;
  end
  else
    NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpIndexCombo }

function TabcnpIndexCombo.GetText: string;
begin
  Result := inherited Text;
end;

function TabcnpIndexCombo.LoadItemsFromDataSet: Integer;
begin
  Result := inherited LoadItemsFromDataSet;
end;

{ TabcCustomnpRangeEdit }

constructor TabcCustomnpRangeEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FGoToFirst := True;
  FSeparator := sabcnpRangeSep;
  Width := 125;
  Height := 25;
end;

destructor TabcCustomnpRangeEdit.Destroy;
begin
  FDataset := nil;
  FAdapter.Free;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcCustomnpRangeEdit.ApplyRange;
var
  SepPos: Integer;
  StartRange, EndRange: string;
begin
  if Enabled then
    with FAdapter do
      {set dataset range}
      if Text = '' then begin
        CancelRange;
        if FGoToFirst then FDataset.First;
      end
      else begin
        SepPos := Pos(FSeparator, Text);
        StartRange := '';
        EndRange := '';
        if SepPos = 0 then begin
          StartRange := Text;
          if FIxFieldType = ftString then
            EndRange := Text + #255
          else
            EndRange := Text;
        end
        else begin
          StartRange := abcLRTrim(Copy(Text, 1, SepPos - 1));
          EndRange := abcLRTrim(Copy(Text, SepPos + Length(Separator), Length(Text)));
          if EndRange <> '' then begin
            if FIxFieldType = ftString then
              EndRange := EndRange + #255;
          end
          else
            {set max value for field type}
            EndRange := abcMaxFieldType(FIxFieldType);
        end;
        try
          SetRangeStart;
          IndexFields[0].AsString := StartRange;
          SetRangeEnd;
          IndexFields[0].AsString := EndRange;
          ApplyRange;
          if FGoToFirst then FDataset.First;
        except
          on Exception do begin
            CancelRange;
            if FGoToFirst then FDataset.First;
            raise;
          end;
        end;
      end;
  UpdateFocus;
end;

procedure TabcCustomnpRangeEdit.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FAdapter.Free;
    FAdapter := nil;
    FDataset := Value;
    if Value <> nil then
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
  end;
end;

procedure TabcCustomnpRangeEdit.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  SetDataset(Message.Dataset);
  if FAdapter <> nil then
    IndexChanged
  else
    Caption := '';
  if Message.LinkActive then
    CheckEnabled
  else
    Enabled := False;
end;

procedure TabcCustomnpRangeEdit.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  if FAdapter <> nil then
    with FAdapter do
      if (IndexName <> FIxName) or (IndexFieldNames <> FIxFieldNames) or
         (IndexFieldCount <> FIxFieldCount) then
        IndexChanged;
end;

procedure TabcCustomnpRangeEdit.CheckEnabled;
begin
  if (FDataset <> nil) and FDataset.Active and (FAdapter <> nil) then
    with FAdapter do
      if CanSetRange and CanIndex and (IndexDefs <> nil) then
      begin
        if IndexFieldCount > 0 then
          FIxFieldType := IndexFields[0].DataType
        else
          FIxFieldType := ftUnknown;
        if FIxFieldType in [ftString..ftDateTime] then
          Enabled := True
        else
          Enabled := False;
      end
      else
        Enabled := False
  else
    Enabled := False;
end;

procedure TabcCustomnpRangeEdit.IndexChanged;
begin
  with FAdapter do
    if CanSetRange and CanIndex and (IndexDefs <> nil) then
    begin
      IndexDefs.Update;
      FIxName := IndexName;
      FIxFieldNames := IndexFieldNames;
      FIxFieldCount := IndexFieldCount;
      if IndexFieldCount > 0 then
        FIxFieldType := IndexFields[0].DataType
      else
        FIxFieldType := ftUnknown;
      if FIxFieldType in [ftString..ftDateTime] then
        Enabled := True
      else
        Enabled := False;
    end
    else
    begin
      FIxName := '';
      FIxFieldNames := '';
      FIxFieldCount := 0;
      FIxFieldType := ftUnknown;
      Enabled := False;
    end;
  Caption := '';
end;

procedure TabcCustomnpRangeEdit.KeyPress(var Key: Char);
begin
  if (Key = #13) then begin
    Key := #0;
    inherited KeyPress(Key);
    ApplyRange;
  end
  else
    inherited KeyPress(Key);
end;

procedure TabcCustomnpRangeEdit.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if [csLoading, csDestroying] * ComponentState <> [] then Exit;
  if (AParent <> nil) then
    if (Parent is TabcCustomNavToolbar) then
      with TabcNavToolbar(Parent) do
        if (DataSource <> nil) then
          SetDataset(Datasource.Dataset)
        else
          SetDataset(nil)
    else if (Parent is TabcCustomNavPanel) then
      with TabcNavPanel(Parent) do
        if (DataSource <> nil) then
          SetDataset(Datasource.Dataset)
        else
          SetDataset(nil)
    else
      SetDataset(nil)
  else
    SetDataset(nil);
  CheckEnabled;
end;

procedure TabcCustomnpRangeEdit.UpdateFocus;
{ focus reset and fix for active control lockup with MDI child DB grids }
var
  ChildForm: TForm;
  ParentForm: TCustomForm;
begin
  ParentForm := GetParentForm(Self);
  if (ParentForm is TForm) and (TForm(ParentForm).FormStyle = fsMDIForm) then begin
    ChildForm := TForm(ParentForm).ActiveMDIChild;
    if ChildForm <> nil then
      with ChildForm do begin
        if (ActiveControl <> nil) and ActiveControl.HandleAllocated then
          Windows.SetFocus(ActiveControl.Handle);
      end;
  end
  else
    NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpFilterExpression }

constructor TabcnpFilterExpression.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  GroupIndex := GetTickCount; {try to get a unique group for designing}
  AllowAllUp := True;
  Down := False;
  FFilter := TabcDBDataSetFilter.Create(Self);
  with FFilter do begin
    AutoActivate := False;
    OnActivate := CheckButtonState;
    OnDeActivate := CheckButtonState;
  end;
  FAllowEdit := True;
end;

destructor TabcnpFilterExpression.Destroy;
begin
  FFilter.Free;
  inherited Destroy;
end;

procedure TabcnpFilterExpression.CheckButtonState(Sender: TObject);
begin
  Down := not FFilter.Active;
  if (csDesigning in ComponentState) then
    ValidParentForm(Self).Designer.Modified;
  DoFilterActiveChangeEvents;    
end;

procedure TabcnpFilterExpression.DoFilterActiveChangeEvents;
begin
  if [csDestroying, csLoading, csReading] * ComponentState = [] then
    if Down then
      if Assigned(FOnActivate) then
        FOnActivate(Self)
      else
    else
      if Assigned(FOnDeactivate) then
        FOnDeactivate(Self);
end;

procedure TabcnpFilterExpression.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  if not Message.LinkActive then
    CheckEnabled
  else if FLoadedFilter <> '' then
  begin
    Filter := FLoadedFilter;
    FLoadedFilter := '';
    if FLoadedActive then
    begin
      Active := True;
      Down := Active;
    end;
  end;
end;

procedure TabcnpFilterExpression.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  inherited;
  if Down and (Adapter <> nil) and Adapter.SingleFilter then
    if DataSet.Filter <> FFilter.Filter then
    begin
      FFilter.Active := False;
      Down := False;
    end;
end;

procedure TabcnpFilterExpression.CheckEnabled;
var
  bEnabled: Boolean;
begin
  if DataSet <> FFilter.DataSet then begin
    if Down then FFilter.Active := False;
    FFilter.Filter := '';
    FFilter.DataSet := Dataset;
  end;
  bEnabled := FFilter.CanActivate or
    ((Filter = '') and
      AllowEdit and
      (DataSet <> nil) and
       (FFilter.Adapter <> nil) and
      FFilter.Adapter.CanFilter
    );
  if bEnabled and Assigned(FOnCheckEnabled) then
    FOnCheckEnabled(Self, DataSet, bEnabled);
  Enabled := bEnabled;
end;

procedure TabcnpFilterExpression.Click;
begin
  inherited Down := not inherited Down;
  inherited Click;
end;

procedure TabcnpFilterExpression.ValidClick;
var
  FilterEvent: TabcFilterRecordEvent;
begin
  inherited Down := not inherited Down;
  if Down then begin
    FilterEvent := OnFilterRecord;
    if (FFilter.Filter = '') and not Assigned(FilterEvent) and FAllowEdit then Edit;
    if (FFilter.Filter <> '') or Assigned(FilterEvent) then
      FFilter.Activate
    else
      Down := False;
  end
  else
    FFilter.DeActivate;
  NavPanelNotifier.FocusActiveControl;
end;

function TabcnpFilterExpression.Edit: Boolean;
begin
  if FFilter.Active then FFilter.AutoActivate := True;
  if (csDesigning in ComponentState) then
  begin
    if (NavPanel <> nil) and (NavPanel.Datasource <> nil) then
      FFilter.DataSet := NavPanel.Datasource.Dataset
    else if (NavToolbar <> nil) and (NavToolbar.Datasource <> nil) then
      FFilter.DataSet := NavToolbar.Datasource.Dataset;
    if FLoadedFilter <> '' then
    begin
      FFilter.Filter := FLoadedFilter;
      FLoadedFilter := '';
    end;
  end;
  Result := FFilter.Edit;
  FFilter.AutoActivate := False;
  if Result and Assigned(FOnEdited) then
    FOnEdited(Self);
end;

function TabcnpFilterExpression.GetDown: Boolean;
begin
  Result := inherited Down;
end;

function TabcnpFilterExpression.GetFilter: string;
begin
  Result := FFilter.Filter;
end;

function TabcnpFilterExpression.GetFilterOptions: TFilterOptions;
begin
  Result := FFilter.FilterOptions;
end;

function TabcnpFilterExpression.GetGoToFirst: Boolean;
begin
  Result := FFilter.GoToFirst;
end;

function TabcnpFilterExpression.GetHelpContext: THelpContext;
begin
  Result := FFilter.HelpContext;
end;

function TabcnpFilterExpression.GetHint: string;
begin
  Result := inherited Hint;
end;

function TabcnpFilterExpression.GetOnFilterRecord: TabcFilterRecordEvent;
begin
  Result := FFilter.OnFilterRecord;
end;

procedure TabcnpFilterExpression.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if FAllowEdit and (Button = mbRight) then Edit;
end;

function TabcnpFilterExpression.ParamByName(const FieldName: string): TParam;
begin
  Result := FFilter.ParamByName(FieldName);
end;

function TabcnpFilterExpression.GetActive: Boolean;
begin
  Result := FFilter.Active;
end;

procedure TabcnpFilterExpression.SetActive(Value: Boolean);
begin
  if Value <> FFilter.Active then
  begin
    if (csLoading in ComponentState) then
      FLoadedActive := Value
    else
      FFilter.Active := Value;
    Down := FFilter.Active;
  end;
end;

procedure TabcnpFilterExpression.SetDown(Value: Boolean);
begin
  inherited Down := Value;
  if Value then
    inherited Hint := FHintDown
  else
    inherited Hint := FHintUp;
end;

procedure TabcnpFilterExpression.SetFilter(Value: string);
begin
  if Value <> FFilter.Filter then
  begin
    FFilter.Filter := Value;
    if (csLoading in ComponentState) then
      FLoadedFilter := Value;
  end;
end;

procedure TabcnpFilterExpression.SetFilterOptions(Value: TFilterOptions);
begin
  if Value <> FFilter.FilterOptions then begin
    FFilter.FilterOptions := Value;
  end;
end;

procedure TabcnpFilterExpression.SetGoToFirst(Value: Boolean);
begin
  if Value <> FFilter.GoToFirst then begin
    FFilter.GoToFirst := Value;
  end;
end;

procedure TabcnpFilterExpression.SetHelpContext(Value: THelpContext);
begin
  if Value <> FFilter.HelpContext then
    FFilter.HelpContext := Value;
end;

procedure TabcnpFilterExpression.SetHintDown(Value: string);
begin
  if Value <> FHintDown then begin
    FHintDown := Value;
    if Down then inherited Hint := Value;
  end;
end;

procedure TabcnpFilterExpression.SetHintUp(Value: string);
begin
  if Value <> FHintUp then begin
    FHintUp := Value;
    if not Down then inherited Hint := Value;
  end;
end;

procedure TabcnpFilterExpression.SetOnFilterRecord(Value: TabcFilterRecordEvent);
begin
  FFilter.OnFilterRecord := Value;
end;

{ TabcnpFilterActive }

constructor TabcnpFilterActive.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  GroupIndex := GetTickCount; {try to get a unique group for designing}
  AllowAllUp := True;
  Down := False;
  FFilter := TabcDBDataSetFilter.Create(Self);
  with FFilter do begin
    OnActivate := CheckButtonState;
    OnDeActivate := CheckButtonState;
  end;
end;

destructor TabcnpFilterActive.Destroy;
begin
  FFilter.Free;
  inherited Destroy;
end;

function TabcnpFilterActive.ApplyFilter: Boolean;
{
This function tries to set a filter using the field of the active control.
Returns true if a filter was set
Returns false if an error occurs while setting the filter.
}
var
  SFilter: string;
begin
  Result := False;
  FFilterChanging := True;
  try
    try
      if (Field = nil) or not (Field.DataType in [ftString..ftDateTime]) then Exit;
      SFilter := Field.FieldName;
      case Field.DataType of
        ftString:
          SFilter := SFilter + sabcnpFltEquals + #39 + Field.AsString + #39;
        ftSmallInt..ftWord:
          SFilter := SFilter + sabcnpFltEquals + Field.AsString;
        ftBoolean:
          if not Field.AsBoolean then
            SFilter := sabcnpFltNot + SFilter;
        ftFloat..ftBCD:
          SFilter := SFilter + sabcnpFltEquals + Field.AsString;
        ftDate..ftDateTime:
          SFilter := SFilter + sabcnpFltEquals + #39 + Field.AsString + #39;
      end;
      FFilter.Filter := SFilter;
      Result := FFilter.Active;
    except
      Exit;
    end;
  finally
    FFilterChanging := False;
  end;
end;

procedure TabcnpFilterActive.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  if not Message.LinkActive then CheckEnabled;
end;

procedure TabcnpFilterActive.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  inherited;
  if Down and (Adapter <> nil) and Adapter.SingleFilter then
    if {DataSet.Filter} Adapter.Filter <> FFilter.Filter then
    begin
      FFilter.Active := False;
      Down := False;
    end;
end;

procedure TabcnpFilterActive.CheckButtonState(Sender: TObject);
begin
  Down := not FFilter.Active;
  if (csDesigning in ComponentState) then
    ValidParentForm(Self).Designer.Modified;
end;

procedure TabcnpFilterActive.CheckEnabled;
begin
  if DataSet <> FFilter.DataSet then begin
    if Down then
      FFilter.Active := False;
    Down := False;
    FFilter.DataSet := Dataset;
    FFilter.Filter := '';
  end;
  Enabled := (DataSet <> nil) and
             (DataSet.Active) and
             (Adapter <> nil) and
             Adapter.CanFilter and
             ((Field <> nil) and
             (Field.DataSet = DataSet) and
              (Field.DataType in [ftString..ftDateTime]));
end;

procedure TabcnpFilterActive.Click;
begin
  inherited Down := not inherited Down;
  inherited Click;
end;

procedure TabcnpFilterActive.ValidClick;
begin
  inherited Down := not inherited Down;
  if Down then begin
    if not ApplyFilter then begin
      Down := False;
      if Assigned(FOnNoFilter) then FOnNoFilter(Self);
    end;
  end
  else
  begin
    FFilterChanging := True;
    FFilter.DeActivate;
    FFilterChanging := False;
  end;
  NavPanelNotifier.FocusActiveControl;
end;

function TabcnpFilterActive.GetDown: Boolean;
begin
  Result := inherited Down;
end;

function TabcnpFilterActive.GetFilterOptions: TFilterOptions;
begin
  Result := FFilter.FilterOptions;
end;

function TabcnpFilterActive.GetGoToFirst: Boolean;
begin
  Result := FFilter.GoToFirst;
end;

function TabcnpFilterActive.GetHint: string;
begin
  Result := inherited Hint;
end;

procedure TabcnpFilterActive.SetDown(Value: Boolean);
begin
  inherited Down := Value;
  if Value then
    inherited Hint := FHintDown
  else
    inherited Hint := FHintUp;
end;

procedure TabcnpFilterActive.SetFilterOptions(Value: TFilterOptions);
begin
  if Value <> FFilter.FilterOptions then begin
    FFilter.FilterOptions := Value;
  end;
end;

procedure TabcnpFilterActive.SetGoToFirst(Value: Boolean);
begin
  if Value <> FFilter.GoToFirst then begin
    FFilter.GoToFirst := Value;
  end;
end;

procedure TabcnpFilterActive.SetHintDown(Value: string);
begin
  if Value <> FHintDown then begin
    FHintDown := Value;
    if Down then inherited Hint := Value;
  end;
end;

procedure TabcnpFilterActive.SetHintUp(Value: string);
begin
  if Value <> FHintUp then begin
    FHintUp := Value;
    if not Down then inherited Hint := Value;
  end;
end;

{-------------------------- FILTER, SEARCH ETC BUTTONS ------------------------}

{ TabcnpRange }

procedure TabcnpRange.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and (Adapter <> nil) and Adapter.CanSetRange
    and (DataSet.State = dsBrowse);
end;

procedure TabcnpRange.ValidClick;
begin
  ExecabcafKeyRange(Self, DataSet, Adapter, FDlgHelpButton, FDlgHelpContext);
end;

{ TabcnpFindKey }

procedure TabcnpFindKey.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and DataSet.Active and (Adapter <> nil) and Adapter.CanFindKey
    and (DataSet.State = dsBrowse);
end;

procedure TabcnpFindKey.ValidClick;
begin
  ExecabcafFindKey(Self, DataSet, Adapter, FDlgHelpButton, FDlgHelpContext);
  NavPanelNotifier.FocusActiveControl;
end;

{ TabcnpQueryParams }

procedure TabcnpQueryParams.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  if not Message.LinkActive then CheckEnabled;
end;

procedure TabcnpQueryParams.CheckEnabled;
begin
  Enabled := (DataSet <> nil) and (Adapter <> nil) and Adapter.CanSetParams and (Adapter.ParamCount > 0);
end;

procedure TabcnpQueryParams.ValidClick;
begin
  ExecabcafQueryParams(Self, DataSet, Adapter, FDlgHelpButton, FDlgHelpContext);
  NavPanelNotifier.FocusActiveControl;
end;

end.
