unit abcnpinf;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Classes, Graphics, Messages, Controls, StdCtrls, Menus, Dialogs, DB,
  abcnp, abcnpbtn, abcfact, abcda;

type

{ TabcnpRecordCount }

  TabcnpRecordCount = class(TCustomLabel)
  private
    FCurrentRecord, FRecordCount: Longint;
    FDataSet: TDataSet;
    FDisabledCaption: string;
    FMask: string;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDDataSetChanged(var Message: TabcCDDataSetChanged); message ABC_CD_DATASETCHANGED;
    procedure abcCDEditingChanged(var Message: TabcCDEditingChanged); message ABC_CD_EDITINGCHANGED;
    procedure abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled); message ABC_CD_DATASETSCROLLED;
    procedure CheckEnabled;
    procedure SetDataset(Value: TDataset);
    procedure SetMask(Value: string);
  protected
    procedure DataChange;
    procedure Loaded; override;
    procedure RefreshRecordCount;
    procedure RefreshCaption;
    procedure SetParent(AParent: TWinControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DisabledCaption: string read FDisabledCaption write FDisabledCaption;
    property Mask: string read FMask write SetMask;
    {from CustomLabel}
    property Align;
    property Alignment;
    property AutoSize default False;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Transparent;
    property ShowHint;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
{$IFDEF DELPHI6}
    property OnMouseEnter;
    property OnMouseLeave;
{$ENDIF}
  end;

  TabcnpState = class(TCustomLabel)
  private
    FAdapter: TabcDatasetAdapter;
    FDataSet: TDataSet;
    FDisabledCaption: string;
    FMask: string;
    FStates: TStrings;
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure abcCDEditingChanged(var Message: TabcCDEditingChanged); message ABC_CD_EDITINGCHANGED;
    procedure CheckEnabled;
    procedure SetDataset(Value: TDataset);
    procedure SetMask(Value: string);
    procedure SetStates(Value: TStrings);
  protected
    procedure DataChange;
    procedure Loaded; override;
    procedure SetParent(AParent: TWinControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DisabledCaption: string read FDisabledCaption write FDisabledCaption;
    property Mask: string read FMask write SetMask;
    property States: TStrings read FStates write SetStates;
    {from CustomLabel}
    property Align;
    property Alignment;
    property AutoSize default False;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Transparent;
    property ShowHint;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
{$IFDEF DELPHI6}
    property OnMouseEnter;
    property OnMouseLeave;
{$ENDIF}
  end;

{-------------------------- DB INFO BUTTON ------------------------------------}

  { TabcnpDBInfo }

  TabcnpDBInfo = class(TabcCustomnpBtn)
  private
    FLinkActive: Boolean;
    procedure AddStr(Str: string; Indent: Integer; List: TStrings);
    procedure abcCDActiveChanged(var Message: TabcCDActiveChanged); message ABC_CD_ACTIVECHANGED;
    procedure GetComponentInfo(Info: TStrings);
  protected
    procedure CheckEnabled; override;
    procedure ValidClick; override;
  public
    procedure ShowDBInfo;
  end;

implementation

uses SysUtils, abcconst, abcutild, abcutilf, abcfdbme, abcfdbim,
     abcfinfo, abcnpcon;

{ TabcnpRecordCount }

constructor TabcnpRecordCount.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ShowAccelChar := False;
  AutoSize := False;
  Width := 125;
  Height := 25;
  FMask := sabcnpiMask;
  FDisabledCaption := sabcnpiDisabled;
  FCurrentRecord := 5;
  FRecordCount := 0;
end;

destructor TabcnpRecordCount.Destroy;
begin
  FDataSet := nil;
  inherited Destroy;
end;

procedure TabcnpRecordCount.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FDataSet := Value;
  end;
end;

procedure TabcnpRecordCount.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  if not (csDestroying in ComponentState) then
  begin
    SetDataset(Message.DataSet);
    if Message.LinkActive then begin
      CheckEnabled;
    end
    else begin
      Enabled := False;
      DataChange;
    end;
  end;
end;

procedure TabcnpRecordCount.abcCDDataSetChanged(var Message: TabcCDDataSetChanged);
begin
  CheckEnabled;
end;

procedure TabcnpRecordCount.abcCDDataSetScrolled(var Message: TabcCDDataSetScrolled);
begin
  DataChange;
end;

procedure TabcnpRecordCount.abcCDEditingChanged(var Message: TabcCDEditingChanged);
begin
  DataChange;
end;

procedure TabcnpRecordCount.CheckEnabled;
begin
  if not (csDestroying in ComponentState) then
  begin
    Enabled := (FDataSet <> nil) and (FDataSet.Active);
    DataChange;
  end;
end;

procedure TabcnpRecordCount.DataChange;
begin
  if not (csDestroying in ComponentState) then
    if (FDataSet <> nil) then
      RefreshRecordCount
    else
      if csDesigning in ComponentState then
        Caption := FMask
      else begin
        FCurrentRecord := 0;
        FRecordCount := 0;
        RefreshCaption;
      end;
end;

procedure TabcnpRecordCount.Loaded;
begin
  inherited Loaded;
  DataChange;
end;

procedure TabcnpRecordCount.RefreshCaption;
var
  RecMarker, CountMarker, TempCaption: string;
  MarkerPos: integer;
begin
  RecMarker := sabcnpiRecMarker;
  CountMarker := sabcnpiCountMarker;
  if not Enabled then
    if (csDesigning in ComponentState) then
      Caption := Name
    else
      Caption := FDisabledCaption
  else begin
    TempCaption := FMask;
    MarkerPos := Pos(RecMarker, TempCaption);
    if MarkerPos > 0 then begin
      Delete(TempCaption, MarkerPos, Length(RecMarker));
      Insert(InttoStr(FCurrentRecord), TempCaption, MarkerPos);
    end;
    MarkerPos := Pos(CountMarker, TempCaption);
    if MarkerPos > 0 then begin
      Delete(TempCaption, MarkerPos, Length(CountMarker));
      Insert(InttoStr(FRecordCount), TempCaption, MarkerPos);
    end;
    Caption := TempCaption;
  end;
end;

procedure TabcnpRecordCount.RefreshRecordCount;
var
  TempCurRec, TempRecCount: LongInt;
begin
  if (FDataSet <> nil) and FDataSet.Active then
    with FDataSet do
    begin
      try
        TempRecCount := RecordCount;
      except
        TempRecCount := -1;
      end;
      UpdateCursorPos;
      if EOF then
        if State = dsInsert then
          TempCurRec := TempRecCount + 1
        else
          TempCurRec := TempRecCount
      else
        TempCurRec := RecNo;
    end
  else begin
    TempCurRec := 0;
    TempRecCount := 0;
  end;
  if (FCurrentRecord <> TempCurRec) or (FRecordCount <> TempRecCount) then begin
    FCurrentRecord := TempCurRec;
    FRecordCount := TempRecCount;
    RefreshCaption;
  end;
end;

procedure TabcnpRecordCount.SetMask(Value: string);
begin
  if Value <> FMask then begin
    FMask := Value;
    FCurrentRecord := 5; {to force a recalc of caption}
    FRecordCount := 0;
    DataChange;
  end;
end;

procedure TabcnpRecordCount.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (AParent = nil) or (csDestroying in ComponentState) then Exit;
  if (Parent is TabcCustomNavToolbar) then
    with TabcNavToolbar(Parent) do
      if (DataSource <> nil) then
        SetDataset(DataSource.DataSet)
      else
        SetDataset(nil)
  else if (Parent is TabcCustomNavPanel) then
    with TabcNavPanel(Parent) do
      if (DataSource <> nil) then
        SetDataset(DataSource.DataSet)
      else
        SetDataset(nil)
  else
    SetDataset(nil);
  CheckEnabled;
  DataChange;
end;

{ TabcnpState }

constructor TabcnpState.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStates := TStringList.Create;
  FStates.Add(sabcnpiInactive);
  FStates.Add(sabcnpiBrowse);
  FStates.Add(sabcnpiEdit);
  FStates.Add(sabcnpiInsert);
  FStates.Add(sabcnpiSetKey);
  FStates.Add(sabcnpiCalcFields);
  ShowAccelChar := False;
  AutoSize := False;
  Width := 125;
  Height := 25;
  FMask := sabcnpiDSMask;
  FDisabledCaption := sabcnpiDisabled;
end;

destructor TabcnpState.Destroy;
begin
  FStates.Free;
  FDataSet := nil;
  FAdapter.Free;
  FAdapter := nil;
  inherited Destroy;
end;

procedure TabcnpState.SetDataset(Value: TDataset);
begin
  if Value <> FDataset then
  begin
    FAdapter.Free;
    FAdapter := nil;
    FDataSet := Value;
    if Value <> nil then
      FAdapter := abcDSAdapterFactory.CreateAdapter(Value);
  end;
end;

procedure TabcnpState.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  if not (csDestroying in ComponentState) then
  begin
    SetDataset(Message.DataSet);
    if Message.LinkActive then
      CheckEnabled
    else begin
      Enabled := False;
      DataChange;
    end;
  end;
end;

procedure TabcnpState.abcCDEditingChanged(var Message: TabcCDEditingChanged);
begin
  DataChange;
end;

procedure TabcnpState.CheckEnabled;
begin
  if not (csDestroying in ComponentState) then
  begin
    Enabled := (FDataSet <> nil);
    DataChange;
  end;
end;

procedure TabcnpState.DataChange;
var
  Index: integer;
  DataSetMarker, StateMarker, TempCaption: string;
  StateName: string;
  MarkerPos: integer;
begin
  if (csDestroying in ComponentState) then Exit;
  DataSetMarker := sabcnpiDSNameMarker;
  StateMarker := sabcnpiDSStateMarker;
  if not Enabled then
    if (csDesigning in ComponentState) then
      Caption := Name
    else
      Caption := FDisabledCaption
  else if FDataSet = nil then
    if (csDesigning in ComponentState) then
      Caption := Name
    else
      Caption := FDisabledCaption
  else begin
    TempCaption := FMask;

    {dataset description}
    MarkerPos := Pos(DataSetMarker, TempCaption);
    if MarkerPos > 0 then begin
      Delete(TempCaption, MarkerPos, Length(DataSetMarker));
      if FAdapter <> nil then
        Insert(FAdapter.Description, TempCaption, MarkerPos);
    end;

    {dataset state description}
    MarkerPos := Pos(StateMarker, TempCaption);
    if MarkerPos > 0 then begin
      Index := Ord(FDataSet.State);
      if Index > FStates.Count - 1 then
        case FDataSet.State of
          dsInactive:
            StateName := sabcnpiInactive;
          dsBrowse:
            StateName := sabcnpiBrowse;
          dsEdit:
            StateName := sabcnpiEdit;
          dsInsert:
            StateName := sabcnpiInsert;
          dsSetKey:
            StateName := sabcnpiSetkey;
          dsCalcFields:
            StateName := sabcnpiCalcFields;
        else
//            StateName := sabcnpiOtherState;
        end
      else
        StateName := FStates.Strings[Index];
      Delete(TempCaption, MarkerPos, Length(StateMarker));
      Insert(StateName, TempCaption, MarkerPos);
    end;

    Caption := TempCaption;
  end;
end;

procedure TabcnpState.Loaded;
begin
  inherited Loaded;
  DataChange;
end;

procedure TabcnpState.SetMask(Value: string);
begin
  if Value <> FMask then begin
    FMask := Value;
    DataChange;
  end;
end;

procedure TabcnpState.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (AParent = nil) or (csDestroying in ComponentState) then Exit;
  if (Parent is TabcCustomNavToolbar) then
    with TabcNavToolbar(Parent) do
      if (DataSource <> nil) then
        SetDataset(DataSource.DataSet)
      else
        SetDataset(nil)
  else if (Parent is TabcCustomNavPanel) then
    with TabcNavPanel(Parent) do
      if (DataSource <> nil) then
        SetDataset(DataSource.DataSet)
      else
        SetDataset(nil)
  else
    SetDataset(nil);
  CheckEnabled;
  DataChange;
end;

procedure TabcnpState.SetStates(Value: TStrings);
begin
  FStates.Assign(Value);
  DataChange;
end;

{-------------------------- DB INFO BUTTON ------------------------------------}

{ TabcnpDBInfo }

procedure TabcnpDBInfo.AddStr(Str: string; Indent: Integer; List: TStrings);
var
  IndentStr: string;
  I: integer;
begin
  IndentStr := Str;
  for I := 1 to Indent do
    IndentStr := sabcnpiIndent + IndentStr;
  List.Add(IndentStr);
end;

procedure TabcnpDBInfo.abcCDActiveChanged(var Message: TabcCDActiveChanged);
begin
  inherited;
  FLinkActive := Message.LinkActive;
  CheckEnabled;
end;

procedure TabcnpDBInfo.CheckEnabled;
begin
  Enabled := (DataSet <> nil);
end;

procedure TabcnpDBInfo.ValidClick;
begin
  ShowDBInfo;
  NavPanelNotifier.FocusActiveControl;
end;

procedure TabcnpDBInfo.GetComponentInfo(Info: TStrings);
var
  Indent: Integer;
  DataSource: TDataSource;
begin
  Indent := 0;
  AddStr(sabcnpiComp,Indent,Info);
  Inc(Indent);
  AddStr(sabcnpiNameEq + Name,Indent,Info);
  AddStr(sabcnpiOCNameEq + Owner.ClassName,Indent,Info);
  AddStr(sabcnpiParentEq + Parent.Name,Indent,Info);
  AddStr(sabcnpiParentClassEq + Parent.ClassName,Indent,Info);
  {DataSource info}
  if Parent is TabcCustomNavToolbar then
    DataSource := TabcNavToolbar(Parent).DataSource
  else if Parent is TabcCustomNavPanel then
    DataSource := TabcNavPanel(Parent).DataSource
  else
    Datasource := nil;

    if DataSource = nil then
      AddStr(sabcnpiParentDSNil,Indent,Info)
    else
      with DataSource do begin
        AddStr(sabcnpiParentDSEq + Name,Indent,Info);
        AddStr(sabcnpiParentDSAuto + ABC_BOOLEANS[AutoEdit],Indent,Info);
        AddStr(sabcnpiParentDSEnabled + ABC_BOOLEANS[Enabled],Indent,Info);
        AddStr(sabcnpiParentDSState + ABC_DATASETSTATES[State],Indent,Info);
        if DataSet = nil then
          AddStr(sabcnpiParentDSDSNil,Indent,Info)
        else
          AddStr(sabcnpiParentDSDSEq + DataSet.Name,Indent,Info);
      end;
  AddStr('',0,Info);
end;

procedure TabcnpDBInfo.ShowDBInfo;
var
  Form: TabcsfDSInfo;
begin
  if (Dataset = nil) then
    ShowMessage(sabcnpiDataSetNil)
  else begin
    Form := TabcsfDSInfo(
      abcFormFactory.CreateFormOfClass(TabcsfDSInfo, Self));
    try
      with Adapter, Form do begin
        GetComponentInfo(memComp.Lines);
        GetDataSetInfo(memDataSet.Lines);
        GetFieldInfo(memFields.Lines);
        GetIndexInfo(memIndexes.Lines);
        GetParamInfo(memParams.Lines);
        GetDatabaseInfo(memDataBase.Lines);
        GetSessionInfo(memSession.Lines);
        GetDBEngineInfo(memEngine.Lines);
        Caption := sabcnpiDSInfo;
        if DataSet.Name <> '' then
          Caption := Caption + sabcnpifor + DataSet.Name;
        ShowModal;
      end;
    finally
      Form.Free;
    end;
  end;
end;

end.
