unit abcpage;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, {Dialogs,} CommDlg;

type

  { TabcPrintMargins }

  TabcPrintUnits = (apuInches, apuMillimeters);

  TabcPrintMargins = class(TPersistent)
  private
    FLeft: Double;
    FTop: Double;
    FRight: Double;
    FBottom: Double;
    FUnits: TabcPrintUnits;
    FOnChange: TNotifyEvent;
    function GetSysRect: TRect;
    procedure SetSysRect(Value: TRect);
    function GetPrinterPixels: TRect;
    procedure SetPrinterPixels(Value: TRect);
    function GetPrinterClientRect: TRect;
    function GetScreenPixels: TRect;
    procedure SetScreenPixels(Value: TRect);
    procedure SetLeft(Value: Double);
    procedure SetTop(Value: Double);
    procedure SetRight(Value: Double);
    procedure SetBottom(Value: Double);
    procedure SetUnits(Value: TabcPrintUnits);
  protected
    procedure DoChange; dynamic;
  public
    constructor Create; virtual;
    procedure Assign(Source: TPersistent); override;
    property AsSysRect: TRect read GetSysRect write SetSysRect;
    property AsPrinterPixels: TRect read GetPrinterPixels write SetPrinterPixels;
    property PrinterClientRect: TRect read GetPrinterClientRect;
    property AsScreenPixels: TRect read GetScreenPixels write SetScreenPixels;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property Units: TabcPrintUnits read FUnits write SetUnits default apuInches;
  published
    property Left: Double read FLeft write SetLeft;
    property Top: Double read FTop write SetTop;
    property Right: Double read FRight write SetRight;
    property Bottom: Double read FBottom write SetBottom;
  end;

  { TabcPrintMarginsUnits }

  TabcPrintMarginsUnits = class(TabcPrintMargins)
  published
    property Units;
  end;

  { TabcPageSetupDialog }

  TabcPageSetupOption = (apsDefaultMinMargins, apsDisableMargins,
                         apsDisableOrientation, apsDisablePagePainting,
                         apsDisablePaper, apsDisablePrinter,
                         apsUseMargins, apsUseMinMargins,
                         apsShowHelp, apsHideNetworkBtn);

  TabcPageSetupOptions = set of TabcPageSetupOption;

  TabcPageSetupDialog = class(TComponent)
  private
    FCtl3D: Boolean;
    FDefWndProc: Pointer;
    FHelpContext: THelpContext;
    FHandle: HWnd;
    FObjectInstance: Pointer;
    FTemplate: PChar;
    FOnClose: TNotifyEvent;
    FOnShow: TNotifyEvent;
    FMargins: TabcPrintMargins;
    FMinMargins: TabcPrintMargins;
    FOptions: TabcPageSetupOptions;
    FUnits: TabcPrintUnits;
    procedure SetMargins(Value: TabcPrintMargins);
    procedure SetMinMargins(Value: TabcPrintMargins);
    procedure SetOptions(Value: TabcPageSetupOptions);
    procedure SetUnits(Value: TabcPrintUnits);
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMInitDialog(var Message: TWMInitDialog); message WM_INITDIALOG;
    procedure WMNCDestroy(var Message: TWMNCDestroy); message WM_NCDESTROY;
    procedure MainWndProc(var Message: TMessage);
  protected
    procedure DoClose; dynamic;
    procedure DoShow; dynamic;
    procedure WndProc(var Message: TMessage); virtual;
    function MessageHook(var Msg: TMessage): Boolean; virtual;
    function TaskModalDialog(DialogFunc: Pointer; var DialogData): Bool; virtual;
    property Template: PChar read FTemplate write FTemplate;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean; virtual;
    procedure DefaultHandler(var Message); override;
    property Handle: HWnd read FHandle;
  published
    property Ctl3D: Boolean read FCtl3D write FCtl3D default True;
    property HelpContext: THelpContext read FHelpContext write FHelpContext default 0;
    property Margins: TabcPrintMargins read FMargins write SetMargins;
    property MinMargins: TabcPrintMargins read FMinMargins write SetMinMargins;
    property Options: TabcPageSetupOptions read FOptions write SetOptions
      default [apsUseMargins];
    property Units: TabcPrintUnits read FUnits write SetUnits default apuInches;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

implementation

uses Printers;

const
  C_MM_PER_INCH = 25.37;

{ Common Dialog Support Functions }

var
  CreationControl: TabcPageSetupDialog = nil;
  HelpMsg: Cardinal;

{$IFDEF DELPHI5}

{ Center the given window on the screen }

procedure CenterWindow(Wnd: HWnd);
var
  Rect: TRect;
  Monitor: TMonitor;
begin
  GetWindowRect(Wnd, Rect);
  if Application.MainForm <> nil then
    Monitor := Application.MainForm.Monitor
  else
    Monitor := Screen.Monitors[0];
  SetWindowPos(Wnd, 0,
    Monitor.Left + ((Monitor.Width - Rect.Right + Rect.Left) div 2),
    Monitor.Top + ((Monitor.Height - Rect.Bottom + Rect.Top) div 3),
    0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
end;

{$ELSE}

procedure CenterWindow(Wnd: HWnd);
var
  Rect: TRect;
begin
  GetWindowRect(Wnd, Rect);
  SetWindowPos(Wnd, 0,
    (GetSystemMetrics(SM_CXSCREEN) - Rect.Right + Rect.Left) div 2,
    (GetSystemMetrics(SM_CYSCREEN) - Rect.Bottom + Rect.Top) div 3,
    0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
end;

{$ENDIF}

{ Generic dialog hook. Centers the dialog on the screen in response to
  the WM_INITDIALOG message }

function DialogHook(Wnd: HWnd; Msg: UINT; WParam: WPARAM; LParam: LPARAM): UINT; stdcall;
begin
  Result := 0;
  if Msg = WM_INITDIALOG then
  begin
    CenterWindow(Wnd);
    CreationControl.FHandle := Wnd;
    CreationControl.FDefWndProc := Pointer(SetWindowLong(Wnd, GWL_WNDPROC,
      Longint(CreationControl.FObjectInstance)));
    CallWindowProc(CreationControl.FObjectInstance, Wnd, Msg, WParam, LParam);
    CreationControl := nil;
  end;
end;

{ Printer dialog routines }

procedure GetPrinter(var DeviceMode, DeviceNames: THandle);
var
  Device, Driver, Port: array[0..79] of char;
  DevNames: PDevNames;
  Offset: PChar;
begin
  Printer.GetPrinter(Device, Driver, Port, DeviceMode);
  if DeviceMode <> 0 then
  begin
    DeviceNames := GlobalAlloc(GHND, SizeOf(TDevNames) +
     StrLen(Device) + StrLen(Driver) + StrLen(Port) + 3);
    DevNames := PDevNames(GlobalLock(DeviceNames));
    try
      Offset := PChar(DevNames) + SizeOf(TDevnames);
      with DevNames^ do
      begin
        wDriverOffset := Longint(Offset) - Longint(DevNames);
        Offset := StrECopy(Offset, Driver) + 1;
        wDeviceOffset := Longint(Offset) - Longint(DevNames);
        Offset := StrECopy(Offset, Device) + 1;
        wOutputOffset := Longint(Offset) - Longint(DevNames);;
        StrCopy(Offset, Port);
      end;
    finally
      GlobalUnlock(DeviceNames);
    end;
  end;
end;

procedure SetPrinter(DeviceMode, DeviceNames: THandle);
var
  DevNames: PDevNames;
begin
  DevNames := PDevNames(GlobalLock(DeviceNames));
  try
    with DevNames^ do
      Printer.SetPrinter(PChar(DevNames) + wDeviceOffset,
        PChar(DevNames) + wDriverOffset,
        PChar(DevNames) + wOutputOffset, DeviceMode);
  finally
    GlobalUnlock(DeviceNames);
    GlobalFree(DeviceNames);
  end;
end;

function CopyData(Handle: THandle): THandle;
var
  Src, Dest: PChar;
  Size: Integer;
begin
  if Handle <> 0 then
  begin
    Size := GlobalSize(Handle);
    Result := GlobalAlloc(GHND, Size);
    if Result <> 0 then
      try
        Src := GlobalLock(Handle);
        Dest := GlobalLock(Result);
        if (Src <> nil) and (Dest <> nil) then Move(Src^, Dest^, Size);
      finally
        GlobalUnlock(Handle);
        GlobalUnlock(Result);
      end
  end
  else
    Result := 0;
end;

{ TabcPrintMargins }

constructor TabcPrintMargins.Create;
begin
  FLeft := 1;
  FTop := 1;
  FRight := 1;
  FBottom := 1;
  FUnits := apuInches;
end;

procedure TabcPrintMargins.Assign(Source: TPersistent);
begin
  if Source is TabcPrintMargins then
    with TabcPrintMargins(Source) do
    begin
      Self.FLeft := Left;
      Self.FTop := Top;
      Self.FRight := Right;
      Self.FBottom := Bottom;
      Self.FUnits := Units;
      DoChange;
    end
  else
    inherited Assign(Source);
end;

procedure TabcPrintMargins.DoChange;
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

function TabcPrintMargins.GetSysRect: TRect;
var
  Mult: Integer;
begin
  if Units = apuInches then
    Mult := 1000
  else
    Mult := 100;
  Result := Rect(Trunc(FLeft * Mult), Trunc(FTop * Mult),
                 Trunc(FRight * Mult), Trunc(FBottom * Mult));
end;

procedure TabcPrintMargins.SetSysRect(Value: TRect);
var
  Divisor: Integer;
begin
  if Units = apuInches then
    Divisor := 1000
  else
    Divisor := 100;
  FLeft := Value.Left / Divisor;
  FTop := Value.Top / Divisor;
  FRight := Value.Right / Divisor;
  FBottom := Value.Bottom / Divisor;
  DoChange;
end;

function TabcPrintMargins.GetPrinterClientRect: TRect;
//adjust exactly for printer device caps - non-printable area
var
  DC: hDC;
  pixelsX, pixelsY, physX, physY, physOffsetX, physOffsetY: integer;
begin
  DC := Printer.Handle;
  pixelsX := GetDeviceCaps(DC, LOGPIXELSX);
  pixelsY := GetDeviceCaps(DC, LOGPIXELSY);
  physX := GetDeviceCaps(DC, PHYSICALWIDTH);
  physY := GetDeviceCaps(DC, PHYSICALHEIGHT);
  physOffsetX := GetDeviceCaps(DC, PHYSICALOFFSETX);
  physOffsetY := GetDeviceCaps(DC, PHYSICALOFFSETY);

  if Units = apuInches then
    Result := Rect(Trunc(FLeft * pixelsX), Trunc(FTop * pixelsY),
                   Trunc(FRight * pixelsX), Trunc(FBottom * pixelsY))
  else
    Result := Rect(Trunc(FLeft * pixelsX / C_MM_PER_INCH),
                   Trunc(FTop * pixelsY / C_MM_PER_INCH),
                   Trunc(FRight * pixelsX / C_MM_PER_INCH),
                   Trunc(FBottom * pixelsY / C_MM_PER_INCH));
  with Result do
  begin
    Left := Left - physOffsetX;
    Top := Top - physOffsetY;
    Right := physX - Right - physOffsetX;
    Bottom := physY - Bottom - physOffsetY;
  end;

(* Testing Only - Draw Rectangle on printer canvas:
  with Printer, Result do
  begin
    BeginDoc;
    Canvas.Rectangle(Left, Top, Right, Bottom);
    EndDoc;
  end;
*)

end;

function TabcPrintMargins.GetPrinterPixels: TRect;
var
  DC: hDC;
  pixelsX, pixelsY: integer;
begin
  DC := Printer.Handle;
  pixelsX := GetDeviceCaps(DC, LOGPIXELSX);
  pixelsY := GetDeviceCaps(DC, LOGPIXELSY);

  if Units = apuInches then
    Result := Rect(Trunc(FLeft * pixelsX), Trunc(FTop * pixelsY),
                   Trunc(FRight * pixelsX), Trunc(FBottom * pixelsY))
  else
    Result := Rect(Trunc(FLeft * pixelsX / C_MM_PER_INCH),
                   Trunc(FTop * pixelsY / C_MM_PER_INCH),
                   Trunc(FRight * pixelsX / C_MM_PER_INCH),
                   Trunc(FBottom * pixelsY / C_MM_PER_INCH));
end;

procedure TabcPrintMargins.SetPrinterPixels(Value: TRect);
var
  DC: hDC;
  pixelsX, pixelsY: integer;
begin
  DC := Printer.Handle;
  pixelsX := GetDeviceCaps(DC, LOGPIXELSX);
  pixelsY := GetDeviceCaps(DC, LOGPIXELSY);

  if Units = apuInches then
  begin
    FLeft := Value.Left / pixelsX;
    FTop := Value.Top / pixelsY;
    FRight := Value.Right / pixelsX;
    FBottom := Value.Bottom / pixelsY;
  end
  else
  begin
    FLeft := Value.Left * C_MM_PER_INCH / pixelsX;
    FTop := Value.Top * C_MM_PER_INCH / pixelsY;
    FRight := Value.Right * C_MM_PER_INCH / pixelsX;
    FBottom := Value.Bottom * C_MM_PER_INCH / pixelsY;
  end;
  DoChange;
end;

function TabcPrintMargins.GetScreenPixels: TRect;
var
  ScreenDC: hDC;
  pixelsX, pixelsY: integer;
begin
  ScreenDC := GetDC(0);
  pixelsX := GetDeviceCaps(ScreenDC, LOGPIXELSX);
  pixelsY := GetDeviceCaps(ScreenDC, LOGPIXELSY);
  ReleaseDC(0, ScreenDC);

  if Units = apuInches then
    Result := Rect(Trunc(FLeft * pixelsX), Trunc(FTop * pixelsY),
                   Trunc(FRight * pixelsX), Trunc(FBottom * pixelsY))
  else
    Result := Rect(Trunc(FLeft * pixelsX / C_MM_PER_INCH),
                   Trunc(FTop * pixelsY / C_MM_PER_INCH),
                   Trunc(FRight * pixelsX / C_MM_PER_INCH),
                   Trunc(FBottom * pixelsY / C_MM_PER_INCH));
end;

procedure TabcPrintMargins.SetScreenPixels(Value: TRect);
var
  ScreenDC: hDC;
  pixelsX, pixelsY: integer;
begin
  ScreenDC := GetDC(0);
  pixelsX := GetDeviceCaps(ScreenDC, LOGPIXELSX);
  pixelsY := GetDeviceCaps(ScreenDC, LOGPIXELSY);
  ReleaseDC(0, ScreenDC);

  if Units = apuInches then
  begin
    FLeft := Value.Left / pixelsX;
    FTop := Value.Top / pixelsY;
    FRight := Value.Right / pixelsX;
    FBottom := Value.Bottom / pixelsY;
  end
  else
  begin
    FLeft := Value.Left * C_MM_PER_INCH / pixelsX;
    FTop := Value.Top * C_MM_PER_INCH / pixelsY;
    FRight := Value.Right * C_MM_PER_INCH / pixelsX;
    FBottom := Value.Bottom * C_MM_PER_INCH / pixelsY;
  end;
  DoChange;
end;

procedure TabcPrintMargins.SetLeft(Value: Double);
begin
  if Value <> FLeft then
  begin
    FLeft := Value;
    DoChange;
  end;
end;

procedure TabcPrintMargins.SetTop(Value: Double);
begin
  if Value <> FTop then
  begin
    FTop := Value;
    DoChange;
  end;
end;

procedure TabcPrintMargins.SetRight(Value: Double);
begin
  if Value <> FRight then
  begin
    FRight := Value;
    DoChange;
  end;
end;

procedure TabcPrintMargins.SetBottom(Value: Double);
begin
  if Value <> FBottom then
  begin
    FBottom := Value;
    DoChange;
  end;
end;

procedure TabcPrintMargins.SetUnits(Value: TabcPrintUnits);
begin
  if Value <> FUnits then
  begin
    FUnits := Value;
    DoChange;
  end;
end;

{ TabcPageSetupDialog }

constructor TabcPageSetupDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCtl3D := True;
  FMargins := TabcPrintMargins.Create;
  FMinMargins := TabcPrintMargins.Create;
  FOptions := [apsUseMargins];
  FUnits := apuInches;
  FObjectInstance := {$IFDEF DELPHI6}Classes.{$ENDIF}MakeObjectInstance(MainWndProc);
end;

destructor TabcPageSetupDialog.Destroy;
begin
  if FObjectInstance <> nil then {$IFDEF DELPHI6}Classes.{$ENDIF}FreeObjectInstance(FObjectInstance);
  FMargins.Free;
  FMinMargins.Free;
  inherited Destroy;
end;

function TabcPageSetupDialog.Execute: Boolean;
const
  DialogOptions: array[TabcPageSetupOption] of DWORD = (
    PSD_DEFAULTMINMARGINS, PSD_DISABLEMARGINS,
    PSD_DISABLEORIENTATION, PSD_DISABLEPAGEPAINTING,
    PSD_DISABLEPAPER, PSD_DISABLEPRINTER,
    PSD_MARGINS, PSD_MINMARGINS, PSD_SHOWHELP, PSD_NONETWORKBUTTON);
var
  PageSetupRec: TPageSetupDlg;
  Option: TabcPageSetupOption;
  DevHandle: THandle;

begin
  with PageSetupRec do
  begin
    ZeroMemory(@PageSetupRec, SizeOf(TPageSetupDlg));
    lStructSize := SizeOf(TPageSetupDlg);
    hInstance := SysInit.HInstance;
    GetPrinter(DevHandle, hDevNames);
    hDevMode := CopyData(DevHandle);

    Flags := PSD_ENABLEPAGESETUPHOOK;
    for Option := Low(Option) to High(Option) do
      if Option in Options then
        Flags := Flags or DialogOptions[Option];

    if apsHideNetworkBtn in Options then
      if (Win32MajorVersion < 4) or ((Win32MajorVersion = 4) and (Win32Platform = VER_PLATFORM_WIN32_WINDOWS)) then
        //fix for Windows95 Microsoft bug
        Flags := Flags and not PSD_NONETWORKBUTTON;

    if Units = apuInches then
      Flags := Flags or PSD_INTHOUSANDTHSOFINCHES
    else
      Flags := Flags or PSD_INHUNDREDTHSOFMILLIMETERS;

    if apsUseMargins in Options then
      rtMargin := FMargins.AsSysRect;
    if apsUseMinMargins in Options then
      rtMinMargin := FMinMargins.AsSysRect;

    lpfnPageSetupHook := DialogHook;
    hWndOwner := Application.Handle;

    CreationControl := Self;

    Result := TaskModalDialog(@PageSetupDlg, PageSetupRec);

    if Result then
    begin
      FMargins.AsSysRect := rtMargin;
      FMinMargins.AsSysRect := rtMinMargin;
      SetPrinter(hDevMode, hDevNames);
    end
    else begin
      if hDevMode <> 0 then GlobalFree(hDevMode);
      if hDevNames <> 0 then GlobalFree(hDevNames);
    end;
  end;
end;

function TabcPageSetupDialog.MessageHook(var Msg: TMessage): Boolean;
begin
  Result := False;
  if (Msg.Msg = HelpMsg) and (FHelpContext <> 0) then
  begin
    Application.HelpContext(FHelpContext);
    Result := True;
  end;
end;

procedure TabcPageSetupDialog.DefaultHandler(var Message);
begin
  if FHandle <> 0 then
    with TMessage(Message) do
      Result := CallWindowProc(FDefWndProc, FHandle, Msg, WParam, LParam)
  else inherited DefaultHandler(Message);
end;

procedure TabcPageSetupDialog.MainWndProc(var Message: TMessage);
begin
  try
    WndProc(Message);
  except
    Application.HandleException(Self);
  end;
end;

procedure TabcPageSetupDialog.WndProc(var Message: TMessage);
begin
  Dispatch(Message);
end;

procedure TabcPageSetupDialog.WMDestroy(var Message: TWMDestroy);
begin
  inherited;
  DoClose;
end;

procedure TabcPageSetupDialog.WMInitDialog(var Message: TWMInitDialog);
begin
  { Called only by non-explorer style dialogs }
  DoShow;
  { Prevent any further processing }
  Message.Result := 0;
end;

procedure TabcPageSetupDialog.WMNCDestroy(var Message: TWMNCDestroy);
begin
  inherited;
  FHandle := 0;
end;

function TabcPageSetupDialog.TaskModalDialog(DialogFunc: Pointer; var DialogData): Bool;
type
  TDialogFunc = function(var DialogData): Bool stdcall;
var
  ActiveWindow: HWnd;
  WindowList: Pointer;
  FPUControlWord: Word;
{$IFDEF DELPHI6}
  FocusState: TFocusState;
{$ENDIF}
begin
  ActiveWindow := GetActiveWindow;
  WindowList := DisableTaskWindows(0);
{$IFDEF DELPHI6}
  FocusState := SaveFocusState;
{$ENDIF}
  try
    Application.HookMainWindow(MessageHook);
    asm
      // Avoid FPU control word change in NETRAP.dll, NETAPI32.dll, etc
      FNSTCW  FPUControlWord
    end;
    try
      CreationControl := Self;
      Result := TDialogFunc(DialogFunc)(DialogData);
    finally
      asm
        FNCLEX
        FLDCW FPUControlWord
      end;
      Application.UnhookMainWindow(MessageHook);
    end;
  finally
    EnableTaskWindows(WindowList);
    SetActiveWindow(ActiveWindow);
{$IFDEF DELPHI6}
    RestoreFocusState(FocusState);
{$ENDIF}
  end;
end;

procedure TabcPageSetupDialog.DoClose;
begin
  if Assigned(FOnClose) then FOnClose(Self);
end;

procedure TabcPageSetupDialog.DoShow;
begin
  if Assigned(FOnShow) then FOnShow(Self);
end;

procedure TabcPageSetupDialog.SetMargins(Value: TabcPrintMargins);
begin
  FMargins.Assign(Value);
end;

procedure TabcPageSetupDialog.SetMinMargins(Value: TabcPrintMargins);
begin
  FMinMargins.Assign(Value);
end;

procedure TabcPageSetupDialog.SetOptions(Value: TabcPageSetupOptions);
begin
  if Value <> FOptions then
  begin
    {mutually exclusive options}
    if (apsDefaultMinMargins in Value)
      and (apsUseMargins in Value)
      and (apsUseMinMargins in Value) then
      if apsDefaultMinMargins in FOptions then
        FOptions := Value - [apsDefaultMinMargins]
      else
        FOptions := Value - [apsUseMargins, apsUseMinMargins]
    else
      FOptions := Value;
  end;
end;

procedure TabcPageSetupDialog.SetUnits(Value: TabcPrintUnits);
begin
  FUnits := Value;
  FMargins.Units := Value;
  FMinMargins.Units := Value;
end;

end.
