unit abcpanel;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  ExtCtrls, Menus, ComCtrls, abcutilg, Dialogs, abcapp;

type

  { TabcCustomEdgePanel }

  TabcCustomEdgePanel = class(TCustomPanel)
  private
    FBevelEdges: TBevelEdges;
    FTransparent: Boolean;
    procedure SetBevelEdges(Value: TBevelEdges);
    procedure SetTransparent(const Value: Boolean);
    procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
    procedure WMMove(var Message: TWMMove); message WM_MOVE;
  protected
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Paint; override;
    property BevelEdges: TBevelEdges read FBevelEdges write SetBevelEdges
      default [beLeft, beTop, beRight, beBottom];
    property Transparent: Boolean read FTransparent write SetTransparent default False;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Invalidate; override;
  end;

  { TabcEdgePanel }

  TabcEdgePanel = class(TabcCustomEdgePanel)
  public
    property DockManager;
  published
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property BorderStyle;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Caption;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragKind;
    property DockSite;
    property UseDockManager default True;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Transparent;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDrag;
    property OnCanResize;
    property OnConstrainedResize;
    property OnDockDrop;
    property OnDockOver;
    property OnGetSiteInfo;
    property OnStartDock;
    property OnEndDock;
    property OnUnDock;
  end;

  { TabcHintBar }
  { Note: different versions for Delphi 1 and Delphi 2 to take advantage of
    the TStatusBar style and features such as TSizeGrip.
    Some display properties are different.  This may cause an initial
    error when upgrading a Delphi 1 form into Delphi 2.  Just ignore the errors.}

  TabcHintBar = class(TStatusBar)
  private
    FHint, FHintPrefix, FHintSuffix: string;
    FOnActiveControlChange: TNotifyEvent;
    FOnHint: TNotifyEvent;
    FShowActiveControl: Boolean;
    FShowEmpty: Boolean;
    procedure DoAppOnHint(Sender: TObject);
    procedure DoScreenActiveControlChange(Sender: TObject);
    procedure DoShowCaption;
    function GetSimpleText: string;
    procedure SetHintPrefix(Value: string);
    procedure SetHintSuffix(Value: string);
    procedure SetShowActiveControl(Value: Boolean);
    procedure SetShowEmpty(Value: Boolean);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property HintPrefix: string read FHintPrefix write SetHintPrefix;
    property HintSuffix: string read FHintSuffix write SetHintSuffix;
    property ShowActiveControl: Boolean read FShowActiveControl write SetShowActiveControl default False;
    property ShowEmpty: Boolean read FShowEmpty write SetShowEmpty default False;
    property SimplePanel default True;
    property SimpleText: string read GetSimpleText; {hide unused property from OI}
    property OnActiveControlChange: TNotifyEvent read FOnActiveControlChange write FOnActiveControlChange;
    property OnHint: TNotifyEvent read FOnHint write FOnHint;
  end;

  { TabcCustomDragPanel }

  TSizeMoveOption = (smMove, smMoveInParent, smSizeLeft, smSizeRight, smSizeTop, smSizeBottom);

  TabcSizeMoveOptions = set of TSizeMoveOption;

  TabcCustomDragPanel = class(TabcEdgePanel)
  private
    FMSLocked: Boolean;
    FMoveMaxLeft: Integer;
    FMoveMaxTop: Integer;
    FMoveMinLeft: Integer;
    FMoveMinTop: Integer;
    FMoving: Boolean;
    FOnMove: TNotifyEvent;
    FOnSize: TNotifyEvent;
    FOptions: TabcSizeMoveOptions;
    FSizeMaxHeight: Integer;
    FSizeMaxWidth: Integer;
    FSizeMinHeight: Integer;
    FSizeMinWidth: Integer;
    FSizing: Boolean;
    procedure SetMoveMaxLeft(Value: Integer);
    procedure SetMoveMaxTop(Value: Integer);
    procedure SetMoveMinLeft(Value: Integer);
    procedure SetMoveMinTop(Value: Integer);
    procedure SetSizeMinHeight(Value: Integer);
    procedure SetSizeMinWidth(Value: Integer);
    procedure SetSizeMaxHeight(Value: Integer);
    procedure SetSizeMaxWidth(Value: Integer);
    procedure SetOptions(Value: TabcSizeMoveOptions);
    procedure WMGetMinMaxInfo(var Message: TMessage); message WM_GETMINMAXINFO;
    procedure WMMove(var Message: TWMMove); message WM_MOVE;
    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMWindowPosChanging(var Message: TMessage); message WM_WINDOWPOSCHANGING;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    property MoveMaxLeft: Integer read FMoveMaxLeft write SetMoveMaxLeft default 0;
    property MoveMaxTop: Integer read FMoveMaxTop write SetMoveMaxTop default 0;
    property MoveMinLeft: Integer read FMoveMinLeft write SetMoveMinLeft default 0;
    property MoveMinTop: Integer read FMoveMinTop write SetMoveMinTop default 0;
    property MSLocked: Boolean read FMSLocked write FMSLocked default False;
    property SizeMaxHeight: Integer read FSizeMaxHeight write SetSizeMaxHeight default 0;
    property SizeMaxWidth: Integer read FSizeMaxWidth write SetSizeMaxWidth default 0;
    property SizeMinHeight: Integer read FSizeMinHeight write SetSizeMinHeight default 30;
    property SizeMinWidth: Integer read FSizeMinWidth write SetSizeMinWidth default 30;
    property Options: TabcSizeMoveOptions read FOptions write SetOptions default
      [smMove, smMoveInParent, smSizeLeft, smSizeRight, smSizeTop, smSizeBottom];
    property OnMove: TNotifyEvent read FOnMove write FOnMove;
    property OnSize: TNotifyEvent read FOnSize write FOnSize;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  end;

  { TabcDragPanel }

  TabcDragPanel = class(TabcCustomDragPanel)
  published
    property BevelInner default bvNone;
    property BevelOuter default bvNone;
    property MSLocked;
    property MoveMaxLeft;
    property MoveMaxTop;
    property MoveMinLeft;
    property MoveMinTop;
    property Options;
    property SizeMaxHeight;
    property SizeMaxWidth;
    property SizeMinHeight;
    property SizeMinWidth;
    property OnMove;
    property OnSize;
  end;

  { TabcCustomSpeedPanel }

  TabcPosInt = 0..Maxint;

  TabcCustomSpeedPanel = class(TabcCustomEdgePanel)
  private
    FAligningControls: Boolean;
    FCtrlCols: TabcPosInt;
    FCtrlHeight: TabcPosInt;
    FCtrlRows: TabcPosInt;
    FCtrlWidth: TabcPosInt;
    FInsertCols: Boolean;
    FMarginCol: Integer;
    FMarginH: Integer;
    FMarginRow: Integer;
    FMarginV: Integer;
    FOrderingControls: Boolean;
    FSizeVisibleOnly: Boolean;
    FSorted: Boolean;
    function GetVisibleControlCount: Integer;
    procedure OrderControls(NewOrder: TList);
    procedure SetCtrlCols(Value: TabcPosInt);
    procedure SetCtrlHeight(Value: TabcPosInt);
    procedure SetCtrlRows(Value: TabcPosInt);
    procedure SetCtrlWidth(Value: TabcPosInt);
    procedure SetMarginCol(Value: Integer);
    procedure SetMarginH(Value: Integer);
    procedure SetMarginRow(Value: Integer);
    procedure SetMarginV(Value: Integer);
    procedure SetSizeVisibleOnly(Value: Boolean);
    procedure SetSorted(Value: Boolean);
  protected
    procedure AdjustSize; override;
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
    property CtrlCols: TabcPosInt read FCtrlCols write SetCtrlCols default 0;
    property CtrlHeight: TabcPosInt read FCtrlHeight write SetCtrlHeight default 24;
    property CtrlRows: TabcPosInt read FCtrlRows write SetCtrlRows default 1;
    property CtrlWidth: TabcPosInt read FCtrlWidth write SetCtrlWidth default 24;
    property InsertCols: Boolean read FInsertCols write FInsertCols default False;
    property MarginCol: Integer read FMarginCol write SetMarginCol default 0;
    property MarginH: Integer read FMarginH write SetMarginH default 5;
    property MarginRow: Integer read FMarginRow write SetMarginRow default 0;
    property MarginV: Integer read FMarginV write SetMarginV default 5;
    property SizeVisibleOnly: Boolean read FSizeVisibleOnly
               write SetSizeVisibleOnly default False;
    property Sorted: Boolean read FSorted write SetSorted default True;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AlignControlRows; virtual;
    procedure SortControls;
  end;

  { TabcSpeedPanel }

  TabcSpeedPanel = class(TabcCustomSpeedPanel)
  public
    property DockManager;
  published
    property Align;
    {property Alignment; NOT Published }
    property Anchors;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DockSite;
    property UseDockManager default True;
    property DragMode;
    property Enabled;
    {property Caption; NOT Published }
    property Color;
    property Ctl3D;
    property CtrlCols;
    property CtrlHeight;
    property CtrlRows;
    property CtrlWidth;
    property Font;
    property InsertCols;
    {property Locked; NOT Published}
    property MarginCol;
    property MarginH;
    property MarginRow;
    property MarginV;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property SizeVisibleOnly;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Transparent;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDrag;
    property OnCanResize;
    property OnConstrainedResize;
    property OnDockDrop;
    property OnDockOver;
    property OnGetSiteInfo;
    property OnUnDock;
    property OnStartDock;
    property OnEndDock;
  end;

  { TabcPictureGroup }

  TabcPictureGroup = class(TabcEdgePanel)
  private
    FAlignPicture: TAlignment;
    FLineRaised: Boolean;
    FMarginL: Integer;
    FMarginT: Integer;
    FMarginR: Integer;
    FMarginB: Integer;
    FOffset: Integer;
    FPicture: TPicture;
    procedure PictureChange(Sender: TObject);
    procedure SetAlignPicture(const Value: TAlignment);
    procedure SetLineRaised(Value: Boolean);

    function GetInnerRect: TRect;
    procedure SetMarginL(Value: Integer);
    procedure SetMarginT(Value: Integer);
    procedure SetMarginR(Value: Integer);
    procedure SetMarginB(Value: Integer);

    procedure SetOffset(Value: Integer);
    procedure SetPicture(const Value: TPicture);
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
  protected
    procedure AdjustClientRect(var Rect: TRect); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property InnerRect: TRect read GetInnerRect;
  published
    property AlignPicture: TAlignment read FAlignPicture write SetAlignPicture default taLeftJustify;
    property LineRaised: Boolean read FLineRaised write SetLineRaised default False;
    property MarginL: Integer read FMarginL write SetMarginL default 60;
    property MarginT: Integer read FMarginT write SetMarginT default 35;
    property MarginR: Integer read FMarginR write SetMarginR default 20;
    property MarginB: Integer read FMarginB write SetMarginB default 10;
    property Offset: Integer read FOffset write SetOffset default 10;
    property Picture: TPicture read FPicture write SetPicture;
    {inherited}
    property Alignment default taLeftJustify;
    property BevelOuter default bvNone;
  end;

implementation

uses abcutilm;

{ TabcCustomEdgePanel }

constructor TabcCustomEdgePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FBevelEdges := [beLeft, beTop, beRight, beBottom];
end;

procedure TabcCustomEdgePanel.AlignControls(AControl: TControl;
  var Rect: TRect);
begin
  inherited;
  if Transparent then Invalidate;
end;

procedure TabcCustomEdgePanel.CreateParams(var Params: TCreateParams);
begin
  inherited;
  with Params do
    if Transparent then
      ExStyle := ExStyle or WS_EX_TRANSPARENT;
end;

procedure TabcCustomEdgePanel.Invalidate;
var
 R: TRect;
 i: Integer;
begin
  if Transparent and (Parent <> nil) and Parent.HandleAllocated then
  begin
    R := BoundsRect;
    InvalidateRect(Parent.Handle, @R, True);
    for i := 0 to ControlCount - 1 do
      Controls[i].Invalidate;
  end;
  inherited;
end;

procedure TabcCustomEdgePanel.Paint;
var
  Rect: TRect;
  TopColor, BottomColor: TColor;
  Text: array[0..255] of Char;
  FontHeight: Integer;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;

begin
  if (BevelEdges = [beLeft, beTop, beRight, beBottom]) and not Transparent then
    inherited Paint
  else with Canvas do begin
    Rect := GetClientRect;
    Brush.Color := Color;
    if not Transparent then
      FillRect(Rect);
    if BevelOuter <> bvNone then begin
      AdjustColors(BevelOuter);
      abcEdgeFrame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth, BevelEdges);
    end;
    Frame3D(Canvas, Rect, Color, Color, BorderWidth);
    if BevelInner <> bvNone then begin
      AdjustColors(BevelInner);
      abcEdgeFrame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth, BevelEdges);
    end;
    if Caption <> '' then begin
      Brush.Style := bsClear;
      Font := Self.Font;
      FontHeight := TextHeight('W');
      with Rect do begin
        Top := ((Bottom + Top) - FontHeight) shr 1;
        Bottom := Top + FontHeight;
      end;
      StrPCopy(Text, Caption);
      DrawText(Handle, Text, StrLen(Text), Rect, (DT_EXPANDTABS or
        DT_VCENTER) or Alignments[Alignment]);
    end;
  end;
end;

procedure TabcCustomEdgePanel.SetBevelEdges(Value: TBevelEdges);
begin
  if Value <> FBevelEdges then begin
    FBevelEdges := Value;
    Invalidate;
  end;
end;

procedure TabcCustomEdgePanel.SetTransparent(const Value: Boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    if Value then
      ControlStyle := ControlStyle - [csOpaque]
    else
      ControlStyle := ControlStyle + [csOpaque];
    if HandleAllocated then RecreateWnd;
    if not (csDestroying in ComponentState) then Invalidate;
  end;
end;

procedure TabcCustomEdgePanel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  if not Transparent then
    inherited
  else
    Message.Result := 1;
end;

procedure TabcCustomEdgePanel.WMMove(var Message: TWMMove);
begin
  inherited;
  if Transparent then Invalidate;
end;

{ TabcHintBar }

constructor TabcHintBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 20;
  ParentFont := False;
  Font.Name := 'MS Sans Serif';
  Font.Size := 8;
  Font.Style := [];
  if not (csDesigning in AOwner.ComponentState) then
  begin
    abcApplicationEvents.RegisterOnHint(DoAppOnHint);
    abcScreenEvents.RegisterOnActiveControlChange(DoScreenActiveControlChange);
  end;
  SimplePanel := True;
end;

procedure TabcHintBar.DoShowCaption;

  procedure SetNewText(s: string);
  begin
    if SimplePanel then
      inherited SimpleText := S
    else if Panels.Count > 0 then
      Panels[0].Text := S;
  end;

begin
  if Enabled and (FHint <> '') or FShowEmpty then
    SetNewText(FHintPrefix + FHint + FHintSuffix)
  else
    SetNewText('');
end;

function TabcHintBar.GetSimpleText;
begin
  Result := inherited SimpleText;
end;

destructor TabcHintBar.Destroy;
begin
  if not (csDesigning in ComponentState) then
  begin
    abcApplicationEvents.UnRegisterOnHint(DoAppOnHint);
    abcScreenEvents.UnRegisterOnActiveControlChange(DoScreenActiveControlChange);
  end;
  inherited Destroy;
end;

procedure TabcHintBar.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  DoShowCaption;
end;

procedure TabcHintBar.DoAppOnHint(Sender: TObject);
begin
  if not FShowActiveControl then begin
    FHint := Application.Hint;
    DoShowCaption;
  end;
  if Assigned(FOnHint) then FOnHint(Sender);
end;

procedure TabcHintBar.DoScreenActiveControlChange(Sender: TObject);
var
  SepPos: Integer;
begin
  if FShowActiveControl then begin
    if Screen.ActiveControl = nil then
      FHint := ''
    else begin
      FHint := Screen.ActiveControl.Hint;
      SepPos := Pos('|', FHint);
      if SepPos <> 0 then FHint := Copy(FHint, SepPos + 1, Length(FHint));
    end;
    DoShowCaption;
  end;
  if Assigned(FOnActiveControlChange) then FOnActiveControlChange(Sender);
end;

procedure TabcHintBar.SetShowActiveControl(Value: Boolean);
begin
  if Value <> FShowActiveControl then begin
    FShowActiveControl := Value;
    if Value then
      if Screen.ActiveControl = nil then
        FHint := ''
      else
        FHint := Screen.ActiveControl.Hint
    else
      FHint := Application.Hint;
    DoShowCaption;
  end;
end;

procedure TabcHintBar.SetHintPrefix(Value: string);
begin
  if Value <> FHintPrefix then begin
    FHintPrefix := Value;
    DoShowCaption;
  end;
end;

procedure TabcHintBar.SetHintSuffix(Value: string);
begin
  if Value <> FHintSuffix then begin
    FHintSuffix := Value;
    DoShowCaption;
  end;
end;

procedure TabcHintBar.SetShowEmpty(Value: Boolean);
begin
  if Value <> FShowEmpty then begin
    FShowEmpty := Value;
    DoShowCaption;
  end;
end;

{ TabcCustomDragPanel }

constructor TabcCustomDragPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BevelInner := bvNone;
  BevelOuter := bvNone;
  FOptions := [smMove, smMoveInParent, smSizeLeft, smSizeRight, smSizeTop, smSizeBottom];
  FSizeMinHeight := 30;
  FSizeMinWidth := 30;
end;

procedure TabcCustomDragPanel.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_THICKFRAME;
end;

procedure TabcCustomDragPanel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
const
  SC_DRAGMOVE : Longint = $F012;
begin
  if not FMSLocked and (smMove in Options) and (Align = alNone) then begin
    FMoving := True;
    ReleaseCapture;
    SendMessage(Handle, WM_SYSCOMMAND, SC_DRAGMOVE, 0);
  end
  else
    inherited MouseDown(Button, Shift, X, Y);
end;

procedure TabcCustomDragPanel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  if FMSLocked or (Align = alClient) then Exit;
  if (Left > FMoveMaxLeft) and (FMoveMaxLeft <> 0) then FMoveMaxLeft := Left;
  if (Left < FMoveMinLeft) then FMoveMinLeft := Left;
  if (Top > FMoveMaxTop) and (FMoveMaxTop <> 0) then FMoveMaxTop := Top;
  if (Top < FMoveMinTop) then FMoveMinTop := Top;
  if (Width > FSizeMaxWidth) and (FSizeMaxWidth <> 0) then FSizeMaxWidth := Width;
  if (Width < FSizeMinWidth) then FSizeMinWidth := Width;
  if (Height > FSizeMaxHeight) and (FSizeMaxHeight <> 0) then FSizeMaxHeight := Height;
  if (Height < FSizeMinHeight) then FSizeMinHeight := Height;
end;

procedure TabcCustomDragPanel.SetMoveMaxLeft(Value: Integer);
begin
  if Value <> FMoveMaxLeft then begin
    if (Value < Left) and (Value <> 0) then
      FMoveMaxLeft := Left
    else
      FMoveMaxLeft := Value;
  end;
end;

procedure TabcCustomDragPanel.SetMoveMaxTop(Value: Integer);
begin
  if Value <> FMoveMaxTop then begin
    if (Value < Top) and (Value <> 0) then
      FMoveMaxTop := Top
    else
      FMoveMaxTop := Value;
  end;
end;

procedure TabcCustomDragPanel.SetMoveMinLeft(Value: Integer);
begin
  if Value <> FMoveMinLeft then begin
    if Value > Left then
      FMoveMinLeft := Left
    else
      FMoveMinLeft := Value;
  end;
end;

procedure TabcCustomDragPanel.SetMoveMinTop(Value: Integer);
begin
  if Value <> FMoveMinTop then begin
    if Value > Top then
      FMoveMinTop := Top
    else
      FMoveMinTop := Value;
  end;
end;

procedure TabcCustomDragPanel.SetSizeMinHeight(Value: Integer);
begin
  if Value <> FSizeMinHeight then begin
    if Value > Height then
      FSizeMinHeight := Height
    else
      FSizeMinHeight := Value;
  end;
end;

procedure TabcCustomDragPanel.SetSizeMinWidth(Value: Integer);
begin
  if Value <> FSizeMinWidth then begin
    if Value > Width then
      FSizeMinWidth := Width
    else
      FSizeMinWidth := Value;
  end;
end;

procedure TabcCustomDragPanel.SetSizeMaxHeight(Value: Integer);
begin
  if Value <> FSizeMaxHeight then begin
    if (Value < Height) and (Value <> 0) then
      FSizeMaxHeight := Height
    else
      FSizeMaxHeight := Value;
  end;
end;

procedure TabcCustomDragPanel.SetSizeMaxWidth(Value: Integer);
begin
  if Value <> FSizeMaxWidth then begin
    if (Value < Width) and (Value <> 0) then
      FSizeMaxWidth := Width
    else
      FSizeMaxWidth := Value;
  end;
end;

procedure TabcCustomDragPanel.SetOptions(Value: TabcSizeMoveOptions);
begin
  if Value <> FOptions then begin
    FOptions := Value;
    if smMoveInParent in Value then
      FOptions := FOptions + [smMove];
  end;
end;

procedure TabcCustomDragPanel.WMGetMinMaxInfo(var Message: TMessage);
begin
  with PMinMaxInfo(Message.lParam)^ do begin
    ptMinTrackSize := Point(FSizeMinWidth, FSizeMinHeight);
    if FSizeMaxWidth <> 0 then
      ptMaxTrackSize.x := FSizeMaxWidth;
    if FSizeMaxHeight <> 0 then
      ptMaxTrackSize.y := FSizeMaxHeight;
  end;
end;

procedure TabcCustomDragPanel.WMMove(var Message: TWMMove);
begin
  inherited;
  if FMoving then begin
    FMoving := False;
    Parent.Realign;
    if Assigned(FOnMove) then FOnMove(Self);
  end;
end;

procedure TabcCustomDragPanel.WMNCHitTest(var Message: TWMNCHitTest);
begin
  inherited;
  with Message do begin
    if FMSLocked or (Align = alClient) then
      Result := HTCLIENT
    else
      case Result of
        Windows.HTNOWHERE..Windows.HTCLIENT: ;
        Windows.HTLEFT:
          if not ((smSizeLeft in Options) and (Align in [alNone, alRight]))  then
            Result := Windows.HTCLIENT;
        Windows.HTRIGHT:
          if not ((smSizeRight in Options) and (Align in [alNone, alLeft])) then
            Result := Windows.HTCLIENT;
        Windows.HTTOP:
          if not ((smSizeTop in Options) and (Align in [alNone, alBottom])) then
            Result := Windows.HTCLIENT;
        Windows.HTTOPLEFT:
          if not ((smSizeTop in Options) and (Align in [alNone, alBottom])) then
            if ((smSizeLeft in Options) and (Align in [alNone, alRight])) then
              Result := Windows.HTLEFT
            else
              Result := Windows.HTCLIENT
          else if not ((smSizeLeft in Options) and (Align in [alNone, alRight])) then
            if (Align in [alNone, alBottom]) then
              Result := Windows.HTTOP
            else
              Result := Windows.HTCLIENT;
        Windows.HTTOPRIGHT:
          if not ((smSizeTop in Options) and (Align in [alNone, alBottom])) then
            if ((smSizeRight in Options) and (Align in [alNone, alLeft])) then
              Result := Windows.HTRIGHT
            else
              Result := Windows.HTCLIENT
          else if not ((smSizeRight in Options) and (Align in [alNone, alLeft])) then
            if (Align in [alNone, alBottom]) then
              Result := Windows.HTTOP
            else
              Result := Windows.HTCLIENT;
        Windows.HTBOTTOM:
          if not ((smSizeBottom in Options) and (Align in [alNone, alTop])) then
            Result := Windows.HTCLIENT;
        Windows.HTBOTTOMLEFT:
          if not ((smSizeBottom in Options) and (Align in [alNone, alTop])) then
            if ((smSizeLeft in Options) and (Align in [alNone, alRight])) then
              Result := Windows.HTLEFT
            else
              Result := Windows.HTCLIENT
          else if not ((smSizeLeft in Options) and (Align in [alNone, alRight])) then
            if (Align in [alNone, alTop]) then
              Result := Windows.HTBOTTOM
            else
              Result := Windows.HTCLIENT;
        Windows.HTBOTTOMRIGHT:
          if not ((smSizeBottom in Options) and (Align in [alNone, alTop])) then
            if ((smSizeRight in Options) and (Align in [alNone, alLeft])) then
              Result := Windows.HTRIGHT
            else
              Result := Windows.HTCLIENT
          else if not ((smSizeRight in Options) and (Align in [alNone, alLeft])) then
            if (Align in [alNone, alTop]) then
              Result := Windows.HTBOTTOM
            else
              Result := Windows.HTCLIENT;
      end;
    if Result in [Windows.HTLEFT..Windows.HTBOTTOMRIGHT] then
      FSizing := True;
  end;
end;

procedure TabcCustomDragPanel.WMSize(var Message: TWMSize);
begin
  inherited;
  if FSizing then begin
    FSizing := False;
    Parent.Realign;
    if Assigned(FOnSize) then FOnSize(Self);
  end;
end;

procedure TabcCustomDragPanel.WMWindowPosChanging(var Message: TMessage);
begin
  if FMoving then
    with PWindowPos(Message.lParam)^ do begin
      if x < FMoveMinLeft then
        x := FMoveMinLeft;
      if y < FMoveMinTop then
        y := FMoveMinTop;
      if (FMoveMaxLeft <> 0) and (x > FMoveMaxLeft) then
        x := FMoveMaxLeft;
      if (FMoveMaxTop <> 0) and (y > FMoveMaxTop) then
        y := FMoveMaxTop;
      if (smMoveInParent in Options) then
        with Parent do begin
          if x < 0 then x := 0;
          if y < 0 then y := 0;
          if (x + cx) > ClientWidth then
            if cx < ClientWidth then
              x := ClientWidth - cx
            else
              x := 0;
          if (y + cy) > ClientHeight then
            if cy < ClientHeight then
              y := ClientHeight - cy
            else
              y := 0;
        end;
    end;
end;

{ TabcCustomSpeedPanel }

function GetColSize(Items: TabcPosInt; var Rows: TabcPosInt; RoundUp: Boolean): TabcPosInt;
var
  CalcRows: TabcPosInt;
begin
  if Items = 0 then begin
    Rows := 0;
    Result := 0;
  end
  else if Rows >= Items then begin
    Rows := Items;
    Result := 1;
  end
  else begin
    if Rows = 0 then Rows := 1;
    Result := abcCeil(Items / Rows);
    CalcRows := abcCeil(Items / Result);
    if (CalcRows <> Rows) then
      if RoundUp then begin
        Dec(Result);
        Rows := Items div Result;
      end
      else
        Rows := CalcRows;
  end;
end;

function GetRowSize(Items: TabcPosInt; var Cols: TabcPosInt; RoundUp: Boolean): TabcPosInt;
var
  CalcCols: TabcPosInt;
begin
  if Items = 0 then begin
    Cols := 0;
    Result := 0;
  end
  else if Cols >= Items then begin
    Cols := Items;
    Result := 1;
  end
  else begin
    if Cols = 0 then Cols := 1;
    Result := abcCeil(Items / Cols);
    CalcCols := abcCeil(Items / Result);
    if (CalcCols <> Cols) then
      if RoundUp then begin
        Dec(Result);
        Cols := Items div Result;
      end
      else
        Cols := CalcCols;
  end;
end;

constructor TabcCustomSpeedPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FCtrlHeight := 24;
  FCtrlWidth := 24;
  FMarginH := 5;
  FMarginV := 5;
  FSorted := True;
  FCtrlRows := 1;
end;

procedure TabcCustomSpeedPanel.AlignControlRows;
var
  I: integer;
  ColCount: Integer;
  XPos, YPos: Integer;
  PanelBorder, PanelWidth, PanelHeight: Integer;
begin
  if not (csLoading in ComponentState) and AutoSize then begin
    if ControlCount = 0 then Exit;
    FAligningControls := True;
    if FSorted then SortControls;
    {check rows x cols in case of insert, delete control, visible settings}
    if FSizeVisibleOnly then
      FCtrlRows := GetRowSize(GetVisibleControlCount, FCtrlCols, FInsertCols)
    else
      FCtrlRows := GetRowSize(ControlCount, FCtrlCols, FInsertCols);
    PanelBorder := BorderWidth;
    if (BevelInner <> bvNone) then
      Inc(PanelBorder, BevelWidth);
    if (BevelOuter <> bvNone) then
      Inc(PanelBorder, BevelWidth);

    ColCount := 1;
    XPos := PanelBorder + FMarginH;
    YPos := PanelBorder + FMarginV;
    for I := 0 to ControlCount - 1 do
      with Controls[I] do begin
        SetBounds(XPos, YPos, FCtrlWidth, FCtrlHeight);
        if Visible or not FSizeVisibleOnly then
          if ColCount = FCtrlCols then begin
            XPos := PanelBorder + FMarginH;
            YPos := YPos + FCtrlHeight + FMarginRow;
            ColCount := 1;
          end
          else begin
            Inc(ColCount);
            XPos := XPos + FCtrlWidth + FMarginCol;
          end;
      end;

    PanelWidth := FCtrlCols * (FCtrlWidth + FMarginCol) - FMarginCol
                    + 2 * (FMarginH + PanelBorder);
    PanelHeight := FCtrlRows * (FCtrlHeight + FMarginRow) - FMarginRow
                    + 2 * (FMarginV + PanelBorder);

    case Align of
      alTop, alBottom:
          PanelWidth  := Width;
      alNone: {do nothing};
      alLeft, alRight:
          PanelHeight := Height;
      alClient:
        begin
          PanelHeight := Height;
          PanelWidth  := Width;
        end;
    end;
    SetBounds(Left, Top, PanelWidth, PanelHeight);
    FAligningControls := False;
  end;
end;

procedure TabcCustomSpeedPanel.AdjustSize;
begin
  AlignControlRows;
end;

procedure TabcCustomSpeedPanel.AlignControls(AControl: TControl; var Rect: TRect);
begin
  inherited AlignControls(AControl, Rect);
  if not (FAligningControls or FOrderingControls) then AlignControlRows;
end;

function TabcCustomSpeedPanel.GetVisibleControlCount: Integer;
var
  I: integer;
begin
  Result := ControlCount;
  for I := 0 to ControlCount - 1 do
    if not Controls[I].Visible then
      Dec(Result);
end;

procedure TabcCustomSpeedPanel.OrderControls(NewOrder: TList);
var
  I: Integer;
begin
  if NewOrder.Count <> ControlCount then
    raise Exception.Create('List Invalid')
  else begin
    FOrderingControls := True;
    for I := 0 to NewOrder.Count - 1 do
      TControl(NewOrder.Items[I]).Parent := Self;
    FOrderingControls := False;
  end;
end;

procedure TabcCustomSpeedPanel.SetCtrlCols(Value: TabcPosInt);
begin
  if Value <> FCtrlCols then
    if (csLoading in ComponentState) then
      FCtrlCols := Value
    else begin
      FCtrlRows := GetRowSize(ControlCount, Value, (Value > FCtrlCols));
      FCtrlCols := Value;
      AlignControlRows;
    end;
end;

procedure TabcCustomSpeedPanel.SetCtrlHeight(Value: TabcPosInt);
begin
  if Value <> FCtrlHeight then begin
    FCtrlHeight := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SetCtrlRows(Value: TabcPosInt);
begin
  if Value <> FCtrlRows then
    if (csLoading in ComponentState) then
      FCtrlRows := Value
    else begin
      FCtrlCols := GetColSize(ControlCount, Value, (Value > FCtrlRows));
      FCtrlRows := Value;
      AlignControlRows;
    end;
end;

procedure TabcCustomSpeedPanel.SetCtrlWidth(Value: TabcPosInt);
begin
  if Value <> FCtrlWidth then begin
    FCtrlWidth := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SetMarginCol(Value: Integer);
begin
  if Value <> FMarginCol then begin
    FMarginCol := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SetMarginH(Value: Integer);
begin
  if Value <> FMarginH then begin
    FMarginH := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SetMarginRow(Value: Integer);
begin
  if Value <> FMarginRow then begin
    FMarginRow := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SetMarginV(Value: Integer);
begin
  if Value <> FMarginV then begin
    FMarginV := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SetSizeVisibleOnly(Value: Boolean);
begin
  if Value <> FSizeVisibleOnly then begin
    FSizeVisibleOnly := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SetSorted(Value: Boolean);
begin
  if Value <> FSorted then begin
    FSorted := Value;
    AlignControlRows;
  end;
end;

procedure TabcCustomSpeedPanel.SortControls;
var
  NewOrder: TList;
  I, J: Integer;
begin
  if ControlCount > 0 then begin
    NewOrder := TList.Create;
    try
      NewOrder.Add(Controls[0]);
      for I := 1 to ControlCount - 1 do
        for J := NewOrder.Count - 1 downto 0 do
          if (Controls[I].Top > TControl(NewOrder.Items[J]).Top) or
             ((Controls[I].Top = TControl(NewOrder.Items[J]).Top) and
             (Controls[I].Left > TControl(NewOrder.Items[J]).Left)) then begin
            if J = NewOrder.Count - 1 then
              NewOrder.Add(Controls[I])
            else
              NewOrder.Insert(J + 1, Controls[I]);
            Break;
          end
          else
            if J = 0 then NewOrder.Insert(0, Controls[I]);
      OrderControls(NewOrder);
    finally
      NewOrder.Free;
    end;
  end;
end;

{ TabcPictureGroup }

constructor TabcPictureGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Alignment := taLeftJustify;
  BevelOuter := bvNone;
  ControlStyle := ControlStyle + [csSetCaption];
  FOffset := 10;
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChange;
  FMarginL := 60;
  FMarginT := 35;
  FMarginR := 20;
  FMarginB := 10;
  SetBounds(Left, Top, 300, 150);
end;

function TabcPictureGroup.GetInnerRect: TRect;
begin
  Result := ClientRect;
  with Result do
  begin
    Left := Left + FMarginL;
    Top := Top + FMarginT;
    Right := Right - FMarginR;
    Bottom := Bottom - FMarginB;
  end;
end;

procedure TabcPictureGroup.AdjustClientRect(var Rect: TRect);
begin
  inherited AdjustClientRect(Rect);
  with Rect do
  begin
    Left := Left + FMarginL;
    Top := Top + FMarginT;
    Right := Right - FMarginR;
    Bottom := Bottom - FMarginB;
  end;
end;

procedure TabcPictureGroup.SetMarginL(Value: Integer);
begin
  if Value <> FMarginL then
  begin
    FMarginL := Value;
    Realign;
  end;
end;

procedure TabcPictureGroup.SetMarginT(Value: Integer);
begin
  if Value <> FMarginT then
  begin
    FMarginT := Value;
    Realign;
  end;
end;

procedure TabcPictureGroup.SetMarginR(Value: Integer);
begin
  if Value <> FMarginR then
  begin
    FMarginR := Value;
    Realign;
  end;
end;

procedure TabcPictureGroup.SetMarginB(Value: Integer);
begin
  if Value <> FMarginB then
  begin
    FMarginB := Value;
    Realign;
  end;
end;

destructor TabcPictureGroup.Destroy;
begin
  FPicture.Free;
  inherited Destroy;
end;

procedure TabcPictureGroup.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus and
      (ssAlt in KeyDataToShiftState(KeyData)) then
    begin
      SelectFirst;
      Result := 1;
    end
    else
      inherited;
end;

procedure TabcPictureGroup.SetAlignPicture(const Value: TAlignment);
begin
  if Value <> FAlignPicture then
  begin
    FAlignPicture := Value;

    {adjust margins}
    case Value of
      taLeftJustify:
        begin
          FMarginL := 60;
          FMarginT := 35;
          FMarginR := 20;
          FMarginB := 10;
        end;
      taRightJustify:
        begin
          FMarginL := 20;
          FMarginT := 35;
          FMarginR := 60;
          FMarginB := 10;
        end;
      taCenter:
        begin
          FMarginL := 20;
          FMarginT := 80;
          FMarginR := 20;
          FMarginB := 10;
        end;
    end;
    ReAlign;
    if FPicture.Graphic <> nil then Invalidate;
  end;
end;

procedure TabcPictureGroup.SetOffset(Value: Integer);
begin
  if Value <> FOffset then
  begin
    FOffset := Value;
    Invalidate;
  end;
end;

procedure TabcPictureGroup.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TabcPictureGroup.PictureChange(Sender: TObject);
begin
  if FPicture.Graphic <> nil then
    FPicture.Graphic.Transparent := True;
  Invalidate;
end;

procedure TabcPictureGroup.SetLineRaised(Value: Boolean);
begin
  if Value <> FLineRaised then
  begin
    FLineRaised := Value;
    Invalidate;
  end;
end;

procedure TabcPictureGroup.Paint;
var
  Rect, LineRect: TRect;
  TopColor, BottomColor: TColor;
  Text: array[0..255] of Char;
  FontHeight: Integer;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;

begin
  with Canvas do begin
    Rect := GetClientRect;
    Brush.Color := Color;
    if not Transparent then
      FillRect(Rect);
    if BevelOuter <> bvNone then begin
      AdjustColors(BevelOuter);
      abcEdgeFrame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth, BevelEdges);
    end;
    Frame3D(Canvas, Rect, Color, Color, BorderWidth);
    if BevelInner <> bvNone then begin
      AdjustColors(BevelInner);
      abcEdgeFrame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth, BevelEdges);
    end;

    InflateRect(Rect, -FOffset, -FOffset);
    LineRect := Rect;

    Brush.Style := bsSolid;
    Brush.Color := Color;
    Font := Self.Font;
    FontHeight := TextHeight('W');
    with Rect do
    begin
      Bottom := Top + FontHeight;
      LineRect.Top := Top + FontHeight div 2;
    end;

    with Pen do
    begin
      Style := psSolid;
      Mode := pmCopy;
      Width := 1;
      if FLineRaised then
        Color := clBtnHighlight
      else
        Color := clBtnShadow;
    end;

    {draw full width line}

    with LineRect do
    begin
      MoveTo(Left, Top);
      LineTo(Right, Top);
      Top := Top + 1;
      MoveTo(Left, Top);
      if FLineRaised then
        Pen.Color := clBtnShadow
      else
        Pen.Color := clBtnHighlight;
      LineTo(Right, Top);
    end;

    if Caption <> '' then begin
      case Alignment of
        taLeftJustify:
          StrPCopy(Text, Caption + '  ');
        taRightJustify:
          StrPCopy(Text, '  ' + Caption);
        taCenter:
          StrPCopy(Text, '  ' + Caption + '  ');
      end;
      DrawText(Handle, Text, StrLen(Text), Rect, (DT_EXPANDTABS or
        DT_VCENTER) or Alignments[Alignment]);
    end;

    if Picture.Graphic <> nil then
      with LineRect do
      begin
        case FAlignPicture of
          taRightJustify:
            Left := Right - Picture.Graphic.Width;
          taCenter:
            Left := (Left + Right - Picture.Graphic.Width) div 2;
        end;
        Draw(Left, Rect.Bottom + FOffset, Picture.Graphic);
      end;
  end;
end;

end.
