unit abcpanfr;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{
  Property Editors for TabcPageFrame
}

interface

uses Classes, SysUtils, Controls, Forms,
     {$IFDEF DELPHI6}
     DesignIntf, DesignEditors
     {$ELSE}
     DsgnIntf
     {$ENDIF}
     ;

type

{ TabcPageProperty }

  TabcPageProperty = class(TComponentProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TabcPageFrameEditor }

  TabcPageFrameEditor = class(TDefaultEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

implementation

uses consts, comstrs, abcdsncn, abcanfr, abcutilf, abcprop;

{ TabcPageProperty }

function TabcPageProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TabcPageProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  ThisComp, AComp: TComponent;
  Par: TWinControl;
begin
  ThisComp := GetComponent(0) as TComponent;
  Par := abcParentFormorFrameOf(ThisComp);
  if Par = nil then Exit;
  for I := 0 to Par.ComponentCount - 1 do
  begin
    AComp := Par.Components[I];
    if (AComp.Name <> '') and (AComp is TabcPage) and
      (TabcPage(AComp).Frame = ThisComp) then
      if Par is TForm then
        Proc(AComp.Name)
      else
        Proc(Par.Name + '.' + AComp.Name);
  end;
end;

{ TabcPageFrameEditor }

procedure TabcPageFrameEditor.ExecuteVerb(Index: Integer);
var
  tmpIndex: Integer;
  Frame: TabcCustomPageFrame;
  Page: TabcPage;
{$IFDEF DELPHI6}
  Designer: IDesigner;
{$ELSE}
  Designer: IFormDesigner;
{$ENDIF}
begin
{$IFDEF DELPHI5}
  if not IsInInlined then
    tmpIndex := Index
  else
    tmpIndex := Index + 1;
{$ELSE}
    tmpIndex := Index;
{$ENDIF}
  if Component is TabcPage then
    Frame := TabcCustomPageFrame(TabcPage(Component).Frame)
  else
    Frame := TabcCustomPageFrame(Component);
  if Frame <> nil then
  begin
    Designer := Self.Designer;
    case tmpIndex of
      0:
      begin
        Page := TabcPage.Create(Designer.GetRoot);
        try
          Page.Name := abcDesignerUniqueName(Designer, TabcPage.ClassName);
          Page.Parent := Frame;
          Page.Frame := Frame;
        except
          Page.Free;
          raise;
        end;
        Frame.ActivePage := Page;
        Designer.SelectComponent(Page);
        Designer.Modified;
      end;
      1..2:
      begin
        Page := Frame.FindNextPage(Frame.ActivePage,
          tmpIndex = 1);
        if (Page <> nil) and (Page <> Frame.ActivePage) then
        begin
          Frame.ActivePage := Page;
          if Component is TabcPage then Designer.SelectComponent(Page);
          Designer.Modified;
        end;
      end;
      3: Designer.SelectComponent(Frame);
    end;
  end;
end;

function TabcPageFrameEditor.GetVerb(Index: Integer): string;
var
  tmpIndex: Integer;
begin
{$IFDEF DELPHI5}
  if not IsInInlined then
    tmpIndex := Index
  else
    tmpIndex := Index + 1;
{$ELSE}
    tmpIndex := Index;
{$ENDIF}
  case tmpIndex of
    0: Result := sabcpropdsgnNewPage;
    1: Result := sabcpropdsgnNextPage;
    2: Result := sabcpropdsgnPrevPage;
    3: Result := sabcpropdsgnSelFrame;
  else
    Result := '';
  end;
end;

function TabcPageFrameEditor.GetVerbCount: Integer;
begin
{$IFDEF DELPHI5}
  if not IsInInlined then
    Result := 4
  else
    Result := 3;
{$ELSE}
    Result := 4;
{$ENDIF}
end;

end.
