unit abcpbar;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{
  Property Editors for TabcButtonBar
}

interface

uses Classes, SysUtils, Controls, Forms, Dialogs,
     {$IFDEF DELPHI6}
     DesignIntf, DesignEditors
     {$ELSE}
     DsgnIntf
     {$ENDIF}
     ;

type

{ TabcButtonPageProperty }

  TabcButtonPageProperty = class(TComponentProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TabcButtonBarEditor }

  TabcButtonBarEditor = class(TDefaultEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

implementation

uses consts, comstrs, abcdsncn, abcbar, abcbarlv, abcutilf, abcprop;

{ TabcButtonPageProperty }

function TabcButtonPageProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TabcButtonPageProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  ThisComp, AComp: TComponent;
  Par: TWinControl;
begin
  ThisComp := GetComponent(0) as TComponent;
  Par := abcParentFormorFrameOf(ThisComp);
  if Par = nil then Exit;
  for I := 0 to Par.ComponentCount - 1 do
  begin
    AComp := Par.Components[I];
    if (AComp.Name <> '') and (AComp is TabcButtonPage) and
      (TabcButtonPage(AComp).ButtonBar = ThisComp) then
      if Par is TForm then
        Proc(AComp.Name)
      else
        Proc(Par.Name + '.' + AComp.Name);
  end;
end;

{ TabcButtonBarEditor }

procedure TabcButtonBarEditor.ExecuteVerb(Index: Integer);
var
  tmpIndex: Integer;
  ButtonBar: TabcButtonBar;
  Page: TabcButtonPage;
  List: TabcButtonList;
{$IFDEF DELPHI6}
  Designer: IDesigner;
{$ELSE}
  Designer: IFormDesigner;
{$ENDIF}
begin
{$IFDEF DELPHI5}
  if not IsInInlined then
    tmpIndex := Index
  else
    tmpIndex := Index + 2;
{$ELSE}
    tmpIndex := Index;
{$ENDIF}
  if Component is TabcButtonPage then
    ButtonBar := TabcButtonBar(TabcButtonPage(Component).ButtonBar)
  else
    ButtonBar := TabcButtonBar(Component);
  if ButtonBar <> nil then
  begin
    Designer := Self.Designer;
    case tmpIndex of
      0..1:
        begin
          Page := TabcButtonPage.Create(Designer.GetRoot);
          try
            Page.Name := abcDesignerUniqueName(Designer, TabcButtonPage.ClassName);
            Page.Parent := ButtonBar;
            Page.ButtonBar := ButtonBar;
          except
            Page.Free;
            raise;
          end;
          ButtonBar.ActivePage := Page;
          Designer.SelectComponent(Page);
          if tmpIndex = 1 then
          //Add a ButtonList also
          begin
            List := TabcButtonList.Create(Designer.GetRoot);
            try
              List.Name := abcDesignerUniqueName(Designer, TabcButtonList.ClassName);
              List.Parent := Page;
            except
              List.Free;
              raise;
            end;
            Designer.SelectComponent(List);
          end;
          Designer.Modified;
        end;
      2:
          Designer.SelectComponent(ButtonBar.ActivePage);
      3..4:
        begin
          Page := ButtonBar.FindNextPage(ButtonBar.ActivePage,
            tmpIndex = 3, False);
          if (Page <> nil) and (Page <> ButtonBar.ActivePage) then
          begin
            ButtonBar.ActivePage := Page;
            if Component is TabcButtonPage then Designer.SelectComponent(Page);
            Designer.Modified;
          end;
        end;
    end;
  end;
end;

function TabcButtonBarEditor.GetVerb(Index: Integer): string;
var
  tmpIndex: Integer;
begin
{$IFDEF DELPHI5}
  if not IsInInlined then
    tmpIndex := Index
  else
    tmpIndex := Index + 2;
{$ELSE}
    tmpIndex := Index;
{$ENDIF}
  case tmpIndex of
    0: Result := sabcpropdsgnNewPage;
    1: Result := sabcpropdsgnNewPageWithList;
    2: Result := sabcpropdsgnSelectActivePage;
    3: Result := sabcpropdsgnNextPage;
    4: Result := sabcpropdsgnPrevPage;
  else
    Result := '';
  end;
end;

function TabcButtonBarEditor.GetVerbCount: Integer;
begin
{$IFDEF DELPHI5}
  if not IsInInlined then
    Result := 5
  else
    Result := 3;
{$ELSE}    
    Result := 5;
{$ENDIF}
end;

end.
