unit abcpdbt;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses SysUtils, TypInfo,
{$IFDEF DELPHI6}
     DesignIntf, DesignEditors
{$ELSE}
     DsgnIntf
{$ENDIF}
     ;

type

  TabcDBTreeSourcesEditor = class(TComponentEditor)
  private
{$IFDEF DELPHI6}
    FDataSourcesEditor: IProperty;
    procedure GetDataSourcesEditor(const Prop: IProperty);
{$ELSE}
    FDataSourcesEditor: TPropertyEditor;
    procedure GetDataSourcesEditor(PropertyEditor: TPropertyEditor);
{$ENDIF}
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

implementation

{ TabcDBTreeSourcesEditor }

function TabcDBTreeSourcesEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TabcDBTreeSourcesEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := '&Edit DataSources';
  else
    Result := '';
  end;
end;

{$IFDEF DELPHI6}
procedure TabcDBTreeSourcesEditor.GetDataSourcesEditor(const Prop: IProperty);
var
  PropName: string;
begin
  if FDataSourcesEditor = nil then
  begin
    PropName := Prop.GetName;
    if CompareText(PropName, 'DataSources') = 0 then
      FDataSourcesEditor := Prop;
  end;
end;
{$ELSE}
procedure TabcDBTreeSourcesEditor.GetDataSourcesEditor(PropertyEditor: TPropertyEditor);
var
  PropName: string;
begin
  if FDataSourcesEditor = nil then
  begin
    PropName := PropertyEditor.GetName;
    if CompareText(PropName, 'DataSources') = 0 then
      FDataSourcesEditor := PropertyEditor;
  end;
end;
{$ENDIF}

procedure TabcDBTreeSourcesEditor.ExecuteVerb(Index: Integer);
{returns the TCollection property editor on the DataSources property
 and executes its Edit method}
var
{$IFDEF DELPHI6}
  Components: IDesignerSelections;
{$ELSE}
{$IFDEF DELPHI5}
  Components: TDesignerSelectionList;
{$ELSE}
  Components: TComponentList;
{$ENDIF}
{$ENDIF}
begin
{$IFDEF DELPHI6}
  Components := CreateSelectionList;
{$ELSE}
{$IFDEF DELPHI5}
  Components := TDesignerSelectionList.Create;
{$ELSE}
  Components := TComponentList.Create;
{$ENDIF}
{$ENDIF}
  try
    Components.Add(Component);
    FDataSourcesEditor := nil;
    GetComponentProperties(Components, [tkClass], Designer, GetDataSourcesEditor);
    if Assigned(FDataSourcesEditor) then FDataSourcesEditor.Edit;
  finally
{$IFDEF DELPHI6}
    Components := nil;
{$ELSE}
    Components.Free;
{$ENDIF}
  end;
end;

end.
