unit abcpedto;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{property & component editors for File Open Edit components}

interface

uses Classes, Forms, Dialogs, abcedtop,
{$IFDEF DELPHI6}
     DesignIntf, DesignEditors
{$ELSE}
     DsgnIntf
{$ENDIF}
     ;

type

  { TabcFileOpenEditEditor }

  TabcFileOpenEditEditor = class(TComponentEditor)
  private
    procedure Preview;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TabcOpenFilenameProperty
  Property editor for filenames.  Displays an File Open Dialog
  for the name of the file.
}

  TabcOpenFilenameProperty = class(TStringProperty)
  protected
    function GetHelpContext: THelpContext; virtual;
    function GetFilter: string; virtual;
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

implementation

{ TabcFileOpenEditEditor }

function TabcFileOpenEditEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TabcFileOpenEditEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := '&Preview Dialog';
  else
    Result := '';
  end;
end;

procedure TabcFileOpenEditEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: Preview;
  else
  end;
end;

procedure TabcFileOpenEditEditor.Preview;
begin
  TabcFileOpenEdit(Component).OpenDialog.Dialog.Execute;
end;

{ TabcOpenFilenameProperty }

procedure TabcOpenFilenameProperty.Edit;
begin
  with TOpenDialog.Create(nil) do
    try
      Filename := GetValue;
      Filter := GetFilter;
      HelpContext := GetHelpContext;
      if HelpContext = 0 then
        Options := Options + [ofPathMustExist, ofFileMustExist]
      else
        Options := Options + [ofShowHelp, ofPathMustExist, ofFileMustExist];
      if Execute then SetValue(Filename);
    finally
      Free;
    end;
end;

function TabcOpenFilenameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paRevertable];
end;

function TabcOpenFilenameProperty.GetFilter: string;
begin
  Result := 'All Files (*.*)|*.*';
end;

function TabcOpenFilenameProperty.GetHelpContext: THelpContext;
begin
  Result := 0;
end;

end.
