unit abcpedts;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{property & component editors for File Save Edit component}

interface

uses Classes, Forms, Dialogs, abcedtsv,
{$IFDEF DELPHI6}
     DesignIntf, DesignEditors
{$ELSE}
     DsgnIntf
{$ENDIF}
     ;

type

  { TabcFileSaveEditEditor }

  TabcFileSaveEditEditor = class(TComponentEditor)
  private
    procedure Preview;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TabcSaveFilenameProperty
  Property editor for filenames.  Displays an File Save Dialog
  for the name of the file.
}

  TabcSaveFilenameProperty = class(TStringProperty)
  protected
    function GetHelpContext: THelpContext; virtual;
    function GetFilter: string; virtual;
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

implementation

{ TabcFileSaveEditEditor }

function TabcFileSaveEditEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TabcFileSaveEditEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := '&Preview Dialog';
  else
    Result := '';
  end;
end;

procedure TabcFileSaveEditEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: Preview;
  else
  end;
end;

procedure TabcFileSaveEditEditor.Preview;
begin
  TabcFileSaveEdit(Component).SaveDialog.Dialog.Execute;
end;

{ TabcSaveFilenameProperty }

procedure TabcSaveFilenameProperty.Edit;
begin
  with TSaveDialog.Create(nil) do
    try
      Filename := GetValue;
      Filter := GetFilter;
      HelpContext := GetHelpContext;
      if HelpContext = 0 then
        Options := Options + [ofPathMustExist, ofFileMustExist]
      else
        Options := Options + [ofShowHelp, ofPathMustExist, ofFileMustExist];
      if Execute then SetValue(Filename);
    finally
      Free;
    end;
end;

function TabcSaveFilenameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paRevertable];
end;

function TabcSaveFilenameProperty.GetFilter: string;
begin
  Result := 'All Files (*.*)|*.*';
end;

function TabcSaveFilenameProperty.GetHelpContext: THelpContext;
begin
  Result := 0;
end;

end.
