unit abcphint;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  ExptIntf,
  VirtIntf,
  ToolIntf,
  Windows,
  abcdfabh;

(* NOTE This breaks the Delphi 3 and C++ Builder 3 Menus 
   with a AV when the help menu is clicked.
*)

type

  TabcBrowseCompsExpert = class(TIExpert)
  private
    FabcBrowseCompsMenuItem: TIMenuItemIntf;
    procedure abcBrowseCompsMenuClick(Sender: TIMenuItemIntf);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute; override;

    function GetAuthor: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;

    function GetIDString: string; override;
    function GetMenuText: string; override;
    function GetName: string; override;

    function GetPage: string; override;
    function GetState: TExpertState; override;

    function GetStyle: TExpertStyle; override;
    property abcBrowseCompsMenuItem: TIMenuItemIntf read FabcBrowseCompsMenuItem;
  end;

procedure HandleException;

implementation

uses SysUtils, Classes, Dialogs;

procedure HandleException;
begin
  ToolServices.RaiseException(ReleaseException);
end;

{ TabcBrowseCompsExpert }

constructor TabcBrowseCompsExpert.Create;
var
  MainMenu: TIMainMenuIntf;
  CompInstallAfterMenu: TIMenuItemIntf;
  CompParentMenu: TIMenuItemIntf;
  MenuItems: TIMenuItemIntf;

(* Inspect Delphi Menus when name changes in new version
  imenuitem,jmenuitem: TIMenuItemIntf;
  i,im,j,jm: integer;
*)

begin
  inherited Create;

  MainMenu := ToolServices.GetMainMenu;
  if MainMenu <> nil then
  try
    MenuItems := MainMenu.GetMenuItems;
    if MenuItems <> nil then
    try

(*  Inspect Delphi Menus
      im := MenuItems.GetItemCount;
      for i := 0 to im-1 do
      begin
        imenuitem := MenuItems.GetItem(i);
        ShowMessage(Inttostr(i) + ': ' + imenuitem.GetName);
        jm := imenuitem.GetItemCount;
        for j := 0 to jm-1 do
        begin
          jmenuitem := imenuitem.GetItem(j);
          ShowMessage(Inttostr(i) + '.' + Inttostr(j) + ': ' + jmenuitem.GetName);
        end;
      end;
*)


      CompParentMenu := MainMenu.FindMenuItem('HelpMenu');
      CompInstallAfterMenu := MainMenu.FindMenuItem('HelpAboutItem');

      if CompInstallAfterMenu <> nil then
      try
        if CompParentMenu <> nil then
        try
          CompParentMenu.AddRef;
          FabcBrowseCompsMenuItem := CompParentMenu.InsertItem(CompInstallAfterMenu.GetIndex + 1,
            GetMenuText, 'abcBrowseCompsMenuItem', '', 0, 0, 0,
            [mfVisible, mfEnabled], abcBrowseCompsMenuClick);
        finally
          CompParentMenu.Free;
        end;
      finally
        CompInstallAfterMenu.Free;
      end;
    finally
      MenuItems.Free;
    end;
  finally
    MainMenu.Free;
  end;
end;

destructor TabcBrowseCompsExpert.Destroy;
begin
  abcBrowseCompsMenuItem.Free;
  inherited Destroy;
end;

procedure TabcBrowseCompsExpert.abcBrowseCompsMenuClick(Sender: TIMenuItemIntf);
begin
  Execute;
end;

function TabcBrowseCompsExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TabcBrowseCompsExpert.GetName: string;
begin
  Result := 'ABC Browse Components Expert';
end;

function TabcBrowseCompsExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

function TabcBrowseCompsExpert.GetIDString: string;
begin
  Result := 'ObjectiveSWTechnology.ABCBrowseComponentExpert';
end;

function TabcBrowseCompsExpert.GetMenuText: string;
begin
  Result := 'Browse ABC Components...';
end;

procedure TabcBrowseCompsExpert.Execute;
begin
  try
    abcShowIDEHintAbout;
  except
    HandleException;
  end;
end;

function TabcBrowseCompsExpert.GetAuthor: string;
begin
  Result := '';
end;

function TabcBrowseCompsExpert.GetComment: string;
begin
  Result := '';
end;

function TabcBrowseCompsExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

function TabcBrowseCompsExpert.GetPage: string;
begin
  Result := '';
end;

end.


