unit abcpilst;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses SysUtils, Windows, Classes, Graphics, Controls, TypInfo, ImgList,
{$IFDEF DELPHI6}
     DesignIntf, DesignEditors, VCLeditors
{$ELSE}
     DsgnIntf
{$ENDIF}
     ;

type

{$IFDEF DELPHI6}
  TabcImageIndexProperty = class(TIntegerProperty, ICustomPropertyListDrawing)
  private
    function GetImages: TImageList;
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    // ICustomPropertyListDrawing
    procedure ListMeasureHeight(const Value: string; ACanvas: TCanvas;
      var AHeight: Integer);
    procedure ListMeasureWidth(const Value: string; ACanvas: TCanvas;
      var AWidth: Integer);
    procedure ListDrawValue(const Value: string; ACanvas: TCanvas;
      const ARect: TRect; ASelected: Boolean);
  end;
{$ELSE}
  TabcImageIndexProperty = class(TIntegerProperty)
  private
    function GetImages: TImageList;
  public
    procedure ListMeasureWidth(const Value: string; ACanvas: TCanvas;
      var AWidth: Integer); override;
    procedure ListMeasureHeight(const Value: string; ACanvas: TCanvas;
      var AHeight: Integer); override;
    procedure ListDrawValue(const Value: string; ACanvas: TCanvas;
      const ARect: TRect; ASelected: Boolean); override;
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;
{$ENDIF}

implementation

{ TabcImageIndexProperty }

function TabcImageIndexProperty.GetImages: TImageList;
var
  Comp: TPersistent;
  Par: TWinControl;
begin
  Result := nil;
  Comp := GetComponent(0);
  if IsPublishedProp(Comp, 'Images') then
    Result := TImageList(GetOrdProp(Comp, 'Images'))
  else if IsPublishedProp(Comp, 'ImageList') then
    Result := TImageList(GetOrdProp(Comp, 'ImageList'))
  else if Comp is TControl then
  begin
    Par := TControl(Comp).Parent;
    if (Par <> nil) then
      if IsPublishedProp(Par, 'Images') then
        Result := TImageList(GetOrdProp(Par, 'Images'))
      else if IsPublishedProp(Par, 'ImageList') then
        Result := TImageList(GetOrdProp(Par, 'ImageList'));
  end;
end;

function TabcImageIndexProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

procedure TabcImageIndexProperty.GetValues(Proc: TGetStrProc);
var
  iList: TImageList;
  i: integer;
begin
  iList := GetImages;
  if iList <> nil then
    for i := 0 to iList.Count - 1 do
    begin
      Proc(InttoStr(i));
    end;
end;

procedure TabcImageIndexProperty.ListDrawValue(const Value: string;
  ACanvas: TCanvas; const ARect: TRect; ASelected: Boolean);
var
  iList: TImageList;
  idx, vRight: Integer;
  brushColor: TColor;
begin
  vRight := ARect.Left;
  with ACanvas do
  try
    iList := GetImages;
    if iList <> nil then
    begin
      idx := StrtoInt(Value);
      if (idx >= 0) and (idx < iList.Count) then
      begin
        brushColor := ACanvas.Brush.Color;
        if ASelected then
          ACanvas.Brush.Color := clHighlight
        else
          ACanvas.Brush.Color := clWindow;
        ACanvas.FillRect(ARect);
        iList.Draw(ACanvas, ARect.Left + 2, ARect.Top + 2, idx, True);
        ACanvas.Brush.Color := brushColor;
        vRight := vRight + iList.Width + 4;
      end;
    end;
  finally
{$IFDEF DELPHI6}
    DefaultPropertyListDrawValue(Value, ACanvas, Rect(vRight, ARect.Top, ARect.Right, ARect.Bottom), ASelected);
{$ELSE}
    inherited ListDrawValue(Value, ACanvas,
                            Rect(vRight, ARect.Top, ARect.Right, ARect.Bottom),
                            ASelected);
{$ENDIF}
  end;
end;

procedure TabcImageIndexProperty.ListMeasureWidth(const Value: string;
  ACanvas: TCanvas; var AWidth: Integer);
var
  iList: TImageList;
begin
  iList := GetImages;
  if iList <> nil then
    AWidth := iList.Width + 2;
end;

procedure TabcImageIndexProperty.ListMeasureHeight(const Value: string;
  ACanvas: TCanvas; var AHeight: Integer);
var
  iList: TImageList;
begin
  iList := GetImages;
  if iList <> nil then
    AHeight := iList.Height + 4;
end;

end.
