unit abcppts;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,
  abcspr,
{$IFDEF DELPHI6}
  DesignIntf, DesignEditors
{$ELSE}
  DsgnIntf
{$ENDIF}
     ;

type

  TabcPointListProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  TabcEditPointsDialog = class(TForm)
    lstPoints: TListBox;
    Bevel1: TBevel;
    btnOk: TButton;
    btnCancel: TButton;
    btnReplace: TButton;
    btnAdd: TButton;
    btnDelete: TButton;
    edtX: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    edtY: TEdit;
    btnUp: TButton;
    btnDown: TButton;
    btnLoad: TButton;
    btnSave: TButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lstPointsClick(Sender: TObject);
    procedure edtXKeyPress(Sender: TObject; var Key: Char);
    procedure edtXChange(Sender: TObject);
    procedure btnReplaceClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure edtXEnter(Sender: TObject);
  private
    { Private declarations }
    FPoints: TabcPointList;
    FEditChanged: Boolean;
    FShowingSelection: Boolean;
    procedure SetPoints(const Points: TabcPointList);
    procedure ShowSelection;
    procedure UpdateButtons;
    procedure LoadFromFile(FName: string);
    procedure SaveToFile(FName: string);
    function IsValidInteger(s: string): Boolean;
  public
    { Public declarations }
  end;

function abcEditPoints(const Points: TabcPointList): Boolean;

implementation

{$R *.DFM}

procedure TabcPointListProperty.Edit;
begin
  if abcEditPoints(TabcPointList(GetOrdValue)) then
    Modified;
end;

function TabcPointListProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TabcPointListProperty.GetValue: string;
begin
  FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;

const
  DEF_INT = -MAXINT;

function abcEditPoints(const Points: TabcPointList): Boolean;
var
  i: integer;
begin
  with TabcEditPointsDialog.Create(nil) do
  try
    SetPoints(Points);
    Result := (ShowModal = mrOk);
    if Result then
    begin
      Points.Clear;
      for i := 0 to FPoints.Count - 1 do
        Points.AddPoint(PPoint(FPoints[i])^);
    end;
  finally
    Free;
  end;
end;

procedure TabcEditPointsDialog.FormCreate(Sender: TObject);
begin
  FPoints := TabcPointList.Create;
end;

procedure TabcEditPointsDialog.FormDestroy(Sender: TObject);
begin
  FPoints.Free;
end;

procedure TabcEditPointsDialog.SetPoints(const Points: TabcPointList);
var
  i: integer;
begin
  FPoints.Clear;
  lstPoints.Clear;
  for i := 0 to Points.Count - 1 do
  begin
    FPoints.AddPoint(PPoint(Points[i])^);
    with PPoint(Points[i])^ do
      lstPoints.Items.Add(Format('%d , %d', [X, Y]));
  end;
  if lstPoints.Items.Count > 0 then
    lstPoints.ItemIndex := 0;
  UpdateButtons;
  ShowSelection;
end;

procedure TabcEditPointsDialog.UpdateButtons;
var
  ix: integer;
  xyAddChange: Boolean;
begin
  ix := lstPoints.ItemIndex;
  xyAddChange := FEditChanged and IsValidInteger(edtX.Text) and IsValidInteger(edtY.Text);
  btnReplace.Enabled :=  xyAddChange and (ix > -1);
  btnAdd.Enabled := xyAddChange;
  btnDelete.Enabled := (ix > -1);
  btnUp.Enabled := (ix > 0);
  btnDown.Enabled := (ix > -1) and (ix < lstPoints.Items.Count - 1);
  btnSave.Enabled := (lstPoints.Items.Count > 0);
end;

procedure TabcEditPointsDialog.ShowSelection;
var
  ix: integer;
begin
  FShowingSelection := True;
  ix := lstPoints.ItemIndex;
  if ix > -1 then
    with PPoint(FPoints[ix])^ do
    begin
      edtX.Text := InttoStr(X);
      edtY.Text := InttoStr(Y);
    end
  else
  begin
    edtX.Text := '';
    edtY.Text := '';
  end;
  FShowingSelection := False;
end;

procedure TabcEditPointsDialog.lstPointsClick(Sender: TObject);
begin
  UpdateButtons;
  ShowSelection;
end;

procedure TabcEditPointsDialog.edtXKeyPress(Sender: TObject;
  var Key: Char);
begin
  if not ((Key in ['-', '0'..'9', ^V, ^C, ^X]) or (Word(Key) in [VK_BACK, VK_DELETE])) then
    Key := #0;
end;

function TabcEditPointsDialog.IsValidInteger(s: string): Boolean;
begin
  Result := StrToIntDef(s, DEF_INT) <> DEF_INT;
end;

procedure TabcEditPointsDialog.edtXChange(Sender: TObject);
begin
  if not FShowingSelection then
  begin
    with TEdit(Sender) do
      if IsValidInteger(Text) then
        FEditChanged := True
      else
        Text := '';
      UpdateButtons;
  end;
end;

procedure TabcEditPointsDialog.btnReplaceClick(Sender: TObject);
var
  ix: integer;
begin
  ix := lstPoints.ItemIndex;
  if ix > -1 then
  begin
    with PPoint(FPoints[ix])^ do
    begin
      X := StrToInt(edtX.Text);
      Y := StrToInt(edtY.Text);
      lstPoints.Items[ix] := Format('%d , %d', [X, Y]);
    end;
    FEditChanged := False;
  end;
end;

procedure TabcEditPointsDialog.btnAddClick(Sender: TObject);
var
  newPt: TPoint;
begin
  newPt.X := StrToInt(edtX.Text);
  newPt.Y := StrToInt(edtY.Text);
  lstPoints.Items.Add(Format('%d , %d', [newPt.X, newPt.Y]));
  lstPoints.ItemIndex := FPoints.AddPoint(newPt);
  FEditChanged := False;
  UpdateButtons;
end;

procedure TabcEditPointsDialog.btnDeleteClick(Sender: TObject);
var
  ix: integer;
begin
  ix := lstPoints.ItemIndex;
  if ix > -1 then
  begin
    FPoints.DeletePoint(ix);
    lstPoints.Items.Delete(ix);
    if ix = FPoints.Count then
      lstPoints.ItemIndex := ix - 1
    else
      lstPoints.ItemIndex := ix;
    UpdateButtons;
    ShowSelection;
  end;
end;

procedure TabcEditPointsDialog.btnUpClick(Sender: TObject);
var
  ix: integer;
begin
  ix := lstPoints.ItemIndex;
  if ix > 0 then
  begin
    FPoints.Exchange(ix, ix-1);
    lstPoints.Items.Exchange(ix, ix-1);
    lstPoints.ItemIndex := ix - 1;
    UpdateButtons;
  end;
end;

procedure TabcEditPointsDialog.btnDownClick(Sender: TObject);
var
  ix: integer;
begin
  ix := lstPoints.ItemIndex;
  if (ix > -1) and (ix < lstPoints.Items.Count - 1) then
  begin
    FPoints.Exchange(ix, ix+1);
    lstPoints.Items.Exchange(ix, ix+1);
    lstPoints.ItemIndex := ix + 1;
    UpdateButtons;
  end;
end;

procedure TabcEditPointsDialog.btnLoadClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
    LoadFromFile(OpenDialog1.FileName);
end;

procedure TabcEditPointsDialog.btnSaveClick(Sender: TObject);
begin
  if SaveDialog1.Execute then
    SaveToFile(SaveDialog1.FileName);
end;

procedure TabcEditPointsDialog.LoadFromFile(FName: string);
var
  tmpStrings: TStringList;
  tmpPoints: TabcPointList;
  Pt: TPoint;
  i, commaPos: integer;
  s: string;
begin
  tmpStrings := TStringList.Create;
  try
    tmpStrings.LoadFromFile(FName);
    tmpPoints := TabcPointList.Create;
    try
      for i := 0 to tmpStrings.Count - 1 do
      begin
        s := tmpStrings[i];
        commaPos := Pos(',', s);
        if commaPos > 0 then
        begin
          Pt.X := StrToIntDef(Trim(Copy(s,1,commaPos-1)), DEF_INT);
          Pt.Y := StrToIntDef(Trim(Copy(s,commaPos+1, 255)), DEF_INT);
          if (Pt.X <> DEF_INT) and (Pt.Y <> DEF_INT) then
            tmpPoints.AddPoint(Pt);
        end;
      end;
      if tmpPoints.Count > 0 then
        SetPoints(tmpPoints)
      else
        MessageDlg(
          'Could not read points from file.' + #13#10 + #13#10 +
          'Expecting file format:' + #13#10 +
          '  0,1' + #13#10 +
          '  1,-2' + #13#10 +
          '  ...',
          mtError,
          [mbOk],
          0);
    finally
      tmpPoints.Free;
    end;
  finally
    tmpStrings.Free;
  end;
end;

procedure TabcEditPointsDialog.SaveToFile(FName: string);
begin
  lstPoints.Items.SaveToFile(FName);
end;

procedure TabcEditPointsDialog.edtXEnter(Sender: TObject);
begin
  TEdit(Sender).SelectAll;
end;

end.
