unit abcpspr;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{::This unit contains the designtime component editor for <See Class=TabcSpriteList>.<P>
Use the registration procedures to add new sprite classes to the designer.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ComCtrls, ExtCtrls, ImgList, ToolWin, ActnList,
  abcspr,
{$IFDEF DELPHI6}
  DesignIntf, DesignEditors, DesignWindows
{$ELSE}
  DsgnIntf, DsgnWnds
{$IFNDEF DELPHI5}
  , LibIntf, LibConst
{$ENDIF}
{$ENDIF}
     ;

{$IFNDEF T2H}  //editor is not documented
	 
type
  { TabcdfSpriteListEditor }

  TabcdfSpriteListEditor = class(TDesignWindow)
    ImageList1: TImageList;
    SpriteListBox: TListBox;
    ToolBar1: TToolBar;
    btnNew: TToolButton;
    btnDelete: TToolButton;
    btnForward: TToolButton;
    btnBack: TToolButton;
    btnSpace: TToolButton;
    mnuSpriteClasses: TPopupMenu;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpriteListBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SpriteListBoxClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnForwardClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
  private
    FSpriteList: TabcSpriteList;
{$IFNDEF DELPHI6}
    FDesigner: IFormDesigner;
{$ENDIF}
    procedure SetSpriteList(const Value: TabcSpriteList);
    procedure SpriteListBoxSelectionChanged;
    procedure UpdateDisplay;
    procedure UpdateSpriteList;
    procedure UpdateCaption;
    procedure UpdateSelection;
    procedure Cut;
    procedure Copy;
    procedure Paste;
    procedure SelectAll;
    procedure SelectNone;
    procedure SelectSprite(Sprite: TabcSprite);
    procedure AddSprite(SpriteClass: TabcSpriteClass);
    procedure RemoveSprites;
    procedure CheckSpriteDelete;
{$IFDEF DELPHI5}
    procedure CheckSpriteAdd;
{$ENDIF}
    procedure MoveSprites(ToFront: Boolean);
    procedure CreateSpriteMenuItems;
    procedure FreeSpriteMenuItems;
    procedure SpriteMenuItemClick(Sender: TObject);
  protected
    procedure Activated; override;
{$IFNDEF DELPHI6}
    function UniqueName(Component: TComponent): string; override;
{$ENDIF}
  public
{$IFDEF DELPHI5}
    function GetEditState: TEditState; override;
{$ENDIF}
{$IFDEF DELPHI6}
    function EditAction(Action: TEditAction): Boolean; override;
    procedure ItemDeleted(const ADesigner: IDesigner; Item: TPersistent); override;
    procedure ItemsModified(const Designer: IDesigner); override;
    procedure SelectionChanged(const ADesigner: IDesigner; const ASelection: IDesignerSelections); override;
    procedure DesignerClosed(const Designer: IDesigner; AGoingDormant: Boolean); override;
{$ELSE}
{$IFDEF DELPHI5}
    procedure EditAction(Action: TEditAction); override;
    procedure SelectionChanged(ASelection: TDesignerSelectionList); override;
{$ELSE}
    procedure EditAction(Action: LibIntf.TEditAction); override;
    procedure SelectionChanged(ASelection: TComponentList); override;
{$ENDIF}
    procedure ComponentDeleted(Component: IPersistent); override;
    procedure FormClosed(AForm: TCustomForm); override;
    procedure FormModified; override;
    property Designer: IFormDesigner read FDesigner write FDesigner;
{$ENDIF}
    property SpriteList: TabcSpriteList read FSpriteList write SetSpriteList;
  end;

  { TabcSpriteListComponentEditor }

  TabcSpriteListComponentEditor = class (TComponentEditor)
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;
{$ENDIF} {T2H}  //editor is not documented

{::Registers a sprite class with the <See Class=TabcSpriteList> design editor.<p>
For example:<br>
<Code>
  abcRegisterSpriteClass(TabcBitmapSprite, 'Bitmap Sprite', True);
  abcRegisterSpriteClass(TabcPictureSprite, 'Picture Sprite', False);
</Code>
<p>
The IsDefaultNewSprite class is the sprite added when the New button on the designer is clicked.<br>
All sprite classes are listed on the menu when the designer New button dropdown arrow is clicked.
}
procedure abcRegisterSpriteClass(SpriteClass: TabcSpriteClass; MenuCaption: string; IsDefaultNewSprite: Boolean);

{::Unregisters a sprite class with the <See Class=TabcSpriteList> design editor.
}
procedure abcUnregisterSpriteClass(SpriteClass: TabcSpriteClass);

implementation

{$R *.DFM}

{$IFDEF DELPHI5}
uses DesignConst;
{$ENDIF}

var
  SpriteClasses: TStringList = nil;
  SpriteListEditorsList: TList = nil;

procedure abcRegisterSpriteClass(SpriteClass: TabcSpriteClass; MenuCaption: string; IsDefaultNewSprite: Boolean);
begin
  if SpriteClasses = nil then
    SpriteClasses := TStringList.Create;
  RegisterClass(SpriteClass);
  RegisterNoIcon([SpriteClass]);
  if SpriteClasses.IndexOfObject(TObject(SpriteClass)) <> -1 then
    Exit;
  if MenuCaption = '' then
    MenuCaption := SpriteClass.ClassName;
  if IsDefaultNewSprite then
    SpriteClasses.InsertObject(0, MenuCaption, TObject(SpriteClass))
  else
    SpriteClasses.AddObject(MenuCaption, TObject(SpriteClass));
end;

procedure abcUnregisterSpriteClass(SpriteClass: TabcSpriteClass);
var
  i: integer;
begin
  if SpriteClasses <> nil then
  begin
    i := SpriteClasses.IndexOfObject(TObject(SpriteClass));
    if i <> -1 then
    begin
      UnRegisterClass(SpriteClass);
      SpriteClasses.Delete(i);
    end;
  end;
end;

procedure ShowSpriteListEditor(ADesigner: IDesigner; ASpriteList: TabcSpriteList);
var
  I: Integer;
  Form: TabcdfSpriteListEditor;
begin
  if SpriteListEditorsList = nil then
    SpriteListEditorsList := TList.Create;
  for I := 0 to SpriteListEditorsList.Count-1 do
  begin
    Form := TabcdfSpriteListEditor(SpriteListEditorsList[I]);
    with Form do
      if (Designer = ADesigner) and (SpriteList = ASpriteList) then
      begin
        Show;
        BringToFront;
        Exit;
      end;
  end;
  Form := TabcdfSpriteListEditor.Create(Application);
  with Form do
  try
    Designer := ADesigner {$IFNDEF DELPHI6} as IFormDesigner{$ENDIF};
    SpriteList := ASpriteList;
    Show;
  except
    Free;
  end;
end;

{ TabcSpriteListComponentEditor }

procedure TabcSpriteListComponentEditor.Edit;
begin
  ShowSpriteListEditor(Designer, TabcSpriteList(Component));
end;

procedure TabcSpriteListComponentEditor.ExecuteVerb(Index: Integer);
begin
  Edit;
end;

function TabcSpriteListComponentEditor.GetVerb(Index: Integer): string;
begin
  Result := 'Sprite List Editor...';
end;

function TabcSpriteListComponentEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;


{ TabcdfSpriteListEditor }

procedure TabcdfSpriteListEditor.UpdateDisplay;
begin
  UpdateSpriteList;
  UpdateCaption;
  UpdateSelection;
  SpriteListBoxSelectionChanged;
end;

procedure TabcdfSpriteListEditor.UpdateCaption;
var
  NewCaption: string;
begin
  if (SpriteList <> nil) and (SpriteList.Owner <> nil) then
    NewCaption := 'Editing - ' + SpriteList.Owner.Name + '.' + SpriteList.Name;
  if Caption <> NewCaption then Caption := NewCaption;
end;

procedure TabcdfSpriteListEditor.UpdateSpriteList;
var
  EnableList: Boolean;
  I: Integer;
  Sprite: TabcSprite;
begin
  SpriteListBox.Clear;
  EnableList := False;
  try
    if SpriteList = nil then Exit;
    for I := 0 to SpriteList.Count - 1 do
    begin
      Sprite := FSpriteList.Sprites[I];
      if not (csDestroying in Sprite.ComponentState) and
         (Sprite.Owner = SpriteList.Owner) then
        SpriteListbox.Items.AddObject(Sprite.Name, Sprite);
    end;
    EnableList := True;
  finally
    SpriteListBox.Enabled := EnableList;
    SpriteListBoxSelectionChanged;
  end;
end;

procedure TabcdfSpriteListEditor.UpdateSelection;
var
  I: Integer;
  Sprite: TabcSprite;
{$IFDEF DELPHI6}
  ComponentList: IDesignerSelections;
{$ELSE}
{$IFDEF DELPHI5}
  ComponentList: TDesignerSelectionList;
{$ELSE}
  ComponentList: TComponentList;
{$ENDIF}
{$ENDIF}
begin
  if Active then
    if SpriteListBox.SelCount > 0 then
    begin
{$IFDEF DELPHI6}
      ComponentList := CreateSelectionList;
{$ELSE}
{$IFDEF DELPHI5}
      ComponentList := TDesignerSelectionList.Create;
{$ELSE}
      ComponentList := TComponentList.Create;
{$ENDIF}
{$ENDIF}
      try
        with SpriteListBox do
          for I := 0 to Items.Count - 1 do
          begin
            Sprite := TabcSprite(Items.Objects[I]);
            if Sprite <> nil then
            begin
              if Items[I] <> Sprite.Name then
                Items[I] := Sprite.Name;
              if Selected[I] then
                ComponentList.Add(Sprite);
            end;
          end;
        if ComponentList.Count = 0 then ComponentList.Add(SpriteList);
      except
{$IFDEF DELPHI6}
        ComponentList := nil;
{$ELSE}
        ComponentList.Free;
{$ENDIF}
        raise;
      end;
      Designer.SetSelections(ComponentList);
    end
    else
      Designer.SelectComponent(SpriteList);
end;

{$IFDEF DELPHI5}
function TabcdfSpriteListEditor.GetEditState: TEditState;
begin
  Result := [];
  if ClipboardComponents then Result := [esCanPaste];
  if SpriteListBox.SelCount > 0 then
    Result := Result + [esCanCopy, esCanCut, esCanDelete];
end;
{$ENDIF}

procedure TabcdfSpriteListEditor.Cut;
begin
  CheckSpriteDelete;
  Copy;
  RemoveSprites;
end;

procedure TabcdfSpriteListEditor.Copy;
var
  I: Integer;
{$IFDEF DELPHI6}
  ComponentList: IDesignerSelections;
{$ELSE}
{$IFDEF DELPHI5}
  ComponentList: TDesignerSelectionList;
{$ELSE}
  ComponentList: TComponentList;
{$ENDIF}
{$ENDIF}
begin
{$IFDEF DELPHI6}
  ComponentList := CreateSelectionList;
{$ELSE}
{$IFDEF DELPHI5}
  ComponentList := TDesignerSelectionList.Create;
{$ELSE}
  ComponentList := TComponentList.Create;
{$ENDIF}
{$ENDIF}
  try
    with SpriteListBox do
      for I := 0 to Items.Count - 1 do
        if Selected[I] then
          ComponentList.Add(TComponent(Items.Objects[I]));
    CopyComponents(SpriteList.Owner, ComponentList);
  finally
{$IFDEF DELPHI6}
    ComponentList := nil;
{$ELSE}
    ComponentList.Free;
{$ENDIF}
  end;
end;

procedure TabcdfSpriteListEditor.Paste;
var
  I, Index: Integer;
  Sprite: TabcSprite;
{$IFDEF DELPHI6}
  ComponentList: IDesignerSelections;
{$ELSE}
{$IFDEF DELPHI5}
  ComponentList: TDesignerSelectionList;
{$ELSE}
  ComponentList: TComponentList;
{$ENDIF}
{$ENDIF}
begin
{$IFDEF DELPHI6}
  ComponentList := CreateSelectionList;
{$ELSE}
{$IFDEF DELPHI5}
  ComponentList := TDesignerSelectionList.Create;
{$ELSE}
  ComponentList := TComponentList.Create;
{$ENDIF}
{$ENDIF}
  try
    try
      PasteComponents(SpriteList.Owner, SpriteList, ComponentList);
    finally
      UpdateDisplay;
    end;
    try
      with SpriteListBox do
        for I := 0 to Items.Count - 1 do Selected[I] := False;
      for I := 0 to ComponentList.Count - 1 do
        if ComponentList[I] is TabcSprite then
        begin
          Sprite := TabcSprite(ComponentList[I]);
          Sprite.Name := UniqueName(Sprite);
          Index := SpriteListBox.Items.IndexOf(Sprite.Name);
          if Index <> -1 then
            SpriteListBox.Selected[Index] := True;
        end;
    finally
      UpdateDisplay;
    end;
  finally
{$IFDEF DELPHI6}
    ComponentList := nil;
{$ELSE}
    ComponentList.Free;
{$ENDIF}
  end;
end;

procedure TabcdfSpriteListEditor.SelectAll;
var
  I: Integer;
begin
  with SpriteListBox do
    for I := 0 to Items.Count - 1 do Selected[I] := True;
end;

procedure TabcdfSpriteListEditor.SelectNone;
var
  I: Integer;
begin
  with SpriteListBox do
    for I := 0 to Items.Count - 1 do Selected[I] := False;
  UpdateSelection;
end;

procedure TabcdfSpriteListEditor.SelectSprite(Sprite: TabcSprite);
var
  I: Integer;
begin
  SelectNone;
  with SpriteListBox do
  begin
    I := Items.IndexOfObject(Sprite);
    if I <> -1 then
      Selected[I] := True;
  end;
  UpdateSelection;
end;

procedure TabcdfSpriteListEditor.RemoveSprites;
var
  I, Focused: Integer;
begin
  CheckSpriteDelete;
  try
    Focused := SpriteListBox.ItemIndex;
    with SpriteListBox do
      for I := Items.Count - 1 downto 0 do
        if Selected[I] then
          TabcSprite(Items.Objects[I]).Free;
    Designer.Modified;
  finally
    UpdateDisplay;
  end;
  if Focused <> -1 then
  begin
    if Focused > (SpriteListBox.Items.Count -1) then
      Focused := SpriteListBox.Items.Count - 1;
    SpriteListBox.ItemIndex := Focused;
    if Focused > -1 then
      SpriteListBox.Selected[Focused] := True;
    UpdateSelection;
    SpriteListBoxSelectionChanged;
  end;
  if SpriteListBox.CanFocus then
    SpriteListBox.SetFocus;
end;

procedure TabcdfSpriteListEditor.AddSprite(SpriteClass: TabcSpriteClass);
var
  NewSprite: TabcSprite;
begin
{$IFDEF DELPHI5}
  CheckSpriteAdd;
{$ENDIF}  
  NewSprite := SpriteClass.Create(Designer.GetRoot);
  NewSprite.Name := UniqueName(NewSprite);
  SpriteList.Add(NewSprite);
  SpriteList.Sorted := True;
  Designer.Modified;
  UpdateDisplay;
  SelectSprite(NewSprite);
end;

procedure TabcdfSpriteListEditor.MoveSprites(ToFront: Boolean);
var
  I: Integer;
begin
  with SpriteListBox do
    for I := 0 to Items.Count-1 do
      if Selected[I] then
        with TabcSprite(Items.Objects[I]) do
          if ToFront then
          begin
            if (ZOrder > 0) then
              ZOrder := ZOrder - 1;
          end
          else
            ZOrder := ZOrder + 1;
  SpriteList.Sorted := True;
  Designer.Modified;
  UpdateDisplay;
end;

procedure TabcdfSpriteListEditor.CheckSpriteDelete;
var
  I: Integer;
begin
  with SpriteListBox do
    for I := 0 to Items.Count-1 do
      if Selected[I] and (csAncestor in TabcSprite(Items.Objects[I]).ComponentState) then
        raise Exception.Create(SCantDeleteAncestor);
end;

{$IFDEF DELPHI5}
procedure TabcdfSpriteListEditor.CheckSpriteAdd;
begin
  if (FSpriteList <> nil) and (FSpriteList.Owner <> nil) and
    (csInline in FSpriteList.Owner.ComponentState) then
    raise Exception.Create(SCantAddToFrame);
end;
{$ENDIF}

//Q. what is the exact need for this WM_ACTIVATE design message??

procedure TabcdfSpriteListEditor.Activated;
var
  Msg: TMessage;
{$IFDEF DELPHI6}
  Root: TComponent;
{$ENDIF}
begin
  Msg.Msg := WM_ACTIVATE;
  Msg.WParam := 1;
{$IFDEF DELPHI6}
  Root := Designer.Root;
  if Root is TCustomForm then
    TCustomForm(Root).Designer.IsDesignMsg(TCustomForm(Root), Msg);
{$ELSE}
  Designer.IsDesignMsg(Designer.Form, Msg);
{$ENDIF}
  try
    UpdateCaption;
    UpdateSelection;
  except
    SpriteListBox.Items.Clear;
  end;
end;

{$IFNDEF DELPHI6}
function TabcdfSpriteListEditor.UniqueName(Component: TComponent): string;
begin
  Result := Designer.UniqueName(Component.ClassName);
end;
{$ENDIF}

procedure TabcdfSpriteListEditor.SetSpriteList(
  const Value: TabcSpriteList);
begin
  FSpriteList := Value;
  if Value = nil then
    Close
  else
    UpdateDisplay;
end;

procedure TabcdfSpriteListEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TabcdfSpriteListEditor.FormCreate(Sender: TObject);
begin
  CreateSpriteMenuItems;
  SpriteListEditorsList.Add(self);
end;

procedure TabcdfSpriteListEditor.FormDestroy(Sender: TObject);
begin
  if SpriteListEditorsList <> nil then
    SpriteListEditorsList.Remove(self);
  FreeSpriteMenuItems;
end;

procedure TabcdfSpriteListEditor.SpriteListBoxKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  i: integer;
begin
  if (Shift = []) then
    if (Key = VK_ESCAPE) then
    begin
      with SpriteListBox do
        for i := 0 to Items.Count - 1 do
          Selected[i] := False;
      SpriteListBoxSelectionChanged;
    end
    else if (Key = VK_DELETE) then
      RemoveSprites
    else if (Key = VK_INSERT) then
      //get default sprite class?
      AddSprite(TabcSpriteClass(SpriteClasses.Objects[0]));
end;

procedure TabcdfSpriteListEditor.SpriteListBoxClick(Sender: TObject);
begin
  SpriteListBoxSelectionChanged;
  UpdateSelection;
end;

procedure TabcdfSpriteListEditor.SpriteListBoxSelectionChanged;
var
  isSelected: Boolean;
begin
  isSelected := SpriteListBox.SelCount > 0;
  btnDelete.Enabled := isSelected;
  btnForward.Enabled := isSelected;
  btnBack.Enabled := isSelected;
end;

procedure TabcdfSpriteListEditor.CreateSpriteMenuItems;
var
  i: integer;
begin
  with SpriteClasses do
    for i := 0 to Count - 1 do
      mnuSpriteClasses.Items.Add(NewItem(
        Strings[i],
        0, False, True,
        SpriteMenuItemClick,
        0,
        ''));
end;

procedure TabcdfSpriteListEditor.FreeSpriteMenuItems;
var
  i: integer;
  menuItem: TMenuItem;
begin
  with mnuSpriteClasses.Items do
    for i := Count - 1 downto 0 do
    begin
      menuItem := mnuSpriteClasses.Items[i];
      Remove(menuItem);
      menuItem.Free;
    end;
end;

procedure TabcdfSpriteListEditor.SpriteMenuItemClick(Sender: TObject);
begin
  if Sender is TMenuItem then
    AddSprite(TabcSpriteClass(SpriteClasses.Objects[TMenuItem(Sender).MenuIndex]));
end;

procedure TabcdfSpriteListEditor.btnNewClick(Sender: TObject);
begin
  AddSprite(TabcSpriteClass(SpriteClasses.Objects[0]));
end;

procedure TabcdfSpriteListEditor.btnDeleteClick(Sender: TObject);
begin
  RemoveSprites;
end;

procedure TabcdfSpriteListEditor.btnForwardClick(Sender: TObject);
begin
  MoveSprites(True);
end;

procedure TabcdfSpriteListEditor.btnBackClick(Sender: TObject);
begin
  MoveSprites(False);
end;

{$IFDEF DELPHI6}
function TabcdfSpriteListEditor.EditAction(Action: TEditAction): Boolean;
begin
  Result := True;
  case Action of
    eaCut: Cut;
    eaCopy: Copy;
    eaPaste: Paste;
    eaDelete: RemoveSprites;
    eaSelectAll:
      begin
        SelectAll;
        UpdateSelection;
      end;
  else
    Result := False;
  end;
end;
{$ELSE}
procedure TabcdfSpriteListEditor.EditAction(Action: TEditAction);
begin
  case Action of
    eaCut: Cut;
    eaCopy: Copy;
    eaPaste: Paste;
    eaDelete: RemoveSprites;
    eaSelectAll:
      begin
        SelectAll;
        UpdateSelection;
      end;
  end;
end;
{$ENDIF}

{$IFDEF DELPHI6}
procedure TabcdfSpriteListEditor.DesignerClosed(const Designer: IDesigner; AGoingDormant: Boolean);
begin
  if Designer = Self.Designer then
    SpriteList := nil;
end;
{$ELSE}
procedure TabcdfSpriteListEditor.FormClosed(AForm: TCustomForm);
begin
  if Designer.Form = AForm then
    SpriteList := nil;
end;
{$ENDIF}

{$IFDEF DELPHI6}
procedure TabcdfSpriteListEditor.ItemsModified(const Designer: IDesigner);
begin
  if (Designer = Self.Designer) and (SpriteList <> nil) then
    UpdateDisplay;
end;
{$ELSE}
procedure TabcdfSpriteListEditor.FormModified;
begin
  if (SpriteList <> nil) then
    UpdateDisplay;
end;
{$ENDIF}

{$IFDEF DELPHI6}
procedure TabcdfSpriteListEditor.ItemDeleted(const ADesigner: IDesigner; Item: TPersistent);
begin
  if Item = SpriteList then
    SpriteList := nil
  else if (Item is TabcSprite) and
          (TabcSprite(Item).SpriteList = SpriteList) then
    UpdateDisplay;
end;
{$ELSE}
procedure TabcdfSpriteListEditor.ComponentDeleted(Component: IPersistent);
var
  vItem: TPersistent;
begin
  vItem := ExtractPersistent(Component);
  if vItem = SpriteList then
    SpriteList := nil
  else if (vItem is TabcSprite) and
          (TabcSprite(vItem).SpriteList = SpriteList) then
    UpdateDisplay;
end;
{$ENDIF}

{$IFDEF DELPHI6}
procedure TabcdfSpriteListEditor.SelectionChanged(const ADesigner: IDesigner; const ASelection: IDesignerSelections);
{$ELSE}
{$IFDEF DELPHI5}
procedure TabcdfSpriteListEditor.SelectionChanged(ASelection: TDesignerSelectionList);
{$ELSE}
procedure TabcdfSpriteListEditor.SelectionChanged(ASelection: TComponentList);
{$ENDIF}
{$ENDIF}
var
  I: Integer;
  S: Boolean;

  function InSelection(Component: TComponent): Boolean;
  var
    I: Integer;
  begin
    Result := True;
    if ASelection <> nil then
      with ASelection do
        for I := 0 to Count - 1 do
          if Component = Items[I] then
            Exit;
    Result := False;
  end;

begin
  with SpriteListBox do
    for I := 0 to Items.Count - 1 do
    begin
      if Items[i] <> TComponent(Items.Objects[I]).Name then
        Items[i] := TComponent(Items.Objects[I]).Name;
      S := InSelection(TComponent(Items.Objects[I]));
      if Selected[I] <> S then Selected[I] := S;
    end;
  SpriteListBoxSelectionChanged;
end;

initialization

finalization
  SpriteClasses.Free;
  SpriteClasses := nil;
  SpriteListEditorsList.Free;
  SpriteListEditorsList := nil;

end.
