unit abcptips;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{property & component editors for tips dialog}

interface

uses SysUtils, Classes, ExptIntf, IniFiles, Controls, Forms, Dialogs,
     abctips, abcdftip, abcprop,
{$IFDEF DELPHI6}
     DesignIntf, DesignEditors
{$ELSE}
     DsgnIntf
{$ENDIF}
     ;

type

  { TabcWelcomeTipsEditor }

  TabcWelcomeTipsEditor = class(TComponentEditor)
  private
    procedure EditTips;
    procedure Preview;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  TabcIniFileNameProperty = class(TabcFileNameProperty)
  protected
    function GetTitle: string; override;
    function GetFilter: string; override;
  end;

implementation

{ TabcWelcomeTipsEditor }

function TabcWelcomeTipsEditor.GetVerbCount: Integer;
begin
  Result := 2;
end;

function TabcWelcomeTipsEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := '&Edit INI Tip Text';
    1: Result := '&Preview Tip Dialog';
  else
    Result := '';
  end;
end;

procedure TabcWelcomeTipsEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: EditTips;
    1: Preview;
  else
  end;
end;

procedure TabcWelcomeTipsEditor.EditTips;
{edit the tips INI File}
var
  abcWelcomeTipsEditDialog: TabcdfWelcomeTipsEditDialog;
  INIFile: TIniFile;
  i: integer;
  sTemp: string;
const
  C_INI_TIP_SECTION = 'TIPS';
  C_INI_TIP_TEXT_SECTION = 'TIP TEXT';
  C_INI_TIP_SHOW = 'ShowOnStartup';
  C_INI_NEXT_TIP = 'NextTip';
begin
  with TabcWelcomeTips(Component) do
  begin
    if (IniFileName = '') then raise Exception.Create('IniFileName required');
    if IniLocal then
      {add project dpr path to filename}
      INIFile := TIniFile.Create(ExtractFilePath(ToolServices.GetProjectName) + IniFileName)
    else
      INIFile := TIniFile.Create(IniFileName);

    abcWelcomeTipsEditDialog := TabcdfWelcomeTipsEditDialog.Create(nil);
    with abcWelcomeTipsEditDialog, INIFile do
    try
      edtNextTip.Text := InttoStr(ReadInteger(C_INI_TIP_SECTION,C_INI_NEXT_TIP,1));
      chkShowOnStartup.Checked := ReadBool(C_INI_TIP_SECTION,C_INI_TIP_SHOW,True);
      memTipText.Lines.Clear;
      i := 1;
      sTemp := ReadString(C_INI_TIP_TEXT_SECTION, 'Tip' + Inttostr(i), '');
      while sTemp <> '' do
      begin
        memTipText.Lines.Add(sTemp);
        i := i + 1;
        sTemp := ReadString(C_INI_TIP_TEXT_SECTION, 'Tip' + Inttostr(i), '');
      end;
      if ShowModal = mrOk then
      begin
        WriteInteger(C_INI_TIP_SECTION,C_INI_NEXT_TIP,StrToIntDef(edtNextTip.Text, 1));
        WriteBool(C_INI_TIP_SECTION,C_INI_TIP_SHOW,chkShowOnStartup.Checked);
        EraseSection(C_INI_TIP_TEXT_SECTION);
        with memTipText.Lines do
          for i := 0 to Count - 1 do
            WriteString(C_INI_TIP_TEXT_SECTION, 'Tip' + InttoStr(i+1), Strings[i]);
      end;
    finally
      abcWelcomeTipsEditDialog.Free;
      INIFile.Free;
    end;
  end;

end;

procedure TabcWelcomeTipsEditor.Preview;
{copy the component to a runtime version, adjust ini file path and execute}
var
  RunComponent: TabcWelcomeTips;
begin
  with TabcWelcomeTips(Component) do
  begin
    RunComponent := TabcWelcomeTips.Create(nil);
    try
      RunComponent.Caption := Caption;
      RunComponent.CaptionGreeting := CaptionGreeting;
      RunComponent.CaptionProgress := CaptionProgress;
      RunComponent.CaptionShowNext := CaptionShowNext;
      RunComponent.CaptionTip := CaptionTip;
      if IniLocal then
        {add project dpr path to filename}
        RunComponent.IniFileName := ExtractFilePath(ToolServices.GetProjectName) + IniFileName
      else
        RunComponent.IniFileName := IniFileName;
      RunComponent.IniLocal := False;
      RunComponent.SharedTipFileName := SharedTipFileName;
      RunComponent.ShowModal := True;
      RunComponent.ShowProgress := ShowProgress;
      RunComponent.VisibleButtons := VisibleButtons;

      RunComponent.Execute;

    finally
      RunComponent.Free;
    end;
  end;
end;

{ TabcIniFileNameProperty }

function TabcIniFileNameProperty.GetTitle: string;
begin
  Result := 'Open Ini File';
end;

function TabcIniFileNameProperty.GetFilter: string;
begin
  Result := 'Ini Files (*.ini)|*.ini|All Files (*.*)|*.*';
end;

end.
