unit abcpvers;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{property & component editors for Version Label component}

interface

uses SysUtils, Classes, abcversn,
{$IFDEF DELPHI6}
     DesignIntf, DesignEditors
{$ELSE}
     DsgnIntf
{$ENDIF}
     ;

type

  { TabcVersionInfoKeyProperty }

  TabcVersionInfoKeyProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

  { TabcVersionInfoLabelEditor }

  TabcVersionInfoLabelEditor = class(TDefaultEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

implementation

{ TabcVersionInfoKeyProperty }

const
  C_STANDARD_VER_KEYS : array [0..11] of string =
    ('CompanyName',
     'FileDescription',
     'FileVersion',
     'InternalName',
     'LegalCopyright',
     'LegalTrademarks',
     'OriginalFilename',
     'ProductName',
     'ProductVersion',
     'Comments',
     'BuildFlags',
     'CustomKeyName');

  C_STANDARD_VER_CAPTIONS : array [0..11] of string =
    ('Company Name:',
     'File Description:',
     'File Version:',
     'Internal Name:',
     'Legal Copyright:',
     'Legal Trademarks:',
     'Original Filename:',
     'Product Name:',
     'Product Version:',
     'Comments:',
     'Build Flags:',
     'Custom Caption:');

function TabcVersionInfoKeyProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TabcVersionInfoKeyProperty.GetValues(Proc: TGetStrProc);
var
  i: integer;
begin
  for i := Low(C_STANDARD_VER_KEYS) to High(C_STANDARD_VER_KEYS) do
    Proc(C_STANDARD_VER_KEYS[i]);
end;

procedure TabcVersionInfoKeyProperty.SetValue(const Value: string);
var
  i: integer;
begin
  inherited SetValue(Value);
  for i := Low(C_STANDARD_VER_KEYS) to High(C_STANDARD_VER_KEYS) do
    if CompareText(Value, C_STANDARD_VER_KEYS[i]) = 0 then
    begin
      {set corresponding Caption property}
      TabcVersionInfoItem(GetComponent(0)).Caption := C_STANDARD_VER_CAPTIONS[i];
      Break;
    end;
end;

{ TabcVersionInfoLabelEditor }

function TabcVersionInfoLabelEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TabcVersionInfoLabelEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'Add &Default Items';
  else
    Result := '';
  end;
end;

procedure TabcVersionInfoLabelEditor.ExecuteVerb(Index: Integer);
var
  i: integer;
  NewItem: TabcVersionInfoItem;
begin
  case Index of
    0:
    with TabcVersionLabel(Component).VersionInfo do
    begin
      Clear;
      for i := Low(C_STANDARD_VER_KEYS) to High(C_STANDARD_VER_KEYS) - 2 do
      begin
        NewItem := Add;
        NewItem.Key := C_STANDARD_VER_KEYS[i];
        NewItem.Caption := C_STANDARD_VER_CAPTIONS[i];
      end;
    end;
  else
  end;
end;

end.
