unit abcrched;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}


{
Not Implemented:
 EditorWidth property
 EditorTitle
 Printer Sizing or wysiwyg
   EmFormatRange
   EMSetRect
 OLE Embedding
 Tabs in popup editor
 MDI child or other non-modal support in editor.

Bug fixes:
  - fixed bug redrawing text when resizing
  - painting a bitmap image of rich text still does not position text or wrap exactly as shown onscreen,
    probably due to bug in microsoft windows API see Q142320
  - improved wordwrapping sizing when drawing offscreen
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, RichEdit, Menus, abcpage, abcforms, abcfact;

type

  { TabcCustomRichEdit }

  TabcREPopupMenuOption = (arpEdit, arpCancel,
                           arpClear, arpLoadTemplate,
                           arpFileOpen, arpFileSaveAs,
                           arpPrint, arpPageSetup,
                           arpUndo, arpCut, arpCopy, arpPaste, arpDelete, arpSelectAll,
                           arpBold, arpItalic, arpUnderline,
                           arpFont, arpBullets,
                           arpFind, arpReplace);
  TabcREPopupMenuOptions = set of TabcREPopupMenuOption;

  TabcREEditorOption = (areFileNew,
                        areFileOpen,
                        areFileLoadTemplate,
                        areFileSave,
                        areFilePrint,
                        areFilePageSetup,
                        areEditMenu,
                        areSearchMenu,
                        areFormatMenu,
                        areFileToolbar,
                        areEditToolbar,
                        areFindToolbar,
                        areFontToolbar,
                        areFormatToolbar,
                        areStatusBar,
                        areConfirmSaveOnClose,
                        areCancelOnEsc);
  TabcREEditorOptions = set of TabcREEditorOption;

  TabcREEditWidth = (arwScreen, arwPrinter);

  TabcFileDroppedEvent = procedure(Sender: TObject; var FileName: string; var Accept: Boolean) of object;
  
  TabcCustomRichEdit = class(TCustomRichEdit)
  private
    FAcceptDroppedFiles: Boolean;
    FEditorOptions: TabcREEditorOptions;
    FEditorTitle: string;
    FEditorWidth: TabcREEditWidth;
    FEditorWindowState: TWindowState;
    FFileName: TFileName;
    FFileTemplate: TFileName;
    FDesignSaveRTF: TMemoryStream;

    FOnAfterDialog: TabcModalDialogEvent;
    FOnBeforeDialog: TabcModalDialogEvent;
    FOnCancel: TNotifyEvent;
    FOnFileDropped: TabcFileDroppedEvent;
    {find & replace dialogs}
    FFindDlg: TFindDialog;
    FReplaceDlg: TReplaceDialog;
    FFindText, FReplaceText: string;
    FFound: Boolean;
    FStartPos: Integer;
    FSearchTypes: TSearchTypes;
    FSaveTextRect: TRect;

    {Printing}
    FPrintTitle: string;
    FPrintMargins: TabcPrintMarginsUnits;

    {popup menu}
    FPopupOptions: TabcREPopupMenuOptions;
    FPopupMenu: TPopupMenu;
    FmiEdit, FmiSep1,
    FmiClear, FmiCancel, FmiLoadTemplate,
    FmiFileOpen, FmiFileSaveAs,
    FmiPrint, FmiPageSetup, FmiSep2,
    FmiUndo, FmiCut, FmiCopy, FmiPaste, FmiDelete, FmiSelectAll, FmiSep3,
    FmiBold, FmiItalic, FmiUnderline, FmiSep4,
    FmiFont, FmiBullets, FmiSep5,
    FmiFind, FmiReplace: TMenuItem;
    procedure FmiEditClick(Sender: TObject);
    procedure FmiCancelClick(Sender: TObject);
    procedure FmiClearClick(Sender: TObject);
    procedure FmiLoadTemplateClick(Sender: TObject);
    procedure FmiFileOpenClick(Sender: TObject);
    procedure FmiFileSaveAsClick(Sender: TObject);
    procedure FmiPrintClick(Sender: TObject);
    procedure FmiPageSetupClick(Sender: TObject);
    procedure FmiUndoClick(Sender: TObject);
    procedure FmiCutClick(Sender: TObject);
    procedure FmiCopyClick(Sender: TObject);
    procedure FmiPasteClick(Sender: TObject);
    procedure FmiDeleteClick(Sender: TObject);
    procedure FmiSelectAllClick(Sender: TObject);
    procedure FmiBoldClick(Sender: TObject);
    procedure FmiItalicClick(Sender: TObject);
    procedure FmiUnderlineClick(Sender: TObject);
    procedure FmiFontClick(Sender: TObject);
    procedure FmiBulletsClick(Sender: TObject);
    procedure FmiFindClick(Sender: TObject);
    procedure FmiReplaceClick(Sender: TObject);

    procedure ActivateFormWindow(Sender: TObject);
    procedure FindDialogFind(Sender: TObject);
    procedure Find;
    procedure FindFromPos;
    procedure PrintMarginsChanged(Sender: TObject);
    procedure ReplaceAll;
    procedure ReplaceDialogReplace(Sender: TObject);
    procedure ReplaceSelection;
    procedure SetAcceptDroppedFiles(Value: Boolean);
    procedure SetEditorWidth(Value: TabcREEditWidth);
    procedure SetPrintMargins(Value: TabcPrintMarginsUnits);
    procedure AdjustTextRect;
    {Filing}
    procedure ReadRichText(Stream: TStream);
    procedure WriteRichText(Stream: TStream);

    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMPrintClient(var Message: TWMPaint); message WM_PRINTCLIENT;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure DefineProperties(Filer: TFiler); override;
    procedure Loaded; override;
    function IsRichTextStored(Filer: TFiler): Boolean; virtual;
    function CanModify: Boolean; virtual;
    function CreatePopupMenu: TPopupMenu; virtual;
    procedure DoMenuPopup(Sender: TObject); virtual;
    procedure FreePopupMenu; virtual;
    procedure PaintClientTextToDC(DC: HDC);

    {can be published}
    property AcceptDroppedFiles: Boolean read FAcceptDroppedFiles write SetAcceptDroppedFiles default True;
    property EditorOptions: TabcREEditorOptions read FEditorOptions write FEditorOptions
      default [areFilePrint, areFilePageSetup, areEditMenu, areSearchMenu, areFormatMenu,
               areEditToolbar, areFindToolbar, areFontToolbar, areFormatToolbar,
               areStatusBar, areCancelOnEsc];
    property EditorTitle: string read FEditorTitle write FEditorTitle;
    property EditorWidth: TabcREEditWidth read FEditorWidth write SetEditorWidth default arwScreen;
    property EditorWindowState: TWindowState read FEditorWindowState write FEditorWindowState default wsNormal;
    property FileName: TFileName read FFileName write FFileName;
    property FileTemplate: TFileName read FFileTemplate write FFileTemplate;
    property Lines stored False; //replaced by custom filer methods in DefineProperties
    property OnAfterDialog: TabcModalDialogEvent read FOnAfterDialog write FOnAfterDialog;
    property OnBeforeDialog: TabcModalDialogEvent read FOnBeforeDialog write FOnBeforeDialog;
    property OnCancel: TNotifyEvent read FOnCancel write FOnCancel;
    property OnFileDropped: TabcFileDroppedEvent read FOnFileDropped write FOnFileDropped;
    property PopupOptions: TabcREPopupMenuOptions read FPopupOptions write FPopupOptions
      default [arpEdit,
               arpPrint, arpPageSetup,
               arpClear, arpCut, arpCopy, arpPaste, arpDelete, arpSelectAll,
               arpBold, arpItalic, arpUnderline,
               arpFont, arpBullets,
               arpFind, arpReplace];

    property PrintTitle: string read FPrintTitle write FPrintTitle;
    property PrintMargins: TabcPrintMarginsUnits read FPrintMargins write SetPrintMargins;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear; override;
    procedure Print(const Caption: string); override;
    procedure BeginEditing; virtual;
    procedure CancelChanges; virtual;
    procedure DoPrint; virtual;
    function Edit: Boolean; virtual;
    procedure FindNext;
    procedure LoadFromStrings(Value: TStrings); virtual;
    function LoadTemplate: Boolean; virtual;
    function OpenFile(FileName: string): Boolean; virtual;
    function Save: Boolean; virtual;

    procedure ShowFindDialog; virtual;
    function ShowFontDialog: Boolean; virtual;
    function ShowOpenDialog: Boolean; virtual;
    function ShowPageSetupDialog: Boolean; virtual;
    function ShowPrintDialog: Boolean; virtual;
    procedure ShowReplaceDialog; virtual;
    function ShowSaveDialog: Boolean; virtual;
    procedure PaintToRect(Dest, Format: HDC; DestR: TRect; ALayout: TTextLayout);
  end;

  TabcRichEdit = class(TabcCustomRichEdit)
  published
    property AcceptDroppedFiles;
    property Align;
    property Alignment;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property EditorOptions;
    property EditorTitle;
//    property EditorWidth; not implemented
    property EditorWindowState;
    property Enabled;
    property FileName;
    property FileTemplate;
    property Font;
    property HideSelection;
    property HideScrollBars;
    property ImeMode;
    property ImeName;
    property Lines;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PlainText;
    {property PopupMenu;}
    property PopupOptions;
    property PrintTitle;
    property PrintMargins;
    property ReadOnly;
    property ScrollBars;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property WantReturns;
    property WantTabs;
    property WordWrap;
    property OnAfterDialog;
    property OnBeforeDialog;
    property OnChange;
    property OnCancel;
    property OnClick;
{$IFDEF DELPHI5}
    property OnContextPopup;
{$ENDIF}        
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEndDock;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnStartDock;
    property OnEnter;
    property OnExit;
    property OnFileDropped;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResizeRequest;
    property OnSelectionChange;
    property OnStartDrag;
    property OnProtectChange;
    property OnSaveClipboard;
  end;

implementation

uses Clipbrd, Printers, ShellAPI, abcutilf, abcfrch, abcconst;

{ TabcCustomRichEdit }

constructor TabcCustomRichEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAcceptDroppedFiles := True;
  FEditorOptions := [areFilePrint, areFilePageSetup,
    areEditMenu, areSearchMenu, areFormatMenu, areEditToolbar, areFindToolbar,
    areFontToolbar, areFormatToolbar, areStatusBar, areCancelOnEsc];
  FEditorWidth := arwScreen;
  FEditorWindowState := wsNormal;
  FPopupMenu := CreatePopupMenu;
  PopupMenu := FPopupMenu;
  FPopupOptions := [arpEdit,
                    arpPrint, arpPageSetup,
                    arpClear, arpCut, arpCopy, arpPaste, arpDelete, arpSelectAll,
                    arpBold, arpItalic, arpUnderline,
                    arpFont, arpBullets,
                    arpFind, arpReplace];
  FPrintMargins := TabcPrintMarginsUnits.Create;
  FPrintMargins.OnChange := PrintMarginsChanged;
end;

destructor TabcCustomRichEdit.Destroy;
begin
  FFindDlg.Free;
  FReplaceDlg.Free;
  FDesignSaveRTF.Free;
  FreePopupMenu;
  FPrintMargins.Free;
  inherited Destroy;
end;

procedure TabcCustomRichEdit.Loaded;
begin
  inherited Loaded;
  Modified := False;
end;

procedure TabcCustomRichEdit.AdjustTextRect;
var
  r: TRect;
begin
  Perform(EM_GETRECT,0,LParam(@r));
  //small adjustment helps formatting for wordwrap
  r.Right := r.Right - 1;
  if not EqualRect(FSaveTextRect, r) then
  begin
    Perform(EM_SETRECT,0,LParam(@r));
    FSaveTextRect := r;
  end;
end;

procedure TabcCustomRichEdit.WMSize(var Message: TWMSize);
begin
  inherited;
  //fix bug causing text to redraw incorrectly when resizing
  AdjustTextRect;
end;

procedure TabcCustomRichEdit.AssignTo(Dest: TPersistent);
begin
  if Dest is TabcCustomRichEdit then
    with TabcCustomRichEdit(Dest) do
    begin
      AcceptDroppedFiles := Self.AcceptDroppedFiles;
      Alignment := Self.Alignment;
      Color := Self.Color;
      EditorOptions := Self.EditorOptions;
      EditorTitle := Self.EditorTitle;
      EditorWidth := Self.EditorWidth;
      FileName := Self.FileName;
      FileTemplate := Self.FileTemplate;
      Font := Self.Font;
      HelpContext := Self.HelpContext;
      HideScrollBars := Self.HideScrollBars;
      HideSelection := Self.HideSelection;
      Hint := Self.Hint;
      IMEMode := Self.IMEMode;
      IMEName := Self.IMEName;
      LoadFromStrings(Self.Lines);
      MaxLength := Self.MaxLength;
      PlainText := Self.PlainText;
      PopupOptions := Self.PopupOptions;
      PrintMargins := Self.PrintMargins;
      ReadOnly := Self.ReadOnly;
      Scrollbars := Self.Scrollbars;
      ShowHint := Self.ShowHint;
      WantReturns := Self.WantReturns;
      WantTabs := Self.WantTabs;
      WordWrap := Self.WordWrap;
    end
  else
    inherited;
end;

procedure TabcCustomRichEdit.DestroyWnd;
begin
  if csDesigning in ComponentState then
    if Lines.Count > 0 then
    begin
      if FDesignSaveRTF = nil then
        FDesignSaveRTF := TMemoryStream.Create
      else
        FDesignSaveRTF.Clear;
      WriteRichText(FDesignSaveRTF);
    end;
  inherited;
end;

procedure TabcCustomRichEdit.CreateWnd;
begin
  inherited;
  SetAcceptDroppedFiles(AcceptDroppedFiles);
  if (csDesigning in ComponentState) then
    if FDesignSaveRTF <> nil then
    begin
      BeginEditing;
      FDesignSaveRTF.Seek(0,0);
      ReadRichText(FDesignSaveRTF);
      FDesignSaveRTF.Free;
      FDesignSaveRTF := nil;
      Modified := False;
    end;
end;

procedure TabcCustomRichEdit.LoadFromStrings(Value: TStrings);
var
  strStream: TStringStream;
begin
  BeginEditing;
  strStream := TStringStream.Create('');
  try
    Value.SaveToStream(strStream);
    strStream.Seek(0,0);
    Lines.LoadFromStream(strStream);
  finally
    strStream.Free;
  end;
end;

{Filing}

procedure TabcCustomRichEdit.DefineProperties(Filer: TFiler);
begin
  inherited;
  Filer.DefineBinaryProperty('RichText', ReadRichText, WriteRichText, IsRichTextStored(Filer));
end;

function TabcCustomRichEdit.IsRichTextStored(Filer: TFiler): Boolean;
var
  Ancestor: TabcCustomRichEdit;
begin
  Ancestor := TabcCustomRichEdit(Filer.Ancestor);
  if Ancestor <> nil then
    Result := not Lines.Equals(Ancestor.Lines)
  else
    Result := (Lines.Count > 0);
end;

procedure TabcCustomRichEdit.ReadRichText(Stream: TStream);
var
  strStream: TStringStream;
  iSize: integer;
begin
  strStream := TStringStream.Create('');
  try
    Stream.Read(iSize, SizeOf(iSize));
    if iSize > 0 then
    begin
      strStream.CopyFrom(Stream, iSize);
      strStream.Seek(0,0);
      Lines.LoadFromStream(strStream);
    end;
  finally
    strStream.Free;
  end;
end;

procedure TabcCustomRichEdit.WriteRichText(Stream: TStream);
var
  strStream: TStringStream;
  iSize: integer;
begin
  strStream := TStringStream.Create('');
  try
    Lines.SaveToStream(strStream);
    strStream.Seek(0,0);
    iSize := strStream.Size;
    Stream.WriteBuffer(iSize, SizeOf(iSize));
    Stream.CopyFrom(strStream, iSize);
  finally
    strStream.Free;
  end;
end;

function TabcCustomRichEdit.Edit: Boolean;  {returns true if changed}
var
  Dialog: TabcsfRichEditDialog;
begin
  Dialog := TabcsfRichEditDialog(
    abcFormFactory.CreateFormOfClass(TabcsfRichEditDialog, Application));
  with Dialog do
  try
    {set properties to match this control}
    Self.AssignTo(Editor);
    ShowHint := Self.ShowHint;
    Editor.ParentShowHint := True;
    Editor.PopupOptions := Editor.PopupOptions - [arpEdit];
    Editor.ReadOnly := not CanModify;
    Editor.OnFileDropped := OnFileDropped;
    if FEditorTitle <> '' then
      Caption := FEditorTitle;
    WindowState := FEditorWindowState;
    SetEditorOptions(Self.EditorOptions);
    if Assigned(FOnBeforeDialog) then
      FOnBeforeDialog(Self, Dialog);
    ShowModal;
    if Assigned(FOnAfterDialog) then
      FOnAfterDialog(Self, Dialog);
    Result := (ModalResult = mrOk);
    if Result then
    begin
      {update selected properties of control}
//      EditorWidth := Editor.EditorWidth;
      FileName := Editor.FileName;
      LoadFromStrings(Editor.Lines);
      PrintMargins := Editor.PrintMargins;
    end;
  finally
    Free;
  end;
end;

function TabcCustomRichEdit.CreatePopupMenu: TPopupMenu;
begin
  FmiEdit := NewItem(sabcRichEditPopupMenuEdit, 0, False, True, FmiEditClick, 0, 'miEdit');
  FmiSep1 := NewItem('-', 0, False, False, nil, 0, 'miSep1');
  FmiClear := NewItem(sabcRichEditPopupMenuClear, 0, False, True, FmiClearClick, 0, 'miClear');
  FmiCancel := NewItem(sabcRichEditPopupMenuCancel, 0, False, True, FmiCancelClick, 0, 'miCancel');
  FmiLoadTemplate := NewItem(sabcRichEditPopupMenuLoadTemplate, 0, False, True, FmiLoadTemplateClick, 0, 'miLoadTemplate');
  FmiFileOpen := NewItem(sabcRichEditPopupMenuFileOpen, 0, False, True, FmiFileOpenClick, 0, 'miFileOpen');
  FmiFileSaveAs := NewItem(sabcRichEditPopupMenuFileSaveAs, 0, False, True, FmiFileSaveAsClick, 0, 'miFileSaveAs');
  FmiPrint := NewItem(sabcRichEditPopupMenuPrint, 0, False, True, FmiPrintClick, 0, 'miPrint');
  FmiPageSetup := NewItem(sabcRichEditPopupMenuPageSetup, 0, False, True, FmiPageSetupClick, 0, 'miPageSetup');
  FmiSep2 := NewItem('-', 0, False, False, nil, 0, 'miSep2');
  FmiUndo := NewItem(sabcRichEditPopupMenuUndo, ShortCut(Ord('Z'), [ssCtrl]), False, True, FmiUndoClick, 0, 'miUndo');
  FmiCut := NewItem(sabcRichEditPopupMenuCut, ShortCut(Ord('X'), [ssCtrl]), False, True, FmiCutClick, 0, 'miCut');
  FmiCopy := NewItem(sabcRichEditPopupMenuCopy, ShortCut(Ord('C'), [ssCtrl]), False, True, FmiCopyClick, 0, 'miCopy');
  FmiPaste := NewItem(sabcRichEditPopupMenuPaste, ShortCut(Ord('V'), [ssCtrl]), False, True, FmiPasteClick, 0, 'miPaste');
  FmiDelete := NewItem(sabcRichEditPopupMenuDelete, ShortCut(VK_DELETE, [ssCtrl]), False, True, FmiDeleteClick, 0, 'miDelete');
  FmiSelectAll := NewItem(sabcRichEditPopupMenuSelectAll, 0, False, True, FmiSelectAllClick, 0, 'miSelectAll');
  FmiSep3 := NewItem('-', 0, False, False, nil, 0, 'miSep3');
  FmiBold := NewItem(sabcRichEditPopupMenuBold, 0, False, True, FmiBoldClick, 0, 'miBold');
  FmiItalic := NewItem(sabcRichEditPopupMenuItalic, 0, False, True, FmiItalicClick, 0, 'miItalic');
  FmiUnderline := NewItem(sabcRichEditPopupMenuUnderline, 0, False, True, FmiUnderlineClick, 0, 'miUnderline');
  FmiSep4 := NewItem('-', 0, False, False, nil, 0, 'miSep4');
  FmiFont := NewItem(sabcRichEditPopupMenuFont, 0, False, True, FmiFontClick, 0, 'miFont');
  FmiBullets := NewItem(sabcRichEditPopupMenuBullet, 0, False, True, FmiBulletsClick, 0, 'miBullets');
  FmiSep5 := NewItem('-', 0, False, False, nil, 0, 'miSep5');
  FmiFind := NewItem(sabcRichEditPopupMenuFind, ShortCut(Ord('F'), [ssCtrl]), False, True, FmiFindClick, 0, 'miFind');
  FmiReplace := NewItem(sabcRichEditPopupMenuReplace, ShortCut(Ord('R'), [ssCtrl]), False, True, FmiReplaceClick, 0, 'miReplace');

  Result := NewPopupMenu(Self, Name + 'Popup', paLeft, True,
    [FmiEdit, FmiCancel, FmiSep1,
     FmiClear, FmiLoadTemplate, FmiFileOpen, FmiFileSaveAs,
     FmiPrint, FmiPageSetup, FmiSep2,
     FmiUndo, FmiCut, FmiCopy, FmiPaste, FmiDelete, FmiSelectAll, FmiSep3,
     FmiBold, FmiItalic, FmiUnderline, FmiSep4,
     FmiFont, FmiBullets, FmiSep5,
     FmiFind, FmiReplace]);
  Result.OnPopup := DoMenuPopup;
end;

procedure TabcCustomRichEdit.FreePopupMenu;
begin
  FmiEdit.Free;
  FmiCancel.Free;
  FmiSep1.Free;
  FmiClear.Free;
  FmiLoadTemplate.Free;
  FmiFileOpen.Free;
  FmiFileSaveAs.Free;
  FmiPrint.Free;
  FmiPageSetup.Free;
  FmiSep2.Free;
  FmiUndo.Free;
  FmiCut.Free;
  FmiCopy.Free;
  FmiPaste.Free;
  FmiDelete.Free;
  FmiSelectAll.Free;
  FmiSep3.Free;
  FmiBold.Free;
  FmiItalic.Free;
  FmiUnderline.Free;
  FmiSep4.Free;
  FmiFont.Free;
  FmiBullets.Free;
  FmiSep5.Free;
  FmiFind.Free;
  FmiReplace.Free;
  FPopupMenu.Free;
end;

procedure TabcCustomRichEdit.DoMenuPopup(Sender: TObject);
{set item visibility depending on options}
begin
  FmiEdit.Visible := (arpEdit in FPopupOptions);
  FmiEdit.Enabled := CanModify;
  FmiCancel.Visible := (arpCancel in FPopupOptions);
  FmiCancel.Enabled := CanModify;
  FmiSep1.Visible := (([arpEdit, arpCancel] * FPopupOptions) <> []);
  FmiClear.Visible := (arpClear in FPopupOptions);
  FmiClear.Enabled := CanModify;
  FmiLoadTemplate.Visible := (arpLoadTemplate in FPopupOptions);
  FmiLoadTemplate.Enabled := CanModify and (FileTemplate <> '');
  FmiFileOpen.Visible := (arpFileOpen in FPopupOptions);
  FmiFileOpen.Enabled := CanModify;
  FmiFileSaveAs.Visible := arpFileSaveAs in FPopupOptions;
  FmiPrint.Visible := arpPrint in FPopupOptions;
  FmiPageSetup.Visible := arpPageSetup in FPopupOptions;
  FmiSep2.Visible := (([arpClear, arpLoadTemplate, arpFileOpen, arpFileSaveAs,
    arpPrint, arpPageSetup] * FPopupOptions) <> []);
  FmiUndo.Visible := (arpUndo in FPopupOptions);
  FmiUndo.Enabled := CanModify and (Perform(EM_CANUNDO, 0, 0) <> 0);
  FmiCut.Visible := (arpCut in FPopupOptions);
  FmiCut.Enabled := CanModify and (SelLength > 0);
  FmiCopy.Visible := arpCopy in FPopupOptions;
  FmiCopy.Enabled := (SelLength > 0);
  FmiPaste.Visible := (arpPaste in FPopupOptions);
  FmiPaste.Enabled := CanModify and (Perform(EM_CANPASTE,0,0) <> 0);
  FmiDelete.Visible := (arpDelete in FPopupOptions);
  FmiDelete.Enabled := CanModify and (SelLength > 0);
  FmiSelectAll.Visible := arpSelectAll in FPopupOptions;
  FmiSep3.Visible := (([arpUndo, arpCut, arpCopy, arpPaste, arpDelete, arpSelectAll] * FPopupOptions) <> []);
  FmiBold.Visible := arpBold in FPopupOptions;
  FmiBold.Checked := fsBold in SelAttributes.Style;
  FmiBold.Enabled := CanModify;
  FmiItalic.Visible := arpItalic in FPopupOptions;
  FmiItalic.Checked := fsItalic in SelAttributes.Style;
  FmiItalic.Enabled := CanModify;
  FmiUnderline.Visible := arpUnderline in FPopupOptions;
  FmiUnderline.Checked := fsUnderline in SelAttributes.Style;
  FmiUnderline.Enabled := CanModify;
  FmiSep4.Visible := (([arpBold, arpItalic, arpUnderline] * FPopupOptions) <> []);
  FmiBullets.Visible := arpBullets in FPopupOptions;
  FmiBullets.Checked := (Paragraph.Numbering = nsBullet);
  FmiBullets.Enabled := CanModify;
  FmiFont.Visible := arpFont in FPopupOptions;
  FmiFont.Enabled := CanModify;
  FmiSep5.Visible := (([arpBullets, arpFont] * FPopupOptions) <> []);
  FmiFind.Visible := arpFind in FPopupOptions;
  FmiReplace.Visible := arpReplace in FPopupOptions;
  FmiReplace.Enabled := CanModify;
end;

function TabcCustomRichEdit.CanModify: Boolean;
begin
  Result := not ReadOnly;
end;

procedure TabcCustomRichEdit.BeginEditing;
begin
  Perform(EM_SETMODIFY, Integer(True), 0);
end;

procedure TabcCustomRichEdit.FmiEditClick(Sender: TObject);
begin
  Edit;
end;

procedure TabcCustomRichEdit.FmiCancelClick(Sender: TObject);
begin
  CancelChanges;
end;

procedure TabcCustomRichEdit.FmiClearClick(Sender: TObject);
begin
  BeginEditing;
  Clear;
end;

procedure TabcCustomRichEdit.FmiLoadTemplateClick(Sender: TObject);
begin
  LoadTemplate;
end;

procedure TabcCustomRichEdit.FmiFileOpenClick(Sender: TObject);
begin
  ShowOpenDialog;
end;

procedure TabcCustomRichEdit.FmiFileSaveAsClick(Sender: TObject);
begin
  ShowSaveDialog;
end;

procedure TabcCustomRichEdit.FmiPrintClick(Sender: TObject);
begin
  ShowPrintDialog;
end;

procedure TabcCustomRichEdit.FmiPageSetupClick(Sender: TObject);
begin
  ShowPageSetupDialog;
end;

procedure TabcCustomRichEdit.FmiUndoClick(Sender: TObject);
begin
  Perform(EM_UNDO, 0, 0);
end;

procedure TabcCustomRichEdit.FmiCutClick(Sender: TObject);
begin
  CutToClipboard;
end;

procedure TabcCustomRichEdit.FmiCopyClick(Sender: TObject);
begin
  CopyToClipboard;
end;

procedure TabcCustomRichEdit.FmiPasteClick(Sender: TObject);
begin
  PasteFromClipboard;
end;

procedure TabcCustomRichEdit.FmiDeleteClick(Sender: TObject);
begin
  BeginEditing;
  ClearSelection;
end;

procedure TabcCustomRichEdit.FmiSelectAllClick(Sender: TObject);
begin
  SelectAll;
end;

procedure TabcCustomRichEdit.FmiBoldClick(Sender: TObject);
begin
  BeginEditing;
  with SelAttributes do
    if (fsBold in Style) then
      Style := Style - [fsBold]
    else
      Style := Style + [fsBold];
end;

procedure TabcCustomRichEdit.FmiItalicClick(Sender: TObject);
begin
  BeginEditing;
  with SelAttributes do
    if (fsItalic in Style) then
      Style := Style - [fsItalic]
    else
      Style := Style + [fsItalic];
end;

procedure TabcCustomRichEdit.FmiUnderlineClick(Sender: TObject);
begin
  BeginEditing;
  with SelAttributes do
    if (fsUnderline in Style) then
      Style := Style - [fsUnderline]
    else
      Style := Style + [fsUnderline];
end;

procedure TabcCustomRichEdit.FmiFontClick(Sender: TObject);
begin
  ShowFontDialog;
end;

procedure TabcCustomRichEdit.FmiBulletsClick(Sender: TObject);
begin
  BeginEditing;
  with Paragraph do
    if Numbering = nsNone then
      Numbering := nsBullet
    else
      Numbering := nsNone;
end;

procedure TabcCustomRichEdit.FmiFindClick(Sender: TObject);
begin
  ShowFindDialog;
end;

procedure TabcCustomRichEdit.FmiReplaceClick(Sender: TObject);
begin
  ShowReplaceDialog;
end;

procedure TabcCustomRichEdit.CancelChanges;
begin
  if Assigned(FOnCancel) then
    FOnCancel(Self)
  else
    {try to revert to saved file contents, if any}
    if (FileName <> '') and FileExists(FileName) then
      Lines.LoadFromFile(FileName);
end;

procedure TabcCustomRichEdit.ShowFindDialog;
begin
  if FFindDlg = nil then
    FFindDlg := TFindDialog.Create(Self);
  with FFindDlg do
  begin
    FindText := SelText;
    FFindText := FindText;
    FFound := False;
    OnFind := FindDialogFind;
    Execute;
  end;
end;

function TabcCustomRichEdit.ShowFontDialog: Boolean;
begin
  Result := False;
  with TFontDialog.Create(Self) do
  try
    Device := fdBoth;
    Font.Assign(SelAttributes);
    if Execute then
    begin
      BeginEditing;
      SelAttributes.Assign(Font);
      Result := True;
    end;
  finally
    Free;
  end;
  SetFocus;
end;

procedure TabcCustomRichEdit.ShowReplaceDialog;
begin
  if FReplaceDlg = nil then
    FReplaceDlg := TReplaceDialog.Create(Self);
  with FReplaceDlg do
  begin
    FindText := SelText;
    FFindText := FindText;
    FFound := False;
    OnFind := FindDialogFind;
    OnReplace := ReplaceDialogReplace;
    Execute;
  end;
end;

function TabcCustomRichEdit.ShowOpenDialog: Boolean;
begin
  Result := False;
  with TOpenDialog.Create(nil) do
  try
    DefaultExt := 'rtf';
    if Self.FileName <> '' then
    begin
      FileName := ExtractFileName(Self.FileName);
      InitialDir := ExtractFilePath(Self.FileName);
    end;
    Filter := sabcRichEditFileDialogFilter;
    if Execute then
      Result := OpenFile(FileName);
  finally
    Free;
  end;
  SetFocus;
end;

function TabcCustomRichEdit.OpenFile(FileName: string): Boolean;
begin
  Result := False;
  if FileExists(FileName) then
  begin
    BeginEditing;
    Lines.LoadFromFile(FileName);
    Self.FileName := FileName;
    Result := True;
  end;
end;

function TabcCustomRichEdit.ShowPageSetupDialog: Boolean;
begin
  Result := False;
  with TabcPageSetupDialog.Create(nil) do
  try
    Margins := FPrintMargins;
    Units := FPrintMargins.Units;
    if Execute then
    begin
      BeginEditing;
      FPrintMargins.Assign(Margins);
      Result := True;
    end;
  finally
    Free;
  end;
  SetFocus;
end;

function TabcCustomRichEdit.ShowPrintDialog: Boolean;
begin
  Result := False;
  with TPrintDialog.Create(nil) do
  try
    if Execute then
    begin
      DoPrint;
      Result := True;
    end;
  finally
    Free;
  end;
  SetFocus;
end;

function TabcCustomRichEdit.ShowSaveDialog: Boolean;
begin
  result := False;
  with TSaveDialog.Create(nil) do
  try
    DefaultExt := 'rtf';
    if Self.FileName <> '' then
    begin
      FileName := ExtractFileName(Self.FileName);
      InitialDir := ExtractFilePath(Self.FileName);
    end;
    Filter := sabcRichEditFileDialogFilter;
    if Execute then
    begin
      Lines.SaveToFile(FileName);
      Self.FileName := FileName;
      Result := True;
    end;
  finally
    Free;
  end;
  SetFocus;
end;

function TabcCustomRichEdit.Save: Boolean;
begin
  if FileName <> '' then
  begin
    Lines.SaveToFile(FileName);
    Result := True;
  end
  else
    Result := ShowSaveDialog;
end;

function TabcCustomRichEdit.LoadTemplate: Boolean;
begin
  if not FileExists(FileTemplate) then
    Result := False
  else
  begin
    BeginEditing;
    Lines.LoadFromFile(FileTemplate);
    FileName := '';
    Result := True;
  end;
end;

procedure TabcCustomRichEdit.FindDialogFind(Sender: TObject);
begin
  FSearchTypes := [];
  if frMatchCase in TFindDialog(Sender).Options then
    FSearchTypes := FSearchTypes + [stMatchCase];
  if frWholeWord in TFindDialog(Sender).Options then
    FSearchTypes := FSearchTypes + [stWholeWord];
  if FFindText <> TFindDialog(Sender).FindText then FFound := False;
  if FFound then begin
    FindNext;
    if not FFound then
      MessageDlg(Format(sabcRichEditCannotFind, [FFindText]), mtInformation, [mbOK], 0);
  end
  else begin
    FFindText := TFindDialog(Sender).FindText;
    Find;
    if not FFound then
      MessageDlg(Format(sabcRichEditCannotFind, [FFindText]), mtInformation, [mbOK], 0);
  end;
  ActivateFormWindow(Sender);
end;

procedure TabcCustomRichEdit.ReplaceDialogReplace(Sender: TObject);
begin
  if FFindText <> FReplaceDlg.FindText then FFound := False;
  FReplaceText := FReplaceDlg.ReplaceText;
  if (frReplaceAll) in FReplaceDlg.Options then begin
    FFindText := FReplaceDlg.FindText;
    FReplaceText := FReplaceDlg.ReplaceText;
    FSearchTypes := [];
    if frMatchCase in FReplaceDlg.Options then
      FSearchTypes := FSearchTypes + [stMatchCase];
    if frWholeWord in FReplaceDlg.Options then
      FSearchTypes := FSearchTypes + [stWholeWord];
    ReplaceAll;
  end
  else if FFound then begin
    ReplaceSelection;
    FindNext;
  end;
  ActivateFormWindow(Sender);
end;

procedure TabcCustomRichEdit.Find;
begin
  FStartPos := 0;
  FindFromPos;
end;

procedure TabcCustomRichEdit.FindFromPos;
var
  FindPos: Integer;
begin
  FFound := False;
  if FFindText = '' then
    Exit;

  FindPos := FindText(FFindText, FStartPos, GetTextLen - FStartPos,  FSearchTypes);

  if FindPos <> -1 then begin
    FFound := True;
    {highlight found text}
    SelStart := FindPos;
    SelLength := Length(FFindText);
  end;
end;

procedure TabcCustomRichEdit.FindNext;
begin
  FStartPos := SelStart + SelLength;
  FindFromPos;
end;

procedure TabcCustomRichEdit.ReplaceSelection;
begin
  abcStrToClip(FReplaceText);
  PasteFromClipboard;
end;

procedure TabcCustomRichEdit.ReplaceAll;
begin
  FStartPos := 0;
  FindFromPos;
  while FFound do begin
    ReplaceSelection;
    FindNext;
  end;
end;

procedure TabcCustomRichEdit.ActivateFormWindow(Sender: TObject);
{
 This procedure activates the form and the window while the
 Find or Replace dialog box is open.
 Assumes the Sender is either a TFindDialog or a TReplaceDialog.
}
var
  TargetLine, TopLine, LinesHalfHeight: Integer;
  parForm: TCustomForm;
begin
  parForm := GetParentForm(Self);
  if (parForm = nil) or not (parForm is TForm) then Exit;
  if TForm(parForm).FormStyle = fsMDIChild then
    BringWindowToTop(Application.MainForm.Handle);
  BringWindowToTop(Handle);
  {Scroll memo selection into center of view}
  if SelStart <> 0 then begin
    LinesHalfHeight := ((Height div (- Font.Height)) + 1) div 2;
    TargetLine := Perform(EM_LINEFROMCHAR, SelStart, 0);
    TopLine := Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
    Perform(EM_LINESCROLL,0,TargetLine - TopLine - LinesHalfHeight);
  end;
  SetFocus;

  if Sender is TFindDialog then
    SetWindowPos(
      TFindDialog(Sender).Handle,
      HWND_TOP,
      0,0,0,0,
      SWP_NOACTIVATE + SWP_NOMOVE + SWP_NOSIZE)
  else if Sender is TReplaceDialog then
    SetWindowPos(
      TReplaceDialog(Sender).Handle,
      HWND_TOP,
      0,0,0,0,
      SWP_NOACTIVATE + SWP_NOMOVE + SWP_NOSIZE);
end;

{ Printing Support }

procedure TabcCustomRichEdit.SetPrintMargins(Value: TabcPrintMarginsUnits);
begin
  FPrintMargins.Assign(Value);
end;

procedure TabcCustomRichEdit.PrintMarginsChanged(Sender: TObject);
begin
 // not implemented
end;

procedure TabcCustomRichEdit.SetEditorWidth(Value: TabcREEditWidth);
begin
 // not implemented
end;

procedure TabcCustomRichEdit.DoPrint;
begin
  Print(FPrintTitle);
end;

procedure TabcCustomRichEdit.Print(const Caption: string);
begin
  PageRect := FPrintMargins.PrinterClientRect;
  inherited;
end;

procedure TabcCustomRichEdit.SetAcceptDroppedFiles(Value: Boolean);
begin
  FAcceptDroppedFiles := Value;
  if HandleAllocated then DragAcceptFiles(Handle, Value);
end;

procedure TabcCustomRichEdit.WMDropFiles(var Msg: TWMDropFiles);
var
  CFileName: array[0..MAX_PATH] of Char;
  sFileName: string;
  Accept: Boolean;
begin
  try
    if AcceptDroppedFiles and (DragQueryFile(Msg.Drop, 0, CFileName, MAX_PATH) > 0) then
    begin
      Accept := True;
      sFileName := CFileName;
      if Assigned(FOnFileDropped) then
        FOnFileDropped(Self, sFileName, Accept);
      if Accept then
        OpenFile(sFileName);
      Msg.Result := 0;
    end;
  finally
    DragFinish(Msg.Drop);
  end;
end;

procedure TabcCustomRichEdit.PaintToRect(Dest, Format: HDC; DestR: TRect; ALayout: TTextLayout);
var
  Range: TFormatRange;
  LogX, LogY, OldMapMode: Integer;
  OldColor: TColorRef;
  DestRTwips: TRect;
begin
  Perform(EM_FORMATRANGE, 0, 0);
  OldMapMode := SetMapMode(Dest, MM_TEXT);
  OldColor := SetBkColor(Dest, ColorToRGB(Color));
  FillChar(Range, SizeOf(TFormatRange), 0);
  LogX := GetDeviceCaps(Dest, LOGPIXELSX);
  LogY := GetDeviceCaps(Dest, LOGPIXELSY);
  with Range do
  begin
    hdc := Dest;
    hdcTarget := Format;
    rc.left := DestR.Left * 1440 div LogX;
    rc.top := DestR.Top * 1440 div LogY;
    rc.right := DestR.Right * 1440 div LogX;
    rc.bottom := DestR.Bottom * 1440 div LogY;
    DestRTwips := rc;
    rcPage := rc;
    chrg.cpMin := -1;
    chrg.cpMax := -1;
  end;
  if ALayout <> tlTop then
  begin
    Perform(EM_FORMATRANGE, 0, Longint(@Range));
    with Range do
    if ALayout = tlBottom then
      OffsetRect(rc, 0, DestRTwips.Bottom - rc.Bottom)
    else
      OffsetRect(rc, 0, (DestRTwips.Bottom - rc.Bottom) div 2);
  end;
  Perform(EM_FORMATRANGE, 0, 0);
  Perform(EM_FORMATRANGE, 1, Longint(@Range));
  Perform(EM_FORMATRANGE, 0, 0);
  SetMapMode(Dest, OldMapMode);
  SetBkColor(Dest, OldColor);
end;

procedure TabcCustomRichEdit.WMPrintClient(var Message: TWMPaint);
begin
  PaintClientTextToDC(Message.DC);
end;

procedure TabcCustomRichEdit.WMPaint(var Message: TWMPaint);
begin
  if csPaintCopy in ControlState then
    PaintClientTextToDC(Message.DC)
  else
    inherited;
end;

procedure TabcCustomRichEdit.PaintClientTextToDC(DC: HDC);
//text rendering when painting to another DC
var
  Range: TFormatRange;
  LogX, LogY, OldMapMode: Integer;
  OldColor: TColorRef;
  TopIndex: TPoint;
begin
  OldMapMode := SetMapMode(DC, MM_TEXT);
  OldColor := SetBkColor(DC, ColorToRGB(Color));
  try
    FillChar(Range, SizeOf(TFormatRange), 0);
    LogX := GetDeviceCaps(DC, LOGPIXELSX);
    LogY := GetDeviceCaps(DC, LOGPIXELSY);
    with Range do
    begin
      hdc := DC;
      hdcTarget := DC;

      AdjustTextRect;

      rc.left := FSaveTextRect.Left * 1440 div LogX;
      rc.top := FSaveTextRect.Top * 1440 div LogY;
      rc.right := FSaveTextRect.Right * 1440 div LogX;
      rc.bottom := (FSaveTextRect.Bottom + $40) * 1440 div LogY;
      rcPage := rc;

      TopIndex.X:= 1;
      TopIndex.Y:= 1;
      chrg.cpMin := Perform(EM_CHARFROMPOS, 0, Longint(@TopIndex));
      chrg.cpMax := -1;

    end;
    Perform(EM_FORMATRANGE, 0, 0);
    Perform(EM_FORMATRANGE, 1, Longint(@Range));
    Perform(EM_FORMATRANGE, 0, 0);
  finally
    SetMapMode(DC, OldMapMode);
    SetBkColor(DC, OldColor);
  end;
end;

procedure TabcCustomRichEdit.Clear;
begin
  BeginEditing;
  inherited;
  Modified := True;
end;

end.
