unit abcscrol;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{ custom version of TScrollBox with unpublished properties }
{ removing dock site property for TabcDBEditWin }

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, StdCtrls;

type

{ TabcCustomScrollBox }

  TabcCustomScrollBox = class(TScrollingWinControl)
  private
    FBorderStyle: TBorderStyle;
    FOnScrollH: TScrollEvent;
    FOnScrollV: TScrollEvent;
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMNCHitTest(var Message: TMessage); message WM_NCHITTEST;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
  protected
    procedure CalcAutoRange;
    procedure CreateParams(var Params: TCreateParams); override;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property OnScrollH: TScrollEvent read FOnScrollH write FOnScrollH;
    property OnScrollV: TScrollEvent read FOnScrollV write FOnScrollV;
  public
    constructor Create(AOwner: TComponent); override;
  published
(*  Publishable in descendants
    property Align;
    property Anchors;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle default bsSingle;
    property BiDiMode;
    property Constraints;
    property DockSite;
    property DragKind;
    property ParentBiDiMode;
    property AutoScroll;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Color nodefault;
    property Ctl3D;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnCanResize;
    property OnConstrainedResize;
    property OnDockDrop;
    property OnDockOver;
    property OnEndDock;
    property OnGetSiteInfo;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnStartDock;
    property OnUnDock;
    property OnStartDrag;
*)
  end;


implementation

type
  TScrollingWinControlAccess = class(TScrollingWinControl);

{ TabcCustomScrollBox }

constructor TabcCustomScrollBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
    csSetCaption, csDoubleClicks];
  Width := 185;
  Height := 41;
  FBorderStyle := bsSingle;
end;

procedure TabcCustomScrollBox.CreateParams(var Params: TCreateParams);
const
  BorderStyles: array[TBorderStyle] of DWORD = (0, WS_BORDER);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or BorderStyles[FBorderStyle];
    if NewStyleControls and Ctl3D and (FBorderStyle = bsSingle) then
    begin
      Style := Style and not WS_BORDER;
      ExStyle := ExStyle or WS_EX_CLIENTEDGE;
    end;
  end;
end;

procedure TabcCustomScrollBox.SetBorderStyle(Value: TBorderStyle);
begin
  if Value <> FBorderStyle then
  begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

procedure TabcCustomScrollBox.WMHScroll(var Message: TWMHScroll);
var
  ScrollPos: Integer;
begin
  if Assigned(FOnScrollH) then
    with Message do begin
      ScrollPos := Pos;
      FOnScrollH(Self, TScrollCode(ScrollCode), ScrollPos);
      Pos := ScrollPos;
    end;
  inherited;
end;

procedure TabcCustomScrollBox.WMVScroll(var Message: TWMVScroll);
var
  ScrollPos: Integer;
begin
  if Assigned(FOnScrollV) then
    with Message do begin
      ScrollPos := Pos;
      FOnScrollV(Self, TScrollCode(ScrollCode), ScrollPos);
      Pos := ScrollPos;
    end;
  inherited;
end;

procedure TabcCustomScrollBox.WMSize(var Message: TWMSize);
begin
  inherited;
  CalcAutoRange;
end;

procedure TabcCustomScrollBox.CalcAutoRange;
begin
  if AutoScroll then
  begin
    DisableAutoRange;
    EnableAutoRange;
  end;
end;

procedure TabcCustomScrollBox.WMNCHitTest(var Message: TMessage);
begin
  DefaultHandler(Message);
end;

procedure TabcCustomScrollBox.CMCtl3DChanged(var Message: TMessage);
begin
  if NewStyleControls and (FBorderStyle = bsSingle) then RecreateWnd;
  inherited;
end;

end.
