unit abcshape;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  ExtCtrls, Menus, Forms;

type

  { TabcCustomLine }

  TabcArrows = (arNone, arStart, arEnd, arBoth);
  TabcArrowStyle = (ahLine, ahBox, ahSolid);
  TabcArrowSize = 2..50;
  TabcDirection = (diHorizontal, diVertical, diToptoBottom, diBottomtoTop);

  TabcCustomLine = class(TGraphicControl)
  private
    FArrows: TabcArrows;
    FArrowSize: TabcArrowSize;
    FArrowStyle: TabcArrowStyle;
    FClipMouse: Boolean;
    FColor: TColor;
    FDirection: TabcDirection;
    FDrawPts: array[0..5] of TPoint;
    FLineStyle: TPenStyle;
    FLineWidth: integer;
    FMouseClipRgn: hRgn;
    FSinAngle, FCosAngle: Double;
    procedure SetArrows(Value: TabcArrows);
    procedure SetArrowSize(Value: TabcArrowSize);
    procedure SetArrowStyle(Value: TabcArrowStyle);
    procedure SetClipMouse(Value: Boolean);
    procedure SetColor(Value: TColor);
    procedure SetDirection(Value: TabcDirection);
    procedure SetLineStyle(Value: TPenStyle);
    procedure SetLineWidth(Value: integer);
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure WMMButtonDown(var Message: TWMMButtonDown); message WM_MBUTTONDOWN;
    procedure WMMButtonDblClk(var Message: TWMMButtonDblClk); message WM_MBUTTONDBLCLK;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMRButtonDblClk(var Message: TWMRButtonDblClk); message WM_RBUTTONDBLCLK;
    procedure WMRButtonDown(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
  protected
    function GetMouseClipRgn: hRgn; dynamic;
    procedure UpdateDrawMovePoints; dynamic;
    procedure Paint; override;
    procedure Loaded; override;
    property Arrows: TabcArrows read FArrows write SetArrows default arBoth;
    property ArrowSize: TabcArrowSize read FArrowSize write SetArrowSize default 10;
    property ArrowStyle: TabcArrowStyle read FArrowStyle write SetArrowStyle default ahSolid;
    property ClipMouse: Boolean read FClipMouse write SetClipMouse default False;
    property Color: TColor read FColor write SetColor default clBlack;
    property Direction: TabcDirection read FDirection write SetDirection default diHorizontal;
    property LineStyle: TPenStyle read FLineStyle write SetLineStyle default psSolid;
    property LineWidth: integer read FLineWidth write SetLineWidth default 1;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  end;

  { TabcLine }
  TabcLine = class(TabcCustomLine)
  published
    property ArrowStyle;
    property Arrows;
    property ArrowSize;
    property ClipMouse;
    property Color;
    property Direction;
    property LineStyle;
    property LineWidth;
    {publish inherited properties}
    property Action;
    property Anchors;
    property Constraints;
    property DragCursor;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TabcShape }

  TabcCornerWidth = 0..MaxInt;
  TabcVerticalAlign = (vaTop, vaCenter, vaBottom);

  TabcShape = class(TShape)
  private
    FAlignment: TAlignment;
    FAlignVertical: TabcVerticalAlign;
    FBevelInner: TPanelBevel;
    FBevelOuter: TPanelBevel;
    FBevelWidth: TBevelWidth;
    FBorderWidth: TBorderWidth;
    FColorHighlight: TColor;
    FColorShadow: TColor;
    FCornerWidth: TabcCornerWidth;
    FWordWrap: Boolean;
    procedure SetAlignment(Value: TAlignment);
    procedure SetAlignVertical(Value: TabcVerticalAlign);
    procedure SetBevelInner(Value: TPanelBevel);
    procedure SetBevelOuter(Value: TPanelBevel);
    procedure SetBevelWidth(Value: TBevelWidth);
    procedure SetBorderWidth(Value: TBorderWidth);
    procedure SetColorHighlight(Value: TColor);
    procedure SetColorShadow(Value: TColor);
    procedure SetCornerWidth(Value: TabcCornerWidth);
    procedure SetWordWrap(Value: Boolean);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property Align;
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property AlignVertical: TabcVerticalAlign read FAlignVertical write SetAlignVertical default vaCenter;
    property BevelInner: TPanelBevel read FBevelInner write SetBevelInner default bvLowered;
    property BevelOuter: TPanelBevel read FBevelOuter write SetBevelOuter default bvRaised;
    property BevelWidth: TBevelWidth read FBevelWidth write SetBevelWidth default 1;
    property BorderWidth: TBorderWidth read FBorderWidth write SetBorderWidth default 0;
    property Caption;
    property ColorHighlight: TColor read FColorHighlight write SetColorHighlight default clBtnHighlight;
    property ColorShadow: TColor read FColorShadow write SetColorShadow default clBtnShadow;
    property CornerWidth: TabcCornerWidth read FCornerWidth write SetCornerWidth;
    property Enabled;
    property Font;
    property Hint;
    property ParentFont;
    property PopupMenu;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default True;
    property OnResize;
    property OnClick;
    property OnDblClick;
  end;

  { TabcFrame }

  TabcFrame = class(TGraphicControl)
  private
    FBevelInner: TPanelBevel;
    FBevelOuter: TPanelBevel;
    FBevelWidth: TBevelWidth;
    FBorderWidth: TBorderWidth;
    FBorderStyle: TBorderStyle;
    FColorHighlight: TColor;
    FColorShadow: TColor;
    FCornerWidth: TabcCornerWidth;
    procedure SetBevelInner(Value: TPanelBevel);
    procedure SetBevelOuter(Value: TPanelBevel);
    procedure SetBevelWidth(Value: TBevelWidth);
    procedure SetBorderWidth(Value: TBorderWidth);
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetColorHighlight(Value: TColor);
    procedure SetColorShadow(Value: TColor);
    procedure SetCornerWidth(Value: TabcCornerWidth);
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property Anchors;
    property BiDiMode;
    property Constraints;
    property ParentBiDiMode;
    property Align;
    property BevelInner: TPanelBevel read FBevelInner write SetBevelInner default bvLowered;
    property BevelOuter: TPanelBevel read FBevelOuter write SetBevelOuter default bvRaised;
    property BevelWidth: TBevelWidth read FBevelWidth write SetBevelWidth default 1;
    property BorderWidth: TBorderWidth read FBorderWidth write SetBorderWidth default 0;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsNone;
    property Color default clBtnFace;
    property ColorHighlight: TColor read FColorHighlight write SetColorHighlight default clBtnHighlight;
    property ColorShadow: TColor read FColorShadow write SetColorShadow default clBtnShadow;
    property CornerWidth: TabcCornerWidth read FCornerWidth write SetCornerWidth default 20;
    property Enabled;
    property Hint;
    property ParentColor default False;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
  end;

implementation

uses abcutilg;

{ TabcCustomLine }

constructor TabcCustomLine.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FArrowStyle := ahSolid;
  FArrows := arBoth;
  FArrowSize := 10;
  FColor := clBlack;
  FDirection := diHorizontal;
  FLineStyle := psSolid;
  FLineWidth := 1;
  Width := 100;
  Height := 20;
end;

destructor TabcCustomLine.Destroy;
begin
  DeleteObject(FMouseClipRgn);
  inherited Destroy;
end;

function TabcCustomLine.GetMouseClipRgn: hRgn;
var
  Mid: Integer;
  RgnPoints: array[0..3] of TPoint;
  ArrowRgn: HRgn;
  dX, dY, dW: Integer;
begin
  Result := 0;
  {get line region adjusted for LineWidth and angle}
  dW := (LineWidth + 1) div 2;
  case FDirection of
    diHorizontal:
      begin
        Mid := Height div 2;
        Result := CreateRectRgn(0, Mid - dW, Width, Mid + dW);
      end;
    diVertical:
      begin
        Mid := Width div 2;
        Result := CreateRectRgn(Mid - dW, 0, Mid + dW, Height);
      end;
    diToptoBottom:
      begin
        dX := Round(dw * FSinAngle);
        dY := Round(dw * FCosAngle);
        RgnPoints[0] := Point(dX, -dY);
        RgnPoints[1] := Point(Width + dX, Height - dY);
        RgnPoints[2] := Point(Width - dX, Height + dY);
        RgnPoints[3] := Point(-dX, dY);
        Result := CreatePolygonRgn(RgnPoints, 4, ALTERNATE);
      end;
    diBottomtoTop:
      begin
        dX := Round(dw * FSinAngle);
        dY := Round(dw * FCosAngle);
        RgnPoints[0] := Point(- dX, Height - dY);
        RgnPoints[1] := Point(Width - dX, - dY);
        RgnPoints[2] := Point(Width + dX, dY);
        RgnPoints[3] := Point(dX, Height + dY);
        Result := CreatePolygonRgn(RgnPoints, 4, ALTERNATE);
      end;
  end;
  {Arrow regions}
  if FArrows in [arStart, arBoth] then begin
    RgnPoints[0] := FDrawPts[0];
    RgnPoints[1] := FDrawPts[2];
    RgnPoints[2] := FDrawPts[3];
    ArrowRgn := CreatePolygonRgn(RgnPoints, 3, ALTERNATE);
    CombineRgn(Result, Result, ArrowRgn, RGN_OR);
    DeleteObject(ArrowRgn);
  end;
  if FArrows in [arEnd, arBoth] then begin
    RgnPoints[0] := FDrawPts[1];
    RgnPoints[1] := FDrawPts[4];
    RgnPoints[2] := FDrawPts[5];
    ArrowRgn := CreatePolygonRgn(RgnPoints, 3, ALTERNATE);
    CombineRgn(Result, Result, ArrowRgn, RGN_OR);
    DeleteObject(ArrowRgn);
  end;
end;

procedure TabcCustomLine.Loaded;
begin
  inherited Loaded;
  UpdateDrawMovePoints;
end;

procedure TabcCustomLine.SetArrows(Value: TabcArrows);
begin
  if Value <> FArrows then begin
    FArrows := Value;
    UpdateDrawMovePoints;
  end;
end;

procedure TabcCustomLine.SetArrowSize(Value: TabcArrowSize);
begin
  if Value <> FArrowSize then begin
    FArrowSize := Value;
    UpdateDrawMovePoints;
  end;
end;

procedure TabcCustomLine.SetArrowStyle(Value: TabcArrowStyle);
begin
  if Value <> FArrowStyle then begin
    FArrowStyle := Value;
    Invalidate;
  end;
end;

procedure TabcCustomLine.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  UpdateDrawMovePoints;
end;

procedure TabcCustomLine.SetClipMouse(Value: Boolean);
begin
  if Value <> FClipMouse then begin
    FClipMouse := Value;
    if Value then begin
      DeleteObject(FMouseClipRgn);
      FMouseClipRgn := GetMouseClipRgn;
    end;
  end;
end;

procedure TabcCustomLine.SetColor(Value: TColor);
begin
  if Value <> FColor then begin
    FColor := Value;
    Canvas.Pen.Color := Value;
    Invalidate;
  end;
end;

procedure TabcCustomLine.SetDirection(Value: TabcDirection);
begin
  if Value <> FDirection then begin
    FDirection := Value;
    UpdateDrawMovePoints;
  end;
end;

procedure TabcCustomLine.SetLineStyle(Value: TPenStyle);
begin
  if Value <> FLineStyle then begin
    FLineStyle := Value;
    Invalidate;
  end;
end;

procedure TabcCustomLine.SetLineWidth(Value: integer);
begin
  if Value <> FLineWidth then begin
    FLineWidth := Value;
    Canvas.Pen.Width := Value;
    UpdateDrawMovePoints;
  end;
end;

procedure TabcCustomLine.Paint;
begin
  with Canvas do begin
    {Draw Line}
    Brush.Style := bsClear;
    SetBkMode(Handle, TRANSPARENT);
    Pen.Style := FLineStyle;
    MoveTo(FDrawPts[0].X, FDrawPts[0].Y);
    LineTo(FDrawPts[1].X, FDrawPts[1].Y);
    {Draw Arrows}
    Pen.Style := psSolid;
    Brush.Color := Pen.Color;
    Brush.Style := bsSolid;
    if (FArrows in [arStart, arBoth]) then
      case FArrowStyle of
        ahLine: PolyLine([FDrawPts[2], FDrawPts[0], FDrawPts[3]]);
        ahBox: PolyLine([FDrawPts[0], FDrawPts[2], FDrawPts[3], FDrawPts[0]]);
        ahSolid: Polygon([FDrawPts[0], FDrawPts[2], FDrawPts[3]]);
      end;
    if (FArrows in [arEnd, arBoth]) then
      case FArrowStyle of
        ahLine: PolyLine([FDrawPts[4], FDrawPts[1], FDrawPts[5]]);
        ahBox: PolyLine([FDrawPts[1], FDrawPts[4], FDrawPts[5], FDrawPts[1]]);
        ahSolid: Polygon([FDrawPts[1], FDrawPts[4], FDrawPts[5]]);
      end;
  end;
end;

procedure TabcCustomLine.UpdateDrawMovePoints;
{
 Recalculates Angles, Drawing points for line and arrows, and mouse clip region
}
var
  Angle: Double;
  CurrCentre: TPoint;

  procedure RotateDrawPoint(Centre: TPoint; var Point:TPoint);
  var
    Rotated: TPoint;
    AdjustSin: Double;
  begin
    Point.X := Point.X - Centre.X;
    Point.Y := Point.Y - Centre.Y;
    if FDirection = diBottomtoTop then
      AdjustSin := - FSinAngle
    else
      AdjustSin := FSinAngle;
    Rotated.X := Round(Point.X * FCosAngle - Point.Y * AdjustSin);
    Rotated.Y := Round(Point.X * AdjustSin + Point.Y * FCosAngle);
    Rotated.X := Rotated.X + Centre.X;
    Rotated.Y := Rotated.Y + Centre.Y;
    Point := Rotated;
  end;

  procedure SetDiagonalArrowPoints;
  begin
    with ClientRect do begin
      FDrawPts[2] := Point(Left + FArrowSize, Top + (Height - FArrowSize) div 2);
      FDrawPts[3] := FDrawPts[2];
      FDrawPts[3].Y := FDrawPts[3].Y + FArrowSize;
      FDrawPts[4] := Point(Right - FArrowSize, FDrawPts[2].Y);
      FDrawPts[5] := Point(FDrawPts[4].X, FDrawPts[3].Y);
      CurrCentre := Point(Left + Width div 2, Top + Height div 2);
      RotateDrawPoint(CurrCentre, FDrawPts[2]);
      RotateDrawPoint(CurrCentre, FDrawPts[3]);
      RotateDrawPoint(CurrCentre, FDrawPts[4]);
      RotateDrawPoint(CurrCentre, FDrawPts[5]);
    end;
  end;

begin
  Angle := ArcTan(Height / Width);
  FSinAngle := Sin(Angle);
  FCosAngle := Cos(Angle);
  with ClientRect do
    case FDirection of
      diHorizontal: begin
        FDrawPts[0] := Point(Left, Top + Height div 2);
        FDrawPts[1] := Point(Right - 1, FDrawPts[0].Y);
        FDrawPts[2] := Point(Left + FArrowSize, FDrawPts[0].Y - FArrowSize div 2);
        FDrawPts[3] := FDrawPts[2];
        FDrawPts[3].Y := FDrawPts[3].Y + FArrowSize;
        FDrawPts[4] := Point(Right - FArrowSize, FDrawPts[2].Y);
        FDrawPts[5] := Point(FDrawPts[4].X, FDrawPts[3].Y);
      end;
      diVertical: begin
        FDrawPts[0] := Point(Left + Width div 2, Bottom - 1);
        FDrawPts[1] := Point(FDrawPts[0].X, Top);
        FDrawPts[2] := Point(FDrawPts[0].X - FArrowSize div 2, Bottom - FArrowSize);
        FDrawPts[3] := FDrawPts[2];
        FDrawPts[3].X := FDrawPts[3].X + FArrowSize;
        FDrawPts[4] := Point(FDrawPts[2].X, Top + FArrowSize);
        FDrawPts[5] := Point(FDrawPts[3].X, FDrawPts[4].Y);
      end;
      diToptoBottom: begin
        FDrawPts[0] := TopLeft;
        FDrawPts[1] := Point(Right - 1, Bottom - 1);
        SetDiagonalArrowPoints;
      end;
      diBottomtoTop: begin
        FDrawPts[0] := Point(Left, Bottom - 1);
        FDrawPts[1] := Point(Right - 1, Top);
        SetDiagonalArrowPoints;
      end;
    end;
  if FClipMouse then begin
    DeleteObject(FMouseClipRgn);
    FMouseClipRgn := GetMouseClipRgn;
  end;
  Invalidate;
end;

{
Following message handlers ignore mouse movement that is not close to the drawn line.
}

procedure TabcCustomLine.WMLButtonDown(var Message: TWMLButtonDown);
begin
  if not FClipMouse or PtInRegion(FMouseClipRgn, Message.XPos, Message.YPos) then inherited;
end;

procedure TabcCustomLine.WMRButtonDown(var Message: TWMRButtonDown);
begin
  if not FClipMouse or PtInRegion(FMouseClipRgn, Message.XPos, Message.YPos) then inherited;
end;

procedure TabcCustomLine.WMMButtonDown(var Message: TWMMButtonDown);
begin
  if not FClipMouse or PtInRegion(FMouseClipRgn, Message.XPos, Message.YPos) then inherited;
end;

procedure TabcCustomLine.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  if not FClipMouse or PtInRegion(FMouseClipRgn, Message.XPos, Message.YPos) then inherited;
end;

procedure TabcCustomLine.WMRButtonDblClk(var Message: TWMRButtonDblClk);
begin
  if not FClipMouse or PtInRegion(FMouseClipRgn, Message.XPos, Message.YPos) then inherited;
end;

procedure TabcCustomLine.WMMButtonDblClk(var Message: TWMMButtonDblClk);
begin
  if not FClipMouse or PtInRegion(FMouseClipRgn, Message.XPos, Message.YPos) then inherited;
end;

procedure TabcCustomLine.WMMouseMove(var Message: TWMMouseMove);
begin
  if not FClipMouse or PtInRegion(FMouseClipRgn, Message.XPos, Message.YPos) then inherited;
end;

{ TabcShape }

constructor TabcShape.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csSetCaption, csCaptureMouse, csClickEvents, csDoubleClicks];
  FColorHighlight := clBtnHighlight;
  FColorShadow := clBtnShadow;
  Width := 185;
  Height := 41;
  FAlignment := taCenter;
  FAlignVertical := vaCenter;
  FWordWrap := True;
  BevelInner := bvLowered;
  BevelOuter := bvRaised;
  BevelWidth := 1;
  Brush.Color := clBtnFace;
  CornerWidth := 20;
end;

procedure TabcShape.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TabcShape.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TabcShape.Paint;
var
  X, Y, W, H, S, C: Integer;
  TextFormat: Word;
  ARect, ATextRect: TRect;
  TopColor, BottomColor: TColor;
  Text: array[0..255] of Char;
  FontHeight: Integer;

const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  VertAlignments: array[TabcVerticalAlign] of Word = (DT_TOP, DT_VCENTER, DT_BOTTOM);

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := FColorHighlight;
    if Bevel = bvLowered then TopColor := FColorShadow;
    BottomColor := FColorShadow;
    if Bevel = bvLowered then BottomColor := FColorHighlight;
  end;

begin

  {draw Shape}
  with Canvas do begin
    Pen := self.Pen;
    Brush := self.Brush;
    X := Pen.Width div 2;
    Y := X;
    W := Width - Pen.Width + 1;
    H := Height - Pen.Width + 1;
    if W < H then S := W else S := H;
    if W < H then C := W div 2 else C := H div 2;
    if C > FCornerWidth then C := FCornerWidth;
    if Shape in [stSquare, stRoundSquare, stCircle] then begin
      Inc(X, (W - S) div 2);
      Inc(Y, (H - S) div 2);
      W := S;
      H := S;
    end;
    case Shape of
      stRectangle, stSquare:
        Rectangle(X, Y, X + W, Y + H);
      stRoundRect, stRoundSquare:
        RoundRect(X, Y, X + W, Y + H, 2 * C, 2 * C);
      stCircle, stEllipse:
        Ellipse(X, Y, X + W, Y + H);
    end;
  end;

  ARect := Rect(X + Pen.Width, Y + Pen.Width, X + W - Pen.Width - 1, Y + H - Pen.Width - 1);

  {draw Bevels}
  Canvas.Pen.Mode := pmCopy;
  Canvas.Pen.Style := psSolid;

  if BevelOuter <> bvNone then begin
    AdjustColors(BevelOuter);
    case Shape of
      stRectangle, stSquare:
        abcRoundFrame3D(Canvas, ARect, TopColor, BottomColor, BevelWidth, 0);
      stRoundRect, stRoundSquare:
        abcRoundFrame3D(Canvas, ARect, TopColor, BottomColor, BevelWidth, C);
      stCircle, stEllipse:
        abcEllipseFrame3D(Canvas, ARect, TopColor, BottomColor, BevelWidth);
    end;
  end;
  if BorderWidth <> 0 then
    case Shape of
      stRectangle, stSquare:
        abcRoundFrame3D(Canvas, ARect, Brush.Color, Brush.Color, BorderWidth, 0);
      stRoundRect, stRoundSquare:
        abcRoundFrame3D(Canvas, ARect, Brush.Color, Brush.Color, BorderWidth, C);
      stCircle, stEllipse:
        abcEllipseFrame3D(Canvas, ARect, Brush.Color, Brush.Color, BorderWidth);
    end;

  abcRoundFrame3D(Canvas, ARect, Brush.Color, Brush.Color, BorderWidth, C);
  if BevelInner <> bvNone then begin
    AdjustColors(BevelInner);
    case Shape of
      stRectangle, stSquare:
        abcRoundFrame3D(Canvas, ARect, TopColor, BottomColor, BevelWidth, 0);
      stRoundRect, stRoundSquare:
        abcRoundFrame3D(Canvas, ARect, TopColor, BottomColor, BevelWidth, C);
      stCircle, stEllipse:
        abcEllipseFrame3D(Canvas, ARect, TopColor, BottomColor, BevelWidth);
    end;
  end;

  {draw Text}
  with ARect do begin
    Inc(Left);
    Inc(Top);
    Dec(Right);
    Dec(Bottom);
  end;
  if Shape in [stCircle, stEllipse] then
    {fit text to inside of ellipse}
    with ARect do begin
      W := (Right - Left) div 2;
      H := (Bottom - Top) div 2;
      {calculate dX, dY}
      X := W - Round(W / sqrt(2));
      Y := H - Round(H / sqrt(2));
      Inc(Left, X);
      Inc(Top, Y);
      Dec(Right, X);
      Dec(Bottom, Y);
    end
  else if Shape in [stRoundRect, stRoundSquare] then
    with ARect do begin
      C := Round( C * (1 - 1 / sqrt(2)));
      Inc(Left, C);
      Inc(Top, C);
      Dec(Right, C);
      Dec(Bottom, C);
    end;
  ATextRect := ARect;
  with Canvas do begin
    Pen := self.Pen;
    Brush := self.Brush;
    Font := self.Font;
    GetTextBuf(Text, SizeOf(Text));
    TextFormat := DT_EXPANDTABS or Alignments[Alignment];
    if FWordWrap then begin
      TextFormat := TextFormat or DT_WORDBREAK or DT_CALCRECT;
      DrawText(Handle, Text, StrLen(Text), ATextRect, TextFormat);
      with ATextRect do begin
        if Right > ARect.Right then
          Right := ARect.Right;
        case FAlignVertical of
          vaTop: begin
            Inc(Top);
            Inc(Bottom);
          end;
          vaCenter: begin
            FontHeight := Bottom - Top;
            Top := (Height - FontHeight) div 2;
            Bottom := (Height + FontHeight) div 2;
          end;
          vaBottom: begin
            FontHeight := Bottom - Top;
            Top := ARect.Bottom - FontHeight;
            Bottom := ARect.Bottom;
          end;
        end;
        if Top < ARect.Top then begin
          Bottom := Bottom + (ARect.Top - Top);
          Top := ARect.Top;
        end;
        if Bottom > ARect.Bottom then Bottom := ARect.Bottom;
        {adjust X difference}
        X := (ARect.Right - ARect.Left) - (Right - Left);
        if (X > 0) then
          case Alignment of
            taLeftJustify:
              {do nothing};
            taCenter:
              with ATextRect do begin
                X := X div 2;
                Inc(Left, X);
                Inc(Right, X);
              end;
            taRightJustify:
              with ATextRect do begin
                Inc(Left, X);
                Inc(Right, X);
              end;
          end;
      end;
      TextFormat := TextFormat and not DT_CALCRECT;
      DrawText(Handle, Text, StrLen(Text), ATextRect, TextFormat);
    end
    else begin
      TextFormat := TextFormat or VertAlignments[FAlignVertical] or DT_SINGLELINE;
      DrawText(Handle, Text, StrLen(Text), ATextRect, TextFormat);
    end;
  end;
end;

procedure TabcShape.SetAlignment(Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;

procedure TabcShape.SetAlignVertical(Value: TabcVerticalAlign);
begin
  if Value <> FAlignVertical then begin
    FAlignVertical := Value;
    invalidate;
  end;
end;

procedure TabcShape.SetBevelInner(Value: TPanelBevel);
begin
  FBevelInner := Value;
  Invalidate;
end;

procedure TabcShape.SetBevelOuter(Value: TPanelBevel);
begin
  FBevelOuter := Value;
  Invalidate;
end;

procedure TabcShape.SetBevelWidth(Value: TBevelWidth);
begin
  FBevelWidth := Value;
  Invalidate;
end;

procedure TabcShape.SetBorderWidth(Value: TBorderWidth);
begin
  FBorderWidth := Value;
  Invalidate;
end;

procedure TabcShape.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  SetCornerWidth(FCornerWidth);
end;

procedure TabcShape.SetColorHighlight(Value: TColor);
begin
  if Value <> FColorHighlight then begin
    FColorHighlight := Value;
    Invalidate;
  end;
end;

procedure TabcShape.SetColorShadow(Value: TColor);
begin
  if Value <> FColorShadow then begin
    FColorShadow := Value;
    Invalidate;
  end;
end;

procedure TabcShape.SetCornerWidth(Value: TabcCornerWidth);
var
  CalcWidth: Integer;
begin
  CalcWidth := Value;
  if CalcWidth > Height div 2 then
    CalcWidth := Height div 2;
  if CalcWidth > Width div 2 then
    CalcWidth := Width div 2;
  FCornerWidth := CalcWidth;
  Invalidate;
end;

procedure TabcShape.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then
  begin
    FWordWrap := Value;
    Invalidate;
  end;
end;

{ TabcFrame }

constructor TabcFrame.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csClickEvents, csDoubleClicks];
  FColorHighlight := clBtnHighlight;
  FColorShadow := clBtnShadow;
  Width := 185;
  Height := 41;
  BevelInner := bvLowered;
  BevelOuter := bvRaised;
  BevelWidth := 1;
  FBorderStyle := bsNone;
  Color := clBtnFace;
end;

procedure TabcFrame.Paint;
var
  Rect: TRect;
  TopColor, BottomColor: TColor;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := FColorHighlight;
    if Bevel = bvLowered then TopColor := FColorShadow;
    BottomColor := FColorShadow;
    if Bevel = bvLowered then BottomColor := FColorHighlight;
  end;

begin
  Rect := GetClientRect;
  with inherited Canvas do begin
    SetBkMode(Handle, TRANSPARENT);
    Brush.Style := bsClear;
  end;
  if FBorderStyle <> bsNone then
    abcRoundFrame3D(Canvas, Rect, clBlack, clBlack, 1, FCornerWidth);
  if BevelOuter <> bvNone then
  begin
    AdjustColors(BevelOuter);
    abcRoundFrame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth, FCornerWidth);
  end;
  abcRoundFrame3D(Canvas, Rect, Color, Color, BorderWidth, FCornerWidth);
  if BevelInner <> bvNone then
  begin
    AdjustColors(BevelInner);
    abcRoundFrame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth, FCornerWidth);
  end;
end;

procedure TabcFrame.SetBevelInner(Value: TPanelBevel);
begin
  FBevelInner := Value;
  Invalidate;
end;

procedure TabcFrame.SetBevelOuter(Value: TPanelBevel);
begin
  FBevelOuter := Value;
  Invalidate;
end;

procedure TabcFrame.SetBevelWidth(Value: TBevelWidth);
begin
  FBevelWidth := Value;
  Invalidate;
end;

procedure TabcFrame.SetBorderWidth(Value: TBorderWidth);
begin
  FBorderWidth := Value;
  Invalidate;
end;

procedure TabcFrame.SetBorderStyle(Value: TBorderStyle);
begin
  if FBorderStyle <> Value then
  begin
    FBorderStyle := Value;
    Invalidate;
  end;
end;

procedure TabcFrame.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  SetCornerWidth(FCornerWidth);
end;

procedure TabcFrame.SetColorHighlight(Value: TColor);
begin
  if Value <> FColorHighlight then begin
    FColorHighlight := Value;
    Invalidate;
  end;
end;

procedure TabcFrame.SetColorShadow(Value: TColor);
begin
  if Value <> FColorShadow then begin
    FColorShadow := Value;
    Invalidate;
  end;
end;

procedure TabcFrame.SetCornerWidth(Value: TabcCornerWidth);
var
  CalcWidth: Integer;
begin
  CalcWidth := Value;
  if CalcWidth > Height div 2 then
    CalcWidth := Height div 2;
  if CalcWidth > Width div 2 then
    CalcWidth := Width div 2;
  FCornerWidth := CalcWidth;
  Invalidate;
end;

procedure TabcFrame.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

end.
