unit abcspla;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{
  Important Note about inheritance:
  If you have an inheritance layer in your main form,
  set AutoShow to true only on the final descendant on which
  you want the splash form to show.
  Otherwise the splash form will be shown from the first ancestor form class
  where AutoShow is true.
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  abcfspla, abcfact, abcwanim;

type

  TabcSplash = class(TComponent)
  private
    { Private declarations }
    FAllowLocalBmp: Boolean;
    FAutoShow: Boolean;
    FBevelWidth: Word;
    FDuration: Word;
    FHideOnClick: Boolean;
    FPicture: TPicture;
    FPictureVGA: TPicture;
    FShapeForm: Boolean;
    FShapeTolerance: TColor;
    FShowModal: Boolean;
    FSplashForm: TabcsfSplash;
    FStayOnTop: Boolean;
    FOnHide: TNotifyEvent;
    FOnShow: TNotifyEvent;
    FAutoShowOnDestroy: Boolean;
    FAnimateHide: TabcAnimateWin;
    FAnimateShow: TabcAnimateWin;
    procedure AnimateChanged(Sender: TObject);
    procedure SetAnimateHide(const Value: TabcAnimateWin);
    procedure SetAnimateShow(const Value: TabcAnimateWin);
    procedure SetPicture(Value: TPicture);
    procedure SetPictureVGA(Value: TPicture);
    procedure SetAutoShow(Value: Boolean);
    procedure SplashFormShow(Sender: TObject);
    procedure SplashFormHide(Sender: TObject);
    procedure SetShapeForm(const Value: Boolean);
    function GetCommandLineDisabled: Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Assign(Source : TPersistent); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Show; dynamic;
    procedure Hide; dynamic;
    property CommandLineDisabled: Boolean read GetCommandLineDisabled;
  published
    { Published declarations }
    property AllowLocalBmp: Boolean read FAllowLocalBmp write FAllowLocalBmp default True;
    property AnimateShow: TabcAnimateWin read FAnimateShow write SetAnimateShow;
    property AnimateHide: TabcAnimateWin read FAnimateHide write SetAnimateHide;
    property BevelWidth: Word read FBevelWidth write FBevelWidth default 5;
    property Duration: Word read FDuration write FDuration default 5000;
    property HideOnClick: Boolean read FHideOnClick write FHideOnClick default True;
    property Picture: TPicture read FPicture write SetPicture;
    property PictureVGA: TPicture read FPictureVGA write SetPictureVGA;
    property ShapeForm: Boolean read FShapeForm write SetShapeForm default False;
    property ShapeTolerance: TColor read FShapeTolerance write FShapeTolerance default $101010;
    property ShowModal: Boolean read FShowModal write FShowModal default False;
    property StayOnTop: Boolean read FStayOnTop write FStayOnTop default True;
    property OnHide: TNotifyEvent read FOnHide write FOnHide;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
    {NB This must be Last! Written as last property to trigger splash screen display}
    property AutoShow: Boolean read FAutoShow write SetAutoShow;
    property AutoShowOnDestroy: Boolean read FAutoShowOnDestroy write FAutoShowOnDestroy default False;
  end;

implementation

uses abcutilg;

constructor TabcSplash.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAllowLocalBmp := True;
  FBevelWidth := 5;
  FDuration := 5000;
  FPicture := TPicture.Create;
  FPictureVGA := TPicture.Create;
  FHideOnClick := True;
  FStayOnTop := True;
  FAutoShow := True;
  FShapeTolerance := $101010;
  FAnimateHide := TabcAnimateWin.Create;
  FAnimateHide.OnChange := AnimateChanged;
  FAnimateShow := TabcAnimateWin.Create;
  FAnimateShow.OnChange := AnimateChanged;
end;

destructor TabcSplash.Destroy;
begin
  if not (csDesigning in ComponentState) and
     AutoShowOnDestroy and
     (Duration <> 0) and
     not CommandLineDisabled then
  begin
    Show;
    while (FSplashForm <> nil) do
      Application.ProcessMessages;
  end;
  FAnimateHide.Free;
  FAnimateShow.Free;
  FPicture.Free;
  FPictureVGA.Free;
  inherited Destroy;
end;

procedure TabcSplash.Assign(Source : TPersistent);
begin
  if Source is TabcSplash then
    with TabcSplash(Source) do
    begin
      Self.BevelWidth := BevelWidth;
      Self.Duration := Duration;
      Self.HideOnClick := HideOnClick;
      Self.Picture := Picture;
      Self.PictureVGA := PictureVGA;
      Self.ShapeForm := ShapeForm;
      Self.ShapeTolerance := ShapeTolerance;
      Self.ShowModal := ShowModal;
      Self.StayOnTop := StayOnTop;
      Self.OnHide := OnHide;
      Self.OnShow := OnShow;
      Self.AutoShow := AutoShow;
      Self.AnimateShow := AnimateShow;
      Self.AnimateHide := AnimateHide;
      Self.AutoShowOnDestroy := AutoShowOnDestroy;
    end
  else
    inherited Assign(Source);
end;

procedure TabcSplash.AnimateChanged(Sender: TObject);
//shaped region only supports blend effects
begin
  if (AnimateShow.Effect in [aweRoll, aweSlide, aweCenter]) or
     (AnimateHide.Effect in [aweRoll, aweSlide, aweCenter]) then
    FShapeForm := False;
end;

procedure TabcSplash.Show;
var
  frmLeft, frmTop, frmWidth, frmHeight: Integer;
  searchFile: TFileName;
begin
  if (Picture.Graphic = nil) and (PictureVGA.Graphic = nil) then
    raise Exception.Create('Picture required');

  if FSplashForm = nil then
    FSplashForm := TabcsfSplash(
      abcFormFactory.CreateFormOfClass(TabcsfSplash, Self))
  else
    Exit;
  with FSplashForm do
  begin
    OnShow := SplashFormShow;
    OnHide := SplashFormHide;
    if StayOnTop then
      FormStyle := fsStayOnTop
    else
      FormStyle := fsNormal;
    tmrClose.Interval := Duration;
    CloseOnClick := HideOnClick;
    pnlSplash.BevelWidth := BevelWidth;
    if FAllowLocalBmp then
    begin
      searchFile := ChangeFileExt(Application.ExeName, '.bmp');
      if FileExists(searchFile) then
        imgSplash.Picture.LoadFromFile(searchFile);
    end;
    if imgSplash.Picture.Graphic = nil then
      if abcScreenIsVGA and (PictureVGA.Graphic <> nil) then
        imgSplash.Picture := PictureVGA
      else
        if (Picture.Graphic <> nil) then
          imgSplash.Picture := Picture
        else
          imgSplash.Picture := PictureVGA;
    if ShapeForm then
      with abcShapeForm1 do
      begin
        Tolerance := ShapeTolerance;
        Picture := imgSplash.Picture;
        Visible := True;
      end
    else
      pnlSplash.Visible := True;
    {calc size and position of form}
    frmWidth := imgSplash.Picture.Graphic.Width + 2 * BevelWidth;
    frmHeight := imgSplash.Picture.Graphic.Height + 2 * BevelWidth;
    frmLeft := (Screen.Width - frmWidth) div 2;
    frmTop := (Screen.Height - frmHeight) div 2;
    SetBounds(frmLeft, frmTop, frmWidth, frmHeight);
    if Self.ShowModal then
    begin
      if (Duration = 0) then tmrClose.Interval := 5000;  {force modal form to close}
      ShowModal;
    end
    else
      Show;
  end;
  Application.ProcessMessages;
end;

procedure TabcSplash.Hide;
begin
  if FSplashForm <> nil then
    FSplashForm.Close;
end;

procedure TabcSplash.SplashFormShow(Sender: TObject);
begin
  AnimateShow.Show(FSplashForm);
  if Assigned(FOnShow) then FOnShow(Self);
end;

procedure TabcSplash.SplashFormHide(Sender: TObject);
begin
  AnimateHide.Hide(FSplashForm);
  if Assigned(FOnHide) then FOnHide(Self);
  FSplashForm := nil;
end;

procedure TabcSplash.SetPicture(Value: TPicture);
begin
  if Value <> FPicture then FPicture.Assign(Value);
end;

procedure TabcSplash.SetPictureVGA(Value: TPicture);
begin
  if Value <> FPictureVGA then FPictureVGA.Assign(Value);
end;

procedure TabcSplash.SetAutoShow(Value: Boolean);
begin
  FAutoShow := Value;
  if Value and
     (csLoading in ComponentState) and
     not (csDesigning in ComponentState) and
     not CommandLineDisabled then
    Show;
end;

procedure TabcSplash.SetAnimateHide(const Value: TabcAnimateWin);
begin
  FAnimateHide.Assign(Value);
end;

procedure TabcSplash.SetAnimateShow(const Value: TabcAnimateWin);
begin
  FAnimateShow.Assign(Value);
end;

procedure TabcSplash.SetShapeForm(const Value: Boolean);
begin
  if Value <> FShapeForm then
  begin
    FShapeForm := Value;
    if Value then
    begin
      if (AnimateShow.Effect in [aweRoll, aweSlide, aweCenter]) then
        AnimateShow.Effect := aweBlend;
      if (AnimateHide.Effect in [aweRoll, aweSlide, aweCenter]) then
        AnimateHide.Effect := aweBlend;
    end;
  end;
end;

function TabcSplash.GetCommandLineDisabled: Boolean;
const
  C_NO_SPLASH_1 = '-ns';
  C_NO_SPLASH_2 = '/ns';
var
  i: integer;
  sParam: string;
begin
  Result := False;
  {check command line params for -ns or /ns options}
  for i := 1 to ParamCount do
  begin
    sParam := LowerCase(ParamStr(i));
    if (sParam = C_NO_SPLASH_1) or (sParam = C_NO_SPLASH_2) then
    begin
      Result := True;
      Exit;
    end;
  end;
end;

end.
