unit abcsplit;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}


{ Splitter controls for ABC:

TabcSplitBar:            Simple bar for scrolling two TWinControls in a parent
TabcSplitPane:           Non-scrolling window for a pane of a splitter, inherits from TWinControl
TabcScrollingSplitPane:  Scrolling window for a pane of a splitter, inherits from TScrollBox
TabcCustomSplit:         protected implementation of splitter common methods
TabcCustomBiSplit:       protected implementation of 2 way splitter common methods
TabcCustomTriSplit:      protected implementation of 3 way splitter common methods
TabcCustomQuadSplit:     protected implementation of 4 way splitter common methods
TabcCustomMultiSplit:    protected implementation of 2,3, or 4 way splitter common methods
TabcBiSplit:             published implementation of 2 way splitter using TabcSplitPanes
TabcSBiSplit:            published implementation of 2 way splitter using TabcScrollingSplitPanes
TabcTriSplit:            published implementation of 3 way splitter using TabcSplitPanes
TabcSTriSplit:           published implementation of 3 way splitter using TabcScrollingSplitPanes
TabcQuadSplit:           published implementation of 4 way splitter using TabcSplitPanes
TabcSQuadSplit:          published implementation of 4 way splitter using TabcScrollingSplitPanes
TabcMultiSplit:          published implementation of 2,3, or 4 way splitter using TabcSplitPanes
TabcSMultiSplit:         published implementation of 2,3, or 4 way splitter using TabcScrollingSplitPanes

NB: Scroll Keys
  Set TabStop to true and focus the control
  Use Left, Right, Up, Down and Shift keys to move the split bar
  Set KeyIncrement to change the scroll amount
}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, StdCtrls,
  ExtCtrls, Forms, Menus, abcmsg, abcutilg, abcpanel, abcscrol;

type

  { common types }

  TabcThickness = 1..10;
  TabcPercent = 0..100;
  TabcPosInt = 0..MaxInt;
  TabcBiSplitLayout = (bslVert, bslHorz);

  TabcSplitMoveStyle = (amsBlack, amsDark, amsMedium, amsMediumLight, amsLight, amsNoLine, amsImmediate);

  { TabcCustomSplitBar }

  TabcCustomSplitBar = class(TabcCustomEdgePanel)
  private
    FBrush: TBrush;
    FSaveBrush: HBrush;
    FSplitMoveStyle: TabcSplitMoveStyle;
    FMoveDC: HDC;
    FDragLineVisible: Boolean;

    FCursor: TCursor;
    FKeyIncrement: TabcPosInt;
    FLastMovePos: TPoint;
    FLayout: TabcBiSplitLayout;
    FMoving: Boolean;
    FMax: TabcPosInt;
    FMin: TabcPosInt;
    FOnSplitMove: TNotifyEvent;
    FPercent: TabcPercent;
    FThickness: TabcThickness;
    FWindowA: TWinControl;
    FWindowB: TWinControl;

    procedure AllocMoveDC;
    procedure FreeMoveDC;
    procedure StopMoving;

    {to be removed}
    function GetMoveImmediate: Boolean;
    procedure SetMoveImmediate(Value: Boolean);

    procedure AlignWindows;
    function CanMove: Boolean;
    function ClientToParent(Point: TPoint): TPoint;
    procedure ClipMovePos(var Pos: TPoint);
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMWantSpecialKey(var Msg: TCMWantSpecialKey); message CM_WANTSPECIALKEY;
    procedure DrawDragLine(Pos: TPoint);
    function GetPos: TabcPosInt;
    function GetSplitMovePos(var X, Y: Integer): TPoint;
    procedure MoveDragLine(Pos: TPoint);
    procedure MoveSplitPosition(Pos: TPoint);
    procedure SetCursor(Value: TCursor);
    procedure SetLayout(Value: TabcBiSplitLayout);
    procedure SetMax(Value: TabcPosInt);
    procedure SetMin(Value: TabcPosInt);
    procedure SetPercent(Value: TabcPercent);
    procedure SetPos(Value: TabcPosInt);
    procedure SetThickness(Value: TabcThickness);
    procedure SetWindowA(Value: TWinControl);
    procedure SetWindowB(Value: TWinControl);
    procedure UpdateCursor;
    procedure UpdatePercent;
  protected
    function GetParentClientRect: TRect; dynamic;
{$IFDEF DELPHI6}
    function DesignWndProc(var Message: TMessage): Boolean; override;
{$ENDIF}
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    { properties to publish }
    property Cursor: TCursor read FCursor write SetCursor default crDefault;
    property KeyIncrement: TabcPosInt read FKeyIncrement write FKeyIncrement default 10;
    property Layout: TabcBiSplitLayout read FLayout write SetLayout default bslVert;
    property MoveImmediate: Boolean read GetMoveImmediate write SetMoveImmediate default False;  //to be removed
    property Max: TabcPosInt read FMax write SetMax;
    property Min: TabcPosInt read FMin write SetMin;
    property OnSplitMove: TNotifyEvent read FOnSplitMove write FOnSplitMove;
    property Percent: TabcPercent read FPercent write SetPercent stored False;
    property Pos: TabcPosInt read GetPos write SetPos stored False;
    property SplitMoveStyle: TabcSplitMoveStyle read FSplitMoveStyle write FSplitMoveStyle default amsBlack;
    property Thickness: TabcThickness read FThickness write SetThickness default 4;
    property WindowA: TWinControl read FWindowA write SetWindowA;
    property WindowB: TWinControl read FWindowB write SetWindowB;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  end;

  { TabcSplitBar }

  TabcSplitBar = class(TabcCustomSplitBar)
  published
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BorderStyle;
    property Color;
    property Cursor;
    property Enabled;
    property Hint;
    property KeyIncrement;
    property Layout;
    property MoveImmediate stored False;  //to be removed
    property Max;
    property Min;
    property ParentColor;
    property ParentShowHint;
    property Percent;
    property Pos;
    property ShowHint;
    property SplitMoveStyle;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property WindowA;
    property WindowB;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMove;
  end;

  { TabcSplitPane }

  TabcCustomSplit = class;

  TabcSplitPane = class(TWinControl)
  private
    FSplitter: TabcCustomSplit;
    function GetPaneIndex: Integer;
    procedure SetPaneIndex(Value: Integer);
    procedure SetSplitter(Value: TabcCustomSplit);
  protected
{$IFDEF DELPHI6}
    procedure SetParent(AParent: TWinControl); override;
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadState(Reader: TReader); override;
    property Splitter: TabcCustomSplit read FSplitter write SetSplitter;
  published
    property PaneIndex: Integer read GetPaneIndex write SetPaneIndex stored False;
    property Left stored False;
    property Top stored False;
    property Width stored False;
    property Height stored False;
    property Color;

    {new TWinControl properties}
    property BevelEdges;
    property BevelKind;
    property BevelInner;
    property BevelOuter default bvNone;
    property BevelWidth;
    property BorderWidth;
    property DragCursor;
    property DragMode;
    property DragKind;
    property DockSite;
    property Enabled;
    property Font;
    property ParentColor default True;
    property ParentFont default True;
    property ParentShowHint default True;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnDockOver;
    property OnDockDrop;
    property OnGetSiteInfo;
    property OnUnDock;
    property OnStartDock;
    property OnEndDock;
  end;

  { TabcScrollingSplitPane }

  TabcScrollingSplitPane = class(TabcCustomScrollBox)
  private
    FSplitter: TabcCustomSplit;
    function GetPaneIndex: Integer;
    procedure SetPaneIndex(Value: Integer);
    procedure SetSplitter(Value: TabcCustomSplit);
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
  protected
{$IFDEF DELPHI6}
    procedure SetParent(AParent: TWinControl); override;
{$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadState(Reader: TReader); override;
    property Splitter: TabcCustomSplit read FSplitter write SetSplitter;
  published
    property PaneIndex: Integer read GetPaneIndex write SetPaneIndex stored False;
    property Left stored False;
    property Top stored False;
    property Width stored False;
    property Height stored False;
    property AutoScroll;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelKind;
    property BevelWidth;
    property BorderWidth;
    property Color;
    property DragCursor;
    property DragMode;
    property AutoSize;
    property DragKind;
    property DockSite;
    property Enabled;
    property Font;
    property ParentColor default True;
    property ParentFont default True;
    property ParentShowHint default True;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnDockOver;
    property OnDockDrop;
    property OnGetSiteInfo;
    property OnUnDock;
    property OnStartDock;
    property OnEndDock;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnScrollH;
    property OnScrollV;
  end;

  { TabcCustomSplit }

  TabcCustomSplit = class(TabcCustomEdgePanel)
  private
    FInitialized: Boolean;
    FBrush: TBrush;
    FSaveBrush: HBrush;
    FSplitMoveStyle: TabcSplitMoveStyle;
    FMoveDC: HDC;
    FDragLineVisible: Boolean;

    FBorderOffset: Integer;
    FCursor: TCursor;
    FDrawing: Boolean;
    FHitTest: TPoint;
    FKeyIncrement: Integer;
    FLastMovePos: TPoint;
    FMoveCursor: TCursor;
    FMoving: Boolean;
    FOnSplitMoveH: TNotifyEvent;
    FOnSplitMoveV: TNotifyEvent;
    FPaneList: TList;
    FSplitBevel: TBevelCut;
    FSplitColor: TColor;
    FSplitHLock: Boolean;
    FSplitHMax: TabcPosInt;
    FSplitHMin: TabcPosInt;
    FSplitHMouseMin: TabcPosInt;
    FSplitHMouseMax: TabcPosInt;
    FSplitHPercent: TabcPercent;
    FSplitHPos: TabcPosInt;
    FSplitHScaled: Boolean;
    FThickness: TabcThickness;
    FSplitVLock: Boolean;
    FSplitVMax: TabcPosInt;
    FSplitVMin: TabcPosInt;
    FSplitVMouseMax: TabcPosInt;
    FSplitVMouseMin: TabcPosInt;
    FSplitVPercent: TabcPercent;
    FSplitVPos: TabcPosInt;
    FSplitVScaled: Boolean;

    procedure AllocMoveDC;
    procedure FreeMoveDC;
    procedure StopMoving;
    procedure DrawOneDragLineAt(DrawRect: TRect);

    {to be removed}
    function GetMoveImmediate: Boolean;
    procedure SetMoveImmediate(Value: Boolean);

    function CheckSplitLock(ACursor: TCursor): TCursor;
    procedure ClipMovePos(var Pos: TPoint);
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure DrawDragLines(CRect: TRect; Pos: TPoint);
    function GetBevelInner: TPanelBevel;
    function GetBevelOuter: TPanelBevel;
    function GetBevelWidth: Integer;
    function GetBorderStyle: TBorderStyle;
    function GetBorderWidth: Integer;
    procedure InitSplitDefaults;
    procedure MoveDragLines(Pos: TPoint);
    procedure MoveSplitPosition(Pos: TPoint);
    procedure SetBevelInner(Value: TPanelBevel);
    procedure SetBevelOuter(Value: TPanelBevel);
    procedure SetBevelWidth(Value: Integer);
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetBorderWidth(Value: Integer);
    procedure SetCursor(Value: TCursor);
    procedure SetSplitBevel(Value: TBevelCut);
    procedure SetSplitColor(Value: TColor);
    procedure SetSplitHMax(Value: TabcPosInt);
    procedure SetSplitHMin(Value: TabcPosInt);
    procedure SetSplitHPercent(Value: TabcPercent);
    procedure SetSplitHPos(Value: TabcPosInt);
    procedure SetSplitVMax(Value: TabcPosInt);
    procedure SetSplitVMin(Value: TabcPosInt);
    procedure SetSplitVPercent(Value: TabcPercent);
    procedure SetSplitVPos(Value: TabcPosInt);
    procedure SetThickness(Value: TabcThickness);
    procedure UpdateBorderOffset;
    procedure UpdateSplitHPos(Value: TabcPosInt);
    procedure UpdateSplitVPos(Value: TabcPosInt);
    procedure WMNCHitTest(var Msg: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMSetCursor(var Msg: TWMSetCursor); message WM_SETCURSOR;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure AMInitSplitSize(var Msg: TMessage); message AM_INITSPLITSIZE;
  protected
    procedure AddPane(Value: TWinControl);
    procedure RemovePane(Value: TWinControl);
    function BetweenXMouseLimits(X: Integer): Boolean;
    function BetweenYMouseLimits(Y: Integer): Boolean;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
{$IFDEF DELPHI6}
    function DesignWndProc(var Message: TMessage): Boolean; override;
{$ENDIF}
    procedure DestroyPanes; virtual;
    procedure SetPaneBounds(APaneIndex, ALeft, ATop, AWidth, AHeight: integer);
    procedure DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer); virtual;
    procedure DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor); virtual;
    procedure DoResizePanes(ARect: TRect); virtual;
    function GetMaxPanes: Integer; virtual;
    function GetPaneClass: TWinControlClass; virtual;
    function GetMoveCursor(X,Y: Integer): TCursor; virtual;
    function GetPane(Index: Integer): TWinControl;
    function GetPaneCount: Integer;
    function GetSplitMovePos(var X, Y: Integer): TPoint; virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
    procedure PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); virtual;
    procedure PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); virtual;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    {Readonly props for access from inherited classes - don't publish}
    property MoveCursor: TCursor read FMoveCursor;
    property PaneList: TList read FPaneList;
    property SplitHMouseMax: TabcPosInt read FSplitHMouseMax;
    property SplitHMouseMin: TabcPosInt read FSplitHMouseMin;
    property SplitVMouseMax: TabcPosInt read FSplitVMouseMax;
    property SplitVMouseMin: TabcPosInt read FSplitVMouseMin;
    property BorderOffset: Integer read FBorderOffset;

    {Following can be published}
    property BevelInner: TPanelBevel read GetBevelInner write SetBevelInner default bvNone;
    property BevelOuter: TPanelBevel read GetBevelOuter write SetBevelOuter default bvRaised;
    property BevelWidth: Integer read GetBevelWidth write SetBevelWidth default 1;
    property BorderStyle: TBorderStyle read GetBorderStyle write SetBorderStyle default bsNone;
    property BorderWidth: Integer read GetBorderWidth write SetBorderWidth;
    property Cursor: TCursor read FCursor write SetCursor default crDefault;
    property KeyIncrement: Integer read FKeyIncrement write FKeyIncrement default 10;
    property MoveImmediate: Boolean read GetMoveImmediate write SetMoveImmediate default False;  //to be removed
    property SplitBevel: TBevelCut read FSplitBevel write SetSplitBevel
      default bvRaised;
    property SplitColor: TColor read FSplitColor write SetSplitColor default clBtnFace;
    property SplitHMax: TabcPosInt read FSplitHMax write SetSplitHMax;
    property SplitHMin: TabcPosInt read FSplitHMin write SetSplitHMin;
    property SplitHPercent: TabcPercent read FSplitHPercent write SetSplitHPercent stored False;
    property SplitHPos: TabcPosInt read FSplitHPos write SetSplitHPos;
    property SplitHScaled: Boolean read FSplitHScaled write FSplitHScaled default False;
    property SplitMoveStyle: TabcSplitMoveStyle read FSplitMoveStyle write FSplitMoveStyle default amsBlack;
    property SplitVMax: TabcPosInt read FSplitVMax write SetSplitVMax;
    property SplitVMin: TabcPosInt read FSplitVMin write SetSplitVMin;
    property SplitVPercent: TabcPercent read FSplitVPercent write SetSplitVPercent stored False;
    property SplitVPos: TabcPosInt read FSplitVPos write SetSplitVPos;
    property SplitVScaled: Boolean read FSplitVScaled write FSplitVScaled default False;
    property Thickness: TabcThickness read FThickness write SetThickness default 2;
    property OnSplitMoveH: TNotifyEvent read FOnSplitMoveH write FOnSplitMoveH;
    property OnSplitMoveV: TNotifyEvent read FOnSplitMoveV write FOnSplitMoveV;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CreatePanes; virtual;
    procedure ResizePanes;
    procedure Rotate(Clockwise: Boolean);
    procedure Exchange(Index1, Index2: Integer);
    procedure Move(CurrIndex, NewIndex: Integer);
    property MaxPanes: Integer read GetMaxPanes;
    property Pane[Index: Integer]: TWinControl read GetPane;  {see redeclared public defs in descendant classes}
    property PaneClass: TWinControlClass read GetPaneClass;
    property PaneCount: Integer read GetPaneCount;
    {can be published}
    property SplitHLock: Boolean read FSplitHLock write FSplitHLock default False;
    property SplitVLock: Boolean read FSplitVLock write FSplitVLock default False;
  end;

  { TabcCustomBiSplit }

  TabcCustomBiSplit = class(TabcCustomSplit)
  private
    FLayout: TabcBiSplitLayout;
    procedure CMWantSpecialKey(var Msg: TCMWantSpecialKey); message CM_WANTSPECIALKEY;
    procedure SetLayout(Value: TabcBiSplitLayout);
  protected
    procedure DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer); override;
    procedure DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor); override;
    procedure DoResizePanes(ARect: TRect); override;
    function GetMoveCursor(X,Y: Integer): TCursor; override;
    function GetMaxPanes: Integer; override;
    function GetSplitMovePos(var X, Y: Integer): TPoint; override;
    {Following can be published}
    property Layout: TabcBiSplitLayout read FLayout write SetLayout default bslVert;
  end;

  { TabcBiSplit }

  TabcBiSplit = class(TabcCustomBiSplit)
  private
    function GetSplitPane(Index: Integer): TabcSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
  public
    property Pane[Index: Integer]: TabcSplitPane read GetSplitPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Color;
    property Constraints;
    property Ctl3D;
    property Cursor;
    property Enabled;
    property Font;
    property KeyIncrement;
    property Layout;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

  { TabcSBiSplit }

  TabcSBiSplit = class(TabcCustomBiSplit)
  private
    FPaneScrolling: Boolean;
    FSyncHScroll: Boolean;
    FSyncVScroll: Boolean;
    function GetScrollPane(Index: Integer): TabcScrollingSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
    procedure PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
    procedure PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
  public
    property Pane[Index: Integer]: TabcScrollingSplitPane read GetScrollPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Color;
    property Constraints;
    property Ctl3D;
    property Cursor;
    property Enabled;
    property Font;
    property KeyIncrement;
    property Layout;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property SyncHScroll: Boolean read FSyncHScroll write FSyncHScroll;
    property SyncVScroll: Boolean read FSyncVScroll write FSyncVScroll;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

  { TabcCustomTriSplit }

  TabcTriSplitLayout = (tslLeftTopBottom, tslRightTopBottom, tslLeftRightBottom, tslTopLeftRight);

  TabcCustomTriSplit = class(TabcCustomSplit)
  private
    FLayout: TabcTriSplitLayout;
    procedure CMWantSpecialKey(var Msg: TCMWantSpecialKey); message CM_WANTSPECIALKEY;
    procedure SetLayout(Value: TabcTriSplitLayout);
  protected
    procedure DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer); override;
    procedure DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor); override;
    procedure DoResizePanes(ARect: TRect); override;
    function GetMoveCursor(X,Y: Integer): TCursor; override;
    function GetMaxPanes: Integer; override;
    function GetSplitMovePos(var X, Y: Integer): TPoint; override;
    {Following can be published}
    property Layout: TabcTriSplitLayout read FLayout write SetLayout default tslLeftTopBottom;
  end;

  { TabcTriSplit }

  TabcTriSplit = class(TabcCustomTriSplit)
  private
    function GetSplitPane(Index: Integer): TabcSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
  public
    property Pane[Index: Integer]: TabcSplitPane read GetSplitPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Layout;
    property Constraints;
    property Cursor;
    property Enabled;
    property Color;
    property Ctl3D;
    property Font;
    property KeyIncrement;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

  { TabcSTriSplit }

  TabcSTriSplit = class(TabcCustomTriSplit)
  private
    FPaneScrolling: Boolean;
    FSyncHScroll: Boolean;
    FSyncVScroll: Boolean;
    function GetScrollPane(Index: Integer): TabcScrollingSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
    procedure PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
    procedure PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
  public
    property Pane[Index: Integer]: TabcScrollingSplitPane read GetScrollPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Layout;
    property Cursor;
    property Enabled;
    property Color;
    property Constraints;
    property Ctl3D;
    property Font;
    property KeyIncrement;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property SyncHScroll: Boolean read FSyncHScroll write FSyncHScroll;
    property SyncVScroll: Boolean read FSyncVScroll write FSyncVScroll;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

  { TabcCustomQuadSplit }

  TabcCustomQuadSplit = class(TabcCustomSplit)
  private
    procedure CMWantSpecialKey(var Msg: TCMWantSpecialKey); message CM_WANTSPECIALKEY;
  protected
    procedure DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer); override;
    procedure DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor); override;
    procedure DoResizePanes(ARect: TRect); override;
    function GetMoveCursor(X,Y: Integer): TCursor; override;
    function GetMaxPanes: Integer; override;
    function GetSplitMovePos(var X, Y: Integer): TPoint; override;
  end;

  { TabcQuadSplit }

  TabcQuadSplit = class(TabcCustomQuadSplit)
  private
    function GetSplitPane(Index: Integer): TabcSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
  public
    property Pane[Index: Integer]: TabcSplitPane read GetSplitPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Cursor;
    property Enabled;
    property Color;
    property Constraints;
    property Ctl3D;
    property Font;
    property KeyIncrement;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

  { TabcSQuadSplit }

  TabcSQuadSplit = class(TabcCustomQuadSplit)
  private
    FPaneScrolling: Boolean;
    FSyncHScroll: Boolean;
    FSyncVScroll: Boolean;
    function GetScrollPane(Index: Integer): TabcScrollingSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
    procedure PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
    procedure PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
  public
    property Pane[Index: Integer]: TabcScrollingSplitPane read GetScrollPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Cursor;
    property Enabled;
    property Color;
    property Constraints;
    property Ctl3D;
    property Font;
    property KeyIncrement;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property SyncHScroll: Boolean read FSyncHScroll write FSyncHScroll;
    property SyncVScroll: Boolean read FSyncVScroll write FSyncVScroll;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

  { TabcCustomMultiSplit }

  TabcMultiSplitLayout = (mslSingle, mslVert, mslHorz, mslLeftTopBottom, mslRightTopBottom,
    mslLeftRightBottom, mslTopLeftRight, mslFourWay);

  TabcCustomMultiSplit = class(TabcCustomSplit)
  private
    FLayout: TabcMultiSplitLayout;
    procedure CMWantSpecialKey(var Msg: TCMWantSpecialKey); message CM_WANTSPECIALKEY;
    procedure SetLayout(Value: TabcMultiSplitLayout);
  protected
    procedure DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer); override;
    procedure DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor); override;
    procedure DoResizePanes(ARect: TRect); override;
    function GetMoveCursor(X,Y: Integer): TCursor; override;
    function GetMaxPanes: Integer; override;
    function GetSplitMovePos(var X, Y: Integer): TPoint; override;
    {Following can be published}
    property Layout: TabcMultiSplitLayout read FLayout write SetLayout default mslFourWay;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  { TabcMultiSplit }

  TabcMultiSplit = class(TabcCustomMultiSplit)
  private
    function GetSplitPane(Index: Integer): TabcSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
  public
    property Pane[Index: Integer]: TabcSplitPane read GetSplitPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Layout;
    property Cursor;
    property Enabled;
    property Color;
    property Constraints;
    property Ctl3D;
    property Font;
    property KeyIncrement;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

  { TabcSMultiSplit }

  TabcSMultiSplit = class(TabcCustomMultiSplit)
  private
    FPaneScrolling: Boolean;
    FSyncHScroll: Boolean;
    FSyncVScroll: Boolean;
    function GetScrollPane(Index: Integer): TabcScrollingSplitPane;
  protected
    function GetPaneClass: TWinControlClass; override;
    procedure PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
    procedure PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer); override;
  public
    property Pane[Index: Integer]: TabcScrollingSplitPane read GetScrollPane;
  published
    property Align;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth;
    property Layout;
    property Cursor;
    property Enabled;
    property Color;
    property Constraints;
    property Ctl3D;
    property Font;
    property KeyIncrement;
    property MoveImmediate stored False;  {to be removed}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property SplitBevel;
    property SplitColor;
    property SplitHLock;
    property SplitHMax;
    property SplitHMin;
    property SplitHPercent;
    property SplitHPos;
    property SplitHScaled;
    property SplitMoveStyle;
    property SplitVLock;
    property SplitVMax;
    property SplitVMin;
    property SplitVPercent;
    property SplitVPos;
    property SplitVScaled;
    property SyncHScroll: Boolean read FSyncHScroll write FSyncHScroll;
    property SyncVScroll: Boolean read FSyncVScroll write FSyncVScroll;
    property TabOrder;
    property TabStop;
    property Thickness;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnSplitMoveH;
    property OnSplitMoveV;
  end;

const
  abccrSize = TCursor(-23);

implementation

uses abcconst, abcutilf;

{$R abccur32.res}

const
  DRAG_BRUSH_COLORS: array[amsDark..amsLight] of TColor =
    (clWhite, $00C0C0C0, $00808080, $00404040);

{ TabcCustomSplitBar }

function TabcCustomSplitBar.GetMoveImmediate: Boolean;
begin
  Result := FSplitMoveStyle = amsImmediate;
end;

procedure TabcCustomSplitBar.SetMoveImmediate(Value: Boolean);
begin
  if Value then
    FSplitMoveStyle := amsImmediate
  else if FSplitMoveStyle = amsImmediate then
    FSplitMoveStyle := amsMedium;
end;

constructor TabcCustomSplitBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
    csOpaque, csDoubleClicks];
  FCursor := crDefault;
  FKeyIncrement := 10;
  FThickness := 4;
  SetBounds(Left, Top, 4, 200);
end;

destructor TabcCustomSplitBar.Destroy;
begin
  FBrush.Free;
  inherited Destroy;
end;

procedure TabcCustomSplitBar.AlignWindows;
{
Called when WindowA/WindowB are set/reset to change the alignment of this window
and its siblings; and to set the cursor.
Also called when Layout changes to realign the windows.
}
var
  ParentRect: TRect;
  WindowAWidth, WindowAHeight: Integer;
begin
  if (csLoading in ComponentState) then Exit;
  if (WindowA = nil) or (WindowB = nil) then
    Align := alNone
  else begin
    ParentRect := GetParentClientRect;
    with ParentRect do
      if (Layout = bslVert) then begin
        WindowAWidth := (Right - Left) div 2;
        WindowAHeight := (Bottom - Top);
        WindowA.Align := alLeft;
        WindowA.SetBounds(Left, Top, WindowAWidth, WindowAHeight);
        Align := alLeft;
        SetBounds(Left + WindowAWidth, Top, Thickness, WindowAHeight);
        WindowB.Align := alClient;
      end
      else begin
        WindowAWidth := (Right - Left);
        WindowAHeight := (Bottom - Top) div 2;
        WindowA.Align := alTop;
        WindowA.SetBounds(Left, Top, WindowAWidth, WindowAHeight);
        Align := alTop;
        SetBounds(Left, Top + WindowAHeight, WindowAWidth, Thickness);
        WindowB.Align := alClient;
      end;
  end;
  UpdateCursor;
end;

function TabcCustomSplitBar.CanMove: Boolean;
begin
  Result := Enabled and (WindowA <> nil) and (WindowB <> nil) and (Parent <> nil);
end;

function TabcCustomSplitBar.ClientToParent(Point: TPoint): TPoint;
begin
  if Parent = nil then
    Result := Point
  else begin
    Result := ClientToScreen(Point);
    Result := Parent.ScreenToClient(Result);
  end;
end;

procedure TabcCustomSplitBar.ClipMovePos(var Pos: TPoint);
var
  ParentRect: TRect;
begin
  ParentRect := GetParentClientRect;
  with Pos do
    if Layout = bslVert then begin
      if X < Min then begin
        X := Min;
        if X <= ParentRect.Left then
          X := ParentRect.Left;
      end
      else if X < ParentRect.Left then
        X := ParentRect.Left
      else if (X > Max) and (Max <> 0) then
        X := Max
      else if X > (ParentRect.Right - Width - 1) then
        X := ParentRect.Right - Width - 1;
    end
    else begin
      if Y < Min then begin
        Y := Min;
        if Y < ParentRect.Top then
          Y := ParentRect.Top;
      end
      else if Y < ParentRect.Top then
        Y := ParentRect.Top
      else if (Y > Max) and (Max <> 0) then
        Y := Max
      else if Y > (ParentRect.Bottom - Height - 1) then
        Y := ParentRect.Bottom - Height - 1;
    end;
end;

procedure TabcCustomSplitBar.CMDesignHitTest(var Message: TCMDesignHitTest);
begin
  if CanMove then
    Message.Result := 1
  else
    inherited;
end;

{$IFDEF DELPHI6}
function TabcCustomSplitBar.DesignWndProc(var Message: TMessage): Boolean;
begin
  Result :=
    CanMove and
    ((Message.Msg = WM_LBUTTONDOWN) or
     (Message.Msg = WM_LBUTTONUP) or
     (Message.Msg = WM_MOUSEMOVE) or
     (Message.Msg = WM_LBUTTONDBLCLK));
end;
{$ENDIF}

procedure TabcCustomSplitBar.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  UpdateCursor;
end;

procedure TabcCustomSplitBar.CMWantSpecialKey(var Msg: TCMWantSpecialKey);
begin
  inherited;
  with Msg do
    if (((CharCode in [VK_LEFT, VK_RIGHT]) and (Layout = bslVert)) or
        ((CharCode in [VK_UP, VK_DOWN]) and (Layout = bslHorz))
       ) and CanMove then
      Result := 1;
end;

procedure TabcCustomSplitBar.AllocMoveDC;
begin
  FMoveDC := GetDCEx(Parent.Handle, 0, DCX_CACHE or DCX_CLIPSIBLINGS
    or DCX_LOCKWINDOWUPDATE);

  if SplitMoveStyle in [amsDark..amsLight]  then
  begin
    if FBrush = nil then
    begin
      FBrush := TBrush.Create;
      FBrush.Bitmap := AllocPatternBitmap(
        clBlack, DRAG_BRUSH_COLORS[SplitMoveStyle]);
    end;
    FSaveBrush := SelectObject(FMoveDC, FBrush.Handle);
  end;
end;

procedure TabcCustomSplitBar.FreeMoveDC;
begin
  if FSaveBrush <> 0 then
    SelectObject(FMoveDC, FSaveBrush);
  ReleaseDC(Parent.Handle, FMoveDC);
  if FBrush <> nil then
  begin
    FBrush.Free;
    FBrush := nil;
  end;
end;

procedure TabcCustomSplitBar.StopMoving;
begin
  if FDragLineVisible then DrawDragLine(FLastMovePos);
  FreeMoveDC;
end;

procedure TabcCustomSplitBar.DrawDragLine(Pos: TPoint);
begin
  FDragLineVisible := not FDragLineVisible;
  with Pos do
    if Layout = bslVert then
      PatBlt(FMoveDC, X, Top, Thickness, Height, PATINVERT)
    else
      PatBlt(FMoveDC, Left, Y, Width, Thickness, PATINVERT);
end;

function TabcCustomSplitBar.GetParentClientRect: TRect;
var
  BevelSize: Integer;
begin
  if Parent = nil then
    Result := Rect(0,0,0,0)
  else begin
    Result := Parent.ClientRect;
    if Parent is TPanel then
      with TPanel(Parent) do begin
        BevelSize := BorderWidth;
        if BevelOuter <> bvNone then Inc(BevelSize, BevelWidth);
        if BevelInner <> bvNone then Inc(BevelSize, BevelWidth);
        InflateRect(Result, -BevelSize, -BevelSize);
      end;
  end;
end;

function TabcCustomSplitBar.GetPos: TabcPosInt;
  function Max(I, J: Integer): Integer;
  begin
    if I > J then
      Result := I
    else
      Result := J;
  end;
begin
  if Layout = bslVert then
    Result := Max(0, Left)
  else
    Result := Max(0, Top);
end;

function TabcCustomSplitBar.GetSplitMovePos(var X, Y: Integer): TPoint;
begin
  Result := ClientToParent(Point(X,Y));
  ClipMovePos(Result);
end;

procedure TabcCustomSplitBar.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Key in [VK_LEFT..VK_DOWN]) then begin
    case Key of
      VK_LEFT, VK_UP:
        if (ssShift in Shift) then
          Pos := 0
        else
          Pos := Pos - FKeyIncrement;
      VK_RIGHT, VK_DOWN:
        if (ssShift in Shift) then
          if Layout = bslVert then
            Pos := Parent.ClientWidth
          else
            Pos := Parent.ClientHeight
        else
          Pos := Pos + FKeyIncrement;
    end;
    Key := 0;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TabcCustomSplitBar.Loaded;
var
  SavePos: Integer;
begin
  inherited Loaded;
  SavePos := Pos;
  AlignWindows;
  Pos := SavePos;
end;

procedure TabcCustomSplitBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and CanMove and not FMoving then begin
    AllocMoveDC;
    FLastMovePos := GetSplitMovePos(X, Y);
    if (SplitMoveStyle <> amsImmediate) then
      MoveDragLine(FLastMovePos);
    FMoving := True;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TabcCustomSplitBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  MovePos: TPoint;
begin
  if FMoving then begin
    MovePos := GetSplitMovePos(X, Y);
    if SplitMoveStyle = amsImmediate then
      MoveSplitPosition(MovePos)
    else
      MoveDragLine(MovePos);
    FLastMovePos := MovePos;
  end;
  inherited MouseMove(Shift, X, Y);
end;

procedure TabcCustomSplitBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FMoving then begin
    FMoving := False;
    MoveSplitPosition(GetSplitMovePos(X, Y));
    StopMoving;
  end;
  inherited MouseUp(Button, Shift, X, Y);
end;

procedure TabcCustomSplitBar.MoveDragLine(Pos: TPoint);
begin
  if SplitMoveStyle in [amsBlack..amsLight] then
  begin
    if FMoving then DrawDragLine(FLastMovePos);
    DrawDragLine(Pos);
  end;
end;

procedure TabcCustomSplitBar.MoveSplitPosition(Pos: TPoint);
var
  AdjustedPos: TPoint;
  ParentRect: TRect;
begin
  if CanMove then begin
    ParentRect := GetParentClientRect;
    AdjustedPos := Pos;
    ClipMovePos(AdjustedPos);
    if Layout = bslVert then
      WindowA.Width := AdjustedPos.X - ParentRect.Left
    else
      WindowA.Height := AdjustedPos.Y - ParentRect.Top;
    UpdatePercent;
    abcDesignerModified(Self);
    if Assigned(FOnSplitMove) then FOnSplitMove(Self);
  end;
end;

procedure TabcCustomSplitBar.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = WindowA then
      WindowA := nil
    else if AComponent = WindowB then
      WindowB := nil;
end;

procedure TabcCustomSplitBar.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if Layout = bslVert then begin
    if AWidth < Low(TabcThickness) then
      AWidth := Low(TabcThickness)
    else if AWidth > High(TabcThickness) then
      AWidth := High(TabcThickness);
  end
  else begin
    if AHeight < Low(TabcThickness) then
      AHeight := Low(TabcThickness)
    else if AHeight > High(TabcThickness) then
      AHeight := High(TabcThickness);
  end;
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcCustomSplitBar.SetCursor;
begin
  if Value <> FCursor then begin
    FCursor := Value;
    UpdateCursor;
  end;
end;

procedure TabcCustomSplitBar.SetLayout(Value: TabcBiSplitLayout);
begin
  if Value <> FLayout then begin
    FLayout := Value;
    AlignWindows;
  end;
end;

procedure TabcCustomSplitBar.SetMax(Value: TabcPosInt);
begin
  if Value <> FMax then begin
    if Value <> 0 then begin
      if Value < (Min + Thickness) then
        FMax := Min + Thickness
      else
        FMax := Value;
      if Pos > FMax then Pos := FMax;
    end
    else
      FMax := 0;
  end;
end;

procedure TabcCustomSplitBar.SetMin(Value: TabcPosInt);
var
  CalcMin: Integer;
begin
  if Value <> FMin then begin
    CalcMin := Value;
    if (Max <> 0) and (CalcMin > Max) then
      CalcMin := Max - Thickness;
    if CalcMin < 0 then CalcMin := 0;
    FMin := CalcMin;
    if Pos < CalcMin then Pos := CalcMin;
  end;
end;

procedure TabcCustomSplitBar.SetPercent(Value: TabcPercent);
var
  CalcPos: TabcPosInt;
begin
  if CanMove then begin
    if Layout = bslVert then
      CalcPos := Round(Value / 100 * Parent.ClientWidth)
    else
      CalcPos := Round(Value / 100 * Parent.ClientHeight);
    Pos := CalcPos;
  end;
end;

procedure TabcCustomSplitBar.SetPos(Value: TabcPosInt);
var
  MovePos: TPoint;
begin
  if CanMove and (Value <> Pos) then begin
    if Layout = bslVert then
      MovePos := Point(Value, Top)
    else
      MovePos := Point(Left, Value);
    MoveSplitPosition(MovePos);
  end;
end;

procedure TabcCustomSplitBar.SetThickness(Value: TabcThickness);

  function Min(I, J: Integer): Integer;
  begin
    if I < J then
      Result := I
    else
      Result := J;
  end;

begin
  if Value <> FThickness then begin
    FThickness := Value;
    if Layout = bslVert then
      if Parent <> nil then
        Width := Min(Value, Parent.ClientWidth - 1)
      else
        Width := Value
    else
      if Parent <> nil then
        Height := Min(Value, Parent.ClientHeight - 1)
      else
        Height := Value;
  end;
end;

procedure TabcCustomSplitBar.SetWindowA(Value: TWinControl);
begin
  if Value <> FWindowA then begin
    FWindowA := Value;
    AlignWindows;
  end;
end;

procedure TabcCustomSplitBar.SetWindowB(Value: TWinControl);
begin
  if Value <> FWindowB then begin
    FWindowB := Value;
    AlignWindows;
  end;
end;

procedure TabcCustomSplitBar.UpdateCursor;
begin
  if CanMove then
    if Layout = bslVert then
      inherited Cursor := crHSplit
    else
      inherited Cursor := crVSplit
  else
    inherited Cursor := FCursor;
end;

procedure TabcCustomSplitBar.UpdatePercent;
begin
  if Parent = nil then
    FPercent := 0
  else begin
    if Layout = bslVert then
      if Left >= (Parent.ClientWidth - Width - 1) then
        FPercent := 100
      else
        FPercent := Round(Left / Parent.ClientWidth * 100)
    else
      if Top >= (Parent.ClientHeight - Height - 1) then
        FPercent := 100
      else
        FPercent := Round(Top / Parent.ClientHeight * 100);
  end;
end;

{ TabcSplitPane }

constructor TabcSplitPane.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csDoubleClicks];
  BevelOuter := bvNone;
  ParentColor := True;
  ParentFont := True;
  ParentShowHint := True;
end;

destructor TabcSplitPane.Destroy;
begin
  if FSplitter <> nil then FSplitter.RemovePane(Self);
  inherited Destroy;
end;

function TabcSplitPane.GetPaneIndex: Integer;
begin
  if FSplitter <> nil then
    Result := FSplitter.FPaneList.IndexOf(Self)
  else
    Result := -1;
end;

procedure TabcSplitPane.SetPaneIndex(Value: Integer);
var
  MaxPaneIndex: Integer;
begin
  if FSplitter <> nil then
  begin
    MaxPaneIndex := FSplitter.FPaneList.Count - 1;
    if Value > MaxPaneIndex then
      raise EListError.CreateFmt(sabcPaneIndexError, [Value, MaxPaneIndex]);
    FSplitter.Move(PaneIndex, Value);
  end;
end;

procedure TabcSplitPane.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  if Reader.Parent is TabcCustomSplit then
    Splitter := TabcCustomSplit(Reader.Parent);
end;

{$IFDEF DELPHI6}
procedure TabcSplitPane.SetParent(AParent: TWinControl);
begin
  if (AParent <> nil) and not (AParent is TabcCustomSplit) then
    Abort
  else
    inherited;
end;
{$ENDIF}

procedure TabcSplitPane.SetSplitter(Value: TabcCustomSplit);
begin
  if FSplitter <> Value then
  begin
    if FSplitter <> nil then FSplitter.RemovePane(Self);
    Parent := Value;
    if Value <> nil then Value.AddPane(Self);
  end;
end;

{ TabcScrollingSplitPane }

constructor TabcScrollingSplitPane.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csDoubleClicks];
  ParentColor := True;
  ParentFont := True;
  ParentShowHint := True;
  BorderStyle := bsNone;
end;

destructor TabcScrollingSplitPane.Destroy;
begin
  if FSplitter <> nil then FSplitter.RemovePane(Self);
  inherited Destroy;
end;

function TabcScrollingSplitPane.GetPaneIndex: Integer;
begin
  if FSplitter <> nil then
    Result := FSplitter.FPaneList.IndexOf(Self)
  else
    Result := -1;
end;

procedure TabcScrollingSplitPane.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  if Reader.Parent is TabcCustomSplit then
    Splitter := TabcCustomSplit(Reader.Parent);
end;

{$IFDEF DELPHI6}
procedure TabcScrollingSplitPane.SetParent(AParent: TWinControl);
begin
  if (AParent <> nil) and not (AParent is TabcCustomSplit) then
    Abort
  else
    inherited;
end;
{$ENDIF}

procedure TabcScrollingSplitPane.SetPaneIndex(Value: Integer);
var
  MaxPaneIndex: Integer;
begin
  if FSplitter <> nil then
  begin
    MaxPaneIndex := FSplitter.FPaneList.Count - 1;
    if Value > MaxPaneIndex then
      raise EListError.CreateFmt(sabcPaneIndexError, [Value, MaxPaneIndex]);
    FSplitter.Move(PaneIndex, Value);
  end;
end;

procedure TabcScrollingSplitPane.SetSplitter(Value: TabcCustomSplit);
begin
  if FSplitter <> Value then
  begin
    if FSplitter <> nil then FSplitter.RemovePane(Self);
    Parent := Value;
    if Value <> nil then Value.AddPane(Self);
  end;
end;

procedure TabcScrollingSplitPane.WMHScroll(var Message: TWMHScroll);
var
  ScrollPos: integer;
begin
  inherited;
  ScrollPos := Message.Pos;
  if FSplitter <> nil then FSplitter.PaneScrollH(Self, TScrollCode(Message.ScrollCode), ScrollPos);
end;

procedure TabcScrollingSplitPane.WMVScroll(var Message: TWMVScroll);
var
  ScrollPos: integer;
begin
  inherited;
  ScrollPos := Message.Pos;
  if FSplitter <> nil then FSplitter.PaneScrollV(Self, TScrollCode(Message.ScrollCode), ScrollPos);
end;

{ TabcCustomSplit }

function TabcCustomSplit.GetMoveImmediate: Boolean;
begin
  Result := FSplitMoveStyle = amsImmediate;
end;

procedure TabcCustomSplit.SetMoveImmediate(Value: Boolean);
begin
  if Value then
    FSplitMoveStyle := amsImmediate
  else if FSplitMoveStyle = amsImmediate then
    FSplitMoveStyle := amsMedium;
end;

constructor TabcCustomSplit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csClickEvents, csSetCaption, csAcceptsControls];
  InitSplitDefaults;
  if ([csLoading,csReading] * ComponentState = []) and
     ([csLoading,csReading] * AOwner.ComponentState = []) then
  begin
    FInitialized := True;
    Invalidate;
  end;
end;

procedure TabcCustomSplit.InitSplitDefaults;
begin
  BevelOuter := bvRaised;
  BevelWidth := 1;
  FCursor := crDefault;
  Width := 100;
  Height := 100;
  FBorderOffset := 1;
  FSplitBevel := bvRaised;
  FSplitColor := clBtnFace;
  FKeyIncrement   := 10;
  FSplitHScaled   := False;
  FSplitVScaled   := False;
  FThickness      := 2;
  FSplitHPos      := 50;
  FSplitHMouseMin := 48;
  FSplitHMouseMax := 51;
  FSplitVPos      := 50;
  FSplitVMouseMin := 48;
  FSplitVMouseMax := 51;
  FPaneList := TList.Create;
end;

procedure TabcCustomSplit.CreatePanes;
{
Virtual method for creating panes, implemented in descendant classes.
Sequence of Panes:
  Panes are stored in FPaneList in the order they are created or read from the
  stream.
  Rotate or Exchange can be used to change the order at runtime.
  Order is very important to the various layouts for tri, quad and multisplit.
  The DoResizePanes method assumes the convention of numbering panes clockwise
  from the top left of the splitter.
}
var
  APane: TWinControl;
  i: integer;
begin
  if (Parent <> nil) and not (csAcceptsControls in Parent.ControlStyle) then Exit;
  for i := FPaneList.Count to MaxPanes - 1 do
  begin
    APane := PaneClass.Create(Owner);
    APane.Parent := Self;
    AddPane(APane);
  end;
end;

procedure TabcCustomSplit.DestroyPanes;
var
  i: integer;
begin
  for i := FPaneList.Count - 1 downto 0 do
    Pane[i].Free;
end;

destructor TabcCustomSplit.Destroy;
begin
  DestroyPanes;
  FPaneList.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure TabcCustomSplit.AddPane(Value: TWinControl);
begin
  FPaneList.Add(Value);
  if Value is TabcSplitPane then
    TabcSplitPane(Value).FSplitter := Self
  else
  if Value is TabcScrollingSplitPane then
    TabcScrollingSplitPane(Value).FSplitter := Self;
  if not (csLoading in ComponentState) then ResizePanes;
end;

procedure TabcCustomSplit.RemovePane(Value: TWinControl);
begin
  if Value is TabcSplitPane then
    TabcSplitPane(Value).FSplitter := nil
  else
  if Value is TabcScrollingSplitPane then
    TabcScrollingSplitPane(Value).FSplitter := nil;
  FPaneList.Remove(Value);
  if not (csDestroying in ComponentState) then ResizePanes;
end;

function TabcCustomSplit.BetweenXMouseLimits(X: Integer): Boolean;
begin
  if (X >= FSplitHMouseMin) and
    ((X <= FSplitHMouseMax) or (FSplitHMouseMax = 0)) then
    Result := True
  else
    Result := False;
end;

function TabcCustomSplit.BetweenYMouseLimits(Y: Integer): Boolean;
begin
  if (Y >= FSplitVMouseMin) and
    ((Y <= FSplitVMouseMax) or (FSplitVMouseMax = 0)) then
    Result := True
  else
    Result := False;
end;

function TabcCustomSplit.CheckSplitLock(ACursor: TCursor): TCursor;
begin
  {checks locking to restrict cursor}
  Result := ACursor;
  case ACursor of
    crHSplit:
      if FSplitHLock then Result := FCursor;
    crVSplit:
      if FSplitVLock then Result := FCursor;
    abccrSize:
      if FSplitHLock then
        if FSplitVLock then
          Result := FCursor
        else
          Result := crVSplit
      else
        if FSplitVLock then
          Result := crHSplit;
  end;
end;

procedure TabcCustomSplit.ClipMovePos(var Pos: TPoint);
var
  AdjustThickness: Integer;
begin
//  if FInitializing then Exit;
  with Pos do begin
    if X > Width - FBorderOffset then
      X := Width - FBorderOffset;
    if (X > FSplitHMax) and (FSplitHMax <> 0) then
      X := FSplitHMax
    else
    if (X < FSplitHMin) then
      X := FSplitHMin;

    if Y > Height - FBorderOffset then
      Y := Height - FBorderOffset;
    if (Y > FSplitVMax) and (FSplitVMax <> 0) then
      Y := FSplitVMax
    else
    if (Y < FSplitVMin) then
      Y := FSplitVMin;

    AdjustThickness := 1 + Thickness div 2 + FBorderOffset;

    if X < AdjustThickness then X := AdjustThickness;
    if Y < AdjustThickness then Y := AdjustThickness;
    if BorderStyle = bsSingle then
      Inc(AdjustThickness, 2);
    if Thickness Mod 2 = 1 then
      Inc(AdjustThickness);
    if X > (Width - AdjustThickness) then X := Width - AdjustThickness;
    if Y > (Height - AdjustThickness) then Y := (Height - AdjustThickness);
  end;
end;

procedure TabcCustomSplit.CMDesignHitTest(var Message: TCMDesignHitTest);
begin
{$IFDEF DELPHI6}
  if FMoving then
    Message.Result := 1
  else if (SplitHLock and SplitVLock) or (FPaneList.Count = 0) then
    inherited
  else
    Message.Result := 0;
{$ELSE}
  Message.Result := 1;
  if (SplitHLock and SplitVLock) or (FPaneList.Count = 0) then inherited;
{$ENDIF}
end;

{$IFDEF DELPHI6}
function TabcCustomSplit.DesignWndProc(var Message: TMessage): Boolean;
begin
  Result :=
    (not SplitHLock or not SplitVLock) and
    (FPaneList.Count > 0) and
    ((Message.Msg = WM_LBUTTONDOWN) or
     ((Message.Msg = WM_LBUTTONUP) and FMoving) or
     (Message.Msg = WM_MOUSEMOVE) or
     (Message.Msg = WM_LBUTTONDBLCLK));
end;
{$ENDIF}

procedure TabcCustomSplit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_CLIPCHILDREN;
end;

procedure TabcCustomSplit.CreateWnd;
begin
  inherited;
  if not (csLoading in ComponentState) and not FInitialized then
    PostMessage(Handle, AM_INITSPLITSIZE, 0, 0);
end;

procedure TabcCustomSplit.DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer);
begin
end;

procedure TabcCustomSplit.DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor);
begin
end;

procedure TabcCustomSplit.DoResizePanes(ARect: TRect);
begin
end;

procedure TabcCustomSplit.DrawDragLines(CRect: TRect; Pos: TPoint);
var
  BOffset: Integer;
  ScrPos: TPoint;
begin
  if BorderStyle = bsNone then
    BOffset := FBorderOffset
  else
    BOffset := FBorderOffset + 1;
  {adjust Pos to Screen Pos}
  with Pos, CRect do begin
    if X < BOffset then
      ScrPos.X := Left + BOffset
    else if X > Width - BOffset then
      ScrPos.X := Right - BOffset
    else
      ScrPos.X := Left + X;
    if Y < BOffset then
      ScrPos.Y := Top + BOffset
    else if Y > Height - BOffset then
      ScrPos.Y := Bottom - BOffset
    else
      ScrPos.Y := Top + Y;
  end;
  InflateRect(CRect, -BOffset, -BOffset);
  DoDrawDragLines(CRect, ScrPos, BOffset);
  FDragLineVisible := not FDragLineVisible;
end;

procedure TabcCustomSplit.Exchange(Index1, Index2: Integer);
begin
  FPaneList.Exchange(Index1, Index2);
  ResizePanes;
end;

procedure TabcCustomSplit.Move(CurrIndex, NewIndex: Integer);
begin
  FPaneList.Move(CurrIndex, NewIndex);
  ResizePanes;
end;

function TabcCustomSplit.GetBevelInner: TPanelBevel; begin Result := inherited BevelInner; end;
function TabcCustomSplit.GetBevelOuter: TPanelBevel; begin Result := inherited BevelOuter; end;
function TabcCustomSplit.GetBevelWidth: Integer; begin Result := inherited BevelWidth; end;
function TabcCustomSplit.GetBorderStyle: TBorderStyle; begin Result := inherited BorderStyle; end;
function TabcCustomSplit.GetBorderWidth: Integer; begin Result := inherited BorderWidth; end;

function TabcCustomSplit.GetMoveCursor(X,Y: Integer): TCursor;
begin
  Result := 0;
end;

function TabcCustomSplit.GetMaxPanes: Integer;
begin
  Result := 0;
end;

function TabcCustomSplit.GetPaneClass: TWinControlClass;
begin
  Result := TWinControl;
end;

function TabcCustomSplit.GetPane(Index: Integer): TWinControl;
begin
  if Index < FPaneList.Count then
    Result := TWinControl(FPaneList.Items[Index])
  else
    Result := nil;
end;

function TabcCustomSplit.GetPaneCount: Integer;
begin
  Result := FPaneList.Count;
end;

function TabcCustomSplit.GetSplitMovePos(var X, Y: Integer): TPoint;
begin
  Result := Point(0,0);
end;

procedure TabcCustomSplit.KeyDown(var Key: Word; Shift: TShiftState);
{
NOTE: This method assumes CM_WANTSPECIALKEYS is used in descendant classes
      to filter the keys
}
var
  NewPos: TPoint;
begin
  if (Key in [VK_LEFT..VK_DOWN]) then begin
    case Key of
      VK_LEFT:
        if (ssShift in Shift) then
          NewPos := Point(0, SplitVPos)
        else
          NewPos := Point(SplitHPos - KeyIncrement, SplitVPos);
      VK_UP:
        if (ssShift in Shift) then
          NewPos := Point(SplitHPos, 0)
        else
          NewPos := Point(SplitHPos, SplitVPos - KeyIncrement);
      VK_RIGHT:
        if (ssShift in Shift) then
          NewPos := Point(Width, SplitVPos)
        else
          NewPos := Point(SplitHPos + KeyIncrement, SplitVPos);
      VK_DOWN:
        if (ssShift in Shift) then
          NewPos := Point(SplitHPos, Height)
        else
          NewPos := Point(SplitHPos, SplitVPos + KeyIncrement);
    end;
    MoveSplitPosition(NewPos);
    Key := 0;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TabcCustomSplit.Loaded;
begin
  inherited Loaded;
  UpdateBorderOffset;
  ResizePanes;
  PostMessage(Handle, AM_INITSPLITSIZE, 0, 0);
end;

procedure TabcCustomSplit.AMInitSplitSize(var Msg: TMessage);
begin
  UpdateSplitHPos(SplitHPos);
  UpdateSplitVPos(SplitVPos);
  ResizePanes;
  FInitialized := True;
end;

procedure TabcCustomSplit.AllocMoveDC;
begin
  FMoveDC := GetDCEx(Parent.Handle, 0, DCX_CACHE or DCX_CLIPSIBLINGS
    or DCX_LOCKWINDOWUPDATE);

  if SplitMoveStyle in [amsDark..amsLight]  then
  begin
    if FBrush = nil then
    begin
      FBrush := TBrush.Create;
      FBrush.Bitmap := AllocPatternBitmap(
        clBlack, DRAG_BRUSH_COLORS[SplitMoveStyle]);
    end;
    FSaveBrush := SelectObject(FMoveDC, FBrush.Handle);
  end;
end;

procedure TabcCustomSplit.FreeMoveDC;
begin
  if FSaveBrush <> 0 then
    SelectObject(FMoveDC, FSaveBrush);
  ReleaseDC(Parent.Handle, FMoveDC);
  if FBrush <> nil then
  begin
    FBrush.Free;
    FBrush := nil;
  end;
end;

procedure TabcCustomSplit.StopMoving;
begin
  if FDragLineVisible then MoveDragLines(FLastMovePos);
  FreeMoveDC;
end;

procedure TabcCustomSplit.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if ((Button = mbLeft) and
      (not FSplitHLock or not FSplitVLock or (csDesigning in ComponentState))) then
      if FMoveCursor <> FCursor then begin
        AllocMoveDC;
        FLastMovePos := GetSplitMovePos(X, Y);
        if (SplitMoveStyle <> amsImmediate) then
          MoveDragLines(FLastMovePos);                     //check cursor change???
        FMoving := True;
      end;
    inherited MouseDown(Button, Shift, X, Y);
end;

procedure TabcCustomSplit.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  MovePos: TPoint;
begin
  if FDrawing then Exit;
  if FMoving then
    if SplitMoveStyle = amsImmediate then begin
      MovePos := GetSplitMovePos(X, Y);
      if (MovePos.X <> FLastMovePos.X) or (MovePos.Y <> FLastMovePos.Y) then begin
        FLastMovePos := MovePos;
        MoveSplitPosition(MovePos);
      end;
    end
    else
      MoveDragLines(Point(X,Y))
  else begin
    FMoveCursor := CheckSplitLock(GetMoveCursor(X,Y));
    inherited Cursor := FMoveCursor;
  end;
end;

procedure TabcCustomSplit.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FMoving then begin
    FMoving := False;
    StopMoving;
    MoveSplitPosition(GetSplitMovePos(X, Y));
    inherited Cursor := FCursor;
  end;
  inherited MouseUp(Button, Shift, X, Y);
end;

procedure TabcCustomSplit.MoveDragLines(Pos: TPoint);
var
  ARect: TRect;
  AdjustedPos: TPoint;
begin
  FDrawing := True;
  AdjustedPos := Pos;
  ClipMovePos(AdjustedPos);
  GetWindowRect(Handle, ARect);
  if SplitMoveStyle in [amsBlack..amsLight] then
  begin
    if FMoving then DrawDragLines(ARect, FLastMovePos);
    DrawDragLines(ARect, AdjustedPos);
  end;
  FLastMovePos := AdjustedPos;
  FDrawing := False;
end;

procedure TabcCustomSplit.MoveSplitPosition(Pos: TPoint);
var
  AdjustedPos: TPoint;
begin
  AdjustedPos := Pos;
  ClipMovePos(AdjustedPos);
  SplitHPos := AdjustedPos.X;
  SplitVPos := AdjustedPos.Y;
  abcDesignerModified(Self);
end;

procedure TabcCustomSplit.Paint;
var
  ARect: TRect;
  TopColor, BottomColor: TColor;

  procedure AdjustSplitColors(Bevel: TBevelCut);
  begin
    case Bevel of
      bvRaised:
      begin
        TopColor := clBtnHighlight;
        BottomColor := clBtnShadow;
      end;
      bvLowered:
      begin
        TopColor := clBtnShadow;
        BottomColor := clBtnHighlight;
      end;
    else {bvNone, bvSpace}
      begin
        TopColor := Color;
        BottomColor := Color;
      end;
    end;
  end;

begin
  with Canvas.Pen do begin
    Width := 1;
    Mode := pmCopy;
    Color := clBlack;
  end;
  inherited Paint;
  AdjustSplitColors(FSplitBevel);
  ARect := GetClientRect;
  InflateRect(ARect, -FBorderOffset, -FBorderOffset);
  DoPaintSplitLines(Canvas, ARect, TopColor, BottomColor);
end;

procedure TabcCustomSplit.ResizePanes;
var
  ARect: TRect;
begin
  ARect := ClientRect;
  InflateRect(ARect, -FBorderOffset, -FBorderOffset);
  DoResizePanes(ARect);
  if not (csLoading in ComponentState) then Invalidate;
end;

procedure TabcCustomSplit.Rotate(Clockwise: Boolean);
begin
  if FPaneList.Count > 1 then
    if Clockwise then
      Move(FPaneList.Count - 1, 0)
    else
      Move(0, FPaneList.Count - 1);
end;

procedure TabcCustomSplit.SetBevelInner(Value: TPanelBevel);
begin
  if Value <> BevelInner then begin
    inherited BevelInner := Value;
    UpdateBorderOffset;
  end;
end;

procedure TabcCustomSplit.SetBevelOuter(Value: TPanelBevel);
begin
  if Value <> BevelOuter then begin
    inherited BevelOuter := Value;
    UpdateBorderOffset;
  end;
end;

procedure TabcCustomSplit.SetBevelWidth(Value: Integer);
begin
  if Value <> BevelWidth then begin
    inherited BevelWidth := Value;
    UpdateBorderOffset;
  end;
end;

procedure TabcCustomSplit.SetBorderStyle(Value: TBorderStyle);
begin
  if Value <> BorderStyle then begin
    inherited BorderStyle := Value;
    UpdateBorderOffset;
  end;
end;

procedure TabcCustomSplit.SetBorderWidth(Value: Integer);
begin
  if Value <> BorderWidth then begin
    inherited BorderWidth := Value;
    UpdateBorderOffset;
  end;
end;

procedure TabcCustomSplit.SetCursor(Value: TCursor);
var
  I: integer;
begin
  if (Value <> FCursor) then begin
    FCursor := Value;
    inherited Cursor := Value;
    {set cursor on Panes}
    if not (csLoading in ComponentState) then
      for I := 0 to FPaneList.Count - 1 do
        Pane[I].Cursor := Value;
  end;
end;

procedure TabcCustomSplit.SetPaneBounds(APaneIndex, ALeft, ATop, AWidth, AHeight: Integer);
begin
  if APaneIndex < FPaneList.Count then
    Pane[APaneIndex].SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TabcCustomSplit.SetSplitHMin(Value: TabcPosInt);
begin
  if Value <> FSplitHMin then
    if Value = 0 then
      FSplitHMin := Value
    else
      if ((Value <= FSplitHMax) or (FSplitHMax = 0)) and (Value < Width) then begin
        FSplitHMin := Value;
        if Value > FSplitHPos then SplitHPos := Value;
      end;
end;

procedure TabcCustomSplit.SetSplitHMax(Value: TabcPosInt);
begin
  if Value <> FSplitHMax then
    if Value = 0 then
      FSplitHMax := Value
    else
      if Value >= FSplitHMin then begin
        if (Value <= Width) then
          FSplitHMax := Value
        else
          FSplitHMax := Width;
        if FSplitHMax < FSplitHPos then SplitHPos := Value;
      end;
end;

procedure TabcCustomSplit.SetSplitHPos(Value: TabcPosInt);
begin
  if Value <> SplitHPos then begin
    UpdateSplitHPos(Value);
    if Assigned(FOnSplitMoveH) then FOnSplitMoveH(Self);
  end;
end;

procedure TabcCustomSplit.SetSplitHPercent(Value: TabcPercent);
var
  CalcPos: TabcPosInt;
begin
  if (Value = 100) or (Width = 0) then
    CalcPos := Width
  else
    CalcPos := Round(Value / 100 * Width);
  SplitHPos := CalcPos;
end;

procedure TabcCustomSplit.SetSplitVMin(Value: TabcPosInt);
begin
  if Value <> FSplitVMin then
    if Value = 0 then
      FSplitVMin := Value
    else
      if ((Value <= FSplitVMax) or (FSplitVMax = 0)) and (Value < Height)  then begin
          FSplitVMin := Value;
          if Value > FSplitVPos then SplitVPos := Value;
      end;
end;

procedure TabcCustomSplit.SetSplitVMax(Value: TabcPosInt);
begin
  if Value <> FSplitVMax then
    if Value = 0 then
      FSplitVMax := Value
    else
      if Value >= FSplitVMin then begin
        if (Value <= Height) then
          FSplitVMax := Value
        else
          FSplitVMax := Height;
        if FSplitVMax < FSplitVPos then SplitVPos := Value;
      end;
end;

procedure TabcCustomSplit.SetSplitVPos(Value: TabcPosInt);
begin
  if Value <> SplitVPos then begin
    UpdateSplitVPos(Value);
    if Assigned(FOnSplitMoveV) then FOnSplitMoveV(Self);
  end;
end;

procedure TabcCustomSplit.SetSplitVPercent(Value: TabcPercent);
var
  CalcPos: TabcPosInt;
begin
  if (Value = 100)  or (Height = 0) then
    CalcPos := Height
  else
    CalcPos := Round(Value / 100 * Height);
  SplitVPos := CalcPos;
end;

procedure TabcCustomSplit.SetSplitBevel(Value: TBevelCut);
begin
  if Value <> FSplitBevel then begin
    FSplitBevel := Value;
    if not (csLoading in ComponentState) then Invalidate;
  end;
end;

procedure TabcCustomSplit.SetSplitColor(Value: TColor);
begin
  if Value <> FSplitColor then begin
    FSplitColor := Value;
    if not (csLoading in ComponentState) then Invalidate;
  end;
end;

procedure TabcCustomSplit.SetThickness(Value: TabcThickness);
begin
  if Value <> FThickness then begin
    FThickness := Value;
    UpdateSplitHPos(SplitHPos);
    UpdateSplitVPos(SplitVPos);
  end;
end;

procedure TabcCustomSplit.UpdateBorderOffset;
begin
  FBorderOffset := 0;
  Inc(FBorderOffset, BorderWidth);
  if BevelOuter <> bvNone then
    Inc(FBorderOffset, BevelWidth);
  if BevelInner <> bvNone then
    Inc(FBorderOffset, BevelWidth);
  if not (csLoading in ComponentState) then
  begin
    UpdateSplitHPos(SplitHPos);
    UpdateSplitVPos(SplitVPos);
  end;
end;

procedure TabcCustomSplit.UpdateSplitHPos(Value: TabcPosInt);
var
  AdjustedPos: TPoint;
begin
  if (csLoading in ComponentState) then
    FSplitHPos := Value
  else
    if (Width <> 0) then
    begin
      AdjustedPos := Point(Value, 0);
      ClipMovePos(AdjustedPos);
      FSplitHPos := AdjustedPos.X;
      FSplitHPercent := Round (FSplitHPos / Width * 100 );
      FSplitHMouseMin := FSplitHPos - 1 - Thickness div 2;
      FSplitHMouseMax := FSplitHPos + (Thickness + 1) div 2;
      if FInitialized then
        ResizePanes;
    end;
end;

procedure TabcCustomSplit.UpdateSplitVPos(Value: TabcPosInt);
var
  AdjustedPos: TPoint;
begin
  if (csLoading in ComponentState) then
    FSplitVPos := Value
  else
    if (Height <> 0) then
    begin
      AdjustedPos := Point(0,Value);
      ClipMovePos(AdjustedPos);
      FSplitVPos := AdjustedPos.Y;
      FSplitVPercent := Round (FSplitVPos / Height * 100 );
      FSplitVMouseMin := FSplitVPos - 1 - Thickness div 2;
      FSplitVMouseMax := FSplitVPos + (Thickness + 1) div 2;
      if FInitialized then
        ResizePanes;
    end;
end;

procedure TabcCustomSplit.WMNCHitTest(var Msg: TWMNCHitTest);
begin
  FHitTest := SmallPointtoPoint(Msg.Pos);
  inherited;
end;

procedure TabcCustomSplit.WMSetCursor(var Msg: TWMSetCursor);
var
  Cur: HCURSOR;
  PosCursor: TCursor;
begin
  Cur := 0;
  if (csDesigning in ComponentState) and (Msg.HitTest = HTCLIENT) then
  begin
    FHitTest := ScreenToClient(FHitTest);
    PosCursor := CheckSplitLock(GetMoveCursor(FHitTest.X, FHitTest.Y));
    case PosCursor of
      crDefault:
        Cur := 0; {allow designer grab handles to set the cursor when over a child control}
      abccrSize:
        Cur := Screen.Cursors[abccrSize];
      crHSplit:
        Cur := Screen.Cursors[crHSplit];
      crVSplit:
        Cur := Screen.Cursors[crVSplit];
    end;
  end;
  if Cur <> 0 then
    Windows.SetCursor(Cur)
  else
    inherited;
end;

procedure TabcCustomSplit.WMSize(var Message: TWMSize);
var
  NewPos: TPoint;
begin
  inherited;
  if not ((csLoading in ComponentState) or (Width = 0) or (Height = 0)) then
  begin
    if SplitHScaled then
      NewPos.X := Round(Width / 100 * SplitHPercent)
    else
      NewPos.X := SplitHPos;
    if SplitVScaled then
      NewPos.Y := Round(Height / 100 * SplitVPercent)
    else
      NewPos.Y := SplitVPos;
    MoveSplitPosition(NewPos);
    ResizePanes;
  end;
end;

procedure TabcCustomSplit.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: integer;
begin
  for I := 0 to FPaneList.Count - 1 do
    Proc(Pane[I]);
end;

procedure TabcCustomSplit.PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
begin
end;

procedure TabcCustomSplit.PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
begin
end;

procedure TabcCustomSplit.DrawOneDragLineAt(DrawRect: TRect);
//temporary hack
//change coords higher up to use parent not screen dims
var
  DrawPt: TPoint;
begin
  DrawPt := Parent.ScreenToClient(DrawRect.TopLeft);
  with DrawPt, DrawRect do
    PatBlt(FMoveDC, X, Y, Right - Left, Bottom - Top, PATINVERT);
end;

{ TabcCustomBiSplit }

procedure TabcCustomBiSplit.CMWantSpecialKey(var Msg: TCMWantSpecialKey);
begin
  inherited;
  with Msg do
    if ((Layout = bslVert) and
        (CharCode in [VK_LEFT, VK_RIGHT]) and not SplitHLock)
        or
       ((Layout = bslHorz) and
        (CharCode in [VK_UP, VK_DOWN]) and not SplitVLock) then
      Result := 1;
end;

procedure TabcCustomBiSplit.DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer);
var
  DrawRect: TRect;
  DragPos: TPoint;
begin
  {centre pos to drag thickness}
  DragPos.X := ScrPos.X - Thickness div 2;
  DragPos.Y := ScrPos.Y - Thickness div 2;

  with DragPos, CRect do
    case MoveCursor of
      crHSplit:
        begin
          DrawRect := Rect(X, Top, X + Thickness, Bottom);
          DrawOneDragLineAt(DrawRect);
        end;
      crVSplit:
        begin
          DrawRect := Rect(Left, Y, Right, Y + Thickness);
          DrawOneDragLineAt(DrawRect);
        end;
    end;
end;

procedure TabcCustomBiSplit.DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor);
begin
  with ACanvas, ARect do
    if FLayout = bslVert then
    begin
      Pen.Color := TopColor;
      MoveTo(SplitHMouseMin, Top);
      LineTo(SplitHMouseMin, Bottom);
      Pen.Color := BottomColor;
      MoveTo(SplitHMouseMax, Bottom - 1);
      LineTo(SplitHMouseMax, Top - 1);
      if SplitColor <> Color then
      begin
        Brush.Color := SplitColor;
        ARect.Left := SplitHMouseMin + 1;
        ARect.Right := SplitHMouseMax;
        FillRect(ARect);
      end;
    end
    else
    begin
      Pen.Color := TopColor;
      MoveTo(Left, SplitVMouseMin);
      LineTo(Right, SplitVMouseMin);
      Pen.Color := BottomColor;
      MoveTo(Right - 1, SplitVMouseMax);
      LineTo(Left - 1, SplitVMouseMax);
      if SplitColor <> Color then
      begin
        Brush.Color := SplitColor;
        ARect.Top := SplitVMouseMin + 1;
        ARect.Bottom := SplitVMouseMax;
        FillRect(ARect);
      end;
    end;
end;

procedure TabcCustomBiSplit.DoResizePanes(ARect: TRect);
var
  CWidth, CHeight: integer;
begin
  with ARect do
  begin
    CWidth := Right - Left;
    CHeight := Bottom - Top;
    if FLayout = bslVert then
    begin
      SetPaneBounds(0, Left, Top, SplitHMouseMin - BorderOffset, CHeight);
      SetPaneBounds(1, SplitHMouseMax + 1, Top,
        (Right - SplitHMouseMax - 1), CHeight);
    end
    else
    begin
      SetPaneBounds(0, Left, Top, CWidth,
        SplitVMouseMin - BorderOffset);
      SetPaneBounds(1, Left, SplitVMouseMax + 1,
        CWidth, (Bottom - SplitVMouseMax - 1));
    end;
  end;
end;

function TabcCustomBiSplit.GetMoveCursor(X,Y: Integer): TCursor;
begin
  if FLayout = bslVert then
    if BetweenXMouseLimits(X) then
      Result := crHSplit
    else
      Result := Cursor
  else
    if BetweenYMouseLimits(Y) then
      Result := crVSplit
    else
      Result := Cursor;
end;

function TabcCustomBiSplit.GetMaxPanes: Integer;
begin
  Result := 2;
end;

function TabcCustomBiSplit.GetSplitMovePos(var X, Y: Integer): TPoint;
begin
  case MoveCursor of
    crHSplit:
      Result := Point(X,SplitVPos);
    crVSplit:
      Result := Point(SplitHPos,Y);
  end;
end;

procedure TabcCustomBiSplit.SetLayout(Value: TabcBiSplitLayout);
begin
  if Value <> FLayout then
  begin
    FLayout := Value;
    if not (csLoading in ComponentState) then ResizePanes;
  end;
end;

{ TabcBiSplit }

function TabcBiSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcSplitPane;
end;

function TabcBiSplit.GetSplitPane(Index: Integer): TabcSplitPane;
begin
  Result := TabcSplitPane(inherited GetPane(Index));
end;

{ TabcSBiSplit }

function TabcSBiSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcScrollingSplitPane;
end;

function TabcSBiSplit.GetScrollPane(Index: Integer): TabcScrollingSplitPane;
begin
  Result := TabcScrollingSplitPane(inherited GetPane(Index));
end;

procedure TabcSBiSplit.PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: integer;
begin
  if FSyncHScroll and not FPaneScrolling then
  begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_HSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

procedure TabcSBiSplit.PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: Integer;
begin
  if FSyncVScroll and not FPaneScrolling then
  begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_VSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

{ TabcCustomTriSplit }

procedure TabcCustomTriSplit.CMWantSpecialKey(var Msg: TCMWantSpecialKey);
begin
  inherited;
  with Msg do
    if ((CharCode in [VK_LEFT, VK_RIGHT]) and not SplitHLock) or
        ((CharCode in [VK_UP, VK_DOWN]) and not SplitVLock) then
      Result := 1;
end;

procedure TabcCustomTriSplit.DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer);
var
  DrawRect: TRect;
  DragPos: TPoint;
begin
  {centre pos to drag thickness}
  DragPos.X := ScrPos.X - Thickness div 2;
  DragPos.Y := ScrPos.Y - Thickness div 2;

  with DragPos, CRect do
    case MoveCursor of
      crHSplit:
        case FLayout of
          tslLeftTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              if X < (Left + SplitHMouseMin - BOffset - Thickness) then begin
                DrawRect := Rect(X + Thickness,
                  Top + SplitVPos - BOffset - Thickness div 2,
                  Left + SplitHMouseMin - BOffset,
                  Top + SplitVPos - BOffset + (Thickness + 1) div 2);
                DrawOneDragLineAt(DrawRect);
              end;
            end;
          tslRightTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              if X > (Left + SplitHMouseMax - BOffset + 1) then begin
                DrawRect := Rect(Left + SplitHMouseMax - BOffset + 1,
                  Top + SplitVPos - BOffset - Thickness div 2,
                  X,
                  Top + SplitVPos - BOffset + (Thickness + 1) div 2);
                DrawOneDragLineAt(DrawRect);
              end;
            end;
          tslLeftRightBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Top + SplitVMouseMin - BOffset);
              DrawOneDragLineAt(DrawRect);
            end;
          tslTopLeftRight:
            begin
              DrawRect := Rect(X, Top + SplitVMouseMax - BOffset + 1, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
            end;
        end;
      crVSplit:
        case FLayout of
         tslLeftTopBottom:
          begin
            DrawRect := Rect(Left + SplitHMouseMax - BOffset + 1, Y, Right, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
          end;
         tslRightTopBottom:
          begin
            DrawRect := Rect(Left, Y, Left + SplitHMouseMin - BOffset, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
          end;
         tslLeftRightBottom:
          begin
            DrawRect := Rect(Left, Y, Right, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
            if (Y > Top + SplitVMouseMax - BOffset + 1) then begin
              DrawRect := Rect(Left + SplitHPos - BOffset - Thickness div 2,
                Top + SplitVMouseMax - BOffset + 1,
                Left + SplitHPos - BOffset + (Thickness + 1) div 2,
                Y);
              DrawOneDragLineAt(DrawRect);
            end;
          end;
         tslTopLeftRight:
          begin
            DrawRect := Rect(Left, Y, Right, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
            if (Y < Top + SplitVMouseMin - BOffset - Thickness) then begin
              DrawRect := Rect(Left + SplitHPos - BOffset - Thickness div 2,
                Y + Thickness,
                Left + SplitHPos - BOffset + (Thickness + 1) div 2,
                Top + SplitVMouseMin - BOffset);
              DrawOneDragLineAt(DrawRect);
            end;
          end;
        end;
      abccrSize:
        case FLayout of
         tslLeftTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(X + Thickness, Y, Right, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
            end;
          tslRightTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(Left, Y, X, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
            end;
          tslLeftRightBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Y);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(Left, Y, Right, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
            end;
          tslTopLeftRight:
            begin
              DrawRect := Rect(Left, Y, Right, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(X, Y + Thickness, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
            end;
        end;
    end;
end;

procedure TabcCustomTriSplit.DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor);
begin
  with ACanvas, ARect do
    case FLayout of

      tslLeftTopBottom:
        begin
          Pen.Color := TopColor;
          MoveTo(SplitHMouseMin, Top);
          LineTo(SplitHMouseMin, Bottom);
          MoveTo(SplitHMouseMax, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          Pen.Color := BottomColor;
          MoveTo(SplitHMouseMax, Top);
          LineTo(SplitHMouseMax, SplitVMouseMin);
          MoveTo(Right - 1, SplitVMouseMax);
          LineTo(SplitHMouseMax, SplitVMouseMax);
          LineTo(SplitHMouseMax, Bottom);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            FillRect(ARect);
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            ARect.Left := SplitHMouseMax;
            if BorderStyle = bsNone then
              ARect.Right := Width - BorderOffset
            else
              ARect.Right := Width - BorderOffset - 2;
            FillRect(ARect);
          end;
        end;

      tslRightTopBottom:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(SplitHMouseMin, SplitVMouseMin);
          LineTo(SplitHMouseMin, Top - 1);
          MoveTo(SplitHMouseMin, SplitVMouseMax + 1);
          LineTo(SplitHMouseMin, Bottom);
          Pen.Color := BottomColor;
          MoveTo(Left, SplitVMouseMax);
          LineTo(SplitHMouseMin + 1, SplitVMouseMax);
          MoveTo(SplitHMouseMax, Top);
          LineTo(SplitHMouseMax, Bottom);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            ARect.Right := SplitHMouseMin + 1;
            FillRect(ARect);
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            ARect.Top := BorderOffset;
            if BorderStyle = bsNone then
              ARect.Bottom := Height - BorderOffset
            else
              ARect.Bottom := Height - BorderOffset - 2;
            FillRect(ARect);
          end;
        end;

      tslLeftRightBottom:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(SplitHMouseMin, SplitVMouseMin);
          LineTo(SplitHMouseMin, Top - 1);
          MoveTo(SplitHMouseMax, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          Pen.Color := BottomColor;
          MoveTo(SplitHMouseMax, Top);
          LineTo(SplitHMouseMax, SplitVMouseMin);
          MoveTo(Left, SplitVMouseMax);
          LineTo(Right, SplitVMouseMax);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            ARect.Bottom := SplitVMouseMax;
            FillRect(ARect);
            ARect.Left := BorderOffset;
            if BorderStyle = bsNone then
              ARect.Right := Width - BorderOffset
            else
              ARect.Right := Width - BorderOffset - 2;
            ARect.Top := SplitVMouseMin + 1;
            FillRect(ARect);
          end;
        end;

      tslTopLeftRight:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          MoveTo(SplitHMouseMin, SplitVMouseMax + 1);
          LineTo(SplitHMouseMin, Bottom);
          Pen.Color := BottomColor;
          MoveTo(Left, SplitVMouseMax);
          LineTo(SplitHMouseMin + 1, SplitVMouseMax);
          MoveTo(Right - 1, SplitVMouseMax);
          LineTo(SplitHMouseMax, SplitVMouseMax);
          LineTo(SplitHMouseMax, Bottom);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            FillRect(ARect);
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            if BorderStyle = bsNone then
              ARect.Bottom := Height - BorderOffset
            else
              ARect.Bottom := Height - BorderOffset - 2;
            FillRect(ARect);
          end;
        end;
    end;
end;

procedure TabcCustomTriSplit.DoResizePanes(ARect: TRect);
var
  CWidth, CHeight: integer;
begin
  with ARect do begin
    CWidth := Right - Left;
    CHeight := Bottom - Top;
    case FLayout of

       tslLeftTopBottom:
        begin
          SetPaneBounds(0, Left, Top,
            SplitHMouseMin - BorderOffset, CHeight);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), SplitVMouseMin - BorderOffset);
          SetPaneBounds(2, SplitHMouseMax + 1,SplitVMouseMax + 1,
            (Right - SplitHMouseMax - 1),(Bottom - SplitVMouseMax - 1));
        end;

      tslRightTopBottom:
        begin
          SetPaneBounds(0, Left, Top,
            SplitHMouseMin - BorderOffset, SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), CHeight);
          SetPaneBounds(2, Left, SplitVMouseMax + 1,
            SplitHMouseMin - BorderOffset, (Bottom - SplitVMouseMax - 1));
        end;

      tslLeftRightBottom:
        begin
          SetPaneBounds(0, Left, Top,
            SplitHMouseMin - BorderOffset, SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), SplitVMouseMin - BorderOffset);
          SetPaneBounds(2, Left, SplitVMouseMax + 1,
            CWidth, (Bottom - SplitVMouseMax - 1));
        end;

      tslTopLeftRight:
        begin
          SetPaneBounds(0, Left, Top, CWidth, SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, SplitHMouseMax + 1, SplitVMouseMax + 1,
            (Right - SplitHMouseMax - 1), (Bottom - SplitVMouseMax - 1));
          SetPaneBounds(2, Left, SplitVMouseMax + 1,
            SplitHMouseMin - BorderOffset, (Bottom - SplitVMouseMax - 1));
        end;
    end;
  end;
end;

function TabcCustomTriSplit.GetMoveCursor(X,Y: Integer): TCursor;
begin
  case FLayout of

    tslLeftTopBottom:
      if BetweenXMouseLimits(X) then
        if BetweenYMouseLimits(Y) then
          Result := abccrSize
        else
          Result := crHSplit
      else
        if BetweenYMouseLimits(Y) and
          (X > SplitHMouseMax) then
          Result := crVsplit
        else
          Result := Cursor;

    tslRightTopBottom:
      if BetweenXMouseLimits(X) then
        if BetweenYMouseLimits(Y) then
          Result := abccrSize
        else
          Result := crHSplit
      else
        if BetweenYMouseLimits(Y) and
          (X < SplitHMouseMin) then
            Result := crVsplit
        else
          Result := Cursor;

    tslLeftRightBottom:
      if BetweenYMouseLimits(Y) then
        if BetweenXMouseLimits(X) then
          Result := abccrSize
        else
          Result := crVSplit
      else
        if BetweenXMouseLimits(X) and
          (Y < SplitVMouseMin) then
          Result := crHSplit
        else
          Result := Cursor;

    tslTopLeftRight:
      if BetweenYMouseLimits(Y) then
        if BetweenXMouseLimits(X) then
          Result := abccrSize
        else
          Result := crVSplit
      else
        if BetweenXMouseLimits(X) and
          (Y > SplitVMouseMax) then
          Result := crHSplit
        else
          Result := Cursor;
  else
    Result := Cursor;
  end;
end;

function TabcCustomTriSplit.GetMaxPanes: Integer;
begin
  Result := 3;
end;

function TabcCustomTriSplit.GetSplitMovePos(var X, Y: Integer): TPoint;
begin
  case MoveCursor of
    crHSplit:
      Result := Point(X,SplitVPos);
    crVSplit:
      Result := Point(SplitHPos,Y);
    abccrSize:
      Result := Point(X,Y);
  end;
end;

procedure TabcCustomTriSplit.SetLayout(Value: TabcTriSplitLayout);
begin
  if Value <> FLayout then begin
    FLayout := Value;
    if not (csLoading in ComponentState) then ResizePanes;
  end;
end;

{ TabcTriSplit }

function TabcTriSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcSplitPane;
end;

function TabcTriSplit.GetSplitPane(Index: Integer): TabcSplitPane;
begin
  Result := TabcSplitPane(inherited GetPane(Index));
end;

{ TabcSTriSplit }

function TabcSTriSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcScrollingSplitPane;
end;

function TabcSTriSplit.GetScrollPane(Index: Integer): TabcScrollingSplitPane;
begin
  Result := TabcScrollingSplitPane(inherited GetPane(Index));
end;

procedure TabcSTriSplit.PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: integer;
begin
  if FSyncHScroll and not FPaneScrolling then begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_HSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

procedure TabcSTriSplit.PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: Integer;
begin
  if FSyncVScroll and not FPaneScrolling then begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_VSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

{ TabcCustomQuadSplit }

procedure TabcCustomQuadSplit.CMWantSpecialKey(var Msg: TCMWantSpecialKey);
begin
  inherited;
  with Msg do
    if ((CharCode in [VK_LEFT, VK_RIGHT]) and not SplitHLock) or
       ((CharCode in [VK_UP, VK_DOWN]) and not SplitVLock) then
      Result := 1;
end;

procedure TabcCustomQuadSplit.DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer);
var
  DrawRect: TRect;
  DragPos: TPoint;
begin
  {centre pos to drag thickness}
  DragPos.X := ScrPos.X - Thickness div 2;
  DragPos.Y := ScrPos.Y - Thickness div 2;

  with DragPos, CRect do
    case MoveCursor of
      crHSplit:
        begin
          DrawRect := Rect(X, Top, X + Thickness, Bottom);
          DrawOneDragLineAt(DrawRect);
        end;
      crVSplit:
        begin
          DrawRect := Rect(Left, Y, Right, Y + Thickness);
          DrawOneDragLineAt(DrawRect);
        end;
      abccrSize:
        begin
          DrawRect := Rect(X, Top, X + Thickness, Bottom);
          DrawOneDragLineAt(DrawRect);
          DrawRect := Rect(Left, Y, Right, Y + Thickness);
          DrawOneDragLineAt(DrawRect);
        end;
    end;
end;

procedure TabcCustomQuadSplit.DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor);
begin
  with ACanvas, ARect do begin
    Pen.Color := TopColor;
    MoveTo(Left, SplitVMouseMin);
    LineTo(SplitHMouseMin, SplitVMouseMin);
    LineTo(SplitHMouseMin, Top - 1);
    MoveTo(SplitHMouseMax, SplitVMouseMin);
    LineTo(Right, SplitVMouseMin);
    MoveTo(SplitHMouseMin, SplitVMouseMax + 1);
    LineTo(SplitHMouseMin, Bottom);
    Pen.Color := BottomColor;
    MoveTo(Left, SplitVMouseMax);
    LineTo(SplitHMouseMin + 1, SplitVMouseMax);
    MoveTo(SplitHMouseMax, Top);
    LineTo(SplitHMouseMax, SplitVMouseMin);
    MoveTo(Right - 1, SplitVMouseMax);
    LineTo(SplitHMouseMax, SplitVMouseMax);
    LineTo(SplitHMouseMax, Bottom);
    if SplitColor <> Color then begin
      Brush.Color := SplitColor;
      ARect.Top := SplitVMouseMin + 1;
      ARect.Bottom := SplitVMouseMax;
      FillRect(ARect);
      ARect.Left := SplitHMouseMin + 1;
      ARect.Right := SplitHMouseMax;
      ARect.Top := BorderOffset;
      if BorderStyle = bsNone then
        ARect.Bottom := Height - BorderOffset
      else
        ARect.Bottom := Height - BorderOffset - 2;
      FillRect(ARect);
    end;
  end;
end;

procedure TabcCustomQuadSplit.DoResizePanes(ARect: TRect);
begin
  with ARect do begin
    SetPaneBounds(0, Left, Top,
      SplitHMouseMin - BorderOffset, SplitVMouseMin - BorderOffset);
    SetPaneBounds(1, SplitHMouseMax + 1, Top,
      (Right - SplitHMouseMax - 1), SplitVMouseMin - BorderOffset);
    SetPaneBounds(2, SplitHMouseMax + 1, SplitVMouseMax + 1,
      (Right - SplitHMouseMax - 1), (Bottom - SplitVMouseMax - 1));
    SetPaneBounds(3, Left, SplitVMouseMax + 1,
      SplitHMouseMin - BorderOffset, (Bottom - SplitVMouseMax - 1));
  end;
end;

function TabcCustomQuadSplit.GetMoveCursor(X,Y: Integer): TCursor;
begin
  if BetweenXMouseLimits(X) then
    if BetweenYMouseLimits(Y) then
      Result := abccrSize
    else
      Result := crHSplit
  else
    if BetweenYMouseLimits(Y) then
      Result := crVSplit
    else
      Result := Cursor;
end;

function TabcCustomQuadSplit.GetMaxPanes: Integer;
begin
  Result := 4;
end;

function TabcCustomQuadSplit.GetSplitMovePos(var X, Y: Integer): TPoint;
begin
  case MoveCursor of
    crHSplit:
      Result := Point(X,SplitVPos);
    crVSplit:
      Result := Point(SplitHPos,Y);
    abccrSize:
      Result := Point(X,Y);
  end;
end;

{ TabcQuadSplit }

function TabcQuadSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcSplitPane;
end;

function TabcQuadSplit.GetSplitPane(Index: Integer): TabcSplitPane;
begin
  Result := TabcSplitPane(inherited GetPane(Index));
end;

{ TabcSQuadSplit }

function TabcSQuadSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcScrollingSplitPane;
end;

function TabcSQuadSplit.GetScrollPane(Index: Integer): TabcScrollingSplitPane;
begin
  Result := TabcScrollingSplitPane(inherited GetPane(Index));
end;

procedure TabcSQuadSplit.PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: integer;
begin
  if FSyncHScroll and not FPaneScrolling then begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_HSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

procedure TabcSQuadSplit.PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: Integer;
begin
  if FSyncVScroll and not FPaneScrolling then begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_VSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

{ TabcCustomMultiSplit }

constructor TabcCustomMultiSplit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLayout := mslFourWay;
end;

procedure TabcCustomMultiSplit.CMWantSpecialKey(var Msg: TCMWantSpecialKey);
begin
  inherited;
  with Msg do
    if (Layout <> mslSingle) and
       (((CharCode in [VK_LEFT, VK_RIGHT]) and not SplitHLock and (Layout <> mslHorz)) or
       ((CharCode in [VK_UP, VK_DOWN]) and not SplitVLock and (Layout <> mslVert))) then
      Result := 1;
end;

procedure TabcCustomMultiSplit.DoDrawDragLines(CRect: TRect; ScrPos: TPoint; BOffset: Integer);
var
  DrawRect: TRect;
  DragPos: TPoint;
begin
  {centre pos to drag thickness}
  DragPos.X := ScrPos.X - Thickness div 2;
  DragPos.Y := ScrPos.Y - Thickness div 2;

  with DragPos, CRect do
    case MoveCursor of
      crHSplit:
        case FLayout of
          mslVert, mslFourWay:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
            end;
          mslLeftTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              if X < (Left + SplitHMouseMin - BOffset - Thickness) then begin
                DrawRect := Rect(X + Thickness,
                  Top + SplitVPos - BOffset - Thickness div 2,
                  Left + SplitHMouseMin - BOffset,
                  Top + SplitVPos - BOffset + (Thickness + 1) div 2);
                DrawOneDragLineAt(DrawRect);
              end;
            end;
          mslRightTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              if X > (Left + SplitHMouseMax - BOffset + 1) then begin
                DrawRect := Rect(Left + SplitHMouseMax - BOffset + 1,
                  Top + SplitVPos - BOffset - Thickness div 2,
                  X,
                  Top + SplitVPos - BOffset + (Thickness + 1) div 2);
                DrawOneDragLineAt(DrawRect);
              end;
            end;
          mslLeftRightBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Top + SplitVMouseMin - BOffset);
              DrawOneDragLineAt(DrawRect);
            end;
          mslTopLeftRight:
            begin
              DrawRect := Rect(X, Top + SplitVMouseMax - BOffset + 1, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
            end;
        end;
      crVSplit:
        case FLayout of
         mslHorz, mslFourWay:
          begin
            DrawRect := Rect(Left, Y, Right, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
          end;
         mslLeftTopBottom:
          begin
            DrawRect := Rect(Left + SplitHMouseMax - BOffset + 1, Y, Right, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
          end;
         mslRightTopBottom:
          begin
            DrawRect := Rect(Left, Y, Left + SplitHMouseMin - BOffset, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
          end;
         mslLeftRightBottom:
          begin
            DrawRect := Rect(Left, Y, Right, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
            if (Y > Top + SplitVMouseMax - BOffset + 1) then begin
              DrawRect := Rect(Left + SplitHPos - BOffset - Thickness div 2,
                Top + SplitVMouseMax - BOffset + 1,
                Left + SplitHPos - BOffset + (Thickness + 1) div 2,
                Y);
              DrawOneDragLineAt(DrawRect);
            end;
          end;
         mslTopLeftRight:
          begin
            DrawRect := Rect(Left, Y, Right, Y + Thickness);
            DrawOneDragLineAt(DrawRect);
            if (Y < Top + SplitVMouseMin - BOffset - Thickness) then begin
              DrawRect := Rect(Left + SplitHPos - BOffset - Thickness div 2,
                Y + Thickness,
                Left + SplitHPos - BOffset + (Thickness + 1) div 2,
                Top + SplitVMouseMin - BOffset);
              DrawOneDragLineAt(DrawRect);
            end;
          end;
        end;
      abccrSize:
        case FLayout of
          mslLeftTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(X + Thickness, Y, Right, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
            end;
          mslRightTopBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(Left, Y, X, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
            end;
          mslLeftRightBottom:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Y);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(Left, Y, Right, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
            end;
          mslTopLeftRight:
            begin
              DrawRect := Rect(Left, Y, Right, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(X, Y + Thickness, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
            end;
          mslFourWay:
            begin
              DrawRect := Rect(X, Top, X + Thickness, Bottom);
              DrawOneDragLineAt(DrawRect);
              DrawRect := Rect(Left, Y, Right, Y + Thickness);
              DrawOneDragLineAt(DrawRect);
            end;
        end;
    end;
end;

procedure TabcCustomMultiSplit.DoPaintSplitLines(ACanvas: TCanvas; ARect: TRect; TopColor, BottomColor: TColor);
begin
  with ACanvas, ARect do
    case FLayout of

      mslSingle:
        {do nothing};

      mslVert:
        begin
          Pen.Color := TopColor;
          MoveTo(SplitHMouseMin, Top);
          LineTo(SplitHMouseMin, Bottom);
          Pen.Color := BottomColor;
          MoveTo(SplitHMouseMax, Bottom - 1);
          LineTo(SplitHMouseMax, Top - 1);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            FillRect(ARect);
          end;
        end;

      mslHorz:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          Pen.Color := BottomColor;
          MoveTo(Right - 1, SplitVMouseMax);
          LineTo(Left - 1, SplitVMouseMax);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            FillRect(ARect);
          end;
        end;

      mslLeftTopBottom:
        begin
          Pen.Color := TopColor;
          MoveTo(SplitHMouseMin, Top);
          LineTo(SplitHMouseMin, Bottom);
          MoveTo(SplitHMouseMax, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          Pen.Color := BottomColor;
          MoveTo(SplitHMouseMax, Top);
          LineTo(SplitHMouseMax, SplitVMouseMin);
          MoveTo(Right - 1, SplitVMouseMax);
          LineTo(SplitHMouseMax, SplitVMouseMax);
          LineTo(SplitHMouseMax, Bottom);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            FillRect(ARect);
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            ARect.Left := SplitHMouseMax;
            if BorderStyle = bsNone then
              ARect.Right := Width - BorderOffset
            else
              ARect.Right := Width - BorderOffset - 2;
            FillRect(ARect);
          end;
        end;

      mslRightTopBottom:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(SplitHMouseMin, SplitVMouseMin);
          LineTo(SplitHMouseMin, Top - 1);
          MoveTo(SplitHMouseMin, SplitVMouseMax + 1);
          LineTo(SplitHMouseMin, Bottom);
          Pen.Color := BottomColor;
          MoveTo(Left, SplitVMouseMax);
          LineTo(SplitHMouseMin + 1, SplitVMouseMax);
          MoveTo(SplitHMouseMax, Top);
          LineTo(SplitHMouseMax, Bottom);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            ARect.Right := SplitHMouseMin + 1;
            FillRect(ARect);
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            ARect.Top := BorderOffset;
            if BorderStyle = bsNone then
              ARect.Bottom := Height - BorderOffset
            else
              ARect.Bottom := Height - BorderOffset - 2;
            FillRect(ARect);
          end;
        end;

      mslLeftRightBottom:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(SplitHMouseMin, SplitVMouseMin);
          LineTo(SplitHMouseMin, Top - 1);
          MoveTo(SplitHMouseMax, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          Pen.Color := BottomColor;
          MoveTo(SplitHMouseMax, Top);
          LineTo(SplitHMouseMax, SplitVMouseMin);
          MoveTo(Left, SplitVMouseMax);
          LineTo(Right, SplitVMouseMax);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            ARect.Bottom := SplitVMouseMax;
            FillRect(ARect);
            ARect.Left := BorderOffset;
            if BorderStyle = bsNone then
              ARect.Right := Width - BorderOffset
            else
              ARect.Right := Width - BorderOffset - 2;
            ARect.Top := SplitVMouseMin + 1;
            FillRect(ARect);
          end;
        end;

      mslTopLeftRight:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          MoveTo(SplitHMouseMin, SplitVMouseMax + 1);
          LineTo(SplitHMouseMin, Bottom);
          Pen.Color := BottomColor;
          MoveTo(Left, SplitVMouseMax);
          LineTo(SplitHMouseMin + 1, SplitVMouseMax);
          MoveTo(Right - 1, SplitVMouseMax);
          LineTo(SplitHMouseMax, SplitVMouseMax);
          LineTo(SplitHMouseMax, Bottom);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            FillRect(ARect);
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            if BorderStyle = bsNone then
              ARect.Bottom := Height - BorderOffset
            else
              ARect.Bottom := Height - BorderOffset - 2;
            FillRect(ARect);
          end;
        end;

      mslFourWay:
        begin
          Pen.Color := TopColor;
          MoveTo(Left, SplitVMouseMin);
          LineTo(SplitHMouseMin, SplitVMouseMin);
          LineTo(SplitHMouseMin, Top - 1);
          MoveTo(SplitHMouseMax, SplitVMouseMin);
          LineTo(Right, SplitVMouseMin);
          MoveTo(SplitHMouseMin, SplitVMouseMax + 1);
          LineTo(SplitHMouseMin, Bottom);
          Pen.Color := BottomColor;
          MoveTo(Left, SplitVMouseMax);
          LineTo(SplitHMouseMin + 1, SplitVMouseMax);
          MoveTo(SplitHMouseMax, Top);
          LineTo(SplitHMouseMax, SplitVMouseMin);
          MoveTo(Right - 1, SplitVMouseMax);
          LineTo(SplitHMouseMax, SplitVMouseMax);
          LineTo(SplitHMouseMax, Bottom);
          if SplitColor <> Color then begin
            Brush.Color := SplitColor;
            ARect.Top := SplitVMouseMin + 1;
            ARect.Bottom := SplitVMouseMax;
            FillRect(ARect);
            ARect.Left := SplitHMouseMin + 1;
            ARect.Right := SplitHMouseMax;
            ARect.Top := BorderOffset;
            if BorderStyle = bsNone then
              ARect.Bottom := Height - BorderOffset
            else
              ARect.Bottom := Height - BorderOffset - 2;
            FillRect(ARect);
          end;
        end;
    end;
end;

procedure TabcCustomMultiSplit.DoResizePanes(ARect: TRect);
var
  CWidth, CHeight: integer;

  procedure PaneSetVisible(Index: Integer; FVisible: Boolean);
  {crude device to hide panes at designtime}
  begin
    if Index < FPaneList.Count then
      with Pane[Index] do begin
        if FVisible then begin
          if not Visible then Visible := True;
        end
        else begin
          if Visible then Visible := False;
          if (csDesigning in ComponentState) then SetBounds(0,0,0,0);
        end;
      end;
  end;

begin
  with ARect do begin
    CWidth := Right - Left;
    CHeight := Bottom - Top;
    case FLayout of

      mslSingle:
        begin
          SetPaneBounds(0, Left, Top, CWidth, CHeight);
          PaneSetVisible(1, False);
          PaneSetVisible(2, False);
          PaneSetVisible(3, False);
        end;

      mslVert:
        begin
          SetPaneBounds(0, Left, Top, SplitHMouseMin - BorderOffset, CHeight);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), CHeight);
          PaneSetVisible(1, True);
          PaneSetVisible(2, False);
          PaneSetVisible(3, False);
        end;

      mslHorz:
        begin
          SetPaneBounds(0, Left, Top, CWidth,
            SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, Left, SplitVMouseMax + 1,
            CWidth, (Bottom - SplitVMouseMax - 1));
          PaneSetVisible(1, True);
          PaneSetVisible(2, False);
          PaneSetVisible(3, False);
        end;

       mslLeftTopBottom:
        begin
          SetPaneBounds(0, Left, Top,
            SplitHMouseMin - BorderOffset, CHeight);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), SplitVMouseMin - BorderOffset);
          SetPaneBounds(2, SplitHMouseMax + 1,SplitVMouseMax + 1,
            (Right - SplitHMouseMax - 1),(Bottom - SplitVMouseMax - 1));
          PaneSetVisible(1, True);
          PaneSetVisible(2, True);
          PaneSetVisible(3, False);
        end;

      mslRightTopBottom:
        begin
          SetPaneBounds(0, Left, Top,
            SplitHMouseMin - BorderOffset, SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), CHeight);
          SetPaneBounds(2, Left, SplitVMouseMax + 1,
            SplitHMouseMin - BorderOffset, (Bottom - SplitVMouseMax - 1));
          PaneSetVisible(1, True);
          PaneSetVisible(2, True);
          PaneSetVisible(3, False);
        end;

      mslLeftRightBottom:
        begin
          SetPaneBounds(0, Left, Top,
            SplitHMouseMin - BorderOffset, SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), SplitVMouseMin - BorderOffset);
          SetPaneBounds(2, Left, SplitVMouseMax + 1,
            CWidth, (Bottom - SplitVMouseMax - 1));
          PaneSetVisible(1, True);
          PaneSetVisible(2, True);
          PaneSetVisible(3, False);
        end;

      mslTopLeftRight:
        begin
          SetPaneBounds(0, Left, Top, CWidth, SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, SplitHMouseMax + 1, SplitVMouseMax + 1,
            (Right - SplitHMouseMax - 1), (Bottom - SplitVMouseMax - 1));
          SetPaneBounds(2, Left, SplitVMouseMax + 1,
            SplitHMouseMin - BorderOffset, (Bottom - SplitVMouseMax - 1));
          PaneSetVisible(1, True);
          PaneSetVisible(2, True);
          PaneSetVisible(3, False);
        end;

      mslFourWay:
        begin
          SetPaneBounds(0, Left, Top,
            SplitHMouseMin - BorderOffset, SplitVMouseMin - BorderOffset);
          SetPaneBounds(1, SplitHMouseMax + 1, Top,
            (Right - SplitHMouseMax - 1), SplitVMouseMin - BorderOffset);
          SetPaneBounds(2, SplitHMouseMax + 1, SplitVMouseMax + 1,
            (Right - SplitHMouseMax - 1), (Bottom - SplitVMouseMax - 1));
          SetPaneBounds(3, Left, SplitVMouseMax + 1,
            SplitHMouseMin - BorderOffset, (Bottom - SplitVMouseMax - 1));
          PaneSetVisible(1, True);
          PaneSetVisible(2, True);
          PaneSetVisible(3, True);
        end;
    end;
  end;
end;

function TabcCustomMultiSplit.GetMoveCursor(X,Y: Integer): TCursor;
begin
  case FLayout of

    mslSingle:
      Result := Cursor;

    mslVert:
      if BetweenXMouseLimits(X) then
        Result := crHSplit
      else
        Result := Cursor;

    mslHorz:
      if BetweenYMouseLimits(Y) then
        Result := crVSplit
      else
        Result := Cursor;

    mslLeftTopBottom:
      if BetweenXMouseLimits(X) then
        if BetweenYMouseLimits(Y) then
          Result := abccrSize
        else
          Result := crHSplit
      else
        if BetweenYMouseLimits(Y) and
          (X > SplitHMouseMax) then
          Result := crVsplit
        else
          Result := Cursor;

    mslRightTopBottom:
      if BetweenXMouseLimits(X) then
        if BetweenYMouseLimits(Y) then
          Result := abccrSize
        else
          Result := crHSplit
      else
        if BetweenYMouseLimits(Y) and
          (X < SplitHMouseMin) then
            Result := crVsplit
        else
          Result := Cursor;

    mslLeftRightBottom:
      if BetweenYMouseLimits(Y) then
        if BetweenXMouseLimits(X) then
          Result := abccrSize
        else
          Result := crVSplit
      else
        if BetweenXMouseLimits(X) and
          (Y < SplitVMouseMin) then
          Result := crHSplit
        else
          Result := Cursor;

    mslTopLeftRight:
      if BetweenYMouseLimits(Y) then
        if BetweenXMouseLimits(X) then
          Result := abccrSize
        else
          Result := crVSplit
      else
        if BetweenXMouseLimits(X) and
          (Y > SplitVMouseMax) then
          Result := crHSplit
        else
          Result := Cursor;

    mslFourWay:
      if BetweenXMouseLimits(X) then
        if BetweenYMouseLimits(Y) then
          Result := abccrSize
        else
          Result := crHSplit
      else
        if BetweenYMouseLimits(Y) then
          Result := crVSplit
        else
          Result := Cursor;
  else
    Result := Cursor;
  end;
end;

function TabcCustomMultiSplit.GetMaxPanes: Integer;
begin
  Result := 4;
end;

function TabcCustomMultiSplit.GetSplitMovePos(var X, Y: Integer): TPoint;
begin
  case MoveCursor of
    crHSplit:
      Result := Point(X,SplitVPos);
    crVSplit:
      Result := Point(SplitHPos,Y);
    abccrSize:
      Result := Point(X,Y);
  end;
end;

procedure TabcCustomMultiSplit.SetLayout(Value: TabcMultiSplitLayout);
begin
  if Value <> FLayout then begin
    FLayout := Value;
    if not (csLoading in ComponentState) then ResizePanes;
  end;
end;

{ TabcMultiSplit }

function TabcMultiSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcSplitPane;
end;

function TabcMultiSplit.GetSplitPane(Index: Integer): TabcSplitPane;
begin
  Result := TabcSplitPane(inherited GetPane(Index));
end;

{ TabcSMultiSplit }

function TabcSMultiSplit.GetPaneClass: TWinControlClass;
begin
  Result := TabcScrollingSplitPane;
end;

function TabcSMultiSplit.GetScrollPane(Index: Integer): TabcScrollingSplitPane;
begin
  Result := TabcScrollingSplitPane(inherited GetPane(Index));
end;

procedure TabcSMultiSplit.PaneScrollH(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: integer;
begin
  if FSyncHScroll and not FPaneScrolling then begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_HSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

procedure TabcSMultiSplit.PaneScrollV(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  ScrollMsg: TWMScroll;
  I: Integer;
begin
  if FSyncVScroll and not FPaneScrolling then begin
    FPaneScrolling := True;
    ScrollMsg.Msg := WM_VSCROLL;
    ScrollMsg.ScrollCode := SmallInt(ScrollCode);
    ScrollMsg.Pos := ScrollPos;
    ScrollMsg.ScrollBar := 0;
    with TMessage(ScrollMsg) do
      for I := 0 to FPaneList.Count - 1 do
        if Sender <> Pane[I] then
          SendMessage(Pane[I].Handle, Msg, wParam, LParam);
    FPaneScrolling := False;
  end;
end;

initialization
{$IFDEF DELPHI6}
  ActivateClassGroup(TControl);
{$ENDIF}
  RegisterClasses([TabcSplitPane, TabcScrollingSplitPane]);
  Screen.Cursors[abccrSize] := LoadCursor(hInstance, 'ABCCRSIZE');
  
finalization
  UnRegisterClasses([TabcSplitPane, TabcScrollingSplitPane]);
  Screen.Cursors[abccrSize] := 0;

end.


