unit abcsprfr;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses Classes, Windows, Controls, StdCtrls, Forms,
     abcfxrch, abcsprfx;

type

  { TabcRichEffectSprite }

  {:A picture sprite with a rich text formatted (RTF) caption and transition effects between images.
  This class draws blend, fade, wipe or shaped effects whenever the Picture or ImageIndex or RichText changes.<p>
  The sprite can display either an image or text or both using the <See Property=ShowPicture> and <See Property=ShowText> properties.<P>
  The <See Property=RichText> is drawn centered over any Picture or Imagelist image.
  The text is clipped to the picture dimensions and is transparently masked over the image.<p>
  For sprites with a plain text caption, see class <See Class=TabcEffectSprite>.<P>
  For non-sprite transition effect components, see classes
  <A HRef="TabcEffectsImage_Component">TabcEffectsImage</A>,
  <A HRef="TabcRichEffectsImage_Component">TabcRichEffectsImage</A>, and
  <A HRef="TabcDBEffectsImage_Component">TabcDBEffectsImage</A>.<P>
  }

  TabcRichEffectSprite = class(TabcEffectSprite)
  private
    FRichText: TabcRichText;
    procedure SetRichText(Value: TabcRichText);
  protected
    procedure DoClear; override;
    procedure DrawImageText(R: TRect); override;
    function HasText: Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    {::The RTF data to be drawn on the sprite.
     <A HRef="TabcRichText_Component">TabcRichText</A> encapsulates RTF data without a visual control.}
    property RichText: TabcRichText read FRichText write SetRichText;
  end;

implementation

{ TabcRichEffectSprite }

constructor TabcRichEffectSprite.Create(AOwner: TComponent);
begin
  inherited;
  FRichText := TabcRichText.Create;
  FRichText.OnChange := PictureChanged;
end;

destructor TabcRichEffectSprite.Destroy;
begin
  Destroying;
  FRichText.Free;
  inherited;
end;

procedure TabcRichEffectSprite.DoClear;
begin
  inherited;
  FRichText.Clear;
end;

procedure TabcRichEffectSprite.DrawImageText(R: TRect);
begin
  inherited;
  with FRichText do
    if not Empty then
      PaintToRect(Bitmap.Canvas, R, Layout);
end;

function TabcRichEffectSprite.HasText: Boolean;
begin
  Result := not FRichText.Empty or (Caption <> '');
end;

procedure TabcRichEffectSprite.SetRichText(Value: TabcRichText);
begin
  if Value = nil then
    FRichText.Clear
  else if Value <> FRichText then
    FRichText.Assign(Value);
end;

end.
