unit abcsubfm;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, TabNotBk, Tabs, Dialogs, abcpanel, abcfx;

type

  TabcFormNotifyEvent = procedure (Sender: TObject; const Form: TScrollingWinControl) of object;

  TabcFormChangingEvent = procedure (Sender: TObject; const NewFormName: string;
    var AllowChange: Boolean) of object;

  TabcSubFormAction = (afaCreate, afaActivate, afaDeactivate, afaHide, afaClose, afaDestroy, afaShowExisting);
  TabcSubFormActions = set of TabcSubFormAction;

  { TabcLRNoteBook }

  TabcLRNoteBook = class(TNotebook)
  private
    function GetPageIndex: integer;
    procedure SetPageIndex(Value: integer);
  protected
    procedure DestroyPageWnd(Index: integer); virtual;
    procedure RecreatePageWnd(Index: integer); virtual;
    procedure ShowControl(AControl: TControl); override;
  published
    property PageIndex read GetPageIndex write SetPageIndex;
  end;

  { TabcFormNoteBook }

  TabcFormCreateDestroyOption = (fcOnLoad, fcOnPageChange);

  TabcFormNoteBook = class(TabcLRNoteBook)
  private
    FFormCreateDestroy: TabcFormCreateDestroyOption;
    FHandleException: Boolean;
    FOnCreateForm: TabcFormNotifyEvent;
    FOnDestroyForm: TabcFormNotifyEvent;
    function GetActiveForm: string;
    function GetActivePage: string;
    function GetForm(Index: Integer): TScrollingWinControl;
    procedure SetActiveForm(const Value: string);
    procedure SetActivePage(const Value: string);
    procedure SetForms(Value: TStrings);
    procedure DestroyForms;
  protected
    FForms: TStrings;
    procedure CreateForm(Index: Integer); virtual;
    procedure DestroyForm(Index: Integer); virtual;
    procedure DestroyPageWnd(Index: integer); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RecreatePageWnd(Index: integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property FormObjects[Index: Integer]: TScrollingWinControl read GetForm;
  published
    property ActiveForm: string read GetActiveForm write SetActiveForm stored False;
    property ActivePage: string read GetActivePage write SetActivePage stored False;
    property FormCreateDestroy: TabcFormCreateDestroyOption read FFormCreateDestroy
      write FFormCreateDestroy default fcOnLoad;
    property Forms: TStrings read FForms write SetForms;
    property HandleException: Boolean read FHandleException write FHandleException default True;
    property OnCreateForm: TabcFormNotifyEvent read FOnCreateForm write FOnCreateForm;
    property OnDestroyForm: TabcFormNotifyEvent read FOnDestroyForm write FOnDestroyForm;
  end;

  { TabcLRTabbedNoteBook }

  TabcLRTabbedNoteBook = class(TTabbedNotebook)
  private
    FOnChange: TPageChangeEvent;
    procedure InheritedPageChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
  protected
    procedure ShowControl(AControl: TControl); override;
    procedure RecreatePageWnd(Index: integer); virtual;
    procedure DestroyPageWnd(Index: integer); virtual;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property OnChange: TPageChangeEvent read FOnChange write FOnChange;
  end;

  { TabcFormTabbedNoteBook }

  TabcFormTabbedNoteBook = class(TabcLRTabbedNoteBook)
  private
    FFormCreateDestroy: TabcFormCreateDestroyOption;
    FHandleException: Boolean;
    FOnCreateForm: TabcFormNotifyEvent;
    FOnDestroyForm: TabcFormNotifyEvent;
    function GetActiveForm: string;
    function GetForm(Index: Integer): TScrollingWinControl;
    procedure DestroyForms;
    procedure SetForms(Value: TStrings);
    procedure SetActiveForm(const Value: string);
  protected
    FForms: TStrings;
    procedure CreateForm(Index: Integer); virtual;
    procedure DestroyForm(Index: Integer); virtual;
    procedure DestroyPageWnd(Index: integer); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RecreatePageWnd(Index: integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property FormObjects[Index: Integer]: TScrollingWinControl read GetForm;
  published
    property ActiveForm: string read GetActiveForm write SetActiveForm stored False;
    property FormCreateDestroy: TabcFormCreateDestroyOption read FFormCreateDestroy
      write FFormCreateDestroy default fcOnLoad;
    property Forms: TStrings read FForms write SetForms;
    property HandleException: Boolean read FHandleException write FHandleException default True;
    property OnCreateForm: TabcFormNotifyEvent read FOnCreateForm write FOnCreateForm;
    property OnDestroyForm: TabcFormNotifyEvent read FOnDestroyForm write FOnDestroyForm;
  end;

  { TabcFormPanel }

  TabcFormPanel = class(TabcCustomEdgePanel)
  private
    FAutoCreate: Boolean;
    FAutoDestroy: Boolean;
    FFormActions: TabcSubFormActions;
    FFormName: string;
    FHandleException: Boolean;
    FPaintImage: Boolean;
    FSaveImage: TBitmap;
    FTransition: TabcTransition;
    FOnCreateForm: TabcFormNotifyEvent;
    FOnDestroyForm: TabcFormNotifyEvent;
    FOnLoadForm: TabcFormNotifyEvent;
    FOnChanging: TabcFormChangingEvent;
    FOnChange: TNotifyEvent;
    FFormChanging: Boolean;
    procedure CancelTransition(Sender:TObject; var Cancel: Boolean);
    procedure SavePanelImage;
    procedure ClearPanelImage;
    function GetFormLoaded: Boolean;
    procedure SetAutoCreate(Value: Boolean);
    procedure SetAutoDestroy(Value: Boolean);
    procedure SetFormLoaded(Value: Boolean);
    procedure DoSetFormLoaded(Value: Boolean);
    procedure SetFormName(Value: string);
    procedure SetTransition(Value: TabcTransition);
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
    procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
    procedure SetFormActions(const Value: TabcSubFormActions);
  protected
    FForm: TScrollingWinControl;
    FSkipFormPropertyChanges: Boolean;
    procedure CreateHandle; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
    function CreateForm: TScrollingWinControl; virtual;
    function FindExistingForm: TScrollingWinControl; virtual;
    procedure DestroyForm; virtual;
    procedure SetForm(Value: TScrollingWinControl); virtual;
    procedure DoSetForm(Value: TScrollingWinControl); virtual;
    function CanChange(NewFormName: string): Boolean; virtual;
  public
    property Form: TScrollingWinControl read FForm write SetForm;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property FormLoaded: Boolean read GetFormLoaded write SetFormLoaded;
  published
    property AutoCreate: Boolean read FAutoCreate write SetAutoCreate default True;
    property AutoDestroy: Boolean read FAutoDestroy write SetAutoDestroy default True;
    property FormActions: TabcSubFormActions read FFormActions write SetFormActions
      default [afaCreate, afaActivate, afaDeActivate, afaHide, afaClose, afaDestroy];
    property FormName: string read FFormName write SetFormName;
    property HandleException: Boolean read FHandleException write FHandleException default True;
    property Transition: TabcTransition read FTransition write SetTransition;
    property Transparent;
    property OnCreateForm: TabcFormNotifyEvent read FOnCreateForm write FOnCreateForm;
    property OnDestroyForm: TabcFormNotifyEvent read FOnDestroyForm write FOnDestroyForm;
    property OnLoadForm: TabcFormNotifyEvent read FOnLoadForm write FOnLoadForm;
    property OnChanging: TabcFormChangingEvent read FOnChanging write FOnChanging;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property Align;
    {property Alignment;}
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    {property DragCursor;
    property DragMode;}
    property Enabled;
    property Caption;
    property Color;
    {property Ctl3D;
    property Font;
    property Locked;}
    property ParentColor;
    {property ParentCtl3D;
    property ParentFont;}
    property ParentShowHint;
    {property PopupMenu;}
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    {property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;}
    property OnEnter;
    property OnExit;
    {property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;}
  end;

  TabcLRComboBox = class(TComboBox)
  private
    FSaveItemIndex: Integer;
  protected
    procedure CreateHandle; override;
    procedure DestroyWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TabcLRListBox = class(TListBox)
  private
    FSaveItemIndex: Integer;
    FSaveSelectedItems: TStringList;
  protected
    procedure CreateHandle; override;
    procedure DestroyWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TabcAutoTabSet }

  TabcAutoTabSet = class(TTabSet)
  private
    FNoteBook: TNoteBook;
    FOnChange: TTabChangeEvent;
    procedure InheritedTabChange(Sender: TObject; NewTab: Integer; var AllowChange: Boolean);
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure UpdateTabs;
  published
    property NoteBook: TNoteBook read FNoteBook write FNoteBook;
    property OnChange: TTabChangeEvent read FOnChange write FOnChange;
    property TabIndex stored False;
    property Tabs stored False;
  end;

implementation

uses abcconst, abcutilf;

type
  TScrollingWinControlClass = class of TScrollingWinControl;

{ utility }

procedure CallFormActivate(AForm: TForm; Sender: TObject);
begin
  if (AForm <> nil) and Assigned(AForm.OnActivate) then
    AForm.OnActivate(Sender);
end;

{ TabcLRComboBox }

constructor TabcLRComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSaveItemIndex := -1;
end;

procedure TabcLRComboBox.CreateHandle;
begin
  inherited CreateHandle;
  if FSaveItemIndex <> -1 then
    ItemIndex := FSaveItemIndex;
end;

procedure TabcLRComboBox.DestroyWnd;
begin
  FSaveItemIndex := ItemIndex;
  inherited DestroyWnd;
end;

{ TabcLRListBox }

constructor TabcLRListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSaveItemIndex := -1;
  FSaveSelectedItems := TStringList.Create;
end;

destructor TabcLRListBox.Destroy;
begin
  FSaveSelectedItems.Free;
  inherited Destroy;
end;

procedure TabcLRListBox.CreateHandle;
var
  I: integer;
begin
  inherited CreateHandle;
  if FSaveItemIndex <> -1 then
    ItemIndex := FSaveItemIndex;
  if MultiSelect and (FSaveSelectedItems.Count > 0) then
    for I := 0 to FSaveSelectedItems.Count - 1 do
      Selected[StrToInt(FSaveSelectedItems.Strings[I])] := True;
end;

procedure TabcLRListBox.DestroyWnd;
var
  I: integer;
begin
  FSaveItemIndex := ItemIndex;
  if MultiSelect then
    for I := 0 to Items.Count - 1 do
      if Selected[I] then
        FSaveSelectedItems.Add(Inttostr(I));
  inherited DestroyWnd;
end;

{$WARNINGS OFF}

{ internal class definitions }

type

  { TabcCreateDestroyWinCtrl }

  TabcCreateDestroyWinCtrl = class(TWinControl)
  public
    procedure DestroyHandle;
    procedure RecreateWnd;
  end;

  { TabcShowCustomControl }

  TabcShowCustomControl = class(TCustomControl)
  public
    procedure ShowControl(AControl: TControl);
  end;

{ implementation of internal classes }
{ TabcCreateDestroyWinCtrl }

{
This class is used to redeclare protected methods of TWinControl.
Used in this module to destroy/recreate window handles of TPage.
}

procedure TabcCreateDestroyWinCtrl.DestroyHandle;
begin
  inherited DestroyHandle;
end;

procedure TabcCreateDestroyWinCtrl.RecreateWnd;
begin
  inherited RecreateWnd;
end;

{ TabcShowCustomControl }

{
This class is used to redeclare protected methods of TCustomControl.
Used in this module to call ShowControl and skip one level of inheritance.
}

procedure TabcShowCustomControl.ShowControl(AControl: TControl);
begin
  inherited ShowControl(AControl);
end;

{$WARNINGS ON}

{ TabcLRNoteBook }

{
TNotebook that creates/destroys window handles when changing pages.
Changes are implemented by replacing PageIndex property access method.
}

function TabcLRNoteBook.GetPageIndex: integer;
begin
  Result := inherited PageIndex;
end;

procedure TabcLRNoteBook.SetPageIndex(Value: integer);
var
  CurrentPage: integer;
  ParentForm: TCustomForm;
begin
  CurrentPage := PageIndex;
  if (CurrentPage <> Value) and (Value >= 0) and (Value < Pages.Count) then begin
    if (csDesigning in ComponentState) then
    begin
      inherited PageIndex := Value;
      Exit;
    end;
    ParentForm := ValidParentForm(self);
    LockWindowUpdate(ParentForm.Handle);
    inherited PageIndex := Value;
    RecreatePageWnd(Value);
    DestroyPageWnd(CurrentPage);
    LockWindowUpdate(0);
    if (csDesigning in ComponentState) then
      with ParentForm do
        if Designer <> nil then Designer.Modified;
  end;
end;

procedure TabcLRNoteBook.RecreatePageWnd(Index: integer);
begin
  TabcCreateDestroyWinCtrl(Pages.Objects[Index]).RecreateWnd;
  TPage(Pages.Objects[Index]).Parent := self;
end;

procedure TabcLRNoteBook.DestroyPageWnd(Index: integer);
begin
  TabcCreateDestroyWinCtrl(Pages.Objects[Index]).DestroyHandle;
end;

procedure TabcLRNoteBook.ShowControl(AControl: TControl);
var
  I: Integer;
begin
  for I := 0 to Pages.Count - 1 do
    if TPage(Pages.Objects[I]) = AControl then
    begin
      SetPageIndex(I);
      Exit;
    end;
  TabcShowCustomControl(self).ShowControl(AControl);
end;

{ TabcFormNoteBook }

constructor TabcFormNoteBook.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForms := TStringList.Create;
  FHandleException := True;
end;

destructor TabcFormNoteBook.Destroy;
begin
  DestroyForms;
  FForms.Free;
  FForms := nil;
  inherited Destroy;
end;

procedure TabcFormNoteBook.CreateForm(Index: Integer);
var
  AClass: TPersistentClass;
  AFormClass: TScrollingWinControlClass;
  AForm: TScrollingWinControl;
begin
  if (Index < 0) or (Index > FForms.Count - 1) then Exit;
  try
    if FForms.Strings[Index] = '' then Exit; {allow notebook pages without a form}
    AClass := FindClass('T' + FForms.Strings[Index]);

    AFormClass := TScrollingWinControlClass(AClass);
    AForm := AFormClass.Create(Self);
    with AForm do begin
      if AForm is TForm then
        with TForm(AForm) do
        begin
          {change the form style}
          BorderStyle := bsNone;
          BorderIcons := [];
          FormStyle := fsNormal;
        end;
      {set the form's parent wnd}
      Parent := TPage(Pages.Objects[Index]);
      if Align = alNone then
        Align := alClient
      else
        Align := Align;
      if AForm is TForm then
        TForm(AForm).AutoScroll := True;
    end;
    FForms.Objects[Index] := AForm;
    if Assigned(FOnCreateForm) then FOnCreateForm(Self, AForm);
  except
    on E: Exception do
      if HandleException then
        ShowMessage(E.Message)
      else
        raise;
  end;
end;

procedure TabcFormNoteBook.DestroyForm(Index: Integer);
var
  AForm: TScrollingWinControl;
begin
  if (Index < 0) or (Index > FForms.Count - 1) then Exit;
  try
    with FForms do
      if Objects[Index] <> nil then begin
        AForm := TScrollingWinControl(Objects[Index]);
        if Assigned(FOnDestroyForm) then FOnDestroyForm(Self, AForm);
        if not (csDestroying in AForm.ComponentState) then
          if AForm is TForm then
            TForm(AForm).Release
          else
            AForm.Free;
        Objects[Index] := nil;
      end;
  except
    on E: Exception do
      if HandleException then
        ShowMessage(E.Message)
      else
        raise;
  end;
end;

procedure TabcFormNoteBook.DestroyForms;
var
  I: integer;
begin
  for I := 0 to FForms.Count - 1 do
    DestroyForm(I);
end;

procedure TabcFormNoteBook.Notification(AComponent: TComponent; Operation: TOperation);
var
  i: integer;
begin
  inherited;
  if (Operation = opRemove) and (FForms <> nil) then
    if AComponent is TScrollingWinControl then
    begin
      i := FForms.IndexOfObject(AComponent);
      if i <> -1 then
        FForms.Objects[i] := nil;
    end;
end;

procedure TabcFormNoteBook.DestroyPageWnd(Index: integer);
begin
  inherited DestroyPageWnd(Index);
  if not (csDesigning in ComponentState) then
    if FFormCreateDestroy = fcOnPageChange then
      DestroyForm(Index);
end;

function TabcFormNoteBook.GetActiveForm: string;
begin
  if (PageIndex > -1) and (PageIndex < Forms.Count) then
    Result := Forms[PageIndex]
  else
    Result := '';
end;

function TabcFormNoteBook.GetActivePage: string;
begin
  Result := inherited ActivePage;
end;

procedure TabcFormNoteBook.SetActiveForm(const Value: string);
begin
  PageIndex := Forms.IndexOf(Value);
end;

procedure TabcFormNoteBook.SetActivePage(const Value: string);
begin
  PageIndex := Pages.IndexOf(Value);
end;

function TabcFormNoteBook.GetForm(Index: Integer): TScrollingWinControl;
begin
  if (Index < 0) or (Index > FForms.Count - 1) then
    Result := nil
  else
    Result := TScrollingWinControl(FForms.Objects[Index]);
end;

procedure TabcFormNoteBook.Loaded;
var
  I: Integer;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then begin
    if FFormCreateDestroy = fcOnLoad then
      for I := 0 to FForms.Count - 1 do
        CreateForm(I)
    else
      CreateForm(PageIndex);
    if FormObjects[PageIndex] <> nil then
      FormObjects[PageIndex].Show;
  end;
end;

procedure TabcFormNoteBook.RecreatePageWnd(Index: integer);
begin
  inherited RecreatePageWnd(Index);
  if not (csDesigning in ComponentState) then begin
    if FFormCreateDestroy = fcOnPageChange then
      CreateForm(Index);
    if FormObjects[Index] <> nil then
      with FormObjects[Index] do
      begin
        if FormObjects[Index] is TForm then
        begin
          TForm(FormObjects[Index]).Show;
          CallFormActivate(TForm(FormObjects[Index]), Self);
        end
        else
          TScrollingWinControl(FormObjects[Index]).Visible := True;
      end;
  end;
end;

procedure TabcFormNoteBook.SetForms(Value: TStrings);
begin
  if (csDesigning in ComponentState) then
    FForms.Assign(Value)
  else
    raise EInvalidOperation.Create('Forms can be replaced at design time only');
end;

{ TabcLRTabbedNoteBook }

{
TTabbedNotebook that creates/destroys window handles when changing pages.
Changes are implemented by replacing the inherited OnChange event.
}

constructor TabcLRTabbedNoteBook.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited OnChange := InheritedPageChange;
end;

procedure TabcLRTabbedNoteBook.InheritedPageChange(Sender: TObject;
  NewTab: Integer; var AllowChange: Boolean);
var
  ParentForm: TCustomForm;
begin
  if Assigned(FOnChange) then FOnChange(Sender, NewTab, AllowChange);
  if AllowChange then begin
    if (csDesigning in ComponentState) then
      Exit;
    ParentForm := ValidParentForm(self);
    LockWindowUpdate(ParentForm.Handle);
    RecreatePageWnd(NewTab);
    DestroyPageWnd(PageIndex);
    LockWindowUpdate(0);
    if (csDesigning in ComponentState) then
      with ParentForm do
        if Designer <> nil then Designer.Modified;
  end;
end;

procedure TabcLRTabbedNoteBook.RecreatePageWnd(Index: integer);
begin
  TabcCreateDestroyWinCtrl(Pages.Objects[Index]).RecreateWnd;
  TPage(Pages.Objects[Index]).Parent := self;
end;

procedure TabcLRTabbedNoteBook.DestroyPageWnd(Index: integer);
begin
  TabcCreateDestroyWinCtrl(Pages.Objects[Index]).DestroyHandle;
end;

procedure TabcLRTabbedNoteBook.ShowControl(AControl: TControl);
var
  I: Integer;
begin
  for I := 0 to Pages.Count - 1 do
    if TPage(Pages.Objects[I]) = AControl then
    begin
      SetPageIndex(I);
      Exit;
    end;
  TabcShowCustomControl(self).ShowControl(AControl);
end;

{ TabcFormTabbedNoteBook }

constructor TabcFormTabbedNoteBook.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForms := TStringList.Create;
  FHandleException := True;
end;

destructor TabcFormTabbedNoteBook.Destroy;
begin
  DestroyForms;
  FForms.Free;
  FForms := nil;
  inherited Destroy;
end;

procedure TabcFormTabbedNoteBook.CreateForm(Index: Integer);
var
  AClass: TPersistentClass;
  AFormClass: TScrollingWinControlClass;
  AForm: TScrollingWinControl;
begin
  if (Index < 0) or (Index > FForms.Count - 1) then Exit;
  try
    if FForms.Strings[Index] = '' then Exit; {allow notebook pages without a form}
    AClass := FindClass('T' + FForms.Strings[Index]);

    AFormClass := TScrollingWinControlClass(AClass);
    AForm := AFormClass.Create(Self);
    with AForm do begin
      if AForm is TForm then
        with TForm(AForm) do
        begin
          {change the form style}
          BorderStyle := bsNone;
          BorderIcons := [];
          FormStyle := fsNormal;
        end;
      {set the form's parent wnd}
      Parent := TPage(Pages.Objects[Index]);
      if Align = alNone then
        Align := alClient
      else
        Align := Align;
      if AForm is TForm then
        TForm(AForm).AutoScroll := True;
    end;
    FForms.Objects[Index] := AForm;
    if Assigned(FOnCreateForm) then FOnCreateForm(Self, AForm);
  except
    on E: Exception do
      if HandleException then
        ShowMessage(E.Message)
      else
        raise;
  end;
end;

procedure TabcFormTabbedNoteBook.DestroyForm(Index: Integer);
var
  AForm: TScrollingWinControl;
begin
  if (Index < 0) or (Index > FForms.Count - 1) then Exit;
  try
    with FForms do
      if Objects[Index] <> nil then begin
        AForm := TScrollingWinControl(Objects[Index]);
        if Assigned(FOnDestroyForm) then FOnDestroyForm(Self, AForm);
        if not (csDestroying in AForm.ComponentState) then
          if AForm is TForm then
            TForm(AForm).Release
          else
            AForm.Free;
        Objects[Index] := nil;
      end;
  except
    on E: Exception do
      if HandleException then
        ShowMessage(E.Message)
      else
        raise;
  end;
end;

procedure TabcFormTabbedNoteBook.DestroyForms;
var
  I: integer;
begin
  for I := 0 to FForms.Count - 1 do
    DestroyForm(I);
end;

procedure TabcFormTabbedNoteBook.Notification(AComponent: TComponent; Operation: TOperation);
var
  i: integer;
begin
  inherited;
  if (Operation = opRemove) and (FForms <> nil) then
    if AComponent is TScrollingWinControl then
    begin
      i := FForms.IndexOfObject(AComponent);
      if i <> -1 then
        FForms.Objects[i] := nil;
    end;
end;

procedure TabcFormTabbedNoteBook.DestroyPageWnd(Index: integer);
begin
  inherited DestroyPageWnd(Index);
  if not (csDesigning in ComponentState) then
    if FFormCreateDestroy = fcOnPageChange then
      DestroyForm(Index);
end;

function TabcFormTabbedNoteBook.GetActiveForm: string;
begin
  if (PageIndex > -1) and (PageIndex < Forms.Count) then
    Result := Forms[PageIndex]
  else
    Result := '';
end;

function TabcFormTabbedNoteBook.GetForm(Index: Integer): TScrollingWinControl;
begin
  if (Index < 0) or (Index > FForms.Count - 1) then
    Result := nil
  else
    Result := TScrollingWinControl(FForms.Objects[Index]);
end;

procedure TabcFormTabbedNoteBook.Loaded;
var
  I: Integer;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then begin
    if FFormCreateDestroy = fcOnLoad then
      for I := 0 to FForms.Count - 1 do
        CreateForm(I)
    else
      CreateForm(PageIndex);
    if FormObjects[PageIndex] <> nil then
      FormObjects[PageIndex].Show;
  end;
end;

procedure TabcFormTabbedNoteBook.RecreatePageWnd(Index: integer);
begin
  inherited RecreatePageWnd(Index);
  if not (csDesigning in ComponentState) then begin
    if FFormCreateDestroy = fcOnPageChange then
      CreateForm(Index);
    if FormObjects[Index] <> nil then
      with FormObjects[Index] do
      begin
        if FormObjects[Index] is TForm then
        begin
          TForm(FormObjects[Index]).Show;
          CallFormActivate(TForm(FormObjects[Index]), Self);
        end
        else
          TScrollingWinControl(FormObjects[Index]).Visible := True;
      end;
  end;
end;

procedure TabcFormTabbedNoteBook.SetActiveForm(const Value: string);
begin
  PageIndex := Forms.IndexOf(Value);
end;

procedure TabcFormTabbedNoteBook.SetForms(Value: TStrings);
begin
  if (csDesigning in ComponentState) then
    FForms.Assign(Value)
  else
    raise EInvalidOperation.Create('Forms can be replaced at design time only');
end;

{ TabcFormPanel }

constructor TabcFormPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForm := nil;
  FFormName := '';
  FHandleException := True;
  FAutoCreate := True;
  FAutoDestroy := True;
  FTransition := TabcTransition.Create;
  FTransition.OnEffectStep := CancelTransition;
  FFormActions := [afaCreate, afaActivate, afaDeActivate, afaHide, afaClose, afaDestroy];
end;

destructor TabcFormPanel.Destroy;
begin
  if AutoDestroy then DestroyForm;
  ClearPanelImage;
  FTransition.Free;
  inherited Destroy;
end;

function TabcFormPanel.CreateForm: TScrollingWinControl;
var
  AClass: TPersistentClass;
  AFormClass: TScrollingWinControlClass;
begin
  Result := nil;
  try
    if FFormName = '' then
      raise Exception.Create(sabcErrFormNameExpected);
    AClass := FindClass('T' + FFormName);
    AFormClass := TScrollingWinControlClass(AClass);
    Result := AFormClass.Create(Self);
    if Assigned(FOnCreateForm) then FOnCreateForm(Self, Result);
  except
    on E: Exception do
      if HandleException then
        ShowMessage(E.Message)
      else
        raise;
  end;
end;

procedure TabcFormPanel.DestroyForm;
begin
  if FForm <> nil then
    try
      if Assigned(FOnDestroyForm) then FOnDestroyForm(Self, FForm);
      if not (csDestroying in FForm.ComponentState) then
        if FForm is TForm then
          TForm(FForm).Release
        else
          FForm.Free;
      FForm := nil;
    except
      on E: Exception do
        if HandleException then
          ShowMessage(E.Message)
        else
          raise;
    end;
end;

procedure TabcFormPanel.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FForm) then
    FForm := nil;
end;

function TabcFormPanel.FindExistingForm: TScrollingWinControl;
var
  AClassName: string;
  i: integer;
begin
  Result := nil;
  try
    if FFormName = '' then
      raise Exception.Create(sabcErrFormNameExpected);
    AClassName := 'T' + FFormName;
    with Application do
      for i := 0 to ComponentCount - 1 do
        if Components[i].ClassName = AClassName then
        begin
          Result := TScrollingWinControl(Components[i]);
          Break;
        end;
  except
    on E: Exception do
      if HandleException then
        ShowMessage(E.Message)
      else
        raise;
  end;
end;

procedure TabcFormPanel.CreateHandle;
begin
  inherited CreateHandle;
  if not (csDesigning in ComponentState) and
     ([afaCreate, afaShowExisting] * FormActions <> []) and
     (FForm = nil) and
     (FFormName <> '') then
    FormLoaded := True;
end;

function TabcFormPanel.GetFormLoaded: Boolean;
begin
  Result := FForm <> nil;
end;

procedure TabcFormPanel.Paint;
begin
  if FPaintImage then
    Canvas.Draw(0,0,FSaveImage)
  else
  begin
    inherited Paint;
    if (FForm <> nil) then FForm.Repaint;
  end;
end;

procedure TabcFormPanel.SetAutoCreate(Value: Boolean);
begin
  if Value <> FAutoCreate then begin
    FAutoCreate := Value;
    if Value then
      FFormActions := FFormActions + [afaCreate]
    else
      FFormActions := FFormActions - [afaCreate];
    if not (csDesigning in ComponentState) then
      if Value then
        SetFormName(FFormName)
      else if FForm <> nil then
        SetForm(nil);
  end;
end;

procedure TabcFormPanel.SetAutoDestroy(Value: Boolean);
begin
  if Value <> FAutoDestroy then begin
    FAutoDestroy := Value;
    if Value then
      FFormActions := FFormActions + [afaDestroy]
    else
      FFormActions := FFormActions - [afaDestroy];
  end;
end;

procedure TabcFormPanel.SavePanelImage;
begin
  ClearPanelImage;
  FSaveImage := abcCopyWindowFromDesktop(Self, True);
end;

procedure TabcFormPanel.ClearPanelImage;
begin
  if FSaveImage <> nil then
  begin
    FSaveImage.Free;
    FSaveImage := nil;
  end;
end;

function TabcFormPanel.CanChange(NewFormName: string): Boolean;
begin
  Result := True;
  if (csLoading in ComponentState) or (FForm = nil) then
    Exit;
  if (FForm <> nil) and ([afaClose, afaDestroy] * FormActions <> []) then
    if FForm is TForm then
    begin
      Result := TForm(FForm).CloseQuery;
      if not Result then Exit;
    end;
  if Assigned(FOnChanging) then FOnChanging(Self, NewFormName, Result);
end;

procedure TabcFormPanel.DoSetForm(Value: TScrollingWinControl);
var
  ClientR, OffScreenR, DrawR, RemainingR: TRect;
  NewAlign: TAlign;
  bmp: TBitmap;
begin
  if (Value <> FForm) then
  begin
    ClientR := ClientRect;
    AdjustClientRect(ClientR);
    if FForm <> nil then
    begin
      SavePanelImage;
      FPaintImage := True;
      with FForm do
      begin
        if afaDeActivate in FormActions then Perform(CM_DEACTIVATE,0,0);
        if afaHide in FormActions then Hide;
        if afaClose in FormActions then
          if FForm is TForm then
            TForm(FForm).Close
          else
            FForm.Visible := False;
        Parent := nil;
      end;
      if afaDestroy in FormActions then
        DestroyForm;
    end;
    FForm := nil;

    if (Value <> nil) then
    begin
      with Value do
      begin
        if not FSkipFormPropertyChanges then
        begin
          {change the form style}
          if Value is TForm then
            with TForm(Value) do
            begin
              BorderStyle := bsNone;
              BorderIcons := [];
              FormStyle := fsNormal;
            end;

          if Align = alNone then
            NewAlign := alClient
          else
            NewAlign := Align;
          //size and show new form off screen client aligned
          Align := alNone;
          if Value is TForm then
            TForm(Value).AutoScroll := True;
        end
        else
        begin
          NewAlign := Align;
          Align := alNone;
        end;

        OffScreenR := ClientR;
        case NewAlign of
          alNone, alClient: ;
          alTop:
            OffScreenR.Bottom := OffScreenR.Top + Height;
          alBottom:
            OffScreenR.Top := OffScreenR.Bottom - Height;
          alLeft:
            OffScreenR.Right := OffScreenR.Left + Width;
          alRight:
            OffScreenR.Left := OffScreenR.Right - Width;
        end;
        DrawR := OffScreenR;
        OffsetRect(OffScreenR, -OffScreenR.Right, -OffScreenR.Bottom);
        with OffScreenR do SetBounds(Left, Top, Right-Left, Bottom-Top);
        Parent := self;
        Show;
        if Transition.Enabled then
          Transition.DrawWindowAt(Value, Self.Canvas, True, DrawR);
        SubtractRect(RemainingR, ClientR, DrawR);
        if Transition.NoRedraw and Transition.Enabled then
          SetWindowPos(Handle, HWND_TOP, DrawR.Left, DrawR.Top, Width, Height, SWP_NOREDRAW)
        else
          SetWindowPos(Handle, HWND_TOP, DrawR.Left, DrawR.Top, Width, Height, SWP_SHOWWINDOW);
        if afaActivate in FormActions then
        begin
          SetFocus;
          Perform(CM_ACTIVATE,0,0);
        end;
        Align := NewAlign;
      end;
      FPaintImage := False;
      if not IsRectEmpty(RemainingR) then
        Invalidate;
      FForm := Value;
      if Assigned(FOnLoadForm) then FOnLoadForm(Self, Value);
    end
    else
      if Transparent then
      begin
        DrawR := BoundsRect;
        if Parent.HandleAllocated then
          InvalidateRect(Parent.Handle, @DrawR,  TRUE);
        Invalidate;
      end
      else if Transition.Enabled and not (csDestroying in ComponentState) then
      begin
        FPaintImage := False;
        bmp := abcGetControlBitmap(Self);
        FPaintImage := True;
        with bmp do
        try
          Transition.DrawAt(Canvas, Self.Canvas, Rect(0,0,Width, Height));
        finally
          Free;
        end;
      end;
    FPaintImage := False;
    ClearPanelImage;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcFormPanel.SetForm(Value: TScrollingWinControl);
var
  NewFormName: string;
begin
  if FFormChanging then Exit;
  FFormChanging := True;
  try
  if Value <> nil then
    NewFormName := Copy(Value.ClassName, 2, 256)
  else
    NewFormName := '';
  if CanChange(NewFormName) then
    DoSetForm(Value);
  finally
    FFormChanging := False;
  end;
end;

procedure TabcFormPanel.CMShowingChanged(var Message: TMessage);
begin
  inherited;
  if Visible and (FForm is TForm) then
    CallFormActivate(TForm(FForm), Self);
end;

procedure TabcFormPanel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  if not FPaintImage then
    inherited
  else
    Message.Result := 1;
end;

procedure TabcFormPanel.SetFormLoaded(Value: Boolean);
var
  NewFormName: string;
begin
  if Value then
    NewFormName := FormName
  else
    NewFormName := '';
  if CanChange(NewFormName) then
    DoSetFormLoaded(Value);
end;

procedure TabcFormPanel.DoSetFormLoaded(Value: Boolean);
var
  AForm: TScrollingWinControl;
begin
  AForm := nil;
  if Value then
  begin
    if (afaShowExisting in FormActions) then
      AForm := FindExistingForm;
    if (AForm = nil) and (afaCreate in FormActions) then
      AForm := CreateForm;
    DoSetForm(AForm);
  end
  else
    DoSetForm(nil);
end;

procedure TabcFormPanel.SetFormName(Value: string);
begin
  if CanChange(Value) then
  begin
    FFormName := Value;
    if ([csLoading, csDesigning] * ComponentState = []) and
       ([afaCreate, afaShowExisting] * FormActions <> []) and
       HandleAllocated then
      DoSetFormLoaded(Value <> '');
  end;
end;

procedure TabcFormPanel.SetTransition(Value: TabcTransition);
begin
  FTransition.Assign(Value);
end;

procedure TabcFormPanel.SetFormActions(const Value: TabcSubFormActions);
begin
  FFormActions := Value;
  if afaCreate in Value then
    AutoCreate := True
  else
    AutoCreate := False;
  if afaDestroy in Value then
    AutoDestroy := True
  else
    AutoDestroy := False;
end;

procedure TabcFormPanel.CancelTransition(Sender:TObject; var Cancel: Boolean);
begin
  Cancel := Application.Terminated or (csDestroying in ComponentState) or not Visible;
end;

{ TabcAutoTabSet }

constructor TabcAutoTabSet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited OnChange := InheritedTabChange;
end;

procedure TabcAutoTabSet.Loaded;
begin
  inherited Loaded;
  UpdateTabs;
end;

procedure TabcAutoTabSet.InheritedTabChange(Sender: TObject; NewTab: Integer; var AllowChange: Boolean);
begin
  if Assigned(FOnChange) then FOnChange(Sender, NewTab, AllowChange);
  if AllowChange and (FNoteBook <> nil) then
    if FNoteBook is TabcFormNoteBook then
      TabcFormNoteBook(FNoteBook).PageIndex := NewTab
    else if FNoteBook is TabcLRNoteBook then
      TabcLRNoteBook(FNoteBook).PageIndex := NewTab
    else
      FNoteBook.PageIndex := NewTab;
end;

procedure TabcAutoTabSet.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = FNoteBook then FNoteBook := nil;
end;

procedure TabcAutoTabSet.UpdateTabs;
var
  I: Integer;
begin
  inherited Tabs.Clear;
  if not (csDesigning in ComponentState) then
    if FNoteBook <> nil then begin
      for I := 0 to FNoteBook.Pages.Count - 1 do
        inherited Tabs.Add(FNoteBook.Pages.Strings[I]);
      inherited TabIndex := FNoteBook.PageIndex;
    end;
end;

end.
