unit abctctls;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Based on Toolbar97 version 1.76a Licensed from Jordan Russell - Thanks Jordan!
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, Classes, Controls, Forms, Menus, Graphics, Buttons,
  ImgList, ActnList, StdCtrls, ExtCtrls;

const
  abcDefaultDropdownArrowWidth = 9;
  
type

  {$IFNDEF DELPHI5}
    TImageIndex = type Integer;
  {$ENDIF}


  { TabcToolbarButton }

  TabcTBButtonDisplayMode = (admBoth, admGlyphOnly, admTextOnly);
  TabcTBButtonState = (atsUp, atsDisabled, atsDown, atsExclusive, atsMouseIn);
  TabcTBNumGlyphs = 1..5;
  TabcTBButtonDropdownEvent = procedure(Sender: TObject;
    var ShowMenu, RemoveClicks: Boolean) of object;

  TabcToolbarButton = class(TGraphicControl)
  private
    FAllowAllUp: Boolean;
    FAlignment: TAlignment;
    FCancel: Boolean;
    FDefault: Boolean;
    FDisplayMode: TabcTBButtonDisplayMode;
    FDown: Boolean;
    FDropdownAlways: Boolean;
    FDropdownArrow: Boolean;
    FDropdownArrowWidth: Integer;
    FDropdownCombo: Boolean;
    FDropdownMenu: TPopupMenu;
    FFlat: Boolean;
    FGlyph: Pointer;
    FGroupIndex: Integer;
    FHelpContext: THelpContext;
    FHighlightWhenDown: Boolean;
    FLayout: TButtonLayout;
    FMargin: Integer;
    FModalResult: TModalResult;
    FNoBorder: Boolean;
    FOldDisabledStyle: Boolean;
    FOpaque: Boolean;
    FRepeating: Boolean;
    FRepeatDelay, FRepeatInterval: Integer;
    FShowBorderWhenInactive: Boolean;
    FSpacing: Integer;
    FWordWrap: Boolean;
    FOnDropdown: TabcTBButtonDropdownEvent;
    FOnMouseEnter, FOnMouseExit: TNotifyEvent;
    { Internal }
    FInClick: Boolean;
    FMouseInControl: Boolean;
    FMouseIsDown: Boolean;
    FMenuIsDown: Boolean;
    FUsesDropdown: Boolean;
    FRepeatTimer: TTimer;
    procedure GlyphChanged(Sender: TObject);
    procedure UpdateExclusive;
    procedure SetAlignment (Value: TAlignment);
    procedure SetAllowAllUp (Value: Boolean);
    function GetCallDormant: Boolean;
    procedure SetCallDormant (Value: Boolean);
    procedure SetDown (Value: Boolean);
    procedure SetDisplayMode (Value: TabcTBButtonDisplayMode);
    procedure SetDropdownAlways (Value: Boolean);
    procedure SetDropdownArrow (Value: Boolean);
    procedure SetDropdownArrowWidth (Value: Integer);
    procedure SetDropdownCombo (Value: Boolean);
    procedure SetDropdownMenu (Value: TPopupMenu);
    procedure SetFlat (Value: Boolean);
    function GetGlyph: TBitmap;
    procedure SetGlyph (Value: TBitmap);
    function GetGlyphMask: TBitmap;
    procedure SetGlyphMask (Value: TBitmap);
    procedure SetGroupIndex (Value: Integer);
    procedure SetHighlightWhenDown (Value: Boolean);
    function GetImageIndex: TImageIndex;
    procedure SetImageIndex (Value: TImageIndex);
    function GetImages: TCustomImageList;
    procedure SetImages (Value: TCustomImageList);
    procedure SetLayout (Value: TButtonLayout);
    procedure SetMargin (Value: Integer);
    procedure SetNoBorder (Value: Boolean);
    function GetNumGlyphs: TabcTBNumGlyphs;
    procedure SetNumGlyphs (Value: TabcTBNumGlyphs);
    procedure SetOldDisabledStyle (Value: Boolean);
    procedure SetOpaque (Value: Boolean);
    procedure SetSpacing (Value: Integer);
    procedure SetWordWrap (Value: Boolean);
    procedure RemoveButtonMouseTimer;
    procedure Redraw (const Erase: Boolean);
    function PointInButton (X, Y: Integer): Boolean;
    procedure ButtonMouseTimerHandler (Sender: TObject);
    procedure RepeatTimerHandler (Sender: TObject);
    function IsCheckedStored: Boolean;
    function IsHelpContextStored: Boolean;
    function IsImageIndexStored: Boolean;
    procedure WMLButtonDblClk (var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMDialogChar (var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMDialogKey (var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged (var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMSysColorChange (var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure WMCancelMode (var Message: TWMCancelMode); message WM_CANCELMODE;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
  protected
    FState: TabcTBButtonState;
    function GetPalette: HPALETTE; override;
    procedure Loaded; override;
    procedure Notification (AComponent: TComponent; Operation: TOperation); override;
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove (Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
    procedure ActionChange (Sender: TObject; CheckDefaults: Boolean); override;
    function GetActionLinkClass: TControlActionLinkClass; override;
    procedure AssignTo (Dest: TPersistent); override;
  public
    procedure CancelMenuClick;
    property CallDormant: Boolean read GetCallDormant write SetCallDormant;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure MouseEntered;
    procedure MouseLeft;
  published
    property Action;
    property Anchors;
    property Constraints;
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property AllowAllUp: Boolean read FAllowAllUp write SetAllowAllUp default False;
    property Cancel: Boolean read FCancel write FCancel default False;
    property Color default clBtnFace;
    property GroupIndex: Integer read FGroupIndex write SetGroupIndex default 0;
    property Default: Boolean read FDefault write FDefault default False;
    property DisplayMode: TabcTBButtonDisplayMode read FDisplayMode write SetDisplayMode default admBoth;
    property Down: Boolean read FDown write SetDown stored IsCheckedStored default False;
    property DragCursor;
    property DragMode;
    property DropdownAlways: Boolean read FDropdownAlways write SetDropdownAlways default False;
    property DropdownArrow: Boolean read FDropdownArrow write SetDropdownArrow default True;
    property DropdownArrowWidth: Integer read FDropdownArrowWidth write SetDropdownArrowWidth default abcDefaultDropdownArrowWidth;
    property DropdownCombo: Boolean read FDropdownCombo write SetDropdownCombo default False;
    property DropdownMenu: TPopupMenu read FDropdownMenu write SetDropdownMenu;
    property Caption;
    property Enabled;
    property Flat: Boolean read FFlat write SetFlat default True;
    property Font;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    property GlyphMask: TBitmap read GetGlyphMask write SetGlyphMask;
    property HelpContext: THelpContext read FHelpContext write FHelpContext stored IsHelpContextStored default 0;
    property HighlightWhenDown: Boolean read FHighlightWhenDown write SetHighlightWhenDown default True;
    property ImageIndex: TImageIndex read GetImageIndex write SetImageIndex stored IsImageIndexStored default -1;
    property Images: TCustomImageList read GetImages write SetImages;
    property Layout: TButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property Margin: Integer read FMargin write SetMargin default -1;
    property ModalResult: TModalResult read FModalResult write FModalResult default 0;
    property NoBorder: Boolean read FNoBorder write SetNoBorder default False;
    property NumGlyphs: TabcTBNumGlyphs read GetNumGlyphs write SetNumGlyphs default 1;
    property OldDisabledStyle: Boolean read FOldDisabledStyle write SetOldDisabledStyle default False;
    property Opaque: Boolean read FOpaque write SetOpaque default True;
    property ParentFont;
    property ParentColor default False;
    property ParentShowHint;
    property Repeating: Boolean read FRepeating write FRepeating default False;
    property RepeatDelay: Integer read FRepeatDelay write FRepeatDelay default 400;
    property RepeatInterval: Integer read FRepeatInterval write FRepeatInterval default 100;
    property ShowBorderWhenInactive: Boolean read FShowBorderWhenInactive write FShowBorderWhenInactive default False;
    property ShowHint;
    property Spacing: Integer read FSpacing write SetSpacing default 4;
    property Visible;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default False;

    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropdown: TabcTBButtonDropdownEvent read FOnDropdown write FOnDropdown;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseExit write FOnMouseExit;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  { TabcToolbarButtonActionLink }

  TabcToolbarButtonActionLink = class(TControlActionLink)
  protected
    FClient: TabcToolbarButton;
    procedure AssignClient (AClient: TObject); override;
    function IsCheckedLinked: Boolean; override;
    function IsHelpContextLinked: Boolean; override;
    function IsImageIndexLinked: Boolean; override;
    procedure SetChecked (Value: Boolean); override;
    procedure SetHelpContext (Value: THelpContext); override;
    procedure SetImageIndex (Value: Integer); override;
  end;

  TabcToolbarButtonActionLinkClass = class of TabcToolbarButtonActionLink;

  { TabcToolbarEdit }

  TabcToolbarEdit = class(TCustomEdit)
  private
    MouseInControl: Boolean;
    procedure DrawNCArea (const DrawToDC: Boolean; const ADC: HDC;
      const Clip: HRGN);
    procedure NewAdjustHeight;
    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave (var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMKillFocus (var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMNCCalcSize (var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCPaint (var Message: TMessage); message WM_NCPAINT;
    procedure WMPrint (var Message: TMessage); message WM_PRINT;
    procedure WMPrintClient (var Message: TMessage); message WM_PRINTCLIENT;
    procedure WMSetFocus (var Message: TWMSetFocus); message WM_SETFOCUS;
  protected
    procedure Loaded; override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property AutoSelect;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property Align;
    property CharCase;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnEndDock;
    property OnStartDock;
  end;

var
  abcButtonsStayDown: Boolean = True;
  abcButtonMouseInControl: TabcToolbarButton = nil;

function abcControlIsTBControl (AControl: TControl): Boolean;
procedure abcRegisterTBControlClass (AClass: TClass);
procedure abcUnregisterTBControlClass (AClass: TClass);

implementation

uses
  SysUtils, Consts, CommCtrl, abctcmn, abcutilg;

var
  { See TabcToolbarButton.ButtonMouseTimerHandler for info on this }
  ButtonMouseTimer: TTimer = nil;

  abcControlTBList: TList = nil;

  abcTBEditCount: Integer = 0;

const
  DropdownComboSpace = 2;

function abcControlIsTBControl (AControl: TControl): Boolean;
var
  I: Integer;
begin
  Result := False;
  if Assigned(AControl) and Assigned(abcControlTBList) then
    for I := 0 to abcControlTBList.Count-1 do
      if AControl is TClass(abcControlTBList[I]) then begin
        Result := True;
        Break;
      end;
end;

procedure abcRegisterTBControlClass (AClass: TClass);
begin
  if abcControlTBList = nil then abcControlTBList := TList.Create;
  abcControlTBList.Add (AClass);
end;

procedure abcUnregisterTBControlClass (AClass: TClass);
begin
  if Assigned(abcControlTBList) then begin
    abcControlTBList.Remove (AClass);
    if abcControlTBList.Count = 0 then begin
      abcControlTBList.Free;
      abcControlTBList := nil;
    end;
  end;
end;

{ TabcToolbarButtonActionLink - internal }

procedure TabcToolbarButtonActionLink.AssignClient (AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TabcToolbarButton;
end;

function TabcToolbarButtonActionLink.IsCheckedLinked: Boolean;
begin
  Result := inherited IsCheckedLinked and
    (FClient.Down = (Action as TCustomAction).Checked);
end;

function TabcToolbarButtonActionLink.IsHelpContextLinked: Boolean;
begin
  Result := inherited IsHelpContextLinked and
    (FClient.HelpContext = (Action as TCustomAction).HelpContext);
end;

function TabcToolbarButtonActionLink.IsImageIndexLinked: Boolean;
begin
  Result := inherited IsImageIndexLinked and
    (FClient.ImageIndex = (Action as TCustomAction).ImageIndex);
end;

procedure TabcToolbarButtonActionLink.SetChecked (Value: Boolean);
begin
  if IsCheckedLinked then FClient.Down := Value;
end;

procedure TabcToolbarButtonActionLink.SetHelpContext (Value: THelpContext);
begin
  if IsHelpContextLinked then FClient.HelpContext := Value;
end;

procedure TabcToolbarButtonActionLink.SetImageIndex (Value: Integer);
begin
  if IsImageIndexLinked then FClient.ImageIndex := Value;
end;

{ TabcToolbarButton - internal }

type
  TabcGlyphList = class(TImageList)
  private
    Used: TBits;
    FCount: Integer;
    function AllocateIndex: Integer;
  public
    constructor CreateSize (AWidth, AHeight: Integer);
    destructor Destroy; override;
    function Add (Image, Mask: TBitmap): Integer;
    function AddMasked (Image: TBitmap; MaskColor: TColor): Integer;
    procedure Delete (Index: Integer);
    property Count: Integer read FCount;
  end;

  TabcGlyphCache = class
  private
    GlyphLists: TList;
  public
    constructor Create;
    destructor Destroy; override;
    function GetList(AWidth, AHeight: Integer): TabcGlyphList;
    procedure ReturnList(List: TabcGlyphList);
    function Empty: Boolean;
  end;

  TBoolInt = record
    B: Boolean;
    I: Integer;
  end;

  TCustomImageListAccess = class(TCustomImageList);

  TSmallMemoryStream = class(TMemoryStream)
  protected
    function Realloc(var NewCapacity: Longint): Pointer; override;
  end;

  TabcTBButtonGlyph = class
  private
    FStream, FMaskStream: TSmallMemoryStream;
    FOriginal, FOriginalMask: TBitmap;
    FOriginalSize, FMaskSize: TPoint;
    FCallDormant: Boolean;
    FGlyphList: array[Boolean] of TabcGlyphList;
    FImageIndex: TImageIndex;
    FImageList: TCustomImageList;
    FImageChangeLink: TChangeLink;
    FIndexs: array[Boolean, TabcTBButtonState] of Integer;
    FTransparentColor: TColor;
    FNumGlyphs: TabcTBNumGlyphs;
    FOnChange: TNotifyEvent;
    FOldDisabledStyle: Boolean;
    procedure GlyphChanged (Sender: TObject);
    function GetGlyph: TBitmap;
    procedure SetGlyph (Value: TBitmap);
    function GetGlyphMask: TBitmap;
    procedure SetGlyphMask (Value: TBitmap);
    procedure SetNumGlyphs (Value: TabcTBNumGlyphs);
    procedure UpdateNumGlyphs;
    procedure Invalidate;
    function CreateButtonGlyph (State: TabcTBButtonState): TBoolInt;
    procedure DrawButtonGlyph (Canvas: TCanvas; const GlyphPos: TPoint;
      State: TabcTBButtonState);
    procedure DrawButtonText (Canvas: TCanvas;
      const Caption: string; TextBounds: TRect;
      WordWrap: Boolean; Alignment: TAlignment; State: TabcTBButtonState);
    procedure DrawButtonDropArrow (Canvas: TCanvas; const X, Y, AWidth: Integer;
      State: TabcTBButtonState);
    procedure CalcButtonLayout (Canvas: TCanvas; const Client: TRect;
      const Offset: TPoint; DrawGlyph, DrawCaption: Boolean;
      const Caption: string; WordWrap: Boolean;
      Layout: TButtonLayout; Margin, Spacing: Integer; DropArrow: Boolean;
      DropArrowWidth: Integer; var GlyphPos, ArrowPos: TPoint;
      var TextBounds: TRect);
    procedure RestoreGlyph;
    procedure RestoreGlyphMask;
  public
    constructor Create;
    destructor Destroy; override;
    { returns the text rectangle }
    function Draw (Canvas: TCanvas; const Client: TRect; const Offset: TPoint;
      DrawGlyph, DrawCaption: Boolean; const Caption: string; WordWrap: Boolean;
      Alignment: TAlignment; Layout: TButtonLayout; Margin, Spacing: Integer;
      DropArrow: Boolean; DropArrowWidth: Integer; State: TabcTBButtonState): TRect;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    property GlyphMask: TBitmap read GetGlyphMask write SetGlyphMask;
    property NumGlyphs: TabcTBNumGlyphs read FNumGlyphs write SetNumGlyphs;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;


{ TabcGlyphList }

constructor TabcGlyphList.CreateSize(AWidth, AHeight: Integer);
begin
  inherited CreateSize (AWidth, AHeight);
  Used := TBits.Create;
end;

destructor TabcGlyphList.Destroy;
begin
  Used.Free;
  inherited;
end;

function TabcGlyphList.AllocateIndex: Integer;
begin
  Result := Used.OpenBit;
  if Result >= Used.Size then
  begin
    Result := inherited Add(nil, nil);
    Used.Size := Result + 1;
  end;
  Used[Result] := True;
end;

function TabcGlyphList.Add (Image, Mask: TBitmap): Integer;
begin
  Result := AllocateIndex;
  Replace (Result, Image, Mask);
  Inc (FCount);
end;

function TabcGlyphList.AddMasked (Image: TBitmap; MaskColor: TColor): Integer;
  procedure BugfreeReplaceMasked (Index: Integer; NewImage: TBitmap; MaskColor: TColor);
    procedure CheckImage (Image: TGraphic);
    begin
      if Image = nil then Exit;
      if (Image.Height < Height) or (Image.Width < Width) then
        raise EInvalidOperation.Create(SInvalidImageSize);
    end;
  var
    TempIndex: Integer;
    Image, Mask: TBitmap;
  begin
    if HandleAllocated then begin
      CheckImage(NewImage);
      TempIndex := inherited AddMasked(NewImage, MaskColor);
      if TempIndex <> -1 then
        try
          Image := nil;
          Mask := nil;
          try
            Image := TBitmap.Create;
            Image.Height := Height;
            Image.Width := Width;
            Mask := TBitmap.Create;
            Mask.Monochrome := True;
            { ^ Prevents the "invisible glyph" problem when used with certain
                color schemes. (Fixed in Delphi 3.01) }
            Mask.Height := Height;
            Mask.Width := Width;
            ImageList_Draw (Handle, TempIndex, Image.Canvas.Handle, 0, 0, ILD_NORMAL);
            ImageList_Draw (Handle, TempIndex, Mask.Canvas.Handle, 0, 0, ILD_MASK);
            if not ImageList_Replace(Handle, Index, Image.Handle, Mask.Handle) then
              raise EInvalidOperation.Create(SReplaceImage);
          finally
            Image.Free;
            Mask.Free;
          end;
        finally
          inherited Delete(TempIndex);
        end
      else
        raise EInvalidOperation.Create(SReplaceImage);
    end;
    Change;
  end;
begin
  Result := AllocateIndex;
  { This works two very serious bugs in the Delphi 2/BCB and Delphi 3
    implementations of the ReplaceMasked method. In the Delphi 2 and BCB
    versions of the ReplaceMasked method, it incorrectly uses ILD_NORMAL as
    the last parameter for the second ImageList_Draw call, in effect causing
    all white colors to be considered transparent also. And in the Delphi 2/3
    and BCB versions it doesn't set Monochrome to True on the Mask bitmap,
    causing the bitmaps to be invisible on certain color schemes. }
  BugfreeReplaceMasked (Result, Image, MaskColor);
  Inc (FCount);
end;

procedure TabcGlyphList.Delete (Index: Integer);
begin
  if Used[Index] then begin
    Dec(FCount);
    Used[Index] := False;
  end;
end;

{ TabcGlyphCache }

constructor TabcGlyphCache.Create;
begin
  inherited;
  GlyphLists := TList.Create;
end;

destructor TabcGlyphCache.Destroy;
begin
  GlyphLists.Free;
  inherited;
end;

function TabcGlyphCache.GetList(AWidth, AHeight: Integer): TabcGlyphList;
var
  I: Integer;
begin
  for I := GlyphLists.Count - 1 downto 0 do begin
    Result := GlyphLists[I];
    with Result do
      if (AWidth = Width) and (AHeight = Height) then Exit;
  end;
  Result := TabcGlyphList.CreateSize(AWidth, AHeight);
  GlyphLists.Add(Result);
end;

procedure TabcGlyphCache.ReturnList(List: TabcGlyphList);
begin
  if List = nil then Exit;
  if List.Count = 0 then begin
    GlyphLists.Remove(List);
    List.Free;
  end;
end;

function TabcGlyphCache.Empty: Boolean;
begin
  Result := GlyphLists.Count = 0;
end;

var
  GlyphCache: TabcGlyphCache = nil;
  Pattern: TBitmap = nil;
  PatternBtnFace, PatternBtnHighlight: TColor;
  ButtonCount: Integer = 0;

procedure CreateBrushPattern;
var
  X, Y: Integer;
begin
  PatternBtnFace := GetSysColor(COLOR_BTNFACE);
  PatternBtnHighlight := GetSysColor(COLOR_BTNHIGHLIGHT);
  Pattern := TBitmap.Create;
  with Pattern do begin
    Width := 8;
    Height := 8;
    with Canvas do begin
      Brush.Style := bsSolid;
      Brush.Color := clBtnFace;
      FillRect (Rect(0, 0, Width, Height));
      for Y := 0 to 7 do
        for X := 0 to 7 do
          if Odd(Y) = Odd(X) then  { toggles between even/odd pixels }
            Pixels[X, Y] := clBtnHighlight;     { on even/odd rows }
    end;
  end;
end;

{ TSmallMemoryStream }

function TSmallMemoryStream.Realloc(var NewCapacity: Integer): Pointer;
const
  SmallMemoryDelta = $50;
begin
  if NewCapacity > 0 then
    NewCapacity := (NewCapacity + (SmallMemoryDelta - 1)) and not (SmallMemoryDelta - 1);
  Result := Memory;
  if NewCapacity <> Capacity then
    ReallocMem(Result, NewCapacity);
end;

{ TabcTBButtonGlyph }

constructor TabcTBButtonGlyph.Create;
var
  B: Boolean;
  I: TabcTBButtonState;
begin
  inherited;
  FCallDormant := True;
  FImageIndex := -1;
  FOriginal := TBitmap.Create;
  FOriginal.OnChange := GlyphChanged;
  FOriginalSize := Point(0, 0);
  FOriginalMask := TBitmap.Create;
  FOriginalMask.OnChange := GlyphChanged;
  FMaskSize := Point(0, 0);
  if ABC_GLYPHS_SAVE_GDI then
  begin
    FStream := TSmallMemoryStream.Create;
    FMaskStream := TSmallMemoryStream.Create;
  end;
  FNumGlyphs := 1;
  for B := False to True do
    for I := Low(I) to High(I) do
      FIndexs[B, I] := -1;
  if GlyphCache = nil then
    GlyphCache := TabcGlyphCache.Create;
end;

destructor TabcTBButtonGlyph.Destroy;
begin
  FOriginalMask.Free;
  FOriginal.Free;
  FImageChangeLink.Free;
  Invalidate;
  if Assigned(GlyphCache) and GlyphCache.Empty then begin
    GlyphCache.Free;
    GlyphCache := nil;
  end;
  if ABC_GLYPHS_SAVE_GDI then
  begin
    FStream.Free;
    FMaskStream.Free;
  end;
  inherited;
end;

procedure TabcTBButtonGlyph.Invalidate;
var
  B: Boolean;
  I: TabcTBButtonState;
begin
  for B := False to True do begin
    for I := Low(I) to High(I) do
      if FIndexs[B, I] <> -1 then begin
        FGlyphList[B].Delete (FIndexs[B, I]);
        FIndexs[B, I] := -1;
      end;
    GlyphCache.ReturnList (FGlyphList[B]);
    FGlyphList[B] := nil;
  end;
  if ABC_GLYPHS_SAVE_GDI then
  begin
    FOriginalSize := Point(0, 0);
    FStream.Clear;
    FMaskStream.Clear;
    FMaskSize := Point(0, 0);
  end;
end;

procedure TabcTBButtonGlyph.GlyphChanged (Sender: TObject);
begin
  if (Sender = FOriginal) and (FOriginal.Width <> 0) and (FOriginal.Height <> 0) then
    FTransparentColor := FOriginal.Canvas.Pixels[0, FOriginal.Height-1] or $02000000;
  Invalidate;
  FOriginalSize := Point(FOriginal.Width, FOriginal.Height);
  FMaskSize := Point(FOriginalMask.Width, FOriginalMask.Height);
  if Assigned(FOnChange) then FOnChange (Self);
end;

procedure TabcTBButtonGlyph.UpdateNumGlyphs;
var
  Glyphs: Integer;
begin
  if (FOriginal.Width <> 0) and (FOriginal.Height <> 0) and
     (FOriginal.Width mod FOriginal.Height = 0) then begin
    Glyphs := FOriginal.Width div FOriginal.Height;
    if Glyphs > High(TabcTBNumGlyphs) then Glyphs := 1;
  end
  else
    Glyphs := 1;
  SetNumGlyphs (Glyphs);
end;

procedure TabcTBButtonGlyph.SetGlyph (Value: TBitmap);
begin
  Invalidate;
  FOriginal.Assign (Value);
  FOriginalSize := Point(FOriginal.Width, FOriginal.Height);
  UpdateNumGlyphs;
end;

procedure TabcTBButtonGlyph.SetGlyphMask (Value: TBitmap);
begin
  Invalidate;
  FOriginalMask.Assign (Value);
  FMaskSize := Point(FOriginalMask.Width, FOriginalMask.Height);
end;

procedure TabcTBButtonGlyph.SetNumGlyphs (Value: TabcTBNumGlyphs);
begin
  Invalidate;
  if (FImageList <> nil) or (Value < Low(TabcTBNumGlyphs)) or
     (Value > High(TabcTBNumGlyphs)) then
    FNumGlyphs := 1
  else
    FNumGlyphs := Value;
  GlyphChanged (nil);
end;

function TabcTBButtonGlyph.CreateButtonGlyph (State: TabcTBButtonState): TBoolInt;
const
  ROP_DSPDxax = $00E20746;
  ROP_PSDPxax = $00B8074A;
  ROP_DSna = $00220326;  { D & ~S }

const
  ROPs: array[Boolean] of DWORD = (ROP_PSDPxax, ROP_DSPDxax);
var
  OriginalBmp, OriginalMaskBmp, TmpImage, DDB, MonoBmp, MaskBmp, UseMaskBmp: TBitmap;
  I: TabcTBButtonState;
  B: Boolean;
  AddPixels, IWidth, IHeight, IWidthA, IHeightA: Integer;
  IRect, IRectA, SourceRect, R: TRect;
  DC: HDC;
  UsesMask: Boolean;
  IsHighColorDIB: Boolean;
begin
  if (State <> atsDisabled) and (Ord(State) >= NumGlyphs) then
    State := atsUp;
  Result.B := True;
  Result.I := FIndexs[True, State];
  if Result.I = -1 then begin
    Result.B := False;
    Result.I := FIndexs[False, State];
  end;
  if Result.I <> -1 then Exit;
  if FImageList = nil then begin
    if (FOriginalSize.X or FOriginalSize.Y) = 0 then Exit;
    if ABC_GLYPHS_SAVE_GDI then
    begin
      RestoreGlyph;
      RestoreGlyphMask;
    end;
    UsesMask := (FMaskSize.X <> 0) and (FMaskSize.Y <> 0);
  end
  else begin
    if (FImageIndex < 0) or (FImageIndex >= FImageList.Count) then Exit;
    UsesMask := False;
  end;
  B := State <> atsDisabled;
  { + AddPixels is to make sure the highlight color on generated disabled glyphs
    doesn't get cut off }
  if FImageList = nil then begin
    IWidthA := FOriginalSize.X div FNumGlyphs;
    IHeightA := FOriginalSize.Y;
  end
  else begin
    IWidthA := TCustomImageListAccess(FImageList).Width;
    IHeightA := TCustomImageListAccess(FImageList).Height;
  end;
  IRectA := Rect(0, 0, IWidthA, IHeightA);
  AddPixels := Ord(State = atsDisabled);
  IWidth := IWidthA + AddPixels;
  IHeight := IHeightA + AddPixels;
  IRect := Rect(0, 0, IWidth, IHeight);
  if FGlyphList[B] = nil then begin
    if GlyphCache = nil then
      GlyphCache := TabcGlyphCache.Create;
    FGlyphList[B] := GlyphCache.GetList(IWidth, IHeight);
  end;
  IsHighColorDIB := (FImageList = nil) and (FOriginal.PixelFormat > pf4bit);
  OriginalBmp := nil;
  OriginalMaskBmp := nil;
  TmpImage := nil;
  MaskBmp := nil;
  try
    OriginalBmp := TBitmap.Create;
    OriginalBmp.Assign (FOriginal);
    OriginalMaskBmp := TBitmap.Create;
    OriginalMaskBmp.Assign (FOriginalMask);
    TmpImage := TBitmap.Create;
    TmpImage.Width := IWidth;
    TmpImage.Height := IHeight;
    TmpImage.Canvas.Brush.Color := clBtnFace;
    if FImageList = nil then
      TmpImage.Palette := abcCopyPalette(OriginalBmp.Palette);
    I := State;
    if Ord(I) >= NumGlyphs then I := atsUp;
    SourceRect := Bounds(Ord(I) * IWidthA, 0, IWidthA, IHeightA);
    if FImageList <> nil then begin
      MaskBmp := TBitmap.Create;
      MaskBmp.Monochrome := True;
      MaskBmp.Width := IWidthA;
      MaskBmp.Height := IHeightA;
      ImageList_Draw (FImageList.Handle, FImageIndex, MaskBmp.Canvas.Handle,
        0, 0, ILD_MASK);
    end;

    if State <> atsDisabled then begin
      if FImageList = nil then begin
        TmpImage.Canvas.CopyRect (IRectA, OriginalBmp.Canvas, SourceRect);
        if not UsesMask then
          { Use clDefault instead of FTransparentColor whereever possible to
            ensure compatibility with all video drivers when using high-color
            (> 4 bpp) DIB glyphs }
          FIndexs[B, State] := FGlyphList[B].AddMasked(TmpImage, clDefault)
        else begin
          MonoBmp := TBitmap.Create;
          try
            MonoBmp.Monochrome := True;
            MonoBmp.Width := IWidth;
            MonoBmp.Height := IHeight;
            MonoBmp.Canvas.CopyRect (IRectA, OriginalMaskBmp.Canvas, SourceRect);
            FIndexs[B, State] := FGlyphList[B].Add(TmpImage, MonoBmp);
          finally
            MonoBmp.Free;
          end;
        end;
      end
      else begin
        ImageList_Draw (FImageList.Handle, FImageIndex, TmpImage.Canvas.Handle,
          0, 0, ILD_NORMAL);
        FIndexs[B, State] := FGlyphList[B].Add(TmpImage, MaskBmp);
      end;
    end
    else begin
      MonoBmp := nil;
      DDB := nil;
      try
        MonoBmp := TBitmap.Create;
        { Uses the abcCopyBitmapToDDB to work around a Delphi 3 flaw. If you copy
          a DIB to a second bitmap via Assign, change the HandleType of the
          second bitmap to bmDDB, then try to read the Handle property, Delphi
          converts it back to a DIB. }
        if FImageList = nil then
          DDB := abcCopyBitmapToDDB(OriginalBmp)
        else begin
          DDB := TBitmap.Create;
          DDB.Width := IWidthA;
          DDB.Height := IHeightA;
          ImageList_Draw (FImageList.Handle, FImageIndex, DDB.Canvas.Handle,
            0, 0, ILD_NORMAL);
        end;
        if NumGlyphs > 1 then
          with TmpImage.Canvas do begin
            CopyRect (IRectA, DDB.Canvas, SourceRect);

            { Convert white to clBtnHighlight }
            if not IsHighColorDIB then
              abcGenerateMaskBitmap (MonoBmp, DDB, SourceRect.TopLeft,
                IRectA.BottomRight, [GetNearestColor(OriginalBmp.Canvas.Handle, clWhite)])
            else
              abcGenerateMaskBitmapFromDIB (MonoBmp, OriginalBmp, SourceRect.TopLeft,
                IRectA.BottomRight, [clWhite]);
            abcReplaceBitmapColorsFromMask (MonoBmp, TmpImage, IRectA.TopLeft,
              IRectA.BottomRight, clBtnHighlight);

            { Convert gray to clBtnShadow }
            if not IsHighColorDIB then
              abcGenerateMaskBitmap (MonoBmp, DDB, SourceRect.TopLeft,
                IRectA.BottomRight, [GetNearestColor(OriginalBmp.Canvas.Handle, clGray)])
            else
              abcGenerateMaskBitmapFromDIB (MonoBmp, OriginalBmp, SourceRect.TopLeft,
                IRectA.BottomRight, [clGray]);
            abcReplaceBitmapColorsFromMask (MonoBmp, TmpImage, IRectA.TopLeft,
              IRectA.BottomRight, clBtnShadow);

            if not UsesMask then begin
              { Generate the transparent mask in MonoBmp. The reason why
                it doesn't just use a mask color is because the mask needs
                to be of the glyph -before- the clBtnHighlight/Shadow were
                translated }
              if not IsHighColorDIB then
                abcGenerateMaskBitmap (MonoBmp, DDB,
                  SourceRect.TopLeft, IRectA.BottomRight, FTransparentColor)
              else
                abcGenerateMaskBitmapFromDIB (MonoBmp, OriginalBmp,
                  SourceRect.TopLeft, IRectA.BottomRight, [-1]);
            end
            else
              MonoBmp.Canvas.CopyRect (IRectA, OriginalMaskBmp.Canvas, SourceRect);
            with MonoBmp do begin
              Width := Width + AddPixels;
              Height := Height + AddPixels;
              { Set the additional bottom and right row on disabled glyph
                masks to white so that it always shines through, since the
                bottom and right row on TmpImage was left uninitialized }
              Canvas.Pen.Color := clWhite;
              Canvas.PolyLine ([Point(0, Height-1), Point(Width-1, Height-1),
                Point(Width-1, -1)]);
            end;

            FIndexs[B, State] := FGlyphList[B].Add(TmpImage, MonoBmp);
          end
        else begin
          { Create a disabled version }
          if FOldDisabledStyle then begin
            { "Old" TSpeedButton style }
            if FImageList = nil then begin
              if not UsesMask then begin
                if IsHighColorDIB then
                  abcGenerateMaskBitmapFromDIB (MonoBmp, OriginalBmp,
                    SourceRect.TopLeft, IRectA.BottomRight, [clBlack])
                else begin
                  with MonoBmp do begin
                    Assign (DDB);  { must be a DDB for this to work right }
                    Canvas.Brush.Color := clBlack;
                    Monochrome := True;
                  end;
                end;
              end
              else begin
                MonoBmp.Assign (DDB);  { must be a DDB for this to work right }
                with TBitmap.Create do
                  try
                    Monochrome := True;
                    Width := OriginalMaskBmp.Width;
                    Height := OriginalMaskBmp.Height;
                    R := Rect(0, 0, Width, Height);
                    Canvas.CopyRect (R, OriginalMaskBmp.Canvas, R);
                    DC := Canvas.Handle;
                    with MonoBmp.Canvas do begin
                      BitBlt (Handle, 0, 0, IWidthA, IHeightA, DC,
                        SourceRect.Left, SourceRect.Top, ROP_DSna);
                      BitBlt (Handle, 0, 0, IWidthA, IHeightA, DC,
                        SourceRect.Left, SourceRect.Top, SRCPAINT);
                    end;
                  finally
                    Free;
                  end;
                MonoBmp.Canvas.Brush.Color := clBlack;
                MonoBmp.Monochrome := True;
              end
            end
            else begin
              with MonoBmp do begin
                Width := IWidthA;
                Height := IHeightA;
                Canvas.Brush.Color := clWhite;
                Canvas.FillRect (IRectA);
                ImageList_Draw (FImageList.Handle, FImageIndex, Canvas.Handle,
                  0, 0, ILD_TRANSPARENT);
                Canvas.Brush.Color := clBlack;
                Monochrome := True;
              end;
            end;
          end
          else begin
            { The new Office 97 / MFC look }
            if not UsesMask and (FImageList = nil) then begin
              with TmpImage.Canvas do begin
                if not IsHighColorDIB then
                  abcGenerateMaskBitmap (MonoBmp, DDB, IRectA.TopLeft,
                    IRectA.BottomRight, [FTransparentColor, clWhite, clSilver])
                else
                  abcGenerateMaskBitmapFromDIB (MonoBmp, OriginalBmp,
                    SourceRect.TopLeft, IRectA.BottomRight, [-1, clWhite, clSilver]);
              end;
            end
            else begin
              { Generate the mask in MonoBmp. Make clWhite and clSilver transparent. }
              if not IsHighColorDIB then
                abcGenerateMaskBitmap (MonoBmp, DDB, SourceRect.TopLeft,
                  IRectA.BottomRight, [clWhite, clSilver])
              else
                abcGenerateMaskBitmapFromDIB (MonoBmp, OriginalBmp, SourceRect.TopLeft,
                  IRectA.BottomRight, [clWhite, clSilver]);
              if FImageList = nil then
                UseMaskBmp := OriginalMaskBmp
              else
                UseMaskBmp := MaskBmp;
              { and all the white colors in UseMaskBmp }
              with TBitmap.Create do
                try
                  Monochrome := True;
                  Width := UseMaskBmp.Width;
                  Height := UseMaskBmp.Height;
                  R := Rect(0, 0, Width, Height);
                  Canvas.CopyRect (R, UseMaskBmp.Canvas, R);
                  DC := Canvas.Handle;
                  with MonoBmp.Canvas do begin
                    BitBlt (Handle, 0, 0, IWidthA, IHeightA, DC,
                      SourceRect.Left, SourceRect.Top, ROP_DSna);
                    BitBlt (Handle, 0, 0, IWidthA, IHeightA, DC,
                      SourceRect.Left, SourceRect.Top, SRCPAINT);
                  end;
                finally
                  Free;
                end;
            end;
          end;

          with TmpImage.Canvas do begin
            Brush.Color := clBtnFace;
            FillRect (IRect);
            Brush.Color := clBtnHighlight;
            DC := Handle;
            SetTextColor (DC, clBlack);
            SetBkColor (DC, clWhite);
            BitBlt (DC, 1, 1, IWidthA, IHeightA,
              MonoBmp.Canvas.Handle, 0, 0, ROPs[FOldDisabledStyle]);
            Brush.Color := clBtnShadow;
            DC := Handle;
            SetTextColor (DC, clBlack);
            SetBkColor (DC, clWhite);
            BitBlt (DC, 0, 0, IWidthA, IHeightA,
              MonoBmp.Canvas.Handle, 0, 0, ROPs[FOldDisabledStyle]);
          end;

          FIndexs[B, State] := FGlyphList[B].AddMasked(TmpImage, clBtnFace);
        end;
      finally
        DDB.Free;
        MonoBmp.Free;
      end;
    end;
  finally
    MaskBmp.Free;
    TmpImage.Free;
    OriginalMaskBmp.Free;
    OriginalBmp.Free;
  end;
  Result.B := B;
  Result.I := FIndexs[B, State];

  if FCallDormant then
    FOriginal.Dormant;
  FOriginalMask.Dormant;

  if ABC_GLYPHS_SAVE_GDI then
  begin
    FStream.Clear;
    if not FOriginal.Empty then
      FOriginal.SaveToStream(FStream);
    FOriginal.OnChange := nil;
    FOriginal.Assign(nil);
    FOriginal.OnChange := GlyphChanged;

    FMaskStream.Clear;
    if not FOriginalMask.Empty then
      FOriginalMask.SaveToStream(FMaskStream);
    FOriginalMask.OnChange := nil;
    FOriginalMask.Assign(nil);
    FOriginalMask.OnChange := GlyphChanged;
  end;
end;

procedure TabcTBButtonGlyph.DrawButtonGlyph (Canvas: TCanvas; const GlyphPos: TPoint;
  State: TabcTBButtonState);
var
  Index: TBoolInt;
begin
  Index := CreateButtonGlyph(State);
  if Index.I <> -1 then
    ImageList_DrawEx (FGlyphList[Index.B].Handle, Index.I, Canvas.Handle,
      GlyphPos.X, GlyphPos.Y, 0, 0, CLR_NONE, CLR_NONE, ILD_TRANSPARENT);
end;

procedure TabcTBButtonGlyph.DrawButtonText (Canvas: TCanvas; const Caption: string;
  TextBounds: TRect; WordWrap: Boolean; Alignment: TAlignment;
  State: TabcTBButtonState);
const
  AlignmentFlags: array[TAlignment] of UINT = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Format: UINT;
begin
  Format := DT_VCENTER or AlignmentFlags[Alignment];
  if not WordWrap then
    Format := Format or DT_SINGLELINE
  else
    Format := Format or DT_WORDBREAK;
  with Canvas do begin
    Brush.Style := bsClear;
    if State = atsDisabled then begin
      OffsetRect (TextBounds, 1, 1);
      Font.Color := clBtnHighlight;
      DrawText (Handle, PChar(Caption), Length(Caption), TextBounds, Format);
      OffsetRect (TextBounds, -1, -1);
      Font.Color := clBtnShadow;
      DrawText (Handle, PChar(Caption), Length(Caption), TextBounds, Format);
    end
    else
      DrawText (Handle, PChar(Caption), Length(Caption), TextBounds, Format);
  end;
end;

procedure TabcTBButtonGlyph.DrawButtonDropArrow (Canvas: TCanvas;
  const X, Y, AWidth: Integer; State: TabcTBButtonState);
var
  X2: Integer;
begin
  with Canvas do begin
    X2 := X + AWidth div 2;
    if State = atsDisabled then begin
      Pen.Color := clBtnHighlight;
      Brush.Color := clBtnHighlight;
      Polygon ([Point(X2-1, Y+1), Point(X2+3, Y+1), Point(X2+1, Y+3)]);
      Pen.Color := clBtnShadow;
      Brush.Color := clBtnShadow;
      Polygon ([Point(X2-2, Y), Point(X2+2, Y), Point(X2, Y+2)]);
    end
    else begin
      Pen.Color := Font.Color;
      Brush.Color := Font.Color;
      Polygon ([Point(X2-2, Y), Point(X2+2, Y), Point(X2, Y+2)]);
    end;
  end;
end;

procedure TabcTBButtonGlyph.CalcButtonLayout (Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; DrawGlyph, DrawCaption: Boolean; const Caption: string;
  WordWrap: Boolean; Layout: TButtonLayout; Margin, Spacing: Integer;
  DropArrow: Boolean; DropArrowWidth: Integer; var GlyphPos, ArrowPos: TPoint;
  var TextBounds: TRect);
var
  TextPos: TPoint;
  ClientSize, GlyphSize, TextSize, ArrowSize: TPoint;
  HasGlyph: Boolean;
  TotalSize: TPoint;
  Format: UINT;
  Margin1, Spacing1: Integer;
  LayoutLeftOrRight: Boolean;
begin
  { calculate the item sizes }
  ClientSize := Point(Client.Right-Client.Left, Client.Bottom-Client.Top);

  GlyphSize.X := 0;
  GlyphSize.Y := 0;
  if DrawGlyph then begin
    if FImageList = nil then begin
      if FOriginal <> nil then begin
        GlyphSize := Point(FOriginalSize.X div FNumGlyphs, FOriginalSize.Y);
      end;
    end
    else begin
      GlyphSize.X := TCustomImageListAccess(FImageList).Width;
      GlyphSize.Y := TCustomImageListAccess(FImageList).Height;
    end;
  end;
  HasGlyph := (GlyphSize.X <> 0) and (GlyphSize.Y <> 0);

  if DropArrow then begin
    ArrowSize.X := DropArrowWidth;
    ArrowSize.Y := 3;
  end
  else begin
    ArrowSize.X := 0;
    ArrowSize.Y := 0;
  end;

  LayoutLeftOrRight := Layout in [blGlyphLeft, blGlyphRight];
  if not LayoutLeftOrRight and not HasGlyph then begin
    Layout := blGlyphLeft;
    LayoutLeftOrRight := True;
  end;

  if DrawCaption and (Caption <> '') then begin
    TextBounds := Rect(0, 0, Client.Right-Client.Left, 0);
    if LayoutLeftOrRight then
      Dec (TextBounds.Right, ArrowSize.X);
    Format := DT_CALCRECT;
    if WordWrap then begin
      Format := Format or DT_WORDBREAK;
      Margin1 := 4;
      if LayoutLeftOrRight and HasGlyph then begin
        if Spacing = -1 then
          Spacing1 := 4
        else
          Spacing1 := Spacing;
        Dec (TextBounds.Right, GlyphSize.X + Spacing1);
        if Margin <> -1 then
          Margin1 := Margin
        else
        if Spacing <> -1 then
          Margin1 := Spacing;
      end;
      Dec (TextBounds.Right, Margin1 * 2);
    end;
    DrawText (Canvas.Handle, PChar(Caption), Length(Caption), TextBounds, Format);
    TextSize := Point(TextBounds.Right - TextBounds.Left, TextBounds.Bottom -
      TextBounds.Top);
  end
  else begin
    TextBounds := Rect(0, 0, 0, 0);
    TextSize := Point(0,0);
  end;

  { If the layout has the glyph on the right or the left, then both the
    text and the glyph are centered vertically.  If the glyph is on the top
    or the bottom, then both the text and the glyph are centered horizontally.}
  if LayoutLeftOrRight then begin
    GlyphPos.Y := (ClientSize.Y - GlyphSize.Y + 1) div 2;
    TextPos.Y := (ClientSize.Y - TextSize.Y + 1) div 2;
  end
  else begin
    GlyphPos.X := (ClientSize.X - GlyphSize.X - ArrowSize.X + 1) div 2;
    TextPos.X := (ClientSize.X - TextSize.X + 1) div 2;
    if not HasGlyph then
      ArrowPos.X := TextPos.X + TextSize.X
    else
      ArrowPos.X := GlyphPos.X + GlyphSize.X;
  end;

  { if there is no text or no bitmap, then Spacing is irrelevant }
  if (TextSize.X = 0) or (TextSize.Y = 0) or not HasGlyph then
    Spacing := 0;

  { adjust Margin and Spacing }
  if Margin = -1 then begin
    if Spacing = -1 then begin
      TotalSize := Point(GlyphSize.X + TextSize.X + ArrowSize.X,
        GlyphSize.Y + TextSize.Y);
      if LayoutLeftOrRight then
        Margin := (ClientSize.X - TotalSize.X) div 3
      else
        Margin := (ClientSize.Y - TotalSize.Y) div 3;
      Spacing := Margin;
    end
    else begin
      TotalSize := Point(GlyphSize.X + Spacing + TextSize.X + ArrowSize.X,
        GlyphSize.Y + Spacing + TextSize.Y);
      if LayoutLeftOrRight then
        Margin := (ClientSize.X - TotalSize.X + 1) div 2
      else
        Margin := (ClientSize.Y - TotalSize.Y + 1) div 2;
    end;
  end
  else begin
    if Spacing = -1 then begin
      TotalSize := Point(ClientSize.X - (Margin + GlyphSize.X + ArrowSize.X),
        ClientSize.Y - (Margin + GlyphSize.Y));
      if LayoutLeftOrRight then
        Spacing := (TotalSize.X - TextSize.X) div 2
      else
        Spacing := (TotalSize.Y - TextSize.Y) div 2;
    end;
  end;

  case Layout of
    blGlyphLeft: begin
        GlyphPos.X := Margin;
        TextPos.X := GlyphPos.X + GlyphSize.X + Spacing;
        ArrowPos.X := TextPos.X + TextSize.X;
      end;
    blGlyphRight: begin
        ArrowPos.X := ClientSize.X - Margin - ArrowSize.X;
        GlyphPos.X := ArrowPos.X - GlyphSize.X;
        TextPos.X := GlyphPos.X - Spacing - TextSize.X;
      end;
    blGlyphTop: begin
        GlyphPos.Y := Margin;
        TextPos.Y := GlyphPos.Y + GlyphSize.Y + Spacing;
      end;
    blGlyphBottom: begin
        GlyphPos.Y := ClientSize.Y - Margin - GlyphSize.Y;
        TextPos.Y := GlyphPos.Y - Spacing - TextSize.Y;
      end;
  end;
  Inc (ArrowPos.X);
  if not HasGlyph then
    ArrowPos.Y := TextPos.Y + (TextSize.Y - ArrowSize.Y) div 2
  else
    ArrowPos.Y := GlyphPos.Y + (GlyphSize.Y - ArrowSize.Y) div 2;

  { fixup the result variables }
  with GlyphPos do begin
    Inc (X, Client.Left + Offset.X);
    Inc (Y, Client.Top + Offset.Y);
  end;
  with ArrowPos do begin
    Inc (X, Client.Left + Offset.X);
    Inc (Y, Client.Top + Offset.Y);
  end;
  OffsetRect (TextBounds, TextPos.X + Client.Left + Offset.X,
    TextPos.Y + Client.Top + Offset.X);
end;

function TabcTBButtonGlyph.Draw (Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; DrawGlyph, DrawCaption: Boolean; const Caption: string;
  WordWrap: Boolean; Alignment: TAlignment; Layout: TButtonLayout;
  Margin, Spacing: Integer; DropArrow: Boolean; DropArrowWidth: Integer;
  State: TabcTBButtonState): TRect;
var
  GlyphPos, ArrowPos: TPoint;
begin
  CalcButtonLayout (Canvas, Client, Offset, DrawGlyph, DrawCaption, Caption,
    WordWrap, Layout, Margin, Spacing, DropArrow, DropArrowWidth, GlyphPos,
    ArrowPos, Result);
  if DrawGlyph then
    DrawButtonGlyph (Canvas, GlyphPos, State);
  if DrawCaption then
    DrawButtonText (Canvas, Caption, Result, WordWrap, Alignment, State);
  if DropArrow then
    DrawButtonDropArrow (Canvas, ArrowPos.X, ArrowPos.Y, DropArrowWidth, State);
end;

function TabcTBButtonGlyph.GetGlyph: TBitmap;
begin
  if ABC_GLYPHS_SAVE_GDI then
    RestoreGlyph;
  Result := FOriginal;
end;

procedure TabcTBButtonGlyph.RestoreGlyph;
begin
  if FOriginal.Empty and (FStream.Size > 0) then
  begin
    FStream.Position := 0;
    FOriginal.LoadFromStream(FStream);
  end;
end;

function TabcTBButtonGlyph.GetGlyphMask: TBitmap;
begin
  if ABC_GLYPHS_SAVE_GDI then
    RestoreGlyphMask;
  Result := FOriginalMask;
end;

procedure TabcTBButtonGlyph.RestoreGlyphMask;
begin
  if FOriginalMask.Empty and (FMaskStream.Size > 0) then
  begin
    FMaskStream.Position := 0;
    FOriginalMask.LoadFromStream(FMaskStream);
  end;
end;

{ TabcToolbarButton }

procedure ButtonHookProc (Code: TabcHookProcCode; Wnd: HWND; WParam: WPARAM; LParam: LPARAM);
var
  P: TPoint;
begin
  case Code of
    ahpSendActivateApp:
      if (WParam = 0) and Assigned(abcButtonMouseInControl) and
         not abcButtonMouseInControl.FShowBorderWhenInactive then
        abcButtonMouseInControl.MouseLeft;
    ahpPostMouseMove: begin
        if Assigned(abcButtonMouseInControl) then begin
          GetCursorPos (P);
          if FindDragTarget(P, True) <> abcButtonMouseInControl then
            abcButtonMouseInControl.MouseLeft;
        end;
      end;
  end;
end;

constructor TabcToolbarButton.Create (AOwner: TComponent);
begin
  inherited;

  if ButtonMouseTimer = nil then begin
    ButtonMouseTimer := TTimer.Create(nil);
    ButtonMouseTimer.Enabled := False;
    ButtonMouseTimer.Interval := 125;  { 8 times a second }
  end;

  abcInstallHookProc (ButtonHookProc, [ahpSendActivateApp, ahpPostMouseMove],
    csDesigning in ComponentState);

  SetBounds (Left, Top, 23, 22);
  ControlStyle := [csCaptureMouse, csDoubleClicks, csOpaque];
  Color := clBtnFace;
  FGlyph := TabcTBButtonGlyph.Create;
  TabcTBButtonGlyph(FGlyph).OnChange := GlyphChanged;
  ParentFont := True;
  FAlignment := taCenter;
  FFlat := True;
  FHighlightWhenDown := True;
  FOpaque := True;
  FSpacing := 4;
  FMargin := -1;
  FLayout := blGlyphLeft;
  FDropdownArrow := True;
  FDropdownArrowWidth := abcDefaultDropdownArrowWidth;
  FRepeatDelay := 400;
  FRepeatInterval := 100;
  Inc (ButtonCount);
end;

destructor TabcToolbarButton.Destroy;
begin
  RemoveButtonMouseTimer;
  TabcTBButtonGlyph(FGlyph).Free;
  { The Notification method, which is sometimes called while the component is
    being destroyed, reads FGlyph and expects it to be valid, so it must be
    reset to nil }
  FGlyph := nil;
  abcUninstallHookProc (ButtonHookProc);
  Dec (ButtonCount);
  if ButtonCount = 0 then begin
    Pattern.Free;
    Pattern := nil;
    ButtonMouseTimer.Free;
    ButtonMouseTimer := nil;
  end;
  inherited;
end;

procedure TabcToolbarButton.Paint;
const
  EdgeStyles: array[Boolean, Boolean] of UINT = (
    (EDGE_RAISED, EDGE_SUNKEN),
    (BDR_RAISEDINNER, BDR_SUNKENOUTER));
  FlagStyles: array[Boolean] of UINT = (BF_RECT or BF_SOFT or BF_MIDDLE, BF_RECT);
var
  UseBmp: Boolean;
  Bmp: TBitmap;
  DrawCanvas: TCanvas;
  PaintRect, R: TRect;
  Offset: TPoint;
  StateDownOrExclusive, DropdownComboShown, UseDownAppearance, DrawBorder: Boolean;
begin
  UseBmp := FOpaque or not FFlat;
  if UseBmp then
    Bmp := TBitmap.Create
  else
    Bmp := nil;
  try
    if UseBmp then begin
      Bmp.Width := Width;
      Bmp.Height := Height;
      DrawCanvas := Bmp.Canvas;
      with DrawCanvas do begin
        Brush.Color := Color;
        FillRect (ClientRect);
      end;
    end
    else
      DrawCanvas := Canvas;
    DrawCanvas.Font := Self.Font;
    PaintRect := Rect(0, 0, Width, Height);

    StateDownOrExclusive := FState in [atsDown, atsExclusive];
    DropdownComboShown := FDropdownCombo and FUsesDropdown;
    UseDownAppearance := StateDownOrExclusive and
      (not DropdownComboShown or not FMenuIsDown);
    DrawBorder := (csDesigning in ComponentState) or
      (not FNoBorder and (not FFlat or StateDownOrExclusive or (FMouseInControl and (FState <> atsDisabled))));

    if DropdownComboShown then begin
      if DrawBorder then begin
        R := PaintRect;
        Dec (R.Right, DropdownComboSpace);
        R.Left := R.Right - DropdownArrowWidth;
        DrawEdge (DrawCanvas.Handle, R,
          EdgeStyles[FFlat, StateDownOrExclusive and FMenuIsDown],
          FlagStyles[FFlat]);
      end;
      Dec (PaintRect.Right, DropdownArrowWidth + DropdownComboSpace);
    end;
    if DrawBorder then
      DrawEdge (DrawCanvas.Handle, PaintRect, EdgeStyles[FFlat, UseDownAppearance],
        FlagStyles[FFlat]);
    if not FNoBorder then begin
      if FFlat then
        InflateRect (PaintRect, -1, -1)
      else
        InflateRect (PaintRect, -2, -2);
    end;

    if UseDownAppearance then begin
      if (FState = atsExclusive) and (not FFlat or not FMouseInControl) and
         FHighlightWhenDown then begin
        if Pattern = nil then CreateBrushPattern;
        DrawCanvas.Brush.Bitmap := Pattern;
        DrawCanvas.FillRect(PaintRect);
      end;
      Offset.X := 1;
      Offset.Y := 1;
    end
    else begin
      Offset.X := 0;
      Offset.Y := 0;
    end;

    TabcTBButtonGlyph(FGlyph).Draw (DrawCanvas, PaintRect, Offset,
      FDisplayMode <> admTextOnly, FDisplayMode <> admGlyphOnly,
      Caption, FWordWrap, FAlignment, FLayout, FMargin, FSpacing,
      FDropdownArrow and not FDropdownCombo and FUsesDropdown,
      DropdownArrowWidth, FState);
    if DropdownComboShown then
      TabcTBButtonGlyph(FGlyph).DrawButtonDropArrow (DrawCanvas, Width-DropdownArrowWidth-2,
        Height div 2 - 1, DropdownArrowWidth, FState);

    if UseBmp then
      Canvas.Draw (0, 0, Bmp);
  finally
    Bmp.Free;
  end;
end;

procedure TabcToolbarButton.RemoveButtonMouseTimer;
begin
  if abcButtonMouseInControl = Self then begin
    ButtonMouseTimer.Enabled := False;
    abcButtonMouseInControl := nil;
  end;
end;

procedure TabcToolbarButton.Loaded;
var
  State: TabcTBButtonState;
begin
  inherited;
  if Enabled then
    State := atsUp
  else
    State := atsDisabled;
  TabcTBButtonGlyph(FGlyph).CreateButtonGlyph (State);
end;

procedure TabcToolbarButton.Notification (AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then begin
    if AComponent = DropdownMenu then DropdownMenu := nil;
    if Assigned(FGlyph) and (AComponent = Images) then Images := nil;
  end;
end;

function TabcToolbarButton.PointInButton (X, Y: Integer): Boolean;
begin
  Result := (X >= 0) and
    (X < ClientWidth-((DropdownArrowWidth+DropdownComboSpace) * Ord(FDropdownCombo and FUsesDropdown))) and
    (Y >= 0) and (Y < ClientHeight);
end;

procedure TabcToolbarButton.MouseDown (Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if not Enabled then begin
    inherited;
    Exit;
  end;
  if Button <> mbLeft then begin
    MouseEntered;
    inherited;
  end
  else begin
    { We know mouse has to be over the control if the mouse went down. }
    MouseEntered;
    FMenuIsDown := FUsesDropdown and (not FDropdownCombo or
      (X >= Width-(DropdownArrowWidth+DropdownComboSpace)));
    try
      if not FDown then begin
        FState := atsDown;
        Redraw (True);
      end
      else
        if FAllowAllUp then
          Redraw (True);
      if not FMenuIsDown then
        FMouseIsDown := True;
      inherited;
      if FMenuIsDown then
        Click
      else
        if FRepeating then begin
          Click;
          if not Assigned(FRepeatTimer) then
            FRepeatTimer := TTimer.Create(Self);
          FRepeatTimer.Enabled := False;
          FRepeatTimer.Interval := FRepeatDelay;
          FRepeatTimer.OnTimer := RepeatTimerHandler;
          FRepeatTimer.Enabled := True;
        end;
    finally
      FMenuIsDown := False;
    end;
  end;
end;

procedure TabcToolbarButton.MouseMove (Shift: TShiftState; X, Y: Integer);
var
  P: TPoint;
  NewState: TabcTBButtonState;
  PtInButton: Boolean;
begin
  inherited;

  { Check if mouse just entered the control. It works better to check this
    in MouseMove rather than using CM_MOUSEENTER, since the VCL doesn't send
    a CM_MOUSEENTER in all cases
    Use FindDragTarget instead of PtInRect since we want to check based on
    the Z order }
  P := ClientToScreen(Point(X, Y));
  if (abcButtonMouseInControl <> Self) and (FindDragTarget(P, True) = Self) then begin
    if Assigned(abcButtonMouseInControl) then
      abcButtonMouseInControl.MouseLeft;
    { Like Office 97, only draw the active borders when the application is active }
    if FShowBorderWhenInactive or abcApplicationIsActive then begin
      abcButtonMouseInControl := Self;
      ButtonMouseTimer.OnTimer := ButtonMouseTimerHandler;
      ButtonMouseTimer.Enabled := True;
      MouseEntered;
    end;
  end;

  if FMouseIsDown then begin
    PtInButton := PointInButton(X, Y);
    if PtInButton and Assigned(FRepeatTimer) then
      FRepeatTimer.Enabled := True;
    if FDown then
      NewState := atsExclusive
    else begin
      if PtInButton then
        NewState := atsDown
      else
        NewState := atsUp;
    end;
    if NewState <> FState then begin
      FState := NewState;
      Redraw (True);
    end;
  end;
end;

procedure TabcToolbarButton.RepeatTimerHandler (Sender: TObject);
var
  P: TPoint;
begin
  FRepeatTimer.Interval := FRepeatInterval;
  GetCursorPos (P);
  P := ScreenToClient(P);
  if Repeating and FMouseIsDown and MouseCapture and PointInButton(P.X, P.Y) then
    Click
  else
    FRepeatTimer.Enabled := False;
end;

procedure TabcToolbarButton.WMCancelMode (var Message: TWMCancelMode);
begin
  FRepeatTimer.Free;
  FRepeatTimer := nil;
  if FMouseIsDown then begin
    FMouseIsDown := False;
    MouseLeft;
  end;
  { Delphi's default processing of WM_CANCELMODE sends a "fake" WM_LBUTTONUP
    message to the control, so inherited must only be called after setting
    FMouseIsDown to False }
  inherited;
end;

procedure TabcToolbarButton.WMLButtonDown(var Message: TWMLButtonDown);
//when closing form with abcPopupMenu showing
begin
  inherited;
  if (csDestroying in ComponentState) then
    Message.Result := 1;
end;

procedure TabcToolbarButton.MouseUp (Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  FRepeatTimer.Free;
  FRepeatTimer := nil;
  { Remove active border when right button is clicked }
  if (Button = mbRight) and Enabled then begin
    FMouseIsDown := False;
    MouseLeft;
  end;
  inherited;
  if (Button = mbLeft) and FMouseIsDown then begin
    FMouseIsDown := False;
    if PointInButton(X, Y) and not FRepeating then
      Click
    else
      MouseLeft;
  end;
end;

procedure TabcToolbarButton.Click;
var
  Popup, ShowMenu, RemoveClicks: Boolean;
  SaveAlignment: TPopupAlignment;
  SaveTrackButton: TTrackButton;
  ptPopup: TPoint;
  RepostList: TList; {pointers to TMsg's}
  Msg: TMsg;
  Repost: Boolean;
  I: Integer;
  P: TPoint;
  Form: TCustomForm;
  DockPos: TabcGetToolbarDockPosType;
begin
  if FRepeating and not FMenuIsDown then begin
    inherited;
    Exit;
  end;
  FInClick := True;
  try
    if (GroupIndex <> 0) and not FMenuIsDown then
      SetDown (not FDown);

    Popup := FUsesDropdown and (not FDropdownCombo or FMenuIsDown);
    if abcButtonsStayDown or Popup then begin
      if FState in [atsUp, atsMouseIn] then begin
        FState := atsDown;
        Redraw (True);
      end;
    end
    else begin
      if FState = atsDown then begin
        if FDown and (FGroupIndex <> 0) then
          FState := atsExclusive
        else
          FState := atsUp;
        Redraw (True);
      end;
    end;

    { Stop tracking }
    MouseLeft;
    if not Popup then begin
      Form := GetParentForm(Self);
      if Form <> nil then Form.ModalResult := ModalResult;
      inherited;
    end
    else begin
      if not FDropdownCombo then
        inherited;
      { It must release its capture before displaying the popup menu since
        this control uses csCaptureMouse. If it doesn't, the VCL seems to
        get confused and think the mouse is still captured even after the
        popup menu is displayed, causing mouse problems after the menu is
        dismissed. }
      MouseCapture := False;
      ShowMenu := Assigned(FDropdownMenu);
      RemoveClicks := True;
      if Assigned(FOnDropdown) then
        FOnDropdown (Self, ShowMenu, RemoveClicks);
      try
        if Assigned(FDropdownMenu) and ShowMenu then begin
          SaveAlignment := DropdownMenu.Alignment;
          SaveTrackButton := DropdownMenu.TrackButton;
          try
            DropdownMenu.Alignment := paLeft;
            ptPopup := Point(0, Height);

            if Assigned(abcGetToolbarDockPosProc) then begin
              DockPos := abcGetToolbarDockPosProc(Parent);
              { Drop out right or left side }
              case DockPos of
                agtLeft:
                  ptPopup := Point(Width, 0);
                agtRight:
                  begin
                    ptPopup := Point(0, 0);
                    DropdownMenu.Alignment := paRight;
                  end;
              end;
            end;

            ptPopup := ClientToScreen(ptPopup);
            with DropdownMenu do begin
              PopupComponent := Self;
              { In Delphi versions prior to 4 it avoids using the Popup method
                of TPopupMenu because it always uses the "track right button"
                flag (which disallowed the "click and drag" selecting motion many
                people are accustomed to). Delphi 4 has a TrackButton property
                to control the tracking button, so it can use the Popup method. }
                if NewStyleControls then
                  TrackButton := tbLeftButton
                else
                  TrackButton := tbRightButton;
                Popup (ptPopup.X, ptPopup.Y);
            end;
          finally
            if Assigned(DropdownMenu) then
            begin
              DropdownMenu.Alignment := SaveAlignment;
              DropdownMenu.TrackButton := SaveTrackButton;
            end;
          end;
        end;
      finally
        if RemoveClicks then begin
          { To prevent a mouse click from redisplaying the menu, filter all
            mouse up/down messages, and repost the ones that don't need
            removing. This is sort of bulky, but it's the only way I could
            find that works perfectly and like Office 97. }
          RepostList := TList.Create;
          try
            while PeekMessage(Msg, 0, WM_LBUTTONDOWN, WM_MBUTTONDBLCLK,
               PM_REMOVE or PM_NOYIELD) do
               { ^ The WM_LBUTTONDOWN to WM_MBUTTONDBLCLK range encompasses all
                 of the DOWN and DBLCLK messages for the three buttons }
              with Msg do begin
                Repost := True;
                case Message of
                  WM_QUIT: begin
                      { Throw back any WM_QUIT messages }
                      PostQuitMessage (wParam);
                      Break;
                    end;
                  WM_LBUTTONDOWN, WM_LBUTTONDBLCLK,
                  WM_RBUTTONDOWN, WM_RBUTTONDBLCLK,
                  WM_MBUTTONDOWN, WM_MBUTTONDBLCLK: begin
                      P := SmallPointToPoint(TSmallPoint(lParam));
                      Windows.ClientToScreen (hwnd, P);
                      if FindDragTarget(P, True) = Self then
                        Repost := False;
                    end;
                end;
                if Repost then begin
                  RepostList.Add (AllocMem(SizeOf(TMsg)));
                  PMsg(RepostList.Last)^ := Msg;
                end;
              end;
          finally
            for I := 0 to RepostList.Count-1 do begin
              with PMsg(RepostList[I])^ do
                PostMessage (hwnd, message, wParam, lParam);
              FreeMem (RepostList[I]);
            end;
            RepostList.Free;
          end;
        end;
      end;
    end;
  finally
    CancelMenuClick;
  end;
end;

procedure TabcToolbarButton.CancelMenuClick;
begin
  FInClick := False;
  if FState = atsDown then
    FState := atsUp;
  { Need to check if it's destroying in case the OnClick handler freed
    the button. If it doesn't check this here, it can sometimes cause an
    access violation }
  if not(csDestroying in ComponentState) then begin
    Redraw (True);
    MouseLeft;
  end;
end;

function TabcToolbarButton.GetPalette: HPALETTE;
begin
  Result := Glyph.Palette;
end;

function TabcToolbarButton.GetGlyph: TBitmap;
begin
  Result := TabcTBButtonGlyph(FGlyph).Glyph;
end;

procedure TabcToolbarButton.SetGlyph (Value: TBitmap);
begin
  TabcTBButtonGlyph(FGlyph).Glyph := Value;
  Redraw (True);
end;

function TabcToolbarButton.GetGlyphMask: TBitmap;
begin
  Result := TabcTBButtonGlyph(FGlyph).GlyphMask;
end;

procedure TabcToolbarButton.SetGlyphMask (Value: TBitmap);
begin
  TabcTBButtonGlyph(FGlyph).GlyphMask := Value;
  Redraw (True);
end;

procedure TabcToolbarButton.SetHighlightWhenDown (Value: Boolean);
begin
  if FHighlightWhenDown <> Value then begin
    FHighlightWhenDown := Value;
    if Down then
      Redraw (True);
  end;
end;

function TabcToolbarButton.GetImageIndex: TImageIndex;
begin
  Result := TabcTBButtonGlyph(FGlyph).FImageIndex;
end;

procedure TabcToolbarButton.SetImageIndex (Value: TImageIndex);
begin
  if TabcTBButtonGlyph(FGlyph).FImageIndex <> Value then begin
    TabcTBButtonGlyph(FGlyph).FImageIndex := Value;
    if Assigned(TabcTBButtonGlyph(FGlyph).FImageList) then
      TabcTBButtonGlyph(FGlyph).GlyphChanged (nil);
  end;
end;

function TabcToolbarButton.GetImages: TCustomImageList;
begin
  Result := TabcTBButtonGlyph(FGlyph).FImageList;
end;

procedure TabcToolbarButton.SetImages (Value: TCustomImageList);
begin
  with TabcTBButtonGlyph(FGlyph) do
    if FImageList <> Value then begin
      if FImageList <> nil then
        FImageList.UnRegisterChanges (FImageChangeLink);
      FImageList := Value;
      if FImageList <> nil then begin
        if FImageChangeLink = nil then begin
          FImageChangeLink := TChangeLink.Create;
          FImageChangeLink.OnChange := GlyphChanged;
        end;
        FImageList.RegisterChanges (FImageChangeLink);
        FImageList.FreeNotification (Self);
      end
      else begin
        FImageChangeLink.Free;
        FImageChangeLink := nil;
      end;
      UpdateNumGlyphs;
    end;
end;

function TabcToolbarButton.GetNumGlyphs: TabcTBNumGlyphs;
begin
  Result := TabcTBButtonGlyph(FGlyph).NumGlyphs;
end;

procedure TabcToolbarButton.SetNumGlyphs (Value: TabcTBNumGlyphs);
begin
  if Value < Low(TabcTBNumGlyphs) then
    Value := Low(TabcTBNumGlyphs)
  else
  if Value > High(TabcTBNumGlyphs) then
    Value := High(TabcTBNumGlyphs);
  if Value <> TabcTBButtonGlyph(FGlyph).NumGlyphs then begin
    TabcTBButtonGlyph(FGlyph).NumGlyphs := Value;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.GlyphChanged(Sender: TObject);
begin
  Redraw (True);
end;

procedure TabcToolbarButton.UpdateExclusive;
var
  I: Integer;
  Ctl: TControl;
begin
  if (FGroupIndex <> 0) and (Parent <> nil) then
    with Parent do
      for I := 0 to ControlCount-1 do begin
        Ctl := Controls[I];
        if (Ctl <> Self) and (Ctl is TabcToolbarButton) then
          with TabcToolbarButton(Ctl) do
            if FGroupIndex = Self.FGroupIndex then begin
              if Self.Down and FDown then begin
                FDown := False;
                FState := atsUp;
                Redraw (True);
              end;
              FAllowAllUp := Self.AllowAllUp;
            end;
      end;
end;

procedure TabcToolbarButton.SetDown (Value: Boolean);
begin
  if FGroupIndex = 0 then
    Value := False;
  if Value <> FDown then begin
    if FDown and (not FAllowAllUp) then Exit;
    FDown := Value;
    if not Enabled then
      FState := atsDisabled
    else begin
      if Value then
        FState := atsExclusive
      else
        FState := atsUp;
    end;
    Redraw (True);
    if Value then UpdateExclusive;
  end;
end;

procedure TabcToolbarButton.SetFlat (Value: Boolean);
begin
  if FFlat <> Value then begin
    FFlat := Value;
    if FOpaque or not FFlat then
      ControlStyle := ControlStyle + [csOpaque]
    else
      ControlStyle := ControlStyle - [csOpaque];
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetGroupIndex (Value: Integer);
begin
  if FGroupIndex <> Value then begin
    FGroupIndex := Value;
    UpdateExclusive;
  end;
end;

procedure TabcToolbarButton.SetLayout (Value: TButtonLayout);
begin
  if FLayout <> Value then begin
    FLayout := Value;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetMargin (Value: Integer);
begin
  if (FMargin <> Value) and (Value >= -1) then begin
    FMargin := Value;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetNoBorder (Value: Boolean);
begin
  if FNoBorder <> Value then begin
    FNoBorder := Value;
    Invalidate;
  end;
end;

procedure TabcToolbarButton.SetOldDisabledStyle (Value: Boolean);
begin
  if FOldDisabledStyle <> Value then begin
    FOldDisabledStyle := Value;
    with TabcTBButtonGlyph(FGlyph) do begin
      FOldDisabledStyle := Value;
      Invalidate;
    end;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetOpaque (Value: Boolean);
begin
  if FOpaque <> Value then begin
    FOpaque := Value;
    if FOpaque or not FFlat then
      ControlStyle := ControlStyle + [csOpaque]
    else
      ControlStyle := ControlStyle - [csOpaque];
    Invalidate;
  end;
end;

procedure TabcToolbarButton.Redraw (const Erase: Boolean);
var
  AddedOpaque: Boolean;
begin
  if FOpaque or not FFlat or not Erase then begin
    { Temporarily add csOpaque to the style. This prevents Invalidate from
      erasing, which isn't needed when Erase is false. }
    AddedOpaque := False;
    if not(csOpaque in ControlStyle) then begin
      AddedOpaque := True;
      ControlStyle := ControlStyle + [csOpaque];
    end;
    try
      Invalidate;
    finally
      if AddedOpaque then
        ControlStyle := ControlStyle - [csOpaque];
    end;
  end
  else
  if not(FOpaque or not FFlat) then
    Invalidate;
end;

procedure TabcToolbarButton.SetSpacing (Value: Integer);
begin
  if Value <> FSpacing then begin
    FSpacing := Value;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetAllowAllUp (Value: Boolean);
begin
  if FAllowAllUp <> Value then begin
    FAllowAllUp := Value;
    UpdateExclusive;
  end;
end;

procedure TabcToolbarButton.SetDropdownMenu (Value: TPopupMenu);
var
  NewUsesDropdown: Boolean;
begin
  if FDropdownMenu <> Value then begin
    FDropdownMenu := Value;
    if Assigned(Value) then begin
      Value.FreeNotification (Self);
    end;
    NewUsesDropdown := FDropdownAlways or Assigned(Value);
    if FUsesDropdown <> NewUsesDropdown then begin
      FUsesDropdown := NewUsesDropdown;
      if FDropdownArrow or FDropdownCombo then
        Redraw (True);
    end;

  end;
end;

procedure TabcToolbarButton.SetWordWrap (Value: Boolean);
begin
  if FWordWrap <> Value then begin
    FWordWrap := Value;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetAlignment (Value: TAlignment);
begin
  if FAlignment <> Value then begin
    FAlignment := Value;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetDropdownAlways (Value: Boolean);
var
  NewUsesDropdown: Boolean;
begin
  if FDropdownAlways <> Value then begin
    FDropdownAlways := Value;
    NewUsesDropdown := Value or Assigned(FDropdownMenu);
    if FUsesDropdown <> NewUsesDropdown then begin
      FUsesDropdown := NewUsesDropdown;
      if FDropdownArrow or FDropdownCombo then
        Redraw (True);
    end;
  end;
end;

procedure TabcToolbarButton.SetDropdownArrow (Value: Boolean);
begin
  if FDropdownArrow <> Value then begin
    FDropdownArrow := Value;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetDropdownArrowWidth (Value: Integer);
var
  Diff: Integer;
begin
  if Value < 7 then Value := 7;
  if FDropdownArrowWidth <> Value then begin
    Diff := Value - FDropdownArrowWidth;
    FDropdownArrowWidth := Value;
    if not(csLoading in ComponentState) and FDropdownCombo then
      Width := Width + Diff;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetDropdownCombo (Value: Boolean);
var
  W: Integer;
begin
  if FDropdownCombo <> Value then begin
    FDropdownCombo := Value;
    if not(csLoading in ComponentState) then begin
      if Value then
        Width := Width + (DropdownArrowWidth + DropdownComboSpace)
      else begin
        W := Width - (DropdownArrowWidth + DropdownComboSpace);
        if W < 1 then W := 1;
        Width := W;
      end;
    end;
    Redraw (True);
  end;
end;

procedure TabcToolbarButton.SetDisplayMode (Value: TabcTBButtonDisplayMode);
begin
  if FDisplayMode <> Value then begin
    FDisplayMode := Value;
    Redraw (True);
  end;
end;

function TabcToolbarButton.GetCallDormant: Boolean;
begin
  Result := TabcTBButtonGlyph(FGlyph).FCallDormant;
end;

procedure TabcToolbarButton.SetCallDormant (Value: Boolean);
begin
  TabcTBButtonGlyph(FGlyph).FCallDormant := Value;
end;

function TabcToolbarButton.IsCheckedStored: Boolean;
begin
  Result := (ActionLink = nil) or not TabcToolbarButtonActionLink(ActionLink).IsCheckedLinked;
end;

function TabcToolbarButton.IsHelpContextStored: Boolean;
begin
  Result := (ActionLink = nil) or not TabcToolbarButtonActionLink(ActionLink).IsHelpContextLinked;
end;

function TabcToolbarButton.IsImageIndexStored: Boolean;
begin
  Result := (ActionLink = nil) or not TabcToolbarButtonActionLink(ActionLink).IsImageIndexLinked;
end;

procedure TabcToolbarButton.ActionChange (Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
    begin
      if not CheckDefaults or (Self.Down = False) then
        Self.Down := Checked;
      if not CheckDefaults or (Self.HelpContext = 0) then
        Self.HelpContext := HelpContext;
      if not CheckDefaults or (Self.ImageIndex = -1) then
        Self.ImageIndex := ImageIndex;
    end;
end;

function TabcToolbarButton.GetActionLinkClass: TControlActionLinkClass;
begin
  Result := TabcToolbarButtonActionLink;
end;

procedure TabcToolbarButton.AssignTo (Dest: TPersistent);
begin
  inherited;
  if Dest is TCustomAction then
    TCustomAction(Dest).Checked := Self.Down;
end;

procedure TabcToolbarButton.WMLButtonDblClk (var Message: TWMLButtonDblClk);
begin
  inherited;
  if FDown then DblClick;
end;

procedure TabcToolbarButton.CMEnabledChanged (var Message: TMessage);
begin
  if not Enabled then begin
    FState := atsDisabled;
    FMouseInControl := False;
    FMouseIsDown := False;
    RemoveButtonMouseTimer;
    Perform (WM_CANCELMODE, 0, 0);
  end
  else
  if FState = atsDisabled then
    if FDown and (FGroupIndex <> 0) then
      FState := atsExclusive
    else
      FState := atsUp;
  Redraw (True);
end;

procedure TabcToolbarButton.CMDialogChar (var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Assigned(Parent) and Parent.CanFocus and
       Enabled and Visible and (DisplayMode <> admGlyphOnly) and
      (ssAlt in KeyDataToShiftState(KeyData)) then begin
      { NOTE: There is a bug in TSpeedButton where accelerator keys are still
        processed even when the button is not visible. The 'and Visible'
        corrects it, so TabcToolbarButton doesn't have this problem. }
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TabcToolbarButton.CMDialogKey (var Message: TCMDialogKey);
begin
  with Message do
    if (((CharCode = VK_RETURN) and FDefault) or
        ((CharCode = VK_ESCAPE) and FCancel)) and
       (KeyDataToShiftState(Message.KeyData) = []) and
       Assigned(Parent) and Parent.CanFocus and Enabled and Visible then begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TabcToolbarButton.CMFontChanged (var Message: TMessage);
begin
  Redraw (True);
end;

procedure TabcToolbarButton.CMTextChanged (var Message: TMessage);
begin
  Redraw (True);
end;

procedure TabcToolbarButton.CMSysColorChange (var Message: TMessage);
begin
  inherited;
  if Assigned(Pattern) and
     ((PatternBtnFace <> TColor(GetSysColor(COLOR_BTNFACE))) or
      (PatternBtnHighlight <> TColor(GetSysColor(COLOR_BTNHIGHLIGHT)))) then begin
    Pattern.Free;
    Pattern := nil;
  end;
  with TabcTBButtonGlyph(FGlyph) do begin
    Invalidate;
    CreateButtonGlyph (FState);
  end;
end;

procedure TabcToolbarButton.MouseEntered;
begin
  if Enabled and not FMouseInControl then begin
    FMouseInControl := True;
    if FState = atsUp then
      FState := atsMouseIn;
    if FFlat or (NumGlyphs >= 5) then
      Redraw (FDown or (NumGlyphs >= 5));
    if Assigned(FOnMouseEnter) then
      FOnMouseEnter (Self);
  end;
end;

procedure TabcToolbarButton.MouseLeft;
var
  OldState: TabcTBButtonState;
begin
  if abcButtonMouseInControl = Self then
    RemoveButtonMouseTimer;

  if Enabled and FMouseInControl and not FMouseIsDown then begin
    FMouseInControl := False;
    RemoveButtonMouseTimer;
    OldState := FState;
    if (FState = atsMouseIn) or (not FInClick and (FState = atsDown)) then begin
      if FDown and (FGroupIndex <> 0) then
        FState := atsExclusive
      else
        FState := atsUp;
    end;
    if FFlat or ((NumGlyphs >= 5) or ((OldState = atsMouseIn) xor (FState <> OldState))) then
      Redraw (True);
    if Assigned(FOnMouseExit) then
      FOnMouseExit (Self);
  end;
end;

procedure TabcToolbarButton.ButtonMouseTimerHandler (Sender: TObject);
var
  P: TPoint;
begin
  { The button mouse timer is used to periodically check if mouse has left.
    Normally it receives a CM_MOUSELEAVE, but the VCL does not send a
    CM_MOUSELEAVE if the mouse is moved quickly from the button to another
    application's window. For some reason, this problem doesn't seem to occur
    on Windows NT 4 -- only 95 and 3.x.

    The timer (which ticks 8 times a second) is only enabled when the
    application is active and the mouse is over a button, so it uses virtually
    no processing power.

    For something interesting to try: If you want to know just how often this
    is called, try putting a Beep call in here }

  GetCursorPos (P);
  if FindDragTarget(P, True) <> Self then
    MouseLeft;
end;


{ TabcToolbarEdit - internal }

constructor TabcToolbarEdit.Create (AOwner: TComponent);
begin
  inherited;
  AutoSize := False;
  Ctl3D := False;
  BorderStyle := bsNone;
  ControlStyle := ControlStyle - [csFramed]; {fixes a VCL bug with Win 3.x}
  Height := 19;
  if abcTBEditCount = 0 then
    abcRegisterTBControlClass (TabcToolbarEdit);
  Inc (abcTBEditCount);
end;

destructor TabcToolbarEdit.Destroy;
begin
  Dec (abcTBEditCount);
  if abcTBEditCount = 0 then
    abcUnregisterTBControlClass (TabcToolbarEdit);
  inherited;
end;

procedure TabcToolbarEdit.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseInControl := True;
  DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarEdit.CMMouseLeave (var Message: TMessage);
begin
  inherited;
  MouseInControl := False;
  DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarEdit.NewAdjustHeight;
var
  DC: HDC;
  SaveFont: HFONT;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics (DC, Metrics);
  SelectObject (DC, SaveFont);
  ReleaseDC (0, DC);

  Height := Metrics.tmHeight + 6;
end;

procedure TabcToolbarEdit.Loaded;
begin
  inherited;
  if not(csDesigning in ComponentState) then
    NewAdjustHeight;
end;

procedure TabcToolbarEdit.CMEnabledChanged (var Message: TMessage);
const
  EnableColors: array[Boolean] of TColor = (clBtnFace, clWindow);
begin
  inherited;
  Color := EnableColors[Enabled];
end;

procedure TabcToolbarEdit.CMFontChanged (var Message: TMessage);
begin
  inherited;
  if not((csDesigning in ComponentState) and (csLoading in ComponentState)) then
    NewAdjustHeight;
end;

procedure TabcToolbarEdit.WMSetFocus (var Message: TWMSetFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarEdit.WMKillFocus (var Message: TWMKillFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarEdit.WMNCCalcSize (var Message: TWMNCCalcSize);
begin
  InflateRect (Message.CalcSize_Params^.rgrc[0], -3, -3);
end;

procedure TabcToolbarEdit.WMNCPaint (var Message: TMessage);
begin
  DrawNCArea (False, 0, HRGN(Message.WParam));
end;

procedure TabcToolbarEdit.DrawNCArea (const DrawToDC: Boolean; const ADC: HDC;
  const Clip: HRGN);
var
  DC: HDC;
  R: TRect;
  BtnFaceBrush, WindowBrush: HBRUSH;
begin
  if not DrawToDC then
    DC := GetWindowDC(Handle)
  else
    DC := ADC;
  try
    { Use update region }
    if not DrawToDC then
      abcSelectNCUpdateRgn (Handle, DC, Clip);

    { This works around WM_NCPAINT problem described at top of source code }
    {no!  R := Rect(0, 0, Width, Height);}
    GetWindowRect (Handle, R);  OffsetRect (R, -R.Left, -R.Top);
    BtnFaceBrush := GetSysColorBrush(COLOR_BTNFACE);
    WindowBrush := GetSysColorBrush(COLOR_WINDOW);
    if ((csDesigning in ComponentState) and Enabled) or
       (not(csDesigning in ComponentState) and
        (Focused or (MouseInControl and not abcControlIsTBControl(Screen.ActiveControl)))) then begin
      DrawEdge (DC, R, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      with R do begin
        FillRect (DC, Rect(Left, Top, Left+1, Bottom-1), BtnFaceBrush);
        FillRect (DC, Rect(Left, Top, Right-1, Top+1), BtnFaceBrush);
      end;
      DrawEdge (DC, R, BDR_SUNKENINNER, BF_BOTTOMRIGHT);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, WindowBrush);
    end
    else begin
      FrameRect (DC, R, BtnFaceBrush);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, BtnFaceBrush);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, WindowBrush);
    end;
  finally
    if not DrawToDC then
      ReleaseDC (Handle, DC);
  end;
end;

procedure EditNCPaintProc (Wnd: HWND; DC: HDC; AppData: Longint);
begin
  TabcToolbarEdit(AppData).DrawNCArea (True, DC, 0);
end;

procedure TabcToolbarEdit.WMPrint (var Message: TMessage);
begin
  abcHandleWMPrint (Handle, Message, EditNCPaintProc, Longint(Self));
end;

procedure TabcToolbarEdit.WMPrintClient (var Message: TMessage);
begin
  abcHandleWMPrintClient (Self, Message);
end;

end.
