unit abctdbe;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Based on Toolbar97 version 1.76a Licensed from Jordan Russell - Thanks Jordan!
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, Classes, Controls, Forms, Graphics, StdCtrls,
  DBCtrls;

type
  { TabcToolbarDBEdit }

  TabcToolbarDBEdit = class(TDBEdit)
  private
    MouseInControl: Boolean;
    procedure DrawNCArea (const DrawToDC: Boolean; const ADC: HDC;
      const Clip: HRGN);
    procedure NewAdjustHeight;
    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave (var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMKillFocus (var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMNCCalcSize (var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCPaint (var Message: TMessage); message WM_NCPAINT;
    procedure WMPrint (var Message: TMessage); message WM_PRINT;
    procedure WMPrintClient (var Message: TMessage); message WM_PRINTCLIENT;
    procedure WMSetFocus (var Message: TWMSetFocus); message WM_SETFOCUS;
  protected
    procedure Loaded; override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
  end;

implementation

uses abctcmn, abctctls;

var
  abcTBDBEditCount: Integer = 0;

{ TabcToolbarDBEdit - internal }

constructor TabcToolbarDBEdit.Create (AOwner: TComponent);
begin
  inherited;
  AutoSize := False;
  Ctl3D := False;
  BorderStyle := bsNone;
  ControlStyle := ControlStyle - [csFramed]; {fixes a VCL bug with Win 3.x}
  Height := 19;
  if abcTBDBEditCount = 0 then
    abcRegisterTBControlClass (TabcToolbarDBEdit);
  Inc (abcTBDBEditCount);
end;

destructor TabcToolbarDBEdit.Destroy;
begin
  Dec (abcTBDBEditCount);
  if abcTBDBEditCount = 0 then
    abcUnregisterTBControlClass (TabcToolbarDBEdit);
  inherited;
end;

procedure TabcToolbarDBEdit.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseInControl := True;
  DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarDBEdit.CMMouseLeave (var Message: TMessage);
begin
  inherited;
  MouseInControl := False;
  DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarDBEdit.NewAdjustHeight;
var
  DC: HDC;
  SaveFont: HFONT;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics (DC, Metrics);
  SelectObject (DC, SaveFont);
  ReleaseDC (0, DC);

  Height := Metrics.tmHeight + 6;
end;

procedure TabcToolbarDBEdit.Loaded;
begin
  inherited;
  if not(csDesigning in ComponentState) then
    NewAdjustHeight;
end;

procedure TabcToolbarDBEdit.CMEnabledChanged (var Message: TMessage);
const
  EnableColors: array[Boolean] of TColor = (clBtnFace, clWindow);
begin
  inherited;
  Color := EnableColors[Enabled];
end;

procedure TabcToolbarDBEdit.CMFontChanged (var Message: TMessage);
begin
  inherited;
  if not((csDesigning in ComponentState) and (csLoading in ComponentState)) then
    NewAdjustHeight;
end;

procedure TabcToolbarDBEdit.WMSetFocus (var Message: TWMSetFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarDBEdit.WMKillFocus (var Message: TWMKillFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    DrawNCArea (False, 0, 0);
end;

procedure TabcToolbarDBEdit.WMNCCalcSize (var Message: TWMNCCalcSize);
begin
  InflateRect (Message.CalcSize_Params^.rgrc[0], -3, -3);
end;

procedure TabcToolbarDBEdit.WMNCPaint (var Message: TMessage);
begin
  DrawNCArea (False, 0, HRGN(Message.WParam));
end;

procedure TabcToolbarDBEdit.DrawNCArea (const DrawToDC: Boolean; const ADC: HDC;
  const Clip: HRGN);
var
  DC: HDC;
  R: TRect;
  BtnFaceBrush, WindowBrush: HBRUSH;
begin
  if not DrawToDC then
    DC := GetWindowDC(Handle)
  else
    DC := ADC;
  try
    { Use update region }
    if not DrawToDC then
      abcSelectNCUpdateRgn (Handle, DC, Clip);

    { This works around WM_NCPAINT problem described at top of source code }
    {no!  R := Rect(0, 0, Width, Height);}
    GetWindowRect (Handle, R);  OffsetRect (R, -R.Left, -R.Top);
    BtnFaceBrush := GetSysColorBrush(COLOR_BTNFACE);
    WindowBrush := GetSysColorBrush(COLOR_WINDOW);
    if ((csDesigning in ComponentState) and Enabled) or
       (not(csDesigning in ComponentState) and
        (Focused or (MouseInControl and not abcControlIsTBControl(Screen.ActiveControl)))) then begin
      DrawEdge (DC, R, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      with R do begin
        FillRect (DC, Rect(Left, Top, Left+1, Bottom-1), BtnFaceBrush);
        FillRect (DC, Rect(Left, Top, Right-1, Top+1), BtnFaceBrush);
      end;
      DrawEdge (DC, R, BDR_SUNKENINNER, BF_BOTTOMRIGHT);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, WindowBrush);
    end
    else begin
      FrameRect (DC, R, BtnFaceBrush);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, BtnFaceBrush);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, WindowBrush);
    end;
  finally
    if not DrawToDC then
      ReleaseDC (Handle, DC);
  end;
end;

procedure EditNCPaintProc (Wnd: HWND; DC: HDC; AppData: Longint);
begin
  TabcToolbarDBEdit(AppData).DrawNCArea (True, DC, 0);
end;

procedure TabcToolbarDBEdit.WMPrint (var Message: TMessage);
begin
  abcHandleWMPrint (Handle, Message, EditNCPaintProc, Longint(Self));
end;

procedure TabcToolbarDBEdit.WMPrintClient (var Message: TMessage);
begin
  abcHandleWMPrintClient (Self, Message);
end;

end.
