unit abctileb;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses Windows, Messages, SysUtils, Classes, Graphics;

type

  { TabcGradient }

  TabcColorCount = 1..236;

  TabcDrawDirection = (ddVertical, ddHorizontal, ddLRDiagonal, ddRLDiagonal,ddCenter);

  TabcGradientDrawMethod = (agdDraw, agdStretch, agdWinAPI);

  TabcGradient = class(TPersistent)
  private
    FBuffer: TBitmap;
    FRefreshBuffer: Boolean;
    FColorCount: TabcColorCount;
    FDirection: TabcDrawDirection;
    FDrawBuffered: Boolean;
    FDrawMethod: TabcGradientDrawMethod;
    FColorTo: TColor;
    FColorFrom: TColor;
    FFromR, FFromG, FFromB: Byte;
    FMidR, FMidG, FMidB: Byte;
    FToR, FToG, FToB: Byte;
    FOnChange: TNotifyEvent;
    function GetStepColor(Step: TabcColorCount): TColor;
    procedure SetColorCount(const Value: TabcColorCount);
    procedure SetColorFrom(const Value: TColor);
    procedure SetColorTo(const Value: TColor);
    procedure SetDirection(const Value: TabcDrawDirection);
    procedure SetDrawBuffered(const Value: Boolean);
    procedure SetDrawMethod(const Value: TabcGradientDrawMethod);
    procedure Paint(const DC: HDC; const ARect: TRect);
    procedure PaintVertical(const DC: HDC; const ARect: TRect);
    procedure PaintHorizontal(const DC: HDC; const ARect: TRect);
    procedure PaintLRDiagonal(const DC: HDC; const ARect: TRect);
    procedure PaintRLDiagonal(const DC: HDC; const ARect: TRect);
    procedure PaintCenter(const DC: HDC; const ARect: TRect);
    procedure WinAPIFill(const DC: HDC; const ARect: TRect);
    procedure WinAPIFillVertical(const DC: HDC; const ARect: TRect);
    procedure WinAPIFillHorizontal(const DC: HDC; const ARect: TRect);
    procedure WinAPIFillLRDiagonal(const DC: HDC; const ARect: TRect);
    procedure WinAPIFillRLDiagonal(const DC: HDC; const ARect: TRect);
    procedure WinAPIFillCenter(const DC: HDC; const ARect: TRect);
  protected
    procedure InnerDrawToDC(const DC: HDC; const ARect: TRect; const aDrawMethod: TabcGradientDrawMethod);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Draw(const Canvas: TCanvas; const ARect: TRect);
    procedure DrawToDC(const DC: HDC; const ARect: TRect);
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property ColorCount: TabcColorCount read FColorCount write SetColorCount default 236;
    property ColorFrom: TColor read FColorFrom write SetColorFrom default clBlue;
    property ColorTo: TColor read FColorTo write SetColorTo default clBlack;
    property Direction: TabcDrawDirection read FDirection write SetDirection default ddVertical;
    property DrawBuffered: Boolean read FDrawBuffered write SetDrawBuffered default False;
    property DrawMethod: TabcGradientDrawMethod read FDrawMethod write SetDrawMethod default agdDraw;
  end;

  { TabcTileBack }

  TabcTileBackStyle = (
    tbsNone,
    tbsGradient,
    tbsPicCenter, tbsPicClip, tbsPicFit, tbsPicHeight, tbsPicStretch, tbsPicTile,
    tbsPicWidth);

  TabcTileBack = class(TPersistent)
  private
    FGradient: TabcGradient;
    FPicture: TPicture;
    FStyle: TabcTileBackStyle;
    FOnChange: TNotifyEvent;
    procedure DrawPicture(const Canvas: TCanvas; const ARect: TRect);
    procedure SetGradient(const Value: TabcGradient);
    procedure SetPicture(const Value: TPicture);
    procedure SetStyle(const Value: TabcTileBackStyle);
    procedure SetOnChange(const Value: TNotifyEvent);
  public
    constructor Create;
    destructor Destroy; override;
    function Empty: Boolean;
    procedure Assign(Source: TPersistent); override;
    procedure Draw(const Canvas: TCanvas; const ARect: TRect);
    property OnChange: TNotifyEvent read FOnChange write SetOnChange;
  published
    property Gradient: TabcGradient read FGradient write SetGradient;
    property Picture: TPicture read FPicture write SetPicture;
    property Style: TabcTileBackStyle read FStyle write SetStyle default tbsNone;
  end;

implementation

{ GradientFill API Dynamic load for Win 5 and up }

{ Fix D5 API declaration of COLOR16 }

type
  COLOR16 = Word;

  TTriVertex = packed record
    x: Longint;
    y: Longint;
    Red: COLOR16;
    Green: COLOR16;
    Blue: COLOR16;
    Alpha: COLOR16;
  end;
  PTriVertex = ^TTriVertex;

  TGradientRect = packed record
    UpperLeft: ULONG;
    LowerRight: ULONG;
  end;
  PGradientRect = ^TGradientRect;

  TGradientTriangle = packed record
    Vertex1: ULONG;
    Vertex2: ULONG;
    Vertex3: ULONG;
  end;
  PGradientTriangle = ^TGradientTriangle;

  TGradientFillProc = function(DC: HDC; p2: PTriVertex; p3: ULONG; p4: Pointer; p5, p6: ULONG): BOOL; stdcall;

var
  msImgProcsLoaded: Boolean = False;
  msImgDLLHandle: HMODULE = 0;
  mGradientFill: TGradientFillProc = nil;

function IsMSImg32Platform: boolean;
var
  Ver: TOsVersionInfo;
begin
  Ver.dwOSVersionInfoSize := sizeof(Ver);
  GetVersionEx(Ver);
  if (Ver.dwPlatformID = VER_PLATFORM_WIN32_NT) and
     (Ver.dwMajorVersion >= 5) then
       Result := true
  else Result := false;
end;

{$IFDEF DELPHI5}
{$ELSE}

const
  msimg32   = 'msimg32.dll';
  GRADIENT_FILL_RECT_H = $00000000;
  GRADIENT_FILL_RECT_V = $00000001;
  GRADIENT_FILL_TRIANGLE = $00000002;
  GRADIENT_FILL_OP_FLAG = $000000ff;

function SafeLoadLibrary(const Filename: string): HMODULE;
var
  OldMode: UINT;
  FPUControlWord: Word;
begin
  OldMode := SetErrorMode(SEM_NOOPENFILEERRORBOX);
  try
    asm
      FNSTCW  FPUControlWord
    end;
    try
      Result := LoadLibrary(PChar(Filename));
    finally
      asm
        FNCLEX
        FLDCW FPUControlWord
      end;
    end;
  finally
    SetErrorMode(OldMode);
  end;
end;

{$ENDIF}

procedure LoadMSImgProcs;
var
  msImgDLL: HModule;
begin
  if msImgProcsLoaded then Exit;
  msImgProcsLoaded := True;
  if IsMSImg32Platform then
  begin
    msImgDLL := GetModuleHandle(msimg32);
    if msimgDLL = 0 then
    begin
      msImgDLLHandle := SafeLoadLibrary(msimg32);
      msImgDLL := msImgDLLHandle;
    end;
    if msImgDLL <> 0 then
       @mGradientFill := GetProcAddress(msImgDLL, 'GradientFill');
  end;
end;

function ByteToColor16(const AColor: Byte): Word;
begin
  Result := AColor shl 8;
end;

{ TabcGradient }

constructor TabcGradient.Create;
begin
  FColorCount := 236;
  ColorFrom := clBlue;
  ColorTo := clBlack;
  FDirection := ddVertical;
end;

destructor TabcGradient.Destroy;
begin
  FBuffer.Free;
  FBuffer := nil;
  inherited Destroy;
end;

procedure TabcGradient.Assign(Source: TPersistent);
begin
  if Source is TabcGradient then
    with TabcGradient(Source) do
    begin
      Self.FColorCount := ColorCount;
      Self.FColorFrom := ColorFrom;
      Self.FColorTo := ColorTo;
      Self.FDirection := Direction;
      FRefreshBuffer := True;
      if Assigned(FOnChange) then FOnChange(Self);
    end
  else
    inherited Assign(Source);
end;

procedure TabcGradient.Draw(const Canvas: TCanvas; const ARect: TRect);
begin
  DrawToDC(Canvas.Handle, ARect);
end;

procedure TabcGradient.DrawToDC(const DC: HDC; const ARect: TRect);
var
  realDrawMethod: TabcGradientDrawMethod;
  rWidth, rHeight: integer;
begin
  {check if winAPI supported}
  realDrawMethod := FDrawMethod;
  if (realDrawMethod = agdWinAPI) and msImgProcsLoaded and (@mGradientFill = nil) then
    realDrawMethod := agdDraw;
  if DrawBuffered then
  begin
    rWidth := (ARect.Right - ARect.Left);
    rHeight := (ARect.Bottom - ARect.Top);
    FRefreshBuffer := FRefreshBuffer or
      ((FDrawMethod <> agdStretch) and
       ((rWidth <> FBuffer.Width) or (rHeight <> FBuffer.Height)));
    if FRefreshBuffer then
    begin
      if DrawMethod <> agdStretch then
      begin
        FBuffer.Width := rWidth;
        FBuffer.Height := rHeight;
      end
      else
      begin
        FBuffer.Width := FColorCount;
        FBuffer.Height := FColorCount;
      end;
      InnerDrawToDC(FBuffer.Canvas.Handle, Rect(0, 0, FBuffer.Width, FBuffer.Height), realDrawMethod);
      FRefreshBuffer := False;
    end;
    if DrawMethod <> agdStretch then
      BitBlt(
        DC, ARect.Left, ARect.Top, rWidth, rHeight,
        FBuffer.Canvas.Handle, 0, 0,
        SRCCOPY)
    else
      StretchBlt(
        DC, ARect.Left, ARect.Top, rWidth, rHeight,
        FBuffer.Canvas.Handle, 0, 0, FBuffer.Width, FBuffer.Height,
        SRCCOPY);
  end
  else
    InnerDrawToDC(DC, ARect, realDrawMethod);
end;

procedure TabcGradient.InnerDrawToDC(const DC: HDC; const ARect: TRect; const aDrawMethod: TabcGradientDrawMethod);
begin
  if aDrawMethod <> agdWinAPI then
    Paint(DC, ARect)
  else
    WinAPIFill(DC, ARect);
end;

function TabcGradient.GetStepColor(Step: TabcColorCount): TColor;
var
  R,G,B: Byte;
begin
  R:= Round(FFromR + (FToR-FFromR)*(Step/FColorCount));
  G:= Round(FFromG + (FToG-FFromG)*(Step/FColorCount));
  B:= Round(FFromB + (FToB-FFromB)*(Step/FColorCount));
  Result:= RGB(R,G,B);
end;

procedure TabcGradient.Paint(const DC: HDC; const ARect: TRect);
var
  savedDC: Integer;
begin
  savedDC := SaveDC(DC);
  try
    SelectObject(DC, GetStockObject(NULL_PEN));
    case FDirection of
      ddVertical:
        PaintVertical(DC, ARect);
      ddHorizontal:
        PaintHorizontal(DC, ARect);
      ddLRDiagonal:
        PaintLRDiagonal(DC, ARect);
      ddRLDiagonal:
        PaintRLDiagonal(DC, ARect);
      ddCenter:
        PaintCenter(DC, ARect);
    end;
  finally
    RestoreDC(DC, savedDC);
  end;
end;

procedure TabcGradient.PaintVertical(const DC: HDC; const ARect: TRect);
var
  Count: Word;
  Rect1: TRect;
  AHeight: Integer;
  hBR: HBRUSH;
begin
  hBR := 0;
  with Rect1 do
  begin
    Left:= ARect.Left;
    Right:= ARect.Right;
    Bottom:= ARect.Top;
    AHeight := ARect.Bottom - ARect.Top;
    for Count := 1 to FColorCount do
    begin
      if hBR <> 0 then
        DeleteObject(hBR);
      hBR := CreateSolidBrush(COLORREF(GetStepColor(Count)));
      Top:= Bottom;
      Bottom:= ARect.Top + Round(AHeight * (Count / FColorCount));
      Windows.FillRect(DC, Rect1, hBR);
    end;
  end;
  DeleteObject(hBR);
end;

procedure TabcGradient.PaintHorizontal(const DC: HDC; const ARect: TRect);
var
  Count: Word;
  Rect1: TRect;
  AWidth: Integer;
  hBR: HBRUSH;
begin
  hBR := 0;
  with Rect1 do
  begin
    Top:= ARect.Top;
    Bottom:= ARect.Bottom;
    Right:= ARect.Left;
    AWidth := ARect.Right - ARect.Left;
    for Count := 1 to FColorCount do
    begin
      if hBR <> 0 then
        DeleteObject(hBR);
      hBR := CreateSolidBrush(COLORREF(GetStepColor(Count)));
      Left:= Right;
      Right:= ARect.Left + Round(AWidth * (Count / FColorCount));
      Windows.FillRect(DC, Rect1, hBR);
    end;
  end;
  DeleteObject(hBR);
end;

procedure TabcGradient.PaintLRDiagonal(const DC: HDC; const ARect: TRect);
var
  Count,Mid: Word;
  Points: array[0..3] of TPoint;
  AWidth, AHeight: Integer;
  hBR, saveBR: HBRUSH;
begin
  saveBR := SelectObject(DC, GetStockObject(NULL_BRUSH));
  Mid := Round(FColorCount/2);
  Points[0].Y := ARect.Top;
  Points[1].X := ARect.Left;
  Points[1].Y := ARect.Top;
  Points[2].X := ARect.Left;
  Points[2].Y := ARect.Top;
  Points[3].X := ARect.Left;
  AWidth := ARect.Right - ARect.Left;
  AHeight := ARect.Bottom - ARect.Top;
  for Count:= 1 to Mid do
  begin
    hBR := SelectObject(DC, CreateSolidBrush(COLORREF(GetStepColor(Count))));
    DeleteObject(hBR);
    Points[0].X := Points[1].X;
    Points[1].X := ARect.Left + Round(AWidth * (2*Count/FColorCount));
    Points[3].Y := Points[2].Y;
    Points[2].Y := ARect.Top + Round(AHeight * (2*Count/FColorCount));
    Windows.Polygon(DC, Points, 4);
  end;
  Points[0].X := ARect.Right;
  Points[1].X := ARect.Right;
  Points[1].Y := ARect.Top;
  Points[2].X := ARect.Left;
  Points[2].Y := ARect.Bottom;
  Points[3].Y := ARect.Bottom;
  for Count := Mid+1 to FColorCount do
  begin
    hBR := SelectObject(DC, CreateSolidBrush(COLORREF(GetStepColor(Count))));
    DeleteObject(hBR);
    Points[0].Y := Points[1].Y;
    Points[1].Y := ARect.Top + Round(AHeight * ((2*Count/FColorCount)-1));
    Points[3].X := Points[2].X;
    Points[2].X := ARect.Left + Round(AWidth * ((2*Count/FColorCount)-1));
    Windows.Polygon(DC, Points, 4);
  end;
  DeleteObject(SelectObject(DC, saveBR));
end;

procedure TabcGradient.PaintRLDiagonal(const DC: HDC; const ARect: TRect);
var
  Count,Mid: Word;
  Points: array[0..3] of TPoint;
  AWidth, AHeight: Integer;
  hBR, saveBR: HBRUSH;
begin
  saveBR := SelectObject(DC, GetStockObject(NULL_BRUSH));
  Mid:= Round(FColorCount/2);
  Points[0].Y:= ARect.Top;
  Points[1].X:= ARect.Right;
  Points[1].Y:= ARect.Top;
  Points[2].X:= ARect.Right;
  Points[2].Y:= ARect.Top;
  Points[3].X:= ARect.Right;
  AWidth := ARect.Right - ARect.Left;
  AHeight := ARect.Bottom - ARect.Top;
  for Count:= 1 to Mid do
  begin
    hBR := SelectObject(DC, CreateSolidBrush(COLORREF(GetStepColor(Count))));
    DeleteObject(hBR);
    Points[0].X:= Points[1].X;
    Points[1].X:= ARect.Right - Round(AWidth * (2*Count/FColorCount));
    Points[3].Y:= Points[2].Y;
    Points[2].Y:= ARect.Top + Round(AHeight * (2*Count/FColorCount));
    Windows.Polygon(DC, Points, 4);
  end;
  Points[0].X:= ARect.Left;
  Points[1].X:= ARect.Left;
  Points[1].Y:= ARect.Top;
  Points[2].X:= ARect.Right;
  Points[2].Y:= ARect.Bottom;
  Points[3].Y:= ARect.Bottom;
  for Count:= Mid+1 to FColorCount do
  begin
    hBR := SelectObject(DC, CreateSolidBrush(COLORREF(GetStepColor(Count))));
    DeleteObject(hBR);
    Points[0].Y:= Points[1].Y;
    Points[1].Y:= ARect.Left + Round(AHeight * ((2*Count/FColorCount)-1));
    Points[3].X:= Points[2].X;
    Points[2].X:= ARect.Right - Round(AWidth * ((2*Count/FColorCount)-1));
    Windows.Polygon(DC, Points, 4);
  end;
  DeleteObject(SelectObject(DC, saveBR));
end;

procedure TabcGradient.PaintCenter(const DC: HDC; const ARect: TRect);
var
  Count: Word;
  Rect1: TRect;
  AWidth, AHeight, dx, dy: Integer;
  hBR: HBRUSH;
begin
  with Rect1 do
  begin
    AWidth := ARect.Right - ARect.Left;
    AHeight := ARect.Bottom - ARect.Top;
    hBR := CreateSolidBrush(COLORREF(FColorFrom));
    Windows.FillRect(DC, ARect, hBR);
    for Count := 1 to FColorCount do
    begin
      if hBR <> 0 then
        DeleteObject(hBR);
      hBR := CreateSolidBrush(COLORREF(GetStepColor(Count)));
      dx := Round((AWidth/2) * (Count/FColorCount));
      Left:= ARect.Left + dx;
      Right:= ARect.Right - dx;
      dy := Round((AHeight/2) * (Count/FColorCount));
      Top:= ARect.Top + dy;
      Bottom:= ARect.Bottom - dy;
      Windows.FillRect(DC, Rect1, hBR);
    end;
  end;
  DeleteObject(hBR);
end;

procedure TabcGradient.WinAPIFill(const DC: HDC; const ARect: TRect);
begin
  case FDirection of
    ddVertical:
      WinAPIFillVertical(DC, ARect);
    ddHorizontal:
      WinAPIFillHorizontal(DC, ARect);
    ddLRDiagonal:
      WinAPIFillLRDiagonal(DC, ARect);
    ddRLDiagonal:
      WinAPIFillRLDiagonal(DC, ARect);
    ddCenter:
      WinAPIFillCenter(DC, ARect);
  end;
end;

procedure TabcGradient.WinAPIFillVertical(const DC: HDC; const ARect: TRect);
var
  vertices: packed array [0..1] of TTriVertex;
  GR: TGradientRect;
begin
  with vertices[0] do
  begin
    x := ARect.Left;
    y := ARect.Top;
    Red := ByteToColor16(FFromR);
    Green := ByteToColor16(FFromG);
    Blue := ByteToColor16(FFromB);
    Alpha := 0;
  end;
  with vertices[1] do
  begin
    x := ARect.Right;
    y := ARect.Bottom;
    Red := ByteToColor16(FToR);
    Green := ByteToColor16(FToG);
    Blue := ByteToColor16(FToB);
    Alpha := 0;
  end;
  GR.UpperLeft := 0;
  GR.LowerRight := 1;
  mGradientFill(
    DC,
    @vertices,
    2,
    @GR,
    1,
    GRADIENT_FILL_RECT_V);
end;

procedure TabcGradient.WinAPIFillHorizontal(const DC: HDC; const ARect: TRect);
var
  vertices: packed array [0..1] of TTriVertex;
  GR: TGradientRect;
begin
  with vertices[0] do
  begin
    x := ARect.Left;
    y := ARect.Top;
    Red := ByteToColor16(FFromR);
    Green := ByteToColor16(FFromG);
    Blue := ByteToColor16(FFromB);
    Alpha := 0;
  end;
  with vertices[1] do
  begin
    x := ARect.Right;
    y := ARect.Bottom;
    Red := ByteToColor16(FToR);
    Green := ByteToColor16(FToG);
    Blue := ByteToColor16(FToB);
    Alpha := 0;
  end;
  GR.UpperLeft := 0;
  GR.LowerRight := 1;
  mGradientFill(
    DC,
    @vertices,
    2,
    @GR,
    1,
    GRADIENT_FILL_RECT_H);
end;

procedure TabcGradient.WinAPIFillLRDiagonal(const DC: HDC; const ARect: TRect);
var
  vertices: packed array [0..3] of TTriVertex;
  triangles: packed array [0..1] of TGradientTriangle;
begin
  with vertices[0] do
  begin
    x := ARect.Left;
    y := ARect.Top;
    Red := ByteToColor16(FFromR);
    Green := ByteToColor16(FFromG);
    Blue := ByteToColor16(FFromB);
    Alpha := 0;
  end;
  with vertices[1] do
  begin
    x := ARect.Right;
    y := ARect.Top;
    Red := ByteToColor16(FMidR);
    Green := ByteToColor16(FMidG);
    Blue := ByteToColor16(FMidB);
    Alpha := 0;
  end;
  with vertices[2] do
  begin
    x := ARect.Left;
    y := ARect.Bottom;
    Red := ByteToColor16(FMidR);
    Green := ByteToColor16(FMidG);
    Blue := ByteToColor16(FMidB);
    Alpha := 0;
  end;
  with vertices[3] do
  begin
    x := ARect.Right;
    y := ARect.Bottom;
    Red := ByteToColor16(FToR);
    Green := ByteToColor16(FToG);
    Blue := ByteToColor16(FToB);
    Alpha := 0;
  end;
  with triangles[0] do
  begin
    Vertex1 := 0;
    Vertex2 := 1;
    Vertex3 := 2;
  end;
  with triangles[1] do
  begin
    Vertex1 := 1;
    Vertex2 := 2;
    Vertex3 := 3;
  end;
  mGradientFill(
    DC,
    @vertices,
    4,
    @triangles,
    2,
    GRADIENT_FILL_TRIANGLE);
end;

procedure TabcGradient.WinAPIFillRLDiagonal(const DC: HDC; const ARect: TRect);
var
  vertices: packed array [0..3] of TTriVertex;
  triangles: packed array [0..1] of TGradientTriangle;
begin
  with vertices[0] do
  begin
    x := ARect.Left;
    y := ARect.Top;
    Red := ByteToColor16(FMidR);
    Green := ByteToColor16(FMidG);
    Blue := ByteToColor16(FMidB);
    Alpha := 0;
  end;
  with vertices[1] do
  begin
    x := ARect.Right;
    y := ARect.Top;
    Red := ByteToColor16(FFromR);
    Green := ByteToColor16(FFromG);
    Blue := ByteToColor16(FFromB);
    Alpha := 0;
  end;
  with vertices[2] do
  begin
    x := ARect.Left;
    y := ARect.Bottom;
    Red := ByteToColor16(FToR);
    Green := ByteToColor16(FToG);
    Blue := ByteToColor16(FToB);
    Alpha := 0;
  end;
  with vertices[3] do
  begin
    x := ARect.Right;
    y := ARect.Bottom;
    Red := ByteToColor16(FMidR);
    Green := ByteToColor16(FMidG);
    Blue := ByteToColor16(FMidB);
    Alpha := 0;
  end;
  with triangles[0] do
  begin
    Vertex1 := 0;
    Vertex2 := 1;
    Vertex3 := 3;
  end;
  with triangles[1] do
  begin
    Vertex1 := 0;
    Vertex2 := 2;
    Vertex3 := 3;
  end;
  mGradientFill(
    DC,
    @vertices,
    4,
    @triangles,
    2,
    GRADIENT_FILL_TRIANGLE);
end;

procedure TabcGradient.WinAPIFillCenter(const DC: HDC; const ARect: TRect);
var
  vertices: packed array [0..4] of TTriVertex;
  triangles: packed array [0..3] of TGradientTriangle;
begin
  with vertices[0] do
  begin
    x := ARect.Left;
    y := ARect.Top;
    Red := ByteToColor16(FFromR);
    Green := ByteToColor16(FFromG);
    Blue := ByteToColor16(FFromB);
    Alpha := 0;
  end;
  vertices[1] := vertices[0];
  vertices[2] := vertices[0];
  vertices[3] := vertices[0];
  with vertices[1] do
  begin
    x := ARect.Right;
    y := ARect.Top;
  end;
  with vertices[2] do
  begin
    x := ARect.Left;
    y := ARect.Bottom;
  end;
  with vertices[3] do
  begin
    x := ARect.Right;
    y := ARect.Bottom;
  end;
  with vertices[4] do
  begin
    x := (ARect.Left + ARect.Right) div 2;
    y := (ARect.Top + ARect.Bottom) div 2;
    Red := ByteToColor16(FToR);
    Green := ByteToColor16(FToG);
    Blue := ByteToColor16(FToB);
    Alpha := 0;
  end;
  with triangles[0] do
  begin
    Vertex1 := 0;
    Vertex2 := 1;
    Vertex3 := 4;
  end;
  with triangles[1] do
  begin
    Vertex1 := 0;
    Vertex2 := 4;
    Vertex3 := 2;
  end;
  with triangles[2] do
  begin
    Vertex1 := 2;
    Vertex2 := 4;
    Vertex3 := 3;
  end;
  with triangles[3] do
  begin
    Vertex1 := 4;
    Vertex2 := 1;
    Vertex3 := 3;
  end;
  mGradientFill(
    DC,
    @vertices,
    5,
    @triangles,
    4,
    GRADIENT_FILL_TRIANGLE);
end;

procedure TabcGradient.SetColorCount(const Value: TabcColorCount);
begin
  if Value <> FColorCount then
  begin
    FColorCount := Value;
    FRefreshBuffer := True;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TabcGradient.SetColorFrom(const Value: TColor);
begin
  if Value <> FColorFrom then
  begin
    FColorFrom := Value;
    FFromR:= GetRValue(DWORD(FColorFrom));
    FFromG:= GetGValue(DWORD(FColorFrom));
    FFromB:= GetBValue(DWORD(FColorFrom));
    FMidR := (FFromR + FToR) div 2;
    FMidG := (FFromG + FToG) div 2;
    FMidB := (FFromB + FToB) div 2;
    FRefreshBuffer := True;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TabcGradient.SetColorTo(const Value: TColor);
begin
  if Value <> FColorTo then
  begin
    FColorTo := Value;
    FToR:= GetRValue(DWORD(FColorTo));
    FToG:= GetGValue(DWORD(FColorTo));
    FToB:= GetBValue(DWORD(FColorTo));
    FMidR := (FFromR + FToR) div 2;
    FMidG := (FFromG + FToG) div 2;
    FMidB := (FFromB + FToB) div 2;
    FRefreshBuffer := True;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TabcGradient.SetDirection(const Value: TabcDrawDirection);
begin
  if Value <> FDirection then
  begin
    FDirection := Value;
    FRefreshBuffer := True;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TabcGradient.SetDrawBuffered(const Value: Boolean);
begin
  if Value <> FDrawBuffered then
  begin
    FDrawBuffered := Value;
    if not Value and (DrawMethod = agdStretch) then
      DrawMethod := agdDraw;
    if Value and (FBuffer = nil) then
    begin
      FBuffer := TBitmap.Create;
      FBuffer.Transparent := False;
      FRefreshBuffer := True;
    end
    else if (FBuffer <> nil) then
    begin
      FBuffer.Free;
      FBuffer := nil;
    end;
  end;
end;

procedure TabcGradient.SetDrawMethod(const Value: TabcGradientDrawMethod);
begin
  if Value <> FDrawMethod then
  begin
    FDrawMethod := Value;
    if (Value = agdStretch) and not DrawBuffered then
      DrawBuffered := True;
    if (Value = agdWinAPI) and not msImgProcsLoaded then
      LoadMSImgProcs;
    FRefreshBuffer := True;
  end;
end;

{ TabcTileBack }

constructor TabcTileBack.Create;
begin
  FPicture := TPicture.Create;
  FGradient := TabcGradient.Create;
end;

destructor TabcTileBack.Destroy;
begin
  FPicture.Free;
  FGradient.Free;
  inherited Destroy;
end;

procedure TabcTileBack.Assign(Source: TPersistent);
begin
  if Source is TabcTileBack then
    with TabcTileBack(Source) do
    begin
      Self.FStyle := Style;
      Self.Gradient := Gradient;
      Self.Picture := Picture;
    end
  else
    inherited Assign(Source);
end;

function TabcTileBack.Empty: Boolean;
begin
  Result :=
    (Style = tbsNone) or
    ((Style in
      [tbsPicClip, tbsPicFit, tbsPicHeight, tbsPicStretch, tbsPicTile, tbsPicWidth])
      and ((Picture = nil) or (Picture.Graphic = nil) or (Picture.Graphic.Empty)));
end;

procedure TabcTileBack.Draw(const Canvas: TCanvas; const ARect: TRect);
begin
  case Style of
    tbsNone:
      Exit;
    tbsGradient:
      Gradient.Draw(Canvas, ARect);
  else
    DrawPicture(Canvas, ARect);
  end;
end;

procedure TabcTileBack.DrawPicture(const Canvas: TCanvas; const ARect: TRect);
var
  Dest: TRect;
  XPos, YPos, RWidth, RHeight: Integer;
  PicRatio, ImageRatio: Double;
begin
  if (Picture = nil) or (Picture.Graphic = nil) or (Picture.Graphic.Empty) then
    Exit;
  RWidth := ARect.Right - ARect.Left;
  RHeight := ARect.Bottom - ARect.Top;
  case Style of
    tbsPicClip:
      Canvas.Draw(ARect.Left, ARect.Top, Picture.Graphic);
    tbsPicCenter:
      begin
        XPos:= ARect.Left + (RWidth - Picture.Width) div 2;
        YPos:= ARect.Top + (RHeight - Picture.Height) div 2;
        Canvas.Draw(Xpos,YPos, Picture.Graphic);
      end;
    tbsPicFit:
      if (Picture.Width > 0) and (Picture.Height > 0) then
      begin
        PicRatio:= Picture.Height / Picture.Width;
        ImageRatio:= RHeight/RWidth;
        if PicRatio > ImageRatio then
          begin
            XPos:= Trunc(RHeight / PicRatio);
            YPos:= RHeight;
          end
        else
          begin
            XPos:= RWidth;
            YPos:= Trunc(RWidth * PicRatio);
          end;
        Dest := Rect(0,0,XPos,YPos);
        OffsetRect(Dest, ARect.Left, ARect.Top);
        Canvas.StretchDraw(Dest, Picture.Graphic);
      end;
    tbsPicHeight:
      begin
        XPos:= Trunc(Picture.Width * (RHeight / Picture.Height));
        YPos:= RHeight;
        Dest := Rect(0,0,XPos,YPos);
        OffsetRect(Dest, ARect.Left, ARect.Top);
        Canvas.StretchDraw(Dest, Picture.Graphic);
      end;
    tbsPicStretch:
      Canvas.StretchDraw(ARect, Picture.Graphic);
    tbsPicTile:
      begin
        XPos:= ARect.Left;
        While XPos < ARect.Right do
        begin
          YPos:= ARect.Top;
          While YPos < ARect.Bottom do
          begin
            Canvas.Draw(XPos, YPos, Picture.Graphic);
            YPos:= YPos + Picture.Height;
          end;
          XPos:= XPos + Picture.Width;
        end;
      end;
    tbsPicWidth:
      begin
        XPos:= RWidth;
        YPos:= Trunc(RWidth * (Picture.Height / Picture.Width));
        Dest := Rect(0,0,XPos,YPos);
        OffsetRect(Dest, ARect.Left, ARect.Top);
        Canvas.StretchDraw(Dest, Picture.Graphic);
      end;
  else
  end;
end;

procedure TabcTileBack.SetGradient(const Value: TabcGradient);
begin
  FGradient.Assign(Value);
end;

procedure TabcTileBack.SetOnChange(const Value: TNotifyEvent);
begin
  FOnChange := Value;
  FGradient.OnChange := Value;
  FPicture.OnChange := Value;
end;

procedure TabcTileBack.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TabcTileBack.SetStyle(const Value: TabcTileBackStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

initialization

finalization
  if msImgDLLHandle <> 0 then
    FreeLibrary(msImgDLLHandle);

end.
