unit abctimag;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, ExtCtrls, Menus;

type

  TabcPicDrawStyle = (dsAlign, dsCenter, dsFit, dsHeight, dsStretch, dsTile, dsWidth);

  TabcPicHAlign = (bhLeft, bhCenter, bhRight);
  TabcPicVAlign = (bvTop, bvCenter, bvBottom);

  TabcPicAlign = class(TPersistent)
  private
    FOnChange: TNotifyEvent;
    FHorz: TabcPicHAlign;
    FHOffset: integer;
    FVert: TabcPicVAlign;
    FVOffset: integer;
    procedure SetHorz(Value: TabcPicHAlign);
    procedure SetHOffset(Value: integer);
    procedure SetVert(Value: TabcPicVAlign);
    procedure SetVOffset(Value: integer);
  public
    constructor Create;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Horz: TabcPicHAlign read FHorz write SetHorz;
    property HOffset: integer read FHOffset write SetHOffset;
    property Vert: TabcPicVAlign read FVert write SetVert;
    property VOffset: integer read FVOffset write SetVOffset;
  end;

  { TabcMaskedBitmap }

  TabcMaskedBitmap = class
  private
    FBitmap: TBitmap;
    FMask: TBitmap;
    FMasked: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    property Bitmap: TBitmap read FBitmap;
    property Mask: TBitmap read FMask;
    procedure SetBitmap(Image: TBitmap; MaskColor: TColor);
    procedure SetBitmapMask(Image, Mask: TBitmap);
    procedure Draw(Canvas: TCanvas; X, Y: Integer);
    procedure StretchDraw(Canvas: TCanvas; ARect: TRect);
    property Masked: Boolean read FMasked;
  end;

  TabcTransImage = class(TGraphicControl)
  private
    FBevelInner: TPanelBevel;
    FBevelOuter: TPanelBevel;
    FBevelWidth: TBevelWidth;
    FBorderWidth: TBorderWidth;
    FBorderStyle: TBorderStyle;

    FPicture: TPicture;
    FPicAlign: TabcPicAlign;
    FPicDrawStyle: TabcPicDrawStyle;
    FTransparent: Boolean;
    FTransparentColor: TColor;
    FMaskedBitmap: TabcMaskedBitmap;
    FIconBitmap: TBitmap;
    procedure SetBevelInner(Value: TPanelBevel);
    procedure SetBevelOuter(Value: TPanelBevel);
    procedure SetBevelWidth(Value: TBevelWidth);
    procedure SetBorderWidth(Value: TBorderWidth);
    procedure SetBorderStyle(Value: TBorderStyle);

    procedure SetPicAlign(Value: TabcPicAlign);
    procedure SetPicDrawStyle(Value: TabcPicDrawStyle);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparentColor(Value: TColor);
    procedure PictureChanged(Sender: TObject);
    procedure SetPicture(Value: TPicture);
    procedure UpdateMask;
    function GetCanvas: TCanvas;
    procedure InvalidateParentRect;
  protected
    procedure AdjustSize; override;
    function GetPalette: HPALETTE; override;
    procedure Paint; override;
    procedure PaintTransparentImage(ARect: TRect); dynamic;
    procedure PaintNormalImage(ARect: TRect); dynamic;
    procedure PicAlignChange(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas: TCanvas read GetCanvas;
  published
    property Picture: TPicture read FPicture write SetPicture;
    property PicAlign: TabcPicAlign read FPicAlign write SetPicAlign;
    property PicDrawStyle: TabcPicDrawStyle read FPicDrawStyle write SetPicDrawStyle;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property TransparentColor: TColor read FTransparentColor write SetTransparentColor;
    property BevelInner: TPanelBevel read FBevelInner write SetBevelInner default bvNone;
    property BevelOuter: TPanelBevel read FBevelOuter write SetBevelOuter default bvNone;
    property BevelWidth: TBevelWidth read FBevelWidth write SetBevelWidth default 1;
    property BorderWidth: TBorderWidth read FBorderWidth write SetBorderWidth default 0;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsNone;
    property Align;
    property AutoSize;
    property Anchors;
    property Constraints;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

uses Consts;

{ TabcMaskedBitmap }

constructor TabcMaskedBitmap.Create;
begin
  FBitmap := TBitmap.Create;
  FMask := TBitmap.Create;
  FMasked := False;
end;

procedure TabcMaskedBitmap.SetBitmap(Image: TBitmap; MaskColor: TColor);
var
  NMask: TBitmap;
  NImage: TBitmap;
begin
  FBitmap.Assign(Image);
  NMask := TBitmap.Create;
  try
    NMask.Assign(Image);
    NMask.Canvas.Brush.Color := MaskColor;
    NMask.Monochrome := True;
    NImage := TBitmap.Create;
    try
      NImage.Assign(Image);
      with NImage.Canvas do
      begin
        CopyMode := cmSrcAnd;
        Brush.Color := clBlack;
        Font.Color := clWhite;
        Draw(0, 0, NMask);
      end;
      FBitmap.Assign(NImage);
      FMask.Assign(NMask);
      FMasked := True;
    finally
      NImage.Free;
    end;
  finally
    NMask.Free;
  end;
end;

procedure TabcMaskedBitmap.SetBitmapMask(Image, Mask: TBitmap);
begin
  FBitmap.Assign(Image);
  FMasked := Assigned(Mask);
  if FMasked then begin
    FMask.Monochrome := True;
    FMask.Height := FBitmap.Height;
    FMask.Width := FBitmap.Width;
    FMask.Canvas.Draw(0, 0, Mask);
  end;
end;

destructor TabcMaskedBitmap.Destroy;
begin
  FBitmap.Free;
  FMask.Free;
  inherited Destroy;
end;

procedure TabcMaskedBitmap.Draw(Canvas: TCanvas; X, Y: Integer);
var
  ImageRect, CanvasRect: TRect;
  OldMode: TCopyMode;
  OldTextColor, OldBkColor: Longint;
begin
  CanvasRect := Rect(X, Y, X + FBitmap.Width, Y + FBitmap.Height);
  ImageRect := Rect(0, 0, FBitmap.Width, FBitmap.Height);
  with Canvas do
    if not Masked then begin
      OldMode := CopyMode;
      CopyRect(CanvasRect, FBitmap.Canvas, ImageRect);
      CopyMode := OldMode;
    end
    else begin
      OldMode := CopyMode;
      CopyMode := cmSrcAnd;
      OldTextColor := SetTextColor(Handle, clBlack);
      OldBkColor := SetBkColor(Handle, clWhite);
      CopyRect(CanvasRect, FMask.Canvas, ImageRect);
      CopyMode := cmSrcInvert;
      CopyRect(CanvasRect, FBitmap.Canvas, ImageRect);
      CopyMode := OldMode;
      SetTextColor(Handle, OldTextColor);
      SetBkColor(Handle, OldBkColor);
    end;
end;

procedure TabcMaskedBitmap.StretchDraw(Canvas: TCanvas; ARect: TRect);
var
  ImageRect: TRect;
  OldMode: TCopyMode;
  OldTextColor, OldBkColor: Longint;
begin
  ImageRect := Rect(0, 0, FBitmap.Width, FBitmap.Height);
  with Canvas do
    if not Masked then begin
      OldMode := CopyMode;
      CopyRect(ARect, FBitmap.Canvas, ImageRect);
      CopyMode := OldMode;
    end
    else begin
      OldMode := CopyMode;
      CopyMode := cmSrcAnd;
      OldTextColor := SetTextColor(Handle, clBlack);
      OldBkColor := SetBkColor(Handle, clWhite);
      CopyRect(ARect, FMask.Canvas, ImageRect);
      CopyMode := cmSrcInvert;
      CopyRect(ARect, FBitmap.Canvas, ImageRect);
      CopyMode := OldMode;
      SetTextColor(Handle, OldTextColor);
      SetBkColor(Handle, OldBkColor);
    end;
end;

{ TabcPicAlign }

constructor TabcPicAlign.Create;
begin
  FHorz := bhCenter;
  FVert := bvCenter;
end;

procedure TabcPicAlign.SetHorz(Value: TabcPicHAlign);
begin
  if Value <> FHorz then begin
    FHorz := Value;
    if assigned(FOnChange) then FOnChange(self);
  end;
end;

procedure TabcPicAlign.SetHOffset(Value: integer);
begin
  if Value <> FHOffset then begin
    FHOffset := Value;
    if assigned(FOnChange) then FOnChange(self);
  end;
end;

procedure TabcPicAlign.SetVert(Value: TabcPicVAlign);
begin
  if Value <> FVert then begin
    FVert := Value;
    if assigned(FOnChange) then FOnChange(self);
  end;
end;

procedure TabcPicAlign.SetVOffset(Value: integer);
begin
  if Value <> FVOffset then begin
    FVOffset := Value;
    if assigned(FOnChange) then FOnChange(self);
  end;
end;

{ TabcTransImage }

constructor TabcTransImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FPicAlign := TabcPicAlign.Create;
  FPicAlign.OnChange := PicAlignChange;
  FMaskedBitmap := TabcMaskedBitmap.Create;
  FIconBitmap := TBitmap.Create;
  ControlStyle := ControlStyle +
    [csCaptureMouse, csClickEvents, csDoubleClicks] - [csSetCaption, csOpaque] ;
  Height := 105;
  Width := 105;
  BevelWidth := 1;
  FBorderStyle := bsNone;
end;

destructor TabcTransImage.Destroy;
begin
  Destroying;
  FPicture.Free;
  FPicAlign.Free;
  FMaskedBitmap.Free;
  FIconBitmap.Free;
  inherited Destroy;
end;

procedure TabcTransImage.UpdateMask;
var
  TempBitmap: TBitmap;
begin
  if (csDestroying in ComponentState) then Exit;
  if FPicture.Graphic = nil then
    FMaskedBitmap.SetBitmap(nil, FTransparentColor)
  else
    if FPicture.Graphic is TBitmap then
      FMaskedBitmap.SetBitmap(FPicture.Bitmap, FTransparentColor)
    else begin
      TempBitmap := TBitmap.Create;
      try
        with TempBitmap do begin
          Width := FPicture.Width;
          Height := FPicture.Height;
          Canvas.Draw(0,0,FPicture.Graphic);
          FMaskedBitmap.SetBitmap(TempBitmap, FTransparentColor);
        end;
      finally
        TempBitmap.Free;
      end;
    end;
  Invalidate;
end;

function TabcTransImage.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture.Graphic is TBitmap then
    Result := TBitmap(FPicture.Graphic).Palette;
end;

function TabcTransImage.GetCanvas: TCanvas;
var
  Bitmap: TBitmap;
begin
  if Picture.Graphic = nil then
  begin
    Bitmap := TBitmap.Create;
    try
      Bitmap.Width := Width;
      Bitmap.Height := Height;
      Picture.Graphic := Bitmap;
    finally
      Bitmap.Free;
    end;
  end;
  if Picture.Graphic is TBitmap then
    Result := TBitmap(Picture.Graphic).Canvas
  else
    raise EInvalidOperation.Create(SImageCanvasNeedsBitmap);
end;

procedure TabcTransImage.AdjustSize;
begin
  if AutoSize then
  begin
    if (Picture.Width > 0) and (Picture.Height > 0) then
      SetBounds(Left, Top, Picture.Width, Picture.Height);
    PicDrawStyle := dsCenter;
  end
  else
    inherited;
end;

procedure TabcTransImage.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
  FPicture.OnChange := PictureChanged;
end;

procedure TabcTransImage.PictureChanged(Sender: TObject);
begin
  if (csDestroying in ComponentState) then Exit;
  if (FPicture.Graphic is TIcon) then begin
    FIconBitmap.Width := FPicture.Width;
    FIconBitmap.Height := FPicture.Height;
    FIconBitmap.Canvas.Draw(0,0,FPicture.Graphic);
  end;
  if AutoSize and (Picture.Width > 0) and (Picture.Height > 0) then
    SetBounds(Left, Top, Picture.Width, Picture.Height);
  UpdateMask;
end;

procedure TabcTransImage.SetBevelInner(Value: TPanelBevel);
begin
  if FBevelInner <> Value then begin
    FBevelInner := Value;
    Invalidate;
  end;
end;

procedure TabcTransImage.SetBevelOuter(Value: TPanelBevel);
begin
  if FBevelOuter <> Value then begin
    FBevelOuter := Value;
    Invalidate;
  end;
end;

procedure TabcTransImage.SetBevelWidth(Value: TBevelWidth);
begin
  if FBevelWidth <> Value then begin
    FBevelWidth := Value;
    Invalidate;
  end;
end;

procedure TabcTransImage.SetBorderWidth(Value: TBorderWidth);
begin
  if FBorderWidth <> Value then begin
    FBorderWidth := Value;
    Invalidate;
  end;
end;

procedure TabcTransImage.SetBorderStyle(Value: TBorderStyle);
begin
  if FBorderStyle <> Value then
  begin
    FBorderStyle := Value;
    Invalidate;
  end;
end;

procedure TabcTransImage.SetPicAlign(Value: TabcPicAlign);
begin
  if Value <> FPicAlign then begin
    FPicAlign := Value;
    FPicAlign.OnChange := PicAlignChange;
  end;
end;

procedure TabcTransImage.SetPicDrawStyle(Value: TabcPicDrawStyle);
begin
  if Value <> FPicDrawStyle then begin
    FPicDrawStyle := Value;
    Invalidate;
  end;
end;

procedure TabcTransImage.SetTransparent(Value: Boolean);
begin
  if Value <> FTransparent then begin
    FTransparent := Value;
    if FTransparent then InvalidateParentRect;
    Invalidate;
  end;
end;

procedure TabcTransImage.InvalidateParentRect;
var
  ARect: TRect;
begin
  if (csDestroying in ComponentState) then Exit;
  if Parent <> nil then begin
    ARect := ClientRect;
    ARect.TopLeft := Parent.ScreentoClient(ClienttoScreen(ARect.TopLeft));
    ARect.BottomRight := Parent.ScreentoClient(ClienttoScreen(ARect.BottomRight));
    InvalidateRect(Parent.Handle, @ARect, True);
  end;
end;

procedure TabcTransImage.SetTransparentColor(Value: TColor);
begin
  if Value <> FTransparentColor then begin
    FTransparentColor := Value;
    UpdateMask;
  end;
end;

procedure TabcTransImage.PicAlignChange(Sender: TObject);
begin
  if (csDestroying in ComponentState) then Exit;
  SetPicDrawStyle(dsAlign);
  Invalidate;
end;

procedure TabcTransImage.Paint;
var
  ARect: TRect;
  TopColor, BottomColor: TColor;
  BorderSize: Integer;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;

begin
  ARect := ClientRect;
  {calc size of borders}
  BorderSize := 0;
  if (BorderStyle = bsSingle) then
    Inc(BorderSize);
  if BevelOuter <> bvNone then
    Inc(BorderSize, BevelWidth);
  Inc(BorderSize, BorderWidth);
  if BevelInner <> bvNone then
    Inc(BorderSize, BevelWidth);
  with ARect do begin
    Inc(Left, BorderSize);
    Dec(Right, BorderSize);
    Inc(Top, BorderSize);
    Dec(Bottom, BorderSize);
  if (Left > Right) or (Top > Bottom) then
    ARect := ClientRect;
  end;

  with inherited Canvas do begin
    Brush.Color := Color;
    Brush.Style := bsSolid;
    if not FTransparent then FillRect(ARect);
    if FPicture.Graphic <> nil then begin
      if FTransparent then
        PaintTransparentImage(ARect)
      else
        PaintNormalImage(ARect);
    end;

    ARect := ClientRect;

    Pen.Style := psSolid;
    Pen.Color := clBlack;
    Pen.Width := 1;
    Brush.Style := bsClear;
    if BorderStyle = bsNone then begin
      if csDesigning in ComponentState then begin
        Pen.Style := psDash;
        Rectangle(0, 0, Width, Height);
        Pen.Style := psSolid;
      end;
    end
    else begin
      Rectangle(0, 0, Width, Height);
      Inc(ARect.Left);
      Dec(ARect.Right);
      Inc(ARect.Top);
      Dec(ARect.Bottom);
    end;

  end;

  {now draw borders}
  if BevelOuter <> bvNone then
  begin
    AdjustColors(BevelOuter);
    Frame3D(inherited Canvas, ARect, TopColor, BottomColor, BevelWidth);
  end;
  Frame3D(inherited Canvas, ARect, Color, Color, BorderWidth);
  if BevelInner <> bvNone then
  begin
    AdjustColors(BevelInner);
    Frame3D(inherited Canvas, ARect, TopColor, BottomColor, BevelWidth);
  end;

end;

procedure TabcTransImage.PaintNormalImage(ARect: TRect);
{
paints the image inside the border dimensions passed in Rect
using the BmpDrawStyle and BmpAlign properties
}
var
  Dest: TRect;
  RWidth, RHeight, XPos, YPos: Integer;
  PicRatio, DestRatio: Double;
begin

  RWidth := ARect.Right - ARect.Left;
  RHeight := ARect.Bottom - ARect.Top;

  XPos := 0;
  YPos := 0;

  case FPicDrawStyle of
    dsAlign:
      begin
        case FPicAlign.Horz of
          bhLeft:
            XPos := ARect.Left;
          bhCenter:
            XPos := (ARect.Left + ARect.Right - FPicture.Width) div 2;
          bhRight:
            XPos := ARect.Right - FPicture.Width;
        end;
        case FPicAlign.Vert of
          bvTop:
            YPos := ARect.Top;
          bvCenter:
            YPos := (ARect.Top + ARect.Bottom - FPicture.Height) div 2;
          bvBottom:
            YPos := ARect.Bottom - FPicture.Height;
        end;
        XPos := XPos + FPicAlign.HOffset;
        YPos := YPos + FPicAlign.VOffset;
        Dest := Rect(XPos, YPos, XPos + FPicture.Width, YPos + FPicture.Height);
        with inherited Canvas do
          StretchDraw(Dest, FPicture.Graphic);
      end;

    dsCenter:
      begin
        XPos := (ARect.Left + ARect.Right - FPicture.Width) div 2;
        YPos := (ARect.Top + ARect.Bottom - FPicture.Height) div 2;
        with inherited Canvas do
          Draw(XPos, YPos, FPicture.Graphic);
      end;
    dsFit:
      if (FPicture.Width > 0) and (FPicture.Height > 0) then begin
        PicRatio:= FPicture.Height / FPicture.Width;
        DestRatio:= RHeight/RWidth;
        if PicRatio > DestRatio then begin
          XPos:= Trunc(RHeight / PicRatio);
          YPos:= RHeight;
        end
        else begin
          XPos:= RWidth;
          YPos:= Trunc(RWidth * PicRatio);
        end;
        Dest := ARect;
        Dest.Right := Dest.Left + XPos;
        Dest.Bottom := Dest.Top + YPos;
        with inherited Canvas do
          if (Picture.Graphic is TIcon) then
            StretchDraw(Dest, FIconBitmap)
          else
            StretchDraw(Dest, FPicture.Graphic);
      end;
    dsHeight:
      begin
        Dest := ARect;
        Dest.Right := Dest.Left + Trunc(FPicture.Width * (RHeight / FPicture.Height));
        with inherited Canvas do
          if (Picture.Graphic is TIcon) then
            StretchDraw(Dest, FIconBitmap)
          else
            StretchDraw(Dest, FPicture.Graphic);
     end;
    dsStretch:
        with inherited Canvas do
          if (Picture.Graphic is TIcon) then
            StretchDraw(ARect, FIconBitmap)
          else
            StretchDraw(ARect, FPicture.Graphic);
    dsTile:
      begin
        XPos:= ARect.Left;
        while XPos < RWidth do begin
          YPos:= ARect.Top;
          while YPos < RHeight do begin
            with inherited Canvas do
              Draw(XPos, YPos, FPicture.Graphic);
            YPos:= YPos + FPicture.Height;
          end;
          XPos:= XPos + FPicture.Width;
        end;
      end;
    dsWidth:
      begin
        Dest := ARect;
        Dest.Bottom := Dest.Top + Trunc(FPicture.Height * (RWidth / FPicture.Width));
        with inherited Canvas do
          if (Picture.Graphic is TIcon) then
            StretchDraw(Dest, FIconBitmap)
          else
            StretchDraw(Dest, FPicture.Graphic);
      end;
  end;

end;

procedure TabcTransImage.PaintTransparentImage(ARect: TRect);
{
paints the image inside the border dimensions passed in Rect
using the BmpDrawStyle, BmpAlign and Transparent properties
}
var
  Dest: TRect;
  RWidth, RHeight, XPos, YPos: Integer;
  PicRatio, DestRatio: Double;
begin

  RWidth := ARect.Right - ARect.Left;
  RHeight := ARect.Bottom - ARect.Top;

  XPos := 0;
  YPos := 0;

  case FPicDrawStyle of
    dsAlign:
      begin
        case FPicAlign.Horz of
          bhLeft:
            XPos := ARect.Left;
          bhCenter:
            XPos := (ARect.Left + ARect.Right - FPicture.Width) div 2;
          bhRight:
            XPos := ARect.Right - FPicture.Width;
        end;
        case FPicAlign.Vert of
          bvTop:
            YPos := ARect.Top;
          bvCenter:
            YPos := (ARect.Top + ARect.Bottom - FPicture.Height) div 2;
          bvBottom:
            YPos := ARect.Bottom - FPicture.Height;
        end;
        XPos := XPos + FPicAlign.HOffset;
        YPos := YPos + FPicAlign.VOffset;
        FMaskedBitmap.Draw(inherited Canvas, XPos, YPos);
      end;
    dsCenter:
      begin
        XPos := (ARect.Left + ARect.Right - FPicture.Width) div 2;
        YPos := (ARect.Top + ARect.Bottom - FPicture.Height) div 2;
        FMaskedBitmap.Draw(inherited Canvas, XPos, YPos);
      end;
    dsFit:
      if (FPicture.Width > 0) and (FPicture.Height > 0) then begin
        PicRatio:= FPicture.Height / FPicture.Width;
        DestRatio:= RHeight/RWidth;
        if PicRatio > DestRatio then begin
          XPos:= Trunc(RHeight / PicRatio);
          YPos:= RHeight;
        end
        else begin
          XPos:= RWidth;
          YPos:= Trunc(RWidth * PicRatio);
        end;
        Dest := ARect;
        Dest.Right := Dest.Left + XPos;
        Dest.Bottom := Dest.Top + YPos;
        FMaskedBitmap.StretchDraw(inherited Canvas, Dest);
      end;
    dsHeight:
      begin
        Dest := ARect;
        Dest.Right := Dest.Left + Trunc(FPicture.Width * (RHeight / FPicture.Height));
        FMaskedBitmap.StretchDraw(inherited Canvas, Dest);
     end;
    dsStretch:
        FMaskedBitmap.StretchDraw(inherited Canvas, ARect);
    dsTile:
      begin
        XPos:= ARect.Left;
        while XPos < RWidth do begin
          YPos:= ARect.Top;
          while YPos < RHeight do begin
            FMaskedBitmap.Draw(inherited Canvas, XPos, YPos);
            YPos:= YPos + FPicture.Height;
          end;
          XPos:= XPos + FPicture.Width;
        end;
      end;
    dsWidth:
      begin
        Dest := ARect;
        Dest.Bottom := Dest.Top + Trunc(FPicture.Height * (RWidth / FPicture.Width));
        FMaskedBitmap.StretchDraw(inherited Canvas, Dest);
      end;
  end;
end;

end.

