unit abctips;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  IniFiles, abcfact;

type
  TabcTipButton = (atbNextTip, atbHelpContents, atbHelpWhatsNew, atbClose);
  TabcTipButtons = set of TabcTipButton;

  TabcCustomWelcomeTips = class(TComponent)
  private
    { Private declarations }
    FAutoShow: Boolean;
    FCaption: string;
    FCaptionGreeting: TStrings;
    FCaptionProgress: string;
    FCaptionShowNext: string;
    FCaptionTip: string;
    FForm: TForm;
    FHelpContents: THelpContext;
    FHelpWhatsNew: THelpContext;
    FProgressMax: Integer;
    FProgressStep: Integer;
    FProgressPos: Integer;
    FProgressMin: Integer;
    FShowProgress: Boolean;
    FShowModal: Boolean;
    FStayOnTop: Boolean;
    FVisibleButtons: TabcTipButtons;
    procedure SetCaptionGreeting(Value: TStrings);
    procedure SetCaptionProgress(Value: string);
    procedure SetProgressMax(const Value: Integer);
    procedure SetProgressMin(const Value: Integer);
    procedure SetProgressPos(const Value: Integer);
    procedure SetProgressStep(const Value: Integer);
    procedure SetShowProgress(const Value: Boolean);
    function GetForm: TForm;
  protected
    { Protected declarations }
    procedure Loaded; override;

    function GetShowOnStartup: Boolean; virtual; abstract;
    procedure SetShowOnStartup(Value: Boolean); virtual; abstract;

    property AutoShow: Boolean read FAutoShow write FAutoShow default True;
    property Caption: string read FCaption write FCaption;
    property CaptionGreeting: TStrings read FCaptionGreeting write SetCaptionGreeting;
    property CaptionProgress: string read FCaptionProgress write SetCaptionProgress;
    property CaptionShowNext: string read FCaptionShowNext write FCaptionShowNext;
    property CaptionTip: string read FCaptionTip write FCaptionTip;
    property HelpContents: THelpContext read FHelpContents write FHelpContents;
    property HelpWhatsNew: THelpContext read FHelpWhatsNew write FHelpWhatsNew;
    property ProgressMin: Integer read FProgressMin write SetProgressMin default 0;
    property ProgressMax: Integer read FProgressMax write SetProgressMax default 100;
    property ProgressPos: Integer read FProgressPos write SetProgressPos default 0;
    property ProgressStep: Integer read FProgressStep write SetProgressStep default 10;
    property ShowProgress: Boolean read FShowProgress write SetShowProgress default False;
    property ShowModal: Boolean read FShowModal write FShowModal default False;
    property StayOnTop: Boolean read FStayOnTop write FStayOnTop default True;
    property VisibleButtons: TabcTipButtons read FVisibleButtons write FVisibleButtons default [atbNextTip, atbClose];
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute; virtual;
    procedure AfterExecute; virtual;
    procedure ExecuteIfSelected; virtual;
    function GetNextTip: string; virtual; abstract;
    procedure StepBy(Delta: Integer);
    procedure StepIt;
    procedure StepByMsg(Delta: Integer; Msg: string);
    procedure StepItMsg(Msg: string);
    property Form: TForm read GetForm;
    property ShowOnStartup: Boolean read GetShowOnStartup write SetShowOnStartup;
  published
    { Published declarations }
  end;

  TabcGetTipEvent = procedure(Sender: TObject; var TipIndex: Integer; var TipText: string) of object;
  TabcShowDialogEvent = procedure(Sender: TObject; var ShowDialog: Boolean) of object;

  TabcWelcomeTips = class(TabcCustomWelcomeTips)
  private
    { Private declarations }
    FIniFile: TIniFile;
    FIniFileName: TFileName;
    FIniLocal: Boolean;
    FReadOnly: Boolean;
    FSharedTipIniFile: TIniFile;
    FSharedTipFileName: TFileName;
    FTipIndex: Integer;
    FOnGetShow: TabcShowDialogEvent;
    FOnSetShow: TabcShowDialogEvent;
    FOnGetTip: TabcGetTipEvent;
  protected
    { Protected declarations }
    function GetShowOnStartup: Boolean; override;
    procedure SetShowOnStartup(Value: Boolean); override;
    procedure OpenIniFiles;
    procedure CloseIniFiles;
  public
    { Public declarations }
    destructor Destroy; override;
    procedure Execute; override;
    procedure AfterExecute; override;
    function GetNextTip: string; override;
    property ReadOnly: Boolean read FReadOnly write FReadOnly;
    property TipIndex: Integer read FTipIndex write FTipIndex;
  published
    { Published declarations }
    property AutoShow;
    property Caption;
    property CaptionGreeting;
    property CaptionProgress;
    property CaptionShowNext;
    property CaptionTip;
    property HelpContents;
    property HelpWhatsNew;
    property IniLocal: Boolean read FIniLocal write FIniLocal default False;
    property IniFileName: TFilename read FIniFileName write FIniFileName;
    property ProgressMin;
    property ProgressMax;
    property ProgressPos;
    property ProgressStep;
    property SharedTipFileName: TFilename read FSharedTipFileName write FSharedTipFileName;
    property ShowProgress;
    property ShowModal;
    property StayOnTop;
    property OnGetShow: TabcShowDialogEvent read FOnGetShow write FOnGetShow;
    property OnSetShow: TabcShowDialogEvent read FOnSetShow write FOnSetShow;
    property OnGetTip: TabcGetTipEvent read FOnGetTip write FOnGetTip;
    property VisibleButtons;
  end;

implementation

uses abcutilf, abcfti32, abcconst;

procedure TabcCustomWelcomeTips.SetCaptionProgress(Value: string);
begin
  if Value <> FCaptionProgress then
  begin
    FCaptionProgress := Value;
    if Form <> nil then
      TabcsfTip(Form).lblProgress.Caption :=
        abcReplaceAll(sabcWelcomeAppTitleMask, Application.Title, Value, True, False);
  end;
end;

procedure TabcCustomWelcomeTips.SetProgressMax(const Value: Integer);
begin
  if Value <> FProgressMax then
  begin
    FProgressMax := Value;
    if Form <> nil then
      TabcsfTip(Form).ProgressBar.Max := Value;
  end;
end;

procedure TabcCustomWelcomeTips.SetProgressMin(const Value: Integer);
begin
  if Value <> FProgressMin then
  begin
    FProgressMin := Value;
    if Form <> nil then
      TabcsfTip(Form).ProgressBar.Min := Value;
  end;
end;

procedure TabcCustomWelcomeTips.SetProgressPos(const Value: Integer);
begin
  if Value <> FProgressPos then
  begin
    FProgressPos := Value;
    if Form <> nil then
      TabcsfTip(Form).ProgressBar.Position := Value;
  end;
end;

procedure TabcCustomWelcomeTips.SetProgressStep(const Value: Integer);
begin
  if Value <> FProgressStep then
  begin
    FProgressStep := Value;
    if Form <> nil then
      TabcsfTip(Form).ProgressBar.Step := Value;
  end;
end;

procedure TabcCustomWelcomeTips.SetShowProgress(const Value: Boolean);
begin
  if Value <> FShowProgress then
  begin
    FShowProgress := Value;
    if Form <> nil then
      with TabcsfTip(Form) do
      begin
        pnlProgress.Visible := Value;
        AdjustFormSize;
      end;
  end;
end;

function TabcCustomWelcomeTips.GetForm: TForm;
begin
  if (FForm <> nil) and not (csDestroying in FForm.ComponentState) then
    Result := FForm
  else
    Result := nil;
end;

procedure TabcCustomWelcomeTips.StepBy(Delta: Integer);
begin
  if Form <> nil then
    TabcsfTip(Form).ProgressBar.StepBy(Delta);
end;

procedure TabcCustomWelcomeTips.StepIt;
begin
  if Form <> nil then
    TabcsfTip(Form).ProgressBar.StepIt;
end;

procedure TabcCustomWelcomeTips.StepByMsg(Delta: Integer; Msg: string);
begin
  if Form <> nil then
    with TabcsfTip(Form) do
    begin
      ProgressBar.StepBy(Delta);
      lblProgress.Caption :=
        abcReplaceAll(sabcWelcomeAppTitleMask, Application.Title, Msg, True, False);
    end;
end;

procedure TabcCustomWelcomeTips.StepItMsg(Msg: string);
begin
  if Form <> nil then
    with TabcsfTip(Form) do
    begin
      ProgressBar.StepIt;
      lblProgress.Caption :=
        abcReplaceAll(sabcWelcomeAppTitleMask, Application.Title, Msg, True, False);
    end;
end;

constructor TabcCustomWelcomeTips.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCaption := sabcWelcomeCaption;
  FCaptionTip := sabcWelcomeCaptionTip;
  FCaptionGreeting := TStringList.Create;
  FCaptionShowNext := sabcWelcomeCaptionShowNext;
  FCaptionGreeting.Add(sabcWelcomeCaptionGreeting1);
  FCaptionProgress := sabcWelcomeCaptionProgress;
  FAutoShow := True;
  FVisibleButtons := [atbNextTip, atbClose];
  FStayOnTop := True;
  FProgressMax := 100;
  FProgressStep := 10;
end;

destructor TabcCustomWelcomeTips.Destroy;
begin
  FCaptionGreeting.Free;
  inherited Destroy;
end;

procedure TabcCustomWelcomeTips.Loaded;
begin
  inherited Loaded;
  if AutoShow and ShowOnStartup and not (csDesigning in ComponentState) then
    Execute;
end;

procedure TabcCustomWelcomeTips.ExecuteIfSelected;
begin
  if ShowOnStartup then Execute;
end;

procedure TabcCustomWelcomeTips.Execute;
var
  frmABCTip: TabcsfTip;
begin
  frmABCTip := TabcsfTip(
    abcFormFactory.CreateFormOfClass(TabcsfTip, Self));
  FForm := frmABCTip;
  with frmABCTip do
  begin
    btnNext.Visible := (atbNextTip in VisibleButtons);
    btnHelp.Visible := (atbHelpContents in VisibleButtons);
    btnHelp.HelpContext := HelpContents;
    btnWhatsNew.Visible := (atbHelpWhatsNew in VisibleButtons);
    btnWhatsNew.HelpContext := HelpWhatsNew;
    btnClose.Visible := (atbClose in VisibleButtons);
    Caption := Self.Caption;
    {substitute Real Application Title for #AppTitle#}
    lblGreeting.Lines.Text :=
      abcReplaceAll(sabcWelcomeAppTitleMask, Application.Title, Self.CaptionGreeting.Text, True, False);
    chkShowNext.Caption :=
      abcReplaceAll(sabcWelcomeAppTitleMask, Application.Title, Self.CaptionShowNext, True, False);
    lblDidYouKnow.Caption := Self.CaptionTip;
    lblProgress.Caption :=
      abcReplaceAll(sabcWelcomeAppTitleMask, Application.Title, Self.CaptionProgress, True, False);
    ProgressBar.Max := Self.ProgressMax;
    ProgressBar.Min := Self.ProgressMin;
    ProgressBar.Position := Self.ProgressPos;
    ProgressBar.Step := Self.ProgressStep;
    pnlProgress.Visible := Self.ShowProgress;
    if Self.StayOnTop then
      FormStyle := fsStayOnTop
    else
      FormStyle := fsNormal;
    if Self.ShowModal then
      ShowModal
    else
      Show;
  end;
end;

procedure TabcCustomWelcomeTips.AfterExecute;
begin
  FForm := nil;
end;

procedure TabcCustomWelcomeTips.SetCaptionGreeting(Value: TStrings);
begin
  if Value <> FCaptionGreeting then
    FCaptionGreeting.Assign(Value);
end;

{ TabcWelcomeTips }

{Tips dialog using INI File or custom event to get tip text}

destructor TabcWelcomeTips.Destroy;
begin
  CloseIniFiles;
  inherited;
end;

procedure TabcWelcomeTips.Execute;
begin
  OpenIniFiles;
  inherited;
end;

procedure TabcWelcomeTips.AfterExecute;
begin
  inherited;
  CloseIniFiles;
end;

procedure TabcWelcomeTips.OpenIniFiles;
var
  FileNameToOpen: TFileName;
begin
  if (csDesigning in ComponentState) then Exit;
  if FIniFile <> nil then CloseIniFiles;
  if (FIniFileName <> '') then
  begin
    if IniLocal then
      FileNameToOpen := ExtractFilePath(Application.ExeName) + FIniFileName
    else
      FileNameToOpen := FIniFileName;
    if FileExists(FileNameToOpen) then
      FIniFile := TIniFile.Create(FileNameToOpen)
    else
      MessageDlg(
        Format(sabcWelcomeWarningIniNotFound, [FileNameToOpen]),
        mtWarning, [mbOk], 0);
    if FSharedTipFileName <> '' then
      if FileExists(FSharedTipFileName) then
        FSharedTipIniFile := TIniFile.Create(FSharedTipFileName)
      else
        MessageDlg(
          Format(sabcWelcomeWarningSharedIniNotFound, [FSharedTipFileName]),
          mtWarning, [mbOk], 0);
  end;
end;

procedure TabcWelcomeTips.CloseIniFiles;
begin
  FIniFile.Free;
  FIniFile := nil;
  FSharedTipIniFile.Free;
  FSharedTipIniFile := nil;
end;

function TabcWelcomeTips.GetShowOnStartup: Boolean;
var
  NeedToClose: boolean;
begin
  Result := True;
  NeedToClose := (FIniFile = nil);
  if NeedToClose then
    OpenIniFiles;
  try
    if Assigned(FOnGetShow) then
      FOnGetShow(Self, Result)
    else if FIniFile = nil then
      Result := False
    else
      Result := FIniFile.ReadBool(sabcWelcomeIniTipSection, sabcWelcomeIniTipShow, True);
  finally
    if NeedToClose then
      CloseIniFiles;
  end;
end;

procedure TabcWelcomeTips.SetShowOnStartup(Value: Boolean);
begin
  if Assigned(FOnSetShow) then
    FOnSetShow(Self, Value)
  else if ((FIniFile <> nil) and not ReadOnly) then
    FIniFile.WriteBool(sabcWelcomeIniTipSection, sabcWelcomeIniTipShow, Value);
end;

function TabcWelcomeTips.GetNextTip: string;
var
  TipSource: TIniFile;
begin
  Result := '';
  if Assigned(FOnGetTip) then
    FOnGetTip(Self, FTipIndex, Result)
  else
  begin
    if FIniFile = nil then Exit;
    if not ReadOnly then
      FTipIndex := FIniFile.ReadInteger(sabcWelcomeIniTipSection, sabcWelcomeIniNextTip, 1);

    if (FSharedTipIniFile = nil) then
      TipSource := FIniFile
    else
      TipSource := FSharedTipIniFile;
    Result := TipSource.ReadString(sabcWelcomeIniTipTextSection, sabcWelcomeIniTipPrefix + InttoStr(FTipIndex), '');
    if Result = '' then
    begin
      FTipIndex := 1;
      Result := TipSource.ReadString(sabcWelcomeIniTipTextSection, sabcWelcomeIniTipPrefix + InttoStr(FTipIndex), '');
    end;
    {increment the tip count}
    FTipIndex := FTipIndex + 1;
    if not ReadOnly then
      FIniFile.WriteInteger(sabcWelcomeIniTipSection, sabcWelcomeIniNextTip, FTipIndex);
  end;
end;

end.
