unit abctitle;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  abccompf;

type

  { TabcTitleGradient }

  TabcTitleOption = (atoUseSystemColors, atoUseSystemFonts, atoUseSystemFontColors, atoMDIItalic,
    atoDrawInactive, atoReverseColors, atoDisableOnVer5, atoDisableOn256Color);

  TabcTitleOptions = set of TabcTitleOption;

  TabcCaptionDrawDefault = (addGradient, addIcon, addButtons, addText);
  TabcCaptionDrawDefaults = set of TabcCaptionDrawDefault;

  TabcCustomDrawCaptionEvent = procedure(Sender: TObject; IsActive: Boolean; Remainder: TRect; const CaptionImage: TBitmap) of object;

  TabcTitleGradient = class(TabcFormHook)
  private
    FCanvas: TCanvas;
    FbmpActive, FbmpInactive: TBitmap;
    FFont: TFont;
    FChildFont: TFont;
    FOptions: TabcTitleOptions;
    FActive: Boolean;
    FEnabled, FInternalEnabled: Boolean;
    FSaveNCUpdateRgn, FNewUpdateRgn: HRgn;
    FColorActive, FColorActiveHighlight: TColor;
    FColorInactive, FColorInactiveHighlight: TColor;
    FCaption: string;
    FDefaultDrawing: TabcCaptionDrawDefaults;
    FOnCustomDraw: TabcCustomDrawCaptionEvent;
    FInit: Boolean;
    function DoWMNCActivate(var Message: TMessage): Boolean;
    function DoWMSetCursor(var Message: TMessage): Boolean;
    function DoAMTitleRefresh(var Message: TMessage): Boolean;
    procedure DoBeforeWMNCPaint(var Message: TMessage);
    procedure DoAfterWMNCPaint(var Message: TMessage);
    procedure DoAfterWMNCActivate(var Message: TMessage);
    procedure DoAfterWMSize(var Message: TMessage);
    procedure DoAfterWMWindowPosChanged(var Message: TMessage);
    procedure DoAfterWMText(var Message: TMessage);
    procedure SetColorActive(const Value: TColor);
    procedure SetColorInactive(const Value: TColor);
    procedure SetColorActiveHighlight(const Value: TColor);
    procedure SetColorInactiveHighlight(const Value: TColor);
    procedure SetDefaultDrawing(const Value: TabcCaptionDrawDefaults);
    procedure SetEnabled(const Value: Boolean);
    procedure SetFont(Value: TFont);
    procedure SetChildFont(Value: TFont);
    function IsFontStored: Boolean;
    function IsChildFontStored: Boolean;
    procedure FontChanged(Sender: TObject);
    procedure SetOptions(const Value: TabcTitleOptions);
    procedure SetOnCustomDraw(const Value: TabcCustomDrawCaptionEvent);
  protected
    function DoProcessMessage(var Message: TMessage): Boolean; override;
    procedure AfterProcessMessage(var Message: TMessage); override;
    procedure DoOnSubClass; override;
    procedure Loaded; override;
    procedure PaintOwnerTitleBar;
    function GetCaptionRect: TRect;
    procedure CheckCaptionBitmaps(const DrawR: TRect);
    procedure RepaintCaptionBitmaps;
    procedure PaintFormCaption(const ABitmap: TBitmap; UseActiveColors: Boolean);
    procedure DoDrawBackground(const ACanvas: TCanvas; UseActiveColors: Boolean; var PaintR: TRect);
    procedure DoDrawIcon(const ACanvas: TCanvas; var PaintR: TRect);
    procedure DoDrawButtons(const ACanvas: TCanvas; var PaintR: TRect);
    procedure DoDrawCaptionText(const ACanvas: TCanvas; UseActiveColors: Boolean; var PaintR: TRect);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
    property CaptionRect: TRect read GetCaptionRect;
  published
    property ChildFont: TFont read FChildFont write SetChildFont stored IsChildFontStored;
    property ColorActive: TColor read FColorActive write SetColorActive default clNone;
    property ColorActiveHighlight: TColor read FColorActiveHighlight write SetColorActiveHighlight default clNone;
    property ColorInactive: TColor read FColorInactive write SetColorInactive default clNone;
    property ColorInactiveHighlight: TColor read FColorInactiveHighlight write SetColorInactiveHighlight default clNone;
    property DefaultDrawing: TabcCaptionDrawDefaults read FDefaultDrawing write SetDefaultDrawing
      default [addGradient, addButtons, addIcon, addText];
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Font: TFont read FFont write SetFont stored IsFontStored;
    property Options: TabcTitleOptions read FOptions write SetOptions
      default [atoUseSystemColors, atoUseSystemFonts];
    property OnCustomDraw: TabcCustomDrawCaptionEvent read FOnCustomDraw write SetOnCustomDraw;
  end;

implementation

uses abcconst, abcmsg, abcutilf, abcutilg;

var
  mNonClientMetrics: TNonClientMetrics;
  mIsWindows5, mScreenIsHighColor: Boolean;

  mActiveColorFrom, mActiveColorTo,
  mInactiveColorFrom, mInactiveColorTo: TColor;

procedure UpdateNCMetrics;
begin
  mNonClientMetrics.cbSize := sizeof(TNonClientMetrics);
  SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @mNonClientMetrics, 0);
  mScreenIsHighColor := abcScreenIsHighColor;
end;

procedure UpdateSysColors;
begin
  mActiveColorFrom := GetSysColor(COLOR_ACTIVECAPTION);
  mInactiveColorFrom := GetSysColor(COLOR_INACTIVECAPTION);
  if mIsWindows5 then
  begin
    mActiveColorTo := GetSysColor(COLOR_GRADIENTACTIVECAPTION);
    mInactiveColorTo := GetSysColor(COLOR_GRADIENTINACTIVECAPTION);
  end
  else
  begin
    mActiveColorTo := abcLightenCaptionColor(mActiveColorFrom);
    mInactiveColorTo := abcLightenCaptionColor(mInactiveColorFrom);
  end;
end;

{ TabcTitleGradient }

constructor TabcTitleGradient.Create(AOwner: TComponent);
var
  i: Integer;
  found: Boolean;
begin
  inherited;
  FInit := True;
  {make sure its the only instance for this form}
  found := False;
  with AOwner do
    for i := ComponentCount - 1 downto 0 do
      if Components[i] is TabcTitleGradient then
        if found then
          raise EComponentError.CreateFmt(
            sabcErrorOwnerSingleComponent, [Self.ClassName, AOwner.Name])
        else
          found := True;
  FCanvas := TCanvas.Create;
  FFont := TFont.Create;
  FFont.OnChange := FontChanged;
  FChildFont := TFont.Create;
  FChildFont.OnChange := FontChanged;
  FEnabled := True;
  FDefaultDrawing := [addGradient, addButtons, addIcon, addText];
  FOptions := [atoUseSystemColors, atoUseSystemFonts];
  FbmpActive := TBitmap.Create;
  FbmpInactive := TBitmap.Create;
  FColorActive := clNone;
  FColorActiveHighlight := clNone;
  FColorInactive := clNone;
  FColorInactiveHighlight := clNone;
  FActive := True;
end;

destructor TabcTitleGradient.Destroy;
begin
  if FNewUpdateRgn <> 0 then
  begin
    DeleteObject(FNewUpdateRgn);
    FNewUpdateRgn := 0;
  end;
  FCanvas.Free;
  FChildFont.Free;
  FFont.Free;
  FbmpActive.Free;
  FbmpInactive.Free;
  inherited;
end;

procedure TabcTitleGradient.Loaded;
begin
  inherited;
  if not (csDesigning in ComponentState) then
  begin
    FCaption := OwnerForm.Caption;
    UpdateNCMetrics;
    SetEnabled(FEnabled);
    UpdateSysColors;
  end;
end;

procedure TabcTitleGradient.DoOnSubClass;
begin
  FActive := True;
  PaintOwnerTitleBar;
end;

function TabcTitleGradient.DoProcessMessage(var Message: TMessage): Boolean;
begin
  Result := False;
  if not (csDesigning in ComponentState) then
    case Message.Msg of
      WM_NCACTIVATE:
        Result := DoWMNCActivate(Message);
      WM_NCPAINT:
        DoBeforeWMNCPaint(Message);
      WM_SETCURSOR:
        Result := DoWMSetCursor(Message);
      WM_SYSCOMMAND:
        PaintOwnerTitleBar;
      WM_DESTROY:
        if OwnerForm.FormStyle = fsMDIChild then
          PostMessage(Application.MainForm.Handle, AM_TITLEREFRESH, 0, 0);
      AM_TITLEREFRESH:
        Result := DoAMTitleRefresh(Message);
    end;
end;

procedure TabcTitleGradient.AfterProcessMessage(var Message: TMessage);
begin
  if not (csDesigning in ComponentState) then
    case Message.Msg of
      WM_NCACTIVATE:
        DoAfterWMNCActivate(Message);
      WM_NCPAINT:
        DoAfterWMNCPaint(Message);
      WM_NCLBUTTONDOWN, WM_SYSCOMMAND, WM_ENTERIDLE:
        PaintOwnerTitleBar;
      WM_SIZE:
        DoAfterWMSize(Message);
      WM_WINDOWPOSCHANGED:
        DoAfterWMWindowPosChanged(Message);
      WM_GETTEXT, WM_SETTEXT:
        DoAfterWMText(Message);
      WM_SETTINGCHANGE:
        begin
          UpdateNCMetrics;
          SetEnabled(FEnabled);
          PaintOwnerTitleBar;
        end;
      WM_SYSCOLORCHANGE:
        begin
          UpdateSysColors;
          PaintOwnerTitleBar;
        end;
      WM_SETICON:
        Refresh;
    end;
end;

function TabcTitleGradient.DoWMNCActivate(var Message: TMessage): Boolean;
begin
  if FInternalEnabled then
  begin
    Message.Result := 1;
    with OwnerForm do
    begin
      //allow default processing for [fsMDIChild, fsMDIForm]         ????
      Result := FormStyle in [fsNormal, fsStayOnTop];
      if (FormStyle = fsMDIForm) and (ActiveMDIChild <> nil) and
         (ActiveMDIChild.HandleAllocated) then
        SendMessage(ActiveMDIChild.Handle, WM_NCACTIVATE, Message.WParam, Message.LParam);
    end;
  end
  else
    Result := False;
end;

procedure TabcTitleGradient.DoAfterWMNCActivate(var Message: TMessage);
begin
  FActive := BOOL(Message.wParam);
  PaintOwnerTitleBar;
end;

procedure TabcTitleGradient.DoAfterWMSize(var Message: TMessage);
begin
  if (TWMSize(Message).SizeType in [SIZE_MAXIMIZED, SIZE_RESTORED]) then
    PaintOwnerTitleBar;
end;

procedure TabcTitleGradient.DoAfterWMWindowPosChanged(var Message: TMessage);
begin
  if (OwnerForm.FormStyle = fsMDIChild) then
    PostMessage(Application.MainForm.Handle, AM_TITLEREFRESH, 0, 0);
  if FInit then
  begin
    FInit := False;
    PostMessage(OwnerForm.Handle, AM_TITLEREFRESH, 0, 0);
  end;
end;

function TabcTitleGradient.DoAMTitleRefresh(var Message: TMessage): Boolean;
begin
  Result := True;
  Refresh;
end;

function TabcTitleGradient.DoWMSetCursor(var Message: TMessage): Boolean;
//stop button repaint on cursor change
begin
  if FInternalEnabled then
    with TWMSetCursor(Message) do
    begin
      Result := 1;
      case HitTest of
        HTLEFT, HTRIGHT:
          SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZEWE)));
        HTTOP, HTBOTTOM:
          SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZENS)));
        HTTOPLEFT, HTBOTTOMRIGHT:
          SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZENWSE)));
        HTTOPRIGHT, HTBOTTOMLEFT:
          SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZENESW)));
      else
        Result := 0;
      end;
    end;
  Result := Message.Result = 1;
end;

procedure TabcTitleGradient.DoBeforeWMNCPaint(var Message: TMessage);
{
  check if caption is in update region and if it is remove it
  to allow default form painting
}
var
  UpdateRgn, capRgn: HRgn;
  capRect, wndRect: TRect;
begin
  if not FInternalEnabled then Exit;
  GetWindowRect(OwnerHandle, wndRect);
  capRect := CaptionRect;
  OffsetRect(capRect, wndRect.Left, wndRect.Top);
  UpdateRgn := Message.WParam;
  if (UpdateRgn > 1) and not RectInRegion(UpdateRgn, capRect) then
    Exit;

  FNewUpdateRgn := CreateRectRgnIndirect(wndRect);
  capRgn := CreateRectRgnIndirect(capRect);
  try
    if (UpdateRgn > 1) then
      CombineRgn(FNewUpdateRgn, UpdateRgn, capRgn, RGN_DIFF)
    else
    begin
      UpdateRgn := CreateRectRgnIndirect(wndRect);
      try
        CombineRgn(FNewUpdateRgn, UpdateRgn, capRgn, RGN_DIFF);
      finally
        DeleteObject(UpdateRgn);
      end;
    end;
  finally
    DeleteObject(capRgn);
  end;
  FSaveNCUpdateRgn := Message.WParam;
  Message.WParam := FNewUpdateRgn;

end;

procedure TabcTitleGradient.DoAfterWMNCPaint(var Message: TMessage);
begin
  if FSaveNCUpdateRgn <> 0 then
  begin
    Message.WParam := FSaveNCUpdateRgn;
    FSaveNCUpdateRgn := 0;
    DeleteObject(FNewUpdateRgn);
    FNewUpdateRgn := 0;
    PaintOwnerTitleBar;
  end;
end;

procedure TabcTitleGradient.DoAfterWMText(var Message: TMessage);
begin
  if FCaption <> OwnerForm.Caption then
  begin
    FCaption := OwnerForm.Caption;
    if FInternalEnabled then PostMessage(OwnerHandle, AM_TITLEREFRESH, 0, 0);
  end;
end;

procedure TabcTitleGradient.PaintOwnerTitleBar;
var
  DrawR: TRect;
  SaveIndex: Integer;
  bmpDraw: TBitmap;
begin
  if not FInternalEnabled or
     (OwnerForm.BorderStyle = bsNone) then
    Exit;

  DrawR := CaptionRect;

  CheckCaptionBitmaps(DrawR);

  if FActive then
    bmpDraw := FbmpActive
  else
    bmpDraw := FbmpInactive;

  with FCanvas do
  begin
    Handle := GetWindowDC(OwnerHandle);
    SaveIndex := SaveDC(Handle);
    Lock;
    try
      Draw(DrawR.Left, DrawR.Top, bmpDraw);
    finally
      Unlock;
      RestoreDC(Handle, SaveIndex);
      ReleaseDC(Handle, OwnerHandle);
      Handle := 0;
    end;
  end;
end;

procedure TabcTitleGradient.CheckCaptionBitmaps(const DrawR: TRect);
{
  Check bitmaps are still the size of the drawing rect
  If not, repaint them.
}
begin
  with FbmpActive, DrawR do
    if (Width <> (Right - Left)) or
       (Height <> (Bottom - Top)) then
    begin
      Width := Right - Left;
      Height := Bottom - Top;
      FbmpInActive.Width := Width;
      FbmpInActive.Height := Height;
      RepaintCaptionBitmaps;
    end;
end;

procedure TabcTitleGradient.RepaintCaptionBitmaps;
begin
  PaintFormCaption(FbmpActive, True);
  PaintFormCaption(FbmpInActive, False);
end;

procedure TabcTitleGradient.PaintFormCaption(const ABitmap: TBitmap; UseActiveColors: Boolean);
var
  Remainder: TRect;
  SaveIndex: Integer;
begin
  with ABitmap do
  begin
    Remainder := Rect(0,0,Width,Height);
    with Canvas do
    begin
      SaveIndex := SaveDC(Handle);
      Lock;
      try
        if addGradient in DefaultDrawing then
          DoDrawBackground(Canvas, UseActiveColors, Remainder);

        if (biSystemMenu in OwnerForm.BorderIcons) and
           (OwnerForm.BorderStyle in [bsSingle, bsSizeable]) and
           (addIcon in DefaultDrawing) then
          DoDrawIcon(Canvas, Remainder);

        if addButtons in DefaultDrawing then
          DoDrawButtons(Canvas, Remainder);

        if addText in DefaultDrawing then
          DoDrawCaptionText(Canvas, UseActiveColors, Remainder);

        if Assigned(OnCustomDraw) then
          OnCustomDraw(Self, UseActiveColors, Remainder, ABitmap);

      finally
        Unlock;
        RestoreDC(Handle, SaveIndex);
      end;
    end;
  end;
end;

procedure TabcTitleGradient.DoDrawBackground(const ACanvas: TCanvas; UseActiveColors: Boolean; var PaintR: TRect);
var
  clrFrom, clrTo, clrTemp: TColor;
  FromR, FromG, FromB, ToR, ToG, ToB, StepR, StepG, StepB: Byte;
  paintW, paintH, gradW, stepWidth, stepX: integer;
  stepRect: TRect;
begin

  if atoUseSystemColors in Options then
    if UseActiveColors then
    begin
  		clrFrom	:= mActiveColorFrom;
  		clrTo	:= mActiveColorTo;
    end
    else
    begin
      clrFrom	:= mInactiveColorFrom;
      clrTo	:= mInactiveColorTo;
    end
  else
    if UseActiveColors then
    begin
  		clrFrom	:= FColorActive;
  		clrTo	:= FColorActiveHighlight;
    end
    else
    begin
      clrFrom	:= FColorInactive;
      clrTo	:= FColorInactiveHighlight;
    end;

  if (atoReverseColors in Options) then
  begin
    clrTemp := clrFrom;
    clrFrom := clrTo;
    clrTo := clrTemp;
  end;

	FromR := GetRValue(DWORD(clrFrom));
	FromG := GetGValue(DWORD(clrFrom));
	FromB := GetBValue(DWORD(clrFrom));

	ToR := GetRValue(DWORD(clrTo));
	ToG := GetGValue(DWORD(clrTo));
	ToB := GetBValue(DWORD(clrTo));

  with PaintR, ACanvas do
  begin

    paintW := Right - Left;
    paintH := Bottom - Top;

    stepX := paintW * 5 div 6;  // start 5/6 of the way right
    gradW := stepX;
    if gradW <= 64 then
      stepWidth := 1
    else
      stepWidth := gradW div 64;

    // Paint far right 1/6 of caption the background color
    Brush.Color := clrTo;

    FillRect(Rect(stepX, 0, paintW, paintH));

    stepRect := Rect(stepX, 0, paintW - stepWidth, paintH);

    while (stepX > stepWidth) do
    begin

      stepX := stepX - stepWidth;

      if (ToR > FromR) then
        StepR := ToR - (ToR - FromR) * (gradW - stepX) div gradW
      else
        StepR := ToR + (FromR - ToR) * (gradW - stepX) div gradW;

      if (ToG > FromG) then
        StepG := ToG - (ToG - FromG) * (gradW - stepX) div gradW
      else
        StepG := ToG + (FromG - ToG) * (gradW - stepX) div gradW;


      if (ToB > FromB) then
        StepB := ToB - (ToB - FromB) * (gradW - stepX) div gradW
      else
        StepB := ToB + (FromB - ToB) * (gradW - stepX) div gradW;

      Brush.Color := RGB(StepR, StepG, StepB);

      OffsetRect(stepRect, -stepWidth, 0);

      FillRect(stepRect);

    end;

    // Paint what's left of the caption with the beginning color

    Brush.Color := clrFrom;
    FillRect(Rect(0,0,stepX, paintH));

    Left := Left + 2;

  end;

end;

procedure TabcTitleGradient.DoDrawIcon(const ACanvas: TCanvas; var PaintR: TRect);
var
  iWidth, iHeight: Integer;
  frmIcon, copiedIcon: HICON;
  bDestroy: Boolean;

begin
  bDestroy := False;
  frmIcon := 0;
  
  iWidth := GetSystemMetrics(SM_CXSMICON);
  iHeight := GetSystemMetrics(SM_CYSMICON);
  with PaintR do
  try
    if OwnerForm.Icon.Handle <> 0 then
      frmIcon := OwnerForm.Icon.Handle
    else if Application.Icon.Handle <> 0 then
      frmIcon := Application.Icon.Handle
    else
    begin
      frmIcon := LoadIcon(0, IDI_APPLICATION);
      bDestroy := True;
    end;

    copiedIcon := CopyImage(frmIcon, IMAGE_ICON, iWidth, iHeight, LR_COPYFROMRESOURCE);

    try

      DrawIconEx(ACanvas.Handle, Left, Top + 1,
        copiedIcon, 0, 0, 0, 0, DI_NORMAL);

      Left := Left + iWidth + 2;

    finally
      DestroyIcon(copiedIcon);
    end;

  finally
    if bDestroy then
      DestroyIcon(frmIcon);
  end;

end;

procedure TabcTitleGradient.DoDrawButtons(const ACanvas: TCanvas; var PaintR: TRect);
var
  buttonWidth, buttonHeight, buttonsDrawn, buttonFlags: integer;
  buttonR: TRect;
begin
  with OwnerForm do
  begin
    if not (biSystemMenu in BorderIcons) then Exit;
    if not (BorderStyle in [bsToolWindow, bsSizeToolWin]) then
    begin
      buttonWidth := mNonClientMetrics.iCaptionWidth;
      buttonHeight := mNonClientMetrics.iCaptionHeight;
      buttonR := Rect(0,0,buttonWidth,buttonHeight);
      OffsetRect(buttonR, PaintR.Right - buttonWidth, 0);
      InflateRect(buttonR, 0, -2);
      buttonR.Right := buttonR.Right - 2;
      buttonsDrawn := buttonWidth + 2;  //close and edges
      DrawFrameControl(ACanvas.Handle, buttonR, DFC_CAPTION, DFCS_CAPTIONCLOSE);

      if BorderStyle <> bsDialog then
      begin
        if BorderIcons * [biMinimize, biMaximize] <> [] then
        begin
          buttonsDrawn := buttonsDrawn + 2 * buttonWidth + 1;  //min, max buttons
          OffsetRect(buttonR, -buttonWidth, 0);
          if IsZoomed(OwnerHandle) then
            buttonFlags := DFCS_CAPTIONRESTORE
          else
            buttonFlags := DFCS_CAPTIONMAX;
          if not (biMaximize in BorderIcons) then buttonFlags := buttonFlags or DFCS_INACTIVE;
          DrawFrameControl(ACanvas.Handle, buttonR, DFC_CAPTION, buttonFlags);
          OffsetRect(buttonR, -buttonWidth+2, 0);
          if IsIconic(OwnerHandle) then
            buttonFlags := DFCS_CAPTIONRESTORE
          else
            buttonFlags := DFCS_CAPTIONMIN;
          if not (biMinimize in BorderIcons) then buttonFlags := buttonFlags or DFCS_INACTIVE;
          DrawFrameControl(ACanvas.Handle, buttonR, DFC_CAPTION, buttonFlags);
          if BorderIcons * [biMinimize, biMaximize] <> [biMinimize, biMaximize] then
            if biHelp in BorderIcons then
            begin
              buttonsDrawn := buttonsDrawn + buttonWidth + 1; //biHelp
              OffsetRect(buttonR, -buttonWidth-2, 0);
              DrawFrameControl(ACanvas.Handle, buttonR, DFC_CAPTION, DFCS_CAPTIONHELP);
            end;
        end
        else
          if biHelp in BorderIcons then
          begin
            buttonsDrawn := buttonsDrawn + buttonWidth + 1; //biHelp with no min or max
            OffsetRect(buttonR, -buttonWidth, 0);
            DrawFrameControl(ACanvas.Handle, buttonR, DFC_CAPTION, DFCS_CAPTIONHELP);
          end;
      end
      else
        if biHelp in BorderIcons then
        begin
          buttonsDrawn := buttonsDrawn + buttonWidth + 1; //biHelp on dialog
          OffsetRect(buttonR, -buttonWidth, 0);
          DrawFrameControl(ACanvas.Handle, buttonR, DFC_CAPTION, DFCS_CAPTIONHELP);
        end;
    end
    else
    begin
      buttonWidth := mNonClientMetrics.iSmCaptionWidth;
      buttonHeight := mNonClientMetrics.iSmCaptionHeight;
      buttonR := Rect(0,0,buttonWidth,buttonHeight);
      OffsetRect(buttonR, PaintR.Right - buttonWidth, 0);
      InflateRect(buttonR, 0, -2);
      buttonR.Right := buttonR.Right - 2;
      DrawFrameControl(ACanvas.Handle, buttonR, DFC_CAPTION, DFCS_CAPTIONCLOSE);
      buttonsDrawn := buttonWidth + 2;  //close and edges
    end;
  end;
  with PaintR do
    Right := Right - buttonsDrawn + 2;
end;

procedure TabcTitleGradient.DoDrawCaptionText(const ACanvas: TCanvas; UseActiveColors: Boolean; var PaintR: TRect);
var
  captionFont: LOGFONT;
  captionMain, captionChild: string;
  capPos: Integer;
  calcR: TRect;
  Flags: Integer;

  procedure SetFontColors;
  begin
    if [atoUseSystemFonts, atoUseSystemFontColors] * Options <> [] then
      if UseActiveColors then
        ACanvas.Font.Color := clCaptionText
      else
        ACanvas.Font.Color := clInactiveCaptionText;
  end;

begin
  with ACanvas do
  begin
    if atoUseSystemFonts in Options then
    begin
      if not (OwnerForm.BorderStyle in [bsToolWindow, bsSizeToolWin]) then
        captionFont := mNonClientMetrics.lfCaptionFont
      else
        captionFont := mNonClientMetrics.lfSmCaptionFont;
      Font.Handle := CreateFontIndirect(captionFont);
    end
    else
      Font := Self.Font;

    SetFontColors;

    SetBkMode(Handle, TRANSPARENT);

    Flags := DT_LEFT or DT_VCENTER or DT_SINGLELINE or DT_END_ELLIPSIS;

    calcR := PaintR;

    if (OwnerForm.FormStyle <> fsMDIForm) or
       (OwnerForm.ActiveMDIChild = nil) or
       not IsZoomed(OwnerForm.ActiveMDIChild.Handle) or
       ([atoUseSystemFonts, atoMDIItalic] * Options = [atoUseSystemFonts]) then
    begin
      // default drawing
      DrawText(Handle, PChar(FCaption), Length(FCaption), calcR,
         Flags or DT_CALCRECT);
      DrawText(Handle, PChar(FCaption), Length(FCaption), PaintR,
         Flags);
      if calcR.Right < (PaintR.Right-1) then
        PaintR.Left := calcR.Right + 1
      else
        SetRectEmpty(PaintR);
    end
    else
    begin
      capPos := Pos('-', FCaption);
      if capPos = 0 then
        capPos := Length(FCaption);
      captionMain := Copy(FCaption, 1, capPos);
      captionChild := Copy(FCaption, capPos+1, Length(FCaption));

      if atoMDIItalic in Options then
        Font.Style := Font.Style + [fsItalic];

      DrawText(Handle, PChar(captionMain), Length(captionMain), calcR,
         Flags or DT_CALCRECT);

      DrawText(Handle, PChar(captionMain), Length(captionMain), PaintR,
         Flags);

      PaintR.Left := calcR.Right + 1;
      calcR := PaintR;

      if not (atoUseSystemFonts in Options) then
      begin
        Font := ChildFont;
        SetFontColors;
      end
      else if atoMDIItalic in Options then
        Font.Style := Font.Style - [fsItalic];

      DrawText(Handle, PChar(captionChild), Length(captionChild), calcR,
         Flags or DT_CALCRECT);
      DrawText(Handle, PChar(captionChild), Length(captionChild), PaintR,
         Flags);
         
      if calcR.Right < (PaintR.Right-1) then
        PaintR.Left := calcR.Right + 1
      else
        SetRectEmpty(PaintR);
    end;
  end;
end;

function TabcTitleGradient.GetCaptionRect: TRect;
// returns window caption rect in client coordinates
var
  fWidth: Integer;
begin
  GetWindowRect(OwnerHandle, Result);
  fWidth := Result.Right - Result.Left;
  with Result do
  begin
    if (OwnerForm.BorderStyle in [bsSingle, bsDialog, bsToolWindow]) or IsIconic(OwnerHandle) then
    begin
      Left := GetSystemMetrics(SM_CXFIXEDFRAME);
      Top := GetSystemMetrics(SM_CYFIXEDFRAME);
    end
    else
    begin
      Left := GetSystemMetrics(SM_CXSIZEFRAME);
      Top := GetSystemMetrics(SM_CYSIZEFRAME);
    end;
    Right := fWidth - Left;
    Bottom := Top - GetSystemMetrics(SM_CYBORDER);
    if not (OwnerForm.BorderStyle in [bsToolWindow, bsSizeToolWin]) then
      Bottom := Bottom + GetSystemMetrics(SM_CYCAPTION)
    else
      Bottom := Bottom + GetSystemMetrics(SM_CYSMCAPTION);
  end;
end;

procedure TabcTitleGradient.SetEnabled(const Value: Boolean);
begin
  FEnabled := Value;
  FInternalEnabled :=
    Value and
    not (mIsWindows5 and (atoDisableOnVer5 in Options)) and
    (mScreenIsHighColor or not (atoDisableOn256Color in Options));
  if not (csDesigning in ComponentState) and OwnerForm.Showing then
    if FInternalEnabled then
    begin
      UpdateNCMetrics;
      UpdateSysColors;
      Refresh;
    end
    else
      PostMessage(OwnerHandle, WM_NCACTIVATE,
        Integer(OwnerForm.Active or (OwnerForm.FormStyle = fsMDIForm)), 0);
end;

procedure TabcTitleGradient.SetOptions(const Value: TabcTitleOptions);
var
  setSysColors, setSysFonts: Boolean;
begin
  if Value <> FOptions then
  begin
    setSysColors := (atoUseSystemColors in Value) and not (atoUseSystemColors in FOptions);
    setSysFonts := (atoUseSystemFonts in Value) and not (atoUseSystemFonts in FOptions);
    FOptions := Value;
    if atoUseSystemFonts in FOptions then
      FOptions := FOptions + [atoUseSystemFontColors];
    if setSysColors then
    begin
      FColorActive := clNone;
      FColorActiveHighlight := clNone;
      FColorInactive := clNone;
      FColorInactiveHighlight := clNone;
    end;
    if not (csLoading in ComponentState) and setSysFonts then
    begin
      FFont.Free;
      FFont := TFont.Create;
      FFont.OnChange := FontChanged;
      FChildFont.Free;
      FChildFont := TFont.Create;
      FChildFont.OnChange := FontChanged;
    end;
    UpdateSysColors;
    SetEnabled(FEnabled);
  end;
end;

procedure TabcTitleGradient.SetColorActive(const Value: TColor);
begin
  if Value <> FColorActive then
  begin
    FColorActive := Value;
    if not (csLoading in ComponentState) then
    begin
      if FColorActiveHighlight = clNone then
        FColorActiveHighlight := abcLightenCaptionColor(Value);
      FOptions := FOptions - [atoUseSystemColors];
      if not (csDesigning in ComponentState) then
        Refresh;
    end;
  end;
end;

procedure TabcTitleGradient.SetColorActiveHighlight(const Value: TColor);
begin
  if Value <> FColorActiveHighlight then
  begin
    FColorActiveHighlight := Value;
    if not (csLoading in ComponentState) then
    begin
      FOptions := FOptions - [atoUseSystemColors];
      if not (csDesigning in ComponentState) then
        Refresh;
    end;
  end;
end;

procedure TabcTitleGradient.SetColorInactive(const Value: TColor);
begin
  if Value <> FColorInactive then
  begin
    FColorInactive := Value;
    if not (csLoading in ComponentState) then
    begin
      if FColorInactiveHighlight = clNone then
        FColorInactiveHighlight := abcLightenCaptionColor(Value);
      FOptions := FOptions - [atoUseSystemColors];
      if not (csDesigning in ComponentState) then
        Refresh;
    end;
  end;
end;

procedure TabcTitleGradient.SetColorInactiveHighlight(const Value: TColor);
begin
  if Value <> FColorInactiveHighlight then
  begin
    FColorInactiveHighlight := Value;
    if not (csLoading in ComponentState) then
    begin
      FOptions := FOptions - [atoUseSystemColors];
      if not (csDesigning in ComponentState) then
        Refresh;
    end;
  end;
end;

procedure TabcTitleGradient.Refresh;
begin
  //force redraw
  FbmpActive.Width := 0;
  if (OwnerForm.FormStyle <> fsMDIChild) then
    PaintOwnerTitleBar
  else if IsZoomed(OwnerHandle) then
    PostMessage(Application.MainForm.Handle, AM_TITLEREFRESH, 0, 0)
  else
    PaintOwnerTitleBar;
end;

procedure TabcTitleGradient.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
  if [csLoading, csDesigning] * ComponentState <> [] then
    Refresh;
end;

function TabcTitleGradient.IsFontStored: Boolean;
begin
  Result := abcIsFontStored(Font);
end;

procedure TabcTitleGradient.SetChildFont(Value: TFont);
begin
  FChildFont.Assign(Value);
  if [csLoading, csDesigning] * ComponentState <> [] then
    Refresh;
end;

function TabcTitleGradient.IsChildFontStored: Boolean;
begin
  Result := abcIsFontStored(ChildFont);
end;

procedure TabcTitleGradient.FontChanged(Sender: TObject);
begin
  if not (csLoading in ComponentState) then
  begin
    FOptions := FOptions - [atoUseSystemFonts, atoUseSystemFontColors];
    if not (csDesigning in ComponentState) then
      Refresh;
  end;
end;

procedure TabcTitleGradient.SetDefaultDrawing(
  const Value: TabcCaptionDrawDefaults);
begin
  FDefaultDrawing := Value;
  if [csLoading, csDesigning] * ComponentState <> [] then
    Refresh;
end;

procedure TabcTitleGradient.SetOnCustomDraw(
  const Value: TabcCustomDrawCaptionEvent);
begin
  FOnCustomDraw := Value;
  if [csLoading, csDesigning] * ComponentState <> [] then
    Refresh;
end;

(*
  Custom Rect removed, since there are windows buttons underneath our painting
  that can't be moved or fully hidden...

  TabcAdjustRectEvent = procedure(Sender: TObject; var ARect: TRect) of object;
   FOnCalcCaptionRect: TabcAdjustRectEvent;
    procedure SetOnCalcCaptionRect(const Value: TabcAdjustRectEvent);
    property OnCalcCaptionRect: TabcAdjustRectEvent read FOnCalcCaptionRect write SetOnCalcCaptionRect;
  if Assigned(OnCalcCaptionRect) then
  begin
    tmpRect := Result;
    OnCalcCaptionRect(Self, tmpRect);
    Result := tmpRect;
  end;
procedure TabcTitleGradient.SetOnCalcCaptionRect(
  const Value: TabcAdjustRectEvent);
begin
  FOnCalcCaptionRect := Value;
  if [csLoading, csDesigning] * ComponentState <> [] then
    Refresh;
end;
*)

initialization
  mIsWindows5 := LoByte(LoWord(GetVersion)) > 4;

end.
