unit abctray;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, Messages, ShellAPI, SysUtils, Classes, Graphics, Controls, Forms,
  Menus, Dialogs, ExtCtrls, ImgList,
  abccompf, abcmsg;

type

{$IFNDEF DELPHI5}
  TImageIndex = type Integer;
{$ENDIF}

  { TabcTrayIcon }

  TabcTrayIconOption = (
    atiShowOnClick, atiShowOnDblClick, atiMenuDefaultOnDblClick,
    atiHideApp, atiHideOnMinimize, atiHideOnClose,
    atiWin95Service, atiTerminateOnLogoff, atiMonitorTray);
// these pairs are mutually exclusive:
// [atiShowOnClick, atiShowOnDblClick] and [atiShowOnDblClick, atiMenuDefaultOnDblClick]
// [atiWin95Service, atiTerminateOnLogoff]
//   - if atiTerminateOnLogoff is false, atiWin95Service must be true
//   - if atiWin95Service is false, atiTerminateOnLogoff must be true
//default [atiShowOnDblClick, atiHideApp, atiHideOnClose, atiTerminateOnLogoff];

  TabcTrayIconOptions = set of TabcTrayIconOption;

{$IFNDEF DELPHI5}
  TContextPopupEvent = procedure(Sender: TObject; MousePos: TPoint; var Handled: Boolean) of object;
{$ENDIF}

  TabcTrayIcon = class(TabcFormHook)
  private
    { Private declarations }
    FAnimated: Boolean;
    FAnimateTimer: TTimer;
    FCursorPos: TPoint;
    FHint: string;
    FIconData: TNotifyIconData;
    FIcon: TIcon;
    FImages: TImageList;
    FImageIndex: TImageIndex;
    FMonitor: TTimer;
    FOptions: TabcTrayIconOptions;
    FOnContextPopup: TContextPopupEvent;
    FOnMouseDown: TMouseEvent;
    FOnMouseUp: TMouseEvent;
    FOnMouseMove: TMouseMoveEvent;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FPopupMenu: TPopupMenu;
    FServiceRegistered: Boolean;
    FTrayAdded: Boolean;
    FVisible: Boolean;
    procedure SetAnimated(Value: Boolean);
    function GetAnimateInterval: Cardinal;
    procedure SetAnimateInterval(Value: Cardinal);
    procedure AnimateIcon(Sender: TObject);
    function GetMonitorInterval: Cardinal;
    procedure SetMonitorInterval(Value: Cardinal);
    procedure MonitorCheckTray(Sender: TObject);
    procedure UpdateCursorPos;
    procedure IconChanged(Sender: TObject);
    function GetIconHandle: HICON;
    function IsIconStored: Boolean;
    procedure SetHint(const Value: string);
    procedure SetIcon(const Value: TIcon);
    procedure SetOptions(const Value: TabcTrayIconOptions);
    procedure SetImages(Value: TImageList);
    procedure SetImageIndex(Value: TImageIndex);
    procedure SetVisible(const Value: Boolean);
  protected
    { Protected declarations }
    procedure TrayMouseEvent(var Message: TMessage); dynamic;
    procedure ContextPopup(MousePos: TPoint); dynamic;
    procedure Click(Button: TMouseButton); dynamic;
    procedure DblClick(Button: TMouseButton); dynamic;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); dynamic;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); dynamic;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); dynamic;
    function DoProcessMessage(var Message: TMessage): Boolean; override;
    function DoProcessAppMessage(var Message: TMessage): Boolean; override;
    procedure AfterProcessMessage(var Message: TMessage); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddIcon; dynamic;
    procedure DeleteIcon; dynamic;
    procedure ShowForm; dynamic;
    procedure DoMenuDefault; dynamic;
  published
    { Published declarations }
    property Animated: Boolean read FAnimated write SetAnimated default False;
    property AnimateInterval: Cardinal read GetAnimateInterval write SetAnimateInterval default 700;
    property Hint: string read FHint write SetHint;
    property Icon: TIcon read FIcon write SetIcon stored IsIconStored;
    property Images: TImageList read FImages write SetImages;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default 0;
    property MonitorInterval: Cardinal read GetMonitorInterval write SetMonitorInterval default 5000;
    property Options: TabcTrayIconOptions read FOptions write SetOptions
      default [atiShowOnDblClick, atiHideApp, atiHideOnClose, atiTerminateOnLogoff];
    property PopupMenu: TPopupMenu read FPopupMenu write FPopupMenu;
    property Visible: Boolean read FVisible write SetVisible default True;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnContextPopup: TContextPopupEvent read FOnContextPopup write FOnContextPopup;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnMouseDown: TMouseEvent read FOnMouseDown write FOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
    property OnMouseUp: TMouseEvent read FOnMouseUp write FOnMouseUp;
  end;

implementation

var
  mIE4Installed: Boolean = False;
  mIsNT351: Boolean = False;
  mTaskBarCreated: DWord = 0;

procedure abcRegWin95ServiceProcess(Reg: Boolean);
type
  TRegisterServiceProc = function(dwProcessId, dwType: DWORD): DWORD; stdcall;
var
  hKernel: HMODULE;
  pReg: TRegisterServiceProc;
begin
  hKernel := LoadLibrary('kernel32.dll');
  if hKernel <> 0 then
  begin
    @pReg := GetProcAddress(hKernel, 'RegisterServiceProcess');
    if @pReg <> nil then
    begin
      if Reg then
        pReg(0,1)
      else
        pReg(0,0);
    end;
    FreeLibrary(hKernel);
  end;
end;

{ TabcTrayIcon }

constructor TabcTrayIcon.Create(AOwner: TComponent);
begin
  inherited;
  FIcon := TIcon.Create;
{$IFDEF DELPHI5}
  FIcon.Width := GetSystemMetrics(SM_CXSMICON);
  FIcon.Height := GetSystemMetrics(SM_CYSMICON);
{$ENDIF}
  FIcon.OnChange := IconChanged;
  FVisible := True;
  FOptions := [atiShowOnDblClick, atiHideApp, atiHideOnClose, atiTerminateOnLogoff];
  FMonitor := TTimer.Create(Self);
  with FMonitor do
  begin
    Enabled := False;
    Interval := 5000;
    OnTimer := MonitorCheckTray;
  end;
  FAnimateTimer := TTimer.Create(Self);
  with FAnimateTimer do
  begin
    Enabled := False;
    Interval := 700;
    OnTimer := AnimateIcon;
  end;
end;

destructor TabcTrayIcon.Destroy;
begin
  FAnimateTimer.Free;
  FMonitor.Free;
  DeleteIcon;
  FIcon.Free;
  if FServiceRegistered then
  begin
    abcRegWin95ServiceProcess(False);
    FServiceRegistered := False;
  end;
  inherited;
end;

procedure TabcTrayIcon.MonitorCheckTray(Sender: TObject);
//check valid icon handle in case tray is destroyed or user logs off on NT
begin
  if FTrayAdded then
    if not Shell_NotifyIcon(NIM_MODIFY, @FIconData) then
    begin
      FMonitor.Enabled := False;
      DeleteIcon;
      AddIcon;
      FMonitor.Enabled := True;
    end;
end;

procedure TabcTrayIcon.AddIcon;
begin
  if FTrayAdded or mIsNT351 then Exit;
  with FIconData do
  begin
    cbSize := SizeOf(FIconData);
    Wnd := OwnerHandle;
    uID := UINT(Self);
    uFlags := NIF_MESSAGE or NIF_ICON or NIF_TIP;
    hIcon := GetIconHandle;
    uCallbackMessage := AM_TRAYMOUSEEVENT;
    StrLCopy(szTip, PChar(FHint), 63);
  end;
  Shell_NotifyIcon(NIM_Add, @FIconData);
  FTrayAdded := True;
  FMonitor.Enabled := (atiMonitorTray in Options);
end;

procedure TabcTrayIcon.DeleteIcon;
begin
  if not FTrayAdded or mIsNT351 then Exit;
  try
    Shell_NotifyIcon(NIM_DELETE, @FIconData);
  except
  end;
  FTrayAdded := False;
end;

function TabcTrayIcon.DoProcessMessage(var Message: TMessage): Boolean;
begin
  Result := False;
  case Message.Msg of
    AM_TRAYMOUSEEVENT:
      begin
        Result := True;
        TrayMouseEvent(Message);
      end;
    WM_SYSCOMMAND:
      if (Message.WParam = SC_MINIMIZE) and (atiHideOnMinimize in Options) then
      begin
        Result := True;
        ShowWindow(OwnerHandle, SW_HIDE);
        if Application.MainForm = OwnerForm then
          ShowWindow(Application.Handle, SW_HIDE);
      end;
    WM_CLOSE:
      if atiHideOnClose in Options then
      begin
        Result := True;
        OwnerForm.Hide;
      end;
  end;
end;

procedure TabcTrayIcon.AfterProcessMessage(var Message: TMessage);
begin
  inherited;
end;

function TabcTrayIcon.DoProcessAppMessage(var Message: TMessage): Boolean;
begin
  Result := False;
  case Message.Msg of
    WM_QUIT:
      DeleteIcon;
    WM_USERCHANGED:
      if not (atiTerminateOnLogoff in Options) then
        PostMessage(Application.Handle, AM_REFRESHTRAYICON, 0, 0);
    WM_QUERYENDSESSION:
      if not (atiTerminateOnLogoff in Options) and
        (DWORD(Message.lParam) = ENDSESSION_LOGOFF) then
      begin
        Result := True;
        Message.Result := 1;
      end;
    WM_ENDSESSION:
      if not (atiTerminateOnLogoff in Options) and
        BOOL(Message.wParam) and
        (DWORD(Message.lParam) = ENDSESSION_LOGOFF) then
      begin
        Result := True;
      end;
    AM_REFRESHTRAYICON:
      if (not Application.Terminated) and Visible then
      begin
        Result := True;
        DeleteIcon;
        Sleep(1000);
        AddIcon;
      end;
  else
    if (Message.Msg = mTaskBarCreated) and Visible then
    begin
      Result := True;
      DeleteIcon;
      AddIcon;
    end;
  end;
end;

procedure TabcTrayIcon.TrayMouseEvent(var Message: TMessage);
begin
  {only process messages for this icon - allow multiple icons on a form}
  if UINT(Message.wParam) <> FIconData.uID then
    Exit;
  UpdateCursorPos;
  case Message.LParam of
    WM_MOUSEMOVE:
      MouseMove([], FCursorPos.X, FCursorPos.Y);
    WM_LBUTTONDOWN:
      MouseDown(mbLeft, [], FCursorPos.X, FCursorPos.Y);
    WM_RBUTTONDOWN:
      MouseDown(mbRight, [], FCursorPos.X, FCursorPos.Y);
    WM_MBUTTONDOWN:
      MouseDown(mbMiddle, [], FCursorPos.X, FCursorPos.Y);
    WM_LBUTTONUP:
      begin
        {OnMouseUp, option - show form, OnClick}
        MouseUp(mbLeft, [], FCursorPos.X, FCursorPos.Y);
        Click(mbLeft);
      end;
    WM_RBUTTONUP:
      begin
        {OnMouseUp, OnContextPopup, show popupmenu}
        MouseUp(mbRight, [], FCursorPos.X, FCursorPos.Y);
        Click(mbRight);
      end;
    WM_MBUTTONUP:
      begin
        {OnMouseUp, OnClick}
        MouseUp(mbMiddle, [], FCursorPos.X, FCursorPos.Y);
        Click(mbMiddle);
      end;
    WM_LBUTTONDBLCLK:
      DblClick(mbLeft);
    WM_MBUTTONDBLCLK:
      DblClick(mbMiddle);
    WM_RBUTTONDBLCLK:
      DblClick(mbRight);
  end;
end;

procedure TabcTrayIcon.ContextPopup(MousePos: TPoint);
var
  Handled: Boolean;
begin
  Handled := False;
  if Assigned(FOnContextPopup) then FOnContextPopup(Self, MousePos, Handled);
  if Handled then Exit;
  if (FPopupMenu <> nil) and FPopupMenu.AutoPopup then
  begin
    FPopupMenu.PopupComponent := Self;
    FPopupMenu.Popup(MousePos.X, MousePos.Y);
  end;
end;

procedure TabcTrayIcon.ShowForm;
begin
  OwnerForm.Show;
  SetForegroundWindow(OwnerHandle);
  ShowWindow(OwnerHandle, SW_RESTORE);
  if [atiHideApp, atiHideOnMinimize] * Options <> [] then
    if Application.MainForm = OwnerForm then
      ShowWindow(Application.Handle, SW_RESTORE);
end;

procedure TabcTrayIcon.Click(Button: TMouseButton);
begin
  case Button of
    mbLeft:
      if atiShowOnClick in Options then
        ShowForm;
    mbRight:
      ContextPopup(FCursorPos);
  end;
  if Assigned(FOnClick) then FOnClick(Self);
end;

procedure TabcTrayIcon.DblClick(Button: TMouseButton);
begin
  if Button = mbLeft then
    if atiShowOnDblClick in Options then
      ShowForm
    else if atiMenuDefaultOnDblClick in Options then
      DoMenuDefault;
  if Assigned(FOnDblClick) then FOnDblClick(Self);
end;

procedure TabcTrayIcon.DoMenuDefault;
var
  i: integer;
begin
  if (FPopupMenu <> nil) then
    with FPopupMenu.Items do
      for i := 0 to Count - 1 do
        if Items[i].Default then
          Items[i].Click;
end;

procedure TabcTrayIcon.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if Assigned(FOnMouseDown) then FOnMouseDown(Self, Button, Shift, X, Y);
end;

procedure TabcTrayIcon.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(FOnMouseMove) then FOnMouseMove(Self, Shift, X, Y);
end;

procedure TabcTrayIcon.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  if Assigned(FOnMouseUp) then FOnMouseUp(Self, Button, Shift, X, Y);
end;

procedure TabcTrayIcon.IconChanged(Sender: TObject);
begin
  if FTrayAdded then
  begin
    FIconData.hIcon := GetIconHandle;
    Shell_NotifyIcon(NIM_MODIFY, @FIconData);
  end;
end;

function TabcTrayIcon.GetIconHandle: HICON;
begin
  Result := FIcon.Handle;
  if Result = 0 then
  begin
    Result := OwnerForm.Icon.Handle;
    if Result = 0 then
    begin
      Result := Application.Icon.Handle;
      if Result = 0 then
        Result := LoadIcon(0, IDI_APPLICATION);
    end;
  end;
end;

function TabcTrayIcon.IsIconStored: Boolean;
begin
  Result := (Images = nil) and not FIcon.Empty;
end;

procedure TabcTrayIcon.SetHint(const Value: string);
begin
  FHint := Value;
  if (Length(Value) > 63) then
    FHint := Copy(Value, 1, 63)
  else
    FHint := Value;
  if FTrayAdded then
  begin
    StrLCopy(FIconData.szTip, PChar(FHint), 63);
    Shell_NotifyIcon(NIM_MODIFY, @FIconData);
  end;
end;

procedure TabcTrayIcon.SetIcon(const Value: TIcon);
begin
  FIcon.Assign(Value);
end;

procedure TabcTrayIcon.SetOptions(const Value: TabcTrayIconOptions);
// these pairs are mutually exclusive:
// [atiShowOnClick, atiShowOnDblClick] and [atiShowOnDblClick, atiMenuDefaultOnDblClick]
// App Termination - overrides WM_QUERYENDSESSION and WM_ENDSESSION termination
//   - if atiTerminateOnLogoff is false, atiWin95Service must also be true
//   - if atiWin95Service is false, atiTerminateOnLogoff must be true
var
  NewValue: TabcTrayIconOptions;
begin
  NewValue := Value;
  {remove mutually exclusive values}
  if [atiShowOnClick, atiShowOnDblClick] * NewValue = [atiShowOnClick, atiShowOnDblClick] then
    if atiShowOnDblClick in Options then
      NewValue := NewValue - [atiShowOnDblClick]
    else
      NewValue := NewValue - [atiShowOnClick];

  if [atiShowOnDblClick, atiMenuDefaultOnDblClick] * NewValue = [atiShowOnDblClick, atiMenuDefaultOnDblClick] then
    if atiShowOnDblClick in Options then
      NewValue := NewValue - [atiShowOnDblClick]
    else
      NewValue := NewValue - [atiMenuDefaultOnDblClick];

  if (atiWin95Service in Options) and not (atiWin95Service in NewValue) then
      NewValue := NewValue + [atiTerminateOnLogoff];
  if not (atiTerminateOnLogoff in NewValue) then
      NewValue := NewValue + [atiWin95Service];

  FOptions := NewValue;
  if FTrayAdded then
    FMonitor.Enabled := (atiMonitorTray in NewValue);
end;

procedure TabcTrayIcon.Loaded;
begin
  inherited;
  if (csDesigning in ComponentState) then Exit;
  SetVisible(FVisible);
  if atiHideApp in Options then
  begin
    OwnerForm.Visible := False;
    Application.ShowMainForm := False;
    ShowWindow(Application.Handle, SW_HIDE);
  end;
  if atiWin95Service in Options then
  begin
    FServiceRegistered := True;
    abcRegWin95ServiceProcess(True);
  end;
end;

function TabcTrayIcon.GetMonitorInterval: Cardinal;
begin
  Result := FMonitor.Interval;
end;

procedure TabcTrayIcon.SetMonitorInterval(Value: Cardinal);
begin
  FMonitor.Interval := Value;
end;

procedure TabcTrayIcon.SetVisible(const Value: Boolean);
begin
  FVisible := Value;
  if [csLoading, csDesigning] * ComponentState = [] then
    if Value then
      AddIcon
    else
      DeleteIcon;
end;

procedure TabcTrayIcon.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) then
    if (AComponent = FPopupMenu) then
      FPopupMenu := nil
    else if (AComponent = FImages) then
      Images := nil;
end;

procedure TabcTrayIcon.UpdateCursorPos;
begin
  GetCursorPos(FCursorPos);
end;

function IsIE4orHigherInstalled: Boolean;
var
  RegKey: HKEY;
begin
  Result := False;
  if RegOpenKey(
    HKEY_LOCAL_MACHINE,
    'SOFTWARE\Microsoft\Internet Explorer',
    RegKey
    ) = ERROR_SUCCESS then
  try
    Result := RegQueryValueEx(RegKey, 'Version', nil, nil, nil, nil) = ERROR_SUCCESS;
  finally
    RegCloseKey(RegKey);
  end;
end;

procedure TabcTrayIcon.AnimateIcon(Sender: TObject);
begin
  if ImageIndex < Images.Count - 1 then
    ImageIndex := ImageIndex + 1
  else
    ImageIndex := 0;
end;

function TabcTrayIcon.GetAnimateInterval: Cardinal;
begin
  Result := FAnimateTimer.Interval;
end;

procedure TabcTrayIcon.SetAnimated(Value: Boolean);
begin
  if Value <> FAnimated then
  begin
    FAnimated := Value;
    FAnimateTimer.Enabled :=
      Value and Assigned(Images) and not (csDesigning in ComponentState);
  end;
end;

procedure TabcTrayIcon.SetAnimateInterval(Value: Cardinal);
begin
  FAnimateTimer.Interval := Value;
end;

procedure TabcTrayIcon.SetImageIndex(Value: TImageIndex);
begin
  if Value <> FImageIndex then
  begin
    FImageIndex := Value;
    if Assigned(Images) then
      if (Value >= 0) and (Value < Images.Count) then
        Images.GetIcon(Value, Icon);
  end;
end;

procedure TabcTrayIcon.SetImages(Value: TImageList);
begin
  if Value <> FImages then
  begin
    FImages := Value;
    if Value = nil then
      Animated := False
    else
    begin
      if ImageIndex >= Value.Count then
        ImageIndex := -1
      else if (ImageIndex >= 0) then
        Value.GetIcon(ImageIndex, Icon);
      FAnimateTimer.Enabled :=
        Animated and not (csDesigning in ComponentState);
    end;
  end;
end;

initialization
  mIE4Installed := IsIE4orHigherInstalled;
  mIsNT351 := (Win32MajorVersion <= 3) and (Win32Platform = VER_PLATFORM_WIN32_NT);

  if mIE4Installed then
    mTaskBarCreated := RegisterWindowMessage('TaskbarCreated');

end.
