unit abcufx;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{
NOTE:
  These transition effects assume the clip region is the same size or larger
  than the bitmap to be drawn.
  If this is not true, the caller should scale the bitmap before drawing.
  For performance, adjustment of drawing to clip regions is not implemented.
}

interface

uses Windows, Messages, Graphics, Classes, abcbtn;

type
  TabcTransitionEffect = (
    {1. Patterns}
      {Skip}
    atfSkipHorizontal,
    atfSkipVertical,
    atfSkipRectangles,
    atfSkipDiagonals,
    atfSkipDiagonalSquares,
      {Box}
    atfBoxIn,
    atfBoxOut,
      {Blinds}
    atfBlindsHorz,
    atfBlindsVert,
      {Dissolve}
    atfDissolve,
    atfDissolveHorz,
    atfDissolveVert,
      {Wind}
    atfWindIn,
    atfWindOut,
    {2. Special Effects}
      {Pour}
    atfPourDown,
    atfPourLeft,
    atfPourRight,
    atfPourUp,
      {Quad}
    atfQuadCover,
    atfQuadUncover,
    atfQuadWipeIn,
    atfQuadWipeOut,
    atfQuadUnfold,
    atfQuadFold,
    atfQuadFoldCopy,
    atfQuadExpand,
    atfQuadShrink,
    atfQuadShrinkCopy,
      {Shapes}
    atfShapeSawtooth,
    atfShapeStarBurst,
    atfShapeBroadStar,
    atfShapeParallelogram,
    atfShapeLens1,
    atfShapeLens2,
    atfShapeSpiralClockwise,
    atfShapeSpiralAntiClockWise,
    {3. Resize}
      {Fold}
    atfFoldDown,
    atfFoldLeft,
    atfFoldRight,
    atfFoldUp,
    atfFoldCenterHorz,
    atfFoldCenterVert,
      {Unfold}
    atfUnfoldDown,
    atfUnfoldLeft,
    atfUnfoldRight,
    atfUnfoldUp,
    atfUnfoldCenterHorz,
    atfUnfoldCenterVert,
      {Squash}
    atfSquashDown,
    atfSquashLeft,
    atfSquashRight,
    atfSquashUp,
    atfSquashCenterHorz,
    atfSquashCenterVert,
      {UnSquash}
    atfUnSquashDown,
    atfUnSquashLeft,
    atfUnSquashRight,
    atfUnSquashUp,
    atfUnSquashCenterHorz,
    atfUnSquashCenterVert,
      {Shrink}
    atfShrinkLeftDown,
    atfShrinkLeftUp,
    atfShrinkRightDown,
    atfShrinkRightUp,
    atfShrinkCenter,
      {Expand}
    atfExpandLeftDown,
    atfExpandLeftUp,
    atfExpandRightDown,
    atfExpandRightUp,
    atfExpandCenter,
      {Zoom}
    atfZoomIn,
    atfZoomOut,
    atfZoomDown,
    atfZoomLeft,
    atfZoomRight,
    atfZoomUp,
    atfZoomLeftDown,
    atfZoomLeftUp,
    atfZoomRightDown,
    atfZoomRightUp,
    {Move}
      {Push}
    atfPushDown,
    atfPushLeft,
    atfPushRight,
    atfPushUp,
      {Cover}
    atfCoverDown,
    atfCoverLeft,
    atfCoverRight,
    atfCoverUp,
    atfCoverLeftDown,
    atfCoverLeftUp,
    atfCoverRightDown,
    atfCoverRightUp,
    atfCoverCenterHorzIn,
    atfCoverCenterHorzOut,
    atfCoverCenterVertIn,
    atfCoverCenterVertOut,
      {Uncover}
    atfUncoverDown,
    atfUncoverLeft,
    atfUncoverRight,
    atfUncoverUp,
    atfUncoverLeftDown,
    atfUncoverLeftUp,
    atfUncoverRightDown,
    atfUncoverRightUp,
    atfUncoverCenterHorzIn,
    atfUncoverCenterHorzOut,
    atfUncoverCenterVertIn,
    atfUncoverCenterVertOut,
    {4. Wipe}
      {1D}
    atfWipeDown,
    atfWipeLeft,
    atfWipeRight,
    atfWipeUp,
      {Diagonal}
    atfWipeLeftDown,
    atfWipeLeftUp,
    atfWipeRightDown,
    atfWipeRightUp,
      {2D}
    atfWipeRectLeftDown,
    atfWipeRectLeftUp,
    atfWipeRectRightDown,
    atfWipeRectRightUp,
      {2D Reverse}
    atfCropLeftDown,
    atfCropLeftUp,
    atfCropRightDown,
    atfCropRightUp,
      {Center}
    atfWipeCenterHorzIn,
    atfWipeCenterHorzOut,
    atfWipeCenterVertIn,
    atfWipeCenterVertOut,
      {Clock}
    atfWipeClockwise,
    atfWipeAntiClockwise,
    {5. Other}
      {Plain}
    atfNone,
    atfCut,
    atfCutThroughBlack,
    atfFadeThroughBlack,
      {Selection}
    atfRandom,
    atfSelectionRandom,
    atfSelectionSequence
    );

  TabcTransitionThread = class(TThread)
  private
    FCancelled: Boolean;

    FDestCopy, FBuffer: TBitmap;

    FDestCopyDC, FBufferDC: HDC;

    i, j, lastpos: integer;
    id: integer;
    nFrames, xFrames, yFrames: integer;

    xl, yl, halfsize, minsize, xr, yr,
    inwidth, inheight, outwidth, outheight: Integer;
    stilldrawing: Boolean;

    x1, x2, y1, y2, dx, dy: Integer;

    x, y, w, h: Integer;

    lxtimes, lytimes: integer;

    xMin, xMax, yMin, yMax, xDraw, yDraw: Integer;

    mid: Integer;

    halfWidth, halfHeight{, i}: Integer;
    ptPolyMask: array[0..3] of TPoint;

    FadeBrushes: array[1..4] of HBrush;

    midx, midy: Integer;

    FSaveStretchMode: Integer;

    t: Integer;

    drawx, drawy: Integer;

    CeilHeight, CeilWidth, ndiags: Integer;

    ptDestOffset: TPoint;

    halfW, halfH: integer;
    xs, ys, td, tt, minx, miny, maxx, maxy, scalex, scaley: double;

    FSpiralPoints: TList;

    procedure CopyDest;
    function ClockPoint(Index: Integer): TPoint;
    procedure InitSpiralPoints(nPoints: Integer);
    procedure FreeSpiralPoints;
    function SpiralPoint(Frame: Integer): TPoint;
    function CreateFadeBrush(Darkness: Integer): hBrush;
    procedure InitBuffer;
    procedure CopySourceToBuffer;
    procedure CopyBufferToDest;
    procedure CopyDestToBuffer;
    procedure DrawCopyToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
    procedure DrawLockedToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
    procedure DrawOffsetToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
    procedure DrawCopyLocked(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
    procedure DrawCopyOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
    procedure StretchCopyOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
    procedure StretchCopyToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
    procedure StretchOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
    procedure StretchToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
    procedure BrushFadeOut(i: Integer);
    procedure BrushFadeIn(i: Integer);
    procedure SetDestStretchMode;
    procedure RestoreDestStretchMode;
    procedure DoCancelEvent;
  protected
    FSource, FDest: TCanvas;
    FDestX, FDestY: Integer;
    FWidth, FHeight: Integer;
    FSourceX, FSourceY: Integer;
    FThickness, FDuration, FPauseAfter: Integer;
    FEffect: TabcTransitionEffect;
    FOnCancel: TabcCancelEvent;
    FFrameDuration, FStartNextFrame: DWORD;
    FSingleFrameMode: Boolean;
    function TryLockSourceDest: Boolean;
    procedure UnlockSourceDest;
    function CheckCancel: Boolean;
    procedure SleepUntilNextFrame;
    procedure DoDraw(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
    procedure DrawOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
    procedure InitializeEffect; virtual;
    procedure DrawFrame; virtual;
    procedure DrawFrames; virtual;
    procedure Execute; override;
    function StillDrawingFrames: Boolean; virtual;
  public
    constructor Create(
      const aSource, aDest: TCanvas;
      aDestX, aDestY, aWidth, aHeight: Integer;
      aSourceX, aSourceY: Integer;
      aEffect: TabcTransitionEffect;
      aThickness, aDuration, aPauseAfter: Integer;
      aOnCancel: TabcCancelEvent;
      aOnTerminate: TNotifyEvent;
      aSuspended: Boolean;
      aSingleFrameMode: Boolean
      );
    destructor Destroy; override;
  end;

  TabcTransitionThreadClass = class of TabcTransitionThread;

function abcThreadTransition (
           Effect: TabcTransitionEffect;
           const EffectSelection: TList;
           var EffectSelectionIndex: Integer;
           const Source, Dest: TCanvas;
           DestX, DestY, Width, Height: Integer;
           SourceX, SourceY: Integer;
           Thickness, Duration, PauseAfter: Integer;
           OnCancel: TabcCancelEvent;
           OnTerminate: TNotifyEvent;
           Suspended: Boolean;
           SingleFrameMode: Boolean
         ): TabcTransitionThread;

function abcSizedEffect(Effect: TabcTransitionEffect): Boolean;

procedure abcInitializeDissolve;

implementation

uses abcutilm;

const
  RANDARRAYSIZE = 128;
  ZOOM_SCALE_FACTOR = 2; {decrease for finer grained zoom}
  ZOOM_SPEED_FACTOR = 8; {increase for faster drawing (less steps)}
  ZOOM_OUT_FRAMES = 64;

  SAWTOOTH_FRAMES = 240;
  SAWTOOTH_FRAME_STEPS = 20;

  SPIRAL_FRAMES = 100;  {nb try parallelogram with 500 points - another nice effect}
  SPIRAL_A = 1.0;
  SPIRAL_DD = 0;

  SPIRAL_TD:
    array[atfShapeSawtooth..atfShapeSpiralAntiClockWise] of double
      = (0.15, 3.0, 4.0, 4.7, 0.7, 5.0, 0.3, 6.0);

type
  TabcSpiralPoint = array[0..1] of Double;
  PabcSpiralPoint = ^TabcSpiralPoint;

type
  TabcRandPoints = class
  public
    Rands: array[0..RANDARRAYSIZE-1] of Byte;  {single dimension random series}
    RandXs: array[0..RANDARRAYSIZE-1, 0..RANDARRAYSIZE-1] of Byte;  {together these two arrays give a series}
    RandYs: array[0..RANDARRAYSIZE-1, 0..RANDARRAYSIZE-1] of Byte;  {of random points in 128x128 square}
    constructor Create;
  end;

constructor TabcRandPoints.Create;

  procedure FreePtList(L: TList);
  var
    i: integer;
    pp: PPoint;
  begin
    with L do
    begin
      for i := Count - 1 downto 0 do
      begin
        pp := PPoint(Items[i]);
        Dispose(pp);
      end;
      Free;
    end;
  end;

var
  px, py, i, j: Integer;
  ptList, ptList2: TList;
  ppt, pp: PPoint;
begin
  inherited Create;
  //init random points
  Randomize;
  ptList := TList.Create;
  ptList2 := TList.Create;
  try
    {make a list of points}
    for px := 0 to RANDARRAYSIZE-1 do
    begin
      for py := 0 to RANDARRAYSIZE-1 do
      begin
        New(ppt);
        ppt^.x := px;
        ppt^.y := py;
        ptList.Add(ppt);
      end;
      New(pp);
      pp^.x := px;
      pp^.y := 0;
      ptList2.Add(pp);
    end;
    {sort randomly into target arrays}
    for px := 0 to RANDARRAYSIZE-1 do
    begin
      for py := 0 to RANDARRAYSIZE-1 do
      begin
        i := Random(ptList.Count - 1);
        ppt := PPoint(ptList.Items[i]);
        RandXs[px, py] := ppt^.x;
        RandYs[px, py] := ppt^.y;
        Dispose(ppt);
        ptList.Delete(i);
      end;
      j := Random(ptList2.Count - 1);
      pp := PPoint(ptList2.Items[j]);
      Rands[px] := pp^.x;
      Dispose(pp);
      ptList2.Delete(j);
    end;
  finally
    FreePtList(ptList);
    FreePtList(ptList2);
  end;
end;

var
  RandPoints: TabcRandPoints;

procedure abcInitializeDissolve;
begin
  if RandPoints = nil then
    RandPoints := TabcRandPoints.Create;
end;

{ TabcTransitionThread }

constructor TabcTransitionThread.Create(
  const aSource, aDest: TCanvas;
  aDestX, aDestY, aWidth, aHeight: Integer;
  aSourceX, aSourceY: Integer;
  aEffect: TabcTransitionEffect;
  aThickness, aDuration, aPauseAfter: Integer;
  aOnCancel: TabcCancelEvent;
  aOnTerminate: TNotifyEvent;
  aSuspended: Boolean;
  aSingleFrameMode: Boolean
  );
begin
  FSource := aSource;
  FDest := aDest;
  FDestX := aDestX;
  FDestY := aDestY;
  FWidth := aWidth;
  FHeight := aHeight;
  FSourceX := aSourceX;
  FSourceY := aSourceY;
  FEffect := aEffect;
  FThickness := aThickness;
  FDuration := aDuration;
  if aPauseAfter > 1 then
    FPauseAfter := aPauseAfter
  else
    FPauseAfter := 1;
  FOnCancel := aOnCancel;
  OnTerminate := aOnTerminate;
//  FreeOnTerminate := True; fails in D6 with Waitfor??  seems ok now, test again with sprites
  FreeOnTerminate := True;
  FSingleFrameMode := aSingleFrameMode;
  inherited Create(True);
  InitializeEffect;
  if not aSuspended then
    Resume;
end;

procedure TabcTransitionThread.DoCancelEvent;
begin
  FOnCancel(Self, FCancelled);
end;

function TabcTransitionThread.CheckCancel: Boolean;
begin
  Result := False;
  if Assigned(FOnCancel) then
  begin
    FCancelled := False;
    if not Terminated then
      Synchronize(DoCancelEvent);
    Result := FCancelled;
    if Result then Terminate;
  end;
end;

procedure TabcTransitionThread.DoDraw(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
begin
  if not Terminated then
    BitBlt (FDest.Handle, dstX, dstY, dstWidth, dstHeight,
            FSource.Handle, srcX, srcY,
            SRCCOPY);
end;

procedure TabcTransitionThread.DrawOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
begin
  DoDraw(FDestX+dstX, FDestY+dstY, dstWidth, dstHeight, FSourceX+srcX, FSourceY+srcY);
end;

function TabcTransitionThread.TryLockSourceDest: Boolean;
begin
  Result := (not Terminated) and FDest.TryLock;
  if Result then
  try
    Result := (not Terminated) and FSource.TryLock;
  finally
    if not Result then FDest.Unlock;
  end;
end;

procedure TabcTransitionThread.UnlockSourceDest;
begin
  FSource.Unlock;
  FDest.Unlock;
end;

procedure TabcTransitionThread.SleepUntilNextFrame;
var
  sleeptime: DWORD;
begin
  if CheckCancel or Terminated then
    Exit;
  sleeptime := GetTickCount;
  if sleeptime < FStartNextFrame then
    Sleep(FStartNextFrame - sleeptime)
  else
    Sleep(0);  
  FStartNextFrame := FStartNextFrame + FFrameDuration;
end;

procedure TabcTransitionThread.Execute;
begin
  DrawFrames;
  if not Terminated then
  begin
    if TryLockSourceDest then
    try
      DrawOffset(0, 0, FWidth, FHeight, 0, 0);
    finally
      UnlockSourceDest;
    end;
    GDIFlush;
    if not Terminated then
    begin
      Sleep(FPauseAfter);
      Terminate;
    end;
  end
  else
    GDIFlush;
end;

destructor TabcTransitionThread.Destroy;
begin
  if FDestCopy <> nil then
  begin
    if FDestCopyDC <> 0 then
    begin
      FDestCopyDC := 0;
      FDestCopy.Canvas.Unlock;
    end;
    FDestCopy.Free;
    FDestCopy := nil;
  end;
  if FBuffer <> nil then
  begin
    if FBufferDC <> 0 then
    begin
      FBufferDC := 0;
      FBuffer.Canvas.Unlock;
    end;
    FBuffer.Free;
    FBuffer := nil;
  end;
  if FadeBrushes[1] <> 0 then
  begin
    DeleteObject(FadeBrushes[1]);
    DeleteObject(FadeBrushes[2]);
    DeleteObject(FadeBrushes[3]);
    DeleteObject(FadeBrushes[4]);
  end;
  FreeSpiralPoints;
  inherited;
end;

procedure TabcTransitionThread.CopyDest;
//initializeEffect
begin
  if Terminated then Exit;
  if FDestCopy = nil then
    FDestCopy := TBitmap.Create
  else
  begin
    FDestCopyDC := 0;
    FDestCopy.Canvas.Unlock;
  end;
  if Terminated then Exit;
  with FDestCopy do
  begin
    Width := FWidth;
    Height := FHeight;
    Canvas.Lock;
    FDestCopyDC := Canvas.Handle;
    if Terminated then Exit;
    FDest.Lock;
    try
      BitBlt(FDestCopyDC, 0, 0, FWidth, FHeight, FDest.Handle, FDestX, FDestY, SRCCOPY);
    finally
      FDest.Unlock;
    end;
  end;
end;

procedure TabcTransitionThread.InitBuffer;
//initializeEffect
begin
  if Terminated then Exit;
  if FBuffer = nil then
    FBuffer := TBitmap.Create
  else
  begin
    FBufferDC := 0;
    FBuffer.Canvas.Unlock;
  end;
  if Terminated then Exit;
  with FBuffer do
  begin
    Width := FWidth;
    Height := FHeight;
    FBuffer.Canvas.Lock;
    FBufferDC := FBuffer.Canvas.Handle;
  end;
  SetStretchBltMode(FBufferDC, COLORONCOLOR);
end;

procedure TabcTransitionThread.CopySourceToBuffer;
begin
  if not Terminated then
    BitBlt(FBufferDC, 0, 0, FWidth, FHeight, FSource.Handle, FSourceX, FSourceY, SRCCOPY);
end;

procedure TabcTransitionThread.CopyBufferToDest;
begin
  if not Terminated then
    BitBlt(FDest.Handle, FDestX, FDestY, FWidth, FHeight, FBufferDC, 0, 0, SRCCOPY);
end;

procedure TabcTransitionThread.CopyDestToBuffer;
//initializeEffect
begin
  if Terminated then Exit;
  FDest.Lock;
  try
    BitBlt(FBufferDC, 0, 0, FWidth, FHeight, FDest.Handle, FDestX, FDestY, SRCCOPY);
  finally
    FDest.Unlock;
  end;
end;

procedure TabcTransitionThread.DrawCopyToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
begin
  if not Terminated then
    BitBlt (FBufferDC, dstX, dstY, dstWidth, dstHeight,
            FDestCopyDC, srcX, srcY,
            SRCCOPY);
end;

procedure TabcTransitionThread.DrawLockedToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
begin
  if not Terminated then
    BitBlt (FBufferDC, dstX, dstY, dstWidth, dstHeight,
            FSource.Handle, srcX, srcY,
            SRCCOPY);
end;

procedure TabcTransitionThread.DrawOffsetToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
begin
  if not Terminated then
    DrawLockedToBuffer(dstX, dstY, dstWidth, dstHeight, FSourceX + srcX, FSourceY + srcY);
end;

procedure TabcTransitionThread.DrawCopyLocked(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
begin
  if not Terminated then
    BitBlt (FDest.Handle, dstX, dstY, dstWidth, dstHeight,
            FDestCopyDC, srcX, srcY,
            SRCCOPY);
end;

procedure TabcTransitionThread.DrawCopyOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY: Integer);
begin
  if Terminated then Exit;
  DrawCopyLocked(FDestX+dstX, FDestY+dstY, dstWidth, dstHeight, srcX, srcY);
end;

procedure TabcTransitionThread.StretchCopyOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
begin
  if not Terminated then
    StretchBlt (FDest.Handle, FDestX + dstX, FDestY + dstY, dstWidth, dstHeight,
            FDestCopyDC, srcX, srcY, srcWidth, srcHeight,
            SRCCOPY);
end;

procedure TabcTransitionThread.StretchCopyToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
begin
  if not Terminated then
    StretchBlt (FBufferDC, dstX, dstY, dstWidth, dstHeight,
            FDestCopyDC, srcX, srcY, srcWidth, srcHeight,
            SRCCOPY);
end;

procedure TabcTransitionThread.StretchOffset(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
begin
  if not Terminated then
    StretchBlt (FDest.Handle, FDestX + dstX, FDestY + dstY, dstWidth, dstHeight,
            FSource.Handle, FSourceX + srcX, FSourceY + srcY, srcWidth, srcHeight,
            SRCCOPY);
end;

procedure TabcTransitionThread.StretchToBuffer(dstX, dstY, dstWidth, dstHeight, srcX, srcY, srcWidth, srcHeight: Integer);
begin
  if not Terminated then
    StretchBlt (FBufferDC, dstX, dstY, dstWidth, dstHeight,
            FSource.Handle, FSourceX + srcX, FSourceY + srcY, srcWidth, srcHeight,
            SRCCOPY);
end;

procedure TabcTransitionThread.SetDestStretchMode;
begin
  if not Terminated then
    FSaveStretchMode := SetStretchBltMode(FDest.Handle, COLORONCOLOR);
end;

procedure TabcTransitionThread.RestoreDestStretchMode;
begin
  if not Terminated then
    SetStretchBltMode(FDest.Handle, FSaveStretchMode);
end;

procedure TabcTransitionThread.InitializeEffect;
var
  m, d: Integer;
begin
  if Terminated then Exit;
  case FEffect of
    atfNone: ;
    atfSkipHorizontal:
    begin
      m := FWidth mod FThickness;
      d := FWidth div FThickness;
      if (m > 0) then
        if Odd(d) then
            j := d * FThickness
        else
                j := (d - 1) * FThickness
      else
        if Odd(d) then
                j := FWidth - (FThickness shl 1)
        else
                j := FWidth - FThickness;
      i := 0;
      FFrameDuration := FDuration * 2 div d;
    end;

    atfSkipVertical:
    begin
      m := FHeight mod FThickness;
      d := FHeight div FThickness;
      if (m > 0) then
        if Odd(d) then
            j := d * FThickness
        else
                j := (d - 1) * FThickness
      else
        if Odd(d) then
                j := FHeight - (FThickness shl 1)
        else
                j := FHeight - FThickness;
      i := 0;
      FFrameDuration := FDuration * 2 div d;
    end;

    atfSkipDiagonals:
    begin
      id := 0;
      for m := 0 to 3 do
      begin
        ptPolyMask[m].x := 0;
        ptPolyMask[m].y := 0;
      end;
      CeilWidth := abcCeil(FWidth / FThickness);
      CeilHeight := abcCeil(FHeight / FThickness);
      ndiags := CeilWidth + CeilHeight;
      CeilWidth := CeilWidth * FThickness;
      CeilHeight := CeilHeight * FThickness;
      FFrameDuration := abcCeil(FDuration / ndiags);
    end;

    atfSkipDiagonalSquares:
    begin
      xFrames :=  abcCeil(FWidth / FThickness);
      yFrames :=  abcCeil(FHeight / FThickness);
      nFrames := xFrames + yFrames;
      FFrameDuration := FDuration div nFrames;
      id := 0;
    end;

    atfSkipRectangles:
    begin
      xl := 0;
      yl := 0;
      if FWidth > FHeight then
        minsize := FHeight
      else
        minsize := FWidth;
      if Odd(minsize) then
        xr := minsize shr 1
      else
        xr := minsize shr 1 + 1;

      if not Odd(xr) then xr := xr + 1;

      yr := xr;

      halfsize := xr;

      if FHeight < FWidth then
        FFrameDuration := FDuration * 4 div FHeight
      else
        FFrameDuration := FDuration * 4 div FWidth;
    end;

    atfBoxIn:
    begin
      if FWidth > FHeight then
      begin
        dx := FThickness * FWidth div FHeight;
        dy := FThickness;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        dx := FThickness;
        dy := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
      x1 := dx;
      x2 := FWidth - dx;
      y1 := dy;
      y2 := FHeight - dy;
    end;

    atfBoxOut:
    begin
      if FWidth > FHeight then
      begin
        h := FThickness;
        w := h * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        w := FThickness;
        h := w * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;

      x := (FWidth - w) div 2;
      y := (FHeight - h) div 2;

    end;

    atfBlindsHorz, atfBlindsVert:
    begin
      i := 0;
      FFrameDuration := FDuration div FThickness;
    end;

    atfDissolve, atfDissolveHorz, atfDissolveVert:
    begin
      abcInitializeDissolve;
      FFrameDuration := FDuration div RANDARRAYSIZE;
      lxtimes := abcCeil(FWidth / RANDARRAYSIZE);
      lytimes := abcCeil(FHeight / RANDARRAYSIZE);
      i := 0;
    end;

    atfWindIn, atfWindOut:
    begin
      if FWidth > FHeight then
      begin
        dX := FThickness * FWidth div FHeight;
        dY := FThickness;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        dX := FThickness;
        dY := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;

      if FEffect = atfWindIn then
      begin
        xMin := -dX;
        xMax := FWidth;
        yMin := 0;
        yMax := FHeight;
      end
      else
      begin
        xMin := (FWidth - dX) div 2;
        xMax := xMin + dX;
        yMin := (FHeight - dY) div 2;
        yMax := yMin + dY;
      end;
    end;

    atfPourDown:
    begin
      FFrameDuration := FDuration * FThickness div FHeight;
      i := FHeight - FThickness;
    end;

    atfPourLeft:
    begin
      FFrameDuration := FDuration * FThickness div FWidth;
      i := 0;
      lastpos := FWidth - FThickness;
    end;

    atfPourRight:
    begin
      FFrameDuration := FDuration * FThickness div FWidth;
      i := FWidth - FThickness;
    end;

    atfPourUp:
    begin
      FFrameDuration := FDuration * FThickness div FHeight;
      i := 0;
      lastpos := FHeight - FThickness;
    end;

    atfQuadCover:
    begin
      InitBuffer;
      CopyDestToBuffer;
      midx := FWidth div 2;
      midy := FHeight div 2;
      if FWidth > FHeight then
      begin
        y := FThickness;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        x := FThickness;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
    end;

    atfQuadUncover:
    begin
      CopyDest;
      InitBuffer;
      midx := FWidth div 2;
      midy := FHeight div 2;
      if FWidth > FHeight then
      begin
        y := midy - FThickness div 2;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        x := midx - FThickness div 2;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
    end;

    atfQuadWipeIn:
    begin
      InitBuffer;
      CopyDestToBuffer;
      midx := FWidth div 2;
      if FWidth > FHeight then
      begin
        y := FThickness;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        x := FThickness;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
    end;

    atfQuadWipeOut:
    begin
      CopyDest;
      InitBuffer;
      if FWidth > FHeight then
      begin
        y := (FWidth - FThickness) div 2;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        x := (FHeight - FThickness) div 2;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
    end;

    atfQuadUnfold:
    begin
      InitBuffer;
      CopyDestToBuffer;
      midx := FWidth div 2;
      midy := FHeight div 2;
      if FWidth > FHeight then
      begin
        y := FThickness;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        x := FThickness;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
    end;

    atfQuadFold, atfQuadFoldCopy:
    begin
      CopyDest;
      InitBuffer;
      midx := FWidth div 2;
      midy := FHeight div 2;
      if FWidth > FHeight then
      begin
        y := midy - FThickness div 2;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        x := midx - FThickness div 2;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
    end;

    atfQuadExpand:
    begin
      InitBuffer;
      CopyDestToBuffer;
      midx := FWidth div 2;
      midy := FHeight div 2;

      if FWidth > FHeight then
      begin
        y := FThickness * 2;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 4 div FHeight;
        drawy := FHeight div 4 - FThickness;
        drawx := drawy * FWidth div FHeight;
      end
      else
      begin
        x := FThickness * 2;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 4 div FWidth;
        drawx := FWidth div 4 - FThickness;
        drawy := drawx * FHeight div FWidth;
      end;
    end;

    atfQuadShrink, atfQuadShrinkCopy:
    begin
      CopyDest;
      InitBuffer;
      midx := FWidth div 2;
      midy := FHeight div 2;
      if FWidth > FHeight then
      begin
        y := midy - FThickness;
        x := y * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness * 4 div FHeight;
        drawy := FThickness;
        drawx := drawy * FWidth div FHeight;
      end
      else
      begin
        x := midx - FThickness;
        y := x * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 4 div FWidth;
        drawx := FThickness;
        drawy := drawx * FHeight div FWidth;
      end;
    end;

    atfShapeSawTooth:
    begin
      td := SPIRAL_TD[FEffect];
      InitSpiralPoints(SAWTOOTH_FRAMES * SAWTOOTH_FRAME_STEPS);
      FFrameDuration := FDuration div SAWTOOTH_FRAMES;
      i := 0;
      j := 0;
    end;

    atfShapeStarBurst..atfShapeSpiralAntiClockwise:
    begin
      td := SPIRAL_TD[FEffect];
      InitSpiralPoints(SPIRAL_FRAMES);
      FFrameDuration := FDuration div SPIRAL_FRAMES;
      ptPolyMask[0].x := halfW;
      ptPolyMask[0].y := halfH;
      ptPolyMask[2].x := halfW;
      ptPolyMask[2].y := halfH;
      i := 0;
    end;

    atfFoldDown:
    begin
      CopyDest;
      i := 0;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfFoldLeft:
    begin
      CopyDest;
      i := FThickness;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfFoldRight:
    begin
      CopyDest;
      i := 0;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfFoldUp:
    begin
      CopyDest;
      i := FThickness;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfFoldCenterHorz:
    begin
      CopyDest;
      InitBuffer;
      mid := FWidth div 2;
      x1 := FThickness;
      x2 := FWidth - FThickness;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
    end;

    atfFoldCenterVert:
    begin
      CopyDest;
      InitBuffer;
      mid := FHeight div 2;
      y1 := FThickness;
      y2 := FHeight - FThickness;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
    end;

    atfUnfoldDown:
    begin
      i := 0;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfUnfoldLeft:
    begin
      i := FThickness;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfUnfoldRight:
    begin
      i := 0;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfUnfoldUp:
    begin
      i := FThickness;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfUnfoldCenterHorz:
    begin
      CopyDest;
      InitBuffer;
      mid := FWidth div 2;
      x1 := mid - FThickness div 2;
      x2 := mid + FThickness div 2;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
    end;

    atfUnfoldCenterVert:
    begin
      CopyDest;
      InitBuffer;
      mid := FHeight div 2;
      y1 := mid - FThickness div 2;
      y2 := mid + FThickness div 2;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
    end;

    atfSquashDown:
    begin
      CopyDest;
      t := 0;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfSquashLeft:
    begin
      CopyDest;
      t := FThickness;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfSquashRight:
    begin
      CopyDest;
      t := 0;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfSquashUp:
    begin
      CopyDest;
      t := FThickness;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfSquashCenterHorz:
    begin
      CopyDest;
      InitBuffer;
      mid := FWidth div 2;
      x1 := FThickness;
      x2 := FWidth - FThickness;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
    end;

    atfSquashCenterVert:
    begin
      CopyDest;
      InitBuffer;
      mid := FHeight div 2;
      y1 := FThickness;
      y2 := FHeight - FThickness;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
    end;

    atfUnsquashDown:
    begin
      CopyDest;
      t := 0;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfUnsquashLeft:
    begin
      CopyDest;
      t := FThickness;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfUnsquashRight:
    begin
      CopyDest;
      t := 0;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfUnsquashUp:
    begin
      CopyDest;
      t := FThickness;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfUnsquashCenterHorz:
    begin
      CopyDest;
      InitBuffer;
      mid := FWidth div 2;
      x1 := mid - FThickness div 2;
      x2 := mid + FThickness div 2;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
    end;

    atfUnsquashCenterVert:
    begin
      CopyDest;
      InitBuffer;
      mid := FHeight div 2;
      y1 := mid - FThickness div 2;
      y2 := mid + FThickness div 2;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
    end;

    atfShrinkLeftDown..atfShrinkRightUp:
    begin
      CopyDest;
      InitBuffer;
      if FWidth > FHeight then
      begin
        x := FThickness * FWidth div FHeight;
        y := FThickness;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        x := FThickness;
        y := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;
    end;

    atfShrinkCenter:
    begin
      CopyDest;
      InitBuffer;
      if FWidth > FHeight then
      begin
        dx := FThickness * FWidth div FHeight;
        dy := FThickness;
        FFrameDuration := FDuration * FThickness * 2 div FHeight;
      end
      else
      begin
        dx := FThickness;
        dy := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness * 2 div FWidth;
      end;
      x1 := dx;
      x2 := FWidth - dx;
      y1 := dy;
      y2 := FHeight - dy;
    end;

    atfExpandLeftDown..atfExpandRightUp:
    begin
      if FWidth > FHeight then
      begin
        x := FThickness * FWidth div FHeight;
        y := FThickness;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        x := FThickness;
        y := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;
    end;

    atfExpandCenter:
    begin
      if FWidth > FHeight then
      begin
        h := FThickness;
        w := h * FWidth div FHeight;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        w := FThickness;
        h := w * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;
      x := (FWidth - w) div 2;
      y := (FHeight - h) div 2;
    end;

    atfZoomIn:
    begin
      if FWidth > FHeight then
      begin
        dy := FThickness * ZOOM_SCALE_FACTOR * ZOOM_SPEED_FACTOR;
        dx := dy * FWidth div FHeight;
      end
      else
      begin
        dx := FThickness * ZOOM_SCALE_FACTOR * ZOOM_SPEED_FACTOR;
        dy := dx * FHeight div FWidth;
      end;

      x1 := - FWidth * ZOOM_SCALE_FACTOR;
      x2 := FWidth * (ZOOM_SCALE_FACTOR+1);
      y1 := -FHeight * ZOOM_SCALE_FACTOR;
      y2 := FHeight * (ZOOM_SCALE_FACTOR+1);

      FFrameDuration := FDuration * dx div (-x1);
    end;

    atfZoomOut:
    begin
      CopyDest;
      if FWidth > FHeight then
      begin
        dy := FThickness;
        dx := dy * FWidth div FHeight;
      end
      else
      begin
        dx := FThickness;
        dy := dx * FHeight div FWidth;
      end;
      x1 := -dx;
      x2 := FWidth + dx;
      y1 := -dy;
      y2 := FHeight + dy;
      FFrameDuration := FDuration div ZOOM_OUT_FRAMES;
      i := 0;
    end;

    atfZoomDown:
    begin
      y := - FHeight * ZOOM_SCALE_FACTOR;
      dy := FThickness * ZOOM_SPEED_FACTOR;
      FFrameDuration := FDuration * dy div (-y);
    end;

    atfZoomLeft:
    begin
      x := FWidth * (ZOOM_SCALE_FACTOR+1);
      dx := FThickness * ZOOM_SPEED_FACTOR;
      FFrameDuration := FDuration * dx div (x-FWidth);
    end;

    atfZoomRight:
    begin
      x := - FWidth * ZOOM_SCALE_FACTOR;
      dx := FThickness * ZOOM_SPEED_FACTOR;
      FFrameDuration := FDuration * dx div (-x);
    end;

    atfZoomUp:
    begin
      y := FHeight * (ZOOM_SCALE_FACTOR+1);
      dy := FThickness * ZOOM_SPEED_FACTOR;
      FFrameDuration := FDuration * dy div (y-FHeight);
    end;

    atfZoomLeftDown:
    begin
      if FWidth > FHeight then
      begin
        dy := FThickness * ZOOM_SPEED_FACTOR;
        dx := dy * FWidth div FHeight;
      end
      else
      begin
        dx := FThickness * ZOOM_SPEED_FACTOR;
        dy := dx * FHeight div FWidth;
      end;
      x := FWidth * (ZOOM_SCALE_FACTOR+1);
      y := - FHeight * ZOOM_SCALE_FACTOR;
      FFrameDuration := FDuration * dy div (-y);
    end;

    atfZoomLeftUp:
    begin
      if FWidth > FHeight then
      begin
        dy := FThickness * ZOOM_SPEED_FACTOR;
        dx := dy * FWidth div FHeight;
      end
      else
      begin
        dx := FThickness * ZOOM_SPEED_FACTOR;
        dy := dx * FHeight div FWidth;
      end;
      x := FWidth * (ZOOM_SCALE_FACTOR+1);
      y := FHeight * (ZOOM_SCALE_FACTOR+1);
      FFrameDuration := FDuration * dy div (y-FWidth);
    end;

    atfZoomRightDown:
    begin
      if FWidth > FHeight then
      begin
        dy := FThickness * ZOOM_SPEED_FACTOR;
        dx := dy * FWidth div FHeight;
      end
      else
      begin
        dx := FThickness * ZOOM_SPEED_FACTOR;
        dy := dx * FHeight div FWidth;
      end;
      x := - FWidth * ZOOM_SCALE_FACTOR;
      y := - FHeight * ZOOM_SCALE_FACTOR;
      FFrameDuration := FDuration * dy div (-y);
    end;

    atfZoomRightUp:
    begin
      if FWidth > FHeight then
      begin
        dy := FThickness * ZOOM_SPEED_FACTOR;
        dx := dy * FWidth div FHeight;
      end
      else
      begin
        dx := FThickness * ZOOM_SPEED_FACTOR;
        dy := dx * FHeight div FWidth;
      end;
      x := - FWidth * ZOOM_SCALE_FACTOR;
      y := FHeight * (ZOOM_SCALE_FACTOR+1);
      FFrameDuration := FDuration * dx div (-x);
    end;

    atfPushDown, atfPushUp:
    begin
      CopyDest;
      FFrameDuration := FDuration * FThickness div FHeight;
      i := FThickness;
    end;

    atfPushLeft, atfPushRight:
    begin
      CopyDest;
      FFrameDuration := FDuration * FThickness div FWidth;
      i := FThickness;
    end;

    atfCoverDown, atfCoverUp:
    begin
      FFrameDuration := FDuration * FThickness div FHeight;
      i := FThickness;
    end;

    atfCoverLeft, atfCoverRight:
    begin
      FFrameDuration := FDuration * FThickness div FWidth;
      i := FThickness;
    end;

    atfCoverLeftDown..atfCoverRightUp:
    begin
      if FWidth > FHeight then
      begin
        x := FThickness * FWidth div FHeight;
        y := FThickness;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        x := FThickness;
        y := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;
    end;

    atfCoverCenterHorzIn:
    begin
      mid := FWidth div 2;
      x := FThickness;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
    end;

    atfCoverCenterHorzOut:
    begin
      mid := FWidth div 2;
      x1 := mid - FThickness div 2;
      x2 := mid + FThickness div 2;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
    end;

    atfCoverCenterVertIn:
    begin
      mid := FHeight div 2;
      y := FThickness;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
    end;

    atfCoverCenterVertOut:
    begin
      mid := FHeight div 2;
      y1 := mid - FThickness div 2;
      y2 := mid + FThickness div 2;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
    end;

    atfUncoverDown, atfUncoverUp:
    begin
      CopyDest;
      FFrameDuration := FDuration * FThickness div FHeight;
      i := FThickness;
    end;

    atfUncoverLeft, atfUncoverRight:
    begin
      CopyDest;
      FFrameDuration := FDuration * FThickness div FWidth;
      i := FThickness;
    end;

    atfUncoverLeftDown..atfUncoverRightUp:
    begin
      CopyDest;
      if FWidth > FHeight then
      begin
        x := FThickness * FWidth div FHeight;
        y := FThickness;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        x := FThickness;
        y := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;
    end;

    atfUncoverCenterHorzIn, atfUncoverCenterHorzOut:
    begin
      CopyDest;
      InitBuffer;
      mid := FWidth div 2;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
      if FEffect = atfUncoverCenterHorzIn then
        x := FThickness
      else
      begin
        x1 := mid - FThickness div 2;
        x2 := mid + FThickness div 2;
      end;
    end;

    atfUncoverCenterVertIn, atfUncoverCenterVertOut:
    begin
      CopyDest;
      InitBuffer;
      mid := FHeight div 2;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
      if FEffect = atfUncoverCenterVertIn then
        y := FThickness
      else
      begin
        y1 := mid - FThickness div 2;
        y2 := mid + FThickness div 2;
      end;
    end;

    atfWipeDown:
    begin
      i := - FThickness;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfWipeLeft:
    begin
      i := 0;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfWipeRight:
    begin
      i := - FThickness;
      FFrameDuration := FDuration * FThickness div FWidth;
    end;

    atfWipeUp:
    begin
      i := 0;
      FFrameDuration := FDuration * FThickness div FHeight;
    end;

    atfWipeLeftDown..atfWipeRightUp:
    begin
      i := 0;
      nFrames := abcCeil(FWidth/FThickness) + abcCeil(FHeight/FThickness);
      FFrameDuration := FDuration div nFrames;
      case FEffect of
        atfWipeLeftDown:
        begin
          x1 := FWidth;
          y1 := 0;
        end;
        atfWipeLeftUp:
        begin
          x1 := FWidth - FThickness;
          y1 := FHeight;
        end;
        atfWipeRightDown:
        begin
          x1 := - FThickness;
          y1 := 0;
        end;
        atfWipeRightUp:
        begin
          x1 := 0;
          y1 := FHeight;
        end;
      else
      end;
    end;

    atfWipeRectLeftDown..atfWipeRectRightUp:
    begin
      if FWidth > FHeight then
      begin
        x := FThickness * FWidth div FHeight;
        y := FThickness;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        x := FThickness;
        y := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;
    end;

    atfCropLeftDown..atfCropRightUp:
    begin
      if FWidth > FHeight then
      begin
        x := FThickness * FWidth div FHeight;
        y := FThickness;
        FFrameDuration := FDuration * FThickness div FHeight;
      end
      else
      begin
        x := FThickness;
        y := FThickness * FHeight div FWidth;
        FFrameDuration := FDuration * FThickness div FWidth;
      end;
    end;

    atfWipeCenterHorzIn, atfWipeCenterHorzOut:
    begin
      mid := FWidth div 2;
      FFrameDuration := FDuration * FThickness div FWidth * 2;
      if FEffect = atfWipeCenterHorzIn then
      begin
        x1 := FThickness;
        x2 := FWidth - FThickness;
      end
      else
      begin
        x1 := mid - FThickness div 2;
        x2 := mid + FThickness div 2;
      end;
    end;

    atfWipeCenterVertIn, atfWipeCenterVertOut:
    begin
      mid := FHeight div 2;
      FFrameDuration := FDuration * FThickness div FHeight * 2;
      if FEffect = atfWipeCenterVertIn then
      begin
        y1 := FThickness;
        y2 := FHeight - FThickness;
      end
      else
      begin
        y1 := mid - FThickness div 2;
        y2 := mid + FThickness div 2;
      end;
    end;

    atfWipeClockwise, atfWipeAntiClockwise:
    begin
      i := 0;
      FFrameDuration := FDuration div 16;
      halfWidth := abcCeil(FWidth / 2);
      halfHeight := abcCeil(FHeight / 2);
      ptPolyMask[0].x := halfWidth;
      ptPolyMask[0].y := halfHeight;
      ptPolyMask[2].x := halfWidth;
      ptPolyMask[2].y := 0;
    end;

    atfCutThroughBlack:
    begin
      FFrameDuration := FDuration div 2;
      i := 0;
    end;

    atfFadeThroughBlack:
    begin
      InitBuffer;
      FFrameDuration := FDuration div 8;
      FadeBrushes[1] := CreateFadeBrush(1);
      FadeBrushes[2] := CreateFadeBrush(2);
      FadeBrushes[3] := CreateFadeBrush(3);
      FadeBrushes[4] := CreateFadeBrush(4);
      i := 0;
    end;

  else
  end;
end;

{------------------------------------------------------------------------------}

function TabcTransitionThread.ClockPoint(Index: Integer): TPoint;
const
  X_STEPS : array[0..15] of Integer =
   (3,4,4,4,4,4,3,2,1,0,0,0,0,0,1,2);
  X_STEPS_ANTICLOCKWISE : array[0..15] of Integer =
   (1,0,0,0,0,0,1,2,3,4,4,4,4,4,3,2);
  Y_STEPS : array[0..15] of Integer =
   (0,0,1,2,3,4,4,4,4,4,3,2,1,0,0,0);
begin
  if FEffect = atfWipeClockwise then
    Result.x := FWidth * X_STEPS[Index] div 4
  else
    Result.x := FWidth * X_STEPS_ANTICLOCKWISE[Index] div 4;
  Result.y := FHeight * Y_STEPS[Index] div 4;
end;

function TabcTransitionThread.CreateFadeBrush(Darkness: Integer): hBrush;
var
  bmp: TBitmap;
  x, y: integer;
begin
  bmp := TBitmap.Create;
  try
    with bmp do
    begin
      Monochrome := True;
      Width := 8;
      Height := 8;
      case Darkness of
        1:
          for x := 0 to 7 do
            for y := 0 to 7 do
              if Odd(x) and Odd(y) then
                Canvas.Pixels[x,y] := clBlack
              else
                Canvas.Pixels[x,y] := clWhite;
        2:
          for x := 0 to 7 do
            for y := 0 to 7 do
              if Odd(y) then
                Canvas.Pixels[x,y] := clWhite
              else
                if Odd(x) then
                  Canvas.Pixels[x,y] := clWhite
                else
                  Canvas.Pixels[x,y] := clBlack;
        3:
          for x := 0 to 7 do
            for y := 0 to 7 do
              if Odd(x) <> Odd(y) then
                Canvas.Pixels[x,y] := clWhite
              else
                Canvas.Pixels[x,y] := clBlack;
        4:
          for x := 0 to 7 do
            for y := 0 to 7 do
              if Odd(x) and Odd(y) then
                Canvas.Pixels[x,y] := clWhite
              else
                Canvas.Pixels[x,y] := clBlack;
      end;
      Result := CreatePatternBrush(Handle);
    end;
  finally
    bmp.Free;
  end;
end;

procedure TabcTransitionThread.BrushFadeOut(i: Integer);
// i = 1 to 4
var
  oldBrush: Integer;
begin
  if Terminated then Exit;
  oldBrush := SelectObject(FDest.Handle, FadeBrushes[i]);
  try
    if not Terminated then
      PatBlt (FDest.Handle, FDestX, FDestY, FDestX+FWidth, FDestY+FHeight, $A000C9);
  finally
    SelectObject(FDest.Handle, oldBrush);
  end;
end;

procedure TabcTransitionThread.BrushFadeIn(i: Integer);
// i = 4 downto 1
var
  oldBrush: HBRUSH;
begin
  if Terminated then Exit;
  CopySourceToBuffer;
  if Terminated then Exit;
  oldBrush := SelectObject(FBufferDC, FadeBrushes[i]);
  try
    if not Terminated then
      PatBlt (FBufferDC, 0, 0, FWidth, FHeight, $A000C9);
  finally
    SelectObject(FBufferDC, oldBrush);
  end;
  if Terminated then Exit;
  CopyBufferToDest;
end;

procedure TabcTransitionThread.InitSpiralPoints(nPoints: Integer);
var
  i: integer;
  Pt: PabcSpiralPoint;
begin

  if FSpiralPoints <> nil then
    FreeSpiralPoints;

  halfW := FWidth shr 1;
  halfH := FHeight shr 1;

  FSpiralPoints := TList.Create;
  FSpiralPoints.Capacity := nPoints;

  minx := 0;
  miny := 0;
  maxx := 0;
  maxy := 0;

  for i := 1 to nPoints do
  begin
    New(Pt);
    tt := SPIRAL_DD + (i-1) * td;
    xs := SPIRAL_A * tt * cos(tt);
    ys := SPIRAL_A * tt * sin(tt);
    if xs < minx then
      minx := xs
    else if xs > maxx then
      maxx := xs;
    Pt^[0] := xs;
    if ys < miny then
      miny := ys
    else if ys > maxy then
      maxy := ys;
    Pt^[1] := ys;
    FSpiralPoints.Add(Pt);
  end;

  if maxx < abs(minx) then
    maxx := abs(minx);

  if maxy < abs(miny) then
    maxy := abs(miny);

  scaley := halfH / maxy;
  scalex := halfW / maxx;

  if scalex > scaley then
    scaley := scalex
  else
    scalex := scaley;
end;

procedure TabcTransitionThread.FreeSpiralPoints;
var
  i: integer;
  Pt: PabcSpiralPoint;
begin
  if FSpiralPoints <> nil then
  with FSpiralPoints do
  begin
    for i := Count - 1 downto 0 do
    begin
      Pt := PabcSpiralPoint(Items[i]);
      Dispose(Pt);
    end;
    Free;
    FSpiralPoints := nil;
  end;
end;

function TabcTransitionThread.SpiralPoint(Frame: Integer): TPoint;
var
  Pt: PabcSpiralPoint;
begin
  if (FSpiralPoints <> nil) and (Frame < FSpiralPoints.Count) then
  begin
    Pt := PabcSpiralPoint(FSpiralPoints[Frame]);
    Result.X := Trunc(Pt^[0] * scalex) + halfW;
    Result.Y := Trunc(Pt^[1] * scaley) + halfH;
  end
  else
    Result := Point(0,0);
end;

procedure TabcTransitionThread.DrawFrame;
var
  ay, lx, ly, px, py, rgnFound: Integer;
  newClipRgn, oldClipRgn: HRGN;
  inner: Integer;
  aPoint: TPoint;
begin
  if Terminated then Exit;
  case FEffect of

    atfSkipHorizontal:
    begin
      DrawOffset(i, 0, FThickness, FHeight, i, 0);
      DrawOffset(j, 0, FThickness, FHeight, j, 0);
      i := i + (FThickness shl 1);
      j := j - (FThickness shl 1);
    end;

    atfSkipVertical:
    begin
      DrawOffset(0, i, FWidth, FThickness, 0, i);
      DrawOffset(0, j, FWidth, FThickness, 0, j);
      i := i + (FThickness shl 1);
      j := j - (FThickness shl 1);
    end;

    atfSkipDiagonals:
    begin
      id := id + 1;
      if (id = 1) then
      begin
        ptPolyMask[0].x := 0;
        ptPolyMask[0].y := 0;
        ptPolyMask[1].x := FThickness;
        ptPolyMask[1].y := 0;
        ptPolyMask[2].x := 0;
        ptPolyMask[2].y := FThickness;
        newClipRgn := CreatePolygonRgn (ptPolyMask, 3, WINDING);
      end
      else if (id = 2) and not Odd(ndiags) then
      begin
        ptPolyMask[0].x := CeilWidth;
        ptPolyMask[0].y := CeilHeight;
        ptPolyMask[1].x := CeilWidth - FThickness;
        ptPolyMask[1].y := CeilHeight;
        ptPolyMask[2].x := CeilWidth;
        ptPolyMask[2].y := CeilHeight - FThickness;
        newClipRgn := CreatePolygonRgn (ptPolyMask, 3, WINDING);
      end
      else
      begin
        if Odd(id) then
        begin
          ptPolyMask[0].x := (id - 1) * FThickness;
          ptPolyMask[1].x := ptPolyMask[0].x + FThickness;
          if (ptPolyMask[0].x > CeilWidth) then
          begin
            ptPolyMask[0].y := ptPolyMask[0].x - CeilWidth;
            ptPolyMask[0].x := CeilWidth;
          end
          else
            ptPolyMask[0].y := 0;
          if (ptPolyMask[1].x > CeilWidth) then
          begin
            ptPolyMask[1].y := ptPolyMask[1].x - CeilWidth;
            ptPolyMask[1].x := CeilWidth;
          end
          else
            ptPolyMask[1].y := 0;
          ptPolyMask[2].y := id * FThickness;
          ptPolyMask[3].y := ptPolyMask[2].y - FThickness;
          if (ptPolyMask[2].y > CeilHeight) then
          begin
            ptPolyMask[2].x := ptPolyMask[2].y - CeilHeight;
            if ptPolyMask[2].x < FThickness then
              ptPolyMask[2].x := 0;
            ptPolyMask[2].y := CeilHeight;
          end
          else
            ptPolyMask[2].x := 0;
          if (ptPolyMask[3].y > CeilHeight) then
          begin
            ptPolyMask[3].x := ptPolyMask[3].y - CeilHeight;
            ptPolyMask[3].y := CeilHeight;
          end
          else
            ptPolyMask[3].x := 0;
        end
        else
        begin
          if Odd(ndiags) then
            ptPolyMask[0].x := CeilWidth - (id - 1) * FThickness
          else
            ptPolyMask[0].x := CeilWidth - (id - 2) * FThickness;
          ptPolyMask[1].x := ptPolyMask[0].x - FThickness;
          if (ptPolyMask[0].x < 0) then
          begin
            ptPolyMask[0].y := CeilHeight + ptPolyMask[0].x;
            ptPolyMask[0].x := 0;
          end
          else
            ptPolyMask[0].y := CeilHeight;
          if (ptPolyMask[1].x < 0) then
          begin
            ptPolyMask[1].y := CeilHeight + ptPolyMask[1].x;
            ptPolyMask[1].x := 0;
          end
          else
            ptPolyMask[1].y := CeilHeight;

          if Odd(ndiags) then
            ptPolyMask[2].y := CeilHeight - id * FThickness
          else
            ptPolyMask[2].y := CeilHeight - (id - 1) * FThickness;
          ptPolyMask[3].y := ptPolyMask[2].y + FThickness;
          if (ptPolyMask[2].y < 0) then
          begin
            ptPolyMask[2].x :=  CeilWidth + ptPolyMask[2].y;
            ptPolyMask[2].y := 0;
          end
          else
           ptPolyMask[2].x := CeilWidth;
          if (ptPolyMask[3].y < 0) then
          begin
            ptPolyMask[3].x := CeilWidth + ptPolyMask[3].y;
            ptPolyMask[3].y := 0;
          end
          else
            ptPolyMask[3].x := CeilWidth;
        end;
        newClipRgn := CreatePolygonRgn (ptPolyMask, 4, WINDING);
      end;

      if newClipRgn <> 0 then
      begin
        rgnFound := -1;
        oldClipRgn := CreateRectRgn(0,0,1,1);
        try
          if Terminated then Exit;
          if not GetViewportOrgEx(FDest.Handle, ptDestOffset) then Exit;
          if Terminated then Exit;
          OffsetRgn(newClipRgn, ptDestOffset.X + FDestX, ptDestOffset.Y + FDestY);
          if Terminated then Exit;
          rgnFound := GetClipRgn(FDest.Handle, oldClipRgn);
          if rgnFound = -1 then Exit;
          if SelectClipRgn(FDest.Handle, newClipRgn) = ERROR then Exit;
          if Terminated then Exit;
          DrawOffset(0, 0, FWidth, FHeight, 0, 0);
        finally
          DeleteObject(newClipRgn);
          if rgnFound = 1 then
            SelectClipRgn(FDest.Handle, oldClipRgn)
          else if rgnFound = 0 then
            SelectClipRgn(FDest.Handle, 0);
          DeleteObject(oldClipRgn);
        end;
      end;
    end;

    atfSkipDiagonalSquares:
    begin
      id := id + 1;
      if Odd(id) then begin
        i := id * FThickness;
        j := (id - i div FThickness) * FThickness;
        while (j <= FHeight) do
        begin
          i := i - FThickness;
          DrawOffset(i, j, FThickness, FThickness, i, j);
          j := j + FThickness;
        end;
      end
      else
      begin
        if Odd(nFrames) then
          i := (xFrames + 1 - id) * FThickness
        else
          i := (xFrames + 2 - id) * FThickness;
        j := yFrames * FThickness;
        while (i <= FWidth) do
        begin
          j := j - FThickness;
          DrawOffset(i, j, FThickness, FThickness, i, j);
          i := i + FThickness;
        end;
      end;
    end;

    atfSkipRectangles:
    begin
      stilldrawing := FALSE;
      if (xl <= halfsize) and (yl <= halfsize) then
      begin
        if xl = 0 then
          inwidth := FWidth - 1
        else
          inwidth := FWidth - (xl shl 1);
        if yl = 0 then
          inheight := FHeight - 1
        else
          inheight := FHeight - (yl shl 1);
        DrawOffset(xl, yl, inwidth, 1, xl, yl);
        DrawOffset(xl, yl, 1, inheight, xl, yl);
        DrawOffset(xl, yl + inheight, inwidth + 1, 1, xl, yl + inheight);
        DrawOffset(xl + inwidth, yl, 1, inheight + 1, xl + inwidth, yl);
        stilldrawing := TRUE;
      end;

      if (xr >= 0) and (yr >= 0) then
      begin
        outwidth := FWidth - (xr shl 1);
        outheight := FHeight - (yr shl 1);
        DrawOffset(xr, yr, outwidth, 1, xr, yr);
        DrawOffset(xr, yr, 1, outheight, xr, yr);
        DrawOffset(xr, yr + outheight, outwidth + 1, 1, xr, yr + outheight);
        DrawOffset(xr + outwidth, yr, 1, outheight + 1, xr + outwidth, yr);
        stilldrawing := TRUE;
      end;
      if stilldrawing then
      begin
        xl := xl + 2;
        yl := yl + 2;
        xr := xr - 2;
        yr := yr - 2;
      end;
    end;

    atfBoxIn:
    begin
      DrawOffset(x2, 0, x1, FHeight, x2, 0);
      DrawOffset(0, 0, x1, FHeight, 0, 0);
      DrawOffset(0, 0, FWidth, y1, 0, 0);
      DrawOffset(0, y2, FWidth, y1, 0, y2);
      x1 := x1 + dx;
      x2 := x2 - dx;
      y1 := y1 + dy;
      y2 := y2 - dy;
    end;

    atfBoxOut:
    begin
      DrawOffset(x, y, w, h, x, y);
      if FWidth > FHeight then
      begin
        h := h + FThickness;
        w := h * FWidth div FHeight;
      end
      else
      begin
        w := w + FThickness;
        h := w * FHeight div FWidth;
      end;

      x := (FWidth - w) div 2;
      y := (FHeight - h) div 2;
    end;

    atfBlindsHorz:
    begin
      j := i;
      while j < FHeight do
      begin
        if Terminated then Exit;
        DrawOffset(0, j, FWidth, 1, 0, j);
        j := j + FThickness;
      end;
      i := i + 1;
    end;

    atfBlindsVert:
    begin
      j := i;
      while j < FWidth do
      begin
        if Terminated then Exit;
        DrawOffset(j, 0, 1, FHeight, j, 0);
        j := j + FThickness;
      end;
      i := i + 1;
    end;

    atfDissolve:
    begin
      for ay := 0 to RANDARRAYSIZE-1 do
        for lx := 0 to lxtimes - 1 do
          for ly := 0 to lytimes - 1 do
          begin
            if Terminated then Exit;
            px := lx * RANDARRAYSIZE + RandPoints.RandXs[i, ay];
            if px >= FWidth then Continue;
            py := ly * RANDARRAYSIZE + RandPoints.RandYs[i, ay];
            if py >= FHeight then Continue;
//            DrawOffset(px, py, FThickness, FThickness, px, py);
            DrawOffset(px, py, 2, 2, px, py);
          end;
      i := i + 1;
    end;

    atfDissolveHorz:
    begin
      for ly := 0 to lytimes - 1 do
      begin
        if Terminated then Exit;
        py := ly * RANDARRAYSIZE + RandPoints.Rands[i];
        if py >= FHeight then Continue;
//        DrawOffset(0, py, FWidth, FThickness, 0, py);
        DrawOffset(0, py, FWidth, 2, 0, py);
      end;
      i := i + 1;
    end;

    atfDissolveVert:
    begin
      for lx := 0 to lxtimes - 1 do
      begin
        if Terminated then Exit;
        px := lx * RANDARRAYSIZE + RandPoints.Rands[i];
        if px >= FWidth then Continue;
//        DrawOffset(px, 0, FThickness, FHeight, px, 0);
        DrawOffset(px, 0, 2, FHeight, px, 0);
      end;
      i := i + 1;
    end;

    atfWindIn:
    begin
      xDraw := xMin;
      yDraw := yMin;

      while xDraw < xMax do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        xDraw := xDraw + dX;
      end;
      xDraw := xDraw - dX;
      xMax := xDraw;
      while yDraw < yMax do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        yDraw := yDraw + dY;
      end;
      yDraw := yDraw - dY;
      yMax := yDraw;
      while xDraw >= xMin do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        xDraw := xDraw - dX;
      end;
      xDraw := xDraw + dX;
      if xDraw <= xMin then
        xDraw := xDraw + dX;
      xMin := xDraw;

      while yDraw >= yMin do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        yDraw := yDraw - dY;
      end;
      yDraw := yDraw + dY;
      if yDraw <= yMin then
        yDraw := yDraw + dY;
      yMin := yDraw;
    end;

    atfWindOut:
    begin
      xDraw := xMin;
      yDraw := yMin;
      while xDraw < xMax do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        xDraw := xDraw + dX;
      end;
      xMax := xDraw + dX;
      xMin := xMin - dX;
      while yDraw < yMax do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        yDraw := yDraw + dY;
      end;
      yMax := yDraw + dY;
      while xDraw > xMin do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        xDraw := xDraw - dX;
      end;
      xMin := xDraw;
      while yDraw >= yMin do
      begin
        DrawOffset(xDraw, yDraw, dX, dY, xDraw, yDraw);
        if Terminated then Exit;
        yDraw := yDraw - dY;
      end;
      yMin := yDraw;
    end;

    atfPourDown:
    begin
      j := i + FThickness;
      while j > 0 do
      begin
        j := j - FThickness;
        if j < 0 then j := 0;
        DrawOffset(0, j, FWidth, FThickness, 0, i);
        if Terminated then Exit;
      end;
      i := i - FThickness;
    end;

    atfPourLeft:
    begin
      j := i;
      while (j < FWidth) do
      begin
        if j > lastpos then j := lastpos;
        DrawOffset(j, 0, FThickness, FHeight, i, 0);
        j := j + FThickness;
        if Terminated then Exit;
      end;
      if i > lastpos then i := lastpos;
      i := i + FThickness;
    end;

    atfPourRight:
    begin
      j := i + FThickness;
      while j > 0 do
      begin
        j := j - FThickness;
        if j < 0 then j := 0;
        DrawOffset(j, 0, FThickness, FHeight, i, 0);
        if Terminated then Exit;
      end;
      i := i - FThickness;
    end;

    atfPourUp:
    begin
      j := i;
      while (j < FHeight) do
      begin
        if j > lastpos then j := lastpos;
        DrawOffset(0, j, FWidth, FThickness, 0, i);
        j := j + FThickness;
        if Terminated then Exit;
      end;
      if i > lastpos then i := lastpos;
      i := i + FThickness;
    end;

    atfQuadCover:
    begin
      DrawOffsetToBuffer(0, 0, x, y, midx - x, midy - y);
      DrawOffsetToBuffer(FWidth - x, 0, x, y, midx, midy - y);
      DrawOffsetToBuffer(0, FHeight - y, x, y, midx - x, midy);
      DrawOffsetToBuffer(FWidth - x, FHeight - y, x, y, midx, midy);
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfQuadUncover:
    begin
      CopySourceToBuffer;
      DrawCopyToBuffer(0, 0, x, y, midx - x, midy - y);
      DrawCopyToBuffer(FWidth - x, 0, x, y, midx, midy - y);
      DrawCopyToBuffer(0, FHeight - y, x, y, midx - x, midy);
      DrawCopyToBuffer(FWidth - x, FHeight - y, x, y, midx, midy);
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y - FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x - FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfQuadWipeIn:
    begin
      DrawOffsetToBuffer(0, 0, x, y, 0, 0);
      DrawOffsetToBuffer(FWidth - x, 0, x, y, FWidth - x, 0);
      DrawOffsetToBuffer(0, FHeight - y, x, y, 0, FHeight - y);
      DrawOffsetToBuffer(FWidth - x, FHeight - y, x, y, FWidth - x, FHeight - y);
      CopyBuffertoDest;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfQuadWipeOut:
    begin
      CopySourceToBuffer;
      DrawCopyToBuffer(0, 0, x, y, 0, 0);
      DrawCopyToBuffer(FWidth - x, 0, x, y, FWidth - x, 0);
      DrawCopyToBuffer(0, FHeight - y, x, y, 0, FHeight - y);
      DrawCopyToBuffer(FWidth - x, FHeight - y, x, y, FWidth - x, FHeight - y);
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y - FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x - FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfQuadUnfold:
    begin
      StretchToBuffer(0, 0, x, y, 0, 0, midx, midy);
      StretchToBuffer(FWidth - x, 0, x, y, midx, 0, midx, midy);
      StretchToBuffer(0, FHeight - y, x, y, 0, midy, midx, midy);
      StretchToBuffer(FWidth - x, FHeight - y, x, y, midx, midy, midx, midy);
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfQuadFold, atfQuadFoldCopy:
    begin
      CopySourceToBuffer;
      if FEffect = atfQuadFold then
      begin
        StretchCopyToBuffer(0, 0, x, y, 0, 0, midx, midy);
        StretchCopyToBuffer(FWidth - x, 0, x, y, midx, 0, midx, midy);
        StretchCopyToBuffer(0, FHeight - y, x, y, 0, midy, midx, midy);
        StretchCopyToBuffer(FWidth - x, FHeight - y, x, y, midx, midy, midx, midy);
      end
      else
      begin
        StretchCopyToBuffer(0, 0, x, y, 0, 0, FWidth, FHeight);
        StretchCopyToBuffer(FWidth - x, 0, x, y, 0, 0, FWidth, FHeight);
        StretchCopyToBuffer(0, FHeight - y, x, y, 0, 0, FWidth, FHeight);
        StretchCopyToBuffer(FWidth - x, FHeight - y, x, y, 0, 0, FWidth, FHeight);
      end;
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y - FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x - FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfQuadExpand:
    begin
      StretchToBuffer(drawx, drawy, x, y, 0, 0, midx, midy);
      StretchToBuffer(midx + drawx, drawy, x, y, midx, 0, midx, midy);
      StretchToBuffer(drawx, midy + drawy, x, y, 0, midy, midx, midy);
      StretchToBuffer(midx + drawx, midy + drawy, x, y, midx, midy, midx, midy);
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y + FThickness * 2;
        x := y * FWidth div FHeight;
        drawy := drawy - FThickness;
        drawx := drawy * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness * 2;
        y := x * FHeight div FWidth;
        drawx := drawx - FThickness;
        drawy := drawx * FHeight div FWidth;
      end;
    end;

    atfQuadShrink, atfQuadShrinkCopy:
    begin
      CopySourceToBuffer;
      if FEffect = atfQuadShrink then
      begin
        StretchCopyToBuffer(drawx, drawy, x, y, 0, 0, midx, midy);
        StretchCopyToBuffer(midx + drawx, drawy, x, y, midx, 0, midx, midy);
        StretchCopyToBuffer(drawx, midy + drawy, x, y, 0, midy, midx, midy);
        StretchCopyToBuffer(midx + drawx, midy + drawy, x, y, midx, midy, midx, midy);
      end
      else
      begin
        StretchCopyToBuffer(drawx, drawy, x, y, 0, 0, FWidth, FHeight);
        StretchCopyToBuffer(midx + drawx, drawy, x, y, 0, 0, FWidth, FHeight);
        StretchCopyToBuffer(drawx, midy + drawy, x, y, 0, 0, FWidth, FHeight);
        StretchCopyToBuffer(midx + drawx, midy + drawy, x, y, 0, 0, FWidth, FHeight);
      end;
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y - FThickness * 2;
        x := y * FWidth div FHeight;
        drawy := drawy + FThickness;
        drawx := drawy * FWidth div FHeight;
      end
      else
      begin
        x := x - FThickness * 2;
        y := x * FHeight div FWidth;
        drawx := drawx + FThickness;
        drawy := drawx * FHeight div FWidth;
      end;
    end;

    atfShapeSawTooth:
    begin
      i := i + 1;
      for inner := 1 to SAWTOOTH_FRAME_STEPS do
      begin
        j := j + 1;
        aPoint := SpiralPoint(j);
        if Terminated then Exit;
        DrawOffset(aPoint.X - 10, aPoint.Y - 10, 20, 20, aPoint.X - 10, aPoint.Y - 10);
        if Terminated then Exit;
      end;
    end;

    atfShapeStarBurst..atfShapeSpiralAntiClockwise:
    begin
      i := i + 1;
      ptPolyMask[1] := ptPolyMask[2];
      ptPolyMask[2] := SpiralPoint(i);

      newClipRgn := CreatePolygonRgn (ptPolyMask, 3, WINDING);

      if newClipRgn <> 0 then
      begin
        rgnFound := -1;
        oldClipRgn := CreateRectRgn(0,0,1,1);
        try
          if Terminated then Exit;
          if not GetViewportOrgEx(FDest.Handle, ptDestOffset) then Exit;
          if Terminated then Exit;
          OffsetRgn(newClipRgn, ptDestOffset.X + FDestX, ptDestOffset.Y + FDestY);
          if Terminated then Exit;
          rgnFound := GetClipRgn(FDest.Handle, oldClipRgn);
          if rgnFound = -1 then Exit;
          if SelectClipRgn(FDest.Handle, newClipRgn) = ERROR then Exit;
          if Terminated then Exit;
          DrawOffset(0, 0, FWidth, FHeight, 0, 0);
        finally
          DeleteObject(newClipRgn);
          if rgnFound = 1 then
            SelectClipRgn(FDest.Handle, oldClipRgn)
          else if rgnFound = 0 then
            SelectClipRgn(FDest.Handle, 0);
          DeleteObject(oldClipRgn);
        end;
      end;
    end;

    atfFoldDown:
    begin
      SetDestStretchMode;
      DrawOffset(0, 0, FWidth, i, 0, 0);
      StretchCopyOffset(0, i, FWidth, FHeight - i, 0, 0, FWidth, FHeight);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfFoldLeft:
    begin
      SetDestStretchMode;
      StretchCopyOffset(0, 0, FWidth - i, FHeight, 0, 0, FWidth, FHeight);
      DrawOffset(FWidth - i, 0, i, FHeight, FWidth - i, 0);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfFoldRight:
    begin
      SetDestStretchMode;
      DrawOffset(0, 0, i, FHeight, 0, 0);
      StretchCopyOffset(i, 0, FWidth - i, FHeight, 0, 0, FWidth, FHeight);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfFoldUp:
    begin
      SetDestStretchMode;
      StretchCopyOffset(0, 0, FWidth, FHeight - i, 0, 0, FWidth, FHeight);
      DrawOffset(0, FHeight - i, FWidth, i, 0, FHeight - i);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfFoldCenterHorz:
    begin
      CopySourceToBuffer;
      StretchCopyToBuffer(x1, 0, x2 - x1, FHeight, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      x1 := x1 + FThickness;
      x2 := x2 - FThickness;
    end;

    atfFoldCenterVert:
    begin
      CopySourceToBuffer;
      StretchCopyToBuffer(0, y1, FWidth, y2 - y1, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      y1 := y1 + FThickness;
      y2 := y2 - FThickness;
    end;

    atfUnfoldDown:
    begin
      SetDestStretchMode;
      StretchOffset(0, 0, FWidth, i, 0, 0, FWidth, FHeight);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnfoldLeft:
    begin
      SetDestStretchMode;
      StretchOffset(FWidth - i, 0, i, FHeight, 0, 0,  FWidth, FHeight);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnfoldRight:
    begin
      SetDestStretchMode;
      StretchOffset(0, 0, i, FHeight, 0, 0, FWidth, FHeight);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnfoldUp:
    begin
      SetDestStretchMode;
      StretchOffset(0, FHeight - i, FWidth, i, 0, 0, FWidth, FHeight);
      i := i + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnfoldCenterHorz:
    begin
      DrawCopyToBuffer(0, 0, FWidth, FHeight, 0, 0);
      StretchToBuffer(x1, 0, x2 - x1, FHeight, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      x1 := x1 - FThickness;
      x2 := x2 + FThickness;
    end;

    atfUnfoldCenterVert:
    begin
      DrawCopyToBuffer(0, 0, FWidth, FHeight, 0, 0);
      StretchToBuffer(0, y1, FWidth, y2 - y1, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      y1 := y1 - FThickness;
      y2 := y2 + FThickness;
    end;

    atfSquashDown:
    begin
      SetDestStretchMode;
      DrawOffset(0, 0, FWidth, t, 0, FHeight-t);
      StretchCopyOffset(0, t, FWidth, FHeight - t, 0, 0, FWidth, FHeight);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfSquashLeft:
    begin
      SetDestStretchMode;
      StretchCopyOffset(0, 0, FWidth - t, FHeight, 0, 0,  FWidth, FHeight);
      DrawOffset(FWidth - t, 0, t, FHeight, 0, 0);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfSquashRight:
    begin
      SetDestStretchMode;
      DrawOffset(0, 0, t, FHeight, FWidth - t, 0);
      StretchCopyOffset(t, 0, FWidth - t, FHeight, 0, 0, FWidth, FHeight);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfSquashUp:
    begin
      SetDestStretchMode;
      StretchCopyOffset(0, 0, FWidth, FHeight - t, 0, 0, FWidth, FHeight);
      DrawOffset(0, FHeight - t, FWidth, t, 0, 0);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfSquashCenterHorz:
    begin
      DrawOffsetToBuffer(0, 0, x1, FHeight, mid - x1, 0);
      DrawOffsetToBuffer(x2, 0, FWidth - x2, FHeight, mid, 0);
      StretchCopyToBuffer(x1, 0, x2 - x1, FHeight, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      x1 := x1 + FThickness;
      x2 := x2 - FThickness;
    end;

    atfSquashCenterVert:
    begin
      DrawOffsetToBuffer(0, 0, FWidth, y1, 0, mid - y1);
      DrawOffsetToBuffer(0, y2, FWidth, FHeight - y2, 0, mid);
      StretchCopyToBuffer(0, y1, FWidth, y2 - y1, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      y1 := y1 + FThickness;
      y2 := y2 - FThickness;
    end;

    atfUnsquashDown:
    begin
      SetDestStretchMode;
      StretchOffset(0, 0, FWidth, t, 0, 0, FWidth, FHeight);
      DrawCopyOffset(0, t, FWidth, FHeight-t, 0, 0);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnsquashLeft:
    begin
      SetDestStretchMode;
      DrawCopyOffset(0, 0, FWidth - t, FHeight, t, 0);
      StretchOffset(FWidth - t, 0, t, FHeight, 0, 0,  FWidth, FHeight);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnsquashRight:
    begin
      SetDestStretchMode;
      StretchOffset(0, 0, t, FHeight, 0, 0, FWidth, FHeight);
      DrawCopyOffset(t, 0, FWidth - t, FHeight, 0, 0);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnsquashUp:
    begin
      SetDestStretchMode;
      DrawCopyOffset(0, 0, FWidth, FHeight - t, 0, t);
      StretchOffset(0, FHeight - t, FWidth, t, 0, 0, FWidth, FHeight);
      t := t + FThickness;
      RestoreDestStretchMode;
    end;

    atfUnsquashCenterHorz:
    begin
      DrawCopyToBuffer(0, 0, x1, FHeight, mid - x1, 0);
      DrawCopyToBuffer(x2, 0, FWidth - x2, FHeight, mid, 0);
      StretchToBuffer(x1, 0, x2 - x1, FHeight, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      x1 := x1 - FThickness;
      x2 := x2 + FThickness;
    end;

    atfUnsquashCenterVert:
    begin
      DrawCopyToBuffer(0, 0, FWidth, y1, 0, mid - y1);
      DrawCopyToBuffer(0, y2, FWidth, FHeight - y2, 0, mid);
      StretchToBuffer(0, y1, FWidth, y2 - y1, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      y1 := y1 - FThickness;
      y2 := y2 + FThickness;
    end;

    atfShrinkLeftDown..atfShrinkRightUp:
    begin
      CopySourceToBuffer;
      case FEffect of
        atfShrinkLeftDown:
          StretchCopyToBuffer(0, y, FWidth - x, FHeight - y, 0, 0, FWidth, FHeight);
        atfShrinkLeftUp:
          StretchCopyToBuffer(0, 0, FWidth - x, FHeight - y, 0, 0, FWidth, FHeight);
        atfShrinkRightDown:
          StretchCopyToBuffer(x, y, FWidth - x, FHeight - y, 0, 0, FWidth, FHeight);
        atfShrinkRightUp:
          StretchCopyToBuffer(x, 0, FWidth - x, FHeight - y, 0, 0, FWidth, FHeight);
      else
      end;
      CopyBufferToDest;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfShrinkCenter:
    begin
      CopySourceToBuffer;
      StretchCopyToBuffer(x1, y1, x2 - x1, y2-y1, 0, 0, FWidth, FHeight);
      CopyBufferToDest;
      x1 := x1 + dx;
      x2 := x2 - dx;
      y1 := y1 + dy;
      y2 := y2 - dy;
    end;

    atfExpandLeftDown..atfExpandRightUp:
    begin
      SetDestStretchMode;
      case FEffect of
        atfExpandLeftDown:
          StretchOffset(FWidth - x, 0, x, y, 0, 0, FWidth, FHeight);
        atfExpandLeftUp:
          StretchOffset(FWidth - x, FHeight - y, x, y, 0, 0, FWidth, FHeight);
        atfExpandRightDown:
          StretchOffset(0, 0, x, y, 0, 0, FWidth, FHeight);
        atfExpandRightUp:
          StretchOffset(0, FHeight - y, x, y, 0, 0, FWidth, FHeight);
      else
      end;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
      RestoreDestStretchMode;
    end;

    atfExpandCenter:
    begin
      SetDestStretchMode;
      StretchOffset(x, y, w, h, 0, 0, FWidth, FHeight);
      if FWidth > FHeight then
      begin
        h := h + FThickness;
        w := h * FWidth div FHeight;
      end
      else
      begin
        w := w + FThickness;
        h := w * FHeight div FWidth;
      end;

      x := (FWidth - w) div 2;
      y := (FHeight - h) div 2;
      RestoreDestStretchMode;
    end;

    atfZoomIn:
    begin
      SetDestStretchMode;
      StretchOffset(x1, y1, x2-x1, y2-y1, 0, 0, FWidth, FHeight);
      x1 := x1 + dx;
      x2 := x2 - dx;
      y1 := y1 + dy;
      y2 := y2 - dy;
      RestoreDestStretchMode;
    end;

    atfZoomOut:
    begin
      SetDestStretchMode;
      StretchCopyOffset(x1, y1, x2-x1, y2-y1, 0, 0, FWidth, FHeight);
      x1 := x1 - dx;
      x2 := x2 + dx;
      y1 := y1 - dy;
      y2 := y2 + dy;
      RestoreDestStretchMode;
      i := i + 1;
    end;

    atfZoomDown:
    begin
      SetDestStretchMode;
      StretchOffset(0, y, FWidth, FHeight - y, 0, 0, FWidth, FHeight);
      y := y + dy;
      RestoreDestStretchMode;
    end;

    atfZoomLeft:
    begin
      SetDestStretchMode;
      StretchOffset(0, 0, x, FHeight, 0, 0, FWidth, FHeight);
      x := x - dx;
      RestoreDestStretchMode;
    end;

    atfZoomRight:
    begin
      SetDestStretchMode;
      StretchOffset(x, 0, FWidth - x, FHeight, 0, 0, FWidth, FHeight);
      x := x + dx;
      RestoreDestStretchMode;
    end;

    atfZoomUp:
    begin
      SetDestStretchMode;
      StretchOffset(0, 0, FWidth, y, 0, 0, FWidth, FHeight);
      y := y - dy;
      RestoreDestStretchMode;
    end;

    atfZoomLeftDown:
    begin
      SetDestStretchMode;
      StretchOffset(0, y, x, FHeight - y, 0, 0, FWidth, FHeight);
      x := x - dx;
      y := y + dy;
      RestoreDestStretchMode;
    end;

    atfZoomLeftUp:
    begin
      SetDestStretchMode;
      StretchOffset(0, 0, x, y, 0, 0, FWidth, FHeight);
      x := x - dx;
      y := y - dy;
      RestoreDestStretchMode;
    end;

    atfZoomRightDown:
    begin
      SetDestStretchMode;
      StretchOffset(x, y, FWidth - x, FHeight - y, 0, 0, FWidth, FHeight);
      x := x + dx;
      y := y + dy;
      RestoreDestStretchMode;
    end;

    atfZoomRightUp:
    begin
      SetDestStretchMode;
      StretchOffset(x, 0, FWidth - x, y, 0, 0, FWidth, FHeight);
      x := x + dx;
      y := y - dy;
      RestoreDestStretchMode;
    end;

    atfPushDown:
    begin
      DrawOffset(0, 0, FWidth, i, 0, FHeight - i);
      DrawCopyOffset(0, i, FWidth, FHeight - i, 0, 0);
      i := i + FThickness;
    end;

    atfPushLeft:
    begin
      DrawOffset(FWidth - i, 0, i, FHeight, 0, 0);
      DrawCopyOffset(0, 0, FWidth-i, FHeight, i, 0);
      i := i + FThickness;
    end;

    atfPushRight:
    begin
      DrawOffset(0, 0, i, FHeight, FWidth - i, 0);
      DrawCopyOffset(i, 0, FWidth - i, FHeight, 0, 0);
      i := i + FThickness;
    end;

    atfPushUp:
    begin
      DrawOffset(0, FHeight - i, FWidth, i, 0, 0);
      DrawCopyOffset(0, 0, FWidth, FHeight - i, 0, i);
      i := i + FThickness;
    end;

    atfCoverDown:
    begin
      DrawOffset(0, 0, FWidth, i, 0, FHeight - i);
      i := i + FThickness;
    end;

    atfCoverLeft:
    begin
      DrawOffset(FWidth - i, 0, i, FHeight, 0, 0);
      i := i + FThickness;
    end;

    atfCoverRight:
    begin
      DrawOffset(0, 0, i, FHeight, FWidth - i, 0);
      i := i + FThickness;
    end;

    atfCoverUp:
    begin
      DrawOffset(0, FHeight - i, FWidth, i, 0, 0);
      i := i + FThickness;
    end;

    atfCoverLeftDown..atfCoverRightUp:
    begin
      case FEffect of
        atfCoverLeftDown:
          DrawOffset(FWidth - x, 0, x, y, 0, FHeight - y);
        atfCoverLeftUp:
          DrawOffset(FWidth - x, FHeight - y, x, y, 0, 0);
        atfCoverRightDown:
          DrawOffset(0, 0, x, y, FWidth - x, FHeight - y);
        atfCoverRightUp:
          DrawOffset(0, FHeight - y, x, y, FWidth - x, 0);
      else
      end;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfCoverCenterHorzIn:
    begin
      DrawOffset(0, 0, x, FHeight, mid - x, 0);
      DrawOffset(FWidth - x, 0, x, FHeight, mid, 0);
      x := x + FThickness;
    end;

    atfCoverCenterHorzOut:
    begin
      DrawOffset(x1, 0, mid-x1, FHeight, 0, 0);
      DrawOffset(mid, 0, x2-mid, FHeight, FWidth - x2 + mid, 0);
      x1 := x1 - FThickness;
      x2 := x2 + FThickness;
    end;

    atfCoverCenterVertIn:
    begin
      DrawOffset(0, 0, FWidth, y, 0, mid - y);
      DrawOffset(0, FHeight - y, FWidth, y, 0, mid);
      y := y + FThickness;
    end;

    atfCoverCenterVertOut:
    begin
      DrawOffset(0, y1, FWidth, mid-y1, 0, 0);
      DrawOffset(0, mid, FWidth, y2-mid, 0, FHeight - y2 + mid);
      y1 := y1 - FThickness;
      y2 := y2 + FThickness;
    end;

    atfUncoverDown:
    begin
      DrawOffset(0, 0, FWidth, i, 0, 0);
      DrawCopyOffset(0, i, FWidth, FHeight - i, 0, 0);
      i := i + FThickness;
    end;

    atfUncoverLeft:
    begin
      DrawOffset(FWidth - i, 0, i, FHeight, FWidth - i, 0);
      DrawCopyOffset(0, 0, FWidth - i, FHeight, i, 0);
      i := i + FThickness;
    end;

    atfUncoverRight:
    begin
      DrawOffset(0, 0, i, FHeight, 0, 0);
      DrawCopyOffset(i, 0, FWidth - i, FHeight, 0, 0);
      i := i + FThickness;
    end;

    atfUncoverUp:
    begin
      DrawOffset(0, FHeight - i, FWidth, i, 0, FHeight - i);
      DrawCopyOffset(0, 0, FWidth, FHeight - i, 0, i);
      i := i + FThickness;
    end;

    atfUncoverLeftDown..atfUncoverRightUp:
    begin
      case FEffect of
        atfUncoverLeftDown:
        begin
          DrawOffset(0, 0, FWidth, y, 0, 0);
          DrawOffset(FWidth - x, y, x, FHeight-y, FWidth - x, y);
          DrawCopyOffset(0, y, FWidth - x, FHeight-y, x, 0);
        end;
        atfUncoverLeftUp:
        begin
          DrawOffset(0, FHeight - y, FWidth, y, 0, FHeight - y);
          DrawOffset(FWidth - x, 0, x, FHeight-y, FWidth - x, 0);
          DrawCopyOffset(0, 0, FWidth - x, FHeight-y, x, y);
        end;
        atfUncoverRightDown:
        begin
          DrawOffset(0, 0, FWidth, y, 0, 0);
          DrawOffset(0, y, x, FHeight-y, 0, y);
          DrawCopyOffset(x, y, FWidth - x, FHeight-y, 0, 0);
        end;
        atfUncoverRightUp:
        begin
          DrawOffset(0, FHeight - y, FWidth, y, 0, FHeight - y);
          DrawOffset(0, 0, x, FHeight-y, 0, 0);
          DrawCopyOffset(x, 0, FWidth - x, FHeight-y, 0, y);
        end;
      else
      end;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfUncoverCenterHorzIn:
    begin
      CopySourceToBuffer;
      DrawCopyToBuffer(x, 0, mid - x, FHeight, 0, 0);
      DrawCopyToBuffer(mid, 0, mid - x, FHeight, FWidth - mid + x, 0);
      CopyBufferToDest;
      x := x + FThickness;
    end;

    atfUncoverCenterHorzOut:
    begin
      CopySourceToBuffer;
      DrawCopyToBuffer(0, 0, x1, FHeight, mid - x1, 0);
      DrawCopyToBuffer(x2, 0, FWidth - x2, FHeight, mid, 0);
      CopyBufferToDest;
      x1 := x1 - FThickness;
      x2 := x2 + FThickness;
    end;

    atfUncoverCenterVertIn:
    begin
      CopySourceToBuffer;
      DrawCopyToBuffer(0, y, FWidth, mid - y, 0, 0);
      DrawCopyToBuffer(0, mid, FWidth, mid - y, 0, FHeight - mid + y);
      CopyBufferToDest;
      y := y + FThickness;
    end;

    atfUncoverCenterVertOut:
    begin
      CopySourceToBuffer;
      DrawCopyToBuffer(0, 0, FWidth, y1, 0, mid - y1);
      DrawCopyToBuffer(0, y2, FWidth, FHeight - y2, 0, mid);
      CopyBufferToDest;
      y1 := y1 - FThickness;
      y2 := y2 + FThickness;
    end;

    atfWipeDown:
    begin
      DrawOffset(0, i, FWidth, FThickness, 0, i);
      i := i + FThickness;
    end;

    atfWipeLeft:
    begin
      DrawOffset(FWidth - i, 0, FThickness, FHeight, FWidth - i, 0);
      i := i + FThickness;
    end;

    atfWipeRight:
    begin
      DrawOffset(i, 0, FThickness, FHeight, i, 0);
      i := i + FThickness;
    end;

    atfWipeUp:
    begin
      DrawOffset(0, FHeight - i, FWidth, FThickness, 0, FHeight - i);
      i := i + FThickness;
    end;

    atfWipeLeftDown:
    begin
      i := i + 1;
      if x1 > 0 then
        x1 := x1 - FThickness
      else
        y1 := y1 + FThickness;
      x2 := x1;
      y2 := y1;
      repeat
        DrawOffset(x2, y2, FThickness, FThickness, x2, y2);
        if Terminated then Exit;
        x2 := x2 + FThickness;
        y2 := y2 + FThickness;
      until (x2 < 0) or (y2 >= FHeight) or (x2 >= FWidth);
    end;

    atfWipeLeftUp:
    begin
      i := i + 1;
      if y1 > 0 then
        y1 := y1 - FThickness
      else
        x1 := x1 - FThickness;
      x2 := x1;
      y2 := y1;
      repeat
        DrawOffset(x2, y2, FThickness, FThickness, x2, y2);
        if Terminated then Exit;
        x2 := x2 - FThickness;
        y2 := y2 + FThickness;
      until (y2 >= FHeight) or (x2 < -FThickness);
    end;

    atfWipeRightDown:
    begin
      i := i + 1;
      if x1 < FWidth then
        x1 := x1 + FThickness
      else
        y1 := y1 + FThickness;
      x2 := x1;
      y2 := y1;
      repeat
        DrawOffset(x2, y2, FThickness, FThickness, x2, y2);
        if Terminated then Exit;
        x2 := x2 - FThickness;
        y2 := y2 + FThickness;
      until (x2 < -FThickness) or (y2 >= FHeight) or (x2 >= FWidth);
    end;

    atfWipeRightUp:
    begin
      i := i + 1;
      if y1 > 0 then
        y1 := y1 - FThickness
      else
        x1 := x1 + FThickness;
      x2 := x1;
      y2 := y1;
      repeat
        DrawOffset(x2, y2, FThickness, FThickness, x2, y2);
        if Terminated then Exit;
        x2 := x2 + FThickness;
        y2 := y2 + FThickness;
      until (y2 < 0) or (y2 >= FHeight) or (x2 >= FWidth);
    end;

    atfWipeRectLeftDown:
    begin
      DrawOffset(FWidth - x, 0, x, y, FWidth - x, 0);
      if Terminated then Exit;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfWipeRectLeftUp:
    begin
      DrawOffset(FWidth - x, FHeight - y, x, y, FWidth - x, FHeight - y);
      if Terminated then Exit;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfWipeRectRightDown:
    begin
      DrawOffset(0, 0, x, y, 0, 0);
      if Terminated then Exit;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfWipeRectRightUp:
    begin
      DrawOffset(0, FHeight - y, x, y, 0, FHeight - y);
      if Terminated then Exit;
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfCropLeftDown:
    begin
      DrawOffset(0, 0, FWidth, y, 0, 0);
      DrawOffset(FWidth - x, y, x, FHeight-y, FWidth - x, y);
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfCropLeftUp:
    begin
      DrawOffset(0, FHeight - y, FWidth, y, 0, FHeight - y);
      DrawOffset(FWidth - x, 0, x, FHeight-y, FWidth - x, 0);
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfCropRightDown:
    begin
      DrawOffset(0, 0, FWidth, y, 0, 0);
      DrawOffset(0, y, x, FHeight-y, 0, y);
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfCropRightUp:
    begin
      DrawOffset(0, FHeight - y, FWidth, y, 0, FHeight - y);
      DrawOffset(0, 0, x, FHeight-y, 0, 0);
      if FWidth > FHeight then
      begin
        y := y + FThickness;
        x := y * FWidth div FHeight;
      end
      else
      begin
        x := x + FThickness;
        y := x * FHeight div FWidth;
      end;
    end;

    atfWipeCenterHorzIn:
    begin
      DrawOffset(0, 0, x1, FHeight, 0, 0);
      DrawOffset(x2, 0, FWidth - x2, FHeight, x2, 0);
      x1 := x1 + FThickness;
      x2 := x2 - FThickness;
    end;

    atfWipeCenterHorzOut:
    begin
      DrawOffset(x1, 0, x2-x1, FHeight, x1, 0);
      x1 := x1 - FThickness;
      x2 := x2 + FThickness;
    end;

    atfWipeCenterVertIn:
    begin
      DrawOffset(0, 0, FWidth, y1, 0, 0);
      DrawOffset(0, y2, FWidth, FHeight - y2, 0, y2);
      y1 := y1 + FThickness;
      y2 := y2 - FThickness;
    end;

    atfWipeCenterVertOut:
    begin
      DrawOffset(0, y1, FWidth, y2-y1, 0, y1);
      y1 := y1 - FThickness;
      y2 := y2 + FThickness;
    end;

    atfWipeClockwise, atfWipeAntiClockwise:
    begin
      ptPolyMask[1] := ptPolyMask[2];
      ptPolyMask[2] := ClockPoint(i);
      newClipRgn := 0;
      rgnFound := -1;
      oldClipRgn := CreateRectRgn(0,0,1,1);
      try
        newClipRgn := CreatePolygonRgn (ptPolyMask, 3, WINDING);
        if Terminated then Exit;
        if newClipRgn <> 0 then
        begin
          if not GetViewportOrgEx(FDest.Handle, ptDestOffset) then Exit;
          if Terminated then Exit;
          OffsetRgn(newClipRgn, ptDestOffset.X + FDestX, ptDestOffset.Y + FDestY);
          if Terminated then Exit;
          rgnFound := GetClipRgn(FDest.Handle, oldClipRgn);
          if rgnFound = -1 then Exit;
          if SelectClipRgn(FDest.Handle, newClipRgn) = ERROR then Exit;
          if Terminated then Exit;
          DrawOffset(0, 0, FWidth, FHeight, 0, 0);
        end;
      finally
        if newClipRgn <> 0 then
          DeleteObject(newClipRgn);
        if rgnFound = 1 then
          SelectClipRgn(FDest.Handle, oldClipRgn)
        else if rgnFound = 0 then
          SelectClipRgn(FDest.Handle, 0);
        DeleteObject(oldClipRgn);
      end;
      i := i + 1;
    end;

    atfCut:
      DrawOffset(0, 0, FWidth, FHeight, 0, 0);

    atfCutThroughBlack:
    begin
      if i = 0 then
      begin
          if not Terminated then
          FillRect(
            FDest.Handle,
            Rect(FDestX, FDestY, FDestX + FWidth, FDestY + FHeight),
            GetStockObject(BLACK_BRUSH));
      end
      else
        DrawOffset(0, 0, FWidth, FHeight, 0, 0);
      i := i + 1;
    end;

    atfFadeThroughBlack:
    begin
      case i of
        0: BrushFadeOut(1);
        1: BrushFadeOut(2);
        2: BrushFadeOut(3);
        3: BrushFadeOut(4);
        4: BrushFadeIn(4);
        5: BrushFadeIn(3);
        6: BrushFadeIn(2);
        7: BrushFadeIn(1);
      else
      end;
      i := i + 1;
    end;

  else
  end;
end;

{------------------------------------------------------------------------------}

function TabcTransitionThread.StillDrawingFrames: Boolean;
begin
  case FEffect of
    atfNone:
      Result := False;
    atfSkipHorizontal:
      Result := ((j > 0) or (i < FWidth));

    atfSkipVertical:
      Result := ((j > 0) or (i < FHeight));

    atfSkipDiagonals:
      Result := (id < ndiags);

    atfSkipDiagonalSquares:
      Result := (id < nFrames);

    atfSkipRectangles:
      Result := (xl <= halfsize) or (yl <= halfsize);

    atfBoxIn:
      Result := (x1 < x2) and (y1 < y2);

    atfBoxOut:
      Result := (x >= 0) or (y >= 0);

    atfBlindsHorz, atfBlindsVert:
      Result := (i < FThickness);

    atfDissolve, atfDissolveHorz, atfDissolveVert:
      Result := i < RANDARRAYSIZE;

    atfWindIn:
      Result := (xMin < xMax) or (yMin < yMax);

    atfWindOut:
      Result := (xMin > -dX) or (xMax < FWidth) or (yMin > -dY) or (yMax < FHeight);

    atfPourDown:
      Result := (i >= 0);

    atfPourLeft:
      Result := (i < FWidth);

    atfPourRight:
      Result := (i >= 0);

    atfPourUp:
      Result := (i < FHeight);

    atfQuadCover, atfQuadWipeIn:
      Result := (x < midx);

    atfQuadUncover, atfQuadWipeOut:
      Result := (x > 0);

    atfQuadUnfold, atfQuadExpand:
      Result := (x < midx);

    atfQuadFold, atfQuadFoldCopy, atfQuadShrink, atfQuadShrinkCopy:
      Result := (x > 0);

    atfShapeSawTooth:
      Result := (i < SAWTOOTH_FRAMES);

    atfShapeStarBurst..atfShapeSpiralAntiClockwise:
      Result := (i < SPIRAL_FRAMES);

    atfFoldDown, atfFoldUp:
      Result := (i < FHeight);

    atfFoldLeft, atfFoldRight:
      Result := (i < FWidth);

    atfFoldCenterHorz:
      Result := (x1 < mid);

    atfFoldCenterVert:
      Result := (y1 < mid);

    atfUnfoldDown, atfUnfoldUp:
      Result := (i < FHeight);

    atfUnfoldLeft, atfUnfoldRight:
      Result := (i < FWidth);

    atfUnfoldCenterHorz:
      Result := (x1 > 0);

    atfUnfoldCenterVert:
      Result := (y1 > 0);

    atfSquashDown, atfSquashUp:
      Result := (t < FHeight);

    atfSquashLeft, atfSquashRight:
      Result := (t < FWidth);

    atfSquashCenterHorz:
      Result := (x1 < mid);

    atfSquashCenterVert:
      Result := (y1 < mid);

    atfUnsquashDown, atfUnsquashUp:
      Result := (t < FHeight);

    atfUnsquashLeft, atfUnsquashRight:
      Result := (t < FWidth);

    atfUnsquashCenterHorz:
      Result := (x1 > 0);

    atfUnsquashCenterVert:
      Result := (y1 > 0);

    atfShrinkLeftDown..atfShrinkRightUp:
      Result := (x < FWidth) or (y < FHeight);

    atfShrinkCenter:
      Result := (x1 < x2) and (y1 < y2);

    atfExpandLeftDown..atfExpandRightUp:
      Result := (x < FWidth) or (y < FHeight);

    atfExpandCenter:
      Result := (x >= 0) or (y >= 0);

    atfZoomIn:
      Result := (x1 < 0) and (y1 < 0);

    atfZoomOut:
      Result := (i < ZOOM_OUT_FRAMES);

    atfZoomDown:
      Result := (y < 0);

    atfZoomLeft:
      Result := (x > FWidth);

    atfZoomRight:
      Result := (x < 0);

    atfZoomUp:
      Result := (y > FHeight);

    atfZoomLeftDown:
      Result := (y < 0);

    atfZoomLeftUp:
      Result := (y > FHeight);

    atfZoomRightDown:
      Result := (y < 0);

    atfZoomRightUp:
      Result := (x < 0);

    atfPushDown, atfPushUp:
      Result := (i < FHeight);

    atfPushLeft, atfPushRight:
      Result := (i < FWidth);

    atfCoverDown, atfCoverUp:
      Result := (i < FHeight);

    atfCoverLeft, atfCoverRight:
      Result := (i < FWidth);

    atfCoverLeftDown..atfCoverRightUp:
      Result := (x < FWidth) and (y < FHeight);

    atfCoverCenterHorzIn:
      Result := (x < mid);

    atfCoverCenterHorzOut:
      Result := (x1 > 0);

    atfCoverCenterVertIn:
      Result := (y < mid);

    atfCoverCenterVertOut:
      Result := (y1 > 0);

    atfUncoverDown, atfUncoverUp:
      Result := (i < FHeight);

    atfUncoverLeft, atfUncoverRight:
      Result := (i < FWidth);

    atfUncoverLeftDown..atfUncoverRightUp:
      Result := (x < FWidth) and (y < FHeight);

    atfUncoverCenterHorzIn:
      Result := (x < mid);

    atfUncoverCenterHorzOut:
      Result := (x1 > 0);

    atfUncoverCenterVertIn:
      Result := (y < mid);

    atfUncoverCenterVertOut:
      Result := (y1 > 0);

    atfWipeDown, atfWipeUp:
      Result := (i < FHeight);

    atfWipeLeft, atfWipeRight:
      Result := (i < FWidth);

    atfWipeLeftDown..atfWipeRightUp:
      Result := i < nFrames;

    atfWipeRectLeftDown..atfCropRightUp:
      Result := (x < FWidth) and (y < FHeight);

    atfWipeCenterHorzIn:
      Result := (x1 < mid);

    atfWipeCenterHorzOut:
      Result := (x1 > 0);

    atfWipeCenterVertIn:
      Result := (y1 < mid);

    atfWipeCenterVertOut:
      Result := (y1 > 0);

    atfWipeClockwise, atfWipeAntiClockwise:
      Result := i < 16;

    atfCut:
      Result := False;

    atfCutThroughBlack:
      Result := i < 2;

    atfFadeThroughBlack:
      Result := i < 8;

  else
    Result := False;
  end;
end;

procedure TabcTransitionThread.DrawFrames;
begin
  if Terminated then Exit;
  FStartNextFrame := GetTickCount + FFrameDuration;
  while StillDrawingFrames do
  begin
    if TryLockSourceDest then
    try
      DrawFrame;
    finally
      UnlockSourceDest;
    end;
    if Terminated then Exit;
    SleepUntilNextFrame;
    if Terminated then Exit;
  end;
end;

function abcThreadTransition (
           Effect: TabcTransitionEffect;
           const EffectSelection: TList;
           var EffectSelectionIndex: Integer;
           const Source, Dest: TCanvas;
           DestX, DestY, Width, Height: Integer;
           SourceX, SourceY: Integer;
           Thickness, Duration, PauseAfter: Integer;
           OnCancel: TabcCancelEvent;
           OnTerminate: TNotifyEvent;
           Suspended: Boolean;
           SingleFrameMode: Boolean
         ): TabcTransitionThread;
var
  EffectToCreate : TabcTransitionEffect;
  i: integer;
begin
  Result := nil;
  if (Source = nil) or
     (Dest = nil) or
     (Width = 0) or
     (Height = 0) or
     (Thickness = 0) then
    Exit;

  if Effect in [atfRandom, atfSelectionRandom, atfSelectionSequence] then
  {these special effects values need substitution}
  begin
    if Effect = atfRandom then
      EffectToCreate := TabcTransitionEffect(Random(Ord(atfRandom)-1))
    else if Effect = atfSelectionRandom then
    begin
      if (EffectSelection <> nil) and (EffectSelection.Count > 0) then
      begin
        i := Integer(EffectSelection[Random(EffectSelection.Count)]);
        if (i >= 0) and (i <= Ord(atfRandom)) then
          EffectToCreate := TabcTransitionEffect(i)
        else
          EffectToCreate := atfNone;
      end
      else
        EffectToCreate := atfNone;
    end
    else
    begin
      if (EffectSelection <> nil) and (EffectSelection.Count > 0) then
      begin
        EffectSelectionIndex := EffectSelectionIndex + 1;
        if EffectSelectionIndex >= EffectSelection.Count then
          EffectSelectionIndex := 0;
        i := Integer(EffectSelection.Items[EffectSelectionIndex]);
        if (i >= 0) and (i <= Ord(atfRandom)) then
          EffectToCreate := TabcTransitionEffect(i)
        else
          EffectToCreate := atfNone;
      end
      else
        EffectToCreate := atfNone;
    end;
  end
  else
    EffectToCreate := Effect;

  Result := TabcTransitionThread.Create(
    Source, Dest,
    DestX, DestY, Width, Height,
    SourceX, SourceY,
    EffectToCreate,
    Thickness, Duration, PauseAfter,
    OnCancel,
    OnTerminate,
    Suspended,
    SingleFrameMode
    );

end;

function abcSizedEffect(Effect: TabcTransitionEffect): Boolean;
{returns true if effect size/thickness parameter is used}
begin
  Result := not (Effect in [
    atfSkipRectangles,
    atfWipeClockwise,
    atfWipeAntiClockwise,
    atfNone,
    atfCut,
    atfCutThroughBlack,
    atfFadeThroughBlack,
    atfShapeSawtooth,
    atfShapeStarBurst,
    atfShapeBroadStar,
    atfShapeParallelogram,
    atfShapeLens1,
    atfShapeLens2,
    atfShapeSpiralClockwise,
    atfShapeSpiralAntiClockWise]);
end;

initialization

finalization
  RandPoints.Free;
  RandPoints := nil;
  
end.
