unit abcutil;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, SysUtils, Messages, Classes, Controls, ExtCtrls, Forms, Dialogs, ShellAPI;

type
  TLaunchChecks = (lcDate, lcTime, lcTimeLapse, lcWeekDay, lcMonthDay);
  TabcApplyChecks = set of TLaunchChecks;

  TabcLaunchWeekDay = (ldSunday, ldMonday, ldTuesday, ldWednesday,
                  ldThursday, ldFriday, ldSaturday);

  TabcDay = 1..31;
  TabcMonth = 1..12;
  TabcYear = 1..9999;

  TabcHour = 0..23;
  TabcMin = 0..59;
  TabcSec = 0..59;
  TabcMSec = 0..999;

  {TabcDateNum}

  TabcDateNum = class(TPersistent)
  private
    FDay: TabcDay;
    FMonth: TabcMonth;
    FYear: TabcYear;
    FOnChange: TNotifyEvent;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    procedure SetDate(ADate: TDateTime);
    procedure SetDay(Value: TabcDay);
    procedure SetMonth(Value: TabcMonth);
    procedure SetYear(Value: TabcYear);
    function GetDate: TDateTime;
    property Date: TDateTime read GetDate write SetDate;
  published
    property Day: TabcDay read FDay write SetDay default 1;
    property Month: TabcMonth read FMonth write SetMonth default 1;
    property Year: TabcYear read FYear write SetYear default 1;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {TabcTimeNum}

  TabcTimeNum = class(TPersistent)
  private
    FHour: TabcHour;
    FMin: TabcMin;
    FSec: TabcSec;
    FMSec: TabcMSec;
    FOnChange: TNotifyEvent;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    procedure SetHour(Value: TabcHour);
    procedure SetMin(Value: TabcMin);
    procedure SetSec(Value: TabcSec);
    procedure SetMSec(Value: TabcMSec);
    procedure SetTime(ATime: TDateTime);
    function GetTime: TDateTime;
    property Time: TDateTime read GetTime write SetTime;
  published
    property Hour: TabcHour read FHour write SetHour;
    property Min: TabcMin read FMin write SetMin;
    property Sec: TabcSec read FSec write SetSec;
    property MSec: TabcMSec read FMSec write SetMSec;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {TabcBeforeLaunchEvent}

  TabcBeforeLaunchEvent = procedure (Sender: TObject; var Cancel: Boolean) of object;

  {TabcAfterLaunchEvent}

  TabcAfterLaunchEvent = procedure (Sender: TObject; Result: Integer) of object;

  { TabcShowWindowStyle }

  TabcShowWindowStyle = (aswHide, aswMaximize, aswMinimize, aswRestore,
                         aswShow, aswShowDefault,
                         aswShowMaximized, aswShowMinimized,
                         aswShowMinNoActivate, aswShowNA, aswShowNoActivate,
                         aswShowNormal);

  function abcASWEnumToSW_CONST(Value: TabcShowWindowStyle): integer;

type

  {TabcLauncher}

  TabcLaunchType = (altProcessCommandLine, altShellExecFile, altShellExecFiles);

  TabcShellMaskOption = (asmConnectNetDrive, asmExpandEnvStrings, asmDDEWait,
    asmNoErrorMsg, asmKeepProcess, asmAllVerbs);

  TabcShellMaskOptions = set of TabcShellMaskOption;

  TabcShellExecParams = class(TPersistent)
  private
    FDirectory: string;
    FParameters: string;
    FFileName: string;
    FVerb: string;
    FOptions: TabcShellMaskOptions;
    FFileNames: TStrings;
    procedure SetFileNames(const Value: TStrings);
    function GetMaskFlags: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property FileName: string read FFileName write FFileName;
    property FileNames: TStrings read FFileNames write SetFileNames;
    property Parameters: string read FParameters write FParameters;
    property Directory: string read FDirectory write FDirectory;
    property Verb: string read FVerb write FVerb;
    property Options: TabcShellMaskOptions read FOptions write FOptions
      default [asmKeepProcess, asmAllVerbs];
  end;

  TabcLauncher = class(TComponent)
  private
    FContinuous: Boolean;
    FLaunchProcessID: DWORD;
    FLaunchResult: Integer;
    FLaunchError: DWORD;
    FTimer: TTimer;
    FCommandLine: string;
    FInterval: Word;
    FWeekDay: TabcLaunchWeekDay;
    FMonthDay: TabcDay;
    FDate: TabcDateNum;
    FTime: TabcTimeNum;
    FTimeLapse: TabcTimeNum;
    FNextLapseTime: TDateTime;
    FChecks: TabcApplyChecks;
    FEnabled: Boolean;
    FBeforeLaunch: TabcBeforeLaunchEvent;
    FAfterLaunch: TabcAfterLaunchEvent;
    FOnTimer: TNotifyEvent;
    FWait: Boolean;
    FWindowState: TabcShowWindowStyle;
    FBeforeWait: TabcAfterLaunchEvent;
    FLaunchType: TabcLaunchType;
    FShellParams: TabcShellExecParams;
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Word);
    procedure SetDate(Value: TabcDateNum);
    procedure SetTime(Value: TabcTimeNum);
    procedure SetTimeLapse(Value: TabcTimeNum);
    procedure SetChecks(Value: TabcApplyChecks);
    procedure SetShellParams(const Value: TabcShellExecParams);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property LaunchResult: Integer read FLaunchResult write FLaunchResult;
    property LaunchError: DWORD read FLaunchError;
    property LaunchProcessID: DWORD read FLaunchProcessID;
    function GetDateTime: TDateTime;
    procedure SetDateTime(Value: TDateTime);
    procedure Reset;
    procedure ResetDateTime(Value: TDateTime);
    procedure CheckLaunch(Sender: TObject);
    procedure LaunchSequence(Sender: TObject);
    function ExecCommandLine(Sender: TObject): Integer;
    function ExecShellFile(Sender: TObject): Integer;
    function ExecShellFiles(Sender: TObject): Integer;
    function Execute: Boolean;
    property NextLapseTime: TDateTime read FNextLapseTime;
  published
    property Checks: TabcApplyChecks read FChecks write SetChecks default [lcTime];
    property CommandLine: string read FCommandLine write FCommandLine;
    property Continuous: Boolean read FContinuous write FContinuous default False;
    property Date: TabcDateNum read FDate write SetDate;
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Word read FInterval write SetInterval default 1000;
    property MonthDay: TabcDay read FMonthDay write FMonthDay default 1;
    property Time: TabcTimeNum read FTime write SetTime;
    property TimeLapse: TabcTimeNum read FTimeLapse write SetTimeLapse;
    property Wait: Boolean read FWait write FWait;
    property WeekDay: TabcLaunchWeekDay read FWeekDay write FWeekDay;
    property WindowState: TabcShowWindowStyle read FWindowState write FWindowState
      default aswShowDefault;
    property LaunchType: TabcLaunchType read FLaunchType write FLaunchType
      default altProcessCommandLine;
    property ShellParams: TabcShellExecParams read FShellParams write SetShellParams;
    property BeforeLaunch: TabcBeforeLaunchEvent read FBeforeLaunch write FBeforeLaunch;
    property AfterLaunch: TabcAfterLaunchEvent read FAfterLaunch write FAfterLaunch;
    property BeforeWait: TabcAfterLaunchEvent read FBeforeWait write FBeforeWait;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  end;

  {TabcSingleInstance}

  TabcHaltEvent = procedure (Sender: TObject;
    PreviousInstances: Integer; var HaltThisInstance: Boolean) of object;

  TabcTerminateMethod = (atmAppTerminate, atmCloseMainForm, atmExitProcess, atmHalt, atmTerminateProcess);

  TabcSingleInstance = class(TComponent)
  private
    FMutex: THandle;
    FSemaphore: THandle;
    FStopMsg: string;
    FActivate: Boolean;
    FClosePrevious: Boolean;
    FMultipleInstances: Boolean;
    FOnPreviousInstance: TabcHaltEvent;
    FTerminateBy: TabcTerminateMethod;
    procedure CheckTerminate;
    function GetMutexName: string;
    function GetInstanceCount: Integer;
    function GetSemaphoreName: string;
    procedure InitSemaphore;
    procedure SetActivate(Value: Boolean);
    procedure SetClosePrevious(Value: Boolean);
    procedure SetMultipleInstances(Value: Boolean);
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure HaltIfRunning;
    property InstanceCount: Integer read GetInstanceCount;
  published
    property Activate: Boolean read FActivate write SetActivate default True;
    property ClosePrevious: Boolean read FClosePrevious write SetClosePrevious default False;
    property MultipleInstances: Boolean read FMultipleInstances write SetMultipleInstances default False;
    property StopMsg: string read FStopMsg write FStopMsg;
    property TerminateBy: TabcTerminateMethod read FTerminateBy write FTerminateBy default atmAppTerminate;
    property OnPreviousInstance: TabcHaltEvent read FOnPreviousInstance write FOnPreviousInstance;
  end;

procedure abcExploreFolder(Path: string; Maximized: Boolean);
function abcExecuteFile(FileName: string; Maximized: Boolean): Boolean;
function abcPrintFile(FileName: string): Boolean;

implementation

function abcASWEnumToSW_CONST(Value: TabcShowWindowStyle): integer;
const
  absSWConsts: array[TabcShowWindowStyle] of integer = (
                        SW_HIDE, SW_MAXIMIZE, SW_MINIMIZE, SW_RESTORE,
                        SW_SHOW, SW_SHOWDEFAULT,
                        SW_SHOWMAXIMIZED, SW_SHOWMINIMIZED,
                        SW_SHOWMINNOACTIVE, SW_SHOWNA, SW_SHOWNOACTIVATE,
                        SW_SHOWNORMAL);
begin
  Result := absSWConsts[Value];
end;

constructor TabcTimeNum.Create;
begin
  SetTime(Now);
end;

procedure TabcTimeNum.Assign(Source: TPersistent);
begin
  if Source is TabcTimeNum then
    SetTime(TabcTimeNum(Source).GetTime)
  else
    inherited Assign(Source);
end;

procedure TabcTimeNum.SetTime(ATime: TDateTime);
var
  Hour1,Min1,Sec1, MSec1: Word;
begin
  if ATime <> GetTime then begin
    DecodeTime(ATime, Hour1, Min1, Sec1, MSec1);
    FHour := Hour1;
    FMin := Min1;
    FSec := Sec1;
    FMSec := MSec1;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

function TabcTimeNum.GetTime: TDateTime;
begin
  Result := EncodeTime(FHour, FMin, FSec, FMSec);
end;

procedure TabcTimeNum.SetHour(Value: TabcHour);
begin
  if Value <> FHour then begin
    FHour := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcTimeNum.SetMin(Value: TabcMin);
begin
  if Value <> FMin then begin
    FMin := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcTimeNum.SetSec(Value: TabcSec);
begin
  if Value <> FSec then begin
    FSec := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcTimeNum.SetMSec(Value: TabcMSec);
begin
  if Value <> FMSec then begin
    FMSec := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

constructor TabcDateNum.Create;
begin
  FDay := 1;
  FMonth := 1;
  FYear := 1;
  SetDate(Now);
end;

procedure TabcDateNum.Assign(Source: TPersistent);
begin
  if Source is TabcDateNum then
    SetDate(TabcDateNum(Source).GetDate)
  else
    inherited Assign(Source);
end;

procedure TabcDateNum.SetDate(ADate: TDateTime);
var
  Year1,Month1,Day1: Word;
begin
  if ADate <> GetDate then begin
    DecodeDate(ADate, Year1, Month1, Day1);
    FYear := Year1;
    FMonth := Month1;
    FDay := Day1;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

function TabcDateNum.GetDate: TDateTime;
begin
  try
    Result := EncodeDate(FYear, FMonth, FDay);
  except
    Result := 0;
  end;
end;

procedure TabcDateNum.SetDay(Value: TabcDay);
begin
  if Value <> FDay then begin
    FDay := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcDateNum.SetMonth(Value: TabcMonth);
begin
  if Value <> FMonth then begin
    FMonth := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TabcDateNum.SetYear(Value: TabcYear);
begin
  if Value <> FYear then begin
    FYear := Value;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

{ TabcLauncher }

constructor TabcLauncher.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLaunchResult := -1;
  FDate := TabcDateNum.Create;
  FTime := TabcTimeNum.Create;
  FTimeLapse := TabcTimeNum.Create;
  FTimeLapse.Time := 0;
  if not(csDesigning in ComponentState) then begin
    FTimer := TTimer.Create(Self);
    FTimer.OnTimer := CheckLaunch;
  end;
  Interval := 1000;
  Enabled := True;
  FChecks := [lcTime];
  FMonthDay := 1;
  FWindowState := aswShowDefault;
  FShellParams := TabcShellExecParams.Create;
end;

destructor TabcLauncher.Destroy;
begin
  FTimer.Free;
  FDate.Free;
  FTime.Free;
  FTimeLapse.Free;
  FShellParams.Free;
  inherited Destroy;
end;

procedure TabcLauncher.CheckLaunch(Sender: TObject);

  function LaunchDayofMonth(Value: TabcDay): Boolean;
  var
    Year1,Month1,Day1: Word;
  begin
    Result := False;
    DecodeDate(Now,Year1,Month1,Day1);
    if Day1 = Value then
      Result := True
    else
      if (Value = 31) and (Day1 = 30) and (Month1 in [4,6,9,11]) then
        Result := True
      else
        if (Value = 31) and (Day1 in [28,29]) and (Month1 = 2) then
          Result := True;
  end;

  function TimeToLaunch: Boolean;
  var
    LaunchDateTime: TDateTime;
  begin
    Result := True;
    LaunchDateTime := Now;
    if (lcDate in FChecks) then
      LaunchDateTime := EncodeDate(FDate.Year,FDate.Month,FDate.Day)
        + Frac(LaunchDateTime);
    if (lcTime in FChecks) then
      LaunchDateTime := Trunc(LaunchDateTime)
        + EncodeTime(FTime.Hour, FTime.Min, FTime.Sec, FTime.MSec);
    {Perform each required check}
    if (lcTimeLapse in FChecks) then
      Result := Result and (LaunchDateTime >= FNextLapseTime);
    if (lcDate in FChecks) or (lcTime in FChecks) then
      Result := Result and (Now >= LaunchDateTime);
    if (lcWeekDay in FChecks) then
      Result := Result and (Ord(FWeekDay) = DayofWeek(Now)-1);
    if (lcMonthDay in FChecks) then
      Result := Result and LaunchDayofMonth(FMonthDay);
  end;

{Main proc starts here}
begin
  if Assigned(FOnTimer) then FOnTimer(Sender);
  if TimetoLaunch then
    LaunchSequence(Sender);
end;

procedure TabcLauncher.LaunchSequence(Sender: TObject);
var
  Cancelled: Boolean;
  wasEnabled: Boolean;
begin
  wasEnabled := Enabled;
  {Turn off the timer so it won't trigger more than once}
  FTimer.Enabled := False;
  {Execute BeforeLaunch event procedure}
  Cancelled := False;
  if Assigned(FBeforeLaunch) then BeforeLaunch(Sender, Cancelled);
  if Cancelled then
    FLaunchResult := -2
  else
    case LaunchType of
      altProcessCommandLine:
        FLaunchResult := ExecCommandLine(Sender);
      altShellExecFile:
        FLaunchResult := ExecShellFile(Sender);
      altShellExecFiles:
        FLaunchResult := ExecShellFiles(Sender);
    end;
  if (lcTimeLapse in FChecks) then
    FNextLapseTime := Now + FTimeLapse.Time;

  if FContinuous then
    FTimer.Enabled := wasEnabled
  else
    FEnabled := False;

  {Execute AfterLaunch event procedure}
  if Assigned(FAfterLaunch) then AfterLaunch(Sender, FLaunchResult);
end;

function TabcLauncher.Execute: Boolean;
begin
  Result := False;
  case LaunchType of
    altProcessCommandLine:
      Result := ExecCommandLine(Self) > 32;
    altShellExecFile:
      Result := ExecShellFile(Self) > 32;
    altShellExecFiles:
      Result := ExecShellFiles(Self) > 32;
  end;
end;

function TabcLauncher.ExecCommandLine(Sender: TObject): integer;
var
  ProcessInfo: TProcessInformation;
  StartupInfo: TStartupInfo;
begin
  FLaunchError := 0;
  FLaunchProcessID := 0;
  if FCommandLine <> '' then
  begin
    {initialize params}
    ZeroMemory(@StartupInfo, SizeOf(TStartupInfo));
    with StartupInfo do
    begin
      cb := SizeOf(TStartupInfo);
      dwFlags := STARTF_USESHOWWINDOW;
      wShowWindow := abcASWEnumToSW_CONST(FWindowState);
    end;
    if CreateProcess(nil, PChar(FCommandLine), nil, nil, False, 0, nil, nil, StartupInfo, ProcessInfo) then
    begin
      CloseHandle(ProcessInfo.hThread);
      Result := ProcessInfo.hProcess;
      FLaunchProcessID := ProcessInfo.dwProcessId;
      if FWait then
      begin
        {Execute BeforeWait event procedure}
        if Assigned(FBeforeWait) then FBeforeWait(Sender, Result);
        WaitForSingleObject(Result, INFINITE);
      end;
      CloseHandle(Result);
    end
    else
    begin
      FLaunchError := GetLastError;
      Result := 0
    end
  end
  else
    Result := -3;  {nothing to execute}
end;

function TabcLauncher.ExecShellFile(Sender: TObject): Integer;
var
  ShellExecInfo: TShellExecuteInfo;
begin
  Result := -3;  {nothing to execute}
  FLaunchError := 0;

  if FShellParams.FileName <> '' then
  begin
    {initialize params}
    ZeroMemory(@ShellExecInfo, SizeOf(TShellExecuteInfo));
    with ShellExecInfo do
    begin
      cbSize := SizeOf(TShellExecuteInfo);
      fMask := FShellParams.GetMaskFlags;
      Wnd := Application.Handle;
      lpVerb := PChar(FShellParams.Verb);
      lpParameters := PChar(FShellParams.Parameters);
      lpDirectory := PChar(FShellParams.Directory);
      nShow := abcASWEnumToSW_CONST(FWindowState);
      lpFile := PChar(FShellParams.FileName);
    end;
    if FWait then
      ShellExecInfo.fMask := ShellExecInfo.fMask or SEE_MASK_NOCLOSEPROCESS;
    if ShellExecuteEx(@ShellExecInfo) then
    begin
      Result := ShellExecInfo.hInstApp;
      if FWait then
      begin
        {Execute BeforeWait event procedure}
        if Assigned(FBeforeWait) then FBeforeWait(Sender, Result);
        WaitForSingleObject(ShellExecInfo.hProcess, INFINITE);
        CloseHandle(ShellExecInfo.hProcess);
      end;
    end
    else begin
      FLaunchError := GetLastError;
      Result := 0;
    end;
  end;
end;

function TabcLauncher.ExecShellFiles(Sender: TObject): Integer;
var
  ShellExecInfo: TShellExecuteInfo;
  i: integer;
begin
  Result := -3;  {nothing to execute}
  FLaunchError := 0;
  if (FShellParams.FileNames.Count > 0) then
  begin
    {initialize params}
    ZeroMemory(@ShellExecInfo, SizeOf(TShellExecuteInfo));
    with ShellExecInfo do
    begin
      cbSize := SizeOf(TShellExecuteInfo);
      fMask := FShellParams.GetMaskFlags;
      Wnd := Application.Handle;
      lpVerb := PChar(FShellParams.Verb);
      lpParameters := PChar(FShellParams.Parameters);
      lpDirectory := PChar(FShellParams.Directory);
      nShow := abcASWEnumToSW_CONST(FWindowState);
    end;
    if FWait then
      ShellExecInfo.fMask := ShellExecInfo.fMask or SEE_MASK_NOCLOSEPROCESS;
    with FShellParams.FileNames do
      for i := 0 to Count - 1 do
      begin
        ShellExecInfo.lpFile := PChar(Strings[i]);
        if ShellExecuteEx(@ShellExecInfo) then
        begin
          Result := ShellExecInfo.hInstApp;
          if FWait then
          begin
            {Execute BeforeWait event procedure}
            if Assigned(FBeforeWait) then FBeforeWait(Sender, Result);
            WaitForSingleObject(ShellExecInfo.hProcess, INFINITE);
            CloseHandle(ShellExecInfo.hProcess);
          end;
        end
        else
        begin
          FLaunchError := GetLastError;
          Result := 0;
          Exit;
        end;
      end;
  end;
end;

procedure TabcLauncher.SetEnabled(Value: Boolean);
begin
  FEnabled := Value;
  if not(csDesigning in ComponentState) then
  begin
    FTimer.Enabled := Value;
    if Value and (lcTimeLapse in FChecks) then
    begin
      FNextLapseTime := Now + FTimeLapse.Time;
    end
    else
      FNextLapseTime := 0;
  end;
end;

procedure TabcLauncher.SetInterval(Value: Word);
begin
  if Value <> FInterval then begin
    FInterval := Value;
    if not(csDesigning in ComponentState) then
      FTimer.Interval := Value;
  end;
end;

procedure TabcLauncher.SetDate(Value: TabcDateNum);
begin
  FDate := Value;
end;

procedure TabcLauncher.SetShellParams(const Value: TabcShellExecParams);
begin
  FShellParams.Assign(Value);
end;

procedure TabcLauncher.SetTime(Value: TabcTimeNum);
begin
  FTime := Value;
end;

procedure TabcLauncher.SetTimeLapse(Value: TabcTimeNum);
begin
  FTimeLapse := Value;
  if (lcTimeLapse in FChecks) and Enabled and not (csDesigning in ComponentState) then
    FNextLapseTime := Now + FTimeLapse.Time;
end;

procedure TabcLauncher.SetChecks(Value: TabcApplyChecks);
begin
  FChecks := Value;
  if (lcTimeLapse in Value) then
    FContinuous := True;
end;

function TabcLauncher.GetDateTime: TDateTime;
var
  DatePart, TimePart: TDateTime;
begin
  with Date do
    DatePart := EncodeDate(Year,Month,Day);
  with Time do
    TimePart := EncodeTime(Hour,Min,Sec,MSec);
  Result := DatePart + TimePart;
end;

procedure TabcLauncher.SetDateTime(Value: TDateTime);
var
  Year1,Month1,Day1: Word;
  Hour1,Min1,Sec1, MSec1: Word;
begin
  DecodeDate(Value,Year1,Month1,Day1);
  Date.Year := Year1;
  Date.Month := Month1;
  Date.Day := Day1;
  DecodeTime(Value,Hour1,Min1,Sec1, MSec1);
  Time.Hour := Hour1;
  Time.Min := Min1;
  Time.Sec := Sec1;
  Time.MSec := MSec1;
end;


procedure TabcLauncher.Reset;
begin
  FLaunchResult := -1;
  Enabled := True;
end;

procedure TabcLauncher.ResetDateTime(Value: TDateTime);
begin
  SetDateTime(Value);
  Reset;
end;

{ TabcSingleInstance }

const
  ABC_SI_SEMAPHORE_MAX = 65535;

function TabcSingleInstance.GetInstanceCount: Integer;
//returns total number of instances, including this one.
begin
  if FSemaphore <> 0 then
  begin
    ReleaseSemaphore(
      FSemaphore,      // handle of the semaphore object
      1,               // amount to add to current count
      @Result          // address of previous count
    );
    WaitForSingleObject(FSemaphore, INFINITE); //reduce count again
    Result := ABC_SI_SEMAPHORE_MAX - Result;
  end
  else
    Result := 1;
end;

constructor TabcSingleInstance.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActivate := True;
  FMutex := CreateMutex(nil, True, PChar(GetMutexName));
  if (FMutex <> 0) and (GetLastError <> 0) then FMutex := 0;
end;

function TabcSingleInstance.GetMutexName: string;
begin
  Result := 'TabcSingleInstance in application ' + Application.Title;
end;

function TabcSingleInstance.GetSemaphoreName: string;
begin
  Result := 'TabcSingleInstance Semaphore in application ' + Application.Title;
end;

destructor TabcSingleInstance.Destroy;
begin
  if FMutex <> 0 then CloseHandle(FMutex);
  if FSemaphore <> 0 then
  begin
    ReleaseSemaphore(FSemaphore, 1, nil);
    CloseHandle(FSemaphore);
  end;
  inherited Destroy;
end;

procedure TabcSingleInstance.Loaded;
begin
  inherited Loaded;
  HaltIfRunning;
end;

procedure TabcSingleInstance.HaltIfRunning;
begin
  if not (csDesigning in ComponentState) then
  begin
    if MultipleInstances then
      InitSemaphore;
    if (FMutex = 0) then
      CheckTerminate;
  end;
end;

procedure TabcSingleInstance.InitSemaphore;
const
  SEMAPHORE_MODIFY_STATE = $0002;
begin
  if FSemaphore = 0 then
  begin
    FSemaphore := CreateSemaphore(nil, ABC_SI_SEMAPHORE_MAX, ABC_SI_SEMAPHORE_MAX, PChar(GetSemaphoreName));
    if (FSemaphore = 0) and (GetLastError = ERROR_ALREADY_EXISTS) then
      FSemaphore := OpenSemaphore(SEMAPHORE_MODIFY_STATE, False, PChar(GetSemaphoreName));
    if FSemaphore <> 0 then
      WaitForSingleObject(FSemaphore, INFINITE);
  end;
end;

procedure TabcSingleInstance.CheckTerminate;
var
  AppWindow, ActiveWindow: HWND;
  ClassChar, WindowChar: array[0..255] of char;
  prevInstances: Integer;
  HaltThisInstance: Boolean;

  procedure HaltOrTerminateApp;
  begin
    case TerminateBy of
      atmAppTerminate:
        begin
          Application.ShowMainForm := False;
          Application.Terminate;
        end;
      atmCloseMainForm:
        if Application.MainForm <> nil then
          Application.MainForm.Close;
      atmExitProcess:
        ExitProcess(0);
      atmHalt:
        Halt;
      atmTerminateProcess:
        TerminateProcess(GetCurrentProcess, 0);
    end;
  end;

begin

  if MultipleInstances then
  begin
    prevInstances := InstanceCount - 1;
    if Assigned(FOnPreviousInstance) then
    begin
      HaltThisInstance := True;
      FOnPreviousInstance(Self, prevInstances, HaltThisInstance);
      if not HaltThisInstance then
        Exit;
    end
    else
      Exit;
  end;

  if FStopMsg <> '' then
    MessageBox(Application.Handle, PChar(FStopMsg), PChar(Application.Title), MB_ICONEXCLAMATION);
  if FActivate or FClosePrevious then begin
    {Find previous instance using Window Caption}
    GetClassName(Application.Handle, ClassChar, 255);
    GetWindowText(Application.Handle, WindowChar, 255);
    SetWindowText(Application.Handle, '');
    AppWindow := FindWindow(ClassChar, WindowChar);
    SetWindowText(Application.Handle, WindowChar);
    if (AppWindow <> 0) then
    begin
      {Other instance detected}
      if FClosePrevious then
        PostMessage(AppWindow, WM_QUIT, 0, 0)
      else
      begin
        if IsIconic(AppWindow) then
          {Restore when minimised}
          ShowWindow(AppWindow, SW_RESTORE)
        else
        begin
          ActiveWindow := GetLastActivePopup(AppWindow);
          if (ActiveWindow <> 0) and
              IsWindowVisible(ActiveWindow) and
              IsWindowEnabled(ActiveWindow) and
              (ActiveWindow <> AppWindow) then
            SetForegroundWindow(ActiveWindow);
        end;
        HaltOrTerminateApp;
      end;
    end
    else
      HaltOrTerminateApp;  //couldn't find app window
  end
  else
    HaltOrTerminateApp;
end;

procedure TabcSingleInstance.SetActivate(Value: Boolean);
begin
  FActivate := Value;
  if Value then
  begin
    FClosePrevious := False;
  end;
end;

procedure TabcSingleInstance.SetClosePrevious(Value: Boolean);
begin
  FClosePrevious := Value;
  if Value then
  begin
    FActivate := False;
    FMultipleInstances := False;
  end;
end;

procedure TabcSingleInstance.SetMultipleInstances(Value: Boolean);
begin
  FMultipleInstances := Value;
  if Value then
    FClosePrevious := False;
end;

{ TabcShellExecParams }

procedure TabcShellExecParams.Assign(Source: TPersistent);
begin
  if Source is TabcShellExecParams then
    with TabcShellExecParams(Source) do
    begin
      Self.FileName := FileName;
      Self.FileNames := FileNames;
      Self.Parameters := Parameters;
      Self.Directory := Directory;
      Self.Verb := Verb;
      Self.Options := Options;
    end
  else
    inherited;
end;

constructor TabcShellExecParams.Create;
begin
  FFileNames := TStringList.Create;
  FOptions := [asmKeepProcess, asmAllVerbs];
end;

destructor TabcShellExecParams.Destroy;
begin
  FFileNames.Free;
  inherited Destroy;
end;

procedure TabcShellExecParams.SetFileNames(const Value: TStrings);
begin
  FFileNames.Assign(Value);
end;

function TabcShellExecParams.GetMaskFlags: Integer;
begin
  Result := 0;
  if asmConnectNetDrive in Options then
    Result := Result or SEE_MASK_CONNECTNETDRV;
  if asmExpandEnvStrings in Options then
    Result := Result or SEE_MASK_DOENVSUBST;
  if asmDDEWait in Options then
    Result := Result or SEE_MASK_FLAG_DDEWAIT;
  if asmNoErrorMsg in Options then
    Result := Result or SEE_MASK_FLAG_NO_UI;
  if asmKeepProcess in Options then
    Result := Result or SEE_MASK_NOCLOSEPROCESS;
  if asmAllVerbs in Options then
    Result := Result or SEE_MASK_INVOKEIDLIST;
end;

procedure abcExploreFolder(Path: string; Maximized: Boolean);
const
  SHOW_FLAGS: array[Boolean] of Integer = (SW_SHOWNORMAL, SW_SHOWMAXIMIZED);
begin
  ShellExecute(Application.Handle, 'explore', PChar(Path), nil, nil, SHOW_FLAGS[Maximized]);
end;

function abcExecuteFile(FileName: string; Maximized: Boolean): Boolean;
const
  SHOW_FLAGS: array[Boolean] of Integer = (SW_SHOWNORMAL, SW_SHOWMAXIMIZED);
begin
  Result :=
    ShellExecute(Application.Handle, 'open', PChar(FileName), nil, nil, SHOW_FLAGS[Maximized]) > 32;
end;

function abcPrintFile(FileName: string): Boolean;
begin
  Result :=
    ShellExecute(Application.Handle, 'print', PChar(FileName), nil, nil, SW_HIDE) > 32;
end;

end.
