unit abcutild;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{utility functions for databases}

interface

uses SysUtils, DB;

function abcDataSetReadOnly(DataSet: TDataSet): Boolean;
  {Returns true for nil dataset, readonly table, query or stored proc
   with canmodify = false}

function abcMaxFieldType(FieldType: TFieldType): string;
  {Returns a max value as a string for field types ftString..ftDateTime}

function abcGetDataSetDesc(DataSet: TDataSet): string;
  {Returns TableName or Query or StoredProc component name,
   or table component name if global var abc_UseTableComponentNames is set to true}

var abc_UseTableComponentNames: Boolean;

function abcGetTableTypeMask(TableType: Byte): string;
  {Returns '*.DBF' etc file masks, depending on TableType; Supports Apollo}

implementation

uses abcda;

function abcDataSetReadOnly(DataSet: TDataSet): Boolean;
begin
  if DataSet = nil then
    Result := True
  else
    Result := not Dataset.CanModify;
end;

function abcMaxFieldType(FieldType: TFieldType): string;
var
  ResultChars: array[0..255] of char;
begin
  case FieldType of
    ftString:
      begin
        FillChar(ResultChars, 255, #255);
        ResultChars[255] := #0;
        Result := StrPas(ResultChars);
      end;
    ftSmallint:
      Result := '32767';
    ftInteger:
      Result := '2147483647';
    ftWord:
      Result := '65535';
    ftBoolean:
      Result := '1';
    ftFloat, ftCurrency, ftBCD:
      Result := '1.7E308';
    ftDate:
      Result := DateToStr(EncodeDate(9999,12,31));
    ftTime:
      Result := TimeToStr(EncodeTime(23,59,59,999));
    ftDateTime:
      Result := DateTimeToStr(EncodeDate(9999,12,31) + EncodeTime(23,59,59,999));
  else
    Result := '';
  end;
end;

function abcGetDataSetDesc(DataSet: TDataSet): string;
begin
  if DataSet = nil then
    Result := 'nil'
  else
    with abcDSAdapterFactory.CreateAdapter(Dataset) do
    try
      Result := Description;
    finally
      Free;
    end;
end;

function abcGetTableTypeMask(TableType: Byte): string;
begin
  case TableType of
    0: Result := '';
    1: Result := '*.DB';
    2: Result := '*.DBF';
    3: Result := '*.TXT';
    4: Result := '*.DBF';
    {for Apollo DB Engine}
    5: Result := '*.DBF';
    6: Result := '*.DBF';
  else
    Result := '';
  end;
end;

initialization
  abc_UseTableComponentNames := False;

end.
