unit abcutilw;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

{ Windows API Utility Functions and Types }

interface

uses Windows, SysUtils, Classes, abcutil;

function abcGetRegString(Key: HKey; SubKey: string): string;

function abcLaunchBrowserAtURL(URL: string; WindowState: TabcShowWindowStyle): Boolean;

function abcSendEmail(const mailto, cc, bcc, subject: string; const body: TStrings; WindowState: TabcShowWindowStyle): Boolean;

function abcMakeMailToCommand(const mailto, cc, bcc, subject: string; const body: TStrings): string;

function abcLaunchDefaultEmail(URLCommand: string; WindowState: TabcShowWindowStyle): Boolean;

function abcGetWinTempPath: string;

function abcGetWinTempFile: string;

function abcNetworkIsConnected: Boolean;

implementation

uses abcutilf;

function abcGetRegString(Key: HKey; SubKey: string): string;
var
  hK: HKey;
  pBuf: array[0..511] of char;
  lResult, lSize: Longint;
begin
  Result := '';
  lResult := RegOpenKeyEx(Key, PChar(SubKey), 0, KEY_QUERY_VALUE, hK);
  if lResult = ERROR_SUCCESS then
  begin
    lSize := 512;
    RegQueryValue(hK, nil, pBuf, lSize);
    if lSize > 0 then
      Result := pBuf;
    RegCloseKey(hK);
  end;
end;

function abcLaunchBrowserAtURL(URL: string; WindowState: TabcShowWindowStyle): Boolean;
var
  sLaunch: string;
  iPos: integer;
begin
  Result := False;
  sLaunch := abcGetRegString(HKEY_CLASSES_ROOT, '.htm');
  if sLaunch <> '' then
  begin
    sLaunch := sLaunch + '\shell\open\command';
    sLaunch := abcGetRegString(HKEY_CLASSES_ROOT, sLaunch);
    if sLaunch <> '' then
    begin
      iPos := Pos('"%1"', sLaunch);
      if iPos = 0 then
        iPos := Pos('%1', sLaunch);
      if iPos <> 0 then
        sLaunch := Copy(sLaunch, 1, iPos-1);
      sLaunch := sLaunch + ' ' + URL;
      Result := (WinExec(PChar(sLaunch), abcASWEnumToSW_CONST(WindowState)) >= 32);
    end;
  end;
end;

{ Send Email }

function abcSendEmail(const mailto, cc, bcc, subject: string; const body: TStrings; WindowState: TabcShowWindowStyle): Boolean;
begin
  Result := abcLaunchDefaultEmail(abcMakeMailToCommand(mailto, cc, bcc, subject, body), WindowState);
end;

function abcMakeMailToCommand(const mailto, cc, bcc, subject: string; const body: TStrings): string;
const
  MAIL_TAG_TO = 'mailto:';
  MAIL_TAG_CC = 'cc=';
  MAIL_TAG_BCC = 'bcc=';
  MAIL_TAG_SUBJECT = 'subject=';
  MAIL_TAG_BODY = 'body=';
  MAIL_TAG_SEP = '&';
  MAIL_TAG_SEP_PARAMS = '?';
var
  sParams: string;
begin
  Result := MAIL_TAG_TO + abcEncodeASCIItoHex(mailto);
  sParams := '';
  if cc <> '' then
    sParams := sParams + MAIL_TAG_SEP + MAIL_TAG_CC + abcEncodeASCIItoHex(cc);
  if bcc <> '' then
    sParams := sParams + MAIL_TAG_SEP + MAIL_TAG_BCC + abcEncodeASCIItoHex(bcc);
  if subject <> '' then
    sParams := sParams + MAIL_TAG_SEP + MAIL_TAG_SUBJECT + abcEncodeASCIItoHex(subject);
  if body.Count > 0 then
    sParams := sParams + MAIL_TAG_SEP + MAIL_TAG_BODY + abcEncodeASCIItoHex(body.Text);
  if sParams <> '' then
    Result := Result + MAIL_TAG_SEP_PARAMS + Copy(sParams, Length(MAIL_TAG_SEP) + 1, Maxint);
end;


function abcLaunchDefaultEmail(URLCommand: string; WindowState: TabcShowWindowStyle): Boolean;
var
  sLaunch: string;
  iPos: integer;
begin
  Result := False;
  sLaunch := abcGetRegString(HKEY_CLASSES_ROOT,'mailto\shell\open\command');
  if sLaunch <> '' then
  begin
    iPos := Pos('"%1"', sLaunch);
    if iPos = 0 then
      iPos := Pos('%1', sLaunch);
    if iPos <> 0 then
      sLaunch := Copy(sLaunch, 1, iPos-1);
    sLaunch := sLaunch + ' ' + URLCommand;
    Result := (WinExec(PChar(sLaunch), abcASWEnumToSW_CONST(WindowState)) >= 32);
  end;
end;

function abcGetWinTempPath: string;
var
  pTempPath: array[0..255] of char;
begin
  GetTempPath(255, pTempPath);
  Result := StrPas(pTempPath);
end;

function abcGetWinTempFile: string;
var
  pTempPath: array[0..255] of char;
  pTempFile: array[0..255] of char;
begin
  GetTempPath(255, pTempPath);
  GetTempFileName(pTempPath, '', 0, pTempFile);
  Result := StrPas(pTempFile);
end;

function abcNetworkIsConnected: Boolean;
var
  lphEnum: THandle;
begin
  Result := False;
  if WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_ANY, 0, nil, lphEnum) <> NO_ERROR then
    Exit;
  Result := (WNetCloseEnum(lphEnum) = NO_ERROR);
end;

end.
