unit abcversn;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, StdCtrls;

type

  { TabcVersionLabel }

  TabcCustomVersionLabel = class;

  TabcVersionInfoItem = class(TCollectionItem)
  private
    FKey: string;
    FCaption: string;
    FValue: string;
    FVisible: Boolean;
    procedure SetCaption(Value: string);
    procedure SetKey(Value: string);
    procedure SetVisible(Value: Boolean);
  protected
    function GetDisplayName: string; override;
    procedure SetDisplayName(const Value: string); override;
  public
    constructor Create(Collection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
  published
    property Key: string read FKey write SetKey;
    property Caption: string read FCaption write SetCaption;
    property Value: string read FValue write FValue;
    property Visible: Boolean read FVisible write SetVisible default True;
  end;

  TabcVersionInfoCollection = class(TCollection)
  private
    FVersionLabel: TabcCustomVersionLabel;
    FOnUpdate: TNotifyEvent;
    function GetItem(Index: Integer): TabcVersionInfoItem;
    procedure SetItem(Index: Integer; Value: TabcVersionInfoItem);
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
    procedure GetAllVersionInfo; virtual;
  public
    constructor Create(AVersionLabel: TabcCustomVersionLabel);
    function Add: TabcVersionInfoItem;
    property Items[Index: Integer]: TabcVersionInfoItem read GetItem write SetItem; default;
    property OnUpdate: TNotifyEvent read FOnUpdate write FOnUpdate;
  end;

  TabcCustomVersionLabel = class(TCustomLabel)
  private
    FVersionInfo: TabcVersionInfoCollection;
    FDoubleSpaced: Boolean;
    FTabStop: Byte;
    function GetItemByKey(AKey: string): TabcVersionInfoItem;
    function GetKeyValue(AKey: string): string;
    procedure SetDoubleSpaced(Value: Boolean);
    procedure SetTabStop(Value: Byte);
    procedure SetVersionInfo(Value: TabcVersionInfoCollection);
    function GetOnUpdateVersionInfo: TNotifyEvent;
    procedure SetOnUpdateVersionInfo(Value: TNotifyEvent);
  protected
    procedure DoDrawText(var Rect: TRect; Flags: Longint); override;
    procedure UpdateCaption; virtual;
    procedure Loaded; override;
    procedure Paint; override;
    property DoubleSpaced: Boolean read FDoubleSpaced write SetDoubleSpaced default True;
    property TabStop: Byte read FTabStop write SetTabStop default 10;
    property ItemByKey[AKey: string]: TabcVersionInfoItem read GetItemByKey;
    property ValueByKey[AKey: string]: string read GetKeyValue;
    property VersionInfo: TabcVersionInfoCollection read FVersionInfo write SetVersionInfo;
    property OnUpdateVersionInfo: TNotifyEvent read GetOnUpdateVersionInfo write SetOnUpdateVersionInfo;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
  end;

  TabcVersionLabel = class(TabcCustomVersionLabel)
  public
    property ItemByKey;
    property ValueByKey;
  published
    property DoubleSpaced;
    property TabStop;
    property VersionInfo;
    property OnUpdateVersionInfo;
    {inherited label properties}
    {property Align;}
    {property Alignment;}
    {property AutoSize;}
    {property Caption;}
    property Color;
    {property DragCursor;}
    {property DragMode;}
    property Enabled;
    {property FocusControl;}
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    {property PopupMenu;}
    {property ShowAccelChar;}
    {property ShowHint;}
    property Transparent;
    {property Layout;}
    property Visible;
    {property WordWrap;}
    property OnClick;
    property OnDblClick;
    {property OnDragDrop;}
    {property OnDragOver;}
    {property OnEndDrag;}
    {property OnMouseDown;}
    {property OnMouseMove;}
    {property OnMouseUp;}
    {property OnStartDrag;}
  end;

function abcGetFileVersion(const AFileName: TFileName): string;
function abcGetFileVersionKey(const AFileName: TFileName; const AKeyValue: string): string;
function abcGetFileVersionKeys(const AFileName: TFileName; const AKeyValues, AResultStrings: TStrings): Boolean;

procedure abcGetWindowsVersionStrings(const strings: TStrings);

resourcestring
  sabcWin32Ver = 'Win32 OS Version:';
  sabcWin32VerA = '  Platform = ';
  sabcWin32VerB = '  MajorVersion = ';
  sabcWin32VerC = '  MinorVersion = ';
  sabcWin32VerD = '  BuildNumber = ';
  sabcWin32VerE = '  CSDVersion = ';
  sabcWin32PlatformWIN32s = 'WIN32s';
  sabcWin32PlatformWIN32Windows = 'WIN32_WINDOWS';
  sabcWin32PlatformWIN32NT = 'WIN32_NT';
  sabcWin32PlatformOther = 'WIN32_OTHER';

implementation

{ TabcVersionInfoItem }

constructor TabcVersionInfoItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FVisible := True;
  FValue := '';
end;

procedure TabcVersionInfoItem.Assign(Source: TPersistent);
begin
  if Source is TabcVersionInfoItem then
  begin
    FCaption := TabcVersionInfoItem(Source).Caption;
    FKey := TabcVersionInfoItem(Source).Key;
    FValue := '';
  end
  else
    inherited Assign(Source);
end;

function TabcVersionInfoItem.GetDisplayName: string;
begin
  Result := Key;
end;

procedure TabcVersionInfoItem.SetDisplayName(const Value: string);
begin
  Key := Value;
end;

procedure TabcVersionInfoItem.SetCaption(Value: string);
begin
  FCaption := Value;
  Changed(False);
end;

procedure TabcVersionInfoItem.SetKey(Value: string);
begin
  FKey := Value;
  Changed(False);
end;

procedure TabcVersionInfoItem.SetVisible(Value: Boolean);
begin
  FVisible := Value;
  Changed(False);
end;

{ TabcVersionInfoCollection }

constructor TabcVersionInfoCollection.Create(AVersionLabel: TabcCustomVersionLabel);
begin
  inherited Create(TabcVersionInfoItem);
  FVersionLabel := AVersionLabel;
end;

function TabcVersionInfoCollection.Add: TabcVersionInfoItem;
begin
  Result := TabcVersionInfoItem(inherited Add);
end;

function TabcVersionInfoCollection.GetItem(Index: Integer): TabcVersionInfoItem;
begin
  Result := TabcVersionInfoItem(inherited GetItem(Index));
end;

function TabcVersionInfoCollection.GetOwner: TPersistent;
begin
  Result := FVersionLabel;
end;

procedure TabcVersionInfoCollection.SetItem(Index: Integer; Value: TabcVersionInfoItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TabcVersionInfoCollection.Update(Item: TCollectionItem);
{lookup all item values}
begin
  if not (csDesigning in FVersionLabel.ComponentState) then
    GetAllVersionInfo;
  if Assigned(FOnUpdate) then FOnUpdate(Self);
  FVersionLabel.UpdateCaption;
end;

procedure TabcVersionInfoCollection.GetAllVersionInfo;
var
  keyStrings, resultStrings: TStringList;
  verExists: Boolean;
  i: integer;
begin
  keyStrings := TStringList.Create;
  resultStrings := TStringList.Create;
  try
    for i := 0 to Count - 1 do
      keyStrings.Add(Items[i].Key);
    verExists := abcGetFileVersionKeys(Application.ExeName, keyStrings, resultStrings);
    for i := 0 to Count - 1 do
      if verExists then
        Items[i].FValue := resultStrings[i]
      else
        Items[i].FValue := '';
  finally
    keyStrings.Free;
    resultStrings.Free;
  end;
end;

{ TabcCustomVersionLabel }

constructor TabcCustomVersionLabel.Create(AOwner: TComponent);
begin
  inherited Create(aOwner);
  AutoSize := False;
  WordWrap := True;
  FVersionInfo := TabcVersionInfoCollection.Create(Self);
  FDoubleSpaced := True;
  FTabStop := 10;
end;

destructor TabcCustomVersionLabel.Destroy;
begin
  FVersioninfo.Free;
  inherited Destroy;
end;

procedure TabcCustomVersionLabel.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
    FVersionInfo.Update(nil);
end;

function TabcCustomVersionLabel.GetItemByKey(AKey: string): TabcVersionInfoItem;
var
  i: integer;
begin
  Result := nil;
  with FVersionInfo do
    for i := 0 to Count - 1 do
      if Items[i].Key = AKey then
      begin
        Result := Items[i];
        Break;
      end;
end;

function TabcCustomVersionLabel.GetKeyValue(AKey: string): string;
var
  i: integer;
begin
  Result := '';
  with FVersionInfo do
    for i := 0 to Count - 1 do
      if Items[i].Key = AKey then
      begin
        Result := Items[i].Value;
        Break;
      end;
end;

procedure TabcCustomVersionLabel.UpdateCaption;
var
  i: integer;
  sCaption, sTab: string;
begin
  sCaption := '';
  if TabStop <> 0 then
    sTab := #9
  else
    sTab := '';
  if VersionInfo.Count = 0 then
    if (csDesigning in ComponentState) then
      sCaption := '<VersionInfo needed>'
    else
      sCaption := ''
  else
    with VersionInfo do
      for i := 0 to Count - 1 do
        with Items[i] do
          if Visible then
          begin
            if (csDesigning in ComponentState) then
              sCaption := sCaption + Caption + sTab + '<not available at designtime>' + #13#10
            else
              sCaption := sCaption + Caption + sTab + Value + #13#10;
            if DoubleSpaced then sCaption := sCaption + #13#10;
          end;
  Caption := sCaption;
end;

procedure TabcCustomVersionLabel.DoDrawText(var Rect: TRect; Flags: Longint);
var
  Text: string;
begin
  Text := GetLabelText;
  if (Flags and DT_CALCRECT <> 0) and (Text = '') then Text := Text + ' ';
  Canvas.Font := Font;
  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
  end
  else
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
end;

procedure TabcCustomVersionLabel.Paint;
var
  Rect: TRect;
  Flags: Integer;
  TabStopFlag: Word;
begin
  {draw text with adjusted tabstop}
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    if TabStop <> 0 then
    begin
      {set tabstop}
      TabStopFlag := TabStop;
      TabStopFlag := TabStopFlag shl 8;
      Flags := DT_NOPREFIX or DT_WORDBREAK or DT_LEFT or DT_EXPANDTABS or DT_TABSTOP or TabStopFlag;
    end
    else
      Flags := DT_NOPREFIX or DT_WORDBREAK or DT_LEFT;
    DoDrawText(Rect, Flags);
  end;
end;

procedure TabcCustomVersionLabel.SetDoubleSpaced(Value: Boolean);
begin
  FDoubleSpaced := Value;
  UpdateCaption;
end;

procedure TabcCustomVersionLabel.SetTabStop(Value: Byte);
begin
  FTabStop := value;
  UpdateCaption;
  invalidate;
end;

function TabcCustomVersionLabel.GetOnUpdateVersionInfo: TNotifyEvent;
begin
  Result := FVersionInfo.OnUpdate;
end;

procedure TabcCustomVersionLabel.SetOnUpdateVersionInfo(Value: TNotifyEvent);
begin
  FVersionInfo.OnUpdate := Value;
end;

procedure TabcCustomVersionLabel.SetVersionInfo(Value: TabcVersionInfoCollection);
begin
  FVersionInfo.Assign(Value);
end;

function abcGetFileVersion(const AFileName: TFileName): string;
begin
  Result := abcGetFileVersionKey(AFileName, 'FileVersion');
end;

function abcGetFileVersionKey(const AFileName: TFileName; const AKeyValue: string): string;
var
  keyStrings, resultStrings: TStringList;
begin
  Result := '';
  keyStrings := TStringList.Create;
  resultStrings := TStringList.Create;
  try
    keyStrings.Add(AKeyValue);
    if abcGetFileVersionKeys(AFileName, keyStrings, resultStrings) then
      Result := resultStrings[0];
  finally
    keyStrings.Free;
    resultStrings.Free;
  end;
end;

function abcGetFileVersionKeys(const AFileName: TFileName; const AKeyValues, AResultStrings: TStrings): Boolean;
var
  dummy, valueLen, langLen: UINT;
  i, temp, verInfoSize: integer;
  infoBuf, langBuf, valueBuf: PChar;
  pVersionValue: Pointer;
  sValueKey, sLanguageLocale: string;

  function GetBuildFlags: string;
  var
   fixedFileInfo: PVSFixedFileInfo;
   valueLen: UINT;
  begin
    Result := '';
    if VerQueryValue(infoBuf, '\', Pointer(fixedFileInfo), valueLen) then
      if (valueLen > 1) then
      begin
        if fixedFileInfo.dwFileFlags and VS_FF_DEBUG <> 0 then
            Result := Result + ', Debug';
        if fixedFileInfo.dwFileFlags and VS_FF_INFOINFERRED <> 0  then
            Result := Result + ', InfoInferred';
        if fixedFileInfo.dwFileFlags and VS_FF_PATCHED <> 0 then
            Result := Result + ', Patched';
        if fixedFileInfo.dwFileFlags and VS_FF_PRERELEASE <> 0 then
            Result := Result + ', Pre-Release';
        if fixedFileInfo.dwFileFlags and VS_FF_PRIVATEBUILD <> 0  then
            Result := Result + ', Private';
        if fixedFileInfo.dwFileFlags and VS_FF_SPECIALBUILD <> 0  then
            Result := Result + ', Special';
        if Result <> '' then Delete(Result, 1, 2);
      end;
  end;

begin
  Result := False;
  AResultStrings.Clear;
  verInfoSize := GetFileVersionInfoSize(PChar(AFileName), dummy);
  if verInfoSize > 0 then
  begin
    Result := True;
    infoBuf := StrAlloc(verInfoSize+1);
    try
      GetFileVersionInfo(PChar(AFileName), 0, verInfoSize, infoBuf);
      {get Language Locale ID}
      VerQueryValue(infoBuf, '\VarFileInfo\Translation', pointer(langBuf), langLen);
      if langLen >= 4 then
      begin
          temp := 0;
          StrLCopy(@temp, langBuf, 2);
          sLanguageLocale:= Format('%4.4x', [temp]);
          temp := 0;
          StrLCopy(@temp, langBuf+2, 2);
          sLanguageLocale := sLanguageLocale + Format('%4.4x', [temp]);
      end
      else
        sLanguageLocale := '040904E4';  {US English, ANSI Code Page}

      sValueKey := '\StringFileInfo\'+sLanguageLocale+'\';

      {get strings}
      for i := 0 to AKeyValues.Count - 1 do
      if AKeyValues[i] = 'BuildFlags' then
        AResultStrings.Add(GetBuildFlags)
      else
      begin
        if VerQueryValue(infoBuf,
                         pchar(sValueKey + AKeyValues[i]),
                         pVersionValue,
                         valueLen) then
        begin
          valueBuf := StrAlloc(valueLen);
          try
            StrLCopy(valueBuf, PChar(pVersionValue), valueLen);
            AResultStrings.Add(string(valueBuf));
          finally
            StrDispose(valueBuf);
          end;
        end
        else
          AResultStrings.Add('');
      end;
    finally
      StrDispose(infoBuf);
    end;
  end;
end;

procedure abcGetWindowsVersionStrings(const strings: TStrings);
var
  s: string;
begin
  with strings do
  begin

    Add(sabcWin32Ver);

    case Win32Platform of
      VER_PLATFORM_WIN32s:
        s := sabcWin32PlatformWIN32s;
      VER_PLATFORM_WIN32_WINDOWS:
        s := sabcWin32PlatformWIN32Windows;
      VER_PLATFORM_WIN32_NT:
        s := sabcWin32PlatformWIN32NT;
    else
      s := sabcWin32PlatformOther;
    end;

    Add(sabcWin32VerA + s);

    Add(sabcWin32VerB + InttoStr(Win32MajorVersion));

    Add(sabcWin32VerC + InttoStr(Win32MinorVersion));

    Add(sabcWin32VerD + InttoStr(Win32BuildNumber));

    Add(sabcWin32VerE + Win32CSDVersion);

  end;
end;

end.
