unit abcwtch;

(*------------------------------------------------------------------------------

Advanced Business Components
ABC for Delphi
Version 6

Copyright  1995, 2001 Objective Software Technology Pty Limited ACN 068 640 353
Functionality derived from Delphi VCL Source Code
is Copyright  1995, 2001 Borland Software Corporation

------------------------------------------------------------------------------*)

{$I abcdefs.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Forms, Dialogs, abcfwtch;

type

{TabcStopWatch}

  TabcLogWhen = (lwAfterStop, lwBeforeReset);
  TabcLogFormat = (lfSeconds, lfTicks);

  TabcStopWatch = class(TComponent)
  private
    FLaps: Longint;
    FStartTicks: Longint;
    FStopTicks: Longint;
    FTotalTicks: Longint;
    FLastTicks: Longint;
    FLogFile: TFileName;
    FLogWhen: TabcLogWhen;
    FLogFormat: TabcLogFormat;
    FLogFileHandle: TextFile;
    function GetAverage: Longint;
    procedure OpenLogFile;
    procedure CloseLogFile;
    procedure WriteLogStopTime;
    procedure WriteLogReset;
  public
    property Laps: Longint read FLaps;
    property TotalTime: Longint read FTotalTicks;
    property LastLapTime: Longint read FLastTicks;
    property AvgLapTime: Longint read GetAverage;
    procedure Start;
    procedure Stop;
    procedure Reset;
    procedure Show;
  published
    property LogFile: TFileName read FLogFile write FLogFile;
    property LogFormat: TabcLogFormat read FLogFormat write FLogFormat;
    property LogWhen: TabcLogWhen read FLogWhen write FLogWhen;
  end;

implementation

{TabcStopWatch}

function TabcStopWatch.GetAverage: Longint;
begin
  if FLaps = 0 then
    GetAverage := 0
  else
    GetAverage := Round(FTotalTicks / FLaps);
end;

procedure TabcStopWatch.Start;
begin
  FStartTicks:= GetTickCount;
end;

procedure TabcStopWatch.Stop;
begin
  FStopTicks:= GetTickCount;
  FLastTicks:= FStopTicks - FStartTicks;
  FTotalTicks:= FTotalTicks + FLastTicks;
  Inc(FLaps);
  if (FLogWhen = lwAfterStop) and (FlogFile <> '') then
    WriteLogStopTime;
end;

procedure TabcStopWatch.Reset;
begin
  if FlogFile <> '' then begin
    if (FLogWhen = lwBeforeReset) then
      WriteLogStopTime;
    WriteLogReset;
  end;
  FLaps := 0;
  FStartTicks := 0;
  FStopTicks := 0;
  FTotalTicks := 0;
  FLastTicks := 0;
end;

procedure TabcStopWatch.Show;
var
  TotalSecs, AvgSecs, LastLapSecs: Single;
begin
  with TabcsfStopWatch.Create(nil) do
  try
    Caption := Name;
    lblLaps.Caption := IntToStr(FLaps);
    if FLogFormat = lfSeconds then begin
      TotalSecs := FTotalTicks / 1000;
      LastLapSecs := FLastTicks / 1000;
      AvgSecs := AvgLapTime / 1000;
      lblTotal.Caption := Format('%.3n %s',[TotalSecs,' seconds']);
      lblAverage.Caption := Format('%.3n %s',[AvgSecs,' seconds']);
      lblLastLap.Caption := Format('%.3n %s',[LastLapSecs,' seconds']);
    end
    else begin
      lblTotal.Caption := InttoStr(FTotalTicks) + ' mS';
      lblAverage.Caption := InttoStr(AvgLapTime) + ' mS';
      lblLastLap.Caption := InttoStr(FLastTicks) + ' mS';
    end;
    ShowModal;
  finally
    Free;
  end;
end;

procedure TabcStopWatch.OpenLogFile;
begin
  AssignFile(FLogFileHandle, FLogFile);
  try
    Append(FLogFileHandle);
  except
    on EInOutError do begin
      Rewrite(FLogFileHandle);
      {add headings for CSV entries in the file}
      Writeln(FLogFileHandle, 'Name, Laps, Total Time, Average Time, Last Lap');
    end;
  end;
end;

procedure TabcStopWatch.CloseLogFile;
begin
  CloseFile(FLogFileHandle);
end;

procedure TabcStopWatch.WriteLogReset;
{write a 'Reset + component name' message to the logfile}
begin
  OpenLogFile;
  Writeln(FLogFileHandle, 'Reset ' + Name);
  CloseLogFile;
end;

procedure TabcStopWatch.WriteLogStopTime;
{write times at last stop to the logfile}
{format is comma separated values of:
 TabcStopWatch.Name, TabcStopWatch.FLaps, TabcStopWatch.FTotalTicks, TabcStopWatch.GetAverage, TabcStopWatch.FLastTicks}
var
  TotalSecs, AvgSecs, LastLapSecs: Single;
begin
  OpenLogFile;
  Write(FLogFileHandle, Name + ',');
  Write(FLogFileHandle, InttoStr(FLaps) + ',');
  if FLogFormat = lfSeconds then begin
    TotalSecs := FTotalTicks / 1000;
    AvgSecs := Round(AvgLapTime)/1000;
    LastLapSecs := FLastTicks / 1000;
    Write(FLogFileHandle, Format('%.3n',[TotalSecs]) + ',');
    Write(FLogFileHandle, Format('%.3n',[AvgSecs]) + ',');
    Writeln(FLogFileHandle, Format('%.3n',[LastLapSecs]));
  end
  else begin
    Write(FLogFileHandle, InttoStr(FTotalTicks) + ',');
    Write(FLogFileHandle, InttoStr(AvgLapTime) + ',');
    Writeln(FLogFileHandle, InttoStr(FLastTicks));
  end;
  CloseLogFile;
end;

end.
